/*
 * Mylene0fa.c
 * Copyright (C) 2002-2012, 2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * BUGS
 *
 * If cache size is too small bad things will happen.  Have to check that it is
 * greater than e0fa - loss + smth...
 */
/*
 * TODO
 *
 *.extend libx1f4i2 so that it tells application whether a write will block,
 * and if so how much does the application has to sleep before trying again.
 * Kind of done.  Have to fix stream merging, too.
 *.make sure output data is flushed when exiting upon Mylene1f4 request and
 * !__SYNCRONIZE_STREAMS__.
 */

#include <config.h>

#if defined HAVE_FEATURES_H
# include <features.h>
#endif				/* HAVE_FEATURES_H */

#include <errno.h>
#include <getopt.h>
#if defined HAVE_LIBx1f4i0
# include <libx1f4i0.h>
#endif				/* HAVE_LIBx1f4i0 */
#include <libx1f4i2.h>
#include <signal.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/select.h>
#include <sys/shm.h>
#include <sys/time.h>
#include <sys/types.h>

#if !defined HAVE_LIBx1f4i0
# include <cardinal-wx.h>
# include <cardinal-x.h>
# include <integral-x.h>
#endif				/* !HAVE_LIBx1f4i0 */
#include <logique.0.h>
#include <logique.1.h>
#include <logique.3.h>

#if WORDS_BIGENDIAN
# define COMPLETIONS			0x7fffffff
#else
# define COMPLETIONS			0xffffffff
#endif				/* WORDS_BIGENDIAN */

static int annoy_device(struct context_state *);
static int delay_header(struct context_state *);
static int write_header(struct context_state *);

static void clean_up(int);
static void usage(void);

static void *downward;

const char start_again[] = "1f47\0\0\0\0\0\0\0\0\0\0\0\0";

static int
annoy_device(struct context_state *context_data)
{
    int status;

    context_data->detached = 0;
    context_data->paused = 0;

    status = delay_header(context_data);
    if (status) {
    } else {
	context_data->completions = COMPLETIONS;
	context_data->e0fa_index = 0;
	context_data->e1f4_size = 0;
	context_data->head_state = NULL;
	context_data->outgoing = 0;
	context_data->output = NULL;
	context_data->output_size = 0;
	context_data->pending_1f4 = 0;
#if !__Mylene0fa_LINE_INTER__
	context_data->segment = NULL;
#endif				/* !__Mylene0fa_LINE_INTER__ */
	context_data->tail_state = NULL;

#if 0
	/*
	 * better not set it, for we may have been restarted
	 */
	context_data->state[1] = 0;
#endif				/* 0 */

	context_data->state[Mylene0fa_B1st_STATE] = Mylene0fa_B1st_PLAYING;

	ADVANCE_COMPLETIONS(context_data->completions);

	do {
	    if (context_data->detached) {
		context_data->state[Mylene0fa_B1st_STATE] |=
		    Mylene0fa_B1st_DELAYED;
	    } else {
		status = Mylene0fa_setup_device(context_data);
		if (status) {
		    break;
		}
	    }

	    status = Mylene0fa_class_mylene(context_data);
	    if (status) {
		break;
	    }

	    context_data->down = 1;

	    status = Mylene0fa_cycle_device(context_data);

	    if (context_data->down) {
	    } else {
		int excess;

		excess = Mylene0fa_flush_device(context_data, 1);
		if (status) {
		} else {
		    status = excess;
		}
	    }

	    {
		void *xaudio;

		xaudio = context_data->xaudio;
		if (xaudio) {
		    int excess;

		    /*
		     * BUGS
		     *
		     * Small race condition here: if a signal arrives just
		     * after resetting context_data->xaudio device will not be
		     * closed...
		     */
		    context_data->xaudio = NULL;
		    excess = x1f4_lose_xaudio(&xaudio);
		    if (status) {
		    } else {
			status = excess;
		    }
		}
	    }
	} while (0);

#if !__Mylene0fa_LINE_INTER__
	{
	    void *segment;

	    segment = context_data->segment;
	    if (segment) {
		free(segment);
	    }
	}
#endif				/* !__Mylene0fa_LINE_INTER__ */
	{
	    void *output;

	    output = context_data->output;
	    if (output) {
		free(output);
	    }
	}
	{
	    struct state_type *state_data;

	    state_data = context_data->head_state;
	    while (state_data) {
		struct state_type *free_state;

		free_state = state_data;
		state_data = state_data->state_data;
		free(free_state);
	    }
	}
    }

    return status;
}


static int
delay_header(struct context_state *context_data)
{
    fd_set fd_set_0;
    const char *header;
    int status = 0;
    unsigned advance = 16;

    header = start_again;

    FD_ZERO(&fd_set_0);
    FD_SET(STDIN_FILENO, &fd_set_0);

    while (1) {
	if (select(STDIN_FILENO + 1, &fd_set_0, NULL, NULL, NULL) == -1) {
	    if (errno == EINTR) {
	    } else {
		fprintf(stderr, "%s: cannot select standard input: %s\n",
			context_data->Mylene0fa, strerror(errno));
		status = 1;
		break;
	    }
	} else {
	    char c;

	    status = read(STDIN_FILENO, &c, 1);
	    if (status != 1) {
		fprintf(stderr, "%s: cannot read standard input: %s\n",
			context_data->Mylene0fa, strerror(errno));
		status = 1;
		break;
	    } else {
		status = c != *header;
		if (status) {
		    if (advance != 1) {
			break;
		    } else {
			if (c != 1) {
			    break;
			} else {
			    context_data->detached = 0x800;
			    context_data->paused = 0x1f4;
			}
		    }
		}
		{
		    header++;
		    advance--;
		    if (advance) {
		    } else {
			status = write_header(context_data);

			break;
		    }
		}
	    }
	}
    }

    return status;
}


static int
write_header(struct context_state *context_data)
{
    int status;

    status = write(STDOUT_FILENO, start_again, 16);
    status -= 16;
    if (status) {
	if (status == -17) {
	    fprintf(stderr, "%s: cannot write standard output: %s\n",
		    context_data->Mylene0fa, strerror(errno));
	} else {
	    fprintf(stderr, "%s: short standard output write: %d\n",
		    context_data->Mylene0fa, status + 16);
	}
    }

    return status;
}


static void
clean_up(int none)
{
    void *xaudio;

    xaudio = ((struct context_state *) downward)->xaudio;
    if (xaudio) {
	((struct context_state *) downward)->xaudio = NULL;
	x1f4_fini_xaudio(&xaudio, 1);
    }

    exit(1);
}


static void
usage(void)
{
    puts("Usage: Mylene0fa [OPTIONS] CACHE_SIZE SHMID\n\
Annoy audio device.\n\
\n\
Options:\n\
  -d, --device AUDIO		set device to AUDIO (`" x1f4_AUDIO_DEVICE "'\
 by default)\n\
  -l, --latency MICROSECONDS	wait for cache at least MICROSECONDS worth\n\
				before start playing\n\
      --help			display this help and exit\n\
      --version			output version information and exit");
}


int
main(int argc, char **argv)
{
    int status;
    struct context_state context;

    context.Mylene0fa = argv[0];
    context.device = x1f4_AUDIO_DEVICE;
    context.xaudio = NULL;

    unsetenv("POSIXLY_CORRECT");

    {
	const char *latency_specs = NULL;

	while (1) {
	    char c;
	    static struct option long_options[] = {
/* *INDENT-OFF* */
		{   "device",	    0x01,   NULL,   0x64    },
		{   "latency",	    0x01,   NULL,   0x6c    },
		{   "version",	    0x00,   NULL,   0x56    },
		{   "help",	    0x00,   NULL,   0x48    },
		{   NULL,	    0x00,   NULL,   0x00    }
/* *INDENT-ON* */
	    };

	    c = getopt_long(argc, argv, "d:l:", long_options, NULL);

	    if (!~c) {
		break;
	    }

	    switch (c) {
	    case 'H':
		usage();

		return 0;
	    case 'V':
		printf("%s (%s) %s\n", argv[0], PACKAGE, VERSION);

		return 0;
	    case 'd':
		context.device = optarg;
		break;
	    case 'l':
		latency_specs = optarg;
		break;
	    case '?':
		return 1;
	    }
	}

	if (!latency_specs) {
	    context.latency = 0;
	} else {
	    if (x1f4_parse_xcardinal
		(&context.latency, latency_specs, NULL, 0)) {
		fprintf(stderr, "%s: cannot parse latency specification: `%s"
			"'\n", argv[0], latency_specs);

		return 1;
	    }
	}
    }

    if (optind + 2 != argc) {
	fprintf(stderr, "%s: wrong number of arguments\nType `%s --help' for "
		"more information.\n", argv[0], argv[0]);

	return 1;
    }

    if (x1f4_parse_wxcardinal
	(&context.cache_size, argv[optind + 0], NULL, 0)) {
	fprintf(stderr, "%s: cannot parse cache size specification: `%s'\n",
		argv[0], argv[optind + 0]);

	return 1;
    } else if (!context.cache_size) {
	fprintf(stderr, "%s: I won't go for null buffer sizes\n", argv[0]);

	return 1;
    }
    if (x1f4_parse_xintegral(&context.shmid, argv[optind + 1], NULL, 0)) {
	fprintf(stderr, "%s: cannot parse shmid specification: `%s'\n",
		argv[0], argv[optind + 1]);

	return 1;
    }

    context.cache_size *= 36864;

    downward = &context;

    signal(SIGHUP, clean_up);
    signal(SIGINT, clean_up);
    signal(SIGQUIT, clean_up);
    signal(SIGILL, clean_up);
    signal(SIGABRT, clean_up);
    signal(SIGFPE, clean_up);
#if 0
    signal(SIGKILL, clean_up);
#endif				/* 0 */
    signal(SIGSEGV, clean_up);
    signal(SIGPIPE, clean_up);
    signal(SIGALRM, clean_up);
    signal(SIGTERM, clean_up);
    signal(SIGUSR1, clean_up);
    signal(SIGUSR2, clean_up);
#if 0
    signal(SIGCHLD, clean_up);
    signal(SIGCONT, clean_up);
    signal(SIGSTOP, clean_up);
#endif				/* 0 */
    signal(SIGTSTP, clean_up);
    signal(SIGTTIN, clean_up);
    signal(SIGTTOU, clean_up);

    if ((context.audio = shmat(context.shmid, 0, 0)) == (void *) -1) {
	fprintf(stderr, "%s: cannot attach `%s'\n", argv[0], argv[optind + 1]);

	return 1;
    }

    context.state = (unsigned *) ((char *) context.audio + context.cache_size);
    context.atime = (struct timeval *) (context.state + 128);

    status = annoy_device(&context);

    shmdt(context.audio);

    return status;
}
