/*
 * logique.1.c
 * Copyright (C) 2006-2008, 2015, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <errno.h>
#include <libx1f4i2.h>
#include <stddef.h>
#include <stdio.h>
#include <unistd.h>

#include <sys/select.h>
#include <sys/time.h>
#include <sys/types.h>

#include <logique.0.h>
#include <logique.1.h>
#include <logique.2.h>
#include <logique.3.h>

int
Mylene0fa_merge_device(struct context_state *context_data)
{
    int status = 0;
# if HERE_UNDERFLOW == EXIT_UNDERFLOW
    int stream = 0;
# endif				/* HERE_UNDERFLOW == EXIT_UNDERFLOW */
    unsigned block, size, *state, wait_e1f4 = 0;
#if HERE_SLEEP == Mylene0fa_SLEEP
    unsigned delay = 0;
#else
# define delay				context_data->delay
#endif				/* HERE_SLEEP == Mylene0fa_SLEEP */
    void *audio, *xaudio;

    audio = context_data->audio;
    block = context_data->block;
    size = context_data->size;
    state = context_data->state;
    xaudio = context_data->xaudio;

    while (1) {
	if (wait_e1f4) {
	    usleep(wait_e1f4);
	}

	{
	    struct state_type *head_state;

	    head_state = context_data->head_state;
	    if (!head_state) {
		break;
	    } else {
		if (~head_state->e0fa_index) {
		    status = Mylene0fa_index_stream(context_data, &wait_e1f4);
		    if (status) {
			break;
		    } else {
			if (!~head_state->e0fa_index) {
			    block = context_data->block;
#if HERE_SLEEP == Mylene0fa_SLEEP
			    delay = context_data->delay;
#endif				/* HERE_SLEEP == Mylene0fa_SLEEP */
			    size = context_data->size;
			    xaudio = context_data->xaudio;
#if HERE_UNDERFLOW == EXIT_UNDERFLOW
			    stream = 1;
#endif				/* HERE_UNDERFLOW == EXIT_UNDERFLOW */
			    wait_e1f4 = 0;
			}
		    }
		} else {
		    unsigned e0fa_index, post_index;

		    e0fa_index = context_data->e0fa_index;
		    post_index = head_state->post_index;
		    if (e0fa_index == post_index) {
#if WAIT_SLEEP == libx1f4i2_SLEEP
			status = Mylene0fa_flush_stream(context_data, 0);
#else
			status = Mylene0fa_class_stream
			    (context_data, &wait_e1f4);
#endif				/* WAIT_SLEEP == libx1f4i2_SLEEP */
			if (status) {
			    break;
			}
		    } else {
			unsigned cache;

			if (~post_index) {
			    cache = post_index;
			} else {
			    cache = state[0];
			}
			cache -= e0fa_index;
			if (size < cache) {
			    cache += size;
			}
			if (cache < block) {
			    if (~post_index) {
				status = Mylene0fa_merge_stream(context_data);
				if (status) {
				    break;
				} else {
#if HERE_UNDERFLOW == EXIT_UNDERFLOW
				    stream = 0;
#endif				/* HERE_UNDERFLOW == EXIT_UNDERFLOW */
				    wait_e1f4 = 0;
				}
			    } else {
#if HERE_UNDERFLOW == WAIT_UNDERFLOW
				wait_e1f4 = 65536;
#else
				if (!stream) {
				    status =
					Mylene0fa_class_device(context_data);
				    if (!status) {
					wait_e1f4 = delay;
				    } else {
					Mylene0fa_shift_device
					    (context_data,
					     Mylene0fa_flow_error);
					break;
				    }
				} else {
				    wait_e1f4 = 65536;
				}
#endif				/* HERE_UNDERFLOW == WAIT_UNDERFLOW */
			    }
			} else {
#if !__Mylene0fa_LINE_INTER__
			    unsigned first;
#endif				/* !__Mylene0fa_LINE_INTER__ */

#if !__Mylene0fa_LINE_INTER__
			    first = size - e0fa_index;
			    if (first < block) {
				status = Mylene0fa_catch_stream(context_data);
			    } else {
				if (cache < first) {
				    first = cache;
				}
#else
# define first				cache
#endif				/* !__Mylene0fa_LINE_INTER__ */
				status = Mylene0fa_play_xaudio
				    (xaudio, (char *) audio + e0fa_index,
				     first, context_data->atime);
#if !__Mylene0fa_LINE_INTER__
			    }
#else
# undef first
#endif				/* !__Mylene0fa_LINE_INTER__ */
			    if (status < 0) {
				Mylene0fa_shift_device
				    (context_data, status << 4);
				break;
			    } else {
				if (!status) {
#if HERE_SLEEP == libx1f4i2_SLEEP
				    status = -1;
				    break;
#else
				    wait_e1f4 = delay >> 1;
#endif				/* HERE_SLEEP == libx1f4i2_SLEEP */
				} else {
				    if (!context_data->miss_stamp) {
					Mylene0fa_stamp_mylene(context_data);
				    }

				    UPDATE_LOSS(context_data, size, state,
						status);
				    e0fa_index += status;
				    if (e0fa_index < size) {
				    } else {
					e0fa_index -= size;
				    }
				    context_data->e0fa_index = e0fa_index;
				    context_data->state[Mylene0fa_REAL_STATE] =
					e0fa_index;
#if HERE_UNDERFLOW == EXIT_UNDERFLOW
				    stream = 0;
#endif				/* HERE_UNDERFLOW == EXIT_UNDERFLOW */
#if HERE_SLEEP == libx1f4i2_SLEEP
				    wait_e1f4 = 0;
#else
				    wait_e1f4 = delay;
#endif				/* HERE_SLEEP == libx1f4i2_SLEEP */
				}
			    }
			}
		    }
		}
	    }
	}
    }

    return status;
}
