/*
 * logique.2.c
 * Copyright (C) 2006-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <errno.h>
#include <libx1f4i2.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/time.h>

#include <logique.0.h>
#include <logique.1.h>
#include <logique.3.h>

#define __debug_timestamp__		0

#if __debug_timestamp__
# include <stdio.h>
#endif				/* __debug_timestamp__ */

#define C1_SECONDS(time)		((time)->tv_sec)
#define C6_SECONDS(time)		((time)->tv_usec)

#define G1_SECONDS(time)		((time).tv_sec)
#define G6_SECONDS(time)		((time).tv_usec)

#if __debug_timestamp__
# if defined __GNUC__
#  define STAMP_DEBUG(format, args...) \
    do {								      \
	fprintf(stderr, "Mylene0fa.ST" format, ## args);		      \
    } while (0)
#  define WRITE_DEBUG(format, args...) \
    do {								      \
	fprintf(stderr, "Mylene0fa.WX" format, ## args);		      \
    } while (0)
# else
#  define STAMP_DEBUG			debug
#  define WRITE_DEBUG			debug
# endif				/* __GNUC__ */
#endif				/* __debug_timestamp__ */

#if WORDS_BIGENDIAN
# define CARDINAL(data, base) \
    ((data)[(base) + 0] << 030 | (data)[(base) + 1] << 020		      \
     | (data)[(base) + 2] << 010 | (data)[(base) + 3])
#else
# define CARDINAL(data, base) \
    ((data)[(base) + 0] | (data)[(base) + 1] << 010			      \
     | (data)[(base) + 2] << 020 | (data)[(base) + 3] << 030)
#endif				/* WORDS_BIGENDIAN */

#if __debug_timestamp__
# define CARDinAL(buffer, base) \
    ((buffer)[(base) + 0] << 030 | (buffer)[(base) + 1] << 020		      \
     | (buffer)[(base) + 2] << 010 | (buffer)[(base) + 3] << 000)
#endif				/* __debug_timestamp__ */

#if __debug_timestamp__
# define CARDiNAL(buffer, base) \
    ((unsigned) CARDinAL(buffer, base))
#endif				/* __debug_timestamp__ */

#if 1
# define SET_CARDINAL(buffer, base, card) \
    {									      \
	(buffer)[(base) + 0] = (card) >> 030;				      \
	(buffer)[(base) + 1] = (card) >> 020;				      \
	(buffer)[(base) + 2] = (card) >> 010;				      \
	(buffer)[(base) + 3] = (card) >> 000;				      \
    }
#endif				/* 1 */

#if 1
# define SET_CARDiNAL(buffer, base, card) \
    {									      \
	unsigned cardinal;						      \
									      \
	cardinal = card;						      \
	SET_CARDINAL(buffer, base, cardinal);				      \
    }
#endif				/* 1 */

int
Mylene0fa_break_mylene(struct context_state *context_data, int *clear,
		       int state)
{
    int clear_mylene = 1, status;

    if (state == 0002) {
	context_data->state[Mylene0fa_B1st_STATE] ^= Mylene0fa_B1st_PLAYING;

	if (context_data->detached) {
	    context_data->detached ^= 0x1f4;

	    clear_mylene = 0;

	    status = Mylene0fa_class_mylene(context_data);
	    if (status) {
		clear_mylene = 1;
	    }
	} else {
	    struct state_type *head_state;

	    context_data->paused ^= 0x1f4;

	    clear_mylene = 0;

	    status = 0;

	    head_state = context_data->head_state;
	    if (!head_state) {
	    } else {
		if (~head_state->e0fa_index) {
		} else if (!context_data->paused) {
		} else {
		    status = Mylene0fa_reset_stream(context_data, 0);
		    if (status) {
			clear_mylene = 1;
		    }
		}
	    }

	    if (!status) {
		status = Mylene0fa_class_mylene(context_data);
		if (status) {
		    clear_mylene = 1;
		}
	    }
	}
    } else if ((state & ~1) == 0014) {
	if (context_data->detached) {
	    context_data->detached = 0x800 | (0x1f4 & -(state & 1));

	    if (state == 0014) {
		context_data->state[Mylene0fa_B1st_STATE] |=
		    Mylene0fa_B1st_PLAYING;
	    } else {
		context_data->state[Mylene0fa_B1st_STATE] &=
		    ~Mylene0fa_B1st_PLAYING;
	    }

	    clear_mylene = 0;

	    status = Mylene0fa_class_mylene(context_data);
	    if (status) {
		clear_mylene = 1;
	    }
	} else if (context_data->paused ^ (0x1f4 & -(state & 1))) {
	    status = Mylene0fa_break_mylene(context_data, &clear_mylene, 0002);
	} else {
	    clear_mylene = 0;

	    status = 0;
	}
    } else if ((state & ~1) == 0040) {
	if ((context_data->detached ^ ~state << 11) & 0x800) {
	    clear_mylene = 0;

	    status = 0;

	    if (state == 0041) {
		status = Mylene0fa_setup_device(context_data);
		if (status) {
		    clear_mylene = 1;
		} else {
		    status = Mylene0fa_class_mylene(context_data);
		    if (status) {
			clear_mylene = 1;
		    } else {
			context_data->paused = context_data->detached & 0x1f4;
			context_data->detached = 0;
		    }
		}
	    } else {
		do {
		    struct state_type *head_state;

		    head_state = context_data->head_state;
		    if (!head_state) {
		    } else {
			if (~head_state->e0fa_index) {
			} else {
			    int do_stamp;

			    do_stamp = head_state->do_stamp;
			    if (context_data->paused) {
				head_state->do_stamp = 0;
			    }
			    status = Mylene0fa_reset_stream(context_data, 1);
			    if (status) {
				clear_mylene = 1;
			    } else {
				head_state->do_stamp = do_stamp;
			    }

			    break;
			}
		    }

		    {
			if (context_data->xaudio) {
			    status = Mylene0fa_clear_device(context_data);
			    if (status) {
				clear_mylene = 1;
			    }
			}
		    }
		} while (0);

		if (!status) {
		    status = Mylene0fa_class_mylene(context_data);
		    if (status) {
			clear_mylene = 1;
		    } else {
			context_data->detached = 0x800 | context_data->paused;
			context_data->paused = 0x1f4;
		    }
		}
	    }
	} else {
	    clear_mylene = 0;

	    status = 0;
	}
    } else if (state == 0177) {
	if (!context_data->head_state) {
	    status = 0;
	} else if (context_data->paused) {
	    status = 0;
	} else {
#if __SYNCRONIZE_STREAMS__
	    status = 1;
	    /*
	     * ...
	     */
#else
	    status = Mylene0fa_merge_device(context_data);
#endif				/* __SYNCRONIZE_STREAMS__ */
	}
    } else if (state == 0400) {
	if (!context_data->head_state) {
	    status = 0;
	} else {
	    status = Mylene0fa_flush_device(context_data, 1);
	}
    } else if (state == 0401) {
	struct state_type *head_state;

	clear_mylene = 0;

	status = 0;

	head_state = context_data->head_state;
	if (!head_state) {
	} else {
	    if (~head_state->e0fa_index) {
		if (context_data->paused) {
		    status = Mylene0fa_space_stream(context_data);
		    if (status) {
			clear_mylene = 1;
		    }
		} else {
		    head_state->post_index = head_state->e0fa_index;
		}
	    } else {
		status = Mylene0fa_flush_stream(context_data, 1);
		if (status) {
		    clear_mylene = 1;
		}
	    }
	}
    } else {
	status = state;
    }

    *clear = clear_mylene;

    return status;
}


int
Mylene0fa_class_mylene(struct context_state *context_data)
{
    unsigned delete;
    unsigned char c;

    delete = context_data->state[Mylene0fa_B1st_STATE];
    if (delete & Mylene0fa_B1st_DELAYED) {
	if (delete & Mylene0fa_B1st_PLAYING) {
	    c = 0 | 2 | 4;
	} else {
	    c = 1 | 2 | 4;
	}
    } else {
	if (delete & Mylene0fa_B1st_PLAYING) {
	    c = 0 | 0 | 4;
	} else {
	    c = 1 | 0 | 4;
	}
    }

    return Mylene0fa_queue_mylene(context_data, 1, &c);
}


int
Mylene0fa_pause_mylene(struct context_state *context_data)
{
    struct timeval *atime;
    unsigned char stamp[010];

    context_data->miss_stamp = 0;

    atime = context_data->atime;
    SET_CARDiNAL(stamp, 000, C1_SECONDS(atime));
    SET_CARDiNAL(stamp, 004, C6_SECONDS(atime));

#if __debug_timestamp__
    WRITE_DEBUG(": %2x %2x %2x %2x %2x %2x %2x %2x\n", stamp[000] | 0100,
		stamp[001],
		stamp[002],
		stamp[003],
		stamp[004],
		stamp[005],
		stamp[006],
		stamp[007]);
    STAMP_DEBUG(":  system:    %02u:%02u:%02u:%02u\n",
		CARDiNAL(stamp, 000) / 3600 % 24,
		CARDiNAL(stamp, 000) / 60 % 60,
		CARDiNAL(stamp, 000) % 60,
		CARDiNAL(stamp, 004) * 60 / 1000000);
#endif				/* __debug_timestamp__ */

    stamp[0] |= 0100;

    return Mylene0fa_queue_mylene(context_data, 010, stamp);
}


int
Mylene0fa_queue_mylene(struct context_state *context_data, unsigned size,
		       void *data)
{
    int status = 1;

    do {
	if (context_data->output_size - context_data->outgoing < size) {
	    void *swap;

	    swap = realloc
		(context_data->output, context_data->output_size + 512);
	    if (!swap) {
		break;
	    }

	    context_data->output = swap;
	    context_data->output_size += 512;
	}

	memcpy((char *) context_data->output + context_data->outgoing, data,
	       size);
	context_data->outgoing += size;

	status = 0;
    } while (0);

    return status;
}


#if !__Mylene0fa_KSET_INTER__
int
Mylene0fa_stamp_mylene(struct context_state *context_data)
{
    struct timeval *atime, ctime;
    unsigned char stamp[020];

    context_data->miss_stamp = 1;

    atime = context_data->atime;
    x1f4_eset_xaudio(context_data->xaudio, atime);
    SET_CARDiNAL(stamp, 000, C1_SECONDS(atime));
    SET_CARDiNAL(stamp, 004, C6_SECONDS(atime));
    gettimeofday(&ctime, NULL);
    SET_CARDiNAL(stamp, 010, G1_SECONDS(ctime));
    SET_CARDiNAL(stamp, 014, G6_SECONDS(ctime));

# if __debug_timestamp__
    WRITE_DEBUG(": %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %"
		"2x %2x\n", stamp[000] | 0040,
		stamp[001],
		stamp[002],
		stamp[003],
		stamp[004],
		stamp[005],
		stamp[006],
		stamp[007],
		stamp[010],
		stamp[011],
		stamp[012],
		stamp[013],
		stamp[014],
		stamp[015],
		stamp[016],
		stamp[017]);
    STAMP_DEBUG(":  system:    %02u:%02u:%02u:%02u\n",
		CARDiNAL(stamp, 000) / 3600 % 24,
		CARDiNAL(stamp, 000) / 60 % 60,
		CARDiNAL(stamp, 000) % 60,
		CARDiNAL(stamp, 004) * 60 / 1000000);
    STAMP_DEBUG(":   clock:    %02u:%02u:%02u:%02u\n",
		CARDiNAL(stamp, 010) / 3600 % 24,
		CARDiNAL(stamp, 010) / 60 % 60,
		CARDiNAL(stamp, 010) % 60,
		CARDiNAL(stamp, 014) * 60 / 1000000);
# endif				/* __debug_timestamp__ */

    stamp[0] |= 0040;

    return Mylene0fa_queue_mylene(context_data, 020, stamp);
}
#endif				/* !__Mylene0fa_KSET_INTER__ */


#if __Mylene0fa_KSET_INTER__
int
Mylene0fa_stamp_mylene(struct context_state *context_data)
{
    int status;
    struct timeval atime, ctime;

    status = x1f4_kset_xaudio(context_data->xaudio, &atime, &ctime);
    if (status) {
	status = 0;
    } else {
	unsigned char stamp[020];

	context_data->miss_stamp = 1;

	SET_CARDiNAL(stamp, 000, G1_SECONDS(atime));
	SET_CARDiNAL(stamp, 004, G6_SECONDS(atime));
	SET_CARDiNAL(stamp, 010, G1_SECONDS(ctime));
	SET_CARDiNAL(stamp, 014, G6_SECONDS(ctime));

# if __debug_timestamp__
	WRITE_DEBUG(": %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %"
		    "2x %2x %2x\n", stamp[000] | 0040,
		    stamp[001],
		    stamp[002],
		    stamp[003],
		    stamp[004],
		    stamp[005],
		    stamp[006],
		    stamp[007],
		    stamp[010],
		    stamp[011],
		    stamp[012],
		    stamp[013],
		    stamp[014],
		    stamp[015],
		    stamp[016],
		    stamp[017]);
	STAMP_DEBUG(":  system:    %02u:%02u:%02u:%02u\n",
		    CARDiNAL(stamp, 000) / 3600 % 24,
		    CARDiNAL(stamp, 000) / 60 % 60,
		    CARDiNAL(stamp, 000) % 60,
		    CARDiNAL(stamp, 004) * 60 / 1000000);
	STAMP_DEBUG(":   clock:    %02u:%02u:%02u:%02u\n",
		    CARDiNAL(stamp, 010) / 3600 % 24,
		    CARDiNAL(stamp, 010) / 60 % 60,
		    CARDiNAL(stamp, 010) % 60,
		    CARDiNAL(stamp, 014) * 60 / 1000000);
# endif				/* __debug_timestamp__ */

	stamp[0] |= 0040;

	status = Mylene0fa_queue_mylene(context_data, 020, stamp);
    }

    return status;
}
#endif				/* __Mylene0fa_KSET_INTER__ */


int
Mylene0fa_trans_mylene(struct context_state *context_data)
{
    int do_skip = 1, status;
    unsigned e1f4_size;
    unsigned char *e1f4_data;

    e1f4_data = context_data->e1f4_data;
    e1f4_size = context_data->e1f4_size;

    if (!context_data->pending_1f4) {
	status = read(STDIN_FILENO, e1f4_data + e1f4_size, 0x40 - e1f4_size);
	if (status < 1) {
	    if (status == -1) {
		if (errno == EINTR) {
		    status = 0;
		} else {
		    status = 1;
		    Mylene0fa_shift_device(context_data, Mylene0fa_read_error);
		}
	    } else {
		status = 1;
		Mylene0fa_shift_device(context_data, Mylene0fa_miss_error);
	    }
	} else {
	    e1f4_size += status;
	    status = 0;
	    do_skip = 0;
	}
    } else {
	context_data->pending_1f4 = 0;
	status = 0;
	do_skip = 0;
    }

    if (!do_skip) {
	while (e1f4_size) {
	    int c;

	    c = e1f4_data[0];
	    if (c < 128) {
		int clear = 0;

		if (!c) {
		    status = 0400;
		    break;
		} else if (c == 0001) {
		    clear = 1;
		    status = 0401;
		} else if (c == 0002 || c == 0014 || c == 0015 || c == 0040
			   || c == 0041) {
		    clear = 1;
		    status = c;
		} else if (c == 0177) {
		    status = 0177;
		    break;
		} else {
		    status = 1;
		    Mylene0fa_shift_device(context_data, Mylene0fa_data_error);
		    break;
		}

		e1f4_size--;
		if (e1f4_size) {
		    memmove(e1f4_data, e1f4_data + 1, e1f4_size);
		}
		if (clear) {
		    if (e1f4_size) {
			context_data->pending_1f4 = 1;
		    }

		    break;
		}
	    } else {
		if (c == 0200) {
		    if (007 < e1f4_size) {
			struct state_type *state_data;

			state_data = context_data->tail_state;
			if (!state_data) {
			    status = 1;
			    Mylene0fa_shift_device
				(context_data, Mylene0fa_tail_error);
			    break;
			} else if (~state_data->post_index) {
			    status = 1;
			    Mylene0fa_shift_device
				(context_data, Mylene0fa_post_error);
			    break;
			} else {
			    state_data->post_index = CARDINAL(e1f4_data, 4);

			    e1f4_size -= 010;
			    if (e1f4_size) {
				memmove(e1f4_data, e1f4_data + 010, e1f4_size);
#if __SYNCRONIZE_STREAMS__
				context_data->pending_1f4 = 1;
#endif				/* __SYNCRONIZE_STREAMS__ */
			    }

#if __Mylene0fa_LINE_INTER__
			    if (~state_data->e0fa_index) {
			    } else {
				Mylene0fa_trail_stream(context_data);
			    }
#endif				/* __Mylene0fa_LINE_INTER__ */

#if __SYNCRONIZE_STREAMS__
			    break;
#endif				/* __SYNCRONIZE_STREAMS__ */
			}
		    } else {
			break;
		    }
		} else if (c == 0201) {
#if __SYNCRONIZE_STREAMS__
		    if (context_data->head_state) {
			status = 1;
			Mylene0fa_shift_device
			    (context_data, Mylene0fa_head_error);
			/*
			 * ...
			 */
			break;
		    }
#endif				/* __SYNCRONIZE_STREAMS__ */

		    if (027 < e1f4_size) {
			struct state_type *state_data;

			state_data = (struct state_type *)
			     malloc(sizeof(struct state_type));
			if (!state_data) {
			    status = 1;
			    break;
			}

			state_data->bias_0 = CARDINAL(e1f4_data, 16);
			state_data->bias_1 = CARDINAL(e1f4_data, 20);
			state_data->channels = (e1f4_data[3] & 1) + 1;
			state_data->do_stamp = e1f4_data[3] & 128;
			state_data->e0fa_index = CARDINAL(e1f4_data, 4);
			state_data->post_index = ~0;
			state_data->size = CARDINAL(e1f4_data, 8);
			state_data->sampling_frequency =
			    e1f4_data[1] << 010 | e1f4_data[2];
			state_data->state_data = NULL;
#if !__SYNCRONIZE_STREAMS__
			if (context_data->head_state) {
			    context_data->tail_state->state_data = state_data;
			} else {
#endif				/* !__SYNCRONIZE_STREAMS__ */
			    context_data->head_state = state_data;
#if !__SYNCRONIZE_STREAMS__
			}
#endif				/* !__SYNCRONIZE_STREAMS__ */
			context_data->tail_state = state_data;

			e1f4_data[0] = 3;
			if (Mylene0fa_queue_mylene
			    (context_data, 1, e1f4_data)) {
			    status = 1;
			    break;
			}

#if !__SYNCRONIZE_STREAMS__
			/*
			 * should reset atime, but cannot decide in what
			 * conditions...
			 */
			if (0) {
#elif 0
			}
#else
			if (1) {
#endif				/* !__SYNCRONIZE_STREAMS__ */
			    struct timeval *atime;

			    atime = context_data->atime;
			    C1_SECONDS(atime) = 0;
			    C6_SECONDS(atime) = 0;
			}

			e1f4_size -= 030;
			if (e1f4_size) {
			    memmove(e1f4_data, e1f4_data + 030, e1f4_size);
			}
		    } else {
			break;
		    }
		} else {
		    status = 1;
		    Mylene0fa_shift_device(context_data, Mylene0fa_data_error);
		    break;
		}
	    }
	}
    }

    context_data->e1f4_size = e1f4_size;

    return status;
}


int
Mylene0fa_write_mylene(struct context_state *context_data)
{
    int status;

    status =
	write(STDOUT_FILENO, context_data->output, context_data->outgoing);
    if (status < 0) {
	if (errno == EINTR) {
	    status = 0;
	}
    } else {
	if (status) {
	    if (status == context_data->outgoing) {
		context_data->outgoing = 0;
	    } else {
		context_data->outgoing -= status;
		memmove(context_data->output, context_data->output + status,
			context_data->outgoing);
	    }

	    status = 0;
	}
    }

    return status;
}
