/*
 * logique.3.c
 * Copyright (C) 2006-2012, 2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <libx1f4i2.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#if HERE_UNDERFLOW == EXIT_UNDERFLOW
# include <sys/time.h>
#endif				/* HERE_UNDERFLOW == EXIT_UNDERFLOW */

#include <logique.0.h>
#include <logique.1.h>
#include <logique.2.h>
#include <logique.3.h>

#if WAIT_SLEEP == Mylene0fa_SLEEP
# define CLASS_MISS			256
# define CLASS_WAIT			524288
#endif				/* WAIT_SLEEP == Mylene0fa_SLEEP */

#if WAIT_SLEEP == Mylene0fa_SLEEP
/*
 * define small enough for fine progress report
 */
# undef CLASS_WAIT
# define CLASS_WAIT			65536
#endif				/* WAIT_SLEEP == Mylene0fa_SLEEP */

#define C1_SECONDS(time)		((time)->tv_sec)
#define C6_SECONDS(time)		((time)->tv_usec)

#define G1_SECONDS(time)		((time).tv_sec)
#define G6_SECONDS(time)		((time).tv_usec)

#if SIZEOF_UNSIGNED_LONG < 8
typedef double ACCELERATION;
#else
typedef unsigned long ACCELERATION;
#endif				/* SIZEOF_UNSIGNED_LONG < 8 */

#if !__Mylene0fa_LINE_INTER__
int
Mylene0fa_catch_stream(struct context_state *context_data)
{
    void *audio, *data;
    unsigned block, e0fa_index, first;

    audio = context_data->audio;
    block = context_data->block;
    data = context_data->segment;
    e0fa_index = context_data->e0fa_index;
    first = context_data->size - e0fa_index;
    memcpy(data, (char *) audio + e0fa_index, first);
    memcpy(data + first, audio, block - first);

    return Mylene0fa_play_xaudio
	(context_data->xaudio, data, block, context_data->atime);
}
#endif				/* !__Mylene0fa_LINE_INTER__ */


#if HERE_UNDERFLOW == EXIT_UNDERFLOW
int
Mylene0fa_class_device(struct context_state *context_data)
{
    int false;
    struct timeval down;

    x1f4_down_xaudio(context_data->xaudio, &down);
    if (down.tv_sec * 1000000 + down.tv_usec
	< context_data->delay * PANIC_THRESHOLD) {
	false = 1;
    } else {
	false = 0;
    }

    return false;
}
#endif				/* HERE_UNDERFLOW == EXIT_UNDERFLOW */


#if WAIT_SLEEP == Mylene0fa_SLEEP
int
Mylene0fa_class_stream(struct context_state *context_data, unsigned *wait)
{
    int status = 0;
    unsigned second_a, second_f;

    /*
     * second_a/second_f should have been set to whatever x1f4_down_xaudio is
     * outputting, only OSS is broken (again) and this time libx1f4i2/libxaudio
     * won't cover it...
     */

    second_f = context_data->second_f;
    if (!~second_f) {
	struct timeval down, left;
	unsigned system_a, system_f;

	x1f4_down_xaudio(context_data->xaudio, &left);

	gettimeofday(&down, NULL);

	second_a = G1_SECONDS(left);
	second_f = G6_SECONDS(left);

	system_a = second_a + G1_SECONDS(down);
	system_f = second_f + G6_SECONDS(down);
	if (system_f < 1000000) {
	} else {
	    system_f -= 1000000;
	    system_a++;
	}

	context_data->second_a = system_a;
	context_data->second_f = system_f;

	context_data->violet_a = second_a;
	context_data->violet_f = second_f;
    } else {
	struct timeval down;

	gettimeofday(&down, NULL);

	second_a = context_data->second_a;

	if (second_a < G1_SECONDS(down)) {
	    second_a = 0;
	    second_f = 0;
	} else {
	    do {
		second_a -= G1_SECONDS(down);

		if (second_f < G6_SECONDS(down)) {
		    if (second_a) {
		    } else {
			second_f = 0;
			break;
		    }

		    second_f += 1000000;
		    second_a--;
		}

		second_f -= G6_SECONDS(down);
	    } while (0);
	    if (second_a | second_f) {
		struct timeval *atime;
		unsigned violet_a, violet_f;

		violet_a = context_data->violet_a;
		violet_f = context_data->violet_f;

		violet_a -= second_a;
		if (violet_f < second_f) {
		    violet_f += 1000000;
		    violet_a--;
		}

		violet_f -= second_f;

		atime = context_data->atime;

		violet_a += C1_SECONDS(atime);
		violet_f += C6_SECONDS(atime);
		if (violet_f < 1000000) {
		} else {
		    violet_f -= 1000000;
		    violet_a++;
		}

		C1_SECONDS(atime) = violet_a;
		C6_SECONDS(atime) = violet_f;

		context_data->violet_a = second_a;
		context_data->violet_f = second_f;
	    }
	}
    }

    if (second_a) {
	*wait = CLASS_WAIT;
    } else {
	if (CLASS_MISS < second_f) {
	    if (second_f < CLASS_WAIT) {
		*wait = second_f - CLASS_MISS;
	    } else {
		*wait = CLASS_WAIT;
	    }
	} else {
# if 0
	    status = Mylene0fa_flush_stream(context_data, 1);
# else
	    int excess;

	    status = Mylene0fa_clear_stream(context_data);

	    excess = x1f4_flat_xaudio(context_data->xaudio);
	    if (excess) {
		Mylene0fa_shift_device(context_data, status << 4);
	    } else {
		context_data->down = 1;
	    }
	    if (status) {
	    } else {
		status = excess;
	    }
	    excess = x1f4_cast_xaudio
		(context_data->xaudio, context_data->atime);
	    if (excess) {
		Mylene0fa_shift_device(context_data, status << 4);
	    }
	    if (status) {
	    } else {
		status = excess;
	    }
	    if (status) {
	    } else {
		status = Mylene0fa_shift_stream(context_data);
	    }
# endif				/* 0 */
	}
    }

    return status;
}
#endif				/* WAIT_SLEEP == Mylene0fa_SLEEP */


int
Mylene0fa_clear_device(struct context_state *context_data)
{
    int status;

    status = x1f4_fini_xaudio(&context_data->xaudio, 1);
    if (status) {
	Mylene0fa_shift_device(context_data, status << 4);
    }

    context_data->xaudio = NULL;

    context_data->state[Mylene0fa_B1st_STATE] |= Mylene0fa_B1st_DELAYED;

    return status;
}


int
Mylene0fa_clear_stream(struct context_state *context_data)
{
    int status = 0;
    struct state_type *head_state;

#if !__Mylene0fa_LINE_INTER__
    if (context_data->segment) {
	free(context_data->segment);
	context_data->segment = NULL;
    }
#endif				/* !__Mylene0fa_LINE_INTER__ */

    head_state = context_data->head_state;

    if (context_data->tail_state == head_state) {
	context_data->tail_state = NULL;
	context_data->head_state = NULL;
    } else {
	context_data->head_state = head_state->state_data;
    }

    if (context_data->miss_stamp) {
	if (head_state->do_stamp) {
	    if (~head_state->e0fa_index) {
	    } else {
		status = Mylene0fa_pause_mylene(context_data);
	    }
	}
    }

    free(head_state);

    return status;
}


int
Mylene0fa_flush_device(struct context_state *context_data, int now)
{
    int excess, status;

    status = Mylene0fa_clear_stream(context_data);

    excess = x1f4_clip_xaudio(context_data->xaudio, now, context_data->atime);
    if (excess) {
	Mylene0fa_shift_device(context_data, excess << 4);
    } else {
	context_data->down = 1;
    }
    if (status) {
    } else {
	status = excess;
    }

    return status;
}


int
Mylene0fa_flush_stream(struct context_state *context_data, int now)
{
    int status;

    status = Mylene0fa_flush_device(context_data, now);
    if (status) {
    } else {
	status = Mylene0fa_shift_stream(context_data);
    }

    return status;
}


int
Mylene0fa_index_stream(struct context_state *context_data, unsigned *wait)
{
    int status;
    unsigned latency;

    latency = context_data->latency;
    if (!latency) {
	status = Mylene0fa_setup_stream(context_data);
    } else {
	struct state_type *head_state;
	unsigned cache, e0fa_index, post_index, speed;

	head_state = context_data->head_state;

	e0fa_index = head_state->e0fa_index;

	post_index = head_state->post_index;
	if (~post_index) {
	    cache = post_index;
	} else {
	    cache = context_data->state[0];
	}
	cache -= e0fa_index;
	if (head_state->size < cache) {
	    cache += head_state->size;
	}

	speed = head_state->sampling_frequency * head_state->channels << 1;

	do {
	    ACCELERATION major, minor;

	    major = (ACCELERATION) cache * 1000000;
	    minor = (ACCELERATION) latency * speed;
	    if (major < minor) {
		if ((cache << 1) < head_state->size) {
		    if (~head_state->post_index) {
		    } else {
			status = 0;
			*wait = 4096;
			context_data->state[1] = e0fa_index;
			context_data->state[Mylene0fa_REAL_STATE] = e0fa_index;
			break;
		    }
		} else {
		}
	    } else {
	    }

	    status = Mylene0fa_setup_stream(context_data);
	} while (0);
    }

    return status;
}


int
Mylene0fa_merge_stream(struct context_state *context_data)
{
    int status;
    unsigned cache, e0fa_index;
#if !__Mylene0fa_LINE_INTER__
    unsigned first;
#endif				/* !__Mylene0fa_LINE_INTER__ */
    unsigned size;
    void *audio;

    e0fa_index = context_data->e0fa_index;
    audio = context_data->audio;
    size = context_data->size;

    cache = context_data->state[0] - e0fa_index;
    if (size < cache) {
	cache += size;
    }

    audio = (char *) audio + e0fa_index;

#if !__Mylene0fa_LINE_INTER__
    first = size - e0fa_index;
    if (first < cache) {
	void *data;

	data = context_data->segment;
	memcpy(data, audio, first);
	memcpy((char *) data + first, context_data->audio, cache - first);
	audio = data;
    } else {
    }
#endif				/* !__Mylene0fa_LINE_INTER__ */

    status = x1f4_xlay_xaudio
	(context_data->xaudio, audio, cache, context_data->atime);
    if (status < 0) {
	Mylene0fa_shift_device(context_data, status << 4);
    } else {
	if (!status) {
	    Mylene0fa_shift_device(context_data, status << 4);
	    status = 1;
	} else {
	    if (context_data->miss_stamp) {
	    } else {
		Mylene0fa_stamp_mylene(context_data);
	    }

	    UPDATE_LOSS(context_data, size, context_data->state, status);
	    e0fa_index += status;
	    if (e0fa_index < size) {
	    } else {
		e0fa_index -= size;
	    }

	    context_data->e0fa_index = e0fa_index;
	    context_data->state[Mylene0fa_REAL_STATE] = e0fa_index;

	    status = 0;
	}
    }

    return status;
}


int
Mylene0fa_reset_stream(struct context_state *context_data, int clear)
{
    int status;
    unsigned loss;

    status = x1f4_vset_xaudio
	(context_data->xaudio, &loss, context_data->atime);
    if (status) {
	Mylene0fa_shift_device(context_data, status << 4);
    } else {
	do {
	    if (clear) {
		int excess;

		context_data->head_state->bias_0 = 0;

		status = x1f4_side_xaudio
		    (context_data->xaudio, &context_data->head_state->bias_1);

		excess = x1f4_fini_xaudio(&context_data->xaudio, 1);
		if (status) {
		} else {
		    status = excess;
		}

		context_data->xaudio = NULL;

		context_data->state[Mylene0fa_B1st_STATE] |=
		    Mylene0fa_B1st_DELAYED;

		if (excess) {
		    Mylene0fa_shift_device(context_data, excess << 4);
		    break;
		}
	    }

	    {
		unsigned diff, e0fa_index, loss_index, slip;

		slip = context_data->head_state->channels << 1;

		e0fa_index = context_data->e0fa_index;
		loss_index = context_data->loss_index;
		diff = e0fa_index - loss_index;
		if (e0fa_index < loss_index) {
		    diff += context_data->size;
		}
		if (diff < loss) {
		    loss = diff;
		}
		if (e0fa_index < loss) {
		    e0fa_index += context_data->size;
		}
		e0fa_index -= loss;
		slip = e0fa_index % slip;
		loss += slip;
		e0fa_index -= slip;
		context_data->e0fa_index = e0fa_index;
		if (clear) {
		    context_data->head_state->bias_1 -= loss;
		    context_data->head_state->e0fa_index = e0fa_index;
		} else {
		    context_data->flush_loss += loss;
		    x1f4_pass_xaudio(context_data->xaudio, loss);
		}

		/*
		 * FEATURES
		 *
		 * should I set Mylene0fa_REAL_STATE?
		 */

		if (context_data->miss_stamp) {
		    if (context_data->head_state->do_stamp) {
			status = Mylene0fa_pause_mylene(context_data);
		    }
		}
	    }
	} while (0);
    }

    return status;
}


int
Mylene0fa_setup_device(struct context_state *context_data)
{
    int status;
    unsigned busy = 2;

    while (busy) {
	status = x1f4_open_xaudio
	    (&context_data->xaudio, context_data->device,
	     &context_data->aerror);
	if (status) {
	    /*
	     * might have been busy and becoming free.
	     */
	    busy--;
	    if (busy) {
		usleep(32768);
	    } else {
		break;
	    }
	} else {
	    break;
	}
    }
    if (status) {
	Mylene0fa_shift_device(context_data, status << 4);
    } else {
	context_data->state[Mylene0fa_B1st_STATE] &= ~Mylene0fa_B1st_DELAYED;
    }

    return status;
}


int
Mylene0fa_setup_stream(struct context_state *context_data)
{
    int status;

    do {
	void *xaudio;

	xaudio = context_data->xaudio;
	if (xaudio) {
	} else {
	    status = Mylene0fa_setup_device(context_data);
	    if (status) {
		break;
	    } else {
		xaudio = context_data->xaudio;
	    }
	}

	{
	    struct state_type *head_state;

	    head_state = context_data->head_state;

	    status = x1f4_mode_xaudio
		(xaudio, head_state->sampling_frequency, head_state->channels,
		 x1f4_S16N_AUDIO, 0);
	    if (status) {
		Mylene0fa_shift_device(context_data, status << 4);
	    } else {
		unsigned block, size;
#if !__Mylene0fa_LINE_INTER__
		void *data;
#endif				/* !__Mylene0fa_LINE_INTER__ */

		x1f4_size_xaudio(xaudio, &block);

		size = head_state->size;

#if !__Mylene0fa_LINE_INTER__
		data = malloc(block);
		if (!data) {
		    x1f4_deck_xaudio(xaudio, 0);

		    status = 1;
		} else {
#elif 0
		}
#else
		{
#endif				/* !__Mylene0fa_LINE_INTER__ */
		    unsigned e0fa_index;

		    e0fa_index = head_state->e0fa_index;

		    context_data->down = 0;

		    context_data->block = block;
		    context_data->e0fa_index = e0fa_index;
		    context_data->loss_index = e0fa_index;
#if !__Mylene0fa_LINE_INTER__
		    context_data->segment = data;
#endif				/* !__Mylene0fa_LINE_INTER__ */
		    context_data->size = size;

		    context_data->miss_stamp = head_state->do_stamp ^ 128;

		    context_data->state[1] = e0fa_index;
		    context_data->state[Mylene0fa_REAL_STATE] = e0fa_index;

#if WAIT_SLEEP == Mylene0fa_SLEEP
		    context_data->second_f = ~0;
#endif				/* WAIT_SLEEP == Mylene0fa_SLEEP */

		    head_state->e0fa_index = ~0;

		    x1f4_bias_xaudio(xaudio, head_state->bias_1);

#if HERE_SLEEP == Mylene0fa_SLEEP
		    x1f4_walk_xaudio(xaudio, &context_data->delay);
#endif				/* HERE_SLEEP == Mylene0fa_SLEEP */

#if __Mylene0fa_LINE_INTER__
		    x1f4_line_xaudio
			(xaudio, context_data->audio, e0fa_index,
			 context_data->state, size);
#endif				/* __Mylene0fa_LINE_INTER__ */

#if __Mylene0fa_LINE_INTER__
		    if (~head_state->post_index) {
			x1f4_tile_xaudio(xaudio, head_state->post_index);
		    }
#endif				/* __Mylene0fa_LINE_INTER__ */

		    x1f4_mean_xaudio(xaudio, &context_data->flush_loss);
		}
	    }
	}
    } while (0);

    return status;
}


int
Mylene0fa_shift_stream(struct context_state *context_data)
{
    int status;
    unsigned completions;

    completions = context_data->completions;
    ADVANCE_COMPLETIONS(completions);
    context_data->completions = completions;
    status = Mylene0fa_queue_mylene(context_data, 4, &completions);

    return status;
}


int
Mylene0fa_space_stream(struct context_state *context_data)
{
    int status;

    status = Mylene0fa_clear_stream(context_data);
    if (status) {
    } else {
	status = Mylene0fa_shift_stream(context_data);
	if (status) {
	} else {
	    context_data->state[1] = context_data->state[0];
	}
    }

    return status;
}


#if __Mylene0fa_LINE_INTER__
int
Mylene0fa_trail_stream(struct context_state *context_data)
{
    x1f4_tile_xaudio
	(context_data->xaudio, context_data->head_state->post_index);

    return 0;
}
#endif				/* __Mylene0fa_LINE_INTER__ */
