/*
 * fsvv.0.c
 * Copyright (C) 2008-2009, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <fspost.h>
#include <fsvv-defs.h>
#include <fsvv-inter.h>
#include <fsvv-types.h>
#include <mask.0.h>
#include <trans.h>
#include <tslist.h>

#define fs_shift(list, ts_1) \
    ((char *) ts_shift(list) + (ts_1))
#define ts_shift(list) \
    ((char *) (list) + sizeof(struct fslist_type))

static int fini_tsxx(void *);
static int init_fsxx(void *, void *, unsigned, struct screen_type *);
static int init_tsxx(void *, unsigned, struct screen_type *);

static int
fini_tsxx(void *tslist)
{
    return x1f4_flat_tslist(tslist);
}


static int
init_fsxx(void *fspost, void *list, unsigned flags,
	  struct screen_type *screen_data)
{
    struct x1f4_fspost_type delete;

    delete.link = _libx1f4l2_pipe_fslist;
    delete.text = list;

    delete.trans = screen_data->trans_data;

    return x1f4_fast_fspost(fspost, flags, &delete);
}


static int
init_tsxx(void *tslist, unsigned flags, struct screen_type *screen_data)
{
    struct x1f4_tslist_type delete;

    delete.trans = screen_data->trans_data;

    return x1f4_fast_tslist(tslist, flags, &delete);
}

int
x1f4_init_fslist(void **fslist, unsigned flags,
		 struct screen_type *screen_data)
{
    int (*link) (void *, void **, unsigned), status;
    unsigned fs_2, ts_1;
    void *list, *text;

    if (flags & TRANS_MASK) {
	struct trans_type *trans_data;

	trans_data = screen_data->trans_data;

	link = trans_data->link;
	text = trans_data->data;
    } else {
	link = _libx1f4l2_link_data;
	text = (void *) 0;
    }

    x1f4_call_fspost(&fs_2);
    x1f4_call_tslist(&ts_1);

    status = link(text, &list, sizeof(struct fslist_type) + ts_1 + fs_2);
    if (status) {
	status = LINK_ERROR;
    } else {
	struct fslist_type *fslist_data;

	fslist_data = list;

	status = init_tsxx(ts_shift(list), flags, screen_data);
	if (status) {
	} else {
	    status = init_fsxx(fs_shift(list, ts_1), list, flags, screen_data);
	    if (status) {
		fini_tsxx(ts_shift(list));
	    }
	}

	if (status) {
	    int (*free) (void *, void *);

	    if (flags & TRANS_MASK) {
		free = ((struct trans_type *) screen_data->trans_data)->free;
	    } else {
		free = _libx1f4l2_free_data;
	    }

	    free(text, list);
	} else {
	    *fslist = ts_shift(list);

	    fslist_data->link_0.base = screen_data->base;
	    fslist_data->link_0.ts_1 = ts_1;
	}
    }

    return status;
}
