/*
 * fsxx.2.c
 * Copyright (C) 2008-2009, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <string.h>

#include <fsxx-defs.h>
#include <fsxx-types.h>

#define fspost(fspost)			((struct fspost_type *) (fspost))

static int fast_post(struct fspost_type *, struct fsnode_type *, unsigned,
		     const char *, unsigned);
static int find_base(struct fslist_type *, const char *, unsigned, unsigned *);
static int head_post(struct fspost_type *, struct fsnode_type *, const char *,
		     unsigned, const char *, unsigned);
static int link_post(struct fspost_type *, struct fsnode_type *, const char *,
		     unsigned);
static int lock_post(struct fspost_type *, struct fsnode_type *, const char *,
		     unsigned, const char *, unsigned);
static int peek_data(const char **, unsigned *, const char **, unsigned *);
static int peek_none(const char **, unsigned *, const char **, unsigned *);
static int posh_post(struct fspost_type *, struct fsnode_type *, const char *,
		     unsigned);
static int push_post(struct fspost_type *, struct fsnode_type *, const char *,
		     unsigned);
static int slip_post(struct fspost_type *, struct fsnode_type *, unsigned,
		     const char *, unsigned, const char *, unsigned);

static int
fast_post(struct fspost_type *fspost_data, struct fsnode_type *fsnode_data,
	  unsigned slip, const char *data, unsigned size)
{
    int status;

    if (0) {
    } else {
	if (0) {
	} else {
	    struct fsnode_type *fsnode_post;

	    fsnode_post = fsnode_data->fslist.fslist[slip];

	    fsnode_data->miss++;

	    status = posh_post(fspost_data, fsnode_post, data, size);
	    if (status) {
		fsnode_data->miss--;
	    }
	}
    }

    return status;
}


static int
find_base(struct fslist_type *fslist_data, const char *data, unsigned size,
	  unsigned *slip)
{
    int delete;
    struct fsnode_type **fslist;
    unsigned head = 0, line;

    fslist = fslist_data->fslist;

    line = fslist_data->size - 1;

    while (1) {
	int lock;
	struct fsnode_type *fsnode_data;
	unsigned away, half, spin;

	half = (head + line) >> 1;
	fsnode_data = fslist[half];
	spin = fsnode_data->size;
	away = spin;
	if (size < away) {
	    away = size;
	}
	lock = memcmp(data, fsnode_data + 1, away);
	if (!lock) {
	    if (away < spin) {
		lock = -1;
	    } else {
		if (away < size) {
		    lock = 1;
		}
	    }
	}
	if (!lock) {
	    delete = 1;

	    *slip = half;

	    break;
	} else {
	    if (head == line) {
		delete = 0;
		if (lock < 0) {
		    *slip = half;
		} else {
		    *slip = half + 1;
		}

		break;
	    } else {
		if (0 < lock) {
		    head = half + 1;
		} else {
		    if (half != head) {
			line = half - 1;
		    } else {
			delete = 0;
			*slip = head;

			break;
		    }
		}
	    }
	}
    }

    return delete;
}


static int
head_post(struct fspost_type *fspost_data, struct fsnode_type *fsnode_data,
	  const char *base, unsigned miss, const char *data, unsigned size)
{
    int (*link) (void *, void **, unsigned), status;
    void *fslist, *fsnode, *text;

    link = fspost_data->link_m.link;
    text = fspost_data->link_m.data;

    status = link(text, &fsnode, sizeof(struct fsnode_type) + miss);
    if (status) {
	status = LINK_ERROR;
    } else {
	status = link(text, &fslist, sizeof(struct fsnode_type *));
	if (status) {
	    status = LINK_ERROR;

	    fspost_data->link_m.free(text, fsnode);
	} else {
	    struct fsnode_type **fsdata, *fsnode_post;

	    fsnode_post = fsnode;

	    fsdata = fslist;

	    fsnode_post->fslist.size = 0;
	    fsnode_post->fsnode_data = fsnode_data;
	    fsnode_post->miss = 0;
	    fsnode_post->size = miss;

	    *fsdata = fsnode_post;

	    fsnode_data->fslist.size = 1;
	    fsnode_data->fslist.fslist = fslist;
	    fsnode_data->miss++;

	    status = posh_post(fspost_data, fsnode_post, data, size);
	    if (status) {
		int (*free) (void *, void *);

		free = fspost_data->link_m.free;

		free(text, fslist);
		free(text, fsnode);

		fsnode_data->miss--;
		fsnode_data->fslist.size = 0;
	    } else {
		memcpy(fsnode_post + 1, base, miss);
	    }
	}
    }

    return status;
}


static int
link_post(struct fspost_type *fspost_data, struct fsnode_type *fsnode_data,
	  const char *data, unsigned size)
{
    int status;

    status = fspost_data->link_c.link
	(fspost_data->link_c.text, data, size, fsnode_data);
    if (!status) {
	fsnode_data->miss++;
    }

    return status;
}


static int
lock_post(struct fspost_type *fspost_data, struct fsnode_type *fsnode_data,
	  const char *base, unsigned miss, const char *data, unsigned size)
{
    int status;

    if (!fsnode_data->fslist.size) {
	if (1) {
	    status = head_post
		(fspost_data, fsnode_data, base, miss, data, size);
	}
    } else {
	unsigned slip;

	if (find_base(&fsnode_data->fslist, base, miss, &slip)) {
	    status = fast_post(fspost_data, fsnode_data, slip, data, size);
	} else {
	    status = slip_post
		(fspost_data, fsnode_data, slip, base, miss, data, size);
	}
    }

    return status;
}


static int
peek_data(const char **peek, unsigned *fast, const char **slip, unsigned *dick)
{
    const char *data;
    unsigned miss, size;

    data = *slip;
    miss = *dick;

    size = miss;

    *peek = data;

    while (size) {
	if (*data == 0x2f) {
	    break;
	} else {
	    data++;
	    size--;
	}
    }

    *fast = miss - size;

    while (size) {
	if (*data != 0x2f) {
	    break;
	} else {
	    data++;
	    size--;
	}
    }

    *dick = size;
    *slip = data;

    return 0;
}


static int
peek_none(const char **peek, unsigned *fast, const char **slip, unsigned *dick)
{
    unsigned size;

    size = *dick;

    if (!size) {
	*dick = 0;
	*fast = 0;
    } else {
	const char *data;

	data = *slip;

	if (*data != 0x2f) {
	    peek_data(peek, fast, slip, dick);
	} else {
	    data++;
	    size--;
	    while (size) {
		if (*data != 0x2f) {
		    break;
		} else {
		    data++;
		    size--;
		}
	    }

	    if (size) {
		*dick = size;
		*fast = 0;
		*slip = data;
	    } else {
		*dick = 1;
		*fast = 0;
		*slip = data - 1;
	    }
	}
    }

    return 0;
}


static int
posh_post(struct fspost_type *fspost_data, struct fsnode_type *fsnode_data,
	  const char *data, unsigned size)
{
    const char *base;
    int status;
    unsigned miss;

    peek_data(&base, &miss, &data, &size);

    if (size) {
	status = lock_post(fspost_data, fsnode_data, base, miss, data, size);
    } else {
	status = link_post(fspost_data, fsnode_data, base, miss);
    }

    return status;
}


static int
push_post(struct fspost_type *fspost_data, struct fsnode_type *fsnode_data,
	  const char *data, unsigned size)
{
    const char *base;
    int status;
    unsigned miss;

    peek_none(&base, &miss, &data, &size);

    if (size) {
	status = lock_post(fspost_data, fsnode_data, base, miss, data, size);
    } else {
	status = link_post(fspost_data, fsnode_data, base, miss);
    }

    return status;
}


static int
slip_post(struct fspost_type *fspost_data, struct fsnode_type *fsnode_data,
	  unsigned slip, const char *base, unsigned miss, const char *data,
	  unsigned size)
{
    int status;
    void *fslist, *fsnode, *text;

    text = fspost_data->link_m.data;

    status = fspost_data->link_m.link
	(text, &fsnode, sizeof(struct fsnode_type) + miss);
    if (status) {
	status = LINK_ERROR;
    } else {
	unsigned list;

	fslist = fsnode_data->fslist.fslist;

	list = fsnode_data->fslist.size;

	status = fspost_data->link_m.mode
	    (text, &fslist, sizeof(struct fsnode_type *) * (list + 1));
	if (status) {
	    status = MODE_ERROR;

	    fspost_data->link_m.free(text, fsnode);
	} else {
	    struct fsnode_type *fsnode_post;

	    fsnode_post = fsnode;

	    fsnode_post->fslist.size = 0;
	    fsnode_post->fsnode_data = fsnode_data;
	    fsnode_post->miss = 0;
	    fsnode_post->size = miss;

	    fsnode_data->fslist.size++;
	    fsnode_data->fslist.fslist = fslist;
	    fsnode_data->miss++;

	    status = posh_post(fspost_data, fsnode_post, data, size);
	    if (status) {
		int excess;

		excess = fspost_data->link_m.mode
		    (text, &fslist, sizeof(struct fsnode_type *) * list);
		fspost_data->link_m.free(text, fsnode);

		fsnode_data->miss--;
		fsnode_data->fslist.size--;
		if (!excess) {
		    fsnode_data->fslist.fslist = fslist;
		}
	    } else {
		struct fsnode_type **fsdata;

		memcpy(fsnode_post + 1, base, miss);

		fsdata = fslist;

		if (slip == list) {
		    fsdata[list] = fsnode_post;
		} else {
		    fsdata += slip;
		    memmove(fsdata + 1, fsdata,
			    (list - slip) * sizeof(struct fsnode_type *));
		    *fsdata = fsnode_post;
		}
	    }
	}
    }

    return status;
}


int
x1f4_push_fspost(void *fspost, const char *data, unsigned size)
{
    return push_post
	(fspost, &fspost(fspost)->link_a.fsmiss.fsnode, data, size);
}
