/*
 * fsxx.3.c
 * Copyright (C) 2008-2009, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <fsxx-types.h>

#define BASE_SIZE			32

#define fspost(fspost)			((struct fspost_type *) (fspost))

static int dump_link(struct fsnode_type *, unsigned, void *,
		     int (*) (void *, const char *, unsigned));
static int dump_node(struct fsnode_type *, unsigned, void *,
		     int (*) (void *, const char *, unsigned));
static int head_dump(struct fspost_type *, void *,
		     int (*) (void *, const char *, unsigned));
static int head_left(void *, int (*) (void *, const char *, unsigned),
		     unsigned);
static int miss_node(void *, int (*) (void *, const char *, unsigned),
		     unsigned);
static int type_node(char **, unsigned);

static const char digits[] = "0123456789", f_space[] = "                ";

static int
dump_link(struct fsnode_type *fsnode_data, unsigned left, void *data,
	  int (*type) (void *, const char *, unsigned))
{
    int status;

    status = head_left(data, type, left << 2);
    if (status) {
    } else {
	status = type(data, (void *) (fsnode_data + 1), fsnode_data->size);
	if (status) {
	} else {
	    status = type(data, " (", 2);
	    if (status) {
	    } else {
		status = miss_node(data, type, fsnode_data->miss);
		if (status) {
		} else {
		    status = type(data, ")\n", 2);
		    if (status) {
		    } else {
			status = dump_node(fsnode_data, left + 1, data, type);
		    }
		}
	    }
	}
    }

    return status;
}


static int
dump_node(struct fsnode_type *fsnode_data, unsigned left, void *data,
	  int (*type) (void *, const char *, unsigned))
{
    int status;
    unsigned size;

    size = fsnode_data->fslist.size;
    if (!size) {
	status = 0;
    } else {
	struct fsnode_type **fslist;

	fslist = fsnode_data->fslist.fslist;
	while (size) {
	    status = dump_link(*fslist, left, data, type);
	    if (status) {
		break;
	    } else {
		size--;
		fslist++;
	    }
	}
    }

    return status;
}


static int
head_dump(struct fspost_type *fspost_data, void *data,
	  int (*type) (void *, const char *, unsigned))
{
    int status;

    status = type(data, "-(", 2);
    if (status) {
    } else {
	status = miss_node(data, type, fspost_data->link_a.fsmiss.fsnode.miss);
	if (status) {
	} else {
	    status = type(data, ")\n", 2);
	}
    }

    return status;
}


static int
head_left(void *data, int (*this) (void *, const char *, unsigned),
	  unsigned space)
{
    int status = 0;

    while (space & ~15) {
	space -= 16;
	status = this(data, f_space, 16);
	if (status) {
	    space = 0;
	}
    }
    if (space) {
	status = this(data, f_space, space);
    }

    return status;
}


static int
miss_node(void *data, int (*this) (void *, const char *, unsigned), unsigned c)
{
    char aime[BASE_SIZE], *mode;

    mode = aime + BASE_SIZE;
    type_node(&mode, c);

    return this(data, mode, aime + BASE_SIZE - mode);
}


static int
type_node(char **output, unsigned c)
{
    char *o;

    o = *output;

    do {
	--o;
	*o = digits[c % 10];
	c /= 10;
    } while (c);

    *output = o;

    return 0;
}


int
x1f4_dump_fspost(void *fspost, void *data,
		 int (*type) (void *, const char *, unsigned))
{
    int status;

    status = head_dump(fspost, data, type);
    if (status) {
    } else {
	status = dump_node
	    (&fspost(fspost)->link_a.fsmiss.fsnode, 0, data, type);
    }

    return status;
}
