/*
 * fsxx.5.c
 * Copyright (C) 2008-2009, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>

#include <fsxx-defs.h>
#include <fsxx-types.h>

#define fsnode(fsnode)			((struct fsnode_type *) (fsnode))
#define fspost(fspost)			((struct fspost_type *) (fspost))

static int miss_link(struct fspost_type *, struct fsnode_type *,
		     struct fsnode_type *);
static int miss_post(struct fspost_type *, struct fsnode_type *,
		     struct fsnode_type *);

static int
miss_link(struct fspost_type *fspost_data, struct fsnode_type *fsnode_data,
	  struct fsnode_type *fsnode_text)
{
    int status;
    struct fsnode_type **fsmiss;
    unsigned miss, size;
    void *fslist;

    fslist = fsnode_data->fslist.fslist;

    size = fsnode_data->fslist.size;

    fsmiss = fslist;
    miss = size;
    while (1) {
	miss--;
	if (*fsmiss != fsnode_text) {
	    fsmiss++;
	} else {
	    memmove(fsmiss, fsmiss + 1, miss * sizeof(struct fsnode_type *));

	    size--;

	    fsnode_data->fslist.size = size;

	    if (!size) {
		status = fspost_data->link_m.free
		    (fspost_data->link_m.data, fslist);
		if (status) {
		    status = FREE_ERROR;
		} else {
		}
	    } else {
		status = fspost_data->link_m.mode
		    (fspost_data->link_m.data, &fslist,
		     size * sizeof(struct fsnode_type *));
		if (status) {
		    status = MODE_ERROR;
		} else {
		    fsnode_data->fslist.fslist = fslist;
		}
	    }

	    break;
	}
    }

    return status;
}


static int
miss_post(struct fspost_type *fspost_data, struct fsnode_type *fsnode_data,
	  struct fsnode_type *fsnode_text)
{
    int status;
    unsigned miss;

    miss = fsnode_data->miss;
    miss--;
    if (miss) {
	status = miss_link(fspost_data, fsnode_data, fsnode_text);
	if (status) {
	} else {
	    fsnode_data->miss = miss;
	    fsnode_data = fsnode_data->fsnode_data;
	    while (fsnode_data) {
		fsnode_data->miss--;
		fsnode_data = fsnode_data->fsnode_data;
	    }
	}
    } else {
	status = miss_post(fspost_data, fsnode_data->fsnode_data, fsnode_data);
	if (status) {
	} else {
	    status = fspost_data->link_m.free
		(fspost_data->link_m.data, fsnode_data->fslist.fslist);
	    if (status) {
		status = FREE_ERROR;
	    } else {
		status = fspost_data->link_m.free
		    (fspost_data->link_m.data, fsnode_data);
		if (status) {
		    status = FREE_ERROR;
		}
	    }
	}
    }

    return status;
}


int
x1f4_miss_fspost(void *fspost, void *fsnode)
{
    int status;
    unsigned miss;

    miss = fsnode(fsnode)->miss;
    miss--;
    if (miss) {
	struct fsnode_type *fsnode_data;

	fsnode(fsnode)->miss = miss;

	fsnode_data = fsnode(fsnode)->fsnode_data;
	while (fsnode_data) {
	    fsnode_data->miss--;
	    fsnode_data = fsnode_data->fsnode_data;
	}

	status = 0;
    } else {
	status = miss_post(fspost, fsnode(fsnode)->fsnode_data, fsnode);
	if (status) {
	} else {
	    status = fspost(fspost)->link_m.free
		(fspost(fspost)->link_m.data, fsnode);
	    if (status) {
		status = FREE_ERROR;
	    }
	}
    }

    return status;
}
