/*
 * tslist.h
 * Copyright (C) 2008-2011, 2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBx1f4_TSLIST_H__
#define __LIBx1f4_TSLIST_H__

#include <errors.h>

/*
 * attribute bit definitions
 */
#define X1f4_TSLIST_TRANS_MASK		(1u << 0x00)

/*
 * error definitions
 */
#define X1f4_TSLIST_DECK_ERROR		LIBx1f4L2_DECK_ERROR
#define X1f4_TSLIST_FREE_ERROR		LIBx1f4L2_FREE_ERROR
#define X1f4_TSLIST_LINK_ERROR		LIBx1f4L2_LINK_ERROR
#define X1f4_TSLIST_MODE_ERROR		LIBx1f4L2_MODE_ERROR
#define X1f4_TSLIST_SIZE_ERROR		LIBx1f4L2_SIZE_ERROR

typedef struct x1f4_tslist_type {
    void *trans;
} x1f4_tslist_type;

#if !defined __x1f4_trans_type_defined__
typedef struct x1f4_trans_type {
    int (*free) (void *, void *), (*link) (void *, void **, unsigned),
	(*mode) (void *, void **, unsigned);
    void *data;
} x1f4_trans_type;
#endif				/* !__x1f4_trans_type_defined__ */

#if !defined __x1f4_trans_type_defined__
# define __x1f4_trans_type_defined__
#endif				/* !__x1f4_trans_type_defined__ */

extern int x1f4_call_tslist(unsigned *);
extern int x1f4_clip_tslist
    (void *, unsigned, void *, int (*) (void *, void *));
extern int x1f4_deck_tslist(void *, int);
extern int x1f4_deux_tslist(void *, unsigned, void *, unsigned);
extern int x1f4_ever_tslist(void *, void **);
extern int x1f4_fast_tslist(void *, unsigned, struct x1f4_tslist_type *);
extern int x1f4_fgxp_tslist
    (void *, void *, unsigned, int (*) (void *, void *), unsigned *);
extern int x1f4_find_tslist
    (void *, void *, unsigned, int (*) (void *, void *), void **);
extern int x1f4_fine_tslist(void *, void *, int (*) (void *, void *));
extern int x1f4_fini_tslist(void **);
extern int x1f4_flat_tslist(void *);
extern int x1f4_flip_tslist(void *);
extern int x1f4_flow_tslist(void *);
extern int x1f4_fmxp_tslist
    (void *, void *, unsigned, int (*) (void *, void *), unsigned *);
extern int x1f4_fxxp_tslist
    (void *, void *, unsigned, int (*) (void *, void *), unsigned *);
extern int x1f4_head_tslist(void *, unsigned, void **);
extern int x1f4_high_tslist(void *);
extern int x1f4_init_tslist(void **, unsigned, struct x1f4_tslist_type *);
extern int x1f4_land_tslist
    (void *, void *, int (*) (void *, void *), unsigned, unsigned, void **);
extern int x1f4_last_tslist(void **, void *);
extern int x1f4_lead_tslist(void *, void **);
extern int x1f4_lime_tslist(void *, void *, int (*) (void *, void *));
extern int x1f4_line_tslist
    (void *, void *, int (*) (void *, unsigned *, const void *),
     int (*) (void *, void *, const void *), int (*) (void *, void *), void *);
extern int x1f4_llxp_tslist
    (void *, void *, unsigned, int (*) (void *, void *), unsigned *);
extern int x1f4_lmxp_tslist
    (void *, void *, unsigned, int (*) (void *, void *), unsigned *);
extern int x1f4_lxxp_tslist
    (void *, void *, unsigned, int (*) (void *, void *), unsigned *);
extern int x1f4_mind_tslist(void *);
extern int x1f4_miss_tslist(void *, unsigned);
extern int x1f4_mode_tslist(void *, unsigned, unsigned, void **);
extern int x1f4_move_tslist(void *, unsigned, unsigned);
extern int x1f4_next_tslist(void **, void *);
extern int x1f4_nice_tslist(void *, void *, int (*) (void *, void *));
extern int x1f4_peek_tslist(void *, unsigned, void **);
extern int x1f4_pull_tslist
    (void *, unsigned, void *, int (*) (void *, void *));
extern int x1f4_push_tslist(void *, unsigned, unsigned, void **);
extern int x1f4_rand_tslist(void *);
extern int x1f4_seek_tslist
    (void *, void *, unsigned, int (*) (void *, void *), unsigned *);
extern int x1f4_side_tslist(void *, void *);
extern int x1f4_size_tslist(void *, unsigned *);
extern int x1f4_skew_tslist(void *, void *);
extern int x1f4_slip_tslist(void *);
extern int x1f4_sort_tslist
    (void *, void *, unsigned, int (*) (void *, void *, void *));
extern int x1f4_spin_tslist(void *, unsigned, unsigned);
extern int x1f4_tail_tslist(void *, unsigned, void **);
extern int x1f4_text_tslist
    (void **, void *, int (*) (void *, unsigned *, const void *),
     int (*) (void *, void *, const void *), int (*) (void *, void *), void *);
extern int x1f4_tick_tslist
    (void *, void *, unsigned, int (*) (void *, void *), unsigned *, void **);
extern int x1f4_tile_tslist(unsigned *, void *);
extern int x1f4_turn_tslist(void *);
extern int x1f4_well_tslist(void *, void *, int (*) (void *, void *));
extern int x1f4_wind_tslist(void *, unsigned, void *, unsigned);

#endif				/* __LIBx1f4_TSLIST_H__ */
