/*
 * tsxx.o.c
 * Copyright (C) 2008-2012, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <tsxx-defs.h>
#include <tsxx-names.h>
#include <tsxx-types.h>

#define tslist(tslist)			((struct tslist_type *) (tslist))

static int copy_miss(struct tslist_type *, struct fpnode_type **,
		     struct fpnode_type *, struct fpnode_type *,
		     int (*) (void *, unsigned *, const void *),
		     int (*) (void *, void *, const void *),
		     int (*) (void *, void *), void *);
static int copy_node(struct tslist_type *, struct fpnode_type **,
		     struct fpnode_type **, struct fpnode_type *,
		     int (*) (void *, unsigned *, const void *),
		     int (*) (void *, void *, const void *),
		     int (*) (void *, void *), void *);
static int free_node(struct tslist_type *, struct fpnode_type *,
		     int (*) (void *, void *), void *);

static int
copy_miss(struct tslist_type *tslist_data, struct fpnode_type **fsnode,
	  struct fpnode_type *fpnode_copy, struct fpnode_type *fpnode_data,
	  int (*size) (void *, unsigned *, const void *),
	  int (*copy) (void *, void *, const void *),
	  int (*free) (void *, void *), void *context)
{
    int status;

    do {
	if (fpnode_data->fqnode.fbnode.bits & LAST_T) {
	    fpnode_copy->fpnode_data[0] = *fsnode;
	} else {
	    status = copy_node
		(tslist_data, fsnode, fpnode_copy->fpnode_data + 0,
		 fpnode_data->fpnode_data[0], size, copy, free, context);
	    if (status) {
		break;
	    } else {
	    }
	}

	{
	    struct fpnode_type *last_fpnode;

	    last_fpnode = *fsnode;
	    if (last_fpnode->fqnode.fbnode.bits & NEXT_T) {
		last_fpnode->fpnode_data[1] = fpnode_copy;
	    }
	}

	*fsnode = fpnode_copy;

	fpnode_copy->fqnode = fpnode_data->fqnode;

	if (fpnode_data->fqnode.fbnode.bits & NEXT_T) {
	    status = 0;
	} else {
	    status = copy_node
		(tslist_data, fsnode, fpnode_copy->fpnode_data + 1,
		 fpnode_data->fpnode_data[1], size, copy, free, context);
	    if (status) {
		if (fpnode_data->fqnode.fbnode.bits & LAST_T) {
		    free_node(tslist_data, fpnode_copy->fpnode_data[0], free,
			      context);
		}
	    } else {
	    }
	}
    } while (0);

    return status;
}


static int
copy_node(struct tslist_type *tslist_data, struct fpnode_type **fsnode,
	  struct fpnode_type **fpnode, struct fpnode_type *fpnode_data,
	  int (*size) (void *, unsigned *, const void *),
	  int (*copy) (void *, void *, const void *),
	  int (*free) (void *, void *), void *context)
{
    int status;
    unsigned down;
    void *tsnode;

    size(context, &down, fpnode_data + 1);

    status = tslist_data->link_m.link
	(tslist_data->link_m.data, &tsnode, sizeof(struct fpnode_type) + down);
    if (status) {
	status = LINK_ERROR;
    } else {
	struct fpnode_type *fpnode_copy;

	fpnode_copy = tsnode;

	status = copy(context, fpnode_copy + 1, fpnode_data + 1);
	if (status) {
	    status = CALL_ERROR;

	    tslist_data->link_m.free(tslist_data->link_m.data, tsnode);
	} else {
	    status = copy_miss
		(tslist_data, fsnode, fpnode_copy, fpnode_data, size, copy,
		 free, context);
	    if (status) {
		free(context, fpnode_copy + 1);

		tslist_data->link_m.free(tslist_data->link_m.data, tsnode);
	    } else {
		*fpnode = tsnode;
	    }
	}
    }

    return status;
}


static int
free_node(struct tslist_type *tslist_data, struct fpnode_type *fpnode_data,
	  int (*free) (void *, void *), void *context)
{
    if (fpnode_data->fqnode.fbnode.bits & LAST_T) {
    } else {
	free_node(tslist_data, fpnode_data->fpnode_data[0], free, context);
    }

    if (fpnode_data->fqnode.fbnode.bits & NEXT_T) {
    } else {
	free_node(tslist_data, fpnode_data->fpnode_data[1], free, context);
    }

    free(context, fpnode_data + 1);

    tslist_data->link_m.free(tslist_data->link_m.data, fpnode_data);

    return -1;
}


int
_libx1f4l2_rail_tslist(void *tcdata, void *tctext,
		       int (*size) (void *, unsigned *, const void *),
		       int (*copy) (void *, void *, const void *),
		       int (*free) (void *, void *), void *context)
{
    int status;
    struct fpnode_type fbnode, *fsnode_data;

    fbnode.fqnode.fbnode.bits = NEXT_T;

    fsnode_data = &fbnode;

    status = copy_node
	(tcdata, &fsnode_data,
	 tslist(tcdata)->link_a.fplist.fplink.fpnode_data + 0,
	 tslist(tctext)->link_a.fplist.fplink.fpnode_data[0], size, copy, free,
	 context);
    if (status) {
    } else {
	fbnode.fpnode_data[1]->fpnode_data[0] = NULL;
	fsnode_data->fpnode_data[1] = NULL;

	tslist(tcdata)->link_a.fplist.fplink.fqnode =
	    tslist(tctext)->link_a.fplist.fplink.fqnode;
    }

    return status;
}
