/*
 * 1f44.0.c
 * Copyright (C) 2004-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <errno.h>
#include <fcntl.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/stat.h>
#include <sys/types.h>

#include <defs.h>
#include <inter.h>
#include <lcard.h>
#include <types.h>

#define CARDINAL(buffer, base) \
    ((buffer)[(base) + 0] << 030 | (buffer)[(base) + 1] << 020		      \
     | (buffer)[(base) + 2] << 010 | (buffer)[(base) + 3] << 000)

#define DATA(x1f44v_data) \
    (((struct x1f44v_type *) (x1f44v_data))->data)
#define MISS(x1f44v_data) \
    (((struct x1f44v_type *) (x1f44v_data))->error)

static int eead_1f440(void *, void *, unsigned);
static int eead_1f441(void *, void *, unsigned);
static int get_record(struct record_type *, unsigned, lcard_type);
static int read_1f440(void *, void *, unsigned);
static int read_1f441(void *, void *, unsigned);
static int seek_1f440(void *, void *, int);
static int seek_1f441(void *, void *, int);
static int tell_1f440(void *, void *);
static int tell_1f441(void *, void *);

const struct x1f44e_type _seek_1f44_0_0[] = {
/* *INDENT-OFF* */
    {	eead_1f440,	read_1f440,	seek_1f440,	tell_1f440	},
    {	eead_1f441,	read_1f441,	seek_1f441,	tell_1f441	}
/* *INDENT-ON* */
};

static int
eead_1f440(void *x1f44v_data, void *data, unsigned size)
{
    int fd, status = 0;
    struct x1f440_type *x1f440_data;

    x1f440_data = DATA(x1f44v_data);

    fd = x1f440_data->fd;

    while (1) {
	int success;

	success = read(fd, data, size);
	if (success != -1) {
	    if (!success) {
		break;
	    } else {
		data = (char *) data + success;
		size -= success;
		status += success;
		if (!size) {
		    break;
		}
	    }
	} else {
	    if (errno == EINTR) {
	    } else {
		MISS(x1f44v_data) = READ_ERROR;
		status = -1;
		break;
	    }
	}
    }

    if (status != -1) {
	LCARD_ADD_CARD1(x1f440_data->offset, status);
    }

    return status;
}


static int
eead_1f441(void *x1f44v_data, void *data, unsigned size)
{
    int status = 0;

    do {
	int fd;
	lcard_type entry_offset, text;
	struct record_type *head_record, *record_data;
	struct x1f441_type *x1f441_data;
	unsigned count;

	x1f441_data = DATA(x1f44v_data);

	count = x1f441_data->count;
	head_record = x1f441_data->record_data;

	fd = x1f441_data->fd;

	if (fd == -1) {
	    x1f441_data->index =
		get_record(head_record, count, x1f441_data->offset);
	    record_data = head_record + x1f441_data->index;
	    fd = open(record_data->path, O_RDONLY);
	    if (fd == -1) {
		MISS(x1f44v_data) = OPEN_ERROR;
		status = -1;
		break;
	    }
	    x1f441_data->fd = fd;
	    LCARD_SET_CARD2(entry_offset, x1f441_data->offset);
	    LCARD_SUBTRACT_CARD2(entry_offset, record_data->deck);
	    {
		lcard_type file_offset;

		LCARD_SET_CARD2(file_offset, entry_offset);
		LCARD_ADD_CARD2(file_offset, record_data->head);
		if (LCARD_IS_SET(file_offset)) {
		    if (lseek(fd, LCARD_GET_CARD(file_offset), SEEK_SET)
			== -1) {
			MISS(x1f44v_data) = SEEK_ERROR;
			status = -1;
			break;
		    }
		}
	    }
	} else {
	    record_data = head_record + x1f441_data->index;
	    LCARD_SET_CARD2(entry_offset, x1f441_data->offset);
	    LCARD_SUBTRACT_CARD2(entry_offset, record_data->deck);
	}

	LCARD_SET_CARD2(text, entry_offset);
	LCARD_ADD_CARD1(text, size);
	if (!LCARD_LESS_CARD2(record_data->size, text)) {
	    {
		while (1) {
		    int success;

		    success = read(fd, data, size);
		    if (success != -1) {
			if (!success) {
			    break;
			} else {
			    data = (char *) data + success;
			    size -= success;
			    status += success;
			    if (!size) {
				break;
			    }
			}
		    } else {
			if (errno == EINTR) {
			} else {
			    MISS(x1f44v_data) = READ_ERROR;
			    status = -1;
			    break;
			}
		    }
		}

		if (status != -1) {
		    LCARD_ADD_CARD1(x1f441_data->offset, status);
		}
	    }
	} else {
	    unsigned transfer;

	    LCARD_SET_CARD2(text, record_data->size);
	    LCARD_SUBTRACT_CARD2(text, entry_offset);
	    transfer = LCARD_GET_CARD(text);
	    do {
		int excess = 0;

		size -= transfer;

		excess = 0;

		while (1) {
		    int success;

		    success = read(fd, data, transfer);
		    if (success != -1) {
			if (!success) {
			    break;
			} else {
			    data = (char *) data + success;
			    transfer -= success;
			    excess += success;
			    if (!transfer) {
				break;
			    }
			}
		    } else {
			if (errno == EINTR) {
			} else {
			    MISS(x1f44v_data) = READ_ERROR;
			    excess = -1;
			    break;
			}
		    }
		}

		if (excess == -1) {
		    status = -1;
		    break;
		} else {
		    status += excess;
		    LCARD_ADD_CARD1(x1f441_data->offset, excess);
		    if (transfer) {
			break;
		    }
		}

		if (size) {
		    if (x1f441_data->index == count - 1) {
			break;
		    }
		    excess = close(fd);
		    if (excess == -1) {
			MISS(x1f44v_data) = LOSE_ERROR;
			break;
		    }
		    record_data++;
		    x1f441_data->index++;
		    fd = open(record_data->path, O_RDONLY);
		    if (fd == -1) {
			MISS(x1f44v_data) = OPEN_ERROR;
			x1f441_data->fd = -1;
			break;
		    } else {
			x1f441_data->fd = fd;
		    }
		    if (LCARD_IS_SET(record_data->head)) {
			if (lseek(fd, LCARD_GET_CARD(record_data->head),
				  SEEK_SET) == -1) {
			    MISS(x1f44v_data) = SEEK_ERROR;
			    break;
			}
		    }
		    LCARD_SET_CARD1(text, size);
		    if (LCARD_LESS_CARD2(text, record_data->size)) {
			transfer = size;
		    } else {
			transfer = LCARD_GET_CARD(record_data->size);
		    }
		} else {
		    break;
		}
	    } while (1);
	}
    } while (0);

    return status;
}


static int
get_record(struct record_type *record_data, unsigned count, lcard_type offset)
{
    int index;

    count--;

    LCARD_ADD_CARD1(offset, 1);
    if (LCARD_LESS_CARD2(record_data[count].deck, offset)) {
	index = count;
    } else {
	unsigned finish, half = 0, start = 0;

	LCARD_SUBTRACT_CARD1(offset, 1);
	finish = count;
	while (start < finish) {
	    half = (finish + start) >> 1;
	    if (LCARD_LESS_CARD2(offset, record_data[half].deck)) {
		finish = half;
	    } else {
		start = half + 1;
	    }
	}

	if (start) {
	    start--;
	}

	index = start;
    }

    return index;
}


static int
read_1f440(void *x1f44v_data, void *data, unsigned size)
{
    int status;
    struct x1f440_type *x1f440_data;

    x1f440_data = DATA(x1f44v_data);

    status = read(x1f440_data->fd, data, size);
    if (status == -1) {
	MISS(x1f44v_data) = READ_ERROR;
    } else {
	LCARD_ADD_CARD1(x1f440_data->offset, status);
    }

    return status;
}


static int
read_1f441(void *x1f44v_data, void *data, unsigned size)
{
    int status = 0;

    do {
	int excess, fd;
	lcard_type entry_offset, text;
	struct record_type *head_record, *record_data;
	struct x1f441_type *x1f441_data;
	unsigned count;

	x1f441_data = DATA(x1f44v_data);

	count = x1f441_data->count;
	head_record = x1f441_data->record_data;

	fd = x1f441_data->fd;

	if (fd == -1) {
	    x1f441_data->index =
		get_record(head_record, count, x1f441_data->offset);
	    record_data = head_record + x1f441_data->index;
	    fd = open(record_data->path, O_RDONLY);
	    if (fd == -1) {
		MISS(x1f44v_data) = OPEN_ERROR;
		status = -1;
		break;
	    }
	    x1f441_data->fd = fd;
	    LCARD_SET_CARD2(entry_offset, x1f441_data->offset);
	    LCARD_SUBTRACT_CARD2(entry_offset, record_data->deck);
	    {
		lcard_type file_offset;

		LCARD_SET_CARD2(file_offset, entry_offset);
		LCARD_ADD_CARD2(file_offset, record_data->head);
		if (LCARD_IS_SET(file_offset)) {
		    if (lseek(fd, LCARD_GET_CARD(file_offset), SEEK_SET)
			== -1) {
			MISS(x1f44v_data) = SEEK_ERROR;
			status = -1;
			break;
		    }
		}
	    }
	} else {
	    record_data = head_record + x1f441_data->index;
	    LCARD_SET_CARD2(entry_offset, x1f441_data->offset);
	    LCARD_SUBTRACT_CARD2(entry_offset, record_data->deck);
	}

	LCARD_SET_CARD2(text, entry_offset);
	LCARD_ADD_CARD1(text, size);
	if (!LCARD_LESS_CARD2(record_data->size, text)) {
	    {
		status = read(fd, data, size);
		if (status == -1) {
		    MISS(x1f44v_data) = READ_ERROR;
		} else {
		    LCARD_ADD_CARD1(x1f441_data->offset, status);
		}
	    }
	} else {
	    unsigned transfer;

	    LCARD_SET_CARD2(text, record_data->size);
	    LCARD_SUBTRACT_CARD2(text, entry_offset);
	    transfer = LCARD_GET_CARD(text);
	    do {
		excess = read(fd, data, transfer);
		if (excess == -1) {
		    MISS(x1f44v_data) = READ_ERROR;
		    status = -1;
		    break;
		}
		status += excess;
		LCARD_ADD_CARD1(x1f441_data->offset, excess);
		if (excess != transfer) {
		    break;
		}
		size -= transfer;
		if (size) {
		    data = (char *) data + transfer;
		    if (x1f441_data->index == count - 1) {
			break;
		    }
		    excess = close(fd);
		    if (excess == -1) {
			MISS(x1f44v_data) = LOSE_ERROR;
			break;
		    }
		    record_data++;
		    x1f441_data->index++;
		    fd = open(record_data->path, O_RDONLY);
		    if (fd == -1) {
			MISS(x1f44v_data) = OPEN_ERROR;
			x1f441_data->fd = -1;
			break;
		    } else {
			x1f441_data->fd = fd;
		    }
		    if (LCARD_IS_SET(record_data->head)) {
			if (lseek(fd, LCARD_GET_CARD(record_data->head),
				  SEEK_SET) == -1) {
			    MISS(x1f44v_data) = SEEK_ERROR;
			    break;
			}
		    }
		    LCARD_SET_CARD1(text, size);
		    if (LCARD_LESS_CARD2(text, record_data->size)) {
			transfer = size;
		    } else {
			transfer = LCARD_GET_CARD(record_data->size);
		    }
		} else {
		    break;
		}
	    } while (1);
	}
    } while (0);

    return status;
}


static int
seek_1f440(void *x1f44v_data, void *offset, int whence)
{
    int status;
    lcard_type seek_offset;
    struct x1f440_type *x1f440_data;

    LCARD_SET_CARD2(seek_offset, *(lcard_type *) offset);

    x1f440_data = DATA(x1f44v_data);

    status = lseek
	(x1f440_data->fd, LCARD_GET_CARD(seek_offset), whence);
    if (status < 0) {
	MISS(x1f44v_data) = SEEK_ERROR;
    } else {
	status = 0;
	if (whence == SEEK_CUR) {
	    LCARD_ADD_CARD2(x1f440_data->offset, seek_offset);
	} else if (whence == SEEK_SET) {
	    LCARD_SET_CARD2(x1f440_data->offset, seek_offset);
	} else if (whence == SEEK_END) {
	    /*
	     * FEATURES
	     *
	     *.SEEK_ENDing may be allowed, but I'm too lazy to...
	     */
	    status = -1;
	}
    }

    return status;
}


static int
seek_1f441(void *x1f44v_data, void *offset, int whence)
{
    int status = 0;
    lcard_type seek_offset, x1f441_offset;
    struct x1f441_type *x1f441_data;

    x1f441_data = DATA(x1f44v_data);

    LCARD_SET_CARD2(seek_offset, *(lcard_type *) offset);
    LCARD_SET_CARD2(x1f441_offset, x1f441_data->offset);
    if (whence == SEEK_CUR) {
	LCARD_ADD_CARD2(seek_offset, x1f441_offset);
	if (LCARD_LESS_CARD2(x1f441_data->size, seek_offset)) {
	    LCARD_SET_CARD2(seek_offset, x1f441_data->size);
	}
    } else if (whence == SEEK_SET) {
	if (LCARD_LESS_CARD2(x1f441_data->size, seek_offset)) {
	    LCARD_SET_CARD2(seek_offset, x1f441_data->size);
	}
    } else if (whence == SEEK_END) {
	status = -1;
    }

    if (status == -1) {
    } else if (x1f441_data->fd == -1) {
	LCARD_SET_CARD2(x1f441_data->offset, seek_offset);
    } else {
	do {
	    lcard_type text;
	    struct record_type *record_data;

	    record_data = x1f441_data->record_data + x1f441_data->index;
	    LCARD_SET_CARD2(text, record_data->deck);
	    if (LCARD_LESS_CARD2(seek_offset, text)) {
	    } else {
		LCARD_ADD_CARD2(text, record_data->size);
		LCARD_ADD_CARD1(text, 1);
		if (LCARD_LESS_CARD2(seek_offset, text)) {
		    int lseek_offset;

		    if (LCARD_LESS_CARD2(seek_offset, x1f441_offset)) {
			LCARD_SET_CARD2(text, x1f441_offset);
			LCARD_SUBTRACT_CARD2(text, seek_offset);
			lseek_offset = -(int) LCARD_GET_CARD(text);
		    } else {
			LCARD_SET_CARD2(text, seek_offset);
			LCARD_SUBTRACT_CARD2(text, x1f441_offset);
			lseek_offset = LCARD_GET_CARD(text);
		    }
		    status = lseek(x1f441_data->fd, lseek_offset, SEEK_CUR);
		    if (status == -1) {
			MISS(x1f44v_data) = SEEK_ERROR;
		    } else {
			status = 0;
			LCARD_SET_CARD2(x1f441_data->offset, seek_offset);
		    }
		    break;
		}
	    }

	    status = close(x1f441_data->fd);
	    if (status == -1) {
		MISS(x1f44v_data) = LOSE_ERROR;
	    } else {
		x1f441_data->fd = -1;
		LCARD_SET_CARD2(x1f441_data->offset, seek_offset);
	    }
	} while (0);
    }

    return status;
}


static int
tell_1f440(void *x1f44v_data, void *offset)
{
#if 1
    lcard_type *output;

    output = offset;
    LCARD_SET_CARD2(*output,
		    ((struct x1f440_type *) DATA(x1f44v_data))->offset);

    return 0;
#else
    int status;

    status = lseek
	(((struct x1f440_type *) DATA(x1f44v_data))->fd, 0, SEEK_CUR);
    if (status < 0) {
	MISS(x1f44v_data) = SEEK_ERROR;
    } else {
	lcard_type *output;

	output = offset;
	LCARD_SET_CARD1(*output, status);
	status = 0;
    }

    return status;
#endif				/* 1 */
}


static int
tell_1f441(void *x1f44v_data, void *offset)
{
    lcard_type *output;

    output = offset;
    LCARD_SET_CARD2(*output,
		    ((struct x1f441_type *) DATA(x1f44v_data))->offset);

    return 0;
}
