/*
 * 1f44.6.c
 * Copyright (C) 2004-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <fcntl.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/stat.h>
#include <sys/types.h>

#include <defs.h>
#include <inter.h>
#include <lcard.h>
#include <types.h>

#define CARDINAL(buffer, base) \
    ((buffer)[(base) + 0] << 030 | (buffer)[(base) + 1] << 020		      \
     | (buffer)[(base) + 2] << 010 | (buffer)[(base) + 3] << 000)

extern const struct x1f44e_type _seek_1f44_0_0[];

int
seek_1f44_init(struct x1f44v_type *x1f44v_data, const char *path, void *offset)
{
    int fd, status = 0;

    x1f44v_data->type = X1f440_DATA;

    fd = open(path, O_RDONLY);
    if (fd == -1) {
	x1f44v_data->error = OPEN_ERROR;
	status = -1;
    } else {
	x1f44v_data->data = NULL;

	do {
	    lcard_type x1f441_size;
	    struct record_type *record_data = NULL;
	    struct x1f441_type *x1f441_data;
	    unsigned count;
	    unsigned char buffer[0x10];

	    if (read(fd, buffer, 16) != 16) {
		break;
	    }
	    if (memcmp(buffer, "1f44\0\0\0\0", 8)) {
		break;
	    }

	    LCARD_SET_CARD1(x1f441_size, 0);

	    count = buffer[0xc] << 010 | buffer[0xd] << 000;
	    if (count) {
		unsigned i = 0;

		if (!(record_data = (struct record_type *)
		      malloc(count * sizeof(struct record_type)))) {
		    break;
		}

		for (; i < count; i++) {
		    unsigned size;

		    record_data->path = NULL;

		    if (read(fd, buffer, 16) != 16) {
			break;
		    }
		    size = buffer[0xe] << 010 | buffer[0xf] << 000;
		    if (size) {
			if (!(record_data->path = (char *) malloc(size + 1))) {
			    break;
			}
			if (read(fd, record_data->path, size) != size) {
			    break;
			}
			if (size & 15) {
			    if (lseek(fd, 16 - (size & 15), SEEK_CUR) == -1) {
				break;
			    }
			}
			record_data->path[size] = 0;
		    } else {
			break;
		    }

		    {
			unsigned deck;

			deck = CARDINAL(buffer, 0x0);
			LCARD_SET_CARD1(record_data->head, deck);
			deck = CARDINAL(buffer, 0x4);
			LCARD_SET_CARD1(record_data->size, deck);
			LCARD_SET_CARD2(record_data->deck, x1f441_size);
			LCARD_ADD_CARD1(x1f441_size, deck);
		    }

		    record_data++;
		}

		if (i == count) {
		    record_data -= count;
		} else {
		    record_data -= i;
		    _seek_1f44_free_entries(record_data - i, i + 1);
		    break;
		}
	    } else {
		break;
	    }

	    if (!(x1f441_data = (struct x1f441_type *)
		  malloc(sizeof(struct x1f441_type)))) {
		_seek_1f44_free_entries(record_data, count);
		break;
	    }

	    x1f44v_data->data = x1f441_data;
	    x1f44v_data->type = X1f441_DATA;

	    x1f441_data->record_data = record_data;
	    x1f441_data->index = 0;
	    x1f441_data->fd = -1;
	    x1f441_data->count = count;
	    LCARD_SET_CARD2(x1f441_data->size, x1f441_size);
	    if (!offset) {
		LCARD_SET_CARD1(x1f441_data->offset, 0);
	    } else {
		LCARD_SET_CARD2(x1f441_data->offset, *(lcard_type *) offset);
	    }
	} while (0);

	if (x1f44v_data->data) {
	    close(fd);

	    x1f44v_data->eead = _seek_1f44_0_0[1].eead;
	    x1f44v_data->read = _seek_1f44_0_0[1].read;
	    x1f44v_data->seek = _seek_1f44_0_0[1].seek;
	    x1f44v_data->tell = _seek_1f44_0_0[1].tell;
	} else {
	    status = -1;

	    do {
		struct x1f440_type *x1f440_data;

		if (offset) {
		    if (lseek(fd, LCARD_GET_CARD(*(lcard_type *) offset),
			      SEEK_SET) == -1) {
			break;
		    }
		} else {
		    if (lseek(fd, 0, SEEK_SET) == -1) {
			break;
		    }
		}
		if (!(x1f440_data = (struct x1f440_type *)
		      malloc(sizeof(struct x1f440_type)))) {
		    break;
		}

		x1f44v_data->data = x1f440_data;
		x1f44v_data->type = X1f440_DATA;

		x1f440_data->fd = fd;
		if (!offset) {
		    LCARD_SET_CARD1(x1f440_data->offset, 0);
		} else {
		    LCARD_SET_CARD2(x1f440_data->offset,
				    *(lcard_type *) offset);
		}

		status = 0;
	    } while (0);

	    if (status) {
		close(fd);
	    } else {
		seek_1f44_link(x1f44v_data);
	    }
	}
    }

    return status;
}
