/*
 * 1f44.a.c
 * Copyright (C) 2004-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#include <defs.h>
#include <inter.h>
#include <lcard.h>
#include <types.h>

#define CARDINAL(buffer, base) \
    ((buffer)[(base) + 0] << 030 | (buffer)[(base) + 1] << 020		      \
     | (buffer)[(base) + 2] << 010 | (buffer)[(base) + 3] << 000)

extern const struct x1f44e_type _seek_1f44_0_0[];

int
seek_1f44_read(struct x1f44v_type *x1f44v_data, unsigned count, void *data,
	       int (*read) (void *, void *, unsigned))
{
    int status = 0;

    x1f44v_data->data = NULL;

    do {
	lcard_type x1f441_size;
	struct record_type *record_data = NULL;
	struct x1f441_type *x1f441_data;
	unsigned char buffer[0x10];

	LCARD_SET_CARD1(x1f441_size, 0);

	if (count) {
	    unsigned i;

	    if (!(record_data = (struct record_type *)
		  malloc(count * sizeof(struct record_type)))) {
		break;
	    }

	    for (i = count; i; i--) {
		unsigned size;

		record_data->path = NULL;

		if (read(data, buffer, 16) != 16) {
		    break;
		}
		size = buffer[0xe] << 010 | buffer[0xf] << 000;
		if (size) {
		    if (!(record_data->path = (char *) malloc(size + 1))) {
			break;
		    }
		    if (read(data, record_data->path, size) != size) {
			break;
		    }
		    if (size & 15) {
			unsigned deck;
			unsigned char buffer[0x10];

			deck = 16 - (size & 15);
			if (read(data, buffer, deck) != deck) {
			    break;
			}
		    }
		    record_data->path[size] = 0;
		} else {
		    break;
		}

		{
		    unsigned deck;

		    deck = CARDINAL(buffer, 0x0);
		    LCARD_SET_CARD1(record_data->head, deck);
		    deck = CARDINAL(buffer, 0x4);
		    LCARD_SET_CARD1(record_data->size, deck);
		    LCARD_SET_CARD2(record_data->deck, x1f441_size);
		    LCARD_ADD_CARD1(x1f441_size, deck);
		}

		record_data++;
	    }

	    if (!i) {
		record_data -= count;
	    } else {
		i = count - i;
		record_data -= i;
		_seek_1f44_free_entries(record_data, i + 1);
		break;
	    }
	} else {
	    break;
	}

	if (!(x1f441_data = (struct x1f441_type *)
	      malloc(sizeof(struct x1f441_type)))) {
	    _seek_1f44_free_entries(record_data, count);
	    break;
	}

	x1f44v_data->data = x1f441_data;
	x1f44v_data->type = X1f441_DATA;

	x1f441_data->record_data = record_data;
	x1f441_data->count = count;
	LCARD_SET_CARD2(x1f441_data->size, x1f441_size);
    } while (0);

    if (x1f44v_data->data) {
	x1f44v_data->eead = _seek_1f44_0_0[1].eead;
	x1f44v_data->read = _seek_1f44_0_0[1].read;
	x1f44v_data->seek = _seek_1f44_0_0[1].seek;
	x1f44v_data->tell = _seek_1f44_0_0[1].tell;
    } else {
	status = -1;
    }

    return status;
}
