/*
 * 1f44.g.c
 * Copyright (C) 2007-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <errno.h>
#include <string.h>

#include <defs.h>
#include <types.h>

static const char digits[] = "0123456789";

int
seek_1f44_stat(struct x1f44v_type *x1f44v_data, const char *path, void *data,
	       int (*type) (void *, const char *))
{
    int status;

    do {
	status = type(data, "cannot ");
	if (status) {
	    break;
	}
	{
	    int error;

	    error = x1f44v_data->error;
	    if (error == LOSE_ERROR) {
		status = type(data, "close");
		if (status) {
		    break;
		}
	    } else if (error == OPEN_ERROR) {
		status = type(data, "open");
		if (status) {
		    break;
		}
	    } else if (error == READ_ERROR) {
		status = type(data, "read");
		if (status) {
		    break;
		}
	    } else if (error == SEEK_ERROR) {
		status = type(data, "seek");
		if (status) {
		    break;
		}
	    } else {
		status = type(data, "hell knows what");
		if (status) {
		    break;
		}
	    }
	}
	status = type(data, " `");
	if (status) {
	    break;
	}
	status = type(data, path);
	if (status) {
	    break;
	}
	status = type(data, "'");
	if (status) {
	    break;
	}
	if (x1f44v_data->type == X1f441_DATA) {
	    struct x1f441_type *x1f441_data;
	    unsigned index;

	    x1f441_data = x1f44v_data->data;

	    index = x1f441_data->index;

	    status = type(data, " (indexed ");
	    if (status) {
		break;
	    }
	    {
		char *o, v[21];
		unsigned c;

		c = index;

		o = v + 20;
		*o = 0;

		do {
		    --o;
		    *o = digits[c % 10];
		    c /= 10;
		} while (c);
		status = type(data, o);
		if (status) {
		    break;
		}
	    }
	    status = type(data, ", `");
	    if (status) {
		break;
	    }
	    status = type(data, x1f441_data->record_data[index].path);
	    if (status) {
		break;
	    }
	    status = type(data, "')");
	    if (status) {
		break;
	    }
	}
	status = type(data, ": ");
	if (status) {
	    break;
	}
	status = type(data, strerror(errno));
	if (status) {
	    break;
	}
    } while (0);

    return status;
}
