/*
 * lcard.h
 * Copyright (C) 2004-2007, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __seek_1f44_LCARD_H__
#define __seek_1f44_LCARD_H__

#include <config.h>

#if defined ENABLE_ANSI
# undef __GNUC__
#endif				/* ENABLE_ANSI */

#if defined __GNUC__
# define LONG_LONG_UNSIGNED_DEFINITION	0
#endif				/* __GNUC__ */
#define LONG_UNSIGNED_DEFINITION	1
#define STRUCT_DEFINITION		2
#define UNSIGNED_DEFINITION		3

#if SIZEOF_UNSIGNED_LONG == 8
# define LCARD_DEFINITION		LONG_UNSIGNED_DEFINITION
#elif defined __GNUC__
# define LCARD_DEFINITION		LONG_LONG_UNSIGNED_DEFINITION
#else
# define LCARD_DEFINITION		STRUCT_DEFINITION
#endif				/* __GNUC__ */

#if LCARD_DEFINITION == STRUCT_DEFINITION
# define LCARD_ADD_CARD1(card_0, card_1) \
    {									      \
	if ((unsigned) ~0 - (card_0).minor < (card_1)) {		      \
	    (card_0).major++;						      \
	}								      \
	(card_0).minor += (card_1);					      \
    }
#else
# define LCARD_ADD_CARD1(card_0, card_1) \
    {									      \
	(card_0) += (card_1);						      \
    }
#endif				/* LCARD_DEFINITION == STRUCT_DEFINITION */

#if LCARD_DEFINITION == STRUCT_DEFINITION
# define LCARD_ADD_CARD2(card_0, card_1) \
    {									      \
	if ((unsigned) ~0 - (card_0).minor < (card_1).minor) {		      \
	    (card_0).major++;						      \
	}								      \
	(card_0).major += (card_1).major;				      \
	(card_0).minor += (card_1).minor;				      \
    }
#else
# define LCARD_ADD_CARD2(card_0, card_1) \
    {									      \
	(card_0) += (card_1);						      \
    }
#endif				/* LCARD_DEFINITION == STRUCT_DEFINITION */

#if LCARD_DEFINITION == STRUCT_DEFINITION
# define LCARD_GET_CARD(card) \
    (card).minor
#else
# define LCARD_GET_CARD(card) \
    (card)
#endif				/* LCARD_DEFINITION == STRUCT_DEFINITION */

#if LCARD_DEFINITION == STRUCT_DEFINITION
# define LCARD_EQUAL_CARD2(card_0, card_1) \
    ((card_0).major == (card_1).major && (card_0).minor == (card_1).minor)
#else
# define LCARD_EQUAL_CARD2(card_0, card_1) \
    ((card_0) == (card_1))
#endif				/* LCARD_DEFINITION == STRUCT_DEFINITION */

#if LCARD_DEFINITION == STRUCT_DEFINITION
# define LCARD_IS_SET(card) \
    ((card).major || (card).minor)
#else
# define LCARD_IS_SET(card) \
    (card)
#endif				/* LCARD_DEFINITION == STRUCT_DEFINITION */

#if LCARD_DEFINITION == STRUCT_DEFINITION
# define LCARD_LESS_CARD2(card_0, card_1) \
    ((card_0).major < (card_1).major					      \
     || ((card_0).major == (card_1).major && (card_0).minor < (card_1).minor))
#else
# define LCARD_LESS_CARD2(card_0, card_1) \
    ((card_0) < (card_1))
#endif				/* LCARD_DEFINITION == STRUCT_DEFINITION */

#if LCARD_DEFINITION == STRUCT_DEFINITION
# define LCARD_SET_CARD1(card_0, card_1) \
    {									      \
	(card_0).major = 0;						      \
	(card_0).minor = (card_1);					      \
    }
#else
# define LCARD_SET_CARD1(card_0, card_1) \
    {									      \
	(card_0) = (card_1);						      \
    }
#endif				/* LCARD_DEFINITION == STRUCT_DEFINITION */

#if LCARD_DEFINITION == STRUCT_DEFINITION
# define LCARD_SET_CARD2(card_0, card_1) \
    {									      \
	(card_0) = (card_1);						      \
    }
#else
# define LCARD_SET_CARD2(card_0, card_1) \
    {									      \
	(card_0) = (card_1);						      \
    }
#endif				/* LCARD_DEFINITION == STRUCT_DEFINITION */

#if LCARD_DEFINITION == STRUCT_DEFINITION
# define LCARD_SUBTRACT_CARD1(card_0, card_1) \
    {									      \
	if ((card_0).minor < (card_1)) {				      \
	    (card_0).major--;						      \
	}								      \
	(card_0).minor -= (card_1);					      \
    }
#else
# define LCARD_SUBTRACT_CARD1(card_0, card_1) \
    {									      \
	(card_0) -= (card_1);						      \
    }
#endif				/* LCARD_DEFINITION == STRUCT_DEFINITION */

#if LCARD_DEFINITION == STRUCT_DEFINITION
# define LCARD_SUBTRACT_CARD2(card_0, card_1) \
    {									      \
	if ((card_0).minor < (card_1).minor) {				      \
	    (card_0).major--;						      \
	}								      \
	(card_0).major -= (card_1).major;				      \
	(card_0).minor -= (card_1).minor;				      \
    }
#else
# define LCARD_SUBTRACT_CARD2(card_0, card_1) \
    {									      \
	(card_0) -= (card_1);						      \
    }
#endif				/* LCARD_DEFINITION == STRUCT_DEFINITION */

#if LCARD_DEFINITION == LONG_LONG_UNSIGNED_DEFINITION
typedef long long unsigned lcard_type;
#elif LCARD_DEFINITION == LONG_UNSIGNED_DEFINITION
typedef long unsigned lcard_type;
#elif LCARD_DEFINITION == STRUCT_DEFINITION
typedef struct lcard_type {
    unsigned major, minor;
} lcard_type;
#else
typedef unsigned lcard_type;
#endif				/* LCARD_DEFINITION == LONG_LONG_UNSIGNED... */

#endif				/* __seek_1f44_LCARD_H__ */
