/*
 * pnon.h
 * Copyright (C) 2006-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __LIBx1f4_PNON_H__
#define __LIBx1f4_PNON_H__

#include <config.h>

#define PNON_C_TYPE_DEFINITION		0
#define PNON_STRUCT_DEFINITION		1

#if SIZEOF_UNSIGNED_LONG == 8
# define PNON_C_BITS			64
# define PNON_C_HALF			32
# define PNON_C_MASK			4294967295u
# define PNON_C_SIZE			8
# define PNON_C_TYPE			unsigned long
# define PNON_DEFINITION		PNON_C_TYPE_DEFINITION
#else
# define PNON_C_BITS			32
# define PNON_C_HALF			16
# define PNON_C_MASK			65535
# define PNON_C_SIZE			4
# define PNON_C_TYPE			unsigned
# define PNON_DEFINITION		PNON_STRUCT_DEFINITION
#endif				/* SIZEOF_UNSIGNED_LONG == 8 */

/*
 * addition
 */
#if PNON_DEFINITION == PNON_C_TYPE_DEFINITION
# define PNON_ADD_CARD(pnon, card) \
    {									      \
	(pnon) += (card);						      \
    }
#elif 0
# define PNON_ADD_CARD(pnon, card) \
    {									      \
	if (~(PNON_C_TYPE) 0 - (pnon).b < (PNON_C_TYPE) (card)) {	      \
	    (pnon).a++;							      \
	}								      \
	(pnon).b += (card);						      \
    }
#else
# define PNON_ADD_CARD(pnon, card) \
    {									      \
	(pnon).b += (card);						      \
	if ((pnon).b < (PNON_C_TYPE) (card)) {				      \
	    (pnon).a++;							      \
	}								      \
    }
#endif				/* PNON_DEFINITION == PNON_C_TYPE_DEFINITION */

/*
 * addition
 */
#if PNON_DEFINITION == PNON_C_TYPE_DEFINITION
# define PNON_ADD_PNON(pnon, pnOn) \
    {									      \
	(pnon) += (pnOn);						      \
    }
#elif 0
# define PNON_ADD_PNON(pnon, pnOn) \
    {									      \
	if (~(PNON_C_TYPE) 0 - (pnon).b < (pnOn).b) {			      \
	    (pnon).a++;							      \
	}								      \
	(pnon).a += (pnOn).a;						      \
	(pnon).b += (pnOn).b;						      \
    }
#else
# define PNON_ADD_PNON(pnon, pnOn) \
    {									      \
	(pnon).a += (pnOn).a;						      \
	(pnon).b += (pnOn).b;						      \
	if ((pnon).b < (pnOn).b) {					      \
	    (pnon).a++;							      \
	}								      \
    }
#endif				/* PNON_DEFINITION == PNON_C_TYPE_DEFINITION */

/*
 * non zero test
 */
#if PNON_DEFINITION == PNON_C_TYPE_DEFINITION
# define PNON_DAY_DATA(pnon) \
    (pnon)
#elif 0
# define PNON_DAY_DATA(pnon) \
    ((pnon).a || (pnon).b)
#else
# define PNON_DAY_DATA(pnon) \
    ((pnon).a | (pnon).b)
#endif				/* PNON_DEFINITION == PNON_C_TYPE_DEFINITION */

/*
 * conversion
 */
#if PNON_DEFINITION == PNON_C_TYPE_DEFINITION
# define PNON_GET_DATA(pnon) \
    (pnon)
#else
# define PNON_GET_DATA(pnon) \
    ((pnon).b)
#endif				/* PNON_DEFINITION == PNON_C_TYPE_DEFINITION */

/*
 * setting
 */
#if PNON_DEFINITION == PNON_C_TYPE_DEFINITION
# define PNON_SET_CARD(pnon, card) \
    {									      \
	(pnon) = (card);						      \
    }
#else
# define PNON_SET_CARD(pnon, card) \
    {									      \
	(pnon).a = 0;							      \
	(pnon).b = (card);						      \
    }
#endif				/* PNON_DEFINITION == PNON_C_TYPE_DEFINITION */

/*
 * setting
 */
#if PNON_DEFINITION == PNON_C_TYPE_DEFINITION
# define PNON_SET_DATA(pnon) \
    {									      \
	(pnon) = 1;							      \
    }
#else
# define PNON_SET_DATA(pnon) \
    {									      \
	(pnon).a = 0;							      \
	(pnon).b = 1;							      \
    }
#endif				/* PNON_DEFINITION == PNON_C_TYPE_DEFINITION */

/*
 * setting
 */
#if PNON_DEFINITION == PNON_C_TYPE_DEFINITION
# define PNON_SET_PAIR(pnon, c1st, c2nd) \
    {									      \
	(pnon) = (PNON_C_TYPE) (c1st) << PNON_C_HALF | (c2nd);		      \
    }
#else
# define PNON_SET_PAIR(pnon, c1st, c2nd) \
    {									      \
	(pnon).a = (c1st);						      \
	(pnon).b = (c2nd);						      \
    }
#endif				/* PNON_DEFINITION == PNON_C_TYPE_DEFINITION */

/*
 * setting
 */
#if PNON_DEFINITION == PNON_C_TYPE_DEFINITION
# define PNON_SET_PNON(pnon, pnOn) \
    {									      \
	(pnon) = (pnOn);						      \
    }
#else
# define PNON_SET_PNON(pnon, pnOn) \
    {									      \
	(pnon).a = (pnOn).a;						      \
	(pnon).b = (pnOn).b;						      \
    }
#endif				/* PNON_DEFINITION == PNON_C_TYPE_DEFINITION */

/*
 * subtraction
 */
#if PNON_DEFINITION == PNON_C_TYPE_DEFINITION
# define PNON_SLL_CARD(pnon, card) \
    {									      \
	(pnon) -= (card);						      \
    }
#else
# define PNON_SLL_CARD(pnon, card) \
    {									      \
	if ((pnon).b < (PNON_C_TYPE) (card)) {				      \
	    (pnon).a--;							      \
	}								      \
	(pnon).b -= (card);						      \
    }
#endif				/* PNON_DEFINITION == PNON_C_TYPE_DEFINITION */

/*
 * subtraction
 */
#if PNON_DEFINITION == PNON_C_TYPE_DEFINITION
# define PNON_SLL_PNON(pnon, pnOn) \
    {									      \
	(pnon) -= (pnOn);						      \
    }
#else
# define PNON_SLL_PNON(pnon, pnOn) \
    {									      \
	if ((pnon).b < (pnOn).b) {					      \
	    (pnon).a--;							      \
	}								      \
	(pnon).a -= (pnOn).a;						      \
	(pnon).b -= (pnOn).b;						      \
    }
#endif				/* PNON_DEFINITION == PNON_C_TYPE_DEFINITION */

/*
 * (struct definition null major) squaring
 */
#if PNON_DEFINITION == PNON_C_TYPE_DEFINITION
# define PNON_SQE_DATA(pnon) \
    {									      \
	(pnon) *= (pnon);						      \
    }
#else
# define PNON_SQE_DATA(pnon) \
    {									      \
	pnon_type *__pnon_data;						      \
									      \
	__pnon_data = &(pnon);						      \
	__libx1f4i0_pnon_0_0_2[0](__pnon_data, __pnon_data);		      \
    }
#endif				/* PNON_DEFINITION == PNON_C_TYPE_DEFINITION */

/*
 * multiplication
 */
#if PNON_DEFINITION == PNON_C_TYPE_DEFINITION
# define PNON_SLX_CARD(pnon, card) \
    {									      \
	(pnon) *= (card);						      \
    }
#else
# define PNON_SLX_CARD(pnon, card) \
    {									      \
	pnon_type *__pnon_data;						      \
	unsigned __card;						      \
									      \
	__card = (card);						      \
	__pnon_data = &(pnon);						      \
	__libx1f4i0_pnon_0_1_2[0](__pnon_data, &__card);		      \
    }
#endif				/* PNON_DEFINITION == PNON_C_TYPE_DEFINITION */

/*
 * division
 */
#if PNON_DEFINITION == PNON_C_TYPE_DEFINITION
# define PNON_VLX_CARD(pnon, card) \
    {									      \
	(pnon) /= (card);						      \
    }
#else
# define PNON_VLX_CARD(pnon, card) \
    {									      \
	pnon_type *__pnon_data;						      \
	unsigned __card;						      \
									      \
	__card = (card);						      \
	__pnon_data = &(pnon);						      \
	__libx1f4i0_pnon_0_2_2[0](__pnon_data, &__card);		      \
    }
#endif				/* PNON_DEFINITION == PNON_C_TYPE_DEFINITION */

/*
 * resetting
 */
#if PNON_DEFINITION == PNON_C_TYPE_DEFINITION
# define PNON_XOR_DATA(pnon) \
    {									      \
	(pnon) = 0;							      \
    }
#else
# define PNON_XOR_DATA(pnon) \
    {									      \
	(pnon).a = 0;							      \
	(pnon).b = 0;							      \
    }
#endif				/* PNON_DEFINITION == PNON_C_TYPE_DEFINITION */

#if PNON_DEFINITION == PNON_C_TYPE_DEFINITION
typedef PNON_C_TYPE pnon_type;
#else
typedef struct pnon_type {
    PNON_C_TYPE a, b;
} pnon_type;
#endif				/* PNON_DEFINITION == PNON_C_TYPE_DEFINITION */

#if PNON_DEFINITION == PNON_STRUCT_DEFINITION
extern void (*const __libx1f4i0_pnon_0_0_2[]) (void *, void *),
    (*const __libx1f4i0_pnon_0_1_2[]) (void *, void *),
    (*const __libx1f4i0_pnon_0_2_2[]) (void *, void *);
#endif				/* PNON_DEFINITION == PNON_STRUCT_DEFINITION */

#endif				/* __LIBx1f4_PNON_H__ */
