/*
 * printf6.c
 * Copyright (C) 2002-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <limits.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define __USE_LIBx1f4__			0
#define __ENABLE_PRINT__		0

#if __USE_LIBx1f4__
# include <inter.h>
# include <types.h>
#endif				/* __USE_LIBx1f4__ */

#define ALTERNATE			(1 << 0x10)
#define INDEXED				(1 << 0x11)
#define UNINDEXED			(1 << 0x12)

#define ITERATE				(1 << 0x13)

#define DETAILED			(1 << 0x14)

#define CONDITIONAL			(1 << 0x15)
#define DECOMPOSE			(1 << 0x16)
#define OTHERWISE			(1 << 0x17)
#define NEGATE				(1 << 0x18)

#define SPLIT				(1 << 0x1f)

#define USER_FLAGS			(ALTERNATE)

#define HAS_COMPONENTS			(1 << 0x00)

#define FORMAT				0x100
#define RETURN				0x101
#define STRING				0x102

#define COUNT_STAMP			(1 << 0)
#define WIDTH_STAMP			(1 << 1)

#define STAMP_ORDER			8

#define STAMP_BASE			(1 << STAMP_ORDER)
#define STAMP_MASK			(~0 << STAMP_ORDER)

#define isddigit(c) \
    (0x2f < (c) && (c) < 0x3a)
#define isodigit(c) \
    (0x2f < (c) && (c) < 0x38)

#if !__USE_LIBx1f4__
# define is_spaCe(c) \
    (c_type_2[(c) >> 5] & (unsigned) 1 << ((c) & 31))
# define is_space(c) \
    (c_type_1[(c) >> 5] & (unsigned) 1 << ((c) & 31))
# define isxdigit(c) \
    (c_type_3[(c) >> 5] & (unsigned) 1 << ((c) & 31))
#endif				/* !__USE_LIBx1f4__ */

#define max(x, y) \
    (((x) < (y)) ? (y) : (x))

#if __USE_LIBx1f4__
# define add_iTem2			x1f4_add_iTem2
# define free_item			x1f4_free_item
# if __ENABLE_PRINT__
#  define print_item			x1f4_print_item
# endif				/* __ENABLE_PRINT__ */
#endif				/* __USE_LIBx1f4__ */

#define RUN_COUNT(list_data, xparser_data) \
    (((list_data)->r_stamp ^ (COUNT_STAMP | (xparser_data)->r_stamp))	      \
     & (COUNT_STAMP | STAMP_MASK))
#define RUN_WIDTH(list_data, xparser_data) \
    (((list_data)->r_stamp ^ (WIDTH_STAMP | (xparser_data)->r_stamp))	      \
     & (STAMP_MASK | WIDTH_STAMP))

#define COPY_FIELD_0(list, data) \
    {									      \
	(list)->flags |= SPLIT;						      \
	(list)->field.count_field = (data)->count_field;		      \
    }

#define COPY_FIELD_1(list, data) \
    {									      \
	(list)->field.count_field = (data)->count_field;		      \
	(list)->field.print_field = (data)->print_field;		      \
    }

#define COPY_FIELD__(list, data) \
    {									      \
	if ((data)->efield_data) {					      \
	    COPY_FIELD_0((list), (data));				      \
	} else {							      \
	    COPY_FIELD_1((list), (data));				      \
	}								      \
    }

#define COUNT_FIELD(printer, list_data, extent) \
    {									      \
	if (!(list_data)->size) {					      \
	    (extent) = (list_data)->field.count_field			      \
		((printer), (list_data)->flags & USER_FLAGS);		      \
	} else {							      \
	    xparser_data->tail_data = (list_data);			      \
	    (extent) = (list_data)->field.count_field			      \
		((printer), ((list_data)->flags & USER_FLAGS) | DETAILED);    \
	}								      \
    }

#define PRINT_FIELD(printer, list_data, flags) \
    {									      \
	if ((list_data)->size) {					      \
	    xparser_data->tail_data = (list_data);			      \
	    (list_data)->field.print_field				      \
		((printer), ((flags) & USER_FLAGS) | DETAILED);		      \
	} else {							      \
	    (list_data)->field.print_field((printer), (flags) & USER_FLAGS);  \
	}								      \
    }

typedef struct efield_type {
    const char *class;
    const struct efield_type *efield_data;
    unsigned (*count_field) (void *, unsigned), flags, index;
    void (*print_field) (void *, unsigned);
} efield_type;

typedef struct f6link_type {
   unsigned (*fix_ailes) (void *, char *, unsigned);
   void (*put_space) (void *, unsigned, char *, unsigned),
	(*put_ailes) (void *, char *, unsigned),
	(*set_index) (void *, unsigned), (*set_scope) (void *, unsigned),
	(*get_scope) (void *, unsigned);
} f6link_type;

typedef struct ffield_type {
    const char *class;
    const struct ffield_type *efield_data;
    unsigned (*count_field) (void *, unsigned), flags, index;
    void (*print_field) (void *, unsigned);
} ffield_type;

typedef struct frame_type {
    const struct ffield_type *field_data;
    struct frame_type *parent, *previous;
} frame_type;

#if !__USE_LIBx1f4__
typedef struct item_type {
    struct item_type *item_data;
    unsigned from, to;
} item_type;
#endif				/* !__USE_LIBx1f4__ */

typedef struct list_type {
    char *space;
    struct {
	unsigned (*count_field) (void *, unsigned);
	void (*print_field) (void *, unsigned);
    } field;
    struct list_type *list_data;
    unsigned code, count_x_width, flags, *list, r_stamp, scope, size,
	space_length, width;
} list_type;

typedef struct xitem_type {
    struct item_type *item_data;
    struct xitem_type *xitem_data;
} xitem_type;

typedef struct string_type {
    char *data;
    struct string_type *string_data;
    unsigned size;
} string_type;

typedef struct xparser_type {
    const char *specs;
    int level, record, scope;
    struct list_type *head_data, *tail_data;
    struct string_type *head_string, *tail_string;
    struct xitem_type *head_xitem, *tail_xitem;
    unsigned (*fix_ailes) (void *, char *, unsigned), r_stamp;
    void (*get_scope) (void *, unsigned), *printer,
	(*put_space) (void *, unsigned, char *, unsigned),
	(*put_ailes) (void *, char *, unsigned),
	(*set_index) (void *, unsigned), (*set_scope) (void *, unsigned);
} xparser_type;

static int add_field(struct xparser_type *, unsigned, unsigned, unsigned,
		     char *, unsigned);
static int add_string(struct xparser_type *, const char *, unsigned);
#if !__USE_LIBx1f4__
static int add_iTem2(struct item_type **, struct item_type **, char *,
		     unsigned);
#endif				/* !__USE_LIBx1f4__ */
static int add_items(struct xparser_type *, const char *);
static int field_format(struct xparser_type *, struct frame_type *);
static int frame_format(struct xparser_type *, const struct ffield_type *,
			struct frame_type *, struct frame_type *);
static int parse_format(struct xparser_type *, struct frame_type *);

static unsigned get_width(struct xparser_type *);

#if __ENABLE_PRINT__
static void flush_user(FILE *, int, unsigned *);
#endif				/* __ENABLE_PRINT__ */
#if !__USE_LIBx1f4__
static void free_item(struct item_type **);
#endif				/* !__USE_LIBx1f4__ */
static void lookup_code(const struct ffield_type **,
			const struct ffield_type *, unsigned);
static void parse_decimal(unsigned *, const char **);
#if __ENABLE_PRINT__
static void print_format(FILE *, struct xparser_type *);
#endif				/* __ENABLE_PRINT__ */
#if __ENABLE_PRINT__ && !__USE_LIBx1f4__
static void print_item(FILE *, struct item_type *);
#endif				/* __ENABLE_PRINT__ && !__USE_LIBx1f4__ */
#if __ENABLE_PRINT__
static void print_string(FILE *, char *, unsigned);
#endif				/* __ENABLE_PRINT__ && !__USE_LIBx1f4__ */
#if __ENABLE_PRINT__
static void put_user(FILE *, unsigned, char *, unsigned *);
#endif				/* __ENABLE_PRINT__ */
static void run_field(struct xparser_type *, struct list_type *);
static void run_format(struct xparser_type *);
static void run_subformat(struct xparser_type *, struct list_type *);
static void skip_format(struct xparser_type *);

#if !__USE_LIBx1f4__
static const unsigned c_type_1[] = {
/* *INDENT-OFF* */
    0x00003e00, 0x00000001, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000
/* *INDENT-ON* */
}, c_type_2[] = {
/* *INDENT-OFF* */
    0x00003e00, 0x00001001, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000
/* *INDENT-ON* */
}, c_type_3[] = {
/* *INDENT-OFF* */
    0x00000000, 0x03ff0000, 0x0000007e, 0x0000007e,
    0x00000000, 0x00000000, 0x00000000, 0x00000000
/* *INDENT-ON* */
};
#endif				/* !__USE_LIBx1f4__ */

static int
add_field(struct xparser_type *xparser_data, unsigned c, unsigned flags,
	  unsigned width, char *space, unsigned space_length)
{
    struct list_type *list_data;

    if (!(list_data = (struct list_type *) malloc(sizeof(struct list_type)))) {
	return -1;
    }

    if (!xparser_data->head_data) {
	xparser_data->head_data = list_data;
    } else {
	xparser_data->tail_data->list_data = list_data;
    }

    xparser_data->tail_data = list_data;

    list_data->code = c;
    list_data->flags = flags;
    list_data->list = NULL;
    list_data->r_stamp = 0;
    list_data->space = space;
    list_data->space_length = space_length;
    list_data->size = 0;
    list_data->width = width;

    return 0;
}


static int
add_string(struct xparser_type *xparser_data, const char *string,
	   unsigned length)
{
    char *partial;
    struct string_type *this_string;
    unsigned c, i;

    if (!(this_string = (struct string_type *)
	  malloc(sizeof(struct string_type) + length + 0))) {
	return -1;
    }

    if (!xparser_data->head_string) {
	xparser_data->head_string = this_string;
    } else {
	xparser_data->tail_string->string_data = this_string;
    }
    xparser_data->tail_string = this_string;
    partial = (char *) (this_string + 1);
    this_string->data = partial;
    for (i = length; i; i--) {
	if (*string == '\\') {
	    switch (*++string) {
	    case 'a':
		*partial++ = '\a';
		break;
	    case 'b':
		*partial++ = '\b';
		break;
	    case 'f':
		*partial++ = '\f';
		break;
	    case 'n':
		*partial++ = '\n';
		break;
	    case 'r':
		*partial++ = '\r';
		break;
	    case 't':
		*partial++ = '\t';
		break;
	    case 'v':
		*partial++ = '\v';
		break;
	    default:
		if (isodigit(*string)) {
		    c = *string - 0x30;
		    if (2 < i && isodigit(string[1])) {
			c <<= 3;
			c += string[1] - 0x30;
			if (3 < i && isodigit(string[2])) {
			    c <<= 3;
			    c += string[2] - 0x30;
			    string++;
			    i--;
			}
			string++;
			i--;
		    }
		    *partial++ = c;
		} else {
		    *partial++ = *string;
		}
	    }
	    i--;
	    string++;
	} else {
	    *partial++ = *string++;
	}
    }

    this_string->size = partial - this_string->data;

    return 0;
}


#if !__USE_LIBx1f4__
static int
add_iTem2(struct item_type **head_item, struct item_type **tail_item,
	  char *text, unsigned terminator)
{
    char *save_text;

    save_text = text;

    while (1) {
	while (is_space(*(unsigned char *) text)) {
	    text++;
	}
	if (*text == 0x30) {
	    text++;
	    if ((*(unsigned char *) text & 0xdf) == 0x58) {
		text++;
		if (!isxdigit(*text)) {
		    return 0;
		}
		do {
		    text++;
		} while (isxdigit(*text));
	    } else {
		while (isodigit(*text)) {
		    text++;
		}
	    }
	    while (is_space(*(unsigned char *) text)) {
		text++;
	    }
	} else {
	    if (isddigit(*text)) {
		do {
		    text++;
		} while (isddigit(*text));
		while (is_space(*(unsigned char *) text)) {
		    text++;
		}
	    } else {
		return 0;
	    }
	}
	if (*text == 0x2d) {
	    text++;
	    while (is_space(*(unsigned char *) text)) {
		text++;
	    }
	    if (*text == 0x30) {
		text++;
		if ((*(unsigned char *) text & 0xdf) == 0x58) {
		    text++;
		    if (!isxdigit(*text)) {
			return 0;
		    }
		    do {
			text++;
		    } while (isxdigit(*text));
		} else {
		    while (isodigit(*text)) {
			text++;
		    }
		}
		while (is_space(*(unsigned char *) text)) {
		    text++;
		}
	    } else {
		if (isddigit(*text)) {
		    do {
			text++;
		    } while (isddigit(*text));
		    while (is_space(*(unsigned char *) text)) {
			text++;
		    }
		} else {
		    return 0;
		}
	    }
	}
	if (*text == terminator) {
	    break;
	} else {
	    if (*text != 0x2c) {
		return 0;
	    } else {
		text++;
	    }
	}
    }

    text = save_text;

    while (*text != terminator) {
	struct item_type *my_item_data;
	unsigned from = 0, to = ~0;

	while (is_spaCe(*(unsigned char *) text)) {
	    text++;
	}
	if (*text == terminator) {
	    break;
	}
	if (*text == 0x30) {
	    text++;
	    if ((*(unsigned char *) text & 0xdf) == 0x58) {
		text++;
		do {
		    from <<= 4;
		    if (*text < 0x3a) {
			from += *text - 0x30;
		    } else if (*text < 0x47) {
			from += *text - 0x37;
		    } else {
			from += *text - 0x57;
		    }
		    text++;
		} while (isxdigit(*text));
	    } else {
		while (isodigit(*text)) {
		    from <<= 3;
		    from += *text - 0x30;
		    text++;
		}
	    }
	    while (is_space(*(unsigned char *) text)) {
		text++;
	    }
	} else {
	    do {
		from *= 10;
		from += *text - 0x30;
		text++;
	    } while (isddigit(*text));
	    while (is_space(*(unsigned char *) text)) {
		text++;
	    }
	}
	if (*text == 0x2d) {
	    text++;
	    while (is_space(*(unsigned char *) text)) {
		text++;
	    }
	    if (*text == 0x30) {
		to = 0;
		text++;
		if ((*(unsigned char *) text & 0xdf) == 0x58) {
		    text++;
		    do {
			to <<= 4;
			if (*text < 0x3a) {
			    to += *text - 0x30;
			} else if (*text < 0x47) {
			    to += *text - 0x37;
			} else {
			    to += *text - 0x57;
			}
			text++;
		    } while (isxdigit(*text));
		} else {
		    while (isodigit(*text)) {
			to <<= 3;
			to += *text - 0x30;
			text++;
		    }
		}
		while (is_space(*(unsigned char *) text)) {
		    text++;
		}
	    } else {
		to = 0;
		do {
		    to *= 10;
		    to += *text - 0x30;
		    text++;
		} while (isddigit(*text));
		while (is_space(*(unsigned char *) text)) {
		    text++;
		}
	    }
	} else {
	    to = from;
	}

	if (!(my_item_data =
	      (struct item_type *) malloc(sizeof(struct item_type)))) {
	    return -1;
	}

	my_item_data->from = from;
	my_item_data->to = to;

	if (*head_item) {
	    (*tail_item)->item_data = my_item_data;
	} else {
	    *head_item = my_item_data;
	}
	*tail_item = my_item_data;
    }

    (*tail_item)->item_data = NULL;

    return 1;
}
#endif				/* !__USE_LIBx1f4__ */


static int
add_items(struct xparser_type *xparser_data, const char *specs)
{
    struct item_type *item_data;
    struct xitem_type *xitem_data;

    if (!(xitem_data = (struct xitem_type *)
	  malloc(sizeof(struct xitem_type)))) {
	 return -1;
    }

    xitem_data->item_data = NULL;
    if (add_iTem2(&xitem_data->item_data, &item_data, (char *) specs, '}')
	!= 1) {
	free(xitem_data);

	return -1;
    }

    if (!xparser_data->head_xitem) {
	xparser_data->head_xitem = xitem_data;
    } else {
	xparser_data->tail_xitem->xitem_data = xitem_data;
    }
    xparser_data->tail_xitem = xitem_data;

    return 0;
}


static int
field_format(struct xparser_type *xparser_data, struct frame_type *frame_data)
{
    const char *string;
    int status = 0;
    unsigned *list = NULL, size = 0;

    string = xparser_data->specs;

    do {
	char *space = NULL;
	const struct ffield_type *efield_data, *ffield_data;
	struct frame_type *parent;
	unsigned flags = 0, scope = 0, space_length = 0, width = 0;

	parent = frame_data;
	if (*string == ':') {
	    status = -1;

	    string++;
	    if (!isddigit(*string)) {
		break;
	    }
	    parse_decimal(&scope, &string);
	    if (xparser_data->scope < scope) {
		break;
	    }
	    if (*string != ':') {
		break;
	    }
	    string++;

	    status = 0;

	    if (scope) {
		unsigned i;

		xparser_data->scope -= scope;
		for (i = scope; i; i--) {
		    parent = parent->parent;
		}
	    }
	}
	efield_data = parent->field_data;
	ffield_data = efield_data;
	if (*string == '#') {
	    flags = ALTERNATE;
	    string++;
	}
	if (*string == '<') {
	    const char *partial;

	    status = -1;

	    string++;
	    for (partial = string; *partial; partial++) {
		if (*partial == '\\') {
		    if (!partial[1]) {
			break;
		    } else {
			partial++;
		    }
		} else if (*partial == '%' || *partial == '('
			   || *partial == ')') {
		    break;
		} else if (*partial == '>') {
		    status = 0;
		    break;
		}
	    }
	    if (status) {
		break;
	    }
	    if (status) {
		break;
	    } else {
		status = add_string(xparser_data, string, partial - string);
		if (status) {
		    break;
		} else {
		    string = partial + 1;
		}
	    }

	    space = xparser_data->tail_string->data;
	    space_length = xparser_data->tail_string->size;
	}
	if (*string == '-') {
	    if (isddigit(string[1])) {
		flags |= NEGATE;
		string++;
		parse_decimal(&width, &string);
	    }
	} else {
	    if (isddigit(*string)) {
		parse_decimal(&width, &string);
	    }
	}
	while (*string == '.') {
	    if (!isddigit(string[1])) {
		break;
	    } else {
		if (!size) {
		    list = (unsigned *) malloc(sizeof(unsigned));
		    if (!list) {
			status = -1;
			break;
		    }
		} else {
		    if (!(size & (size - 1))) {
			unsigned *swap;

			swap = (unsigned *) realloc
			    (list - size, sizeof(unsigned) * (size << 1));
			if (!swap) {
			    status = -1;
			    break;
			} else {
			    list = swap + size;
			}
		    }
		}
		string++;
		parse_decimal(list, &string);
		list++;
		size++;
	    }
	}
	if (status) {
	    if (list) {
		free(list - size);
		list = NULL;
	    }

	    break;
	} else {
	    if (list) {
		list -= size;
		if (size & (size - 1)) {
		    unsigned *swap;

		    swap = (unsigned *) realloc
			(list, sizeof(unsigned) * size);
		    if (swap) {
			list = swap;
		    }
		}
	    }
	}
	if (*string == '{' || *string == '(') {
	    do {
		const char *partial;
		int has_components = 0;
		const struct ffield_type *field_data = NULL;
		struct list_type *list_data;
		unsigned code;

		status = -1;

		partial = string;
		if (*partial == '{') {
		    partial++;
		    for (; *partial && *partial != '}'; partial++);
		    if (*partial) {
			partial++;
		    } else {
			break;
		    }
		    has_components = 1;
		    if (*partial & 128) {
			break;
		    }
		    if (partial[1] != '(') {
			break;
		    } else {
		    }
		    lookup_code(&field_data, ffield_data,
				*(unsigned char *) partial);
		    if (!field_data) {
			break;
		    }
		    if (field_data->print_field) {
			break;
		    }
		    if (!field_data->efield_data) {
			break;
		    }
		    code = *partial;

		    partial++;

		    efield_data = (const struct ffield_type *)
			field_data->efield_data;
		} else {
		    code = FORMAT;
		}
		if (add_field(xparser_data, code, flags, width, space,
			      space_length)) {
		    break;
		}
		list_data = xparser_data->tail_data;
		if (list) {
		    list_data->list = list;
		    list = NULL;
		    list_data->size = size;
		}
		if (has_components) {
		    if (add_items(xparser_data, string + 1)) {
			break;
		    }
		    list_data->flags |= DECOMPOSE;
		}
		xparser_data->specs = partial + 1;
		if (field_data || scope) {
		    COPY_FIELD__(list_data, field_data);
		    xparser_data->scope++;
		    if (frame_format
			(xparser_data, efield_data, parent,
			 frame_data)) {
			break;
		    }
		    xparser_data->scope--;
		} else {
		    if (parse_format(xparser_data, frame_data)) {
			break;
		    }
		}
		if (scope) {
		    list_data->flags |= INDEXED;
		    list_data->scope = scope;
		}
		partial = xparser_data->specs;
		string = partial;

		status = 0;
	    } while (0);
	} else if (*string & 128) {
	    status = -1;
	} else {
	    const struct ffield_type *field_data;

	    lookup_code
		(&field_data, ffield_data, *(unsigned char *) string);
	    if (!field_data) {
		status = -1;
		break;
	    }
	    if (!field_data->print_field) {
		do {
		    const char *partial;
		    struct list_type *list_data;
		    unsigned code;

		    status = -1;

		    partial = string;
		    code = *partial;
		    if (!code || partial[1] != '(') {
			break;
		    }
		    if (!field_data->efield_data) {
			break;
		    }
		    efield_data = (const struct ffield_type *)
			field_data->efield_data;
		    flags |= field_data->flags & ITERATE;
		    if (add_field(xparser_data, code, flags, width,
				  space, space_length)) {
			break;
		    }
		    list_data = xparser_data->tail_data;
		    if (list) {
			list_data->list = list;
			list = NULL;
			list_data->size = size;
		    }
		    xparser_data->specs = partial + 2;
		    {
			COPY_FIELD_0(list_data, field_data);
			xparser_data->scope++;
			if (frame_format
			    (xparser_data, efield_data, parent,
			     frame_data)) {
			    break;
			}
			xparser_data->scope--;
		    }
		    partial = xparser_data->specs;
		    list_data->flags |= CONDITIONAL;
		    if (scope) {
			list_data->flags |= INDEXED;
			list_data->scope = scope;
		    }
		    if (!(flags & ITERATE)) {
			if (partial[1] == '(') {
			    if (add_field(xparser_data, OTHERWISE, 0,
					  0, NULL, 0)) {
				break;
			    }
			    xparser_data->specs = partial + 2;
			    if (parse_format
				(xparser_data, frame_data)) {
				break;
			    }
			    partial = xparser_data->specs;
			}
		    }
		    string = partial;

		    status = 0;
		} while (0);
	    } else {
		if (field_data->efield_data) {
		    status = -1;
		} else {
		    struct list_type *list_data;

		    status = add_field(xparser_data, *string, flags,
				       width, space, space_length);
		    list_data = xparser_data->tail_data;
		    if (list) {
			list_data->list = list;
			list = NULL;
			list_data->size = size;
		    }
		    COPY_FIELD_1(list_data, field_data);
		}
	    }
	}
	{
	    {
		xparser_data->scope += scope;
	    }
	}
    } while (0);
    if (status) {
	if (list) {
	    free(list);
	}
    } else {
	string++;
	xparser_data->specs = string;
    }

    return status;
}


static int
frame_format(struct xparser_type *xparser_data,
	     const struct ffield_type *field_data, struct frame_type *parent,
	     struct frame_type *previous)
{
    int status;
    struct frame_type frame_data;

    frame_data.field_data = field_data;
    frame_data.parent = parent;
    frame_data.previous = previous;

    status = parse_format(xparser_data, &frame_data);

    return status;
}


static int
parse_format(struct xparser_type *xparser_data, struct frame_type *frame_data)
{
    const char *string;
    int status = 0;

    xparser_data->level++;

    string = xparser_data->specs;

    while (1) {
	if (!*string) {
	    if (xparser_data->level) {
		status = -1;
	    }
	    break;
	} else if (*string == '%') {
	    string++;
	    if (*string == '%') {
		status = add_field(xparser_data, 0x25, 0, 0, NULL, 0);
		if (status) {
		    break;
		}
		string++;
	    } else {
		xparser_data->specs = string;
		status = field_format(xparser_data, frame_data);
		if (status) {
		    break;
		} else {
		    string = xparser_data->specs;
		}
	    }
	} else if (*string == '(') {
	    status = -1;
	    break;
	} else if (*string == ')') {
	    if (xparser_data->level) {
		status = add_field(xparser_data, RETURN, 0, 0, NULL, 0);
		if (status) {
		} else {
		    xparser_data->specs = string;
		}
	    } else {
		status = -1;
	    }
	    break;
	} else {
	    const char *partial;

	    for (partial = string; *partial; partial++) {
		if (*partial == '\\') {
		    if (!partial[1]) {
			status = -1;
			break;
		    } else {
			partial++;
		    }
		} else if (*partial == '%' || *partial == '('
			   || *partial == ')') {
		    break;
		}
	    }
	    if (status) {
		break;
	    } else {
		status = add_string(xparser_data, string, partial - string);
		if (status) {
		    break;
		} else {
		    status = add_field(xparser_data, STRING, 0, 0, NULL, 0);
		    if (status) {
			break;
		    } else {
			string = partial;
		    }
		}
	    }
	}
    }

    xparser_data->level--;

    return status;
}


static unsigned
get_width(struct xparser_type *xparser_data)
{
    struct list_type *list_data;
    struct string_type *string_data;
    struct xitem_type *xitem_data;
    unsigned ing = 0;
    void *printer;

    xparser_data->scope++;

    printer = xparser_data->printer;

    string_data = xparser_data->head_string;
    xitem_data = xparser_data->head_xitem;
    for (list_data = xparser_data->head_data; list_data;
	 list_data = list_data->list_data) {
	unsigned code;

	code = list_data->code;
	if (code == 0x25) {
	    ing += xparser_data->fix_ailes(printer, "%", 1);
	} else if (code < 256) {
	    unsigned width;

	    if (list_data->space) {
		string_data = string_data->string_data;
	    }
	    width = list_data->width;
	    if (!(list_data->flags & SPLIT)) {
		unsigned size;

		if (!xparser_data->record) {
		    COUNT_FIELD(printer, list_data, size);
		} else if (RUN_WIDTH(list_data, xparser_data)) {
		    list_data->r_stamp = WIDTH_STAMP | xparser_data->r_stamp;
		    COUNT_FIELD(printer, list_data, size);
		    list_data->count_x_width = size;
		} else {
		    size = list_data->count_x_width;
		}

		ing += size < width ? width : size;
	    } else {
		int decompose, iterate, record;
		struct item_type *item_data = NULL;
		unsigned from, limit = ~0, scope, to;

		if (list_data->flags & INDEXED) {
		    scope = list_data->scope;
		    xparser_data->scope -= scope;
		    xparser_data->set_scope(printer, xparser_data->scope);
		} else {
		    scope = 0;
		}

		record = xparser_data->record;

		decompose = list_data->flags & DECOMPOSE;
		iterate = list_data->flags & ITERATE;
		if (decompose) {
		    item_data = xitem_data->item_data;
		    xitem_data = xitem_data->xitem_data;
		    from = item_data->from;
		    to = item_data->to;
		    item_data = item_data->item_data;

		    xparser_data->record = 0;
		} else if (iterate) {
		    from = 0;
		    to = ~0;

		    xparser_data->record = 0;
		} else {
		    from = to = 0;
		}

		if (list_data->field.count_field) {
		    if (!xparser_data->record) {
			limit = list_data->field.count_field(printer, INDEXED);
		    } else if (RUN_COUNT(list_data, xparser_data)) {
			list_data->r_stamp =
			    COUNT_STAMP | xparser_data->r_stamp;
			limit = list_data->field.count_field(printer, 0);
			list_data->count_x_width = limit;
		    } else {
			limit = list_data->count_x_width;
		    }
		}

		do {
		    int skip = 0;
		    unsigned current, ink = 1;

		    current = from;
		    if (decompose | iterate) {
			if (from < to) {
			    if (from < limit) {
				if (!~limit) {
				    to++;
				} else if (to < limit) {
				    to++;
				} else {
				    to = limit;
				}
			    } else {
				skip = 1;
			    }
			} else {
			    if (!~limit) {
				ink = -1;
				to--;
			    } else if (to < limit) {
				ink = -1;
				to--;
				if (from < limit) {
				} else {
				    from = limit - 1;
				}
			    } else {
				skip = 1;
			    }
			}
		    } else {
			if (limit) {
			    to++;
			} else {
			    skip = 1;
			}
		    }
		    if (skip) {
			xparser_data->head_data = list_data->list_data;
			xparser_data->head_string = string_data;
			xparser_data->head_xitem = xitem_data;

			skip_format(xparser_data);
		    } else {
			for (current = from; current != to; current += ink) {
			    unsigned size;

			    xparser_data->head_data = list_data->list_data;
			    xparser_data->head_string = string_data;
			    xparser_data->head_xitem = xitem_data;

			    if (decompose | iterate) {
				xparser_data->set_index(printer, current);
			    }

			    size = get_width(xparser_data);
			    ing += size < width ? width : size;
			}
		    }
		    if (item_data) {
			from = item_data->from;
			to = item_data->to;
			item_data = item_data->item_data;
		    } else {
			break;
		    }
		} while (1);

		if (decompose | iterate) {
		    list_data->field.count_field(printer, UNINDEXED);
		} else {
		    if (xparser_data->head_data->list_data
			&& (xparser_data->head_data->list_data->code
			    == OTHERWISE)) {
			xparser_data->head_data =
			    xparser_data->head_data->list_data->list_data;

			if (limit) {
			    skip_format(xparser_data);
			} else {
			    unsigned size;

			    size = get_width(xparser_data);
			    ing += size < width ? width : size;
			}
		    }
		}

		list_data = xparser_data->head_data;
		string_data = xparser_data->head_string;
		xitem_data = xparser_data->head_xitem;

		xparser_data->record = record;

		if (scope) {
		    xparser_data->get_scope(printer, xparser_data->scope);
		    xparser_data->scope += scope;
		}
	    }
	} else if (code == FORMAT) {
	    {
		unsigned scope, size, width;

		scope = list_data->scope;
		xparser_data->scope -= scope;

		if (list_data->space) {
		    string_data = string_data->string_data;
		}

		xparser_data->head_data = list_data->list_data;
		xparser_data->head_string = string_data;
		xparser_data->head_xitem = xitem_data;

		width = list_data->width;
		size = get_width(xparser_data);
		ing += size < width ? width : size;

		list_data = xparser_data->head_data;
		string_data = xparser_data->head_string;
		xitem_data = xparser_data->head_xitem;

		xparser_data->scope += scope;
	    }
	} else if (code == RETURN) {
	    {
		xparser_data->head_data = list_data;
		xparser_data->head_string = string_data;
		break;
	    }
	} else {
	    {
		ing += xparser_data->fix_ailes
		    (printer, string_data->data, string_data->size);
		string_data = string_data->string_data;
	    }
	}
    }

    xparser_data->scope--;

    return ing;
}


#if __ENABLE_PRINT__
static void
flush_user(FILE *file, int three, unsigned *buffered)
{
    if (~*buffered) {
	unsigned c;

	c = *buffered;
	if (three) {
	    putc('\\', file);
	    putc(((c >> 6) & 0x07) + 0x30, file);
	    putc(((c >> 3) & 0x07) + 0x30, file);
	    putc(((c >> 0) & 0x07) + 0x30, file);
	} else if (7 < c) {
	    putc('\\', file);
	    putc(((c >> 3) & 0x07) + 0x30, file);
	    putc(((c >> 0) & 0x07) + 0x30, file);
	} else {
	    putc('\\', file);
	    putc(((c >> 0) & 0x07) + 0x30, file);
	}
	*buffered = ~0;
    }
}
#endif				/* __ENABLE_PRINT__ */


#if !__USE_LIBx1f4__
static void
free_item(struct item_type **item_data)
{
    struct item_type *current_item_data, *head_item_data;

    head_item_data = *item_data;

    while (head_item_data) {
	current_item_data = head_item_data;
	head_item_data = head_item_data->item_data;
	free(current_item_data);
    }

    *item_data = NULL;
}
#endif				/* !__USE_LIBx1f4__ */


static void
lookup_code(const struct ffield_type **field_data,
	    const struct ffield_type *ffield_data, unsigned code)
{
    const struct ffield_type *efield_data = NULL;
    unsigned index;

    index = ffield_data->index;
    while (index)  {
	if (index == code) {
	    efield_data = ffield_data;
	    break;
	} else {
	    ffield_data++;
	    index = ffield_data->index;
	}
    }

    *field_data = efield_data;
}


static void
parse_decimal(unsigned *cardinal, const char **e)
{
    const char *s;
    int c;
    unsigned l;

    s = *e;

    c = *(const unsigned char *) s;
    l = c - '0';
    s++;
    c = *(const unsigned char *) s;
    while (isddigit(c)) {
	if (l < UINT_MAX / 10) {
	    l *= 10;
	} else {
	    l = UINT_MAX;
	}
	c -= '0';
	if (l < UINT_MAX - c) {
	    l += c;
	} else {
	    l = UINT_MAX;
	}
	s++;
	c = *(const unsigned char *) s;
    }

    *e = s;

    *cardinal = l;
}


#if __ENABLE_PRINT__
static void
print_format(FILE *file, struct xparser_type *xparser_data)
{
    struct list_type *list_data;
    struct string_type *string_data;
    struct xitem_type *xitem_data;
    unsigned level, this_level;

    this_level = xparser_data->level;
    level = ++(xparser_data->level);

    string_data = xparser_data->head_string;
    xitem_data = xparser_data->head_xitem;
    for (list_data = xparser_data->head_data; list_data;
	 list_data = list_data->list_data) {
	unsigned code;

	for (code = level; code; code--) {
	    fputc('\t', file);
	}
	code = list_data->code;
	if (code == 0x25) {
	    fputs("%\n", file);
	} else if (code < 256) {
	    int subsequent = 0;

	    fprintf(file, "FIELD: <%c %04o> (", code, code);
	    if (list_data->flags & ALTERNATE) {
		fputs("ALTERNATE", file);
		subsequent = 1;
	    }
	    if (list_data->flags & DECOMPOSE) {
		if (subsequent) {
		    fputs(" | ", file);
		} else {
		    subsequent = 1;
		}
		fputs("DECOMPOSE", file);
	    }
	    fputs(", ", file);
	    if (list_data->space) {
		fputc('<', file);
		print_string(file, string_data->data, string_data->size);
		fputc('>', file);
		string_data = string_data->string_data;
	    }
	    if (list_data->flags & NEGATE) {
		fputc('-', file);
	    }
	    fprintf(file, "%u)", list_data->width);

	    if (list_data->flags & (CONDITIONAL | DECOMPOSE)) {
		if (list_data->flags & DECOMPOSE) {
		    fputs(" { ", file);
		    print_item(file, xitem_data->item_data);
		    fputs(" }\n", file);
		    xitem_data = xitem_data->xitem_data;
		} else {
		    fputc('\n', file);
		}

		xparser_data->head_data = list_data->list_data;
		xparser_data->head_string = string_data;
		xparser_data->head_xitem = xitem_data;
		xparser_data->level = level;

		print_format(file, xparser_data);

		list_data = xparser_data->head_data;

		if (list_data->list_data
		    && list_data->list_data->code == OTHERWISE) {
		    for (code = level; code; code--) {
			fputc('\t', file);
		    }
		    fputs("OTHERWISE:\n", file);

		    xparser_data->head_data =
			list_data->list_data->list_data;

		    print_format(file, xparser_data);

		    list_data = xparser_data->head_data;
		}

		level = xparser_data->level;
		list_data = xparser_data->head_data;
		string_data = xparser_data->head_string;
		xitem_data = xparser_data->head_xitem;
	    } else {
		fputc('\n', file);
	    }
	} else {
	    switch (code) {
	    case FORMAT:
		fputs("FORMAT: (", file);
		if (list_data->flags & ALTERNATE) {
		    fputs("ALTERNATE", file);
		}
		fputs(", ", file);
		if (list_data->space) {
		    fputc('<', file);
		    print_string(file, string_data->data, string_data->size);
		    fputc('>', file);
		    string_data = string_data->string_data;
		}
		if (list_data->flags & NEGATE) {
		    fputc('-', file);
		}
		fprintf(file, "%u)\n", list_data->width);
		level++;
		break;
	    case RETURN:
		level--;
		fputs("RETURN\n", file);

		if (level == this_level) {
		    xparser_data->head_data = list_data;
		    xparser_data->head_string = string_data;
		    xparser_data->head_xitem = xitem_data;
		    xparser_data->level = level;

		    return;
		}

		break;
	    case STRING:
		fputs("STRING: <", file);
		print_string(file, string_data->data, string_data->size);
		fputs(">\n", file);
		string_data = string_data->string_data;
	    }
	}
    }

    xparser_data->level = --level;
}
#endif				/* __ENABLE_PRINT__ */


#if __ENABLE_PRINT__ && !__USE_LIBx1f4__
void
print_item(FILE * file, struct item_type *item_data)
{
    struct item_type *head_item_data;

    head_item_data = item_data;

    while (head_item_data) {
	fprintf(file, "%u", head_item_data->from);
	if (head_item_data->from != head_item_data->to) {
	    fprintf(file, "..%u", head_item_data->to);
	}
	head_item_data = head_item_data->item_data;
	if (head_item_data) {
	    fputs(", ", file);
	}
    }
}
#endif				/* __ENABLE_PRINT__ && !__USE_LIBx1f4__ */


#if __ENABLE_PRINT__
static void
print_string(FILE *file, char *data, unsigned size)
{
    unsigned buffered = ~0;

    while (*data) {
	put_user(file, *data++, "\\n", &buffered);
    }
    flush_user(file, 0, &buffered);
}
#endif				/* __ENABLE_PRINT__ && !__USE_LIBx1f4__ */


#if __ENABLE_PRINT__
static void
put_user(FILE *file, unsigned byte, char *new_line, unsigned *buffered)
{
    if (byte < 0x07 || (0x0d < byte && byte < 0x20) || 0x7e < byte) {
	flush_user(file, 0, buffered);
	if (byte < 64) {
	    *buffered = byte;
	} else {
	    putc('\\', file);
	    putc(((byte >> 6) & 0x07) + 0x30, file);
	    putc(((byte >> 3) & 0x07) + 0x30, file);
	    putc(((byte >> 0) & 0x07) + 0x30, file);
	}
    } else {
	flush_user(file, 0x2f < byte && byte < 0x38, buffered);
	if (byte == 0x07) {
	    fputs("\\a", file);
	} else if (byte == 0x08) {
	    fputs("\\b", file);
	} else if (byte == 0x09) {
	    fputs("\\t", file);
	} else if (byte == 0x0a) {
	    fputs(new_line, file);
	} else if (byte == 0x0b) {
	    fputs("\\v", file);
	} else if (byte == 0x0c) {
	    fputs("\\f", file);
	} else if (byte == 0x0d) {
	    fputs("\\r", file);
	} else if (byte == 0x22) {
	    fputs("\\\"", file);
	} else if (byte == 0x5c) {
	    fputs("\\\\", file);
	} else {
	    putc(byte, file);
	}
    }
}
#endif				/* __ENABLE_PRINT__ */


static void
run_field(struct xparser_type *xparser_data, struct list_type *list_data)
{
    unsigned after_space = 0, flags, width;
    void *printer;

    printer = xparser_data->printer;

    flags = list_data->flags;
    width = list_data->width;
    if (width) {
	unsigned size;

	if (!xparser_data->record) {
	    COUNT_FIELD(printer, list_data, size);
	} else if (RUN_WIDTH(list_data, xparser_data)) {
	    list_data->r_stamp = WIDTH_STAMP | xparser_data->r_stamp;
	    COUNT_FIELD(printer, list_data, size);
	    list_data->count_x_width = size;
	} else {
	    size = list_data->count_x_width;
	}

	if (size < width) {
	    if (flags & NEGATE) {
		after_space = width - size;
	    } else {
		xparser_data->put_space
		    (printer, width - size, list_data->space,
		     list_data->space_length);
	    }
	}
    }
    PRINT_FIELD(printer, list_data, flags);
    if (after_space) {
	xparser_data->put_space
	    (printer, after_space, list_data->space, list_data->space_length);
    }
}


static void
run_format(struct xparser_type *xparser_data)
{
    struct list_type *list_data;
    struct string_type *string_data;
    struct xitem_type *xitem_data;
    void *printer;

    xparser_data->scope++;

    printer = xparser_data->printer;

    string_data = xparser_data->head_string;
    xitem_data = xparser_data->head_xitem;
    for (list_data = xparser_data->head_data; list_data;
	 list_data = list_data->list_data) {
	unsigned code;

	code = list_data->code;
	if (code == 0x25) {
	    xparser_data->put_ailes(printer, "%", 1);
	} else if (code < 256) {
	    if (list_data->space) {
		string_data = string_data->string_data;
	    }
	    if (!(list_data->flags & SPLIT)) {
		run_field(xparser_data, list_data);
	    } else {
		int decompose, iterate, record;
		struct item_type *item_data = NULL;
		unsigned from, limit = ~0, scope, to;

		if (list_data->flags & INDEXED) {
		    scope = list_data->scope;
		    xparser_data->scope -= scope;
		    xparser_data->set_scope(printer, xparser_data->scope);
		} else {
		    scope = 0;
		}

		record = xparser_data->record;

		decompose = list_data->flags & DECOMPOSE;
		iterate = list_data->flags & ITERATE;
		if (decompose) {
		    item_data = xitem_data->item_data;
		    xitem_data = xitem_data->xitem_data;
		    from = item_data->from;
		    to = item_data->to;
		    item_data = item_data->item_data;

		    xparser_data->record = 0;
		} else if (iterate) {
		    from = 0;
		    to = ~0;

		    xparser_data->record = 0;
		} else {
		    from = to = 0;
		}

		if (list_data->field.count_field) {
		    if (!xparser_data->record) {
			limit = list_data->field.count_field(printer, INDEXED);
		    } else if (RUN_COUNT(list_data, xparser_data)) {
			list_data->r_stamp =
			    COUNT_STAMP | xparser_data->r_stamp;
			limit = list_data->field.count_field(printer, 0);
			list_data->count_x_width = limit;
		    } else {
			limit = list_data->count_x_width;
		    }
		}

		do {
		    int skip = 0;
		    unsigned current, ink = 1;

		    current = from;
		    if (decompose | iterate) {
			if (from < to) {
			    if (from < limit) {
				if (!~limit) {
				    to++;
				} else if (to < limit) {
				    to++;
				} else {
				    to = limit;
				}
			    } else {
				skip = 1;
			    }
			} else {
			    if (!~limit) {
				ink = -1;
				to--;
			    } else if (to < limit) {
				ink = -1;
				to--;
				if (from < limit) {
				} else {
				    from = limit - 1;
				}
			    } else {
				skip = 1;
			    }
			}
		    } else {
			if (limit) {
			    to++;
			} else {
			    skip = 1;
			}
		    }
		    if (skip) {
			xparser_data->head_data = list_data->list_data;
			xparser_data->head_string = string_data;
			xparser_data->head_xitem = xitem_data;

			skip_format(xparser_data);
		    } else {
			for (current = from; current != to; current += ink) {
			    xparser_data->head_data = list_data->list_data;
			    xparser_data->head_string = string_data;
			    xparser_data->head_xitem = xitem_data;

			    if (decompose | iterate) {
				xparser_data->set_index(printer, current);
			    }

			    run_subformat(xparser_data, list_data);
			}
		    }
		    if (item_data) {
			from = item_data->from;
			to = item_data->to;
			item_data = item_data->item_data;
		    } else {
			break;
		    }
		} while (1);

		if (decompose | iterate) {
		    list_data->field.count_field(printer, UNINDEXED);
		} else {
		    if (xparser_data->head_data->list_data
			&& (xparser_data->head_data->list_data->code
			    == OTHERWISE)) {
			xparser_data->head_data =
			    xparser_data->head_data->list_data->list_data;

			if (limit) {
			    skip_format(xparser_data);
			} else {
			    run_subformat(xparser_data, list_data);
			}
		    }
		}

		list_data = xparser_data->head_data;
		string_data = xparser_data->head_string;
		xitem_data = xparser_data->head_xitem;

		xparser_data->record = record;

		if (scope) {
		    xparser_data->get_scope(printer, xparser_data->scope);
		    xparser_data->scope += scope;
		}
	    }
	} else if (code == FORMAT) {
	    {
		unsigned scope;

		scope = list_data->scope;
		xparser_data->scope -= scope;

		if (list_data->space) {
		    string_data = string_data->string_data;
		}

		xparser_data->head_data = list_data->list_data;
		xparser_data->head_string = string_data;
		xparser_data->head_xitem = xitem_data;

		run_subformat(xparser_data, list_data);

		list_data = xparser_data->head_data;
		string_data = xparser_data->head_string;
		xitem_data = xparser_data->head_xitem;

		xparser_data->scope += scope;
	    }
	} else if (code == RETURN) {
	    {
		xparser_data->head_data = list_data;
		xparser_data->head_string = string_data;
		break;
	    }
	} else {
	    {
		xparser_data->put_ailes
		    (printer, string_data->data, string_data->size);
		string_data = string_data->string_data;
	    }
	}
    }

    xparser_data->scope--;
}


static void
run_subformat(struct xparser_type *xparser_data, struct list_type *list_data)
{
    unsigned after_space = 0, flags, width;
    void *printer;

    printer = xparser_data->printer;

    flags = list_data->flags;
    width = list_data->width;

    if (width) {
	struct list_type *save_list_data;
	struct string_type *save_string_data;
	struct xitem_type *save_xitem_data;
	unsigned size;

	save_list_data = xparser_data->head_data;
	save_string_data = xparser_data->head_string;
	save_xitem_data = xparser_data->head_xitem;

	size = get_width(xparser_data);

	xparser_data->head_data = save_list_data;
	xparser_data->head_string = save_string_data;
	xparser_data->head_xitem = save_xitem_data;

	if (size < width) {
	    if (flags & NEGATE) {
		after_space = width - size;
	    } else {
		xparser_data->put_space
		    (printer, width - size, list_data->space,
		     list_data->space_length);
	    }
	}
    }

    run_format(xparser_data);

    if (after_space) {
	xparser_data->put_space
	    (printer, after_space, list_data->space, list_data->space_length);
    }
}


static void
skip_format(struct xparser_type *xparser_data)
{
    struct list_type *list_data;
    struct string_type *string_data;
    struct xitem_type *xitem_data;
    unsigned level = 0;

    string_data = xparser_data->head_string;
    xitem_data = xparser_data->head_xitem;
    for (list_data = xparser_data->head_data; list_data;
	 list_data = list_data->list_data) {
	unsigned code;

	code = list_data->code;
	if (code == 0x25) {
	} else if (code < 256) {
	    if (list_data->space) {
		string_data = string_data->string_data;
	    }
	    if (list_data->flags & (CONDITIONAL | DECOMPOSE)) {
		if (list_data->flags & DECOMPOSE) {
		    xitem_data = xitem_data->xitem_data;
		}

		xparser_data->head_data = list_data->list_data;
		xparser_data->head_string = string_data;
		xparser_data->head_xitem = xitem_data;

		skip_format(xparser_data);

		list_data = xparser_data->head_data;

		if (list_data->list_data
		    && list_data->list_data->code == OTHERWISE) {
		    xparser_data->head_data =
			list_data->list_data->list_data;

		    skip_format(xparser_data);

		    list_data = xparser_data->head_data;
		}

		list_data = xparser_data->head_data;
		string_data = xparser_data->head_string;
		xitem_data = xparser_data->head_xitem;
	    } else {
	    }
	} else {
	    switch (code) {
	    case FORMAT:
		if (list_data->space) {
		    string_data = string_data->string_data;
		}
		level++;
		break;
	    case RETURN:
		level--;

		if (!~level) {
		    xparser_data->head_data = list_data;
		    xparser_data->head_string = string_data;
		    xparser_data->head_xitem = xitem_data;

		    return;
		}

		break;
	    case STRING:
		string_data = string_data->string_data;
	    }
	}
    }
}


void
x1f4_detail_format6(struct xparser_type *xparser_data, unsigned *size,
		    unsigned **list)
{
    *list = xparser_data->tail_data->list;
    *size = xparser_data->tail_data->size;
}


void
x1f4_free_format6(struct xparser_type **xparser_return)
{
    {
	struct list_type *head_list;

	head_list = (*xparser_return)->head_data;
	while (head_list) {
	    void *free_data;

	    free_data = head_list->list;
	    if (free_data) {
		free(free_data);
	    }
	    free_data = head_list;
	    head_list = head_list->list_data;
	    free(free_data);
	}
    }

    {
	struct xitem_type *head_xitem;

	head_xitem = (*xparser_return)->head_xitem;
	while (head_xitem) {
	    void *free_data;

	    free_data = head_xitem;
	    head_xitem = head_xitem->xitem_data;
	    free_item((struct item_type **)
		      &((struct xitem_type *) free_data)->item_data);
	    free(free_data);
	}
    }

    {
	struct string_type *head_string;

	head_string = (*xparser_return)->head_string;
	while (head_string) {
	    void *free_data;

	    free_data = head_string;
	    head_string = head_string->string_data;
	    free(free_data);
	}
    }

    free(*xparser_return);
}


int
x1f4_parse_format6(struct xparser_type **xparser_return,
		   const struct ffield_type *field_data,
		   const struct f6link_type *flink_data, const char *specs)
{
    int status;
    struct xparser_type *xparser_data;

    if (!(xparser_data = (struct xparser_type *)
	  malloc(sizeof(struct xparser_type)))) {
	return -1;
    }

    xparser_data->head_data = NULL;
    xparser_data->head_string = NULL;
    xparser_data->head_xitem = NULL;
    xparser_data->level = -1;
    xparser_data->r_stamp = 0;
    xparser_data->record = 1;
    xparser_data->scope = 0;
    xparser_data->specs = specs;
    xparser_data->tail_data = NULL;
    xparser_data->tail_string = NULL;
    xparser_data->tail_xitem = NULL;
    status = frame_format(xparser_data, field_data, NULL, NULL);
    if (xparser_data->tail_data) {
	xparser_data->tail_data->list_data = NULL;
    }
    if (xparser_data->tail_xitem) {
	xparser_data->tail_xitem->xitem_data = NULL;
    }
    if (xparser_data->tail_string) {
	xparser_data->tail_string->string_data = NULL;
    }

    if (status) {
	x1f4_free_format6(&xparser_data);
    } else {
	if (xparser_data->head_data) {
	    xparser_data->fix_ailes = flink_data->fix_ailes;
	    xparser_data->get_scope = flink_data->get_scope;
	    xparser_data->put_space = flink_data->put_space;
	    xparser_data->put_ailes = flink_data->put_ailes;
	    xparser_data->set_index = flink_data->set_index;
	    xparser_data->set_scope = flink_data->set_scope;

	    *xparser_return = xparser_data;
	} else {
	    *xparser_return = NULL;

	    x1f4_free_format6(&xparser_data);
	}
    }

    return status;
}


#if __ENABLE_PRINT__
void
x1f4_print_format6(FILE *file, struct xparser_type *xparser_data)
{
    struct list_type *list_data;
    struct string_type *string_data;
    struct xitem_type *xitem_data;

    list_data = xparser_data->head_data;
    string_data = xparser_data->head_string;
    xitem_data = xparser_data->head_xitem;

    xparser_data->level = -1;
    print_format(file, xparser_data);

    xparser_data->head_data = list_data;
    xparser_data->head_string = string_data;
    xparser_data->head_xitem = xitem_data;
}
#endif				/* __ENABLE_PRINT__ */


void
x1f4_run_format6(struct xparser_type *xparser_data, void *printer)
{
    struct list_type *list_data;
    struct string_type *string_data;
    struct xitem_type *xitem_data;

    list_data = xparser_data->head_data;
    string_data = xparser_data->head_string;
    xitem_data = xparser_data->head_xitem;

    xparser_data->printer = printer;

    xparser_data->r_stamp += STAMP_BASE;

    xparser_data->scope = -1;

    run_format(xparser_data);

    xparser_data->head_data = list_data;
    xparser_data->head_string = string_data;
    xparser_data->head_xitem = xitem_data;

    if (!(xparser_data->r_stamp + STAMP_BASE)) {
	xparser_data->r_stamp = 0;
	for (; list_data; list_data = list_data->list_data) {
	    list_data->r_stamp = 0;
	}
    }
}
