/*
 * printp6.c
 * Copyright (C) 2006-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>

#define put_space(file, size) \
    fwrite(fill_space, (size), 1, (file))

typedef struct efield_type {
    const char *class;
    const struct efield_type *efield_data;
    unsigned (*count_field) (void *, unsigned), flags, index;
    void (*print_field) (void *, unsigned);
} efield_type;

typedef struct ffield_type {
    const char *class;
    const struct ffield_type *efield_data;
    unsigned (*count_field) (void *, unsigned), flags, index;
    void (*print_field) (void *, unsigned);
} ffield_type;

static void print_ffields(FILE *, unsigned, const struct ffield_type *);

static const char fill_space[] = "                                ";

static void
print_ffields(FILE *file, unsigned head, const struct ffield_type *ffield_data)
{
    const struct ffield_type *cfield_data;
    unsigned index;

    cfield_data = ffield_data;

    index = ffield_data->index;
    while (index) {
	int skip = 0;
	const struct ffield_type *efield_data;

	efield_data = ffield_data->efield_data;
	if (efield_data) {
	    const struct ffield_type *lfield_data;

	    lfield_data = cfield_data;
	    while (lfield_data != ffield_data) {
		const struct ffield_type *dfield_data;

		dfield_data = lfield_data->efield_data;
		if (dfield_data == efield_data) {
		    skip = 1;
		    break;
		}

		lfield_data++;
	    }
	}
	if (!skip) {
	    int repeat = 0;
	    unsigned space = 29;

	    put_space(file, head);

	    fputc(index, file);
	    if (efield_data) {
		const struct ffield_type *lfield_data;
		unsigned Index;

		lfield_data = ffield_data + 1;
		Index = lfield_data->index;
		while (Index) {
		    const struct ffield_type *dfield_data;

		    dfield_data = lfield_data->efield_data;
		    if (dfield_data == efield_data) {
			fputc(',', file);
			fputc(' ', file);
			fputc(Index, file);
			space -= 3;
			repeat = 1;
		    }

		    lfield_data++;
		    Index = lfield_data->index;
		}
	    }

	    {
		const char *class;

		put_space(file, space);
		class = ffield_data->class;
		if (class) {
		    fputs(class, file);
		}
		if (repeat) {
		    const struct ffield_type *lfield_data;
		    unsigned Index;

		    lfield_data = ffield_data + 1;
		    Index = lfield_data->index;
		    while (Index) {
			const struct ffield_type *dfield_data;

			dfield_data = lfield_data->efield_data;
			if (dfield_data == efield_data) {
			    fputc(',', file);
			    fputc(' ', file);
			    class = lfield_data->class;
			    if (class) {
				fputs(class, file);
			    }
			}

			lfield_data++;
			Index = lfield_data->index;
		    }
		}
	    }

	    fputc(00012, file);

	    {
		if (efield_data) {
		    print_ffields(file, head + 2, efield_data);
		}
	    }
	}

	ffield_data++;
	index = ffield_data->index;
    }
}


void
x1f4_pprint_ffields6(FILE *file, const struct ffield_type *ffield_data)
{
    print_ffields(file, 2, ffield_data);
}
