/*
 * salist.c
 * Copyright (C) 2006-2007, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <config.h>

#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#if SIZEOF_VOID_P == SIZEOF_LONG
# define integral_q			unsigned long
#else
# define integral_q			unsigned
#endif				/* SIZEOF_VOID_P == SIZEOF_LONG */

typedef struct sanode_type {
    struct sanode_type *last_sanode, *next_sanode;
    void *data;
} sanode_type;

typedef struct salist_type {
    struct sanode_type *sanode_data, *sanode_swap;
    unsigned base;
} salist_type;

int
x1f4_fini_salist(void **salist)
{
    struct salist_type *salist_data;
    struct sanode_type *sanode_data;

    salist_data = *salist;
    sanode_data = salist_data->sanode_data;
    while (sanode_data) {
	struct sanode_type *next_sanode;

	next_sanode = sanode_data;
	sanode_data = sanode_data->next_sanode;
	free(next_sanode->data);
    }
    sanode_data = salist_data->sanode_swap;
    while (sanode_data) {
	struct sanode_type *next_sanode;

	next_sanode = sanode_data;
	sanode_data = sanode_data->next_sanode;
	free(next_sanode->data);
    }

    free(salist_data);

    return 0;
}


int
x1f4_free_salist(void *salist, void *data)
{
    struct salist_type *salist_data;
    struct sanode_type *sanode_data;
    unsigned deck;

    salist_data = salist;

    deck = sizeof(struct sanode_type);
    deck += (16 - (deck & 15)) & 15;
    sanode_data = (void *) ((char *) data - deck);
    {
	struct sanode_type *last_sanode, *next_sanode;

	last_sanode = sanode_data->last_sanode;
	next_sanode = sanode_data->next_sanode;
	if (next_sanode) {
	    next_sanode->last_sanode = last_sanode;
	}
	if (last_sanode) {
	    last_sanode->next_sanode = next_sanode;
	} else {
	    salist_data->sanode_data = next_sanode;
	}
    }

    sanode_data->next_sanode = salist_data->sanode_swap;
    salist_data->sanode_swap = sanode_data;

    return 0;
}


int
x1f4_init_salist(void **salist, unsigned base)
{
    int status;
    struct salist_type *salist_data;

    salist_data = (struct salist_type *) malloc(sizeof(struct salist_type));
    if (!salist_data) {
	status = -1;
    } else {
	status = 0;

	salist_data->base = base;
	salist_data->sanode_data = NULL;
	salist_data->sanode_swap = NULL;

	*salist = salist_data;
    }

    return status;
}


int
x1f4_link_salist(void *salist, void **link)
{
    int status;
    struct salist_type *salist_data;
    struct sanode_type *sanode_data;
    unsigned deck;

    salist_data = salist;

    deck = sizeof(struct sanode_type);
    deck += (16 - (deck & 15)) & 15;

    sanode_data = salist_data->sanode_swap;
    if (sanode_data) {
	if (1) {
	    struct sanode_type *list_sanode;

	    status = 0;

	    list_sanode = salist_data->sanode_data;
	    salist_data->sanode_data = sanode_data;

	    salist_data->sanode_swap = sanode_data->next_sanode;

	    sanode_data->last_sanode = NULL;
	    sanode_data->next_sanode = list_sanode;
	    if (list_sanode) {
		list_sanode->last_sanode = sanode_data;
	    }

	    *link = (void *) ((char *) sanode_data + deck);
	}
    } else {
	unsigned flow, size;
	void *data;

	size = salist_data->base;

	flow = size + deck + 15;

	data = malloc(flow);
	if (!data) {
	    status = -1;
	} else {
	    struct sanode_type *list_sanode;

	    status = 0;

	    {
		integral_q post;
		unsigned skip;

		post = (integral_q) data;
		skip = post;
		skip = (16 - (skip & 15)) & 15;
		sanode_data = (void *) ((char *) data + skip);
	    }

	    list_sanode = salist_data->sanode_data;
	    salist_data->sanode_data = sanode_data;

	    sanode_data->last_sanode = NULL;
	    sanode_data->next_sanode = list_sanode;
	    if (list_sanode) {
		list_sanode->last_sanode = sanode_data;
	    }

	    sanode_data->data = data;

	    *link = (void *) ((char *) sanode_data + deck);
	}
    }

    return status;
}


int
x1f4_post_salist(void *salist, unsigned *size)
{
    struct salist_type *salist_data;
    struct sanode_type *sanode_data;
    unsigned deck = 0;

    salist_data = salist;
    sanode_data = salist_data->sanode_data;
    while (sanode_data) {
	sanode_data = sanode_data->next_sanode;
	deck++;
    }

    *size = deck;

    return 0;
}


int
x1f4_size_salist(void *salist, unsigned *size)
{
    struct salist_type *salist_data;
    struct sanode_type *sanode_data;
    unsigned deck = 0;

    salist_data = salist;
    sanode_data = salist_data->sanode_data;
    while (sanode_data) {
	sanode_data = sanode_data->next_sanode;
	deck++;
    }

    *size = deck * salist_data->base;

    return 0;
}
