/*
 * 1f42.c
 * Copyright (C) 2005-2010, 2015, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * endless, pointless, 1f42...
 */

#include <config.h>

#include <errno.h>
#if defined HAVE_LIBLEWD
# include <lewd.h>
#endif				/* HAVE_LIBLEWD */
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <1f44.h>
#if !defined HAVE_LIBLEWD
# include <basename.h>
#endif				/* !HAVE_LIBLEWD */
#include <access.h>
#include <defs.h>
#include <lcard.h>
#include <inter.h>
#include <types.h>

#if WORDS_BIGENDIAN
# define AUDIO_MASK			0xffffffe0
# define AUDIO_PREFIX			0x000001c0
# define AUDIO_STREAM_ID		0x000001c0
#else
# define AUDIO_MASK			0xe0ffffff
# define AUDIO_PREFIX			0xc0010000
# define AUDIO_STREAM_ID		0xc0010000
#endif				/* WORDS_BIGENDIAN */

#if WORDS_BIGENDIAN
# define AUDIO_A52A_MASK		0xfffffff8
# define AUDIO_A52A_PREFIX		0x00000080
# define AUDIO_A52A_TRACK		0x00000080
#else
# define AUDIO_A52A_MASK		0xf8ffffff
# define AUDIO_A52A_PREFIX		0x80000000
# define AUDIO_A52A_TRACK		0x80000000
#endif				/* WORDS_BIGENDIAN */

#if WORDS_BIGENDIAN
# define AUDIO_LPCM_MASK		0xffffffff
# define AUDIO_LPCM_PREFIX		0x000000a0
#else
# define AUDIO_LPCM_MASK		0xffffffff
# define AUDIO_LPCM_PREFIX		0xa0000000
#endif				/* WORDS_BIGENDIAN */

#if WORDS_BIGENDIAN
# define VIDEO_MASK			0xfffffff0
# define VIDEO_PREFIX			0x000001e0
# define VIDEO_STREAM_ID		0x000001e0
#else
# define VIDEO_MASK			0xf0ffffff
# define VIDEO_PREFIX			0xe0010000
# define VIDEO_STREAM_ID		0xe0010000
#endif				/* WORDS_BIGENDIAN */

#define A52A_CLASS			1
#define LPCM_CLASS			2
#define MPEG_CLASS			3

#define back_1f42			NULL
#define head_1f42			NULL

#if WORDS_BIGENDIAN
# define stream_id(stream_id)		((stream_id) & 0xff)
#else
# define stream_id(stream_id)		((stream_id) >> 030)
#endif				/* WORDS_BIGENDIAN */

#define false(e)			0

#define CARDINAL(buffer, base) \
    ((buffer)[(base) + 0] << 030 | (buffer)[(base) + 1] << 020		      \
     | (buffer)[(base) + 2] << 010 | (buffer)[(base) + 3] << 000)

#define Source(source) \
    ((struct source_type *) (source))->_

#define Inter6(inter6) \
    ((struct inter6_type *) (inter6))

#define VRECORDS(x1f42a) \
    (((struct x1f42a_type *) (x1f42a))->qrecords)

#if __A52A_LOADER__ == __EXTERNAL_LOADER__
# define _a52a_audio6			&cell_audio6
#else
# define _a52a_audio6
#endif				/* __A52A_LOADER__ == __EXTERNAL_LOADER__ */

typedef struct access_type {
    lcard_type stream, system;
    unsigned detail, temporal_reference;
} access_type;

typedef struct audio6_type {
    int (*clip) (void *, unsigned), (*copy) (void *, void *),
	(*dump) (void *, void *, unsigned *, void **),
	(*load) (void *, void *, unsigned *, void **),
	(*node) (void *, const void **), (*type) (void *, void *);
} audio6_type;

typedef struct inter6_type {
    const struct audio6_type *audio6_data;
    struct sequel_type sequel;
    struct source_type source;
    struct x1f42e_type x1f42e;
    unsigned temporal_reference;
#if __EXPAND_x1f42MAPS__
    void *access;
#endif				/* __EXPAND_x1f42MAPS__ */
    void *context, *device, *entry, *loader;
#if __EXPAND_x1f42MAPS__
    void *socket;
#endif				/* __EXPAND_x1f42MAPS__ */
    void *x1f42, *x1f44;
} inter6_type;

typedef struct x1f440_type {
    int fd;
    struct seek_1f44_type e66c;
} x1f440_type;

typedef struct label_type {
    int video;
    unsigned count, id3, next_index, start_index, temporal_reference;
} label_type;

#if __EXPAND_x1f42MAPS__
static int clip_port(struct inter6_type *, void *, unsigned, void **,
		     struct x1f42a_type *);
#endif				/* __EXPAND_x1f42MAPS__ */
static int fini_1f42(void *, unsigned, void **);
static int init_1f42(void *, unsigned, void **);
#if __ENABLE_x00696e6b__
static int line_1f42(void *, unsigned, void **);
#endif				/* __ENABLE_x00696e6b__ */
static int load_1f42(void *, void *, unsigned *, struct inter6_type *);
static int lock_port(struct inter6_type *, void *, unsigned, void **,
		     struct x1f42a_type *);
static int lose_1f44(struct inter6_type *, struct x1f42a_type *,
		     struct source_type *);
static int lose_66c0(struct inter6_type *);
static int lose_66c1(struct inter6_type *, struct source_type *);
#if __EXPAND_x1f42MAPS__
static int miss_lock(void *, unsigned, void *, void *);
#endif				/* __EXPAND_x1f42MAPS__ */
static int open_1f44(struct inter6_type *, struct x1f42a_type *,
		     struct source_type *);
static int open_66c0(struct inter6_type *, struct x1f42a_type *,
		     struct source_type *);
static int open_66c1(struct inter6_type *, struct x1f42a_type *,
		     struct source_type *);
static int page_miss(struct inter6_type *, void *, struct x1f42a_type *);
#if __EXPAND_x1f42MAPS__
static int plug_lock(struct inter6_type *, struct x1f42a_type *,
		     const struct target_type *);
#endif				/* __EXPAND_x1f42MAPS__ */
static int read_1f42(void *, void *, struct x1f42a_type **, unsigned *,
		     struct label_type *);
static int read_1f44(void *, void *, struct x1f42a_type *, unsigned,
		     unsigned *);
#if __EXPAND_x1f42MAPS__
static int read_entrIes(void *, void *, struct x1f42a_type *,
			struct label_type *, unsigned);
#endif				/* __EXPAND_x1f42MAPS__ */
static int read_entries(void *, void *, struct x1f42a_type *,
			struct label_type *);
static int read_id3x(void *, void *, void *);
static int read_miss(void *, void *, unsigned);
static int read_path(void *, void *, struct x1f42a_type *, unsigned);
#if __EXPAND_x1f42MAPS__
static int read_records(void *, void *, unsigned, struct x1f42a_type *,
			unsigned);
#endif				/* __EXPAND_x1f42MAPS__ */
static int seek_1f42(void *, void *, unsigned, void *);
#if __ENABLE_x00696e6b__
static int skip_1f44(void *, void *, unsigned, unsigned *);
#endif				/* __ENABLE_x00696e6b__ */
static int skip_name(void *, void *, unsigned);
static int skip_records(void *, void *, unsigned, struct x1f42a_type *,
			unsigned, struct label_type *);
static int skip_video(void *, void *, unsigned, unsigned, unsigned *,
		      unsigned char *);
static int type_1f42(void *, void *);
static int type_audio(void *, unsigned);
static int type_class(int, unsigned);
static int type_order(int, unsigned);
static int type_video(unsigned);
static int wind_1f42(void *, unsigned *, unsigned);

#if __ENABLE_x00696e6b__
static struct access_type *seek_data(unsigned, struct access_type *, unsigned);
#endif				/* __ENABLE_x00696e6b__ */

#if __EXPAND_x1f42MAPS__
static void clip_1f42(void *, struct inter6_type *, struct x1f42a_type *,
		      unsigned *, unsigned *);
#endif				/* __EXPAND_x1f42MAPS__ */
static void free_1f42(void *, void *, struct x1f42a_type *, unsigned);
static void free_path(void *, struct x1f42a_type *);
#if __EXPAND_x1f42MAPS__
static void less_1f42(struct inter6_type *, struct x1f42a_type *, unsigned *,
		      struct access_type **);
#endif				/* __EXPAND_x1f42MAPS__ */
static void pick_page(void *, unsigned *, unsigned, unsigned);
static void type_flow(struct inter6_type *, struct x1f42a_type *);
static void wrap_path(struct x1f42a_type *, unsigned, const char *);
#if __EXPAND_x1f42MAPS__
static void wrap_qrecords(struct x1f42a_type *);
#endif				/* __EXPAND_x1f42MAPS__ */

#if __LINK_CELL_LOADER__
static const struct audio6_type cell_audio6 = {
/* *INDENT-OFF* */
    Mylene1f4_clip_cell,
    Mylene1f4_copy_cell,
    Mylene1f4_dump_cell,
    Mylene1f4_load_cell,
    Mylene1f4_node_cell,
    Mylene1f4_type_cell
/* *INDENT-ON* */
};
#endif				/* __LINK_CELL_LOADER__ */
static const struct audio6_type lpcm_audio6 = {
/* *INDENT-OFF* */
    Mylene1f4_clip_lpcm,
    Mylene1f4_copy_lpcm,
    Mylene1f4_dump_lpcm,
    Mylene1f4_load_lpcm,
    Mylene1f4_node_lpcm,
    Mylene1f4_type_lpcm
/* *INDENT-ON* */
}, mpeg_audio6 = {
/* *INDENT-OFF* */
    Mylene1f4_clip_mpeg,
    Mylene1f4_copy_mpeg,
    Mylene1f4_dump_mpeg,
    Mylene1f4_load_mpeg,
    Mylene1f4_node_mpeg,
    Mylene1f4_type_mpeg
/* *INDENT-ON* */
};
static const struct device_type data_device = {
/* *INDENT-OFF* */
    seek_1f42
/* *INDENT-ON* */
};
static const struct target_type data_target = {
/* *INDENT-OFF* */
    back_1f42,
    NULL,
    NULL,
#if __ENABLE_FAST_INTER__
    NULL,
#endif				/* __ENABLE_FAST_INTER__ */
    fini_1f42,
    NULL,
    head_1f42,
    init_1f42,
#if __ENABLE_x00696e6b__
    line_1f42,
#endif				/* __ENABLE_x00696e6b__ */
    NULL,
#if __ENABLE_x00696e6b__
    NULL,
#endif				/* __ENABLE_x00696e6b__ */
#if __EXPAND_x1f42MAPS__
    NULL,
#endif				/* __ENABLE_x00696e6b__ */
    NULL,
    NULL,
    type_1f42,
#if !__LINK_CELL_LOADER__
    wind_1f42
#else
    wind_1f42,
    NULL
#endif				/* !__LINK_CELL_LOADER__ */
/* *INDENT-ON* */
};

#if __EXPAND_x1f42MAPS__
static int
clip_port(struct inter6_type *inter6_data, void *context, unsigned index,
	  void **storage, struct x1f42a_type *x1f42a_data)
{
    int status;
    const struct audio6_type *audio6_data;
    unsigned count, head, tail;

    clip_1f42(context, inter6_data, x1f42a_data, &head, &tail);

    audio6_data = inter6_data->audio6_data;

    status = audio6_data->load
	(context, &inter6_data->source, &count, &inter6_data->loader);
    if (!status) {
	const void *target;

	audio6_data->node(context, &target);
	status = ((const struct target_type *) target)->init
	    (inter6_data->loader, 0, storage);
	if (!status) {
	    status = page_miss(inter6_data, context, x1f42a_data);
	    if (status) {
		status = ((const struct target_type *) target)->fini
		    (inter6_data->loader, 0, storage);
	    } else {
		plug_lock(inter6_data, x1f42a_data, target);

		HEAD(context) = head;
		TAIL(context) = tail;

		inter6_data->x1f42e.index = index;

		audio6_data->clip(inter6_data->loader, tail - head);
	    }
	}

	if (status) {
	    audio6_data->dump
		(context, &inter6_data->source, &count, &inter6_data->loader);
	}
    }

    return status;
}
#endif				/* __EXPAND_x1f42MAPS__ */


static int
fini_1f42(void *loader, unsigned index, void **storage)
{
    int excess, status;
    struct inter6_type *inter6_data;
    void *context;

    inter6_data = loader;

    context = inter6_data->context;

    {
	const void *target;

	inter6_data->audio6_data->node(context, &target);
	status = ((const struct target_type *) target)->fini
	    (inter6_data->loader, 0, storage);
    }
    {
	struct source_type *source_data;

	source_data = &inter6_data->source;

	{
	    unsigned count;

	    excess = inter6_data->audio6_data->dump
		(context, source_data, &count, &inter6_data->loader);
	    if (!status) {
		status = excess;
	    }
	}

	X1f42(context) = NULL;

	{
	    {
		excess = lose_1f44
		    (inter6_data,
		     (struct x1f42a_type *) inter6_data->x1f42 + index,
		     source_data);
		if (!status) {
		    status = excess;
		}
	    }
	}
    }

    return status;
}


static int
init_1f42(void *loader, unsigned index, void **storage)
{
    int status;
    struct inter6_type *inter6_data;
    struct source_type *source_data;
    struct x1f42a_type *x1f42a_data;
    void *context;

    inter6_data = loader;

    x1f42a_data = inter6_data->x1f42;
    x1f42a_data += index;

    context = inter6_data->context;

    source_data = &inter6_data->source;

    status = open_1f44(inter6_data, x1f42a_data, source_data);
    if (!status) {
	if (!status) {
	    void *glass[2];

	    glass[0] = inter6_data;
	    glass[1] = (void *) &data_device;
	    inter6_data->entry = x1f42a_data;
	    inter6_data->loader = glass;

#if __EXPAND_x1f42MAPS__
	    inter6_data->access = NULL;
#endif				/* __EXPAND_x1f42MAPS__ */

	    X1f42(context) = x1f42a_data;

	    type_flow(inter6_data, x1f42a_data);

#if __EXPAND_x1f42MAPS__
# if __ENABLE_CONTROLLING_SERVER__
	    if ((MODE(context) & Mylene1f4_WINDOW_MASK)
		== Mylene1f4_SERVER_WINDOW) {
		status = clip_port
		    (inter6_data, context, index, storage, x1f42a_data);
	    } else {
# endif				/* __ENABLE_CONTROLLING_SERVER__ */
# if __ENABLE_x00696e6b__
		do {
		    {
			unsigned audio_entry;

			audio_entry = ENTRY2(context);
			if (~audio_entry) {
			    if (VRECORDS(x1f42a_data)) {
				status = clip_port
				    (inter6_data, context, index, storage,
				     x1f42a_data);
				break;
			    }
			}
		    }
# endif				/* __ENABLE_x00696e6b__ */
#endif				/* __EXPAND_x1f42MAPS__ */

		    status = lock_port
			(inter6_data, context, index, storage, x1f42a_data);
#if __EXPAND_x1f42MAPS__
# if __ENABLE_x00696e6b__
		} while (0);
# endif				/* __ENABLE_x00696e6b__ */
# if __ENABLE_CONTROLLING_SERVER__
	    }
# endif				/* __ENABLE_CONTROLLING_SERVER__ */
#endif				/* __EXPAND_x1f42MAPS__ */
	}

	if (status) {
	    lose_1f44(inter6_data, x1f42a_data, source_data);
	}
    }

    return status;
}


#if __ENABLE_x00696e6b__
static int
line_1f42(void *loader, unsigned index, void **x1f42a)
{
    struct inter6_type *inter6_data;
    struct x1f42a_type *x1f42a_data;

    inter6_data = loader;

    x1f42a_data = inter6_data->x1f42;
    x1f42a_data += index;

    *x1f42a = x1f42a_data;

    return 0;
}
#endif				/* __ENABLE_x00696e6b__ */


static int
load_1f42(void *context, void *source, unsigned *c_nentries,
	  struct inter6_type *inter6_data)
{
    int status;
    struct label_type label;
    struct x1f42a_type *entries;
    unsigned nentries;

#if __ENABLE_TAGGED_LIST__
    if (FLAGS(context) & Mylene1f4_BREAK_LIST) {
	label.id3 = 0;
    } else {
#endif				/* __ENABLE_TAGGED_LIST__ */
	label.id3 = 128;
#if __ENABLE_TAGGED_LIST__
    }
#endif				/* __ENABLE_TAGGED_LIST__ */
    if (SELECT(context) == Mylene1f4_PAGE_PICKUP) {
	label.start_index = START2(context);
    } else {
	label.start_index = 0;
    }

    status = read_1f42(context, source, &entries, &nentries, &label);
    if (!status) {
	if (label.id3) {
	    status = read_id3x(context, source, entries + nentries);
	    if (status) {
		free_1f42(context, source, entries, nentries);
		free(entries);
	    }
	}
    }
    if (!status) {
	const char *path;
	unsigned size;

	*c_nentries = nentries;

	inter6_data->x1f42 = entries;
	inter6_data->temporal_reference = label.temporal_reference;

	path = Source(source).s1.path;
	size = x1f4_basename(path) - path;

	for (; nentries; nentries--) {
#if __EXPAND_x1f42MAPS__
	    if (VRECORDS(entries)) {
# if __LINK_A52A_LOADER__
#  if __A52A_LOADER__ == __EXTERNAL_LOADER__
		if (entries->type != 0x000003cf
		    && entries->type != 0xf3c003cf) {
#  endif			/* __A52A_LOADER__ == __EXTERNAL_LOADER__ */
# endif				/* __LINK_A52A_LOADER__ */
		    wrap_qrecords(entries);
# if __LINK_A52A_LOADER__
#  if __A52A_LOADER__ == __EXTERNAL_LOADER__
		}
#  endif			/* __A52A_LOADER__ == __EXTERNAL_LOADER__ */
# endif				/* __LINK_A52A_LOADER__ */
	    }
#endif				/* __EXPAND_x1f42MAPS__ */
	    if (size) {
		if (!entries->nentries) {
		    if (entries->source.path != path) {
			if (*entries->source.path != 057) {
			    wrap_path(entries, size, path);
			}
		    }
		} else {
		    seek_1f44_wrap(&entries->source.e66c, size, path);
		}
	    }

	    entries++;
	}
    }

    return status;
}


static int
lock_port(struct inter6_type *inter6_data, void *context, unsigned index,
	  void **storage, struct x1f42a_type *x1f42a_data)
{
    int status;
    const struct audio6_type *audio6_data;
    unsigned count;

    audio6_data = inter6_data->audio6_data;

    status = audio6_data->load
	(context, &inter6_data->source, &count, &inter6_data->loader);
    if (!status) {
	const void *target;

	audio6_data->node(context, &target);
	status = ((const struct target_type *) target)->init
	    (inter6_data->loader, 0, storage);
	if (!status) {
	    status = page_miss(inter6_data, context, x1f42a_data);
	    if (status) {
		status = ((const struct target_type *) target)->fini
		    (inter6_data->loader, 0, storage);
	    } else {
		unsigned head, tail;

#if __ENABLE_x00696e6b__
		{
		    unsigned audio_entry;

		    audio_entry = ENTRY2(context);
		    if (~audio_entry) {
			head = 0;
			tail = x1f42a_data->frames;
		    } else {
#endif				/* __ENABLE_x00696e6b__ */
			tail = x1f42a_data->next_frame;
			if (x1f42a_data == inter6_data->x1f42) {
			    head = inter6_data->temporal_reference;
			} else {
			    head = x1f42a_data[-1].next_frame;
			}
#if __ENABLE_x00696e6b__
		    }
		}
#endif				/* __ENABLE_x00696e6b__ */

		HEAD(context) = head;
		TAIL(context) = tail;

		inter6_data->x1f42e.index = index;

		audio6_data->clip(inter6_data->loader, tail - head);
	    }
	}

	if (status) {
	    audio6_data->dump
		(context, &inter6_data->source, &count, &inter6_data->loader);
	}
    }

    return status;
}


static int
lose_1f44(struct inter6_type *inter6_data, struct x1f42a_type *x1f42a_data,
	  struct source_type *source_data)
{
    int status;

    if (!x1f42a_data->nentries) {
	status = lose_66c0(inter6_data);
    } else {
	status = lose_66c1(inter6_data, source_data);
    }

    return status;
}


static int
lose_66c0(struct inter6_type *inter6_data)
{
    int excess, status;
    struct x1f440_type *x1f44;
    struct source_type source;

    x1f44 = inter6_data->x1f44;

    source._.s1.fd = x1f44->fd;

    status = seek_1f44_clip(&x1f44->e66c);
    excess = Mylene1f4_close_file(inter6_data->context, &source);
    if (status) {
    } else {
	status = excess;
    }

    return status;
}


static int
lose_66c1(struct inter6_type *inter6_data, struct source_type *source_data)
{
    return seek_1f44_lose(source_data->_.s2.e66c);
}


#if __EXPAND_x1f42MAPS__
static int
miss_lock(void *inter, unsigned count, void *stream, void *system)
{
    struct access_type *access_data;
    struct inter6_type *inter6_data;
    struct socket_type *socket_data;
    struct x1f42a_type *x1f42a_data;
    unsigned artificial;

    inter6_data = inter;

    access_data = inter6_data->access;

    if (count) {
	count--;
    }
    if (access_data->temporal_reference < count) {
	count = access_data->temporal_reference;
    }

    access_data->detail = count;
    access_data->stream = *(lcard_type *) stream;
    access_data->system = *(lcard_type *) system;

    x1f42a_data = inter6_data->entry;

    artificial = x1f42a_data->artificial;

    artificial--;

    x1f42a_data->artificial = artificial;

    socket_data = inter6_data->socket;
    if (!artificial) {
	socket_data->call = NULL;
    } else {
	unsigned nrecords;

	nrecords = access_data - (struct access_type *) VRECORDS(x1f42a_data);
	nrecords = x1f42a_data->nrecords - nrecords - 1;
	for (; nrecords && ~access_data->detail; access_data++, nrecords--);
	if (~access_data->detail) {
	    socket_data->call = NULL;
	} else {
	    socket_data->page = access_data->temporal_reference;

	    inter6_data->access = access_data;
	}
    }

    return 0;
}
#endif				/* __EXPAND_x1f42MAPS__ */


static int
open_1f44(struct inter6_type *inter6_data, struct x1f42a_type *x1f42a_data,
	  struct source_type *source_data)
{
    int status;

    if (!x1f42a_data->nentries) {
	status = open_66c0(inter6_data, x1f42a_data, source_data);
    } else {
	status = open_66c1(inter6_data, x1f42a_data, source_data);
    }

    return status;
}


static int
open_66c0(struct inter6_type *inter6_data, struct x1f42a_type *x1f42a_data,
	  struct source_type *source_data)
{
    int status;

    do {
	struct x1f440_type *x1f44;

	x1f44 = inter6_data->x1f44;
	if (!x1f44) {
	    x1f44 = (struct x1f440_type *) malloc(sizeof(struct x1f440_type));
	    if (!x1f44) {
		status = -1;
		break;
	    } else {
		inter6_data->x1f44 = x1f44;
	    }
	}

	{
	    struct source_type source;
	    void *context;

	    context = inter6_data->context;

	    source._.s1.path = x1f42a_data->source.path;
	    status = Mylene1f4_open_file(context, &source);
	    if (!status) {
		do {
		    lcard_type offset;

		    {
			unsigned file_offset;

			file_offset = x1f42a_data->file_offset;
			if (!file_offset) {
			    LCARD_SET_CARD1(offset, 0);
			} else {
			    status = Mylene1f4_seek_file
				(context, &source, file_offset, SEEK_SET);
			    if (status != -1) {
				status = 0;
				LCARD_SET_CARD1(offset, file_offset);
			    } else {
				break;
			    }
			}
		    }

		    {
			int fd;
			struct seek_1f44_type *e66c;

			fd = source._.s1.fd;

			e66c = &x1f44->e66c;
			status = seek_1f44_pull(e66c, &fd, &offset);
			if (!status) {
			    source_data->_.s2.e66c = e66c;
			    LCARD_SET_CARD1
				(source_data->_.s2.size, source._.s1.size);
			    x1f44->fd = fd;
			}
		    }
		} while (0);

		if (status) {
		    Mylene1f4_close_file(context, &source);
		}
	    }
	}
    } while (0);

    return status;
}


static int
open_66c1(struct inter6_type *inter6_data, struct x1f42a_type *x1f42a_data,
	  struct source_type *source_data)
{
    int status;
    lcard_type offset;
    struct seek_1f44_type *e66c;

    e66c = &x1f42a_data->source.e66c;

    LCARD_SET_CARD1(offset, x1f42a_data->file_offset);
    status = seek_1f44_rail(e66c, &offset);
    if (!status) {
	source_data->_.s2.e66c = e66c;
	seek_1f44_size(e66c, &source_data->_.s2.size);
    }

    return status;
}


static int
page_miss(struct inter6_type *inter6_data, void *context,
	  struct x1f42a_type *x1f42a_data)
{
    int false = 0;

    do {
	if (1) {
	    unsigned frame_rate_0_d, frame_rate_0_n, frame_rate_1_d,
		frame_rate_1_n;

	    frame_rate_0_d = RELEASE(context);
	    frame_rate_0_n = SAMPLING(context) * CHANNELS(context) << 1;
	    frame_rate_1_d = x1f42a_data->frame_rate_d;
	    frame_rate_1_n = x1f42a_data->frame_rate_n;
	    if (!~(frame_rate_1_n & frame_rate_1_d)) {
		x1f42a_data->frame_rate_d = frame_rate_1_d;
		x1f42a_data->frame_rate_n = frame_rate_1_n;
	    } else {
		if (frame_rate_0_d * frame_rate_1_n
		    != frame_rate_0_n * frame_rate_1_d) {
		    false = 1;
		    Mylene1f4_fprintf_error
			(context, "%s: frame rate for `%s', entry %u (%u / %"
			 "u) does not match the 1f42 map recorded one (%u / "
			 "%u)\n", Mylene1f4(context),
			 Source(inter6_data->device).s1.path,
			 x1f42a_data
			 - (struct x1f42a_type *) inter6_data->x1f42,
			 frame_rate_0_n, frame_rate_0_d,
			 frame_rate_1_n, frame_rate_1_d);
		    break;
		}
	    }
	}
    } while (0);

    return false;
}


#if __EXPAND_x1f42MAPS__
static int
plug_lock(struct inter6_type *inter6_data, struct x1f42a_type *x1f42a_data,
	  const struct target_type *target_data)
{
    /*
     * 8192 of the tests in this function are pointless...
     */

    if (VRECORDS(x1f42a_data)) {
	int (*plug) (void *, void **);

	plug = target_data->plug;
	if (plug) {
	    if (x1f42a_data->artificial) {
		struct access_type *access_data;

		access_data = inter6_data->access;
		if (access_data) {
		    struct socket_type *socket_data;
		    void *socket;

		    plug(inter6_data->loader, &socket);

		    inter6_data->socket = socket;

		    socket_data = socket;

		    socket_data->back = inter6_data;
		    socket_data->call = miss_lock;
		    socket_data->page = access_data->temporal_reference;
		}
	    }
	}
    }

    return 0;
}
#endif				/* __EXPAND_x1f42MAPS__ */


static int
read_1f42(void *context, void *source, struct x1f42a_type **entries,
	  unsigned *nentries, struct label_type *label_data)
{
    int status = 1;

    do {
#if !__ENABLE_x00696e6b__
# define Count				count
#endif				/* !__ENABLE_x00696e6b__ */

	struct x1f42a_type *x1f42_tables;
	unsigned count, majOr, major, minOr, minor;
#if __ENABLE_x00696e6b__
	unsigned Count;
#endif				/* __ENABLE_x00696e6b__ */
#if __EXPAND_x1f42MAPS__
	unsigned audio_entry;
#endif				/* __EXPAND_x1f42MAPS__ */
	unsigned char buffer[0x10];

	status = Mylene1f4_read_file(context, source, buffer, 16);
	if (status == -1) {
	    break;
	} else {
	    if (status != 16) {
		status = 1;
		break;
	    } else {
		status = 1;
	    }
	}
	if (memcmp(buffer, "\61\146\64\62", 4)) {
	    break;
	}
	major = buffer[0x4] << 010 | buffer[0x5];
	minor = buffer[0x6] << 030 | buffer[0x7];
	if (major || (minor != 1 && minor != 2 && minor != 3)) {
	    break;
	}
	majOr = buffer[0x8] << 010 | buffer[0x9];
	minOr = buffer[0xa] << 010 | buffer[0xb];
	count = buffer[0xc] << 010 | buffer[0xd];
	if ((minOr && !majOr) || (!minOr && majOr) || !count) {
	    break;
	}
#if __EXPAND_x1f42MAPS__
# if __ENABLE_CONTROLLING_SERVER__
	if ((MODE(context) & Mylene1f4_WINDOW_MASK)
	    == Mylene1f4_SERVER_WINDOW) {
	    audio_entry = ~0;
#  if __ENABLE_x00696e6b__
	    Count = count;
#  endif			/* __ENABLE_x00696e6b__ */
	} else {
# endif				/* __ENABLE_CONTROLLING_SERVER__ */
# if __ENABLE_x00696e6b__
	    audio_entry = ENTRY2(context);
	    if (~audio_entry) {
		if (audio_entry < count) {
		    audio_entry = count - audio_entry;
		    Count = 1;
		} else {
		    break;
		}
	    } else {
		Count = count;
		audio_entry = 0;
	    }
# endif				/* __ENABLE_x00696e6b__ */
# if __ENABLE_CONTROLLING_SERVER__
	}
# endif				/* __ENABLE_CONTROLLING_SERVER__ */
#endif				/* __EXPAND_x1f42MAPS__ */
	if (memcmp((char *) buffer + 12, "\0\1\377\375", 4)) {
	    label_data->id3 = 0;
	}
	if (minor != 1) {
	    unsigned flags;

	    status = Mylene1f4_read_file(context, source, buffer, 16);
	    if (status == -1) {
		break;
	    } else {
		if (status != 16) {
		    status = 1;
		    break;
		}
	    }
	    flags = buffer[0];
	    if (minor == 2 && flags & 0100) {
		status = 1;
		break;
	    }
	    if (flags & 0300) {
		status = Mylene1f4_read_file(context, source, buffer, 16);
		if (status == -1) {
		    break;
		} else {
		    if (status != 16) {
			status = 1;
			break;
		    }
		}
		if (flags & 0200) {
		    if (!CARDINAL(buffer, 0x0) || !CARDINAL(buffer, 0x4)) {
			status = 1;
			break;
		    }
		}
		if (flags & 0100) {
		    unsigned h, w, x, y;

		    h = buffer[0xe] << 010 | buffer[0xf];
		    w = buffer[0xc] << 010 | buffer[0xd];
		    x = buffer[0x8] << 010 | buffer[0x9];
		    y = buffer[0xa] << 010 | buffer[0xb];
		    if (!h || !w || majOr < w + x || minOr < h + y) {
			status = 1;
			break;
		    }
		}
	    }
	} else {
	}

	if (!(x1f42_tables = (struct x1f42a_type *)
	      malloc(Count * sizeof(struct x1f42a_type) + label_data->id3))) {
	    status = -1;

	    Mylene1f4_fprintf_error
		(context, "%s: %s\n", Mylene1f4(context), strerror(errno));

	    break;
	}

	memset(x1f42_tables, 0, Count * sizeof(struct x1f42a_type));

	label_data->count = count;
	label_data->video = majOr | minOr;

#if __EXPAND_x1f42MAPS__
	if (audio_entry) {
	    status = read_entrIes
		(context, source, x1f42_tables, label_data, audio_entry);
	} else {
#endif				/* __EXPAND_x1f42MAPS__ */
	    status = read_entries(context, source, x1f42_tables, label_data);
#if __EXPAND_x1f42MAPS__
	}
#endif				/* __EXPAND_x1f42MAPS__ */
	if (status) {
	    free(x1f42_tables);
	} else {
	    *entries = x1f42_tables;
	    *nentries = label_data->count;
	}

#if !__ENABLE_x00696e6b__
# undef Count
#endif				/* !__ENABLE_x00696e6b__ */
    } while (0);

    return status;
}


static int
read_1f44(void *context, void *source, struct x1f42a_type *x1f42a_data,
	  unsigned nentries, unsigned *size)
{
    int status;
    unsigned head;

    status = Mylene1f4_tell_file(context, source, &head);
    if (!status) {
	void *miss[2];

	miss[0] = context;
	miss[1] = source;
	status = seek_1f44_read
	    (&x1f42a_data->source.e66c, nentries, miss, read_miss);
	if (status) {
	    x1f42a_data->source.path = NULL;
	} else {
	    unsigned tail;

	    x1f42a_data->nentries = nentries;

	    status = Mylene1f4_tell_file(context, source, &tail);
	    if (!status) {
		*size = tail - head;
	    }
	}
    }

    return status;
}


#if __EXPAND_x1f42MAPS__
static int
read_entrIes(void *context, void *source, struct x1f42a_type *x1f42_tables,
	     struct label_type *label_data, unsigned audio_entry)
{
    int status = 0;
    unsigned audio_frames = 0, count, progress = 0;
    unsigned char buffer[0x10];

    if (SELECT(context) == Mylene1f4_PAGE_PICKUP) {
	label_data->next_index = START2(context);
    } else {
	label_data->next_index = 0;
    }

    count = label_data->count;
    for (; count; count--) {
# if __ENABLE_x00696e6b__
	int do_skip;
# endif				/* __ENABLE_x00696e6b__ */
	unsigned entry_length, flags, label_length, shift_length, type;

# if __ENABLE_x00696e6b__
	if (~audio_entry) {
	    do_skip = count != audio_entry;
	} else {
	    do_skip = 0;
	}
# endif				/* __ENABLE_x00696e6b__ */

	status = Mylene1f4_read_file(context, source, buffer, 16);
	if (status == -1) {
	    break;
	} else {
	    if (status != 16) {
		status = 1;
		break;
	    }
	}
	if (memcmp(&buffer[0xc], "\377\377", 2)) {
	    label_length = 65535 - (buffer[0xc] << 010 | buffer[0xd]);
	} else {
	    label_length = 0;
	}
	type = CARDINAL(buffer, 0x4);
	if (!label_data->video && type) {
	    status = 1;
	    break;
	} else if (!type) {
	} else if (type_video(type)) {
	} else {
	    status = 1;
	    break;
	}
	if (label_length) {
	    if (skip_name(context, source, label_length) == -1) {
		status = 1;
		break;
	    }
	}
	entry_length = CARDINAL(buffer, 0x0);
	flags = CARDINAL(buffer, 0x8);
	if (flags & 7) {
	    status = 1;
	    break;
	}
	if (flags & 0x3fffff80) {
	    status = 1;
	    break;
	}
	if (!type && flags & 0x8000005f) {
	    status = 1;
	    break;
	}
	if (flags & 32) {
	    unsigned nentries;

	    nentries = buffer[0xe] << 010 | buffer[0xf];
	    if (nentries < 2) {
		status = 1;
		break;
	    }
	    {
# if __ENABLE_x00696e6b__
		if (do_skip) {
		    if (skip_1f44(context, source, nentries, &shift_length)) {
			status = 1;
			break;
		    }
		} else {
# endif				/* __ENABLE_x00696e6b__ */
		    status = read_1f44
			(context, source, x1f42_tables, nentries,
			 &shift_length);
		    if (status) {
			break;
		    }
# if __ENABLE_x00696e6b__
		}
# endif				/* __ENABLE_x00696e6b__ */
		{
		    if (1) {
			label_data->id3 = 0;
		    }
		}
	    }
	} else {
	    shift_length = buffer[0xe] << 010 | buffer[0xf];
	    if (shift_length) {
# if __ENABLE_x00696e6b__
		if (do_skip) {
		    if (skip_name(context, source, shift_length) == -1) {
			status = 1;
			break;
		    }
		} else {
# endif				/* __ENABLE_x00696e6b__ */
		    status = read_path
			(context, source, x1f42_tables, shift_length);
		    if (status) {
			break;
		    }
# if __ENABLE_x00696e6b__
		}
# endif				/* __ENABLE_x00696e6b__ */
		{
		    if (1) {
			label_data->id3 = 0;
		    }
		    {
			shift_length += (16 - (shift_length & 15)) & 15;
		    }
		}
	    } else {
# if __ENABLE_x00696e6b__
		if (do_skip) {
		} else {
# endif				/* __ENABLE_x00696e6b__ */
		    x1f42_tables->source.path = Source(source).s1.path;
# if __ENABLE_x00696e6b__
		}
# endif				/* __ENABLE_x00696e6b__ */
	    }
	}
	entry_length -= 12 + shift_length + label_length
	    + ((16 - (label_length & 15)) & 15) + 16;
	if ((int) entry_length < 0) {
	    status = 1;
	    break;
	}
	status = Mylene1f4_read_file(context, source, buffer, 16);
	if (status == -1) {
	    break;
	} else {
	    if (status != 16) {
		status = 1;
		break;
	    }
	}
	status = skip_video
	    (context, source, type, flags, &entry_length, buffer);
	if (status) {
	    break;
	}
	if (type) {
	    if (1) {
		label_data->id3 = 0;
	    }
	}
	if (flags & (1L << 0x1e)) {
	    int class;
	    unsigned audio_stream_id, frames, nrecords;

	    if (entry_length < 32) {
		status = 1;
		break;
	    }
	    status = Mylene1f4_read_file(context, source, buffer, 16);
	    if (status == -1) {
		break;
	    } else {
		if (status != 16) {
		    status = 1;
		    break;
		}
	    }
	    if (memcmp(&buffer[0xc], "\377\377", 2)) {
		status = 1;
		break;
	    }
	    type = CARDINAL(buffer, 0x4);
	    class = type_audio(context, type);
	    if (class) {
	    } else {
		status = 1;
		break;
	    }
	    flags = CARDINAL(buffer, 0x8);
	    if (flags & 7) {
		status = 1;
		break;
	    }
	    if (flags & 0x7fffff80) {
		status = 1;
		break;
	    }
	    status = Mylene1f4_read_file(context, source, buffer, 16);
	    if (status == -1) {
		break;
	    } else {
		if (status != 16) {
		    status = 1;
		    break;
		}
	    }
# if __ENABLE_x00696e6b__
	    if (!do_skip) {
# endif				/* __ENABLE_x00696e6b__ */
		x1f42_tables->file_offset = CARDINAL(buffer, 0x0);
# if __ENABLE_x00696e6b__
	    }
# endif				/* __ENABLE_x00696e6b__ */
	    audio_stream_id = *(unsigned *) ((unsigned char *) buffer + 4);
	    if (type & 0xffff0000) {
		if (type_class(class, audio_stream_id)) {
		    status = 1;
		    break;
		}
	    } else {
		if (type_order(class, audio_stream_id)) {
		    status = 1;
		    break;
		}
	    }
# if __ENABLE_x00696e6b__
	    if (!do_skip) {
# endif				/* __ENABLE_x00696e6b__ */
		if (class == MPEG_CLASS) {
		    x1f42_tables->audio_id = audio_stream_id;
		} else {
		    x1f42_tables->audio_id = stream_id(audio_stream_id);
		}
# if __ENABLE_x00696e6b__
	    }
# endif				/* __ENABLE_x00696e6b__ */
	    frames = CARDINAL(buffer, 0x8);
	    nrecords = CARDINAL(buffer, 0xc);
	    if (frames < nrecords) {
		status = 1;
		break;
	    }
	    {
		audio_frames += frames;
# if __ENABLE_x00696e6b__
		if (!do_skip) {
# endif				/* __ENABLE_x00696e6b__ */
		    x1f42_tables->frames = frames;
		    x1f42_tables->next_frame = audio_frames;
		    x1f42_tables->type = type;
# if __ENABLE_x00696e6b__
		}
# endif				/* __ENABLE_x00696e6b__ */
	    }
	    entry_length -= 32;
	    if (flags & (1L << 0x04 | 1L << 0x06)) {
		if (entry_length < 16) {
		    status = 1;
		    break;
		} else {
		    if (flags & (1L << 0x06)) {
		    } else {
			if (class == LPCM_CLASS) {
			    status = 1;
			    break;
			}
		    }
		}
# if __ENABLE_x00696e6b__
		if (do_skip) {
		    status = Mylene1f4_seek_file
			(context, source, 16, SEEK_CUR);
		    if (status != -1) {
			status = 0;
		    } else {
			break;
		    }
		} else {
# endif				/* __ENABLE_x00696e6b__ */
		    status = Mylene1f4_read_file(context, source, buffer, 16);
		    if (status == -1) {
			break;
		    } else {
			if (status != 16) {
			    status = 1;
			    break;
			}
		    }
		    if (flags & (1L << 0x04)) {
			x1f42_tables->PTS_0 = buffer[3] & 1;
			x1f42_tables->PTS_1 = CARDINAL(buffer, 0x4);
		    } else {
			x1f42_tables->PTS_0 = ~0;
			x1f42_tables->PTS_1 = ~0;
		    }
		    if (flags & (1L << 0x06)) {
			unsigned frame_rate__;

			frame_rate__ = CARDINAL(buffer, 0x8);
			if (!frame_rate__) {
			    status = 1;
			    break;
			}
			x1f42_tables->frame_rate_n = frame_rate__;
			frame_rate__ = CARDINAL(buffer, 0xc);
			if (!frame_rate__) {
			    status = 1;
			    break;
			}
			x1f42_tables->frame_rate_d = frame_rate__;
		    } else {
			x1f42_tables->frame_rate_n = ~0;
			x1f42_tables->frame_rate_d = ~0;
		    }
# if __ENABLE_x00696e6b__
		}
# endif				/* __ENABLE_x00696e6b__ */
		entry_length -= 16;
	    } else {
		if (class == LPCM_CLASS) {
		    status = 1;
		    break;
		}
# if __ENABLE_x00696e6b__
		if (!do_skip) {
# endif				/* __ENABLE_x00696e6b__ */
		    {
			x1f42_tables->PTS_0 = ~0;
			x1f42_tables->PTS_1 = ~0;
			x1f42_tables->frame_rate_n = ~0;
			x1f42_tables->frame_rate_d = ~0;
		    }
# if __ENABLE_x00696e6b__
		}
# endif				/* __ENABLE_x00696e6b__ */
	    }
	    if (nrecords) {
		if (entry_length < nrecords << 4) {
		    status = 1;
		    break;
		}
		entry_length -= nrecords << 4;

# if __ENABLE_x00696e6b__
		if (do_skip) {
		    status = Mylene1f4_seek_file
			(context, source, nrecords << 4, SEEK_CUR);
		    if (status != -1) {
			status = 0;
		    } else {
			break;
		    }
		} else {
# endif				/* __ENABLE_x00696e6b__ */
		    if ((status = read_records
			 (context, source, type, x1f42_tables, nrecords))) {
			break;
		    } else {
			if (type) {
			    label_data->next_index = 0;
			}
		    }
# if __ENABLE_x00696e6b__
		}
# endif				/* __ENABLE_x00696e6b__ */
	    }
	    if (type & 0xffff0000) {
		label_data->id3 = 0;
	    }
	} else {
	    if (1) {
		label_data->id3 = 0;
	    }
	}

	if (entry_length) {
	    status = 1;
	    break;
	}

# if __ENABLE_x00696e6b__
	if (!do_skip) {
# endif				/* __ENABLE_x00696e6b__ */
	    if (x1f42_tables->frames) {
		progress++;
		x1f42_tables++;
	    } else {
		void *qrecords;

		x1f42_tables->type = 0;
		free_path(source, x1f42_tables);
		qrecords = VRECORDS(x1f42_tables);
		if (qrecords) {
		    VRECORDS(x1f42_tables) = NULL;
		    free(qrecords);
		}
	    }
# if __ENABLE_x00696e6b__
	}
# endif				/* __ENABLE_x00696e6b__ */
    }

# if __ENABLE_x00696e6b__
    if (~audio_entry) {
	if (count == audio_entry) {
	    progress++;
	    x1f42_tables++;
	}
    } else {
# endif				/* __ENABLE_x00696e6b__ */
	if (count) {
	    progress++;
	    x1f42_tables++;
	}
# if __ENABLE_x00696e6b__
    }
# endif				/* __ENABLE_x00696e6b__ */

    if (!status) {
	label_data->count = progress;
    } else {
	free_1f42(context, source, x1f42_tables - progress, progress);
    }

    return status;
}
#endif				/* __EXPAND_x1f42MAPS__ */


static int
read_entries(void *context, void *source, struct x1f42a_type *x1f42_tables,
	     struct label_type *label_data)
{
    int status = 0;
    unsigned audio_frames = 0, count, progress = 0;
    unsigned char buffer[0x10];

    label_data->temporal_reference = 0;
    if (SELECT(context) == Mylene1f4_PAGE_PICKUP) {
	label_data->next_index = START2(context);
    } else {
	label_data->next_index = 0;
    }

    count = label_data->count;
    for (; count; count--) {
	unsigned entry_length, flags, label_length, shift_length, type;

	status = Mylene1f4_read_file(context, source, buffer, 16);
	if (status == -1) {
	    break;
	} else {
	    if (status != 16) {
		status = 1;
		break;
	    }
	}
	if (memcmp(&buffer[0xc], "\377\377", 2)) {
	    label_length = 65535 - (buffer[0xc] << 010 | buffer[0xd]);
	} else {
	    label_length = 0;
	}
	type = CARDINAL(buffer, 0x4);
	if (!label_data->video && type) {
	    status = 1;
	    break;
	} else if (!type) {
	} else if (type_video(type)) {
	} else {
	    status = 1;
	    break;
	}
	if (label_length) {
	    if (skip_name(context, source, label_length) == -1) {
		status = 1;
		break;
	    }
	}
	entry_length = CARDINAL(buffer, 0x0);
	flags = CARDINAL(buffer, 0x8);
	if (flags & 7) {
	    status = 1;
	    break;
	}
	if (flags & 0x3fffff80) {
	    status = 1;
	    break;
	}
	if (!type && flags & 0x8000005f) {
	    status = 1;
	    break;
	}
	if (flags & 32) {
	    unsigned nentries;

	    nentries = buffer[0xe] << 010 | buffer[0xf];
	    if (nentries < 2) {
		status = 1;
		break;
	    }
	    {
		{
		    status = read_1f44
			(context, source, x1f42_tables, nentries,
			 &shift_length);
		    if (status) {
			break;
		    }
		}
		{
		    if (1) {
			label_data->id3 = 0;
		    }
		}
	    }
	} else {
	    shift_length = buffer[0xe] << 010 | buffer[0xf];
	    if (shift_length) {
		{
		    status = read_path
			(context, source, x1f42_tables, shift_length);
		    if (status) {
			break;
		    }
		    if (1) {
			label_data->id3 = 0;
		    }
		    {
			shift_length += (16 - (shift_length & 15)) & 15;
		    }
		}
	    } else {
		{
		    x1f42_tables->source.path = Source(source).s1.path;
		}
	    }
	}
	entry_length -= 12 + shift_length + label_length
	    + ((16 - (label_length & 15)) & 15) + 16;
	if ((int) entry_length < 0) {
	    status = 1;
	    break;
	}
	status = Mylene1f4_read_file(context, source, buffer, 16);
	if (status == -1) {
	    break;
	} else {
	    if (status != 16) {
		status = 1;
		break;
	    }
	}
	status = skip_video
	    (context, source, type, flags, &entry_length, buffer);
	if (status) {
	    break;
	}
	if (type) {
	    if (1) {
		label_data->id3 = 0;
	    }
	}
	if (flags & (1L << 0x1e)) {
	    int class;
	    unsigned audio_stream_id, frames, nrecords;

	    if (entry_length < 32) {
		status = 1;
		break;
	    }
	    status = Mylene1f4_read_file(context, source, buffer, 16);
	    if (status == -1) {
		break;
	    } else {
		if (status != 16) {
		    status = 1;
		    break;
		}
	    }
	    if (memcmp(&buffer[0xc], "\377\377", 2)) {
		status = 1;
		break;
	    }
	    type = CARDINAL(buffer, 0x4);
	    class = type_audio(context, type);
	    if (class) {
	    } else {
		status = 1;
		break;
	    }
	    flags = CARDINAL(buffer, 0x8);
	    if (flags & 7) {
		status = 1;
		break;
	    }
	    if (flags & 0x7fffff80) {
		status = 1;
		break;
	    }
	    status = Mylene1f4_read_file(context, source, buffer, 16);
	    if (status == -1) {
		break;
	    } else {
		if (status != 16) {
		    status = 1;
		    break;
		}
	    }
	    {
		x1f42_tables->file_offset = CARDINAL(buffer, 0x0);
	    }
	    audio_stream_id = *(unsigned *) ((unsigned char *) buffer + 4);
	    if (type & 0xffff0000) {
		if (type_class(class, audio_stream_id)) {
		    status = 1;
		    break;
		}
	    } else {
		if (type_order(class, audio_stream_id)) {
		    status = 1;
		    break;
		}
	    }
	    {
		if (class == MPEG_CLASS) {
		    x1f42_tables->audio_id = audio_stream_id;
		} else {
		    x1f42_tables->audio_id = stream_id(audio_stream_id);
		}
	    }
	    frames = CARDINAL(buffer, 0x8);
	    nrecords = CARDINAL(buffer, 0xc);
	    if (frames < nrecords) {
		status = 1;
		break;
	    }
	    entry_length -= 32;
	    if (flags & (1L << 0x04 | 1L << 0x06)) {
		if (entry_length < 16) {
		    status = 1;
		    break;
		} else {
		    if (flags & (1L << 0x06)) {
		    } else {
			if (class == LPCM_CLASS) {
			    status = 1;
			    break;
			}
		    }
		}
		if (1) {
		    status = Mylene1f4_read_file(context, source, buffer, 16);
		    if (status == -1) {
			break;
		    } else {
			if (status != 16) {
			    status = 1;
			    break;
			}
		    }
		    if (flags & (1L << 0x04)) {
			x1f42_tables->PTS_0 = buffer[3] & 1;
			x1f42_tables->PTS_1 = CARDINAL(buffer, 0x4);
		    } else {
			x1f42_tables->PTS_0 = ~0;
			x1f42_tables->PTS_1 = ~0;
		    }
		    if (flags & (1L << 0x06)) {
			unsigned frame_rate__;

			frame_rate__ = CARDINAL(buffer, 0x8);
			if (!frame_rate__) {
			    status = 1;
			    break;
			}
			x1f42_tables->frame_rate_n = frame_rate__;
			frame_rate__ = CARDINAL(buffer, 0xc);
			if (!frame_rate__) {
			    status = 1;
			    break;
			}
			x1f42_tables->frame_rate_d = frame_rate__;
			if (SELECT(context) == Mylene1f4_TIME_PICKUP) {
			    if (label_data->count == 1) {
				unsigned text;

				pick_page
				    (context, &text,
				     x1f42_tables->frame_rate_n, frame_rate__);
				label_data->next_index = text;
				label_data->start_index = text;
			    }
			}
		    } else {
			x1f42_tables->frame_rate_n = ~0;
			x1f42_tables->frame_rate_d = ~0;
		    }
		}
		entry_length -= 16;
	    } else {
		if (class == LPCM_CLASS) {
		    status = 1;
		    break;
		}
		if (1) {
		    {
			x1f42_tables->PTS_0 = ~0;
			x1f42_tables->PTS_1 = ~0;
			x1f42_tables->frame_rate_n = ~0;
			x1f42_tables->frame_rate_d = ~0;
		    }
		}
	    }
	    {
		audio_frames += frames;
		if (label_data->next_index < frames) {
		    x1f42_tables->frames = frames;
		    x1f42_tables->next_frame = audio_frames;
		    x1f42_tables->type = type;
		} else {
		    if (frames == label_data->next_index) {
			label_data->next_index = 0;
			label_data->temporal_reference = audio_frames;
		    } else {
			label_data->next_index -= frames;
			type = 0;
		    }
		}
	    }
	    if (nrecords) {
		if (entry_length < nrecords << 4) {
		    status = 1;
		    break;
		}
		entry_length -= nrecords << 4;

		{
		    if ((status = skip_records
			 (context, source, type, x1f42_tables, nrecords,
			  label_data))) {
			break;
		    } else {
			if (type) {
			    label_data->next_index = 0;
			}
		    }
		}
	    }
	    if (type & 0xffff0000) {
		label_data->id3 = 0;
	    }
	} else {
	    if (1) {
		label_data->id3 = 0;
	    }
	}

	if (entry_length) {
	    status = 1;
	    break;
	}

	{
	    if (x1f42_tables->frames) {
		progress++;
		x1f42_tables++;
	    } else {
		x1f42_tables->type = 0;
		free_path(source, x1f42_tables);
	    }
	}
    }

    if (count) {
	progress++;
	x1f42_tables++;
    } else {
	status = 0;
    }

    if (!status) {
	label_data->count = progress;
    } else {
	free_1f42(context, source, x1f42_tables - progress, progress);
    }

    return status;
}


static int
read_id3x(void *context, void *source, void *id3x)
{
    int status;

    do {
	unsigned extension_length;
	unsigned char *buffer;

	buffer = id3x;
	status = Mylene1f4_read_file(context, source, buffer, 8);
	if (status == -1) {
	    break;
	} else {
	    if (status != 8) {
		status = 0;
		break;
	    }
	}
	status = 0;
	if (memcmp(buffer + 4, "\377\377\0\0", 4)) {
	    break;
	}
	extension_length = buffer[0x0] << 030 | buffer[0x1] << 020
	    | buffer[0x2] << 010 | buffer[0x3] << 000;
	if (extension_length < 12) {
	    break;
	}
	status = Mylene1f4_seek_file
	    (context, source, extension_length - 4, SEEK_CUR);
	if (status == -1) {
	    break;
	}
	status = Mylene1f4_read_file(context, source, buffer, 8);
	if (status == -1) {
	    break;
	} else {
	    if (status != 8) {
		status = 0;
		break;
	    }
	}
	status = 0;
	if (memcmp(buffer + 4, "\377\376\377\377", 4)) {
	    break;
	}
	extension_length = buffer[0x0] << 030 | buffer[0x1] << 020
	    | buffer[0x2] << 010 | buffer[0x3] << 000;
	if (extension_length != 140) {
	    break;
	}
	status = Mylene1f4_seek_file(context, source, 8, SEEK_CUR);
	if (status == -1) {
	    break;
	}
	status = Mylene1f4_read_file(context, source, buffer, 128);
	if (status == -1) {
	    break;
	} else {
	    if (status != 128) {
		status = 0;
		break;
	    }
	}
	status = 0;
	if (memcmp(buffer, "TAG", 3)) {
	    break;
	}
	status = Mylene1f4_read_file(context, source, buffer, 1);
	if (status == -1) {
	    break;
	} else {
	    if (status) {
		status = 0;
		break;
	    }
	}

	ID3T(context) = Mylene1f4_MERGED_ID3TAG;
	ID3X(context) = id3x;
    } while (0);

    return status;
}


static int
read_miss(void *miss, void *data, unsigned size)
{
    return Mylene1f4_read_file
	(((void **) miss)[0], ((void **) miss)[1], data, size);
}


static int
read_path(void *context, void *source, struct x1f42a_type *x1f42a_data,
	  unsigned size)
{
    int status = 0;

    do {
	char *path;

	if (!(path = (char *) malloc(size + 1))) {
	    status = -1;

	    Mylene1f4_fprintf_error
		(context, "%s: %s\n", Mylene1f4(context), strerror(errno));

	    break;
	}
	x1f42a_data->source.path = path;
	status = Mylene1f4_read_file(context, source, path, size);
	if (status == -1) {
	    break;
	} else {
	    if (status != size) {
		status = 1;
		break;
	    }
	}
	path[size] = '\0';
	status = Mylene1f4_seek_file
	    (context, source, (16 - (size & 15)) & 15, SEEK_CUR);
	if (status == -1) {
	    break;
	}

	status = 0;
    } while (0);

    return status;
}


#if __EXPAND_x1f42MAPS__
static int
read_records(void *context, void *source, unsigned type,
	     struct x1f42a_type *entry, unsigned nrecords)
{
    int status;
    struct access_type *access_data = NULL;

    do {
	if (type) {
	    access_data = (struct access_type *)
		malloc(nrecords * sizeof(struct access_type));
	    if (!access_data) {
		status = -1;
		break;
	    }
	}
	if (type) {
	    lcard_type Stream, System;
	    struct access_type *this_access;
	    unsigned count, stream, system, window;
	    unsigned char buffer[0x10];

	    count = nrecords;
	    this_access = access_data;

	    status = Mylene1f4_read_file(context, source, buffer, 16);
	    if (status == -1) {
		break;
	    } else {
		if (status != 16) {
		    status = 1;
		    break;
		}
	    }

	    count--;
	    stream = CARDINAL(buffer, 0x8);
	    system = CARDINAL(buffer, 0xc);
	    if (CARDINAL(buffer, 0x0)) {
		status = 1;
		break;
	    }

	    window = CARDINAL(buffer, 0x0);

	    LCARD_SET_CARD1(System, entry->file_offset);
	    LCARD_ADD_CARD1(System, system);
	    LCARD_SET_CARD2(Stream, System);
	    LCARD_ADD_CARD1(Stream, stream);

	    LCARD_SET_CARD1(entry->stream, stream);
	    LCARD_SET_CARD1(entry->system, system);

	    this_access->detail = CARDINAL(buffer, 0x4);
	    if (window < this_access->detail) {
		status = 1;
		break;
	    }

	    window += entry->next_frame - entry->frames;

	    this_access->temporal_reference = window;

	    LCARD_SET_CARD2(this_access->stream, Stream);
	    LCARD_SET_CARD2(this_access->system, System);

	    this_access++;

	    for (; count; count--) {
		status = Mylene1f4_read_file(context, source, buffer, 16);
		if (status == -1) {
		    break;
		} else {
		    if (status != 16) {
			status = 1;
			break;
		    } else {
			unsigned access;

			access = CARDINAL(buffer, 0x0);
			if (!access) {
			    status = 1;
			    break;
			}

			window = window + access;

			this_access->temporal_reference = window;

			this_access->detail = CARDINAL(buffer, 0x4);
			if (window < this_access->detail) {
			    status = 1;
			    break;
			}

			system = CARDINAL(buffer, 0xc);
			stream = CARDINAL(buffer, 0x8);

			LCARD_ADD_CARD1(System, system);
			LCARD_SET_CARD2(Stream, System);
			LCARD_ADD_CARD1(Stream, stream);

			LCARD_SET_CARD2(this_access->stream, Stream);
			LCARD_SET_CARD2(this_access->system, System);

			this_access++;
		    }
		}
	    }
	    if (count) {
		break;
	    }
	}

	status = 0;
    } while (0);

    if (access_data) {
	if (status) {
	    free(access_data);
	} else {
	    VRECORDS(entry) = access_data;
	    entry->nrecords = nrecords;
	}
    }

    return status;
}
#endif				/* __EXPAND_x1f42MAPS__ */


static int
seek_1f42(void *loader, void *source, unsigned level, void *cleavage)
{
    int status = 0;
    struct inter6_type *inter6_data;
    struct x1f42a_type *x1f42a_data;

    inter6_data = loader;

    x1f42a_data = inter6_data->entry;

    if (level) {
	if (x1f42a_data->type & 0xffff0000) {
	} else {
	    status = -1;
	}
    } else {
	if (x1f42a_data->type & 0xffff0000) {
	    status = -1;
	} else {
	}
    }

    if (!status) {
	struct seek_1f44_type *e66c;

	e66c = Source(source).s2.e66c;
	status = e66c->seek(e66c, &x1f42a_data->system, SEEK_SET);
	if (!status) {
	    lcard_type *template;

	    template = cleavage;
	    template[0] = x1f42a_data->stream;
	}
    }

    return status;
}


#if __ENABLE_x00696e6b__
static int
skip_1f44(void *context, void *source, unsigned nentries, unsigned *shift)
{
    int status;
    unsigned scale;
    unsigned char buffer[0x10];

    scale = nentries << 4;

    do {
	status = Mylene1f4_read_file(context, source, buffer, 16);
	if (status != 16) {
	    break;
	} else {
	    unsigned size;

	    size = buffer[0xe] << 010 | buffer[0xf];
	    if (!size) {
		status = 0;
	    } else {
		size += (16 - (size & 15)) & 15;
		status = Mylene1f4_seek_file(context, source, size, SEEK_CUR);
		if (status == -1) {
		    break;
		} else {
		    status = 0;
		    scale += size;
		}
	    }
	}

	nentries--;
    } while (nentries);

    *shift = scale;

    return status;
}
#endif				/* __ENABLE_x00696e6b__ */


static int
skip_name(void *context, void *source, unsigned type)
{
    int status = 0;

    if (type) {
	unsigned offset;

	offset = type + ((16 - (type & 15)) & 15);
	status = Mylene1f4_seek_file(context, source, offset, SEEK_CUR);
    }

    return status;
}


static int
skip_records(void *context, void *source, unsigned type,
	     struct x1f42a_type *entry, unsigned nrecords,
	     struct label_type *label_data)
{
    int status;

    do {
	if (type) {
	    lcard_type Stream, System;
	    unsigned next_index;
	    unsigned char buffer[0x10];

	    status = Mylene1f4_read_file(context, source, buffer, 16);
	    if (status == -1) {
		break;
	    } else {
		if (status != 16) {
		    status = 1;
		    break;
		}
	    }

	    nrecords--;
	    if (CARDINAL(buffer, 0x0)) {
		status = 1;
		break;
	    }

	    if (1) {
		unsigned stream, system;

		stream = CARDINAL(buffer, 0x8);
		system = CARDINAL(buffer, 0xc);

		LCARD_SET_CARD1(System, entry->file_offset);
		LCARD_ADD_CARD1(System, system);
		LCARD_SET_CARD2(Stream, System);
		LCARD_ADD_CARD1(Stream, stream);
	    }

	    next_index = label_data->next_index;
	    if (next_index) {
		status = Mylene1f4_read_file(context, source, buffer, 16);
		if (status == -1) {
		    break;
		} else {
		    if (status != 16) {
			status = 1;
			break;
		    } else {
			unsigned shift;

			nrecords--;

			do {
			    shift = CARDINAL(buffer, 0x0);
			    if (shift < next_index + 1) {
				unsigned stream, system;

				next_index -= shift;

				stream = CARDINAL(buffer, 0x8);
				system = CARDINAL(buffer, 0xc);

				LCARD_ADD_CARD1(System, system);
				LCARD_SET_CARD2(Stream, System);
				LCARD_ADD_CARD1(Stream, stream);
			    } else {
				break;
			    }
			    if (!nrecords) {
				break;
			    }
			    status = Mylene1f4_read_file
				(context, source, buffer, 16);
			    if (status == -1) {
				break;
			    } else {
				if (status != 16) {
				    status = 1;
				    break;
				}
			    }

			    nrecords--;
			} while (1);
			if (status != 16) {
			    break;
			}

			label_data->temporal_reference =
			    label_data->start_index - next_index;
		    }
		}
	    }

	    LCARD_SET_CARD2(entry->stream, Stream);
	    LCARD_SET_CARD2(entry->system, System);
	}

	status = Mylene1f4_seek_file(context, source, nrecords << 4, SEEK_CUR);
	if (status == -1) {
	    break;
	}

	status = 0;
    } while (0);

    return status;
}


static int
skip_video(void *context, void *source, unsigned type, unsigned flags,
	   unsigned *length, unsigned char *buffer)
{
    int status = 1;

    do {
	if (type) {
	    unsigned entry_length, nrecords, video_stream_id;

	    entry_length = *length;

	    video_stream_id = *(unsigned *) ((unsigned char *) buffer + 4);
	    if (type & 0xffff0000) {
		if ((video_stream_id & VIDEO_MASK) != VIDEO_PREFIX) {
		    break;
		}
	    } else {
		if (video_stream_id != VIDEO_STREAM_ID) {
		    break;
		}
	    }
	    nrecords = CARDINAL(buffer, 0xc);
	    if (CARDINAL(buffer, 0x8) < nrecords) {
		break;
	    }
	    if (flags & (1L << 0x04 | 1L << 0x06)) {
		if (entry_length < 16) {
		    break;
		}
		status = Mylene1f4_seek_file(context, source, 16, SEEK_CUR);
		if (status == -1) {
		    break;
		}
		entry_length -= 16;
	    } else {
	    }
	    if (nrecords) {
		if (entry_length < nrecords << 4) {
		    break;
		}
		entry_length -= nrecords << 4;
		status = Mylene1f4_seek_file
		    (context, source, nrecords << 4, SEEK_CUR);
		if (status == -1) {
		    break;
		}
	    }
	    if (flags & (1L << 0x1f)) {
		if (entry_length < 16) {
		    break;
		}
		status = Mylene1f4_read_file(context, source, buffer, 16);
		if (status == -1) {
		    break;
		} else {
		    if (status != 16) {
			status = 1;
			break;
		    }
		}
		nrecords = CARDINAL(buffer, 0x0);
		if (nrecords == 1) {
		    break;
		}
		if (!nrecords) {
		    break;
		}
		entry_length -= 16;
		if (entry_length < nrecords << 4) {
		    break;
		}
		entry_length -= nrecords << 4;
		status = Mylene1f4_seek_file
		    (context, source, nrecords << 4, SEEK_CUR);
		if (status == -1) {
		    break;
		}
	    }

	    *length = entry_length;
	} else {
	    if (memcmp(buffer, "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0", 16)) {
		break;
	    } else {
	    }
	}

	status = 0;
    } while (0);

    return status;
}


static int
type_1f42(void *loader, void *target)
{
    int status;

    status = ((struct inter6_type *) loader)->audio6_data->type
	(((struct inter6_type *) loader)->loader, target);
    if (!status) {
	struct target_type *target_data;

	target_data = target;
	target_data->back = data_target.back;
	target_data->fini = data_target.fini;
	target_data->head = data_target.head;
	target_data->init = data_target.init;
#if __ENABLE_x00696e6b__
	target_data->line = data_target.line;
#endif				/* __ENABLE_x00696e6b__ */
	target_data->type = data_target.type;
	target_data->wind = data_target.wind;
    }

    return status;
}


static int
type_audio(void *context, unsigned type)
{
    int class;

    switch (type) {
    case 0x00003333:
    case 0x0000aaaa:
    case 0x0000ffff:
    case 0x33333333:
    case 0xaaaaaaaa:
    case 0xffffffff:
	class = MPEG_CLASS;
	break;
    case 0x17e817e8:
	class = LPCM_CLASS;
	break;
#if __LINK_A52A_LOADER__
    case 0x000003cf:
    case 0xf3c003cf:
	if (CLASS(context) == Mylene1f4_LINK_OUTPUT) {
	    class = A52A_CLASS;
	} else {
	    class = 0;
	}
	break;
#endif				/* __LINK_A52A_LOADER__ */
    default:
	class = 0;
    }

    return class;
}


static int
type_class(int class, unsigned audio_stream_id)
{
    int false = 0;

    if (class == MPEG_CLASS) {
	if ((audio_stream_id & AUDIO_MASK) != AUDIO_PREFIX) {
	    false = 1;
	}
    } else {
	if (class == LPCM_CLASS) {
	    if ((audio_stream_id & AUDIO_LPCM_MASK) != AUDIO_LPCM_PREFIX) {
		false = 1;
	    }
	} else {
#if __LINK_A52A_LOADER__
	    if (class == A52A_CLASS) {
		if ((audio_stream_id & AUDIO_A52A_MASK) != AUDIO_A52A_PREFIX) {
		    false = 1;
		}
	    } else {
#endif				/* __LINK_A52A_LOADER__ */
		if (1) {
		    false = 1;
		}
#if __LINK_A52A_LOADER__
	    }
#endif				/* __LINK_A52A_LOADER__ */
	}
    }

    return false;
}


static int
type_order(int class, unsigned audio_stream_id)
{
    int false = 0;

    if (class == MPEG_CLASS) {
	if (audio_stream_id != AUDIO_STREAM_ID) {
	    false = 1;
	}
    } else {
	if (false(class == LPCM_CLASS)) {
	    if (0) {
		false = 1;
	    }
	} else {
#if __LINK_A52A_LOADER__
	    if (class == A52A_CLASS) {
		if (audio_stream_id != AUDIO_A52A_TRACK) {
		    false = 1;
		}
	    } else {
#endif				/* __LINK_A52A_LOADER__ */
		if (1) {
		    false = 1;
		}
#if __LINK_A52A_LOADER__
	    }
#endif				/* __LINK_A52A_LOADER__ */
	}
    }

    return false;
}


static int
type_video(unsigned type)
{
    return type == 0x00003333 || type == 0x00004444 || type == 0x00005555
	|| type == 0x0000aaaa || type == 0x0000ffff || type == 0x33333333
	|| type == 0x44444444 || type == 0x55555555 || type == 0xaaaaaaaa
	|| type == 0xffffffff;
}


static int
wind_1f42(void *loader, unsigned *frame, unsigned scale)
{
    if (Inter6(loader)->audio6_data == &mpeg_audio6) {
	struct x1f42a_type *x1f42a_data;
	unsigned class;

	if (4 < scale) {
	    scale = 4;
	}

	class = *frame;

	x1f42a_data = X1f42(Inter6(loader)->context);
#if 0
	shift = class - (x1f42a_data->next_frame - x1f42a_data->frames);
#endif				/* 0 */
	if (class < scale) {
	    scale = class;
	}

	*frame = class - scale;
    }

    return 0;
}


#if __ENABLE_x00696e6b__
static struct access_type *
seek_data(unsigned nrecords, struct access_type *access_data, unsigned frame)
{
    nrecords--;

    if (access_data[nrecords].temporal_reference < frame + 1) {
	access_data += nrecords;
    } else {
	unsigned finish, half = 0, start = 0;

	finish = nrecords;
	while (start < finish) {
	    half = (finish + start) >> 1;
	    if (frame < access_data[half].temporal_reference) {
		finish = half;
	    } else {
		start = half + 1;
	    }
	}

	access_data += start;
	if (start) {
	    access_data--;
	}
    }

    return access_data;
}
#endif				/* __ENABLE_x00696e6b__ */


#if __EXPAND_x1f42MAPS__
static void
clip_1f42(void *context, struct inter6_type *inter6_data,
	  struct x1f42a_type *x1f42a_data, unsigned *head, unsigned *tail)
{
    int (*index_frame) (void *, unsigned *);
    struct access_type *access_data;
    unsigned nrecords, start2;

    index_frame = START3(context);
    if (index_frame) {
	index_frame(context, &start2);
    } else {
	if (SELECT(context) == Mylene1f4_PAGE_PICKUP) {
	    start2 = START2(context);
	} else {
	    if (false(SELECT(context) != Mylene1f4_PAGE_PICKUP)) {
		start2 = 0;
	    } else {
		if (inter6_data->x1f42e.count != 1) {
		    start2 = 0;
		} else {
		    unsigned frame_rate_d, frame_rate_n;

		    frame_rate_d = x1f42a_data->frame_rate_d;
		    frame_rate_n = x1f42a_data->frame_rate_n;
		    if (!~(frame_rate_n & frame_rate_d)) {
			start2 = 0;
		    } else {
			pick_page
			    (context, &start2, frame_rate_n, frame_rate_d);
		    }
		}
	    }
	}
    }

    access_data = seek_data
	(x1f42a_data->nrecords, VRECORDS(x1f42a_data), start2);

    nrecords = access_data - (struct access_type *) VRECORDS(x1f42a_data);

    less_1f42(inter6_data, x1f42a_data, &nrecords, &access_data);

    if (access_data->detail) {
	unsigned target;

	target = access_data->temporal_reference - access_data->detail;
	while (nrecords) {
	    nrecords--;
	    access_data--;
	    if (target < access_data->temporal_reference) {
	    } else {
		break;
	    }
	}

	less_1f42(inter6_data, x1f42a_data, &nrecords, &access_data);
    }

    *head = access_data->temporal_reference;
    *tail = x1f42a_data->next_frame;

    LCARD_SET_CARD2(x1f42a_data->stream, access_data->stream);
    LCARD_SET_CARD2(x1f42a_data->system, access_data->system);
}
#endif				/* __EXPAND_x1f42MAPS__ */


static void
free_1f42(void *context, void *source, struct x1f42a_type *x1f42_tables,
	  unsigned ntables)
{
    unsigned i;

    for (i = ntables; i; i--) {
#if __EXPAND_x1f42MAPS__
	void *qrecords;
#endif				/* __ENABLE_x00696e6b__ */

#if __EXPAND_x1f42MAPS__
	qrecords = VRECORDS(x1f42_tables);
	if (qrecords) {
	    free(qrecords);
	}
#endif				/* __ENABLE_x00696e6b__ */

	if (x1f42_tables->nentries) {
	    seek_1f44_free(&x1f42_tables->source.e66c);
	} else {
	    char *path;

	    path = (char *) x1f42_tables->source.path;
	    if (path) {
		if (path != Source(source).s1.path) {
		    free(path);
		}
	    }
	}

	x1f42_tables++;
    }
}


static void
free_path(void *source, struct x1f42a_type *x1f42a_data)
{
    if (x1f42a_data->nentries) {
	x1f42a_data->nentries = 0;
	seek_1f44_free(&x1f42a_data->source.e66c);
	x1f42a_data->source.path = NULL;
    } else {
	const char *path;

	path = x1f42a_data->source.path;
	if (path) {
	    x1f42a_data->source.path = NULL;
	    if (path != Source(source).s1.path) {
		free((char *) path);
	    }
	}
    }
}


#if __EXPAND_x1f42MAPS__
static void
less_1f42(struct inter6_type *inter6_data, struct x1f42a_type *x1f42a_data,
	  unsigned *irecords, struct access_type **access)
{
    struct access_type *access_data;
    unsigned nrecords;

    access_data = *access;

    nrecords = *irecords;

    if (x1f42a_data->artificial) {
	if (!~access_data->detail) {
	    for (; nrecords && !~access_data->detail;
		 access_data--, nrecords--);
	    if (~access_data->detail) {
		inter6_data->access = access_data + 1;
	    } else {
		inter6_data->access = access_data;
	    }
	} else {
	    struct access_type *this_access;
	    unsigned vrecords;

	    vrecords = x1f42a_data->nrecords - nrecords - 1;
	    this_access = access_data;
	    for (; vrecords && ~this_access->detail;
		 this_access++, vrecords--);
	    if (!~this_access->detail) {
		inter6_data->access = this_access;
	    }
	}
    }

    *irecords = nrecords;

    *access = access_data;
}
#endif				/* __EXPAND_x1f42MAPS__ */


static void
pick_page(void *context, unsigned *page, unsigned frame_rate_n,
	  unsigned frame_rate_d)
{
#if SIZEOF_UNSIGNED_LONG != 8
    double text;
#endif					/* SIZEOF_UNSIGNED_LONG != 8 */
    struct extime_type *extime_data;
#if SIZEOF_UNSIGNED_LONG == 8
    unsigned long text;
#endif					/* SIZEOF_UNSIGNED_LONG == 8 */

    extime_data = &START6(context);
    text = (extime_data->h * 3600 + extime_data->m * 60 + extime_data->s
	    + extime_data->x / 60) * frame_rate_n
	+ extime_data->x % 60 * frame_rate_n / 60;
    *page = text / frame_rate_d;
}


static void
type_flow(struct inter6_type *inter6_data, struct x1f42a_type *x1f42a_data)
{
    switch (x1f42a_data->type) {
#if __LINK_A52A_LOADER__
    case 0x000003cf:
    case 0xf3c003cf:
	inter6_data->audio6_data = _a52a_audio6;
	break;
#endif				/* __LINK_A52A_LOADER__ */
    case 0x17e817e8:
	inter6_data->audio6_data = &lpcm_audio6;
	break;
    default:
	inter6_data->audio6_data = &mpeg_audio6;
    }
}


static void
wrap_path(struct x1f42a_type *x1f42a_data, unsigned size, const char *path)
{
    char *storage;
    unsigned base;

    storage = (char *) x1f42a_data->source.path;
    base = strlen(storage);
    storage = (char *) realloc(storage, size + base + 1);
    if (storage) {
	x1f42a_data->source.path = storage;
	memmove(storage + size, storage, base + 1);
	memcpy(storage, path, size);
    }
}


#if __EXPAND_x1f42MAPS__
static void
wrap_qrecords(struct x1f42a_type *x1f42a_data)
{
    struct access_type *access_data, *this_access;
    unsigned frame_rate_d, frame_rate_n, j, next_frame, previous, mapping = 0,
	nrecords;

    access_data = VRECORDS(x1f42a_data);
    nrecords = x1f42a_data->nrecords;

    frame_rate_d = x1f42a_data->frame_rate_d;
    frame_rate_n = x1f42a_data->frame_rate_n;
    if (!~(frame_rate_n & frame_rate_d)) {
	frame_rate_d = 1152;
	frame_rate_n = 44100;
    } else {
    }

    next_frame = x1f42a_data->next_frame;

    this_access = access_data;

    previous = next_frame - x1f42a_data->frames;

    j = nrecords;
    if (this_access->temporal_reference == previous) {
	j--;
	mapping++;
	this_access++;
    }
    for (; j; j--) {
	unsigned current, diff;

	current = this_access->temporal_reference;
	if (current < previous + 1) {
	    break;
	} else if (next_frame < current + 1) {
	    break;
	} else {
	    diff = current - previous;
	    previous = current;
	    mapping += 1 + (diff - 1) * frame_rate_d / frame_rate_n;
	}
	this_access++;
    }
    {
	unsigned diff;

	{
	    diff = next_frame - previous;
	    mapping += 0 + (diff - 1) * frame_rate_d / frame_rate_n;
	}
    }

    if (mapping != nrecords) {
	struct access_type *miss_access;

	miss_access = (struct access_type *)
	    malloc(mapping * sizeof(struct access_type));
	if (!miss_access) {
	    x1f42a_data->artificial = 0;
	} else {
	    x1f42a_data->artificial = mapping - nrecords;
	    x1f42a_data->nrecords = mapping;
	    VRECORDS(x1f42a_data) = miss_access;

	    previous = next_frame - x1f42a_data->frames;

	    this_access = access_data;
	    j = nrecords;
	    if (this_access->temporal_reference == previous) {
		j--;
		miss_access->temporal_reference = previous;
		miss_access->detail = 0;
		LCARD_SET_CARD2(miss_access->stream, this_access->stream);
		LCARD_SET_CARD2(miss_access->system, this_access->system);
		miss_access++;
		this_access++;
	    }
	    for (; j; j--) {
		unsigned current, diff, k = 0;

		current = this_access->temporal_reference;
		if (current < previous + 1) {
		    break;
		} else if (next_frame < current + 1) {
		    break;
		} else {
		    diff = current - previous;
		    diff = (diff - 1) * frame_rate_d / frame_rate_n;
		    for (; k < diff; k++) {
			miss_access->temporal_reference =
			    previous + (k + 1) * frame_rate_n / frame_rate_d;
			miss_access->detail = ~0;
			miss_access++;
		    }
		    previous = current;
		}
		miss_access->temporal_reference = current;
		miss_access->detail = this_access->detail;
		LCARD_SET_CARD2(miss_access->stream, this_access->stream);
		LCARD_SET_CARD2(miss_access->system, this_access->system);
		miss_access++;
		this_access++;
	    }
	    {
		unsigned diff, k = 0;

		{
		    diff = next_frame - previous;
		    diff = (diff - 1) * frame_rate_d / frame_rate_n;
		    for (; k < diff; k++) {
			miss_access->temporal_reference =
			    previous + (k + 1) * frame_rate_n / frame_rate_d;
			miss_access->detail = ~0;
			miss_access++;
		    }
		}
	    }

	    free(access_data);
	}
    }
}
#endif				/* __EXPAND_x1f42MAPS__ */


#if __LINK_CELL_LOADER__
int
Mylene1f4_66c0_1f42(void *loader, int *miss)
{
    *miss = ((struct x1f440_type *) Inter6(loader)->x1f44)->fd;

    return 0;
}
#endif				/* __LINK_CELL_LOADER__ */


#if __LINK_CELL_LOADER__
int
Mylene1f4_ceil_1f42(void *loader, void **ceil)
{
    *ceil = ((struct inter6_type *) loader)->device;

    return 0;
}
#endif				/* __LINK_CELL_LOADER__ */


int
Mylene1f4_copy_1f42(void *context, void *target)
{
    if (1) {
	struct target_type *target_data;

	target_data = target;
	target_data->back = data_target.back;
	target_data->fini = data_target.fini;
	target_data->head = data_target.head;
	target_data->init = data_target.init;
#if __ENABLE_x00696e6b__
	target_data->line = data_target.line;
#endif				/* __ENABLE_x00696e6b__ */
	target_data->type = data_target.type;
	target_data->wind = data_target.wind;
    }

    return 0;
}


int
Mylene1f4_dump_1f42(void *context, void *source, unsigned *count,
		    void **loader)
{
    struct inter6_type *inter6_data;

    inter6_data = *loader;

    Mylene1f4_remove_sequel(context, Mylene1f4_x1f42e_SEQUEL);

    {
	void *x1f44;

	x1f44 = inter6_data->x1f44;
	if (x1f44) {
	    free(x1f44);
	}
    }

    {
	struct x1f42a_type *entries;

	entries = inter6_data->x1f42;
	free_1f42(context, source, entries, *count);
	free(entries);
    }

    free(inter6_data);

    return 0;
}


int
Mylene1f4_load_1f42(void *context, void *source, unsigned *count,
		    void **loader)
{
    int status;
    struct inter6_type *inter6_data;

    inter6_data = (struct inter6_type *) malloc(sizeof(struct inter6_type));
    if (!inter6_data) {
	Mylene1f4_fprintf_error
	    (context, "%s: %s\n", Mylene1f4(context), strerror(errno));

	status = -1;
    } else {
	unsigned nentries;

	status = load_1f42(context, source, &nentries, inter6_data);
	if (status) {
	    free(inter6_data);
	} else {
	    *count = nentries;
	    *loader = inter6_data;

	    inter6_data->context = context;

	    inter6_data->device = source;

	    inter6_data->x1f44 = NULL;

	    inter6_data->x1f42e.count = nentries;

	    inter6_data->sequel.sequel_data = SEQUEL(context);
	    SEQUEL(context) = &inter6_data->sequel;
	    inter6_data->sequel.code = Mylene1f4_x1f42e_SEQUEL;
	    inter6_data->sequel.data = &inter6_data->x1f42e;
	}
    }

    return status;
}
