/*
 * Mylene1f4.c
 * Copyright (C) 2002-2013, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * TODO
 *
 *.should not allocate memory shared for null output
 *.should not allocate pcm data storage memory for null output (some memory
 * should be allocated, though, to store timestamps and so forth)
 *.optimize text output chain (currently one function calls another, that calls
 * yet another, that calls yet another, that calls...)
 */

#include <config.h>

#if defined HAVE_FEATURES_H
# include <features.h>
#endif				/* HAVE_FEATURES_H */

#include <errno.h>
#include <getopt.h>
#if defined HAVE_LIBLEWD
# include <lewd.h>
#endif				/* HAVE_LIBLEWD */
#include <signal.h>
#include <stdarg.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include <sys/time.h>

#if !defined HAVE_LIBLEWD
# include <basename.h>
# include <cardinal-wx.h>
# include <ckeylist.h>
# include <ckeytree.h>
# include <command-e.h>
#endif				/* !HAVE_LIBLEWD */
#include <defs.h>
#include <inter.h>
#include <progress.h>
#include <state.h>
#if !defined HAVE_LIBLEWD
# include <vkeylist.h>
# include <vkeytree.h>
#endif				/* !HAVE_LIBLEWD */

#if 1
# define FLAGS_DEFAULT__0 \
    (Mylene1f4_FUSSY_MISS | Mylene1f4_ID3V2_READ | Mylene1f4_LOCAL_LIST)
#endif				/* 1 */
#if 1
# define FLAGS_DEFAULT___		FLAGS_DEFAULT__0
#endif				/* 1 */

#if __ENABLE_CONTROLLING_SERVER__
# define FUSSY_SERVER \
    (Mylene1f4_FUSSY_FILTER | Mylene1f4_FUSSY_FORMAT | Mylene1f4_FUSSY_OUTPUT)
#endif				/* __ENABLE_CONTROLLING_SERVER__ */

#if 1
# define SHORT_STRING___0		""
#endif				/* 1 */
#if __ENABLE_x1f4LANES__
# define SHORT_STRING___1		SHORT_STRING___0 "!"
#else
# define SHORT_STRING___1		SHORT_STRING___0
#endif				/* __ENABLE_x1f4LANES__ */
#if 1
# define SHORT_STRING___2 \
    SHORT_STRING___1 "1:2:3:A:B:C:D:F:L:P:Sa:b:c:d:e:f:i:l:o:p:qrs:t:vx"
#endif				/* 1 */
#if 1
# define SHORT_STRING____		SHORT_STRING___2
#endif				/* 1 */

#if 0
#elif __ENABLE_FILTER__
# define __ENABLE_EXHELP__		1
#elif __ENABLE_EQUEUE__
# define __ENABLE_EXHELP__		1
#else
# define __ENABLE_EXHELP__		0
#endif				/* 0 */

#define NULL_MISS			Mylene1f4_NULL_MISS
#define NEAR_MISS			Mylene1f4_NEAR_MISS

#define LOAD_LAST			Mylene1f4_LOAD_LAST
#define LOAD_NEXT			Mylene1f4_LOAD_NEXT
#define LOAD_RAND			Mylene1f4_LOAD_RAND

#define LINK_OUTPUT			Mylene1f4_LINK_OUTPUT
#define NULL_OUTPUT			Mylene1f4_NULL_OUTPUT
#define SKIP_OUTPUT			Mylene1f4_SKIP_OUTPUT

#define context(context) \
    ((struct context_state *) (context))

extern const struct x1f4_ckeytree_type _Mylene1f4_process_root[],
    _Mylene1f4_synchronization_root[];
extern const struct x1f4_vkeytree_type _Mylene1f4_dct_64_root[];

static int fini_mylene_0_0(void *);
static int fini_mylene_1_0(void *);
static int fini_mylene_1_1(void *);
static int fini_mylene_1_2(void *);
static int init_mylene_0_0(void *);
static int init_mylene_1_0(void *);
static int init_mylene_1_1(void *);
static int init_mylene_1_2(void *);
static int kill_mylene_0_0(void *);
static int kill_mylene_1_0(void *);
static int line_output_0_0(void *, const char *, unsigned);
static int line_output_1_3(void *, const char *, unsigned);
static int list_xkeytree_2(void *, const char *, unsigned);

static void SIGCHLD_handler(int);
#if __ENABLE_EQUEUE__
static void back_iqueue(struct equeue_type *, unsigned *,
			struct equeue_type **);
#endif				/* __ENABLE_EQUEUE__ */
static void clean_up(void);
#if __ENABLE_EQUEUE__
static void clip_iqueue(void *, struct iqueue_type *);
static void ever_iqueue(void *, struct iqueue_type *);
#endif				/* __ENABLE_EQUEUE__ */
static void flush_up(void);
#if __ENABLE_EQUEUE__
static void free_iqueue(void *, struct iqueue_type *);
static void lose_iqueue(void *, struct iqueue_type *);
static void miss_iqueue(void *, struct iqueue_type *);
static void page_iqueue(void *, struct iqueue_type *, unsigned);
#endif				/* __ENABLE_EQUEUE__ */
static void print_ckey(const struct x1f4_ckeytree_type *, unsigned);
static void print_vkey(const struct x1f4_vkeytree_type *, unsigned);
static void printf_error_0(void *, const char *, va_list);
static void printf_error_1(void *, const char *, va_list);
#if __ENABLE_EQUEUE__
static void rule_iqueue(void *, struct iqueue_type *);
static void size_iqueue(void *, struct iqueue_type *, char *, const char **);
#endif				/* __ENABLE_EQUEUE__ */
static void usage(void);

static const char *null_progress[] = {
/* *INDENT-OFF* */
    NULL,
    NULL,
    NULL
/* *INDENT-ON* */
}, *path_progress[] = {
/* *INDENT-OFF* */
    "%N\n",
    NULL,
    NULL
/* *INDENT-ON* */
}, *tags_progress[] = {
/* *INDENT-OFF* */
    Mylene1f4_tags_PROGRESS_PROGRESS1__,
    NULL,
    NULL
/* *INDENT-ON* */
}, *x1f4_progress[] = {
/* *INDENT-OFF* */
    Mylene1f4_x1f4_PROGRESS_PROGRESS1__,
    Mylene1f4_x1f4_PROGRESS_PROGRESS2__,
    Mylene1f4_x1f4_PROGRESS_PROGRESS3__
/* *INDENT-ON* */
};
static const struct output_type outputs[] = {
/* *INDENT-OFF* */
    {	fini_mylene_0_0,	init_mylene_0_0,
	kill_mylene_0_0,	Mylene1f4_run_excess0			},
    {	NULL,			NULL,
	NULL,			NULL					},
    {	fini_mylene_0_0,	init_mylene_0_0,
	kill_mylene_0_0,	Mylene1f4_run_excess2			},
    {	fini_mylene_1_2,	init_mylene_1_2,
	kill_mylene_1_0,	Mylene1f4_run_excess3			},
#if __ENABLE_CONTROLLING_SERVER__
    {	NULL,			NULL,
	NULL,			NULL					},
    {	NULL,			NULL,
	NULL,			NULL					},
    {	NULL,			NULL,
	NULL,			NULL					},
    {	fini_mylene_1_2,	init_mylene_1_2,
	kill_mylene_1_0,	Mylene1f4_run_excess7			},
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
#if __ENABLE_x1f4LANES__
    {	NULL,			NULL,
	NULL,			NULL					},
    {	NULL,			NULL,
	NULL,			NULL					},
    {	NULL,			NULL,
	NULL,			NULL					},
    {	fini_mylene_1_2,	init_mylene_1_2,
	kill_mylene_1_0,	Mylene1f4_run_excessb			},
#endif				/* __ENABLE_x1f4LANES__ */
/* *INDENT-ON* */
};
static const struct x1f4_ckeytree_type error_0[] = {
/* *INDENT-OFF* */
    {	"abort",	{	NULL_MISS	},	NULL		},
    {	"ignore",	{	NEAR_MISS	},	NULL		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, error_root[] = {
/* *INDENT-OFF* */
    {	"",		{	0		},	error_0		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, order_0[] = {
/* *INDENT-OFF* */
    {	"back",		{	LOAD_LAST	},	NULL		},
    {	"last",		{	LOAD_LAST	},	NULL		},
    {	"next",		{	LOAD_NEXT	},	NULL		},
    {	"previous",	{	LOAD_LAST	},	NULL		},
    {	"random",	{	LOAD_RAND	},	NULL		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, order_root[] = {
/* *INDENT-OFF* */
    {	"",		{	0		},	order_0		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, output_0[] = {
/* *INDENT-OFF* */
    {	"device",	{	LINK_OUTPUT	},	NULL		},
    {	"null",		{	NULL_OUTPUT	},	NULL		},
    {	"skip",		{	SKIP_OUTPUT	},	NULL		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, output_root[] = {
/* *INDENT-OFF* */
    {	"",		{	0		},	output_0	},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
};
static const struct x1f4_vkeytree_type progress_0[] = {
/* *INDENT-OFF* */
    {	"null",		{	null_progress	},	NULL		},
    {	"path",		{	path_progress	},	NULL		},
    {	"tags",		{	tags_progress	},	NULL		},
    {	"x1f4",		{	x1f4_progress	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, progress_root[] = {
/* *INDENT-OFF* */
    {	"",		{	NULL		},	progress_0	},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
};
static void *global;

static int
fini_mylene_0_0(void *context)
{
    return Mylene1f4_free_mylene0fa(context);
}


static int
fini_mylene_1_0(void *context)
{
    int excess, status = 0;

    if (context(context)->pid != -1) {
	if (context(context)->fd_0 != -1) {
	    status = Mylene1f4_lose_mylene0fa(context);
	    if (status) {
		if (context(context)->pid != -1) {
		    Mylene1f4_kill_mylene0fa(context);
		}
	    }
	}
    }

    excess = Mylene1f4_fini_mylene0fa(context);
    if (status) {
    } else {
	status = excess;
    }
    excess = Mylene1f4_free_mylene0fa(context);
    if (status) {
    } else {
	status = excess;
    }

    return status;
}


static int
fini_mylene_1_1(void *context)
{
    struct mirror_type *mirror_data;
    unsigned count;

    mirror_data = context(context)->mirror;
    count = mirror_data->args.argc;
    if (count) {
	char **items;

	items = mirror_data->args.argv;
	free(items[Mylene0fa_PREFIX]);
	free(items);
    }

    return 0;
}


static int
fini_mylene_1_2(void *context)
{
    int excess, status;

    status = fini_mylene_1_0(context);
    excess = fini_mylene_1_1(context);
    if (status) {
    } else {
	status = excess;
    }

#if __RESTART_Mylene0fa__
    free(context(context)->restarts);
#endif				/* __RESTART_Mylene0fa__ */

    return status;
}


static int
init_mylene_0_0(void *context)
{
    return Mylene1f4_deux_mylene0fa(context);
}


static int
init_mylene_1_0(void *context)
{
    int status;

#if !__WAIT_VIOLET__
    context(context)->SIGCHLD_handler = SIGCHLD_handler;
    context(context)->pid = -1;
#endif				/* !__WAIT_VIOLET__ */

    status = Mylene1f4_set_record(context, Mylene1f4_kill_mylene0fa, context);
    status = Mylene1f4_deux_mylene0fa(context);
    if (status) {
    } else {
	status = Mylene1f4_init_mylene0fa(context);
	if (status) {
	} else {
	    status = Mylene1f4_sync_mylene0fa(context);
	    if (status) {
		Mylene1f4_kill_mylene0fa(context);
		Mylene1f4_fini_mylene0fa(context);
	    }
	}
	if (status) {
	    Mylene1f4_free_mylene0fa(context);
	}
    }

    return status;
}


static int
init_mylene_1_1(void *context)
{
    int status;

    do {
	{
	    char *args;
	    struct mirror_type *mirror_data;

	    mirror_data = context(context)->mirror;

	    args = mirror_data->args.argv;
	    if (!args) {
		mirror_data->args.argc = 0;
	    } else {
		char **items;
		unsigned count;

		status = x1f4_esplit_command
		    (args, Mylene0fa_PREFIX, 0, &count, &items);
		if (status) {
		    break;
		} else {
		    if (count) {
			mirror_data->args.argc = count;
			mirror_data->args.argv = items;
		    } else {
			mirror_data->args.argc = 0;
			free(items[Mylene0fa_PREFIX]);
			free(items);
		    }
		}
	    }
	}

	status = 0;
    } while (0);

    return status;
}


static int
init_mylene_1_2(void *context)
{
    int status;

#if __RESTART_Mylene0fa__
    context(context)->restarts =
	malloc(sizeof(struct timeval) << RESTART_ORDER);
    if (context(context)->restarts) {
	context(context)->restarts_count = 0;
#endif				/* __RESTART_Mylene0fa__ */

	status = init_mylene_1_1(context);
	if (status) {
	} else {
	    status = init_mylene_1_0(context);
	    if (status) {
		fini_mylene_1_1(context);
	    }
	}

#if __RESTART_Mylene0fa__
	if (status) {
	    free(context(context)->restarts);
	} else {
	}
    } else {
	status = -1;

	Mylene1f4_fprintf_error
	    (context, "%s: %s\n", context(context)->Mylene1f4,
	     strerror(errno));
    }
#endif				/* __RESTART_Mylene0fa__ */

    return status;
}


static int
kill_mylene_0_0(void *context)
{
    return 0;
}


static int
kill_mylene_1_0(void *context)
{
    return Mylene1f4_kill_mylene0fa(context);
}


static int
line_output_0_0(void *context, const char *data, unsigned size)
{
    if (size) {
	context(context)->do_clear = data[size - 1] != 10;
	fwrite(data, size, 1, context(context)->progress);
    }

    return 0;
}


static int
line_output_1_3(void *context, const char *data, unsigned size)
{
    fwrite(data, size, 1, context(context)->progress);

    return 0;
}


static int
list_xkeytree_2(void *text, const char *data, unsigned size)
{
    unsigned *list;

    list = text;
    if (*list) {
	*list = 0;
    } else {
	list[1] += 1;
	fputc('|', stdout);
    }

    list[1] += size;
    fwrite(data, size, 1, stdout);

    return 0;
}


static void
SIGCHLD_handler(int signal)
{
    ((struct context_state *) global)->SIGCHLD_received = 1;
}


#if __ENABLE_EQUEUE__
static void
back_iqueue(struct equeue_type *equeue_data, unsigned *v,
	    struct equeue_type **tail)
{
    {
	struct equeue_type *equeue_slip;

	equeue_slip = equeue_data->equeue_data;
	if (equeue_slip) {
	    back_iqueue(equeue_slip, v, tail);
	}
    }
    {
	unsigned e;

	e = *v;
	if (~e) {
	    if (e) {
		free(equeue_data);
		*v = e - 1;
	    } else {
		equeue_data->equeue_data = NULL;
		*tail = equeue_data;
		*v = ~0;
	    }
	}
    }
}
#endif				/* __ENABLE_EQUEUE__ */


static void
clean_up(void)
{
    struct context_state *context_data;
    struct record_type *record_data;

    context_data = global;
    record_data = context_data->record_data;
    while (record_data) {
	record_data->data(record_data->text);

	record_data = record_data->record_data;
    }
}


#if __ENABLE_EQUEUE__
static void
clip_iqueue(void *context, struct iqueue_type *iqueue_data)
{
    if (~(iqueue_data->lqueue | 1)) {
	lose_iqueue(context, iqueue_data);
    }
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static void
ever_iqueue(void *context, struct iqueue_type *iqueue_data)
{
    if (~(iqueue_data->lqueue | 1)) {
	page_iqueue(context, iqueue_data, 1);
    }
}
#endif				/* __ENABLE_EQUEUE__ */


static void
flush_up(void)
{
    struct context_state *context_data;
    void *progress;

    context_data = global;

    progress = context_data->progress;

    if (context_data->do_clear) {
	context_data->do_clear = 0;
	if (progress) {
	    fputc('\n', progress);
	}
    }

    if (progress) {
	fflush(progress);
    }
}


#if __ENABLE_EQUEUE__
static void
free_iqueue(void *context, struct iqueue_type *iqueue_data)
{
    struct equeue_type *miss_equeue;

    miss_equeue = iqueue_data->head_equeue;
    if (miss_equeue) {
	do {
	    struct equeue_type *equeue_data;

	    equeue_data = miss_equeue->equeue_data;
	    Mylene1f4_free_equeue(context, miss_equeue);
	    if (equeue_data) {
	    } else {
		break;
	    }
	    miss_equeue = equeue_data->equeue_data;
	    Mylene1f4_free_equeue(context, equeue_data);
	    if (miss_equeue) {
	    } else {
		break;
	    }
	} while (1);
    }
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static void
lose_iqueue(void *context, struct iqueue_type *iqueue_data)
{
    miss_iqueue(context, iqueue_data);
    iqueue_data->lqueue = 0;
    iqueue_data->tail_equeue = NULL;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static void
miss_iqueue(void *context, struct iqueue_type *iqueue_data)
{
    free_iqueue(context, iqueue_data);
    iqueue_data->head_equeue = NULL;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static void
page_iqueue(void *context, struct iqueue_type *iqueue_data, unsigned v)
{
    unsigned lqueue;

    lqueue = iqueue_data->lqueue;
    if (v < lqueue) {
	if (v) {
	    iqueue_data->lqueue = lqueue - v;
	    back_iqueue
		(iqueue_data->head_equeue, &v, &iqueue_data->tail_equeue);
	}
    } else {
	lose_iqueue(context, iqueue_data);
    }
}
#endif				/* __ENABLE_EQUEUE__ */


static void
print_ckey(const struct x1f4_ckeytree_type *root, unsigned header)
{
    unsigned list[2];

    list[0] = 1;
    list[1] = header;

    x1f4_list_ctkey(list, list_xkeytree_2, root);

    if (31 < list[1]) {
	fputs("\n\t\t\t\t", stdout);
    } else {
	unsigned i;

	i = (32 - list[1] + 7) >> 3;
	while (i) {
	    fputc('\t', stdout);
	    i--;
	}
    }
}


static void
print_vkey(const struct x1f4_vkeytree_type *root, unsigned header)
{
    unsigned list[2];

    list[0] = 1;
    list[1] = header;

    x1f4_list_vtkey(list, list_xkeytree_2, root);

    if (31 < list[1]) {
	fputs("\n\t\t\t\t", stdout);
    } else {
	unsigned i;

	i = (32 - list[1] + 7) >> 3;
	while (i) {
	    fputc('\t', stdout);
	    i--;
	}
    }
}


static void
printf_error_0(void *context, const char *format, va_list ap)
{
}


static void
printf_error_1(void *context, const char *format, va_list ap)
{
    struct context_state *context_data;
    void *progress;

    context_data = context;

    progress = context_data->progress;

    if (context_data->do_clear) {
	context_data->do_clear = 0;
	if (progress) {
	    fputc('\n', progress);
	}
    }

    if (progress) {
	fflush(progress);
    }

    vfprintf(stderr, format, ap);
}


#if __ENABLE_EQUEUE__
static void
rule_iqueue(void *context, struct iqueue_type *iqueue_data)
{
    struct equeue_type *equeue_data;

    context(context)->window.server.iqueue = iqueue_data;

    equeue_data = iqueue_data->head_equeue;
    while (equeue_data) {
	Mylene1f4_init_moon(context, equeue_data);
	equeue_data = equeue_data->equeue_data;
    }
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static void
size_iqueue(void *context, struct iqueue_type *iqueue_data,
	    char *specification, const char **error)
{
    if (~(iqueue_data->lqueue | 1)) {
	unsigned cardinal;

	if (x1f4_parse_wxcardinal(&cardinal, specification, NULL, 0)) {
	    iqueue_data->lqueue = ~0 ^ 1;
	    *error = specification;
	} else {
	    page_iqueue(context, iqueue_data, cardinal);
	}
    }
}
#endif				/* __ENABLE_EQUEUE__ */


static void
usage(void)
{
    fputs("Usage: Mylene1f4 [OPTIONS] [FILE]\n\
Play audio FILE(s).\n\
\n\
Options:"
#if __ENABLE_x1f4LANES__
	 "\n\
  -!				talk in lanes"
#endif				/* __ENABLE_x1f4LANES__ */
	 "\n\
  -1, --progress1 FORMAT	set format for printing stream header progress\
\n\
				information to FORMAT\n\
  -2, --progress2 FORMAT	set format for printing frame progress\n\
				information to FORMAT\n\
  -3, --progress3 FORMAT	set format for printing stream trailer progr\
ess\n\
				information to FORMAT\n\
  -A, --album ALBUM		play ALBUM only FILE(s)\n\
  -B, --basename BASENAME	play BASENAME only FILE(s)\n\
  -C, --device-name NAME	set output device first argument to NAME\n\
  -D, --device-args ARGS	set output device extra arguments to ARGS"
#if __ENABLE_FILTER__
	 "\n\
  -F, --filter FILTER		play FILTER only FILE(s)"
#else
	 "\n\
  -F, --filter FILTER		ignored"
#endif				/* __ENABLE_FILTER__ */
	 "\n\
  -L, --load-order ", stdout);
    print_ckey(order_root, 19);
    fputs("set load order\n\
  -P, --path PATH		play PATH only FILE(s)\n\
  -S, --shuffle			shuffle files list\n\
  -a, --artist ARTIST		play ARTIST only FILE(s)\n\
  -b, --buffer SIZE		set output cache size to SIZE * 36864 bytes\n\
  -c, --comment COMMENT		play COMMENT only FILE(s)\n\
  -d, --device PATH		set output device path to PATH\n\
  -e, --start-time [[HOURS:]MINUTES:]SECONDS[,FRAMES]\n\
				set play start stream time\n\
  -f, --start-frame INDEX	skip first INDEX frames"
#if __ENABLE_TAGGED_LIST__
	 "\n\
  -i, --tagged-list PATH	read the files tagged list from PATH"
#else
	 "\n\
  -i, --tagged-list PATH	ignored"
#endif				/* __ENABLE_TAGGED_LIST__ */
	 "\n\
  -l, --list PATH		read the files list from PATH\n\
  -o, --output ", stdout);
    print_ckey(output_root, 15);
    fputs("set output to OUTPUT\n\
  -p, --progress-type ", stdout);
    print_vkey(progress_root, 22);
    fputs("set format for printing progress information\n\
  -q, --quiet			suppress error messages\n\
  -r, --cycle, --repeat		restart playing FILE(s) after playing the la\
st\n\
  -s, --start INDEX		start playing FILE(s) at INDEX\n\
  -t, --title TITLE		play TITLE only FILE(s)\n\
  -v, --verbose			?\n\
  -x, --synchronize		seek audio stream (default)"
#if __LINK_CELL_A52A_LOADER__
	 "\n\
      --a52a-complaints		allow a52a loader complaints\n\
      --a52a-loader PATH	set a52a loader path to PATH"
#else
	 "\n\
      --a52a-complaints		ignored\n\
      --a52a-loader PATH	ignored"
#endif				/* __LINK_CELL_A52A_LOADER__ */
	 "\n\
      --attached		start system audio device attached (default)\n\
      --backward-seek-complaints\n\
				allow backward seek complaints (default)"
#if __ENABLE_EQUEUE__
	 "\n\
      --clear-title		discard the next to be queue program title\n\
      --context-interrupt COUNT	set maximum context uninterrupted instructio\
n\n\
				execution count to COUNT\n\
      --context-list-length LENGTH\n\
				set maximum context execution list length to\n\
				LENGTH"
#else
	 "\n\
      --clear-title		ignored\n\
      --context-interrupt COUNT	ignored\n\
      --context-list-length LENGTH\n\
				ignored"
#endif				/* __ENABLE_EQUEUE__ */
	 "\n\
      --dct64 1st|2nd|mmx	?"
#if __ENABLE_EQUEUE__
	 "\n\
      --deque N			remove last N programs from the execution qu\
eue\n\
      --deque-all		remove all programs from the execution queue\n\
      --deque-last		remove last program from the execution queue"
#else
	 "\n\
      --deque N			ignored\n\
      --deque-all		ignored\n\
      --deque-last		ignored"
#endif				/* __ENABLE_EQUEUE__ */
	 "\n\
      --detached		start system audio device detached"
#if __ENABLE_FILTER__
	 "\n\
      --disable-expression-optimizations, --enable-expression-optimizations\n\
				disable or enable filter expression\n\
				optimizations (enable by default)"
#else
	 "\n\
      --disable-expression-optimizations, --enable-expression-optimizations\n\
				ignored"
#endif				/* __ENABLE_FILTER__ */
	 "\n\
      --disable-id3v2-read, --enable-id3v2-read\n\
				disable or enable ID3 version 2 (ID3v2) read\
ing\n\
				(enable by default)\n\
      --dumb			do not seek for audio stream (default)\n\
      --error ", stdout);
    print_ckey(error_root, 14);
    fputs("set load error behavior (abort by default)"
#if __ENABLE_FILTER__
	 "\n\
      --head-b-operators, --list-b-operators\n\
				list filter available binary operators and e\
xit"
#else
	 "\n\
      --head-b-operators, --list-b-operators\n\
				ignored"
#endif				/* __ENABLE_FILTER__ */
#if __ENABLE_EQUEUE__
	 "\n\
      --head-p-b-operators, --list-p-b-operators\n\
				list program available binary operators and\n\
				exit"
#else
	 "\n\
      --head-p-b-operators, --list-p-b-operators\n\
				ignored"
#endif				/* __ENABLE_EQUEUE__ */
	 "\n\
      --id3v2-complaints	allow ID3v2 parser complaints\n\
      --id3v2-parser PATH	set ID3v2 parser path to PATH\n\
      --id3v2-timeout MICROSECONDS\n\
				set ID3v2 loading timeout to MICROSECONDS\n\
      --id3v2-transfer SIZE	set ID3v2 transfer size to SIZE bytes"
#if __ENABLE_CONTROLLING_SERVER__
	 "\n\
      --inet-server PORT	establish PORT TCP/IP controlling server"
#else
	 "\n\
      --inet-server PORT	ignored"
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
#if __ENABLE_MPEG_EQUALIZER__
	 "\n\
      --layer3-amplification FACTOR\n\
				set amplification for layer3 streams\n\
      --layer3-frequencies FREQUENCIES\n\
				set equalizer band frequencies for layer3\n\
				streams\n\
      --layer3-levels LEVELS	set equalizer band levels for layer3 streams"
#else
	 "\n\
      --layer3-amplification FACTOR\n\
				ignored\n\
      --layer3-frequencies FREQUENCIES\n\
				ignored\n\
      --layer3-levels LEVELS	ignored"
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
	 "\n\
      --list-directives, --pack-directives, --tree-directives\n\
				display formatting directives and exit"
#if __ENABLE_FILTER__
	 "\n\
      --list-fs-operators	list filter available file selection operato\
rs\n\
				and exit"
#else
	 "\n\
      --list-fs-operators	ignored"
#endif				/* __ENABLE_FILTER__ */
#if __ENABLE_FILTER__
	 "\n\
      --list-functions		list filter available functions and exit"
#else
	 "\n\
      --list-functions		ignored"
#endif				/* __ENABLE_FILTER__ */
#if __ENABLE_EQUEUE__
	 "\n\
      --list-p-fs-operators	list program available file selection operat\
ors\n\
				and exit"
#else
	 "\n\
      --list-p-fs-operators	ignored"
#endif				/* __ENABLE_EQUEUE__ */
#if __ENABLE_EQUEUE__
	 "\n\
      --list-p-functions, --list-q-functions\n\
				list program available functions and exit"
#else
	 "\n\
      --list-p-functions, --list-q-functions\n\
				ignored"
#endif				/* __ENABLE_EQUEUE__ */
#if __ENABLE_EQUEUE__
	 "\n\
      --list-p-u-operators	list program available unary operators and e\
xit"
#else
	 "\n\
      --list-p-u-operators	ignored"
#endif				/* __ENABLE_EQUEUE__ */
#if __ENABLE_FILTER__
	 "\n\
      --list-u-operators	list filter available unary operators and ex\
it"
#else
	 "\n\
      --list-u-operators	ignored"
#endif				/* __ENABLE_FILTER__ */
	 "\n\
      --mpeg-synchronization ", stdout);
    print_ckey(_Mylene1f4_synchronization_root, 29);
    fputs("set mpeg audio synchronization method\n\
      --mpeg-synchronization-seek BYTES\n\
				seek no more than this many bytes when looking\
\n\
				for the audio stream\n\
      --mpeg-synchronization-sequence COUNT\n\
				set synchronization frames count"
#if __LINK_CELL_A52A_LOADER__
	 "\n\
      --no-a52a-complaints	suppress a52a loader complaints (default)"
#else
	 "\n\
      --no-a52a-complaints	ignored"
#endif				/* __LINK_CELL_A52A_LOADER__ */
	 "\n\
      --no-backward-seek-complaints\n\
				suppress backward seek complaints\n\
      --no-cycle, --no-repeat	restart not playing FILE(s) after playing th\
e\n\
				last\n\
      --no-id3v2-complaints	suppress ID3v2 parser complaints (default)"
#if __ENABLE_CONTROLLING_SERVER__
	 "\n\
      --no-reuse-address	reuse no address when binding INET server\n\
      --no-server-filter-complaints\n\
				suppress server filter parsing complaints\n\
      --no-server-format-complaints\n\
				suppress server format parsing complaints\n\
      --no-server-progress-complaints\n\
				suppress server progress parsing complaints"
#else
	 "\n\
      --no-reuse-address	ignored\n\
      --no-server-filter-complaints\n\
				ignored\n\
      --no-server-format-complaints\n\
				ignored\n\
      --no-server-progress-complaints\n\
				ignored"
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
	 "\n\
      --no-shuffle		shuffle not files list\n\
      --no-synchronization	seek not audio stream\n\
      --null-process ", stdout);
    print_ckey(_Mylene1f4_process_root, 21);
    puts("set audio decoding process for null output\n\
      --paused			start paused"
#if __ENABLE_CONTROLLING_SERVER__
	 "\n\
      --play-list-length LENGTH	set maximum play list length to LENGTH"
#else
	 "\n\
      --play-list-length LENGTH	ignored"
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
	 "\n\
      --playing			start playing (default)"
#if __ENABLE_EQUEUE__
	 "\n\
      --program-allocation BYTES\n\
				set memory allocation limit for programs to\n\
				BYTES\n\
      --program-interrupt COUNT	set maximum program uninterrupted instructio\
n\n\
				execution count to COUNT"
#else
	 "\n\
      --program-allocation BYTES\n\
				ignored\n\
      --program-interrupt COUNT	ignored"
#endif				/* __ENABLE_EQUEUE__ */
	 "\n\
      --progress-print		print progress information formatting and ex\
it"
#if __ENABLE_EQUEUE__
	 "\n\
      --queue PROGRAM		queue PROGRAM for execution\n\
      --queue-length LENGTH	set maximum execution queue length to LENGTH"
#else
	 "\n\
      --queue PROGRAM		ignored\n\
      --queue-length LENGTH	ignored"
#endif				/* __ENABLE_EQUEUE__ */
#if __ENABLE_CONTROLLING_SERVER__
	 "\n\
      --request-transfer SIZE	set maximum request size to SIZE bytes\n\
      --reuse-address		reuse local address when binding INET server\n\
      --server-filter-complaints\n\
				allow server filter parsing complaints\n\
				(default)\n\
      --server-format-complaints\n\
				allow server format parsing complaints\n\
				(default)\n\
      --server-progress-complaints\n\
				allow server progress parsing complaints\n\
				(default)"
#if __ENABLE_EQUEUE__
	 "\n\
      --store-title TITLE	set the next to be queued program title to\n\
				TITLE"
#else
	 "\n\
      --store-title TITLE	ignored"
#endif				/* __ENABLE_EQUEUE__ */
	 "\n\
      --unix-server PATH	establish PATH local TCP controlling server\n\
      --unix-server-mode MODE	set file permissions for local TCP controlli\
ng\n\
				server to integral MODE"
#else
	 "\n\
      --request-transfer SIZE	ignored\n\
      --reuse-address		ignored\n\
      --server-filter-complaints\n\
				ignored\n\
      --server-format-complaints\n\
				ignored\n\
      --server-progress-complaints\n\
				ignored\n\
      --unix-server PATH	ignored\n\
      --unix-server-mode MODE	ignored"
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
	 "\n\
      --help			display this help and exit\n\
      --version			output version information and exit");
}


int
main(int argc, char **argv)
{
    const char *list_path = NULL;
#if __ENABLE_FILTER__
    const char *filter_specs = NULL;
#endif				/* __ENABLE_FILTER__ */
    int do_shuffle = 0, fini_list = 0, status = 0;
    struct context_state context;
#if __LINK_CELL_A52A_LOADER__
    struct ea52al_type ea52al;
#endif				/* __LINK_CELL_A52A_LOADER__ */
    struct eid3wl_type eid3wl;
#if __ENABLE_EQUEUE__
    struct iqueue_type iqueue;
#endif				/* __ENABLE_EQUEUE__ */
    struct mirror_type mirror;
    const struct output_type *output_data;
    struct printer_state printer;
#if __ENABLE_CONTROLLING_WINDOW__
    unsigned window = 0;
#endif				/* __ENABLE_CONTROLLING_WINDOW__ */

    context.Mylene1f4 = x1f4_basename(argv[0]);
    context.SIGCHLD_received = 0;
#if __ENABLE_x00696e6b__
    context.audio_entry = ~0;
#endif				/* __ENABLE_x00696e6b__ */
#if __NEAR_LINK_BITS__
    context.bits = 0;
#endif				/* __NEAR_LINK_BITS__ */
    context.cache_size = Mylene1f4_CACHE;
    context.class = Mylene1f4_LINK_OUTPUT;
    context.do_clear = 0;
    context.do_explain = 0;
    context.do_repeat = 0;
    context.do_synchronize = 1;
#if __ENABLE_FILTER__
    context.e4deck = NULL;
    context.e4line = 1;
#endif				/* __ENABLE_FILTER__ */
    context.e1f4_index = E1f4_START_INDEX;
#if __LINK_CELL_A52A_LOADER__
    context.ea52al = &ea52al;
#endif				/* __LINK_CELL_A52A_LOADER__ */
    context.eid3wl = &eid3wl;
    context.flags = FLAGS_DEFAULT___;
    context.index_frame = NULL;
    context.line_index = NULL;
    context.load_order = Mylene1f4_LOAD_NEXT;
    context.mirror = &mirror;
    context.no_backward_seek_complaints = 0;
    context.printf_error = printf_error_1;
    context.printer_data = &printer;
    context.progress = NULL;
    context.progress1 = NULL;
    context.progress2 = NULL;
    context.progress3 = NULL;
    context.record_data = NULL;
    context.select = Mylene1f4_PAGE_PICKUP;
    context.start = 0;
#if __WAIT_VIOLET__
    context.violet = NULL;
#endif				/* __WAIT_VIOLET__ */

#if __ENABLE_MPEG_EQUALIZER__
    context.mpeg.amplification = 0;
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
    context.mpeg.dct_64 = NULL;
#if __ENABLE_MPEG_EQUALIZER__
    context.mpeg.l3band_data = NULL;
    context.mpeg.mode = 0;
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
    context.mpeg.process = NULL;
    context.mpeg.seek = 4194304;
    context.mpeg.synchronization = 0;
    context.mpeg.synchronization_sequence = 4;

    context.pickup.page = 0;

    context.text.album.data = NULL;
    context.text.lbase.data = NULL;
    context.text.lpath.data = NULL;
    context.text.match.data = NULL;
    context.text.notes.data = NULL;
    context.text.slack.data = NULL;
    context.text.slack.size = 0;
    context.text.title.data = NULL;

#if __LINK_CELL_A52A_LOADER__
    ea52al.flags = Mylene1f4_EA52AL_QUIET;
    ea52al.loader = NULL;
#endif				/* __LINK_CELL_A52A_LOADER__ */

    eid3wl.flags = Mylene1f4_EID3WL_QUIET;
    eid3wl.parser = NULL;
    eid3wl.timeout = 0;
    eid3wl.transfer = 0;

#if __ENABLE_EQUEUE__
    iqueue.head_equeue = NULL;
    iqueue.lqueue = 0;
    iqueue.tail_equeue = NULL;
#endif				/* __ENABLE_EQUEUE__ */

    mirror.name = Mylene1f4_MIRROR;
    mirror.path = Mylene1f4_OUTPUT;

    mirror.args.argc = 0;
    mirror.args.argv = NULL;

    Mylene1f4_setup_output(&context);

#if __ENABLE_FILTER__
    Mylene1f4_setup_filter(&context);
#endif				/* __ENABLE_FILTER__ */

    unsetenv("POSIXLY_CORRECT");

    {
#if __ENABLE_EQUEUE__
	const char *allocation_deck_specs = NULL;
#endif				/* __ENABLE_EQUEUE__ */
	const char *buffer_specs = NULL;
#if __ENABLE_EQUEUE__
	const char *context_list_length_specs = NULL;
#endif				/* __ENABLE_EQUEUE__ */
	const char *dct64_specs = NULL;
#if __ENABLE_EQUEUE__
	const char *deque_error = NULL;
#endif				/* __ENABLE_EQUEUE__ */
	const char *error_specs = NULL, *id3v2_timeout_specs = NULL,
	    *id3v2_transfer_specs = NULL;
#if __ENABLE_EQUEUE__
	const char *interrupt_class_specs = NULL, *interrupt_count_specs =
	    NULL;
#endif				/* __ENABLE_EQUEUE__ */
#if __ENABLE_MPEG_EQUALIZER__
	const char *layer3_amplification_specs = NULL,
	    *layer3_frequencies_specs = NULL, *layer3_levels_specs = NULL;
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
#if __ENABLE_EQUEUE__
	const char *line_title = NULL;
#endif				/* __ENABLE_EQUEUE__ */
	const char *load_order_specs = NULL,
	    *mpeg_synchronization_seek_specs = NULL,
	    *mpeg_synchronization_sequence_specs = NULL,
	    *mpeg_synchronization_specs = NULL, *output_specs = NULL;
#if __ENABLE_CONTROLLING_SERVER__
	const char *play_list_length_specs = NULL;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
	const char *process_specs = NULL, *progress1_specs = NULL,
	    *progress2_specs = NULL, *progress3_specs = NULL,
	    *progress__specs = NULL;
#if __ENABLE_EQUEUE__
	const char *queue_length_specs = NULL;
#endif				/* __ENABLE_EQUEUE__ */
#if __ENABLE_CONTROLLING_SERVER__
	const char *request_transfer_specs = NULL;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
	const char *start_frame_specs = NULL, *start_specs = NULL,
	    *start_time_specs = NULL;
#if __ENABLE_CONTROLLING_SERVER__
	const char *unix_server_mode_specs = NULL;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
	int fast = ~0, print_progress = 0;
#if __ENABLE_CONTROLLING_SERVER__
	unsigned delay_server = 0, fussy_server = FUSSY_SERVER, pause_server =
	    0, reuse_record = 0;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */

	while (1) {
	    int c;
	    static struct option long_options[] = {
/* *INDENT-OFF* */
		{   "a52a-complaints",
				    0x00,   NULL,   0xaf    },
		{   "a52a-loader",  0x01,   NULL,   0xb0    },
		{   "album",	    0x01,   NULL,   0x41    },
		{   "artist",	    0x01,   NULL,   0x61    },
		{   "attached",	    0x00,   NULL,   0x0d    },
		{   "backward-seek-complaints",
				    0x00,   NULL,   0xce    },
		{   "basename",	    0x01,   NULL,   0x42    },
		{   "buffer",	    0x01,   NULL,   0x62    },
		{   "clear-title",  0x00,   NULL,   0xa6    },
		{   "comment",	    0x01,   NULL,   0x63    },
		{   "context-interrupt",
				    0x01,   NULL,   0x9f    },
		{   "context-list-length",
				    0x01,   NULL,   0xa9    },
		{   "cycle",	    0x00,   NULL,   0x72    },
		{   "dct64",	    0x01,   NULL,   0x10    },
		{   "deque",	    0x01,   NULL,   0xa4    },
		{   "deque-all",    0x00,   NULL,   0xa2    },
		{   "deque-last",   0x00,   NULL,   0xa3    },
		{   "detached",	    0x00,   NULL,   0x0c    },
		{   "device",	    0x01,   NULL,   0x64    },
		{   "device-args",  0x01,   NULL,   0x44    },
		{   "device-name",  0x01,   NULL,   0x43    },
		{   "disable-expression-optimizations",
				    0x00,   NULL,   0xef    },
		{   "disable-id3v2-read",
				    0x00,   NULL,   0xdf    },
		{   "dumb",	    0x00,   NULL,   0xbe    },
		{   "enable-expression-optimizations",
				    0x00,   NULL,   0xee    },
		{   "enable-id3v2-read",
				    0x00,   NULL,   0xde    },
		{   "error",	    0x01,   NULL,   0x80    },
		{   "head-b-operators",
				    0x00,   NULL,   0x99    },
		{   "head-p-b-operators",
				    0x00,   NULL,   0x9a    },
		{   "id3v2-complaints",
				    0x00,   NULL,   0xd9    },
		{   "id3v2-parser", 0x01,   NULL,   0xdc    },
		{   "id3v2-timeout",
				    0x01,   NULL,   0xdb    },
		{   "id3v2-transfer",
				    0x01,   NULL,   0xda    },
		{   "filter",	    0x01,   NULL,   0x46    },
		{   "list",	    0x01,   NULL,   0x6c    },
		{   "inet-server",  0x01,   NULL,   0x08    },
		{   "output",	    0x01,   NULL,   0x6f    },
		{   "layer3-amplification",
				    0x01,   NULL,   0xc1    },
		{   "layer3-frequencies",
				    0x01,   NULL,   0xc2    },
		{   "layer3-levels",
				    0x01,   NULL,   0xc3    },
		{   "list-b-operators",
				    0x00,   NULL,   0x19    },
		{   "list-directives",
				    0x00,   NULL,   0x1a    },
		{   "list-fs-operators",
				    0x00,   NULL,   0x1f    },
		{   "list-functions",
				    0x00,   NULL,   0x1e    },
		{   "list-p-b-operators",
				    0x00,   NULL,   0x9b    },
		{   "list-p-fs-operators",
				    0x00,   NULL,   0x9c    },
		{   "list-p-functions",
				    0x00,   NULL,   0x1d    },
		{   "list-p-u-operators",
				    0x00,   NULL,   0x9d    },
		{   "list-q-functions",
				    0x00,   NULL,   0x1d    },
		{   "list-u-operators",
				    0x00,   NULL,   0x18    },
		{   "load-order",   0x01,   NULL,   0x4c    },
		{   "mpeg-synchronization",
				    0x01,   NULL,   0xc0    },
		{   "mpeg-synchronization-seek",
				    0x01,   NULL,   0xbd    },
		{   "mpeg-synchronization-sequence",
				    0x01,   NULL,   0xbf    },
		{   "no-a52a-complaints",
				    0x00,   NULL,   0xae    },
		{   "no-backward-seek-complaints",
				    0x00,   NULL,   0xcf    },
		{   "no-cycle",	    0x00,   NULL,   0xd2    },
		{   "no-id3v2-complaints",
				    0x00,   NULL,   0xd8    },
		{   "no-repeat",    0x00,   NULL,   0xd2    },
		{   "no-reuse-address",
				    0x00,   NULL,   0x16    },
		{   "no-server-filter-complaints",
				    0x00,   NULL,   0xcb    },
		{   "no-server-format-complaints",
				    0x00,   NULL,   0xcd    },
		{   "no-server-progress-complaints",
				    0x00,   NULL,   0xc9    },
		{   "no-shuffle",   0x00,   NULL,   0xd3    },
		{   "no-synchronization",
				    0x00,   NULL,   0xc7    },
		{   "null-process", 0x01,   NULL,   0x11    },
		{   "pack-directives",
				    0x00,   NULL,   0x1b    },
		{   "path",	    0x01,   NULL,   0x50    },
		{   "paused",	    0x00,   NULL,   0x0f    },
		{   "play-list-length",
				    0x01,   NULL,   0xa7    },
		{   "playing",	    0x00,   NULL,   0x0e    },
		{   "program-allocation",
				    0x01,   NULL,   0xaa    },
		{   "program-interrupt",
				    0x01,   NULL,   0xa0    },
		{   "progress-print",
				    0x00,   NULL,   0x07    },
		{   "progress-type",
				    0x01,   NULL,   0x70    },
		{   "progress1",    0x01,   NULL,   0x31    },
		{   "progress2",    0x01,   NULL,   0x32    },
		{   "progress3",    0x01,   NULL,   0x33    },
		{   "queue",	    0x01,   NULL,   0xa1    },
		{   "queue-length", 0x01,   NULL,   0xa8    },
		{   "quiet",	    0x00,   NULL,   0x71    },
		{   "repeat",	    0x00,   NULL,   0x72    },
		{   "request-transfer",
				    0x01,   NULL,   0x0b    },
		{   "reuse-address",
				    0x00,   NULL,   0x17    },
		{   "server-filter-complaints",
				    0x00,   NULL,   0xca    },
		{   "server-format-complaints",
				    0x00,   NULL,   0xcc    },
		{   "server-progress-complaints",
				    0x00,   NULL,   0xc8    },
		{   "shuffle",	    0x00,   NULL,   0x53    },
		{   "start",	    0x01,   NULL,   0x73    },
		{   "start-time",   0x01,   NULL,   0x65    },
		{   "start-frame",  0x01,   NULL,   0x66    },
		{   "store-title",  0x01,   NULL,   0xa5    },
		{   "synchronize",  0x00,   NULL,   0x78    },
		{   "tagged-list",  0x01,   NULL,   0x69    },
		{   "title",	    0x01,   NULL,   0x74    },
		{   "tree-directives",
				    0x00,   NULL,   0x1c    },
		{   "unix-server",  0x01,   NULL,   0x09    },
		{   "unix-server-mode",
				    0x01,   NULL,   0x0a    },
		{   "verbose",	    0x00,   NULL,   0x76    },
		{   "version",	    0x00,   NULL,   0x56    },
		{   "help",	    0x00,   NULL,   0x48    },
		{   NULL,	    0x00,   NULL,   0x00    }
/* *INDENT-ON* */
	    };

	    c = getopt_long(argc, argv, SHORT_STRING____, long_options, NULL);

	    if (!~c) {
		break;
	    }

	    switch (c) {
#if __ENABLE_x1f4LANES__
	    case '!':
		window = Mylene1f4_WITHIN_WINDOW;
		break;
#endif				/* __ENABLE_x1f4LANES__ */
	    case '1':
		progress1_specs = optarg;
		break;
	    case '2':
		progress2_specs = optarg;
		break;
	    case '3':
		progress3_specs = optarg;
		break;
	    case 'A':
		context.text.album.data = optarg;
		break;
	    case 'B':
		context.text.lbase.data = optarg;
		break;
	    case 'C':
		mirror.name = optarg;
		break;
	    case 'D':
		mirror.args.argv = optarg;
		break;
	    case 'F':
#if __ENABLE_FILTER__
		filter_specs = optarg;
#endif				/* __ENABLE_FILTER__ */
		break;
	    case 'L':
		load_order_specs = optarg;
		break;
	    case 'P':
		context.text.lpath.data = optarg;
		break;
	    case 'S':
		do_shuffle = 1;
		break;
	    case 'a':
		context.text.match.data = optarg;
		break;
	    case 'b':
		buffer_specs = optarg;
		break;
	    case 'c':
		context.text.notes.data = optarg;
		break;
	    case 'd':
		mirror.path = optarg;
		break;
	    case 'e':
		start_frame_specs = NULL;
		start_time_specs = optarg;
		break;
	    case 'f':
		start_frame_specs = optarg;
		start_time_specs = NULL;
		break;
	    case 'i':
#if __ENABLE_TAGGED_LIST__
		list_path = optarg;
		context.flags |= Mylene1f4_BREAK_LIST;
#endif				/* __ENABLE_TAGGED_LIST__ */
		break;
	    case 'l':
		list_path = optarg;
#if __ENABLE_TAGGED_LIST__
		context.flags &= ~Mylene1f4_BREAK_LIST;
#endif				/* __ENABLE_TAGGED_LIST__ */
		break;
	    case 'o':
		output_specs = optarg;
		break;
	    case 'p':
		progress1_specs = NULL;
		progress2_specs = NULL;
		progress3_specs = NULL;
		progress__specs = optarg;
		break;
	    case 'q':
#if __NEED_QUIET__
		context.flags |= Mylene1f4_QUIET_HERE;
#endif				/* __NEED_QUIET__ */
		context.printf_error = printf_error_0;
		break;
	    case 'r':
		context.do_repeat = 1;
		break;
	    case 's':
		start_specs = optarg;
		break;
	    case 't':
		context.text.title.data = optarg;
		break;
	    case 'v':
		context.do_explain++;
		break;
	    case 'x':
		context.do_synchronize = 1;
		break;
	    case 007:
		print_progress = 1;
		break;
	    case 010:
#if __ENABLE_CONTROLLING_SERVER__
		window = Mylene1f4_SERVER_WINDOW;
		context.window.server.port = optarg;
		context.window.server.type = 0;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
		break;
	    case 011:
#if __ENABLE_CONTROLLING_SERVER__
		window = Mylene1f4_SERVER_WINDOW;
		context.window.server.port = optarg;
		context.window.server.type = 1;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
		break;
	    case 012:
#if __ENABLE_CONTROLLING_SERVER__
		unix_server_mode_specs = optarg;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
		break;
	    case 013:
#if __ENABLE_CONTROLLING_SERVER__
		request_transfer_specs = optarg;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
		break;
	    case 014:
#if __ENABLE_CONTROLLING_SERVER__
		delay_server = Mylene1f4_DELAY_SERVER;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
		break;
	    case 015:
#if __ENABLE_CONTROLLING_SERVER__
		delay_server = 0;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
		break;
	    case 016:
#if __ENABLE_CONTROLLING_SERVER__
		pause_server = 0;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
		break;
	    case 017:
#if __ENABLE_CONTROLLING_SERVER__
		pause_server = Mylene1f4_PAUSE_SERVER;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
		break;
	    case 020:
		dct64_specs = optarg;
		break;
	    case 021:
		process_specs = optarg;
		break;
	    case 026:
#if __ENABLE_CONTROLLING_SERVER__
		reuse_record = 0;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
		break;
	    case 027:
#if __ENABLE_CONTROLLING_SERVER__
		reuse_record = Mylene1f4_REUSE_RECORD;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
		break;
#if !__ENABLE_FILTER__
	    case 030:
#endif				/* !__ENABLE_FILTER__ */
#if !__ENABLE_FILTER__
	    case 031:
#endif				/* !__ENABLE_FILTER__ */
#if !__ENABLE_EQUEUE__
	    case 035:
#endif				/* !__ENABLE_EQUEUE__ */
#if !__ENABLE_FILTER__
	    case 036:
#endif				/* !__ENABLE_FILTER__ */
#if !__ENABLE_FILTER__
	    case 037:
#endif				/* !__ENABLE_FILTER__ */
#if !__ENABLE_FILTER__
	    case 153:
#endif				/* !__ENABLE_FILTER__ */
#if !__ENABLE_EQUEUE__
	    case 154:
#endif				/* !__ENABLE_EQUEUE__ */
#if !__ENABLE_EQUEUE__
	    case 155:
#endif				/* !__ENABLE_EQUEUE__ */
#if !__ENABLE_EQUEUE__
	    case 156:
#endif				/* !__ENABLE_EQUEUE__ */
#if !__ENABLE_EQUEUE__
	    case 157:
#endif				/* !__ENABLE_EQUEUE__ */
#if !__ENABLE_EXHELP__
		break;
#endif				/* !__ENABLE_EXHELP__ */
#if __ENABLE_FILTER__
	    case 030:
#endif				/* __ENABLE_FILTER__ */
#if __ENABLE_FILTER__
	    case 031:
#endif				/* __ENABLE_FILTER__ */
	    case 032:
	    case 033:
	    case 034:
#if __ENABLE_EQUEUE__
	    case 035:
#endif				/* __ENABLE_EQUEUE__ */
#if __ENABLE_FILTER__
	    case 036:
#endif				/* __ENABLE_FILTER__ */
#if __ENABLE_FILTER__
	    case 037:
#endif				/* __ENABLE_FILTER__ */
#if __ENABLE_FILTER__
	    case 153:
#endif				/* __ENABLE_FILTER__ */
#if __ENABLE_EQUEUE__
	    case 154:
#endif				/* __ENABLE_EQUEUE__ */
#if __ENABLE_EQUEUE__
	    case 155:
#endif				/* __ENABLE_EQUEUE__ */
#if __ENABLE_EQUEUE__
	    case 156:
#endif				/* __ENABLE_EQUEUE__ */
#if __ENABLE_EQUEUE__
	    case 157:
#endif				/* __ENABLE_EQUEUE__ */
	    case 'H':
	    case 'V':
		if (!~fast) {
		    fast = c;
		}
		break;
	    case 128:
		error_specs = optarg;
		break;
	    case 159:
#if __ENABLE_EQUEUE__
		interrupt_class_specs = optarg;
#endif				/* __ENABLE_EQUEUE__ */
		break;
	    case 160:
#if __ENABLE_EQUEUE__
		interrupt_count_specs = optarg;
#endif				/* __ENABLE_EQUEUE__ */
		break;
	    case 161:
#if __ENABLE_EQUEUE__
		Mylene1f4_link_equeue(&iqueue, optarg, &line_title);
#endif				/* __ENABLE_EQUEUE__ */
		break;
	    case 162:
#if __ENABLE_EQUEUE__
		clip_iqueue(&context, &iqueue);
#endif				/* __ENABLE_EQUEUE__ */
		break;
	    case 163:
#if __ENABLE_EQUEUE__
		ever_iqueue(&context, &iqueue);
#endif				/* __ENABLE_EQUEUE__ */
		break;
	    case 164:
#if __ENABLE_EQUEUE__
		size_iqueue(&context, &iqueue, optarg, &deque_error);
#endif				/* __ENABLE_EQUEUE__ */
		break;
	    case 165:
#if __ENABLE_EQUEUE__
		line_title = optarg;
#endif				/* __ENABLE_EQUEUE__ */
		break;
	    case 166:
#if __ENABLE_EQUEUE__
		line_title = NULL;
#endif				/* __ENABLE_EQUEUE__ */
		break;
	    case 167:
#if __ENABLE_CONTROLLING_SERVER__
		play_list_length_specs = optarg;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
		break;
	    case 168:
#if __ENABLE_EQUEUE__
		queue_length_specs = optarg;
#endif				/* __ENABLE_EQUEUE__ */
		break;
	    case 169:
#if __ENABLE_EQUEUE__
		context_list_length_specs = optarg;
#endif				/* __ENABLE_EQUEUE__ */
		break;
	    case 170:
#if __ENABLE_EQUEUE__
		allocation_deck_specs = optarg;
#endif				/* __ENABLE_EQUEUE__ */
		break;
	    case 174:
#if __LINK_CELL_A52A_LOADER__
		ea52al.flags |= Mylene1f4_EA52AL_QUIET;
#endif				/* __LINK_CELL_A52A_LOADER__ */
		break;
	    case 175:
#if __LINK_CELL_A52A_LOADER__
		ea52al.flags &= ~Mylene1f4_EA52AL_QUIET;
#endif				/* __LINK_CELL_A52A_LOADER__ */
		break;
	    case 176:
#if __LINK_CELL_A52A_LOADER__
		ea52al.loader = optarg;
#endif				/* __LINK_CELL_A52A_LOADER__ */
		break;
	    case 189:
		mpeg_synchronization_seek_specs = optarg;
		break;
	    case 190:
		context.do_synchronize = 0;
		break;
	    case 191:
		mpeg_synchronization_sequence_specs = optarg;
		break;
	    case 192:
		mpeg_synchronization_specs = optarg;
		break;
	    case 193:
#if __ENABLE_MPEG_EQUALIZER__
		layer3_amplification_specs = optarg;
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
		break;
	    case 194:
#if __ENABLE_MPEG_EQUALIZER__
		layer3_frequencies_specs = optarg;
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
		break;
	    case 195:
#if __ENABLE_MPEG_EQUALIZER__
		layer3_levels_specs = optarg;
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
		break;
	    case 199:
		context.do_synchronize = 0;
		break;
	    case 200:
#if __ENABLE_CONTROLLING_SERVER__
		fussy_server |= Mylene1f4_FUSSY_OUTPUT;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
		break;
	    case 201:
#if __ENABLE_CONTROLLING_SERVER__
		fussy_server &= ~Mylene1f4_FUSSY_OUTPUT;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
		break;
	    case 202:
#if __ENABLE_CONTROLLING_SERVER__
		fussy_server |= Mylene1f4_FUSSY_FILTER;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
		break;
	    case 203:
#if __ENABLE_CONTROLLING_SERVER__
		fussy_server &= ~Mylene1f4_FUSSY_FILTER;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
		break;
	    case 204:
#if __ENABLE_CONTROLLING_SERVER__
		fussy_server |= Mylene1f4_FUSSY_FORMAT;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
		break;
	    case 205:
#if __ENABLE_CONTROLLING_SERVER__
		fussy_server &= ~Mylene1f4_FUSSY_FORMAT;
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
		break;
	    case 206:
		context.no_backward_seek_complaints = 0;
		break;
	    case 207:
		context.no_backward_seek_complaints = 1;
		break;
	    case 210:
		context.do_repeat = 0;
		break;
	    case 211:
		do_shuffle = 0;
		break;
	    case 216:
		eid3wl.flags |= Mylene1f4_EID3WL_QUIET;
		break;
	    case 217:
		eid3wl.flags &= ~Mylene1f4_EID3WL_QUIET;
		break;
	    case 218:
		id3v2_transfer_specs = optarg;
		break;
	    case 219:
		id3v2_timeout_specs = optarg;
		break;
	    case 220:
		eid3wl.parser = optarg;
		break;
	    case 222:
		context.flags |= Mylene1f4_ID3V2_READ;
		break;
	    case 223:
		context.flags &= ~Mylene1f4_ID3V2_READ;
		break;
	    case 238:
#if __ENABLE_FILTER__
		context.e4line |= 1;
#endif				/* __ENABLE_FILTER__ */
		break;
	    case 239:
#if __ENABLE_FILTER__
		context.e4line &= ~1;
#endif				/* __ENABLE_FILTER__ */
		break;
	    case '?':
		status = 1;
	    }

	    if (status) {
		break;
	    }
	}

	do {
	    if (status) {
		break;
	    }

	    if (~fast) {
		switch (fast) {
		case 030:
		case 031:
		case 036:
		case 037:
		case 153:
#if __ENABLE_FILTER__
		    Mylene1f4_dump_filter(fast);
#endif				/* __ENABLE_FILTER__ */
		    break;
		case 032:
		case 033:
		case 034:
		    Mylene1f4_dump_format(fast);
		    break;
		case 035:
		case 154:
		case 155:
		case 156:
		case 157:
#if __ENABLE_EQUEUE__
		    Mylene1f4_detail_aime(&context, fast);
#endif				/* __ENABLE_EQUEUE__ */
		    break;
		case 'H':
		    usage();
		    break;
		case 'V':
		    printf("%s (%s) %s\n", argv[0], PACKAGE, VERSION);
		}

		break;
	    }

#if __ENABLE_TAGGED_LIST__
	    if (context.flags & Mylene1f4_BREAK_LIST) {
		context.flags &= ~Mylene1f4_ID3V2_READ;
	    }
#endif				/* __ENABLE_TAGGED_LIST__ */

#if __ENABLE_EQUEUE__
	    if (!~(iqueue.lqueue | 1)) {
		if (!~iqueue.lqueue) {
		} else {
		    Mylene1f4_fprintf_error
			(&context, "%s: cannot parse dequeueing specificatio"
			 "n: `%s'\n", context.Mylene1f4, deque_error);
		}

		status = 1;
		break;
	    }
#endif				/* __ENABLE_EQUEUE__ */

	    if (buffer_specs) {
		status = Mylene1f4_parse_positive
		    (&context, "cache size", buffer_specs,
		     &context.cache_size);
		if (status) {
		    break;
		}
	    }

	    if (error_specs) {
		unsigned error;

		status = Mylene1f4_parse_ctreekey
		    (&context, "error", error_root, error_specs, &error);
		if (status) {
		    break;
		} else {
		    context.flags |= error;
		}
	    }

	    if (mpeg_synchronization_specs) {
		status = Mylene1f4_parse_ctreekey
		    (&context, "mpeg synchronization",
		     _Mylene1f4_synchronization_root,
		     mpeg_synchronization_specs,
		     &context.mpeg.synchronization);
		if (status) {
		    break;
		}
	    }

	    if (mpeg_synchronization_seek_specs) {
		status = Mylene1f4_parse_cardinal
		    (&context, "mpeg synchronization seek",
		     mpeg_synchronization_seek_specs, &context.mpeg.seek);
		if (status) {
		    break;
		}
	    }

	    if (context.mpeg.synchronization == Mylene1f4_SELECT_FRAME) {
		if (mpeg_synchronization_sequence_specs) {
		    status = Mylene1f4_parse_cardinal
			(&context, "mpeg synchronization sequence",
			 mpeg_synchronization_sequence_specs,
			 &context.mpeg.synchronization_sequence);
		    if (status) {
			break;
		    }
		}
	    }

	    if (dct64_specs) {
		status = Mylene1f4_parse_vtreekey
		    (&context, "dct64", _Mylene1f4_dct_64_root, dct64_specs,
		     &context.mpeg.dct_64);
		if (status) {
		    break;
		}
	    }

#if __ENABLE_MPEG_EQUALIZER__
	    if (layer3_amplification_specs) {
		status = Mylene1f4_parse_amplification
		    (&context, layer3_amplification_specs);
		if (status) {
		    break;
		}
	    }
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
	    if (layer3_frequencies_specs) {
		status = Mylene1f4_parse_frequencies
		    (&context, layer3_frequencies_specs);
		if (status) {
		    break;
		}
	    } else {
		status = Mylene1f4_tee_mode(&context);
		if (status) {
		    Mylene1f4_fprintf_error
			(&context, "%s: %s\n", context.Mylene1f4,
			 strerror(errno));
		    break;
		}
	    }
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
	    if (layer3_levels_specs) {
		status = Mylene1f4_parse_levels(&context, layer3_levels_specs);
		if (status) {
		    break;
		}
	    }
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

	    if (output_specs) {
		status = Mylene1f4_parse_ctreekey
		    (&context, "output", output_root, output_specs,
		     &context.class);
		if (status) {
		    break;
		}
	    }

	    if (load_order_specs) {
		status = Mylene1f4_parse_ctreekey
		    (&context, "load order", order_root, load_order_specs,
		     &context.load_order);
		if (status) {
		    break;
		}
	    }

	    if (start_specs) {
		status = Mylene1f4_parse_integral
		    (&context, "start index", start_specs, &context.start);
		if (status) {
		    break;
		}
	    }

	    if (start_frame_specs) {
		status = Mylene1f4_parse_cardinal
		    (&context, "start frame index", start_frame_specs,
		     &context.pickup.page);
		if (status) {
		    break;
		}
	    } else if (start_time_specs) {
		status = Mylene1f4_parse_time
		    (&context, "start time", start_time_specs,
		     &context.pickup.time);
		if (status) {
		    break;
		} else {
		    context.select = Mylene1f4_TIME_PICKUP;
		}
	    }

	    if (id3v2_timeout_specs) {
		status = Mylene1f4_parse_cardinal
		    (&context, "ID3v2 timeout", id3v2_timeout_specs,
		     &eid3wl.timeout);
		if (status) {
		    break;
		}
	    }

	    if (id3v2_transfer_specs) {
		status = Mylene1f4_parse_cardinal
		    (&context, "ID3v2 transfer", id3v2_transfer_specs,
		     &eid3wl.transfer);
		if (status) {
		    break;
		}
	    }

	    if (progress__specs) {
		const void *this;

		status = Mylene1f4_parse_vtreekey
		    (&context, "progress", progress_root, progress__specs,
		     &this);
		if (status) {
		    break;
		} else {
		    const char *const *progress;

		    progress = this;
		    if (progress1_specs) {
		    } else {
			progress1_specs = progress[0];
		    }
		    if (progress2_specs) {
		    } else {
			progress2_specs = progress[1];
		    }
		    if (progress3_specs) {
		    } else {
			progress3_specs = progress[2];
		    }
		}
	    }

	    if (print_progress) {
		puts("progress1:");
		if (progress1_specs) {
		    puts(progress1_specs);
		}
		puts("progress2:");
		if (progress2_specs) {
		    puts(progress2_specs);
		}
		puts("progress3:");
		if (progress3_specs) {
		    puts(progress3_specs);
		}

		fast = 1048576;

		break;
	    }

#if __ENABLE_CONTROLLING_SERVER__
	    if (window == Mylene1f4_SERVER_WINDOW) {
		if (context.window.server.type) {
		    context.window.server.mode = unix_server_mode_specs;
		} else {
		    fussy_server |= reuse_record;
		}

		{
		    context.window.server.setup =
			delay_server | fussy_server | pause_server;
		}

		if (!play_list_length_specs) {
		    context.window.server.list_length = 65536;
		} else {
		    status = Mylene1f4_parse_cardinal
			(&context, "maximum play list length",
			 play_list_length_specs,
			 &context.window.server.list_length);
		    if (status) {
			break;
		    }
		}

		if (!request_transfer_specs) {
		    context.window.server.transfer =
			MAXIMUM_REQUEST_TRANSFER_SIZE;
		} else {
		    status = Mylene1f4_parse_cardinal
			(&context, "ID3v2 timeout", request_transfer_specs,
			 &context.window.server.transfer);
		    if (status) {
			break;
		    }
		}

# if __ENABLE_EQUEUE__
		if (!allocation_deck_specs) {
		    context.window.server.equeue_size = 2097152;
		} else {
		    status = Mylene1f4_parse_cardinal
			(&context, "memory allocation limit for programs",
			 allocation_deck_specs,
			 &context.window.server.equeue_size);
		    if (status) {
			break;
		    }
		}
# endif				/* __ENABLE_EQUEUE__ */

# if __ENABLE_EQUEUE__
		if (!interrupt_class_specs) {
		    context.window.server.intermiss = 65536;
		} else {
		    status = Mylene1f4_parse_cardinal
			(&context, "maximum context uninterrupted instructio"
			 "n execution count", interrupt_class_specs,
			 &context.window.server.intermiss);
		    if (status) {
			break;
		    }
		}
# endif				/* __ENABLE_EQUEUE__ */

# if __ENABLE_EQUEUE__
		if (!interrupt_count_specs) {
		    context.window.server.interrupt = 65536;
		} else {
		    status = Mylene1f4_parse_cardinal
			(&context, "maximum program uninterrupted instructio"
			 "n execution count", interrupt_count_specs,
			 &context.window.server.interrupt);
		    if (status) {
			break;
		    }
		}
# endif				/* __ENABLE_EQUEUE__ */

# if __ENABLE_EQUEUE__
		if (!context_list_length_specs) {
		    context.window.server.context_length = 1024;
		} else {
		    status = Mylene1f4_parse_cardinal
			(&context, "maximum program execution context list l"
			 "ength", context_list_length_specs,
			 &context.window.server.context_length);
		    if (status) {
			break;
		    }
		}
# endif				/* __ENABLE_EQUEUE__ */

# if __ENABLE_EQUEUE__
		if (!queue_length_specs) {
		    context.window.server.queue_length = 4096;
		} else {
		    status = Mylene1f4_parse_cardinal
			(&context, "maximum program execution queue length",
			 queue_length_specs,
			 &context.window.server.queue_length);
		    if (status) {
			break;
		    }
		}
# endif				/* __ENABLE_EQUEUE__ */

# if __ENABLE_EQUEUE__
		if (iqueue.head_equeue) {
		    rule_iqueue(&context, &iqueue);
		} else {
		    context.window.server.iqueue = NULL;
		}
# endif				/* __ENABLE_EQUEUE__ */
	    } else {
# if __ENABLE_EQUEUE__
		if (iqueue.head_equeue) {
		    miss_iqueue(&context, &iqueue);
		}
# endif				/* __ENABLE_EQUEUE__ */
	    }
#endif				/* __ENABLE_CONTROLLING_SERVER__ */

	    {
		status = Mylene1f4_init_output
		    (&context, progress1_specs, progress2_specs,
		     progress3_specs);
		if (status) {
		    break;
		}
	    }
	} while (0);
	if (status) {
	    goto bye;
	}

	if (~fast) {
	    goto bye;
	}

	if (context.class == Mylene1f4_NULL_OUTPUT) {
	    context.ground.futile.process = process_specs;
	}
    }

    output_data = outputs;
    output_data += context.class;
#if __ENABLE_CONTROLLING_WINDOW__
    output_data += window;
    if (output_data->sync) {
    } else {
	Mylene1f4_fprintf_error
	    (&context, "%s: unsupported operating mode\n", argv[0]);
	status = 1;
	goto bye;
    }
#endif				/* __ENABLE_CONTROLLING_WINDOW__ */

    context.run_mylene = output_data->sync;

    gettimeofday(&context.becca, NULL);

    srand(context.becca.tv_sec);

#if __ENABLE_x1f4LANES__
    if (window == Mylene1f4_WITHIN_WINDOW) {
	if (optind != argc) {
	    fprintf(stderr, "%s: too many arguments\nTry `%s --help' for mor"
		    "e information\n", argv[0], argv[0]);
	    context.run_mylene = NULL;
	} else {
	    context.flags &= ~Mylene1f4_LOCAL_LIST;
	}
    } else {
# if __ENABLE_x00696e6b__
#  if 0
	context.audio_entry = ~0;
#  endif			/* 0 */
# endif				/* __ENABLE_x00696e6b__ */
#endif				/* __ENABLE_x1f4LANES__ */

	do {
#if __ENABLE_FILTER__
	    if (filter_specs) {
		status = Mylene1f4_aime_filter
		    (&context, filter_specs, &context.e4deck);
		if (status) {
		    break;
		}
	    }
#endif				/* __ENABLE_FILTER__ */

	    if (list_path) {
		status = Mylene1f4_read_list(&context, list_path, do_shuffle);
		if (status) {
		    break;
		}
		if (context.excess.e1.list87.count) {
		} else {
		    context.run_mylene = NULL;
		    if (1) {
			break;
		    }
		}
	    } else {
		if (optind != argc) {
		    status = Mylene1f4_init_list
			(&context, argc - optind, argv + optind, do_shuffle);
		    if (status) {
			break;
		    }
		} else {
		    context.run_mylene = NULL;
		    if (1) {
			break;
		    }
		}
	    }

	    {
		fini_list = 1;

		if (context.text.album.data) {
		    Mylene1f4_strip_graphique(&context, &context.text.album);
		}
		if (context.text.lbase.data) {
		    Mylene1f4_strip_graphique(&context, &context.text.lbase);
		}
		if (context.text.lpath.data) {
		    Mylene1f4_strip_graphique(&context, &context.text.lpath);
		}
		if (context.text.match.data) {
		    Mylene1f4_strip_graphique(&context, &context.text.match);
		}
		if (context.text.notes.data) {
		    Mylene1f4_strip_graphique(&context, &context.text.notes);
		}
		if (context.text.title.data) {
		    Mylene1f4_strip_graphique(&context, &context.text.title);
		}
	    }
	} while (0);
	if (status) {
	    context.run_mylene = NULL;
	}
#if __ENABLE_x1f4LANES__
    }
#endif				/* __ENABLE_x1f4LANES__ */

    if (context.run_mylene) {
	context.clause_data = NULL;

	context.progress = stdout;
	if (isatty(STDOUT_FILENO)) {
	    context.line_out = line_output_0_0;
	} else {
	    context.line_out = line_output_1_3;
	}

	global = &context;
	status = Mylene1f4_all_signals(context.Mylene1f4, flush_up, clean_up);

#if __ENABLE_CONTROLLING_WINDOW__
	context.mode = window;
#endif				/* __ENABLE_CONTROLLING_WINDOW__ */

	context.clip = 0;
	context.play = 0;

	context.device = output_data;

	if (context.flags & Mylene1f4_QUIET_HERE) {
#if __LINK_CELL_A52A_LOADER__
	    ea52al.flags |= Mylene1f4_EA52AL_QUIET;
#endif				/* __LINK_CELL_A52A_LOADER__ */
	    eid3wl.flags |= Mylene1f4_EID3WL_QUIET;
	}

#if __WAIT_VIOLET__
	context.pid = -1;
#endif				/* __WAIT_VIOLET__ */

#if __WAIT_VIOLET__
	signal(SIGCHLD, SIGCHLD_handler);
#endif				/* __WAIT_VIOLET__ */

	status = context.run_mylene(&context);

#if __WAIT_VIOLET__
	signal(SIGCHLD, SIG_DFL);
#endif				/* __WAIT_VIOLET__ */

	{
	    struct clause_type *clause_data;

	    clause_data = context.clause_data;
	    if (clause_data) {
		context.clause_data = NULL;
		do {
		    struct clause_type *clause_swap;

		    clause_swap = clause_data;
		    clause_data = clause_data->clause_data;
		    free(clause_swap);
		} while (clause_data);
	    }
	}
    }

    if (context.text.slack.data) {
	free(context.text.slack.data);
    }

    if (fini_list) {
	Mylene1f4_fini_list(&context);
    }

#if __ENABLE_FILTER__
    if (context.e4deck) {
	Mylene1f4_fini_filter(&context, &context.e4deck);
    }
#endif				/* __ENABLE_FILTER__ */

    {
	struct record_type *record_data;

	record_data = context.record_data;
	if (record_data) {
	    context.record_data = NULL;
	    do {
		struct record_type *record_swap;

		record_swap = record_data;
		record_data = record_data->record_data;
		free(record_swap);
	    } while (record_data);
	}
    }

  bye:
    Mylene1f4_fini_output(&context);

#if __ENABLE_FILTER__
    Mylene1f4_clear_filter(&context);
#endif				/* __ENABLE_FILTER__ */

    Mylene1f4_clear_output(&context);

#if __ENABLE_MPEG_EQUALIZER__
    Mylene1f4_air_mode(&context);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_EQUEUE__
    free_iqueue(&context, &iqueue);
#endif				/* __ENABLE_EQUEUE__ */

    return status;
}
