/*
 * aime.c
 * Copyright (C) 2007-2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if __ENABLE_EQUEUE__
# include <aime.h>
# if defined HAVE_LIBLEWD
#  include <lewd.h>
# endif				/* HAVE_LIBLEWD */
# if defined HAVE_LIBx1f4l0
#  include <libx1f4l0.h>
# endif				/* HAVE_LIBx1f4l0 */
# include <stdio.h>
# include <stddef.h>
# include <stdlib.h>
# include <string.h>
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# include <access.h>
# include <aime1.h>
# include <defs.h>
# include <excess4.h>
# include <excess7.h>
# include <inter.h>
# include <sfnote.h>
# include <types.h>
# if !defined HAVE_LIBLEWD
#  include <vtkey.h>
# endif				/* !HAVE_LIBLEWD */
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
#else
# undef __ENABLE_FQUEUE__
# define __ENABLE_FQUEUE__		0
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define ABORT_CLASS			1
# define BREAK_CLASS			2
# define QUEUE_CLASS			3
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define LINK_AIME			((void *) -1)
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define I_MODE(i)			(*((X1f4_E4_C_MODE *) (i)))
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define Frame4(context) \
    ((struct frame4_type *) &Frame7((context))->_1.frame4)
# define Frame7(context) \
    ((struct frame7_type *) WINDOW((context)).server.window)
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define a1text(context) \
    ((struct a1text_type *) Frame4((context))->a1text)
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define e4text(context) \
    ((struct e4text_type *) Frame4((context))->lxdeck)
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define offset_p(c1text) \
    ((void *) ((char *) (c1text)))
# define offset_q(c1text) \
    ((void *) ((char *) (c1text)))
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define offset_q_4(frame4) \
    offset_q((frame4)->c1text)
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_FQUEUE__
# define FIX_TIE(context) \
    if (1) {								      \
	struct frame4_type *frame4_data;				      \
									      \
	frame4_data = Frame4((context));				      \
									      \
	if (frame4_data->link & 4) {					      \
	    ((struct c1text_state *)					      \
	     offset_q_4(frame4_data))->o1text.consolidated =		      \
		((struct c1text_state *)				      \
		 offset_q_4(frame4_data))->o1text.suspended;		      \
	} else {							      \
	    ((struct c1text_state *)					      \
	     offset_q_4(frame4_data))->o1text.consolidated =		      \
		WINDOW((context)).server.intermiss;			      \
	}								      \
    }
#elif __ENABLE_EQUEUE__
# define FIX_TIE(context) \
    if (1) {								      \
	if (1) {							      \
	    ((struct c1text_state *)					      \
	     offset_q_4(Frame4((context))))->o1text.consolidated =	      \
		WINDOW((context)).server.intermiss;			      \
	}								      \
    }
#endif				/* __ENABLE_FQUEUE__ */

#if __ENABLE_FQUEUE__
# define SET_TIE(context, status) \
    if (!status) {							      \
	struct frame4_type *frame4_data;				      \
									      \
	frame4_data = Frame4((context));				      \
									      \
	if (1) {							      \
	    ((struct c1text_state *)					      \
	     offset_q_4(frame4_data))->o1text.suspended =		      \
		((struct c1text_state *)				      \
		 offset_q_4(frame4_data))->o1text.consolidated;		      \
	}								      \
    }
#elif __ENABLE_EQUEUE__
# define SET_TIE(context, status)
#endif				/* __ENABLE_FQUEUE__ */

#if __ENABLE_EQUEUE__
typedef X1f4_E4_C_TEXT e4_c_text;
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
typedef struct o1text_type {
    int delete, linked;
#endif				/* __ENABLE_EQUEUE__ */
#if __ENABLE_CHANNEL_EQUEUE__
    unsigned channel, client_id;
#endif				/* __ENABLE_CHANNEL_EQUEUE__ */
#if __ENABLE_EQUEUE__
    unsigned consolidated;
#endif				/* __ENABLE_EQUEUE__ */
#if __ENABLE_FQUEUE__
    unsigned suspended;
#endif				/* __ENABLE_FQUEUE__ */
#if __ENABLE_EQUEUE__
    void *aime, *deck, *functions;
} o1text_type;
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
typedef struct c1text_state {
    struct o1text_type o1text;
} c1text_state;
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
typedef union deck_type {
    struct {
	unsigned post;
    } v1;
} deck_type;
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
extern const struct x1f4_function_type _Mylene1f4_aime_1_F__[],
    _Mylene1f4_aime_1_I_2[], _Mylene1f4_aime_1_I__[], _Mylene1f4_aime_1_L_2[],
    _Mylene1f4_aime_1_L__[], _Mylene1f4_aime_1_q_l[], _Mylene1f4_aime_1_q_s[],
    _Mylene1f4_aime_1____[], _Mylene1f4_aime_3_F__[], _Mylene1f4_aime_3_L_2[],
    _Mylene1f4_aime_3_L__[], _Mylene1f4_aime_3_R__[], _Mylene1f4_aime_3_S__[],
    _Mylene1f4_filter_F__[], _Mylene1f4_filter_N__[], _Mylene1f4_filter_U__[];
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
static int case_aime(void *, int);
static int copy_aime(void *, unsigned, struct equeue_type *);
static int init_1fix(struct sample_type *, struct c1text_state *, void *);
static int init_2fix(struct sample_type *, struct c1text_state *, void *);
static int init_aime(void *, struct equeue_type *);
static int init_text(void *);
static int line_aime(void *, struct equeue_type *);
static int line_text(void *);
static int link_aile(void *);
static int link_aime(void *);
static int nonsense_080101_8(void *, void *, void **);
static int nonsense_080102_0(void *, void *, void **);
static int nonsense_080117_0(void *, void *, void **);
static int nonsense_080117_1(void *, void *, void **);
#endif				/* __ENABLE_EQUEUE__ */
#if __ENABLE_FQUEUE__
static int nonsense_080402_1(void *, void *, void **);
#endif				/* __ENABLE_FQUEUE__ */
#if __ENABLE_EQUEUE__
static int nonsense_081003_0(void *, void *, void **);
static int nonsense_081012_0(void *, void *, void **);
static int slip_aile(void *);
static int slip_aime(void *);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
static void fini_aime(void *);
static void fini_text(void *, struct c1text_state *);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
static const int c_____m__[] = {
/* *INDENT-OFF* */
    X1f4_E4_MODE
/* *INDENT-ON* */
};
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
const struct x1f4_function_type _Mylene1f4_aime___E__[] = {
/* *INDENT-OFF* */
    {	"E_down",		nonsense_080117_1,
	X1f4_E4_VOID,		c_____m__,	1,
	X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK | M1f4_L7_TEXT_LINK,
						6			},
    {	"E_exit",		nonsense_080102_0,
	X1f4_E4_VOID,		NULL,		0,
	X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK | M1f4_L7_TEXT_LINK,
						6			},
#endif				/* __ENABLE_EQUEUE__ */
#if __ENABLE_FQUEUE__
    {	"E_load",		nonsense_080402_1,
	X1f4_E4_VOID,		NULL,		0,
	X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK | M1f4_L7_TEXT_LINK,
						6			},
#endif				/* __ENABLE_FQUEUE__ */
#if __ENABLE_EQUEUE__
    {	"E_loop",		nonsense_080117_0,
	X1f4_E4_VOID,		NULL,		0,
	X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK | M1f4_L7_TEXT_LINK,
						6			},
    {	"E_play",		nonsense_080101_8,
	X1f4_E4_VOID,		NULL,		0,
	X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK | M1f4_L7_TEXT_LINK,
						6			},
    {	NULL,			NULL,
	0,			NULL,		0,
	0,					1			}
/* *INDENT-ON* */
}, _Mylene1f4_aime___F__[] = {
/* *INDENT-OFF* */
    {	"F_abort",		nonsense_081003_0,
	X1f4_E4_VOID,		NULL,		0,
	X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK | M1f4_L7_TEXT_LINK,
						7			},
    {	"F_suspend",		nonsense_081012_0,
	X1f4_E4_VOID,		c_____m__,	1,
	X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK | M1f4_L7_TEXT_LINK,
						9			},
    {	NULL,			NULL,
	0,			NULL,		0,
	0,					1			}
/* *INDENT-ON* */
};
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
static const struct x1f4_linetext_type *NON_nets[] = {
/* *INDENT-OFF* */
    NULL
/* *INDENT-ON* */
};
static const struct x1f4_function_type *NON_sets[] = {
/* *INDENT-OFF* */
    _Mylene1f4_aime_1_F__,
    _Mylene1f4_aime_1_I_2,
    _Mylene1f4_aime_1_I__,
    _Mylene1f4_aime_1_L_2,
    _Mylene1f4_aime_1_L__,
    _Mylene1f4_aime_1____,
    _Mylene1f4_aime_1_q_l,
    _Mylene1f4_aime_1_q_s,
    _Mylene1f4_aime_3_F__,
    _Mylene1f4_aime_3_L_2,
    _Mylene1f4_aime_3_L__,
    _Mylene1f4_aime_3_R__,
    _Mylene1f4_aime_3_S__,
    _Mylene1f4_aime___E__,
    _Mylene1f4_aime___F__,
    _Mylene1f4_filter_F__,
    _Mylene1f4_filter_N__,
    _Mylene1f4_filter_U__,
    NULL
/* *INDENT-ON* */
};
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
static int
case_aime(void *context, int slip)
{
    int status;

    if (1) {
	void *states;

	states = Frame4(context)->line.states;
	if (states) {
	    x1f4_high_sfnote(states);
	    Frame4(context)->line.down = 256;
	    Frame4(context)->line.size = 65536;
	}
    }

    FIX_TIE(context);

    do {
	if (slip) {
	    status = slip_aime(context);
	    if (status) {
		break;
	    } else {
		if (Frame4(context)->logic != LINK_AIME) {
		    break;
		}
	    }
	}
	do {
	    status = link_aime(context);
	    if (status) {
		break;
	    } else {
		if (Frame4(context)->logic != LINK_AIME) {
		    break;
		}
	    }
	} while (1);
    } while (0);

    SET_TIE(context, status);

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
copy_aime(void *context, unsigned bits, struct equeue_type *equeue_data)
{
# if __ENABLE_CHANNEL_SERVER__
    struct c1text_state *c1text_data;
# endif				/* __ENABLE_CHANNEL_SERVER__ */

# if __ENABLE_CHANNEL_SERVER__
    c1text_data = offset_q_4(Frame4(context));
# endif				/* __ENABLE_CHANNEL_SERVER__ */

# if __ENABLE_CHANNEL_SERVER__
    c1text_data->o1text.channel = equeue_data->channel;
    c1text_data->o1text.client_id = equeue_data->client;
# endif				/* __ENABLE_CHANNEL_SERVER__ */

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
init_1fix(struct sample_type *sample_data, struct c1text_state *c1text_data,
	  void *context)
{
    int status;

    if (c1text_data->o1text.linked) {
	status = 0;
	sample_data->operator1s = a1text(context)->prefixing;
    } else {
	status = Mylene1f4_push_aime(context);
	if (status) {
	} else {
	    c1text_data->o1text.linked = 1;
	    sample_data->operator1s = a1text(context)->prefixing;
	}
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
init_2fix(struct sample_type *sample_data, struct c1text_state *c1text_data,
	  void *context)
{
    int status;

    if (c1text_data->o1text.linked) {
	status = 0;
	sample_data->operator2s = a1text(context)->operators;
    } else {
	status = Mylene1f4_push_aime(context);
	if (status) {
	} else {
	    c1text_data->o1text.linked = 1;
	    sample_data->operator2s = a1text(context)->operators;
	}
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
init_aime(void *context, struct equeue_type *equeue_data)
{
    int status;
    struct c1text_state *c1text_data;
    struct frame4_type *frame4_data;
    struct sample_type sample;

    frame4_data = Frame4(context);

    c1text_data = offset_q_4(frame4_data);

    status = init_2fix(&sample, c1text_data, context);
    if (status) {
    } else {
	status = init_1fix(&sample, c1text_data, context);
	if (status) {
	} else {
	    sample.function_set.miss = c1text_data->o1text.functions;

	    status = Mylene1f4_pick_aime(context, equeue_data, &sample);
	    if (status) {
	    } else {
		c1text_data->o1text.aime = equeue_data->data.aime;

		frame4_data->flags |= Mylene1f4_AIME_LINKED;
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
init_text(void *context)
{
    int status;

    status = line_text(context);
    if (status) {
    } else {
	struct c1text_state *c1text_data;

	c1text_data = (struct c1text_state *)
	    malloc(sizeof(struct c1text_state));
	if (!c1text_data) {
	    status = -1;
	} else {
	    status = 0;

	    c1text_data->o1text.aime = NULL;
	    c1text_data->o1text.functions = NULL;

	    c1text_data->o1text.linked = 0;

	    Frame4(context)->c1text = offset_p(c1text_data);
	}
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
line_aime(void *context, struct equeue_type *equeue_data)
{
    struct c1text_state *c1text_data;
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    c1text_data = offset_q_4(frame4_data);

    c1text_data->o1text.aime = equeue_data->data.aime;

    frame4_data->flags |= Mylene1f4_AIME_LINKED;

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
line_text(void *context)
{
    int status;

    status = Mylene1f4_line_filter(context);
    if (status) {
    } else {
	status = Mylene1f4_line_e4lx(context);
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
link_aile(void *context)
{
    return case_aime(context, 0);
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
link_aime(void *context)
{
    int status;
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    while (1) {
	unsigned bits;
	struct equeue_type *equeue_data, *head_equeue;

	equeue_data = frame4_data->iqueue.head_equeue;

	frame4_data->iqueue.lqueue--;

	head_equeue = equeue_data->equeue_data;
	if (head_equeue) {
	    frame4_data->iqueue.head_equeue = head_equeue;
	} else {
	    frame4_data->iqueue.head_equeue = NULL;
	    frame4_data->iqueue.tail_equeue = NULL;
	}

	bits = equeue_data->bits;

	if (bits & Mylene1f4_INTERN_LOGIC) {
	    status = line_aime(context, equeue_data);
	} else {
	    status = init_aime(context, equeue_data);
	}

	if (status) {
	} else {
	    if (1) {
		copy_aime(context, bits, equeue_data);
	    }
	}

	if (status) {
	    Mylene1f4_free_equeue(context, equeue_data);

	    if (head_equeue) {
	    } else {
		frame4_data->logic = NULL;

		status = 0;

		break;
	    }
	} else {
	    frame4_data->logic = slip_aile;

	    frame4_data->dana.eclass = equeue_data;

	    status = slip_aime(context);

	    break;
	}
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
nonsense_080101_8(void *context, void *output, void **input)
{
    ((struct c1text_state *) offset_q_4(Frame4(context)))->o1text.delete =
	BREAK_CLASS;

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
nonsense_080102_0(void *context, void *output, void **input)
{
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    frame4_data->request = Mylene1f4_EXIT_Mylene;

    ((struct c1text_state *) offset_q_4(frame4_data))->o1text.delete =
	BREAK_CLASS;

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
nonsense_080117_0(void *context, void *output, void **input)
{
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    frame4_data->instant = 1;

    ((struct c1text_state *) offset_q_4(frame4_data))->o1text.delete =
	BREAK_CLASS;

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
nonsense_080117_1(void *context, void *output, void **input)
{
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    {
	X1f4_E4_C_MODE mode;

	mode = I_MODE(input[0]);
	if (mode < 2) {
	} else {
	    frame4_data->instant = ((mode - 1) & 0x3fffffff) << 1;
	}
    }

    ((struct c1text_state *) offset_q_4(frame4_data))->o1text.delete =
	BREAK_CLASS;

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_FQUEUE__
static int
nonsense_080402_1(void *context, void *output, void **input)
{
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);
    frame4_data->link = 1;
    ((struct c1text_state *) offset_q_4(frame4_data))->o1text.delete =
	BREAK_CLASS;

    return 0;
}
#endif				/* __ENABLE_FQUEUE__ */


#if __ENABLE_EQUEUE__
static int
nonsense_081003_0(void *context, void *output, void **input)
{
    ((struct c1text_state *) offset_q_4(Frame4(context)))->o1text.delete =
	ABORT_CLASS;

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
nonsense_081012_0(void *context, void *output, void **input)
{
    X1f4_E4_C_MODE mode;
    struct frame4_type *frame4_data;
    unsigned post, sign, tail;

    frame4_data = Frame4(context);

    mode = I_MODE(input[0]);
    if (mode < 0) {
	post = -mode;
	sign = 1;
    } else {
	post = mode;
	sign = 0;
    }

    tail = frame4_data->iqueue.lqueue;

    if (tail + sign < post) {
    } else {
	struct c1text_state *c1text_data;

	if (sign) {
	    post = tail + 1 - post;
	}

	c1text_data = offset_q_4(frame4_data);

	c1text_data->o1text.delete = QUEUE_CLASS;

	((union deck_type *) c1text_data->o1text.deck)->v1.post = post;
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
slip_aile(void *context)
{
    return case_aime(context, 1);
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
slip_aime(void *context)
{
    int relink, status, (*tail) (void *);
    struct c1text_state *c1text_data;
    struct equeue_type *equeue_data;
    struct frame4_type *frame4_data;
    union deck_type deck;
    void *aime;

    frame4_data = Frame4(context);

    c1text_data = offset_q_4(frame4_data);

    equeue_data = frame4_data->dana.eclass;

# if __ENABLE_CHANNEL_SERVER__
    frame4_data->channel = c1text_data->o1text.channel;
    frame4_data->client_id = c1text_data->o1text.client_id;
# endif				/* __ENABLE_CHANNEL_SERVER__ */

    aime = c1text_data->o1text.aime;

    c1text_data->o1text.delete = 0;

    c1text_data->o1text.deck = &deck;

    tail = equeue_data->fine.tail;

    if (tail(aime)) {
	relink = 1;
    } else {
	int (*flat) (void **), (*slip) (void *);
	unsigned interrupt, run = 0;
	void *deck;

	interrupt = WINDOW(context).server.interrupt;
	if (1) {
	    unsigned intermiss;

	    intermiss = c1text_data->o1text.consolidated;
	    if (intermiss) {
		if (intermiss < interrupt) {
		    interrupt = intermiss;
		}
	    }
	}

	frame4_data->dana.eport = equeue_data->slip.aime.e8port;

	frame4_data->dana.equeue = equeue_data;

	frame4_data->dana.flat = 0;

	deck = equeue_data->deck.data;
	flat = equeue_data->deck.flat;

	slip = equeue_data->fine.slip;

	while (1) {
	    status = slip(aime);

	    if (frame4_data->dana.flat) {
		int excess;

		frame4_data->dana.flat = 0;

		excess = flat(&deck);
		if (excess) {
		    if (status) {
		    } else {
			status = excess;
		    }
		}
	    }

	    if (status) {
		if (equeue_data->bits & Mylene1f4_FAILED_LOGIC) {
		} else {
		    Mylene1f4_fail_aime(context, equeue_data);
		}

		relink = 1;

		break;
	    } else {
		if (tail(aime)) {
		    relink = 1;
		    break;
		} else {
		    int delete;

		    delete = c1text_data->o1text.delete;
		    if (delete) {
			if (delete == ABORT_CLASS || delete == QUEUE_CLASS) {
			    relink = 1;
			} else {
# if __ENABLE_FQUEUE__
			    frame4_data->link |= 2;
# endif				/* __ENABLE_FQUEUE__ */
			    relink = 0;
			}

			break;
		    } else {
			if (interrupt) {
			    if (run == interrupt) {
				if (run == WINDOW(context).server.interrupt) {
				    struct equeue_type *equeue_data;

				    equeue_data = frame4_data->dana.eclass;
				    Mylene1f4_ceil_aime
					(context, equeue_data->name);
				} else {
				    Mylene1f4_cell_aime(context);

				    if (1) {
					struct iqueue_type *iqueue_data;

					iqueue_data = &frame4_data->iqueue;
					if (iqueue_data->lqueue) {
					    Mylene1f4_fini_iqueue
						(context,
						 iqueue_data->head_equeue,
						 iqueue_data);
					}
				    }
				}

				relink = 1;

				break;
			    } else {
				run++;
			    }
			}
		    }
		}
	    }
	}

	equeue_data->slip.aime.e8port = frame4_data->dana.eport;

	if (1) {
	    unsigned intermiss;

	    intermiss = c1text_data->o1text.consolidated;
	    if (intermiss) {
		c1text_data->o1text.consolidated = intermiss - run;
	    }
	}
    }
    if (relink) {
	int delete;

	delete = c1text_data->o1text.delete;

# if __ENABLE_FQUEUE__
	frame4_data->link |= 8;
# endif				/* __ENABLE_FQUEUE__ */

	if (delete == QUEUE_CLASS) {
	    Mylene1f4_back_aime(context);
	    Mylene1f4_push_equeue(context, deck.v1.post + 1);
	    delete = 0;
	} else {
	    fini_aime(context);

	    c1text_data->o1text.aime = NULL;

	    if (delete == ABORT_CLASS) {
		delete = 0;
	    }
	}

	if (!frame4_data->iqueue.head_equeue) {
	    frame4_data->logic = NULL;

# if __ENABLE_FQUEUE__
	    if (!delete) {
		if (frame4_data->link & 4) {
		    frame4_data->link |= 16;
		}
	    } else {
		frame4_data->link |= 2;
	    }
# endif				/* __ENABLE_FQUEUE__ */

	    status = 0;
	} else {
	    if (!delete) {
# if __ENABLE_FQUEUE__
		if (frame4_data->link & 4) {
		    status = 0;

		    frame4_data->logic = link_aile;

		    frame4_data->link |= 16;
		} else {
# endif				/* __ENABLE_FQUEUE__ */
		    status = 0;

		    frame4_data->logic = LINK_AIME;
# if __ENABLE_FQUEUE__
		}
# endif				/* __ENABLE_FQUEUE__ */
	    } else {
# if __ENABLE_FQUEUE__
		frame4_data->link |= 2 | 16;
# endif				/* __ENABLE_FQUEUE__ */

		status = 0;

		frame4_data->logic = link_aile;
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static void
fini_aime(void *context)
{
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    frame4_data->flags &= ~Mylene1f4_AIME_LINKED;

    Mylene1f4_free_equeue(context, frame4_data->dana.eclass);
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static void
fini_text(void *context, struct c1text_state *c1text_data)
{
    void *data;

    if (c1text_data->o1text.linked) {
	Mylene1f4_miss_aime(context);
    }

    data = c1text_data->o1text.aime;
    if (data) {
	fini_aime(context);
    }
    data = c1text_data->o1text.functions;
    if (data) {
	x1f4_fini_mxdeck(&data);
    }

    free(c1text_data);
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_CHANNEL_EQUEUE__
int
Mylene1f4_away_aime(void *context)
{
    struct c1text_state *c1text_data;

    c1text_data = offset_q_4(Frame4(context));

    if (1) {
	c1text_data->o1text.client_id = 0;
    }

    return 0;
}
#endif				/* __ENABLE_CHANNEL_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_back_aime(void *context)
{
    int status;
    struct equeue_type *equeue_data;
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    equeue_data = frame4_data->dana.eclass;
    if (0) {
	status = -1;
    } else {
	status = 0;

	equeue_data->equeue_data = frame4_data->iqueue.head_equeue;
	frame4_data->iqueue.head_equeue = equeue_data;
	if (!frame4_data->iqueue.tail_equeue) {
	    frame4_data->iqueue.tail_equeue = equeue_data;
	}

	frame4_data->iqueue.lqueue++;

	if (1) {
	    struct c1text_state *c1text_data;

	    c1text_data = offset_q_4(Frame4(context));

# if __ENABLE_CHANNEL_SERVER__
	    equeue_data->channel = c1text_data->o1text.channel;
	    equeue_data->client = c1text_data->o1text.client_id;
# endif				/* __ENABLE_CHANNEL_SERVER__ */

	    c1text_data->o1text.aime = NULL;

	    frame4_data->flags &= ~Mylene1f4_AIME_LINKED;
	}
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_CHANNEL_EQUEUE__
int
Mylene1f4_case_aime(void *context, unsigned track)
{
    struct c1text_state *c1text_data;

    c1text_data = offset_q_4(Frame4(context));

    if (c1text_data->o1text.channel == track) {
	c1text_data->o1text.client_id = 0;
    }

    return 0;
}
#endif				/* __ENABLE_CHANNEL_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_clear_aime(void *context)
{
    struct c1text_state *c1text_data;

    c1text_data = Frame4(context)->c1text;
    if (c1text_data) {
	fini_text(context, offset_q(c1text_data));
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_link_aime(void *context)
{
    int status;

    do {
	{
	    struct c1text_state *c1text_data;

	    c1text_data = Frame4(context)->c1text;
	    if (c1text_data) {
	    } else {
		status = init_text(context);
		if (status) {
		    break;
		} else {
		    c1text_data = Frame4(context)->c1text;
		}
	    }
	    c1text_data = offset_q(c1text_data);
	    if (c1text_data->o1text.functions) {
	    } else {
		status = Mylene1f4_line_aime
		    (&c1text_data->o1text.functions, e4text(context)->indexset,
		     NON_nets, NON_sets);
		if (status) {
		    break;
		}
	    }
	}

	status = link_aile(context);
    } while (0);

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_lose_aime(void *context)
{
    struct c1text_state *c1text_data;

    c1text_data = Frame4(context)->c1text;
    if (c1text_data) {
	void *aime;

	c1text_data = offset_q(c1text_data);

	aime = c1text_data->o1text.aime;
	if (aime) {
	    fini_aime(context);

	    c1text_data->o1text.aime = NULL;
	}
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_setup_aime(void *context)
{
    Frame4(context)->c1text = NULL;

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */
