/*
 * aime0.c
 * Copyright (C) 2007-2015, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if __ENABLE_EQUEUE__
# include <aime.h>
# include <stddef.h>
# include <stdlib.h>
# include <string.h>
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# include <access.h>
# include <aime1.h>
# include <defs.h>
# include <excess4.h>
# include <excess7.h>
# include <inter.h>
# include <types.h>
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define equeue(equeue) \
    ((struct equeue_type *) (equeue))
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define e4text(context) \
    ((struct e4text_type *) Frame4((context))->lxdeck)
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define C1_BITS \
    X1f4_C1_COMPOSER | X1f4_C1_RESOURCE | X1f4_C1_SCOMMENT | X1f4_C1_SIDELIST \
    | X1f4_C1_TRANSFER
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define A1_BITS \
    X1f4_A1_COMPOSER | X1f4_A1_LEADCAST | X1f4_A1_RESOURCE | X1f4_A1_SCOMMENT \
    | X1f4_A1_SIDELIST | X1f4_A1_TRANSFER
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define Frame4(context) \
    ((struct frame4_type *) &Frame7((context))->_1.frame4)
# define Frame7(context) \
    ((struct frame7_type *) WINDOW((context)).server.window)
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
static int fini_aime(void *, void *);
static int fini_flat(void *, void *, void *);
static int fini_line(void *, void *);
static int init_aime(void *, void *, unsigned, void *);
static int init_flat(void *, void *, unsigned, void *);
static int init_line(void *, void *);
static int init_text(void *, void *, const void **);
static int land_aime(void *, void *, unsigned, void *);
static int land_flat(void *, void *, void *);
static int link_data(const void *, const struct x1f4_function_type *, void **);
static int link_text(void *, void *, struct x1f4_indexset_type *);
static int pick_aime(void *, void *, void *);
static int pick_flat(void *, void *, void *);
static int select_logic(const char *, unsigned, const void *,
			const struct x1f4_function_type **);
static int set_composer(struct x1f4_indexset_type *, void **,
			int (**) (void *, int, int,
				  const struct x1f4_linetext_type **));
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
static int
fini_aime(void *context, void *aime)
{
# if !__MISS_TEXT_FREE__
    x1f4_text_program(aime, free_text, NULL);
# else
    x1f4_flat_program(aime);
# endif				/* !__MISS_TEXT_FREE__ */
    x1f4_fini_program(&aime);

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
fini_flat(void *context, void *slip, void *aime)
{
    x1f4_side_shuffle(&slip, aime, context);
    x1f4_fini_shuffle(&aime);

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
fini_line(void *context, void *equeue)
{
    return x1f4_fini_e4ll(&equeue(equeue)->slip.aime.e4less);
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
init_aime(void *context, void *equeue, unsigned flags, void *c1)
{
    char *logique;
    int status;
    struct x1f4_c1_type *c1_data;
    struct x1f4_c1record_type c1record;
    void *program;

    logique = equeue(equeue)->rate;

    c1_data = c1;

    c1_data->bcollect_set.c1record_data = &c1record;

    init_text(context, equeue, &c1_data->variable_set.context);

    equeue(equeue)->deck.data = equeue(equeue)->slip.aime.e2list;
    equeue(equeue)->deck.flat = x1f4_deck_e2list;

    status = x1f4_init_program(&program, logique, flags, c1);
    if (status) {
	Mylene1f4_stat_aime
	    (context, equeue(equeue)->name, logique, status, 0, &c1record);
    } else {
	x1f4_head_program(program);

	equeue(equeue)->bits |= Mylene1f4_INTERN_LOGIC;

	equeue(equeue)->data.aime = program;
	equeue(equeue)->date.slip = NULL;

	equeue(equeue)->june.look = x1f4_look_program;
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
init_flat(void *context, void *equeue, unsigned flags, void *a1)
{
    char *logique;
    int status;
    struct x1f4_a1_type *a1_data;
    struct x1f4_a1record_type a1record;
    void *shuffle;

    logique = equeue(equeue)->rate;

    a1_data = a1;

    a1_data->bcollect_set.a1record_data = &a1record;

    x1f4_deck_e4ll
	(equeue(equeue)->slip.aime.e4less, &equeue(equeue)->deck.data,
	 &equeue(equeue)->deck.flat);

    status = x1f4_init_shuffle(&shuffle, logique, flags, a1);
    if (status) {
	Mylene1f4_stat_aime
	    (context, equeue(equeue)->name, logique, status, 1, &a1record);
    } else {
	status = x1f4_pipe_e4ll(equeue(equeue)->slip.aime.e4less, shuffle);
	if (status) {
	    x1f4_fini_shuffle(&shuffle);
	} else {
	    void *subtext;

	    x1f4_rail_shuffle(shuffle);

	    status = x1f4_near_shuffle(&subtext, shuffle);
	    if (status) {
		x1f4_fini_shuffle(&shuffle);
	    } else {
		x1f4_head_shuffle(subtext);

		equeue(equeue)->bits |= Mylene1f4_INTERN_LOGIC;

		equeue(equeue)->data.aime = subtext;
		equeue(equeue)->date.slip = shuffle;

		equeue(equeue)->june.look = x1f4_look_shuffle;
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
init_line(void *context, void *equeue)
{
    int status;
    struct frame4_type *frame4_data;
    struct x1f4_e4less_type e4line;
    void *e4less;

    frame4_data = Frame4(context);

    frame4_data->dana.equeue = equeue;

    e4line.misslink_set.data =
	((struct e4text_type *) frame4_data->lxdeck)->e2line;

    x1f4_type_e2list(&e4line.misslink_set);

    e4line.resource_set.data = Frame4(context);

    e4line.resource_set.free = Mylene1f4_free_moon;
    e4line.resource_set.link = Mylene1f4_link_moon;
    e4line.resource_set.mode = Mylene1f4_mode_moon;

    status = x1f4_init_e4ll
	(&e4less, X1f4_E4LL_MISSLINK | X1f4_E4LL_RESOURCE, &e4line);
    if (status) {
    } else {
	equeue(equeue)->slip.aime.e4less = e4less;

	x1f4_fast_e4ll(e4less, &equeue(equeue)->slip.aime.e2list);

	equeue(equeue)->slip.aime.e8port = &frame4_data->dana.plain;
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
init_text(void *context, void *equeue, const void **subtext)
{
    struct x1f4_indexset_type *indexset_data;
    void *data;

    data = equeue(equeue)->text.data;

    indexset_data = e4text(context)->indexset;

    if (data) {
    } else {
	do {
	    void *text;

	    text = indexset_data->variable_set.text;
	    if (text) {
		struct x1f4_mxdeck_type mxdeck;
		struct x1f4_trans_type trans;
		void *data;

		trans.free = Mylene1f4_free_moon;
		trans.link = Mylene1f4_link_moon;
		trans.mode = Mylene1f4_mode_moon;

		trans.data = Frame4(context);

		mxdeck.trans = &trans;

		if (x1f4_init_mxdeck(&data, X1f4_MXDECK_TRANS_MASK, &mxdeck)) {
		    break;
		} else {
		    if (x1f4_fill_mxdeck(data, text)) {
			x1f4_fini_mxdeck(&data);
			if (1) {
			    break;
			}
		    } else {
			text = data;
		    }
		}
	    }

	    link_text(&text, &equeue(equeue)->text.link, indexset_data);

	    *subtext = text;

	    equeue(equeue)->text.data = text;
	} while (0);
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
land_aime(void *context, void *equeue, unsigned flags, void *c1)
{
    int status;

    status = init_line(context, equeue);
    if (status) {
    } else {
	status = init_aime(context, equeue, flags, c1);
	if (status) {
	    fini_line(context, equeue);
	}
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
land_flat(void *context, void *equeue, void *sample)
{
    int status;

    status = init_line(context, equeue);
    if (status) {
    } else {
	status = pick_flat(context, equeue, sample);
	if (status) {
	    fini_line(context, equeue);
	}
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
link_data(const void *context, const struct x1f4_function_type *function_data,
	  void **link)
{
    if (function_data->flags & M1f4_L7_TEXT_LINK) {
	*link = ((const struct sample_type *) context)->function_set.text;
    } else {
	*link = (void *)
	    ((struct x1f4_linetext_type *)
	     ((const char *) function_data
	      - offsetof(struct x1f4_linetext_type, function)))->context;
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
link_text(void *text, void *link, struct x1f4_indexset_type *indexset_data)
{
    const char *name;
    struct x1f4_datatype_type *datatype_data;

    datatype_data = indexset_data->datatype_set.datatype_data;

    name = datatype_data->name;
    while (name) {
	if (datatype_data->size == 6) {
	    if (memcmp(name, "record", 6)) {
	    } else {
		break;
	    }
	}

	datatype_data++;
	name = datatype_data->name;
    }

    if (name) {
	int status, type;

	type = datatype_data->type;

	status = datatype_data->line(datatype_data->context, link);
	if (status) {
	} else {
	    status = x1f4_push_variable
		(text, "context", 7, type, X1f4_E4_READ_ONLY, link);
	    if (status) {
		datatype_data->flat(datatype_data->context, link);
	    }
	}
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
pick_aime(void *context, void *equeue, void *sample)
{
    int status;
    struct frame4_type *frame4_data;
    struct sample_type *sample_data;
    struct x1f4_c1_type c1;
    struct x1f4_indexset_type *indexset_data;
    struct x1f4_track_type track;
    unsigned flags = C1_BITS;

    sample_data = sample;

    if (E4LINE(context) & 1) {
	flags |= X1f4_C1_OPTIMIZE;
# if 0
	c1.context = NULL;
# endif				/* 0 */
    }
    if (1) {
	flags |= X1f4_C1_BCOLLECT;
    }

    if (1) {
	flags |= X1f4_C1_TYPELIST;
    }

    frame4_data = Frame4(context);

    track.line = NULL;
    track.post = NULL;
    track.push = NULL;

    frame4_data->dana.eport = &track;

    frame4_data->dana.equeue = equeue;

    indexset_data = ((struct e4text_type *) frame4_data->lxdeck)->indexset;

    set_composer
	(indexset_data, &c1.composer_set.context, &c1.composer_set.get);
    c1.datatype_set.miss = indexset_data->datatype_set.datatype_data;
    sample_data->function_set.text = context;
    c1.function_set.context = sample_data;
    c1.function_set.fix = link_data;
    c1.function_set.get = select_logic;
    c1.operator1s = sample_data->operator1s;
    c1.operator2s = sample_data->operator2s;
    c1.resource_set.data = frame4_data;
    c1.resource_set.free = Mylene1f4_free_moon;
    c1.resource_set.link = Mylene1f4_link_moon;
    c1.resource_set.mode = Mylene1f4_mode_moon;
    x1f4_pset_lxcast
	((void *) c1.composer_set.context, (void *) &c1.sidetype_set.miss);
    c1.transfer_set.fine = indexset_data->transfer_set.fine;
    c1.variable_set.context = equeue(equeue)->text.data;
    if (c1.variable_set.context) {
    } else {
	c1.variable_set.context = indexset_data->variable_set.text;
    }

    status = land_aime(context, equeue, flags, &c1);
    if (status) {
    } else {
	equeue(equeue)->fine.slip = x1f4_slip_program;
	equeue(equeue)->fine.tail = x1f4_tail_program;
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
pick_flat(void *context, void *equeue, void *sample)
{
    int status;
    struct frame4_type *frame4_data;
    struct sample_type *sample_data;
    struct x1f4_a1_type a1;
    struct x1f4_indexset_type *indexset_data;
    struct x1f4_track_type track;
    unsigned flags = A1_BITS;

    sample_data = sample;

    if (E4LINE(context) & 1) {
	flags |= X1f4_A1_OPTIMIZE;
# if 0
	a1.context = NULL;
# endif				/* 0 */
    }
    if (1) {
	flags |= X1f4_A1_BCOLLECT;
    }

    if (1) {
	flags |= X1f4_A1_TYPELIST;
    }

    frame4_data = Frame4(context);

    track.line = NULL;
    track.post = NULL;
    track.push = NULL;

    frame4_data->dana.eport = &track;

    frame4_data->dana.equeue = equeue;

    indexset_data = ((struct e4text_type *) frame4_data->lxdeck)->indexset;

    a1.autolead_set.context = equeue(equeue)->slip.aime.e4less;
    a1.autolead_set.link = x1f4_link_e4lf;
    a1.autolead_set.pick = x1f4_pick_e4lf;
    a1.autolink_set.data = indexset_data->autolink_set.data;
    a1.autolink_set.link = indexset_data->autolink_set.link;
    set_composer
	(indexset_data, &a1.composer_set.context, &a1.composer_set.get);
    a1.datatype_set.miss = indexset_data->datatype_set.datatype_data;
    sample_data->function_set.text = context;
    a1.function_set.context = sample_data;
    a1.function_set.fix = link_data;
    a1.function_set.get = select_logic;
    a1.operator1s = sample_data->operator1s;
    a1.operator2s = sample_data->operator2s;
    a1.resource_set.data = frame4_data;
    a1.resource_set.free = Mylene1f4_free_moon;
    a1.resource_set.link = Mylene1f4_link_moon;
    a1.resource_set.mode = Mylene1f4_mode_moon;
    x1f4_pset_lxcast
	((void *) a1.composer_set.context, (void *) &a1.sidetype_set.miss);
    a1.transfer_set.fine = indexset_data->transfer_set.fine;
    a1.variable_set.context = equeue(equeue)->text.data;
    if (a1.variable_set.context) {
    } else {
	a1.variable_set.context = indexset_data->variable_set.text;
    }

    init_text(context, equeue, &a1.variable_set.context);

    status = init_flat(context, equeue, flags, &a1);
    if (status) {
    } else {
	equeue(equeue)->fine.slip = x1f4_slip_shuffle;
	equeue(equeue)->fine.tail = x1f4_tail_shuffle;
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
select_logic(const char *f, unsigned length, const void *context,
	     const struct x1f4_function_type **function)
{
    int status;
    struct sample_type *sample_data;
    const void *mind;

    sample_data = (void *) context;
    status =
	x1f4_find_mxpath(sample_data->function_set.miss, f, length, &mind);
    if (status) {
	status = 0;
	if (1) {
	    *function = mind;
	}
    } else {
	status = 1;
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
set_composer(struct x1f4_indexset_type *indexset_data, void **text,
	     int (**lock) (void *, int, int,
			   const struct x1f4_linetext_type **))
{
    struct x1f4_lxslip_type *lxslip_data;

    lxslip_data = indexset_data->sliplong_set.lxslip_data;
    while (lxslip_data->note != x1f4_note_lxcast) {
	lxslip_data++;
    }

    *lock = x1f4_lock_lxcast;

    *text = lxslip_data->slip;

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_fini_aime(void *context, void *equeue)
{
    void *slip;

    slip = equeue(equeue)->date.slip;
    if (slip) {
	fini_flat(context, equeue(equeue)->data.aime, slip);
    } else {
	fini_aime(context, equeue(equeue)->data.aime);
    }

    fini_line(context, equeue);

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_pick_aime(void *context, void *equeue, void *sample)
{
    int status;

    if (x1f4_peer_shuffle(equeue(equeue)->rate, X1f4_A1_SCOMMENT)) {
	status = land_flat(context, equeue, sample);
    } else {
	status = pick_aime(context, equeue, sample);
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */
