/*
 * aime4.c
 * Copyright (C) 2007-2013, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if __ENABLE_EQUEUE__
# include <aime.h>
# include <stddef.h>
# include <stdlib.h>
# include <string.h>
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# include <access.h>
# include <excess4.h>
# include <excess7.h>
# include <inter.h>
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define e4text(context) \
    ((struct e4text_type *) Frame4((context))->lxdeck)
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define Frame4(context) \
    ((struct frame4_type *) &Frame7((context))->_1.frame4)
# define Frame7(context) \
    ((struct frame7_type *) WINDOW((context)).server.window)
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
extern const struct x1f4_operator_type *_Mylene1f4_aime_3_B[],
    *_Mylene1f4_filter_I[];
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
static int init_operators(void *, struct a1text_type *);
static int init_prefixing(void *, struct a1text_type *);
static int link_operators(void *, const struct x1f4_operator_type **,
			  unsigned);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
static int
init_operators(void *context, struct a1text_type *a1text_data)
{
    int status;
    struct x1f4_indexset_type *indexset_data;
    const struct x1f4_operator_type *const *operAtors, *const *operatOrs,
	**operators;
    unsigned count;

    indexset_data = e4text(context)->indexset;

    operAtors = indexset_data->operator_set.operator2s;

    operatOrs = operAtors;
    while (*operatOrs) {
	operatOrs++;
    }

    count = operatOrs - operAtors;

    operators = (const struct x1f4_operator_type **) malloc
	(sizeof(const struct x1f4_operator_type *) * (1 + count + 1));
    if (!operators) {
	status = -1;
    } else {
	status = 0;

	a1text_data->operators = operators;

	memcpy(operators, _Mylene1f4_aime_3_B,
	       sizeof(const struct x1f4_operator_type *) << 0);
	operators += 1;
	memcpy(operators, operAtors,
	       sizeof(const struct x1f4_operator_type *) * (count + 1));

	link_operators(context, _Mylene1f4_aime_3_B, 1);
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
init_prefixing(void *context, struct a1text_type *a1text_data)
{
    int status;
    const struct x1f4_operator_type *const *operAtors, *const *operatOrs,
	**operators;
    unsigned count;

    x1f4_llink_operator1s(&operAtors);

    operatOrs = operAtors;
    while (*operatOrs) {
	operatOrs++;
    }

    count = operatOrs - operAtors;
    operators = (const struct x1f4_operator_type **) malloc
	(sizeof(const struct x1f4_operator_type *) * (8 + count + 1));
    if (!operators) {
	status = -1;
    } else {
	status = 0;

	a1text_data->prefixing = operators;

	memcpy(operators, _Mylene1f4_filter_I,
	       sizeof(const struct x1f4_operator_type *) << 3);
	operators += 8;
	memcpy(operators, operAtors,
	       sizeof(const struct x1f4_operator_type *) * (count + 1));

	link_operators(context, _Mylene1f4_filter_I, 8);
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
link_operators(void *context, const struct x1f4_operator_type **operator,
	       unsigned count)
{
    for (; count; count--) {
	const struct x1f4_2ndscope_type *endscope_data;
	struct x1f4_linetext_type *linetext_data;

	endscope_data = (*operator)->extension2;
	linetext_data = (void *) endscope_data->line;

	linetext_data->context = context;

	operator++;
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_miss_aime(void *context)
{
    struct a1text_type *a1text_data;

    a1text_data = Frame4(context)->a1text;
    if (a1text_data->count != 1) {
	a1text_data->count--;
    } else {
	Mylene1f4_wind_aime(context);
	Frame4(context)->a1text = NULL;
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_push_aime(void *context)
{
    int status;
    struct a1text_type *a1text_data;
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    a1text_data = frame4_data->a1text;

    if (a1text_data) {
	a1text_data->count++;

	status = 0;
    } else {
	a1text_data = (struct a1text_type *)
	    malloc(sizeof(struct a1text_type));
	if (!a1text_data) {
	    status = -1;
	} else {
	    status = init_prefixing(context, a1text_data);
	    if (status) {
		free(a1text_data);
	    } else {
		status = init_operators(context, a1text_data);
		if (status) {
		    free(a1text_data->prefixing);
		    free(a1text_data);
		} else {
		    a1text_data->count = 1;

		    frame4_data->a1text = a1text_data;
		}
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_wind_aime(void *context)
{
    struct a1text_type *a1text_data;

    a1text_data = Frame4(context)->a1text;
    if (a1text_data) {
	free(a1text_data->operators);
	free(a1text_data->prefixing);

	free(a1text_data);
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_xset_aime(void *context)
{
    Frame4(context)->a1text = NULL;

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */
