/*
 * aimea.c
 * Copyright (C) 2007-2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if __ENABLE_EQUEUE__
# include <aime.h>
# if defined HAVE_LIBLEWD
#  include <lewd.h>
# endif				/* HAVE_LIBLEWD */
# if defined HAVE_LIBx1f4l0
#  include <libx1f4l0.h>
# endif				/* HAVE_LIBx1f4l0 */
# include <stddef.h>
# include <stdlib.h>
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# include <access.h>
# include <aime1.h>
# include <defs.h>
# include <excess4.h>
# include <excess7.h>
# include <inter.h>
# include <types.h>
# if !defined HAVE_LIBLEWD
#  include <vtkey.h>
# endif				/* !HAVE_LIBLEWD */
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define ABORT_CLASS			1
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define SLIP_AIME			slip_aime
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define Frame4(context) \
    ((struct frame4_type *) &Frame7((context))->_1.frame4)
# define Frame7(context) \
    ((struct frame7_type *) WINDOW((context)).server.window)
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define a1text(context) \
    ((struct a1text_type *) Frame4((context))->a1text)
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define i1text(context) \
    ((struct i1text_state *) Frame4((context))->i1text)
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define e4text(context) \
    ((struct e4text_type *) Frame4((context))->lxdeck)
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
typedef struct i1text_state {
    int delete, linked;
    void *functions;
} i1text_state;
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
extern const struct x1f4_function_type _Mylene1f4_aime_1_F__[],
    _Mylene1f4_aime_1_I_3[], _Mylene1f4_aime_1_I__[], _Mylene1f4_aime_1_L_3[],
    _Mylene1f4_aime_1_L__[], _Mylene1f4_aime_1___3[], _Mylene1f4_aime_1____[],
    _Mylene1f4_aime_1_q_l[], _Mylene1f4_aime_1_q_s[], _Mylene1f4_aime_3_L__[],
    _Mylene1f4_aime_3_R__[], _Mylene1f4_aime_3_S__[], _Mylene1f4_filter_F__[],
    _Mylene1f4_filter_N__[], _Mylene1f4_filter_U__[];
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
static int init_1fix(struct sample_type *, struct i1text_state *, void *);
static int init_2fix(struct sample_type *, struct i1text_state *, void *);
static int init_aime(void *, struct equeue_type *);
static int init_text(void *);
static int line_text(void *);
static int nonsense_081103_0(void *, void *, void **);
static int slip_aime(void *, struct equeue_type *);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
static void fini_text(void *, struct i1text_state *);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
const struct x1f4_function_type _Mylene1f4_aime_a_F__[] = {
/* *INDENT-OFF* */
    {	"F_abort",		nonsense_081103_0,
	X1f4_E4_VOID,		NULL,		0,
	X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK | M1f4_L7_TEXT_LINK,
						7			},
    {	NULL,			NULL,
	0,			NULL,		0,
	0,					1			}
/* *INDENT-ON* */
};
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
static const struct x1f4_function_type *NON_sets[] = {
/* *INDENT-OFF* */
    _Mylene1f4_aime_1_F__,
    _Mylene1f4_aime_1_I_3,
    _Mylene1f4_aime_1_I__,
    _Mylene1f4_aime_1_L_3,
    _Mylene1f4_aime_1_L__,
    _Mylene1f4_aime_1___3,
    _Mylene1f4_aime_1____,
    _Mylene1f4_aime_1_q_l,
    _Mylene1f4_aime_1_q_s,
    _Mylene1f4_aime_3_L__,
    _Mylene1f4_aime_3_R__,
    _Mylene1f4_aime_3_S__,
    _Mylene1f4_aime_a_F__,
    _Mylene1f4_filter_F__,
    _Mylene1f4_filter_N__,
    _Mylene1f4_filter_U__,
    NULL
/* *INDENT-ON* */
};
static const struct x1f4_linetext_type *NON_nets[] = {
/* *INDENT-OFF* */
    NULL
/* *INDENT-ON* */
};
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
static int
init_1fix(struct sample_type *sample_data, struct i1text_state *i1text_data,
	  void *context)
{
    int status;

    if (i1text_data->linked) {
	status = 0;
	sample_data->operator1s = a1text(context)->prefixing;
    } else {
	status = Mylene1f4_push_aime(context);
	if (status) {
	} else {
	    i1text_data->linked = 1;
	    sample_data->operator1s = a1text(context)->prefixing;
	}
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
init_2fix(struct sample_type *sample_data, struct i1text_state *i1text_data,
	  void *context)
{
    int status;

    if (i1text_data->linked) {
	status = 0;
	sample_data->operator2s = a1text(context)->operators;
    } else {
	status = Mylene1f4_push_aime(context);
	if (status) {
	} else {
	    i1text_data->linked = 1;
	    sample_data->operator2s = a1text(context)->operators;
	}
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
init_aime(void *context, struct equeue_type *equeue_data)
{
    int status;
    struct i1text_state *i1text_data;
    struct frame4_type *frame4_data;
    struct sample_type sample;

    frame4_data = Frame4(context);

    i1text_data = frame4_data->i1text;

    status = init_2fix(&sample, i1text_data, context);
    if (status) {
    } else {
	status = init_1fix(&sample, i1text_data, context);
	if (status) {
	} else {
	    sample.function_set.miss = i1text_data->functions;

	    status = Mylene1f4_pick_aime(context, equeue_data, &sample);
	}
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
init_text(void *context)
{
    int status;

    status = line_text(context);
    if (!status) {
	struct i1text_state *i1text_data;

	i1text_data = (struct i1text_state *)
	    malloc(sizeof(struct i1text_state));
	if (!i1text_data) {
	    status = -1;
	} else {
	    status = 0;

	    i1text_data->functions = NULL;

	    i1text_data->linked = 0;

	    Frame4(context)->i1text = i1text_data;
	}
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
line_text(void *context)
{
    int status;

    status = Mylene1f4_line_filter(context);
    if (status) {
    } else {
	status = Mylene1f4_line_e4lx(context);
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
nonsense_081103_0(void *context, void *output, void **input)
{
    i1text(context)->delete = ABORT_CLASS;

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
slip_aime(void *context, struct equeue_type *equeue_data)
{
    int false, (*tail) (void *);
    struct i1text_state *i1text_data;
    struct frame4_type *frame4_data;
    void *aime;

    frame4_data = Frame4(context);

# if __ENABLE_CHANNEL_SERVER__
    frame4_data->channel = equeue_data->channel;
    frame4_data->client_id = equeue_data->client;
# endif				/* __ENABLE_CHANNEL_SERVER__ */

    aime = equeue_data->data.aime;

    i1text_data = i1text(context);

    i1text_data->delete = 0;

    tail = equeue_data->fine.tail;

    if (tail(aime)) {
	false = 1;
    } else {
	int (*flat) (void **), (*slip) (void *);
	unsigned interrupt, run = 0;
	void *deck;

	interrupt = WINDOW(context).server.interrupt;

	frame4_data->dana.eport = equeue_data->slip.aime.e8port;

	frame4_data->dana.equeue = equeue_data;

	frame4_data->dana.flat = 0;

	deck = equeue_data->deck.data;
	flat = equeue_data->deck.flat;

	slip = equeue_data->fine.slip;

	while (1) {
	    int status;

	    status = slip(aime);

	    if (frame4_data->dana.flat) {
		int excess;

		frame4_data->dana.flat = 0;

		excess = flat(&deck);
		if (excess) {
		    if (status) {
		    } else {
			status = excess;
		    }
		}
	    }

	    if (status) {
		if (equeue_data->bits & Mylene1f4_FAILED_LOGIC) {
		} else {
		    Mylene1f4_fail_aime(context, equeue_data);
		}

		false = 1;

		break;
	    } else {
		if (tail(aime)) {
		    false = 1;
		    break;
		} else {
		    int delete;

		    delete = i1text(context)->delete;
		    if (delete) {
			if (delete == ABORT_CLASS) {
			    false = 1;
			} else {
			    false = 0;
			}

			break;
		    } else {
			if (interrupt) {
			    if (run == interrupt) {
				Mylene1f4_ceil_aime
				    (context, equeue_data->name);

				false = 1;

				break;
			    } else {
				run++;
			    }
			}
		    }
		}
	    }
	}

	equeue_data->slip.aime.e8port = frame4_data->dana.eport;
    }

    return false;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static void
fini_text(void *context, struct i1text_state *i1text_data)
{
    void *data;

    if (i1text_data->linked) {
	Mylene1f4_miss_aime(context);
    }

    data = i1text_data->functions;
    if (data) {
	x1f4_fini_mxdeck(&data);
    }

    free(i1text_data);
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_break_aime(void *context)
{
    struct i1text_state *i1text_data;

    i1text_data = Frame4(context)->i1text;
    if (i1text_data) {
	fini_text(context, i1text_data);
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_fast_aime(void *context, void *equeue)
{
    if (0) {
    } else {
	do {
	    int status;

	    {
		struct i1text_state *i1text_data;

		i1text_data = Frame4(context)->i1text;
		if (i1text_data) {
		} else {
		    status = init_text(context);
		    if (status) {
			break;
		    } else {
			i1text_data = Frame4(context)->i1text;
		    }
		}
		if (i1text_data->functions) {
		} else {
		    status = Mylene1f4_line_aime
			(&i1text_data->functions, e4text(context)->indexset,
			 NON_nets, NON_sets);
		    if (status) {
			break;
		    }
		}
	    }

	    status = init_aime(context, equeue);
	    if (status) {
	    } else {
		SLIP_AIME(context, equeue);
	    }
	} while (0);
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_track_aime(void *context)
{
    Frame4(context)->i1text = NULL;

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */
