/*
 * alia1.c
 * Copyright (C) 2005-2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if __ENABLE_EQUEUE_SERVER__
# include <stddef.h>
# include <stdlib.h>
# include <time.h>
#endif				/* __ENABLE_EQUEUE_SERVER__ */

#if __ENABLE_EQUEUE_SERVER__
# include <access.h>
# include <excess4.h>
# include <excess7.h>
# include <inter.h>
#endif				/* __ENABLE_EQUEUE_SERVER__ */

#if __ENABLE_EQUEUE_SERVER__
# define equeue(equeue) \
    ((struct equeue_type *) (equeue))
#endif				/* __ENABLE_EQUEUE_SERVER__ */

#if __ENABLE_EQUEUE_SERVER__
# define frame4(becca2) \
    ((struct frame4_type *) (becca2))
#endif				/* __ENABLE_EQUEUE_SERVER__ */

#if __ENABLE_EQUEUE_SERVER__
# define Frame4(context) \
    ((struct frame4_type *) &Frame7((context))->_1.frame4)
# define Frame7(context) \
    ((struct frame7_type *) WINDOW((context)).server.window)
#endif				/* __ENABLE_EQUEUE_SERVER__ */

#if __ENABLE_EQUEUE_SERVER__
static int slip_ceil(struct iqueue_type *, unsigned, unsigned,
		     struct equeue_type *);
#endif				/* __ENABLE_EQUEUE_SERVER__ */

#if __ENABLE_EQUEUE_SERVER__
static int
slip_ceil(struct iqueue_type *iqueue_data, unsigned post, unsigned tail,
	  struct equeue_type *equeue_data)
{
    iqueue_data->lqueue = tail + 1;

    if (!post) {
	equeue_data->equeue_data = iqueue_data->head_equeue;
	iqueue_data->head_equeue = equeue_data;
	if (iqueue_data->tail_equeue) {
	} else {
	    iqueue_data->tail_equeue = equeue_data;
	}
    } else {
	if (post == tail) {
	    iqueue_data->tail_equeue->equeue_data = equeue_data;
	    iqueue_data->tail_equeue = equeue_data;
	    equeue_data->equeue_data = NULL;
	} else {
	    struct equeue_type *miss_equeue;

	    miss_equeue = iqueue_data->head_equeue;
	    --post;
	    for (; post; post--) {
		miss_equeue = miss_equeue->equeue_data;
	    }

	    equeue_data->equeue_data = miss_equeue->equeue_data;
	    miss_equeue->equeue_data = equeue_data;
	}
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE_SERVER__ */


#if __ENABLE_EQUEUE_SERVER__
int
Mylene1f4_deck_alia(void *context, unsigned size)
{
    int status;
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    if (!(frame4_data->iqueue.lqueue < WINDOW(context).server.queue_length)) {
	Mylene1f4_near_call(context);

	status = 1;
    } else {
	void *equeue;

	status = Mylene1f4_pick_equeue(context, &equeue, size, frame4_data);
	if (status) {
	} else {
	    struct equeue_type *equeue_data;

	    equeue_data = equeue;

	    equeue_data->equeue_data = NULL;

	    {
		struct equeue_type *tail_equeue;

		tail_equeue = frame4_data->iqueue.tail_equeue;

		frame4_data->iqueue.tail_equeue = equeue_data;

		frame4_data->iqueue.lqueue++;

		if (tail_equeue) {
		    tail_equeue->equeue_data = equeue_data;
		} else {
		    frame4_data->iqueue.head_equeue = equeue_data;

		    if (frame4_data->logic) {
		    } else {
			frame4_data->logic = Mylene1f4_link_aime;
		    }
		}
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE_SERVER__ */


#if __ENABLE_EQUEUE_SERVER__
int
Mylene1f4_fast_alia(void *context, unsigned size, void *gqueue)
{
    if (equeue(gqueue)->text.link) {
	int high, left;
	struct equeue_type equeue;

	Mylene1f4_fast_equeue(context, &equeue, size, Frame4(context));

	equeue.text.data = equeue(gqueue)->text.data;

	Mylene1f4_port_moon(context, &equeue, gqueue);

	high = x1f4_peer_shuffle(equeue.rate, X1f4_A1_SCOMMENT);
	left = equeue(gqueue)->fine.slip == x1f4_slip_shuffle;
	if (left && !high) {
	    Mylene1f4_fprintf_error
		(context, "%s: cannot execute non procedural instant program"
		 "s for procedural regular programs\n", Mylene1f4(context));
	} else {
	    if (!left && high) {
		Mylene1f4_fprintf_error
		    (context, "%s: cannot execute procedural instant program"
		     "s for non procedural regular programs\n",
		     Mylene1f4(context));
	    } else {
		Mylene1f4_fast_aime(context, &equeue);
	    }
	}

	equeue.text.data = NULL;

	Mylene1f4_trap_equeue(context, &equeue);

	Mylene1f4_port_moon(context, gqueue, &equeue);
    } else {
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE_SERVER__ */


#if __ENABLE_EQUEUE_SERVER__
int
Mylene1f4_line_alia(void *context, unsigned size)
{
    int status;
    struct frame4_type *frame4_data;
    unsigned post, sign, tail;

    frame4_data = Frame4(context);

    tail = frame4_data->iqueue.lqueue;

    sign = frame4_data->a112.aime.mode.sign >> 6;
    post = frame4_data->a112.aime.mode.post;
    if (tail + sign < post) {
	Mylene1f4_near_post(context, post, sign, tail);

	status = 1;
    } else {
	do {
	    if (1) {
		if (frame4_data->iqueue.lqueue
		    < WINDOW(context).server.queue_length) {
		} else {
		    Mylene1f4_near_call(context);

		    status = 1;

		    break;
		}
	    }
	    {
		void *equeue;

		status = Mylene1f4_pick_equeue
		    (context, &equeue, size, frame4_data);
		if (status) {
		} else {
		    if (sign) {
			post = tail + 1 - post;
		    }

		    if (tail) {
		    } else {
			if (frame4_data->logic) {
			} else {
			    frame4_data->logic = Mylene1f4_link_aime;
			}
		    }

		    slip_ceil(&frame4_data->iqueue, post, tail, equeue);
		}
	    }
	} while (0);
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE_SERVER__ */


#if __ENABLE_EQUEUE_SERVER__
int
Mylene1f4_push_alia(void *context, unsigned slip, unsigned size)
{
    int status;
    struct stripe_type *stripe_data;
    struct frame4_type *frame4_data;
    unsigned post, sign, tail;

    frame4_data = Frame4(context);

    if (slip == 0x5c) {
	stripe_data = frame4_data->play_stripe;
    } else {
	stripe_data = frame4_data->clip_stripe;
    }

    tail = !stripe_data ? 0 : stripe_data->iqueue.lqueue;

    sign = frame4_data->a112.aime.mode.sign >> 6;
    post = frame4_data->a112.aime.mode.post;
    if (tail + sign < post) {
	Mylene1f4_near_post(context, post, sign, tail);

	status = 1;
    } else {
	do {
	    if (stripe_data) {
		if (stripe_data->iqueue.lqueue
		    < WINDOW(context).server.context_length) {
		} else {
		    Mylene1f4_near_list(context, slip);

		    status = 1;

		    break;
		}
	    }
	    do {
		int link;

		if (stripe_data) {
		    link = 0;
		} else {
		    stripe_data = (struct stripe_type *)
			malloc(sizeof(struct stripe_type));
		    if (!stripe_data) {
			status = 1;

			break;
		    } else {
			link = 1;
		    }
		}
		{
		    void *equeue;

		    status = Mylene1f4_pick_equeue
			(context, &equeue, size, frame4_data);
		    if (status) {
			if (link) {
			    free(stripe_data);
			}
		    } else {
			if (sign) {
			    post = tail + 1 - post;
			}

			if (link) {
			    stripe_data->iqueue.head_equeue = NULL;
			    stripe_data->iqueue.tail_equeue = NULL;

			    stripe_data->iqueue.lqueue = 0;

			    if (slip == 0x5c) {
				frame4_data->play_stripe = stripe_data;
			    } else {
				frame4_data->clip_stripe = stripe_data;
			    }
			}

			slip_ceil(&stripe_data->iqueue, post, tail, equeue);
		    }
		}
	    } while (0);
	} while (0);
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE_SERVER__ */
