/*
 * cell.c
 * Copyright (C) 2007-2011, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * BUGS
 *
 *.there's a huge one over here.  And not easy to fix.  During loading setup
 * (all the reading and writing from and to the cell external loader) all other
 * connections (Mylene0fa, server or lanes controller) are not being watched.
 * Fixing this conditions is anything but easy, only once fixed we head into
 * the real issue: Mylene1f4 is not ready to handle controller requests with no
 * file loaded.  So keeping the connections watched renders futile: we cannot
 * deal with them in this stage.  Nor is any Mylene0fa dialog expected now, so,
 * again, the futility of doing things right towers our lack of inspiration.
 */
/*
 * TODO
 *
 *.start time (lead_cell) option
 *.non blocking io (hopefully that'll make things faster)
 *.excess0 once
 *.excess2 once
 *.man description (..., only the internal loaders are used for skip output and
 * the files that would be loaded externally will account for failures,
 * recommend Mylene0fa buffering (-l))
 *
 *.back_cell and head_cell for non 1f42 targets
 */

#include <config.h>

#if __LINK_CELL_LOADER__

#include <errno.h>
#if defined HAVE_LIBLEWD
# include <lewd.h>
#endif				/* HAVE_LIBLEWD */
#if defined HAVE_LIBx1f4l1
# include <libx1f4l1.h>
#endif				/* HAVE_LIBx1f4l1 */
#if __ENABLE_x00696e6b__
# include <math.h>
#endif				/* __ENABLE_x00696e6b__ */
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/select.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <1f44.h>
#include <access.h>
#if !defined HAVE_LIBLEWD
# include <basename.h>
#endif				/* !HAVE_LIBLEWD */
#include <c213.h>
#if !defined HAVE_LIBLEWD
# include <cardinal.n-d.h>
# include <cardinal.p-d.h>
# include <cardinal.p-f.h>
# include <cardinal.p.h>
#endif				/* !HAVE_LIBLEWD */
#include <defs.h>
#include <inter.h>
#include <lcard.h>
#if !defined HAVE_LIBLEWD
# include <lcardinal.p-f.h>
# include <lcardinal.p.h>
#endif				/* !HAVE_LIBLEWD */
#include <types.h>
#if !defined HAVE_LIBx1f4l1
# include <xshell.h>
#endif				/* !HAVE_LIBx1f4l1 */

#define __DEBUG_EXTERN__		0

#define OFFSET_SIZE			(2 + (8 << 1) + 1)

#define ATRACK_SIZE			(2 + 1)
#define MCACHE_SIZE			21
#define MINDEX_SIZE			21
#define MSHMID_SIZE			21
#define STREAM_SIZE			OFFSET_SIZE
#define SYSTEM_SIZE			OFFSET_SIZE

#define LINE_SIZE \
    (ATRACK_SIZE + MCACHE_SIZE + MINDEX_SIZE + MSHMID_SIZE + STREAM_SIZE      \
     + SYSTEM_SIZE)

#define true(e)				1

#define CARDINAL(buffer, base) \
    ((buffer)[(base) + 0] << 030 | (buffer)[(base) + 1] << 020		      \
     | (buffer)[(base) + 2] << 010 | (buffer)[(base) + 3] << 000)

#define Source(source) \
    ((struct source_type *) (source))->_

#define Inter6(inter6) \
    ((struct inter6_type *) (inter6))

#define Audio6(audio6) \
    ((struct audio6_type *) (audio6))

#define ea52al(context) \
    ((struct ea52al_type *) EA52AL((context)))

#define ea52al_second_a(context)	2
#define ea52al_second_f(context)	0

#define SECOND_a(v)			((v).tv_sec)
#define SECOND_f(v)			((v).tv_usec)

#if __DEBUG_EXTERN__
# define cell_stat_0(fax_0) \
    if (1) {								      \
	head_debug_f();							      \
	fprintf(stderr, (fax_0));					      \
    }
#endif				/* __DEBUG_EXTERN__ */

#if __DEBUG_EXTERN__
# define cell_stat_1(fax_0, fax_1) \
    if (1) {								      \
	head_debug_f();							      \
	fprintf(stderr, (fax_0), (fax_1));				      \
    }
#endif				/* __DEBUG_EXTERN__ */

#if __DEBUG_EXTERN__
# define cell_stat_2(fax_0, fax_1, fax_2) \
    if (1) {								      \
	head_debug_f();							      \
	fprintf(stderr, (fax_0), (fax_1), (fax_2));			      \
    }
#endif				/* __DEBUG_EXTERN__ */

#if __DEBUG_EXTERN__
# define cell_stat_3(fax_0, fax_1, fax_2, fax_3) \
    if (1) {								      \
	head_debug_f();							      \
	fprintf(stderr, (fax_0), (fax_1), (fax_2), (fax_3));		      \
    }
#endif				/* __DEBUG_EXTERN__ */

#if __DEBUG_EXTERN__
# define cell_stat_9(fax_0, fax_1, fax_2, fax_3, fax_4, fax_5, fax_6, fax_7, \
		     fax_8, fax_9)					      \
    if (1) {								      \
	head_debug_f();							      \
	fprintf(stderr, (fax_0), (fax_1), (fax_2), (fax_3), (fax_4), (fax_5), \
		(fax_6), (fax_7), (fax_8), (fax_9));			      \
    }
#endif				/* __DEBUG_EXTERN__ */

#if !__DEBUG_EXTERN__
#elif !__GNUC__
# define head_debug_f() \
    if (1) {								      \
	fprintf(stderr, "%s(%s): ", "cell", __FILE__);			      \
    }
#else
# define head_debug_f() \
    if (1) {								      \
	fprintf(stderr, "%s(%s): ", __FUNCTION__, __FILE__);		      \
    }
#endif				/* !__DEBUG_EXTERN__ */

#if __ENABLE_x00696e6b__
# define SET_CARDINAL(buffer, base, card) \
    {									      \
	(buffer)[(base) + 0] = (card) >> 030;				      \
	(buffer)[(base) + 1] = (card) >> 020;				      \
	(buffer)[(base) + 2] = (card) >> 010;				      \
	(buffer)[(base) + 3] = (card) >> 000;				      \
    }
#endif				/* __ENABLE_x00696e6b__ */

#if __ENABLE_x00696e6b__
# define SET_CARDiNAL(buffer, base, card) \
    {									      \
	unsigned cardinal;						      \
									      \
	cardinal = card;						      \
	SET_CARDINAL(buffer, base, cardinal);				      \
    }
#endif				/* __ENABLE_x00696e6b__ */

#if 0
#elif LCARD_DEFINITION == LONG_UNSIGNED_DEFINITION
# define print_offset(type, offset) \
    {									      \
	x1f4_xprint_lcardinal(&(type), 0, 16, (offset));		      \
	(type)--;							      \
	*(type) = 'x';							      \
	(type)--;							      \
	*(type) = '0';							      \
    }
#elif LCARD_DEFINITION == LONG_LONG_UNSIGNED_DEFINITION
# define print_offset(type, offset) \
    {									      \
	unsigned long long ___type_13_offset;				      \
									      \
	___type_13_offset = (offset);					      \
	x1f4_xprint_flcardinal						      \
	    (&(type), 0, SIZEOF_UNSIGNED_LONG << 1, 16, ___type_13_offset);   \
	___type_13_offset = ___type_13_offset >> (SIZEOF_UNSIGNED_LONG << 3); \
	x1f4_xprint_lcardinal(&(type), 0, 16, ___type_13_offset);	      \
	(type)--;							      \
	*(type) = 'x';							      \
	(type)--;							      \
	*(type) = '0';							      \
    }
#elif LCARD_DEFINITION == STRUCT_DEFINITION
# define print_offset(type, offset) \
    {									      \
	x1f4_xprint_fcardinal						      \
	    (&(type), 0, SIZEOF_UNSIGNED << 1, 16, (offset).minor);	      \
	x1f4_xprint_cardinal(&(type), 0, 16, (offset).major);		      \
	(type)--;							      \
	*(type) = 'x';							      \
	(type)--;							      \
	*(type) = '0';							      \
    }
#endif				/* 0 */

typedef struct audio6_type {
    int fd_0, fd_1, link, tail;
    pid_t wait;
    void *context, *inter6;
} audio6_type;

typedef struct inter6_type {
    struct audio6_type *audio6_data;
    void *context, *link, *q1f42a, *source, *xnone1;
} inter6_type;

static int back_cell(void *);
static int fini_1f44(void *, struct inter6_type *);
static int fini_66c0(void *, struct inter6_type *);
static int fini_66c1(void *, struct inter6_type *);
static int fini_a52a(void *, struct inter6_type *);
static int fini_cell(void *, unsigned, void **);
static int fini_none(void *, struct inter6_type *);
static int head_cell(void *);
static int head_hell(void *);
static int kill_cell(void *, struct inter6_type *);
static int init_1f44(void *, const char *, struct inter6_type *);
static int init_66c0(void *, void *, struct inter6_type *);
static int init_66c1(void *, void *, struct inter6_type *);
static int init_a52a(void *, void *, struct inter6_type *);
static int init_cell(void *, unsigned, void **);
static int init_none(void *, void *, struct inter6_type *);
static int lead_cell(void *, struct audio6_type *, void *, unsigned);
#if __ENABLE_x00696e6b__
static int lock_cell(void *, unsigned char *, unsigned);
#endif				/* __ENABLE_x00696e6b__ */
static int lose_path(void *);
static int lose_self(void *);
static int name_cell(void *, const char **);
static int pull_cell(void *, struct audio6_type *);
static int push_cell(void *, struct audio6_type *, unsigned);
static int slip_cell(void *, struct audio6_type *, void *, unsigned);
static int stat_cell(void *, const char *, unsigned);
static int stat_lead(void *);
static int stat_pull(void *, int);
static int stat_push(void *, int);
static int stat_slip(void *);
static int tail_hell(void *);
static int type_cell(void *, void *);
static int type_hell(void *, const char *, unsigned);
static int wait_cell(void *, int);
static int zero_cell(void *, unsigned);

static const struct target_type data_target = {
/* *INDENT-OFF* */
    back_cell,
    NULL,
    NULL,
#if __ENABLE_FAST_INTER__
    NULL,
#endif				/* __ENABLE_FAST_INTER__ */
    fini_cell,
    NULL,
    head_cell,
    init_cell,
#if __ENABLE_x00696e6b__
    NULL,
#endif				/* __ENABLE_x00696e6b__ */
    NULL,
#if __ENABLE_x00696e6b__
    lock_cell,
#endif				/* __ENABLE_x00696e6b__ */
#if __EXPAND_x1f42MAPS__
    NULL,
#endif				/* __ENABLE_x00696e6b__ */
    NULL,
    NULL,
    type_cell,
#if 0
    NULL
#else
    NULL,
    zero_cell
#endif				/* 0 */
/* *INDENT-ON* */
};

static int
back_cell(void *e1f4_audio)
{
    return -1;
}


static int
fini_1f44(void *context, struct inter6_type *inter6_data)
{
    int excess, fd_x, status = 0;
    struct audio6_type *audio6_data;
    struct violet_type *violet_data;

    audio6_data = inter6_data->audio6_data;

    violet_data = (struct violet_type *) (audio6_data + 1);
    violet_data = (struct violet_type *) ALIGN_ABUFFER(violet_data);

    fd_x = audio6_data->fd_0;
    if (fd_x != -1) {
	status = close(fd_x);
    }
    fd_x = audio6_data->fd_1;
    if (fd_x != -1) {
	excess = close(fd_x);
	if (excess) {
	    status = excess;
	}
    }

    Mylene1f4_deque_violet(context, violet_data);

    if (1) {
	excess = kill_cell(context, inter6_data);
	if (excess) {
	    status = excess;
	}
    }

    return status;
}


static int
fini_66c0(void *context, struct inter6_type *inter6_data)
{
    return fini_1f44(context, inter6_data);
}


static int
fini_66c1(void *context, struct inter6_type *inter6_data)
{
    unlink(inter6_data->link);

    Mylene1f4_get_record(context, lose_path);

    return fini_1f44(context, inter6_data);
}


static int
fini_a52a(void *context, struct inter6_type *inter6_data)
{
    int status;

    if (!inter6_data->link) {
	status = fini_66c0(context, inter6_data);
    } else {
	status = fini_66c1(context, inter6_data);
    }

    return status;
}


static int
fini_cell(void *loader, unsigned index, void **storage)
{
    return 0;
}


static int
fini_none(void *context, struct inter6_type *inter6_data)
{
    int status;
    struct audio6_type *audio6_data;

    audio6_data = inter6_data->audio6_data;

    Mylene1f4_remove_sequel(context, Mylene1f4_XA52A1_SEQUEL);

    status = fini_a52a(context, inter6_data);

    free(audio6_data);

    return status;
}


static int
head_cell(void *e1f4_audio)
{
    return -1;
}


static int
head_hell(void *context)
{
    Mylene1f4_fprintf_error(context, "%s: ", Mylene1f4(context));

    return 0;
}


static int
init_1f44(void *context, const char *path, struct inter6_type *inter6_data)
{
    int status;
    void *xshell;

    status = x1f4_init_xshell(&xshell);
    if (!status) {
	const char *device, *loader;

	loader = ea52al(context)->loader;
	if (loader) {
	    if (!*loader) {
		loader = NULL;
	    }
	}

	if (loader) {
	    device = loader;
	} else {
	    device = __install_prefix "/lib/Mylene1f4/cell.a52a";
	}

	status = access(device, R_OK | X_OK);
	if (status) {
	    Mylene1f4_fprintf_error
		(context, "%s: cannot execute `%s': %s\n", Mylene1f4(context),
		 device, strerror(errno));
	} else {
	    char line[LINE_SIZE], *type;
	    const char *argv[9];
	    struct x1f42a_type *x1f42a_data;
	    struct x1f4_xshell_type Xshell;

	    x1f42a_data = X1f42(context);

	    *argv = !loader ? "cell.a52a" : x1f4_basename(loader);
	    type = line + LINE_SIZE;
	    type--;
	    *type = 0;
	    x1f4_xprint_dcardinal(&type, CACHE(context) / (9 << 12));
	    argv[1] = type;
	    type--;
	    *type = 0;
	    x1f4_xprint_dcardinal(&type, SHMID(context));
	    argv[2] = type;
	    type--;
	    *type = 0;
	    x1f4_xprint_dcardinal(&type, C1f4(context));
	    argv[3] = type;
	    type--;
	    *type = 0;
	    print_offset(type, x1f42a_data->system);
	    argv[4] = type;
	    type--;
	    *type = 0;
	    print_offset(type, x1f42a_data->stream);
	    argv[5] = type;
	    type--;
	    *type = 0;
	    x1f4_xprint_cardinal(&type, 0, 16, x1f42a_data->audio_id);
	    argv[6] = type;
	    argv[7] = path;
	    argv[8] = NULL;

	    Xshell.cleanup.back = inter6_data;
	    Xshell.cleanup.call = lose_self;
	    Xshell.command.argv = (const char *const *) argv;
	    Xshell.command.path = device;
	    Xshell.printer.head = head_hell;
	    Xshell.printer.look = context;
	    Xshell.printer.tail = tail_hell;
	    Xshell.printer.type = type_hell;
	    if (ea52al(context)->flags & Mylene1f4_EA52AL_QUIET) {
		Xshell.trailer.c3rd = X1f4_XSHELL_KILL_C3RD;
	    } else {
		Xshell.trailer.c3rd = X1f4_XSHELL_MISS_C3RD;
	    }

	    status = x1f4_mode_xshell
		(xshell, &Xshell,
		 X1f4_XSHELL_ARGV_MASK | X1f4_XSHELL_BACK_MASK
		 | X1f4_XSHELL_C3RD_MASK | X1f4_XSHELL_CALL_MASK
		 | X1f4_XSHELL_HEAD_MASK | X1f4_XSHELL_LOOK_MASK
		 | X1f4_XSHELL_PATH_MASK | X1f4_XSHELL_TAIL_MASK
		 | X1f4_XSHELL_TYPE_MASK);
	    if (!status) {
#if __DEBUG_EXTERN__
		cell_stat_0("running external loader as:\n");
		cell_stat_9("%s %s %s %s %s %s %s %s %s\n", device, argv[0],
			    argv[1], argv[2], argv[3], argv[4], argv[5],
			    argv[6], argv[7]);
#endif				/* __DEBUG_EXTERN__ */

		status = x1f4_deck_xshell(xshell);
		if (!status) {
		    pid_t wait;
		    struct audio6_type *audio6_data;
		    struct violet_type *violet_data;

		    x1f4_hook_xshell(xshell, &wait);

		    audio6_data = inter6_data->audio6_data;

		    audio6_data->wait = wait;

		    x1f4_pipe_xshell
			(xshell, &audio6_data->fd_0, &audio6_data->fd_1);

		    violet_data = (struct violet_type *) (audio6_data + 1);
		    violet_data = (struct violet_type *)
			ALIGN_ABUFFER(violet_data);

		    violet_data->clip = wait_cell;
		    violet_data->data = audio6_data;
		    violet_data->wait = wait;

		    Mylene1f4_queue_violet(context, violet_data);
		}
	    }
	}

	x1f4_fini_xshell(&xshell);
    }

    return status;
}


static int
init_66c0(void *context, void *source, struct inter6_type *inter6_data)
{
    inter6_data->link = NULL;

    return init_1f44
	(context, ((struct x1f42a_type *) X1f42(context))->source.path,
	 inter6_data);
}


static int
init_66c1(void *context, void *source, struct inter6_type *inter6_data)
{
    char *v66c;
    const char *name;
    int status;
    pid_t here;
    unsigned head = 0, lead;

    here = getpid();

    name = Mylene1f4(context);

    x1f4_nprint_dcardinal(&head, (unsigned) here);

    lead = strlen(name);

    v66c = (char *) malloc(6 + lead + 1 + head + 8);
    if (!v66c) {
	status = -1;
    } else {
	char *text;
	int fd;

	text = v66c;
	memcpy(text, "/tmp/.", 6);
	text += 6;
	memcpy(text, name, lead);
	text += lead;
	*text++ = '-';
	text += head;
	x1f4_xprint_dcardinal(&text, (unsigned) here);
	text += head;
	memcpy(text, "-XXXXXX", 8);

	inter6_data->link = NULL;

	Mylene1f4_set_record(context, lose_path, inter6_data);

	fd = mkstemp(v66c);
	if (fd == -1) {
	    Mylene1f4_get_record(context, lose_path);

	    free(v66c);

	    status = -1;
	} else {
	    status = -1;

	    do {
		struct seek_1f44_type *e66c;
		unsigned count, i = 0;

		e66c = Source(source).s2.e66c;

		seek_1f44_tail(e66c, &count);

		{
		    char x_header[] = "1f44\0\0\0\0\0\0\0\0\0\0\0\0";

		    x_header[0xc] = count >> 010;
		    x_header[0xd] = count >> 000;

		    if (write(fd, x_header, 16) != 16) {
			break;
		    }
		}
		for (; i < count; i++) {
		    char s_header[0x10];
		    const char *path;
		    unsigned miss;

		    seek_1f44_pass(e66c, i, &path);

		    seek_1f44_head(e66c, i, &miss);
		    s_header[0x0] = miss >> 030;
		    s_header[0x1] = miss >> 020;
		    s_header[0x2] = miss >> 010;
		    s_header[0x3] = miss >> 000;
		    seek_1f44_lead(e66c, i, &miss);
		    s_header[0x4] = miss >> 030;
		    s_header[0x5] = miss >> 020;
		    s_header[0x6] = miss >> 010;
		    s_header[0x7] = miss >> 000;
		    s_header[0x8] = 0;
		    s_header[0x9] = 0;
		    s_header[0xa] = 0;
		    s_header[0xb] = 0;
		    s_header[0xc] = 0;
		    s_header[0xd] = 0;
		    miss = strlen(path);
		    s_header[0xe] = miss >> 010;
		    s_header[0xf] = miss >> 000;

		    if (write(fd, s_header, 16) != 16) {
			break;
		    }

		    if (write(fd, path, miss) != miss) {
			break;
		    }
		    if (miss & 15) {
			memset(s_header, 255, 16);
			miss = 16 - (miss & 15);
			if (write(fd, s_header, miss) != miss) {
			    break;
			}
		    }
		}
		if (i == count) {
		    status = close(fd);
		    fd = -1;
		    if (!status) {
			status = init_1f44(context, v66c, inter6_data);
			if (!status) {
			    inter6_data->link = v66c;
			}
		    }
		}
	    } while (0);

	    if (status) {
		free(v66c);
		if (fd != -1) {
		    close(fd);
		}

		Mylene1f4_get_record(context, lose_path);
	    }
	}
    }

    return status;
}


static int
init_a52a(void *context, void *source, struct inter6_type *inter6_data)
{
    int status;

    inter6_data->source = source;

    if (Source(source).s2.e66c->type == SEEK_1f44_x1f440_DATA) {
	status = init_66c0(context, source, inter6_data);
    } else {
	status = init_66c1(context, source, inter6_data);
    }

    return status;
}


static int
init_cell(void *loader, unsigned index, void **storage)
{
    int status;
    struct audio6_type *audio6_data;
    struct inter6_type *inter6_data;
    void *context;

    inter6_data = loader;

    audio6_data = inter6_data->audio6_data;

    context = inter6_data->context;

    status = pull_cell(context, audio6_data);
    if (!status) {
	HEAD(context) = 0;
	TAIL(context) = ~0;

	*storage = audio6_data;
    }

    return status;
}


static int
init_none(void *context, void *source, struct inter6_type *inter6_data)
{
    struct audio6_type *audio6_data;
    int status;

    if (!(audio6_data = (struct audio6_type *)
	  malloc(sizeof(struct audio6_type)
		 + ALIGN_AMALLOC(sizeof(sequel_type))
		 + ALIGN_AMALLOC(sizeof(violet_type))
		 + ALIGN_AMALLOC(sizeof(xa52a1_type))))) {
	Mylene1f4_fprintf_error
	    (context, "%s: %s\n", Mylene1f4(context), strerror(errno));

	status = -1;
    } else {
	struct sequel_type *sequel_data;
	struct violet_type *violet_data;
	struct xa52a1_type *xa52a1_data;
	struct x1f440_type *x1f440_data;

	violet_data = (struct violet_type *) (audio6_data + 1);
	violet_data = (struct violet_type *) ALIGN_ABUFFER(violet_data);

	sequel_data = (struct sequel_type *) (violet_data + 1);
	sequel_data = (struct sequel_type *) ALIGN_ABUFFER(sequel_data);

	xa52a1_data = (struct xa52a1_type *) (sequel_data + 1);
	xa52a1_data = (struct xa52a1_type *) ALIGN_ABUFFER(xa52a1_data);

	x1f440_data = (struct x1f440_type *) (xa52a1_data + 1);
	x1f440_data = (struct x1f440_type *) ALIGN_ZBUFFER(x1f440_data);

	{
	    struct x1f42a_type *x1f42a_data;

	    x1f42a_data = X1f42(context);
	    if (0) {
	    } else {
		xa52a1_data->audio_id = x1f42a_data->audio_id;
	    }
	}

	if (1) {
	    {
		inter6_data->audio6_data = audio6_data;
	    }
	}

	audio6_data->context = context;
	audio6_data->inter6 = inter6_data;
	audio6_data->link = 1;
	audio6_data->tail = 1;

	status = init_a52a(context, source, inter6_data);
	if (status) {
	    free(audio6_data);
	} else {
	    {
		inter6_data->xnone1 = xa52a1_data;
	    }

	    sequel_data->sequel_data = SEQUEL(context);
	    SEQUEL(context) = sequel_data;
	    sequel_data->code = Mylene1f4_XA52A1_SEQUEL;
	    sequel_data->data = xa52a1_data;
	}
    }

    return status;
}


static int
kill_cell(void *context, struct inter6_type *inter6_data)
{
    pid_t wait;
    struct audio6_type *audio6_data;

    audio6_data = inter6_data->audio6_data;

    wait = audio6_data->wait;
    if (wait != -1) {
	audio6_data->wait = -1;

#if __DEBUG_EXTERN__
	cell_stat_1("SIGTERM killing %d external loader...\n", (int) wait);
#endif				/* __DEBUG_EXTERN__ */

	kill(wait, SIGTERM);

	if (Mylene1f4_wait_second(context, &wait)) {
	} else {
	    if (wait != -1) {
#if __DEBUG_EXTERN__
		cell_stat_1("SIGKILL killing %d external loader...\n",
			    (int) wait);
#endif				/* __DEBUG_EXTERN__ */

		kill(wait, SIGKILL);
	    }
	}
    }

    return 0;
}


static int
lead_cell(void *context, struct audio6_type *audio6_data, void *data,
	  unsigned size)
{
    char *lead;
    int fd_0, status;
    unsigned te1st, te6th;

    te1st = ea52al_second_a(context);
    te6th = ea52al_second_f(context);
    if (1) {
	if (!(te6th < 1000000)) {
	    te1st += te6th / 1000000;
	    te6th %= 1000000;
	}
    }
    if (1) {
	struct timeval tv;

	gettimeofday(&tv, NULL);
	te1st += SECOND_a(tv);
	te6th += SECOND_f(tv);
	if (!(te6th < 1000000)) {
	    te1st++;
	    te6th -= 1000000;
	}
    }

    fd_0 = audio6_data->fd_0;

    lead = data;

    while (1) {
	fd_set fd_set_0, *fd_set_0_select;
	struct timeval *selector, timeout;

	if (SIGCHLD_received(context)) {
	    status = Mylene1f4_yway_mylene0fa(context, 0);
	    if (status) {
		break;
	    }
	}

	if (0) {
	} else {
	    fd_set_0_select = &fd_set_0;
	    FD_ZERO(fd_set_0_select);
	    FD_SET(fd_0, fd_set_0_select);
	}

	if (0) {
	} else {
	    unsigned second_a;

	    selector = &timeout;
	    gettimeofday(selector, NULL);
	    second_a = SECOND_a(timeout);
	    if (te1st < second_a) {
		stat_lead(context);
		status = -1;
		break;
	    } else {
		unsigned second_f, to6th;

		second_a = te1st - second_a;
		second_f = SECOND_f(timeout);
		to6th = te6th;
		if (!second_a) {
		    if (to6th < second_f) {
			stat_lead(context);
			status = -1;
			break;
		    }
		} else {
		    if (to6th < second_f) {
			second_a++;
			to6th += 1000000;
		    }
		}

		SECOND_a(timeout) = second_a;
		SECOND_f(timeout) = to6th - second_f;
	    }
	}

	status = select(fd_0 + 1, fd_set_0_select, NULL, NULL, selector);
	if (status == -1) {
	    if (errno != EINTR) {
		break;
	    }
	} else {
	    if (true(fd_set_0_select)) {
		if (FD_ISSET(fd_0, fd_set_0_select)) {
		    char c;

		    status = read(fd_0, &c, 1);
		    if (!status) {
			stat_pull(context, 0);
			status = -1;
			break;
		    } else {
			if (status == -1) {
			    stat_pull(context, 1);
			    break;
			} else {
			    *lead = c;
			    size--;
			    if (size) {
				lead++;
			    } else {
#if __DEBUG_EXTERN__
				struct timeval tv;
				unsigned to1st, to6th;
#endif				/* __DEBUG_EXTERN__ */

#if __DEBUG_EXTERN__
				gettimeofday(&tv, NULL);
				if (te6th < ea52al_second_f(context)) {
				    te6th += 1000000;
				    te1st--;
				}
				te1st -= ea52al_second_a(context);
				te6th -= ea52al_second_f(context);
				to1st = SECOND_a(tv);
				to6th = SECOND_f(tv);
				if (to6th < te6th) {
				    to6th += 1000000;
				    to1st--;
				}
				to1st -= te1st;
				to6th -= te6th;
				cell_stat_2("external loader started phase 1"
					    " in %u, %u\n", to1st, to6th);
#endif				/* __DEBUG_EXTERN__ */
				status = 0;
				break;
			    }
			}
		    }
		}
	    }
	}
    }

    return status;
}


#if __ENABLE_x00696e6b__
static int
lock_cell(void *context, unsigned char *buffer, unsigned reference)
{
    double fractional, integral;
    struct sequel_type *sequel_data;
    struct xa52a1_type *xa52a1_data;

    Mylene1f4_find_sequel
	(context, Mylene1f4_XA52A1_SEQUEL, (void *) &sequel_data);
    xa52a1_data = sequel_data->data;
    fractional = modf
	((double) reference * xa52a1_data->frame_rate_d
	 / xa52a1_data->frame_rate_n, &integral);
    SET_CARDiNAL(buffer, 0, (unsigned) integral);
    SET_CARDiNAL(buffer, 4, (unsigned) (fractional * 1000000));

    return 0;
}
#endif				/* __ENABLE_x00696e6b__ */


static int
lose_path(void *inter6)
{
    return unlink(Inter6(inter6)->link);
}


static int
lose_self(void *inter6)
{
    int status = -1;

    do {
	void *context;

	context = ((struct inter6_type *) inter6)->context;

	if (CLASS(context) == Mylene1f4_LINK_OUTPUT) {
	    if (close(Mylene1ST(context))) {
		break;
	    }
	    if (close(Mylene2ND(context))) {
		break;
	    }
	}

	if (1) {
	    void *source;

	    if (Mylene1f4_ceil_1f42(Inter6(inter6)->q1f42a, &source)) {
		break;
	    }
	    if (Mylene1f4_close_file(context, source)) {
		break;
	    }
	}
	{
	    void *source;

	    source = Inter6(inter6)->source;
	    if (Source(source).s2.e66c->type == SEEK_1f44_x1f440_DATA) {
		struct source_type source;

		Mylene1f4_66c0_1f42(Inter6(inter6)->q1f42a, &source._.s1.fd);
		if (Mylene1f4_close_file(context, &source)) {
		    break;
		}
	    }
	}

	if (Mylene1f4_air_signals()) {
	    break;
	}

	status = 0;
    } while (0);

    return status;
}


static int
name_cell(void *context, const char **name)
{
    const char *path;

    path = ea52al(context)->loader;
    if (!path) {
	path = __install_prefix "/lib/Mylene1f4/cell.a52a";
    } else {
	if (!*path) {
	    path = __install_prefix "/lib/Mylene1f4/cell.a52a";
	}
    }

    *name = path;

    return 0;
}


static int
pull_cell(void *context, struct audio6_type *audio6_data)
{
    unsigned char head[16];
    int status;

    status = lead_cell(context, audio6_data, head, 16);
    if (status) {
	kill_cell(context, audio6_data->inter6);
    } else {
	do {
	    unsigned channels, lead, release, sampling;

	    lead = CARDINAL(head, 0x0);

	    sampling = lead >> 010;
	    if (sampling < 1024 || 1048576 < sampling) {
		stat_cell(context, "sampling frequency", sampling);
		status = 1;
		break;
	    }
	    channels = lead & 0xff;
	    if (1 < channels) {
		stat_cell(context, "channel count", channels + 1);
		status = 1;
		break;
	    }

	    release = head[6] << 010 | head[7] << 000;
	    if (release < 256 || 36864 < release) {
		stat_cell(context, "release size", release);
		status = 1;
		break;
	    }

	    channels++;

	    CHANNELS(context) = channels;
	    RELEASE(context) = release;
	    SAMPLING(context) = sampling;

	    {
		struct sequel_type *sequel_data;
		struct xa52a1_type *xa52a1_data;

		Mylene1f4_find_sequel
		    (context, Mylene1f4_XA52A1_SEQUEL, (void *) &sequel_data);
		xa52a1_data = sequel_data->data;
		xa52a1_data->frame_rate_d = release / channels >> 1;
		xa52a1_data->frame_rate_n = sampling;
	    }
	} while (0);
	if (status) {
	    kill_cell(context, audio6_data->inter6);
	}
    }

    return status;
}


static int
push_cell(void *context, struct audio6_type *audio6_data, unsigned miss)
{
    int status;
    unsigned char *lift, size[4];

#if __DEBUG_EXTERN__
    if (miss) {
	cell_stat_1("telling external loader that it needs to miss first %u "
		    "frames...\n", miss);
    } else {
	cell_stat_0("telling external loader that it needs to miss no frames"
		    "...\n");
    }
#endif				/* __DEBUG_EXTERN__ */

    lift = size + 3;
    if (!(miss & ~127)) {
	*lift = miss;
    } else {
	*lift = miss & 127;
	lift--;
	miss >>= 7;
	*lift = 128 | (miss & 127);
	miss >>= 7;
	miss &= (1 << 14) - 1;
	while (miss) {
	    lift--;
	    *lift = 128 | (miss & 127);
	    miss >>= 7;
	}
    }

    status = slip_cell(context, audio6_data, lift, size + 4 - lift);
    if (status) {
	kill_cell(context, audio6_data->inter6);
    } else {
	audio6_data->link = 0;
    }

    return status;
}


static int
slip_cell(void *context, struct audio6_type *audio6_data, void *data,
	  unsigned size)
{
    char *lead;
    int fd_1, status;
    unsigned te1st, te6th;

    te1st = ea52al_second_a(context);
    te6th = ea52al_second_f(context);
    if (1) {
	if (!(te6th < 1000000)) {
	    te1st += te6th / 1000000;
	    te6th %= 1000000;
	}
    }
    if (1) {
	struct timeval tv;

	gettimeofday(&tv, NULL);
	te1st += SECOND_a(tv);
	te6th += SECOND_f(tv);
	if (!(te6th < 1000000)) {
	    te1st++;
	    te6th -= 1000000;
	}
    }

    fd_1 = audio6_data->fd_1;

    lead = data;

    while (1) {
	fd_set fd_set_1, *fd_set_1_select;
	struct timeval *selector, timeout;

	if (SIGCHLD_received(context)) {
	    status = Mylene1f4_yway_mylene0fa(context, 0);
	    if (status) {
		break;
	    }
	}

	if (0) {
	} else {
	    fd_set_1_select = &fd_set_1;
	    FD_ZERO(fd_set_1_select);
	    FD_SET(fd_1, fd_set_1_select);
	}

	if (0) {
	} else {
	    unsigned second_a;

	    selector = &timeout;
	    gettimeofday(selector, NULL);
	    second_a = SECOND_a(timeout);
	    if (te1st < second_a) {
		stat_slip(context);
		status = -1;
		break;
	    } else {
		unsigned second_f, to6th;

		second_a = te1st - second_a;
		second_f = SECOND_f(timeout);
		to6th = te6th;
		if (!second_a) {
		    if (to6th < second_f) {
			stat_slip(context);
			status = -1;
			break;
		    }
		} else {
		    if (to6th < second_f) {
			second_a++;
			to6th += 1000000;
		    }
		}

		SECOND_a(timeout) = second_a;
		SECOND_f(timeout) = to6th - second_f;
	    }
	}

	status = select(fd_1 + 1, NULL, fd_set_1_select, NULL, selector);
	if (status == -1) {
	    if (errno != EINTR) {
		break;
	    }
	} else {
	    if (true(fd_set_1_select)) {
		if (FD_ISSET(fd_1, fd_set_1_select)) {
		    status = write(fd_1, lead, 1);
		    if (!status) {
			stat_push(context, 0);
			status = -1;
			break;
		    } else {
			if (status == -1) {
			    stat_push(context, 1);
			    break;
			} else {
			    size--;
			    if (size) {
				lead++;
			    } else {
#if __DEBUG_EXTERN__
				struct timeval tv;
				unsigned to1st, to6th;
#endif				/* __DEBUG_EXTERN__ */

#if __DEBUG_EXTERN__
				gettimeofday(&tv, NULL);
				if (te6th < ea52al_second_f(context)) {
				    te6th += 1000000;
				    te1st--;
				}
				te1st -= ea52al_second_a(context);
				te6th -= ea52al_second_f(context);
				to1st = SECOND_a(tv);
				to6th = SECOND_f(tv);
				if (to6th < te6th) {
				    to6th += 1000000;
				    to1st--;
				}
				to1st -= te1st;
				to6th -= te6th;
				cell_stat_2("external loader started phase 2"
					    " in %u, %u\n", to1st, to6th);
#endif				/* __DEBUG_EXTERN__ */
				status = 0;
				break;
			    }
			}
		    }
		}
	    }
	}
    }

    return status;
}


static int
stat_cell(void *context, const char *dimension, unsigned value)
{
    const char *loader;

    name_cell(context, &loader);
    Mylene1f4_fprintf_error
	(context, "%s: unreasonable %s (%u) submitted by `%s' external loade"
	 "r\n", Mylene1f4(context), dimension, value, loader);

    return 0;
}


static int
stat_lead(void *context)
{
    const char *loader;

    name_cell(context, &loader);

#if __DEBUG_EXTERN__
    cell_stat_2("phase 1 lead transfer time (%u, %u) expired for external lo"
		"ader\n", ea52al_second_a(context), ea52al_second_f(context));
#endif				/* __DEBUG_EXTERN__ */
    Mylene1f4_fprintf_error
	(context, "%s: `%s' external loader is taking too much starting up, "
	 "shutting down...\n", Mylene1f4(context), loader);

    return 0;
}


static int
stat_pull(void *context, int error)
{
    const char *loader;

    name_cell(context, &loader);
    Mylene1f4_fprintf_error
	(context, "%s: cannot read `%s' external loader", Mylene1f4(context),
	 loader);
    if (!error) {
	Mylene1f4_fprintf_error(context, "\n");
    } else {
	Mylene1f4_fprintf_error(context, ": %s\n", strerror(errno));
    }

    return 0;
}


static int
stat_push(void *context, int error)
{
    const char *loader;

    name_cell(context, &loader);
    Mylene1f4_fprintf_error
	(context, "%s: cannot write `%s' external loader", Mylene1f4(context),
	 loader);
    if (!error) {
	Mylene1f4_fprintf_error(context, "\n");
    } else {
	Mylene1f4_fprintf_error(context, ": %s\n", strerror(errno));
    }

    return 0;
}


static int
stat_slip(void *context)
{
    const char *loader;

    name_cell(context, &loader);

#if __DEBUG_EXTERN__
    cell_stat_2("phase 2 lead transfer time (%u, %u) expired for external lo"
		"ader\n", ea52al_second_a(context), ea52al_second_f(context));
#endif				/* __DEBUG_EXTERN__ */
    Mylene1f4_fprintf_error
	(context, "%s: `%s' external loader is taking too much starting up, "
	 "shutting down...\n", Mylene1f4(context), loader);

    return 0;
}


static int
tail_hell(void *context)
{
    Mylene1f4_fprintf_error(context, "\n");

    return 0;
}


static int
type_cell(void *loader, void *target)
{
    struct target_type *target_data;

    target_data = target;
    *target_data = data_target;

    return 0;
}


static int
type_hell(void *context, const char *hell, unsigned size)
{
    Mylene1f4_fprintf_error(context, "%.*s", (int) size, hell);

    return 0;
}


static int
wait_cell(void *e1f4_audio, int record)
{
    int status = 1;

    if (Audio6(e1f4_audio)->tail) {
    } else {
	unsigned lead;
	void *context;

	context = Audio6(e1f4_audio)->context;

	lead = STATE(context)[Mylene1f4_CELL_STATE] - C1f4(context);
	if (SIZE(context) < lead) {
	    lead += SIZE(context);
	}

#if __DEBUG_EXTERN__
	cell_stat_1("external loader quitted on us on frame %u\n",
		    REFERENCE(context));
	cell_stat_3("leaving %u extra frame%s decoded (and some %u bytes scr"
		    "ap)\n", lead / RELEASE(context),
		    lead / RELEASE(context) == 1 ? "" : "s",
		    lead % RELEASE(context));
#endif				/* __DEBUG_EXTERN__ */

	if (lead % RELEASE(context)) {
	} else {
	    lead /= RELEASE(context);
	    lead += REFERENCE(context);
	    if (lead == TAIL(context)) {
		status = 0;
	    }
	}

#if __DEBUG_EXTERN__
	if (status) {
	    cell_stat_0("though that won't do it\n");
	} else {
	    cell_stat_0("which is pretty much it...\n");
	}
#endif				/* __DEBUG_EXTERN__ */
    }

    Audio6(e1f4_audio)->wait = -1;

    if (status) {
	const char *loader;
	void *context;

	context = Audio6(e1f4_audio)->context;

	name_cell(context, &loader);

	Mylene1f4_fprintf_error
	    (context, "%s: `%s' external loader", Mylene1f4(context),
	     loader);
	Mylene1f4_stat_exit(context, record);
    }

    return status;
}


static int
zero_cell(void *e1f4_audio, unsigned zero)
{
    int status;

    Audio6(e1f4_audio)->tail = 0;

    status = push_cell(Audio6(e1f4_audio)->context, e1f4_audio, zero);

    return status;
}


int
Mylene1f4_clip_cell(void *loader, unsigned clip)
{
    /*
     * TODO
     *
     *.send clip information to loader, it might just use it...
     */

    return 0;
}


int
Mylene1f4_copy_cell(void *context, void *target)
{
    struct target_type *target_data;

    target_data = target;
    *target_data = data_target;

    return 0;
}


int
Mylene1f4_dump_cell(void *context, void *source, unsigned *count,
		    void **loader)
{
    int status;
    struct inter6_type *inter6_data;

    inter6_data = *loader;
    status = fini_none(context, inter6_data);
    free(inter6_data);

    return status;
}


int
Mylene1f4_load_cell(void *context, void *source, unsigned *count,
		    void **loader)
{
    int status;
    struct inter6_type *inter6_data;

    inter6_data = (struct inter6_type *) malloc(sizeof(struct inter6_type));
    if (!inter6_data) {
	Mylene1f4_fprintf_error
	    (context, "%s: %s\n", Mylene1f4(context), strerror(errno));

	status = -1;
    } else {
	inter6_data->context = context;
	if (true(*loader)) {
	    inter6_data->q1f42a = ((void **) *loader)[0];
	}

	status = init_none(context, source, inter6_data);
	if (status) {
	    free(inter6_data);
	} else {
	    if (0) {
	    } else {
		*count = 1;
		*loader = inter6_data;
	    }
	}
    }

    return status;
}


int
Mylene1f4_node_cell(void *context, const void **address)
{
    *address = &data_target;

    return 0;
}


int
Mylene1f4_type_cell(void *loader, void *target)
{
    return type_cell(loader, target);
}

#endif				/* __LINK_CELL_LOADER__ */
