/*
 * config.h
 * Copyright (C) 2005-2011, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __Mylene1f4_CONFIG_H__
#define __Mylene1f4_CONFIG_H__

#include <autoconfig.h>

#if defined HAVE_ENDIAN_H
# include <endian.h>
#endif				/* HAVE_ENDIAN_H */

#if defined HAVE_ENDIAN_H
# if __BYTE_ORDER == __BIG_ENDIAN
#  define WORDS_BIGENDIAN		1
# endif				/* __BYTE_ORDER == __BIG_ENDIAN */
#endif				/* HAVE_ENDIAN_H */

#if !defined WORDS_BIGENDIAN
# define WORDS_BIGENDIAN		0
#endif				/* !WORDS_BIGENDIAN */

#define Mylene1f4_CACHE			81

#define Mylene0fa_PREFIX		3

#define Mylene1f4_MIRROR		"Mylene0fa"

#define Mylene1f4_OUTPUT		"Mylene0fa"

#if SIZEOF_VOID_P == SIZEOF_LONG
# define integral_q			unsigned long
#else
# define integral_q			unsigned
#endif				/* SIZEOF_VOID_P == SIZEOF_LONG */

#define __ENABLE_x1f4LANES__		1
#define __ENABLE_x00696e6b__		1

#if !__ENABLE_x1f4LANES__
# undef __ENABLE_x00696e6b__
# define __ENABLE_x00696e6b__		0
#endif				/* !__ENABLE_x1f4LANES__ */

#define __ENABLE_CONTROLLING_SERVER__	1

#if __ENABLE_x00696e6b__
# define __EXPAND_x1f42MAPS__		1
#else
# define __EXPAND_x1f42MAPS__		0
#endif				/* __ENABLE_x00696e6b__ */

#if __ENABLE_CONTROLLING_SERVER__
# define __ENABLE_CONTROLLING_WINDOW__	1
#elif __ENABLE_x1f4LANES__
# define __ENABLE_CONTROLLING_WINDOW__	1
#else
# define __ENABLE_CONTROLLING_WINDOW__	0
#endif				/* __ENABLE_CONTROLLING_SERVER__ */

#if __ENABLE_CONTROLLING_SERVER__
# define __ENABLE_SEEK_WINDOW__		1
#elif __ENABLE_x1f4LANES__
# define __ENABLE_SEEK_WINDOW__		1
#else
# define __ENABLE_SEEK_WINDOW__		0
#endif				/* __ENABLE_CONTROLLING_SERVER__ */

#if __ENABLE_CONTROLLING_SERVER__
# define __NONBLOCKING_CLIENT_TRANSFER__ \
					1
#else
# define __NONBLOCKING_CLIENT_TRANSFER__ \
					0
#endif				/* __ENABLE_CONTROLLING_SERVER__ */

#if __ENABLE_CONTROLLING_SERVER__
# define MAXIMUM_REQUEST_TRANSFER_SIZE	4096
#endif				/* __ENABLE_CONTROLLING_SERVER__ */

#if __ENABLE_CONTROLLING_SERVER__
# define __DOUBLE_SELECT__		0
# define __SINGLE_SELECT__		1
#endif				/* __ENABLE_CONTROLLING_SERVER__ */

#if __ENABLE_CONTROLLING_SERVER__
# define __REFLEX_SELECT__		__DOUBLE_SELECT__
#endif				/* __ENABLE_CONTROLLING_SERVER__ */

#if !__ENABLE_CONTROLLING_SERVER__
# define __ENABLE_CHANNEL_SERVER__	0
#elif __REFLEX_SELECT__ != __DOUBLE_SELECT__
# define __ENABLE_CHANNEL_SERVER__	0
#else
# define __ENABLE_CHANNEL_SERVER__	1
#endif				/* !__ENABLE_CONTROLLING_SERVER__ */

#if defined HAVE_LIBAIME
# define __ENABLE_FILTER__		1
#else
# define __ENABLE_FILTER__		0
#endif				/* HAVE_LIBAIME */

#define __ENABLE_TAGGED_LIST__		1

#define __ENABLE_MPEG_EQUALIZER__	1

#if __ENABLE_MPEG_EQUALIZER__
# define __ENABLE_FAST_INTER__		1
#else
# define __ENABLE_FAST_INTER__		0
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if !defined HAVE_LIBAIME
# define __ENABLE_EQUEUE__		0
#elif !__ENABLE_FILTER__
# define __ENABLE_EQUEUE__		0
#elif !__ENABLE_CONTROLLING_SERVER__
# define __ENABLE_EQUEUE__		0
#else
# define __ENABLE_EQUEUE__		1
#endif				/* !HAVE_LIBAIME */

#if !__ENABLE_CONTROLLING_SERVER__
# define __ENABLE_EQUEUE_SERVER__	0
#elif !__ENABLE_EQUEUE__
# define __ENABLE_EQUEUE_SERVER__	0
#else
# define __ENABLE_EQUEUE_SERVER__	1
#endif				/* !__ENABLE_CONTROLLING_SERVER__ */

#if !__ENABLE_EQUEUE__
# define __LINK_LOAD_INTER__		0
#else
# define __LINK_LOAD_INTER__		1
#endif				/* !__ENABLE_EQUEUE__ */

#if !__LINK_LOAD_INTER__
# define __ENABLE_FQUEUE__		0
#else
# define __ENABLE_FQUEUE__		1
#endif				/* !__LINK_LOAD_INTER__ */

#if __ENABLE_EQUEUE__
# define __MISS_TEXT_FREE__		1
#endif				/* __ENABLE_EQUEUE__ */

#if !__ENABLE_CHANNEL_SERVER__
# define __ENABLE_CHANNEL_EQUEUE__	0
#elif !__ENABLE_EQUEUE__
# define __ENABLE_CHANNEL_EQUEUE__	0
#else
# define __ENABLE_CHANNEL_EQUEUE__	1
#endif				/* !__ENABLE_CHANNEL_SERVER__ */

#define __USE_1f44_EEAD__		1

#define __NEED_QUIET__			1

#define __LINK_CELL_LOADER__		1

#if __LINK_CELL_LOADER__
# define __WAIT_VIOLET__		1
#else
# define __WAIT_VIOLET__		0
#endif				/* __LINK_CELL_LOADER__ */

#define __EXTERNAL_LOADER__		1
#define __INTERNAL_LOADER__		2

#define __A52A_LOADER__			__EXTERNAL_LOADER__

#if __A52A_LOADER__ == __EXTERNAL_LOADER__
# define __LINK_A52A_LOADER__		__LINK_CELL_LOADER__
# define __LINK_CELL_A52A_LOADER__	__LINK_CELL_LOADER__
#else
# define __LINK_A52A_LOADER__		0
# define __LINK_CELL_A52A_LOADER__	0
#endif				/* __A52A_LOADER__ == __EXTERNAL_LOADER__ */

#define E1f4_START_INDEX		0

#if __ENABLE_MPEG_EQUALIZER__
# define MPEG_MAX_CLASS			20000
# define MPEG_MIN_CLASS			20
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
# define MPEG_MAX_LEVEL			20
# define MPEG_MIN_LEVEL			-20
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
# define MPEG_MAX_SCALE			20
# define MPEG_MIN_SCALE			-20
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#define __RESTART_Mylene0fa__		1

#if __RESTART_Mylene0fa__
# define RESTART_ORDER			4
#endif				/* __RESTART_Mylene0fa__ */

#if !__ENABLE_CONTROLLING_SERVER__
# define __RECORD_DEVICE__		0
#elif !__RESTART_Mylene0fa__
# define __RECORD_DEVICE__		0
#else
# define __RECORD_DEVICE__		1
#endif				/* !__ENABLE_CONTROLLING_SERVER__ */

#if __ENABLE_CONTROLLING_SERVER__
# define __EDITABLE_PLAYLIST__		1
#endif				/* __ENABLE_CONTROLLING_SERVER__ */

#if !__ENABLE_CONTROLLING_SERVER__
# define __ENABLE_EDIT_SERVER__		0
#elif !__EDITABLE_PLAYLIST__
# define __ENABLE_EDIT_SERVER__		0
#else
# define __ENABLE_EDIT_SERVER__		1
#endif				/* !__ENABLE_CONTROLLING_SERVER__ */

#if __ENABLE_x00696e6b__
# define __NEAR_LINK_BITS__		1
#else
# define __NEAR_LINK_BITS__		0
#endif				/* __ENABLE_x00696e6b__ */

#define SIMPLE_LIST_RECORD_SIZE		4096

#if __ENABLE_TAGGED_LIST__
# define TAGGED_LIST_RECORD_SIZE	6144
#endif				/* __ENABLE_TAGGED_LIST__ */

#endif				/* __Mylene1f4_CONFIG_H__ */
