/*
 * defs.h
 * Copyright (C) 2005, 2006, 2007, 2008, 2009, 2013, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __Mylene1f4_DEFS_H__
#define __Mylene1f4_DEFS_H__

#include <config.h>

/*
 * ea52al definitions
 */
#if __LINK_CELL_A52A_LOADER__
# define Mylene1f4_EA52AL_QUIET		(1L << 0x0)
#endif				/* __LINK_CELL_A52A_LOADER__ */

/*
 * eid3wl definitions
 */
#define Mylene1f4_EID3WL_QUIET		(1L << 0x0)

/*
 * equeue definitions
 */
#if __ENABLE_EQUEUE__
# define Mylene1f4_EXTERN_LOGIC		(1L << 0x0)
# define Mylene1f4_FAILED_LOGIC		(1L << 0x1)
# define Mylene1f4_INTERN_LOGIC		(1L << 0x2)
#endif				/* __ENABLE_EQUEUE__ */

/*
 * error code definitions
 */
#define Mylene1f4_ADVANCE_FAILURE	001
#define Mylene1f4_CHANGED_HEADER	002
#define Mylene1f4_MALLOC_FAILURE	003
#define Mylene1f4_NEAR_FAILURE		021
#define Mylene1f4_SYSTEM_FAILURE	024
#define Mylene1f4_UNEXPECTED_EOF	025
#define Mylene1f4_UNEXPECTED_EOS	026
#define Mylene1f4_UNEXPECTED_LENGTH	036
#define Mylene1f4_e66c_ERROR		037

#define Mylene1f4__ERROR		040

/*
 * extension definitions
 */
#define Mylene1f4_XA52A1_SEQUEL		1
#define Mylene1f4_XEPCM1_SEQUEL		2
#define Mylene1f4_XMPEG0_SEQUEL		3
#define Mylene1f4_XMPEG1_SEQUEL		4
#define Mylene1f4_x1f42e_SEQUEL		5

/*
 * field definitions
 */
#define Mylene1f4_MODE_FRAME		(0L << 0x0)

#define Mylene1f4_MODE_LEMON		(1L << 0x0)
#define Mylene1f4_MODE_LSTAT		(1L << 0x1)
#define Mylene1f4_MODE_NSTAT		(1L << 0x2)

#define Mylene1f4_MODE_FIELD		(7L << 0x0)

#define Mylene1f4_MODE_PRESS		0x3

/*
 * flag definitions
 */
#if __ENABLE_TAGGED_LIST__
# define Mylene1f4_BREAK_LIST		(1ul << 0x00)
#endif				/* __ENABLE_TAGGED_LIST__ */
#define Mylene1f4_ID3V2_READ		(1ul << 0x01)
#define Mylene1f4_LOCAL_LIST		(1ul << 0x02)
#define Mylene1f4_ERROR_MISS \
    (Mylene1f4_NULL_MISS | Mylene1f4_NEAR_MISS)
#if __NEED_QUIET__
# define Mylene1f4_QUIET_HERE		(1ul << 0x04)
#endif				/* __NEED_QUIET__ */
#define Mylene1f4_TILED_LIST		(1ul << 0x05)

/*
 * flag definitions (continued)
 */
#define Mylene1f4_OVER_TRACK		(1ul << 0x06)

/*
 * id3 tag field definitions
 */
#define Mylene1f4_ALBUM_FIELD		2
#define Mylene1f4_GENRE_FIELD		5
#define Mylene1f4_MATCH_FIELD		0
#define Mylene1f4_NOTES_FIELD		4
#define Mylene1f4_SHORT_FIELD		1
#define Mylene1f4_TITLE_FIELD		3

/*
 * id3 tag type definitions
 */
#define Mylene1f4_BROKEN_ID3TAG		0
#define Mylene1f4_MERGED_ID3TAG		1

/*
 * list index definitions
 */
#define Mylene1f4_CLIP_LIST		0
#define Mylene1f4_PLAY_LIST		1

/*
 * load error flag definitions
 */
#define Mylene1f4_NULL_MISS		(0ul << 0x03)
#define Mylene1f4_NEAR_MISS		(1ul << 0x03)

/*
 * load error flag definitions (continued)
 */
#define Mylene1f4_FUSSY_MISS		Mylene1f4_NULL_MISS

/*
 * load order definitions
 */
#define Mylene1f4_LOAD_LAST		Mylene1f4_LOAD_PREVIOUS
#define Mylene1f4_LOAD_NEXT		0156
#define Mylene1f4_LOAD_PREVIOUS		0160
#define Mylene1f4_LOAD_RAND		Mylene1f4_LOAD_RANDOM
#define Mylene1f4_LOAD_RANDOM		0172

/*
 * mode definitions
 */
#if __ENABLE_CONTROLLING_WINDOW__
# define Mylene1f4_WINDOW_BASE		2
#endif				/* __ENABLE_CONTROLLING_WINDOW__ */

#if __ENABLE_CONTROLLING_WINDOW__
# define Mylene1f4_WINDOW_MASK		(3 << Mylene1f4_WINDOW_BASE)
#endif				/* __ENABLE_CONTROLLING_WINDOW__ */

/*
 * mpeg synchronization method definitions
 */
#define Mylene1f4_COTTON_FRAME		0
#define Mylene1f4_DOUBLE_FRAME		1
#define Mylene1f4_SELECT_FRAME		2
#define Mylene1f4_SINGLE_FRAME		3

/*
 * output definitions
 */
#define Mylene1f4_LINK_OUTPUT		3
#define Mylene1f4_NULL_OUTPUT		2
#define Mylene1f4_SKIP_OUTPUT		0

/*
 * pickup method definitions
 */
#define Mylene1f4_PAGE_PICKUP		0
#define Mylene1f4_TIME_PICKUP		1

/*
 * server setup definitions
 */
#if __ENABLE_CONTROLLING_SERVER__
# define Mylene1f4_DELAY_SERVER		(1L << 0x0)
# define Mylene1f4_FUSSY_FILTER		(1L << 0x1)
# define Mylene1f4_FUSSY_FORMAT		(1L << 0x2)
# define Mylene1f4_FUSSY_OUTPUT		(1L << 0x3)
# define Mylene1f4_PAUSE_SERVER		(1L << 0x4)
# define Mylene1f4_REUSE_RECORD		(1L << 0x5)
#endif				/* __ENABLE_CONTROLLING_SERVER__ */

/*
 * state definitions
 */
#define Mylene1f4_B1st_STATE		003
#define Mylene1f4_CELL_STATE		004
#define Mylene1f4_REAL_STATE		002

/*
 * state indexed 3 definitions
 */
#define Mylene1f4_B1st_PLAYING		(1L << 0x0)
#define Mylene1f4_B1st_DELAYED		(1L << 0x1)

/*
 * window definitions
 */
#if __ENABLE_CONTROLLING_SERVER__
# define Mylene1f4_SERVER_WINDOW	(1 << Mylene1f4_WINDOW_BASE)
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
#if __ENABLE_x1f4LANES__
# define Mylene1f4_WITHIN_WINDOW	(2 << Mylene1f4_WINDOW_BASE)
#endif				/* __ENABLE_x1f4LANES__ */
#if __ENABLE_CONTROLLING_WINDOW__
# define Mylene1f4_EXTENT_WINDOW	(0 << Mylene1f4_WINDOW_BASE)
#endif				/* __ENABLE_CONTROLLING_WINDOW__ */

#endif				/* __Mylene1f4_DEFS_H__ */
