/*
 * e0fa.c
 * Copyright (C) 2005-2011, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <errno.h>
#include <string.h>
#include <unistd.h>

#include <access.h>
#include <defs.h>
#include <inter.h>

#define __DEBUG_Mylene0fa__		0

#if WORDS_BIGENDIAN
# define COMPLETIONS(completions) \
    (completions)
#else
# define COMPLETIONS(completions) \
    (((completions) & 127) | ((completions) >> 1 & ~127)		      \
     | ((completions) & 128) << 030)
#endif				/* WORDS_BIGENDIAN */

#if WORDS_BIGENDIAN
# define CARDINAL(data, base) \
    ((data)[(base) + 0] << 030 | (data)[(base) + 1] << 020		      \
     | (data)[(base) + 2] << 010 | (data)[(base) + 3])
#else
# define CARDINAL(data, base) \
    ((data)[(base) + 0] | (data)[(base) + 1] << 010			      \
     | (data)[(base) + 2] << 020 | (data)[(base) + 3] << 030)
#endif				/* WORDS_BIGENDIAN */

#define Mirror(mirror) \
    ((struct mirror_type *) (mirror))

#if __DEBUG_Mylene0fa__
# define R_0fa_1(format, arg1) \
    do {								      \
	fprintf(stderr, "Mylene0fa.rd" format, arg1);			      \
    } while (0)
# define R_0fa_2(format, arg1, arg2) \
    do {								      \
	fprintf(stderr, "Mylene0fa.rd" format, arg1, arg2);		      \
    } while (0)
#endif				/* __DEBUG_Mylene0fa__ */

int
Mylene1f4_read_e0fa(void *context, int pending_exit)
{
    int status = 0;
    unsigned incoming;

    incoming = INCOMING(context);

    do {
	unsigned char *input;

	input = INPUT(context);

	if (Z0fa(context)) {
	    Z0fa(context) = 0;
	} else {
	    status =
		read(Mylene1ST(context), input + incoming, 0x10 - incoming);
	    if (status == -1) {
		if (errno == EINTR) {
		    status = 0;
		} else {
		    Mylene1f4_fprintf_error
			(context, "%s: cannot read `%s': %s\n",
			 Mylene1f4(context), Mirror(MIRROR(context))->path,
			 strerror(errno));
		}

		break;
	    } else {
		if (status) {
		    incoming += status;
		    status = 0;
		} else {
		    if (pending_exit) {
			close(Mylene1ST(context));
			Mylene1ST(context) = -1;
			close(Mylene2ND(context));
			Mylene2ND(context) = -1;
		    } else {
#if __RESTART_Mylene0fa__
#else
			status = -1;
#endif				/* __RESTART_Mylene0fa__ */

			Mylene1f4_fprintf_error
			    (context, "%s: `%s' closed\n", Mylene1f4(context),
			     Mirror(MIRROR(context))->path);

#if __RESTART_Mylene0fa__
			status = Mylene1f4_yway_mylene0fa(context, 0);
#endif				/* __RESTART_Mylene0fa__ */
		    }

		    break;
		}
	    }
	}

	while (incoming) {
	    unsigned c;

	    c = input[0];
#if __DEBUG_Mylene0fa__
	    R_0fa_1(": %02x\n", c);
#endif				/* __DEBUG_Mylene0fa__ */
	    if (c & 128) {
		if (incoming < 4) {
		    break;
		}
		{
		    unsigned completions;

		    completions = CARDINAL(input, 0);
		    completions = COMPLETIONS(completions);
#if __DEBUG_Mylene0fa__
		    R_0fa_2(":     C/S: %6u, %6u\n",
			    completions & ~(1u << 31),
			    STREAMS(context) & ~(1u << 31));
#endif				/* __DEBUG_Mylene0fa__ */
		    if (completions != STREAMS(context)
			|| !(completions & 1u << 31)
			|| !(EXTRA(context) & Mylene1f4_OVER_TRACK)) {
			status = 1;
			break;
		    } else {
			STREAMS(context) &= ~(1u << 31);
		    }
		}

		incoming -= 4;
		if (incoming) {
		    memmove(input, input + 4, incoming);
		    Z0fa(context) = 1;
		}

		break;
	    } else if (c & 64) {
		if (incoming < 010) {
		    break;
		}
		{
		    int (*patch_64) (void *);

		    patch_64 = FINI(context);
		    if (patch_64) {
			status = patch_64(context);
			if (status) {
			    break;
			}
		    }
		}

		incoming -= 010;
		if (incoming) {
		    memmove(input, input + 010, incoming);
		    Z0fa(context) = 1;
		}

		break;
	    } else if (c & 32) {
		if (incoming < 020) {
		    break;
		}
		{
		    int (*patch_32) (void *);

		    patch_32 = INIT(context);
		    if (patch_32) {
			status = patch_32(context);
			if (status) {
			    break;
			}
		    }
		}

		incoming -= 020;
		if (incoming) {
		    memmove(input, input + 020, incoming);
		    Z0fa(context) = 1;
		}

		break;
	    } else {
		switch (c) {
		case 3:
		    if (LINK(context)) {
			LINK(context)--;
		    } else {
			status = 1;
		    }
		    break;
		case 4:
		case 5:
		case 6:
		case 7:
		    WIRE(context) = c & ~4;
		    break;
		default:
		    status = 1;
		}
		if (status) {
		    break;
		} else {
		    incoming--;
		    if (incoming) {
			memmove(input, input + 1, incoming);
		    }
		}
	    }
	}

#if 0
	if (status == 1) {
#elif 0
	}
#else
	if (status) {
#endif				/* 0 */
	    Mylene1f4_fprintf_error
		(context, "%s: `%s' communication breakdown, failed transmis"
		 "sion was %02x\n", Mylene1f4(context),
		 Mirror(MIRROR(context))->path, input[0]);
	}
    } while (0);

    INCOMING(context) = incoming;

    return status;
}
