/*
 * e3e8.c
 * Copyright (C) 2005-2011, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if __ENABLE_CONTROLLING_SERVER__
# include <stdlib.h>
# include <unistd.h>
# include <stddef.h>
# include <string.h>
#endif				/* __ENABLE_CONTROLLING_SERVER__ */

#if __ENABLE_CONTROLLING_SERVER__
# include <access.h>
# include <defs.h>
# include <excess4.h>
# include <excess7.h>
# include <inter.h>
#endif				/* __ENABLE_CONTROLLING_SERVER__ */

#if __ENABLE_CONTROLLING_SERVER__
# define Frame4(context) \
    ((struct frame4_type *) &Frame7((context))->_1.frame4)
# define Frame7(context) \
    ((struct frame7_type *) WINDOW((context)).server.window)
#endif				/* __ENABLE_CONTROLLING_SERVER__ */

#if __ENABLE_CHANNEL_SERVER__
static int fast_away(void *);
#endif				/* __ENABLE_CHANNEL_SERVER__ */
#if __ENABLE_CONTROLLING_SERVER__
static int head_away(void *);
static int slip_away(void *);
#endif				/* __ENABLE_CONTROLLING_SERVER__ */

#if __ENABLE_CONTROLLING_SERVER__
static const char *slippingaway____ = "slippingaway\0\0\0\0";
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
#if __ENABLE_CHANNEL_SERVER__
static const struct every4_type every_channel[] = {
/* *INDENT-OFF* */
    {	Mylene1f4_miss_back,
	Mylene1f4_pipe_back	},
    {	Mylene1f4_slip_back,
	Mylene1f4_text_back	}
/* *INDENT-ON* */
};
#endif				/* __ENABLE_CHANNEL_SERVER__ */

#if __ENABLE_CHANNEL_SERVER__
static int
fast_away(void *context)
{
    int client, status;
    struct frame4_type *frame4_data;
    unsigned char c;

    frame4_data = Frame4(context);

    client = frame4_data->client;

    status = read(client, &c, 1);
    if (status == 1) {
	if (c & ~1) {
	    status = 2;
	} else {
	    frame4_data->trans = slip_away;

	    if (c & 1) {
		frame4_data->every4 = every_channel[1];
		frame4_data->flags |= Mylene1f4_AIME_TARGET;
	    } else {
		frame4_data->every4 = every_channel[0];
		frame4_data->flags &= ~Mylene1f4_AIME_TARGET;
	    }
	}
    }

    if (status != 1) {
	Mylene1f4_lose_e3e8(context);
    }

    return 0;
}
#endif				/* __ENABLE_CHANNEL_SERVER__ */


#if __ENABLE_CONTROLLING_SERVER__
static int
head_away(void *context)
{
    int client, status;
    struct frame4_type *frame4_data;
    unsigned char c;

    frame4_data = Frame4(context);

    client = frame4_data->client;

    status = read(client, &c, 1);
    if (status == 1) {
	unsigned progress;

	progress = frame4_data->progress;
	if (c != slippingaway____[16 - progress]) {
	    status = 2;
	} else {
	    progress--;

# if __REFLEX_SELECT__ == __DOUBLE_SELECT__
	    if (progress == 1) {
		frame4_data->trans = fast_away;
	    } else {
# else
		if (!progress) {
		    frame4_data->trans = slip_away;
		} else {
# endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */
		    frame4_data->progress = progress;
# if __REFLEX_SELECT__ == __DOUBLE_SELECT__
		}
# else
	    }
# endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */
	}
    }

    if (status != 1) {
	Mylene1f4_lose_e3e8(context);
    }

    return 0;
}
#endif				/* __ENABLE_CONTROLLING_SERVER__ */


#if __ENABLE_CONTROLLING_SERVER__
static int
slip_away(void *context)
{
    int client, request = 0, status;
    struct frame4_type *frame4_data;
    unsigned char c;

    frame4_data = Frame4(context);

    client = frame4_data->client;

    status = read(client, &c, 1);
    if (status == 1) {
	if (!frame4_data->a112.ever) {
	    Mylene1f4_look_alia(context, c);
	}
	if (frame4_data->a112.ever) {
	    status = (*frame4_data->a112.ever) (context, c);
	    if (status) {
		status = -1;
	    } else {
		status = 1;
		if (frame4_data->request) {
		    request = 1;
		}
	    }
	}
    }

    if (status != 1) {
	Mylene1f4_lose_e3e8(context);
    }

    return request;
}
#endif				/* __ENABLE_CONTROLLING_SERVER__ */


#if __NONBLOCKING_CLIENT_TRANSFER__
int
Mylene1f4_edge_e3e8(void *context, char **pick, unsigned *slip)
{
    char *data;
    int delete;
    unsigned size;

    data = *pick;
    size = *slip;

    delete = read(Frame4(context)->client, data, size);
    if (delete) {
	if (delete == -1) {
	    delete = 0;
	} else {
	    *pick = data + delete;
	    *slip = size - delete;
	}
    }

    if (delete) {
    } else {
	Mylene1f4_lose_e3e8(context);
    }

    return delete;
}
#endif				/* __NONBLOCKING_CLIENT_TRANSFER__ */


#if __ENABLE_CONTROLLING_SERVER__
int
Mylene1f4_link_e3e8(void *context)
{
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    frame4_data->trans = head_away;

    frame4_data->a112.ever = NULL;

    frame4_data->flags &= ~Mylene1f4_FREE_CLIENT;

# if __ENABLE_CHANNEL_SERVER__
    frame4_data->channel_up = 1;
# endif				/* __ENABLE_CHANNEL_SERVER__ */

    return 0;
}
#endif				/* __ENABLE_CONTROLLING_SERVER__ */


#if __ENABLE_CONTROLLING_SERVER__
int
Mylene1f4_lose_e3e8(void *context)
{
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    close(frame4_data->client);

    frame4_data->client = -1;

# if __ENABLE_CHANNEL_SERVER__
    if (1) {
	unsigned client_up;

	client_up = frame4_data->client_up;
	client_up++;
	if (client_up) {
	} else {
	    client_up = 1;

#  if __ENABLE_EQUEUE__
	    Mylene1f4_away_equeue(context);
#  endif			/* __ENABLE_EQUEUE__ */
	}

	frame4_data->client_up = client_up;
    }
# endif				/* __ENABLE_CHANNEL_SERVER__ */

    if (frame4_data->flags & Mylene1f4_FREE_CLIENT) {
	free(frame4_data->a112.data.text.text);
    }

    {
	char *data;

# if __ENABLE_EQUEUE__
	data = frame4_data->dana.name;
	if (data) {
	    frame4_data->dana.name = NULL;
	    free(data);
	}
# endif				/* __ENABLE_EQUEUE__ */
	data = frame4_data->back.slack.data;
	if (data) {
	    frame4_data->back.slack.data = NULL;
	    free(data);
	}
    }

    return 0;
}
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
