/*
 * equeue.c
 * Copyright (C) 2007-2011, 2013, 2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if __ENABLE_EQUEUE__
# include <aime.h>
# include <stddef.h>
# include <stdlib.h>
# include <string.h>
# include <time.h>
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# include <access.h>
# include <defs.h>
# include <excess4.h>
# include <excess7.h>
# include <inter.h>
# include <types.h>
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define true(e)			1
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define e4text(context) \
    ((struct e4text_type *) Frame4((context))->lxdeck)
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define Frame4(context) \
    ((struct frame4_type *) &Frame7((context))->_1.frame4)
# define Frame7(context) \
    ((struct frame7_type *) WINDOW((context)).server.window)
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
static int lose_record(void *, struct x1f4_indexset_type *);
static int side_equeue(void *, struct equeue_type *);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
static int
lose_record(void *link, struct x1f4_indexset_type *indexset_data)
{
    const char *name;
    struct x1f4_datatype_type *datatype_data;

    datatype_data = indexset_data->datatype_set.datatype_data;

    name = datatype_data->name;
    while (name) {
	if (datatype_data->size == 6) {
	    if (memcmp(name, "record", 6)) {
	    } else {
		break;
	    }
	}

	datatype_data++;
	name = datatype_data->name;
    }

    if (true(name)) {
	datatype_data->flat(datatype_data->context, link);
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
side_equeue(void *context, struct equeue_type *equeue_data)
{
    struct x1f4_dxflat_type *dxflat_data;
    void *data, *link;

    dxflat_data = equeue_data->dxflat;
    if (dxflat_data) {
	dxflat_data->data = NULL;
    }

    data = equeue_data->text.data;
    link = equeue_data->text.link;
    if (data || link) {
	void *aqueue;

	aqueue = Frame4(context)->dana.equeue;

	Frame4(context)->dana.equeue = equeue_data;

	if (data) {
	    x1f4_fini_mxdeck(&data);
	    if (1) {
		equeue_data->text.data = NULL;
	    }
	}

	if (link) {
	    lose_record(&link, e4text(context)->indexset);
	    if (1) {
		equeue_data->text.link = NULL;
	    }
	}

	Frame4(context)->dana.equeue = aqueue;
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_blue_equeue(void *context, void *equeue)
{
    /*
     * side_equeue needs moon still set
     */

    side_equeue(context, equeue);

    Mylene1f4_fini_moon(context, equeue);

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_deck_equeue(void *context, void *equeue)
{
    struct equeue_type *equeue_data;
    void *data;

    equeue_data = equeue;

    if (equeue_data->bits & Mylene1f4_INTERN_LOGIC) {
	Mylene1f4_flat_equeue(context, equeue_data);
    } else {
    }

    data = equeue_data->rate;
    if (data) {
	free(data);
    }

    Mylene1f4_blue_equeue(context, equeue_data);

    data = equeue_data->name;
    if (data) {
	free(data);
    }

    free(equeue);

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_fini_equeue(void *context)
{
    return Mylene1f4_fini_iqueue
	(context, Frame4(context)->iqueue.head_equeue,
	 WINDOW(context).server.iqueue);
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_flat_equeue(void *context, void *equeue)
{
    void *aqueue;

    aqueue = Frame4(context)->dana.equeue;

    Frame4(context)->dana.equeue = equeue;

    Mylene1f4_fini_aime(context, equeue);

    Frame4(context)->dana.equeue = aqueue;

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_free_equeue(void *context, void *equeue)
{
    if (((struct equeue_type *) equeue)->bits & Mylene1f4_EXTERN_LOGIC) {
	Mylene1f4_sand_equeue(context, equeue);
    } else {
	Mylene1f4_deck_equeue(context, equeue);
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_init_equeue(void *context)
{
    struct frame4_type *frame4_data;
    struct iqueue_type *iqueue_data;

    frame4_data = Frame4(context);

    frame4_data->dana.last = BECCA(context).tv_sec;

    iqueue_data = WINDOW(context).server.iqueue;
    if (!iqueue_data) {
	frame4_data->iqueue.head_equeue = NULL;
	frame4_data->iqueue.tail_equeue = NULL;

	frame4_data->iqueue.lqueue = 0;

	frame4_data->dana.clip = 0;

	frame4_data->logic = NULL;
    } else {
	unsigned iqueue;

	iqueue = iqueue_data->lqueue;

	frame4_data->iqueue.head_equeue = iqueue_data->head_equeue;
	frame4_data->iqueue.tail_equeue = iqueue_data->tail_equeue;

	frame4_data->iqueue.lqueue = iqueue;

	frame4_data->dana.clip = iqueue;

	frame4_data->logic = Mylene1f4_link_aime;

	if (1) {
	    struct equeue_type *equeue_data;
	    unsigned clip = 0;

	    equeue_data = frame4_data->iqueue.head_equeue;
	    while (equeue_data) {
		equeue_data->major = 0;
		equeue_data->minor = clip;

		clip++;

		equeue_data = equeue_data->equeue_data;
	    }
	}
    }

    frame4_data->instant = 0;
    frame4_data->link = 0;

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_line_equeue(void *context, void *equeue)
{
    struct equeue_type *equeue_data;

    equeue_data = equeue;

    if (equeue_data->bits & Mylene1f4_INTERN_LOGIC) {
	Mylene1f4_flat_equeue(context, equeue_data);
	if (1) {
	    equeue_data->bits = Mylene1f4_EXTERN_LOGIC;
	}
    } else {
    }

    Mylene1f4_blue_equeue(context, equeue_data);

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_lose_equeue(void *context)
{
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    if (frame4_data->iqueue.lqueue) {
	Mylene1f4_fini_equeue(context);

	{
	    frame4_data->iqueue.lqueue = 0;

	    frame4_data->iqueue.head_equeue = NULL;
	    frame4_data->iqueue.tail_equeue = NULL;

	    if (frame4_data->logic == Mylene1f4_link_aime) {
		frame4_data->logic = NULL;
	    }
	}
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_miss_equeue(void *context, unsigned delete)
{
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    if (frame4_data->iqueue.lqueue == 1) {
	Mylene1f4_lose_equeue(context);
    } else {
	Mylene1f4_miss_iqueue(context, &frame4_data->iqueue, delete);
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_move_equeue(void *context, unsigned insert, unsigned delete)
{
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    if (delete == insert) {
    } else {
	if (delete + 1 == insert) {
	} else {
# if 0
	    struct equeue_type *equeue_data;

	    if (delete < insert) {
		insert -= 1;
	    } else {
	    }

	    if (!delete) {
		equeue_data = frame4_data->iqueue.head_equeue;
		frame4_data->iqueue.head_equeue = equeue_data->equeue_data;
	    } else {
		struct equeue_type *equeue_slip;
		unsigned i;

		equeue_slip = frame4_data->iqueue.head_equeue;
		i = delete - 1;
		for (; i; i--) {
		    equeue_slip = equeue_slip->equeue_data;
		}

		equeue_data = equeue_slip->equeue_data;
		equeue_slip->equeue_data = equeue_data->equeue_data;

		if (equeue_data == frame4_data->iqueue.tail_equeue) {
		    frame4_data->iqueue.tail_equeue = equeue_slip;
		}
	    }

	    if (!insert) {
		equeue_data->equeue_data = frame4_data->iqueue.head_equeue;
		frame4_data->iqueue.head_equeue = equeue_data;
	    } else {
		struct equeue_type *equeue_slip;
		unsigned i;

		insert--;

		equeue_slip = frame4_data->iqueue.head_equeue;
		i = insert;
		for (; i; i--) {
		    equeue_slip = equeue_slip->equeue_data;
		}

		equeue_data->equeue_data = equeue_slip->equeue_data;
		equeue_slip->equeue_data = equeue_data;

		if (equeue_slip == frame4_data->iqueue.tail_equeue) {
		    frame4_data->iqueue.tail_equeue = equeue_data;
		}
	    }
# else
	    if (delete < insert) {
		if (!delete) {
		    struct equeue_type *equeue_data, *equeue_slip;
		    unsigned i;

		    equeue_data = frame4_data->iqueue.head_equeue;
		    equeue_slip = equeue_data->equeue_data;
		    frame4_data->iqueue.head_equeue = equeue_slip;

		    i = insert - 2;
		    for (; i; i--) {
			equeue_slip = equeue_slip->equeue_data;
		    }

		    equeue_data->equeue_data = equeue_slip->equeue_data;
		    equeue_slip->equeue_data = equeue_data;

		    if (equeue_slip == frame4_data->iqueue.tail_equeue) {
			frame4_data->iqueue.tail_equeue = equeue_data;
		    }
		} else {
		    struct equeue_type *equeue_data, *equeue_half,
			*equeue_slip;
		    unsigned i;

		    equeue_slip = frame4_data->iqueue.head_equeue;

		    i = delete - 1;
		    for (; i; i--) {
			equeue_slip = equeue_slip->equeue_data;
		    }

		    equeue_data = equeue_slip->equeue_data;
		    equeue_half = equeue_data->equeue_data;
		    equeue_slip->equeue_data = equeue_half;

		    i = insert - delete - 2;
		    for (; i; i--) {
			equeue_half = equeue_half->equeue_data;
		    }

		    equeue_data->equeue_data = equeue_half->equeue_data;
		    equeue_half->equeue_data = equeue_data;

		    if (equeue_half == frame4_data->iqueue.tail_equeue) {
			frame4_data->iqueue.tail_equeue = equeue_data;
		    }
		}
	    } else {
		if (!insert) {
		    struct equeue_type *equeue_data, *equeue_slip,
			*head_equeue;
		    unsigned i;

		    head_equeue = frame4_data->iqueue.head_equeue;
		    equeue_slip = head_equeue;

		    i = delete - 1;
		    for (; i; i--) {
			equeue_slip = equeue_slip->equeue_data;
		    }

		    equeue_data = equeue_slip->equeue_data;
		    equeue_slip->equeue_data = equeue_data->equeue_data;

		    if (equeue_data == frame4_data->iqueue.tail_equeue) {
			frame4_data->iqueue.tail_equeue = equeue_slip;
		    }

		    equeue_data->equeue_data = head_equeue;
		    frame4_data->iqueue.head_equeue = equeue_data;
		} else {
		    struct equeue_type *equeue_data, *equeue_half,
			*equeue_slip;
		    unsigned i;

		    equeue_half = frame4_data->iqueue.head_equeue;

		    i = insert - 1;
		    for (; i; i--) {
			equeue_half = equeue_half->equeue_data;
		    }

		    equeue_slip = equeue_half->equeue_data;

		    i = delete - insert - 1;
		    for (; i; i--) {
			equeue_slip = equeue_slip->equeue_data;
		    }

		    equeue_data = equeue_slip->equeue_data;
		    equeue_slip->equeue_data = equeue_data->equeue_data;

		    if (equeue_data == frame4_data->iqueue.tail_equeue) {
			frame4_data->iqueue.tail_equeue = equeue_slip;
		    }

		    equeue_data->equeue_data = equeue_half->equeue_data;
		    equeue_half->equeue_data = equeue_data;
		}
	    }
# endif				/* 0 */
	}
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_push_equeue(void *context, unsigned slip)
{
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    if (slip) {
	struct equeue_type *equeue_data;

	equeue_data = frame4_data->iqueue.head_equeue;

	frame4_data->iqueue.head_equeue = equeue_data->equeue_data;

	if (slip == frame4_data->iqueue.lqueue) {
	    frame4_data->iqueue.tail_equeue->equeue_data = equeue_data;
	    frame4_data->iqueue.tail_equeue = equeue_data;
	    equeue_data->equeue_data = NULL;
	    if (slip == 1) {
		frame4_data->iqueue.head_equeue = equeue_data;
	    }
	} else {
	    struct equeue_type *miss_equeue;

	    miss_equeue = frame4_data->iqueue.head_equeue;
	    slip--;
	    --slip;
	    for (; slip; slip--) {
		miss_equeue = miss_equeue->equeue_data;
	    }

	    equeue_data->equeue_data = miss_equeue->equeue_data;
	    miss_equeue->equeue_data = equeue_data;
	}
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_sand_equeue(void *context, void *equeue)
{
    Mylene1f4_line_equeue(context, equeue);

    free(equeue);

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_slip_equeue(void *context, int end, unsigned delete)
{
    do {
	unsigned lqueue;

	lqueue = Frame4(context)->iqueue.lqueue;

	if (end) {
	    if (lqueue < delete) {
		break;
	    } else {
		delete = lqueue - delete;
	    }
	} else {
	    if (delete < lqueue) {
	    } else {
		break;
	    }
	}

	Mylene1f4_miss_equeue(context, delete);
    } while (0);

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_swap_equeue(void *context, int fix, unsigned insert, int end,
		      unsigned delete)
{
    do {
	unsigned lqueue;

	lqueue = Frame4(context)->iqueue.lqueue;

	if (end) {
	    if (lqueue < delete) {
		break;
	    } else {
		delete = lqueue - delete;
	    }
	} else {
	    if (delete < lqueue) {
	    } else {
		break;
	    }
	}
	if (fix) {
	    if (lqueue + 1 < insert) {
		break;
	    } else {
		insert = lqueue + 1 - insert;
	    }
	} else {
	    if (lqueue < insert) {
		break;
	    } else {
	    }
	}

	Mylene1f4_move_equeue(context, insert, delete);
    } while (0);

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_trap_equeue(void *context, void *equeue)
{
    struct equeue_type *equeue_data;
    void *data;

    equeue_data = equeue;

    if (equeue_data->bits & Mylene1f4_INTERN_LOGIC) {
	Mylene1f4_flat_equeue(context, equeue_data);
    } else {
    }

    data = equeue_data->rate;
    if (data) {
	free(data);
    }

    side_equeue(context, equeue_data);

    data = equeue_data->name;
    if (data) {
	free(data);
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */
