/*
 * excess2.c
 * Copyright (C) 2005-2011, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * BUGS
 *
 * stream time computation is wrong since it assumes that all the streams for
 *     object target has the same rate
 */

#include <config.h>

#if defined HAVE_LIBLEWD
# include <lewd.h>
#endif				/* HAVE_LIBLEWD */
#include <stddef.h>

#include <sys/time.h>

#include <access.h>
#if !defined HAVE_LIBLEWD
# include <ckeylist.h>
#endif				/* !HAVE_LIBLEWD */
#include <defs.h>
#include <inter.h>

#if __ENABLE_TAGGED_LIST__
# define __PLAY_TILE__			1
#else
# define __PLAY_TILE__			0
#endif				/* __ENABLE_TAGGED_LIST__ */

#define SECOND_A(v)			((v)->tv_sec)
#define SECOND_F(v)			((v)->tv_usec)

#define Output(context) \
    ((const struct output_type *) DEVICE(context))
#define Window(context) \
    ((struct window_type *) WINDOW(context).extent.window)

#if __PLAY_TILE__
# define PLAY_FILE(context, window) \
    (window).play((context))
#else
# define PLAY_FILE(context, window) \
    play_file((context))
#endif				/* __PLAY_TILE__ */

typedef struct window_type {
#if __PLAY_TILE__
    int (*play) (void *);
#endif				/* __PLAY_TILE__ */
    unsigned skipped;
} window_type;

extern const struct x1f4_ckeytree_type _Mylene1f4_process_root[];

static int clip_run1(void *);
static int clip_run2(void *);
static int clip_run3(void *);
static int eVer_clip(void *, unsigned);
static int line_clip(void *);
static int make_link(void *);
static int plAy_dAta(void *);
static int plAy_data(void *);
static int play_clip(void *);
static int play_dAta(void *, unsigned);
static int play_data(void *, unsigned);
static int play_file(void *);
static int play_list(void *);
#if __ENABLE_TAGGED_LIST__
static int play_tile(void *);
#endif				/* __ENABLE_TAGGED_LIST__ */
static int skip_dAta(void *, unsigned);
static int skip_data(void *, unsigned);

static int
clip_run1(void *context)
{
    int start, status = 0;
    struct window_type window;

    SEQUEL(context) = NULL;
    STATE(context)[0] = 0;
    STATE(context)[1] = 0;
    X1f42(context) = NULL;

    window.skipped = 0;

    WINDOW(context).extent.window = &window;

    start = START(context);

#if __ENABLE_TAGGED_LIST__
    if (FLAGS(context) & Mylene1f4_BREAK_LIST) {
	window.play = play_tile;
    } else {
	window.play = play_file;
    }
#endif				/* __ENABLE_TAGGED_LIST__ */

    while (1) {
	if (1) {
	    int class;

	    status = Mylene1f4_deck_list(context, &start, &class);
	    if (status) {
		break;
	    } else {
		if (class) {
		    break;
		}
	    }
	}

	while (EXCESS(context).e1.list87.clear) {
	    SOURCE(context)._.s1.path = EXCESS(context).e1.list87.file_name;
	    status = PLAY_FILE(context, window);
	    if (status) {
		break;
	    } else {
		if (window.skipped == EXCESS(context).e1.list87.count) {
		    break;
		}
	    }

	    status = Mylene1f4_slip_list(context);
	    if (status) {
		break;
	    }
	}

	if (!REPEAT(context)) {
	    break;
	} else {
	    if (EXCESS(context).e1.list87.clear) {
		break;
	    } else {
		if (LOAD_ORDER(context) == Mylene1f4_LOAD_LAST) {
		    start = -1;
		}
	    }
	}
    }

    return status;
}


static int
clip_run2(void *context)
{
    int status;

    status = Output(context)->init(context);
    if (!status) {
	int excess;

	status = clip_run1(context);
	if (status) {
	    Output(context)->kill(context);
	}
	excess = Output(context)->fini(context);
	if (!status) {
	    status = excess;
	}
    }

    return status;
}


static int
clip_run3(void *context)
{
    int status;
    unsigned process;

    status = Mylene1f4_parse_ctreekey
	(context, "null process", _Mylene1f4_process_root,
	 GROUND(context).futile.process, &process);
    if (!status) {
	MPEG(context).process = &process;

	status = clip_run2(context);
    }

    return status;
}


static int
eVer_clip(void *context, unsigned start2)
{
    int status;
    struct target_type *target_data;
    unsigned count;
    void *storage;

    storage = STORAGE(context);
    target_data = &TARGET(context);

    target_data->skip(storage);

    count = start2 - REFERENCE(context);
    do {
	status = line_clip(context);
	if (status) {
	    break;
	}

	count--;
    } while (count);

    if (!status) {
	target_data->seek(storage);
    }

    return status;
}


static int
line_clip(void *context)
{
    int status;

    do {
	{
	    status = TARGET(context).ever(STORAGE(context));
	    if (status) {
		break;
	    } else {
		unsigned reference;

		reference = REFERENCE(context);
		reference++;
		Mylene1f4_post_time(context, reference);
		REFERENCE(context) = reference;
	    }
	}

	if (PROGRESS2(context)) {
	    Mylene1f4_run_format(context, PROGRESS2(context));
	}
    } while (0);

    return status;
}


static int
make_link(void *context)
{
    SIZE(context) = RELEASE(context);

    return 0;
}


static int
plAy_dAta(void *context)
{
    int status = 0;
    unsigned tail;

    tail = TAIL(context);
    while (REFERENCE(context) < tail) {
	status = play_clip(context);
	if (status) {
	    break;
	}
    }

    return status;
}


static int
plAy_data(void *context)
{
    int status;

    do {
	status = play_clip(context);
    } while (!status);
    if (COMPLETED(context)) {
	status = 0;
    }

    return status;
}


static int
play_clip(void *context)
{
    int status;

    do {
	{
	    status = TARGET(context).link(STORAGE(context));
	    if (status) {
		break;
	    } else {
		unsigned reference;

		reference = REFERENCE(context);
		reference++;
		Mylene1f4_post_time(context, reference);
		REFERENCE(context) = reference;
	    }
	}

	if (PROGRESS2(context)) {
	    Mylene1f4_run_format(context, PROGRESS2(context));
	}
    } while (0);

    return status;
}


static int
play_dAta(void *context, unsigned start2)
{
    int status = 0;

    if (!(start2 < TAIL(context))) {
	REFERENCE(context) = TAIL(context);
    } else {
	COMPLETED(context) = 0;
	REFERENCE(context) = HEAD(context);

	if (REFERENCE(context) < start2) {
	    status = skip_dAta(context, start2);
	}
	if (!status) {
	    if (1) {
		status = plAy_dAta(context);
	    }
	}
    }

    return status;
}


static int
play_data(void *context, unsigned start2)
{
    int status = 0;

    COMPLETED(context) = 0;
    REFERENCE(context) = HEAD(context);

    if (REFERENCE(context) < start2) {
	status = skip_data(context, start2);
    }
    if (!status) {
	if (!COMPLETED(context)) {
	    status = plAy_data(context);
	}
    }

    return status;
}


static int
play_file(void *context)
{
    int status;

    status = Mylene1f4_init_source(context);
    if (status) {
	status = Mylene1f4_near_miss(context);
    } else {
	int excess;

	CLIP(context)++;

	if (Mylene1f4_clip_source(context)) {
	    Window(context)->skipped++;
	} else {
	    Window(context)->skipped = 0;
	    if (1) {
		status = Mylene1f4_line_source(context);
		if (status) {
		} else {
		    status = play_list(context);
		}
	    }
	}

	excess = Mylene1f4_fini_source(context);
	if (!status) {
	    status = excess;
	}
    }

    return status;
}


static int
play_list(void *context)
{
    int status = 0;
    unsigned count;

    count = COUNT(context);
    if (count) {
	unsigned i = 0;
	void *loader;

	SECOND_A(ATIME(context)) = 0;
	SECOND_F(ATIME(context)) = 0;

	REFERENCE(context) = 0;

	loader = LOADER(context);

	for (; i < count; i++) {
	    status = TARGET(context).init(loader, i, &STORAGE(context));
	    if (!status) {
		int excess;
		unsigned start2;

		TARGET(context).type(loader, &TARGET(context));

		if (PROGRESS1(context)) {
		    Mylene1f4_run_format(context, PROGRESS1(context));
		}

		Mylene1f4_fix_start(context, &start2);

		status = make_link(context);
		if (!status) {
		    if (~TAIL(context)) {
			status = play_dAta(context, start2);
		    } else {
			status = play_data(context, start2);
		    }
		    if (!status) {
			{
			    if (PROGRESS3(context)) {
				Mylene1f4_run_format
				    (context, PROGRESS3(context));
			    }
			}
		    }
		}
		excess = TARGET(context).fini(loader, i, &STORAGE(context));
		if (!status) {
		    status = excess;
		}
		if (status) {
		    break;
		}
	    }
	}
    }

    PLAY(context)++;

    return status;
}


#if __ENABLE_TAGGED_LIST__
static int
play_tile(void *context)
{
    int status;

    Mylene1f4_link_source(context);

    CLIP(context)++;

    if (Mylene1f4_clip_source(context)) {
	Window(context)->skipped++;

	status = 0;
    } else {
	Window(context)->skipped = 0;
	if (1) {
	    status = Mylene1f4_init_source(context);
	    if (status) {
		status = Mylene1f4_near_miss(context);
	    } else {
		int excess;

		status = play_list(context);

		excess = Mylene1f4_fini_source(context);
		if (!status) {
		    status = excess;
		}
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_TAGGED_LIST__ */


static int
skip_dAta(void *context, unsigned start2)
{
    int status;

    if (TAIL(context) < start2 + 1) {
	status = 1;
    } else {
	status = eVer_clip(context, start2);
    }

    return status;
}


static int
skip_data(void *context, unsigned start2)
{
    int status;

    status = eVer_clip(context, start2);
    if (status) {
	if (COMPLETED(context)) {
	    TAIL(context) = REFERENCE(context);
	    status = 0;
	}
    }

    return status;
}


int
Mylene1f4_run_excess2(void *context)
{
    int status;

    if (!GROUND(context).futile.process) {
	status = clip_run2(context);
    } else {
	status = clip_run3(context);
    }

    return status;
}
