/*
 * excess3.c
 * Copyright (C) 2005-2011, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <errno.h>
#include <stddef.h>
#include <string.h>
#include <unistd.h>

#include <sys/select.h>
#include <sys/time.h>
#include <sys/types.h>

#include <access.h>
#include <defs.h>
#include <inter.h>

#define __FIX_0fa_WAIT__		1

#define WAIT_COUNT			4

#if __ENABLE_TAGGED_LIST__
# define __PLAY_TILE__			1
#else
# define __PLAY_TILE__			0
#endif				/* __ENABLE_TAGGED_LIST__ */

#define SECOND_A(v)			((v)->tv_sec)
#define SECOND_F(v)			((v)->tv_usec)

#define Output(context) \
    ((const struct output_type *) DEVICE(context))
#define Window(context) \
    ((struct window_type *) WINDOW(context).extent.window)

#if __PLAY_TILE__
# define PLAY_FILE(context, window) \
    (window).play((context))
#else
# define PLAY_FILE(context, window) \
    play_file((context))
#endif				/* __PLAY_TILE__ */

#if !__LINK_CELL_LOADER__
# define zero_data(context, null) \
    0
#else
# define zero_data(context, null) \
    (!TARGET((context)).zero ? 0 : zero_cell((context), (null)))
#endif				/* !__LINK_CELL_LOADER__ */

typedef struct window_type {
    int (*link) (void *);
#if __PLAY_TILE__
    int (*play) (void *);
#endif				/* __PLAY_TILE__ */
    unsigned skipped;
#if __FIX_0fa_WAIT__
    unsigned wait;
#endif				/* __FIX_0fa_WAIT__ */
} window_type;

static int cLip_link(void *);
static int clip_link(void *);
static int clip_run1(void *);
static int eVer_clip(void *, unsigned);
static int line_clip(void *);
#if __LINK_CELL_LOADER__
static int pLay_clip(void *);
#endif				/* __LINK_CELL_LOADER__ */
static int plAy_dAta(void *);
static int plAy_data(void *);
static int play_clip(void *);
static int play_dAta(void *, unsigned);
static int play_data(void *, unsigned);
static int play_deck(void *, unsigned, int (*) (void *));
static int play_file(void *);
static int play_list(void *);
#if __ENABLE_TAGGED_LIST__
static int play_tile(void *);
#endif				/* __ENABLE_TAGGED_LIST__ */
static int skip_dAta(void *, unsigned);
static int skip_data(void *, unsigned);
static int wait_clip(void *);
static int wait_e1f4(void *);
#if __LINK_CELL_LOADER__
static int zero_cell(void *, unsigned);
#endif				/* __LINK_CELL_LOADER__ */

static int
cLip_link(void *context)
{
#if __LINK_CELL_LOADER__
    if (TARGET(context).zero) {
	Window(context)->link = pLay_clip;
    } else {
#endif				/* __LINK_CELL_LOADER__ */
	Window(context)->link = play_clip;
#if __LINK_CELL_LOADER__
    }
#endif				/* __LINK_CELL_LOADER__ */

    return 0;
}


static int
clip_link(void *context)
{
    int status;

    status = Mylene1f4_make_link(context);
    if (!status) {
	cLip_link(context);
    }

    return status;
}


static int
clip_run1(void *context)
{
    int start, status = 0;
    struct window_type window;

    FINI(context) = NULL;
    INIT(context) = NULL;
    LINK(context) = 0;
    SEQUEL(context) = NULL;
    STREAMS(context) = 0;
    X1f42(context) = NULL;
    WIRE(context) = 7;
    Z0fa(context) = 0;

    window.skipped = 0;

    WINDOW(context).extent.window = &window;

    start = START(context);

#if __ENABLE_TAGGED_LIST__
    if (FLAGS(context) & Mylene1f4_BREAK_LIST) {
	window.play = play_tile;
    } else {
	window.play = play_file;
    }
#endif				/* __ENABLE_TAGGED_LIST__ */

    while (1) {
	if (1) {
	    int class;

	    status = Mylene1f4_deck_list(context, &start, &class);
	    if (status) {
		break;
	    } else {
		if (class) {
		    break;
		}
	    }
	}

	while (EXCESS(context).e1.list87.clear) {
	    SOURCE(context)._.s1.path = EXCESS(context).e1.list87.file_name;
	    status = PLAY_FILE(context, window);
	    if (status) {
		break;
	    } else {
		if (window.skipped == EXCESS(context).e1.list87.count) {
		    break;
		}
	    }

	    status = Mylene1f4_slip_list(context);
	    if (status) {
		break;
	    }
	}

	if (!REPEAT(context)) {
	    break;
	} else {
	    if (EXCESS(context).e1.list87.clear) {
		break;
	    } else {
		if (LOAD_ORDER(context) == Mylene1f4_LOAD_LAST) {
		    start = -1;
		}
	    }
	}
    }

    if (!status) {
	if (PID(context) != -1) {
	    status = wait_e1f4(context);
	}
    }

    return status;
}


static int
eVer_clip(void *context, unsigned start2)
{
    int status;
    struct target_type *target_data;
    unsigned count;

    target_data = &TARGET(context);

    count = start2 - REFERENCE(context);

#if __LINK_CELL_LOADER__
    if (target_data->zero) {
	status = zero_cell(context, count);
	if (!status) {
	    REFERENCE(context) += count;
	}
    } else {
#endif				/* __LINK_CELL_LOADER__ */
	target_data->skip(STORAGE(context));

	do {
	    status = line_clip(context);
	    if (status) {
		break;
	    }

	    count--;
	} while (count);

	if (!status) {
	    target_data->seek(STORAGE(context));
	}
#if __LINK_CELL_LOADER__
    }
#endif				/* __LINK_CELL_LOADER__ */

    return status;
}


static int
line_clip(void *context)
{
    int fd_0, fd_1, pending_0fa, status;

    fd_0 = Mylene1ST(context);
    fd_1 = Mylene2ND(context);

    pending_0fa = Z0fa(context);

    while (1) {
	fd_set fd_set_0, *fd_set_0_select, fd_set_1, *fd_set_1_select;
	struct timeval *selector, timeout;
	unsigned extend = 0, nfds = 0;

	if (SIGCHLD_received(context)) {
	    status = Mylene1f4_yway_mylene0fa(context, 0);
	    if (status) {
		break;
	    } else {
		if (1) {
		    break;
		}
	    }
	}

	if (pending_0fa) {
	    fd_set_0_select = NULL;
	} else {
	    fd_set_0_select = &fd_set_0;
	    FD_ZERO(fd_set_0_select);
	    FD_SET(fd_0, fd_set_0_select);
	    nfds = fd_0;
	    extend = 1;
	}
	if (!OUTGOING(context)) {
	    fd_set_1_select = NULL;
	} else {
	    fd_set_1_select = &fd_set_1;
	    FD_ZERO(fd_set_1_select);
	    FD_SET(fd_1, fd_set_1_select);
	    if (nfds < fd_1) {
		nfds = fd_1;
	    }
	    extend = 1;
	}

	if (1) {
	    selector = &timeout;
	    timeout.tv_sec = 0;
	    timeout.tv_usec = 0;
	}

	nfds += extend;

	status = select
	    (nfds, fd_set_0_select, fd_set_1_select, NULL, selector);
	if (status == -1) {
	    if (errno != EINTR) {
		Mylene1f4_fprintf_error
		    (context, "%s: failed select: %s\n", Mylene1f4(context),
		     strerror(errno));
		break;
	    }
	} else {
	    status = 0;

	    if (fd_set_0_select) {
		if (FD_ISSET(fd_0, fd_set_0_select)) {
		    pending_0fa = 1;
		}
	    }
	    if (pending_0fa) {
		if (1) {
		    status = Mylene1f4_read_e0fa(context, 0);
		    if (status) {
			break;
		    }
		}
	    }

	    if (fd_set_1_select) {
		if (FD_ISSET(fd_1, fd_set_1_select)) {
		    status = Mylene1f4_push_mylene0fa(context);
		    if (status) {
			break;
		    }
		}
	    }

	    break;
	}
    }
    if (status) {
    } else {
	do {
	    if (1) {
		status = TARGET(context).ever(STORAGE(context));
		if (status) {
		    break;
		} else {
		    unsigned reference;

		    reference = REFERENCE(context);
		    reference++;
		    Mylene1f4_post_time(context, reference);
		    REFERENCE(context) = reference;
		}
	    }

	    if (PROGRESS2(context)) {
		Mylene1f4_run_format(context, PROGRESS2(context));
	    }
	} while (0);
    }

    return status;
}


#if __LINK_CELL_LOADER__
static int
pLay_clip(void *context)
{
    int do_decode, fd_0, fd_1, pending_0fa, status;
    unsigned wait_d = 0;

    fd_0 = Mylene1ST(context);
    fd_1 = Mylene2ND(context);

    pending_0fa = Z0fa(context);

    if (LINK(context)) {
	do_decode = 0;
    } else {
	unsigned cache;

	cache = C1f4(context) - STATE(context)[1];
	if (SIZE(context) < cache) {
	    cache += SIZE(context);
	}
	do {
	    cache = SIZE(context) - cache;
	    if (RELEASE(context) < cache) {
		cache = STATE(context)[Mylene1f4_CELL_STATE] - C1f4(context);
		if (SIZE(context) < cache) {
		    cache += SIZE(context);
		}
		if (cache < RELEASE(context)) {
		} else {
		    do_decode = 1;
		    wait_d = 0;
		    break;
		}
	    }
	    if (pending_0fa) {
		do_decode = 0;
	    } else if (WIRE(context)) {
		do_decode = 0;
	    } else {
# if __FIX_0fa_WAIT__
		wait_d = Window(context)->wait;
# else
#  if SIZEOF_UNSIGNED_LONG == 8
		unsigned long wait;

		wait = RELEASE(context) * WAIT_COUNT - cache;
		wait *= 1000000;
		wait /= (SAMPLING(context) * CHANNELS(context) << 1);
#  else
		double wait;

		wait = RELEASE(context) * WAIT_COUNT - cache;
		wait /= (SAMPLING(context) * CHANNELS(context) << 1);
		wait *= 1000000;
#  endif				/* SIZEOF_UNSIGNED_LONG == 8 */
		wait_d = wait;
		if (131072 < wait_d) {
		    wait_d = 131072;
		}
# endif				/* __FIX_0fa_WAIT__ */

		do_decode = 2;
	    }
	} while (0);
    }

    while (1) {
	fd_set fd_set_0, *fd_set_0_select, fd_set_1, *fd_set_1_select;
	struct timeval *selector, timeout;
	unsigned extend = 0, nfds = 0;

	if (SIGCHLD_received(context)) {
	    status = Mylene1f4_yway_mylene0fa(context, 0);
	    if (status) {
		break;
	    } else {
		do_decode = 0;
		if (1) {
		    break;
		}
	    }
	}

	if (pending_0fa) {
	    fd_set_0_select = NULL;
	} else {
	    fd_set_0_select = &fd_set_0;
	    FD_ZERO(fd_set_0_select);
	    FD_SET(fd_0, fd_set_0_select);
	    nfds = fd_0;
	    extend = 1;
	}
	if (!OUTGOING(context)) {
	    fd_set_1_select = NULL;
	} else {
	    fd_set_1_select = &fd_set_1;
	    FD_ZERO(fd_set_1_select);
	    FD_SET(fd_1, fd_set_1_select);
	    if (nfds < fd_1) {
		nfds = fd_1;
	    }
	    extend = 1;
	}

	if (pending_0fa) {
	    selector = &timeout;
	    timeout.tv_sec = 0;
	    timeout.tv_usec = 0;
	} else {
	    if (!do_decode) {
		selector = NULL;
	    } else {
		selector = &timeout;
		timeout.tv_sec = 0;
		timeout.tv_usec = wait_d;
	    }
	}

	nfds += extend;

	status = select
	    (nfds, fd_set_0_select, fd_set_1_select, NULL, selector);
	if (status == -1) {
	    if (errno != EINTR) {
		Mylene1f4_fprintf_error
		    (context, "%s: failed select: %s\n", Mylene1f4(context),
		     strerror(errno));
		break;
	    }
	} else {
	    status = 0;

	    if (fd_set_0_select) {
		if (FD_ISSET(fd_0, fd_set_0_select)) {
		    pending_0fa = 1;
		}
	    }
	    if (pending_0fa) {
		if (1) {
		    status = Mylene1f4_read_e0fa(context, 0);
		    if (status) {
			break;
		    }
		}
	    }

	    if (fd_set_1_select) {
		if (FD_ISSET(fd_1, fd_set_1_select)) {
		    status = Mylene1f4_push_mylene0fa(context);
		    if (status) {
			break;
		    }
		}
	    }

	    break;
	}
    }
    if (status) {
    } else {
	do {
	    if (do_decode == 1) {
		if (0) {
		} else {
		    unsigned e1f4_index;

		    e1f4_index = C1f4(context);
		    e1f4_index += RELEASE(context);
		    if (e1f4_index == SIZE(context)) {
			e1f4_index = 0;
		    }
		    C1f4(context) = e1f4_index;
		    STATE(context)[0] = e1f4_index;

		    REFERENCE(context)++;
		}
	    }

	    if (PROGRESS2(context)) {
		Mylene1f4_run_format(context, PROGRESS2(context));
	    }
	} while (0);
    }

    return status;
}
#endif				/* __LINK_CELL_LOADER__ */


static int
plAy_dAta(void *context)
{
    int status = 0;
    unsigned tail;

    tail = TAIL(context);
    while (REFERENCE(context) < tail) {
	status = Window(context)->link(context);
	if (status) {
	    break;
	}
    }
    if (!status) {
	COMPLETED(context) = 1;
    }

    return status;
}


static int
plAy_data(void *context)
{
    int status;

    do {
	status = Window(context)->link(context);
    } while (!status);
    if (COMPLETED(context)) {
	status = 0;
    }

    return status;
}


static int
play_clip(void *context)
{
    int do_decode, fd_0, fd_1, pending_0fa, status;
    unsigned wait_d = 0;

    fd_0 = Mylene1ST(context);
    fd_1 = Mylene2ND(context);

    pending_0fa = Z0fa(context);

    if (LINK(context)) {
	do_decode = 0;
    } else {
	unsigned cache;

	cache = C1f4(context) - STATE(context)[1];
	if (SIZE(context) < cache) {
	    cache += SIZE(context);
	}
	{
	    cache = SIZE(context) - cache;
	    if (RELEASE(context) < cache) {
		do_decode = 1;
		wait_d = 0;
	    } else if (pending_0fa) {
		do_decode = 0;
	    } else if (WIRE(context)) {
		do_decode = 0;
	    } else {
#if __FIX_0fa_WAIT__
		wait_d = Window(context)->wait;
#else
# if SIZEOF_UNSIGNED_LONG == 8
		unsigned long wait;

		wait = RELEASE(context) * WAIT_COUNT - cache;
		wait *= 1000000;
		wait /= (SAMPLING(context) * CHANNELS(context) << 1);
# else
		double wait;

		wait = RELEASE(context) * WAIT_COUNT - cache;
		wait /= (SAMPLING(context) * CHANNELS(context) << 1);
		wait *= 1000000;
# endif					/* SIZEOF_UNSIGNED_LONG == 8 */
		wait_d = wait;
		if (131072 < wait_d) {
		    wait_d = 131072;
		}
#endif				/* __FIX_0fa_WAIT__ */

		do_decode = 2;
	    }
	}
    }

    while (1) {
	fd_set fd_set_0, *fd_set_0_select, fd_set_1, *fd_set_1_select;
	struct timeval *selector, timeout;
	unsigned extend = 0, nfds = 0;

	if (SIGCHLD_received(context)) {
	    status = Mylene1f4_yway_mylene0fa(context, 0);
	    if (status) {
		break;
	    } else {
		do_decode = 0;
		if (1) {
		    break;
		}
	    }
	}

	if (pending_0fa) {
	    fd_set_0_select = NULL;
	} else {
	    fd_set_0_select = &fd_set_0;
	    FD_ZERO(fd_set_0_select);
	    FD_SET(fd_0, fd_set_0_select);
	    nfds = fd_0;
	    extend = 1;
	}
	if (!OUTGOING(context)) {
	    fd_set_1_select = NULL;
	} else {
	    fd_set_1_select = &fd_set_1;
	    FD_ZERO(fd_set_1_select);
	    FD_SET(fd_1, fd_set_1_select);
	    if (nfds < fd_1) {
		nfds = fd_1;
	    }
	    extend = 1;
	}

	if (pending_0fa) {
	    selector = &timeout;
	    timeout.tv_sec = 0;
	    timeout.tv_usec = 0;
	} else {
	    if (!do_decode) {
		selector = NULL;
	    } else {
		selector = &timeout;
		timeout.tv_sec = 0;
		timeout.tv_usec = wait_d;
	    }
	}

	nfds += extend;

	status = select
	    (nfds, fd_set_0_select, fd_set_1_select, NULL, selector);
	if (status == -1) {
	    if (errno != EINTR) {
		Mylene1f4_fprintf_error
		    (context, "%s: failed select: %s\n", Mylene1f4(context),
		     strerror(errno));
		break;
	    }
	} else {
	    status = 0;

	    if (fd_set_0_select) {
		if (FD_ISSET(fd_0, fd_set_0_select)) {
		    pending_0fa = 1;
		}
	    }
	    if (pending_0fa) {
		if (1) {
		    status = Mylene1f4_read_e0fa(context, 0);
		    if (status) {
			break;
		    }
		}
	    }

	    if (fd_set_1_select) {
		if (FD_ISSET(fd_1, fd_set_1_select)) {
		    status = Mylene1f4_push_mylene0fa(context);
		    if (status) {
			break;
		    }
		}
	    }

	    break;
	}
    }
    if (status) {
    } else {
	do {
	    if (do_decode == 1) {
		status = TARGET(context).link(STORAGE(context));
		if (status) {
		    break;
		} else {
		    unsigned e1f4_index;

		    e1f4_index = C1f4(context);
		    e1f4_index += RELEASE(context);
		    if (e1f4_index == SIZE(context)) {
			e1f4_index = 0;
		    }
		    C1f4(context) = e1f4_index;
		    STATE(context)[0] = e1f4_index;

		    REFERENCE(context)++;
		}
	    }

	    if (PROGRESS2(context)) {
		Mylene1f4_run_format(context, PROGRESS2(context));
	    }
	} while (0);
    }

    return status;
}


static int
play_dAta(void *context, unsigned start2)
{
    int status = 0;

    if (!(start2 < TAIL(context))) {
	REFERENCE(context) = TAIL(context);
    } else {
	COMPLETED(context) = 0;
	REFERENCE(context) = HEAD(context);

	if (REFERENCE(context) < start2) {
	    status = skip_dAta(context, start2);
	} else {
	    status = zero_data(context, 0);
	}
	if (!status) {
	    status = play_deck(context, start2, plAy_dAta);
	}
    }

    return status;
}


static int
play_data(void *context, unsigned start2)
{
    int status;

    COMPLETED(context) = 0;
    REFERENCE(context) = HEAD(context);

    if (REFERENCE(context) < start2) {
	status = skip_data(context, start2);
    } else {
	status = zero_data(context, 0);
    }
    if (!status) {
	if (!COMPLETED(context)) {
	    status = play_deck(context, start2, plAy_data);
	}
    }

    return status;
}


static int
play_deck(void *context, unsigned start2, int (*deck) (void *))
{
    int status;

    Mylene1f4_scale_track(context);

    status = Mylene1f4_init_link(context, start2);
    if (!status) {
	status = deck(context);
	if (!status) {
	    int excess;

	    if (COMPLETED(context)) {
		Mylene1f4_shift_track(context);
	    }

	    excess = Mylene1f4_fini_link(context);
	    if (!status) {
		status = excess;
	    }
	}
    }

    return status;
}


static int
play_file(void *context)
{
    int status;

    status = Mylene1f4_init_source(context);
    if (status) {
	status = Mylene1f4_near_miss(context);
    } else {
	int excess;

	CLIP(context)++;

	if (Mylene1f4_clip_source(context)) {
	    Window(context)->skipped++;
	} else {
	    Window(context)->skipped = 0;
	    if (1) {
		status = Mylene1f4_line_source(context);
		if (status) {
		} else {
		    status = play_list(context);
		}
	    }
	}

	excess = Mylene1f4_fini_source(context);
	if (!status) {
	    status = excess;
	}
    }

    return status;
}


static int
play_list(void *context)
{
    int status = 0;
    unsigned count;

    count = COUNT(context);
    if (count) {
	unsigned i = 0;
	void *loader;

	SECOND_A(ATIME(context)) = 0;
	SECOND_F(ATIME(context)) = 0;

	REFERENCE(context) = 0;

	loader = LOADER(context);

	for (; i < count; i++) {
	    status = TARGET(context).init(loader, i, &STORAGE(context));
	    if (!status) {
		int excess;
		unsigned start2;

		TARGET(context).type(loader, &TARGET(context));

#if __FIX_0fa_WAIT__
		Mylene1f4_miss_wait
		    (context, WAIT_COUNT, &Window(context)->wait);
#endif				/* __FIX_0fa_WAIT__ */

		if (PROGRESS1(context)) {
		    Mylene1f4_run_format(context, PROGRESS1(context));
		}

		Mylene1f4_fix_start(context, &start2);

		if (1) {
		    status = clip_link(context);
		}
		if (!status) {
		    if (~TAIL(context)) {
			status = play_dAta(context, start2);
		    } else {
			status = play_data(context, start2);
		    }
		    if (!status) {
			status = wait_e1f4(context);

			if (!status) {
			    void *progressl;

			    progressl = PROGRESS2(context);
			    if (progressl) {
			        Mylene1f4_run_format(context, progressl);
			    }
			    progressl = PROGRESS3(context);
			    if (progressl) {
			        Mylene1f4_run_format(context, progressl);
			    }
			}
		    }
		}
		excess = TARGET(context).fini(loader, i, &STORAGE(context));
		if (!status) {
		    status = excess;
		}
		if (status) {
		    break;
		}
	    }
	}
    }

    PLAY(context)++;

    return status;
}


#if __ENABLE_TAGGED_LIST__
static int
play_tile(void *context)
{
    int status;

    Mylene1f4_link_source(context);

    CLIP(context)++;

    if (Mylene1f4_clip_source(context)) {
	Window(context)->skipped++;

	status = 0;
    } else {
	Window(context)->skipped = 0;
	if (1) {
	    status = Mylene1f4_init_source(context);
	    if (status) {
		status = Mylene1f4_near_miss(context);
	    } else {
		int excess;

		status = play_list(context);

		excess = Mylene1f4_fini_source(context);
		if (!status) {
		    status = excess;
		}
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_TAGGED_LIST__ */


static int
skip_dAta(void *context, unsigned start2)
{
    int status;

    if (TAIL(context) < start2 + 1) {
	status = 1;
    } else {
	status = eVer_clip(context, start2);
    }

    return status;
}


static int
skip_data(void *context, unsigned start2)
{
    int status;

    status = eVer_clip(context, start2);
    if (status) {
	if (COMPLETED(context)) {
	    TAIL(context) = REFERENCE(context);
	    status = 0;
	}
    }

    return status;
}


static int
wait_clip(void *context)
{
    int fd_0, fd_1, pending_0fa, status;
    unsigned wait_2 = 0;
    void *progress2;

    progress2 = PROGRESS2(context);

    fd_0 = Mylene1ST(context);
    fd_1 = Mylene2ND(context);

    pending_0fa = Z0fa(context);

    if (pending_0fa) {
    } else if (progress2) {
#if __FIX_0fa_WAIT__
	wait_2 = Window(context)->wait;
#else
# if SIZEOF_UNSIGNED_LONG == 8
	unsigned long wait;

	wait = RELEASE(context) * WAIT_COUNT;
	wait *= 1000000;
	wait /= (SAMPLING(context) * CHANNELS(context) << 1);
# else
	double wait;

	wait = RELEASE(context) * WAIT_COUNT;
	wait /= (SAMPLING(context) * CHANNELS(context) << 1);
	wait *= 1000000;
# endif					/* SIZEOF_UNSIGNED_LONG == 8 */
	wait_2 = wait;
	if (131072 < wait_2) {
	    wait_2 = 131072;
	}
#endif				/* __FIX_0fa_WAIT__ */
    }

    while (1) {
	fd_set fd_set_0, *fd_set_0_select, fd_set_1, *fd_set_1_select;
	struct timeval *selector, timeout;
	unsigned extend = 0, nfds = 0;

	if (SIGCHLD_received(context)) {
	    status = Mylene1f4_yway_mylene0fa(context, 0);
	    if (status) {
		break;
	    } else {
		if (1) {
		    break;
		}
	    }
	}

	if (pending_0fa) {
	    fd_set_0_select = NULL;
	} else {
	    fd_set_0_select = &fd_set_0;
	    FD_ZERO(fd_set_0_select);
	    FD_SET(fd_0, fd_set_0_select);
	    nfds = fd_0;
	    extend = 1;
	}
	if (!OUTGOING(context)) {
	    fd_set_1_select = NULL;
	} else {
	    fd_set_1_select = &fd_set_1;
	    FD_ZERO(fd_set_1_select);
	    FD_SET(fd_1, fd_set_1_select);
	    if (nfds < fd_1) {
		nfds = fd_1;
	    }
	    extend = 1;
	}

	if (pending_0fa) {
	    selector = &timeout;
	    timeout.tv_sec = 0;
	    timeout.tv_usec = 0;
	} else {
	    if (!progress2) {
		selector = NULL;
	    } else {
		selector = &timeout;
		timeout.tv_sec = 0;
		timeout.tv_usec = wait_2;
	    }
	}

	nfds += extend;

	status = select
	    (nfds, fd_set_0_select, fd_set_1_select, NULL, selector);
	if (status == -1) {
	    if (errno != EINTR) {
		Mylene1f4_fprintf_error
		    (context, "%s: failed select: %s\n", Mylene1f4(context),
		     strerror(errno));
		break;
	    }
	} else {
	    status = 0;

	    if (fd_set_0_select) {
		if (FD_ISSET(fd_0, fd_set_0_select)) {
		    pending_0fa = 1;
		}
	    }
	    if (pending_0fa) {
		if (1) {
		    status = Mylene1f4_read_e0fa(context, 0);
		    if (status) {
			break;
		    }
		}
	    }

	    if (fd_set_1_select) {
		if (FD_ISSET(fd_1, fd_set_1_select)) {
		    status = Mylene1f4_push_mylene0fa(context);
		    if (status) {
			break;
		    }
		}
	    }

	    break;
	}
    }
    if (status) {
    } else {
	if (1) {
	    if (1) {
		if (progress2) {
		    Mylene1f4_run_format(context, progress2);
		}
	    }
	}
    }

    return status;
}


static int
wait_e1f4(void *context)
{
    int status = 0;

    while (STREAMS(context) & (1u << 31)) {
	status = wait_clip(context);
	if (status) {
	    break;
	}
    }

    return status;
}


#if __LINK_CELL_LOADER__
static int
zero_cell(void *context, unsigned count2)
{
    int status;

    status = TARGET(context).zero(STORAGE(context), count2);

    return status;
}
#endif				/* __LINK_CELL_LOADER__ */


int
Mylene1f4_run_excess3(void *context)
{
    int status;

    status = Output(context)->init(context);
    if (!status) {
	int excess;

	status = clip_run1(context);
	if (status) {
	    Output(context)->kill(context);
	}
	excess = Output(context)->fini(context);
	if (!status) {
	    status = excess;
	}
    }

    return status;
}
