/*
 * excess7.c
 * Copyright (C) 2005-2015, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * TODO
 *
 *.page jump request for multiple entries targets
 *.time jump request for multiple entries targets
 *.redo clip_{deck,none} (put some do-while-0 for EQUEUE)
 *
 *.TODO.amplification (there is comment down below tagged like this)
 *.if non mapped stream completes (sets COMPLETED) a rerun has to consider
 * the already found tail.
 */
 
#include <config.h>

#if __ENABLE_CONTROLLING_SERVER__

#include <errno.h>
#include <fcntl.h>
#if defined HAVE_LIBLEWD
# include <lewd.h>
#endif				/* HAVE_LIBLEWD */
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <arpa/inet.h>

#include <netinet/in.h>

#include <sys/select.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/un.h>

#include <access.h>
#if !defined HAVE_LIBLEWD
# include <cardinal-wx.h>
#endif				/* !HAVE_LIBLEWD */
#include <excess4.h>
#include <excess7.h>
#include <defs.h>
#include <inter.h>
#if __ENABLE_EQUEUE__
# include <sfnote.h>
#endif				/* __ENABLE_EQUEUE__ */
#include <types.h>

#define __USE_SHUTDOWN__		1

#define WAIT_COUNT			4

#define f4_LOAD_NEXT			Mylene1f4_LOAD_NEXT
#define f4_LOAD_LAST			Mylene1f4_LOAD_LAST

#define swap_data			Mylene1f4_swap_graphique
#define swap_deck			Mylene1f4_swap_filter
#define swap_miss			Mylene1f4_swap_progress

#define PLAY_LINE			0
#define SEEK_LINE			1

#define SECOND_A(v)			((v)->tv_sec)
#define SECOND_F(v)			((v)->tv_usec)

#define SECOND_a(v)			((v).tv_sec)
#define SECOND_f(v)			((v).tv_usec)

#define false(e)			0
#define true(e)				1

#define slip_miss(context)		0

#define Frame4(context) \
    ((struct frame4_type *) &Frame7((context))->_1.frame4)
#define Frame7(context) \
    ((struct frame7_type *) WINDOW((context)).server.window)
#define Output(context) \
    ((const struct output_type *) DEVICE(context))

#if __EDITABLE_PLAYLIST__
# define POST_LIST(context) \
    clip_list((context))
#else
# define POST_LIST(context) \
    post_list((context))
#endif				/* __EDITABLE_PLAYLIST__ */

#if __PLAY_TILE__
# define PLAY_LIST(context) \
    Frame7((context))->_2.ceil((context))
#else
# define PLAY_LIST(context) \
    play_dana((context))
#endif				/* __PLAY_TILE__ */

#if __PLAY_TILE__
# define PLAY_FILE(context) \
    Frame7((context))->_2.play((context))
#else
# define PLAY_FILE(context) \
    play_file((context))
#endif				/* __PLAY_TILE__ */

#if !__LINK_CELL_LOADER__
# define zero_data(context, null, clear) \
    0
#else
# define zero_data(context, null, clear) \
    (!TARGET((context)).zero ? 0 : zero_cell((context), (null), (clear)))
#endif				/* !__LINK_CELL_LOADER__ */

#if !__ENABLE_EQUEUE__
# define CLIP_STRIPE(context)
#else
# define CLIP_STRIPE(context) \
    if (Frame4((context))->clip_stripe) {				      \
	Mylene1f4_port_aime						      \
	    ((context), &Frame4((context))->clip_stripe,		      \
	     Mylene1f4_CLIP_LIST);					      \
    }
#endif				/* !__ENABLE_EQUEUE__ */

#if !__ENABLE_EQUEUE__
# define PLAY_STRIPE(context)
#else
# define PLAY_STRIPE(context) \
    if (Frame4((context))->play_stripe) {				      \
	Mylene1f4_port_aime						      \
	    ((context), &Frame4((context))->play_stripe,		      \
	     Mylene1f4_PLAY_LIST);					      \
    }
#endif				/* !__ENABLE_EQUEUE__ */

typedef struct flush7_type {
    unsigned count, rerun, speed, state;
} flush7_type;

extern const struct class7_type _Mylene1f4_class7;

static int back_port(void *, unsigned);
static int bind_hack(void *, int, unsigned);
static int bind_inet(void *, int, unsigned);
static int bind_unix(void *, int);
static int cLip_liNk(void *);
static int cLip_link(void *);
static int case_inet(void *, unsigned);
static int case_lane(void *, int);
static int case_less(void *, int);
static int case_link(void *, int);
static int case_mode(void *);
static int case_path(void *);
static int case_port(void *);
static int case_unix(void *, unsigned);
static int case_user(void *);
static int clip_deck(void *);
static int clip_liNk(void *);
static int clip_link(void *);
#if __EDITABLE_PLAYLIST__
static int clip_list(void *);
#endif				/* __EDITABLE_PLAYLIST__ */
static int clip_none(void *);
static int clip_run1(void *);
static int clip_text(void *);
static int eVer_clip(void *, unsigned, int *);
static int fini_port(void *, unsigned);
static int head_port(void *, unsigned);
static int init_port(void *, unsigned, int);
static int line_clip(void *);
static int line_deck(void *, unsigned, int *, void *);
static int line_e1f4(void *, unsigned *);
static int line_flow(void *);
static int line_list(void *);
static int line_seek(void *);
static int lose_path(void *);
static int peak_list(void *);
#if __LINK_CELL_LOADER__
static int pLay_clip(void *);
#endif				/* __LINK_CELL_LOADER__ */
static int plAy_dAta(void *);
static int plAy_data(void *);
static int play_aime(void *);
#if __ENABLE_TAGGED_LIST__
static int play_ceil(void *);
#endif				/* __ENABLE_TAGGED_LIST__ */
static int play_clIp(void *);
static int play_clip(void *);
static int play_dAta(void *, unsigned);
static int play_dana(void *);
static int play_data(void *, unsigned);
static int play_deck(void *, unsigned, int (*) (void *));
static int play_file(void *);
static int play_list(void *);
#if __ENABLE_TAGGED_LIST__
static int play_tile(void *);
#endif				/* __ENABLE_TAGGED_LIST__ */
static int post_list(void *);
static int read_e0fa(void *);
static int sign_list(void *);
static int skip_dAta(void *, unsigned, int *);
static int skip_data(void *, unsigned, int *);
static int slip_e1f4(void *);
static int slip_e3e8(void *);
static int stat_e3e8(void *, const char *);
static int stat_e7d0(void *, const char *);
static int stat_link(void *);
static int stat_net1(void *, const char *);
static int stat_net2(void *, const char *);
static int stat_net4(void *, const char *);
static int swap_neWt(void *);
static int swap_newt(void *);
static int swap_teXt(void *);
static int swap_text(void *);
static int wait_clip(void *);
static int wait_e1f4(void *);
static int wait_e3e8(void *);
#if __LINK_CELL_LOADER__
static int zero_cell(void *, unsigned, int *);
#endif				/* __LINK_CELL_LOADER__ */

static int
back_port(void *context, unsigned index)
{
    int status;
    struct target_type *target_data;
    unsigned select, start2;
    void *loader;

    loader = LOADER(context);

    select = SELECT(context);
    start2 = START2(context);

    SELECT(context) = Mylene1f4_PAGE_PICKUP;
    START2(context) = Frame4(context)->next.deck;

    target_data = &TARGET(context);
    status = target_data->init(LOADER(context), index, &STORAGE(context));
    if (!status) {
	int (*back) (void *);

	target_data->type(loader, target_data);

	back = target_data->back;
	if (back) {
	    status = back(STORAGE(context));
	    if (status) {
		target_data->fini(LOADER(context), index, &STORAGE(context));
	    }
	}
    }

    START2(context) = start2;
    SELECT(context) = select;

    return status;
}


static int
bind_hack(void *context, int server, unsigned port)
{
    int status = 1;

    status = setsockopt
	(server, SOL_SOCKET, SO_REUSEADDR, &status, sizeof(int));
    if (status) {
	Mylene1f4_fprintf_error
	    (context, "%s: cannot set \"reuse address\" option for the INET "
	     "%d server\n", Mylene1f4(context), port);
    }

    return status;
}


static int
bind_inet(void *context, int server, unsigned port)
{
    int status;
    struct sockaddr_in address;

    if (WINDOW(context).server.setup & Mylene1f4_REUSE_RECORD) {
	bind_hack(context, server, port);
    }

    address.sin_family = AF_INET;
    address.sin_port = htons(port);
    address.sin_addr.s_addr = htonl(INADDR_ANY);
    status = bind(server, (struct sockaddr *) &address, sizeof(address));

    return status;
}


static int
bind_unix(void *context, int server)
{
    int status;
    struct sockaddr_un address;

    memset(&address, 0, sizeof(struct sockaddr_un));
    address.sun_family = AF_UNIX;
    strcpy(address.sun_path, WINDOW(context).server.port);
    status = bind(server, (struct sockaddr *) &address, sizeof(address));

    return status;
}


static int
cLip_liNk(void *context)
{
#if __LINK_CELL_LOADER__
    if (TARGET(context).zero) {
	/*
	 * TODO.amplification
	 */
/*	Frame4(context)->play = pLay_clIp;*/
    } else {
#endif				/* __LINK_CELL_LOADER__ */
	Frame4(context)->play = play_clIp;
#if __LINK_CELL_LOADER__
    }
#endif				/* __LINK_CELL_LOADER__ */

    return 0;
}


static int
cLip_link(void *context)
{
#if __LINK_CELL_LOADER__
    if (TARGET(context).zero) {
	Frame4(context)->play = pLay_clip;
    } else {
#endif				/* __LINK_CELL_LOADER__ */
	Frame4(context)->play = play_clip;
#if __LINK_CELL_LOADER__
    }
#endif				/* __LINK_CELL_LOADER__ */

    return 0;
}


static int
case_inet(void *context, unsigned port)
{
    int server, status = 1;

    server = socket(PF_INET, SOCK_STREAM, 0);
    if (server == -1) {
	stat_net2(context, "create");
    } else {
	if (!(server < FD_SETSIZE)) {
	    stat_net1(context, "select");
	} else {
	    status = bind_inet(context, server, port);
	    if (status) {
		stat_net2(context, "bind");
	    } else {
		if (1) {
		    status = listen(server, 1);
		    if (status) {
			stat_net2(context, "listen");
		    } else {
			status = case_lane(context, server);
		    }
		}
	    }
	}

	/*
	 * BUGS
	 *
	 *.well, hmm, `server' should have been passed as reference and
	 * stuff...
	 */
	if (server != -1) {
#if __USE_SHUTDOWN__
	    shutdown(server, SHUT_RDWR);
#endif				/* __USE_SHUTDOWN__ */
	    close(server);
	}
    }

    return status;
}


static int
case_lane(void *context, int server)
{
    int status;

    status = Output(context)->init(context);
    if (status) {
    } else {
	int excess;

	status = case_less(context, server);
	if (status) {
	    Output(context)->kill(context);
	}
	excess = Output(context)->fini(context);
	if (status) {
	} else {
	    status = excess;
	}
    }

    return status;
}


static int
case_less(void *context, int server)
{
    int status;

    do {
	if (WINDOW(context).server.setup & Mylene1f4_PAUSE_SERVER) {
	    status = Mylene1f4_send_mylene0fa(context, 2, NULL);
	}

	status = case_link(context, server);
    } while (0);

    return status;
}


static int
case_link(void *context, int server)
{
    int status;
    struct frame7_type frame7;

    frame7._1.frame4.server = server;

    frame7._1.frame4.context = context;

    frame7._1.frame4.already = 0;
    frame7._1.frame4.request = 0;
    frame7._1.frame4.vertige = 0;

    frame7._1.frame4.skipped = 0;

    frame7._1.frame4.client = -1;

    frame7._1.frame4.flags = 0;

#if __ENABLE_CHANNEL_SERVER__
    frame7._1.frame4.client_up = 1;
#endif				/* __ENABLE_CHANNEL_SERVER__ */

#if __ENABLE_EQUEUE__
    frame7._1.frame4.dana.name = NULL;
#endif				/* __ENABLE_EQUEUE__ */

    frame7._1.frame4.back.slack.data = NULL;

    frame7._1.frame4.select = SELECT(context);
    if (frame7._1.frame4.select == Mylene1f4_PAGE_PICKUP) {
	frame7._1.frame4.pickup.page = START2(context);
    } else {
	frame7._1.frame4.pickup.time = START6(context);
    }

    frame7._1.frame4.i144.post = ~0;
    frame7._1.frame4.i144.sign = Mylene1f4_LOAD_NEXT;

#if __EDITABLE_PLAYLIST__
    frame7._1.frame4.weave = EXCESS(context).e1.list87.count;
#endif				/* __EDITABLE_PLAYLIST__ */

#if __ENABLE_TAGGED_LIST__
    if (FLAGS(context) & Mylene1f4_BREAK_LIST) {
	frame7._2.ceil = play_ceil;
	frame7._2.play = play_tile;
    } else {
	frame7._2.ceil = play_dana;
	frame7._2.play = play_file;
    }
#endif				/* __ENABLE_TAGGED_LIST__ */

#if __RESTART_Mylene0fa__
    WINDOW(context).server.attached_i = SIZE(context) << 1;
#endif				/* __RESTART_Mylene0fa__ */

    WINDOW(context).server.window = &frame7;

#if __ENABLE_EQUEUE__
    Mylene1f4_xset_e4lx(context);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
    Mylene1f4_xset_aime(context);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
    Mylene1f4_track_aime(context);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
    Mylene1f4_reset_aime(context);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
    Mylene1f4_setup_aime(context);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
    if (x1f4_init_sfnote(&frame7._1.frame4.line.states, 0, NULL)) {
	frame7._1.frame4.line.states = NULL;
    }
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
    Mylene1f4_init_extent(context, &frame7._1.frame4.play_stripe);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
    Mylene1f4_init_extent(context, &frame7._1.frame4.clip_stripe);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
    Mylene1f4_init_equeue(context);
#endif				/* __ENABLE_EQUEUE__ */

    status = clip_run1(context);

    if (frame7._1.frame4.client != -1) {
	Mylene1f4_lose_e3e8(context);
    }

#if __ENABLE_EQUEUE__
    Mylene1f4_fini_equeue(context);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
    Mylene1f4_fini_extent(context, &frame7._1.frame4.clip_stripe);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
    Mylene1f4_fini_extent(context, &frame7._1.frame4.play_stripe);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
    if (frame7._1.frame4.line.states) {
	x1f4_fini_sfnote(&frame7._1.frame4.line.states);
    }
#endif				/* __ENABLE_EQUEUE__ */

    /*
     * FEATURES
     *
     * objects destroyed by _clear_aime, _class_aime and _break_aime are
     * referred by the _fini_equeue and _fini_extent functions, hence the
     * _?????_aime functions must be called some time after
     */

#if __ENABLE_EQUEUE__
    Mylene1f4_clear_aime(context);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
    Mylene1f4_class_aime(context);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
    Mylene1f4_break_aime(context);
#endif				/* __ENABLE_EQUEUE__ */

    /*
     * FEATURES
     *
     * objects destroyed by _wind_aime are referred by the _fini_equeue,
     * _fini_extent, _clear_aime and _class_aime functions, hence _wind_aime
     * must be called some time after
     */

#if __ENABLE_EQUEUE__
    Mylene1f4_wind_aime(context);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
    Mylene1f4_wind_e4lx(context);
#endif				/* __ENABLE_EQUEUE__ */

    SELECT(context) = frame7._1.frame4.select;
    if (frame7._1.frame4.select == Mylene1f4_PAGE_PICKUP) {
	START2(context) = frame7._1.frame4.pickup.page;
    } else {
	START6(context) = frame7._1.frame4.pickup.time;
    }

    return status;
}


static int
case_mode(void *context)
{
    int status;

    if (!WINDOW(context).server.mode) {
	status = case_unix(context, 00600);
    } else {
	status = case_user(context);
    }

    return status;
}


static int
case_path(void *context)
{
    int status = 0;

    do {
	{
	    struct sockaddr_un address;
	    unsigned length;

	    length = strlen(WINDOW(context).server.port);
	    if (!(length < sizeof(address.sun_path))) {
		Mylene1f4_fprintf_error
		    (context, "%s: specified path (`%s') is too long (%u byt"
		     "es long, %u bytes too long)\n", Mylene1f4(context),
		     WINDOW(context).server.port, length,
		     length + 1 - sizeof(address.sun_path));

		status = 1;

		break;
	    }
	}

	status = case_mode(context);
    } while (0);

    return status;
}


static int
case_port(void *context)
{
    int status;
    unsigned port;

    status =
	x1f4_parse_wxcardinal(&port, WINDOW(context).server.port, NULL, 0);
    if (status) {
	Mylene1f4_fprintf_error
	    (context, "%s: cannot parse port specification: `%s'\n",
	     Mylene1f4(context), WINDOW(context).server.port);
    } else if (65535 < port) {
	Mylene1f4_fprintf_error
	    (context, "%s: port specification (`%u') is outside valid range "
	     "(0 .. 65535)\n", Mylene1f4(context), port);
    } else {
	status = case_inet(context, port);
    }

    return status;
}


static int
case_unix(void *context, unsigned mode)
{
    int server, status = 1;

    server = socket(AF_UNIX, SOCK_STREAM, 0);
    if (server == -1) {
	stat_net2(context, "create");
    } else {
	if (!(server < FD_SETSIZE)) {
	    stat_net1(context, "select");
	} else {
	    status = bind_unix(context, server);
	    if (status) {
		stat_link(context);
	    } else {
		Mylene1f4_set_record(context, lose_path, context);

		if (chmod(WINDOW(context).server.port, mode)) {
		    status = 1;
		    stat_net4(context, "chmod");
		} else {
		    status = listen(server, 1);
		    if (status) {
			stat_net2(context, "listen");
		    } else {
			status = case_lane(context, server);
		    }
		}

		unlink(WINDOW(context).server.port);

		Mylene1f4_get_record(context, lose_path);
	    }
	}

	/*
	 * BUGS
	 *
	 *.well, hmm, `server' should have been passed as reference and
	 * stuff...
	 */
	if (server != -1) {
#if __USE_SHUTDOWN__
	    shutdown(server, SHUT_RDWR);
#endif				/* __USE_SHUTDOWN__ */
	    close(server);
	}
    }

    return status;
}


static int
case_user(void *context)
{
    int status;
    unsigned mode;

    status =
	x1f4_parse_wxcardinal(&mode, WINDOW(context).server.mode, NULL, 0);
    if (status) {
	Mylene1f4_fprintf_error
	    (context, "%s: cannot parse mode specification: `%s'\n",
	     Mylene1f4(context), WINDOW(context).server.mode);
    } else if ((mode & 00600) != 00600) {
	status = 1;
	Mylene1f4_fprintf_error
	    (context, "%s: I am not even considering running `%s' local serv"
	     "er without read/write permissions (as mode %u requests)\n",
	     Mylene1f4(context), WINDOW(context).server.port, mode);
    } else {
	status = case_unix(context, mode);
    }

    return status;
}


static int
clip_deck(void *context)
{
#if __ENABLE_EQUEUE__
    int status = 0;
#endif				/* __ENABLE_EQUEUE__ */
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    if (!frame4_data->skipped) {
	if (!frame4_data->i144.last) {
	    frame4_data->i144.post = ~0;
	}

	if (~frame4_data->i144.post) {
	} else {
	    unsigned vertige;

	    vertige = frame4_data->vertige;

	    if (!vertige) {
	    } else {
		frame4_data->clip = 0;
		if (vertige & ~1) {
		    vertige -= 2;
		}
		{
		    frame4_data->already++;
		    frame4_data->vertige = vertige;
		}
	    }
	}

#if __ENABLE_EQUEUE__
	if (frame4_data->clip) {
	    int (*logic) (void *);

	    logic = Frame4(context)->logic;
	    if (logic) {
		frame4_data->link = 0;

		status = logic(context);
	    }
	}
#endif				/* __ENABLE_EQUEUE__ */
    }

#if !__ENABLE_EQUEUE__
    return 0;
#else
    return status;
#endif				/* !__ENABLE_EQUEUE__ */
}


static int
clip_liNk(void *context)
{
    int status;
    do {
	struct flush7_type *flush7_data;

	flush7_data = Frame7(context)->_3.line;
	if (flush7_data) {
	    if (flush7_data->count) {
		status = cLip_liNk(context);
		break;
	    }
	}

	status = cLip_link(context);
    } while (0);

    return status;
}


static int
clip_link(void *context)
{
    int status;

    status = Mylene1f4_make_link(context);
    if (status) {
    } else {
	cLip_link(context);
    }

    return status;
}


#if __EDITABLE_PLAYLIST__
static int
clip_list(void *context)
{
    int status;
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    if (!(frame4_data->flags & Mylene1f4_FILE_ERASED)) {
	status = post_list(context);
    } else {
	if (EXCESS(context).e1.list87.count == 1) {
	    /*
	     * TODO
	     *
	     * clear list (for good)
	     */

	    EXCESS(context).e1.list87.clear = 0;

	    REPEAT(context) = 0;

	    status = 0;
	} else {
	    unsigned index;

	    Mylene1f4_fix_index(context, &index);

	    status = post_list(context);
	    if (!status) {
		unsigned class;

		Mylene1f4_fix_index(context, &class);

		if (class != index) {
		    int Class;

		    status = Mylene1f4_ceil_list(context, &Class, index, NULL);
		    if (!status) {
			frame4_data->flags &= ~Mylene1f4_FILE_ERASED;
		    }
		}
	    }
	}
    }

    return status;
}
#endif				/* __EDITABLE_PLAYLIST__ */


static int
clip_none(void *context)
{
#if __ENABLE_EQUEUE__
    int status = 0;
#endif				/* __ENABLE_EQUEUE__ */
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    if (frame4_data->skipped) {
	frame4_data->clip = 1;
    } else {
	if (!frame4_data->i144.last) {
	    frame4_data->i144.post = ~0;
	}

	if (~frame4_data->i144.post) {
	    frame4_data->clip = 1;
	} else {
	    unsigned vertige;

	    vertige = frame4_data->vertige;

	    if (!vertige) {
		frame4_data->clip = 1;
	    } else {
		if (vertige & ~1) {
		    vertige -= 2;
		}
		{
		    frame4_data->already++;
		    frame4_data->vertige = vertige;
		}
	    }
	}

#if __ENABLE_EQUEUE__
	if (frame4_data->clip) {
	    int (*logic) (void *);

	    logic = Frame4(context)->logic;
	    if (logic) {
		frame4_data->link = 0;

		status = logic(context);
	    }
	}
#endif				/* __ENABLE_EQUEUE__ */
    }

#if !__ENABLE_EQUEUE__
    return 0;
#else
    return status;
#endif				/* !__ENABLE_EQUEUE__ */
}


static int
clip_run1(void *context)
{
    int start, status = 0;

    FINI(context) = NULL;
    INIT(context) = NULL;
    LINK(context) = 0;
    SEQUEL(context) = NULL;
    STREAMS(context) = 0;
    X1f42(context) = NULL;
    WIRE(context) = 7;
    Z0fa(context) = 0;

    swap_text(context);
    swap_newt(context);

    start = START(context);

    while (1) {
	Frame4(context)->clip = 1;

	if (1) {
	    int class;

	    status = Mylene1f4_deck_list(context, &start, &class);
	    if (status) {
		break;
	    } else {
		if (class) {
		    break;
		}
	    }
	}

	while (EXCESS(context).e1.list87.clear) {
	    Frame4(context)->i144.last = ~0;

	    SOURCE(context)._.s1.path = EXCESS(context).e1.list87.file_name;
	    status = PLAY_FILE(context);
	    if (status) {
		break;
	    } else {
		if (Frame4(context)->request == Mylene1f4_EXIT_Mylene) {
		    REPEAT(context) = 0;
		    break;
		}
	    }

	    {
		struct frame4_type *frame4_data;

		frame4_data = Frame4(context);

		if (!frame4_data->skipped) {
		    if (~frame4_data->i144.post) {
			if (1) {
			    if (POST_LIST(context)) {
				break;
			    } else {
				frame4_data->already = 0;
				frame4_data->vertige = 0;
			    }
			}
		    } else {
			if (frame4_data->clip) {
			    if (POST_LIST(context)) {
				break;
			    } else {
				frame4_data->already = 0;
			    }
			}
		    }
		} else {
		    if (frame4_data->skipped
			== EXCESS(context).e1.list87.count) {
			break;
		    } else {
			if (1) {
			    if (~frame4_data->i144.post) {
				frame4_data->i144.post = 1;
			    }
			    if (sign_list(context)) {
				break;
			    } else {
				frame4_data->already = 0;
				frame4_data->vertige = 0;
			    }
			}
		    }
		}
	    }
	}

	if (!REPEAT(context)) {
	    break;
	} else {
	    if (EXCESS(context).e1.list87.clear) {
		break;
	    } else {
		if (LOAD_ORDER(context) == Mylene1f4_LOAD_LAST) {
		    start = -1;
		}
	    }
	}
    }

    swap_neWt(context);
    swap_teXt(context);

    if (!status) {
	if (PID(context) != -1) {
	    status = wait_e1f4(context);
	}
    }

    return status;
}


static int
clip_text(void *context)
{
    if (Mylene1f4_clip_source(context)) {
	Frame4(context)->skipped++;
    } else {
	Frame4(context)->skipped = 0;
    }

    return 0;
}


static int
eVer_clip(void *context, unsigned start2, int *clear)
{
    int status;
    struct target_type *target_data;
    unsigned count;

    target_data = &TARGET(context);

    count = start2 - REFERENCE(context);

#if __LINK_CELL_LOADER__
    if (target_data->zero) {
	status = zero_cell(context, count, clear);
	if (!status) {
	    REFERENCE(context) += count;
	}
    } else {
#endif				/* __LINK_CELL_LOADER__ */
	target_data->skip(STORAGE(context));

	do {
	    status = line_clip(context);
	    if (status) {
		break;
	    } else {
		count--;
		if (1) {
		    unsigned request;

		    request = Frame4(context)->request;
		    if (request) {
			if (request == Mylene1f4_EXIT_Mylene
			    || request == Mylene1f4_JUMP_Mylene
			    || request == Mylene1f4_LINE_Mylene
			    || request == Mylene1f4_SKIP_Mylene) {
			    *clear = 1;
			    break;
			}
		    }
		}
	    }
	} while (count);

	if (!status) {
	    target_data->seek(STORAGE(context));
	}
#if __LINK_CELL_LOADER__
    }
#endif				/* __LINK_CELL_LOADER__ */

    return status;
}


static int
fini_port(void *context, unsigned index)
{
    return TARGET(context).fini(LOADER(context), index, &STORAGE(context));
}


static int
head_port(void *context, unsigned index)
{
    int status;
    struct target_type *target_data;
    void *loader;

    loader = LOADER(context);

    target_data = &TARGET(context);
    status = target_data->init(LOADER(context), index, &STORAGE(context));
    if (!status) {
	int (*head) (void *);

	target_data->type(loader, target_data);

	head = target_data->head;
	if (head) {
	    status = head(STORAGE(context));
	    if (status) {
		target_data->fini(LOADER(context), index, &STORAGE(context));
	    }
	}
    }

    return status;
}


static int
init_port(void *context, unsigned index, int miss_port)
{
    int status;

    if (miss_port) {
	status = head_port(context, index);
    } else {
	status = back_port(context, index);
    }
#if __FIX_0fa_WAIT__
    if (status) {
    } else {
	Mylene1f4_miss_wait(context, WAIT_COUNT, &Frame7(context)->_0.wait);
    }
#endif				/* __FIX_0fa_WAIT__ */

    return status;
}


static int
line_clip(void *context)
{
    int do_decode = 1, fd_0, fd_1, pending_0fa, server, status;
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    server = frame4_data->server;

    fd_0 = Mylene1ST(context);
    fd_1 = Mylene2ND(context);

    pending_0fa = Z0fa(context);

    while (1) {
	fd_set fd_set_0, *fd_set_0_select, fd_set_1, *fd_set_1_select;
	int client;
	struct timeval *selector, timeout;
	unsigned extend = 0, nfds = 0;

	if (SIGCHLD_received(context)) {
	    status = Mylene1f4_yway_mylene0fa(context, 0);
	    if (status) {
		break;
	    } else {
		if (1) {
		    break;
		}
	    }
	}

	client = frame4_data->client;

	if (0) {
	    fd_set_0_select = NULL;
	} else {
	    fd_set_0_select = &fd_set_0;
	    FD_ZERO(fd_set_0_select);
	    nfds = server;
	    FD_SET(nfds, fd_set_0_select);
	    extend = 1;
	    if (client != -1) {
		FD_SET(client, fd_set_0_select);
		if (nfds < client) {
		    nfds = client;
		}
	    }
	}
	if (!pending_0fa) {
	    if (!fd_set_0_select) {
		fd_set_0_select = &fd_set_0;
		FD_ZERO(fd_set_0_select);
		nfds = fd_0;
	    } else {
		if (nfds < fd_0) {
		    nfds = fd_0;
		}
	    }
	    FD_SET(fd_0, fd_set_0_select);
	    extend = 1;
	}
	if (1) {
	    fd_set_1_select = NULL;
	}
	if (OUTGOING(context)) {
	    fd_set_1_select = &fd_set_1;
	    FD_ZERO(fd_set_1_select);
	    FD_SET(fd_1, fd_set_1_select);
	    if (nfds < fd_1) {
		nfds = fd_1;
	    }
	    extend = 1;
	}
	if (frame4_data->back.slack.data) {
	    if (frame4_data->back.slide.size) {
		if (!fd_set_1_select) {
		    fd_set_1_select = &fd_set_1;
		    FD_ZERO(fd_set_1_select);
		    extend = 1;
		}
		{
		    FD_SET(client, fd_set_1_select);
		    if (false(nfds < client)) {
			nfds = client;
		    }
		}
	    }
	}

	if (1) {
	    selector = &timeout;
	    timeout.tv_sec = 0;
	    timeout.tv_usec = 0;
	}

	nfds += extend;

	status = select
	    (nfds, fd_set_0_select, fd_set_1_select, NULL, selector);
	if (status == -1) {
	    if (errno != EINTR) {
		Mylene1f4_fprintf_error
		    (context, "%s: failed select: %s\n", Mylene1f4(context),
		     strerror(errno));
		break;
	    }
	} else {
	    status = 0;

	    if (fd_set_0_select) {
		if (FD_ISSET(fd_0, fd_set_0_select)) {
		    pending_0fa = 1;
		}
		if (client != -1) {
		    if (FD_ISSET(client, fd_set_0_select)) {
			if (frame4_data->trans(context)) {
			    unsigned request;

			    request = frame4_data->request;
			    if (request == Mylene1f4_DEUX_Mylene) {
				status = slip_e1f4(context);
				if (status) {
				    break;
				}
			    } else {
				switch (request) {
				case Mylene1f4_EXIT_Mylene:
				case Mylene1f4_JUMP_Mylene:
				case Mylene1f4_LINE_Mylene:
				case Mylene1f4_SKIP_Mylene:
				    do_decode = 0;
				}
			    }
			}
		    }
		}
		/*
		 * the server is to be examined after the client, so
		 * that if the client closed the server accepts
		 * connection requests
		 */
		if (1) {
		    if (FD_ISSET(server, fd_set_0_select)) {
			peak_list(context);
		    }
		}
	    }
	    if (pending_0fa) {
		if (1) {
		    status = read_e0fa(context);
		    if (status) {
			break;
		    }
		}
	    }

	    if (fd_set_1_select) {
		status = _Mylene1f4_class7.pipe_clip
		    (context, fd_set_1_select);
		if (status) {
		    break;
		}
	    }

	    break;
	}
    }
    if (status) {
    } else {
	do {
	    if (do_decode) {
		status = TARGET(context).ever(STORAGE(context));
		if (status) {
		    break;
		} else {
		    unsigned reference;

		    reference = REFERENCE(context);
		    reference++;
		    Mylene1f4_post_time(context, reference);
		    REFERENCE(context) = reference;
		}
	    }

	    if (PROGRESS2(context)) {
		Mylene1f4_run_format(context, PROGRESS2(context));
	    }
	} while (0);
    }

    return status;
}


static int
line_deck(void *context, unsigned rerun, int *fix_start, void *flush7)
{
    if (Frame7(context)->_3.line) {
    } else {
	L1f4(context) = line_e1f4;

	Frame7(context)->_3.line = flush7;

	((struct flush7_type *) flush7)->rerun = rerun;
	((struct flush7_type *) flush7)->speed = ~0;
    }

    *fix_start = 0;

    return line_seek(context);
}


static int
line_e1f4(void *context, unsigned *state)
{
    *state = ((struct flush7_type *) Frame7(context)->_3.line)->speed;

    return 0;
}


static int
line_flow(void *context)
{
    Frame7(context)->_3.line = NULL;

    L1f4(context) = NULL;

    return 0;
}


static int
line_list(void *context)
{
    int status = 0;
    struct flush7_type *flush_data;
    unsigned speed;

    flush_data = Frame7(context)->_3.line;

    speed = flush_data->speed;
    if (!~speed) {
	speed = REFERENCE(context);
	if (1) {
	    flush_data->speed = speed;
	}
    }

    if (LINK(context)) {
	Frame4(context)->next.deck = speed;
	flush_data->count = 0;
    } else {
	unsigned frame, state;

	state = C1f4(context);

	frame = STATE(context)[Mylene1f4_REAL_STATE];
	if (SIZE(context) < frame) {
	    status = -1;
	} else {
	    unsigned cache, class, count;

	    class = state;
	    if (class < frame) {
		class += SIZE(context);
	    }
	    cache = class - frame;

	    class = RELEASE(context);

	    count = (cache + class - 1) / class;
	    if (speed < count) {
		status = -1;
	    } else {
		do {
		    frame = speed - count;

		    if (1) {
			int (*wind) (void *, unsigned *, unsigned);

			wind = TARGET(context).wind;
			if (wind) {
			    status = wind
				(LOADER(context), &frame,
				 SIZE(context) / class - count);
			    if (status) {
				break;
			    }
			}
		    }

		    Frame4(context)->next.deck = frame;

		    count = speed - frame;

		    cache = count * class;
		    if (state < cache) {
			state += SIZE(context);
		    }

		    flush_data->count = count;
		    flush_data->state = state - cache;
		} while (0);
	    }
	}
    }

    return status;
}


static int
line_seek(void *context)
{
    int status;

    if (Frame7(context)->_4.cell == SEEK_LINE) {
	struct flush7_type *flush_data;
	unsigned speed;

	flush_data = Frame7(context)->_3.line;

	speed = flush_data->speed;
	if (~speed) {
	    status = line_list(context);
	} else {
	    Frame4(context)->next.deck = Frame7(context)->_4.seek;
	    line_flow(context);
	    status = 0;
	}
    } else {
	status = line_list(context);
    }

    return status;
}


static int
lose_path(void *context)
{
    unlink(WINDOW(context).server.port);

    return 0;
}


static int
peak_list(void *context)
{
    int new;
    socklen_t size;
    struct sockaddr_in address;
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    size = sizeof(address);
    new = accept(frame4_data->server, (struct sockaddr *) &address, &size);
    if (new != -1) {
	if (frame4_data->client != -1 || FD_SETSIZE < new + 1) {
	    close(new);
	} else {
	    frame4_data->client = new;
	    frame4_data->progress = 16;

#if __NONBLOCKING_CLIENT_TRANSFER__
	    frame4_data->flags |= Mylene1f4_LOCK_CLIENT;
#endif				/* __NONBLOCKING_CLIENT_TRANSFER__ */

#if __NONBLOCKING_CLIENT_TRANSFER__
	    if (1) {
		int flags;

		flags = fcntl(new, F_GETFL);
		if (flags == -1) {
		} else {
		    if (fcntl(new, F_SETFL, flags | O_NONBLOCK) == -1) {
		    } else {
			int class;

			class = fcntl(new, F_GETFL);
			if (class == -1) {
			} else {
			    if (class != (flags | O_NONBLOCK)) {
			    } else {
				frame4_data->flags &= ~Mylene1f4_LOCK_CLIENT;
			    }
			}
		    }
		}
	    }
#endif				/* __NONBLOCKING_CLIENT_TRANSFER__ */

	    Mylene1f4_link_e3e8(context);
	}
    }

    return 0;
}


#if __LINK_CELL_LOADER__
static int
pLay_clip(void *context)
{
    int do_decode, fd_0, fd_1, pending_0fa, server, status;
    struct frame4_type *frame4_data;
    unsigned wait_d = 0;

    frame4_data = Frame4(context);

    server = frame4_data->server;

    fd_0 = Mylene1ST(context);
    fd_1 = Mylene2ND(context);

    pending_0fa = Z0fa(context);

    if (LINK(context)) {
	do_decode = 0;
    } else {
	unsigned cache;

	cache = C1f4(context) - STATE(context)[1];
	if (SIZE(context) < cache) {
	    cache += SIZE(context);
	}
	do {
	    cache = SIZE(context) - cache;
	    if (RELEASE(context) < cache) {
		cache = STATE(context)[Mylene1f4_CELL_STATE] - C1f4(context);
		if (SIZE(context) < cache) {
		    cache += SIZE(context);
		}
		if (cache < RELEASE(context)) {
		} else {
		    do_decode = 1;
		    wait_d = 0;
		    break;
		}
	    }
	    if (pending_0fa) {
		do_decode = 0;
	    } else if (WIRE(context)) {
		do_decode = 0;
	    } else {
# if __FIX_0fa_WAIT__
		wait_d = Frame7(context)->_0.wait;
# else
#  if SIZEOF_UNSIGNED_LONG == 8
		unsigned long wait;

		wait = RELEASE(context) * WAIT_COUNT - cache;
		wait *= 1000000;
		wait /= (SAMPLING(context) * CHANNELS(context) << 1);
#  else
		double wait;

		wait = RELEASE(context) * WAIT_COUNT - cache;
		wait /= (SAMPLING(context) * CHANNELS(context) << 1);
		wait *= 1000000;
#  endif				/* SIZEOF_UNSIGNED_LONG == 8 */
		wait_d = wait;
		if (131072 < wait_d) {
		    wait_d = 131072;
		}
# endif				/* __FIX_0fa_WAIT__ */

		do_decode = 2;
	    }
	} while (0);
    }

    while (1) {
	fd_set fd_set_0, *fd_set_0_select, fd_set_1, *fd_set_1_select;
	int client;
	struct timeval *selector, timeout;
	unsigned extend = 0, nfds = 0;

	if (SIGCHLD_received(context)) {
	    status = Mylene1f4_yway_mylene0fa(context, 0);
	    if (status) {
		break;
	    } else {
		do_decode = 0;
		if (1) {
		    break;
		}
	    }
	}

	client = frame4_data->client;

	if (0) {
	    fd_set_0_select = NULL;
	} else {
	    fd_set_0_select = &fd_set_0;
	    FD_ZERO(fd_set_0_select);
	    nfds = server;
	    FD_SET(nfds, fd_set_0_select);
	    extend = 1;
	    if (client != -1) {
		FD_SET(client, fd_set_0_select);
		if (nfds < client) {
		    nfds = client;
		}
	    }
	}
	if (!pending_0fa) {
	    if (!fd_set_0_select) {
		fd_set_0_select = &fd_set_0;
		FD_ZERO(fd_set_0_select);
		nfds = fd_0;
	    } else {
		if (nfds < fd_0) {
		    nfds = fd_0;
		}
	    }
	    FD_SET(fd_0, fd_set_0_select);
	    extend = 1;
	}
	if (1) {
	    fd_set_1_select = NULL;
	}
	if (OUTGOING(context)) {
	    fd_set_1_select = &fd_set_1;
	    FD_ZERO(fd_set_1_select);
	    FD_SET(fd_1, fd_set_1_select);
	    if (nfds < fd_1) {
		nfds = fd_1;
	    }
	    extend = 1;
	}
	if (frame4_data->back.slack.data) {
	    if (frame4_data->back.slide.size) {
		if (!fd_set_1_select) {
		    fd_set_1_select = &fd_set_1;
		    FD_ZERO(fd_set_1_select);
		    extend = 1;
		}
		{
		    FD_SET(client, fd_set_1_select);
		    if (false(nfds < client)) {
			nfds = client;
		    }
		}
	    }
	}

	if (pending_0fa) {
	    selector = &timeout;
	    timeout.tv_sec = 0;
	    timeout.tv_usec = 0;
	} else {
	    if (!do_decode) {
		selector = NULL;
	    } else {
		selector = &timeout;
		timeout.tv_sec = 0;
		timeout.tv_usec = wait_d;
	    }
	}

	nfds += extend;

	status = select
	    (nfds, fd_set_0_select, fd_set_1_select, NULL, selector);
	if (status == -1) {
	    if (errno != EINTR) {
		Mylene1f4_fprintf_error
		    (context, "%s: failed select: %s\n", Mylene1f4(context),
		     strerror(errno));
		break;
	    }
	} else {
	    status = 0;

	    if (fd_set_0_select) {
		if (FD_ISSET(fd_0, fd_set_0_select)) {
		    pending_0fa = 1;
		}
		if (client != -1) {
		    if (FD_ISSET(client, fd_set_0_select)) {
			if (frame4_data->trans(context)) {
			    unsigned request;

			    request = frame4_data->request;
			    if (request == Mylene1f4_DEUX_Mylene) {
				status = slip_e1f4(context);
				if (status) {
				    break;
				}
			    } else {
				switch (request) {
				case Mylene1f4_EXIT_Mylene:
				case Mylene1f4_JUMP_Mylene:
				case Mylene1f4_SKIP_Mylene:
				    do_decode = 0;
				}
			    }
			}
		    }
		}
		/*
		 * the server is to be examined after the client, so
		 * that if the client closed the server accepts
		 * connection requests
		 */
		if (1) {
		    if (FD_ISSET(server, fd_set_0_select)) {
			peak_list(context);
		    }
		}
	    }
	    if (pending_0fa) {
		if (1) {
		    status = read_e0fa(context);
		    if (status) {
			break;
		    }
		}
	    }

	    if (fd_set_1_select) {
		status = _Mylene1f4_class7.pipe_clip
		    (context, fd_set_1_select);
		if (status) {
		    break;
		}
	    }

	    break;
	}
    }
    if (status) {
    } else {
	do {
	    if (do_decode == 1) {
		if (0) {
		} else {
		    unsigned e1f4_index;

		    e1f4_index = C1f4(context);
		    e1f4_index += RELEASE(context);
		    if (e1f4_index == SIZE(context)) {
			e1f4_index = 0;
		    }
		    C1f4(context) = e1f4_index;
		    STATE(context)[0] = e1f4_index;

		    REFERENCE(context)++;
		}
	    }

	    if (PROGRESS2(context)) {
		Mylene1f4_run_format(context, PROGRESS2(context));
	    }
	} while (0);
    }

    return status;
}
#endif				/* __LINK_CELL_LOADER__ */


static int
plAy_dAta(void *context)
{
    int status = 0;
    unsigned tail;

    tail = TAIL(context);
    while (REFERENCE(context) < tail) {
	status = Frame4(context)->play(context);
	if (status) {
	    break;
	} else {
	    unsigned request;

	    request = Frame4(context)->request;
	    if (request) {
		if (request == Mylene1f4_EXIT_Mylene
		    || request == Mylene1f4_JUMP_Mylene
		    || request == Mylene1f4_LINE_Mylene
		    || request == Mylene1f4_SKIP_Mylene) {
		    break;
		}
	    }
	}
    }
    if (!status) {
	COMPLETED(context) = 1;

	do {
	    unsigned request;

	    request = Frame4(context)->request;
	    if (request == Mylene1f4_LINE_Mylene) {
		if (REFERENCE(context) < tail) {
		    break;
		}
	    }

	    Mylene1f4_shift_track(context);
	} while (0);
    }

    return status;
}


static int
plAy_data(void *context)
{
    int status;

    do {
	status = Frame4(context)->play(context);
	if (status) {
	    break;
	} else {
	    unsigned request;

	    request = Frame4(context)->request;
	    if (request) {
		if (request == Mylene1f4_EXIT_Mylene
		    || request == Mylene1f4_JUMP_Mylene
		    || request == Mylene1f4_LINE_Mylene
		    || request == Mylene1f4_SKIP_Mylene) {
		    break;
		}
	    }
	}
    } while (1);
    do {
	if (COMPLETED(context)) {
	    status = 0;
	} else {
	    if (status) {
		break;
	    } else {
		unsigned request;

		request = Frame4(context)->request;
		if (request == Mylene1f4_LINE_Mylene) {
		    break;
		}
	    }
	}

	Mylene1f4_shift_track(context);
    } while (0);

    return status;
}


/*
 * NOTES
 */
/*
 * Note.1
 *
 * in between any effector functions call other than (program non final
 * effector function) E_load call and some effector function call, the
 * instructions are executed before the old file is replaced by the new one
 * (i.e. after the playback completed for the old file, but before file is
 * unloaded).  And it looks like instructions are executed at the
 * beginning of some file processing (since a new file has just been loaded by
 * a E_load call).  But since the file is going to be unloaded (the E_load call
 * is the program final effector call) the instructions are also executed just
 * before the processing for the current file completes.
 */
static int
play_aime(void *context)
{
    int status;
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    if (!frame4_data->clip) {
	frame4_data->i144.last = 0;
	if (1) {
	    status = PLAY_LIST(context);
	    if (status) {
	    } else {
		if (Frame4(context)->request != Mylene1f4_EXIT_Mylene) {
		    status = clip_none(context);
		}
	    }
	}
    } else {
#if __ENABLE_FQUEUE__
	do {
#endif				/* __ENABLE_FQUEUE__ */
	    CLIP(context)++;

#if __ENABLE_FQUEUE__
	    {
		unsigned link;

		link = frame4_data->link;

		frame4_data->link = link & 16;

		if (link & 1) {
		    int inter = 1;

		    frame4_data->skipped = 0;

		    while (1) {
			int (*logic) (void *);

			logic = Frame4(context)->logic;

			if (!logic) {
			    status = 0;

			    break;
			} else {
			    link = frame4_data->link;
			    link &= 16;
			    link |= 4;
			    frame4_data->link = link;

			    status = logic(context);
			    if (status) {
				break;
			    } else {
				unsigned Link;

				Link = frame4_data->link;

				if (Link & 1) {
				    break;
				} else {
				    if (Link & 2) {
					if (frame4_data->request
					    == Mylene1f4_EXIT_Mylene) {
					    break;
					} else {
					    if (Link & 16) {
						/*
						 * See Note.1
						 */
						break;
					    } else {
						clip_text(context);
						inter = 0;
						break;
					    }
					}
				    }
				}
			    }
			}
		    }
		    if (inter) {
			break;
		    }
		} else {
#endif				/* __ENABLE_FQUEUE__ */
		    clip_text(context);

		    status = 0;
#if __ENABLE_FQUEUE__
		}
	    }
#endif				/* __ENABLE_FQUEUE__ */
	    if (frame4_data->skipped) {
	    } else {
#if __ENABLE_EQUEUE__
		{
		    unsigned instant;

		    instant = frame4_data->instant;
		    if (instant) {
			frame4_data->instant = 0;
			frame4_data->vertige = instant;
		    }
		}
#endif				/* __ENABLE_EQUEUE__ */

		frame4_data->i144.last = 0;
		if (1) {
		    status = PLAY_LIST(context);
		    if (status) {
		    } else {
			if (Frame4(context)->request
			    != Mylene1f4_EXIT_Mylene) {
			    status = clip_deck(context);
			}
		    }
		}
	    }
#if __ENABLE_FQUEUE__
	} while (0);
#endif				/* __ENABLE_FQUEUE__ */
    }

    return status;
}


#if __ENABLE_TAGGED_LIST__
static int
play_ceil(void *context)
{
    int status;

    status = Mylene1f4_init_source(context);
    if (status) {
	status = Mylene1f4_near_miss(context);
    } else {
	status = play_dana(context);

	{
	    int excess;

	    excess = Mylene1f4_fini_source(context);
	    if (status) {
	    } else {
		status = excess;
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_TAGGED_LIST__ */


static int
play_clIp(void *context)
{
    int fd_0, fd_1, pending_0fa, server, status;
    struct frame4_type *frame4_data;
    unsigned wait_d = 0;

    frame4_data = Frame4(context);

    server = frame4_data->server;

    fd_0 = Mylene1ST(context);
    fd_1 = Mylene2ND(context);

    pending_0fa = Z0fa(context);

    if (0) {
    } else {
	{
	    if (1) {
		wait_d = 0;
	    }
	}
    }

    while (1) {
	fd_set fd_set_0, *fd_set_0_select, fd_set_1, *fd_set_1_select;
	int client;
	struct timeval *selector, timeout;
	unsigned extend = 0, nfds = 0;

	if (SIGCHLD_received(context)) {
	    status = Mylene1f4_yway_mylene0fa(context, 0);
	    if (status) {
		break;
	    } else {
		if (1) {
		    break;
		}
	    }
	}

	client = frame4_data->client;

	if (0) {
	    fd_set_0_select = NULL;
	} else {
	    fd_set_0_select = &fd_set_0;
	    FD_ZERO(fd_set_0_select);
	    nfds = server;
	    FD_SET(nfds, fd_set_0_select);
	    extend = 1;
	    if (client != -1) {
		FD_SET(client, fd_set_0_select);
		if (nfds < client) {
		    nfds = client;
		}
	    }
	}
	if (!pending_0fa) {
	    if (!fd_set_0_select) {
		fd_set_0_select = &fd_set_0;
		FD_ZERO(fd_set_0_select);
		nfds = fd_0;
	    } else {
		if (nfds < fd_0) {
		    nfds = fd_0;
		}
	    }
	    FD_SET(fd_0, fd_set_0_select);
	    extend = 1;
	}
	if (1) {
	    fd_set_1_select = NULL;
	}
	if (OUTGOING(context)) {
	    fd_set_1_select = &fd_set_1;
	    FD_ZERO(fd_set_1_select);
	    FD_SET(fd_1, fd_set_1_select);
	    if (nfds < fd_1) {
		nfds = fd_1;
	    }
	    extend = 1;
	}
	if (frame4_data->back.slack.data) {
	    if (frame4_data->back.slide.size) {
		if (!fd_set_1_select) {
		    fd_set_1_select = &fd_set_1;
		    FD_ZERO(fd_set_1_select);
		    extend = 1;
		}
		{
		    FD_SET(client, fd_set_1_select);
		    if (false(nfds < client)) {
			nfds = client;
		    }
		}
	    }
	}

	if (pending_0fa) {
	    selector = &timeout;
	    timeout.tv_sec = 0;
	    timeout.tv_usec = 0;
	} else {
	    if (0) {
		selector = NULL;
	    } else {
		selector = &timeout;
		timeout.tv_sec = 0;
		timeout.tv_usec = wait_d;
	    }
	}

	nfds += extend;

	status = select
	    (nfds, fd_set_0_select, fd_set_1_select, NULL, selector);
	if (status == -1) {
	    if (errno != EINTR) {
		Mylene1f4_fprintf_error
		    (context, "%s: failed select: %s\n", Mylene1f4(context),
		     strerror(errno));
		break;
	    }
	} else {
	    status = 0;

	    if (fd_set_0_select) {
		if (FD_ISSET(fd_0, fd_set_0_select)) {
		    pending_0fa = 1;
		}
		if (client != -1) {
		    if (FD_ISSET(client, fd_set_0_select)) {
			if (frame4_data->trans(context)) {
			    unsigned request;

			    request = frame4_data->request;
			    if (request == Mylene1f4_DEUX_Mylene) {
				status = slip_e1f4(context);
				if (status) {
				    break;
				}
			    } else {
				switch (request) {
				case Mylene1f4_EXIT_Mylene:
				case Mylene1f4_JUMP_Mylene:
				case Mylene1f4_LINE_Mylene:
				case Mylene1f4_SKIP_Mylene:
				    ;
				}
			    }
			}
		    }
		}
		/*
		 * the server is to be examined after the client, so
		 * that if the client closed the server accepts
		 * connection requests
		 */
		if (1) {
		    if (FD_ISSET(server, fd_set_0_select)) {
			peak_list(context);
		    }
		}
	    }
	    if (pending_0fa) {
		if (1) {
		    status = read_e0fa(context);
		    if (status) {
			break;
		    }
		}
	    }

	    if (fd_set_1_select) {
		status = _Mylene1f4_class7.pipe_clip
		    (context, fd_set_1_select);
		if (status) {
		    break;
		}
	    }

	    break;
	}
    }
    if (status) {
    } else {
	do {
	    if (1) {
		struct flush7_type *flush7_data;
		unsigned c1f4_index, e1f4_index;

		flush7_data = Frame7(context)->_3.line;

		e1f4_index = flush7_data->state;

		c1f4_index = C1f4(context);
		C1f4(context) = e1f4_index;

		status = TARGET(context).link(STORAGE(context));
		if (status) {
		    break;
		} else {
		    e1f4_index += RELEASE(context);
		    if (e1f4_index == SIZE(context)) {
			e1f4_index = 0;
		    }
		    flush7_data->state = e1f4_index;

		    C1f4(context) = c1f4_index;

		    REFERENCE(context)++;

		    flush7_data->count--;
		    if (!flush7_data->count) {
			line_flow(context);
			Frame4(context)->play = play_clip;
		    }
		}
	    }

	    if (PROGRESS2(context)) {
		Mylene1f4_run_format(context, PROGRESS2(context));
	    }
	} while (0);
    }

    return status;
}


static int
play_clip(void *context)
{
    int do_decode, fd_0, fd_1, pending_0fa, server, status;
    struct frame4_type *frame4_data;
    unsigned wait_d = 0;

    frame4_data = Frame4(context);

    server = frame4_data->server;

    fd_0 = Mylene1ST(context);
    fd_1 = Mylene2ND(context);

    pending_0fa = Z0fa(context);

    if (LINK(context)) {
	do_decode = 0;
    } else {
	unsigned cache;

	cache = C1f4(context) - STATE(context)[1];
	if (SIZE(context) < cache) {
	    cache += SIZE(context);
	}
	{
	    cache = SIZE(context) - cache;
	    if (RELEASE(context) < cache) {
		do_decode = 1;
		wait_d = 0;
	    } else if (pending_0fa) {
		do_decode = 0;
	    } else if (WIRE(context)) {
		do_decode = 0;
	    } else {
#if __FIX_0fa_WAIT__
		wait_d = Frame7(context)->_0.wait;
#else
# if SIZEOF_UNSIGNED_LONG == 8
		unsigned long wait;

		wait = RELEASE(context) * WAIT_COUNT - cache;
		wait *= 1000000;
		wait /= (SAMPLING(context) * CHANNELS(context) << 1);
# else
		double wait;

		wait = RELEASE(context) * WAIT_COUNT - cache;
		wait /= (SAMPLING(context) * CHANNELS(context) << 1);
		wait *= 1000000;
# endif					/* SIZEOF_UNSIGNED_LONG == 8 */
		wait_d = wait;
		if (131072 < wait_d) {
		    wait_d = 131072;
		}
#endif				/* __FIX_0fa_WAIT__ */

		do_decode = 2;
	    }
	}
    }

    while (1) {
	fd_set fd_set_0, *fd_set_0_select, fd_set_1, *fd_set_1_select;
	int client;
	struct timeval *selector, timeout;
	unsigned extend = 0, nfds = 0;

	if (SIGCHLD_received(context)) {
	    status = Mylene1f4_yway_mylene0fa(context, 0);
	    if (status) {
		break;
	    } else {
		do_decode = 0;
		if (1) {
		    break;
		}
	    }
	}

	client = frame4_data->client;

	if (0) {
	    fd_set_0_select = NULL;
	} else {
	    fd_set_0_select = &fd_set_0;
	    FD_ZERO(fd_set_0_select);
	    nfds = server;
	    FD_SET(nfds, fd_set_0_select);
	    extend = 1;
	    if (client != -1) {
		FD_SET(client, fd_set_0_select);
		if (nfds < client) {
		    nfds = client;
		}
	    }
	}
	if (!pending_0fa) {
	    if (!fd_set_0_select) {
		fd_set_0_select = &fd_set_0;
		FD_ZERO(fd_set_0_select);
		nfds = fd_0;
	    } else {
		if (nfds < fd_0) {
		    nfds = fd_0;
		}
	    }
	    FD_SET(fd_0, fd_set_0_select);
	    extend = 1;
	}
	if (1) {
	    fd_set_1_select = NULL;
	}
	if (OUTGOING(context)) {
	    fd_set_1_select = &fd_set_1;
	    FD_ZERO(fd_set_1_select);
	    FD_SET(fd_1, fd_set_1_select);
	    if (nfds < fd_1) {
		nfds = fd_1;
	    }
	    extend = 1;
	}
	if (frame4_data->back.slack.data) {
	    if (frame4_data->back.slide.size) {
		if (!fd_set_1_select) {
		    fd_set_1_select = &fd_set_1;
		    FD_ZERO(fd_set_1_select);
		    extend = 1;
		}
		{
		    FD_SET(client, fd_set_1_select);
		    if (false(nfds < client)) {
			nfds = client;
		    }
		}
	    }
	}

	if (pending_0fa) {
	    selector = &timeout;
	    timeout.tv_sec = 0;
	    timeout.tv_usec = 0;
	} else {
	    if (!do_decode) {
		selector = NULL;
	    } else {
		selector = &timeout;
		timeout.tv_sec = 0;
		timeout.tv_usec = wait_d;
	    }
	}

	nfds += extend;

	status = select
	    (nfds, fd_set_0_select, fd_set_1_select, NULL, selector);
	if (status == -1) {
	    if (errno != EINTR) {
		Mylene1f4_fprintf_error
		    (context, "%s: failed select: %s\n", Mylene1f4(context),
		     strerror(errno));
		break;
	    }
	} else {
	    status = 0;

	    if (fd_set_0_select) {
		if (FD_ISSET(fd_0, fd_set_0_select)) {
		    pending_0fa = 1;
		}
		if (client != -1) {
		    if (FD_ISSET(client, fd_set_0_select)) {
			if (frame4_data->trans(context)) {
			    unsigned request;

			    request = frame4_data->request;
			    if (request == Mylene1f4_DEUX_Mylene) {
				status = slip_e1f4(context);
				if (status) {
				    break;
				}
			    } else {
				switch (request) {
				case Mylene1f4_EXIT_Mylene:
				case Mylene1f4_JUMP_Mylene:
				case Mylene1f4_LINE_Mylene:
				case Mylene1f4_SKIP_Mylene:
				    do_decode = 0;
				}
			    }
			}
		    }
		}
		/*
		 * the server is to be examined after the client, so
		 * that if the client closed the server accepts
		 * connection requests
		 */
		if (1) {
		    if (FD_ISSET(server, fd_set_0_select)) {
			peak_list(context);
		    }
		}
	    }
	    if (pending_0fa) {
		if (1) {
		    status = read_e0fa(context);
		    if (status) {
			break;
		    }
		}
	    }

	    if (fd_set_1_select) {
		status = _Mylene1f4_class7.pipe_clip
		    (context, fd_set_1_select);
		if (status) {
		    break;
		}
	    }

	    break;
	}
    }
    if (status) {
    } else {
	do {
	    if (do_decode == 1) {
		status = TARGET(context).link(STORAGE(context));
		if (status) {
		    break;
		} else {
		    unsigned e1f4_index;

		    e1f4_index = C1f4(context);
		    e1f4_index += RELEASE(context);
		    if (e1f4_index == SIZE(context)) {
			e1f4_index = 0;
		    }
		    C1f4(context) = e1f4_index;
		    STATE(context)[0] = e1f4_index;

		    REFERENCE(context)++;
		}
	    }

	    if (PROGRESS2(context)) {
		Mylene1f4_run_format(context, PROGRESS2(context));
	    }
	} while (0);
    }

    return status;
}


static int
play_dAta(void *context, unsigned start2)
{
    int status = 0;

    if (!(start2 < TAIL(context))) {
	REFERENCE(context) = TAIL(context);
    } else {
	int clear = 0;

	COMPLETED(context) = 0;
	REFERENCE(context) = HEAD(context);

	if (REFERENCE(context) < start2) {
	    status = skip_dAta(context, start2, &clear);
	} else {
	    status = zero_data(context, 0, &clear);
	}
	if (status) {
	} else {
	    if (clear) {
	    } else {
		status = play_deck(context, start2, plAy_dAta);
	    }
	}
    }

    return status;
}


static int
play_dana(void *context)
{
    int status;

    PLAY_STRIPE(context);
    if (Frame4(context)->request == Mylene1f4_SKIP_Mylene) {
	Frame4(context)->request = 0;
	status = 0;
    } else {
	status = Mylene1f4_line_source(context);
	if (status) {
	} else {
	    status = play_list(context);
	}

	if (status) {
	} else {
	    CLIP_STRIPE(context);
	    if (1) {
		PLAY(context)++;
	    }
	}
    }

    return status;
}


static int
play_data(void *context, unsigned start2)
{
    int clear = 0, status;

    COMPLETED(context) = 0;
    REFERENCE(context) = HEAD(context);

    if (REFERENCE(context) < start2) {
	status = skip_data(context, start2, &clear);
    } else {
	status = zero_data(context, 0, &clear);
    }
    if (status) {
    } else {
	if (clear) {
	} else {
	    if (COMPLETED(context)) {
	    } else {
		status = play_deck(context, start2, plAy_data);
	    }
	}
    }

    return status;
}


static int
play_deck(void *context, unsigned start2, int (*deck) (void *))
{
    int status;

    do {
	unsigned rerun = 0;

	if (Frame7(context)->_3.line) {
	    rerun = ((struct flush7_type *) Frame7(context)->_3.line)->rerun;
	    if (rerun) {
		/*
		 * FEATURES
		 *
		 * this one is a fuckup:
		 * Mylene0fa stream completion report may arrive at any time...
		 * ... which is otherwise okay, only it has to be taken into
		 * account for future development.
		 */
	    }
	} else {
	    Mylene1f4_scale_track(context);

	    status = Mylene1f4_init_link(context, start2);
	    if (status) {
		break;
	    }
	}

	Frame7(context)->_4.cell = PLAY_LINE;

	status = deck(context);

	if (!status) {
	    unsigned request;

	    request = Frame4(context)->request;
	    if (request == Mylene1f4_EXIT_Mylene) {
	    } else if (request == Mylene1f4_LINE_Mylene) {
	    } else {
		int excess;

		if (request == Mylene1f4_JUMP_Mylene) {
		    excess = slip_e3e8(context);
		} else if (request == Mylene1f4_SKIP_Mylene) {
		    excess = slip_e3e8(context);
		} else if (rerun) {
		    excess = slip_miss(context);
		} else {
		    excess = Mylene1f4_fini_link(context);
		}
		if (status) {
		} else {
		    status = excess;
		}
	    }
	}
    } while (0);

    return status;
}


static int
play_file(void *context)
{
    int status;

    status = Mylene1f4_init_source(context);
    if (status) {
	status = Mylene1f4_near_miss(context);
    } else {
	status = play_aime(context);

	{
	    int excess;

	    excess = Mylene1f4_fini_source(context);
	    if (status) {
	    } else {
		status = excess;
	    }
	}
    }

    return status;
}


static int
play_list(void *context)
{
    int status = 0;
    unsigned count;

    count = COUNT(context);
    if (count) {
	int fix_start = 1;
	struct flush7_type flush7;

	Frame7(context)->_3.line = NULL;

	Frame4(context)->index = 0;

	SECOND_A(ATIME(context)) = 0;
	SECOND_F(ATIME(context)) = 0;

	REFERENCE(context) = 0;

	while (1) {
	    status = init_port(context, Frame4(context)->index, fix_start);
	    if (status) {
		break;
	    } else {
		int excess;
		unsigned start2;

		if (fix_start) {
		    if (PROGRESS1(context)) {
			Mylene1f4_run_format(context, PROGRESS1(context));
		    }
		}

		if (fix_start) {
		    Mylene1f4_fix_start(context, &start2);
		} else {
		    fix_start = 1;
		    start2 = Frame4(context)->next.deck;
		}

		Frame7(context)->_4.seek = start2;

		if (Frame7(context)->_3.line) {
		    status = clip_liNk(context);
		} else {
		    status = clip_link(context);
		}
		if (status) {
		} else {
		    Frame7(context)->_4.cell = SEEK_LINE;
		    if (~TAIL(context)) {
			status = play_dAta(context, start2);
		    } else {
			status = play_data(context, start2);
		    }
		    if (Frame4(context)->request == Mylene1f4_LINE_Mylene) {
			status = line_deck(context, 0, &fix_start, &flush7);
		    } else {
			if (Frame7(context)->_3.line) {
			    line_flow(context);
			}
			if (Frame4(context)->request
			    == Mylene1f4_EXIT_Mylene) {
			    excess = wait_e3e8(context);
			    if (status) {
			    } else {
				status = excess;
			    }
			}
			if (status) {
			} else {
			    status = wait_e1f4(context);
			    if (status) {
			    } else {
				if (Frame4(context)->request
				    == Mylene1f4_LINE_Mylene) {
				    status = line_deck
					(context, 1, &fix_start, &flush7);
				} else {
				    void *progressl;

				    progressl = PROGRESS2(context);
				    if (progressl) {
					Mylene1f4_run_format
					    (context, progressl);
				    }
				    if (Frame4(context)->request
					== Mylene1f4_JUMP_Mylene) {
					/*
					 * BUGS
					 *
					 * start should not be fixed if new
					 * entry index does not match current
					 * one
					 */
					fix_start = 0;
				    }
				    if (fix_start) {
					progressl = PROGRESS3(context);
					if (progressl) {
			    		    Mylene1f4_run_format
						(context, progressl);
					}
				    }
				}
			    }
			}
		    }
		}

		excess = fini_port(context, Frame4(context)->index);
		if (status) {
		} else {
		    status = excess;
		}
		if (status) {
		    break;
		} else {
		    unsigned request;

		    request = Frame4(context)->request;
		    if (request == Mylene1f4_EXIT_Mylene) {
			break;
		    } else if (request == Mylene1f4_JUMP_Mylene) {
			/*
			 * case still needed, check BUGS above
			 */
			Frame4(context)->request = 0;
		    } else if (request == Mylene1f4_LINE_Mylene) {
			Frame4(context)->request = 0;
		    } else if (request == Mylene1f4_SKIP_Mylene) {
			Frame4(context)->i144.last = ~0;
			Frame4(context)->request = 0;
			break;
		    } else {
			unsigned index;

			index = Frame4(context)->index;
			index++;
			Frame4(context)->index = index;
			if (index == count) {
			    break;
			}
		    }
		}
	    }
	}
    }

    return status;
}


#if __ENABLE_TAGGED_LIST__
static int
play_tile(void *context)
{
    Mylene1f4_link_source(context);

    return play_aime(context);
}
#endif				/* __ENABLE_TAGGED_LIST__ */


static int
post_list(void *context)
{
    if (LOAD_ORDER(context) == Mylene1f4_LOAD_RAND) {
	Frame4(context)->i144.post = rand() % EXCESS(context).e1.list87.count;
	Frame4(context)->i144.sign = rand() & 1 ? f4_LOAD_NEXT : f4_LOAD_LAST;
    }

    return sign_list(context);
}


static int
read_e0fa(void *context)
{
    return Mylene1f4_read_e0fa
	(context, Frame4(context)->request == Mylene1f4_EXIT_Mylene);
}


/*
 * BUGS
 *
 * the return value of _fast_list is misinterpreted - it indicates an error
 * when not zero, it is interpreted as end of list.  and that is because the
 * surrounding code does not expect this function to fail (it was previously
 * never failing).  the effect is that Mylene1f4 will exit successfully where
 * it should have exited unsuccessfully.  but it will exit, nonetheless.
 */
static int
sign_list(void *context)
{
    int status = 0;
    unsigned post;

    post = Frame4(context)->i144.post;
    if (!~post) {
	EXCESS(context).e1.list87.clear--;
	if (EXCESS(context).e1.list87.clear) {
	    status = Mylene1f4_fast_list(context);
	} else {
	    /*
	     * FEATURES
	     *
	     * sort of stupid.  the surrounding code will handle this properly,
	     * returning 0 is required to continue, even though it appears
	     * there is nothing to continue to (there is something, if `repeat'
	     * is on).
	     */
	}
    } else {
	do {
	    if (LOAD_ORDER(context) == Mylene1f4_LOAD_LAST) {
		unsigned this;

		this = EXCESS(context).e1.list87.clear;
		if (Frame4(context)->i144.sign == Mylene1f4_LOAD_LAST) {
		    if (post < this) {
			this -= post;
			EXCESS(context).e1.list87.clear = this;
			status = Mylene1f4_fast_list(context);
		    } else {
			if (true(LOAD_ORDER(context) != Mylene1f4_LOAD_RAND)
			    && !REPEAT(context)) {
			    status = 1;
			    break;
			} else {
			    unsigned size;

			    size = EXCESS(context).e1.list87.count;
			    post %= size;
			    this += size;
			    this -= post;
			    this--;
			    this %= size;
			    this++;
			    EXCESS(context).e1.list87.clear = this;
			    status = Mylene1f4_fast_list(context);
			}
		    }
		} else {
		    unsigned size;

		    size = EXCESS(context).e1.list87.count;
		    this += post;
		    if (!(size < this)) {
			EXCESS(context).e1.list87.clear = this;
			status = Mylene1f4_fast_list(context);
		    } else {
			if (true(LOAD_ORDER(context) != Mylene1f4_LOAD_RAND)
			    && !REPEAT(context)) {
			    status = 1;
			    break;
			} else {
			    this--;
			    this %= size;
			    this++;
			    EXCESS(context).e1.list87.clear = this;
			    status = Mylene1f4_fast_list(context);
			}
		    }
		}
	    } else {
		unsigned this;

		this = EXCESS(context).e1.list87.clear;
		if (Frame4(context)->i144.sign == Mylene1f4_LOAD_LAST) {
		    unsigned size;

		    size = EXCESS(context).e1.list87.count;
		    this += post;
		    if (!(size < this)) {
			EXCESS(context).e1.list87.clear = this;
			status = Mylene1f4_fast_list(context);
		    } else {
			if (true(LOAD_ORDER(context) != Mylene1f4_LOAD_RAND)
			    && !REPEAT(context)) {
			    status = 1;
			    break;
			} else {
			    this--;
			    this %= size;
			    this++;
			    EXCESS(context).e1.list87.clear = this;
			    status = Mylene1f4_fast_list(context);
			}
		    }
		} else {
		    if (post < this) {
			this -= post;
			EXCESS(context).e1.list87.clear = this;
			status = Mylene1f4_fast_list(context);
		    } else {
			if (true(LOAD_ORDER(context) != Mylene1f4_LOAD_RAND)
			    && !REPEAT(context)) {
			    status = 1;
			    break;
			} else {
			    unsigned size;

			    size = EXCESS(context).e1.list87.count;
			    post %= size;
			    this += size;
			    this -= post;
			    this--;
			    this %= size;
			    this++;
			    EXCESS(context).e1.list87.clear = this;
			    status = Mylene1f4_fast_list(context);
			}
		    }
		}
	    }
	} while (0);
    }

    return status;
}


static int
skip_dAta(void *context, unsigned start2, int *clear)
{
    int status;

    if (TAIL(context) < start2 + 1) {
	status = 1;
    } else {
	status = eVer_clip(context, start2, clear);
    }

    return status;
}


static int
skip_data(void *context, unsigned start2, int *clear)
{
    int status;

    status = eVer_clip(context, start2, clear);
    if (status) {
	if (COMPLETED(context)) {
	    TAIL(context) = REFERENCE(context);
	    status = 0;
	}
    }

    return status;
}


static int
slip_e1f4(void *context)
{
    unsigned back;

    Frame4(context)->request = 0;

    back = Frame4(context)->next.back;

#if __RESTART_Mylene0fa__
    if (back == 0041) {
	struct timeval attached;

	gettimeofday(&attached, NULL);

	WINDOW(context).server.attached_i = STATE(context)[1];

	WINDOW(context).server.attached_a = SECOND_a(attached);
	WINDOW(context).server.attached_f = SECOND_f(attached);
    }
#endif				/* __RESTART_Mylene0fa__ */

    return Mylene1f4_send_mylene0fa(context, back, NULL);
}


static int
slip_e3e8(void *context)
{
    COMPLETED(context) = 1;

    return Mylene1f4_send_mylene0fa(context, 1, NULL);
}


static int
stat_e3e8(void *context, const char *action)
{
    Mylene1f4_fprintf_error
	(context, "%s: cannot %s controlling server", Mylene1f4(context),
	 action);

    return 0;
}


static int
stat_e7d0(void *context, const char *action)
{
    stat_e3e8(context, action);
    Mylene1f4_fprintf_error(context, " `%s'", WINDOW(context).server.port);

    return 0;
}


static int
stat_link(void *context)
{
    stat_e7d0(context, "bind");
    Mylene1f4_fprintf_error(context, ": %s", strerror(errno));
    if (errno == EADDRINUSE) {
	int excess;
	struct stat info;

	excess = stat(WINDOW(context).server.port, &info);
	if (!excess) {
	    Mylene1f4_fprintf_error(context, ": %s", strerror(EEXIST));
	}
    }
    Mylene1f4_fprintf_error(context, "\n");

    return 0;
}


static int
stat_net1(void *context, const char *action)
{
    stat_e3e8(context, action);
    Mylene1f4_fprintf_error(context, "\n");

    return 0;
}


static int
stat_net2(void *context, const char *action)
{
    stat_e3e8(context, action);
    Mylene1f4_fprintf_error(context, ": %s\n", strerror(errno));

    return 0;
}


static int
stat_net4(void *context, const char *action)
{
    stat_e7d0(context, action);
    Mylene1f4_fprintf_error(context, ": %s\n", strerror(errno));

    return 0;
}


static int
swap_neWt(void *context)
{
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    swap_miss(context, &PROGRESS1(context), frame4_data->progress1);
    swap_miss(context, &PROGRESS2(context), frame4_data->progress2);
    swap_miss(context, &PROGRESS3(context), frame4_data->progress3);

    return 0;
}


static int
swap_newt(void *context)
{
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    frame4_data->progress1 = PROGRESS1(context);
    frame4_data->progress2 = PROGRESS2(context);
    frame4_data->progress3 = PROGRESS3(context);

    return 0;
}


static int
swap_teXt(void *context)
{
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

    LOAD_ORDER(context) = frame4_data->text.load_order;

    swap_data(&ALBUM(context), &frame4_data->text.album);
    swap_data(&LBASE(context), &frame4_data->text.lbase);
    swap_data(&LPATH(context), &frame4_data->text.lpath);
    swap_data(&MATCH(context), &frame4_data->text.match);
    swap_data(&NOTES(context), &frame4_data->text.notes);
    swap_data(&TITLE(context), &frame4_data->text.title);

#if __ENABLE_FILTER__
    swap_deck(context, Frame4(context)->e4deck);
#endif				/* __ENABLE_FILTER__ */

    {
	char *data;

	data = frame4_data->text.slack.data;
	if (data) {
	    free(data);
	}
    }
    {
	unsigned size, swap;

	size = SLACK(context).size;
	swap = frame4_data->text.slack.size;
	if (size != swap) {
	    if (!swap) {
		char *data;

		data = SLACK(context).data;
		SLACK(context).data = NULL;
		if (data) {
		    free(data);
		}
	    } else {
		char *data;

		data = (char *) realloc(SLACK(context).data, swap);
		if (data) {
		    SLACK(context).data = data;
		    SLACK(context).size = swap;
		}
	    }
	}
    }

    return 0;
}


static int
swap_text(void *context)
{
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);

#if __ENABLE_FILTER__
    frame4_data->e4deck = E4DECK(context);
#endif				/* __ENABLE_FILTER__ */

    frame4_data->text.album.data = ALBUM(context).data;
    frame4_data->text.album.size = ALBUM(context).size;
    frame4_data->text.lbase.data = LBASE(context).data;
    frame4_data->text.lbase.size = LBASE(context).size;
    frame4_data->text.lpath.data = LPATH(context).data;
    frame4_data->text.lpath.size = LPATH(context).size;
    frame4_data->text.match.data = MATCH(context).data;
    frame4_data->text.match.size = MATCH(context).size;
    frame4_data->text.notes.data = NOTES(context).data;
    frame4_data->text.notes.size = NOTES(context).size;
    frame4_data->text.slack.data = NULL;
    frame4_data->text.slack.size = SLACK(context).size;
    frame4_data->text.title.data = TITLE(context).data;
    frame4_data->text.title.size = TITLE(context).size;

    frame4_data->text.load_order = LOAD_ORDER(context);

    return 0;
}


static int
wait_clip(void *context)
{
    int fd_0, fd_1, pending_0fa, server, status;
    struct frame4_type *frame4_data;
    unsigned wait_2 = 0;
    void *progress2;

    progress2 = PROGRESS2(context);

    frame4_data = Frame4(context);

    server = frame4_data->server;

    fd_0 = Mylene1ST(context);
    fd_1 = Mylene2ND(context);

    pending_0fa = Z0fa(context);

    if (pending_0fa) {
    } else if (progress2) {
#if __FIX_0fa_WAIT__
	wait_2 = Frame7(context)->_0.wait;
#else
# if SIZEOF_UNSIGNED_LONG == 8
	unsigned long wait;

	wait = RELEASE(context) * WAIT_COUNT;
	wait *= 1000000;
	wait /= (SAMPLING(context) * CHANNELS(context) << 1);
# else
	double wait;

	wait = RELEASE(context) * WAIT_COUNT;
	wait /= (SAMPLING(context) * CHANNELS(context) << 1);
	wait *= 1000000;
# endif					/* SIZEOF_UNSIGNED_LONG == 8 */
	wait_2 = wait;
	if (131072 < wait_2) {
	    wait_2 = 131072;
	}
#endif				/* __FIX_0fa_WAIT__ */
    }

    while (1) {
	fd_set fd_set_0, *fd_set_0_select, fd_set_1, *fd_set_1_select;
	int client;
	struct timeval *selector, timeout;
	unsigned extend = 0, nfds = 0;

	if (SIGCHLD_received(context)) {
	    int expect;

	    expect = Frame4(context)->request == Mylene1f4_EXIT_Mylene;
	    status = Mylene1f4_yway_mylene0fa(context, expect);
	    if (status) {
		break;
	    } else {
		if (expect) {
		    if (PID(context) == -1) {
			break;
		    }
		} else {
		    break;
		}
	    }
	}

	client = frame4_data->client;

	if (server == -1) {
	    fd_set_0_select = NULL;
	} else {
	    fd_set_0_select = &fd_set_0;
	    FD_ZERO(fd_set_0_select);
	    nfds = server;
	    FD_SET(nfds, fd_set_0_select);
	    extend = 1;
	    if (client != -1) {
		FD_SET(client, fd_set_0_select);
		if (nfds < client) {
		    nfds = client;
		}
	    }
	}
	if (!pending_0fa) {
	    if (!fd_set_0_select) {
		fd_set_0_select = &fd_set_0;
		FD_ZERO(fd_set_0_select);
		nfds = fd_0;
	    } else {
		if (nfds < fd_0) {
		    nfds = fd_0;
		}
	    }
	    FD_SET(fd_0, fd_set_0_select);
	    extend = 1;
	}
	if (1) {
	    fd_set_1_select = NULL;
	}
	if (OUTGOING(context)) {
	    fd_set_1_select = &fd_set_1;
	    FD_ZERO(fd_set_1_select);
	    FD_SET(fd_1, fd_set_1_select);
	    if (nfds < fd_1) {
		nfds = fd_1;
	    }
	    extend = 1;
	}
	if (frame4_data->back.slack.data) {
	    if (frame4_data->back.slide.size) {
		if (!fd_set_1_select) {
		    fd_set_1_select = &fd_set_1;
		    FD_ZERO(fd_set_1_select);
		    extend = 1;
		}
		{
		    FD_SET(client, fd_set_1_select);
		    if (false(nfds < client)) {
			nfds = client;
		    }
		}
	    }
	}

	if (pending_0fa) {
	    selector = &timeout;
	    timeout.tv_sec = 0;
	    timeout.tv_usec = 0;
	} else {
	    if (!progress2) {
		selector = NULL;
	    } else {
		selector = &timeout;
		timeout.tv_sec = 0;
		timeout.tv_usec = wait_2;
	    }
	}

	nfds += extend;

	status = select
	    (nfds, fd_set_0_select, fd_set_1_select, NULL, selector);
	if (status == -1) {
	    if (errno != EINTR) {
		Mylene1f4_fprintf_error
		    (context, "%s: failed select: %s\n", Mylene1f4(context),
		     strerror(errno));
		break;
	    }
	} else {
	    status = 0;

	    if (fd_set_0_select) {
		if (FD_ISSET(fd_0, fd_set_0_select)) {
		    pending_0fa = 1;
		}
		if (client != -1) {
		    if (FD_ISSET(client, fd_set_0_select)) {
			if (frame4_data->trans(context)) {
			    switch (frame4_data->request) {
			    case Mylene1f4_DEUX_Mylene:
				status = slip_e1f4(context);
				break;
			    case Mylene1f4_EXIT_Mylene:
				status = wait_e3e8(context);
				break;
			    case Mylene1f4_JUMP_Mylene:
				status = slip_e3e8(context);
				break;
			    case Mylene1f4_LINE_Mylene:
				status = slip_miss(context);
				break;
			    case Mylene1f4_SKIP_Mylene:
				status = slip_e3e8(context);
				break;
			    }
			    if (status) {
				break;
			    }
			}
		    }
		}
		/*
		 * the server is to be examined after the client, so
		 * that if the client closed the server accepts
		 * connection requests
		 */
		if (server != -1) {
		    if (FD_ISSET(server, fd_set_0_select)) {
			peak_list(context);
		    }
		}
	    }
	    if (pending_0fa) {
		if (1) {
		    status = read_e0fa(context);
		    if (status) {
			break;
		    } else {
			if (Mylene1ST(context) == -1) {
			    break;
			}
		    }
		}
	    }

	    if (fd_set_1_select) {
		status = _Mylene1f4_class7.pipe_clip
		    (context, fd_set_1_select);
		if (status) {
		    break;
		}
	    }

	    break;
	}
    }
    if (status) {
    } else {
	if (PID(context) == -1) {
	} else {
	    if (1) {
		progress2 = PROGRESS2(context);

		if (progress2) {
		    Mylene1f4_run_format(context, progress2);
		}
	    }
	}
    }

    return status;
}


/*
 * if controller window asked for exit wait_e1f4 renders futile (output channel
 * will be flushed by main logic).  I'll take the trip just for fun...
 */
static int
wait_e1f4(void *context)
{
    int status = 0;

    while (STREAMS(context) & (1u << 31)) {
	status = wait_clip(context);
	if (status) {
	    break;
	} else {
	    unsigned request;

	    request = Frame4(context)->request;
	    if (request) {
		if (request == Mylene1f4_LINE_Mylene) {
		    break;
		} else {
		    if (0) {
		    } else {
			if (request == Mylene1f4_EXIT_Mylene) {
			    if (PID(context) == -1) {
				break;
			    } else {
				if (!OUTGOING(context)) {
				    break;
				} else {
				    if (Mylene1ST(context) == -1) {
					break;
				    }
				}
			    }
			}
		    }
		}
	    }
	}
    }

    return status;
}


static int
wait_e3e8(void *context)
{
    int status, socket;
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);
    socket = frame4_data->client;
    if (socket != -1) {
	Mylene1f4_lose_e3e8(context);
    }
    socket = frame4_data->server;
    if (socket != -1) {
#if __USE_SHUTDOWN__
	shutdown(socket, SHUT_RDWR);
#endif				/* __USE_SHUTDOWN__ */
	close(socket);
	frame4_data->server = -1;
    }

    {
	void *progress2;

	progress2 = PROGRESS2(context);
	if (progress2) {
	    Mylene1f4_fini_format(context, &progress2);

	    frame4_data->progress2 = NULL;

	    PROGRESS2(context) = NULL;
	}
    }

    status = Mylene1f4_send_mylene0fa(context, 0, NULL);

    return status;
}


#if __LINK_CELL_LOADER__
static int
zero_cell(void *context, unsigned count2, int *clear)
{
    int status;

    status = TARGET(context).zero(STORAGE(context), count2);

    return status;
}
#endif				/* __LINK_CELL_LOADER__ */


int
Mylene1f4_run_excess7(void *context)
{
    int status;

    if (!WINDOW(context).server.type) {
	status = case_port(context);
    } else {
	status = case_path(context);
    }

    return status;
}

#endif				/* __ENABLE_CONTROLLING_SERVER__ */
