/*
 * excessb.c
 * Copyright (C) 2005-2012, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

/*
 * FEATURES
 *
 *.mirror usage (as in
 * ((struct everlanes_type *) WINDOW(context).within.evermiss)->mirror usage)
 * is at best unsafe, since the mirror may get overwritten by whatever use to
 * write it.
 */
/*
 * BUGS
 *
 *.looks like this whole detaching mechanism is awfully wrong since it can
 * block for quite a while, and thus commands are not served for too long.
 * Especially the 0602 detaching is likely to cause trouble, since PTS may
 * indicate that Mylene1f4 should start lot later, and of course, in this time
 * the controller may change its mind about course of action (like asking for
 * exit or for playing from a later time offset, offset for which the PTS would
 * indicate an earlier (machine clock) start than the one previously
 * indicated).
 *.there seems to be no proper checking that the current target is an 1f42 one
 * for when in \0ink or \1ink.  Yet this is assumed and most likely a big crash
 * will occur when this miss assertion fails.
 */
/*
 * TODO
 *
 *.Mylene0fa exits like hell, apparently killed by underflood.  This is
 * partially addressed by latency option in Mylene0fa and device arguments
 * option in Mylene1f4.
 *.when in \0ink, paused and skipping, the new stream position is not
 * transmitted to the lanes controller.  Which is not that nice, yet the lanes
 * \0ink protocol should be fitted with a stream position set protocol, and
 * once that is done the matter should be addressed.  Along with replacement of
 * slip_tail (the 0202, 0203 sequence) wherever it occurs (it also occurs that
 * slip_tail is the only lock method consumer, hence the lock method will go,
 * too).
 */

#if __ENABLE_x1f4LANES__

#include <errno.h>
#include <math.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/select.h>
#include <sys/time.h>
#include <sys/types.h>

#include <access.h>
#include <defs.h>
#include <inter.h>
#include <lanes.h>
#include <types.h>

#define __FIX_0fa_WAIT__		1

#define __TYPE_TARGET__			1

#define WAIT_COUNT			4

#define Mylene1f4_DEUX_Mylene		1
#define Mylene1f4_EXIT_Mylene		2
#if __ENABLE_x00696e6b__
# define Mylene1f4_JUMP_Mylene		3
# define Mylene1f4_LINE_Mylene		4
#endif				/* __ENABLE_x00696e6b__ */
#define Mylene1f4_SKIP_Mylene		5
#if __ENABLE_x00696e6b__
# define Mylene1f4_STOP_Mylene		6
#endif				/* __ENABLE_x00696e6b__ */
#define Mylene1f4_WAIT_Mylene		7

#if __ENABLE_x00696e6b__
# define Mylene1f4_ZERO_Mylene		8
#endif				/* __ENABLE_x00696e6b__ */

#define SECOND_A(v)			((v)->tv_sec)
#define SECOND_F(v)			((v)->tv_usec)

#define SECOND_a(t)			((t).tv_sec)
#define SECOND_f(t)			((t).tv_usec)

#define Second_a(context) \
    (Window((context))->second_a)
#define Second_f(context) \
    (Window((context))->second_f)

#define true(e)				1

#define Output(context) \
    ((const struct output_type *) DEVICE(context))
#define Window(context) \
    ((struct window_type *) WINDOW(context).within.window)

#if __ENABLE_x00696e6b__
# define Start2(context)		Window(context)->start2
#else
# define Start2(context)		START2(context)
#endif				/* __ENABLE_x00696e6b__ */

#if !__ENABLE_x00696e6b__
# define fini_link			NULL
# define init_link			NULL
#endif				/* !__ENABLE_x00696e6b__ */

#if __ENABLE_x00696e6b__
# define __slip_away(context) \
    Window((context))->slip((context))
#else
# define __slip_away(context) \
    slip_away((context))
#endif				/* __ENABLE_x00696e6b__ */

#if __ENABLE_x00696e6b__
# define CARDINAL(buffer, base) \
    ((buffer)[(base) + 0] << 030 | (buffer)[(base) + 1] << 020		      \
     | (buffer)[(base) + 2] << 010 | (buffer)[(base) + 3] << 000)
#endif				/* __ENABLE_x00696e6b__ */

#if __ENABLE_x00696e6b__
# define SET_CARDINAL(buffer, base, card) \
    {									      \
	(buffer)[(base) + 0] = (card) >> 030;				      \
	(buffer)[(base) + 1] = (card) >> 020;				      \
	(buffer)[(base) + 2] = (card) >> 010;				      \
	(buffer)[(base) + 3] = (card) >> 000;				      \
    }
#endif				/* __ENABLE_x00696e6b__ */

#if !__LINK_CELL_LOADER__
# define zero_data(context, null, clear) \
    0
#else
# define zero_data(context, null, clear) \
    (!TARGET((context)).zero ? 0 : zero_cell((context), (null), (clear)))
#endif				/* !__LINK_CELL_LOADER__ */

typedef struct window_type {
#if __ENABLE_x00696e6b__
    int (*slip) (void *);
#endif				/* __ENABLE_x00696e6b__ */
    unsigned detached, index, request, underlay;
#if __ENABLE_x00696e6b__
    unsigned first, last_386, miss_link, second_a, second_f, start2, state2;
#endif				/* __ENABLE_x00696e6b__ */
#if __FIX_0fa_WAIT__
    unsigned wait;
#endif				/* __FIX_0fa_WAIT__ */
#if __ENABLE_x00696e6b__
    void *x1f42a;
#endif				/* __ENABLE_x00696e6b__ */
} window_type;

#if __ENABLE_x00696e6b__
static int dana_link(void *, unsigned *);
static int down_time(unsigned *, unsigned *, unsigned, unsigned);
#endif				/* __ENABLE_x00696e6b__ */
static int eVer_clip(void *, unsigned, int *);
#if __ENABLE_x00696e6b__
static int ever_link(void *);
static int ever_miss(void *);
static int fini_link(void *);
#endif				/* __ENABLE_x00696e6b__ */
static int fini_port(void *, unsigned);
#if __ENABLE_x00696e6b__
static int head_link(void *);
static int init_link(void *);
#endif				/* __ENABLE_x00696e6b__ */
static int init_port(void *, unsigned);
static int line_clip(void *);
#if __ENABLE_x00696e6b__
static int line_lane(void *);
static int link_1f42(void *);
static int link_slip(void *);
static int miss_clip(void *);
static int miss_link(void *, unsigned *);
#endif				/* __ENABLE_x00696e6b__ */
#if __LINK_CELL_LOADER__
static int pLAy_dAta(void *);
static int pLAy_data(void *);
static int pLay_clip(void *);
#endif				/* __LINK_CELL_LOADER__ */
static int plAy_dAta(void *);
static int plAy_data(void *);
static int play_clip(void *);
static int play_dAta(void *, unsigned);
static int play_data(void *, unsigned);
static int play_deck(void *);
static int play_file(void *);
#if __ENABLE_x00696e6b__
static int play_lane(void *);
#endif				/* __ENABLE_x00696e6b__ */
static int play_list(void *);
#if __ENABLE_x00696e6b__
static int play_tile(void *);
#endif				/* __ENABLE_x00696e6b__ */
static int read_e0fa(void *);
static int skip_dAta(void *, unsigned, int *);
static int skip_data(void *, unsigned, int *);
#if __ENABLE_x00696e6b__
static int slIp_tail(void *);
#endif				/* __ENABLE_x00696e6b__ */
static int slip_away(void *);
static int slip_e1f4(void *);
#if __ENABLE_x00696e6b__
static int slip_eway(void *);
static int slip_iway(void *);
#endif				/* __ENABLE_x00696e6b__ */
#if __ENABLE_x00696e6b__
static int slip_jump(void *);
#endif				/* __ENABLE_x00696e6b__ */
static int slip_miss(void *);
static int slip_skip(void *);
#if __ENABLE_x00696e6b__
static int slip_tail(void *);
#endif				/* __ENABLE_x00696e6b__ */
static int slip_wait(void *);
static int wait_clip(void *);
static int wait_e0fa(void *);
static int wait_e1f4(void *);
static int wait_exit(void *);
static int wait_list(void *);
#if __LINK_CELL_LOADER__
static int zero_cell(void *, unsigned, int *);
#endif				/* __LINK_CELL_LOADER__ */

static const struct {
    int clear, (*shift) (void *);
} requests[] = {
/* *INDENT-OFF* */
    {	0,	NULL,		},
    {	0,	NULL,		},
    {	1,	NULL,		},
#if __ENABLE_x00696e6b__
    {	0,	slip_jump	},
#else
    {	0,	NULL,		},
#endif				/* __ENABLE_x00696e6b__ */
    {	0,	NULL,		},
    {	1,	slip_skip	},
    {	1,	NULL,		},
    {	1,	slip_wait	}
/* *INDENT-ON* */
};

#if __ENABLE_x00696e6b__
static int
dana_link(void *context, unsigned *miss_link)
{
    struct everlanes_type *everlanes_data;
    unsigned *mirror;

    everlanes_data = WINDOW(context).within.evermiss;
    mirror = everlanes_data->mirror;
    if (mirror[0] == 0x00696e6b) {
	struct x1f42a_type *x1f42a_data;
	unsigned reference;

	x1f42a_data = X1f42(context);
	if (Window(context)->first) {
	    unsigned nextframe;

	    nextframe = x1f42a_data->next_frame;
	    reference = mirror[7] + nextframe - x1f42a_data->frames;
	    if (nextframe < reference) {
		reference = nextframe;
	    }
	} else {
	    int new_start;
	    unsigned nextframe;

	    nextframe = x1f42a_data->next_frame;
	    new_start = mirror[5];
	    if (mirror[6]) {
		/*
		 * BUGS
		 *
		 *.next_frame should be added, frames should be subtracted
		 * (as for Window(context)->first).  yet our only client thinks
		 * not and it might just be right...
		 */
		reference = new_start;
		if (nextframe < reference) {
		    reference = nextframe;
		}
	    } else {
		unsigned cache;

		cache = C1f4(context) - STATE(context)[Mylene1f4_REAL_STATE];
		if (SIZE(context) < cache) {
		    cache += SIZE(context);
		}
		if (mirror[3]) {
		    new_start = -new_start;
		}
		new_start += REFERENCE(context) - cache / RELEASE(context);
		if (new_start < 0) {
		    reference = 0;
		} else if (nextframe < (unsigned) new_start) {
		    reference = nextframe;
		} else {
		    reference = new_start;
		}
	    }
	}

	*miss_link = reference;

	Window(context)->start2 = reference;
    }

    return 0;
}
#endif				/* __ENABLE_x00696e6b__ */


#if __ENABLE_x00696e6b__
static int
down_time(unsigned *major, unsigned *minor, unsigned PTS_0, unsigned PTS_1)
{
    unsigned second_a, second_f;

    second_a = *major;
    second_f = *minor;

    if (PTS_0) {
	double PTS_x, PTS_z;

	PTS_x = 1 << 16;
	PTS_x = PTS_x * PTS_x;
	PTS_x = PTS_x + PTS_1;
	PTS_z = PTS_x / 90000;
	PTS_x = PTS_x - PTS_z * 90000;
	PTS_x = PTS_x + 9;
	PTS_x = PTS_x * 200 / 18;
	second_a += PTS_z;
	second_f += PTS_x;
    } else {
	PTS_0 = PTS_1 / 90000;
	PTS_1 = PTS_1 - PTS_0 * 90000;
	PTS_1 = PTS_1 + 9;
	PTS_1 = PTS_1 * 200 / 18;
	second_a += PTS_0;
	second_f += PTS_1;
    }
    if (!(second_f < 1000000)) {
	second_a++;
	second_f -= 1000000;
    }

    *major = second_a;
    *minor = second_f;

    return 0;
}
#endif				/* __ENABLE_x00696e6b__ */


static int
eVer_clip(void *context, unsigned start2, int *clear)
{
    int status;
    struct target_type *target_data;
    unsigned count;

    target_data = &TARGET(context);

    count = start2 - REFERENCE(context);

#if __LINK_CELL_LOADER__
    if (target_data->zero) {
	status = zero_cell(context, count, clear);
	if (status) {
	} else {
	    REFERENCE(context) += count;
	}
    } else {
#endif				/* __LINK_CELL_LOADER__ */
	target_data->skip(STORAGE(context));

	do {
	    status = line_clip(context);
	    if (status) {
		break;
	    } else {
		count--;
		if (1) {
		    unsigned request;

		    request = Window(context)->request;
		    if (request) {
			if (request == Mylene1f4_EXIT_Mylene
#if __ENABLE_x00696e6b__
			    || request == Mylene1f4_JUMP_Mylene
#endif				/* __ENABLE_x00696e6b__ */
			    || request == Mylene1f4_SKIP_Mylene
#if __ENABLE_x00696e6b__
			    || request == Mylene1f4_STOP_Mylene
#endif				/* __ENABLE_x00696e6b__ */
			    || request == Mylene1f4_WAIT_Mylene) {
			    *clear = 1;
			    break;
			}
		    }
		}
	    }
	} while (count);

	if (status) {
	} else {
	    target_data->seek(STORAGE(context));
	}
#if __LINK_CELL_LOADER__
    }
#endif				/* __LINK_CELL_LOADER__ */

    return status;
}


#if __ENABLE_x00696e6b__
static int
ever_link(void *context)
{
    int status = 0;
    struct everlanes_type *everlanes_data;
    unsigned *mirror;

    everlanes_data = WINDOW(context).within.evermiss;
    mirror = everlanes_data->mirror;
    if (mirror[0] == 0x00696e6b) {
	struct x1f42a_type *x1f42a_data;

	x1f42a_data = X1f42(context);
	if (!x1f42a_data) {
	    status = 1;
	} else {
	    /*
	     * No idea why testing for MPEG helps...
	     */
	    if (0) {
		int excess;
		struct sequel_type *sequel_data;

		excess = Mylene1f4_find_sequel
		    (context, Mylene1f4_XMPEG1_SEQUEL, (void *) &sequel_data);
		if (excess) {
		} else {
		    status = 1;
		}
	    }
	}
	if (!status) {
	    mirror[2] = Window(context)->start2;
	    if (!Mylene1f4_send_lanes(context, 0200, NULL)) {
		status = 1;
	    } else {
		Window(context)->miss_link = 128;

		Window(context)->state2 = 1;
	    }
	}
    } else if (mirror[0] == 0x01696e6b) {
	if (1) {
	    if (everlanes_data->bits & (1 << 000)) {
		Window(context)->miss_link = 128;

		Window(context)->state2 = 1;
	    }
	}
    } else {
	if (1) {
	    if (1) {
		Window(context)->state2 = 0;
	    }

	    Window(context)->start2 = START2(context);
	}
    }

    return status;
}
#endif				/* __ENABLE_x00696e6b__ */


#if __ENABLE_x00696e6b__
static int
ever_miss(void *context)
{
    int status = 0;

    if (0x00696e6b) {
	if (1) {
	    Window(context)->detached = 0;
	    if (1) {
		Window(context)->state2 = 1;
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_x00696e6b__ */


#if __ENABLE_x00696e6b__
static int
fini_link(void *context)
{
    int status;

    if (Window(context)->state2 != 2) {
	status = 1;
    } else {
	struct everlanes_type *everlanes_data;
	unsigned *mirror;

	everlanes_data = WINDOW(context).within.evermiss;
	mirror = everlanes_data->mirror;
	if (mirror[0] != 0x00696e6b) {
	    Window(context)->state2 = 3;

	    /*
	     * STUFF
	     *
	     *.PTS are to be considered whenever the controller will be
	     * informed about link finish...
	     */

	    status = 0;
	} else {
	    unsigned char *input;

	    input = INPUT(context);
	    input[0] ^= 64;
	    status = !Mylene1f4_send_lanes(context, 0203, input);
	    if (status) {
	    } else {
		Window(context)->detached = 0;
		Window(context)->state2 = 3;
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_x00696e6b__ */


static int
fini_port(void *context, unsigned index)
{
    return TARGET(context).fini(LOADER(context), index, &STORAGE(context));

}


#if __ENABLE_x00696e6b__
static int
head_link(void *context)
{
    if (0) {
    } else {
	struct x1f42a_type *x1f42a_data;

	x1f42a_data = Window(context)->x1f42a;
	if (true(x1f42a_data)) {
	    unsigned skip;

	    skip = x1f42a_data->next_frame - x1f42a_data->frames;
	    if (skip) {
		unsigned frame_rate_d, frame_rate_n;

		frame_rate_d = x1f42a_data->frame_rate_d;
		frame_rate_n = x1f42a_data->frame_rate_n;
		if (~(frame_rate_d & frame_rate_n)) {
# if SIZEOF_UNSIGNED_LONG != 8
		    double head, stream_a, stream_f;
# endif					/* SIZEOF_UNSIGNED_LONG != 8 */
		    unsigned second_a, second_f, system_a, system_f;
		    unsigned char *system;
# if SIZEOF_UNSIGNED_LONG == 8
		    unsigned long head;
# endif					/* SIZEOF_UNSIGNED_LONG == 8 */

		    system = INPUT(context);

		    system[0] ^= 32;

		    second_a = CARDINAL(system, 0);
		    second_f = CARDINAL(system, 4);

		    head = skip;

# if SIZEOF_UNSIGNED_LONG == 8
		    head *= frame_rate_d << 1;
		    head *= 1000000;
		    head += frame_rate_n;
		    head /= frame_rate_n << 1;
# else
		    head *= frame_rate_d;
		    head /= frame_rate_n;
# endif					/* SIZEOF_UNSIGNED_LONG == 8 */

# if SIZEOF_UNSIGNED_LONG == 8
		    system_a = head / 1000000;
		    system_f = head % 1000000;
# else
		    stream_f = modf(head, &stream_a);
		    system_a = stream_a;
		    system_f = stream_f * 1000000 + .5;
		    if (system_f < 1000000) {
		    } else {
			system_a++;
			system_f -= 1000000;
		    }
# endif					/* SIZEOF_UNSIGNED_LONG == 8 */

		    do {
			if (second_f < system_f) {
			    if (!(system_a < second_a)) {
				/*
				 * BUGS
				 *
				 *.this is an error and should not be
				 * tolerated (as it currently is)
				 */
				break;
			    } else {
				second_a--;
				second_f += 1000000;
			    }
			} else {
			    if (second_a < system_a) {
				/*
				 * BUGS
				 *
				 *.this is an error and should not be
				 * tolerated (as it currently is)
				 */
				break;
			    }
			}

			second_a -= system_a;
			second_f -= system_f;

			SET_CARDINAL(system, 0, second_a);
			SET_CARDINAL(system, 4, second_f);
		    } while (0);

		    system[0] ^= 32;
		}
	    }
	}
    }

    return 0;
}
#endif				/* __ENABLE_x00696e6b__ */


#if __ENABLE_x00696e6b__
static int
init_link(void *context)
{
    int status;

    if ((Window(context)->state2 & ~2) != 1) {
	status = 1;
    } else {
	struct everlanes_type *everlanes_data;
	unsigned *mirror;

	everlanes_data = WINDOW(context).within.evermiss;
	mirror = everlanes_data->mirror;
	if (mirror[0] == 0x00696e6b) {
	    head_link(context);
	} else {
	    Window(context)->detached = 0;

	    head_link(context);

	    if (mirror[4] & 1) {
	    } else {
		struct x1f42a_type *x1f42a_data;

		x1f42a_data = Window(context)->x1f42a;
		if (true(x1f42a_data)) {
		    unsigned PTS_0, PTS_1;

		    PTS_0 = x1f42a_data->PTS_0;
		    PTS_1 = x1f42a_data->PTS_1;
		    if (~(PTS_0 & PTS_1)) {
			if (1) {
			    unsigned second_a, second_f;
			    unsigned char *system;

			    system = INPUT(context);

			    system[0] ^= 32;

			    second_a = CARDINAL(system, 0);
			    second_f = CARDINAL(system, 4);

			    down_time(&second_a, &second_f, PTS_0, PTS_1);

			    SET_CARDINAL(system, 0, second_a);
			    SET_CARDINAL(system, 4, second_f);

			    system[0] ^= 32;
			}
		    }
		}
	    }
	}
	{
	    unsigned char *input;

	    input = INPUT(context);
	    input[0] ^= 32;
	    status = !Mylene1f4_send_lanes(context, 0202, input);
	    if (status) {
	    } else {
		Window(context)->state2 = 2;
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_x00696e6b__ */


static int
init_port(void *context, unsigned index)
{
    int status;
    struct target_type *target_data;
    void *loader;

    loader = LOADER(context);

    target_data = &TARGET(context);
    status = target_data->init(loader, index, &STORAGE(context));
#if __ENABLE_x00696e6b__ || __FIX_0fa_WAIT__ || __TYPE_TARGET__
    if (status) {
    } else {
#endif				/* __ENABLE_x00696e6b__ || __FIX_0fa_WAIT... */
#if __TYPE_TARGET__
	target_data->type(loader, target_data);
#endif				/* __TYPE_TARGET__ */

#if __ENABLE_x00696e6b__
	if (!Window(context)->first) {
	    status = ever_miss(context);
	} else {
	    status = ever_link(context);
	}
	if (status) {
	    TARGET(context).fini(LOADER(context), index, &STORAGE(context));
	} else {
	    if (Window(context)->miss_link) {
		Window(context)->first = 0;
	    }
#endif				/* __ENABLE_x00696e6b__ */
#if __FIX_0fa_WAIT__
	    Mylene1f4_miss_wait(context, WAIT_COUNT, &Window(context)->wait);
#endif				/* __FIX_0fa_WAIT__ */
#if __ENABLE_x00696e6b__
	}
#endif				/* __ENABLE_x00696e6b__ */
#if __ENABLE_x00696e6b__ || __FIX_0fa_WAIT__ || __TYPE_TARGET__
    }
#endif				/* __ENABLE_x00696e6b__ || __FIX_0fa_WAIT... */

    return status;
}


static int
lane_deck(void *context)
{
    int status;

    status = Output(context)->init(context);
    if (status) {
    } else {
	int excess;

	status = play_deck(context);
	if (status) {
	    Output(context)->kill(context);
	}
	excess = Output(context)->fini(context);
	if (status) {
	} else {
	    status = excess;
	}
    }

    return status;
}


static int
line_clip(void *context)
{
    int do_decode = 1, fd_0, fd_1, pending_0fa, pending_1f4, status;
    struct everlanes_type *everlanes_data;
    struct window_type *window_data;
    unsigned detached, underlay;

    window_data = WINDOW(context).within.window;
    everlanes_data = WINDOW(context).within.evermiss;

    fd_0 = Mylene1ST(context);
    fd_1 = Mylene2ND(context);

    pending_0fa = Z0fa(context);

    detached = window_data->detached;
    underlay = window_data->underlay;

    if (detached | underlay) {
	pending_1f4 = 0;
    } else {
	pending_1f4 = everlanes_data->incoming;
    }

    while (1) {
	fd_set fd_set_0, *fd_set_0_select, fd_set_1, *fd_set_1_select;
	struct timeval *selector, timeout;
	unsigned extend = 0, nfds = 0;

	if (SIGCHLD_received(context)) {
	    status = Mylene1f4_yway_mylene0fa(context, 0);
	    if (status) {
		break;
	    } else {
		if (1) {
		    break;
		}
	    }
	}

	if (pending_1f4) {
	    fd_set_0_select = NULL;
	} else if (detached) {
	    fd_set_0_select = NULL;
	} else if (underlay) {
	    fd_set_0_select = NULL;
	} else {
	    fd_set_0_select = &fd_set_0;
	    FD_ZERO(fd_set_0_select);
	    nfds = STDIN_FILENO;
	    FD_SET(STDIN_FILENO, fd_set_0_select);
	    extend = 1;
	}
	if (!pending_0fa) {
	    if (!fd_set_0_select) {
		fd_set_0_select = &fd_set_0;
		FD_ZERO(fd_set_0_select);
		nfds = fd_0;
	    } else {
		if (nfds < fd_0) {
		    nfds = fd_0;
		}
	    }
	    FD_SET(fd_0, fd_set_0_select);
	    extend = 1;
	}
	if (!OUTGOING(context)) {
	    fd_set_1_select = NULL;
	} else {
	    fd_set_1_select = &fd_set_1;
	    FD_ZERO(fd_set_1_select);
	    FD_SET(fd_1, fd_set_1_select);
	    if (nfds < fd_1) {
		nfds = fd_1;
	    }
	    extend = 1;
	}
	if (underlay) {
	} else if (everlanes_data->outgoing) {
	    if (!fd_set_1_select) {
		fd_set_1_select = &fd_set_1;
		FD_ZERO(fd_set_1_select);
		nfds = STDOUT_FILENO;
	    } else {
		if (nfds < STDOUT_FILENO) {
		    nfds = STDOUT_FILENO;
		}
	    }
	    FD_SET(STDOUT_FILENO, fd_set_1_select);
	    extend = 1;
	}

	if (1) {
	    selector = &timeout;
	    timeout.tv_sec = 0;
	    timeout.tv_usec = 0;
	}

	nfds += extend;

	status = select
	    (nfds, fd_set_0_select, fd_set_1_select, NULL, selector);
	if (status == -1) {
	    if (errno != EINTR) {
		Mylene1f4_fprintf_error
		    (context, "%s: failed select: %s\n", Mylene1f4(context),
		     strerror(errno));
		break;
	    }
	} else {
	    status = 0;

	    if (fd_set_0_select) {
		if (FD_ISSET(fd_0, fd_set_0_select)) {
		    pending_0fa = 1;
		}
		if (FD_ISSET(STDIN_FILENO, fd_set_0_select)) {
		    pending_1f4 = 1;
		}
	    }
	    if (pending_0fa) {
		if (1) {
		    status = read_e0fa(context);
		    if (status) {
			break;
		    }
		}
	    }
	    if (pending_1f4) {
		if (1) {
		    if (1) {
			if (__slip_away(context)) {
			    unsigned request;

			    request = window_data->request;
			    if (request == Mylene1f4_DEUX_Mylene) {
				status = slip_e1f4(context);
				if (status) {
				    break;
				}
			    } else {
				switch (request) {
				case Mylene1f4_EXIT_Mylene:
#if __ENABLE_x00696e6b__
				case Mylene1f4_JUMP_Mylene:
#endif				/* __ENABLE_x00696e6b__ */
				case Mylene1f4_SKIP_Mylene:
#if __ENABLE_x00696e6b__
				case Mylene1f4_STOP_Mylene:
#endif				/* __ENABLE_x00696e6b__ */
				case Mylene1f4_WAIT_Mylene:
				    do_decode = 0;
				}
			    }
			}
		    }
		}
	    }

	    if (fd_set_1_select) {
		if (FD_ISSET(fd_1, fd_set_1_select)) {
		    status = Mylene1f4_push_mylene0fa(context);
		    if (status) {
			break;
		    }
		}
		if (FD_ISSET(STDOUT_FILENO, fd_set_1_select)) {
		    status = Mylene1f4_push_lanes(context);
		    if (status) {
			break;
		    }
		}
	    }

	    break;
	}
    }
    if (status) {
    } else {
	do {
	    if (do_decode == 1) {
		status = TARGET(context).ever(STORAGE(context));
		if (status) {
		    break;
		} else {
		    unsigned reference;

		    reference = REFERENCE(context);
		    reference++;
		    Mylene1f4_post_time(context, reference);
		    REFERENCE(context) = reference;
		}
	    }

	    if (PROGRESS2(context)) {
		Mylene1f4_run_format(context, PROGRESS2(context));
	    }
	} while (0);
    }

    return status;
}


#if __ENABLE_x00696e6b__
static int
line_lane(void *context)
{
    int status = 1;

    do {
	struct x1f42a_type *x1f42a_data;

	x1f42a_data = Window(context)->x1f42a;
# if 0
	if (!~(x1f42a_data->PTS_0 & x1f42a_data->PTS_1)) {
	    Mylene1f4_fprintf_error
		(context, "%s: missing P(resentation)T(ime)S(tamp) for `%s'\n",
		 Mylene1f4(context), SOURCE(context)._.s1.path);
	    break;
	}
# endif				/* 0 */
	if (!~(x1f42a_data->frame_rate_d & x1f42a_data->frame_rate_n)) {
	    Mylene1f4_fprintf_error
		(context, "%s: missing frame rate for `%s'\n",
		 Mylene1f4(context), SOURCE(context)._.s1.path);
	    break;
	}

	status = 0;
    } while (0);

    return status;
}
#endif				/* __ENABLE_x00696e6b__ */


#if __ENABLE_x00696e6b__
static int
link_1f42(void *context)
{
    struct everlanes_type *everlanes_data;
    unsigned *mirror;

    everlanes_data = WINDOW(context).within.evermiss;
    mirror = everlanes_data->mirror;
    if (mirror[0] == 0x00696e6b) {
	ENTRY2(context) = mirror[5];
	START3(context) = dana_link;
    } else {
	ENTRY2(context) = ~0;
	START3(context) = NULL;
    }

    return 0;
}
#endif				/* __ENABLE_x00696e6b__ */


#if __ENABLE_x00696e6b__
static int
link_slip(void *context)
{
    struct everlanes_type *everlanes_data;
    unsigned *mirror;

    everlanes_data = WINDOW(context).within.evermiss;
    mirror = everlanes_data->mirror;
    if (1) {
	ENTRY2(context) = mirror[5];
	START3(context) = miss_link;
    }

    return 0;
}
#endif				/* __ENABLE_x00696e6b__ */


#if __ENABLE_x00696e6b__
static int
miss_clip(void *context)
{
    int fd_0, fd_1, pending_0fa, pending_1f4, status;
    struct everlanes_type *everlanes_data;
    struct window_type *window_data;
    unsigned detached;

    window_data = WINDOW(context).within.window;
    everlanes_data = WINDOW(context).within.evermiss;

    fd_0 = Mylene1ST(context);
    fd_1 = Mylene2ND(context);

    pending_0fa = Z0fa(context);

    detached = window_data->detached;

    if (detached) {
	pending_1f4 = 0;
    } else {
	pending_1f4 = everlanes_data->incoming;
    }

    while (1) {
	fd_set fd_set_0, *fd_set_0_select, fd_set_1, *fd_set_1_select;
	struct timeval *selector, timeout;
	unsigned extend = 0, nfds = 0;

	if (SIGCHLD_received(context)) {
	    int expect;

	    expect = Window(context)->request == Mylene1f4_EXIT_Mylene;
	    status = Mylene1f4_yway_mylene0fa(context, expect);
	    if (status) {
		break;
	    } else {
		if (expect) {
		    if (PID(context) == -1) {
			break;
		    }
		} else {
		    break;
		}
	    }
	}

	if (pending_1f4) {
	    fd_set_0_select = NULL;
	} else if (detached) {
	    fd_set_0_select = NULL;
	} else {
	    fd_set_0_select = &fd_set_0;
	    FD_ZERO(fd_set_0_select);
	    nfds = STDIN_FILENO;
	    FD_SET(STDIN_FILENO, fd_set_0_select);
	    extend = 1;
	}
	if (!pending_0fa) {
	    if (!fd_set_0_select) {
		fd_set_0_select = &fd_set_0;
		FD_ZERO(fd_set_0_select);
		nfds = fd_0;
	    } else {
		if (nfds < fd_0) {
		    nfds = fd_0;
		}
	    }
	    FD_SET(fd_0, fd_set_0_select);
	    extend = 1;
	}
	if (!OUTGOING(context)) {
	    fd_set_1_select = NULL;
	} else {
	    fd_set_1_select = &fd_set_1;
	    FD_ZERO(fd_set_1_select);
	    FD_SET(fd_1, fd_set_1_select);
	    if (nfds < fd_1) {
		nfds = fd_1;
	    }
	    extend = 1;
	}
	if (everlanes_data->outgoing) {
	    if (!fd_set_1_select) {
		fd_set_1_select = &fd_set_1;
		FD_ZERO(fd_set_1_select);
		nfds = STDOUT_FILENO;
	    } else {
		if (nfds < STDOUT_FILENO) {
		    nfds = STDOUT_FILENO;
		}
	    }
	    FD_SET(STDOUT_FILENO, fd_set_1_select);
	    extend = 1;
	}

	if (pending_0fa | pending_1f4) {
	    selector = &timeout;
	    SECOND_a(timeout) = 0;
	    SECOND_f(timeout) = 0;
	} else {
	    unsigned second_f;

	    second_f = Window(context)->second_f;
	    if (!~second_f) {
		selector = NULL;
	    } else {
		unsigned second_a;

		selector = &timeout;

		second_a = Window(context)->second_a;
		if (!(second_a | second_f)) {
		    SECOND_a(timeout) = 0;
		    SECOND_f(timeout) = 0;
		} else {
		    struct timeval now;

		    gettimeofday(&now, NULL);
		    if (second_a < SECOND_a(now)) {
			SECOND_a(timeout) = 0;
			SECOND_f(timeout) = 0;
			Second_a(context) = 0;
			Second_f(context) = 0;
		    } else {
			second_a -= SECOND_a(now);
			if (second_a) {
			    if (second_f < SECOND_f(now)) {
				second_f += 1000000;
				second_a--;
			    }
			    {
				SECOND_a(timeout) = second_a;
				SECOND_f(timeout) =
				    second_f - SECOND_f(now);
			    }
			} else {
			    if (SECOND_f(now) < second_f) {
				SECOND_a(timeout) = 0;
				SECOND_f(timeout) =
				    second_f - SECOND_f(now);
			    } else {
				SECOND_a(timeout) = 0;
				SECOND_f(timeout) = 0;
				Second_a(context) = 0;
				Second_f(context) = 0;
			    }
			}
		    }
		}
	    }
	}

	nfds += extend;

	status = select
	    (nfds, fd_set_0_select, fd_set_1_select, NULL, selector);
	if (status == -1) {
	    if (errno != EINTR) {
		Mylene1f4_fprintf_error
		    (context, "%s: failed select: %s\n", Mylene1f4(context),
		     strerror(errno));
		break;
	    }
	} else {
	    if (status) {
		status = 0;
	    } else {
		if (!Window(context)->second_f) {
		    if (!Window(context)->second_a) {
			Window(context)->request =
			    Mylene1f4_ZERO_Mylene;
		    }
		}
	    }

	    if (fd_set_0_select) {
		if (FD_ISSET(fd_0, fd_set_0_select)) {
		    pending_0fa = 1;
		}
		if (FD_ISSET(STDIN_FILENO, fd_set_0_select)) {
		    pending_1f4 = 1;
		}
	    }
	    if (pending_0fa) {
		if (1) {
		    status = read_e0fa(context);
		    if (status) {
			break;
		    } else {
			if (Mylene1ST(context) == -1) {
			    break;
			}
		    }
		}
	    }
	    if (pending_1f4) {
		if (1) {
		    if (1) {
			if (slip_eway(context)) {
			    switch (window_data->request) {
			    case Mylene1f4_EXIT_Mylene:
				status = wait_exit(context);
				break;
			    }
			    if (status) {
				break;
			    }
			}
		    }
		}
	    }

	    if (fd_set_1_select) {
		if (FD_ISSET(fd_1, fd_set_1_select)) {
		    status = Mylene1f4_push_mylene0fa(context);
		    if (status) {
			break;
		    }
		}
		if (FD_ISSET(STDOUT_FILENO, fd_set_1_select)) {
		    status = Mylene1f4_push_lanes(context);
		    if (status) {
			break;
		    }
		}
	    }

	    break;
	}
    }

    return status;
}
#endif				/* __ENABLE_x00696e6b__ */


#if __ENABLE_x00696e6b__
static int
miss_link(void *context, unsigned *miss_link)
{
    struct everlanes_type *everlanes_data;
    unsigned *mirror, reference;

    everlanes_data = WINDOW(context).within.evermiss;
    mirror = everlanes_data->mirror;
    reference = mirror[7];
    *miss_link = reference;
    Window(context)->start2 = reference;

    return 0;
}
#endif				/* __ENABLE_x00696e6b__ */


#if __LINK_CELL_LOADER__
static int
pLAy_dAta(void *context)
{
    int status = 0;
    unsigned tail;

    tail = TAIL(context);
    while (REFERENCE(context) < tail) {
	status = pLay_clip(context);
	if (status) {
	    break;
	} else {
	    unsigned request;

	    request = Window(context)->request;
	    if (request) {
		if (request == Mylene1f4_EXIT_Mylene
# if __ENABLE_x00696e6b__
		    || request == Mylene1f4_JUMP_Mylene
# endif				/* __ENABLE_x00696e6b__ */
		    || request == Mylene1f4_SKIP_Mylene
# if __ENABLE_x00696e6b__
		    || request == Mylene1f4_STOP_Mylene
# endif				/* __ENABLE_x00696e6b__ */
		    || request == Mylene1f4_WAIT_Mylene) {
		    break;
		}
	    }
	}
    }
    if (status) {
    } else {
	COMPLETED(context) = 1;
    }

    return status;
}
#endif				/* __LINK_CELL_LOADER__ */


#if __LINK_CELL_LOADER__
static int
pLAy_data(void *context)
{
    int status;

    do {
	status = pLay_clip(context);
	if (status) {
	    break;
	} else {
	    unsigned request;

	    request = Window(context)->request;
	    if (request) {
		if (request == Mylene1f4_EXIT_Mylene
		    || request == Mylene1f4_SKIP_Mylene
		    || request == Mylene1f4_WAIT_Mylene) {
		    break;
		}
	    }
	}
    } while (1);
    if (COMPLETED(context)) {
	status = 0;
    }

    return status;
}
#endif				/* __LINK_CELL_LOADER__ */


#if __LINK_CELL_LOADER__
static int
pLay_clip(void *context)
{
    int do_decode, fd_0, fd_1, pending_0fa, pending_1f4, status;
    struct everlanes_type *everlanes_data;
    struct window_type *window_data;
    unsigned detached, underlay, wait_d = 0;

    window_data = WINDOW(context).within.window;
    everlanes_data = WINDOW(context).within.evermiss;

    fd_0 = Mylene1ST(context);
    fd_1 = Mylene2ND(context);

    pending_0fa = Z0fa(context);

    detached = window_data->detached;
    underlay = window_data->underlay;

    if (detached | underlay) {
	pending_1f4 = 0;
    } else {
	pending_1f4 = everlanes_data->incoming;
    }

    if (LINK(context)) {
	do_decode = 0;
    } else {
	unsigned cache;

	cache = C1f4(context) - STATE(context)[1];
	if (SIZE(context) < cache) {
	    cache += SIZE(context);
	}
	do {
	    cache = SIZE(context) - cache;
	    if (RELEASE(context) < cache) {
		cache = STATE(context)[Mylene1f4_CELL_STATE] - C1f4(context);
		if (SIZE(context) < cache) {
		    cache += SIZE(context);
		}
		if (cache < RELEASE(context)) {
		} else {
		    do_decode = 1;
		    wait_d = 0;
		    break;
		}
	    }
	    if (pending_0fa | pending_1f4) {
		do_decode = 0;
	    } else if (WIRE(context)) {
		do_decode = 0;
	    } else {
# if __FIX_0fa_WAIT__
		wait_d = Window(context)->wait;
# else
#  if SIZEOF_UNSIGNED_LONG == 8
		unsigned long wait;

		wait = RELEASE(context) * WAIT_COUNT - cache;
		wait *= 1000000;
		wait /= (SAMPLING(context) * CHANNELS(context) << 1);
#  else
		double wait;

		wait = RELEASE(context) * WAIT_COUNT - cache;
		wait /= (SAMPLING(context) * CHANNELS(context) << 1);
		wait *= 1000000;
#  endif				/* SIZEOF_UNSIGNED_LONG == 8 */
		wait_d = wait;
		if (131072 < wait_d) {
		    wait_d = 131072;
		}
# endif				/* __FIX_0fa_WAIT__ */

		do_decode = 2;
	    }
	} while (0);
    }

    while (1) {
	fd_set fd_set_0, *fd_set_0_select, fd_set_1, *fd_set_1_select;
	struct timeval *selector, timeout;
	unsigned extend = 0, nfds = 0;

	if (SIGCHLD_received(context)) {
	    status = Mylene1f4_yway_mylene0fa(context, 0);
	    if (status) {
		break;
	    } else {
		do_decode = 0;
		if (1) {
		    break;
		}
	    }
	}

	if (pending_1f4) {
	    fd_set_0_select = NULL;
	} else if (detached) {
	    fd_set_0_select = NULL;
	} else if (underlay) {
	    fd_set_0_select = NULL;
	} else {
	    fd_set_0_select = &fd_set_0;
	    FD_ZERO(fd_set_0_select);
	    nfds = STDIN_FILENO;
	    FD_SET(STDIN_FILENO, fd_set_0_select);
	    extend = 1;
	}
	if (!pending_0fa) {
	    if (!fd_set_0_select) {
		fd_set_0_select = &fd_set_0;
		FD_ZERO(fd_set_0_select);
		nfds = fd_0;
	    } else {
		if (nfds < fd_0) {
		    nfds = fd_0;
		}
	    }
	    FD_SET(fd_0, fd_set_0_select);
	    extend = 1;
	}
	if (!OUTGOING(context)) {
	    fd_set_1_select = NULL;
	} else {
	    fd_set_1_select = &fd_set_1;
	    FD_ZERO(fd_set_1_select);
	    FD_SET(fd_1, fd_set_1_select);
	    if (nfds < fd_1) {
		nfds = fd_1;
	    }
	    extend = 1;
	}
	if (underlay) {
	} else if (everlanes_data->outgoing) {
	    if (!fd_set_1_select) {
		fd_set_1_select = &fd_set_1;
		FD_ZERO(fd_set_1_select);
		nfds = STDOUT_FILENO;
	    } else {
		if (nfds < STDOUT_FILENO) {
		    nfds = STDOUT_FILENO;
		}
	    }
	    FD_SET(STDOUT_FILENO, fd_set_1_select);
	    extend = 1;
	}

	if (pending_0fa | pending_1f4) {
	    selector = &timeout;
	    timeout.tv_sec = 0;
	    timeout.tv_usec = 0;
	} else {
	    if (!do_decode) {
		selector = NULL;
	    } else {
		selector = &timeout;
		timeout.tv_sec = 0;
		timeout.tv_usec = wait_d;
	    }
	}

	nfds += extend;

	status = select
	    (nfds, fd_set_0_select, fd_set_1_select, NULL, selector);
	if (status == -1) {
	    if (errno != EINTR) {
		Mylene1f4_fprintf_error
		    (context, "%s: failed select: %s\n", Mylene1f4(context),
		     strerror(errno));
		break;
	    }
	} else {
	    status = 0;

	    if (fd_set_0_select) {
		if (FD_ISSET(fd_0, fd_set_0_select)) {
		    pending_0fa = 1;
		}
		if (FD_ISSET(STDIN_FILENO, fd_set_0_select)) {
		    pending_1f4 = 1;
		}
	    }
	    if (pending_0fa) {
		if (1) {
		    status = read_e0fa(context);
		    if (status) {
			break;
		    }
		}
	    }
	    if (pending_1f4) {
		if (1) {
		    if (1) {
			if (__slip_away(context)) {
			    unsigned request;

			    request = window_data->request;
			    if (request == Mylene1f4_DEUX_Mylene) {
				status = slip_e1f4(context);
				if (status) {
				    break;
				}
			    } else {
				switch (request) {
				case Mylene1f4_EXIT_Mylene:
# if __ENABLE_x00696e6b__
				case Mylene1f4_JUMP_Mylene:
# endif				/* __ENABLE_x00696e6b__ */
				case Mylene1f4_SKIP_Mylene:
# if __ENABLE_x00696e6b__
				case Mylene1f4_STOP_Mylene:
# endif				/* __ENABLE_x00696e6b__ */
				case Mylene1f4_WAIT_Mylene:
				    do_decode = 0;
				}
			    }
			}
		    }
		}
	    }

	    if (fd_set_1_select) {
		if (FD_ISSET(fd_1, fd_set_1_select)) {
		    status = Mylene1f4_push_mylene0fa(context);
		    if (status) {
			break;
		    }
		}
		if (FD_ISSET(STDOUT_FILENO, fd_set_1_select)) {
		    status = Mylene1f4_push_lanes(context);
		    if (status) {
			break;
		    }
		}
	    }

	    break;
	}
    }
    if (status) {
    } else {
	do {
	    if (do_decode == 1) {
		if (0) {
		} else {
		    unsigned e1f4_index;

		    e1f4_index = C1f4(context);
		    e1f4_index += RELEASE(context);
		    if (e1f4_index == SIZE(context)) {
			e1f4_index = 0;
		    }
		    C1f4(context) = e1f4_index;
		    STATE(context)[0] = e1f4_index;

		    REFERENCE(context)++;
		}
	    }

	    if (PROGRESS2(context)) {
		Mylene1f4_run_format(context, PROGRESS2(context));
	    }
	} while (0);
    }

    return status;
}
#endif				/* __LINK_CELL_LOADER__ */


static int
plAy_dAta(void *context)
{
    int status = 0;
    unsigned tail;

    tail = TAIL(context);
    while (REFERENCE(context) < tail) {
	status = play_clip(context);
	if (status) {
	    break;
	} else {
	    unsigned request;

	    request = Window(context)->request;
	    if (request) {
		if (request == Mylene1f4_EXIT_Mylene
#if __ENABLE_x00696e6b__
		    || request == Mylene1f4_JUMP_Mylene
#endif				/* __ENABLE_x00696e6b__ */
		    || request == Mylene1f4_SKIP_Mylene
#if __ENABLE_x00696e6b__
		    || request == Mylene1f4_STOP_Mylene
#endif				/* __ENABLE_x00696e6b__ */
		    || request == Mylene1f4_WAIT_Mylene) {
		    break;
		}
	    }
	}
    }
    if (status) {
    } else {
	COMPLETED(context) = 1;
    }

    return status;
}


static int
plAy_data(void *context)
{
    int status;

    do {
	status = play_clip(context);
	if (status) {
	    break;
	} else {
	    unsigned request;

	    request = Window(context)->request;
	    if (request) {
		if (request == Mylene1f4_EXIT_Mylene
		    || request == Mylene1f4_SKIP_Mylene
		    || request == Mylene1f4_WAIT_Mylene) {
		    break;
		}
	    }
	}
    } while (1);
    if (COMPLETED(context)) {
	status = 0;
    }

    return status;
}


static int
play_clip(void *context)
{
    int do_decode, fd_0, fd_1, pending_0fa, pending_1f4, status;
    struct everlanes_type *everlanes_data;
    struct window_type *window_data;
    unsigned detached, underlay, wait_d = 0;

    window_data = WINDOW(context).within.window;
    everlanes_data = WINDOW(context).within.evermiss;

    fd_0 = Mylene1ST(context);
    fd_1 = Mylene2ND(context);

    pending_0fa = Z0fa(context);

    detached = window_data->detached;
    underlay = window_data->underlay;

    if (detached | underlay) {
	pending_1f4 = 0;
    } else {
	pending_1f4 = everlanes_data->incoming;
    }

    if (LINK(context)) {
	do_decode = 0;
    } else {
	unsigned cache;

	cache = C1f4(context) - STATE(context)[1];
	if (SIZE(context) < cache) {
	    cache += SIZE(context);
	}
	{
	    cache = SIZE(context) - cache;
	    if (RELEASE(context) < cache) {
		do_decode = 1;
		wait_d = 0;
	    } else if (pending_0fa | pending_1f4) {
		do_decode = 0;
	    } else if (WIRE(context)) {
		do_decode = 0;
	    } else {
#if __FIX_0fa_WAIT__
		wait_d = Window(context)->wait;
#else
# if SIZEOF_UNSIGNED_LONG == 8
		unsigned long wait;

		wait = RELEASE(context) * WAIT_COUNT - cache;
		wait *= 1000000;
		wait /= (SAMPLING(context) * CHANNELS(context) << 1);
# else
		double wait;

		wait = RELEASE(context) * WAIT_COUNT - cache;
		wait /= (SAMPLING(context) * CHANNELS(context) << 1);
		wait *= 1000000;
# endif					/* SIZEOF_UNSIGNED_LONG == 8 */
		wait_d = wait;
		if (131072 < wait_d) {
		    wait_d = 131072;
		}
#endif				/* __FIX_0fa_WAIT__ */

		do_decode = 2;
	    }
	}
    }

    while (1) {
	fd_set fd_set_0, *fd_set_0_select, fd_set_1, *fd_set_1_select;
	struct timeval *selector, timeout;
	unsigned extend = 0, nfds = 0;

	if (SIGCHLD_received(context)) {
	    status = Mylene1f4_yway_mylene0fa(context, 0);
	    if (status) {
		break;
	    } else {
		do_decode = 0;
		if (1) {
		    break;
		}
	    }
	}

	if (pending_1f4) {
	    fd_set_0_select = NULL;
	} else if (detached) {
	    fd_set_0_select = NULL;
	} else if (underlay) {
	    fd_set_0_select = NULL;
	} else {
	    fd_set_0_select = &fd_set_0;
	    FD_ZERO(fd_set_0_select);
	    nfds = STDIN_FILENO;
	    FD_SET(STDIN_FILENO, fd_set_0_select);
	    extend = 1;
	}
	if (!pending_0fa) {
	    if (!fd_set_0_select) {
		fd_set_0_select = &fd_set_0;
		FD_ZERO(fd_set_0_select);
		nfds = fd_0;
	    } else {
		if (nfds < fd_0) {
		    nfds = fd_0;
		}
	    }
	    FD_SET(fd_0, fd_set_0_select);
	    extend = 1;
	}
	if (!OUTGOING(context)) {
	    fd_set_1_select = NULL;
	} else {
	    fd_set_1_select = &fd_set_1;
	    FD_ZERO(fd_set_1_select);
	    FD_SET(fd_1, fd_set_1_select);
	    if (nfds < fd_1) {
		nfds = fd_1;
	    }
	    extend = 1;
	}
	if (underlay) {
	} else if (everlanes_data->outgoing) {
	    if (!fd_set_1_select) {
		fd_set_1_select = &fd_set_1;
		FD_ZERO(fd_set_1_select);
		nfds = STDOUT_FILENO;
	    } else {
		if (nfds < STDOUT_FILENO) {
		    nfds = STDOUT_FILENO;
		}
	    }
	    FD_SET(STDOUT_FILENO, fd_set_1_select);
	    extend = 1;
	}

	if (pending_0fa | pending_1f4) {
	    selector = &timeout;
	    timeout.tv_sec = 0;
	    timeout.tv_usec = 0;
	} else {
	    if (!do_decode) {
		selector = NULL;
	    } else {
		selector = &timeout;
		timeout.tv_sec = 0;
		timeout.tv_usec = wait_d;
	    }
	}

	nfds += extend;

	status = select
	    (nfds, fd_set_0_select, fd_set_1_select, NULL, selector);
	if (status == -1) {
	    if (errno != EINTR) {
		Mylene1f4_fprintf_error
		    (context, "%s: failed select: %s\n", Mylene1f4(context),
		     strerror(errno));
		break;
	    }
	} else {
	    status = 0;

	    if (fd_set_0_select) {
		if (FD_ISSET(fd_0, fd_set_0_select)) {
		    pending_0fa = 1;
		}
		if (FD_ISSET(STDIN_FILENO, fd_set_0_select)) {
		    pending_1f4 = 1;
		}
	    }
	    if (pending_0fa) {
		if (1) {
		    status = read_e0fa(context);
		    if (status) {
			break;
		    }
		}
	    }
	    if (pending_1f4) {
		if (1) {
		    if (1) {
			if (__slip_away(context)) {
			    unsigned request;

			    request = window_data->request;
			    if (request == Mylene1f4_DEUX_Mylene) {
				status = slip_e1f4(context);
				if (status) {
				    break;
				}
			    } else {
				switch (request) {
				case Mylene1f4_EXIT_Mylene:
#if __ENABLE_x00696e6b__
				case Mylene1f4_JUMP_Mylene:
#endif				/* __ENABLE_x00696e6b__ */
				case Mylene1f4_SKIP_Mylene:
#if __ENABLE_x00696e6b__
				case Mylene1f4_STOP_Mylene:
#endif				/* __ENABLE_x00696e6b__ */
				case Mylene1f4_WAIT_Mylene:
				    do_decode = 0;
				}
			    }
			}
		    }
		}
	    }

	    if (fd_set_1_select) {
		if (FD_ISSET(fd_1, fd_set_1_select)) {
		    status = Mylene1f4_push_mylene0fa(context);
		    if (status) {
			break;
		    }
		}
		if (FD_ISSET(STDOUT_FILENO, fd_set_1_select)) {
		    status = Mylene1f4_push_lanes(context);
		    if (status) {
			break;
		    }
		}
	    }

	    break;
	}
    }
    if (status) {
    } else {
	do {
	    if (do_decode == 1) {
		status = TARGET(context).link(STORAGE(context));
		if (status) {
		    break;
		} else {
		    unsigned e1f4_index;

		    e1f4_index = C1f4(context);
		    e1f4_index += RELEASE(context);
		    if (e1f4_index == SIZE(context)) {
			e1f4_index = 0;
		    }
		    C1f4(context) = e1f4_index;
		    STATE(context)[0] = e1f4_index;

		    REFERENCE(context)++;
		}
	    }

	    if (PROGRESS2(context)) {
		Mylene1f4_run_format(context, PROGRESS2(context));
	    }
	} while (0);
    }

    return status;
}


static int
play_dAta(void *context, unsigned start2)
{
    int status = 0;

    if (!(start2 < TAIL(context))) {
	REFERENCE(context) = TAIL(context);

#if __ENABLE_x00696e6b__
	status = slIp_tail(context);
#endif				/* __ENABLE_x00696e6b__ */
    } else {
	int clear = 0;

	COMPLETED(context) = 0;
	REFERENCE(context) = HEAD(context);

	if (REFERENCE(context) < start2) {
	    status = skip_dAta(context, start2, &clear);
	} else {
	    status = zero_data(context, 0, &clear);
	}
	if (status) {
	} else {
	    if (clear) {
	    } else {
		Mylene1f4_scale_track(context);

#if __ENABLE_x00696e6b__
		BITS(context) = Window(context)->miss_link;
#else
#endif				/* __ENABLE_x00696e6b__ */
		status = Mylene1f4_init_link(context, start2);
		if (status) {
		} else {
#if __LINK_CELL_LOADER__
		    if (TARGET(context).zero) {
			status = pLAy_dAta(context);
		    } else {
#endif				/* __LINK_CELL_LOADER__ */
			status = plAy_dAta(context);
#if __LINK_CELL_LOADER__
		    }
#endif				/* __LINK_CELL_LOADER__ */
		    if (status) {
		    } else {
			unsigned request;

			if (COMPLETED(context)) {
			    Mylene1f4_shift_track(context);
			}

			request = Window(context)->request;
			if (request != Mylene1f4_EXIT_Mylene) {
			    int excess;

			    if (0) {
#if __ENABLE_x00696e6b__
			    } else if (request == Mylene1f4_JUMP_Mylene) {
				excess = slip_miss(context);
#endif				/* __ENABLE_x00696e6b__ */
			    } else if (request == Mylene1f4_SKIP_Mylene) {
				excess = slip_miss(context);
#if __ENABLE_x00696e6b__
			    } else if (request == Mylene1f4_STOP_Mylene) {
				excess = slip_miss(context);
#endif				/* __ENABLE_x00696e6b__ */
			    } else if (request == Mylene1f4_WAIT_Mylene) {
				excess = slip_miss(context);
			    } else {
				excess = Mylene1f4_fini_link(context);
			    }
			    if (status) {
			    } else {
				status = excess;
			    }
			}
		    }
		}
	    }
	}
    }

    return status;
}


static int
play_data(void *context, unsigned start2)
{
    int clear = 0, status = 0;

    COMPLETED(context) = 0;
    REFERENCE(context) = HEAD(context);

    if (REFERENCE(context) < start2) {
	status = skip_data(context, start2, &clear);
    } else {
	status = zero_data(context, 0, &clear);
    }
    if (status) {
    } else {
	if (clear) {
	} else {
	    if (!COMPLETED(context)) {
		Mylene1f4_scale_track(context);

#if __ENABLE_x00696e6b__
		BITS(context) = 0;
#endif				/* __ENABLE_x00696e6b__ */
		status = Mylene1f4_init_link(context, start2);
		if (status) {
		} else {
#if __LINK_CELL_LOADER__
		    if (TARGET(context).zero) {
			status = pLAy_data(context);
		    } else {
#endif				/* __LINK_CELL_LOADER__ */
			status = plAy_data(context);
#if __LINK_CELL_LOADER__
		    }
#endif				/* __LINK_CELL_LOADER__ */
		    if (status) {
		    } else {
			unsigned request;

			if (COMPLETED(context)) {
			    Mylene1f4_shift_track(context);
			}

			request = Window(context)->request;
			if (request != Mylene1f4_EXIT_Mylene) {
			    int excess;

			    if (0) {
#if __ENABLE_x00696e6b__
			    } else if (request == Mylene1f4_JUMP_Mylene) {
				excess = slip_miss(context);
#endif				/* __ENABLE_x00696e6b__ */
			    } else if (request == Mylene1f4_SKIP_Mylene) {
				excess = slip_miss(context);
			    } else if (request == Mylene1f4_WAIT_Mylene) {
				excess = slip_miss(context);
			    } else {
				excess = Mylene1f4_fini_link(context);
			    }
			    if (!status) {
				status = excess;
			    }
			}
		    }
		}
	    }
	}
    }

    return status;
}


static int
play_deck(void *context)
{
    int status = 0;
    struct everlanes_type everlanes;

    everlanes.incoming = 0;
    everlanes.outgoing = 0;

#if __ENABLE_x00696e6b__
    everlanes.bits = 0;
#endif				/* __ENABLE_x00696e6b__ */

    everlanes.mirror[0] = 0;

    everlanes.target = NULL;

    WINDOW(context).within.evermiss = &everlanes;

    FINI(context) = fini_link;
    INIT(context) = init_link;

#if __ENABLE_x00696e6b__
    Window(context)->state2 = 0;
#endif				/* __ENABLE_x00696e6b__ */

    LINK(context) = 0;
    SEQUEL(context) = NULL;
    STREAMS(context) = 0;
    X1f42(context) = NULL;
    WIRE(context) = 7;
    Z0fa(context) = 0;

    while (1) {
	const char *path;

	path = everlanes.target;
	while (!path) {
	    unsigned this = 0;

	    status = Mylene1f4_sync_lanes(context, &this, wait_e0fa);
	    if (status < -1) {
		break;
	    } else if (status == -1) {
		if (this == 0040) {
		    slip_e1f4(context);
		} else {
		    if (this == 0100) {
			status = !Mylene1f4_send_lanes(context, 1, NULL);
			if (status) {
			    break;
			}
		    } else {
			if (this == 0177) {
			    status = 0;
			    break;
			}
		    }
		}
	    } else {
		path = everlanes.target;
	    }
	}
	if (path) {
	} else {
	    break;
	}

	everlanes.target = NULL;

	if (1) {
	    SOURCE(context)._.s1.path = path;
#if __ENABLE_x00696e6b__
	    if (everlanes.mirror[0] != 0x01696e6b) {
#endif				/* __ENABLE_x00696e6b__ */
		status = play_file(context);
#if __ENABLE_x00696e6b__
	    } else {
		status = play_lane(context);
	    }
#endif				/* __ENABLE_x00696e6b__ */
	}

	free((void *) path);

	if (1) {
	    if (status) {
		break;
	    } else {
		unsigned request;

		request = Window(context)->request;
		if (request) {
		    if (request == Mylene1f4_EXIT_Mylene) {
			break;
		    }
		}
	    }
	}
    }

    if (status) {
    } else {
	if (PID(context) != -1) {
	    status = wait_e1f4(context);
	}
    }

    Mylene1f4_ever_lanes(context);

    return status;
}


static int
play_file(void *context)
{
    int status;

#if __ENABLE_x00696e6b__
    link_1f42(context);
#endif				/* __ENABLE_x00696e6b__ */

    status = Mylene1f4_init_source(context);
    if (status) {
	status = Mylene1f4_near_miss(context);
    } else {
	int excess;

#if __ENABLE_x00696e6b__
	do {
	    {
		int (*line) (void *, unsigned, void **);

		line = TARGET(context).line;
		if (line) {
		    status = line
			(LOADER(context), 0, &Window(context)->x1f42a);
		    if (status) {
			break;
		    }
		}
	    }
#endif				/* __ENABLE_x00696e6b__ */
	    {
#if __ENABLE_x00696e6b__
		unsigned start2;
#endif				/* __ENABLE_x00696e6b__ */

		CLIP(context)++;

#if __ENABLE_x00696e6b__
		start2 = START2(context);
#endif				/* __ENABLE_x00696e6b__ */

#if __ENABLE_x00696e6b__
		Window(context)->slip = slip_away;
#endif				/* __ENABLE_x00696e6b__ */

		excess = play_list(context);
		if (status) {
		} else {
		    status = excess;
		}

#if __ENABLE_x00696e6b__
		START2(context) = start2;
#endif				/* __ENABLE_x00696e6b__ */
	    }
#if __ENABLE_x00696e6b__
	} while (0);
#endif				/* __ENABLE_x00696e6b__ */

	excess = Mylene1f4_fini_source(context);
	if (status) {
	} else {
	    status = excess;
	}
    }

    return status;
}


#if __ENABLE_x00696e6b__
static int
play_lane(void *context)
{
    int status;

    link_slip(context);

    status = Mylene1f4_init_source(context);
    if (status) {
	status = Mylene1f4_near_miss(context);
    } else {
	int excess;

	do {
	    {
		int (*line) (void *, unsigned, void **);

		line = TARGET(context).line;
		if (line) {
		} else {
		    status = 1;
		    break;
		}

		status = line(LOADER(context), 0, &Window(context)->x1f42a);
		if (status) {
		    break;
		}
	    }
	    {
		if (line_lane(context)) {
		    status = 1;
		    break;
		}
	    }
	    {
		unsigned start2;

		CLIP(context)++;

		start2 = START2(context);

		Window(context)->slip = slip_iway;

		status = play_tile(context);

		START2(context) = start2;
	    }
	} while (0);

	excess = Mylene1f4_fini_source(context);
	if (status) {
	} else {
	    status = excess;
	}
    }

    return status;
}
#endif				/* __ENABLE_x00696e6b__ */


static int
play_list(void *context)
{
    int status = 0;
    unsigned count;

    count = COUNT(context);
    if (count) {
#if __ENABLE_x00696e6b__
	int fix_start = 1;
#endif				/* __ENABLE_x00696e6b__ */

#if __ENABLE_x00696e6b__
	Window(context)->first = 1;
	Window(context)->miss_link = 0;
#endif				/* __ENABLE_x00696e6b__ */

	Window(context)->index = 0;

	SECOND_A(ATIME(context)) = 0;
	SECOND_F(ATIME(context)) = 0;

	REFERENCE(context) = 0;

#if __ENABLE_x00696e6b__
      x00696e6b:
#endif				/* __ENABLE_x00696e6b__ */
	while (1) {
	    status = init_port(context, Window(context)->index);
	    if (status) {
		break;
	    } else {
		int excess;
		unsigned request = 0, start2;

#if __ENABLE_x00696e6b__
		if (!fix_start) {
		    fix_start = 1;
		} else {
#endif				/* __ENABLE_x00696e6b__ */
		    if (PROGRESS1(context)) {
			Mylene1f4_run_format(context, PROGRESS1(context));
		    }
#if __ENABLE_x00696e6b__
		}
#endif				/* __ENABLE_x00696e6b__ */

#if __ENABLE_x00696e6b__
		if ((((struct everlanes_type *)
		      WINDOW(context).within.evermiss)->mirror[0] & 0xfeffffff)
		    == 0x00696e6b) {
		    start2 = Window(context)->start2;
		} else {
#endif				/* __ENABLE_x00696e6b__ */
		    Mylene1f4_fix_start(context, &start2);
#if __ENABLE_x00696e6b__
		}
#endif				/* __ENABLE_x00696e6b__ */

		status = Mylene1f4_make_link(context);
		if (status) {
		} else {
		    if (~TAIL(context)) {
			status = play_dAta(context, start2);
		    } else {
			status = play_data(context, start2);
		    }
		    request = Window(context)->request;
		    if (request == Mylene1f4_EXIT_Mylene) {
			excess = wait_exit(context);
			if (status) {
			} else {
			    status = excess;
			}
		    }
		    if (status) {
		    } else {
			unsigned reQuest;

			reQuest = Window(context)->request;

			status = wait_list(context);
			if (status) {
			} else {
			    if (1) {
				unsigned Request;

				Request = Window(context)->request;
				if (Request != reQuest) {
				    request = Request;
				}
			    }
			    {
				void *progressl;

				progressl = PROGRESS2(context);
				if (progressl) {
			            Mylene1f4_run_format(context, progressl);
				}
#if __ENABLE_x00696e6b__
				if (request == Mylene1f4_JUMP_Mylene) {
				    fix_start = 0;
				} else {
#endif				/* __ENABLE_x00696e6b__ */
				    progressl = PROGRESS3(context);
				    if (progressl) {
			    		Mylene1f4_run_format
					    (context, progressl);
				    }
#if __ENABLE_x00696e6b__
				}
#endif				/* __ENABLE_x00696e6b__ */
			    }
			}
		    }
		}
		excess = fini_port(context, Window(context)->index);
		if (!status) {
		    status = excess;
		}
		if (status) {
		    break;
		} else {
		    if (request) {
			if (requests[request].shift) {
			    status = requests[request].shift(context);
			    if (status) {
				break;
			    }
			}
			if (requests[request].clear) {
			    break;
			}
		    }
		    {
			unsigned index;

			index = Window(context)->index;
			index++;
			Window(context)->index = index;
			if (index == count) {
			    break;
			}
		    }
		}
	    }
	}

	if (Window(context)->index == count) {
	    while (1u << 31) {
		status = wait_clip(context);
		if (status) {
		    break;
		} else {
		    unsigned request;

		    request = Window(context)->request;
		    if (request) {
			if (requests[request].shift) {
			    status = requests[request].shift(context);
			    if (status) {
				break;
			    }
			}
			if (requests[request].clear) {
			    break;
#if __ENABLE_x00696e6b__
			} else {
			    goto x00696e6b;
#endif				/* __ENABLE_x00696e6b__ */
			}
		    }
		}
	    }
	}
    }

    PLAY(context)++;

    return status;
}


#if __ENABLE_x00696e6b__
static int
play_tile(void *context)
{
    int status;

    Window(context)->last_386 = 0603;
    Window(context)->second_f = ~0;

    while (1) {
	status = miss_clip(context);
	if (status) {
	    break;
	} else {
	    unsigned request;

	    request = Window(context)->request;
	    if (request) {
		if (request == Mylene1f4_LINE_Mylene) {
		    struct everlanes_type *everlanes_data;
		    struct timeval time;
		    struct x1f42a_type *x1f42a_data;
		    unsigned *mirror, second_A, second_F, second_a, second_f;

		    everlanes_data = WINDOW(context).within.evermiss;
		    mirror = everlanes_data->mirror;
		    second_a = mirror[5];
		    second_f = mirror[6];

		    gettimeofday(&time, NULL);
		    second_A = SECOND_a(time);
		    second_F = SECOND_f(time);

		    if (second_A < second_a) {
			status = 1;
			break;
		    }
		    if (second_A ^ second_a) {
		    } else {
			if (second_F < second_f) {
			    status = 1;
			    break;
			}
		    }

		    x1f42a_data = Window(context)->x1f42a;

		    if (mirror[4] & 1) {
		    } else {
			unsigned PTS_0, PTS_1;

			PTS_0 = x1f42a_data->PTS_0;
			PTS_1 = x1f42a_data->PTS_1;
			if (~(PTS_0 & PTS_1)) {
			    down_time(&second_a, &second_f, PTS_0, PTS_1);
			}
		    }

		    if (second_A < second_a) {
			unsigned mirror_7_ = 0;

			mirror_7_ += x1f42a_data->next_frame;
			mirror_7_ -= x1f42a_data->frames;

			mirror[7] = mirror_7_;

			Window(context)->second_a = second_a;
			Window(context)->second_f = second_f;

			Window(context)->start2 = mirror_7_;

			Window(context)->detached = 0602;
		    } else {
			if (second_A ^ second_a) {
			    double frame_i;
			    unsigned mirror_7_;

			    second_A = second_A - second_a;
			    if (second_F < second_f) {
				second_A--;
				second_F += 1000000;
			    }
			    second_F = second_F - second_f;
			    frame_i = second_F;
			    frame_i = frame_i / 1000000;
			    frame_i = frame_i + second_A;
			    frame_i = frame_i * x1f42a_data->frame_rate_n;
			    frame_i = frame_i / x1f42a_data->frame_rate_d;
			    mirror_7_ = frame_i;

			    if (mirror_7_ < x1f42a_data->frames) {
				mirror_7_ += x1f42a_data->next_frame;
				mirror_7_ -= x1f42a_data->frames;

				mirror[7] = mirror_7_;

				Window(context)->second_a = 0;
				Window(context)->second_f = 0;

				Window(context)->start2 = mirror_7_;

				Window(context)->detached = 0602;
			    }
			} else {
			    if (second_F < second_f) {
				unsigned mirror_7_ = 0;

				mirror_7_ += x1f42a_data->next_frame;
				mirror_7_ -= x1f42a_data->frames;

				mirror[7] = mirror_7_;

				Window(context)->second_a = second_a;
				Window(context)->second_f = second_f;

				Window(context)->start2 = mirror_7_;

				Window(context)->detached = 0602;
			    } else {
				double frame_i;
				unsigned mirror_7_;

				Window(context)->second_a = 0;
				Window(context)->second_f = 0;

				second_F = second_F - second_f;
				frame_i = second_F;
				frame_i = frame_i / 1000000;
				frame_i = frame_i * x1f42a_data->frame_rate_n;
				frame_i = frame_i / x1f42a_data->frame_rate_d;
				mirror_7_ = frame_i;

				if (mirror_7_ < x1f42a_data->frames) {
				    mirror_7_ += x1f42a_data->next_frame;
				    mirror_7_ -= x1f42a_data->frames;

				    mirror[7] = mirror_7_;

				    Window(context)->second_a = 0;
				    Window(context)->second_f = 0;

				    Window(context)->start2 = mirror_7_;

				    Window(context)->detached = 0602;
				}
			    }
			}
		    }

		    Window(context)->request = 0;
		} else {
		    if (request == Mylene1f4_ZERO_Mylene) {
			Window(context)->request = 0;
			status = play_list(context);
			if (status) {
			    break;
			} else {
			    request = Window(context)->request;
			    if (request == Mylene1f4_STOP_Mylene) {
				Window(context)->request = 0;
				Window(context)->detached = 0;
				Window(context)->second_f = ~0;
			    } else {
				break;
			    }
			}
		    } else {
			if (request == Mylene1f4_EXIT_Mylene
			    || request == Mylene1f4_SKIP_Mylene
			    || request == Mylene1f4_WAIT_Mylene) {
			    if (requests[request].shift) {
				status = requests[request].shift(context);
			    }

			    break;
			}
		    }
		}
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_x00696e6b__ */


static int
read_e0fa(void *context)
{
    return Mylene1f4_read_e0fa
	(context, Window(context)->request == Mylene1f4_EXIT_Mylene);
}


static int
skip_dAta(void *context, unsigned start2, int *clear)
{
    int status;

    if (TAIL(context) < start2 + 1) {
	status = slIp_tail(context);
    } else {
	status = eVer_clip(context, start2, clear);
    }

    return status;
}


static int
skip_data(void *context, unsigned start2, int *clear)
{
    int status;

    status = eVer_clip(context, start2, clear);
    if (status) {
	if (COMPLETED(context)) {
	    TAIL(context) = REFERENCE(context);
	    status = 0;
	}
    }

    return status;
}


#if __ENABLE_x00696e6b__
static int
slIp_tail(void *context)
{
    int status;
    struct everlanes_type *everlanes_data;
    unsigned *mirror;

    everlanes_data = WINDOW(context).within.evermiss;
    mirror = everlanes_data->mirror;
    if (mirror[0] == 0x00696e6b) {
	status = slip_tail(context);
    } else {
	/*
	 * Now this looks stupid: since the controller is not a \0ink
	 * controller it probably does not know the stream limits!
	 *
	 * BUGS
	 *
	 */
	status = 1;
    }

    return status;
}
#endif				/* __ENABLE_x00696e6b__ */


static int
slip_away(void *context)
{
    int status;
    struct window_type *window_data;
    unsigned command;

    window_data = WINDOW(context).within.window;

    status = Mylene1f4_miss_lanes(context, &command);
    if (status < -1) {
	window_data->detached = 0177;
	window_data->underlay = 1;
	window_data->request = Mylene1f4_EXIT_Mylene;
    } else {
	if (status) {
	    if (command == 0040) {
#if __ENABLE_x00696e6b__
		/*
		 * damn, this looks like a huge bug to me!
		 *
		 * thing is that since this is \0ink the 0040 command reply
		 * should be send before any other commands are processed (or
		 * at least replied), but here I am (very badly) enforcing this
		 * only if state2 (what's that?) is 2 (in other words, if to
		 * the best of my knowledge Mylene0fa is actually playing).
		 * And that's partly (or mainly, or something) because
		 * Mylene0fa will only notify me for change of state only if it
		 * has some data to play (which is not the case when the stream
		 * has completed).
		 *
		 * BUGS
		 *
		 */
		if (Window(context)->state2 == 2) {
		    window_data->detached = 0040;
		}
#endif				/* __ENABLE_x00696e6b__ */
		window_data->request = Mylene1f4_DEUX_Mylene;
	    } else if (command == 0100) {
		window_data->detached = 0100;
		window_data->request = Mylene1f4_WAIT_Mylene;
	    } else if (command == 0177) {
		window_data->detached = 0177;
		window_data->underlay = 1;
		window_data->request = Mylene1f4_EXIT_Mylene;
#if __ENABLE_x00696e6b__
	    } else if (command == 0420) {
	        struct everlanes_type *everlanes_data;
	        unsigned *mirror;

	        everlanes_data = WINDOW(context).within.evermiss;
	        mirror = everlanes_data->mirror;
	        if (mirror[0] != 0x00696e6b) {
		    status = 0;
		} else {
		    if (mirror[4] == 0x50) {
			status = 0;
		    } else {
			window_data->detached = 0420;
			window_data->request = Mylene1f4_JUMP_Mylene;
		    }
		}
#endif				/* __ENABLE_x00696e6b__ */
	    } else {
		status = 0;
	    }
	} else {
	    if (1) {
		status = 1;
		window_data->detached = 0017;
		window_data->request = Mylene1f4_SKIP_Mylene;
	    }
	}
    }

    return status;
}


static int
slip_e1f4(void *context)
{
    Window(context)->request = 0;

    return Mylene1f4_send_mylene0fa(context, 2, NULL);
}


#if __ENABLE_x00696e6b__
static int
slip_eway(void *context)
{
    int status;
    struct window_type *window_data;
    unsigned command;

    window_data = WINDOW(context).within.window;

    status = Mylene1f4_miss_lanes(context, &command);
    if (status < -1) {
	window_data->detached = 0177;
	window_data->underlay = 1;
	window_data->request = Mylene1f4_EXIT_Mylene;
    } else {
	if (status) {
	    if (command == 0040 || command == 0177 || command == 0420) {
		window_data->detached = 0177;
		window_data->underlay = 1;
		window_data->request = Mylene1f4_EXIT_Mylene;
	    } else if (command == 0100) {
		window_data->detached = 0100;
		window_data->request = Mylene1f4_WAIT_Mylene;
	    } else if (command == 0602) {
		if (window_data->last_386 != 0602) {
		    window_data->last_386 = 0602;
		    window_data->request = Mylene1f4_LINE_Mylene;
		} else {
		    window_data->detached = 0177;
		    window_data->underlay = 1;
		    window_data->request = Mylene1f4_EXIT_Mylene;
		}
	    } else if (command == 0603) {
		if (window_data->last_386 != 0603) {
		    window_data->last_386 = 0603;
		    window_data->second_f = ~0;
		    status = 0;
		} else {
		    window_data->detached = 0177;
		    window_data->underlay = 1;
		    window_data->request = Mylene1f4_EXIT_Mylene;
		}
	    } else {
		status = 0;
	    }
	} else {
	    if (1) {
		status = 1;
		window_data->detached = 0017;
		window_data->request = Mylene1f4_SKIP_Mylene;
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_x00696e6b__ */


#if __ENABLE_x00696e6b__
static int
slip_iway(void *context)
{
    int status;
    struct window_type *window_data;
    unsigned command;

    window_data = WINDOW(context).within.window;

    status = Mylene1f4_miss_lanes(context, &command);
    if (status < -1) {
	window_data->detached = 0177;
	window_data->underlay = 1;
	window_data->request = Mylene1f4_EXIT_Mylene;
    } else {
	if (status) {
	    if (command == 0040 || command == 0177 || command == 0420
		|| command == 0602) {
		window_data->detached = 0177;
		window_data->underlay = 1;
		window_data->request = Mylene1f4_EXIT_Mylene;
	    } else if (command == 0100) {
		window_data->detached = 0100;
		window_data->request = Mylene1f4_WAIT_Mylene;
	    } else if (command == 0603) {
		window_data->detached = 0603;
		window_data->last_386 = 0603;
		window_data->request = Mylene1f4_STOP_Mylene;
	    } else {
		status = 0;
	    }
	} else {
	    if (1) {
		status = 1;
		window_data->detached = 0017;
		window_data->request = Mylene1f4_SKIP_Mylene;
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_x00696e6b__ */


#if __ENABLE_x00696e6b__
static int
slip_jump(void *context)
{
    Window(context)->index--;
    Window(context)->request = 0;

    return 0;
}
#endif				/* __ENABLE_x00696e6b__ */


static int
slip_miss(void *context)
{
    COMPLETED(context) = 1;

    return Mylene1f4_send_mylene0fa(context, 1, NULL);
}


static int
slip_skip(void *context)
{
    Window(context)->detached = 0;
    Window(context)->request = 0;

    return 0;
}


#if __ENABLE_x00696e6b__
static int
slip_tail(void *context)
{
    int status;
    unsigned char system[020];

    TARGET(context).lock(context, system, TAIL(context));
    Mylene1f4_code_time(system + 010);

    status = !Mylene1f4_send_lanes(context, 0202, system);
    if (status) {
    } else {
	status = !Mylene1f4_send_lanes(context, 0203, system);
    }

    return status;
}
#endif				/* __ENABLE_x00696e6b__ */


static int
slip_wait(void *context)
{
    Window(context)->detached = 0;
    Window(context)->request = 0;

    return !Mylene1f4_send_lanes(context, 1, NULL);
}


static int
wait_clip(void *context)
{
    int fd_0, fd_1, pending_0fa, pending_1f4, status;
    struct everlanes_type *everlanes_data;
    struct window_type *window_data;
    unsigned detached, underlay, wait_2 = 0;
    void *progress2;

    if (STREAMS(context) & (1u << 31)) {
	progress2 = PROGRESS2(context);
    } else {
	progress2 = NULL;
    }

    window_data = WINDOW(context).within.window;
    everlanes_data = WINDOW(context).within.evermiss;

    fd_0 = Mylene1ST(context);
    fd_1 = Mylene2ND(context);

    pending_0fa = Z0fa(context);

    detached = window_data->detached;
    underlay = window_data->underlay;

    if (detached | underlay) {
	pending_1f4 = 0;
    } else {
	pending_1f4 = everlanes_data->incoming;
    }

    if (pending_0fa | pending_1f4) {
    } else if (progress2) {
#if __FIX_0fa_WAIT__
	wait_2 = Window(context)->wait;
#else
# if SIZEOF_UNSIGNED_LONG == 8
	unsigned long wait;

	wait = RELEASE(context) * WAIT_COUNT;
	wait *= 1000000;
	wait /= (SAMPLING(context) * CHANNELS(context) << 1);
# else
	double wait;

	wait = RELEASE(context) * WAIT_COUNT;
	wait /= (SAMPLING(context) * CHANNELS(context) << 1);
	wait *= 1000000;
# endif					/* SIZEOF_UNSIGNED_LONG == 8 */
	wait_2 = wait;
	if (131072 < wait_2) {
	    wait_2 = 131072;
	}
#endif				/* __FIX_0fa_WAIT__ */
    }

    while (1) {
	fd_set fd_set_0, *fd_set_0_select, fd_set_1, *fd_set_1_select;
	struct timeval *selector, timeout;
	unsigned extend = 0, nfds = 0;

	if (SIGCHLD_received(context)) {
	    int expect;

	    expect = Window(context)->request == Mylene1f4_EXIT_Mylene;
	    status = Mylene1f4_yway_mylene0fa(context, expect);
	    if (status) {
		break;
	    } else {
		if (expect) {
		    if (PID(context) == -1) {
			break;
		    }
		} else {
		    break;
		}
	    }
	}

	if (pending_1f4) {
	    fd_set_0_select = NULL;
	} else if (detached) {
	    fd_set_0_select = NULL;
	} else if (underlay) {
	    fd_set_0_select = NULL;
	} else {
	    fd_set_0_select = &fd_set_0;
	    FD_ZERO(fd_set_0_select);
	    nfds = STDIN_FILENO;
	    FD_SET(STDIN_FILENO, fd_set_0_select);
	    extend = 1;
	}
	if (!pending_0fa) {
	    if (!fd_set_0_select) {
		fd_set_0_select = &fd_set_0;
		FD_ZERO(fd_set_0_select);
		nfds = fd_0;
	    } else {
		if (nfds < fd_0) {
		    nfds = fd_0;
		}
	    }
	    FD_SET(fd_0, fd_set_0_select);
	    extend = 1;
	}
	if (!OUTGOING(context)) {
	    fd_set_1_select = NULL;
	} else {
	    fd_set_1_select = &fd_set_1;
	    FD_ZERO(fd_set_1_select);
	    FD_SET(fd_1, fd_set_1_select);
	    if (nfds < fd_1) {
		nfds = fd_1;
	    }
	    extend = 1;
	}
	if (underlay) {
	} else if (everlanes_data->outgoing) {
	    if (!fd_set_1_select) {
		fd_set_1_select = &fd_set_1;
		FD_ZERO(fd_set_1_select);
		nfds = STDOUT_FILENO;
	    } else {
		if (nfds < STDOUT_FILENO) {
		    nfds = STDOUT_FILENO;
		}
	    }
	    FD_SET(STDOUT_FILENO, fd_set_1_select);
	    extend = 1;
	}

	if (pending_0fa | pending_1f4) {
	    selector = &timeout;
	    timeout.tv_sec = 0;
	    timeout.tv_usec = 0;
	} else {
	    if (!progress2) {
		selector = NULL;
	    } else {
		selector = &timeout;
		timeout.tv_sec = 0;
		timeout.tv_usec = wait_2;
	    }
	}

	nfds += extend;

	status = select
	    (nfds, fd_set_0_select, fd_set_1_select, NULL, selector);
	if (status == -1) {
	    if (errno != EINTR) {
		Mylene1f4_fprintf_error
		    (context, "%s: failed select: %s\n", Mylene1f4(context),
		     strerror(errno));
		break;
	    }
	} else {
	    status = 0;

	    if (fd_set_0_select) {
		if (FD_ISSET(fd_0, fd_set_0_select)) {
		    pending_0fa = 1;
		}
		if (FD_ISSET(STDIN_FILENO, fd_set_0_select)) {
		    pending_1f4 = 1;
		}
	    }
	    if (pending_0fa) {
		if (1) {
		    status = read_e0fa(context);
		    if (status) {
			break;
		    } else {
			if (Mylene1ST(context) == -1) {
			    break;
			}
		    }
		}
	    }
	    if (pending_1f4) {
		if (1) {
		    if (1) {
			if (__slip_away(context)) {
			    switch (window_data->request) {
			    case Mylene1f4_DEUX_Mylene:
				status = slip_e1f4(context);
				break;
			    case Mylene1f4_EXIT_Mylene:
				status = wait_exit(context);
				break;
#if __ENABLE_x00696e6b__
			    case Mylene1f4_JUMP_Mylene:
				status = slip_miss(context);
				break;
#endif				/* __ENABLE_x00696e6b__ */
			    case Mylene1f4_SKIP_Mylene:
				status = slip_miss(context);
				break;
#if __ENABLE_x00696e6b__
			    case Mylene1f4_STOP_Mylene:
				status = slip_miss(context);
				break;
#endif				/* __ENABLE_x00696e6b__ */
			    case Mylene1f4_WAIT_Mylene:
				status = slip_miss(context);
				break;
			    }
			    if (status) {
				break;
			    }
			}
		    }
		}
	    }

	    if (fd_set_1_select) {
		if (FD_ISSET(fd_1, fd_set_1_select)) {
		    status = Mylene1f4_push_mylene0fa(context);
		    if (status) {
			break;
		    }
		}
		if (FD_ISSET(STDOUT_FILENO, fd_set_1_select)) {
		    status = Mylene1f4_push_lanes(context);
		    if (status) {
			break;
		    }
		}
	    }

	    break;
	}
    }
    if (status) {
    } else {
	if (PID(context) == -1) {
	} else {
	    if (STREAMS(context) & (1u << 31)) {
		progress2 = PROGRESS2(context);

		if (progress2) {
		    Mylene1f4_run_format(context, progress2);
		}
	    }
	}
    }

    return status;
}


static int
wait_e0fa(void *context)
{
    int status;

    if (SIGCHLD_received(context)) {
	status = Mylene1f4_yway_mylene0fa(context, 0);
    } else {
	status = 0;
    }

    return status;
}


/*
 * if controller window asked for exit wait_e1f4 renders futile (output channel
 * will be flushed by main logic).  I'll take the trip just for fun...
 */
static int
wait_e1f4(void *context)
{
    int status = 0;

    while (STREAMS(context) & (1u << 31)) {
	status = wait_clip(context);
	if (status) {
	    break;
	} else {
	    unsigned request;

	    request = Window(context)->request;
	    if (request) {
		if (request == Mylene1f4_EXIT_Mylene) {
		    if (PID(context) == -1) {
			break;
		    } else {
			if (!OUTGOING(context)) {
			    break;
			} else {
			    if (Mylene1ST(context) == -1) {
				break;
			    }
			}
		    }
		}
	    }
	}
    }

    return status;
}


static int
wait_exit(void *context)
{
    int status;

    {
	void *progress2;

	progress2 = PROGRESS2(context);
	if (progress2) {
	    Mylene1f4_fini_format(context, &progress2);
	    PROGRESS2(context) = NULL;
	}
    }

    status = Mylene1f4_send_mylene0fa(context, 0, NULL);

    return status;
}


static int
wait_list(void *context)
{
    int status;

    status = wait_e1f4(context);
#if __ENABLE_x00696e6b__
    if (status) {
    } else {
	unsigned *mirror;

	mirror = ((struct everlanes_type *)
	    WINDOW(context).within.evermiss)->mirror;
	if ((mirror[0] & 0xfeffffff) == 0x00696e6b) {
	    if ((Window(context)->state2 & ~2) != 1) {
		status = 1;
	    } else {
		if (mirror[0] == 0x01696e6b) {
		    if (Window(context)->detached == 0602) {
			Window(context)->detached = 0;
		    }
		}
	    }
	}
    }
#endif				/* __ENABLE_x00696e6b__ */

    return status;
}


#if __LINK_CELL_LOADER__
static int
zero_cell(void *context, unsigned count2, int *clear)
{
    int status;

    status = TARGET(context).zero(STORAGE(context), count2);

    return status;
}
#endif				/* __LINK_CELL_LOADER__ */


int
Mylene1f4_run_excessb(void *context)
{
    int fd, status;

    status = Mylene1f4_init_lanes(context, &fd);
    if (status) {
    } else {
	struct window_type window;

	do {
	    if (!(fd < 0)) {
		FILE *progress;

		progress = fdopen(fd, "w");
		if (progress) {
		    PROGRESS(context) = progress;
		    break;
		}
	    }

	    PROGRESS(context) = NULL;

	    Mylene1f4_fini_output(context);
	} while (0);

	{
	    {
		{
		    {
			window.detached = 0;
			window.request = 0;
			window.underlay = 0;

			WINDOW(context).within.window = &window;

			status = lane_deck(context);
		    }
		}
	    }
	}

	Mylene1f4_fini_lanes(context);
    }

    return status;
}

#endif				/* __ENABLE_x1f4LANES__ */
