/*
 * file.c
 * Copyright (C) 2005-2006, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

#include <sys/stat.h>
#include <sys/types.h>

#include <access.h>
#include <inter.h>
#include <types.h>

#define Source(source) \
    ((struct source_type *) (source))->_

int
Mylene1f4_close_file(void *context, void *source)
{
    int status;

    status = close(Source(source).s1.fd);

    return status;
}


int
Mylene1f4_open_file(void *context, void *source)
{
    int fd, status = -1;

    do {
	fd = open(Source(source).s1.path, O_RDONLY);
    } while (fd == -1 && errno == EINTR);
    if (fd == -1) {
	Mylene1f4_fprintf_error
	    (context, "%s: cannot open `%s': %s\n", Mylene1f4(context),
	     Source(source).s1.path, strerror(errno));
    } else {
	struct stat info;

	if (fstat(fd, &info)) {
	    Mylene1f4_fprintf_error
		(context, "%s: cannot stat `%s': %s\n", Mylene1f4(context),
		 Source(source).s1.path, strerror(errno));
	} else {
	    if (!S_ISREG(info.st_mode)) {
		Mylene1f4_fprintf_error
		    (context, "%s: `%s' is not a regular file\n",
		     Mylene1f4(context), Source(source).s1.path);
	    } else {
		status = 0;

		Source(source).s1.fd = fd;
		Source(source).s1.size = info.st_size;
	    }
	}

	if (status) {
	    close(fd);
	}
    }

    return status;
}


int
Mylene1f4_read_file(void *context, void *source, void *buffer, unsigned size)
{
    int status = 0;

    while (1) {
	int fd, success;

	fd = Source(source).s1.fd;

	success = read(fd, buffer, size);
	if (success != -1) {
	    if (!success) {
		break;
	    } else {
		buffer = (char *) buffer + success;
		size -= success;
		status += success;
		if (!size) {
		    break;
		}
	    }
	} else {
	    if (errno == EINTR) {
	    } else {
		status = -1;

		Mylene1f4_fprintf_error
		    (context, "%s: cannot read `%s': %s\n", Mylene1f4(context),
		     Source(source).s1.path, strerror(errno));

		break;
	    }
	}
    }

    return status;
}


int
Mylene1f4_seek_file(void *context, void *source, int offset, int whence)
{
    int status;

    status = lseek(Source(source).s1.fd, offset, whence);
    if (status == -1) {
	Mylene1f4_fprintf_error
	    (context, "%s: cannot seek `%s': %s\n", Mylene1f4(context),
	     Source(source).s1.path, strerror(errno));
    }

    return status;
}


int
Mylene1f4_tell_file(void *context, void *source, unsigned *offset)
{
    int status;

    status = lseek(Source(source).s1.fd, 0, SEEK_CUR);
    if (status != -1) {
	*offset = status;
	status = 0;
    } else {
	Mylene1f4_fprintf_error
	    (context, "%s: cannot seek `%s': %s\n", Mylene1f4(context),
	     Source(source).s1.path, strerror(errno));
    }

    return status;
}
