/*
 * format.c
 * Copyright (C) 2005-2011, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if __ENABLE_EQUEUE__
# include <aime.h>
#endif				/* __ENABLE_EQUEUE__ */
#include <e1f4.h>
#include <grp.h>
#if defined HAVE_LIBLEWD
# include <lewd.h>
#endif				/* HAVE_LIBLEWD */
#if defined HAVE_LIBx1f4l0
# include <libx1f4l0.h>
#endif				/* HAVE_LIBx1f4l0 */
#include <math.h>
#include <pwd.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>

#include <access.h>
#if !defined HAVE_LIBLEWD
# include <basename.h>
#endif				/* !HAVE_LIBLEWD */
#include <c213.h>
#if !defined HAVE_LIBLEWD
# include <cardinal.p-f.h>
#endif				/* !HAVE_LIBLEWD */
#include <defs.h>
#include <format.h>
#include <inter.h>
#include <lcard.h>
#if !defined HAVE_LIBLEWD
# include <printf.h>
#endif				/* !HAVE_LIBLEWD */
#if !defined HAVE_LIBx1f4l0
# include <salist.h>
#endif				/* !HAVE_LIBx1f4l0 */
#if !defined HAVE_LIBLEWD
# include <sstring.n-i.h>
# include <sstring.n.h>
# include <sstring.v-i.h>
# include <sstring.v.h>
#endif				/* !HAVE_LIBLEWD */
#include <types.h>

#define GET_B1st			(1L << (Mylene1f4_MODE_PRESS + 0x0))
#define GET_CACHE			(1L << (Mylene1f4_MODE_PRESS + 0x1))
#define GET_HEAD			(1L << (Mylene1f4_MODE_PRESS + 0x2))
#define GET_INDEX			(1L << (Mylene1f4_MODE_PRESS + 0x3))
#define GET_LENGTH			(1L << (Mylene1f4_MODE_PRESS + 0x4))
#define GET_LSTAT			Mylene1f4_MODE_LSTAT
#define GET_MYLENE			(1L << (Mylene1f4_MODE_PRESS + 0x5))
#define GET_READ			(1L << (Mylene1f4_MODE_PRESS + 0x6))
#define GET_STAT			Mylene1f4_MODE_NSTAT
#define GET_TAIL			(1L << (Mylene1f4_MODE_PRESS + 0x7))
#define GET_TIME			(1L << (Mylene1f4_MODE_PRESS + 0x8))
#define GET_UPTIME			(1L << (Mylene1f4_MODE_PRESS + 0x9))

#define FIX_DECK			(1L << (Mylene1f4_MODE_PRESS + 0xa))
#define FIX_FLOW			(1L << (Mylene1f4_MODE_PRESS + 0xb))

#define COFF_MASK			(1l << COFF_MODE)
#define GOFF_MASK			(1l << GOFF_MODE)
#define LIME_MASK			(1l << LIME_MODE)

#define COFF_MODE			000
#define GOFF_MODE			001
#define LESS_MODE			002
#define LESS_mode			003
#define LIME_MODE			004
#define MASS_MODE			005
#define MASS_mode			006
#define NESS_MODE			007
#define NESS_mode			010
#define TESS_MODE			011
#define TESS_mode			012
#define TIME_MODE			013
#define TIME_mode			014
#define USER_MODE			015
#define USER_mode			016

#define none_MODE			~0

#define x1f4_free_format		x1f4_free_format6
#define x1f4_parse_format		x1f4_parse_format6
#define x1f4_print_format		x1f4_print_format6
#define x1f4_run_format			x1f4_run_format6

#define x1f4_lprint_ffields		x1f4_lprint_ffields6
#define x1f4_pprint_ffields		x1f4_pprint_ffields6
#define x1f4_tprint_ffields		x1f4_tprint_ffields6

#define NEED_COUNT__0			0

#define NEED_COUNT_13			0

#define NEED_PRINT_VERSION		0

#if __ENABLE_CONTROLLING_SERVER__
# define COUNT_Sd			count_Sd
#else
# define COUNT_Sd			count__0

# undef NEED_COUNT__0
# define NEED_COUNT__0			1
#endif				/* __ENABLE_CONTROLLING_SERVER__ */

#if __ENABLE_CONTROLLING_WINDOW__
# define COUNT_Nd			count_Nd
#else
# define COUNT_Nd			count__1
#endif				/* __ENABLE_CONTROLLING_WINDOW__ */

#if __ENABLE_x1f4LANES__
# define COUNT_Ld			count_Ld
#else
# define COUNT_Ld			count__0

# undef NEED_COUNT__0
# define NEED_COUNT__0			1
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_MPEG_EQUALIZER__
# define COUNT_at			count_at
#else
# define COUNT_at			count__0

# undef NEED_COUNT__0
# define NEED_COUNT__0			1
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
# define COUNT_et			count_et
#else
# define COUNT_et			count__0

# undef NEED_COUNT__0
# define NEED_COUNT__0			1
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
# define COUNT_bu			count_bu
#else
# define COUNT_bu			count__0

# undef NEED_COUNT__0
# define NEED_COUNT__0			1
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if !defined __Mylene1f4_version_defined__
# define __have_Mylene1f4_version__	0
#elif !__Mylene1f4_version_defined__
# define __have_Mylene1f4_version__	0
#else
# define __have_Mylene1f4_version__	1
#endif				/* !__Mylene1f4_version_defined__ */

#if __have_Mylene1f4_version__
# define COUNT_AK			count_ak
# define PRINT_AK			print_ak
#else
# define COUNT_AK			count_13
# define PRINT_AK			print_version

# undef NEED_COUNT_13
# define NEED_COUNT_13			1

# undef NEED_PRINT_VERSION
# define NEED_PRINT_VERSION		1
#endif				/* __have_Mylene1f4_version__ */

#if __ENABLE_EQUEUE__
# define COUNT_IK			count_ik
# define PRINT_IK			print_ik
#else
# define COUNT_IK			count_13
# define PRINT_IK			print_version

# undef NEED_COUNT_13
# define NEED_COUNT_13			1

# undef NEED_PRINT_VERSION
# define NEED_PRINT_VERSION		1
#endif				/* __ENABLE_EQUEUE__ */

#define NEED_PRINT__0			0

#if defined HAVE_ST_BLKSIZE
# define COUNT_b6			count_B6
# define PRINT_b6			print_B6
#else
# define COUNT_b6			count__0
# define PRINT_b6			print__0

# undef NEED_COUNT__0
# define NEED_COUNT__0			1

# undef NEED_PRINT__0
# define NEED_PRINT__0			1
#endif				/* HAVE_ST_BLKSIZE */

#if defined HAVE_ST_BLOCKS
# define COUNT_B6			count_b6
# define PRINT_B6			print_b6
#else
# define COUNT_B6			count__0
# define PRINT_B6			print__0

# undef NEED_COUNT__0
# define NEED_COUNT__0			1

# undef NEED_PRINT__0
# define NEED_PRINT__0			1
#endif				/* HAVE_ST_BLOCKS */

#if __ENABLE_MPEG_EQUALIZER__
# define COUNT_lw			count_lw
# define PRINT_lw			print_lw
#else
# define COUNT_lw			count__0
# define PRINT_lw			print__0

# undef NEED_COUNT__0
# define NEED_COUNT__0			1

# undef NEED_PRINT__0
# define NEED_PRINT__0			1
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
# define COUNT_cu			count_cu
# define PRINT_cu			print_cu
#else
# define COUNT_cu			count__0
# define PRINT_cu			print__0

# undef NEED_COUNT__0
# define NEED_COUNT__0			1

# undef NEED_PRINT__0
# define NEED_PRINT__0			1
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
# define COUNT_fv			count_fv
# define COUNT_iv			count_iv
# define COUNT_lv			count_lv
# define PRINT_fv			print_fv
# define PRINT_iv			print_iv
# define PRINT_lv			print_lv
#else
# define COUNT_fv			count__0
# define COUNT_iv			count__0
# define COUNT_lv			count__0
# define PRINT_fv			print__0
# define PRINT_iv			print__0
# define PRINT_lv			print__0

# undef NEED_COUNT__0
# define NEED_COUNT__0			1

# undef NEED_PRINT__0
# define NEED_PRINT__0			1
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#define SECOND_A(v)			((v)->tv_sec)
#define SECOND_F(v)			((v)->tv_usec)

#define SECOND_a(v)			((v).tv_sec)
#define SECOND_f(v)			((v).tv_usec)

#define count_digIts			Mylene1f4_count_pdigits
#define count_digits			Mylene1f4_count_digits

#define print_cardInal			Mylene1f4_print_pcardinal
#define print_cardinal			Mylene1f4_print_cardinal

#define scope_4_e0			_Mylene1f4_format4_0
#define scope_g_e0			_Mylene1f4_formatg_0
#define scope_l_e0			_Mylene1f4_formatl_0
#define scope_t_e0			_Mylene1f4_formatt_0
#define scope_t_e1			_Mylene1f4_formatt_1
#define scope_t_e2			_Mylene1f4_formatt_2

#define B1st(printer) \
    (((struct printer_state *) (printer))->b1st)
#define CCACHE(printer) \
    (((struct printer_state *) (printer))->ccache)
#define DECK(printer) \
    (((struct printer_state *) (printer))->deck)
#define FFLAGS(printer) \
    (((struct printer_state *) (printer))->fflags)
#define FLOW(printer) \
    (((struct printer_state *) (printer))->flow)
#define INDEX(printer) \
    (((struct printer_state *) (printer))->index)
#define MISS(printer) \
    (((struct printer_state *) (printer))->extension)
#define READ(printer) \
    (((struct printer_state *) (printer))->read)
#define RUN_SSTRING(printer) \
    (((struct printer_state *) (printer))->run_sstring)
#define RUN_ZSTRING(printer) \
    (((struct printer_state *) (printer))->run_zstring)

#define Context(printer) \
    (((struct printer_state *) (printer))->context)

#define Excess(printer) \
    EXCESS(Context(printer))
#define Flags(printer) \
    FLAGS(Context(printer))

#define Mirror(mirror) \
    ((struct mirror_type *) (mirror))

#define SCOPE(printer) \
    ((struct ksscan_type *) ((struct printer_state *) (printer))->scope)

#define TYPE_K(printer, index, type) \
    ((type *) SCOPE((printer))->data[(index)])

#define K_1f42(printer) \
    TYPE_K(printer, M66A_SCAN, struct x1f42e_type)
#define K_A52A(printer) \
    TYPE_K(printer, A52A_SCAN, struct xa52a1_type)
#define K_BAND(printer) \
    TYPE_K(printer, BAND_SCAN, struct l3band_type)
#define K_GROUP(printer) \
    TYPE_K(printer, LOOK_SCAN, struct group)
#define K_INFO(printer) \
    TYPE_K(printer, INFO_SCAN, struct stat)
#define K_LPCM(printer) \
    TYPE_K(printer, LPCM_SCAN, struct xepcm1_type)
#define K_MDEV(printer) \
    TYPE_K(printer, MDEV_SCAN, dev_t)
#define K_MPEG(printer) \
    TYPE_K(printer, MPEG_SCAN, struct xmpeg1_type)
#define K_NSSP(printer) \
    TYPE_K(printer, NSSP_SCAN, struct xmpeg0_type)
#define K_STAT(printer) \
    TYPE_K(printer, STAT_SCAN, struct stat_type)
#define K_TIME(printer) \
    TYPE_K(printer, TIME_SCAN, struct fttime_type)
#define K_USER(printer) \
    TYPE_K(printer, USER_SCAN, struct passwd)

#define SA_TIME(printer) \
    (((struct printer_state *) (printer))->sa_time)

#define x1f42(context) \
    ((struct x1f42a_type *) X1f42((context)))

typedef struct atom_type {
    struct atom_type *atom_data;
} atom_type;

typedef struct miss_type {
    struct miss_type *miss_data;
    unsigned code;
} miss_type;

typedef struct stat_type {
    struct stat info;
    unsigned mode;
    void *class, *group, *passwd;
} stat_type;

extern const struct x1f4_efield_type _Mylene1f4_format4_0[],
    _Mylene1f4_formatg_0[], _Mylene1f4_formatl_0[], _Mylene1f4_formatt_0[],
    _Mylene1f4_formatt_1[], _Mylene1f4_formatt_2[];
extern const unsigned _Mylene1f4_c_type_s[];

static int fix_extension(void *, unsigned, unsigned, void **);
static int link_format(void *, struct printer_state *);
static int net_extension(void *, unsigned, void **);
static int print_sstring(void *, const char *, unsigned);
static int print_zstring(void *, const char *);
static int set_extension(void *, unsigned, unsigned, void **);
static int store_sstring(void *, const char *, unsigned);
static int store_zstring(void *, const char *);

#if NEED_COUNT_13
static unsigned count_13(void *, unsigned);
#endif				/* NEED_COUNT_13 */
static unsigned count_15(void *, unsigned);
static unsigned count_A1(void *, unsigned);
static unsigned count_Ah(void *, unsigned);
static unsigned count_Ar(void *, unsigned);
static unsigned count_B1(void *, unsigned);
static unsigned count_B4(void *, unsigned);
#if defined HAVE_ST_BLKSIZE
static unsigned count_B6(void *, unsigned);
#endif				/* HAVE_ST_BLKSIZE */
static unsigned count_B_(void *, unsigned);
static unsigned count_E4(void *, unsigned);
static unsigned count_E_(void *, unsigned);
static unsigned count_Ei(void *, unsigned);
static unsigned count_Em(void *, unsigned);
static unsigned count_En(void *, unsigned);
static unsigned count_F0(void *, unsigned);
static unsigned count_G6(void *, unsigned);
static unsigned count_G_(void *, unsigned);
static unsigned count_L_(void *, unsigned);
#if __ENABLE_x1f4LANES__
static unsigned count_Ld(void *, unsigned);
#endif				/* __ENABLE_x1f4LANES__ */
static unsigned count_M3(void *, unsigned);
static unsigned count_M4(void *, unsigned);
static unsigned count_M6(void *, unsigned);
static unsigned count_M7(void *, unsigned);
static unsigned count_N_(void *, unsigned);
#if __ENABLE_CONTROLLING_WINDOW__
static unsigned count_Nd(void *, unsigned);
#endif				/* __ENABLE_CONTROLLING_WINDOW__ */
static unsigned count_O_(void *, unsigned);
static unsigned count_P1(void *, unsigned);
static unsigned count_P_(void *, unsigned);
static unsigned count_R_(void *, unsigned);
static unsigned count_S_(void *, unsigned);
#if __ENABLE_CONTROLLING_SERVER__
static unsigned count_Sd(void *, unsigned);
#endif				/* __ENABLE_CONTROLLING_SERVER__ */
static unsigned count_U6(void *, unsigned);
static unsigned count_X_(void *, unsigned);
static unsigned count_Y_(void *, unsigned);
#if NEED_COUNT__0
static unsigned count__0(void *, unsigned);
#endif				/* NEED_COUNT__0 */
static unsigned count__1(void *, unsigned);
static unsigned count__2(void *, unsigned);
static unsigned count__9(void *, unsigned);
static unsigned count_a1(void *, unsigned);
static unsigned count_a3(void *, unsigned);
static unsigned count_a6(void *, unsigned);
static unsigned count_a_(void *, unsigned);
#if __have_Mylene1f4_version__
static unsigned count_ak(void *, unsigned);
#endif				/* __have_Mylene1f4_version__ */
#if __ENABLE_MPEG_EQUALIZER__
static unsigned count_at(void *, unsigned);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
#if defined HAVE_ST_BLOCKS
static unsigned count_b6(void *, unsigned);
#endif				/* HAVE_ST_BLOCKS */
static unsigned count_b_(void *, unsigned);
#if __ENABLE_MPEG_EQUALIZER__
static unsigned count_bu(void *, unsigned);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
static unsigned count_c1(void *, unsigned);
static unsigned count_c6(void *, unsigned);
static unsigned count_c_(void *, unsigned);
#if __ENABLE_MPEG_EQUALIZER__
static unsigned count_cu(void *, unsigned);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
static unsigned count_d5(void *, unsigned);
static unsigned count_d6(void *, unsigned);
static unsigned count_d_(void *, unsigned);
static unsigned count_do(void *, unsigned);
static unsigned count_dp(void *, unsigned);
static unsigned count_e4(void *, unsigned);
static unsigned count_e_(void *, unsigned);
static unsigned count_ei(void *, unsigned);
static unsigned count_em(void *, unsigned);
static unsigned count_en(void *, unsigned);
#if __ENABLE_MPEG_EQUALIZER__
static unsigned count_et(void *, unsigned);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
static unsigned count_f4(void *, unsigned);
static unsigned count_f_(void *, unsigned);
static unsigned count_ff(void *, unsigned);
#if __ENABLE_MPEG_EQUALIZER__
static unsigned count_fv(void *, unsigned);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
static unsigned count_g6(void *, unsigned);
static unsigned count_h3(void *, unsigned);
static unsigned count_i6(void *, unsigned);
static unsigned count_i_(void *, unsigned);
static unsigned count_ig(void *, unsigned);
#if __ENABLE_EQUEUE__
static unsigned count_ik(void *, unsigned);
#endif				/* __ENABLE_EQUEUE__ */
#if __ENABLE_MPEG_EQUALIZER__
static unsigned count_iv(void *, unsigned);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
static unsigned count_l0(void *, unsigned);
static unsigned count_l3(void *, unsigned);
static unsigned count_l4(void *, unsigned);
static unsigned count_l_(void *, unsigned);
#if __ENABLE_MPEG_EQUALIZER__
static unsigned count_lv(void *, unsigned);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
#if __ENABLE_MPEG_EQUALIZER__
static unsigned count_lw(void *, unsigned);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
static unsigned count_m3(void *, unsigned);
static unsigned count_m4(void *, unsigned);
static unsigned count_m6(void *, unsigned);
static unsigned count_m7(void *, unsigned);
static unsigned count_n6(void *, unsigned);
static unsigned count_n9(void *, unsigned);
static unsigned count_na(void *, unsigned);
static unsigned count_nb(void *, unsigned);
static unsigned count_no(void *, unsigned);
static unsigned count_np(void *, unsigned);
static unsigned count_odigits(unsigned);
static unsigned count_p2(void *, unsigned);
static unsigned count_pb(void *, unsigned);
static unsigned count_pp(void *, unsigned);
static unsigned count_pr(void *, unsigned);
static unsigned count_qp(void *, unsigned);
static unsigned count_r6(void *, unsigned);
static unsigned count_r_(void *, unsigned);
static unsigned count_s2(void *, unsigned);
static unsigned count_s5(void *, unsigned);
static unsigned count_s6(void *, unsigned);
static unsigned count_s_(void *, unsigned);
static unsigned count_so(void *, unsigned);
static unsigned count_string(void *, char *, unsigned);
static unsigned count_t0(void *, unsigned);
static unsigned count_t1(void *, unsigned);
static unsigned count_t2(void *, unsigned);
static unsigned count_t_(void *, unsigned);
static unsigned count_tf(void *, unsigned);
static unsigned count_u2(void *, unsigned);
static unsigned count_u6(void *, unsigned);
static unsigned count_w_(void *, unsigned);
static unsigned count_x1(void *, unsigned);
static unsigned count_x_(void *, unsigned);
static unsigned count_ym(void *, unsigned);
static unsigned count_z_(void *, unsigned);
static unsigned count_zb(void *, unsigned);
static unsigned scale_t_(void *);
static unsigned set_time(void *, unsigned, time_t);
static unsigned shift_s2(void *);
static unsigned shift_t0(void *);
static unsigned shift_t_(void *);
static unsigned shift_tf(void *);
static unsigned so_L_(void *, struct stat_type *);
static unsigned so_S_(void *, struct stat_type *);
static unsigned stat_file(void *, unsigned, unsigned,
			  unsigned (*) (void *, struct stat_type *));

static void fini_ksscan(struct ksscan_type *);
static void fini_printer(struct printer_state *);
static void get_m4(void *, const char **);
static void get_scope(void *, unsigned);
#if !__DUMMY_INDEX__
static void index___(void *, unsigned);
#endif				/* !__DUMMY_INDEX__ */
#if __ENABLE_MPEG_EQUALIZER__
static void index_bu(void *, unsigned);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
static void init_stat(struct stat_type *);
static void print_A1(void *, unsigned);
static void print_B1(void *, unsigned);
static void print_B4(void *, unsigned);
#if defined HAVE_ST_BLKSIZE
static void print_B6(void *, unsigned);
#endif				/* HAVE_ST_BLKSIZE */
static void print_B_(void *, unsigned);
static void print_C_(void *, unsigned);
static void print_E4(void *, unsigned);
static void print_E_(void *, unsigned);
static void print_Ei(void *, unsigned);
static void print_Em(void *, unsigned);
static void print_En(void *, unsigned);
static void print_F0(void *, unsigned);
static void print_L4(void *, unsigned);
static void print_M4(void *, unsigned);
static void print_M6(void *, unsigned);
static void print_M7(void *, unsigned);
static void print_N_(void *, unsigned);
static void print_O_(void *, unsigned);
static void print_P1(void *, unsigned);
static void print_P_(void *, unsigned);
static void print_R_(void *, unsigned);
static void print_S4(void *, unsigned);
static void print_Sj(void *, unsigned);
static void print_Y_(void *, unsigned);
#if NEED_PRINT__0
static void print__0(void *, unsigned);
#endif				/* NEED_PRINT__0 */
static void print_a1(void *, unsigned);
static void print_a_(void *, unsigned);
#if __have_Mylene1f4_version__
static void print_ak(void *, unsigned);
#endif				/* __have_Mylene1f4_version__ */
#if defined HAVE_ST_BLOCKS
static void print_b6(void *, unsigned);
#endif				/* HAVE_ST_BLOCKS */
static void print_b_(void *, unsigned);
static void print_c1(void *, unsigned);
static void print_c_(void *, unsigned);
#if __ENABLE_MPEG_EQUALIZER__
static void print_cu(void *, unsigned);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
static void print_d5(void *, unsigned);
static void print_d_(void *, unsigned);
static void print_e4(void *, unsigned);
static void print_e_(void *, unsigned);
static void print_ei(void *, unsigned);
static void print_em(void *, unsigned);
static void print_en(void *, unsigned);
static void print_f4(void *, unsigned);
static void print_f_(void *, unsigned);
#if __ENABLE_MPEG_EQUALIZER__
static void print_fv(void *, unsigned);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
static void print_g6(void *, unsigned);
static void print_i6(void *, unsigned);
static void print_i_(void *, unsigned);
static void print_ig(void *, unsigned);
#if __ENABLE_EQUEUE__
static void print_ik(void *, unsigned);
#endif				/* __ENABLE_EQUEUE__ */
#if __ENABLE_MPEG_EQUALIZER__
static void print_iv(void *, unsigned);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
static void print_l4(void *, unsigned);
#if __ENABLE_MPEG_EQUALIZER__
static void print_lv(void *, unsigned);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
#if __ENABLE_MPEG_EQUALIZER__
static void print_lw(void *, unsigned);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
static void print_m4(void *, unsigned);
static void print_m7(void *, unsigned);
static void print_n6(void *, unsigned);
static void print_n9(void *, unsigned);
static void print_na(void *, unsigned);
static void print_np(void *, unsigned);
static void print_ocardinal(void *, unsigned);
static void print_p2(void *, unsigned);
static void print_pp(void *, unsigned);
static void print_qp(void *, unsigned);
static void print_r_(void *, unsigned);
static void print_s6(void *, unsigned);
static void print_s_(void *, unsigned);
static void print_t1(void *, unsigned);
static void print_tq(void *, unsigned);
static void print_u6(void *, unsigned);
#if NEED_PRINT_VERSION
static void print_version(void *, unsigned);
#endif				/* NEED_PRINT_VERSION */
static void print_w_(void *, unsigned);
static void print_x1(void *, unsigned);
static void print_z_(void *, unsigned);
static void put_space(void *, unsigned, char *, unsigned);
static void put_string(void *, char *, unsigned);
static void set_index(void *, unsigned);
static void set_scope(void *, unsigned);
static void store_O_(void *);
static void store_c_(void *);
static void store_e_(void *);
static void store_r_(void *);
static void store_z_(void *);

static const char digits[] = "0123456789";
/* tc{8,c,y} available... */
static const struct x1f4_efield_type tcz_fields[] = {
/* *INDENT-OFF* */
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tcx_fields[] = {
/* *INDENT-OFF* */
    {	"name",
	NULL,		count_np,	0,
	0156,		print_np				},
    {	"path",
	NULL,		count_pp,	0,
	0160,		print_pp				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tcw_fields[] = {
/* *INDENT-OFF* */
    {	"level",
	NULL,		COUNT_lw,	0,
	0154,		PRINT_lw				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tcv_fields[] = {
/* *INDENT-OFF* */
    {	"frequency",
	NULL,		COUNT_fv,	0,
	0146,		PRINT_fv				},
    {	"index",
	NULL,		COUNT_iv,	0,
	0151,		PRINT_iv				},
    {	"level",
	NULL,		COUNT_lv,	0,
	0154,		PRINT_lv				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tcu_fields[] = {
/* *INDENT-OFF* */
    {	"band",
	tcv_fields,	COUNT_bu,	X1f4_FORMAT_ITERATE,
	0142,		NULL					},
    {	"bands count",
	NULL,		COUNT_cu,	0,
	0143,		PRINT_cu				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tct_fields[] = {
/* *INDENT-OFF* */
    {	"amplification",
	tcw_fields,	COUNT_at,	0,
	0141,		NULL					},
    {	"equalizer",
	tcu_fields,	COUNT_et,	0,
	0145,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tcs_fields[] = {
/* *INDENT-OFF* */
    {	"mpeg stream",
	tct_fields,	count__1,	0,
	0155,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tcr_fields[] = {
/* *INDENT-OFF* */
    {	"\"attached\"",
	tcz_fields,	count_Ar,	0,
	0101,		NULL					},
    {	"\"playing\"",
	tcz_fields,	count_pr,	0,
	0160,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tcq_fields[] = {
/* *INDENT-OFF* */
    {	"track id",
	NULL,		count__2,	0,
	0164,		print_tq				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tcp_fields[] = {
/* *INDENT-OFF* */
    {	"reported state",
	tcr_fields,	count_dp,	0,
	0144,		NULL					},
    {	"name",
	NULL,		count_np,	0,
	0156,		print_np				},
    {	"path",
	NULL,		count_pp,	0,
	0160,		print_pp				},
    {	"process id",
	NULL,		count_qp,	0,
	0161,		print_qp				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tco_fields[] = {
/* *INDENT-OFF* */
    {	"device",
	tcp_fields,	count_do,	0,
	0144,		NULL					},
    {	"null",
	tcz_fields,	count_no,	0,
	0156,		NULL					},
    {	"skip",
	tcz_fields,	count_so,	0,
	0163,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tcn_fields[] = {
/* *INDENT-OFF* */
    {	"frame rate denominator",
	NULL,		count_En,	0,
	0105,		print_En				},
    {	"frame rate numerator",
	NULL,		count_en,	0,
	0145,		print_en				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tcm_fields[] = {
/* *INDENT-OFF* */
    {	"frame rate denominator",
	NULL,		count_Em,	0,
	0105,		print_Em				},
    {	"frame rate numerator",
	NULL,		count_em,	0,
	0145,		print_em				},
    {	"system",
	tcq_fields,	count_ym,	0,
	0171,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tcl_fields[] = {
/* *INDENT-OFF* */
    {	"device",
	tcx_fields,	count_do,	0,
	0144,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tck_fields[] = {
/* *INDENT-OFF* */
    {	"Mylene1f4 MPEG audio decoder version",
	NULL,		COUNT_AK,	0,
	0141,		PRINT_AK				},
    {	"aime interpreter version",
	NULL,		COUNT_IK,	0,
	0151,		PRINT_IK				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tcj_fields[] = {
/* *INDENT-OFF* */
    {	"specification",
	NULL,		count_15,	0,
	0123,		print_Sj				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tci_fields[] = {
/* *INDENT-OFF* */
    {	"entry count",
	NULL,		count_Ei,	0,
	0105,		print_Ei				},
    {	"entry index",
	NULL,		count_ei,	0,
	0145,		print_ei				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tch_fields[] = {
/* *INDENT-OFF* */
    {	"multiple entries map",
	tci_fields,	count_Ah,	0,
	0101,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tcg_fields[] = {
/* *INDENT-OFF* */
    {	"index",
	NULL,		count_ig,	0,
	0151,		print_ig				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tcf_fields[] = {
/* *INDENT-OFF* */
    {	"frame",
	tcg_fields,	count_ff,	0,
	0146,		NULL					},
    {	"time",
	scope_t_e0,	count_tf,	X1f4_FORMAT_ITERATE,
	0164,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tce_fields[] = {
/* *INDENT-OFF* */
    {	"start",
	tcf_fields,	count__1,	0,
	0123,		NULL					},
    {	"output",
	tcl_fields,	count__1,	0,
	0126,		NULL,					},
    {	"loaders",
	tcs_fields,	count__1,	0,
	0154,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tcd_fields[] = {
/* *INDENT-OFF* */
    {	"lanes",
	tcz_fields,	COUNT_Ld,	0,
	0114,		NULL					},
    {	"none",
	tcz_fields,	COUNT_Nd,	0,
	0116,		NULL					},
    {	"server",
	scope_4_e0,	COUNT_Sd,	0,
	0123,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tcb_fields[] = {
/* *INDENT-OFF* */
    {	"\"next\" order",
	tcz_fields,	count_nb,	0,
	0156,		NULL					},
    {	"\"previous\" order",
	tcz_fields,	count_pb,	0,
	0160,		NULL					},
    {	"\"random\" order",
	tcz_fields,	count_zb,	0,
	0172,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
}, tca_fields[] = {
/* *INDENT-OFF* */
    {	"name",
	NULL,		count_na,	0,
	0156,		print_na				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tc9_fields[] = {
/* *INDENT-OFF* */
    {	"name",
	NULL,		count_n9,	0,
	0156,		print_n9				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tc7_fields[] = {
/* *INDENT-OFF* */
    {	"major",
	NULL,		count_M7,	0,
	0115,		print_M7				},
    {	"minor",
	NULL,		count_m7,	0,
	0155,		print_m7				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tc6_fields[] = {
/* *INDENT-OFF* */
    {	"blocksize for filesystem I/O",
	NULL,		COUNT_b6,	0,
	0102,		PRINT_b6				},
    {	"group",
	tc9_fields,	count_G6,	0,
	0107,		NULL					},
    {	"protection",
	NULL,		count_M6,	0,
	0115,		print_M6				},
    {	"user",
	tca_fields,	count_U6,	0,
	0125,		NULL					},
    {	"time of last access",
	scope_t_e1,	count_a6,	0,
	0141,		NULL					},
    {	"number of blocks allocated",
	NULL,		COUNT_B6,	0,
	0142,		PRINT_B6				},
    {	"time of last change",
	scope_t_e1,	count_c6,	0,
	0143,		NULL					},
    {	"device",
	tc7_fields,	count_d6,	0,
	0144,		NULL					},
    {	"group ID",
	NULL,		count_g6,	0,
	0147,		print_g6				},
    {	"inode",
	NULL,		count_i6,	0,
	0151,		print_i6				},
    {	"time of last modification",
	scope_t_e1,	count_m6,	0,
	0155,		NULL					},
    {	"number of hard links",
	NULL,		count_n6,	0,
	0156,		print_n6				},
    {	"device type",
	tc7_fields,	count_r6,	0,
	0162,		NULL					},
    {	"total size, in bytes",
	NULL,		count_s6,	0,
	0163,		print_s6				},
    {	"user ID",
	NULL,		count_u6,	0,
	0165,		print_u6				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tc5_fields[] = {
/* *INDENT-OFF* */
    {	"time of day",
	scope_t_e0,	count__1,	0,
	0124,		NULL					},
    {	"day",
	NULL,		count_d5,	0,
	0144,		print_d5				},
    {	"second day",
	tcz_fields,	count_s5,	0,
	0163,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tc4_fields[] = {
/* *INDENT-OFF* */
    {	"bitrate",
	NULL,		count_B4,	0,
	0102,		print_B4				},
    {	"frame rate denominator",
	NULL,		count_E4,	0,
	0105,		print_E4				},
    {	"layer (arabic)",
	NULL,		count__1,	0,
	0114,		print_L4				},
    {	"mode extension",
	NULL,		count_M4,	0,
	0115,		print_M4				},
    {	"specification",
	NULL,		count_15,	0,
	0123,		print_S4				},
    {	"frame rate numerator",
	NULL,		count_e4,	0,
	0145,		print_e4				},
    {	"frame rate (real periodic representation)",
	NULL,		count_f4,	0,
	0146,		print_f4				},
    {	"layer",
	NULL,		count_l4,	0,
	0154,		print_l4				},
    {	"mode",
	NULL,		count_m4,	0,
	0155,		print_m4				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tc3_fields[] = {
/* *INDENT-OFF* */
    {	"mpeg system",
	tcj_fields,	count_M3,	0,
	0115,		NULL					},
    {	"A52a",
	tcn_fields,	count_a3,	0,
	0141,		NULL					},
    {	"1f42",
	tch_fields,	count_h3,	0,
	0150,		NULL					},
    {	"Linear PCM",
	tcm_fields,	count_l3,	0,
	0154,		NULL					},
    {	"mpeg stream",
	tc4_fields,	count_m3,	0,
	0155,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tc2_fields[] = {
/* *INDENT-OFF* */
    {	"process id",
	NULL,		count_p2,	0,
	0160,		print_p2				},
    {	"start time",
	scope_t_e2,	count_s2,	X1f4_FORMAT_ITERATE,
	0163,		NULL					},
    {	"current time",
	scope_t_e2,	count_t2,	0,
	0164,		NULL					},
    {	"up time",
	tc5_fields,	count_u2,	0,
	0165,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tc1_fields[] = {
/* *INDENT-OFF* */
    {	"album",
	NULL,		count_A1,	0,
	0101,		print_A1				},
    {	"basename",
	NULL,		count_B1,	0,
	0102,		print_B1				},
    {	"path",
	NULL,		count_P1,	0,
	0120,		print_P1				},
    {	"artist",
	NULL,		count_a1,	0,
	0141,		print_a1				},
    {	"comment",
	NULL,		count_c1,	0,
	0143,		print_c1				},
    {	"title",
	NULL,		count_t1,	0,
	0164,		print_t1				},
    {	"expression",
	NULL,		count_x1,	0,
	0170,		print_x1				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tc0_fields[] = {
/* *INDENT-OFF* */
    {	"frame count",
	NULL,		count_F0,	0,
	0106,		print_F0				},
    {	"stream length",
	scope_t_e0,	count_l0,	0,
	0154,		NULL					},
    {	"time to end of stream",
	scope_t_e0,	count_t0,	X1f4_FORMAT_ITERATE,
	0164,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
};
static const struct x1f4_f6link_type f6link = {
/* *INDENT-OFF* */
    count_string,
    put_space,
    put_string,
    set_index,
    set_scope,
    get_scope			/* HAVE_LIBLEWD */
/* *INDENT-ON* */
};
static const struct x1f4_ffield_type run_fields[] = {
/* *INDENT-OFF* */
    {	"clip",
	NULL,		count_B_,	0,
	0102,		print_B_				},
    {	"class (Mylene1f4)",
	NULL,		count__9,	0,
	0103,		print_C_				},
    {	"release size",
	NULL,		count_E_,	0,
	0105,		print_E_				},
    {	"",
	scope_g_e0,	count_G_,	0,
	0107,		NULL					},
    {	"file by lstat(2)",
	tc6_fields,	count_L_,	0,
	0114,		NULL,					},
    {	"setup",
	tce_fields,	count__1,	0,
	0115,		NULL					},
    {	"path",
	NULL,		count_N_,	0,
	0116,		print_N_				},
    {	"file offset",
	NULL,		count_O_,	0,
	0117,		print_O_				},
    {	"play",
	NULL,		count_P_,	0,
	0120,		print_P_				},
    {	"controller",
	tcd_fields,	count__1,	0,
	0121,		NULL					},
    {	"name",
	NULL,		count_R_,	0,
	0122,		print_R_				},
    {	"file by stat(2)",
	tc6_fields,	count_S_,	0,
	0123,		NULL,					},
    {	"output",
	tco_fields,	count__1,	0,
	0126,		NULL,					},
    {	"filter",
	tc1_fields,	count_X_,	0,
	0130,		NULL,					},
    {	"version" " (" VERSION ")",
	NULL,		count_Y_,	0,
	0131,		print_Y_				},
    {	"process",
	tc2_fields,	count__1,	0,
	0132,		NULL,					},
    {	"audio device frame index (approximation)",
	NULL,		count_a_,	0,
	0141,		print_a_				},
    {	"basename",
	NULL,		count_b_,	0,
	0142,		print_b_				},
    {	"current file index",
	NULL,		count_c_,	0,
	0143,		print_c_				},
    {	"dirname",
	NULL,		count_d_,	0,
	0144,		print_d_				},
    {	"cache",
	NULL,		count_e_,	0,
	0145,		print_e_				},
    {	"frame index",
	NULL,		count_f_,	0,
	0146,		print_f_				},
    {	"initial file index",
	NULL,		count_i_,	0,
	0151,		print_i_				},
    {	"file list",
	scope_l_e0,	count_l_,	0,
	0154,		NULL					},
    {	"load order",
	tcb_fields,	count__1,	0,
	0157,		NULL					},
    {	"read index",
	NULL,		count_r_,	0,
	0162,		print_r_				},
    {	"sampling",
	NULL,		count_s_,	0,
	0163,		print_s_				},
    {	"stream time",
	scope_t_e0,	count_t_,	X1f4_FORMAT_ITERATE,
	0164,		NULL					},
    {	"source",
	tc3_fields,	count__1,	0,
	0166,		NULL					},
    {	"write index",
	NULL,		count_w_,	0,
	0167,		print_w_				},
    {	"1f42",
	tc0_fields,	count_x_,	0,
	0170,		NULL					},
    {	"detailed version",
	tck_fields,	count__1,	0,
	0171,		NULL					},
    {	"bytes to EOF",
	NULL,		count_z_,	0,
	0172,		print_z_				},
    {	NULL,
	NULL,		NULL,		0,
	0,		NULL					}
/* *INDENT-ON* */
};

static int
fix_extension(void *printer, unsigned code, unsigned size, void **data)
{
    int status;

    status = net_extension(printer, code, data);
    if (status) {
	status = set_extension(printer, code, size, data);
    }

    return status;
}


static int
link_format(void *format, struct printer_state *printer_data)
{
    struct ksscan_type ksscan_data;

    printer_data->fflags |= ~(Mylene1f4_MODE_FIELD | Mylene1f4_MODE_FRAME);
    printer_data->format = format;
    printer_data->scope = &ksscan_data;
    printer_data->scope_failures = 0;

    ksscan_data.atom_data = NULL;
    ksscan_data.scope = 0;

#if !__DUMMY_INDEX__
    ksscan_data.index___ = index___;
#endif				/* !__DUMMY_INDEX__ */

    x1f4_run_format(format, printer_data);

    fini_ksscan(&ksscan_data);

    fini_printer(printer_data);

    return 0;
}


static int
net_extension(void *printer, unsigned code, void **data)
{
    int status = 1;
    struct miss_type *miss_data;
    void *text = NULL;

    miss_data = MISS(printer);
    while (miss_data) {
	if (miss_data->code == code) {
	    void *miss;

	    miss = miss_data + 1;

	    text = ALIGN_ZBUFFER(miss);

	    status = 0;

	    break;
	}

	miss_data = miss_data->miss_data;
    }

    *data = text;

    return status;
}


static int
print_sstring(void *printer, const char *buffer, unsigned length)
{
    if (length) {
	void *context;

	context = Context(printer);
	LINE(context) (context, buffer, length);
    }

    return 0;
}


static int
print_zstring(void *printer, const char *string)
{
    return print_sstring(printer, string, strlen(string));
}


static int
set_extension(void *printer, unsigned code, unsigned size, void **data)
{
    int status = 0;
    void *miss;

#if MISS_SHIFT
    if (!((sizeof(struct miss_type) | ALIGN_Z_PITCH) & MISS_SCRAP)) {
	unsigned scrap, shift;

	shift = MISS_SHIFT;
	scrap = size & MISS_SCRAP;
	if (scrap) {
	    size += MISS_SLEEP;
	}
	size >>= MISS_SHIFT;
	miss = malloc
	    (sizeof(struct miss_type)
	     + ((size + (ALIGN_Z_PITCH >> MISS_SHIFT) - 1) << MISS_SHIFT));
    } else {
#endif				/* MISS_SHIFT */
	miss = malloc(sizeof(struct miss_type) + ALIGN_ZMALLOC(size));
#if MISS_SHIFT
    }
#endif				/* MISS_SHIFT */
    if (!miss) {
	status = -1;
    } else {
	struct miss_type *miss_data;

	miss_data = miss;

	miss_data->miss_data = MISS(printer);
	miss_data->code = code;

	MISS(printer) = miss_data;

	miss = miss_data + 1;

	*data = ALIGN_ZBUFFER(miss);
    }

    return status;
}


static int
store_sstring(void *printer, const char *buffer, unsigned length)
{
    int status = 0;
    struct buffer_type *buffer_data;

    buffer_data = ((struct printer_state *) printer)->data.buffer_data;
    if (!~buffer_data->extent) {
	status = -1;
    } else {
	do {
	    if (buffer_data->extent < buffer_data->length + length + 1) {
		unsigned extent;
		void *data;

		extent = buffer_data->length + length + 1;
		extent += (512 - (extent & 511)) & 511;
		if (!(data = realloc(buffer_data->data, extent))) {
		    buffer_data->extent = ~0;
		    if (buffer_data->data) {
			free(buffer_data->data);
			buffer_data->data = NULL;
		    }

		    status = -1;

		    break;
		}

		buffer_data->data = data;
		buffer_data->extent = extent;
	    }

	    memcpy((char *) buffer_data->data + buffer_data->length, buffer,
		   length);
	    buffer_data->length += length;
	} while (0);
    }

    return status;
}


static int
store_zstring(void *printer, const char *string)
{
    return store_sstring(printer, string, strlen(string));
}


#if NEED_COUNT_13
static unsigned
count_13(void *printer, unsigned flags)
{
    return 015;
}
#endif				/* NEED_COUNT_13 */


static unsigned
count_15(void *printer, unsigned flags)
{
    return 15;
}


static unsigned
count_A1(void *printer, unsigned flags)
{
    void *context;

    context = Context(printer);

    return !ALBUM(context).data ? 0 : ALBUM(context).size;
}


static unsigned
count_Ah(void *printer, unsigned flags)
{
    return K_1f42(printer)->count != 1 ? 1 : 0;
}


static unsigned
count_Ar(void *printer, unsigned flags)
{
    return B1st(printer) & Mylene1f4_B1st_DELAYED ? 0 : 1;
}


static unsigned
count_B1(void *printer, unsigned flags)
{
    void *context;

    context = Context(printer);

    return !LBASE(context).data ? 0 : LBASE(context).size;
}


static unsigned
count_B4(void *printer, unsigned flags)
{
    unsigned count;

    count = K_MPEG(printer)->bitrate;
    if (!~count) {
	count = 1;
    } else {
	count = count_digits(flags, count);
    }

    return count;
}


#if __ENABLE_MPEG_EQUALIZER__
static unsigned
count_at(void *printer, unsigned flags)
{
    return !MPEG(Context(printer)).amplification ? 0 : 1;
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if defined HAVE_ST_BLKSIZE
static unsigned
count_B6(void *printer, unsigned flags)
{
    return count_digits(flags, K_INFO(printer)->st_blksize);
}
#endif				/* HAVE_ST_BLKSIZE */


static unsigned
count_B_(void *printer, unsigned flags)
{
    return count_digits(flags, CLIP(Context(printer)));
}


static unsigned
count_E4(void *printer, unsigned flags)
{
    return count_digits(flags, K_MPEG(printer)->frame_rate_d);
}


static unsigned
count_E_(void *printer, unsigned flags)
{
    return count_digits(flags, RELEASE(Context(printer)));
}


static unsigned
count_Ei(void *printer, unsigned flags)
{
    return count_digits(flags, K_1f42(printer)->count);
}


static unsigned
count_Em(void *printer, unsigned flags)
{
    return count_digits(flags, K_LPCM(printer)->frame_rate_d);
}


static unsigned
count_En(void *printer, unsigned flags)
{
#if !__LINK_A52A_LOADER__
    return 0;
#else
    return count_digits(flags, K_A52A(printer)->frame_rate_d);
#endif				/* !__LINK_A52A_LOADER__ */
}


static unsigned
count_F0(void *printer, unsigned flags)
{
    return count_digits(flags, x1f42(Context(printer))->frames);
}


static unsigned
count_G6(void *printer, unsigned flags)
{
    int status;
    struct stat_type *stat_data;

    stat_data = K_STAT(printer);
    if (stat_data->group) {
	SCOPE(printer)->data[LOOK_SCAN] = stat_data->group;
	status = 1;
    } else {
	unsigned mode, size_max;
	void *miss;

	size_max = sysconf(_SC_GETGR_R_SIZE_MAX);

	mode = stat_data->mode;
	if (mode != none_MODE) {
	    mode -= COFF_MODE;
	    mode += MASS_MODE;
	}
	if (fix_extension
	    (printer, mode, sizeof(struct group) + size_max, &miss)) {
	    status = 0;
	} else {
	    struct group *group;

	    group = (struct group *) miss;
	    status = getgrgid_r
		(stat_data->info.st_gid, group, (char *) (group + 1), size_max,
		 &group);
	    if (status) {
		status = 0;
	    } else {
		SCOPE(printer)->data[LOOK_SCAN] = group;
		stat_data->group = group;
		status = 1;
	    }
	}
    }

    return status;
}


static unsigned
count_G_(void *printer, unsigned flags)
{
    return !ID3X(Context(printer)) ? 0 : 1;
}


static unsigned
count_L_(void *printer, unsigned flags)
{
    return stat_file(printer, GET_LSTAT, COFF_MODE, so_L_);
}


#if __ENABLE_x1f4LANES__
static unsigned
count_Ld(void *printer, unsigned flags)
{
    return ((MODE(Context(printer)) & Mylene1f4_WINDOW_MASK)
	    != Mylene1f4_WITHIN_WINDOW ? 0 : 1);
}
#endif				/* __ENABLE_x1f4LANES__ */


static unsigned
count_M3(void *printer, unsigned flags)
{
    int status;
    struct sequel_type *sequel_data;

    status = Mylene1f4_find_sequel
	(Context(printer), Mylene1f4_XMPEG0_SEQUEL, (void *) &sequel_data);
    if (status) {
	SCOPE(printer)->data[NSSP_SCAN] = sequel_data->data;
    }

    return status;
}


static unsigned
count_M4(void *printer, unsigned flags)
{
    return count_digits(flags, K_MPEG(printer)->mode_extension);
}


static unsigned
count_M6(void *printer, unsigned flags)
{
    return count_odigits(K_INFO(printer)->st_mode);
}


static unsigned
count_M7(void *printer, unsigned flags)
{
    return count_digits(flags, major(*K_MDEV(printer)));
}


static unsigned
count_N_(void *printer, unsigned flags)
{
    unsigned length;

    if (flags & X1f4_FORMAT_ALTERNATE) {
	length = 0;
	x1f4_nprint_sstring
	    (&length, SOURCE(Context(printer))._.s1.path, _Mylene1f4_c_type_s);
    } else {
	length = strlen(SOURCE(Context(printer))._.s1.path);
    }

    return length;
}


#if __ENABLE_CONTROLLING_WINDOW__
static unsigned
count_Nd(void *printer, unsigned flags)
{
    return ((MODE(Context(printer)) & Mylene1f4_WINDOW_MASK)
	    != Mylene1f4_EXTENT_WINDOW ? 0 : 1);
}
#endif				/* __ENABLE_CONTROLLING_WINDOW__ */


static unsigned
count_O_(void *printer, unsigned flags)
{
    unsigned count;

    if (FFLAGS(printer) & GET_HEAD) {
	store_O_(printer);
    }

    if (FFLAGS(printer) & FIX_DECK) {
	count = 13;
    } else {
	count = count_digIts(flags, &DECK(printer));
    }

    return count;
}


static unsigned
count_P1(void *printer, unsigned flags)
{
    void *context;

    context = Context(printer);

    return !LPATH(context).data ? 0 : LPATH(context).size;
}


static unsigned
count_P_(void *printer, unsigned flags)
{
    return count_digits(flags, PLAY(Context(printer)));
}


static unsigned
count_R_(void *printer, unsigned flags)
{
    return strlen(Mylene1f4(Context(printer)));
}


static unsigned
count_S_(void *printer, unsigned flags)
{
    return stat_file(printer, GET_STAT, GOFF_MODE, so_S_);
}


#if __ENABLE_CONTROLLING_SERVER__
static unsigned
count_Sd(void *printer, unsigned flags)
{
    return ((MODE(Context(printer)) & Mylene1f4_WINDOW_MASK)
	    != Mylene1f4_SERVER_WINDOW ? 0 : 1);
}
#endif				/* __ENABLE_CONTROLLING_SERVER__ */


static unsigned
count_U6(void *printer, unsigned flags)
{
    int status;
    struct stat_type *stat_data;

    stat_data = K_STAT(printer);
    if (stat_data->passwd) {
	SCOPE(printer)->data[USER_SCAN] = stat_data->passwd;
	status = 1;
    } else {
	unsigned mode, size_max;
	void *miss;

	size_max = sysconf(_SC_GETPW_R_SIZE_MAX);

	mode = stat_data->mode;
	if (mode != none_MODE) {
	    mode -= COFF_MODE;
	    mode += USER_MODE;
	}
	if (fix_extension
	    (printer, mode, sizeof(struct group) + size_max, &miss)) {
	    status = 0;
	} else {
	    struct passwd *passwd;

	    passwd = (struct passwd *) miss;
	    status = getpwuid_r
		(stat_data->info.st_uid, passwd, (char *) (passwd + 1),
		 size_max, &passwd);
	    if (status) {
		status = 0;
	    } else {
		SCOPE(printer)->data[USER_SCAN] = passwd;
		stat_data->passwd = passwd;
		status = 1;
	    }
	}
    }

    return status;
}


static unsigned
count_X_(void *printer, unsigned flags)
{
    return Flags(printer) & Mylene1f4_LOCAL_LIST ? 1 : 0;
}


static unsigned
count_Y_(void *printer, unsigned flags)
{
    return strlen(VERSION);
}


#if NEED_COUNT__0
static unsigned
count__0(void *printer, unsigned flags)
{
    return 0;
}
#endif				/* NEED_COUNT__0 */


static unsigned
count__1(void *printer, unsigned flags)
{
    return 1;
}


static unsigned
count__2(void *printer, unsigned flags)
{
    return 2;
}


static unsigned
count__9(void *printer, unsigned flags)
{
    return 9;
}


static unsigned
count_a1(void *printer, unsigned flags)
{
    void *context;

    context = Context(printer);

    return !MATCH(context).data ? 0 : MATCH(context).size;
}


static unsigned
count_a3(void *printer, unsigned flags)
{
#if !__LINK_A52A_LOADER__
    return 0;
#else
    int status;
    struct sequel_type *sequel_data;

    status = Mylene1f4_find_sequel
	(Context(printer), Mylene1f4_XA52A1_SEQUEL, (void *) &sequel_data);
    if (status) {
	SCOPE(printer)->data[A52A_SCAN] = sequel_data->data;
    }

    return status;
#endif				/* !__LINK_A52A_LOADER__ */
}


static unsigned
count_a6(void *printer, unsigned flags)
{
    return set_time(printer, LESS_MODE, K_INFO(printer)->st_atime);
}


static unsigned
count_a_(void *printer, unsigned flags)
{
    unsigned frame;
    void *context;

    if (FFLAGS(printer) & GET_CACHE) {
	store_e_(printer);
    }

    context = Context(printer);

    Mylene1f4_line_reference(context, &frame);

    return count_digits(flags, frame - CCACHE(printer) / RELEASE(context));
}


#if __have_Mylene1f4_version__
static unsigned
count_ak(void *printer, unsigned flags)
{
    const char *version;

    Mylene1f4_version(&version);

    return strlen(version);
}
#endif				/* __have_Mylene1f4_version__ */


#if defined HAVE_ST_BLOCKS
static unsigned
count_b6(void *printer, unsigned flags)
{
    return count_digits(flags, K_INFO(printer)->st_blocks);
}
#endif				/* HAVE_ST_BLOCKS */


static unsigned
count_b_(void *printer, unsigned flags)
{
    unsigned length;

    if (flags & X1f4_FORMAT_ALTERNATE) {
	length = 0;
	x1f4_nprint_sstring
	    (&length, x1f4_basename(SOURCE(Context(printer))._.s1.path),
	     _Mylene1f4_c_type_s);
    } else {
	length = strlen(x1f4_basename(SOURCE(Context(printer))._.s1.path));
    }

    return length;
}


#if __ENABLE_MPEG_EQUALIZER__
static unsigned
count_bu(void *printer, unsigned flags)
{
    unsigned router;

    if (flags & X1f4_FORMAT_UNINDEXED) {
	router = 0;

	SCOPE(printer)->index___ = index___;
    } else {
	if (1) {
	    SCOPE(printer)->index___ = index_bu;
	}

	router = MPEG(Context(printer)).count;
    }

    return router;
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


static unsigned
count_c1(void *printer, unsigned flags)
{
    void *context;

    context = Context(printer);

    return !NOTES(context).data ? 0 : NOTES(context).size;
}


static unsigned
count_c6(void *printer, unsigned flags)
{
    return set_time(printer, NESS_MODE, K_INFO(printer)->st_ctime);
}


static unsigned
count_c_(void *printer, unsigned flags)
{
    if (FFLAGS(printer) & GET_INDEX) {
	store_c_(printer);
    }

    return count_digits(flags, INDEX(printer));
}


#if __ENABLE_MPEG_EQUALIZER__
static unsigned
count_cu(void *printer, unsigned flags)
{
    return count_digits(flags, MPEG(Context(printer)).count);
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


static unsigned
count_d5(void *printer, unsigned flags)
{
    return count_digits(flags, K_TIME(printer)->second_a / (3600 * 24));
}


static unsigned
count_d6(void *printer, unsigned flags)
{
    SCOPE(printer)->data[MDEV_SCAN] = &K_INFO(printer)->st_dev;

    return 1;
}


static unsigned
count_d_(void *printer, unsigned flags)
{
    const char *path;
    unsigned length;

    path = SOURCE(Context(printer))._.s1.path;

    length = strlen(path) - strlen(x1f4_basename(path));
    if (flags & X1f4_FORMAT_ALTERNATE) {
	unsigned Length = 0;

	x1f4_nprint_isstring(&Length, path, length, _Mylene1f4_c_type_s);
	length = Length;
    }

    return length;
}


static unsigned
count_do(void *printer, unsigned flags)
{
    return CLASS(Context(printer)) != Mylene1f4_LINK_OUTPUT ? 0 : 1;
}


static unsigned
count_dp(void *printer, unsigned flags)
{
    if (FFLAGS(printer) & GET_B1st) {
	B1st(printer) = STATE(Context(printer))[Mylene1f4_B1st_STATE];

	FFLAGS(printer) &= ~GET_B1st;
    }

    return 1;
}


static unsigned
count_e4(void *printer, unsigned flags)
{
    return count_digits(flags, K_MPEG(printer)->frame_rate_n);
}


static unsigned
count_e_(void *printer, unsigned flags)
{
    if (FFLAGS(printer) & GET_CACHE) {
	store_e_(printer);
    }

    return count_digits(flags, CCACHE(printer));
}


static unsigned
count_ei(void *printer, unsigned flags)
{
    return count_digits(flags, K_1f42(printer)->index);
}


static unsigned
count_em(void *printer, unsigned flags)
{
    return count_digits(flags, K_LPCM(printer)->frame_rate_n);
}


static unsigned
count_en(void *printer, unsigned flags)
{
#if !__LINK_A52A_LOADER__
    return 0;
#else
    return count_digits(flags, K_A52A(printer)->frame_rate_n);
#endif				/* !__LINK_A52A_LOADER__ */
}


#if __ENABLE_MPEG_EQUALIZER__
static unsigned
count_et(void *printer, unsigned flags)
{
    return !MPEG(Context(printer)).l3band_data ? 0 : 1;
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


static unsigned
count_f4(void *printer, unsigned flags)
{
    const char *framerate;
    unsigned count;

    framerate = K_MPEG(printer)->framerate;
    if (!framerate) {
	count = 0;
    } else {
	count = strlen(framerate);
    }

    return count;
}


static unsigned
count_f_(void *printer, unsigned flags)
{
    return count_digits(flags, REFERENCE(Context(printer)));
}


static unsigned
count_ff(void *printer, unsigned flags)
{
    return SELECT(Context(printer)) != Mylene1f4_PAGE_PICKUP ? 0 : 1;
}


#if __ENABLE_MPEG_EQUALIZER__
static unsigned
count_fv(void *printer, unsigned flags)
{
    return Mylene1f4_count_real(printer, flags, K_BAND(printer)->frequency);
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


static unsigned
count_g6(void *printer, unsigned flags)
{
    return count_digits(flags, K_INFO(printer)->st_gid);
}


static unsigned
count_h3(void *printer, unsigned flags)
{
    int status;
    struct sequel_type *sequel_data;

    status = Mylene1f4_find_sequel
	(Context(printer), Mylene1f4_x1f42e_SEQUEL, (void *) &sequel_data);
    if (status) {
	SCOPE(printer)->data[M66A_SCAN] = sequel_data->data;
    }

    return status;
}


static unsigned
count_i6(void *printer, unsigned flags)
{
    return count_digits(flags, K_INFO(printer)->st_ino);
}


static unsigned
count_i_(void *printer, unsigned flags)
{
    unsigned index;

    if (!(Flags(printer) & Mylene1f4_LOCAL_LIST)) {
	index = 0;
    } else {
	index = Excess(printer).e1.list87.index;
    }

    return count_digits(flags, index);
}


static unsigned
count_ig(void *printer, unsigned flags)
{
    return count_digits(flags, START2(Context(printer)));
}


#if __ENABLE_EQUEUE__
static unsigned
count_ik(void *printer, unsigned flags)
{
    const char *version;

    x1f4_aime_version(&version);

    return strlen(version);
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_MPEG_EQUALIZER__
static unsigned
count_iv(void *printer, unsigned flags)
{
    return count_digits
	(flags, K_BAND(printer) - MPEG(Context(printer)).l3band_data);
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


static unsigned
count_l0(void *printer, unsigned flags)
{
    unsigned reference = 1;

    if (!(FFLAGS(printer) & GET_LENGTH)) {
	struct fttime_type *fttime_data;
	struct ksscan_type *ksscan_data;
	void **data, *miss;

	net_extension(printer, LIME_MODE, &miss);
	ksscan_data = SCOPE(printer);
	data = ksscan_data->data;
	fttime_data = miss;
	data[TIME_SCAN] = fttime_data;
    } else {
	void *miss;

	if (fix_extension
	    (printer, LIME_MODE, sizeof(struct fttime_type), &miss)) {
	    reference = 0;
	} else {
	    struct fttime_type *fttime_data;
	    struct ksscan_type *ksscan_data;
	    void **data;

	    fttime_data = (struct fttime_type *) miss;

	    FFLAGS(printer) &= ~GET_LENGTH;
	    ksscan_data = SCOPE(printer);
	    data = ksscan_data->data;
	    data[TIME_SCAN] = fttime_data;

	    {
		double length_1, length_f, length_i;
		unsigned H, M, S;
		void *context;

		context = Context(printer);

		length_1 = x1f42(context)->frames;
		length_1 *= RELEASE(context);
		length_1 /= CHANNELS(context);
		length_1 /= SAMPLING(context);
		length_1 /= 2;

		length_f = modf(length_1, &length_i);

		fttime_data->second_a = length_i;
		fttime_data->second_f = length_f * 1000000;
		S = length_i;
		H = S / 3600 % 24;
		M = S / 60 % 60;
		S = S % 60;
		fttime_data->second_l = H << 020 | M << 010 | S;
	    }
	}
    }

    return reference;
}


static unsigned
count_l3(void *printer, unsigned flags)
{
    int status;
    struct sequel_type *sequel_data;

    status = Mylene1f4_find_sequel
	(Context(printer), Mylene1f4_XEPCM1_SEQUEL, (void *) &sequel_data);
    if (status) {
	SCOPE(printer)->data[LPCM_SCAN] = sequel_data->data;
    }

    return status;
}


static unsigned
count_l4(void *printer, unsigned flags)
{
    return 4 - K_MPEG(printer)->layer;
}


static unsigned
count_l_(void *printer, unsigned flags)
{
    return !(Flags(printer) & Mylene1f4_LOCAL_LIST) ? 0 : 1;
}


#if __ENABLE_MPEG_EQUALIZER__
static unsigned
count_lv(void *printer, unsigned flags)
{
    return Mylene1f4_count_real(printer, flags, K_BAND(printer)->level);
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if __ENABLE_MPEG_EQUALIZER__
static unsigned
count_lw(void *printer, unsigned flags)
{
    return Mylene1f4_count_real
	(printer, flags, MPEG(Context(printer)).amplification);
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


static unsigned
count_m3(void *printer, unsigned flags)
{
    int status;
    struct sequel_type *sequel_data;

    status = Mylene1f4_find_sequel
	(Context(printer), Mylene1f4_XMPEG1_SEQUEL, (void *) &sequel_data);
    if (status) {
	SCOPE(printer)->data[MPEG_SCAN] = sequel_data->data;
    }

    return status;
}


static unsigned
count_m4(void *printer, unsigned flags)
{
    const char *text;

    get_m4(printer, &text);

    return strlen(text);
}


static unsigned
count_m6(void *printer, unsigned flags)
{
    return set_time(printer, TESS_MODE, K_INFO(printer)->st_mtime);
}


static unsigned
count_m7(void *printer, unsigned flags)
{
    return count_digits(flags, minor(*K_MDEV(printer)));
}


static unsigned
count_n6(void *printer, unsigned flags)
{
    return count_digits(flags, K_INFO(printer)->st_nlink);
}


static unsigned
count_n9(void *printer, unsigned flags)
{
    return strlen(K_GROUP(printer)->gr_name);
}


static unsigned
count_na(void *printer, unsigned flags)
{
    return strlen(K_USER(printer)->pw_name);
}


static unsigned
count_nb(void *printer, unsigned flags)
{
    return LOAD_ORDER(Context(printer)) != Mylene1f4_LOAD_NEXT ? 0 : 1;
}


static unsigned
count_no(void *printer, unsigned flags)
{
    return CLASS(Context(printer)) != Mylene1f4_NULL_OUTPUT ? 0 : 1;
}


static unsigned
count_np(void *printer, unsigned flags)
{
    return strlen(Mirror(MIRROR(Context(printer)))->name);
}


static unsigned
count_odigits(unsigned cardinal)
{
    unsigned ing = 0;

    do {
	ing++;
    } while (cardinal >>= 3);

    return ing;
}


static unsigned
count_p2(void *printer, unsigned flags)
{
    return count_digits(flags, getpid());
}


static unsigned
count_pb(void *printer, unsigned flags)
{
    return LOAD_ORDER(Context(printer)) != Mylene1f4_LOAD_LAST ? 0 : 1;
}


static unsigned
count_pp(void *printer, unsigned flags)
{
    return strlen(Mirror(MIRROR(Context(printer)))->path);
}


static unsigned
count_pr(void *printer, unsigned flags)
{
    return B1st(printer) & Mylene1f4_B1st_PLAYING ? 1 : 0;
}


static unsigned
count_qp(void *printer, unsigned flags)
{
    pid_t pid;

    pid = PID(Context(printer));

    return pid == -1 ? 0 : count_digits(pid, flags);
}


static unsigned
count_r6(void *printer, unsigned flags)
{
    int status;
    mode_t mode;
    struct stat *info;

    info = K_INFO(printer);
    mode = info->st_mode;
    if (S_ISBLK(mode) || S_ISCHR(mode)) {
	SCOPE(printer)->data[MDEV_SCAN] = &info->st_rdev;
	status = 1;
    } else {
	status = 0;
    }

    return status;
}


static unsigned
count_r_(void *printer, unsigned flags)
{
    if (FFLAGS(printer) & GET_READ) {
	store_r_(printer);
    }

    return count_digits(flags, READ(printer));
}


static unsigned
count_s2(void *printer, unsigned flags)
{
    unsigned router;

    if (flags & X1f4_FORMAT_UNINDEXED) {
	router = scale_t_(printer);

#if !__DUMMY_RESET__
	SCOPE(printer)->index___ = index___;
#endif				/* !__DUMMY_RESET__ */
    } else {
	if (1) {
#if !__DUMMY_RESET__
	    SCOPE(printer)->index___ = index___;
#endif				/* !__DUMMY_RESET__ */
	}

	router = shift_s2(printer);
    }

    return router;
}


static unsigned
count_s5(void *printer, unsigned flags)
{
    return K_TIME(printer)->second_a / (3600 * 24) != 1 ? 0 : 1;
}


static unsigned
count_s6(void *printer, unsigned flags)
{
    return count_digits(flags, K_INFO(printer)->st_size);
}


static unsigned
count_s_(void *printer, unsigned flags)
{
    return count_digits(flags, SAMPLING(Context(printer)));
}


static unsigned
count_so(void *printer, unsigned flags)
{
    return CLASS(Context(printer)) != Mylene1f4_SKIP_OUTPUT ? 0 : 1;
}


static unsigned
count_string(void *printer, char *data, unsigned size)
{
    return size;
}


static unsigned
count_t0(void *printer, unsigned flags)
{
    unsigned router;

    if (flags & X1f4_FORMAT_UNINDEXED) {
	router = scale_t_(printer);

#if !__DUMMY_RESET__
	SCOPE(printer)->index___ = index___;
#endif				/* !__DUMMY_RESET__ */
    } else {
	if (1) {
#if !__DUMMY_RESET__
	    SCOPE(printer)->index___ = index___;
#endif				/* !__DUMMY_RESET__ */
	}

	router = shift_t0(printer);
    }

    return router;
}


static unsigned
count_t1(void *printer, unsigned flags)
{
    void *context;

    context = Context(printer);

    return !TITLE(context).data ? 0 : TITLE(context).size;
}


static unsigned
count_t2(void *printer, unsigned flags)
{
    int status;

    if (!(FFLAGS(printer) & GET_TIME)) {
	struct fttime_type *fttime_data;
	struct ksscan_type *ksscan_data;
	void **data, *miss;

	net_extension(printer, TIME_MODE, &miss);
	ksscan_data = SCOPE(printer);
	data = ksscan_data->data;
	fttime_data = miss;
	data[TIME_SCAN] = fttime_data;
	status = 1;
    } else {
	void *miss;

	status = fix_extension
	    (printer, TIME_MODE, sizeof(struct fttime_type), &miss);
	if (status) {
	    status = 0;
	} else {
	    struct fttime_type *fttime_data;
	    struct ksscan_type *ksscan_data;
	    struct timeval time;
	    void **data;

	    fttime_data = (struct fttime_type *) miss;

	    FFLAGS(printer) &= ~GET_TIME;
	    ksscan_data = SCOPE(printer);
	    data = ksscan_data->data;
	    data[TIME_SCAN] = fttime_data;
	    status = 1;

	    gettimeofday(&time, NULL);

	    fttime_data->time = SECOND_a(time);

	    fttime_data->second_f = SECOND_f(time);
	}
    }

    return status;
}


static unsigned
count_t_(void *printer, unsigned flags)
{
    unsigned router;

    if (flags & X1f4_FORMAT_UNINDEXED) {
	router = scale_t_(printer);

#if !__DUMMY_RESET__
	SCOPE(printer)->index___ = index___;
#endif				/* !__DUMMY_RESET__ */
    } else {
	if (1) {
#if !__DUMMY_RESET__
	    SCOPE(printer)->index___ = index___;
#endif				/* !__DUMMY_RESET__ */
	}

	router = shift_t_(printer);
    }

    return router;
}


static unsigned
count_tf(void *printer, unsigned flags)
{
    unsigned router;

    if (SELECT(Context(printer)) != Mylene1f4_TIME_PICKUP) {
	router = 0;
    } else if (flags & X1f4_FORMAT_UNINDEXED) {
	router = scale_t_(printer);

#if !__DUMMY_RESET__
	SCOPE(printer)->index___ = index___;
#endif				/* !__DUMMY_RESET__ */
    } else {
	if (1) {
#if !__DUMMY_RESET__
	    SCOPE(printer)->index___ = index___;
#endif				/* !__DUMMY_RESET__ */
	}

	router = shift_tf(printer);
    }

    return router;
}


static unsigned
count_u2(void *printer, unsigned flags)
{
    int status;

    if (!(FFLAGS(printer) & GET_UPTIME)) {
	struct fttime_type *fttime_data;
	struct ksscan_type *ksscan_data;
	void **data, *miss;

	net_extension(printer, TIME_mode, &miss);
	ksscan_data = SCOPE(printer);
	data = ksscan_data->data;
	fttime_data = miss;
	data[TIME_SCAN] = fttime_data;
	status = 1;
    } else {
	status = count_t2(printer, flags);
	if (status) {
	    struct ksscan_type *ksscan_data;
	    struct fttime_type *this_time;
	    void **data, *miss;

	    ksscan_data = SCOPE(printer);
	    data = ksscan_data->data;

	    this_time = data[TIME_SCAN];

	    status = fix_extension
		(printer, TIME_mode, sizeof(struct fttime_type), &miss);
	    if (status) {
		status = 0;
	    } else {
		struct fttime_type *fttime_data;
		unsigned H, M, S, U;
		void *context;

		fttime_data = (struct fttime_type *) miss;

		FFLAGS(printer) &= ~GET_UPTIME;
		data[TIME_SCAN] = fttime_data;
		status = 1;

		context = Context(printer);

		S = this_time->time - SECOND_a(BECCA(context));
		U = this_time->second_f;
		if (U < SECOND_f(BECCA(context))) {
		    U += 1000000;
		    S--;
		}
		U -= SECOND_f(BECCA(context));

		fttime_data->second_a = S;
		fttime_data->second_f = U;
		H = S / 3600 % 24;
		M = S / 60 % 60;
		S = S % 60;
		fttime_data->second_l = H << 020 | M << 010 | S;
	    }
	}
    }

    return status;
}


static unsigned
count_u6(void *printer, unsigned flags)
{
    return count_digits(flags, K_INFO(printer)->st_uid);
}


static unsigned
count_w_(void *printer, unsigned flags)
{
    return count_digits(flags, C1f4(Context(printer)));
}


static unsigned
count_x1(void *printer, unsigned flags)
{
#if __ENABLE_FILTER__
    unsigned count = 0;
    void *context, *e4deck;

    context = Context(printer);
    e4deck = E4DECK(context);
    if (!e4deck) {
    } else {
	Mylene1f4_count_filter(context, e4deck, &count);
    }

    return count;
#else
    return 0;
#endif				/* __ENABLE_FILTER__ */
}


static unsigned
count_x_(void *printer, unsigned flags)
{
    return !X1f42(Context(printer)) ? 0 : 1;
}


static unsigned
count_ym(void *printer, unsigned flags)
{
    void *sequel;

    return !Mylene1f4_find_sequel
	(Context(printer), Mylene1f4_XMPEG0_SEQUEL, &sequel) ? 0 : 1;
}


static unsigned
count_z_(void *printer, unsigned flags)
{
    unsigned count;

    if (FFLAGS(printer) & GET_TAIL) {
	store_z_(printer);
    }

    if (FFLAGS(printer) & FIX_FLOW) {
	count = 13;
    } else {
	count = count_digIts(flags, &FLOW(printer));
    }

    return count;
}


static unsigned
count_zb(void *printer, unsigned flags)
{
    return LOAD_ORDER(Context(printer)) != Mylene1f4_LOAD_RAND ? 0 : 1;
}


static unsigned
scale_t_(void *printer)
{
    x1f4_free_salist(SA_TIME(printer), K_TIME(printer));

    return 0;
}


static unsigned
set_time(void *printer, unsigned mode, time_t time)
{
    struct fttime_type *fttime_data;
    struct stat_type *stat_data;
    unsigned reference = 1;

    stat_data = K_STAT(printer);
    fttime_data = stat_data->class;
    if (!fttime_data) {
	int status;
	unsigned Mode;
	void *miss;

	Mode = stat_data->mode;
	if (Mode != none_MODE) {
	    mode += Mode - COFF_MODE;
	    status = fix_extension
		(printer, mode, sizeof(struct fttime_type), &miss);
	} else {
	    mode = none_MODE;
	    status = set_extension
		(printer, mode, sizeof(struct fttime_type), &miss);
	}
	if (status) {
	    reference = 0;
	} else {
	    fttime_data = (struct fttime_type *) miss;

	    stat_data->class = fttime_data;
	}
    }
    if (fttime_data) {
	SCOPE(printer)->data[TIME_SCAN] = fttime_data;

	fttime_data->time = time;
    }

    return reference;
}


static unsigned
shift_s2(void *printer)
{
    unsigned reference = 1;
    void *sa_time;

    sa_time = SA_TIME(printer);
    if (!sa_time) {
	reference = 0;
    } else {
	void *miss;

	if (x1f4_link_salist(sa_time, &miss)) {
	    reference = 0;
	} else {
	    struct fttime_type *fttime_data;
	    void *context;

	    fttime_data = (struct fttime_type *) miss;

	    SCOPE(printer)->data[TIME_SCAN] = fttime_data;

	    context = Context(printer);

	    fttime_data->time = SECOND_a(BECCA(context));

	    fttime_data->second_f = SECOND_f(BECCA(context));
	}
    }

    return reference;
}


static unsigned
shift_t0(void *printer)
{
    unsigned reference;

    reference = count_l0(printer, 0);
    if (reference) {
	unsigned second_a, second_f;
	struct fttime_type *fttime_data;
	void **data;

	data = SCOPE(printer)->data;
	fttime_data = data[TIME_SCAN];
	second_a = fttime_data->second_a;
	second_f = fttime_data->second_f;
	reference = shift_t_(printer);
	if (reference) {
	    unsigned H, M, S, second_A, second_F;

	    fttime_data = data[TIME_SCAN];
	    second_A = fttime_data->second_a;
	    second_F = fttime_data->second_f;
	    if (second_f < second_F) {
		second_a -= 1;
		second_f += 1000000;
	    }
	    S = second_a - second_A;
	    fttime_data->second_a = second_a - second_A;
	    fttime_data->second_f = second_f - second_F;
	    H = S / 3600 % 24;
	    M = S / 60 % 60;
	    S = S % 60;
	    fttime_data->second_l = H << 020 | M << 010 | S;
	}
    }

    return reference;
}


static unsigned
shift_t_(void *printer)
{
    unsigned reference = 1;
    void *sa_time;

    sa_time = SA_TIME(printer);
    if (!sa_time) {
	reference = 0;
    } else {
	void *miss;

	if (x1f4_link_salist(sa_time, &miss)) {
	    reference = 0;
	} else {
	    struct fttime_type *fttime_data;
	    struct timeval *atime;
	    unsigned H, M, S;
	    void *context;

	    fttime_data = (struct fttime_type *) miss;

	    SCOPE(printer)->data[TIME_SCAN] = fttime_data;

	    context = Context(printer);

	    atime = ATIME(context);
	    S = SECOND_A(atime);
	    fttime_data->second_a = S;
	    fttime_data->second_f = SECOND_F(atime);
	    H = S / 3600 % 24;
	    M = S / 60 % 60;
	    S = S % 60;
	    fttime_data->second_l = H << 020 | M << 010 | S;
	}
    }

    return reference;
}


static unsigned
shift_tf(void *printer)
{
    unsigned reference = 1;
    void *sa_time;

    sa_time = SA_TIME(printer);
    if (!sa_time) {
	reference = 0;
    } else {
	void *miss;

	if (x1f4_link_salist(sa_time, &miss)) {
	    reference = 0;
	} else {
	    struct extime_type *extime_data;
	    struct fttime_type *fttime_data;
	    unsigned H, M, S, X;
	    void *context;

	    fttime_data = (struct fttime_type *) miss;

	    SCOPE(printer)->data[TIME_SCAN] = fttime_data;

	    context = Context(printer);

	    extime_data = &START6(context);
	    X = extime_data->x;
	    S = extime_data->h * 3600 + extime_data->m * 60 + extime_data->s
		+ X / 60;
	    fttime_data->second_a = S;
	    fttime_data->second_f = (X % 60 * 1000000 + 30) / 60;
	    H = S / 3600 % 24;
	    M = S / 60 % 60;
	    S = S % 60;
	    fttime_data->second_l = H << 020 | M << 010 | S;
	}
    }

    return reference;
}


static unsigned
so_L_(void *printer, struct stat_type *stat_data)
{
    unsigned count;

    if (lstat(SOURCE(Context(printer))._.s1.path, &stat_data->info)) {
	count = 0;
    } else {
	count = 1;
	init_stat(stat_data);
    }

    return count;
}


static unsigned
so_S_(void *printer, struct stat_type *stat_data)
{
    unsigned count;

    if (stat(SOURCE(Context(printer))._.s1.path, &stat_data->info)) {
	count = 0;
    } else {
	count = 1;
	init_stat(stat_data);
    }

    return count;
}


static unsigned
stat_file(void *printer, unsigned state, unsigned index,
	  unsigned (*stat) (void *, struct stat_type *))
{
    int status;

    if (!(FFLAGS(printer) & state)) {
	struct ksscan_type *ksscan_data;
	struct stat_type *stat_data;
	void **data, *miss;

	net_extension(printer, index, &miss);
	ksscan_data = SCOPE(printer);
	data = ksscan_data->data;
	stat_data = miss;
	data[INFO_SCAN] = &stat_data->info;
	data[STAT_SCAN] = stat_data;
	status = 1;
    } else {
	void *miss;

	status =
	    fix_extension(printer, index, sizeof(struct stat_type), &miss);
	if (status) {
	    status = 0;
	} else {
	    struct stat_type *stat_data;

	    stat_data = (struct stat_type *) miss;
	    if (!stat(printer, stat_data)) {
		status = 0;
	    } else {
		struct ksscan_type *ksscan_data;
		void **data;

		FFLAGS(printer) &= ~state;
		ksscan_data = SCOPE(printer);
		stat_data->mode = index;
		data = ksscan_data->data;
		data[INFO_SCAN] = &stat_data->info;
		data[STAT_SCAN] = stat_data;
		status = 1;
	    }
	}
    }

    return status;
}


static void
fini_ksscan(struct ksscan_type *ksscan_data)
{
    struct atom_type *atom_data;

    atom_data = ksscan_data->atom_data;
    while (atom_data) {
	struct atom_type *atom_link;

	atom_link = atom_data;
	atom_data = atom_data->atom_data;
	free(atom_link);
    }
}


static void
fini_printer(struct printer_state *printer_data)
{
    struct miss_type *last_miss = NULL, *miss_data;

    miss_data = printer_data->extension;
    while (miss_data) {
	if (miss_data->code != none_MODE) {
	    last_miss = miss_data;
	    miss_data = miss_data->miss_data;
	} else {
	    struct miss_type *miss_link;

	    miss_link = miss_data;
	    miss_data = miss_data->miss_data;
	    free(miss_link);
	    if (last_miss) {
		last_miss->miss_data = miss_data;
	    } else {
		printer_data->extension = miss_data;
	    }
	}
    }
}


static void
get_m4(void *printer, const char **text)
{
    const char *data;
    unsigned mode;

    mode = K_MPEG(printer)->mode;
    switch (mode) {
    case Mylene1f4_dual_channel:
	data = "dual_channel";
	break;
    case Mylene1f4_joint_stereo:
	data = "joint_stereo";
	break;
    case Mylene1f4_single_channel:
	data = "single_channel";
	break;
    default:
	data = "stereo";
    }

    *text = data;
}


static void
get_scope(void *printer, unsigned scope)
{
    if (((struct printer_state *) printer)->scope_failures) {
	((struct printer_state *) printer)->scope_failures--;
    } else {
	struct ksscan_type **ksscan, *ksscan_data, *previous_ksscan;

	ksscan_data = SCOPE(printer);

	previous_ksscan = ksscan_data->previous_ksscan;

	fini_ksscan(ksscan_data);
	free(ksscan_data);

	ksscan = (struct ksscan_type **)
	    (void *) &((struct printer_state *) (printer))->scope;
	*ksscan = previous_ksscan;
    }
}


#if !__DUMMY_INDEX__
static void
index___(void *printer, unsigned index)
{
}
#endif				/* !__DUMMY_INDEX__ */


#if __ENABLE_MPEG_EQUALIZER__
static void
index_bu(void *printer, unsigned index)
{
    SCOPE(printer)->data[BAND_SCAN] =
	MPEG(Context(printer)).l3band_data + index;
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


static void
init_stat(struct stat_type *stat_data)
{
    stat_data->class = NULL;
    stat_data->group = NULL;
    stat_data->passwd = NULL;
}


static void
print_A1(void *printer, unsigned flags)
{
    void *context;
    const void *data;

    context = Context(printer);
    data = ALBUM(context).data;
    if (data) {
	RUN_SSTRING(printer) (printer, data, ALBUM(context).size);
    }
}


static void
print_B1(void *printer, unsigned flags)
{
    void *context;
    const void *data;

    context = Context(printer);
    data = LBASE(context).data;
    if (data) {
	RUN_SSTRING(printer) (printer, data, LBASE(context).size);
    }
}


static void
print_B4(void *printer, unsigned flags)
{
    unsigned count;

    count = K_MPEG(printer)->bitrate;
    if (!~count) {
	RUN_SSTRING(printer) (printer, "?", 1);
    } else {
	print_cardinal(printer, flags, count);
    }
}


#if defined HAVE_ST_BLKSIZE
static void
print_B6(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_INFO(printer)->st_blksize);
}
#endif				/* HAVE_ST_BLKSIZE */


static void
print_B_(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, CLIP(Context(printer)));
}


static void
print_C_(void *printer, unsigned flags)
{
    RUN_SSTRING(printer) (printer, "Mylene1f4", 9);
}


static void
print_E4(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_MPEG(printer)->frame_rate_d);
}


static void
print_E_(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, RELEASE(Context(printer)));
}


static void
print_Ei(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_1f42(printer)->count);
}


static void
print_Em(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_LPCM(printer)->frame_rate_d);
}


static void
print_En(void *printer, unsigned flags)
{
#if __LINK_A52A_LOADER__
    print_cardinal(printer, flags, K_A52A(printer)->frame_rate_d);
#endif				/* __LINK_A52A_LOADER__ */
}


static void
print_F0(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, x1f42(Context(printer))->frames);
}


static void
print_L4(void *printer, unsigned flags)
{
    char c;

    c = 0x30 + 4 - K_MPEG(printer)->layer;
    RUN_SSTRING(printer) (printer, &c, 1);
}


static void
print_M4(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_MPEG(printer)->mode_extension);
}


static void
print_M6(void *printer, unsigned flags)
{
    print_ocardinal(printer, K_INFO(printer)->st_mode);
}


static void
print_M7(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, major(*K_MDEV(printer)));
}


static void
print_N_(void *printer, unsigned flags)
{
    if (flags & X1f4_FORMAT_ALTERNATE) {
	x1f4_vprint_sstring
	    (printer, RUN_SSTRING(printer), SOURCE(Context(printer))._.s1.path,
	     _Mylene1f4_c_type_s);
    } else {
	RUN_ZSTRING(printer)
	    (printer, (char *) SOURCE(Context(printer))._.s1.path);
    }
}


static void
print_O_(void *printer, unsigned flags)
{
    if (FFLAGS(printer) & GET_HEAD) {
	store_O_(printer);
    }

    if (!(FFLAGS(printer) & FIX_DECK)) {
	print_cardInal(printer, flags, &DECK(printer));
    } else {
	RUN_SSTRING(printer) (printer, "not available", 13);
    }
}


static void
print_P1(void *printer, unsigned flags)
{
    void *context;
    const void *data;

    context = Context(printer);
    data = LPATH(context).data;
    if (data) {
	RUN_SSTRING(printer) (printer, data, LPATH(context).size);
    }
}


static void
print_P_(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, PLAY(Context(printer)));
}


static void
print_R_(void *printer, unsigned flags)
{
    RUN_ZSTRING(printer) (printer, (char *) Mylene1f4(Context(printer)));
}


static void
print_S4(void *printer, unsigned flags)
{
    RUN_SSTRING(printer)
	(printer, (K_MPEG(printer)->specification
		   ? "ISO/IEC 11172-3" : "ISO/IEC 13818-3"), 15);
}


static void
print_Sj(void *printer, unsigned flags)
{
    RUN_SSTRING(printer)
	(printer, (K_NSSP(printer)->specification == 11172
		   ? "ISO/IEC 11172-1" : "ISO/IEC 13818-1"), 15);
}


static void
print_Y_(void *printer, unsigned flags)
{
    RUN_ZSTRING(printer) (printer, VERSION);
}


#if NEED_PRINT__0
static void
print__0(void *printer, unsigned flags)
{
    RUN_SSTRING(printer)(printer, "0", 1);
}
#endif				/* NEED_PRINT__0 */


static void
print_a1(void *printer, unsigned flags)
{
    void *context;
    const void *data;

    context = Context(printer);
    data = MATCH(context).data;
    if (data) {
	RUN_SSTRING(printer) (printer, data, MATCH(context).size);
    }
}


static void
print_a_(void *printer, unsigned flags)
{
    unsigned frame;
    void *context;

    if (FFLAGS(printer) & GET_CACHE) {
	store_e_(printer);
    }

    context = Context(printer);

    Mylene1f4_line_reference(context, &frame);

    print_cardinal
	(printer, flags, frame - CCACHE(printer) / RELEASE(context));
}


#if __have_Mylene1f4_version__
static void
print_ak(void *printer, unsigned flags)
{
    const char *version;

    Mylene1f4_version(&version);

    RUN_ZSTRING(printer) (printer, version);
}
#endif				/* __have_Mylene1f4_version__ */


#if defined HAVE_ST_BLOCKS
static void
print_b6(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_INFO(printer)->st_blocks);
}
#endif				/* HAVE_ST_BLOCKS */


static void
print_b_(void *printer, unsigned flags)
{
    if (flags & X1f4_FORMAT_ALTERNATE) {
	x1f4_vprint_sstring
	    (printer, RUN_SSTRING(printer),
	     x1f4_basename(SOURCE(Context(printer))._.s1.path),
	     _Mylene1f4_c_type_s);
    } else {
	RUN_ZSTRING(printer)
	    (printer, x1f4_basename(SOURCE(Context(printer))._.s1.path));
    }
}


static void
print_c1(void *printer, unsigned flags)
{
    void *context;
    const void *data;

    context = Context(printer);
    data = NOTES(context).data;
    if (data) {
	RUN_SSTRING(printer) (printer, data, NOTES(context).size);
    }
}


static void
print_c_(void *printer, unsigned flags)
{
    if (FFLAGS(printer) & GET_INDEX) {
	store_c_(printer);
    }

    print_cardinal(printer, flags, INDEX(printer));
}


#if __ENABLE_MPEG_EQUALIZER__
static void
print_cu(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, MPEG(Context(printer)).count);
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


static void
print_d5(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_TIME(printer)->second_a / (3600 * 24));
}


static void
print_d_(void *printer, unsigned flags)
{
    const char *path;

    path = SOURCE(Context(printer))._.s1.path;
    if (flags & X1f4_FORMAT_ALTERNATE) {
	x1f4_vprint_isstring
	    (printer, RUN_SSTRING(printer), path, x1f4_basename(path) - path,
	     _Mylene1f4_c_type_s);
    } else {
	RUN_SSTRING(printer)
	    (printer, (char *) path, x1f4_basename(path) - path);
    }
}


static void
print_e4(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_MPEG(printer)->frame_rate_n);
}


static void
print_e_(void *printer, unsigned flags)
{
    if (FFLAGS(printer) & GET_CACHE) {
	store_e_(printer);
    }

    print_cardinal(printer, flags, CCACHE(printer));
}


static void
print_ei(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_1f42(printer)->index);
}


static void
print_em(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_LPCM(printer)->frame_rate_n);
}


static void
print_en(void *printer, unsigned flags)
{
#if __LINK_A52A_LOADER__
    print_cardinal(printer, flags, K_A52A(printer)->frame_rate_n);
#endif				/* __LINK_A52A_LOADER__ */
}


static void
print_f4(void *printer, unsigned flags)
{
    const char *framerate;

    framerate = K_MPEG(printer)->framerate;
    if (framerate) {
	RUN_ZSTRING(printer) (printer, framerate);
    }
}


static void
print_f_(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, REFERENCE(Context(printer)));
}


#if __ENABLE_MPEG_EQUALIZER__
static void
print_fv(void *printer, unsigned flags)
{
    Mylene1f4_print_real(printer, flags, K_BAND(printer)->frequency);
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


static void
print_g6(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_INFO(printer)->st_gid);
}


static void
print_i6(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_INFO(printer)->st_ino);
}


static void
print_i_(void *printer, unsigned flags)
{
    unsigned index;

    if (!(Flags(printer) & Mylene1f4_LOCAL_LIST)) {
	index = 0;
    } else {
	index = Excess(printer).e1.list87.index;
    }

    print_cardinal(printer, flags, index);
}


static void
print_ig(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, START2(Context(printer)));
}


#if __ENABLE_EQUEUE__
static void
print_ik(void *printer, unsigned flags)
{
    const char *version;

    x1f4_aime_version(&version);

    RUN_ZSTRING(printer) (printer, version);
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_MPEG_EQUALIZER__
static void
print_iv(void *printer, unsigned flags)
{
    print_cardinal
	(printer, flags, K_BAND(printer) - MPEG(Context(printer)).l3band_data);
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


static void
print_l4(void *printer, unsigned flags)
{
    RUN_SSTRING(printer) (printer, "III", 4 - K_MPEG(printer)->layer);
}


#if __ENABLE_MPEG_EQUALIZER__
static void
print_lv(void *printer, unsigned flags)
{
    Mylene1f4_print_real(printer, flags, K_BAND(printer)->level);
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if __ENABLE_MPEG_EQUALIZER__
static void
print_lw(void *printer, unsigned flags)
{
    Mylene1f4_print_real(printer, flags, MPEG(Context(printer)).amplification);
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


static void
print_m4(void *printer, unsigned flags)
{
    const char *text;

    get_m4(printer, &text);
    RUN_ZSTRING(printer) (printer, (char *) text);
}


static void
print_m7(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, minor(*K_MDEV(printer)));
}


static void
print_n6(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_INFO(printer)->st_nlink);
}


static void
print_n9(void *printer, unsigned flags)
{
    RUN_ZSTRING(printer)(printer, K_GROUP(printer)->gr_name);
}


static void
print_na(void *printer, unsigned flags)
{
    RUN_ZSTRING(printer)(printer, K_USER(printer)->pw_name);
}


static void
print_np(void *printer, unsigned flags)
{
    RUN_ZSTRING(printer)(printer, Mirror(MIRROR(Context(printer)))->name);
}


static void
print_ocardinal(void *printer, unsigned cardinal)
{
    char buffer[32], *output;

    output = (char *) buffer + 32;
    do {
	*--output = digits[cardinal & 7];
    } while ((cardinal >>= 3));

    ((struct printer_state *) (printer))->run_sstring
	(printer, output, buffer + 32 - output);
}


static void
print_p2(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, getpid());
}


static void
print_pp(void *printer, unsigned flags)
{
    RUN_ZSTRING(printer)(printer, Mirror(MIRROR(Context(printer)))->path);
}


static void
print_qp(void *printer, unsigned flags)
{
    pid_t pid;

    pid = PID(Context(printer));
    if (pid == -1) {
    } else {
	print_cardinal(printer, flags, pid);
    }
}


static void
print_r_(void *printer, unsigned flags)
{
    if (FFLAGS(printer) & GET_READ) {
	store_r_(printer);
    }

    print_cardinal(printer, flags, READ(printer));
}


static void
print_s6(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_INFO(printer)->st_size);
}


static void
print_s_(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, SAMPLING(Context(printer)));
}


static void
print_t1(void *printer, unsigned flags)
{
    void *context;
    const void *data;

    context = Context(printer);
    data = TITLE(context).data;
    if (data) {
	RUN_SSTRING(printer) (printer, data, TITLE(context).size);
    }
}


static void
print_tq(void *printer, unsigned flags)
{
    char buffer[2], *string;

    string = buffer + 2;
    x1f4_xprint_fcardinal(&string, 0, 2, 16, K_LPCM(printer)->audio_id);

    RUN_SSTRING(printer) (printer, buffer, 2);
}


static void
print_u6(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_INFO(printer)->st_uid);
}


#if NEED_PRINT_VERSION
static void
print_version(void *printer, unsigned flags)
{
    RUN_SSTRING(printer) (printer, "not available", 13);
}
#endif				/* NEED_PRINT_VERSION */


static void
print_w_(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, C1f4(Context(printer)));
}


static void
print_x1(void *printer, unsigned flags)
{
#if __ENABLE_FILTER__
    void *context, *e4deck;

    context = Context(printer);
    e4deck = E4DECK(context);
    if (e4deck) {
	Mylene1f4_print_filter(context, e4deck, printer, RUN_SSTRING(printer));
    }
#endif				/* __ENABLE_FILTER__ */
}


static void
print_z_(void *printer, unsigned flags)
{
    if (FFLAGS(printer) & GET_TAIL) {
	store_z_(printer);
    }

    if (!(FFLAGS(printer) & FIX_FLOW)) {
	print_cardInal(printer, flags, &FLOW(printer));
    } else {
	RUN_SSTRING(printer) (printer, "not available", 13);
    }
}


static void
put_space(void *printer, unsigned size, char *space, unsigned space_length)
{
    static const char default_space[] = "                ";
    unsigned transfer;

    if (!space || !space_length) {
	space = (char *) default_space;
	space_length = 16;
    }

    for (; size; size -= transfer) {
	if (size < space_length) {
	    transfer = size;
	} else {
	    transfer = space_length;
	}
	RUN_SSTRING(printer) (printer, space, transfer);
    }
}


static void
put_string(void *printer, char *data, unsigned size)
{
    RUN_SSTRING(printer) (printer, data, size);
}


static void
set_index(void *printer, unsigned index)
{
#if !__DUMMY_INDEX__
    SCOPE(printer)->index___(printer, index);
#endif				/* !__DUMMY_INDEX__ */
}


static void
set_scope(void *printer, unsigned scope)
{
    if (((struct printer_state *) printer)->scope_failures) {
	((struct printer_state *) printer)->scope_failures++;
    } else {
	struct ksscan_type *ksscan_data;

	ksscan_data = (struct ksscan_type *)
	    malloc(sizeof(struct ksscan_type));
	if (!ksscan_data) {
	    ((struct printer_state *) printer)->scope_failures++;
	} else {
	    struct ksscan_type **ksscan, *parent_ksscan, *previous_ksscan;

	    previous_ksscan = SCOPE(printer);

	    parent_ksscan = previous_ksscan;
	    while (scope < parent_ksscan->scope) {
		parent_ksscan = parent_ksscan->parent_ksscan;
	    }

	    ksscan_data->atom_data = NULL;
	    ksscan_data->parent_ksscan = parent_ksscan;
	    ksscan_data->previous_ksscan = previous_ksscan;
	    ksscan_data->scope = scope;

#if !__DUMMY_INDEX__
	    ksscan_data->index___ = index___;
#endif				/* !__DUMMY_INDEX__ */

	    memcpy(ksscan_data->data, parent_ksscan->data,
		   sizeof(void *) * KSCANS_STRIDE);

	    ksscan = (struct ksscan_type **)
		(void *) &((struct printer_state *) (printer))->scope;
	    *ksscan = ksscan_data;
	}
    }
}


static void
store_O_(void *printer)
{
    int (*deCk) (void *, void *);
    void *context;

    context = Context(printer);

    deCk = TARGET(context).deck;
    if (deCk) {
	LCARD_SET_CARD1(DECK(printer), 0);
	deCk(STORAGE(context), &DECK(printer));

	FFLAGS(printer) &= ~FIX_DECK;
    }

    FFLAGS(printer) &= ~GET_HEAD;
}


static void
store_c_(void *printer)
{
    Mylene1f4_fix_index(Context(printer), &INDEX(printer));

    FFLAGS(printer) &= ~GET_INDEX;
}


static void
store_e_(void *printer)
{
    unsigned cache;
    void *context;

    context = Context(printer);

    if (LINK(context)) {
	cache = 0;
    } else {
	cache = C1f4(context) - STATE(context)[Mylene1f4_REAL_STATE];
	if (SIZE(context) < cache) {
	    cache += SIZE(context);
	}
    }

    CCACHE(printer) = cache;

    FFLAGS(printer) &= ~GET_CACHE;
}


static void
store_r_(void *printer)
{
    READ(printer) = STATE(Context(printer))[Mylene1f4_REAL_STATE];

    FFLAGS(printer) &= ~GET_READ;
}


static void
store_z_(void *printer)
{
    int (*flOw) (void *, void *);
    void *context;

    context = Context(printer);

    flOw = TARGET(context).flow;
    if (flOw) {
	LCARD_SET_CARD1(FLOW(printer), 0);
	flOw(STORAGE(context), &FLOW(printer));

	FFLAGS(printer) &= ~FIX_FLOW;
    }

    FFLAGS(printer) &= ~GET_TAIL;
}


int
Mylene1f4_aime_format(void *context, const char *text,
		      const char *specification, void **c_format)
{
    int status;

    status = Mylene1f4_init_format(context, specification, c_format);
    if (status) {
	Mylene1f4_fprintf_error
	    (context, "%s: cannot parse %s specification: `%s'\n",
	     Mylene1f4(context), text, specification);
    }

    return status;
}


int
Mylene1f4_clear_format(void *context)
{
    struct printer_state *printer_data;

    printer_data = PRINTER(context);

    {
	void *sa_time;

	sa_time = printer_data->sa_time;
	if (sa_time) {
	    x1f4_fini_salist(&sa_time);
	    printer_data->sa_time = NULL;
	}
    }

    {
	struct miss_type *miss_data;

	miss_data = printer_data->extension;
	if (miss_data) {
	    while (miss_data) {
		struct miss_type *miss_link;

		miss_link = miss_data;
		miss_data = miss_data->miss_data;
		free(miss_link);
	    }

	    printer_data->extension = NULL;
	}
    }

    return 0;
}


int
Mylene1f4_dump_format(int mode)
{
    if (mode == 032) {
	x1f4_lprint_ffields(stdout, run_fields);
    } else if (mode == 033) {
	x1f4_pprint_ffields(stdout, run_fields);
    } else {
	x1f4_tprint_ffields(stdout, run_fields);
    }

    return 0;
}


int
Mylene1f4_fini_format(void *context, void **c_format)
{
    x1f4_free_format(c_format);

    *c_format = NULL;

    return 0;
}


int
Mylene1f4_init_format(void *context, const char *specification,
		      void **c_format)
{
    int status;

    status = x1f4_parse_format
	(c_format, run_fields, &f6link, (char *) specification);

    return status;
}


int
Mylene1f4_run_format(void *context, void *format)
{
    FILE *progress;
    int status;
    struct printer_state *printer_data;

    printer_data = PRINTER(context);

    progress = PROGRESS(context);

    printer_data->data.file = progress;
    printer_data->run_sstring = print_sstring;
    printer_data->run_zstring = print_zstring;

    status = link_format(format, printer_data);

    fflush(progress);

    return status;
}


int
Mylene1f4_set_format(void *context, void *format, void *buffer)
{
    int status;
    struct printer_state *printer_data;

    printer_data = PRINTER(context);

    printer_data->data.buffer_data = buffer;
    printer_data->run_sstring = store_sstring;
    printer_data->run_zstring = store_zstring;

    status = link_format(format, printer_data);

    return status;
}


int
Mylene1f4_setup_format(void *context)
{
    struct printer_state *printer_data;

    printer_data = PRINTER(context);

    printer_data->context = context;

    printer_data->fflags = ~0;

    printer_data->extension = NULL;

    if (x1f4_init_salist(&printer_data->sa_time, sizeof(struct fttime_type))) {
	printer_data->sa_time = NULL;
    }

    return 0;
}
