/*
 * formatg.c
 * Copyright (C) 2005-2010, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if defined HAVE_LIBLEWD
# include <lewd.h>
#endif				/* HAVE_LIBLEWD */

#include <access.h>
#include <defs.h>
#include <inter.h>
#if !defined HAVE_LIBLEWD
# include <printf.h>
# include <sstring.n-i.h>
# include <sstring.v-i.h>
#endif				/* !HAVE_LIBLEWD */

#define RUN_SSTRING(printer) \
    (((struct printer_state *) (printer))->run_sstring)

#define Context(printer) \
    (((struct printer_state *) (printer))->context)

extern const unsigned _Mylene1f4_c_type_s[];

static unsigned count_A0(void *, unsigned);
static unsigned count_A_(void *, unsigned);
static unsigned count__1(void *, unsigned);
static unsigned count_a0(void *, unsigned);
static unsigned count_a_(void *, unsigned);
static unsigned count_c0(void *, unsigned);
static unsigned count_c_(void *, unsigned);
static unsigned count_fIEld(void *, unsigned);
static unsigned count_fIeld(void *, unsigned, unsigned);
static unsigned count_fiEld(void *, unsigned, unsigned);
static unsigned count_field(void *, unsigned, unsigned, unsigned);
static unsigned count_g0(void *, unsigned);
static unsigned count_g_(void *, unsigned);
static unsigned count_t0(void *, unsigned);
static unsigned count_t_(void *, unsigned);
static unsigned count_y0(void *, unsigned);
static unsigned count_y_(void *, unsigned);

static void print_A_(void *, unsigned);
static void print_a_(void *, unsigned);
static void print_c_(void *, unsigned);
static void print_fIeld(void *, unsigned, unsigned);
static void print_field(void *, unsigned, unsigned, unsigned);
static void print_g_(void *, unsigned);
static void print_t_(void *, unsigned);
static void print_y_(void *, unsigned);

static const struct x1f4_efield_type tcz_fields[] = {
/* *INDENT-OFF* */
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tc0_fields[] = {
/* *INDENT-OFF* */
    {	"album",
	tcz_fields,	count_A0,	0,
	0101,		NULL					},
    {	"artist",
	tcz_fields,	count_a0,	0,
	0141,		NULL					},
    {	"comment",
	tcz_fields,	count_c0,	0,
	0143,		NULL					},
    {	"genre",
	tcz_fields,	count_g0,	0,
	0147,		NULL					},
    {	"title",
	tcz_fields,	count_t0,	0,
	0164,		NULL					},
    {	"year",
	tcz_fields,	count_y0,	0,
	0171,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
};

const struct x1f4_efield_type _Mylene1f4_formatg_0[] = {
/* *INDENT-OFF* */
    {	"album",
	NULL,		count_A_,	0,
	0101,		print_A_				},
    {	"artist",
	NULL,		count_a_,	0,
	0141,		print_a_				},
    {	"comment",
	NULL,		count_c_,	0,
	0143,		print_c_				},
    {	"genre",
	NULL,		count_g_,	0,
	0147,		print_g_				},
    {	"title",
	NULL,		count_t_,	0,
	0164,		print_t_				},
    {	"year",
	NULL,		count_y_,	0,
	0171,		print_y_				},
    {	"non zero length",
	tc0_fields,	count__1,	0,
	0172,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
};

static unsigned
count_A0(void *printer, unsigned flags)
{
    unsigned count;

    if (ID3T(Context(printer)) == Mylene1f4_MERGED_ID3TAG) {
	count = count_fiEld(printer, 63, 30);
    } else {
	count = count_fIEld(printer, Mylene1f4_ALBUM_FIELD);
    }

    return !count ? 0 : 1;
}


static unsigned
count_A_(void *printer, unsigned flags)
{
    unsigned count;

    if (ID3T(Context(printer)) == Mylene1f4_MERGED_ID3TAG) {
	count = count_field(printer, flags, 63, 30);
    } else {
	count = count_fIeld(printer, flags, Mylene1f4_ALBUM_FIELD);
    }

    return count;
}


static unsigned
count__1(void *printer, unsigned flags)
{
    return 1;
}


static unsigned
count_a0(void *printer, unsigned flags)
{
    unsigned count;

    if (ID3T(Context(printer)) == Mylene1f4_MERGED_ID3TAG) {
	count = count_fiEld(printer, 33, 30);
    } else {
	count = count_fIEld(printer, Mylene1f4_MATCH_FIELD);
    }

    return !count ? 0 : 1;
}


static unsigned
count_a_(void *printer, unsigned flags)
{
    unsigned count;

    if (ID3T(Context(printer)) == Mylene1f4_MERGED_ID3TAG) {
	count = count_field(printer, flags, 33, 30);
    } else {
	count = count_fIeld(printer, flags, Mylene1f4_MATCH_FIELD);
    }

    return count;
}


static unsigned
count_c0(void *printer, unsigned flags)
{
    unsigned count;

    if (ID3T(Context(printer)) == Mylene1f4_MERGED_ID3TAG) {
	count = count_fiEld(printer, 97, 30);
    } else {
	count = count_fIEld(printer, Mylene1f4_NOTES_FIELD);
    }

    return !count ? 0 : 1;
}


static unsigned
count_c_(void *printer, unsigned flags)
{
    unsigned count;

    if (ID3T(Context(printer)) == Mylene1f4_MERGED_ID3TAG) {
	count = count_field(printer, flags, 97, 30);
    } else {
	count = count_fIeld(printer, flags, Mylene1f4_NOTES_FIELD);
    }

    return count;
}


static unsigned
count_fIEld(void *printer, unsigned offset)
{
    return ((struct id3tag_type *) ID3X(Context(printer)))->size[offset];
}


static unsigned
count_fIeld(void *printer, unsigned flags, unsigned offset)
{
    struct id3tag_type *id3tag;
    unsigned length;

    id3tag = ID3X(Context(printer));

    length = id3tag->size[offset];
    if (flags & X1f4_FORMAT_ALTERNATE) {
	unsigned Length = 0;

	x1f4_nprint_isstring
	    (&Length, id3tag->data[offset], length, _Mylene1f4_c_type_s);
	length = Length;
    }

    return length;
}


static unsigned
count_fiEld(void *printer, unsigned offset, unsigned length)
{
    const char *string;

    string = ID3X(Context(printer));
    string += offset;
    Mylene1f4_shrink_field(&string, &length);

    return length;
}


static unsigned
count_field(void *printer, unsigned flags, unsigned offset, unsigned length)
{
    const char *string;

    string = ID3X(Context(printer));
    string += offset;
    Mylene1f4_shrink_field(&string, &length);
    if (flags & X1f4_FORMAT_ALTERNATE) {
	unsigned Length = 0;

	x1f4_nprint_isstring(&Length, string, length, _Mylene1f4_c_type_s);
	length = Length;
    }

    return length;
}


static unsigned
count_g0(void *printer, unsigned flags)
{
    return 0;
}


static unsigned
count_g_(void *printer, unsigned flags)
{
    return 0;
}


static unsigned
count_t0(void *printer, unsigned flags)
{
    unsigned count;

    if (ID3T(Context(printer)) == Mylene1f4_MERGED_ID3TAG) {
	count = count_fiEld(printer, 3, 30);
    } else {
	count = count_fIEld(printer, Mylene1f4_TITLE_FIELD);
    }

    return !count ? 0 : 1;
}


static unsigned
count_t_(void *printer, unsigned flags)
{
    unsigned count;

    if (ID3T(Context(printer)) == Mylene1f4_MERGED_ID3TAG) {
	count = count_field(printer, flags, 3, 30);
    } else {
	count = count_fIeld(printer, flags, Mylene1f4_TITLE_FIELD);
    }

    return count;
}


static unsigned
count_y0(void *printer, unsigned flags)
{
    unsigned count;

    if (ID3T(Context(printer)) == Mylene1f4_MERGED_ID3TAG) {
	count = count_fiEld(printer, 93, 4);
    } else {
	count = count_fIEld(printer, Mylene1f4_SHORT_FIELD);
    }

    return !count ? 0 : 1;
}


static unsigned
count_y_(void *printer, unsigned flags)
{
    unsigned count;

    if (ID3T(Context(printer)) == Mylene1f4_MERGED_ID3TAG) {
	count = count_field(printer, flags, 93, 4);
    } else {
	count = count_fIeld(printer, flags, Mylene1f4_SHORT_FIELD);
    }

    return count;
}


static void
print_A_(void *printer, unsigned flags)
{
    if (ID3T(Context(printer)) == Mylene1f4_MERGED_ID3TAG) {
	print_field(printer, flags, 63, 30);
    } else {
	print_fIeld(printer, flags, Mylene1f4_ALBUM_FIELD);
    }
}


static void
print_a_(void *printer, unsigned flags)
{
    if (ID3T(Context(printer)) == Mylene1f4_MERGED_ID3TAG) {
	print_field(printer, flags, 33, 30);
    } else {
	print_fIeld(printer, flags, Mylene1f4_MATCH_FIELD);
    }
}


static void
print_c_(void *printer, unsigned flags)
{
    if (ID3T(Context(printer)) == Mylene1f4_MERGED_ID3TAG) {
	print_field(printer, flags, 97, 30);
    } else {
	print_fIeld(printer, flags, Mylene1f4_NOTES_FIELD);
    }
}


static void
print_fIeld(void *printer, unsigned flags, unsigned offset)
{
    struct id3tag_type *id3tag;

    id3tag = ID3X(Context(printer));

    if (flags & X1f4_FORMAT_ALTERNATE) {
	x1f4_vprint_isstring
	    (printer, RUN_SSTRING(printer), id3tag->data[offset],
	     id3tag->size[offset], _Mylene1f4_c_type_s);
    } else {
	RUN_SSTRING(printer)
	    (printer, id3tag->data[offset], id3tag->size[offset]);
    }
}


static void
print_field(void *printer, unsigned flags, unsigned offset, unsigned length)
{
    const char *string;

    string = ID3X(Context(printer));
    string += offset;
    Mylene1f4_shrink_field(&string, &length);

    if (flags & X1f4_FORMAT_ALTERNATE) {
	x1f4_vprint_isstring
	    (printer, RUN_SSTRING(printer), string, length,
	     _Mylene1f4_c_type_s);
    } else {
	RUN_SSTRING(printer) (printer, (char *) string, length);
    }
}


static void
print_g_(void *printer, unsigned flags)
{
}


static void
print_t_(void *printer, unsigned flags)
{
    if (ID3T(Context(printer)) == Mylene1f4_MERGED_ID3TAG) {
	print_field(printer, flags, 3, 30);
    } else {
	print_fIeld(printer, flags, Mylene1f4_TITLE_FIELD);
    }
}


static void
print_y_(void *printer, unsigned flags)
{
    if (ID3T(Context(printer)) == Mylene1f4_MERGED_ID3TAG) {
	print_field(printer, flags, 93, 4);
    } else {
	print_fIeld(printer, flags, Mylene1f4_SHORT_FIELD);
    }
}
