/*
 * formatl.c
 * Copyright (C) 2005-2010, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if defined HAVE_LIBLEWD
# include <lewd.h>
#endif				/* HAVE_LIBLEWD */
#include <stdlib.h>
#include <string.h>

#include <access.h>
#if !defined HAVE_LIBLEWD
# include <basename.h>
#endif				/* !HAVE_LIBLEWD */
#include <defs.h>
#include <format.h>
#include <fslist.h>
#include <inter.h>
#include <list.h>
#if !defined HAVE_LIBLEWD
# include <printf.h>
# include <sstring.n-i.h>
# include <sstring.n.h>
# include <sstring.v-i.h>
# include <sstring.v.h>
#endif				/* !HAVE_LIBLEWD */

#define false(e)			0
#define true(e)				1

#define count_digits			Mylene1f4_count_digits

#define print_cardinal			Mylene1f4_print_cardinal

#define RUN_SSTRING(printer) \
    (((struct printer_state *) (printer))->run_sstring)
#define RUN_ZSTRING(printer) \
    (((struct printer_state *) (printer))->run_zstring)

#define Context(printer) \
    (((struct printer_state *) (printer))->context)

#define Excess(printer) \
    EXCESS(Context(printer))
#define Flags(printer) \
    FLAGS(Context(printer))

#define SCOPE(printer) \
    ((struct ksscan_type *) ((struct printer_state *) (printer))->scope)

#define TYPE_K(printer, index, type) \
    ((type *) SCOPE((printer))->data[(index)])

#define K_RAIL(printer) \
    TYPE_K(printer, RAIL_SCAN, struct rail_type)

typedef struct rail_type {
#if __ENABLE_TAGGED_LIST__
    struct id3tag_type id3tag;
#endif				/* __ENABLE_TAGGED_LIST__ */
    unsigned last, size;
    void *base, *node, *peek, *post;
} rail_type;

extern const unsigned _Mylene1f4_c_type_s[];

static unsigned count_A2(void *, unsigned);
static unsigned count_A3(void *, unsigned);
static unsigned count_N0(void *, unsigned);
static unsigned count__1(void *, unsigned);
static unsigned count_a2(void *, unsigned);
static unsigned count_a3(void *, unsigned);
static unsigned count_b0(void *, unsigned);
static unsigned count_c0(void *, unsigned);
static unsigned count_c2(void *, unsigned);
static unsigned count_c3(void *, unsigned);
static unsigned count_d0(void *, unsigned);
static unsigned count_fIEld(void *, unsigned);
static unsigned count_fIeld(void *, unsigned, unsigned);
static unsigned count_f_(void *, unsigned);
static unsigned count_g2(void *, unsigned);
static unsigned count_g3(void *, unsigned);
static unsigned count_i0(void *, unsigned);
static unsigned count_l_(void *, unsigned);
static unsigned count_s_(void *, unsigned);
static unsigned count_t0(void *, unsigned);
static unsigned count_t2(void *, unsigned);
static unsigned count_t3(void *, unsigned);
static unsigned count_t_(void *, unsigned);
static unsigned count_y2(void *, unsigned);
static unsigned count_y3(void *, unsigned);

static void index___(void *, unsigned);
static void index_f_(void *, unsigned);
static void print_A2(void *, unsigned);
static void print_N0(void *, unsigned);
static void print_a2(void *, unsigned);
static void print_b0(void *, unsigned);
static void print_c0(void *, unsigned);
static void print_c2(void *, unsigned);
static void print_d0(void *, unsigned);
static void print_g2(void *, unsigned);
static void print_fIeld(void *, unsigned, unsigned);
static void print_i0(void *, unsigned);
static void print_l_(void *, unsigned);
static void print_t2(void *, unsigned);
static void print_y2(void *, unsigned);

static const struct x1f4_efield_type tcz_fields[] = {
/* *INDENT-OFF* */
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tc3_fields[] = {
/* *INDENT-OFF* */
    {	"album",
	tcz_fields,	count_A3,	0,
	0101,		NULL					},
    {	"artist",
	tcz_fields,	count_a3,	0,
	0141,		NULL					},
    {	"comment",
	tcz_fields,	count_c3,	0,
	0143,		NULL					},
    {	"genre",
	tcz_fields,	count_g3,	0,
	0147,		NULL					},
    {	"title",
	tcz_fields,	count_t3,	0,
	0164,		NULL					},
    {	"year",
	tcz_fields,	count_y3,	0,
	0171,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tc2_fields[] = {
/* *INDENT-OFF* */
    {	"album",
	NULL,		count_A2,	0,
	0101,		print_A2				},
    {	"artist",
	NULL,		count_a2,	0,
	0141,		print_a2				},
    {	"comment",
	NULL,		count_c2,	0,
	0143,		print_c2				},
    {	"genre",
	NULL,		count_g2,	0,
	0147,		print_g2				},
    {	"title",
	NULL,		count_t2,	0,
	0164,		print_t2				},
    {	"year",
	NULL,		count_y2,	0,
	0171,		print_y2				},
    {	"non zero length",
	tc3_fields,	count__1,	0,
	0172,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, tc0_fields[] = {
/* *INDENT-OFF* */
    {	"path",
	NULL,		count_N0,	0,
	0116,		print_N0				},
    {	"basename",
	NULL,		count_b0,	0,
	0142,		print_b0				},
    {	"current file index",
	NULL,		count_c0,	0,
	0143,		print_c0				},
    {	"dirname",
	NULL,		count_d0,	0,
	0144,		print_d0				},
    {	"initial file index",
	NULL,		count_i0,	0,
	0151,		print_i0				},
    {	"tags",
	tc2_fields,	count_t0,	0,
	0164,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
};

const struct x1f4_efield_type _Mylene1f4_formatl_0[] = {
/* *INDENT-OFF* */
    {	"file",
	tc0_fields,	count_f_,	X1f4_FORMAT_ITERATE,
	0146,		NULL					},
    {	"length",
	NULL,		count_l_,	0,
	0154,		print_l_				},
    {	"\"shuffled\"",
	tcz_fields,	count_s_,	0,
	0163,		NULL					},
    {	"\"tagged\"",
	tcz_fields,	count_t_,	0,
	0164,		NULL					},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
};

static unsigned
count_A2(void *printer, unsigned flags)
{
    return count_fIeld(printer, flags, Mylene1f4_ALBUM_FIELD);
}


static unsigned
count_A3(void *printer, unsigned flags)
{
    return count_fIEld(printer, Mylene1f4_ALBUM_FIELD);
}


static unsigned
count_N0(void *printer, unsigned flags)
{
    struct rail_type *rail_data;
    unsigned length;
    void *base;

    rail_data = K_RAIL(printer);

    base = rail_data->base;

    length = rail_data->size;
    if (!~length) {
	length = strlen(base);
	rail_data->size = length;
    }

    if (flags & X1f4_FORMAT_ALTERNATE) {
	unsigned class;

	class = length;
	length = 0;
	x1f4_nprint_isstring(&length, base, class, _Mylene1f4_c_type_s);
	x1f4_line_fslist(rail_data->node, &length, Mylene1f4_class_string);
    } else {
	x1f4_file_fslist(rail_data->node, &length);
    }

    return length;
}


static unsigned
count__1(void *printer, unsigned flags)
{
    return 1;
}


static unsigned
count_a2(void *printer, unsigned flags)
{
    return count_fIeld(printer, flags, Mylene1f4_MATCH_FIELD);
}


static unsigned
count_a3(void *printer, unsigned flags)
{
    return count_fIEld(printer, Mylene1f4_MATCH_FIELD);
}


static unsigned
count_b0(void *printer, unsigned flags)
{
    unsigned length;

    if (flags & X1f4_FORMAT_ALTERNATE) {
	length = 0;
	x1f4_nprint_sstring
	    (&length, K_RAIL(printer)->base, _Mylene1f4_c_type_s);
    } else {
	struct rail_type *rail_data;

	rail_data = K_RAIL(printer);

	length = rail_data->size;
	if (!~length) {
	    length = strlen(rail_data->base);
	    rail_data->size = length;
	}
    }

    return length;
}


static unsigned
count_c0(void *printer, unsigned flags)
{
    return count_digits(flags, K_RAIL(printer)->last);
}


static unsigned
count_c2(void *printer, unsigned flags)
{
    return count_fIeld(printer, flags, Mylene1f4_NOTES_FIELD);
}


static unsigned
count_c3(void *printer, unsigned flags)
{
    return count_fIEld(printer, Mylene1f4_NOTES_FIELD);
}


static unsigned
count_d0(void *printer, unsigned flags)
{
    unsigned length = 0;

    if (flags & X1f4_FORMAT_ALTERNATE) {
	x1f4_line_fslist
	    (K_RAIL(printer)->node, &length, Mylene1f4_class_string);
    } else {
	x1f4_file_fslist(K_RAIL(printer)->node, &length);
    }

    return length;
}


static unsigned
count_fIEld(void *printer, unsigned offset)
{
    return !K_RAIL(printer)->id3tag.size[offset] ? 0 : 1;
}


static unsigned
count_fIeld(void *printer, unsigned flags, unsigned offset)
{
    struct id3tag_type *id3tag;
    unsigned length;

    id3tag = &K_RAIL(printer)->id3tag;

    length = id3tag->size[offset];
    if (flags & X1f4_FORMAT_ALTERNATE) {
	unsigned Length = 0;

	x1f4_nprint_isstring
	    (&Length, id3tag->data[offset], length, _Mylene1f4_c_type_s);
	length = Length;
    }

    return length;
}


static unsigned
count_f_(void *printer, unsigned flags)
{
    unsigned router;

    if (flags & X1f4_FORMAT_UNINDEXED) {
	struct rail_type *rail_data;

	router = 0;

	rail_data = SCOPE(printer)->data[RAIL_SCAN];
	if (rail_data) {
	    free(rail_data);
	}

	SCOPE(printer)->index___ = index___;
    } else {
	struct rail_type *rail_data;

	rail_data = (struct rail_type *) malloc(sizeof(struct rail_type));
	if (!rail_data) {
	    router = 0;

	    SCOPE(printer)->data[RAIL_SCAN] = NULL;
	} else {
	    SCOPE(printer)->data[RAIL_SCAN] = rail_data;

	    rail_data->last = (1 << 30) | ((1 << 30) - 1);

	    SCOPE(printer)->index___ = index_f_;

	    router = Excess(printer).e1.list87.count;
	}
    }

    return router;
}


static unsigned
count_g2(void *printer, unsigned flags)
{
    return 0;
}


static unsigned
count_g3(void *printer, unsigned flags)
{
    return 0;
}


static unsigned
count_i0(void *printer, unsigned flags)
{
    return count_digits
	(flags, ((struct list21_type *) K_RAIL(printer)->peek)->index);
}


static unsigned
count_l_(void *printer, unsigned flags)
{
    return count_digits(flags, Excess(printer).e1.list87.count);
}


static unsigned
count_s_(void *printer, unsigned flags)
{
    return !(FLAGS(Context(printer)) & Mylene1f4_TILED_LIST) ? 0 : 1;
}


static unsigned
count_t0(void *printer, unsigned flags)
{
#if !__ENABLE_TAGGED_LIST__
    return 0;
#else
    unsigned count;

    if (!(FLAGS(Context(printer)) & Mylene1f4_BREAK_LIST)) {
	count = 0;
    } else {
	count = 1;
	Mylene1f4_wind_list(&K_RAIL(printer)->id3tag, K_RAIL(printer)->peek);
    }

    return count;
#endif				/* __ENABLE_TAGGED_LIST__ */
}


static unsigned
count_t2(void *printer, unsigned flags)
{
    return count_fIeld(printer, flags, Mylene1f4_TITLE_FIELD);
}


static unsigned
count_t3(void *printer, unsigned flags)
{
    return count_fIEld(printer, Mylene1f4_TITLE_FIELD);
}


static unsigned
count_t_(void *printer, unsigned flags)
{
#if !__ENABLE_TAGGED_LIST__
    return 0;
#else
    return !(FLAGS(Context(printer)) & Mylene1f4_BREAK_LIST) ? 0 : 1;
#endif				/* __ENABLE_TAGGED_LIST__ */
}


static unsigned
count_y2(void *printer, unsigned flags)
{
    return count_fIeld(printer, flags, Mylene1f4_SHORT_FIELD);
}


static unsigned
count_y3(void *printer, unsigned flags)
{
    return count_fIEld(printer, Mylene1f4_SHORT_FIELD);
}


static void
index___(void *printer, unsigned index)
{
}


static void
index_f_(void *printer, unsigned index)
{
    struct rail_type *rail_data;
    unsigned last;

    rail_data = K_RAIL(printer);

    last = rail_data->last;
    if (last < index) {
	rail_data->last = index;
	if (index - last == 1) {
	    rail_data->size = ~0;

	    x1f4_next_fslist
		(Excess(printer).e1.list81.window, &rail_data->peek,
		 &rail_data->post, &rail_data->node, &rail_data->base,
		 rail_data->peek);
	} else {
	    rail_data->size = ~0;

	    x1f4_peek_fslist
		(Excess(printer).e1.list81.window, index, &rail_data->peek,
		 &rail_data->post, &rail_data->node, &rail_data->base);
	}
    } else {
	if (index < last) {
	    rail_data->last = index;
	    if (last - index == 1) {
		rail_data->size = ~0;

		x1f4_last_fslist
		    (Excess(printer).e1.list81.window, &rail_data->peek,
		     &rail_data->post, &rail_data->node, &rail_data->base,
		     rail_data->peek);
	    } else {
		rail_data->size = ~0;

		x1f4_peek_fslist
		    (Excess(printer).e1.list81.window, index, &rail_data->peek,
		     &rail_data->post, &rail_data->node, &rail_data->base);
	    }
	}
    }
}


static void
print_A2(void *printer, unsigned flags)
{
    print_fIeld(printer, flags, Mylene1f4_ALBUM_FIELD);
}


static void
print_N0(void *printer, unsigned flags)
{
    struct rail_type *rail_data;
    unsigned length;
    void *base;

    rail_data = K_RAIL(printer);

    base = rail_data->base;

    length = rail_data->size;
    if (!~length) {
	length = strlen(base);
	rail_data->size = length;
    }

    if (flags & X1f4_FORMAT_ALTERNATE) {
	x1f4_line_fslist(rail_data->node, printer, Mylene1f4_shell_string);
	x1f4_vprint_isstring
	    (printer, RUN_SSTRING(printer), base, length, _Mylene1f4_c_type_s);
    } else {
	x1f4_line_fslist(rail_data->node, printer, RUN_SSTRING(printer));
	RUN_SSTRING(printer) (printer, base, length);
    }
}


static void
print_a2(void *printer, unsigned flags)
{
    print_fIeld(printer, flags, Mylene1f4_MATCH_FIELD);
}


static void
print_b0(void *printer, unsigned flags)
{
    if (flags & X1f4_FORMAT_ALTERNATE) {
	x1f4_vprint_sstring
	    (printer, RUN_SSTRING(printer), K_RAIL(printer)->base,
	     _Mylene1f4_c_type_s);
    } else {
	struct rail_type *rail_data;
	unsigned length;

	rail_data = K_RAIL(printer);

	length = rail_data->size;
	if (!~length) {
	    length = strlen(rail_data->base);
	    rail_data->size = length;
	}

	RUN_SSTRING(printer) (printer, rail_data->base, length);
    }
}


static void
print_c0(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_RAIL(printer)->last);
}


static void
print_c2(void *printer, unsigned flags)
{
    print_fIeld(printer, flags, Mylene1f4_NOTES_FIELD);
}


static void
print_d0(void *printer, unsigned flags)
{
    if (flags & X1f4_FORMAT_ALTERNATE) {
	x1f4_line_fslist
	    (K_RAIL(printer)->node, printer, Mylene1f4_shell_string);
    } else {
	x1f4_line_fslist(K_RAIL(printer)->node, printer, RUN_SSTRING(printer));
    }
}


static void
print_fIeld(void *printer, unsigned flags, unsigned offset)
{
    struct id3tag_type *id3tag;

    id3tag = &K_RAIL(printer)->id3tag;

    if (flags & X1f4_FORMAT_ALTERNATE) {
	x1f4_vprint_isstring
	    (printer, RUN_SSTRING(printer), id3tag->data[offset],
	     id3tag->size[offset], _Mylene1f4_c_type_s);
    } else {
	RUN_SSTRING(printer)
	    (printer, id3tag->data[offset], id3tag->size[offset]);
    }
}


static void
print_g2(void *printer, unsigned flags)
{
}


static void
print_i0(void *printer, unsigned flags)
{
    print_cardinal(printer, flags,
		   ((struct list21_type *) K_RAIL(printer)->peek)->index);
}


static void
print_l_(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, Excess(printer).e1.list87.count);
}


static void
print_t2(void *printer, unsigned flags)
{
    print_fIeld(printer, flags, Mylene1f4_TITLE_FIELD);
}


static void
print_y2(void *printer, unsigned flags)
{
    print_fIeld(printer, flags, Mylene1f4_SHORT_FIELD);
}
