/*
 * formatt.c
 * Copyright (C) 2005-2010, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if defined HAVE_LIBLEWD
# include <lewd.h>
#endif				/* HAVE_LIBLEWD */
#include <time.h>

#include <access.h>
#if !defined HAVE_LIBLEWD
# include <cardinal.p-f.h>
#endif				/* !HAVE_LIBLEWD */
#include <format.h>
#include <inter.h>
#if !defined HAVE_LIBLEWD
# include <printf.h>
#endif				/* !HAVE_LIBLEWD */

#define __LOCALTIME_PARANOID__		0

#if __LOCALTIME_PARANOID__
# define MAX2(e)			((e) % 100)
# define MAX4(e)			((e) % 10000)
#else
# define MAX2(e)			(e)
# define MAX4(e)			(e)
#endif				/* __LOCALTIME_PARANOID__ */

#define count_digits			Mylene1f4_count_digits

#define print_cardinal			Mylene1f4_print_cardinal

#define scope_t_e0			_Mylene1f4_formatt_0

#define RUN_SSTRING(printer) \
    (((struct printer_state *) (printer))->run_sstring)

#define SCOPE(printer) \
    ((struct ksscan_type *) ((struct printer_state *) (printer))->scope)

#define TYPE_K(printer, index, type) \
    ((type *) SCOPE((printer))->data[(index)])

#define K_TIME(printer) \
    TYPE_K(printer, TIME_SCAN, struct fttime_type)

static unsigned count_10(void *, unsigned);
static unsigned count_11(void *, unsigned);
static unsigned count_24(void *, unsigned);
static unsigned count_Fl(void *, unsigned);
static unsigned count_Hl(void *, unsigned);
static unsigned count_Ml(void *, unsigned);
static unsigned count_Sl(void *, unsigned);
static unsigned count_T5(void *, unsigned);
static unsigned count_T8(void *, unsigned);
static unsigned count_Yc(void *, unsigned);
static unsigned count__1(void *, unsigned);
static unsigned count__8(void *, unsigned);
static unsigned count_dc(void *, unsigned);
static unsigned count_fl(void *, unsigned);
static unsigned count_hl(void *, unsigned);
static unsigned count_mc(void *, unsigned);
static unsigned count_ml(void *, unsigned);
static unsigned count_sl(void *, unsigned);

static void print_D8(void *, unsigned);
static void print_Fl(void *, unsigned);
static void print_Hl(void *, unsigned);
static void print_Ml(void *, unsigned);
static void print_Sl(void *, unsigned);
static void print_Yc(void *, unsigned);
static void print_d8(void *, unsigned);
static void print_dc(void *, unsigned);
static void print_e8(void *, unsigned);
static void print_fl(void *, unsigned);
static void print_hl(void *, unsigned);
static void print_mc(void *, unsigned);
static void print_ml(void *, unsigned);
static void print_sl(void *, unsigned);
static void print_t8(void *, unsigned);
static void print_tl(void *, unsigned);
static void print_xl(void *, unsigned);
static void print_yl(void *, unsigned);
static void run_time(void *, char);

static const struct x1f4_efield_type tcc_fields[] = {
/* *INDENT-OFF* */
    {	"year",
	NULL,		count_Yc,	0,
	0131,		print_Yc				},
    {	"day",
	NULL,		count_dc,	0,
	0144,		print_dc				},
    {	"month",
	NULL,		count_mc,	0,
	0155,		print_mc				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
};

const struct x1f4_efield_type _Mylene1f4_formatt_0[] = {
/* *INDENT-OFF* */
    {	"frame of second",
	NULL,		count_Fl,	0,
	0106,		print_Fl				},
    {	"hour of day",
	NULL,		count_Hl,	0,
	0110,		print_Hl				},
    {	"minute of hour",
	NULL,		count_Ml,	0,
	0115,		print_Ml				},
    {	"second of minute",
	NULL,		count_Sl,	0,
	0123,		print_Sl				},
    {	"frame",
	NULL,		count_fl,	0,
	0146,		print_fl				},
    {	"hour",
	NULL,		count_hl,	0,
	0150,		print_hl				},
    {	"minute",
	NULL,		count_ml,	0,
	0155,		print_ml				},
    {	"second",
	NULL,		count_sl,	0,
	0163,		print_sl				},
    {	"time expressed as HH:MM:SS",
	NULL,		count__8,	0,
	0164,		print_tl				},
    {	"time expressed as HH:MM:SS:FF",
	NULL,		count_11,	0,
	0170,		print_xl				},
    {	"time expressed as HH:MM:SS,FF",
	NULL,		count_11,	0,
	0171,		print_yl				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, _Mylene1f4_formatt_1[] = {
/* *INDENT-OFF* */
    {	"date",
	tcc_fields,	count__1,	0,
	0101,		NULL					},
    {	"date by ctime(3)",
	NULL,		count_24,	0,
	0104,		print_D8				},
    {	"time of day",
	scope_t_e0,	count_T8,	0,
	0124,		NULL					},
    {	"date expressed as YYYYMMDD",
	NULL,		count__8,	0,
	0144,		print_d8				},
    {	"date expressed as YYYY-MM-DD",
	NULL,		count_10,	0,
	0145,		print_e8				},
    {	"time expressed as HH:MM:SS",
	NULL,		count__8,	0,
	0164,		print_t8				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
}, _Mylene1f4_formatt_2[] = {
/* *INDENT-OFF* */
    {	"date",
	tcc_fields,	count__1,	0,
	0101,		NULL					},
    {	"date by ctime(3)",
	NULL,		count_24,	0,
	0104,		print_D8				},
    {	"time of day",
	scope_t_e0,	count_T5,	0,
	0124,		NULL					},
    {	"date expressed as YYYYMMDD",
	NULL,		count__8,	0,
	0144,		print_d8				},
    {	"date expressed as YYYY-MM-DD",
	NULL,		count_10,	0,
	0145,		print_e8				},
    {	"time expressed as HH:MM:SS",
	NULL,		count__8,	0,
	0164,		print_t8				},
    {	NULL,
	NULL,		NULL,		0,
	0000,		NULL					}
/* *INDENT-ON* */
};

static unsigned
count_10(void *printer, unsigned flags)
{
    return 012;
}


static unsigned
count_11(void *printer, unsigned flags)
{
    return 013;
}


static unsigned
count_24(void *printer, unsigned flags)
{
    return 030;
}


static unsigned
count_Fl(void *printer, unsigned flags)
{
    return count_digits(flags, K_TIME(printer)->second_f * 60 / 1000000);
}


static unsigned
count_Hl(void *printer, unsigned flags)
{
    return count_digits(flags, K_TIME(printer)->second_l >> 020);
}


static unsigned
count_Ml(void *printer, unsigned flags)
{
    return count_digits(flags, K_TIME(printer)->second_l >> 010 & 255);
}


static unsigned
count_Sl(void *printer, unsigned flags)
{
    return count_digits(flags, K_TIME(printer)->second_l & 255);
}


static unsigned
count_T5(void *printer, unsigned flags)
{
    struct fttime_type *fttime_data;
    struct tm *down_time;
    time_t time;

    fttime_data = K_TIME(printer);
    time = fttime_data->time;
    down_time = localtime(&time);
    fttime_data->second_a = time;
    fttime_data->second_l = MAX2(down_time->tm_hour) << 020
	| MAX2(down_time->tm_min) << 010 | MAX2(down_time->tm_sec);

    return 1;
}


static unsigned
count_T8(void *printer, unsigned flags)
{
    struct fttime_type *fttime_data;
    struct tm *down_time;
    time_t time;

    fttime_data = K_TIME(printer);
    time = fttime_data->time;
    down_time = localtime(&time);
    fttime_data->second_a = time;
    fttime_data->second_f = 0;
    fttime_data->second_l = MAX2(down_time->tm_hour) << 020
	| MAX2(down_time->tm_min) << 010 | MAX2(down_time->tm_sec);

    return 1;
}


static unsigned
count_Yc(void *printer, unsigned flags)
{
    return count_digits
	(flags, localtime(&K_TIME(printer)->time)->tm_year + 1900);
}


static unsigned
count__1(void *printer, unsigned flags)
{
    return 001;
}


static unsigned
count__8(void *printer, unsigned flags)
{
    return 010;
}


static unsigned
count_dc(void *printer, unsigned flags)
{
    return count_digits(flags, localtime(&K_TIME(printer)->time)->tm_mday);
}


static unsigned
count_fl(void *printer, unsigned flags)
{
    double f = 0;

    f += K_TIME(printer)->second_f;
    f /= 1000000;
    f += K_TIME(printer)->second_a;
    f *= 60;

    return count_digits(flags, f);
}


static unsigned
count_hl(void *printer, unsigned flags)
{
    return count_digits(flags, K_TIME(printer)->second_a / 3600);
}


static unsigned
count_mc(void *printer, unsigned flags)
{
    return count_digits(flags, localtime(&K_TIME(printer)->time)->tm_mon + 1);
}


static unsigned
count_ml(void *printer, unsigned flags)
{
    return count_digits(flags, K_TIME(printer)->second_a / 60);
}


static unsigned
count_sl(void *printer, unsigned flags)
{
    return count_digits(flags, K_TIME(printer)->second_a);
}


static void
print_D8(void *printer, unsigned flags)
{
    RUN_SSTRING(printer)(printer, ctime(&K_TIME(printer)->time), 24);
}


static void
print_Fl(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_TIME(printer)->second_f * 60 / 1000000);
}


static void
print_Hl(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_TIME(printer)->second_l >> 020);
}


static void
print_Ml(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_TIME(printer)->second_l >> 010 & 255);
}


static void
print_Sl(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_TIME(printer)->second_l & 255);
}


static void
print_Yc(void *printer, unsigned flags)
{
    print_cardinal
	(printer, flags, localtime(&K_TIME(printer)->time)->tm_year + 1900);
}


static void
print_d8(void *printer, unsigned flags)
{
    char buffer[8], *string;
    struct tm *down_time;

    down_time = localtime(&K_TIME(printer)->time);
    string = buffer + 8;
    x1f4_xprint_fcardinal(&string, 0, 2, 10, MAX2(down_time->tm_mday));
    x1f4_xprint_fcardinal(&string, 0, 2, 10, MAX2(down_time->tm_mon + 1));
    x1f4_xprint_fcardinal(&string, 0, 4, 10, MAX4(down_time->tm_year + 1900));

    RUN_SSTRING(printer) (printer, buffer, 8);
}


static void
print_dc(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, localtime(&K_TIME(printer)->time)->tm_mday);
}


static void
print_e8(void *printer, unsigned flags)
{
    char buffer[10], *string;
    struct tm *down_time;

    down_time = localtime(&K_TIME(printer)->time);
    string = buffer + 10;
    x1f4_xprint_fcardinal(&string, 0, 2, 10, MAX2(down_time->tm_mday));
    *--string = '-';
    x1f4_xprint_fcardinal(&string, 0, 2, 10, MAX2(down_time->tm_mon + 1));
    *--string = '-';
    x1f4_xprint_fcardinal(&string, 0, 4, 10, MAX4(down_time->tm_year + 1900));

    RUN_SSTRING(printer) (printer, buffer, 10);
}


static void
print_fl(void *printer, unsigned flags)
{
    double f = 0;

    f += K_TIME(printer)->second_f;
    f /= 1000000;
    f += K_TIME(printer)->second_a;
    f *= 60;

    print_cardinal(printer, flags, f);
}


static void
print_hl(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_TIME(printer)->second_a / 3600);
}


static void
print_mc(void *printer, unsigned flags)
{
    print_cardinal
	(printer, flags, localtime(&K_TIME(printer)->time)->tm_mon + 1);
}


static void
print_ml(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_TIME(printer)->second_a / 60);
}


static void
print_sl(void *printer, unsigned flags)
{
    print_cardinal(printer, flags, K_TIME(printer)->second_a);
}


static void
print_t8(void *printer, unsigned flags)
{
    char buffer[8], *string;
    struct tm *down_time;

    down_time = localtime(&K_TIME(printer)->time);
    string = buffer + 8;
    x1f4_xprint_fcardinal(&string, 0, 2, 10, MAX2(down_time->tm_sec));
    *--string = ':';
    x1f4_xprint_fcardinal(&string, 0, 2, 10, MAX2(down_time->tm_min));
    *--string = ':';
    x1f4_xprint_fcardinal(&string, 0, 2, 10, MAX2(down_time->tm_hour));
    *--string = ':';

    RUN_SSTRING(printer) (printer, buffer, 8);
}


static void
print_tl(void *printer, unsigned flags)
{
    char buffer[8], *string;
    unsigned second_l;

    second_l = K_TIME(printer)->second_l;
    string = buffer + 8;
    x1f4_xprint_fcardinal(&string, 0, 2, 10, second_l & 255);
    *--string = ':';
    x1f4_xprint_fcardinal(&string, 0, 2, 10, second_l >> 010 & 255);
    *--string = ':';
    x1f4_xprint_fcardinal(&string, 0, 2, 10, second_l >> 020);

    RUN_SSTRING(printer) (printer, buffer, 8);
}


static void
print_xl(void *printer, unsigned flags)
{
    run_time(printer, ':');
}


static void
print_yl(void *printer, unsigned flags)
{
    run_time(printer, ',');
}


static void
run_time(void *printer, char cdb)
{
    char buffer[11], *string;
    unsigned second_l;

    second_l = K_TIME(printer)->second_l;
    string = buffer + 11;
    x1f4_xprint_fcardinal
	(&string, 0, 2, 10, K_TIME(printer)->second_f * 60 / 1000000);
    *--string = cdb;
    x1f4_xprint_fcardinal(&string, 0, 2, 10, second_l & 255);
    *--string = ':';
    x1f4_xprint_fcardinal(&string, 0, 2, 10, second_l >> 010 & 255);
    *--string = ':';
    x1f4_xprint_fcardinal(&string, 0, 2, 10, second_l >> 020);

    RUN_SSTRING(printer) (printer, buffer, 11);
}
