/*
 * formatx.c
 * Copyright (C) 2005-2010, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if defined HAVE_LIBLEWD
# include <lewd.h>
#endif				/* HAVE_LIBLEWD */

#include <access.h>
#include <lcard.h>
#if !defined HAVE_LIBLEWD
# include <lcardinal.n-b.h>
# include <lcardinal.n.h>
# include <lcardinal.v-b.h>
# include <lcardinal.v.h>
# include <pcardinal.n-b.h>
# include <pcardinal.n.h>
# include <pcardinal.v-b.h>
# include <pcardinal.v.h>
# include <printf.h>
# include <sstring.n-i.h>
# include <sstring.v-i.h>
#endif				/* !HAVE_LIBLEWD */

#define __LCARD_DEFINITION		LCARD_DEFINITION

#define x1f4_detail_format		x1f4_detail_format6

#define RUN_SSTRING(printer) \
    (((struct printer_state *) (printer))->run_sstring)

static const char digits[] = "0123456789";
extern const unsigned _Mylene1f4_c_type_s[];

int
Mylene1f4_count_digits(unsigned flags, unsigned cardinal)
{
    unsigned ing = 1;

    if (cardinal) {
	for (--ing; cardinal; cardinal /= 10, ing++);
    }
    if (flags & X1f4_FORMAT_ALTERNATE) {
	ing += (ing - 1) / 3;
    }

    return ing;
}


int
Mylene1f4_count_pdigits(unsigned flags, void *cardinal)
{
#if __LCARD_DEFINITION != LONG_UNSIGNED_DEFINITION
    lcard_type offset;
#endif				/* __LCARD_DEFINITION != LONG_UNSIGNED_DE... */
    unsigned digits = 0;

#if __LCARD_DEFINITION == LONG_LONG_UNSIGNED_DEFINITION
    offset = *(lcard_type *) cardinal;
    if (!(flags & X1f4_FORMAT_ALTERNATE)) {
	x1f4_nprint_pcardinal(&digits, 10, offset >> 32, offset);
    } else {
	x1f4_nprint_bpcardinal(&digits, 3, 10, offset >> 32, offset);
    }
#elif LCARD_DEFINITION == LONG_UNSIGNED_DEFINITION
    if (!(flags & X1f4_FORMAT_ALTERNATE)) {
	x1f4_nprint_lcardinal(&digits, 10, *(unsigned long *) cardinal);
    } else {
	x1f4_nprint_blcardinal(&digits, 3, 10, *(unsigned long *) cardinal);
    }
#else
    offset = *(lcard_type *) cardinal;
    if (!(flags & X1f4_FORMAT_ALTERNATE)) {
	x1f4_nprint_pcardinal(&digits, 10, offset.major, offset.minor);
    } else {
	x1f4_nprint_bpcardinal(&digits, 3, 10, offset.major, offset.minor);
    }
#endif				/* __LCARD_DEFINITION != LONG_LONG_UNSIGN... */

    return digits;
}


int
Mylene1f4_print_cardinal(void *printer, unsigned flags, unsigned cardinal)
{
    int status;

    if (!(flags & X1f4_FORMAT_ALTERNATE)) {
	char buffer[10], *output;

	output = (char *) buffer + 10;
	do {
	    *--output = digits[cardinal % 10];
	} while ((cardinal /= 10));

	status = RUN_SSTRING(printer) (printer, output, buffer + 10 - output);
    } else {
	char buffer[13], *output;
	unsigned i = 0;

	output = (char *) buffer + 13;
	do {
	    if ((i & 3) == 3) {
		i++;
		*--output = ',';
	    }
	    *--output = digits[cardinal % 10];
	    i++;
	} while ((cardinal /= 10));

	status = RUN_SSTRING(printer) (printer, output, i);
    }

    return status;
}


int
Mylene1f4_print_pcardinal(void *printer, unsigned flags, void *cardinal)
{
    int status;
#if __LCARD_DEFINITION != LONG_UNSIGNED_DEFINITION
    lcard_type offset;
#endif				/* __LCARD_DEFINITION != LONG_UNSIGNED_DE... */

#if __LCARD_DEFINITION == LONG_LONG_UNSIGNED_DEFINITION
    offset = *(lcard_type *) cardinal;
    if (!(flags & X1f4_FORMAT_ALTERNATE)) {
	status = x1f4_vprint_pcardinal
	    (printer, RUN_SSTRING(printer), 0, 10, offset >> 32, offset);
    } else {
	status = x1f4_vprint_bpcardinal
	    (printer, RUN_SSTRING(printer), 0, 3, 10, offset >> 32, offset);
    }
#elif LCARD_DEFINITION == LONG_UNSIGNED_DEFINITION
    if (!(flags & X1f4_FORMAT_ALTERNATE)) {
	status = x1f4_vprint_lcardinal
	    (printer, RUN_SSTRING(printer), 0, 10,
	     *(unsigned long *) cardinal);
    } else {
	status = x1f4_vprint_blcardinal
	    (printer, RUN_SSTRING(printer), 0, 3, 10,
	     *(unsigned long *) cardinal);
    }
#else
    offset = *(lcard_type *) cardinal;
    if (!(flags & X1f4_FORMAT_ALTERNATE)) {
	status = x1f4_vprint_pcardinal
	    (printer, RUN_SSTRING(printer), 0, 10, offset.major, offset.minor);
    } else {
	status = x1f4_vprint_bpcardinal
	    (printer, RUN_SSTRING(printer), 0, 3, 10, offset.major,
	     offset.minor);
    }
#endif				/* __LCARD_DEFINITION != LONG_LONG_UNSIGN... */

    return status;
}


int
Mylene1f4_class_string(void *winter6, const char *data, unsigned size)
{
    x1f4_nprint_isstring(winter6, data, size, _Mylene1f4_c_type_s);

    return 0;
}


int
Mylene1f4_shell_string(void *printer, const char *data, unsigned size)
{
    return x1f4_vprint_isstring
	(printer, RUN_SSTRING(printer), data, size, _Mylene1f4_c_type_s);
}
