/*
 * fslist.h
 * Copyright (C) 2008-2009, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBx1f4_FSLIST_H__
#define __LIBx1f4_FSLIST_H__

/*
 * attribute bit definitions
 */
#define X1f4_FSLIST_TRANS_MASK		(1u << 0x00)

/*
 * error definitions
 */
#define X1f4_FSLIST_FREE_ERROR		-3
#define X1f4_FSLIST_LINK_ERROR		-4
#define X1f4_FSLIST_MODE_ERROR		-5

#define x1f4_file_fslist(___a, ___b) \
    x1f4_file_fspost((___a), (___b))
#define x1f4_line_fslist(___a, ___b, ___c) \
    x1f4_line_fspost((___a), (___b), (___c))
#define x1f4_rand_fslist(___a) \
    x1f4_rand_tslist((___a))
#define x1f4_size_fslist(___a, ___b) \
    x1f4_size_tslist((___a), (___b))
#define x1f4_type_fslist(___a, ___b) \
    x1f4_type_fspost((___a), (___b))

typedef struct x1f4_fslist_type {
    void *trans;
    unsigned base;
} x1f4_fslist_type;

#if !defined __x1f4_trans_type_defined__
typedef struct x1f4_trans_type {
    int (*free) (void *, void *), (*link) (void *, void **, unsigned),
	(*mode) (void *, void **, unsigned);
    void *context;
} x1f4_trans_type;
#endif				/* !__x1f4_trans_type_defined__ */

#if !defined __x1f4_trans_type_defined__
# define __x1f4_trans_type_defined__
#endif				/* !__x1f4_trans_type_defined__ */

extern int x1f4_back_fslist(void **, void *);
extern int x1f4_beta_fslist(void *, unsigned, void **);
extern int x1f4_file_fspost(void *, unsigned *);
extern int x1f4_find_fslist
    (void *, void *, unsigned, int (*) (void *, void *), void **);
extern int x1f4_fini_fslist(void **);
extern int x1f4_init_fslist(void **, unsigned, struct x1f4_fslist_type *);
extern int x1f4_last_fslist
    (void *, void **, void **, void **, void **, void *);
extern int x1f4_line_fspost
    (void *, void *, int (*) (void *, const char *, unsigned));
extern int x1f4_lock_fslist(void **, void *);
extern int x1f4_miss_fslist(void *, unsigned);
extern int x1f4_next_fslist
    (void *, void **, void **, void **, void **, void *);
extern int x1f4_peek_fslist
    (void *, unsigned, void **, void **, void **, void **);
extern int x1f4_push_fslist
    (void *, unsigned, unsigned, const char *, unsigned, unsigned *, void **);
extern int x1f4_rand_tslist(void *);
extern int x1f4_seek_fslist
    (void *, void *, unsigned, int (*) (void *, void *), unsigned *);
extern int x1f4_size_tslist(void *, unsigned *);
extern int x1f4_sort_fslist
    (void *, void *, unsigned, int (*) (void *, void *, void *));
extern int x1f4_tick_fslist
    (void *, void *, unsigned, int (*) (void *, void *), unsigned *, void **);
extern int x1f4_tile_fslist(unsigned *, void *);
extern int x1f4_type_fspost(void *, void **);

#endif				/* __LIBx1f4_FSLIST_H__ */
