/*
 * lanes.c
 * Copyright (C) 2004-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if __ENABLE_x1f4LANES__
# include <errno.h>
# include <fcntl.h>
# include <math.h>
# include <stddef.h>
# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <unistd.h>
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x1f4LANES__
# include <sys/time.h>
# include <sys/types.h>
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x1f4LANES__
# include <access.h>
# include <defs.h>
# include <inter.h>
# include <lanes.h>
# include <types.h>
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x1f4LANES__
# define __DEBUG_x1f4LANES__		0
#else
# define __DEBUG_x1f4LANES__		0
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x1f4LANES__
# define __debug_x1f4LANES__		0
#else
# define __debug_x1f4LANES__		0
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x1f4LANES__
# if 0
#  define LOSE_70(e)			(e)
# else
#  define LOSE_70(e)			1
# endif				/* 0 */
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x1f4LANES__
# define CHANNEL(context) \
    ((struct window_type *) WINDOW(context).within.channel)
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x1f4LANES__
# define EVERLANES(context) \
    ((struct everlanes_type *) WINDOW(context).within.evermiss)
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x00696e6b__
# define LANES_BITS(context)		(EVERLANES(context)->bits)
#endif				/* __ENABLE_x00696e6b__ */

#if __ENABLE_x1f4LANES__
# define LANES_MIRROR(context)		(EVERLANES(context)->mirror)
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x1f4LANES__
# define LANES_TARGET(context)		(EVERLANES(context)->target)
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x1f4LANES__
# define LANES_INCOMING(context)	(EVERLANES(context)->incoming)
# define LANES_OUTGOING(context)	(EVERLANES(context)->outgoing)
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x1f4LANES__
/*
 * this might have been LOAD_ORDER(context), but controller really expects
 * Mylene1f4_LOAD_NEXT.  Not to say that some of Mylene1f4 load orders lay
 * outside lanes specifications.
 */
# define LANES_LOAD_ORDER(context)	Mylene1f4_LOAD_NEXT
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x1f4LANES__
# if __DEBUG_x1f4LANES__
#  if defined __GNUC__
#   define LANES_DEBUG(format, args...) \
    do {								      \
	fprintf(stderr, "Mylene1f4.IL" format, ## args);		      \
    } while (0)
#   define WRITE_DEBUG(format, args...) \
    do {								      \
	fprintf(stderr, "Mylene1f4.WL" format, ## args);		      \
    } while (0)
#   define XRDWR_DEBUG(format, args...) \
    do {								      \
	fprintf(stderr, "Mylene1f4.XL" format, ## args);		      \
    } while (0)
#   define XREAD_DEBUG(format, args...) \
    do {								      \
	fprintf(stderr, "Mylene1f4.RL" format, ## args);		      \
    } while (0)
#  else
#   define LANES_DEBUG			debug
#   define WRITE_DEBUG			debug
#   define XRDWR_DEBUG			debug
#   define XREAD_DEBUG			debug
#  endif			/* __GNUC__ */
# endif				/* __DEBUG_x1f4LANES__ */
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x1f4LANES__
# if __debug_x1f4LANES__
#  if defined __GNUC__
#   define FRAME_DEBUG(format, args...) \
    do {								      \
	fprintf(stderr, "Mylene1f4.FL" format, ## args);		      \
    } while (0)
#  else
#   define FRAME_DEBUG			debug
#  endif			/* __GNUC__ */
# endif				/* __debug_x1f4LANES__ */
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x00696e6b__
# define true(e)			1
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x00696e6b__
# define SET_BIT(context, bit) \
    (LANES_BITS((context)) |= (1 << (bit)))
#endif				/* __ENABLE_x00696e6b__ */

#if __ENABLE_x00696e6b__
# define SUB_BIT(context, bit) \
    (LANES_BITS((context)) &= ~(1 << (bit)))
#endif				/* __ENABLE_x00696e6b__ */

#if __ENABLE_x1f4LANES__
# define CARDINAL(buffer, base) \
    ((buffer)[(base) + 0] << 030 | (buffer)[(base) + 1] << 020		      \
     | (buffer)[(base) + 2] << 010 | (buffer)[(base) + 3] << 000)
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x1f4LANES__
# define CARDiNAL(buffer, base) \
    ((unsigned) CARDINAL(buffer, base))
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x1f4LANES__
# define SET_CARDINAL(buffer, base, card) \
    {									      \
	(buffer)[(base) + 0] = (card) >> 030;				      \
	(buffer)[(base) + 1] = (card) >> 020;				      \
	(buffer)[(base) + 2] = (card) >> 010;				      \
	(buffer)[(base) + 3] = (card) >> 000;				      \
    }
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x1f4LANES__
# define SET_CARDiNAL(buffer, base, card) \
    {									      \
	unsigned cardinal;						      \
									      \
	cardinal = card;						      \
	SET_CARDINAL(buffer, base, cardinal);				      \
    }
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x1f4LANES__
# define max(x, y) \
    (((x) < (y)) ? (y) : (x))
#endif				/* __ENABLE_x1f4LANES__ */

#if __ENABLE_x1f4LANES__
typedef struct window_type {
    char *target;
    unsigned command, dana[8], driver, length, output_size, size;
    unsigned char data[0x10];
    void *output_data;
} window_type;
#endif				/* __ENABLE_x1f4LANES__ */

#if (__DEBUG_x1f4LANES__ || __debug_x1f4LANES__) && !defined __GNUC__
static void debug(char *, ...);
#endif				/* (__DEBUG_x1f4LANES__ || __debug_x1f4LA... */
#if __ENABLE_x00696e6b__
static void get_index(void *, unsigned char *, int);
#endif				/* __ENABLE_x00696e6b__ */

#if (__DEBUG_x1f4LANES__ || __debug_x1f4LANES__) && !defined __GNUC__
static void
debug(char *format, ...)
{
    va_list ap;

    va_start(ap, format);
    vfprintf(stderr, format, ap);
    va_end(ap);
}
#endif				/* (__DEBUG_x1f4LANES__ || __debug_x1f4LA... */


#if __ENABLE_x00696e6b__
/*
 * the next is not a frame index computation, but a frame index approximation
 */
static void
get_index(void *context, unsigned char *buffer, int entry)
{
    unsigned cache, frame;

    if (LINK(context)) {
	cache = 0;
    } else {
	cache = C1f4(context) - STATE(context)[Mylene1f4_REAL_STATE];
	if (SIZE(context) < cache) {
	    cache += SIZE(context);
	}
    }

    Mylene1f4_line_reference(context, &frame);

    frame -= cache / RELEASE(context);

    if (entry) {
	struct x1f42a_type *x1f42a_data;

	x1f42a_data = X1f42(context);
	if (true(x1f42a_data)) {
	    frame += x1f42a_data->frames;
	    frame -= x1f42a_data->next_frame;
	}
    }

    SET_CARDINAL(buffer, 0, frame);
}
#endif				/* __ENABLE_x00696e6b__ */


#if __ENABLE_x1f4LANES__
int
Mylene1f4_ever_lanes(void *context)
{
    void *data;

    data = LANES_TARGET(context);
    if (data) {
	free(data);
    }

    return 0;
}
#endif				/* __ENABLE_x1f4LANES__ */


#if __ENABLE_x1f4LANES__
int
Mylene1f4_fini_lanes(void *context)
{
    struct window_type *window_data;
    void *data;

    window_data = WINDOW(context).within.channel;

    data = window_data->target;
    if (data) {
	free(data);
    }

    free(window_data);

    return 0;
}
#endif				/* __ENABLE_x1f4LANES__ */


#if __ENABLE_x1f4LANES__
int
Mylene1f4_init_lanes(void *context, int *output_fd)
{
    int status = 0;
    struct window_type *window_data;

    window_data = (struct window_type *) malloc(sizeof(struct window_type));
    if (!window_data) {
	status = -1;
    } else {
	do {
	    fd_set active_fd_set, read_fd_set;
	    unsigned i = 0, progress = 0;
	    unsigned char header[32];

	    FD_ZERO(&active_fd_set);
	    FD_SET(STDIN_FILENO, &active_fd_set);

	    for (; i < 2; i++) {
		int fd, flags;

		fd = i ? STDIN_FILENO : STDOUT_FILENO;
		if ((flags = fcntl(fd, F_GETFL)) == -1
		    || fcntl(fd, F_SETFL, flags | O_NONBLOCK) == -1) {
		    status = -1;
		    break;
		}
	    }
	    if (i != 2) {
		break;
	    }

	    while (progress != 32) {
		read_fd_set = active_fd_set;

		if (0 < select(FD_SETSIZE, &read_fd_set, NULL, NULL, NULL)) {
		    if (FD_ISSET(STDIN_FILENO, &read_fd_set)) {
			if (read(STDIN_FILENO, header + progress, 1) != 1) {
			    break;
			} else {
			    progress++;
			}
		    }
		}
	    }

	    if (progress != 32 || memcmp(header, "1f43\0\0\0\3", 8)) {
		status = -1;
		break;
	    } else {
		int fd;

		fd = CARDINAL(header, 0x0c);

# if __DEBUG_x1f4LANES__
		LANES_DEBUG(":    output fd is %d\n", fd);
# endif				/* __DEBUG_x1f4LANES__ */

		*output_fd = fd;

# if __ENABLE_x00696e6b__
		window_data->driver = CARDINAL(header, 0x1c);
# endif				/* __ENABLE_x00696e6b__ */
	    }
	} while (0);

	if (status) {
	    free(window_data);
	} else {
	    window_data->command = ~0;
	    window_data->size = 0;
	    window_data->output_data = NULL;
	    window_data->target = NULL;

	    WINDOW(context).within.channel = window_data;
	}
    }

    return status;
}
#endif				/* __ENABLE_x1f4LANES__ */


#if __ENABLE_x1f4LANES__
/*
 * return values:
 *     -3	read or write error (or protocol error)
 *     -2	fd was closed
 *     -1	insufficient data available
 *	0	done reading
 */
int
Mylene1f4_miss_lanes(void *context, unsigned *detail)
{
    int status = 0;
    unsigned command, *dana, trans_size;
    unsigned char *trans_data;

    command = CHANNEL(context)->command;

    trans_data = CHANNEL(context)->data;
    trans_size = CHANNEL(context)->size;

    dana = CHANNEL(context)->dana;

    if (!LANES_INCOMING(context)) {
	do {
	    status = read
		(STDIN_FILENO, trans_data + trans_size, 0x10 - trans_size);
	} while (status == -1 && errno == EINTR);
	if (status == -1) {
	    status = -3;
	} else if (!status) {
	    status = -2;
	} else {
	    trans_size += status;
	    status = 0;
	}
    }
    if (!status) {
	unsigned driver, i = 0, *mirror;
	unsigned char c;

	driver = CHANNEL(context)->driver;

	mirror = LANES_MIRROR(context);

	for (; i < trans_size; i++) {
	    int do_append = 0;

	    c = trans_data[i];
	    status = 1;
	    switch (command) {
	    case 0000:
		if (!c) {
# if __DEBUG_x1f4LANES__
		    XREAD_DEBUG(":  0 x1f4 lanes command (no operation)\n");
# endif				/* __DEBUG_x1f4LANES__ */
		    command = ~0;
		} else if (c < 3) {
		    command = c;
		} else if (c == 15) {
		    command = c;
		    dana[1] = 2;
		} else if (c == 16) {
		    command = c;
		    dana[1] = 6;
		} else if (c == 32) {
		    command = c;
		} else if (c == 64) {
		    command = c;
		} else if (c == 127) {
		    command = c;
# if __ENABLE_x00696e6b__
		} else if (c < 128) {
		    return -3;
		} else {
		    switch (driver) {
		    case 0x00696e6b:
			if (c == 128) {
			    command = c;
			    dana[1] = 14;
			} else if (c == 129) {
			    command = c;
			    dana[1] = 14;
			} else if (c == 144) {
			    command = c;
			} else if (c == 145) {
			    command = c;
			} else {
			    switch (dana[0]) {
			    case 0x01696e6b:
				if (c == 130) {
				    command = c;
				    dana[1] = 10;
				} else if (c == 131) {
				    command = c;
				    dana[1] = 10;
				} else {
#  if __DEBUG_x1f4LANES__
				    LANES_DEBUG(": %02x unknown \\0ink comma"
						"nd\n", c);
#  endif			/* __DEBUG_x1f4LANES__ */
				    return -3;
				}
				break;
			    default:
#  if __DEBUG_x1f4LANES__
				LANES_DEBUG(": %02x unknown \\0ink command\n",
					    c);
#  endif			/* __DEBUG_x1f4LANES__ */
				return -3;
			    }
			}
			break;
		    default:
#  if __DEBUG_x1f4LANES__
			LANES_DEBUG(": %02x unknown command\n", c);
#  endif			/* __DEBUG_x1f4LANES__ */
			return -3;
		    }
# else
		} else {
#  if __DEBUG_x1f4LANES__
		    LANES_DEBUG(": %02x unknown command\n", c);
#  endif			/* __DEBUG_x1f4LANES__ */
		    return -3;
# endif				/* __ENABLE_x00696e6b__ */
		}
		break;
	    case 0002:
		do_append = 1;
		break;
	    case 0017:
		switch (--dana[1]) {
		case 000:
		    if (c != 0x6e) {
			if (LOSE_70(c != 0x70)) {
			    return -3;
			}
		    }
		    break;
		case 001:
# if __ENABLE_x00696e6b__
		    if (driver == 0x00696e6b) {
			dana[0] = 0;
		    }
# endif				/* __ENABLE_x00696e6b__ */
		    break;
		default:
		    do_append = 1;
		}
		break;
	    case 0020:
		switch (--dana[1]) {
		case 000:
		    command = 0420;
		case 001:
		case 002:
		case 003:
# if __ENABLE_x00696e6b__
		    dana[4] <<= 8;
		    dana[4] |= c;
# endif				/* __ENABLE_x00696e6b__ */
		    break;
		case 004:
		    if (c != 0x4e && c != 0x50) {
			return -3;
		    } else {
# if __ENABLE_x00696e6b__
			dana[3] = c;
# endif				/* __ENABLE_x00696e6b__ */
		    }
		    break;
		case 005:
# if __ENABLE_x00696e6b__
		    dana[5] = c & 128;
# endif				/* __ENABLE_x00696e6b__ */
		    c &= 127;
		    if (c && c != 0x2d) {
			return -3;
		    } else {
# if __ENABLE_x00696e6b__
			dana[2] = c;
			dana[4] = 0;
# endif				/* __ENABLE_x00696e6b__ */
		    }
		}
		break;
# if __ENABLE_x00696e6b__
	    case 0200:
		switch (--dana[1]) {
		case 000:
		case 001:
		case 002:
		case 003:
		    dana[7] <<= 8;
		    dana[7] |= c;
		    break;
		case 004:
		case 005:
		case 006:
		case 007:
		    break;
		case 010:
		case 011:
		case 012:
		case 013:
		    dana[5] <<= 8;
		    dana[5] |= c;
		    break;
		case 014:
		    if (c != 0x6e) {
			if (LOSE_70(c != 0x70)) {
			    return -3;
			}
		    }
		    break;
		case 015:
		    dana[0] = 0x00696e6b;
		    dana[5] = 0;
		    dana[7] = 0;
		    break;
		default:
		    do_append = 1;
		}
		break;
	    case 0201:
		switch (--dana[1]) {
		case 000:
		case 001:
		case 002:
		case 003:
		case 004:
		case 005:
		case 006:
		case 007:
		    break;
		case 010:
		case 011:
		case 012:
		case 013:
		    dana[5] <<= 8;
		    dana[5] |= c;
		    break;
		case 014:
		    if (c != 0x6e) {
			if (LOSE_70(c != 0x70)) {
			    return -3;
			}
		    }
		    break;
		case 015:
		    dana[0] = 0x01696e6b;
		    dana[5] = 0;
		    break;
		default:
		    do_append = 1;
		}
		break;
	    case 0202:
		switch (--dana[1]) {
		case 000:
		    command += 0400;
		case 001:
		case 002:
		case 003:
		    dana[5] <<= 8;
		    dana[5] |= c;
		    break;
		case 004:
		case 005:
		case 006:
		case 007:
		    dana[4] <<= 8;
		    dana[4] |= c;
		    break;
		case 010:
		    if (c != 0x6e) {
			if (LOSE_70(c != 0x70)) {
			    return -3;
			}
		    }
		    break;
		default:
		    dana[3] = c & 1;
		    dana[4] = 0;
		    dana[5] = 0;
		    if (c & ~1) {
			return -3;
		    }
		    break;
		}
		break;
	    case 0203:
		switch (--dana[1]) {
		case 000:
		    command += 0400;
		    break;
		case 001:
		case 002:
		case 003:
		case 004:
		case 005:
		case 006:
		case 007:
		    break;
		case 010:
		    if (c != 0x6e) {
			if (LOSE_70(c != 0x70)) {
			    return -3;
			}
		    }
		    break;
		default:
		    if (c & ~1) {
			return -3;
		    }
		    break;
		}
		break;
# endif				/* __ENABLE_x00696e6b__ */
	    default:
		if (c) {
		    return -3;
		} else {
		    command = 0;
		}
	    }

	    if (do_append) {
		char *target;
		unsigned length;

		length = CHANNEL(context)->length;
		target = CHANNEL(context)->target;
		if (!(length & 15)) {
		    if (!(target = (char *) realloc(target, length + 16))) {
			return -3;
		    }

		    CHANNEL(context)->target = target;
		}
		target[length] = c;
		length++;
		CHANNEL(context)->length = length;
		if (!c) {
		    status = 0;
		}
	    } else if (~command) {
		if (command == 1) {
		    command = ~0;
		    if (!Mylene1f4_send_lanes(context, 1, NULL)) {
			return -3;
		    }
# if __DEBUG_x1f4LANES__
		    XREAD_DEBUG(":  1 x1f4 lanes command (write to standard "
				"output one 01 byte)\n");
# endif				/* __DEBUG_x1f4LANES__ */
# if __DEBUG_x1f4LANES__
		} else if (command == 15) {
		    if (dana[1] == 1) {
			XREAD_DEBUG(":  f x1f4 lanes command (load path)\n");
		    }
# endif				/* __DEBUG_x1f4LANES__ */
# if __DEBUG_x1f4LANES__
		} else if (command == 16) {
# endif				/* __DEBUG_x1f4LANES__ */
		} else if (command == 32) {
# if __DEBUG_x1f4LANES__
		    XREAD_DEBUG(": 20 x1f4 lanes command (toggle automatic a"
				"dvance delay)\n");
# endif				/* __DEBUG_x1f4LANES__ */
# if __ENABLE_x00696e6b__
		    if (driver != 0x00696e6b) {
			status = 0;
		    } else {
			if (mirror[0] != 0x01696e6b) {
# endif				/* __ENABLE_x00696e6b__ */
			    status = 0;
# if __ENABLE_x00696e6b__
			} else {
			    command = ~0;
#  if __DEBUG_x1f4LANES__
			    XREAD_DEBUG(": will ignore, since I don't know b"
					"etter\n");
#  endif				/* __DEBUG_x1f4LANES__ */
			}
		    }
# endif				/* __ENABLE_x00696e6b__ */
		} else if (command == 64) {
		    status = 0;
# if __ENABLE_x00696e6b__
		    if (driver == 0x00696e6b) {
			dana[0] = 0;
		    }
# endif				/* __ENABLE_x00696e6b__ */
# if __DEBUG_x1f4LANES__
		    XREAD_DEBUG(": 40 x1f4 lanes command (dump loaded target"
				" (if any) and wait)\n");
# endif				/* __DEBUG_x1f4LANES__ */
		} else if (command == 127) {
		    status = 0;
# if __DEBUG_x1f4LANES__
		    XREAD_DEBUG(": 7f x1f4 lanes command (exit)\n");
# endif				/* __DEBUG_x1f4LANES__ */
# if __ENABLE_x00696e6b__ && __DEBUG_x1f4LANES__
		} else if (command == 128) {
		    if (dana[1] == 5) {
			XREAD_DEBUG(": 80 x1f4 lanes command (load path, ena"
				    "ble \\0ink)\n");
		    }
# endif				/* __ENABLE_x00696e6b__ && __DEBUG_x1f4LA... */
# if __ENABLE_x00696e6b__ && __DEBUG_x1f4LANES__
		} else if (command == 129) {
		    if (dana[1] == 5) {
			XREAD_DEBUG(": 81 x1f4 lanes command (load path, ena"
				    "ble \\1ink)\n");
		    }
		} else if (command == 130) {
		} else if (command == 131) {
# endif				/* __ENABLE_x00696e6b__ && __DEBUG_x1f4LA... */
# if __ENABLE_x00696e6b__
		} else if (command == 144) {
		    SUB_BIT(context, 000);
		    command = ~0;
# endif				/* __ENABLE_x00696e6b__ */
# if __ENABLE_x00696e6b__ && __DEBUG_x1f4LANES__
		    XREAD_DEBUG(": 90 x1f4 lanes command (no reply 82 comman"
				"ds)\n");
# endif				/* __ENABLE_x00696e6b__ && __DEBUG_x1f4LA... */
# if __ENABLE_x00696e6b__
		} else if (command == 145) {
		    SET_BIT(context, 000);
		    command = ~0;
# endif				/* __ENABLE_x00696e6b__ */
# if __ENABLE_x00696e6b__ && __DEBUG_x1f4LANES__
		    XREAD_DEBUG(": 91 x1f4 lanes command (82 reply 82 comman"
				"ds)\n");
# endif				/* __ENABLE_x00696e6b__ && __DEBUG_x1f4LA... */
		} else if (command == 272) {
# if __ENABLE_x00696e6b__
		    if (driver == 0x00696e6b) {
			if (mirror[0] == 0x00696e6b) {
			    mirror[3] = dana[2];
			    mirror[4] = dana[3];
			    mirror[5] = dana[4];
			    mirror[6] = dana[5];
			    status = 0;
			} else {
			    command = ~0;
			}
		    } else {
# endif				/* __ENABLE_x00696e6b__ */
			command = ~0;
# if __ENABLE_x00696e6b__
		    }
# endif				/* __ENABLE_x00696e6b__ */
# if __ENABLE_x00696e6b__ && __DEBUG_x1f4LANES__
		    XREAD_DEBUG(": 10 x1f4 lanes command (load target: %c%s%"
				"u%c)\n", dana[5] ? 0x23 : 0x20,
				!dana[2] ? "" : "-", dana[4], dana[3]);
# elif __DEBUG_x1f4LANES__
		    XREAD_DEBUG(": 10 x1f4 lanes command\n");
# endif				/* __ENABLE_x00696e6b__ && __DEBUG_x1f4LA... */
# if __ENABLE_x00696e6b__
		} else if (command == 386) {
		    status = 0;
		    if (1) {
			mirror[4] = dana[3];
			mirror[5] = dana[4];
			mirror[6] = dana[5];
		    }
#  if __DEBUG_x1f4LANES__
		    XREAD_DEBUG(": 82 x1f4 lanes command (animation started "
				"at %02u:%02u:%02u:%02u)\n",
				dana[4] / 3600 % 24, dana[4] / 60 % 60,
				dana[4] % 60, dana[5] * 60 / 1000000);
#  endif			/* __DEBUG_x1f4LANES__ */
		} else if (command == 387) {
		    status = 0;
#  if __DEBUG_x1f4LANES__
		    XREAD_DEBUG(": 83 x1f4 lanes command (animation stopped)"
				"\n");
#  endif			/* __DEBUG_x1f4LANES__ */
# endif				/* __ENABLE_x00696e6b__ */
# if __DEBUG_x1f4LANES__
		} else if (command) {
		    XREAD_DEBUG(": %2x x1f4 lanes command\n", command);
# endif				/* __DEBUG_x1f4LANES__ */
		}
	    }

	    if (status) {
		status = -1;
	    } else if (!status) {
# if __ENABLE_x00696e6b__
		if (command == 32 || command == 127 || command == 272
		    || command == 386 || command == 387) {
# elif 0
		}
# else
		if (command == 32 || command == 127) {
# endif				/* __ENABLE_x00696e6b__ */
		    if (detail) {
			*detail = command;
		    }
		    command = ~0;
		    status = -1;

		    break;
		} else if (~command) {
# if __DEBUG_x1f4LANES__
		    if (!CHANNEL(context)->target) {
			XREAD_DEBUG(":    text: (none)\n");
		    } else {
			XREAD_DEBUG(":    text: <%s>\n",
				    CHANNEL(context)->target);
		    }
# endif				/* __DEBUG_x1f4LANES__ */
		    if (command == 2) {
			status = chdir(CHANNEL(context)->target);
			command = ~0;
			free(CHANNEL(context)->target);
			CHANNEL(context)->target = NULL;
			CHANNEL(context)->length = 0;
			if (status) {
			    return -3;
			} else {
			    status = -1;
			}
# if __ENABLE_x00696e6b__
		    } else if (command == 15 || command == 128
			       || command == 129) {
#  if __DEBUG_x1f4LANES__
			if (command == 128) {
			    XREAD_DEBUG(":   entry: %14u\n", dana[5]);
			    XREAD_DEBUG(":   frame: %14u\n", dana[7]);
			} else if (command == 129) {
			    XREAD_DEBUG(":   entry: %14u\n", dana[5]);
			}
#  endif			/* __DEBUG_x1f4LANES__ */
# else
		    } else if (command == 15) {
# endif				/* __ENABLE_x00696e6b__ */
			command = ~0;
			if (LANES_TARGET(context)) {
			    free(LANES_TARGET(context));
			}
			LANES_TARGET(context) = CHANNEL(context)->target;
			CHANNEL(context)->target = NULL;
			CHANNEL(context)->length = 0;

# if __ENABLE_x00696e6b__
			mirror[0] = dana[0];
			mirror[5] = dana[5];
			mirror[7] = dana[7];
# endif				/* __ENABLE_x00696e6b__ */

			break;
		    } else if (command == 64) {
			if (detail) {
			    *detail = command;
			}
			command = ~0;
			if (LANES_TARGET(context)) {
			    free(LANES_TARGET(context));
			}
			LANES_TARGET(context) = NULL;

			status = -1;

# if __ENABLE_x00696e6b__
			mirror[0] = dana[0];
# endif				/* __ENABLE_x00696e6b__ */

			break;
		    }
		}
	    }
	}

	if (i == trans_size) {
	    trans_size = 0;
	    if (status == -1) {
		if (detail) {
		    *detail = 0;
		}
	    }
	} else {
	    i++;
	    trans_size -= i;
	    if (trans_size) {
		memcpy(trans_data, trans_data + i, trans_size);
	    }
	}
    }

    CHANNEL(context)->size = trans_size;

    CHANNEL(context)->command = command;

    LANES_INCOMING(context) = trans_size;

    return status;
}
#endif				/* __ENABLE_x1f4LANES__ */


#if __ENABLE_x1f4LANES__
int
Mylene1f4_push_lanes(void *context)
{
    int status;
    unsigned outgoing;
    unsigned char *output_data;

    outgoing = LANES_OUTGOING(context);
    output_data = CHANNEL(context)->output_data;

    do {
	status = write(STDOUT_FILENO, output_data, outgoing);
    } while (status == -1 && errno == EINTR);

    if (0 < status) {
# if __DEBUG_x1f4LANES__
	WRITE_DEBUG(":        : %6u, %6u\n", status, outgoing - status);
# endif				/* __DEBUG_x1f4LANES__ */
	if (status == outgoing) {
	    LANES_OUTGOING(context) = 0;
	} else {
	    outgoing -= status;
	    LANES_OUTGOING(context) = outgoing;
	    memmove(output_data, output_data + status, outgoing);
	}

	status = 0;
    }

    return status;
}
#endif				/* __ENABLE_x1f4LANES__ */


#if __ENABLE_x1f4LANES__
int
Mylene1f4_send_lanes(void *context, unsigned dialog, void *data)
{
    unsigned output_size;
    unsigned char buffer[040];

    if (!dialog) {
# if __DEBUG_x1f4LANES__
	WRITE_DEBUG(": %2x\n", dialog);
# endif				/* __DEBUG_x1f4LANES__ */

	buffer[0] = 0;

	output_size = 1;
# if defined __ENABLE_x00696e6b__
    } else if (dialog == 1) {
# else
    } else {
# endif				/* __ENABLE_x00696e6b__ */
# if __DEBUG_x1f4LANES__
	WRITE_DEBUG(":  1\n");
# endif				/* __DEBUG_x1f4LANES__ */

	buffer[0] = 1;
	output_size = 1;
# if __ENABLE_x00696e6b__
    } else {
	if (dialog == 0200) {
	    struct x1f42a_type *x1f42a_data;
	    unsigned *mirror, mirror_2_, frames;

	    x1f42a_data = X1f42(context);
	    buffer[0x00] = dialog;
	    buffer[0x01] = 0;
	    buffer[0x02] = 0;
	    buffer[0x03] = LANES_LOAD_ORDER(context);
	    frames = x1f42a_data->frames;
	    TARGET(context).lock(context, buffer + 0x08, frames);
	    mirror = LANES_MIRROR(context);
	    SET_CARDINAL(buffer, 0x10, frames);
	    mirror_2_ = mirror[2];
	    if (mirror[0] == 0x00696e6b) {
		mirror_2_ += frames;
		mirror_2_ -= x1f42a_data->next_frame;
	    }
	    SET_CARDINAL(buffer, 0x14, mirror_2_);
	    TARGET(context).lock(context, buffer + 0x18, mirror[2]);
#  if __DEBUG_x1f4LANES__
	    WRITE_DEBUG(": %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %"
			"2x %2x %2x %2x\n\t      %2x %2x %2x %2x %2x %2x %2x"
			" %2x %2x %2x %2x %2x %2x %2x %2x %2x\n",
			(unsigned char) buffer[000],
			(unsigned char) buffer[001],
			(unsigned char) buffer[002],
			(unsigned char) buffer[003],
			(unsigned char) buffer[004],
			(unsigned char) buffer[005],
			(unsigned char) buffer[006],
			(unsigned char) buffer[007],
			(unsigned char) buffer[010],
			(unsigned char) buffer[011],
			(unsigned char) buffer[012],
			(unsigned char) buffer[013],
			(unsigned char) buffer[014],
			(unsigned char) buffer[015],
			(unsigned char) buffer[016],
			(unsigned char) buffer[017],
			(unsigned char) buffer[020],
			(unsigned char) buffer[021],
			(unsigned char) buffer[022],
			(unsigned char) buffer[023],
			(unsigned char) buffer[024],
			(unsigned char) buffer[025],
			(unsigned char) buffer[026],
			(unsigned char) buffer[027],
			(unsigned char) buffer[030],
			(unsigned char) buffer[031],
			(unsigned char) buffer[032],
			(unsigned char) buffer[033],
			(unsigned char) buffer[034],
			(unsigned char) buffer[035],
			(unsigned char) buffer[036],
			(unsigned char) buffer[037]);
#  endif			/* __DEBUG_x1f4LANES__ */
#  if __debug_x1f4LANES__
	    FRAME_DEBUG(":  length:    %02u:%02u:%02u:%02u\n",
			CARDiNAL(buffer, 010) / 3600 % 24,
			CARDiNAL(buffer, 010) / 60 % 60,
			CARDiNAL(buffer, 010) % 60,
			CARDiNAL(buffer, 014) * 60 / 1000000);
	    FRAME_DEBUG(":   count: %14u\n", CARDiNAL(buffer, 020));
	    FRAME_DEBUG(":   index: %14u\n", CARDiNAL(buffer, 024));
	    FRAME_DEBUG(":  system:    %02u:%02u:%02u:%02u\n",
			CARDiNAL(buffer, 030) / 3600 % 24,
			CARDiNAL(buffer, 030) / 60 % 60,
			CARDiNAL(buffer, 030) % 60,
			CARDiNAL(buffer, 034) * 60 / 1000000);
#  endif			/* __debug_x1f4LANES__ */

	    output_size = 040;
	} else if (dialog == 0202) {
	    unsigned char *system;

	    buffer[0x00] = dialog;
	    buffer[0x01] = 0;
	    buffer[0x02] = 0;
	    buffer[0x03] = LANES_LOAD_ORDER(context);
	    get_index(context, buffer + 0x04,
		      LANES_MIRROR(context)[0] == 0x00696e6b);
	    system = data;
	    buffer[0x08] = system[0x00];
	    buffer[0x09] = system[0x01];
	    buffer[0x0a] = system[0x02];
	    buffer[0x0b] = system[0x03];
	    buffer[0x0c] = system[0x04];
	    buffer[0x0d] = system[0x05];
	    buffer[0x0e] = system[0x06];
	    buffer[0x0f] = system[0x07];
	    buffer[0x10] = system[0x08];
	    buffer[0x11] = system[0x09];
	    buffer[0x12] = system[0x0a];
	    buffer[0x13] = system[0x0b];
	    buffer[0x14] = system[0x0c];
	    buffer[0x15] = system[0x0d];
	    buffer[0x16] = system[0x0e];
	    buffer[0x17] = system[0x0f];
#  if __DEBUG_x1f4LANES__
	    WRITE_DEBUG(": %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %"
			"2x %2x %2x %2x\n\t      %2x %2x %2x %2x %2x %2x %2x"
			" %2x\n", (unsigned char) buffer[000],
			(unsigned char) buffer[001],
			(unsigned char) buffer[002],
			(unsigned char) buffer[003],
			(unsigned char) buffer[004],
			(unsigned char) buffer[005],
			(unsigned char) buffer[006],
			(unsigned char) buffer[007],
			(unsigned char) buffer[010],
			(unsigned char) buffer[011],
			(unsigned char) buffer[012],
			(unsigned char) buffer[013],
			(unsigned char) buffer[014],
			(unsigned char) buffer[015],
			(unsigned char) buffer[016],
			(unsigned char) buffer[017],
			(unsigned char) buffer[020],
			(unsigned char) buffer[021],
			(unsigned char) buffer[022],
			(unsigned char) buffer[023],
			(unsigned char) buffer[024],
			(unsigned char) buffer[025],
			(unsigned char) buffer[026],
			(unsigned char) buffer[027]);
#  endif			/* __DEBUG_x1f4LANES__ */
#  if __debug_x1f4LANES__
	    FRAME_DEBUG(":   frame: %14u\n", CARDiNAL(buffer, 004));
	    FRAME_DEBUG(":  system:    %02u:%02u:%02u:%02u\n",
			CARDiNAL(buffer, 010) / 3600 % 24,
			CARDiNAL(buffer, 010) / 60 % 60,
			CARDiNAL(buffer, 010) % 60,
			CARDiNAL(buffer, 014) * 60 / 1000000);
	    FRAME_DEBUG(":   clock:    %02u:%02u:%02u:%02u\n",
			CARDiNAL(buffer, 020) / 3600 % 24,
			CARDiNAL(buffer, 020) / 60 % 60,
			CARDiNAL(buffer, 020) % 60,
			CARDiNAL(buffer, 024) * 60 / 1000000);
#  endif			/* __debug_x1f4LANES__ */

	    output_size = 030;
	} else {
	    unsigned char *system;

	    buffer[0x00] = dialog;
	    buffer[0x01] = 0;
	    buffer[0x02] = 0;
	    buffer[0x03] = LANES_LOAD_ORDER(context);
	    get_index(context, buffer + 0x04,
		      LANES_MIRROR(context)[0] == 0x00696e6b);
	    system = data;
	    buffer[0x08] = system[0x00];
	    buffer[0x09] = system[0x01];
	    buffer[0x0a] = system[0x02];
	    buffer[0x0b] = system[0x03];
	    buffer[0x0c] = system[0x04];
	    buffer[0x0d] = system[0x05];
	    buffer[0x0e] = system[0x06];
	    buffer[0x0f] = system[0x07];
#  if __DEBUG_x1f4LANES__
	    WRITE_DEBUG(": %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %"
			"2x %2x %2x %2x\n", (unsigned char) buffer[000],
			(unsigned char) buffer[001],
			(unsigned char) buffer[002],
			(unsigned char) buffer[003],
			(unsigned char) buffer[004],
			(unsigned char) buffer[005],
			(unsigned char) buffer[006],
			(unsigned char) buffer[007],
			(unsigned char) buffer[010],
			(unsigned char) buffer[011],
			(unsigned char) buffer[012],
			(unsigned char) buffer[013],
			(unsigned char) buffer[014],
			(unsigned char) buffer[015],
			(unsigned char) buffer[016],
			(unsigned char) buffer[017]);
#  endif			/* __DEBUG_x1f4LANES__ */
#  if __debug_x1f4LANES__
	    FRAME_DEBUG(":   frame: %14u\n", CARDiNAL(buffer, 004));
	    FRAME_DEBUG(":  system:    %02u:%02u:%02u:%02u\n",
			CARDiNAL(buffer, 010) / 3600 % 24,
			CARDiNAL(buffer, 010) / 60 % 60,
			CARDiNAL(buffer, 010) % 60,
			CARDiNAL(buffer, 014) * 60 / 1000000);
#  endif			/* __debug_x1f4LANES__ */

	    output_size = 020;
	}
# endif				/* __ENABLE_x00696e6b__ */
    }

    {
	unsigned outgoing, outPut_size;
	unsigned char *output_data;

	outgoing = LANES_OUTGOING(context);

	output_data = CHANNEL(context)->output_data;
	outPut_size = CHANNEL(context)->output_size;
	if (outPut_size - outgoing < output_size) {
	    if (!(output_data = realloc(output_data, outPut_size + 512))) {
		return 0;
	    }

	    outPut_size += 512;

	    CHANNEL(context)->output_data = output_data;
	}

	memcpy(output_data + outgoing, buffer, output_size);

	LANES_OUTGOING(context) = outgoing + output_size;
    }

    return 1;
}
#endif				/* __ENABLE_x1f4LANES__ */


#if __ENABLE_x1f4LANES__
/*
 * return values:
 *     -4	external error (Mylene0fa has exited?)
 *     -3	I/O error (or protocol error)
 *     -2	fd was closed
 *     -1	insufficient data available
 *	0	done reading
 */
int
Mylene1f4_sync_lanes(void *context, unsigned *detail,
		     int (*check_environment) (void *context))
{
    fd_set fd_set_0, *fd_set_0_select = NULL, fd_set_1, *fd_set_1_select =
	NULL;
    int status;
    struct timeval *selector = NULL, timeout;
    unsigned nfds = 0;

    do {
	if (1) {
	    if (LANES_INCOMING(context)) {
		selector = &timeout;
		timeout.tv_sec = 0;
		timeout.tv_usec = 0;
	    } else {
		fd_set_0_select = &fd_set_0;
		FD_ZERO(fd_set_0_select);
		FD_SET(STDIN_FILENO, fd_set_0_select);
		nfds = STDIN_FILENO;
	    }
	}
	if (1) {
	    if (LANES_OUTGOING(context)) {
		fd_set_1_select = &fd_set_1;
		FD_ZERO(fd_set_1_select);
		FD_SET(STDOUT_FILENO, fd_set_1_select);
		nfds = max(STDOUT_FILENO, nfds);
	    }
	}

	status = select
	    (nfds + 1, fd_set_0_select, fd_set_1_select, NULL, selector);
	if (status == -1) {
	    if (errno == EINTR) {
		if (check_environment(context)) {
		    status = -4;
		    break;
		}
		continue;
	    } else {
		status = -3;
		break;
	    }
	}

	if (fd_set_1_select && FD_ISSET(STDOUT_FILENO, fd_set_1_select)) {
	    if (Mylene1f4_push_lanes(context)) {
		status = -3;
		break;
	    }
	}

	if (!LANES_INCOMING(context)) {
	    if (!fd_set_0_select) {
		break;
	    } else {
		if (!FD_ISSET(STDIN_FILENO, fd_set_0_select)) {
		    break;
		}
	    }
	}

	status = Mylene1f4_miss_lanes(context, detail);
	if (status < 1) {
	    break;
	}
    } while (0);

    return status;
}
#endif				/* __ENABLE_x1f4LANES__ */
