/*
 * link.c
 * Copyright (C) 2005-2011, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <access.h>
#include <inter.h>

#if defined BITS
#else
# define BITS(context)			0
#endif				/* BITS */

#define NEXT_LEVEL(value, step) \
    ((value) + ((step) - (value) % (step)) % (step))
#define PREVIOUS_LEVEL(value, step) \
    ((value) - (value) % (step))

int
Mylene1f4_fini_link(void *context)
{
    unsigned data[2];

#if WORDS_BIGENDIAN
    data[0] = 128 << 030;
#else
    data[0] = 128;
#endif				/* WORDS_BIGENDIAN */
    data[1] = C1f4(context);

    return Mylene1f4_send_mylene0fa(context, 128, data);
}


int
Mylene1f4_init_link(void *context, unsigned start2)
{
    unsigned data[6];

#if WORDS_BIGENDIAN
    data[0] = SAMPLING(context) << 010 | (CHANNELS(context) - 1)
	| BITS(context) | 129 << 030;
#else
    data[0] = (SAMPLING(context) & 255) << 020
	| SAMPLING(context) >> 010 << 010
	| ((CHANNELS(context) - 1) | BITS(context)) << 030 | 129;
#endif				/* WORDS_BIGENDIAN */
    data[1] = C1f4(context);
    data[2] = SIZE(context);

    {
#if SIZEOF_UNSIGNED_LONG != 8
	double bias;
#endif					/* SIZEOF_UNSIGNED_LONG != 8 */
	unsigned head, start;
#if SIZEOF_UNSIGNED_LONG == 8
	unsigned long bias;
#endif					/* SIZEOF_UNSIGNED_LONG == 8 */

	head = HEAD(context);
	start = start2;
	bias = head < start ? start : head;
	bias *= RELEASE(context);
#if SIZEOF_UNSIGNED_LONG == 8
	data[4] = bias >> 32;
	data[5] = bias;
#else
	data[5] = bias;
	bias /= 65536;
	bias /= 65536;
	data[4] = bias;
#endif					/* SIZEOF_UNSIGNED_LONG == 8 */
    }

    LINK(context)++;

    return Mylene1f4_send_mylene0fa(context, 129, data);
}


#if __RESTART_Mylene0fa__
int
Mylene1f4_line_link(void *context, unsigned e0fa, unsigned start3)
{
    unsigned data[6];

# if WORDS_BIGENDIAN
    data[0] = SAMPLING(context) << 010 | (CHANNELS(context) - 1)
	| BITS(context) | 129 << 030;
# else
    data[0] = (SAMPLING(context) & 255) << 020
	| SAMPLING(context) >> 010 << 010
	| ((CHANNELS(context) - 1) | BITS(context)) << 030 | 129;
# endif				/* WORDS_BIGENDIAN */
    data[1] = e0fa;
    data[2] = SIZE(context);

    {
# if SIZEOF_UNSIGNED_LONG != 8
	double bias;
# endif					/* SIZEOF_UNSIGNED_LONG != 8 */
# if SIZEOF_UNSIGNED_LONG == 8
	unsigned long bias;
# endif					/* SIZEOF_UNSIGNED_LONG == 8 */

	bias = REFERENCE(context);
	bias *= RELEASE(context);
	bias -= start3;
# if SIZEOF_UNSIGNED_LONG == 8
	data[4] = bias >> 32;
	data[5] = bias;
# else
	data[5] = bias;
	bias /= 65536;
	bias /= 65536;
	data[4] = bias;
# endif					/* SIZEOF_UNSIGNED_LONG == 8 */
    }

    LINK(context)++;

    return Mylene1f4_send_mylene0fa(context, 129, data);
}
#endif				/* __RESTART_Mylene0fa__ */


int
Mylene1f4_make_link(void *context)
{
    unsigned cache_size, e1f4_index, release_size, size;

    cache_size = CACHE(context);
    release_size = RELEASE(context);
    e1f4_index = NEXT_LEVEL(C1f4(context), release_size);
    size = PREVIOUS_LEVEL(cache_size, release_size);
    if (size < e1f4_index + release_size) {
	e1f4_index = 0;
    }
    C1f4(context) = e1f4_index;
    SIZE(context) = size;

    STATE(context)[0] = e1f4_index;

    return 0;
}
