/*
 * list.c
 * Copyright (C) 2001-2011, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/stat.h>
#include <sys/types.h>

#include <access.h>
#include <defs.h>
#include <fslist.h>
#include <inter.h>
#include <list.h>

#define BLOCK_SIZE			4096

#define FIELD_SIZE			4096

static int flat_deck(void *, void *, void *);
#if __ENABLE_TAGGED_LIST__
static int read_bias(void *, void *, unsigned *, unsigned *, int,
		     const char *);
#endif				/* __ENABLE_TAGGED_LIST__ */
static int read_data(void *, void *, unsigned *, unsigned *, int,
		     const char *);
#if __ENABLE_TAGGED_LIST__
static int read_deck(void *, void *, char **, unsigned *, unsigned *, int,
		     const char *);
#endif				/* __ENABLE_TAGGED_LIST__ */
static int read_ever(void *, void *, char **, unsigned *, unsigned *, int,
		     const char *);
static int read_list(void *, const char *, int, int);

static int
flat_deck(void *none, void *flat, void *deck)
{
    return ((struct list21_type *) flat)->index
	- ((struct list21_type *) deck)->index;
}


#if __ENABLE_TAGGED_LIST__
static int
read_bias(void *context, void *list, unsigned *flat, unsigned *class, int fd,
	  const char *path)
{
    char block[BLOCK_SIZE], *data = NULL, c = '\0', *line = NULL;
    int last = 0, next = 1, status = 0;
    unsigned dana = 0, field = ~0, index = 0, miss = 0, size = 0, offset1s[7],
	offset2s[7], record = 0, this = 0, zero = 0;

    while (1) {
	if (next) {
	    field = ~0;
	    miss = 0;
	    next = 0;
	}

	if (miss == size) {
	    char *sail;

	    if (!(sail = (char *) realloc(line, miss + 1024))) {
		status = -1;
		break;
	    }

	    size += 1024;
	    line = sail;
	}

	if (this) {
	} else {
	    this = read(fd, block, BLOCK_SIZE);
	    if ((int) this == -1) {
		status = -1;
		break;
	    }
	    last = !this;
	    data = block;
	}
	if (last) {
	} else {
	    c = *data++;
	    this--;
	}

	if (last) {
	    zero = 2;
	} else {
	    if (!c) {
		zero++;
	    } else {
		if (!record || zero) {
		    if (~field) {
			offset2s[field] = miss;
		    }

		    zero = 0;

		    if (c == 'A') {
			field = Mylene1f4_ALBUM_FIELD;
		    } else if (c == 'N') {
			field = 6;
		    } else if (c == 'a') {
			field = Mylene1f4_MATCH_FIELD;
		    } else if (c == 'c') {
			field = Mylene1f4_NOTES_FIELD;
		    } else if (c == 'g') {
			field = Mylene1f4_GENRE_FIELD;
		    } else if (c == 't') {
			field = Mylene1f4_TITLE_FIELD;
		    } else if (c == 'y') {
			field = Mylene1f4_SHORT_FIELD;
		    } else {
			Mylene1f4_fprintf_error
			    (context, "%s: invalid %02x tag in `%s'\n",
			     Mylene1f4(context), c, path);
			status = 13;
			break;
		    }

		    if (record & (1 << field)) {
			unsigned tail;

			tail = offset2s[field];
			if (tail == miss) {
			} else {
			    unsigned head, i, size;

			    head = offset1s[field];

			    memmove(line + head, line + tail, miss - tail);

			    size = tail - head;

			    miss = miss - size;

			    for (i = 0; i < 7; i++) {
				if (i != field) {
				    if (record & (1 << i)) {
					if (head < offset1s[i]) {
					    offset1s[i] -= size;
					    offset2s[i] -= size;
					}
				    } else {
				    }
				}
			    }
			}
		    } else {
			record |= 1 << field;
		    }

		    offset1s[field] = miss;

		    continue;
		}
	    }
	}

	if (zero == 2) {
	    if (miss) {
		if (record & (1 << 6)) {
		} else {
		    Mylene1f4_fprintf_error
			(context, "%s: missing path in `%s' record\n",
			 Mylene1f4(context), path);
		    status = 31;
		    break;
		}

		offset2s[field] = miss;

		if (1) {
		    unsigned offset, post;
		    void *push;

		    post = miss;

		    miss = offset2s[6] - offset1s[6];

		    /*
		     * there should no need for one zero byte if _post_ is
		     * zero, for there is already one - the one ending the file
		     * basename.  More, this byte is even used - for non zero
		     * _post_ cases.
		     */
		    post = post - miss;
		    if (post) {
		    } else {
			post = 1;
		    }

		    if (post < TAGGED_LIST_RECORD_SIZE) {
		    } else {
			Mylene1f4_fprintf_error
			    (context, "%s: exceedingly large record in `%s' "
			     "(%u bytes, limit is %u)\n", Mylene1f4(context),
			     path, post, TAGGED_LIST_RECORD_SIZE);
			status = 33;
			break;
		    }

		    status = x1f4_push_fslist
			(list, index, miss - 1, line + offset1s[6], post,
			 &offset, &push);
		    if (status) {
			break;
		    } else {
			unsigned *offset6s;

			((struct list29_type *) push)->index = index;

			index++;

			if (dana < miss) {
			    dana = miss;
			}

			offset6s = ((struct list29_type *) push)->shift;

			*offset6s = record;

			record &= 63;
			if (!record) {
			    char *flat;
			    unsigned i;

			    flat = push;
			    flat[offset] = 0;

			    offset6s[1] = offset;

			    offset6s += 8;

			    offset++;

			    i = 6;
			    for (; i; i--) {
				*--offset6s = offset;
			    }
			} else {
			    char *flat;
			    unsigned i;

			    flat = (char *) push + offset;

			    offset6s[1] = post + offset;

			    offset6s += 8;

			    post = offset;

			    i = 6;
			    while (i) {
				*--offset6s = post;

				i--;

				if (record & (1 << i)) {
				    unsigned head, size;

				    head = offset1s[i];
				    size = offset2s[i] - head;
				    memcpy(flat, line + head, size);

				    flat += size;

				    post += size;
				}
			    }
			}
		    }
		}

		if (last) {
		    break;
		}

		next = 1;

		record = 0;
	    } else {
		if (record) {
		    Mylene1f4_fprintf_error
			(context, "%s: invalid last record in `%s'\n",
			 Mylene1f4(context), path);
		    status = 31;
		    break;
		}

		if (last) {
		    break;
		}
	    }

	    zero = 0;
	} else {
	    line[miss++] = c;
	}
    }

    if (line) {
	free(line);
    }

    if (status) {
	*class = 0;
    } else {
	*class = index;
	if (1) {
	    *flat = dana;
	}
    }

    return status;
}
#endif				/* __ENABLE_TAGGED_LIST__ */


static int
read_data(void *context, void *list, unsigned *flat, unsigned *class, int fd,
	  const char *path)
{
    char block[BLOCK_SIZE], *data = NULL, c = '\0', *line = NULL;
    int last = 0, next = 1, status = 0;
    unsigned dana = 0, index = 0, miss = 0, size = 0, this = 0;

    while (1) {
	if (next) {
	    miss = 0;
	    next = 0;
	}

	if (miss == size) {
	    char * sail;

	    if (!(sail = (char *) realloc(line, miss + 1024))) {
		status = -1;
		break;
	    }

	    size += 1024;
	    line = sail;
	}

	if (!this) {
	    this = read(fd, block, BLOCK_SIZE);
	    if ((int) this == -1) {
		status = -1;
		break;
	    }
	    last = !this;
	    data = block;
	}
	if (!last) {
	    c = *data++;
	    this--;
	}

	if (!c || last) {
	    if (miss) {
		line[miss++] = '\0';

		if (1) {
		    unsigned offset;
		    void *push;

		    if (miss < SIMPLE_LIST_RECORD_SIZE) {
		    } else {
			Mylene1f4_fprintf_error
			    (context, "%s: exceedingly large record in `%s' "
			     "(%u bytes, limit is %u)\n", Mylene1f4(context),
			     path, miss, SIMPLE_LIST_RECORD_SIZE);
			status = 33;
			break;
		    }

		    status = x1f4_push_fslist
			(list, index, miss, line, 0, &offset, &push);
		    if (status) {
			break;
		    } else {
			((struct list21_type *) push)->index = index;

			index++;

			if (dana < miss) {
			    dana = miss;
			}
		    }
		}

		if (last) {
		    break;
		}

		next = 1;
	    } else {
		if (last) {
		    break;
		}
	    }
	} else {
	    line[miss++] = c;
	}
    }

    if (line) {
	free(line);
    }

    if (status) {
	*class = 0;
    } else {
	*class = index;
	if (1) {
	    *flat = dana;
	}
    }

    return status;
}


#if __ENABLE_TAGGED_LIST__
static int
read_deck(void *context, void *list, char **line, unsigned *flat,
	  unsigned *count, int fd, const char *path)
{
    int status;
    unsigned e;

    if (1) {
	unsigned class;

	status = read_bias(context, list, &e, &class, fd, path);
	if (status) {
	    *count = 0;
	} else {
	    *count = class;
	    if (class) {
		char *fast;

		fast = (char *) malloc(e);
		if (fast) {
		    struct id3tag_type *id3tag_data;

		    id3tag_data = (struct id3tag_type *)
			malloc(sizeof(struct id3tag_type));
		    if (!id3tag_data) {
			free(fast);

			status = -1;
		    } else {
			ID3T(context) = Mylene1f4_BROKEN_ID3TAG;
			ID3X(context) = id3tag_data;

			TAGGED(context).id3tag_data = id3tag_data;

			*flat = e;
			*line = fast;
		    }
		} else {
		    status = -1;
		}
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_TAGGED_LIST__ */


static int
read_ever(void *context, void *list, char **line, unsigned *flat,
	  unsigned *count, int fd, const char *path)
{
    int status;
    unsigned e;

    if (1) {
	unsigned class;

	status = read_data(context, list, &e, &class, fd, path);
	if (status) {
	    *count = 0;
	} else {
	    *count = class;
	    if (class) {
		char *fast;

		fast = (char *) malloc(e);
		if (fast) {
		    *flat = e;
		    *line = fast;
		} else {
		    status = -1;
		}
	    }
	}
    }

    return status;
}


static int
read_list(void *context, const char *path, int fd, int do_shuffle)
{
    char *line = NULL;
    int status;
    struct x1f4_fslist_type fslist;
    unsigned count, flat = 0;
    void *list;

#if __ENABLE_TAGGED_LIST__
    if (FLAGS(context) & Mylene1f4_BREAK_LIST) {
	fslist.base = sizeof(struct list29_type);
    } else {
#endif				/* __ENABLE_TAGGED_LIST__ */
	fslist.base = sizeof(struct list21_type);
#if __ENABLE_TAGGED_LIST__
    }
#endif				/* __ENABLE_TAGGED_LIST__ */

    status = x1f4_init_fslist(&list, 0, &fslist);
    if (status) {
    } else {
#if __ENABLE_TAGGED_LIST__
	if (FLAGS(context) & Mylene1f4_BREAK_LIST) {
	    status = read_deck(context, list, &line, &flat, &count, fd, path);
	} else {
#endif				/* __ENABLE_TAGGED_LIST__ */
	    status = read_ever(context, list, &line, &flat, &count, fd, path);
#if __ENABLE_TAGGED_LIST__
	}
#endif				/* __ENABLE_TAGGED_LIST__ */
	if (status || !count) {
	    if (!count) {
		EXCESS(context).e1.list87.count = 0;
	    }

	    x1f4_fini_fslist(&list);
	} else {
	    EXCESS(context).e1.list81.window = list;
	    EXCESS(context).e1.list81.wynona = line;

	    EXCESS(context).e1.list81.max = flat;

	    EXCESS(context).e1.list82.max = flat;

	    EXCESS(context).e1.list87.count = count;

	    if (do_shuffle) {
		x1f4_rand_fslist(list);
		if (1) {
		    FLAGS(context) |= Mylene1f4_TILED_LIST;
		}
	    }

#if __DUMP_LIST__
	    Mylene1f4_dump_list(list88);
#endif				/* __DUMP_LIST__ */
	}
    }

    return status;
}


int
Mylene1f4_deck_list(void *context, int *start, int *class)
{
    int delete, status;

    do {
	int class;

	EXCESS(context).e1.list87.clear = EXCESS(context).e1.list87.count;

	class = *start;
	if (class) {
	    if (1) {
		int back;
		unsigned count;

		count = EXCESS(context).e1.list87.clear;

		if (class < 0) {
		    class = -class;
		    back = 1;
		} else {
		    back = 0;
		}
		if (!(class < count)) {
		    if (!REPEAT(context)) {
			delete = 1;
			status = 0;
			if (1) {
			    break;
			}
		    } else {
			class %= count;
		    }
		}
		if (back) {
		    class = count - class;
		}

		EXCESS(context).e1.list87.clear = count - class;

		*start = 0;
	    }
	}

	status = Mylene1f4_seek_list(context, class);
	if (status) {
	    delete = 0;
	    if (1) {
		break;
	    }
	}

	if (LOAD_ORDER(context) == Mylene1f4_LOAD_LAST) {
	    Mylene1f4_half_list(context);
	}

	delete = 0;
    } while (0);

    *class = delete;

    return status;
}


int
Mylene1f4_fast_list(void *context)
{
    unsigned index;

    Mylene1f4_fix_index(context, &index);

    return Mylene1f4_seek_list(context, index);
}


int
Mylene1f4_fini_list(void *context)
{
#if __ENABLE_TAGGED_LIST__
    if (FLAGS(context) & Mylene1f4_BREAK_LIST) {
	free(TAGGED(context).id3tag_data);
    }
#endif				/* __ENABLE_TAGGED_LIST__ */

    free(EXCESS(context).e1.list81.wynona);

    if (1) {
	x1f4_fini_fslist(&EXCESS(context).e1.list81.window);
    }

    return 0;
}


int
Mylene1f4_flat_list(void *context)
{
    unsigned index;

    x1f4_sort_fslist(EXCESS(context).e1.list81.window, NULL, 0, flat_deck);

    x1f4_tile_fslist(&index, EXCESS(context).e1.list87.lock);

    FLAGS(context) &= ~Mylene1f4_TILED_LIST;

    EXCESS(context).e1.list87.clear = EXCESS(context).e1.list87.count - index;

    if (LOAD_ORDER(context) == Mylene1f4_LOAD_LAST) {
	Mylene1f4_half_list(context);
    }

    return 0;
}


int
Mylene1f4_half_list(void *context)
{
    EXCESS(context).e1.list87.clear =
	EXCESS(context).e1.list87.count - EXCESS(context).e1.list87.clear + 1;

    return 0;
}


int
Mylene1f4_head_list(void *context, unsigned this)
{
    unsigned index;

    Mylene1f4_fix_index(context, &index);

    return Mylene1f4_seek_list(context, index + this);
}


int
Mylene1f4_init_list(void *context, int argc, char **argv, int do_shuffle)
{
    int status;
    struct x1f4_fslist_type fslist;
    unsigned i;
    void *list;

    fslist.base = sizeof(struct list21_type);

    status = x1f4_init_fslist(&list, 0, &fslist);
    if (status) {
    } else {
	char *line;
	unsigned flat = 0;

	i = 0;
	for (; i < argc; i++) {
	    unsigned offset, size;
	    void *push;

	    size = strlen(*argv);

	    status = x1f4_push_fslist(list, i, size, *argv, 0, &offset, &push);
	    if (status) {
		break;
	    } else {
		argv++;

		((struct list21_type *) push)->index = i;

		if (flat < size) {
		    flat = size;
		}
	    }
	}

	if (status) {
	} else {
	    flat++;

	    line = (char *) malloc(flat << 0);
	    if (line) {
	    } else {
		status = -1;
	    }
	}

	if (status) {
	    x1f4_fini_fslist(&list);
	} else {
	    EXCESS(context).e1.list81.window = list;
	    EXCESS(context).e1.list81.wynona = line;

	    EXCESS(context).e1.list81.max = flat;

	    EXCESS(context).e1.list82.max = flat;

	    EXCESS(context).e1.list87.count = argc;

	    if (do_shuffle) {
		x1f4_rand_fslist(list);
		if (1) {
		    FLAGS(context) |= Mylene1f4_TILED_LIST;
		}
	    }
	}
    }

    return status;
}


int
Mylene1f4_read_list(void *context, const char *path, int do_shuffle)
{
    int fd, status;

    fd = open(path, O_RDONLY);
    if (fd == -1) {
	Mylene1f4_fprintf_error
	    (context, "%s: cannot open `%s': %s\n", Mylene1f4(context), path,
	     strerror(errno));
	status = -1;
    } else {
	status = read_list(context, path, fd, do_shuffle);
	if (0 < status) {
	    /*
	     * TODO
	     */
	}

	close(fd);
    }

    return status;
}


int
Mylene1f4_sand_list(void *context)
{
    unsigned index;

    x1f4_rand_fslist(EXCESS(context).e1.list81.window);

    x1f4_tile_fslist(&index, EXCESS(context).e1.list87.lock);

    FLAGS(context) |= Mylene1f4_TILED_LIST;

    EXCESS(context).e1.list87.clear = EXCESS(context).e1.list87.count - index;

    if (LOAD_ORDER(context) == Mylene1f4_LOAD_LAST) {
	Mylene1f4_half_list(context);
    }

    return 0;
}


int
Mylene1f4_seek_list(void *context, unsigned seek)
{
    int status;

    do {
	void *base, *line, *node, *peek, *post, *slip;

	line = EXCESS(context).e1.list81.wynona;

	if (EXCESS(context).e1.list81.max < EXCESS(context).e1.list82.max) {
	    line = (char *) realloc(line, EXCESS(context).e1.list82.max << 0);
	    if (line) {
		EXCESS(context).e1.list81.wynona = line;

		EXCESS(context).e1.list81.max = EXCESS(context).e1.list82.max;
	    } else {
		status = -1;
		if (1) {
		    break;
		}
	    }
	}

	x1f4_peek_fslist
	    (EXCESS(context).e1.list81.window, seek, &peek, &post, &node,
	     &base);

	slip = line;
	x1f4_type_fslist(node, &slip);
	strcpy(slip, base);

	EXCESS(context).e1.list87.lock = peek;

	EXCESS(context).e1.list87.file_name = line;

	if (1) {
	    EXCESS(context).e1.list87.index =
		((struct list21_type *) peek)->index;
	}

	status = 0;
    } while (0);

    return status;
}


int
Mylene1f4_slip_list(void *context)
{
    int status;
    unsigned clear;

    clear = EXCESS(context).e1.list87.clear;

    clear--;

    if (clear) {
	if (LOAD_ORDER(context) == Mylene1f4_LOAD_LAST) {
	    status = Mylene1f4_tail_list(context, 1);
	} else {
	    status = Mylene1f4_head_list(context, 1);
	}
	if (status) {
	} else {
	    EXCESS(context).e1.list87.clear = clear;
	}
    } else {
	status = 0;
	if (1) {
	    EXCESS(context).e1.list87.clear = clear;
	}
    }

    return status;
}


int
Mylene1f4_tail_list(void *context, unsigned this)
{
    unsigned index;

    Mylene1f4_fix_index(context, &index);

    return Mylene1f4_seek_list(context, index - this);
}
