/*
 * list2.c
 * Copyright (C) 2007-2011, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <stddef.h>
#if __ENABLE_TAGGED_LIST__
# include <string.h>
#endif				/* __ENABLE_TAGGED_LIST__ */

#include <access.h>
#include <defs.h>
#include <fslist.h>
#include <inter.h>
#include <list.h>

#define __BACK_SEEK__			1
#define __BETA_SEEK__			2

#define __e151_SEEK__			__BACK_SEEK__

#define ____e151_SEEK__			__e151_SEEK__

static int case_e103(void *, int *, void *);
static int case_e151(void *, int *, void *);
static int fast_list(void *);
static int flat_deck(void *, void *);
static int line_deck(void *, unsigned *, unsigned, unsigned);
#if __ENABLE_TAGGED_LIST__
static int line_list(void *, unsigned, unsigned, unsigned *, unsigned, void *);
static int line_tags(unsigned *, unsigned *, void *, unsigned, unsigned);
#endif				/* __ENABLE_TAGGED_LIST__ */
static int lock_list(void *, unsigned);
static int pick_list(unsigned);
static int push_list(void *, unsigned, unsigned, unsigned, void *);

static int
case_e103(void *context, int *class, void *rank)
{
    int status;
    unsigned index;

    index = *(unsigned *) rank;

    if (!(index < EXCESS(context).e1.list87.count)) {
	status = 0;
    } else {
	status = Mylene1f4_ceil_list(context, class, index, NULL);
    }

    return status;
}


static int
case_e151(void *context, int *class, void *rank)
{
    int status;
    unsigned index;

    if (FLAGS(context) & Mylene1f4_TILED_LIST) {
	unsigned count;
#if ____e151_SEEK__ == __BACK_SEEK__
	void *peek;
#endif				/* ____e151_SEEK__ == __BACK_SEEK__ */
	void *window;

	window = EXCESS(context).e1.list81.window;

	index = *(unsigned *) rank;

	count = EXCESS(context).e1.list87.count;
#if ____e151_SEEK__ == __BETA_SEEK__
	while (1) {
	    if (count) {
		void *peek;

		count--;

		x1f4_beta_fslist(window, count, &peek);
		if (((struct list21_type *) peek)->index == index) {
		    index = count;
		    if (1) {
			break;
		    }
		}
	    } else {
		index = ~0;
		if (1) {
		    break;
		}
	    }
	}
#elif __e151_SEEK__ == __BACK_SEEK__
	count--;
	x1f4_beta_fslist(window, count, &peek);
	if (((struct list21_type *) peek)->index == index) {
	    index = count;
	} else {
	    while (1) {
		if (count) {
		    count--;

		    x1f4_back_fslist(&peek, peek);
		    if (((struct list21_type *) peek)->index == index) {
			index = count;
			if (1) {
			    break;
			}
		    }
		} else {
		    index = ~0;
		    if (1) {
			break;
		    }
		}
	    }
	}
#endif				/* ____e151_SEEK__ == __BETA_SEEK__ */
    } else {
	struct list21_type list21;

	list21.index = *(unsigned *) rank;

	if (x1f4_seek_fslist
	    (EXCESS(context).e1.list81.window, &list21, 0, flat_deck,
	     &index)) {
	} else {
	    index = ~0;
	}
    }

    if (!~index) {
	status = 0;
    } else {
	status = Mylene1f4_ceil_list(context, class, index, NULL);
    }

    return status;
}


static int
fast_list(void *context)
{
    if (LOAD_ORDER(context) != Mylene1f4_LOAD_LAST) {
	EXCESS(context).e1.list87.clear++;
    }

    return 0;
}


static int
flat_deck(void *flat, void *deck)
{
    return ((struct list21_type *) flat)->index
	- ((struct list21_type *) deck)->index;
}


static int
line_deck(void *context, unsigned *line, unsigned deck, unsigned post)
{
    int delete;
    unsigned count;

    count = EXCESS(context).e1.list87.count;
    if (deck) {
	if (count + 1 < post) {
	    delete = 0;
	} else {
	    delete = 1;
	    *line = count + 1 - post;
	}
    } else {
	if (count + 0 < post) {
	    delete = 0;
	} else {
	    delete = 1;
	    *line = post;
	}
    }

    return delete;
}


#if __ENABLE_TAGGED_LIST__
static int
line_list(void *context, unsigned index, unsigned post, unsigned *tags,
	  unsigned size, void *node)
{
    int status;
    unsigned offset;
    void *list, *push;

    list = EXCESS(context).e1.list81.window;

    if (1) {
	status = x1f4_push_fslist
	    (list, post, size, node, tags[0] ? tags[1] : 1, &offset, &push);
    } else {
    }
    if (status) {
    } else {
	((struct list21_type *) push)->index = index;

	EXCESS(context).e1.list87.count++;

	if (EXCESS(context).e1.list82.max < size + 1) {
	    EXCESS(context).e1.list82.max = size + 1;
	}

	if (tags[0]) {
	    unsigned i, *offset6s;

	    offset6s = ((struct list29_type *) push)->shift;

	    memcpy((char *) push + offset, (char *) node + size + 1, tags[1]);

	    offset6s[0] = tags[0];

	    offset6s++;

	    tags++;

	    i = 7;
	    for (; i; i--) {
		*offset6s++ = *tags++ + offset;
	    }
	} else {
	    unsigned i, *offset6s;

	    offset6s = ((struct list29_type *) push)->shift;

	    *((char *) push + offset) = 0;

	    offset6s[0] = 0;

	    offset6s[1] = offset;

	    offset++;

	    offset6s += 2;

	    i = 6;
	    for (; i; i--) {
		*offset6s++ = offset;
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_TAGGED_LIST__ */


#if __ENABLE_TAGGED_LIST__
static int
line_tags(unsigned *tags, unsigned *flat, void *node, unsigned size,
	  unsigned bits)
{
    int status;

    if (TAGGED_LIST_RECORD_SIZE < size) {
	status = 33;
    } else {
	if (1) {
	    char *path;

	    path = node;
	    if (!*path) {
		status = 1;
	    } else {
		char *miss;
		unsigned deck, i = 6, offset = 0;

		path[size] = 0;

		deck = strlen(path);

		miss = path + deck + 1;

		size -= deck;

		bits &= 63;

		tags += 7;

		while (i) {
		    i--;
		    if (!(bits & (1 << i))) {
			tags[0] = offset;
		    } else {
			if (!(offset < size)) {
			    break;
			} else {
			    if (!miss[0]) {
				break;
			    } else {
				unsigned length;

				tags[0] = offset;

				length = strlen(miss) + 1;

				offset += length;

				miss += length;
			    }
			}
		    }

		    tags--;
		}

		if (i) {
		    status = 1;
		} else {
		    status = 0;

		    *flat = deck;

		    tags[0] = size;

		    *(tags - 1) = bits;
		}
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_TAGGED_LIST__ */


static int
lock_list(void *context, unsigned class)
{
    if (LOAD_ORDER(context) == Mylene1f4_LOAD_LAST) {
	unsigned clear;

	clear = EXCESS(context).e1.list87.clear;
	if (class < clear) {
	    EXCESS(context).e1.list87.clear = clear + 1;
	}
    } else {
	unsigned clear;

	clear = EXCESS(context).e1.list87.clear;
	if (EXCESS(context).e1.list87.count - clear < class) {
	    EXCESS(context).e1.list87.clear = clear + 1;
	}
    }

    return 0;
}


static int
pick_list(unsigned size)
{
    int status;

    if (SIMPLE_LIST_RECORD_SIZE < size) {
	status = 33;
    } else {
	status = 0;
    }

    return status;
}


static int
push_list(void *context, unsigned index, unsigned post, unsigned size,
	  void *node)
{
    int status;
    unsigned offset;
    void *list, *push;

    list = EXCESS(context).e1.list81.window;

    *((char *) node + size) = 0;

#if __ENABLE_TAGGED_LIST__
    if (FLAGS(context) & Mylene1f4_BREAK_LIST) {
	status = x1f4_push_fslist(list, post, size, node, 1, &offset, &push);
    } else {
#endif				/* __ENABLE_TAGGED_LIST__ */
	status = x1f4_push_fslist(list, post, size, node, 0, &offset, &push);
#if __ENABLE_TAGGED_LIST__
    }
#endif				/* __ENABLE_TAGGED_LIST__ */
    if (status) {
    } else {
	((struct list21_type *) push)->index = index;

	EXCESS(context).e1.list87.count++;

	if (EXCESS(context).e1.list82.max < size + 1) {
	    EXCESS(context).e1.list82.max = size + 1;
	}

#if __ENABLE_TAGGED_LIST__
	if (FLAGS(context) & Mylene1f4_BREAK_LIST) {
	    unsigned i, *offset6s;

	    offset6s = ((struct list29_type *) push)->shift;

	    *((char *) push + offset) = 0;

	    offset6s[0] = 0;

	    offset6s[1] = offset;

	    offset++;

	    offset6s += 2;

	    i = 6;
	    for (; i; i--) {
		*offset6s++ = offset;
	    }
	}
#endif				/* __ENABLE_TAGGED_LIST__ */
    }

    return status;
}


int
Mylene1f4_case_list(void *context, int *class, unsigned deck, void *rank)
{
    int status;

    switch (deck) {
    case 0:
	status = case_e103(context, class, rank);
	break;
    default:
	status = case_e151(context, class, rank);
    }

    return status;
}


int
Mylene1f4_ceil_list(void *context, int *class, unsigned index, void *delete)
{
    int status;
    unsigned track;

    Mylene1f4_fix_index(context, &track);
    if (index == track) {
	*class = 1;

	status = 0;
    } else {
	status = x1f4_miss_fslist(EXCESS(context).e1.list81.window, index);
	if (status) {
	} else {
	    unsigned count;

	    *class = 0;

	    count = EXCESS(context).e1.list87.count;

	    if (LOAD_ORDER(context) != Mylene1f4_LOAD_LAST) {
		unsigned clear;

		clear = EXCESS(context).e1.list87.clear;
		if (count - clear < index) {
		    EXCESS(context).e1.list87.clear = clear - 1;
		}
	    } else {
		unsigned clear;

		clear = EXCESS(context).e1.list87.clear;
		if (index < clear) {
		    EXCESS(context).e1.list87.clear = clear - 1;
		}
	    }

	    count--;

	    EXCESS(context).e1.list87.count = count;
	}
    }

    return status;
}


int
Mylene1f4_clip_list(void *context, int *class, int erased, unsigned shift)
{
    unsigned clear, count;
    void *list;

    count = EXCESS(context).e1.list87.count;
    if (count < shift) {
	shift = count;
    }

    clear = EXCESS(context).e1.list87.clear;
    if (LOAD_ORDER(context) == Mylene1f4_LOAD_LAST) {
	clear = count - clear + 1;
    }

    list = EXCESS(context).e1.list81.window;

    if (shift < clear) {
	unsigned index;

	index = count;

	count -= shift;

	if (LOAD_ORDER(context) != Mylene1f4_LOAD_LAST) {
	    EXCESS(context).e1.list87.clear = clear - shift;
	}

	*class = 0;

	do {
	    index--;
	    x1f4_miss_fslist(list, index);
	} while (index != count);
    } else {
	unsigned index;

	index = count;

	if (erased) {
	    if (shift == count) {
	    } else {
		shift++;
	    }
	}

	count -= clear - 1;

	if (LOAD_ORDER(context) != Mylene1f4_LOAD_LAST) {
	    EXCESS(context).e1.list87.clear = 1;
	}

	*class = 1;

	while (count < index) {
	    index--;
	    x1f4_miss_fslist(list, index);
	}

	clear = shift - clear;
	if (clear) {
	    count -= clear;

	    if (LOAD_ORDER(context) == Mylene1f4_LOAD_LAST) {
		EXCESS(context).e1.list87.clear -= clear;
	    }

	    clear = count - 1;
	    index--;
	    do {
		index--;
		x1f4_miss_fslist(list, index);
	    } while (index != clear);
	}
    }

    EXCESS(context).e1.list87.count = count;

    return 0;
}


#if __ENABLE_TAGGED_LIST__
int
Mylene1f4_land_list(void *context, unsigned index, unsigned class,
		    unsigned deck, unsigned post, unsigned size, void *node)
{
    int status;
    unsigned flat, tags[8];

    status = line_tags(tags, &flat, node, size, class);
    if (status) {
    } else {
	unsigned class;

	if (line_deck(context, &class, deck, post)) {
	    status = line_list(context, index, class, tags, flat, node);
	    if (status) {
	    } else {
		lock_list(context, class);
	    }
	} else {
	    status = 0;
	}
    }

    return status;
}
#endif				/* __ENABLE_TAGGED_LIST__ */


#if __ENABLE_TAGGED_LIST__
int
Mylene1f4_line_list(void *context, unsigned index, unsigned class,
		    unsigned size, void *node)
{
    int status;
    unsigned flat, tags[8];

    status = line_tags(tags, &flat, node, size, class);
    if (status) {
    } else {
	unsigned class;

	x1f4_size_fslist(EXCESS(context).e1.list81.window, &class);
	if (1) {
	    status = line_list(context, index, class, tags, flat, node);
	    if (status) {
	    } else {
		fast_list(context);
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_TAGGED_LIST__ */


int
Mylene1f4_post_list(void *context, unsigned index, unsigned deck,
		    unsigned post, unsigned size, void *node)
{
    int status;

    status = pick_list(size);
    if (status) {
    } else {
	unsigned class;

	if (line_deck(context, &class, deck, post)) {
	    status = push_list(context, index, class, size, node);
	    if (status) {
	    } else {
		lock_list(context, class);
	    }
	} else {
	    status = 0;
	}
    }

    return status;
}


int
Mylene1f4_push_list(void *context, unsigned index, unsigned size, void *node)
{
    int status;

    status = pick_list(size);
    if (status) {
    } else {
	unsigned class;

	x1f4_size_fslist(EXCESS(context).e1.list81.window, &class);
	if (1) {
	    status = push_list(context, index, class, size, node);
	    if (status) {
	    } else {
		fast_list(context);
	    }
	}
    }

    return status;
}
