/*
 * misc.c
 * Copyright (C) 2005-2011, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <errno.h>
#if defined HAVE_LIBLEWD
# include <lewd.h>
#endif				/* HAVE_LIBLEWD */
#include <math.h>
#include <signal.h>
#include <stdarg.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <access.h>
#if !defined HAVE_LIBLEWD
# include <cardinal-wx.h>
# include <cardinal-x.h>
# include <ckeytree.h>
#endif				/* !HAVE_LIBLEWD */
#include <defs.h>
#if !defined HAVE_LIBLEWD
# include <integral-x.h>
#endif				/* !HAVE_LIBLEWD */
#include <inter.h>
#include <lcard.h>
#if !defined HAVE_LIBLEWD
# include <lcardinal.v-b.h>
# include <lcardinal.v-f.h>
# include <lcardinal.v-fb.h>
#endif				/* !HAVE_LIBLEWD */
#if !defined HAVE_LIBLEWD
# include <pcardinal.v-fb.h>
# include <vkeytree.h>
#endif				/* !HAVE_LIBLEWD */
#include <types.h>

#define __LCARD_DEFINITION		LCARD_DEFINITION

#if __ENABLE_x00696e6b__
# define G1_SECONDS(time)		((time).tv_sec)
# define G6_SECONDS(time)		((time).tv_usec)
#endif				/* __ENABLE_x00696e6b__ */

#define SECOND_A(v)			((v)->tv_sec)
#define SECOND_F(v)			((v)->tv_usec)

#define STRING(data)			((struct string_type *) (data))

#define SET_EXTIME(extime, H, M, S, X) \
    {									      \
	((struct extime_type *) (extime))->h = (H);			      \
	((struct extime_type *) (extime))->m = (M);			      \
	((struct extime_type *) (extime))->s = (S);			      \
	((struct extime_type *) (extime))->x = (X);			      \
    }

#if __ENABLE_x00696e6b__
# define SET_CARDINAL(buffer, base, card) \
    {									      \
	(buffer)[(base) + 0] = (card) >> 030;				      \
	(buffer)[(base) + 1] = (card) >> 020;				      \
	(buffer)[(base) + 2] = (card) >> 010;				      \
	(buffer)[(base) + 3] = (card) >> 000;				      \
    }
#endif				/* __ENABLE_x00696e6b__ */

#if __ENABLE_x00696e6b__
# define SET_CARDiNAL(buffer, base, card) \
    {									      \
	unsigned cardinal;						      \
									      \
	cardinal = card;						      \
	SET_CARDINAL(buffer, base, cardinal);				      \
    }
#endif				/* __ENABLE_x00696e6b__ */

int
Mylene1f4_close_connections(void *context)
{
    struct clause_type *clause_data;
    int status;

    clause_data = CLAUSE(context);
    if (!clause_data) {
	status = 0;
    } else {
	while (clause_data) {
	    status = clause_data->delete(context);
	    if (status) {
		break;
	    }

	    clause_data = clause_data->clause_data;
	}
    }

    return status;
}


#if __ENABLE_x00696e6b__
int
Mylene1f4_code_time(unsigned char *system)
{
    struct timeval time;

    gettimeofday(&time, NULL);
    SET_CARDiNAL(system, 0, G1_SECONDS(time));
    SET_CARDiNAL(system, 4, G6_SECONDS(time));

    return 0;
}
#endif				/* __ENABLE_x00696e6b__ */


int
Mylene1f4_find_sequel(void *context, unsigned code, void **sequel)
{
    int status = 0;
    struct sequel_type *sequel_data;

    sequel_data = SEQUEL(context);
    if (sequel_data) {
	if (sequel_data->code == code) {
	    *sequel = sequel_data;
	    status = 1;
	} else {
	    struct sequel_type *link_sequel;

	    link_sequel = sequel_data->sequel_data;
	    while (link_sequel) {
		if (link_sequel->code != code) {
		    sequel_data = link_sequel;
		    link_sequel = sequel_data->sequel_data;
		} else {
		    sequel_data->sequel_data = link_sequel->sequel_data;
		    link_sequel->sequel_data = SEQUEL(context);
		    SEQUEL(context) = link_sequel;
		    *sequel = link_sequel;
		    status = 1;

		    break;
		}
	    }
	}
    }

    return status;
}


int
Mylene1f4_fix_index(void *context, unsigned *c_index)
{
    unsigned index = 0;

    if (FLAGS(context) & Mylene1f4_LOCAL_LIST) {
	if (LOAD_ORDER(context) != Mylene1f4_LOAD_LAST) {
	    index = EXCESS(context).e1.list87.count
		- EXCESS(context).e1.list87.clear;
	} else {
	    index = EXCESS(context).e1.list87.clear - 1;
	}
    }

    *c_index = index;

    return 0;
}


int
Mylene1f4_fix_start(void *context, unsigned *c_start)
{
    unsigned start;

    if (SELECT(context) == Mylene1f4_PAGE_PICKUP) {
	start = START2(context);
    } else if (COUNT(context) != 1) {
	start = REFERENCE(context);
	/*
	 * this buggy case should go eventually, since it's here only to
	 * prevent the buggier case below to kick in...
	 *
	 * BUGS
	 *
	 */
    } else {
#if SIZEOF_UNSIGNED_LONG != 8
	double bias;
#endif					/* SIZEOF_UNSIGNED_LONG != 8 */
	struct extime_type *extime_data;
	unsigned second;
#if SIZEOF_UNSIGNED_LONG == 8
	unsigned long bias;
#endif					/* SIZEOF_UNSIGNED_LONG == 8 */

	extime_data = &START6(context);
	second = SAMPLING(context) * CHANNELS(context) << 1;
	bias = (extime_data->h * 3600 + extime_data->m * 60 + extime_data->s
		+ extime_data->x / 60) * second
	    + extime_data->x % 60 * second / 60;
	start = bias / RELEASE(context);
    }

    *c_start = start;

    return 0;
}


int
Mylene1f4_fprintc_error(void *context, const char *data, unsigned size)
{
    return Mylene1f4_fprintf_error(context, "%.*s", (int) size, data);
}


int
Mylene1f4_fprintf_error(void *context, const char *format, ...)
{
    va_list ap;

    va_start(ap, format);
    EPRINTF(context) (context, format, ap);
    va_end(ap);

    return 0;
}


int
Mylene1f4_fprintf_offset(void *context, void *offset)
{
    Mylene1f4_vprintf_offset
	(context, Mylene1f4_fprintc_error, 8, 0, 16, offset);
    Mylene1f4_fprintf_error(context, " (");
    Mylene1f4_vprintf_offset
	(context, Mylene1f4_fprintc_error, 0, 3, 10, offset);
    Mylene1f4_fprintf_error(context, ")");

    return 0;
}


int
Mylene1f4_line_reference(void *context, unsigned *state)
{
    int (*line) (void *, unsigned *), status;

    line = L1f4(context);
    if (line) {
	status = line(context, state);
    } else {
	status = 0;
	*state = REFERENCE(context);
    }

    return status;
}


int
Mylene1f4_near_miss(void *context)
{
    int status;

    if (!(FLAGS(context) & Mylene1f4_ERROR_MISS)) {
	status = 1;
    } else {
	status = 0;
    }

    return status;
}


int
Mylene1f4_parse_cardinal(void *context, const char *text,
			 const char *specification, unsigned *c_cardinal)
{
    int status;
    unsigned cardinal;

    status = x1f4_parse_wxcardinal(&cardinal, specification, NULL, 0);
    if (status) {
	Mylene1f4_fprintf_error
	    (context, "%s: cannot parse %s specification: `%s'\n",
	     Mylene1f4(context), text, specification);
    } else {
	*c_cardinal = cardinal;
    }

    return status;
}


int
Mylene1f4_parse_ctreekey(void *context, const char *text, const void *ctkey,
			 const char *specification, unsigned *c_cardinal)
{
    int status;

    status = x1f4_find_ctkey(specification, ctkey, c_cardinal);
    if (status) {
	Mylene1f4_fprintf_error
	    (context, "%s: cannot parse %s specification: `%s'\n",
	     Mylene1f4(context), text, specification);
    }

    return status;
}


int
Mylene1f4_parse_integral(void *context, const char *text,
			 const char *specification, int *c_integral)
{
    int integral, status;

    status = x1f4_parse_xintegral(&integral, specification, NULL, 0);
    if (status) {
	Mylene1f4_fprintf_error
	    (context, "%s: cannot parse %s specification: `%s'\n",
	     Mylene1f4(context), text, specification);
    } else {
	*c_integral = integral;
    }

    return status;
}


int
Mylene1f4_parse_positive(void *context, const char *text,
			 const char *specification, unsigned *c_positive)
{
    int status;
    unsigned positive;

    status = x1f4_parse_wxcardinal(&positive, specification, NULL, 0);
    if (status) {
	Mylene1f4_fprintf_error
	    (context, "%s: cannot parse %s specification: `%s'\n",
	     Mylene1f4(context), text, specification);
    } else if (!positive) {
	Mylene1f4_fprintf_error
	    (context, "%s: %s must be not null\n", Mylene1f4(context), text);
    } else {
	status = 0;
	*c_positive = positive;
    }

    return status;
}


int
Mylene1f4_parse_time(void *context, const char *text,
		     const char *specification, void *extime)
{
    const char *s, *t;
    int status;
    unsigned a;

    s = specification;
    status = x1f4_parse_wxcardinal(&a, s, &t, 0);
    if (!status) {
	SET_EXTIME(extime, 0, 0, a, 0);
    } else if (s != t) {
	if (*t == ',') {
	    unsigned d;

	    status = x1f4_parse_xcardinal(&d, t + 1, NULL, 0);
	    if (!status) {
		SET_EXTIME(extime, 0, 0, a, d);
	    }
	} else if (*t == ':') {
	    unsigned b;

	    s = t + 1;
	    status = x1f4_parse_xcardinal(&b, s, &t, 0);
	    if (!status) {
		SET_EXTIME(extime, 0, a, b, 0);
	    } else if (s != t) {
		if (*t == ',') {
		    unsigned d;

		    status = x1f4_parse_xcardinal(&d, t + 1, NULL, 0);
		    if (!status) {
			SET_EXTIME(extime, 0, a, b, d);
		    }
		} else if (*t == ':') {
		    unsigned c;

		    s = t + 1;
		    status = x1f4_parse_xcardinal(&c, s, &t, 0);
		    if (!status) {
			SET_EXTIME(extime, a, b, c, 0);
		    } else if (s != t) {
			if (*t == ',') {
			    unsigned d;

			    status = x1f4_parse_xcardinal(&d, t + 1, NULL, 0);
			    if (!status) {
				SET_EXTIME(extime, a, b, c, d);
			    }
			}
		    }
		}
	    }
	}
    }

    if (status) {
	Mylene1f4_fprintf_error
	    (context, "%s: cannot parse %s specification: `%s'\n",
	     Mylene1f4(context), text, specification);
    }

    return status;
}


int
Mylene1f4_parse_vtreekey(void *context, const char *text, const void *vtkey,
			 const char *specification, const void **c_vtreekey)
{
    int status;

    status = x1f4_find_vtkey(specification, vtkey, c_vtreekey);
    if (status) {
	Mylene1f4_fprintf_error
	    (context, "%s: cannot parse %s specification: `%s'\n",
	     Mylene1f4(context), text, specification);
    }

    return status;
}


int
Mylene1f4_post_time(void *context, unsigned reference)
{
    double second_a, second_f;
    struct timeval *atime;

    second_a = reference;
    second_a *= RELEASE(context);
    second_a /= CHANNELS(context) * SAMPLING(context) << 1;
    second_f = modf(second_a, &second_a);
    atime = ATIME(context);
    SECOND_A(atime) = second_a;
    SECOND_F(atime) = second_f * 1000000;

    return 0;
}


int
Mylene1f4_remove_sequel(void *context, unsigned sequel)
{
    struct sequel_type *parent_data, *sequel_data;

    sequel_data = SEQUEL(context);
    parent_data = sequel_data;
    while (sequel_data) {
	if (sequel_data->code != sequel) {
	    parent_data = sequel_data;
	    sequel_data = sequel_data->sequel_data;
	} else {
	    if (sequel_data == parent_data) {
		SEQUEL(context) = sequel_data->sequel_data;
	    } else {
		parent_data->sequel_data = sequel_data->sequel_data;
	    }

	    break;
	}
    }

    return 0;
}


int
Mylene1f4_scale_track(void *context)
{
    STREAMS(context)++;
    STREAMS(context) |= 1u << 31;

    EXTRA(context) &= ~Mylene1f4_OVER_TRACK;

    return 0;
}


int
Mylene1f4_shift_track(void *context)
{
    EXTRA(context) |= Mylene1f4_OVER_TRACK;

    return 0;
}


int
Mylene1f4_shrink_field(const char **string_in_out, unsigned *length_in_out)
{
    const char *string;
    unsigned length;

    string = *string_in_out;
    length = *length_in_out;

    for (; length; length--) {
	unsigned c;

	c = *string;
	if (32 < c && c < 127) {
	    break;
	} else {
	    string++;
	}
    }

    if (!length) {
	*length_in_out = 0;
    } else {
	*length_in_out = length;
	*string_in_out = string;

	for (; length; length--) {
	    unsigned c;

	    c = *string;
	    if (31 < c && c < 127) {
		string++;
	    } else {
		break;
	    }
	}
	for (;; length++) {
	    unsigned c;

	    c = *--string;
	    if (c != 32) {
		break;
	    }
	}

	*length_in_out -= length;
    }

    return 0;
}


int
Mylene1f4_stat_exit(void *context, int record)
{
    if (WIFSIGNALED(record)) {
	char *name;
	int signal;

	signal = WTERMSIG(record);
	Mylene1f4_fprintf_error(context, " was killed by");
	if (!Mylene1f4_try_signals(signal, &name)) {
	    Mylene1f4_fprintf_error(context, " %s signal\n", name);
	} else {
	    Mylene1f4_fprintf_error(context, " signal %d\n", signal);
	}
    } else {
	Mylene1f4_fprintf_error(context, " has exited");
	if (WIFEXITED(record)) {
	    Mylene1f4_fprintf_error(context, " %d", WEXITSTATUS(record));
	}
	if (1) {
	    Mylene1f4_fprintf_error(context, " unexpectedly\n");
	}
    }

    return 0;
}


int
Mylene1f4_strip_graphique(void *context, void *graphique)
{
    const char *slack, *strip;
    struct string_type *string_data;

    string_data = graphique;

    slack = string_data->data;
    strip = slack;
    while (*strip == '*') {
	strip++;
    }
    if (!*strip && strip != slack) {
	string_data->data = NULL;
    } else {
	unsigned count;

	count = strlen(slack);
	if (!(SLACK(context).size < count)) {
	    string_data->size = count;
	} else {
	    void *slack;

	    slack = malloc(count);
	    if (!slack) {
		string_data->data = NULL;
	    } else {
		void *strip;

		string_data->size = count;

		strip = SLACK(context).data;
		SLACK(context).data = slack;
		SLACK(context).size = count;

		if (strip) {
		    free(strip);
		}
	    }
	}
    }

    return 0;
}


#if __ENABLE_FILTER__
int
Mylene1f4_swap_filter(void *context, void *minor)
{
    void *data, *swap;

    data = E4DECK(context);
    swap = minor;
    if (data != swap) {
	E4DECK(context) = swap;
	if (data) {
	    Mylene1f4_fini_filter(context, &data);
	}
    }

    return 0;
}
#endif				/* __ENABLE_FILTER__ */


int
Mylene1f4_swap_graphique(void *major, void *minor)
{
    const char *data, *swap;

    data = STRING(major)->data;
    swap = STRING(minor)->data;
    if (data != swap) {
	STRING(major)->data = swap;
	STRING(major)->size = STRING(minor)->size;
	if (data) {
	    free((char *) data);
	}
    }

    return 0;
}


#if __ENABLE_CONTROLLING_SERVER__
int
Mylene1f4_swap_progress(void *context, void *major, void *minor)
{
    void *data, *swap;

    data = *(void **) major;
    swap = minor;
    if (data != swap) {
	*(void **) major = swap;
	if (data) {
	    Mylene1f4_fini_format(context, &data);
	}
    }

    return 0;
}
#endif				/* __ENABLE_CONTROLLING_SERVER__ */


int
Mylene1f4_vprintf_offset(void *data,
			 int (*this) (void *, const char *, unsigned),
			 unsigned f, unsigned b, unsigned base, void *cardinal)
{
    int status;
#if __LCARD_DEFINITION != LONG_UNSIGNED_DEFINITION
    lcard_type offset;
#endif				/* __LCARD_DEFINITION != LONG_UNSIGNED_DE... */

#if __LCARD_DEFINITION == LONG_LONG_UNSIGNED_DEFINITION
    offset = *(lcard_type *) cardinal;
    status = x1f4_vprint_fbpcardinal
	(data, this, 0, f, b, base, offset >> 32, offset);
#elif LCARD_DEFINITION == LONG_UNSIGNED_DEFINITION
    status = x1f4_vprint_fblcardinal
	(data, this, 0, f, b, base, *(unsigned long *) cardinal);
#else
    offset = *(lcard_type *) cardinal;
    status = x1f4_vprint_fbpcardinal
	(data, this, 0, f, b, base, offset.major, offset.minor);
#endif				/* __LCARD_DEFINITION != LONG_LONG_UNSIGN... */

    return status;
}


int
Mylene1f4_wait_second(void *context, pid_t *miss)
{
    int status = 0;

    if (0) {
	*miss = -1;
    } else {
	pid_t copy;
	struct timeval fast;
	unsigned mind;

	copy = *miss;

	gettimeofday(&fast, NULL);

	fast.tv_sec += 8;

	do {
	    pid_t line;

	    line = waitpid(copy, NULL, WNOHANG);
	    if (line == copy) {
		*miss = -1;

		break;
	    } else {
		if (line == -1) {
		    if (errno == ECHILD) {
			*miss = -1;

			break;
		    } else {
			if (errno == EINTR) {
			} else {
			    status = -1;
			    if (1) {
				break;
			    }
			}
		    }
		}

		if (1) {
		    struct timeval post;

		    gettimeofday(&post, NULL);
		    if (fast.tv_sec < post.tv_sec) {
			break;
		    } else {
			unsigned second;

			second = fast.tv_sec - post.tv_sec;
			if (fast.tv_usec < post.tv_usec) {
			    if (second) {
			    } else {
				if (1) {
				    break;
				}
			    }
			}

			mind = fast.tv_usec - post.tv_usec + second * 1000000;
		    }
		}
	    }
	} while (usleep(mind));
    }

    return status;
}
