/*
 * mode.c
 * Copyright (C) 2007-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * TODO
 *
 *.add sampling frequency
 *.limit the number of band (in mode1.c) to some sesible value (144?)
 */

#include <config.h>

#if __ENABLE_MPEG_EQUALIZER__
# include <e1f4.h>
# include <math.h>
# include <stdlib.h>
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
# include <access.h>
# include <inter.h>
# include <types.h>
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
# define __DUMP_BANDS__			0
# define __DUMP_SCALE__			0
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
# if !defined x1f4_2v010
#  define x1f4_2v010			.00390625
# endif				/* !x1f4_2v010 */
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
# define ZERO_OR_CLOSE_ENOUGH		x1f4_2v010
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
# define INCREMENT(scale)		((Mylene1f4_real *) (scale) + 1)
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
static void fix_spline(double *, struct l3band_type *, unsigned, double);
static void ground_scale(void *, Mylene1f4_real *);
static void init_spline(struct l3band_type *, unsigned);
static void scale_shift(void *, Mylene1f4_real *);
static void spline_bands(void *, Mylene1f4_real *);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
static void
fix_spline(double *spline, struct l3band_type *l3band_data, unsigned n,
	   double x)
{
    double a, b;

    b = 1 - x;
    a = x - 0;
    *spline = (a * a * (2 * a - 3) + 1) * l3band_data->l3band_data->level
	+ (b * b * (2 * b - 3) + 1) * (l3band_data + 1)->l3band_data->level;
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if __ENABLE_MPEG_EQUALIZER__
static void
ground_scale(void *context, Mylene1f4_real *scale)
{
    double ground;
    unsigned i;

    ground = pow(2, MPEG(context).amplification / 10);

    for (i = 144; i; i--) {
	*scale++ = ground;
	*scale++ = ground;
	*scale++ = ground;
	*scale++ = ground;
    }

# if __DUMP_SCALE__
    scale -= 576;
    for (i = 144; i; i--) {
	fprintf(stderr, " %15.7f %15.7f %15.7f %15.7f\n", *scale, scale[1],
		scale[2], scale[3]);
	scale += 4;
    }
# endif				/* __DUMP_SCALE__ */
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if __ENABLE_MPEG_EQUALIZER__
static void
init_spline(struct l3band_type *l3band_data, unsigned count)
{
# if __DUMP_BANDS__
    unsigned i;
# endif				/* __DUMP_BANDS__ */

# if __DUMP_BANDS__
    for (i = 0; i < count; i++) {
	fprintf(stderr, " %15.7f %15.7f\n",
		l3band_data->l3band_data->frequency,
		l3band_data->l3band_data->level);

	l3band_data++;
    }
# endif				/* __DUMP_BANDS__ */
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if __ENABLE_MPEG_EQUALIZER__
static void
scale_shift(void *scale, Mylene1f4_real *shift)
{
    unsigned i = 144;

    for (; i; i--) {
	*shift = *shift * *(Mylene1f4_real *) scale;
	shift++;
	scale = INCREMENT((Mylene1f4_real *) scale);
	*shift = *shift * *(Mylene1f4_real *) scale;
	shift++;
	scale = INCREMENT((Mylene1f4_real *) scale);
	*shift = *shift * *(Mylene1f4_real *) scale;
	shift++;
	scale = INCREMENT((Mylene1f4_real *) scale);
	*shift = *shift * *(Mylene1f4_real *) scale;
	shift++;
	scale = INCREMENT((Mylene1f4_real *) scale);
    }
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if __ENABLE_MPEG_EQUALIZER__
static void
spline_bands(void *context, Mylene1f4_real *scale)
{
    double amplification;
    unsigned count, i, l;
    struct l3band_type *l3band_data;

    amplification = MPEG(context).amplification;

    l3band_data = MPEG(context).l3band_data;

    count = MPEG(context).count;

    init_spline(l3band_data, count);

    l = l3band_data->l3band_data->frequency * 1152 / 44100;
    if (!(l < 576)) {
	for (i = 576; i; i--) {
	    *scale++ = 1;
	}
    } else {
	if (l) {
	    double s;
	    unsigned j;

	    s = pow(2, (l3band_data->l3band_data->level + amplification) / 10);
	    for (j = 0; j < l; j++) {
		*scale++ = s;
	    }
	}
	for (i = 0; i < count - 1; i++) {
	    unsigned j, v;

	    v = (l3band_data + 1)->l3band_data->frequency * 1152 / 44100;
	    if (!(v < 576)) {
		v = 576;
	    }

	    for (j = l; j < v; j++) {
		double spline;

		fix_spline(&spline, l3band_data, count,
			   ((double) (j - l) * ((double) 1 / (v - l))));
		if (0) {
		    if (spline < MPEG_MIN_LEVEL) {
			spline = MPEG_MIN_LEVEL;
		    } else {
			if (MPEG_MAX_LEVEL < spline) {
			    spline = MPEG_MAX_LEVEL;
			}
		    }
		}
		*scale++ = pow(2, (spline + amplification) / 10);
	    }

	    l = v;

	    l3band_data++;
	}
	if (l) {
	    double s;
	    unsigned j;

	    s = pow(2, (l3band_data->l3band_data->level + amplification) / 10);
	    for (j = l; j < 576; j++) {
		*scale++ = s;
	    }
	} else {
	    for (i = 576; i; i--) {
		*scale++ = 1;
	    }
	}
    }

# if __DUMP_SCALE__
    scale -= 576;
    for (i = 144; i; i--) {
	fprintf(stderr, " %15.7f %15.7f %15.7f %15.7f\n", *scale, scale[1],
		scale[2], scale[3]);
	scale += 4;
    }
# endif				/* __DUMP_SCALE__ */
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if __ENABLE_MPEG_EQUALIZER__
int
Mylene1f4_air_mode(void *context)
{
    void *data;

    data = MPEG(context).l3band_data;
    if (data) {
	free(data);
    }

    return 0;
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if __ENABLE_MPEG_EQUALIZER__
int
Mylene1f4_day_mode(void *data)
{
    free(data);

    return 0;
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if __ENABLE_MPEG_EQUALIZER__
int
Mylene1f4_add_mode(void *context, double class)
{
    int status;

    if (class < MPEG_MIN_SCALE) {
	status = 1;
    } else {
	if (MPEG_MAX_SCALE < class) {
	    status = 1;
	} else {
	    status = 0;

	    MPEG(context).amplification = class;
	}
    }

    return status;
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if __ENABLE_MPEG_EQUALIZER__
int
Mylene1f4_fix_mode(void *context, unsigned index, double class)
{
    int status;
    struct l3band_type *l3band_data;

    l3band_data = MPEG(context).l3band_data;
    if (1) {
	if (!(index < MPEG(context).count)) {
	    status = 1;
	} else {
	    if (class < MPEG_MIN_CLASS) {
		status = 1;
	    } else {
		if (MPEG_MAX_CLASS < class) {
		    status = 1;
		} else {
		    double Class, f;
		    unsigned count, i;

		    /*
		     * TODO
		     *
		     *.replace the very stupid way of checking that the new
		     * frequency is not too close to an already existing one
		     */

		    Class = l3band_data[index].frequency;

		    count = MPEG(context).count;

		    l3band_data[index].frequency = class;

		    Mylene1f4_sort_bands(l3band_data, count);

		    f = l3band_data->l3band_data->frequency;
		    l3band_data++;

		    i = count - 1;
		    for (; i; i--) {
			double h;

			h = l3band_data->l3band_data->frequency;
			if (h - f < 1) {
			    break;
			} else {
			    f = h;
			    l3band_data++;
			}
		    }

		    if (!i) {
			status = 0;
		    } else {
			status = 1;

			l3band_data -= count - 1 - i + 1;

			l3band_data[index].frequency = Class;

			Mylene1f4_sort_bands(l3band_data, count);
		    }
		}
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if __ENABLE_MPEG_EQUALIZER__
int
Mylene1f4_nay_mode(void *context)
{
    int mode = 0;
    struct l3band_type *l3band_data;
    unsigned count;

    l3band_data = MPEG(context).l3band_data;
    count = MPEG(context).count;
    for (; count; count--) {
	double level;

	level = l3band_data->level;

	l3band_data++;

	if (-ZERO_OR_CLOSE_ENOUGH < level
	    && level < ZERO_OR_CLOSE_ENOUGH) {
	} else {
	    mode++;
	}
    }

    MPEG(context).mode = mode;

    return 0;
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if __ENABLE_MPEG_EQUALIZER__
int
Mylene1f4_set_mode(void *context, unsigned index, double level)
{
    int status;
    struct l3band_type *l3band_data;

    l3band_data = MPEG(context).l3band_data;
    if (1) {
	if (!(index < MPEG(context).count)) {
	    status = 1;
	} else {
	    if (level < MPEG_MIN_LEVEL) {
		status = 1;
	    } else {
		if (MPEG_MAX_LEVEL < level) {
		    status = 1;
		} else {
		    status = 0;

		    if (-ZERO_OR_CLOSE_ENOUGH < level
			&& level < ZERO_OR_CLOSE_ENOUGH) {
			l3band_data = l3band_data + index;
			if (l3band_data->level) {
			    l3band_data->level = 0;
			    MPEG(context).mode--;
			}
		    } else {
			l3band_data = l3band_data + index;
			if (l3band_data->level) {
			} else {
			    MPEG(context).mode++;
			}
			{
			    l3band_data->level = level;
			}
		    }
		}
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if __ENABLE_MPEG_EQUALIZER__
int
Mylene1f4_tee_mode(void *context)
{
    int status = 0;
    struct l3band_type *l3band_data;

    l3band_data = (struct l3band_type *)
	malloc(sizeof(struct l3band_type) * 10);
    if (!l3band_data) {
	status = -1;
    } else {
	unsigned i = 5;

	status = 0;

	MPEG(context).l3band_data = l3band_data;

	MPEG(context).count = 10;

	for (; i < 15; i++) {
	    l3band_data->frequency = 1 << i;
	    l3band_data->l3band_data = l3band_data;
	    l3band_data->level = 0;
	    l3band_data++;
	}
    }

    return status;
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if __ENABLE_MPEG_EQUALIZER__
int
Mylene1f4_way_mode(void *context, void **l3, unsigned sampling, void *e1f4)
{
    Mylene1f4_real *scale;
    int status;

    scale = (Mylene1f4_real *) malloc(sizeof(Mylene1f4_real) * 576);
    if (!scale) {
	status = -1;
    } else {
	Mylene1f4_audio_attributes_type attributes;

	status = 0;

	*l3 = scale;

	attributes.equalize_0 = scale_shift;
	attributes.equalizer_0 = scale;

	Mylene1f4_set_audio_attributes
	    (e1f4, &attributes,
	     Mylene1f4_AUDIO_EQUALIZER_0_MASK
	     | Mylene1f4_AUDIO_EQUALIZE_0_MASK, 0);

	if (MPEG(context).mode) {
	    spline_bands(context, scale);
	} else {
	    if (MPEG(context).amplification) {
		ground_scale(context, scale);
	    }
	}
    }

    return status;
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
