/*
 * mode2.c
 * Copyright (C) 2007-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if __ENABLE_MPEG_EQUALIZER__
# include <types.h>
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
# define MTHRESHOLD			4
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
static void ceil_bands(struct l3band_type *, unsigned);
static void cell_bands(struct l3band_type *, unsigned);
static void link_bands(struct l3band_type *, unsigned);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
static void
ceil_bands(struct l3band_type *l3band_data, unsigned count)
{
    while (1) {
	struct l3band_type *l3band_deck, *l3band_half, *l3band_miss;

	l3band_deck = l3band_data;
	l3band_miss = l3band_data + count - 1;

	l3band_half = l3band_data + (count >> 1);

	if (l3band_half->l3band_data->frequency
	    < l3band_deck->l3band_data->frequency) {
	    struct l3band_type *l3band_swap___;

	    l3band_swap___ = l3band_deck->l3band_data;
	    l3band_deck->l3band_data = l3band_half->l3band_data;
	    l3band_half->l3band_data = l3band_swap___;
	}
	if (l3band_miss->l3band_data->frequency
	    < l3band_half->l3band_data->frequency) {
	    struct l3band_type *l3band_swap___;

	    l3band_swap___ = l3band_half->l3band_data;
	    l3band_half->l3band_data = l3band_miss->l3band_data;
	    l3band_miss->l3band_data = l3band_swap___;
	    if (l3band_half->l3band_data->frequency
		< l3band_deck->l3band_data->frequency) {
		l3band_swap___ = l3band_deck->l3band_data;
		l3band_deck->l3band_data = l3band_half->l3band_data;
		l3band_half->l3band_data = l3band_swap___;
	    }
	} else {
	}

# define l3band_HALF			l3band_half

	l3band_HALF = l3band_half->l3band_data;

	l3band_deck++;
	l3band_miss--;

	do {
	    while (l3band_deck->l3band_data->frequency
		   < l3band_HALF->frequency) {
		l3band_deck++;
	    }

	    while (l3band_HALF->frequency
		   < l3band_miss->l3band_data->frequency) {
		l3band_miss--;
	    }

	    if (l3band_deck < l3band_miss) {
		struct l3band_type *l3band_swap___;

		l3band_swap___ = l3band_deck->l3band_data;
		l3band_deck->l3band_data = l3band_miss->l3band_data;
		l3band_miss->l3band_data = l3band_swap___;

		l3band_deck++;
		l3band_miss--;
	    } else {
		if (l3band_deck == l3band_miss) {
		    l3band_deck++;
		    l3band_miss--;
		    break;
		}
	    }
	} while (l3band_deck <= l3band_miss);

# undef l3band_HALF

#  if 0
	count = l3band_data - l3band_deck + count;
	if (MTHRESHOLD < count) {
	    ceil_bands(l3band_deck, count);
	}
	{
	    count = l3band_miss - l3band_data + 1;
	    if (MTHRESHOLD < count) {
	    } else {
		break;
	    }
	}
#  else
	{
	    unsigned l_size, r_size;

	    l_size = l3band_data - l3band_deck + count;
	    r_size = l3band_miss - l3band_data + 1;
	    if (l_size < r_size) {
		if (MTHRESHOLD < r_size) {
		    if (MTHRESHOLD < l_size) {
			ceil_bands(l3band_deck, l_size);
		    }
		    count = r_size;
		} else {
		    break;
		}
	    } else {
		if (MTHRESHOLD < l_size) {
		    if (MTHRESHOLD < r_size) {
			ceil_bands(l3band_data, r_size);
		    }
		    l3band_data = l3band_deck;
		    count = l_size;
		} else {
		    break;
		}
	    }
	}
#  endif			/* 0 */
    }
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if __ENABLE_MPEG_EQUALIZER__
static void
cell_bands(struct l3band_type *l3band_data, unsigned count)
{
    if (MTHRESHOLD < count) {
	ceil_bands(l3band_data, count);
    }

    {
	struct l3band_type *l3band_slip;

	l3band_slip = l3band_data + count;

	l3band_data++;
	count--;

	{
	    struct l3band_type *l3band_miss;
	    unsigned i, xsize;

	    l3band_miss = l3band_data - 1;

	    xsize = MTHRESHOLD;
	    if (count < xsize) {
		xsize = count;
	    }

	    for (i = xsize; i; i--) {
		if (l3band_data->l3band_data->frequency
		    < l3band_miss->l3band_data->frequency) {
		    l3band_miss = l3band_data;
		}

		l3band_data++;
	    }

	    l3band_data -= xsize;

	    l3band_data--;

	    if (l3band_miss != l3band_data) {
		struct l3band_type *l3band_swap___;

		l3band_swap___ = l3band_data->l3band_data;
		l3band_data->l3band_data = l3band_miss->l3band_data;
		l3band_miss->l3band_data = l3band_swap___;
	    }

	    l3band_data++;
	}

	l3band_data++;

	for (; l3band_data < l3band_slip; l3band_data++) {
	    struct l3band_type *l3band_miss, l3band, *l3band_text;

	    l3band = *l3band_data;
	    l3band_miss = &l3band;

	    l3band_text = l3band_data - 1;

	    while (l3band_miss->l3band_data->frequency
		   < l3band_text->l3band_data->frequency) {
		l3band_text[1].l3band_data = l3band_text->l3band_data;
		l3band_text--;
	    }

	    l3band_text[1].l3band_data = l3band_miss->l3band_data;
	}
    }
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if __ENABLE_MPEG_EQUALIZER__
static void
link_bands(struct l3band_type *l3band_data, unsigned count)
{
    for (; count; count--) {
	l3band_data->l3band_data = l3band_data;
	l3band_data++;
    }
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


#if __ENABLE_MPEG_EQUALIZER__
int
Mylene1f4_sort_bands(void *bands, unsigned count)
{
    link_bands(bands, count);
    cell_bands(bands, count);

    return 0;
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
