/*
 * moon.c
 * Copyright (C) 2006, 2007, 2008, 2009, 2013, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if __ENABLE_EQUEUE__
# include <stddef.h>
# include <stdlib.h>
# include <string.h>
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# include <access.h>
# include <excess4.h>
# include <excess7.h>
# include <inter.h>
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define equeue(equeue) \
    ((struct equeue_type *) (equeue))
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define frame4(frame4) \
    ((struct frame4_type *) (frame4))
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define Frame4(context) \
    ((struct frame4_type *) &Frame7((context))->_1.frame4)
# define Frame7(context) \
    ((struct frame7_type *) WINDOW((context)).server.window)
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
typedef struct xanode_type {
    struct xanode_type *last_xanode, *next_xanode;
    unsigned size;
} xanode_type;
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
typedef struct xalist_type {
    struct xanode_type *xanode_data;
} xalist_type;
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
int
Mylene1f4_fini_moon(void *context, void *equeue)
{
    struct xanode_type *xanode_data;

    xanode_data = equeue(equeue)->moon.data;
    while (xanode_data) {
	struct xanode_type *next_xanode;

	next_xanode = xanode_data;
	xanode_data = xanode_data->next_xanode;
	free(next_xanode);
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_free_moon(void *frame4, void *data)
{
    struct equeue_type *equeue_data;
    struct xanode_type *xanode_data;
    unsigned deck;

    equeue_data = frame4(frame4)->dana.equeue;

    deck = sizeof(struct xanode_type);
    deck += (16 - (deck & 15)) & 15;
    xanode_data = (void *) ((char *) data - deck);
    {
	struct xanode_type *last_xanode, *next_xanode;

	last_xanode = xanode_data->last_xanode;
	next_xanode = xanode_data->next_xanode;
	if (next_xanode) {
	    next_xanode->last_xanode = last_xanode;
	}
	if (last_xanode) {
	    last_xanode->next_xanode = next_xanode;
	} else {
	    equeue_data->moon.data = next_xanode;
	}
    }

    equeue_data->moon.size += xanode_data->size;

    free(xanode_data);

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_init_moon(void *context, void *equeue)
{
    equeue(equeue)->moon.data = NULL;
    equeue(equeue)->moon.size = WINDOW(context).server.equeue_size;

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_link_moon(void *frame4, void **link, unsigned size)
{
    int status;
    struct equeue_type *equeue_data;

    equeue_data = frame4(frame4)->dana.equeue;

    if (equeue_data->moon.size < size) {
	Mylene1f4_near_moon(frame4(frame4)->context);

	status = -1;
    } else {
	unsigned deck, flow;
	void *data;

	deck = sizeof(struct xanode_type);
	deck += (16 - (deck & 15)) & 15;

	flow = size + deck + 15;

	data = malloc(flow);
	if (!data) {
	    status = -1;
	} else {
	    struct xanode_type *xanode_data, *list_xanode;

	    status = 0;

	    xanode_data = data;

	    list_xanode = equeue_data->moon.data;
	    equeue_data->moon.data = xanode_data;

	    xanode_data->last_xanode = NULL;
	    xanode_data->next_xanode = list_xanode;
	    if (list_xanode) {
		list_xanode->last_xanode = xanode_data;
	    }

	    xanode_data->size = size;

	    equeue_data->moon.size -= size;

	    *link = (void *) ((char *) xanode_data + deck);
	}
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_mode_moon(void *xalist, void **link, unsigned size)
{
    int status;
    void *mode;

    status = Mylene1f4_link_moon(xalist, &mode, size);
    if (!status) {
	void *post;

	post = *link;
	if (post) {
	    struct xanode_type *xanode_data, *last_xanode;
	    unsigned deck, flow;

	    deck = sizeof(struct xanode_type);
	    deck += (16 - (deck & 15)) & 15;

	    xanode_data = (void *) ((char *) mode - deck);
	    last_xanode = (void *) ((char *) post - deck);

	    deck = xanode_data->size;
	    flow = last_xanode->size;
	    if (deck < flow) {
		flow = deck;
	    }

	    memcpy(mode, post, flow);

	    Mylene1f4_free_moon(xalist, post);
	}

	*link = mode;
    }

    return status;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_port_moon(void *context, void *equeue, void *pqueue)
{
    equeue(equeue)->moon.data = equeue(pqueue)->moon.data;
    equeue(equeue)->moon.size = equeue(pqueue)->moon.size;

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */
