/*
 * mpeg.c
 * Copyright (C) 2005-2010, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * TODO
 *
 *.do something about short samples section errors handling (like remove,
 * document, preprocess...)
 *.option for allowing or disallowing decoding errors (like short samples
 * section, invalid backward seek)?
 */
/*
 * BUGS
 *
 *.ID3W(context).transfer is ignored
 *.--mpeg-synchronization-seek is not working for systems, for:
 *    .and if no synchronization is requested the feature is ignored all
 *     together
 */

#include <config.h>

#include <e1f4.h>
#include <errno.h>
#if __ENABLE_x00696e6b__
# include <math.h>
#endif				/* __ENABLE_x00696e6b__ */
#if defined HAVE_LIBLEWD
# include <lewd.h>
#endif				/* HAVE_LIBLEWD */
#if defined HAVE_LIBx1f4l1
# include <libx1f4l1.h>
#endif				/* HAVE_LIBx1f4l1 */
#include <setjmp.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <1f44.h>
#include <access.h>
#if !defined HAVE_LIBLEWD
# include <basename.h>
#endif				/* !HAVE_LIBLEWD */
#include <c213.h>
#if !defined HAVE_LIBLEWD
# include <ckeytree.h>
#endif				/* !HAVE_LIBLEWD */
#include <defs.h>
#include <lcard.h>
#include <mjunk.h>
#include <inter.h>
#if !defined HAVE_LIBx1f4l1
# include <sshell.h>
#endif				/* !HAVE_LIBx1f4l1 */
#include <types.h>
#if !defined HAVE_LIBLEWD
# include <vkeytree.h>
#endif				/* !HAVE_LIBLEWD */

#define __CASE_SYNCHRONIZATION__	0
#define __FIND_SYNCHRONIZATION__	1
#define __HEAD_SYNCHRONIZATION__	2
#define __SEEK_SYNCHRONIZATION__	3

#define __MISS_SYNCHRONIZATION__	4

#define __HERE_SYNCHRONIZATION__	__CASE_SYNCHRONIZATION__

#define __USE_HEADER_HACK_3__		1

#if !__ENABLE_FAST_INTER__
# define __HERE_FAST_INTER__		0
#elif !__ENABLE_MPEG_EQUALIZER__
# define __HERE_FAST_INTER__		0
#else
# define __HERE_FAST_INTER__		1
#endif				/* !__ENABLE_FAST_INTER__ */

#if !defined __Mylene1f4_reset_state_defined__
# define __inter_mylene1f4_reset_state__ \
					1
#elif !__Mylene1f4_reset_state_defined__
# define __inter_mylene1f4_reset_state__ \
					1
#else
# define __inter_mylene1f4_reset_state__ \
					0
#endif				/* !__Mylene1f4_reset_state_defined__ */

#if __inter_mylene1f4_reset_state__
# define Mylene1f4_reset_state		mylene1f4_reset_state
#endif				/* __inter_mylene1f4_reset_state__ */

#if !defined __Mylene1f4_get_allocation_defined__
# define __have_Mylene1f4_get_allocation__ \
					0
#elif !__Mylene1f4_get_allocation_defined__
# define __have_Mylene1f4_get_allocation__ \
					0
#else
# define __have_Mylene1f4_get_allocation__ \
					1
#endif				/* !__Mylene1f4_version_defined__ */

#define AUDIO_FAILURE			(Mylene1f4__ERROR + 0x00)
#define CHANGED_HEADER			(Mylene1f4__ERROR + 0x01)
#define CHANGED_MODE			(Mylene1f4__ERROR + 0x02)
#define CHANGED_SAMPLING		(Mylene1f4__ERROR + 0x03)
#define SEEK_EXCEEDED			(Mylene1f4__ERROR + 0x05)
#define SHORT_SAMPLES_SECTION		(Mylene1f4__ERROR + 0x04)
#define SYNCHRONIZATION_EXHAUSTED	(Mylene1f4__ERROR + 0x16)
#define SYNCHRONIZATION_FAILURE		(Mylene1f4__ERROR + 0x17)

#define NULL_PROCESS			Mylene1f4_FUTILE
#define PCM1_PROCESS			Mylene1f4_4294967291
#define SKIP_PROCESS			Mylene1f4_UTTERLY_FUTILE

#define COTTON_FRAME			Mylene1f4_COTTON_FRAME
#define DOUBLE_FRAME			Mylene1f4_DOUBLE_FRAME
#define SELECT_FRAME			Mylene1f4_SELECT_FRAME
#define SINGLE_FRAME			Mylene1f4_SINGLE_FRAME

#if __HERE_SYNCHRONIZATION__ == __CASE_SYNCHRONIZATION__
# define __find_synchronization__	__FIND_SYNCHRONIZATION__
#elif __HERE_SYNCHRONIZATION__ == __FIND_SYNCHRONIZATION__
# define __find_synchronization__	__FIND_SYNCHRONIZATION__
#else
# define __find_synchronization__	__MISS_SYNCHRONIZATION__
#endif				/* __HERE_SYNCHRONIZATION__ == __CASE_SYN... */

#if __HERE_SYNCHRONIZATION__ == __CASE_SYNCHRONIZATION__
# define __head_synchronization__	__HEAD_SYNCHRONIZATION__
#elif __HERE_SYNCHRONIZATION__ == __HEAD_SYNCHRONIZATION__
# define __head_synchronization__	__HEAD_SYNCHRONIZATION__
#else
# define __head_synchronization__	__MISS_SYNCHRONIZATION__
#endif				/* __HERE_SYNCHRONIZATION__ == __CASE_SYN... */

#if __HERE_SYNCHRONIZATION__ == __CASE_SYNCHRONIZATION__
# define __seek_synchronization__	__SEEK_SYNCHRONIZATION__
#elif __HERE_SYNCHRONIZATION__ == __SEEK_SYNCHRONIZATION__
# define __seek_synchronization__	__SEEK_SYNCHRONIZATION__
#else
# define __seek_synchronization__	__MISS_SYNCHRONIZATION__
#endif				/* __HERE_SYNCHRONIZATION__ == __CASE_SYN... */

#define true(e)				1

#undef SIZE

#define BUFFER(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->buffer)
#define CORRECT_UNDERFLOW(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->correct_underflow)
#define ERROR(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->error)
#define MINIMUM_USEFUL_SIZE(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->minimum_useful_size)
#define PRIVATE(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->private)
#define SIZE(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->size)
#define SYNCWORD_OFFSET(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->syncword_offset)

#define PRIVATE__(whatever, member) \
    (((struct stream_type *) whatever)->member)
#define PRIVATE_A(whatever, member) \
    PRIVATE__(PRIVATE(whatever), member)

#define Private__(whatever, member) \
    (((struct inter6_type *) PRIVATE_A(whatever, loader))->member)

#define Inter5(inter5) \
    ((struct inter5_type *) (inter5))

#define Source(source) \
    ((struct source_type *) (source))->_

#define eid3wl(context) \
    ((struct eid3wl_type *) EID3WL((context)))

#define ALIGN_MBUFFER(bp) \
    ((char *) (bp)							      \
     + ((sizeof(Mylene1f4_real)						      \
	 - ((integral_q) (bp) & (sizeof(Mylene1f4_real) - 1)))		      \
	& (sizeof(Mylene1f4_real) - 1)))

#define ALIGN_MMALLOC(md) \
    ((md) + sizeof(Mylene1f4_real) - 1)

#if __ENABLE_x00696e6b__
# define SET_CARDINAL(buffer, base, card) \
    {									      \
	(buffer)[(base) + 0] = (card) >> 030;				      \
	(buffer)[(base) + 1] = (card) >> 020;				      \
	(buffer)[(base) + 2] = (card) >> 010;				      \
	(buffer)[(base) + 3] = (card) >> 000;				      \
    }
#endif				/* __ENABLE_x00696e6b__ */

#if __ENABLE_x00696e6b__
# define SET_CARDiNAL(buffer, base, card) \
    {									      \
	unsigned cardinal;						      \
									      \
	cardinal = card;						      \
	SET_CARDINAL(buffer, base, cardinal);				      \
    }
#endif				/* __ENABLE_x00696e6b__ */

typedef struct dct64_type {
    void (*data) (Mylene1f4_real *, Mylene1f4_real *, Mylene1f4_real *);
} dct64_type;

typedef struct inter5_type {
    struct seek_1f44_type *e66c;
    struct stream_type *stream_data;
    unsigned ceil, ever;
} inter5_type;

typedef struct inter6_type {
    Mylene1f4_audio_type e1f4_audio;
    int (*seek) (void *, unsigned, unsigned, void *, void *);
    lcard_type stream, system;
    struct socket_type socket;
#if __ENABLE_SEEK_WINDOW__
    void *access;
#endif				/* __ENABLE_SEEK_WINDOW__ */
    void *context, *id3x;
#if __ENABLE_MPEG_EQUALIZER__
    void *instant, *l3;
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
#if __ENABLE_SEEK_WINDOW__
    void *reflex;
#endif				/* __ENABLE_SEEK_WINDOW__ */
    void *xmpeg1;
} inter6_type;

#if __ENABLE_MPEG_EQUALIZER__
typedef struct l3mode_type {
    double amplification;
    struct l3band_type *l3band_data;
    unsigned count;
} l3mode_type;
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

typedef struct x1f440_type {
    int fd;
    struct seek_1f44_type e66c;
    struct source_type source;
} x1f440_type;

#if __inter_mylene1f4_reset_state__
extern void mylene1f4_reset_state(void *);
#endif				/* __inter_mylene1f4_reset_state__ */

extern void III_init_tables(void);

static int back_e1f4(void *, void *, void *);
static int back_mpeg(void *);
#if __ENABLE_SEEK_WINDOW__
static int ceil_page(void *, struct inter6_type *);
#endif				/* __ENABLE_SEEK_WINDOW__ */
#if __ENABLE_MPEG_EQUALIZER__
static int copy_mode(struct inter6_type *, void *);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
static int deck_mpeg(void *, void *);
static int ever_data(void *, unsigned);
static int ever_eead(void *, void *, unsigned);
static int ever_miss(struct inter5_type *);
static int ever_mpeg(void *);
static int ever_read(void *, void *, unsigned);
static int ever_seek(void *, void *, int);
static int ever_tell(void *, void *);
#if __HERE_FAST_INTER__
static int fast_mpeg(void *);
#endif				/* __HERE_FAST_INTER__ */
static int find_data(void *);
static int fini_e1f4(void *, struct inter6_type *);
#if __ENABLE_MPEG_EQUALIZER__
static int fini_mode(struct inter6_type *);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
static int fini_mpeg(void *, unsigned, void **);
static int flow_mpeg(void *, void *);
static int head_hell(void *);
static int head_mpeg(void *);
static int init_e1f4(void *, void *, struct inter6_type *);
#if __ENABLE_MPEG_EQUALIZER__
static int init_mode(struct inter6_type *);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
static int init_mpeg(void *, unsigned, void **);
static int link_mpeg(void *);
#if __ENABLE_x00696e6b__
static int lock_mpeg(void *, unsigned char *, unsigned);
#endif				/* __ENABLE_x00696e6b__ */
static int lose_file(void *);
#if __EXPAND_x1f42MAPS__
static int plug_mpeg(void *, void **);
#endif				/* __ENABLE_x00696e6b__ */
static int seek_1f42(void **, void *, void *);
static int seek_data(void *, void *);
static int seek_e1f4(void *, void *, void *);
static int seek_id3a(void *, void *, struct inter6_type *);
static int seek_id3e(void *, void *, struct inter6_type *);
static int seek_id3i(void *, void *, struct inter6_type *);
static int seek_id3x(void *, void *, struct inter6_type *);
static int seek_mpeg(void *);
#if __ENABLE_SEEK_WINDOW__
static int seek_page(struct inter6_type *);
#endif				/* __ENABLE_SEEK_WINDOW__ */
static int set_output(void *);
static int skip_mpeg(void *);
static int stat_1f44(void *, const char *);
static int tail_hell(void *);
static int type_hell(void *, const char *, unsigned);
static int type_mpeg(void *, void *);
static int wind_mpeg(void *, unsigned *, unsigned);

static void do_release_0(void *, struct Mylene1f4_release_type *);
static void do_release_1(void *, struct Mylene1f4_release_type *);
static void error_exit_1(void *);
static void error_exit_2(void *);
static void error_exit_3(void *);
static void error_exit_4(void *);
static void header_hack_1(struct Mylene1f4_header_type *);
static void header_hack_2(struct Mylene1f4_header_type *);
#if __USE_HEADER_HACK_3__
static void header_hack_3(struct Mylene1f4_header_type *);
#endif				/* __USE_HEADER_HACK_3__ */
static void page_lock(void *, struct inter6_type *);
static void stat_audio(void *, void *);
static void stat_device(void *, void *);
static void stat_failure(void *, int);

static int init_annex_b = 0;
static const struct dct64_type dct64_1st = {
/* *INDENT-OFF* */
    Mylene1f4_dct64_0
/* *INDENT-ON* */
}, dct64_2nd = {
/* *INDENT-OFF* */
    Mylene1f4_dct64_1
/* *INDENT-ON* */
#if defined ENABLE_MMX_CRAP
}, dct64_mmx = {
/* *INDENT-OFF* */
    mpeg_____dct_64___mmx______0
/* *INDENT-ON* */
#endif					/* ENABLE_MMX_CRAP */
};
static const struct target_type data_target = {
/* *INDENT-OFF* */
    back_mpeg,
    deck_mpeg,
    ever_mpeg,
#if __HERE_FAST_INTER__
    fast_mpeg,
#elif __ENABLE_FAST_INTER__
    NULL,
#endif				/* __HERE_FAST_INTER__ */
    fini_mpeg,
    flow_mpeg,
    head_mpeg,
    init_mpeg,
#if __ENABLE_x00696e6b__
    NULL,
#endif				/* __ENABLE_x00696e6b__ */
    link_mpeg,
#if __ENABLE_x00696e6b__
    lock_mpeg,
#endif				/* __ENABLE_x00696e6b__ */
#if __EXPAND_x1f42MAPS__
    plug_mpeg,
#endif				/* __ENABLE_x00696e6b__ */
    seek_mpeg,
    skip_mpeg,
    type_mpeg,
#if !__LINK_CELL_LOADER__
    wind_mpeg
#else
    wind_mpeg,
    NULL
#endif				/* !__LINK_CELL_LOADER__ */
/* *INDENT-ON* */
};
static const struct x1f4_ckeytree_type cotton7_1[] = {
    {	"elect",	{	SELECT_FRAME	},	NULL		},
    {	"ingle",	{	SINGLE_FRAME	},	NULL		},
    {	NULL,		{	0		},	NULL		}
}, cotton7_0[] = {
    {	"double",	{	DOUBLE_FRAME	},	NULL		},
    {	"s",		{	0		},	cotton7_1	},
    {	NULL,		{	0		},	NULL		}
}, process_0[] = {
    {	"4294967291",	{	PCM1_PROCESS	},	NULL		},
    {	"futile",	{	NULL_PROCESS	},	NULL		},
    {	"null",		{	NULL_PROCESS	},	NULL		},
    {	"pcm1",		{	PCM1_PROCESS	},	NULL		},
    {	"real",		{	PCM1_PROCESS	},	NULL		},
    {	"skip",		{	SKIP_PROCESS	},	NULL		},
    {	"utterly-futile",
			{	SKIP_PROCESS	},	NULL		},
    {	NULL,		{	0		},	NULL		}
};
static const struct x1f4_vkeytree_type dct_100_0[] = {
/* *INDENT-OFF* */
    {	"1st",		{	&dct64_1st	},	NULL		},
    {	"2nd",		{	&dct64_2nd	},	NULL		},
#if defined ENABLE_MMX_CRAP
    {	"mmx",		{	&dct64_mmx	},	NULL		},
#endif					/* ENABLE_MMX_CRAP */
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
};

const struct x1f4_ckeytree_type _Mylene1f4_process_root[] = {
    {	"",		{	0		},	process_0	},
    {	NULL,		{	0		},	NULL		}
}, _Mylene1f4_synchronization_root[] = {
    {	"",		{	0		},	cotton7_0	},
    {	NULL,		{	0		},	NULL		}
};
const struct x1f4_vkeytree_type _Mylene1f4_dct_64_root[] = {
/* *INDENT-OFF* */
    {	"",		{	NULL		},	dct_100_0	}
/* *INDENT-ON* */
};

static int
back_e1f4(void *e1f4_audio, void *stream, void *system)
{
    int status;
    struct seek_1f44_type *e66c;

    e66c = PRIVATE_A(e1f4_audio, e66c);

    status = e66c->seek(e66c, system, SEEK_SET);
    if (!status) {
	Mylene1f4_reset_state(e1f4_audio);
	if (PRIVATE_A(e1f4_audio, system)) {
	    Mylene1f4_set_system(e1f4_audio, stream);
	    Mylene1f4_clear_system(e1f4_audio);
	}
    }

    return status;
}


static int
back_mpeg(void *e1f4_audio)
{
    int status;
#if __ENABLE_SEEK_WINDOW__
    void *access;
#endif				/* __ENABLE_SEEK_WINDOW__ */

#if __ENABLE_SEEK_WINDOW__
    access = Private__(e1f4_audio, access);
    if (access) {
	lcard_type stream, system;

	Mylene1f4_back_socket(access, &stream, &system);

	status = back_e1f4(e1f4_audio, &stream, &system);
    } else {
#endif				/* __ENABLE_SEEK_WINDOW__ */
	status = back_e1f4
	    (e1f4_audio, &Private__(e1f4_audio, stream),
	     &Private__(e1f4_audio, system));
#if __ENABLE_SEEK_WINDOW__
    }
#endif				/* __ENABLE_SEEK_WINDOW__ */

    return status;
}


#if __ENABLE_SEEK_WINDOW__
static int
ceil_page(void *context, struct inter6_type *inter6_data)
{
    do {
	unsigned mode;

	mode = MODE(context) & Mylene1f4_WINDOW_MASK;
# if __ENABLE_CONTROLLING_SERVER__
	if (mode == Mylene1f4_SERVER_WINDOW) {
	} else {
# endif				/* __ENABLE_CONTROLLING_SERVER__ */
# if __ENABLE_x1f4LANES__
	    if (mode == Mylene1f4_WITHIN_WINDOW) {
	    } else {
# endif				/* __ENABLE_x1f4LANES__ */
		break;
# if __ENABLE_x1f4LANES__
	    }
# endif				/* __ENABLE_x1f4LANES__ */
# if __ENABLE_CONTROLLING_SERVER__
	}
# endif				/* __ENABLE_CONTROLLING_SERVER__ */

	Mylene1f4_init_socket
	    (context, &inter6_data->reflex, &inter6_data->socket);
    } while (0);

    return 0;
}
#endif				/* __ENABLE_SEEK_WINDOW__ */


#if __ENABLE_MPEG_EQUALIZER__
static int
copy_mode(struct inter6_type *inter6_data, void *context)
{
    int status;
    struct l3mode_type *l3mode_data;

    l3mode_data = (struct l3mode_type *) malloc(sizeof(struct l3mode_type));
    if (!l3mode_data) {
	status = -1;
    } else {
	struct l3band_type *l3band_data;
	unsigned count;

	count = MPEG(context).count;

	l3band_data = (struct l3band_type *)
	    malloc(sizeof(struct l3band_type) * count);
	if (!l3band_data) {
	    free(l3mode_data);

	    status = -1;
	} else {
	    struct l3band_type *l3band_text;
	    unsigned i;

	    status = 0;

	    l3band_text = MPEG(context).l3band_data;

	    l3mode_data->l3band_data = l3band_data;

	    i = count;
	    for (; i; i--) {
		l3band_data->frequency = l3band_text->frequency;
		l3band_data->level = l3band_text->level;
		l3band_data->l3band_data = l3band_data
		    + (l3band_text->l3band_data - l3band_text);
		l3band_data++;
		l3band_text++;
	    }

	    l3mode_data->amplification = MPEG(context).amplification;
	    l3mode_data->count = count;

	    inter6_data->instant = l3mode_data;
	}
    }

    return status;
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


static int
deck_mpeg(void *e1f4_audio, void *deck)
{
    return seek_e1f4(e1f4_audio, deck, NULL);
}


static int
ever_data(void *e1f4_audio, unsigned ceil)
{
    int status;
    struct inter5_type inter5;
    struct seek_1f44_type e66c;
    struct stream_type *stream_data;
    void *h262_system;

    e66c.eead = ever_eead;
    e66c.read = ever_read;
    e66c.seek = ever_seek;
    e66c.tell = ever_tell;

    e66c.data = &inter5;

    stream_data = PRIVATE(e1f4_audio);

    inter5.stream_data = stream_data;

    inter5.e66c = stream_data->e66c;

    stream_data->e66c = &e66c;
    if (1) {
	h262_system = stream_data->system;
	if (h262_system) {
	    Mylene1f4_fit_system(h262_system, &e66c);
	}
    }

    inter5.ceil = ceil;
    inter5.ever = 0;

    status = find_data(e1f4_audio);

    stream_data->e66c = inter5.e66c;
    if (1) {
	if (h262_system) {
	    Mylene1f4_fit_system(h262_system, inter5.e66c);
	}
    }

    return status;
}


static int
ever_eead(void *e66c, void *data, unsigned size)
{
    int status;
    void *inter5;

    inter5 = ((struct seek_1f44_type *) e66c)->data;

    if (Inter5(inter5)->ever < Inter5(inter5)->ceil) {
	struct seek_1f44_type *e66c;

	e66c = Inter5(inter5)->e66c;

	status = e66c->eead(e66c, data, size);
	if (status < 0) {
	} else {
	    Inter5(inter5)->ever += status;
	}
    } else {
	status = ever_miss(inter5);
    }

    return status;
}


static int
ever_miss(struct inter5_type *inter5_data)
{
    longjmp(inter5_data->stream_data->env, SEEK_EXCEEDED);

    return -1;
}


static int
ever_mpeg(void *e1f4_audio)
{
    int status;

    if ((status = setjmp(PRIVATE_A(e1f4_audio, env)))) {
	int do_stat = 1;

	if (!~PRIVATE_A(e1f4_audio, clip)) {
	    if (1) {
		if (status == Mylene1f4_UNEXPECTED_EOF
		    || status == Mylene1f4_UNEXPECTED_EOS
		    || status == SYNCHRONIZATION_FAILURE) {
		    do_stat = 0;
		    COMPLETED(PRIVATE_A(e1f4_audio, context)) = 1;
		}
	    }
	} else {
	    if (status == SHORT_SAMPLES_SECTION) {
		status = 0;
	    }
	}
	if (status) {
	    if (do_stat) {
		if (1) {
		    stat_failure(e1f4_audio, status);
		}
	    }

	    status = 1;
	}
    } else {
	do {
	    struct inter6_type *inter6_data;

	    inter6_data = PRIVATE_A(e1f4_audio, loader);
	    if (inter6_data->socket.call) {
		if (REFERENCE(PRIVATE_A(e1f4_audio, context))
		    == inter6_data->socket.page) {
		    page_lock(e1f4_audio, inter6_data);
		    break;
		}
	    }

	    Mylene1f4_11172_frame(e1f4_audio);
	} while (0);
    }

    return status;
}


static int
ever_read(void *e66c, void *data, unsigned size)
{
    int status;
    void *inter5;

    inter5 = ((struct seek_1f44_type *) e66c)->data;

    if (Inter5(inter5)->ever < Inter5(inter5)->ceil) {
	struct seek_1f44_type *e66c;

	e66c = Inter5(inter5)->e66c;

	status = e66c->read(e66c, data, size);
	if (status < 0) {
	} else {
	    Inter5(inter5)->ever += status;
	}
    } else {
	status = ever_miss(inter5);
    }

    return status;
}


static int
ever_seek(void *ever, void *data, int whence)
{
    int status;
    struct seek_1f44_type *e66c;
    void *inter5;

    inter5 = ((struct seek_1f44_type *) ever)->data;

    e66c = Inter5(inter5)->e66c;

    status = e66c->seek(e66c, data, whence);
    if (!status) {
	if (whence == SEEK_SET) {
	    Inter5(inter5)->ever = LCARD_GET_CARD(*(lcard_type *) data);
	} else {
	    if (true(whence == SEEK_CUR)) {
		Inter5(inter5)->ever += LCARD_GET_CARD(*(lcard_type *) data);
	    } else {
		status = -1;
	    }
	}
    }

    return status;
}


static int
ever_tell(void *ever, void *data)
{
    struct seek_1f44_type *e66c;

    e66c = Inter5(((struct seek_1f44_type *) ever)->data)->e66c;

    return e66c->tell(e66c, data);
}


#if __HERE_FAST_INTER__
static int
fast_mpeg(void *e1f4_audio)
{
    int false;
    struct inter6_type *inter6_data;
    void *context;

    inter6_data = PRIVATE_A(e1f4_audio, loader);

    context = inter6_data->context;

    if (!(MPEG(context).amplification || MPEG(context).mode)) {
	if (!inter6_data->l3) {
	    false = 1;
	} else {
	    false = 0;
	}
    } else {
	struct l3mode_type *l3mode_data;

	l3mode_data = inter6_data->instant;
	if (!l3mode_data) {
	    false = 0;
	} else {
	    if (l3mode_data->amplification != MPEG(context).amplification) {
		false = 0;
	    } else {
		struct l3band_type *l3band_data, *l3band_text;
		unsigned i;

		l3band_data = l3mode_data->l3band_data;

		l3band_text = MPEG(context).l3band_data;

		i = MPEG(context).count;
		for (; i; i--) {
		    struct l3band_type *l3band_fast, *l3band_here;

		    l3band_fast = l3band_text->l3band_data;
		    l3band_here = l3band_data->l3band_data;

		    if (l3band_fast->frequency != l3band_here->frequency
			|| l3band_fast->level != l3band_here->level) {
			break;
		    }

		    l3band_data++;
		    l3band_text++;
		}

		if (i) {
		    false = 0;
		} else {
		    false = 1;
		}
	    }
	}
    }

    return false;
}
#endif				/* __HERE_FAST_INTER__ */


static int
find_data(void *e1f4_audio)
{
    int status = 0;

    if ((status = setjmp(PRIVATE_A(e1f4_audio, env)))) {
	if (status) {
	    if (status == Mylene1f4_UNEXPECTED_EOF
		|| status == Mylene1f4_UNEXPECTED_EOS) {
		if (1) {
		    stat_failure(e1f4_audio, SYNCHRONIZATION_EXHAUSTED);
		}
	    } else {
		if (1) {
		    stat_failure(e1f4_audio, status);
		}
	    }

	    status = 1;
	}
    } else {
#if __HERE_SYNCHRONIZATION__ == __CASE_SYNCHRONIZATION__
	if (MPEG(PRIVATE_A(e1f4_audio, context)).synchronization
	    == SINGLE_FRAME) {
#endif				/* __HERE_SYNCHRONIZATION__ == __CASE_SYN... */
#if __find_synchronization__ == __FIND_SYNCHRONIZATION__
	    Mylene1f4_find_audio(e1f4_audio);
#endif				/* __find_synchronization__ == __FIND_SYN... */
#if __HERE_SYNCHRONIZATION__ == __CASE_SYNCHRONIZATION__
	} else if (MPEG(PRIVATE_A(e1f4_audio, context)).synchronization
		   == DOUBLE_FRAME) {{
#endif				/* __HERE_SYNCHRONIZATION__ == __CASE_SYN... */
#if __seek_synchronization__ == __SEEK_SYNCHRONIZATION__
	    Mylene1f4_seek_audio(e1f4_audio);
#endif				/* __seek_synchronization__ == __SEEK_SYN... */
#if __HERE_SYNCHRONIZATION__ == __CASE_SYNCHRONIZATION__
	}
#endif				/* __HERE_SYNCHRONIZATION__ == __CASE_SYN... */
#if __HERE_SYNCHRONIZATION__ == __CASE_SYNCHRONIZATION__
	} else {
#endif				/* __HERE_SYNCHRONIZATION__ == __CASE_SYN... */
#if __head_synchronization__ == __HEAD_SYNCHRONIZATION__
	    Mylene1f4_synchronize_audio
		(e1f4_audio,
		 MPEG(PRIVATE_A(e1f4_audio,
				context)).synchronization_sequence);
#endif				/* __head_synchronization__ == __HEAD_SYN... */
#if __HERE_SYNCHRONIZATION__ == __CASE_SYNCHRONIZATION__
	}
#endif				/* __HERE_SYNCHRONIZATION__ == __CASE_SYN... */
    }

    return status;
}


static int
fini_e1f4(void *context, struct inter6_type *inter6_data)
{
    Mylene1f4_audio_type e1f4_audio;
    struct stream_type *stream_data;

    if (inter6_data->id3x) {
	free(inter6_data->id3x);
    }

    e1f4_audio = inter6_data->e1f4_audio;

    stream_data = PRIVATE(e1f4_audio);

    Mylene1f4_remove_sequel(context, Mylene1f4_XMPEG1_SEQUEL);

    {
	void *data;

	data = stream_data->data;
	if (data) {
	    free(data);
	}
    }

    if (stream_data->system) {
	Mylene1f4_get_system(e1f4_audio);
    }

    if (!X1f42(context)) {
	seek_1f44_clip(stream_data->e66c);
    }

#if __ENABLE_SEEK_WINDOW__
    if (inter6_data->reflex) {
	Mylene1f4_fini_socket(&inter6_data->reflex);
    }
#endif				/* __ENABLE_SEEK_WINDOW__ */

    free(e1f4_audio);

    return 0;
}


#if __ENABLE_MPEG_EQUALIZER__
static int
fini_mode(struct inter6_type *inter6_data)
{
    void *l3;

    l3 = inter6_data->l3;
    if (l3) {
	struct l3mode_type *l3mode_data;

	l3mode_data = inter6_data->instant;
	if (l3mode_data) {
	    void *instant;

	    instant = l3mode_data->l3band_data;
	    if (instant) {
		free(instant);
	    }

	    free(l3mode_data);
	}

	Mylene1f4_day_mode(l3);
    }

    return 0;
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


static int
fini_mpeg(void *loader, unsigned index, void **storage)
{
#if __ENABLE_MPEG_EQUALIZER__
    fini_mode(loader);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

    return 0;
}


static int
flow_mpeg(void *e1f4_audio, void *flow)
{
    int status;
    lcard_type stream;

    status = seek_e1f4(e1f4_audio, &stream, NULL);
    if (!status) {
	lcard_type offset, *output;
	struct source_type *source_data;

	source_data = PRIVATE_A(e1f4_audio, source);
	LCARD_SET_CARD2(offset, source_data->_.s2.size);
	LCARD_SUBTRACT_CARD2(offset, stream);
	output = flow;
	LCARD_SET_CARD2(*output, offset);
    }

    return status;
}


static int
head_hell(void *context)
{
    Mylene1f4_fprintf_error(context, "%s: ", Mylene1f4(context));

    return 0;
}


static int
head_mpeg(void *e1f4_audio)
{
    return seek_e1f4
	(e1f4_audio, &Private__(e1f4_audio, stream),
	 &Private__(e1f4_audio, system));
}


static int
init_e1f4(void *context, void *source, struct inter6_type *inter6_data)
{
    Mylene1f4_audio_type e1f4_audio;
    int status;
    unsigned s66c, size;

    if (!X1f42(context)) {
	s66c = ALIGN_ZMALLOC(sizeof(struct x1f440_type));
    } else {
	s66c = 0;
    }

    Mylene1f4_audio_size(&size);
    if (!(e1f4_audio = (Mylene1f4_audio_type)
	  malloc(size + ALIGN_AMALLOC(sizeof(struct stream_type))
		 + ALIGN_AMALLOC(sizeof(sequel_type))
		 + ALIGN_AMALLOC(sizeof(xmpeg1_type))
		 + ALIGN_MMALLOC(sizeof(Mylene1f4_real) * (1088 + 2304))
		 + s66c))) {
	Mylene1f4_fprintf_error
	    (context, "%s: %s\n", Mylene1f4(context), strerror(errno));

	status = -1;
    } else {
	struct sequel_type *sequel_data;
	struct stream_type *stream_data;
	struct xmpeg1_type *xmpeg1_data;
	struct x1f440_type *x1f440_data;

	status = 0;

	Mylene1f4_reset_audio(e1f4_audio);

	stream_data = (struct stream_type *) ((char *) e1f4_audio + size);
	stream_data = (struct stream_type *) ALIGN_ABUFFER(stream_data);

	sequel_data = (struct sequel_type *) (stream_data + 1);
	sequel_data = (struct sequel_type *) ALIGN_ABUFFER(sequel_data);

	xmpeg1_data = (struct xmpeg1_type *) (sequel_data + 1);
	xmpeg1_data = (struct xmpeg1_type *) ALIGN_ABUFFER(xmpeg1_data);

	{
	    Mylene1f4_real *polyphase, *transform;
	    struct Mylene1f4_audio_attributes_type attributes;
	    unsigned mask = 0;

	    polyphase = (Mylene1f4_real *) (xmpeg1_data + 1);
	    polyphase = (Mylene1f4_real *) ALIGN_MBUFFER(polyphase);
	    transform = polyphase + 1088;

	    if (!s66c) {
		x1f440_data = NULL;
	    } else {
		x1f440_data = (struct x1f440_type *) (transform + 2304);
		x1f440_data = (struct x1f440_type *)
		    ALIGN_ZBUFFER(x1f440_data);
	    }

	    attributes.abort_decoding = error_exit_2;
	    attributes.header_change_mask = Mylene1f4_AUDIO_LAYER_MASK
#if !__USE_HEADER_HACK_3__
		| Mylene1f4_AUDIO_MODE_MASK
#endif				/* !__USE_HEADER_HACK_3__ */
		| Mylene1f4_AUDIO_SAMPLING_FREQUENCY_MASK
		| Mylene1f4_AUDIO_id_MASK;
	    attributes.output = AUDIO(context);
	    attributes.polyphase_buffer = polyphase;
	    attributes.transform_buffer = transform;

	    if (0) {
		const struct dct64_type *dct_64;

		dct_64 = MPEG(context).dct_64;
		if (dct_64) {
		    attributes.dct_64 = dct_64->data;
		    mask |= Mylene1f4_AUDIO_DCT_64_MASK;
		}
	    }

	    Mylene1f4_set_audio_attributes
		(e1f4_audio, &attributes,
		 Mylene1f4_AUDIO_ABORT_DECODING_MASK
		 | Mylene1f4_AUDIO_CORRECT_UNDERFLOW_MASK
#if 0
		 | Mylene1f4_AUDIO_EQUALIZER_MASK
#endif				/* 0 */
		 | Mylene1f4_AUDIO_OUTPUT_MASK
		 | Mylene1f4_AUDIO_POLYPHASE_BUFFER_MASK
		 | Mylene1f4_AUDIO_TRANSFORM_BUFFER_MASK | mask,
		 Mylene1f4_AUDIO_HEADER_CHANGE_MASK);
	}

	PRIVATE(e1f4_audio) = stream_data;

	stream_data->context = context;
	stream_data->data = NULL;
	stream_data->loader = inter6_data;
	stream_data->size = 0;
	stream_data->system = NULL;

	if (!init_annex_b) {
	    init_annex_b = 1;

	    III_init_tables();
	    Mylene1f4_init_tables();
	}

	Mylene1f4_get_output(e1f4_audio, &stream_data->e1f4_output);

	Mylene1f4_link_stream(e1f4_audio);

	{
	    struct Mylene1f4_audio_attributes_type attributes;

	    attributes.buffer = NULL;
	    attributes.size = 0;
	    attributes.syncword_offset = 0;

	    Mylene1f4_set_audio_attributes
		(e1f4_audio, &attributes,
		 Mylene1f4_AUDIO_BUFFER_MASK
		 | Mylene1f4_AUDIO_SIZE_MASK
		 | Mylene1f4_AUDIO_SYNCWORD_OFFSET_MASK,
		 0);

	    Mylene1f4_clear_state(e1f4_audio);
	}

	{
	    struct x1f42a_type *x1f42a_data;

	    x1f42a_data = X1f42(context);
	    if (!x1f42a_data) {
		int fd;
		struct seek_1f44_type *e66c;
		struct source_type *source_data;

		e66c = &x1f440_data->e66c;

		source_data = &x1f440_data->source;

		stream_data->e66c = e66c;
		stream_data->source = source_data;

		fd = Source(source).s1.fd;

		LCARD_SET_CARD1
		    (source_data->_.s2.size, Source(source).s1.size);
		x1f440_data->fd = fd;

		status = seek_1f44_plug(e66c, &fd);
		if (!status) {
		    status = Mylene1f4_try_system(e1f4_audio, source);
		    if (!status) {
			if (stream_data->system) {
			    Mylene1f4_link_system(e1f4_audio);
			}
		    }
		}
	    } else {
		stream_data->source = source;
		stream_data->e66c = Source(source).s2.e66c;

		do {
		    unsigned type;

		    type = x1f42a_data->type;
		    if (type == 0xffffffff) {
			status = Mylene1f4_fix_system(e1f4_audio, 11172);
		    } else if (type == 0xaaaaaaaa || type == 0x33333333) {
			status = Mylene1f4_fix_system(e1f4_audio, 13818);
		    } else {
			break;
		    }
		    if (!status) {
			if (stream_data->system) {
			    Mylene1f4_link_system(e1f4_audio);
			}
		    }
		} while (0);
	    }
	}
	if (status) {
	    free(e1f4_audio);
	} else {
	    {
		inter6_data->id3x = NULL;
	    }

#if __ENABLE_SEEK_WINDOW__
	    {
		inter6_data->reflex = NULL;
	    }
#endif				/* __ENABLE_SEEK_WINDOW__ */

	    {
		inter6_data->socket.call = NULL;
	    }

	    {
		inter6_data->xmpeg1 = xmpeg1_data;
	    }

	    {
		inter6_data->e1f4_audio = e1f4_audio;
	    }

	    sequel_data->sequel_data = SEQUEL(context);
	    SEQUEL(context) = sequel_data;
	    sequel_data->code = Mylene1f4_XMPEG1_SEQUEL;
	    sequel_data->data = xmpeg1_data;

	    if (!PRIVATE_A(e1f4_audio, system)) {
		inter6_data->seek = Mylene1f4_seek_stream;
	    } else {
		inter6_data->seek = Mylene1f4_seek_system;
	    }
	}
    }

    return status;
}


#if __ENABLE_MPEG_EQUALIZER__
static int
init_mode(struct inter6_type *inter6_data)
{
    inter6_data->instant = NULL;
    inter6_data->l3 = NULL;

    return 0;
}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */


static int
init_mpeg(void *loader, unsigned index, void **storage)
{
    Mylene1f4_audio_type e1f4_audio;
#if __ENABLE_MPEG_EQUALIZER__
    int mode;
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
    struct inter6_type *inter6_data;
    void *context;

    inter6_data = loader;

    e1f4_audio = inter6_data->e1f4_audio;

    context = PRIVATE_A(e1f4_audio, context);

#if __ENABLE_MPEG_EQUALIZER__
    init_mode(inter6_data);
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

#if __ENABLE_MPEG_EQUALIZER__
    mode = MPEG(context).amplification || MPEG(context).mode;
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

    {
	struct Mylene1f4_audio_attributes_type attributes;
#if __ENABLE_MPEG_EQUALIZER__
	unsigned mask = 0;
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

	attributes.do_release = do_release_1;
	attributes.header_hack_1 = header_hack_1;
	attributes.header_hack_2 = NULL;
#if __USE_HEADER_HACK_3__
	attributes.header_hack_3 = NULL;
#endif				/* __USE_HEADER_HACK_3__ */
	if (1) {
	    const unsigned *process;

	    process = MPEG(context).process;
	    if (process) {
		attributes.process = *process;
	    } else {
		attributes.process = Mylene1f4_4294967291;
	    }
	}
#if __ENABLE_MPEG_EQUALIZER__
	if (!mode) {
	    attributes.equalize_0 = NULL;
	    attributes.equalizer_0 = NULL;
	    mask |= Mylene1f4_AUDIO_EQUALIZER_0_MASK
		| Mylene1f4_AUDIO_EQUALIZE_0_MASK;
	}
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

	Mylene1f4_set_audio_attributes
	    (e1f4_audio, &attributes,
#if __ENABLE_MPEG_EQUALIZER__
	     Mylene1f4_AUDIO_DO_RELEASE_MASK | Mylene1f4_AUDIO_PROCESS_MASK
	     | mask,
#else
	     Mylene1f4_AUDIO_DO_RELEASE_MASK | Mylene1f4_AUDIO_PROCESS_MASK,
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
	     Mylene1f4_AUDIO_HEADER_HACK_1_MASK
	     | Mylene1f4_AUDIO_HEADER_HACK_2_MASK
#if __USE_HEADER_HACK_3__
	     | Mylene1f4_AUDIO_HEADER_HACK_3_MASK
#endif				/* __USE_HEADER_HACK_3__ */
	     | 0);

	Mylene1f4_clear_state(e1f4_audio);
    }

#if __ENABLE_MPEG_EQUALIZER__
    if (mode) {
	int excess;
	struct xmpeg1_type *xmpeg1_data;
	unsigned sampling;

	xmpeg1_data = inter6_data->xmpeg1;

	sampling = SAMPLING(context);
	if (!xmpeg1_data->specification) {
	    sampling <<= 1;
	}

	excess = Mylene1f4_way_mode
	    (context, &inter6_data->l3, sampling, e1f4_audio);
	if (!excess) {
	    if (inter6_data->l3) {
		copy_mode(inter6_data, context);
	    }
	}
    }
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

    {
	struct stream_type *stream_data;

	stream_data = PRIVATE(e1f4_audio);
	stream_data->clip = ~0;
	stream_data->synchronization = 0;
    }

    HEAD(context) = 0;
    TAIL(context) = ~0;

#if __ENABLE_SEEK_WINDOW__
    seek_page(inter6_data);
#endif				/* __ENABLE_SEEK_WINDOW__ */

    *storage = e1f4_audio;

    return 0;
}


static int
link_mpeg(void *e1f4_audio)
{
    int status;

    if ((status = setjmp(PRIVATE_A(e1f4_audio, env)))) {
	int do_stat = 1;

	if (!~PRIVATE_A(e1f4_audio, clip)) {
	    if (1) {
		if (status == Mylene1f4_UNEXPECTED_EOF
		    || status == Mylene1f4_UNEXPECTED_EOS
		    || status == SYNCHRONIZATION_FAILURE) {
		    do_stat = 0;
		    COMPLETED(PRIVATE_A(e1f4_audio, context)) = 1;
		}
	    }
	} else {
	    if (status == SHORT_SAMPLES_SECTION) {
		status = 0;
	    }
	}
	if (status) {
	    if (do_stat) {
		if (1) {
		    stat_failure(e1f4_audio, status);
		}
	    }

	    status = 1;
	}
    } else {
	void *context;

	context = PRIVATE_A(e1f4_audio, context);

	*PRIVATE_A(e1f4_audio, e1f4_output) = AUDIO(context) + C1f4(context);

	do {
	    struct inter6_type *inter6_data;

	    inter6_data = PRIVATE_A(e1f4_audio, loader);
	    if (inter6_data->socket.call) {
		if (REFERENCE(context) == inter6_data->socket.page) {
		    page_lock(e1f4_audio, inter6_data);
		    break;
		}
	    }

	    Mylene1f4_11172_frame(e1f4_audio);
	} while (0);
    }

    return status;
}


#if __ENABLE_x00696e6b__
static int
lock_mpeg(void *context, unsigned char *buffer, unsigned reference)
{
    double fractional, integral;
    struct sequel_type *sequel_data;
    struct xmpeg1_type *xmpeg1_data;

    Mylene1f4_find_sequel
	(context, Mylene1f4_XMPEG1_SEQUEL, (void *) &sequel_data);
    xmpeg1_data = sequel_data->data;
    fractional = modf
	((double) reference * xmpeg1_data->frame_rate_d
	 / xmpeg1_data->frame_rate_n, &integral);
    SET_CARDiNAL(buffer, 0, (unsigned) integral);
    SET_CARDiNAL(buffer, 4, (unsigned) (fractional * 1000000));

    return 0;
}
#endif				/* __ENABLE_x00696e6b__ */


static int
lose_file(void *context)
{
    return close(SOURCE(context)._.s1.fd);
}


#if __EXPAND_x1f42MAPS__
static int
plug_mpeg(void *loader, void **socket)
{
    struct inter6_type *inter6_data;

    inter6_data = loader;

    *socket = &inter6_data->socket;

    return 0;
}
#endif				/* __ENABLE_x00696e6b__ */


static int
seek_1f42(void **loader, void *source, void *e1f4_audio)
{
    int status = 0;
    void **glass;

    glass = *loader;
    if (true(glass)) {
	struct device_type *device_data;

	device_data = glass[1];
	if (device_data) {
	    int (*seek) (void *, void *, unsigned, void *);

	    seek = device_data->seek;
	    if (seek) {
		lcard_type cleavage[1];
		unsigned level;

		level = !PRIVATE_A(e1f4_audio, system) ? 0 : 1;
		status = seek(glass[0], source, level, cleavage);
		if (!status) {
		    if (level) {
			Mylene1f4_set_system(e1f4_audio, cleavage);
		    }
		}
	    }
	}
    }

    return status;
}


static int
seek_data(void *e1f4_audio, void *context)
{
    int status;
    unsigned seek;

    seek = MPEG(context).seek;
    if (!seek) {
	status = find_data(e1f4_audio);
    } else {
	status = ever_data(e1f4_audio, seek);
    }

    return status;
}


static int
seek_e1f4(void *e1f4_audio, void *stream, void *system)
{
    unsigned size, syncword_offset;

    syncword_offset = SYNCWORD_OFFSET(e1f4_audio);
    size = SIZE(e1f4_audio);
    if (size < syncword_offset) {
	unsigned shift;

	Mylene1f4_get_audio_data_size(e1f4_audio, &shift);
	size += 4 + shift;
    }

    return Private__(e1f4_audio, seek)
	(e1f4_audio, syncword_offset, size, stream, system);
}


static int
seek_id3a(void *context, void *source, struct inter6_type *inter6_data)
{
    int status;
    void *sshell;

    status = x1f4_init_sshell(&sshell);
    if (!status) {
	const char *argv[3], *parser;
	char lock[4096];
	int excess;
	struct clause_type clause, *clause_data;
	struct x1f4_sshell_type Sshell;

	parser = eid3wl(context)->parser;
	if (parser) {
	    if (!*parser) {
		parser = NULL;
	    }
	}

	argv[0] = !parser ? "id3v2-id3tag" : x1f4_basename(parser);
	argv[1] = Source(source).s1.path;
	argv[2] = NULL;

	clause_data = CLAUSE(context);

	clause.clause_data = clause_data;
	clause.delete = lose_file;

	CLAUSE(context) = &clause;

	Sshell.cleanup.back = context;
	Sshell.cleanup.call = Mylene1f4_close_connections;
	Sshell.command.argv = argv;
	Sshell.command.path =
	    !parser ? __install_prefix "/lib/Mylene1f4/id3v2-id3tag" : parser;
	Sshell.printer.head = head_hell;
	Sshell.printer.look = context;
	Sshell.printer.tail = tail_hell;
	Sshell.printer.type = type_hell;
	Sshell.storage.data = lock;
	Sshell.storage.size = 4096 - 1 - sizeof(struct id3tag_type);
	Sshell.timeout.e6th = eid3wl(context)->timeout;
	if (!Sshell.timeout.e6th) {
	    Sshell.timeout.e6th = 8192;
	}
	if (eid3wl(context)->flags & Mylene1f4_EID3WL_QUIET) {
	    Sshell.trailer.c3rd = X1f4_SSHELL_KILL_C3RD;
	} else {
	    Sshell.trailer.c3rd = X1f4_SSHELL_MISS_C3RD;
	}

	status = x1f4_mode_sshell
	    (sshell, &Sshell,
	     X1f4_SSHELL_ARGV_MASK | X1f4_SSHELL_BACK_MASK
	     | X1f4_SSHELL_C3RD_MASK | X1f4_SSHELL_CALL_MASK
	     | X1f4_SSHELL_E6TH_MASK | X1f4_SSHELL_DATA_MASK
	     | X1f4_SSHELL_HEAD_MASK | X1f4_SSHELL_LOOK_MASK
	     | X1f4_SSHELL_PATH_MASK | X1f4_SSHELL_SIZE_MASK
	     | X1f4_SSHELL_TAIL_MASK | X1f4_SSHELL_TYPE_MASK);
	if (!status) {
	    char *data;
	    unsigned size;

	    status = x1f4_link_sshell(sshell, &data, &size);
	    if (!status) {
		char *over;
		unsigned *ff3a, i, j = 5, tags[5];

		ff3a = tags;
		over = data;
		for (i = size; i; i--) {
		    if (!*over) {
			if (!j) {
			    status = 1;
			} else {
			    *ff3a = size - i + 1;
			    ff3a++;
			    j--;
			}
		    }
		    over++;
		}
		if (j) {
		    status = 1;
		}
		if (!status) {
		    struct id3tag_type *id3tag_data;

		    id3tag_data = (struct id3tag_type *)
			malloc(sizeof(struct id3tag_type) + size + 1);
		    if (!id3tag_data) {
			status = 1;
		    } else {
			char **dAta;
			unsigned ff3c, *sIze;

			over = (char *) (id3tag_data + 1);
			memcpy(over, data, size + 0);
			over[size] = 0;

			dAta = id3tag_data->data;
			sIze = id3tag_data->size;

			ff3a = tags;

			{
			    ff3c = *ff3a;
			    ff3a++;
			    *dAta++ = over;
			    *sIze++ = ff3c - 1;
			}
			for (i = 2; i; i--) {
			    unsigned ff3e;

			    *dAta++ = over + ff3c;
			    ff3e = *ff3a;
			    ff3a++;
			    ff3c++;
			    *sIze++ = ff3e - ff3c;
			    *dAta++ = over + ff3e;
			    ff3c = *ff3a;
			    ff3a++;
			    ff3e++;
			    *sIze++ = ff3c - ff3e;
			}
			{
			    *dAta = over + ff3c;
			    *sIze = size - ff3c;
			}

			inter6_data->id3x = id3tag_data;

			ID3T(context) = Mylene1f4_BROKEN_ID3TAG;
			ID3X(context) = id3tag_data;

			status = 0;
		    }
		}
	    }
	}

	CLAUSE(context) = clause_data;

	excess = x1f4_fini_sshell(&sshell);
	if (excess) {
	    if (!status) {
		status = excess;
	    }
	}
    }

    return !status ? 0 : 1;
}


static int
seek_id3e(void *context, void *source, struct inter6_type *inter6_data)
{
    int status = 1;

    if (1) {
	if (2 < Source(source).s1.size) {
	    if (1) {
		int excess;
		unsigned char buffer[3];

		status = Mylene1f4_read_file(context, source, buffer, 3);
		if (status != -1) {
		    if (status != 3) {
			status = 1;
		    } else {
			if (memcmp(buffer, "ID3", 3)) {
			    status = 1;
			} else {
			    status = seek_id3a(context, source, inter6_data);
			}
		    }

		    excess = Mylene1f4_seek_file(context, source, 0, SEEK_SET);
		    if (!status) {
			status = excess;
		    }
		}
	    }
	}
    }

    return status;
}


static int
seek_id3i(void *context, void *source, struct inter6_type *inter6_data)
{
    int status = 0;

    if (1) {
	if (127 < Source(source).s1.size) {
	    status = Mylene1f4_seek_file(context, source, -128, SEEK_END);
	    if (status != -1) {
		unsigned char buffer[128];

		status = Mylene1f4_read_file(context, source, buffer, 128);
		if (status != -1) {
		    if (status == 128) {
			if (!memcmp(buffer, "TAG", 3)) {
			    void *id3x;

			    id3x = malloc(128);
			    if (id3x) {
				inter6_data->id3x = id3x;

				memcpy(id3x, buffer, 128);

				ID3T(context) = Mylene1f4_MERGED_ID3TAG;
				ID3X(context) = id3x;
			    }
			}
		    }

		    status = Mylene1f4_seek_file(context, source, 0, SEEK_SET);
		}
	    }
	}
    }

    return status;
}


static int
seek_id3x(void *context, void *source, struct inter6_type *inter6_data)
{
    int status;

    if (FLAGS(context) & Mylene1f4_ID3V2_READ) {
	status = seek_id3e(context, source, inter6_data);
	if (status == 1) {
	    status = seek_id3i(context, source, inter6_data);
	}
    } else {
	if (1) {
	    status = seek_id3i(context, source, inter6_data);
	}
    }

    return status;
}


static int
seek_mpeg(void *e1f4_audio)
{
    struct Mylene1f4_audio_attributes_type attributes;

    attributes.abort_decoding = error_exit_2;
    attributes.do_release = do_release_1;
    if (1) {
	const unsigned *process;

	process = MPEG(PRIVATE_A(e1f4_audio, context)).process;
	if (process) {
	    attributes.process = *process;
	} else {
	    attributes.process = Mylene1f4_4294967291;
	}
    }

    Mylene1f4_set_audio_attributes
	(e1f4_audio, &attributes,
	 Mylene1f4_AUDIO_ABORT_DECODING_MASK | Mylene1f4_AUDIO_DO_RELEASE_MASK
	 | Mylene1f4_AUDIO_PROCESS_MASK, 0);

    return 0;
}


#if __ENABLE_SEEK_WINDOW__
static int
seek_page(struct inter6_type *inter6_data)
{
    void *reflex;

    inter6_data->access = NULL;

    reflex = inter6_data->reflex;
    if (reflex) {
	int (*index_frame) (void *, unsigned *);
	unsigned start2;
	void *context;

	context = inter6_data->context;

	index_frame = START3(context);
	if (index_frame) {
	    index_frame(context, &start2);
	} else {
	    if (SELECT(context) == Mylene1f4_PAGE_PICKUP) {
		start2 = START2(context);
	    } else {
# if 1
		start2 = 0;
# else
		unsigned frame_rate_d, frame_rate_n;

		frame_rate_d = x1f42a_data->frame_rate_d;
		frame_rate_n = x1f42a_data->frame_rate_n;
		if (!~(frame_rate_n & frame_rate_d)) {
		    start2 = 0;
		} else {
		    pick_page(context, &start2, frame_rate_n, frame_rate_d);
		}
# endif				/* 1 */
	    }
	}

	Mylene1f4_seek_socket(reflex, &start2, &inter6_data->access);
	if (inter6_data->access) {
	    HEAD(context) = start2;
	}
    }

    return 0;
}
#endif				/* __ENABLE_SEEK_WINDOW__ */


static int
set_output(void *e1f4_audio)
{
    int status;

    if ((status = setjmp(PRIVATE_A(e1f4_audio, env)))) {
	stat_failure(e1f4_audio, status);

	status = 1;
    } else {
	struct Mylene1f4_header_type header;
	unsigned stream;

	if (SIZE(e1f4_audio) - SYNCWORD_OFFSET(e1f4_audio) < 4) {
	    MINIMUM_USEFUL_SIZE(e1f4_audio) = 4 + SYNCWORD_OFFSET(e1f4_audio);
	    Mylene1f4_correct_underflow(e1f4_audio);
	}

	{
	    unsigned char *data;

	    data = BUFFER(e1f4_audio) + SYNCWORD_OFFSET(e1f4_audio);
#if WORDS_BIGENDIAN
	    stream = data[0] << 030 | data[1] << 020 | data[2] << 010
		| data[3];
#else
	    stream = data[0] | data[1] << 010 | data[2] << 020
		| data[3] << 030;
#endif				/* WORDS_BIGENDIAN */
	}

	Mylene1f4_header_semantics(stream, &header);

	if (!header.decoder) {
	    longjmp(PRIVATE_A(e1f4_audio, env), SYNCHRONIZATION_FAILURE);
	}

	{
	    void *context;

	    context = PRIVATE_A(e1f4_audio, context);

	    Mylene1f4_sampling_frequency(&header, &SAMPLING(context));
	    if (header.mode == Mylene1f4_single_channel) {
		CHANNELS(context) = 1;
	    } else {
		CHANNELS(context) = 2;
	    }

	    {
		unsigned release_size;

		Mylene1f4_release_size(stream, &release_size);
		RELEASE(context) = release_size << 1;
	    }

	    {
		struct sequel_type *sequel_data;
		struct xmpeg1_type *xmpeg1_data;

		Mylene1f4_find_sequel
		    (context, Mylene1f4_XMPEG1_SEQUEL, (void *) &sequel_data);
		xmpeg1_data = sequel_data->data;
		xmpeg1_data->layer = header.layer;
		xmpeg1_data->mode = header.mode;
		xmpeg1_data->mode_extension = header.mode_extension;
		xmpeg1_data->specification = header.ID;
		Mylene1f4_bitrate(&header, &xmpeg1_data->bitrate);
		if (xmpeg1_data->bitrate == Mylene1f4_FORBIDDEN
		    || xmpeg1_data->bitrate == Mylene1f4_FREE_FORMAT) {
		    xmpeg1_data->bitrate = ~0;
		}
		Mylene1f4_framerate(&header, &xmpeg1_data->frame_rate_n,
				    &xmpeg1_data->frame_rate_d);
#if !defined __Mylene1f4_framerate_string_defined__
		xmpeg1_data->framerate = "not available";
#elif !__Mylene1f4_framerate_string_defined__
		xmpeg1_data->framerate = "not available";
#else
		Mylene1f4_framerate_string(&header, &xmpeg1_data->framerate);
#endif				/* !__Mylene1f4_framerate_string_defined__ */
	    }
	}
    }

    return status;
}


static int
skip_mpeg(void *e1f4_audio)
{
    struct Mylene1f4_audio_attributes_type attributes;

    attributes.abort_decoding = error_exit_1;
    attributes.do_release = do_release_0;
    attributes.process = Mylene1f4_UTTERLY_FUTILE;

    Mylene1f4_set_audio_attributes
	(e1f4_audio, &attributes,
	 Mylene1f4_AUDIO_ABORT_DECODING_MASK | Mylene1f4_AUDIO_DO_RELEASE_MASK
	 | Mylene1f4_AUDIO_PROCESS_MASK, 0);

    return 0;
}


static int
stat_1f44(void *context, const char *none)
{
    Mylene1f4_fprintf_error(context, "%s", none);

    return 0;
}


static int
tail_hell(void *context)
{
    Mylene1f4_fprintf_error(context, "\n");

    return 0;
}


static int
type_hell(void *context, const char *hell, unsigned size)
{
    Mylene1f4_fprintf_error(context, "%.*s", (int) size, hell);

    return 0;
}


static int
type_mpeg(void *loader, void *target)
{
    struct target_type *target_data;

    target_data = target;
    *target_data = data_target;

    return 0;
}


static int
wind_mpeg(void *loader, unsigned *frame, unsigned scale)
{
    if (1) {
	unsigned class;

	if (4 < scale) {
	    scale = 4;
	}

	class = *frame;

	*frame = class - scale;
    }

    return 0;
}


static void
do_release_0(void *e1f4_whatever, struct Mylene1f4_release_type *release)
{
}


static void
do_release_1(void *e1f4_whatever, struct Mylene1f4_release_type *release)
{
}


static void
error_exit_1(void *e1f4_audio)
{
    if (ERROR(e1f4_audio) == Mylene1f4_AUDIO_INVALID_BACKWARD_SEEK) {
    } else {
	error_exit_2(e1f4_audio);
    }
}


static void
error_exit_2(void *e1f4_audio)
{
    int error, exit_2 = 1;

    error = AUDIO_FAILURE;

    switch (ERROR(e1f4_audio)) {
    case Mylene1f4_AUDIO_FUCKED_UP_AUDIO_HEADER:
	if (PRIVATE_A(e1f4_audio, synchronization)) {
	    error = SYNCHRONIZATION_FAILURE;
	}
	break;
    case Mylene1f4_AUDIO_INVALID_BACKWARD_SEEK:
	exit_2 = 0;
	error_exit_3(e1f4_audio);
	break;
    case Mylene1f4_AUDIO_SHORT_SAMPLES_SECTION:
	error_exit_4(e1f4_audio);
	if (1) {
	    error = SHORT_SAMPLES_SECTION;
	}
    }

    if (exit_2) {
	longjmp(PRIVATE_A(e1f4_audio, env), error);
    }
}


static void
error_exit_3(void *e1f4_audio)
{
    void *context;

    context = PRIVATE_A(e1f4_audio, context);

    if (!BACKSEEK(context)) {
	stat_failure(e1f4_audio, AUDIO_FAILURE);
    }

    memset(AUDIO(context) + C1f4(context), 0, RELEASE(context));
}


static void
error_exit_4(void *e1f4_audio)
{
    void *context;

    context = PRIVATE_A(e1f4_audio, context);

    if (1) {
	stat_failure(e1f4_audio, AUDIO_FAILURE);
    }

    memset(AUDIO(context) + C1f4(context), 0, RELEASE(context));
}


static void
header_hack_1(struct Mylene1f4_header_type *header)
{
    void *context, *e1f4_audio;

    e1f4_audio = header->decoder;

    context = PRIVATE_A(e1f4_audio, context);

    {
	unsigned sampling_frequency;

	Mylene1f4_sampling_frequency(header, &sampling_frequency);
	if (sampling_frequency != SAMPLING(context)) {
	    longjmp(PRIVATE_A(e1f4_audio, env), CHANGED_SAMPLING);
	}
    }
    if (CHANNELS(context) == 1) {
	if (header->mode != Mylene1f4_single_channel) {
	    longjmp(PRIVATE_A(e1f4_audio, env), CHANGED_MODE);
	}
    } else {
	if (header->mode == Mylene1f4_single_channel) {
	    longjmp(PRIVATE_A(e1f4_audio, env), CHANGED_MODE);
	}
    }

    PRIVATE_A(e1f4_audio, synchronization) = 1;

    {
	struct Mylene1f4_audio_attributes_type attributes;

	attributes.header_hack_1 = NULL;
	attributes.header_hack_2 = header_hack_2;
#if __USE_HEADER_HACK_3__
	attributes.header_hack_3 = header_hack_3;
#endif				/* __USE_HEADER_HACK_3__ */

	Mylene1f4_set_audio_attributes
	    (e1f4_audio, &attributes, 0,
	     Mylene1f4_AUDIO_HEADER_HACK_1_MASK
	     | Mylene1f4_AUDIO_HEADER_HACK_2_MASK
#if __USE_HEADER_HACK_3__
	     | Mylene1f4_AUDIO_HEADER_HACK_3_MASK
#endif				/* __USE_HEADER_HACK_3__ */
	     | 0);
    }
}


static void
header_hack_2(struct Mylene1f4_header_type *header)
{
    void *context, *e1f4_audio;

    e1f4_audio = header->decoder;

    context = PRIVATE_A(e1f4_audio, context);

    {
	unsigned sampling_frequency;

	Mylene1f4_sampling_frequency(header, &sampling_frequency);
	if (sampling_frequency != SAMPLING(context)) {
	    longjmp(PRIVATE_A(e1f4_audio, env), CHANGED_SAMPLING);
	}
    }
#if !__USE_HEADER_HACK_3__
    if (CHANNELS(context) == 1) {
	if (header->mode != Mylene1f4_single_channel) {
	    longjmp(PRIVATE_A(e1f4_audio, env), CHANGED_MODE);
	}
    } else {
	if (header->mode == Mylene1f4_single_channel) {
	    longjmp(PRIVATE_A(e1f4_audio, env), CHANGED_MODE);
	}
    }
#endif				/* !__USE_HEADER_HACK_3__ */
    {
	struct xmpeg1_type *xmpeg1_data;

	xmpeg1_data =
	    ((struct inter6_type *) PRIVATE_A(e1f4_audio, loader))->xmpeg1;
	if (header->layer != xmpeg1_data->layer) {
	    longjmp(PRIVATE_A(header->decoder, env), CHANGED_HEADER);
	} else if (header->ID != xmpeg1_data->specification) {
	    longjmp(PRIVATE_A(header->decoder, env), CHANGED_HEADER);
	}
    }
}


#if __USE_HEADER_HACK_3__
static void
header_hack_3(struct Mylene1f4_header_type *header)
{
    longjmp(PRIVATE_A(header->decoder, env), CHANGED_MODE);
}
#endif				/* __USE_HEADER_HACK_3__ */


static void
page_lock(void *e1f4_audio, struct inter6_type *inter6_data)
{
    int excess;
    lcard_type stream, system;

    excess = seek_e1f4(e1f4_audio, &stream, &system);

    /*
     * stupid, but I'll ignore the seek errors...
     *
     * ... and that's perhaps less than smart especially considering that
     * the internal (re)mapper that commanded the seek information should
     * probably be informed 'bout seek failure.
     *
     * FEATURES
     */
    Mylene1f4_11172_frame(e1f4_audio);

    {
	if (!excess) {
#if __have_Mylene1f4_get_allocation__
	    unsigned count;
#else
# define count				0
#endif				/* __have_Mylene1f4_get_allocation__ */

#if __have_Mylene1f4_get_allocation__
	    Mylene1f4_get_allocation(e1f4_audio, &count, 0, NULL);
#endif				/* __have_Mylene1f4_get_allocation__ */

	    inter6_data->socket.call
		(inter6_data->socket.back, count, &stream, &system);

#if __have_Mylene1f4_get_allocation__
#else
# undef count
#endif				/* __have_Mylene1f4_get_allocation__ */
	}
    }
}


static void
stat_audio(void *e1f4_audio, void *context)
{
    char *error_string;

    Mylene1f4_audio_error_string(ERROR(e1f4_audio), &error_string);
    Mylene1f4_fprintf_error
	(context, "audio parse error (frame %u)", REFERENCE(context));
    stat_device(e1f4_audio, context);
    Mylene1f4_fprintf_error(context, " (%s)\n", error_string);
}


static void
stat_device(void *e1f4_audio, void *context)
{
    lcard_type offset;

    Mylene1f4_fprintf_error(context, " in ");
    Mylene1f4_stat_source(context);
    if (!deck_mpeg(e1f4_audio, &offset)) {
	Mylene1f4_fprintf_error(context, " at ");
	Mylene1f4_fprintf_offset(context, &offset);
    }
}


static void
stat_failure(void *e1f4_audio, int error)
{
    void *context;

    context = PRIVATE_A(e1f4_audio, context);

    Mylene1f4_fprintf_error(context, "%s: ", Mylene1f4(context));
    if (error == AUDIO_FAILURE) {
	stat_audio(e1f4_audio, context);
    } else if (error == CHANGED_HEADER) {
	Mylene1f4_fprintf_error(context, "the frame header in ");
	Mylene1f4_stat_source(context);
	Mylene1f4_fprintf_error
	    (context, " changed beyond my abilities to reset audio output\n");
    } else if (error == CHANGED_MODE) {
	Mylene1f4_fprintf_error(context, "the mode (channels count) in ");
	Mylene1f4_stat_source(context);
	Mylene1f4_fprintf_error
	    (context, " changed beyond my abilities to reset audio output\n");
    } else if (error == CHANGED_SAMPLING) {
	Mylene1f4_fprintf_error(context, "the sampling in ");
	Mylene1f4_stat_source(context);
	Mylene1f4_fprintf_error
	    (context, " changed beyond my abilities to reset audio output\n");
    } else if (error == Mylene1f4_ADVANCE_FAILURE) {
	Mylene1f4_fprintf_error(context, "bad ");
	Mylene1f4_stat_source(context);
	Mylene1f4_fprintf_error
	    (context, " 1f42 stream map packet access frame record\n");
    } else if (error == Mylene1f4_MALLOC_FAILURE) {
	Mylene1f4_fprintf_error(context, "%s\n", strerror(errno));
    } else if (error == Mylene1f4_SYSTEM_FAILURE) {
	Mylene1f4_fprintf_error(context, "system parse error");
	stat_device(e1f4_audio, context);
	Mylene1f4_stat_system(e1f4_audio, context);
	Mylene1f4_fprintf_error(context, "\n");
    } else if (error == Mylene1f4_UNEXPECTED_EOF) {
	Mylene1f4_fprintf_error(context, "unexpected EOF");
	stat_device(e1f4_audio, context);
	Mylene1f4_fprintf_error(context, "\n");
    } else if (error == Mylene1f4_UNEXPECTED_EOS) {
	Mylene1f4_fprintf_error(context, "unexpected EOS");
	stat_device(e1f4_audio, context);
	Mylene1f4_fprintf_error(context, "\n");
    } else if (error == Mylene1f4_e66c_ERROR) {
	seek_1f44_stat
	    (PRIVATE_A(e1f4_audio, e66c), SOURCE(context)._.s1.path, context,
	     stat_1f44);
	Mylene1f4_fprintf_error(context, "\n");
    } else if (error == SEEK_EXCEEDED) {
	Mylene1f4_fprintf_error
	    (context, "exceeded seek limit (%u) while looking for mpeg data "
	     "in ", MPEG(context).seek);
	Mylene1f4_stat_source(context);
	Mylene1f4_fprintf_error(context, "\n");
    } else if (error == SYNCHRONIZATION_EXHAUSTED) {
	Mylene1f4_stat_source(context);
	Mylene1f4_fprintf_error
	    (context, " synchronization failed, cannot find mpeg data\n");
    } else if (error == SYNCHRONIZATION_FAILURE) {
	Mylene1f4_stat_source(context);
	Mylene1f4_fprintf_error(context, " synchronization failure\n");
    }
}


int
Mylene1f4_clip_mpeg(void *loader, unsigned clip)
{
    struct inter6_type *inter6_data;

    inter6_data = loader;

    PRIVATE_A(inter6_data->e1f4_audio, clip) = clip;

    return 0;
}


int
Mylene1f4_copy_mpeg(void *context, void *target)
{
    struct target_type *target_data;

    target_data = target;
    *target_data = data_target;

    return 0;
}


int
Mylene1f4_dump_mpeg(void *context, void *source, unsigned *count,
		    void **loader)
{
    int status;
    struct inter6_type *inter6_data;

    inter6_data = *loader;

    status = fini_e1f4(context, inter6_data);

    free(inter6_data);

    return status;
}


int
Mylene1f4_load_mpeg(void *context, void *source, unsigned *count,
		    void **loader)
{
    int status;
    struct inter6_type *inter6_data;

    inter6_data = (struct inter6_type *) malloc(sizeof(struct inter6_type));
    if (!inter6_data) {
	Mylene1f4_fprintf_error
	    (context, "%s: %s\n", Mylene1f4(context), strerror(errno));

	status = -1;
    } else {
	status = init_e1f4(context, source, inter6_data);
	if (status) {
	    free(inter6_data);
	} else {
	    Mylene1f4_audio_type e1f4_audio;

	    e1f4_audio = inter6_data->e1f4_audio;

	    if (*loader) {
		status = seek_1f42(loader, source, e1f4_audio);
	    } else {
#if __ENABLE_SEEK_WINDOW__
		ceil_page(context, inter6_data);
#endif				/* __ENABLE_SEEK_WINDOW__ */

#if __ENABLE_TAGGED_LIST__
		if (FLAGS(context) & Mylene1f4_BREAK_LIST) {
		} else {
#endif				/* __ENABLE_TAGGED_LIST__ */
		    status = seek_id3x(context, source, inter6_data);
#if __ENABLE_TAGGED_LIST__
		}
#endif				/* __ENABLE_TAGGED_LIST__ */
	    }
	    if (!status) {
		do {
		    if (SYNCHRONIZE(context)) {
			status = seek_data(e1f4_audio, context);
			if (status) {
			    break;
			}
		    }

		    status = set_output(e1f4_audio);
		} while (0);
	    }
	    if (status) {
		fini_e1f4(context, inter6_data);

		free(inter6_data);
	    } else {
		*count = 1;
		*loader = inter6_data;

		inter6_data->context = context;
	    }
	}
    }

    return status;
}


int
Mylene1f4_node_mpeg(void *context, const void **address)
{
    *address = &data_target;

    return 0;
}


int
Mylene1f4_type_mpeg(void *loader, void *target)
{
    return type_mpeg(loader, target);
}
