/*
 * note.c
 * Copyright (C) 2010-2013, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if __ENABLE_EQUEUE__
# include <string.h>
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# include <access.h>
# include <defs.h>
# include <excess4.h>
# include <excess7.h>
# include <inter.h>
# include <types.h>
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
# define Frame4(context) \
    ((struct frame4_type *) &Frame7((context))->_1.frame4)
# define Frame7(context) \
    ((struct frame7_type *) WINDOW((context)).server.window)
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
static int link_dana(struct frame4_type *, unsigned, unsigned, void **);
static int link_decq(struct iqueue_type *, unsigned, unsigned, void **);
static int link_list(struct stripe_type *, unsigned, unsigned, void **);
static int link_note(struct frame4_type *, struct equeue_type *, unsigned,
		     unsigned, unsigned, const char *);
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
static int
link_dana(struct frame4_type *frame4_data, unsigned major, unsigned minor,
	  void **equeue)
{
    int delete = 1;

    if (frame4_data->flags & Mylene1f4_AIME_LINKED) {
	struct equeue_type *equeue_data;

	equeue_data = frame4_data->dana.eclass;
	if ((equeue_data->major ^ major) | (equeue_data->minor ^ minor)) {
	} else {
	    delete = 0;

	    *equeue = equeue_data;
	}
    }

    return delete;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
link_decq(struct iqueue_type *iqueue_data, unsigned major, unsigned minor,
	  void **equeue)
{
    int delete = 1;
    unsigned lqueue;

    lqueue = iqueue_data->lqueue;
    if (lqueue) {
	struct equeue_type *equeue_data;

	equeue_data = iqueue_data->head_equeue;
	do {
	    if ((equeue_data->major ^ major) | (equeue_data->minor ^ minor)) {
	    } else {
		delete = 0;

		*equeue = equeue_data;

		break;
	    }

	    lqueue--;

	    equeue_data = equeue_data->equeue_data;
	} while (lqueue);
    }

    return delete;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
link_list(struct stripe_type *stripe_data, unsigned major, unsigned minor,
	  void **equeue)
{
    int delete = 1;

    if (stripe_data) {
	delete = link_decq(&stripe_data->iqueue, major, minor, equeue);
    }

    return delete;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
static int
link_note(struct frame4_type *frame4_data, struct equeue_type *equeue_data,
	  unsigned major, unsigned minor, unsigned size, const char *text)
{
    struct equeue_type *dana_equeue;
    void *data;

    dana_equeue = frame4_data->dana.equeue;

    frame4_data->dana.equeue = equeue_data;

    /*
     * may not consider freeing _report_data_ memory for it will be freed by
     * the moon deallocator
     */

    if (Mylene1f4_link_moon
	(frame4_data, &data, sizeof(struct report_type) + size)) {
    } else {
	struct report_type *report_data, *tail_report;

	report_data = data;

	tail_report = equeue_data->tail_report;

	report_data->report_data = NULL;
	equeue_data->tail_report = report_data;

	if (tail_report) {
	    tail_report->report_data = report_data;
	} else {
	    equeue_data->head_report = report_data;
	}

	report_data->size = size;

	memcpy(report_data + 1, text, size);
    }

    frame4_data->dana.equeue = dana_equeue;

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_link_note(void *context, unsigned major, unsigned minor,
		    unsigned size, const char *text)
{
    void *equeue;

    if (Mylene1f4_pick_note(context, major, minor, &equeue)) {
	link_note(Frame4(context), equeue, major, minor, size, text);
    }

    return 0;
}
#endif				/* __ENABLE_EQUEUE__ */


#if __ENABLE_EQUEUE__
int
Mylene1f4_pick_note(void *context, unsigned major, unsigned minor,
		    void **equeue)
{
    int false = 1;
    struct frame4_type *frame4_data;

    frame4_data = Frame4(context);
    if (link_decq(&frame4_data->iqueue, major, minor, equeue)) {
	if (link_dana(frame4_data, major, minor, equeue)) {
	    if (link_list(frame4_data->clip_stripe, major, minor, equeue)) {
		if (link_list
		    (frame4_data->play_stripe, major, minor, equeue)) {
		    false = 0;
		}
	    }
	}
    }

    return false;
}
#endif				/* __ENABLE_EQUEUE__ */
