/*
 * socket.c
 * Copyright (C) 2007-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if __ENABLE_SEEK_WINDOW__
# include <stddef.h>
# include <stdlib.h>
#endif				/* __ENABLE_SEEK_WINDOW__ */

#if __ENABLE_SEEK_WINDOW__
# include <access.h>
# include <lcard.h>
# include <types.h>
#endif				/* __ENABLE_SEEK_WINDOW__ */

#if __ENABLE_SEEK_WINDOW__
# define ACCESS_RANK			9
#endif				/* __ENABLE_SEEK_WINDOW__ */

#if __ENABLE_SEEK_WINDOW__
typedef struct access_type {
    lcard_type stream, system;
    unsigned detail, temporal_reference;
} access_type;
#endif				/* __ENABLE_SEEK_WINDOW__ */

#if __ENABLE_SEEK_WINDOW__
typedef struct inter6_type {
    struct access_type *access_data, *miss_access;
    unsigned reflex, second;
    void *context, *socket;
} inter6_type;
#endif				/* __ENABLE_SEEK_WINDOW__ */

#if __ENABLE_SEEK_WINDOW__
static int miss_lock(void *, unsigned, void *, void *);
static int slip_lock(void *, unsigned, void *, void *);
#endif				/* __ENABLE_SEEK_WINDOW__ */

#if __ENABLE_SEEK_WINDOW__
static struct access_type *seek_data(unsigned, struct access_type *, unsigned);
#endif				/* __ENABLE_SEEK_WINDOW__ */

#if __ENABLE_SEEK_WINDOW__
static int
miss_lock(void *inter, unsigned count, void *stream, void *system)
{
    struct access_type *access_data;
    struct inter6_type *inter6_data;
    struct socket_type *socket_data;

    inter6_data = inter;

    socket_data = inter6_data->socket;

    access_data = (struct access_type *)
	malloc(sizeof(struct access_type) << ACCESS_RANK);
    if (!access_data) {
	socket_data->call = NULL;
    } else {
	unsigned second;
	void *context;

	inter6_data->access_data = access_data;
	inter6_data->miss_access = access_data;

	context = inter6_data->context;

	second =
	    (SAMPLING(context) * CHANNELS(context) << 1) / RELEASE(context);

	inter6_data->second = second;

	if (count) {
	    count--;
	}

	access_data->detail = count;
	access_data->stream = *(lcard_type *) stream;
	access_data->system = *(lcard_type *) system;

	access_data->temporal_reference = 0;

	socket_data->page = second;

	socket_data->call = slip_lock;

	inter6_data->reflex++;

	inter6_data->miss_access = access_data + 1;
    }

    return 0;
}
#endif				/* __ENABLE_SEEK_WINDOW__ */


#if __ENABLE_SEEK_WINDOW__
static int
slip_lock(void *inter, unsigned count, void *stream, void *system)
{
    do {
	struct access_type *access_data;
	struct inter6_type *inter6_data;
	struct socket_type *socket_data;
	unsigned reference, reflex;

	inter6_data = inter;

	socket_data = inter6_data->socket;

	access_data = inter6_data->miss_access;

	reflex = inter6_data->reflex;
	if (!(reflex & (reflex - 1))) {
	    if (reflex < 1 << ACCESS_RANK) {
	    } else {
		access_data = (struct access_type *)
		    realloc(inter6_data->access_data,
			    sizeof(struct access_type) * (reflex << 1));
		if (!access_data) {
		    socket_data->call = NULL;
		    break;
		} else {
		    inter6_data->access_data = access_data;
		    access_data += reflex;
		    inter6_data->miss_access = access_data;
		}
	    }
	}

	reference = socket_data->page;

	if (count) {
	    count--;
	}
	if (reference < count) {
	    count = reference;
	}

	access_data->detail = count;
	access_data->stream = *(lcard_type *) stream;
	access_data->system = *(lcard_type *) system;

	access_data->temporal_reference = reference;

	socket_data->page = reference + inter6_data->second;

	inter6_data->reflex = reflex + 1;

	inter6_data->miss_access = access_data + 1;
    } while (0);

    return 0;
}
#endif				/* __ENABLE_SEEK_WINDOW__ */


#if __ENABLE_SEEK_WINDOW__
static struct access_type *
seek_data(unsigned nrecords, struct access_type *access_data, unsigned frame)
{
    nrecords--;

    if (access_data[nrecords].temporal_reference < frame + 1) {
	access_data += nrecords;
    } else {
	unsigned finish, half = 0, start = 0;

	finish = nrecords;
	while (start < finish) {
	    half = (finish + start) >> 1;
	    if (frame < access_data[half].temporal_reference) {
		finish = half;
	    } else {
		start = half + 1;
	    }
	}

	access_data += start;
	if (start) {
	    access_data--;
	}
    }

    return access_data;
}
#endif				/* __ENABLE_SEEK_WINDOW__ */


#if __ENABLE_SEEK_WINDOW__
int
Mylene1f4_back_socket(void *access, void *stream, void *system)
{
    lcard_type *output;

    output = stream;
    LCARD_SET_CARD2(*output, ((struct access_type *) access)->stream);
    output = system;
    LCARD_SET_CARD2(*output, ((struct access_type *) access)->system);

    return 0;
}
#endif				/* __ENABLE_SEEK_WINDOW__ */


#if __ENABLE_SEEK_WINDOW__
int
Mylene1f4_fini_socket(void **socket)
{
    struct inter6_type *inter6_data;

    inter6_data = *socket;

    free(inter6_data);

    return 0;
}
#endif				/* __ENABLE_SEEK_WINDOW__ */


#if __ENABLE_SEEK_WINDOW__
int
Mylene1f4_init_socket(void *context, void **socket, void *reflex)
{
    int status;
    struct inter6_type *inter6_data;

    inter6_data = (struct inter6_type *) malloc(sizeof(struct inter6_type));
    if (!inter6_data) {
	status = -1;
    } else {
	status = 0;

	if (1) {
	    struct socket_type *socket_data;

	    socket_data = reflex;

	    socket_data->back = inter6_data;
	    socket_data->call = miss_lock;
	    socket_data->page = 0;
	}

	inter6_data->socket = reflex;

	inter6_data->access_data = NULL;
	inter6_data->miss_access = NULL;

	inter6_data->context = context;

	inter6_data->reflex = 0;

	*socket = inter6_data;
    }

    return status;
}
#endif				/* __ENABLE_SEEK_WINDOW__ */


#if __ENABLE_SEEK_WINDOW__
int
Mylene1f4_seek_socket(void *inter6, unsigned *start2, void **access)
{
    struct access_type *access_data;
    struct inter6_type *inter6_data;
    unsigned reflex;

    inter6_data = inter6;

    reflex = inter6_data->reflex;
    if (!reflex) {
    } else {
	access_data = seek_data
	    (inter6_data->reflex, inter6_data->access_data, *start2);
	reflex = access_data - inter6_data->access_data;
	if (access_data->detail) {
	    unsigned target;

	    target = access_data->temporal_reference - access_data->detail;
	    while (reflex) {
		reflex--;
		access_data--;
		if (target < access_data->temporal_reference) {
		} else {
		    break;
		}
	    }
	}

	*access = access_data;

	*start2 = access_data->temporal_reference;
    }

    return 0;
}
#endif				/* __ENABLE_SEEK_WINDOW__ */
