/*
 * source2.c
 * Copyright (C) 2006-2010, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if defined HAVE_LIBLEWD
# include <lewd.h>
#endif				/* HAVE_LIBLEWD */
#include <string.h>

#include <access.h>
#if !defined HAVE_LIBLEWD
# include <basename.h>
#endif				/* !HAVE_LIBLEWD */
#include <defs.h>
#include <inter.h>
#if !defined HAVE_LIBLEWD
# include <cards.h>
#endif				/* !HAVE_LIBLEWD */

static int match_class(void *);
static int match_entry(void *);
static int match_field(const char *, unsigned, const char *, char *);
static int match_first(void *);
static int match_frame(const char *, unsigned, unsigned, struct id3tag_type *,
		       char *);
static int match_sixth(void *);

static int
match_class(void *context)
{
    int false = 0;

    do {
	if (ALBUM(context).data) {
	    break;
	}
	if (MATCH(context).data) {
	    break;
	}
	if (NOTES(context).data) {
	    break;
	}
	if (TITLE(context).data) {
	    break;
	}

	false = 1;
    } while (0);

    return false;
}


static int
match_entry(void *context)
{
    int false = 0;

    do {
	char *slack;
	const char *class, *slide;

	class = SOURCE(context)._.s1.path;

	slack = SLACK(context).data;

	slide = LBASE(context).data;
	if (slide) {
	    const char *entry;

	    entry = x1f4_basename(class);
	    if (x1f4_match_cards
		(slide, LBASE(context).size, entry, strlen(entry), slack)) {
		break;
	    }
	}
	slide = LPATH(context).data;
	if (slide) {
	    const char *entry;

	    entry = class;
	    if (x1f4_match_cards
		(slide, LPATH(context).size, entry, strlen(entry), slack)) {
		break;
	    }
	}

	false = 1;
    } while (0);

    return false;
}


static int
match_field(const char *major, unsigned bytes, const char *minor, char *slack)
{
    int false;
    unsigned count = 30;

    Mylene1f4_shrink_field(&minor, &count);
    false = x1f4_match_cards(major, bytes, minor, count, slack);

    return false;
}


static int
match_first(void *context)
{
    int false = 0;

    do {
	char *id3x, *slack;
	const char *slide;

	id3x = ID3X(context);

	slack = SLACK(context).data;

	slide = ALBUM(context).data;
	if (slide) {
	    if (match_field(slide, ALBUM(context).size, id3x + 0077, slack)) {
		break;
	    }
	}
	slide = MATCH(context).data;
	if (slide) {
	    if (match_field(slide, MATCH(context).size, id3x + 0041, slack)) {
		break;
	    }
	}
	slide = NOTES(context).data;
	if (slide) {
	    if (match_field(slide, NOTES(context).size, id3x + 0141, slack)) {
		break;
	    }
	}
	slide = TITLE(context).data;
	if (slide) {
	    if (match_field(slide, TITLE(context).size, id3x + 0003, slack)) {
		break;
	    }
	}

	false = 1;
    } while (0);

    return false;
}


static int
match_frame(const char *major, unsigned bytes, unsigned frame,
	    struct id3tag_type *id3tag_data, char *slack)
{
    return x1f4_match_cards
	(major, bytes, id3tag_data->data[frame], id3tag_data->size[frame],
	 slack);
}


static int
match_sixth(void *context)
{
    int false = 0;

    do {
	char *slack;
	const char *slide;
	void *id3v;

	id3v = ID3X(context);

	slack = SLACK(context).data;

	slide = ALBUM(context).data;
	if (slide) {
	    if (match_frame
		(slide, ALBUM(context).size, Mylene1f4_ALBUM_FIELD, id3v,
		 slack)) {
		break;
	    }
	}
	slide = MATCH(context).data;
	if (slide) {
	    if (match_frame
		(slide, MATCH(context).size, Mylene1f4_MATCH_FIELD, id3v,
		 slack)) {
		break;
	    }
	}
	slide = NOTES(context).data;
	if (slide) {
	    if (match_frame
		(slide, NOTES(context).size, Mylene1f4_NOTES_FIELD, id3v,
		 slack)) {
		break;
	    }
	}
	slide = TITLE(context).data;
	if (slide) {
	    if (match_frame
		(slide, TITLE(context).size, Mylene1f4_TITLE_FIELD, id3v,
		 slack)) {
		break;
	    }
	}

	false = 1;
    } while (0);

    return false;
}


int
Mylene1f4_clip_source(void *context)
{
    int status = 1;

    do {
	if (!match_entry(context)) {
	    break;
	}

	if (!ID3X(context)) {
	    if (!match_class(context)) {
		break;
	    }
	} else {
	    if (ID3T(context) == Mylene1f4_MERGED_ID3TAG) {
		if (!match_first(context)) {
		    break;
		}
	    } else {
		if (!match_sixth(context)) {
		    break;
		}
	    }
	}

#if __ENABLE_FILTER__
	if (1) {
	    void *e4deck;

	    e4deck = E4DECK(context);
	    if (e4deck) {
		int mode;

		if (!Mylene1f4_set_filter(context, e4deck, &mode)) {
		    if (!mode) {
			break;
		    }
		}
	    }
	}
#endif				/* __ENABLE_FILTER__ */

	status = 0;
    } while (0);

    return status;
}
