/*
 * state.h
 * Copyright (C) 2005-2012, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __Mylene1f4_STATE_H__
#define __Mylene1f4_STATE_H__

#include <config.h>

#include <signal.h>
#include <stdarg.h>
#include <stdio.h>

#include <sys/time.h>
#include <sys/types.h>

#include <lcard.h>
#include <types.h>

typedef struct printer_state {
    int (*run_sstring) (void *, const char *, unsigned),
	(*run_zstring) (void *, const char *);
    lcard_type deck, flow;
    union {
	FILE *file;
	struct buffer_type *buffer_data;
    } data;
    unsigned b1st, ccache, fflags, index, read, scope_failures;
    void *context, *extension, *format, *sa_time, *scope;
} printer_state;

typedef struct context_state {
    FILE *progress;
    const char *Mylene1f4;
    int completed, do_clear, do_explain, do_repeat, do_synchronize, fd_0, fd_1,
	(*index_frame) (void *, unsigned *),
	(*line_index) (void *, unsigned *),
	(*line_out) (void *, const char *, unsigned),
	no_backward_seek_complaints, (*patch_32) (void *),
	(*patch_64) (void *), (*run_mylene) (void *), shmid, start;
    pid_t pid;
    sig_atomic_t SIGCHLD_received;
    struct {
#if __ENABLE_MPEG_EQUALIZER__
	double amplification;
	int mode;
	struct l3band_type *l3band_data;
	unsigned count;
#endif				/* __ENABLE_MPEG_EQUALIZER__ */
	unsigned seek, synchronization, synchronization_sequence;
	const void *dct_64, *process;
    } mpeg;
    struct {
	struct {
	    char *data;
	    unsigned size;
	} slack;
	struct string_type album, lbase, lpath, match, notes, title;
    } text;
    struct clause_type *clause_data;
    const struct driver_type *driver_data;
    struct printer_state *printer_data;
    struct record_type *record_data;
    struct sequel_type *sequel_data;
    struct source_type source;
    struct target_type target;
    struct timeval becca;
#if __RESTART_Mylene0fa__
    struct timeval *restarts;
#endif				/* __RESTART_Mylene0fa__ */
    struct timeval *time;
#if __ENABLE_TAGGED_LIST__
    union {
	struct {
	    struct id3tag_type *id3tag_data;
	} tagged;
    } atomic;
#endif				/* __ENABLE_TAGGED_LIST__ */
    union {
	struct {
	    struct {
		char *wynona;
		unsigned max;
		void *window;
	    } list81;
	    struct {
		unsigned max;
	    } list82;
	    struct {
		char *file_name;
		unsigned clear, count, index;
		void *lock;
	    } list87;
	} e1;
    } excess;
    union {
	struct {
	    const char *unused1;
	} device;
	struct {
	    const char *process;
	} futile;
	struct {
	    const char *unused1;
	} header;
    } ground;
    union {
	struct extime_type time;
	unsigned page;
    } pickup;
#if __ENABLE_CONTROLLING_WINDOW__
    union {
	struct {
	    void *window;
	} extent;
# if __ENABLE_CONTROLLING_SERVER__
	struct {
	    const char *mode, *port;
	    int type;
# endif				/* __ENABLE_CONTROLLING_SERVER__ */
# if __ENABLE_CONTROLLING_SERVER__ && __RESTART_Mylene0fa__
	    unsigned attached_a, attached_f, attached_i;
# endif				/* __ENABLE_CONTROLLING_SERVER__ && __RES... */
# if __ENABLE_CONTROLLING_SERVER__
	    unsigned context_length;
# endif				/* __ENABLE_CONTROLLING_SERVER__ */
# if __ENABLE_EQUEUE_SERVER__
	    unsigned equeue_size;
# endif				/* __ENABLE_EQUEUE_SERVER__ */
# if __ENABLE_CONTROLLING_SERVER__
	    unsigned intermiss, interrupt, list_length, queue_length, setup,
		transfer;
	    void *iqueue, *window;
	} server;
# endif				/* __ENABLE_CONTROLLING_SERVER__ */
# if __ENABLE_x1f4LANES__
	struct {
	    void *channel, *evermiss, *window;
	} within;
# endif				/* __ENABLE_x1f4LANES__ */
    } window;
#else
    union {
	struct {
	    void *window;
	} extent;
    } window;
#endif				/* __ENABLE_CONTROLLING_WINDOW__ */
#if __ENABLE_x00696e6b__
    unsigned audio_entry;
#endif				/* __ENABLE_x00696e6b__ */
    unsigned audio_release_size;
#if __NEAR_LINK_BITS__
    unsigned bits;
#endif				/* __NEAR_LINK_BITS__ */
    unsigned cache_size, channels, class, clip, count, e1f4_index;
#if __ENABLE_FILTER__
    unsigned e4line;
#endif				/* __ENABLE_FILTER__ */
    unsigned flags, head, id3t, incoming, link, load_order, mode, outgoing,
	output_size, pending_0fa, play;
#if __RESTART_Mylene0fa__
    unsigned restarts_count;
#endif				/* __RESTART_Mylene0fa__ */
    unsigned sampling_frequency, select, size, *state, streams, tail,
	temporal_reference, wire;
    unsigned char input[0x10];
#if !__WAIT_VIOLET__
    void (*SIGCHLD_handler) (int);
#endif				/* !__WAIT_VIOLET__ */
    void *audio;
#if __ENABLE_FILTER__
    void *e4deck;
#endif				/* __ENABLE_FILTER__ */
#if __LINK_CELL_A52A_LOADER__
    void *ea52al;
#endif				/* __LINK_CELL_A52A_LOADER__ */
    void *eid3wl;
#if __ENABLE_FILTER__
    void *filter;
#endif				/* __ENABLE_FILTER__ */
    void *id3x, *loader, *mirror, *output,
	(*printf_error) (void *, const char *, va_list ap),
	*progress1, *progress2, *progress3;
    void *storage;
#if __WAIT_VIOLET__
    void *violet;
#endif				/* __WAIT_VIOLET__ */
    void *x1f42;
    const void *device;
} context_state;

#endif				/* __Mylene1f4_STATE_H__ */
