/*
 * stream.c
 * Copyright (C) 2005-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <e1f4.h>
#include <errno.h>
#include <setjmp.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/types.h>

#include <1f44.h>
#include <access.h>
#include <defs.h>
#include <inter.h>
#include <lcard.h>
#include <types.h>

#define AUDIO_BUFFER_SIZE		(1u << 18)

#undef SIZE

#define BUFFER(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->buffer)
#define MINIMUM_USEFUL_SIZE(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->minimum_useful_size)
#define PRIVATE(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->private)
#define SIZE(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->size)

#define PRIVATE__(e1f4, member) \
    (((struct stream_type *) (e1f4))->member)
#define PRIVATE_A(e1f4, member) \
    PRIVATE__(PRIVATE(e1f4), member)

#define Source(source) \
    ((struct source_type *) (source))->_

#if __USE_1f44_EEAD__
# define try_read(e66c, data, size) \
    (e66c)->eead((e66c), (data), (size))
#endif				/* __USE_1f44_EEAD__ */

#if !__USE_1f44_EEAD__
static int try_read(struct seek_1f44_type *, char *, unsigned);
#endif				/* !__USE_1f44_EEAD__ */

static void deck_stream(void *);

#if !__USE_1f44_EEAD__
static int
try_read(struct seek_1f44_type *e66c, char *data, unsigned size)
{
    int status = 0;

    while (1) {
	int success;

	success = e66c->read(e66c, data, size);
	if (success != -1) {
	    if (!success) {
		break;
	    } else {
		data += success;
		size -= success;
		status += success;
		if (!size) {
		    break;
		}
	    }
	} else {
	    if (errno == EINTR) {
	    } else {
		status = -1;
		break;
	    }
	}
    }

    return status;
}
#endif				/* !__USE_1f44_EEAD__ */


static void
deck_stream(void *stream)
{
    unsigned minimum_useful_size;
    void *data;

    minimum_useful_size = MINIMUM_USEFUL_SIZE(stream);

    data = PRIVATE_A(stream, data);

    if (!SIZE(stream)) {
	unsigned size;

	if (!data) {
	    size = minimum_useful_size;
	    if (size < AUDIO_BUFFER_SIZE) {
		size = AUDIO_BUFFER_SIZE;
	    }
	    data = malloc(size);
	    if (!data) {
		longjmp(PRIVATE_A(stream, env), Mylene1f4_MALLOC_FAILURE);
	    }
	    PRIVATE_A(stream, data) = data;
	    PRIVATE_A(stream, size) = size;
	} else {
	    size = PRIVATE_A(stream, size);
	    if (size < minimum_useful_size) {
		size = minimum_useful_size;
		data = realloc(data, size);
		if (!data) {
		    longjmp(PRIVATE_A(stream, env), Mylene1f4_MALLOC_FAILURE);
		}
		PRIVATE_A(stream, data) = data;
		PRIVATE_A(stream, size) = size;
	    }
	}

	{
	    int read;

	    read = try_read(PRIVATE_A(stream, e66c), data, size);
	    if (read < 0) {
		longjmp(PRIVATE_A(stream, env), Mylene1f4_e66c_ERROR);
	    } else {
		if (read < minimum_useful_size) {
		    longjmp(PRIVATE_A(stream, env), Mylene1f4_UNEXPECTED_EOF);
		}
	    }

	    SIZE(stream) = read;
	    BUFFER(stream) = data;
	}
    } else {
	unsigned size;

	if (PRIVATE_A(stream, size) < minimum_useful_size) {
	    if (BUFFER(stream) != data) {
		memmove(data, BUFFER(stream), SIZE(stream));
		BUFFER(stream) = data;
	    }
	    data = realloc(data, minimum_useful_size);
	    if (!data) {
		longjmp(PRIVATE_A(stream, env), Mylene1f4_MALLOC_FAILURE);
	    }
	    PRIVATE_A(stream, data) = data;
	    PRIVATE_A(stream, size) = minimum_useful_size;

	    size = minimum_useful_size - SIZE(stream);
	} else {
	    size = (char *) BUFFER(stream) - (char *) PRIVATE_A(stream, data);
	    size = PRIVATE_A(stream, size) - size;
	    if (size < minimum_useful_size) {
		memmove(data, BUFFER(stream), SIZE(stream));
		BUFFER(stream) = data;

		size = PRIVATE_A(stream, size) - SIZE(stream);
	    } else {
		size -= SIZE(stream);
	    }
	}

	{
	    int read;

	    read = try_read(PRIVATE_A(stream, e66c),
			    (char *) (BUFFER(stream) + SIZE(stream)), size);
	    if (read < 0) {
		longjmp(PRIVATE_A(stream, env), Mylene1f4_e66c_ERROR);
	    } else {
		if (SIZE(stream) + read < minimum_useful_size) {
		    longjmp(PRIVATE_A(stream, env), Mylene1f4_UNEXPECTED_EOF);
		}
	    }

	    SIZE(stream) += read;
	}
    }
}


int
Mylene1f4_link_stream(void *stream)
{
    struct Mylene1f4_audio_attributes_type attributes;

    attributes.correct_underflow = deck_stream;

    Mylene1f4_set_audio_attributes
	(stream, &attributes, Mylene1f4_AUDIO_CORRECT_UNDERFLOW_MASK, 0);

    return 0;
}


int
Mylene1f4_seek_stream(void *stream, unsigned positive, unsigned negative,
		      void *Stream, void *System)
{
    int status;
    lcard_type offset;
    struct seek_1f44_type *e66c;

    e66c = PRIVATE_A(stream, e66c);

    if (e66c->tell(e66c, &offset)) {
	void *context;

	context = PRIVATE_A(stream, context);

	Mylene1f4_fprintf_error
	    (context, "%s: cannot seek ", Mylene1f4(context));
	Mylene1f4_stat_source(context);
	Mylene1f4_fprintf_error(context, ": %s\n", strerror(errno));

	status = 1;
    } else {
	status = 0;

	LCARD_ADD_CARD1(offset, positive);
	LCARD_SUBTRACT_CARD1(offset, negative);
	if (Stream) {
	    lcard_type *output;

	    output = Stream;
	    LCARD_SET_CARD2(*output, offset);
	}
	if (System) {
	    lcard_type *output;

	    output = System;
	    LCARD_SET_CARD2(*output, offset);
	}
    }

    return status;
}
