/*
 * system.c
 * Copyright (C) 2003-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * BUGS
 *
 *.looks like _seek_system is expecting that some packets corresponding audio
 * stream have been read.  If that is not the case, the behavior is at best
 * undefined.
 */

#include <config.h>

#include <e1f4.h>
#include <errno.h>
#include <h262.h>
#include <setjmp.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/types.h>

#include <1f44.h>
#include <access.h>
#include <c213.h>
#include <defs.h>
#include <inter.h>
#include <lcard.h>
#include <mpeg.h>
#include <system.h>
#include <types.h>

#if WORDS_BIGENDIAN
# define MPEG_program_end_code		0x000001b9
# define first_audio			0x000001c0
# define pack_start_code		0x000001ba
# define system_header_start_code	0x000001bb
#else
# define MPEG_program_end_code		0xb9010000
# define first_audio			0xc0010000
# define pack_start_code		0xba010000
# define system_header_start_code	0xbb010000
#endif				/* WORDS_BIGENDIAN */

#define MISS_FAILURE			(Mylene1f4__ERROR + 0x00)
#define READ_FAILURE			(Mylene1f4__ERROR + 0x01)

#if WORDS_BIGENDIAN
# define start_code(start_code)		((start_code) & 255)
#else
# define start_code(start_code)		((start_code) >> 24)
#endif				/* WORDS_BIGENDIAN */

#define Buffer(h262) \
    (((struct seek_h262_whatever_type *) (h262))->buffer)
#define Error(h262) \
    (((struct seek_h262_whatever_type *) (h262))->error)
#define Private(h262) \
    (((struct seek_h262_whatever_type *) (h262))->private)
#define Size(h262) \
    (((struct seek_h262_whatever_type *) (h262))->size)
#define Start_code(h262) \
    (((struct seek_h262_whatever_type *) (h262))->start_code)

#define Private__(whatever, member) \
    (((struct system_type *) whatever)->member)
#define Private_S(whatever, member) \
    Private__(Private(whatever), member)

#undef SIZE

#define BUFFER(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->buffer)
#define MINIMUM_USEFUL_SIZE(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->minimum_useful_size)
#define PRIVATE(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->private)
#define SIZE(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->size)

#define PRIVATE__(whatever, member) \
    (((struct stream_type *) whatever)->member)
#define PRIVATE_A(whatever, member) \
    PRIVATE__(PRIVATE(whatever), member)

#define Source(source) \
    ((struct source_type *) (source))->_

#if __USE_1f44_EEAD__
# define try_read(e66c, data, size) \
    (e66c)->eead((e66c), (data), (size))
#endif				/* __USE_1f44_EEAD__ */

typedef struct simple_type {
    char buffer[1024];
    jmp_buf *env;
    unsigned excess, seek;
    void *context, *source;
} simple_type;

#if !__USE_1f44_EEAD__
static int try_read(struct seek_1f44_type *, char *, unsigned);
#endif				/* !__USE_1f44_EEAD__ */

static void correct_underflow_0(void *);
static void correct_underflow_1(void *);
static void correct_underflow_2(void *);
static void error_exit_0(void *);
static void error_exit_2(void *);
static void seek_stream(void *, unsigned);
static void stat_failure(void *, int, void *);

#if !__USE_1f44_EEAD__
static int
try_read(struct seek_1f44_type *e66c, char *data, unsigned size)
{
    int status = 0;

    while (1) {
	int success;

	success = e66c->read(e66c, data, size);
	if (success != -1) {
	    if (!success) {
		break;
	    } else {
		data += success;
		size -= success;
		status += success;
		if (!size) {
		    break;
		}
	    }
	} else {
	    if (errno == EINTR) {
	    } else {
		status = -1;
		break;
	    }
	}
    }

    return status;
}
#endif				/* !__USE_1f44_EEAD__ */


static void
correct_underflow_0(void *system)
{
    char *buffer;
    struct simple_type *simple_data;
    unsigned size;

    simple_data = Private(system);

    buffer = simple_data->buffer;

    size = Size(system);

    if (size) {
	memmove(buffer, Buffer(system), size);
    }
    {
	int status;

	status = Mylene1f4_read_file
	    (simple_data->context, simple_data->source, buffer + size,
	     1024 - size);
	if (status == -1) {
	    longjmp(*simple_data->env, READ_FAILURE);
	}

	size += status;
    }
    if (size < 64) {
	unsigned access, excess;

	access = 64 - size;
	excess = simple_data->excess + access;
	if (excess < 64) {
	    simple_data->excess = excess;
	    memset(buffer + size, 0, access);
	} else {
	    longjmp(*simple_data->env, Mylene1f4_UNEXPECTED_EOF);
	}

	size = 64;
    }

    Size(system) = size;
    Buffer(system) = (unsigned char *) buffer;
}


static void
correct_underflow_1(void *system)
{
    char *buffer;
    struct simple_type *simple_data;
    unsigned seek, size;

    simple_data = Private(system);

    buffer = simple_data->buffer;

    seek = simple_data->seek;

    size = Size(system);

    if (size) {
	memmove(buffer, Buffer(system), size);
    }
    {
	int status;
	unsigned S;

	S = 1024 - size;
	if (seek < S) {
	    S = seek;
	}

	status = Mylene1f4_read_file
	    (simple_data->context, simple_data->source, buffer + size, S);
	if (status == -1) {
	    longjmp(*simple_data->env, READ_FAILURE);
	}

	seek -= status;
	size += status;
    }
    if (size < 64) {
	if (!seek) {
	    longjmp(*simple_data->env, MISS_FAILURE);
	} else {
	    unsigned access, excess;

	    access = 64 - size;
	    excess = simple_data->excess + access;
	    if (excess < 64) {
		simple_data->excess = excess;
		memset(buffer + size, 0, access);
	    } else {
		longjmp(*simple_data->env, Mylene1f4_UNEXPECTED_EOF);
	    }

	    size = 64;
	}
    }

    simple_data->seek = seek;

    Size(system) = size;
    Buffer(system) = (unsigned char *) buffer;
}


static void
correct_underflow_2(void *system)
{
    void *storage;

    storage = Private_S(system, storage);

    if (!Size(system)) {
	int status;

	status = try_read(Private_S(system, e66c), storage, 64);
	if (status == -1) {
	    longjmp(PRIVATE_A(Private_S(system, stream), env),
		    Mylene1f4_e66c_ERROR);
	} else {
	    if (!status) {
		if (Private_S(system, excess) == 64) {
		    longjmp(PRIVATE_A(Private_S(system, stream), env),
			    Mylene1f4_UNEXPECTED_EOF);
		}
	    }
	}

	Private_S(system, excess) = 64 - status;
	Size(system) = 64;
	Buffer(system) = storage;
	Private_S(system, offset) = 64;
    } else {
	int status;
	unsigned system_offset;

	system_offset = Private_S(system, offset);
	if (1024 < system_offset + 64) {
	    memmove(storage, Buffer(system), Size(system));
	    system_offset = Size(system);
	    Buffer(system) = storage;
	}
	status = try_read
	    (Private_S(system, e66c), (char *) storage + system_offset, 64);
	if (status == -1) {
	    longjmp(PRIVATE_A(Private_S(system, stream), env),
		    Mylene1f4_e66c_ERROR);
	} else {
	    if (!status) {
		if (Private_S(system, excess) == 64) {
		    longjmp(PRIVATE_A(Private_S(system, stream), env),
			    Mylene1f4_UNEXPECTED_EOF);
		}
	    }
	}

	if (status + Size(system) < 64) {
	    Private_S(system, excess) = 64 - (Size(system) + status);
	    status += Private_S(system, excess);
	} else {
	    Private_S(system, excess) = 0;
	}
	system_offset += status;
	Size(system) += status;
	Private_S(system, offset) = system_offset;
    }
}


static void
error_exit_0(void *system)
{
    longjmp(*((struct simple_type *) Private(system))->env,
	    Mylene1f4_SYSTEM_FAILURE);
}


static void
error_exit_2(void *system)
{
    longjmp(PRIVATE_A(Private_S(system, stream), env),
	    Mylene1f4_SYSTEM_FAILURE);
}


static void
seek_stream(void *system, unsigned offset)
{
    lcard_type delete;
    struct seek_1f44_type *e66c;

    e66c = Private_S(system, e66c);

    Private_S(system, offset) = 0;
    offset -= Size(system);
    LCARD_SET_CARD1(delete, offset);
    if (e66c->seek(e66c, &delete, SEEK_CUR)) {
	longjmp(PRIVATE_A(Private_S(system, stream), env),
		Mylene1f4_e66c_ERROR);
    }

    Size(system) = 0;
}


static void
stat_failure(void *stream, int error, void *source)
{
    void *context;

    context = PRIVATE_A(stream, context);

    Mylene1f4_fprintf_error(context, "%s: ", Mylene1f4(context));
    if (error == MISS_FAILURE) {
	Mylene1f4_fprintf_error
	    (context, "exceeded seek limit (%u) while looking for mpeg data "
	     "in `%s'\n", MPEG(context).seek, Source(source).s1.path);
    } else if (error == Mylene1f4_UNEXPECTED_EOF) {
	Mylene1f4_fprintf_error
	    (context, "unexpected EOF in `%s'\n", Source(source).s1.path);
    } else if (error == READ_FAILURE) {
	Mylene1f4_fprintf_error
	    (context, "cannot read `%s': %s\n",
	     Source(source).s1.path, strerror(errno));
    }
}


int
Mylene1f4_advance_system(void *system, unsigned *packet_length)
{
    lcard_type offset, system_advance;
    struct seek_1f44_type *e66c;
    unsigned diff, size;

    LCARD_SET_CARD2(system_advance, Private_S(system, advance));

    e66c = Private_S(system, e66c);
    if (e66c->tell(e66c, &offset)) {
	longjmp(PRIVATE_A(Private_S(system, stream), env),
		Mylene1f4_e66c_ERROR);
    }
    size = Size(system);
    LCARD_ADD_CARD1(system_advance, size);
    LCARD_SUBTRACT_CARD2(system_advance, offset);
    LCARD_SUBTRACT_CARD1(system_advance, Private_S(system, excess));
    diff = LCARD_GET_CARD(system_advance);
    if (*packet_length < diff) {
	longjmp(PRIVATE_A(Private_S(system, stream), env),
		Mylene1f4_ADVANCE_FAILURE);
    }
    *packet_length -= diff;
    if (size < diff) {
	LCARD_SUBTRACT_CARD1(system_advance, size);
	Buffer(system) += size;
	Size(system) = 0;
	if (e66c->seek(e66c, &system_advance, SEEK_CUR)) {
	    longjmp(PRIVATE_A(Private_S(system, stream), env),
		    Mylene1f4_e66c_ERROR);
	}
    } else {
	Size(system) -= diff;
	Buffer(system) += diff;
    }

    LCARD_SET_CARD1(Private_S(system, advance), 0);

    return 0;
}


int
Mylene1f4_clear_system(void *stream)
{
    void *h262_system;

    h262_system = PRIVATE_A(stream, system);

    seek_h262_reset_system_attributes
	(h262_system,
	 SEEK_H262_SYSTEM_BUFFER_MASK | SEEK_H262_SYSTEM_SIZE_MASK);

    Private_S(h262_system, offset) = 1024;

    return 0;
}


int
Mylene1f4_extend_system(void *stream, unsigned minimum_useful_size)
{
    void *data, *swap;

    if (minimum_useful_size < 65536) {
	minimum_useful_size = 65536;
    }
    data = PRIVATE_A(stream, data);
    swap = realloc(data, minimum_useful_size);
    if (!swap) {
	longjmp(PRIVATE_A(stream, env), Mylene1f4_MALLOC_FAILURE);
    } else {
	BUFFER(stream) -= (char *) data - (char *) swap;
	PRIVATE_A(stream, data) = swap;
	PRIVATE_A(stream, size) = minimum_useful_size;
    }

    return 0;
}


int
Mylene1f4_fit_system(void *system, void *e66c)
{
    Private_S(system, e66c) = e66c;

    return 0;
}


int
Mylene1f4_fix_system(void *stream, unsigned system_type)
{
    int status;
    unsigned size;
    void *h262_system;

    seek_h262_system_size(&size);

    h262_system = (seek_h262_system_type)
	malloc(size + ALIGN_AMALLOC(sizeof(struct system_type))
	       + ALIGN_ZMALLOC(sizeof(struct packet_type) << RECORD_ORDER)
	       + ALIGN_AMALLOC(sizeof(sequel_type))
	       + ALIGN_AMALLOC(sizeof(xmpeg0_type)));
    if (!h262_system) {
	Mylene1f4_fprintf_error
	    (PRIVATE_A(stream, context), "%s: %s\n",
	     Mylene1f4(PRIVATE_A(stream, context)), strerror(errno));

	status = -1;
    } else {
	struct seek_h262_system_attributes_type attributes;

	seek_h262_reset_system(h262_system);

	attributes.abort_decoding = error_exit_2;
	attributes.correct_underflow = correct_underflow_2;
	attributes.seek_stream = seek_stream;

	seek_h262_set_system_attributes
	    (h262_system, &attributes,
	     SEEK_H262_SYSTEM_ABORT_DECODING_MASK
	     | SEEK_H262_SYSTEM_CORRECT_UNDERFLOW_MASK
	     | SEEK_H262_SYSTEM_SEEK_STREAM_MASK);

	{
	    struct packet_type *packet_data;
	    struct sequel_type *sequel_data;
	    struct system_type *system_data;
	    struct xmpeg0_type *xmpeg0_data;

	    system_data = (struct system_type *) ((char *) h262_system + size);
	    system_data = (struct system_type *) ALIGN_ABUFFER(system_data);

	    if (system_type == 11172) {
		system_data->pack_header = seek_h262_artificial_pack_header;
		system_data->packet = seek_h262_packet;
	    } else {
		system_data->pack_header = seek_h262_pack_header;
		system_data->packet = seek_h262_PES_packet;
	    }

	    LCARD_SET_CARD1(system_data->advance, 0);

	    system_data->e66c = PRIVATE_A(stream, e66c);
	    system_data->stream = stream;

	    Private(h262_system) = system_data;

	    packet_data = (struct packet_type *) (system_data + 1);
	    packet_data = (struct packet_type *) ALIGN_ZBUFFER(packet_data);

	    system_data->packet_data = packet_data;
	    system_data->packet_index = 0;

	    system_data->excess = 0;

	    sequel_data = (struct sequel_type *)
		(packet_data + (1 << RECORD_ORDER));
	    sequel_data = (struct sequel_type *) ALIGN_ABUFFER(sequel_data);

	    xmpeg0_data = (struct xmpeg0_type *) (sequel_data + 1);
	    xmpeg0_data = (struct xmpeg0_type *) ALIGN_ABUFFER(xmpeg0_data);

	    xmpeg0_data->specification = system_type;

	    sequel_data->sequel_data = SEQUEL(PRIVATE_A(stream, context));
	    SEQUEL(PRIVATE_A(stream, context)) = sequel_data;
	    sequel_data->code = Mylene1f4_XMPEG0_SEQUEL;
	    sequel_data->data = xmpeg0_data;
	}

	PRIVATE_A(stream, system) = h262_system;

	Mylene1f4_clear_system(stream);

	status = 0;
    }

    return status;
}


int
Mylene1f4_get_system(void *stream)
{
    Mylene1f4_remove_sequel
	(PRIVATE_A(stream, context), Mylene1f4_XMPEG0_SEQUEL);

    free(PRIVATE_A(stream, system));

    return 0;
}


int
Mylene1f4_reset_system(void *stream)
{
    Private_S(PRIVATE_A(stream, system), data).epcm.linger = 0;

    return 0;
}


int
Mylene1f4_seek_system(void *stream, unsigned positive, unsigned negative,
		      void *Stream, void *System)
{
    int descend;
    struct packet_type *packet_data;
    unsigned previous_packet_index;
    void *h262_system;

    h262_system = PRIVATE_A(stream, system);

    packet_data = Private_S(h262_system, packet_data);

    previous_packet_index =
	(RECORD_MASK + Private_S(h262_system, packet_index)) & RECORD_MASK;
    descend = packet_data[previous_packet_index].data_size;
    descend += Private_S(h262_system, excess);
    descend += positive;
    descend -= negative;
    {
	unsigned start_packet_index;

	start_packet_index = previous_packet_index;
	while (descend < 0) {
	    previous_packet_index =
		(previous_packet_index + RECORD_MASK) & RECORD_MASK;
	    if (previous_packet_index == start_packet_index) {
		return 1;
	    }
	    descend += packet_data[previous_packet_index].data_size;
	}
    }
    {
	unsigned deck;
	lcard_type text, *output;

	LCARD_SET_CARD2(text, packet_data[previous_packet_index].file_offset);
	if (System) {
	    output = System;
	    LCARD_SET_CARD2(*output, text);
	}
	if (Stream) {
	    deck = packet_data[previous_packet_index].header_length + descend;
	    LCARD_ADD_CARD1(text, deck);
	    output = Stream;
	    LCARD_SET_CARD2(*output, text);
	}
    }

    return 0;
}


int
Mylene1f4_set_system(void *stream, void *cleavage)
{
    void *h262_system;

    h262_system = PRIVATE_A(stream, system);

    Private_S(h262_system, advance) = *(lcard_type *) cleavage;

    return 0;
}


int
Mylene1f4_stat_system(void *stream, void *context)
{
    char *error_string, *start_code_name;
    void *h262_system;

    h262_system = PRIVATE_A(stream, system);

    seek_h262_system_error_string(Error(h262_system), &error_string);
    seek_h262_start_code_name
	(start_code(Start_code(h262_system)), &start_code_name);
    Mylene1f4_fprintf_error
	(context, " (last decoded start code: 000001%02x (%s), %s)\n",
	 start_code(Start_code(h262_system)), start_code_name, error_string);

    return 0;
}


int
Mylene1f4_try_system(void *stream, void *source)
{
    int status;
    unsigned system_type = ~0;

    do {
	jmp_buf env;
	int offset;
	struct simple_type simple;
	struct seek_h262_whatever_type h262_whatever;
	void *context;

	context = PRIVATE_A(stream, context);

	offset = Mylene1f4_seek_file(context, source, 0, SEEK_CUR);
	if (offset == -1) {
	    status = 1;
	    break;
	}

	h262_whatever.abort_decoding = error_exit_0;
	h262_whatever.buffer = NULL;
	h262_whatever.private = &simple;
	h262_whatever.size = 0;

	simple.context = context;
	simple.env = &env;
	simple.excess = 0;
	simple.source = source;

	if (!MPEG(context).seek) {
	    h262_whatever.correct_underflow = correct_underflow_0;
	} else {
	    simple.seek = MPEG(context).seek;
	    h262_whatever.correct_underflow = correct_underflow_1;
	}

	if ((status = setjmp(env))) {
	    if (status == Mylene1f4_SYSTEM_FAILURE) {
		status = 0;
	    } else {
		stat_failure(stream, status, source);
	    }
	} else {
	    seek_h262_next_start_code(&h262_whatever);
	    if (h262_whatever.start_code == pack_start_code) {
		if (!h262_whatever.size) {
		    h262_whatever.correct_underflow(&h262_whatever);
		}
		{
		    unsigned c;

		    c = h262_whatever.buffer[0];
		    if ((c & 0xf0) == 0x20) {
			system_type = ~11172;
		    } else if ((c & 0xc0) == 0x40) {
			system_type = ~13818;
		    }
		}
	    }
	}

	if (!status) {
	    status = Mylene1f4_seek_file(context, source, offset, SEEK_SET);
	    if (status != -1) {
		status = 0;
	    } else {
		break;
	    }
	}

	if (~system_type) {
	    status = Mylene1f4_fix_system(stream, ~system_type);
	}
    } while (0);

    return status;
}
