/*
 * system2.c
 * Copyright (C) 2007-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <e1f4.h>
#include <h262.h>
#include <setjmp.h>

#include <defs.h>
#include <system.h>
#include <types.h>

#define false(e)			0
#define true(e)				1

#if WORDS_BIGENDIAN
# define private_stream_1		0x000001bd
#else
# define private_stream_1		0xbd010000
#endif				/* WORDS_BIGENDIAN */

#define Buffer(h262) \
    (((struct seek_h262_whatever_type *) (h262))->buffer)
#define Minimum_useful_size(h262) \
    (((struct seek_h262_whatever_type *) (h262))->minimum_useful_size)
#define Private(h262) \
    (((struct seek_h262_whatever_type *) (h262))->private)
#define Size(h262) \
    (((struct seek_h262_whatever_type *) (h262))->size)
#define Start_code(h262) \
    (((struct seek_h262_whatever_type *) (h262))->start_code)

#define Private__(whatever, member) \
    (((struct system_type *) whatever)->member)
#define Private_S(whatever, member) \
    Private__(Private(whatever), member)

#define System(system) \
    ((struct system_type *) (system))

#define h262(what) \
    ((struct seek_h262_whatever_type *) (what))

#undef SIZE

#define PRIVATE(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->private)
#define SIZE(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->size)

#define PRIVATE__(whatever, member) \
    (((struct stream_type *) whatever)->member)
#define PRIVATE_A(whatever, member) \
    PRIVATE__(PRIVATE(whatever), member)

#define stream_id(stream) \
    PRIVATE_A(stream, audio_id)

static void packet_data(void *, unsigned);

static void
packet_data(void *system, unsigned packet_length)
{
    void *stream;

    stream = Private_S(system, stream);

    if (false(Start_code(system) != private_stream_1)) {
	longjmp(PRIVATE_A(stream, env), Mylene1f4_NEAR_FAILURE);
    } else {
	if (packet_length < 7) {
	    longjmp(PRIVATE_A(stream, env), Mylene1f4_NEAR_FAILURE);
	} else {
	    if (Size(system) < 7) {
		if (false(Minimum_useful_size(system) < 7)) {
		    Minimum_useful_size(system) = 7;
		}

		h262(system)->correct_underflow(system);
	    }

	    if (*Buffer(system) != stream_id(stream)) {
		longjmp(PRIVATE_A(stream, env), Mylene1f4_NEAR_FAILURE);
	    } else {
		Private_S(system, data).epcm.header = Buffer(system)[5];
	    }
	}
    }
}


int
Mylene1f4_head_system(void *stream, unsigned *header)
{
    *header = Private_S(PRIVATE_A(stream, system), data).epcm.header;

    return 0;
}


int
Mylene1f4_near_system(void *stream)
{
    void *h262_system;

    h262_system = PRIVATE_A(stream, system);

    if (true(Size(h262_system) < 4)) {
	h262(h262_system)->correct_underflow(h262_system);
    }

    {
	unsigned char *buffer;

	buffer = Buffer(h262_system);
	if (buffer[0] || buffer[1] || buffer[2] != 0x01 || buffer[3] != 0xbd) {
	    longjmp(PRIVATE_A(stream, env), Mylene1f4_NEAR_FAILURE);
	}
    }

    seek_h262_next_start_code(h262_system);

    {
	struct seek_h262_system_attributes_type attributes;

	attributes.packet_data = packet_data;

	seek_h262_set_system_attributes
	    (h262_system, &attributes, SEEK_H262_SYSTEM_PACKET_DATA_MASK);
    }

    System(Private(h262_system))->packet(h262_system);

    return 0;
}
