/*
 * system4.c
 * Copyright (C) 2003-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <e1f4.h>
#include <h262.h>
#include <setjmp.h>
#include <string.h>
#include <unistd.h>

#include <sys/types.h>

#include <1f44.h>
#include <defs.h>
#include <inter.h>
#include <lcard.h>
#include <system.h>
#include <types.h>

#define true(e)				1

#if WORDS_BIGENDIAN
# define MPEG_program_end_code		0x000001b9
# define pack_start_code		0x000001ba
# define private_stream_1		0x000001bd
# define system_header_start_code	0x000001bb
#else
# define MPEG_program_end_code		0xb9010000
# define pack_start_code		0xba010000
# define private_stream_1		0xbd010000
# define system_header_start_code	0xbb010000
#endif				/* WORDS_BIGENDIAN */

#define Buffer(h262) \
    (((struct seek_h262_whatever_type *) (h262))->buffer)
#define Error(h262) \
    (((struct seek_h262_whatever_type *) (h262))->error)
#define Private(h262) \
    (((struct seek_h262_whatever_type *) (h262))->private)
#define Size(h262) \
    (((struct seek_h262_whatever_type *) (h262))->size)
#define Start_code(h262) \
    (((struct seek_h262_whatever_type *) (h262))->start_code)

#define Private__(whatever, member) \
    (((struct system_type *) whatever)->member)
#define Private_S(whatever, member) \
    Private__(Private(whatever), member)

#define h262(what) \
    ((struct seek_h262_whatever_type *) (what))

#undef SIZE

#define BUFFER(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->buffer)
#define MINIMUM_USEFUL_SIZE(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->minimum_useful_size)
#define PRIVATE(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->private)
#define SIZE(e1f4) \
    (((struct Mylene1f4_whatever_type *) (e1f4))->size)

#define PRIVATE__(whatever, member) \
    (((struct stream_type *) whatever)->member)
#define PRIVATE_A(whatever, member) \
    PRIVATE__(PRIVATE(whatever), member)

#define stream_id(stream) \
    PRIVATE_A(stream, audio_id)

#if __USE_1f44_EEAD__
# define try_read(e66c, data, size) \
    (e66c)->eead((e66c), (data), (size))
#endif				/* __USE_1f44_EEAD__ */

#if !__USE_1f44_EEAD__
static int try_read(struct seek_1f44_type *, char *, unsigned);
#endif				/* !__USE_1f44_EEAD__ */

static void copy_packet(void *, unsigned);
static void deck_system(void *);
static void packet_data(void *, unsigned);
static void wrap_packet(void *, unsigned);

#if !__USE_1f44_EEAD__
static int
try_read(struct seek_1f44_type *e66c, char *data, unsigned size)
{
    int status = 0;

    while (1) {
	int success;

	success = e66c->read(e66c, data, size);
	if (success != -1) {
	    if (!success) {
		break;
	    } else {
		data += success;
		size -= success;
		status += success;
		if (!size) {
		    break;
		}
	    }
	} else {
	    if (errno == EINTR) {
	    } else {
		status = -1;
		break;
	    }
	}
    }

    return status;
}
#endif				/* !__USE_1f44_EEAD__ */


static void
copy_packet(void *system, unsigned packet_length)
{
    do {
	unsigned min, transfer;
	void *stream;

	if (LCARD_IS_SET(Private_S(system, advance))) {
	    Mylene1f4_advance_system(system, &packet_length);
	}

	Private_S(system, packet_data_size) = packet_length;

	stream = Private_S(system, stream);

	transfer = MINIMUM_USEFUL_SIZE(stream) - SIZE(stream);

	min = Size(system);
	if (packet_length < min) {
	    min = packet_length;
	}
	if (transfer < min) {
	    min = transfer;
	}
	if (min) {
	    memcpy(BUFFER(stream), Buffer(system), min);
	    BUFFER(stream) += min;
	    packet_length -= min;
	    SIZE(stream) += min;
	    Buffer(system) += min;
	    Size(system) -= min;
	    transfer -= min;
	    if (!transfer) {
		Private_S(system, data).epcm.linger = packet_length;
		if (Size(system)) {
		} else {
		    Private_S(system, offset) = 0;
		}

		break;
	    }
	}
	if (packet_length) {
	    if (transfer < packet_length) {
		Private_S(system, data).epcm.linger = packet_length - transfer;
		packet_length = transfer;
	    } else {
		Private_S(system, data).epcm.linger = 0;
	    }

	    Private_S(system, offset) = 0;

	    {
		int status;

		status = try_read
		    (Private_S(system, e66c), BUFFER(stream), packet_length);
		if (status != (int) packet_length) {
		    Mylene1f4_rack_stripe(stream, status, packet_length);
		} else {
		    BUFFER(stream) += packet_length;
		    SIZE(stream) += packet_length;
		}
	    }
	} else {
	    Private_S(system, data).epcm.linger = 0;

	    if (Size(system)) {
	    } else {
		Private_S(system, offset) = 0;
	    }
	}
    } while (0);
}


static void
deck_system(void *stream)
{
    int have_start_code = 0;
    unsigned minimum_useful_size;
    void *h262_system;

    minimum_useful_size = MINIMUM_USEFUL_SIZE(stream);

    h262_system = PRIVATE_A(stream, system);

    {
	unsigned linger;

	linger = Private_S(h262_system, data).epcm.linger;
	if (linger) {
	    if (minimum_useful_size < linger) {
		Private_S(h262_system, data).epcm.linger =
		    linger - minimum_useful_size;
		linger = minimum_useful_size;
	    }
	    {
		unsigned size;

		size = Size(h262_system);
		if (size) {
		    unsigned min;

		    min = linger;
		    if (size < min) {
			min = size;
		    }
		    memcpy(BUFFER(stream), Buffer(h262_system), min);
		    BUFFER(stream) += min;
		    linger -= min;
		    SIZE(stream) += min;
		    Buffer(h262_system) += min;
		    Size(h262_system) -= min;
		}
	    }
	    if (linger) {
		int status;

		status = try_read
		    (Private_S(h262_system, e66c), BUFFER(stream), linger);
		if (status != (int) linger) {
		    Mylene1f4_rack_stripe(stream, status, linger);
		}

		BUFFER(stream) += linger;
		SIZE(stream) += linger;
	    }
	}
    }

    while (SIZE(stream) < minimum_useful_size) {
	if (!have_start_code) {
	    seek_h262_next_start_code(h262_system);
	}
	{
	    unsigned start_code;

	    start_code = Start_code(h262_system);
	    if (start_code == pack_start_code) {
		Private_S(h262_system, pack_header) (h262_system);
		have_start_code = 1;
	    } else if (system_header_start_code < start_code) {
		wrap_packet(h262_system, start_code);
		have_start_code = 0;
	    } else if (start_code == MPEG_program_end_code) {
		longjmp(PRIVATE_A(stream, env), Mylene1f4_UNEXPECTED_EOS);
	    } else {
		longjmp(PRIVATE_A(stream, env), Mylene1f4_SYSTEM_FAILURE);
	    }
	}
    }
}


static void
packet_data(void *system, unsigned packet_length)
{
    do {
	if (Start_code(system) == private_stream_1) {
	    if (packet_length) {
		void *stream;

		stream = Private_S(system, stream);

		if (Size(system)) {
		} else {
		    h262(system)->correct_underflow(system);
		}
		if (*Buffer(system) == stream_id(stream)) {
		    if (packet_length < 7) {
			longjmp(PRIVATE_A(stream, env),
				Mylene1f4_UNEXPECTED_LENGTH);
		    } else {
			if (6 < Size(system)) {
			} else {
			    h262(system)->correct_underflow(system);
			}

			if (Buffer(system)[5]
			    != Private_S(system, data).epcm.header) {
			    longjmp(PRIVATE_A(stream, env),
				    Mylene1f4_CHANGED_HEADER);
			} else {
			    Buffer(system) += 7;
			    Size(system) -= 7;

			    copy_packet(system, packet_length - 7);

			    break;
			}
		    }
		}
	    }
	}

	if (packet_length < Size(system)) {
	    Buffer(system) += packet_length;
	    Size(system) -= packet_length;
	} else {
	    lcard_type offset;
	    struct seek_1f44_type *e66c;

	    e66c = Private_S(system, e66c);

	    Private_S(system, offset) = 0;
	    packet_length -= Size(system);
	    LCARD_SET_CARD1(offset, packet_length);
	    if (e66c->seek(e66c, &offset, SEEK_CUR)) {
		longjmp(PRIVATE_A(Private_S(system, stream), env),
			Mylene1f4_e66c_ERROR);
	    }
	    Size(system) = 0;
	}
    } while (0);
}


static void
wrap_packet(void *system, unsigned start_code)
{
    if (start_code != private_stream_1) {
	Private_S(system, packet) (system);
    } else {
	lcard_type delete;
	struct packet_type *packet_data;
	struct seek_1f44_type *e66c;
	unsigned packet_index;

	packet_index = Private_S(system, packet_index);

	packet_data = Private_S(system, packet_data) + packet_index;

	e66c = Private_S(system, e66c);

	if (e66c->tell(e66c, &delete)) {
	    longjmp(PRIVATE_A(Private_S(system, stream), env),
		    Mylene1f4_e66c_ERROR);
	} else {
	    lcard_type file_offset;
	    unsigned deck, packet_data_size;

	    LCARD_SET_CARD2(file_offset, delete);
	    deck = Size(system) + 4;
	    LCARD_SUBTRACT_CARD1(file_offset, deck);
	    LCARD_SET_CARD2(packet_data->file_offset, file_offset);
	    Private_S(system, packet) (system);
	    if (e66c->tell(e66c, &delete)) {
		longjmp(PRIVATE_A(Private_S(system, stream), env),
			Mylene1f4_e66c_ERROR);
	    } else {
		LCARD_SET_CARD2(file_offset, delete);
		packet_data_size = Private_S(system, packet_data_size);
		packet_data->data_size = packet_data_size;
		deck = packet_data_size + Size(system)
		    - Private_S(system, data).epcm.linger;
		LCARD_SUBTRACT_CARD1(file_offset, deck);
		LCARD_SUBTRACT_CARD2(file_offset, packet_data->file_offset);
		packet_data->header_length = LCARD_GET_CARD(file_offset);
		if (packet_index == RECORD_MASK) {
		    Private_S(system, packet_index) = 0;
		} else {
		    Private_S(system, packet_index) = packet_index + 1;
		}
	    }
	}
    }
}


int
Mylene1f4_line_system(void *stream)
{
    {
	struct seek_h262_system_attributes_type attributes;

	attributes.packet_data = packet_data;

	seek_h262_set_system_attributes
	    (PRIVATE_A(stream, system), &attributes,
	     SEEK_H262_SYSTEM_PACKET_DATA_MASK);
    }

    {
	struct Mylene1f4_audio_attributes_type attributes;

	attributes.correct_underflow = deck_system;

	Mylene1f4_set_audio_attributes
	    (stream, &attributes, Mylene1f4_AUDIO_CORRECT_UNDERFLOW_MASK, 0);
    }

    return 0;
}
