/*
 * types.h
 * Copyright (C) 2005-2013, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __Mylene1f4_TYPES_H__
#define __Mylene1f4_TYPES_H__

#include <config.h>

#if __ENABLE_EQUEUE__
# include <aime.h>
#endif				/* __ENABLE_EQUEUE__ */
#include <setjmp.h>

#include <sys/types.h>

#include <1f44.h>
#include <lcard.h>

typedef struct buffer_type {
    char *data;
    unsigned extent, length;
} buffer_type;

typedef struct clause_type {
    int (*delete) (void *);
    struct clause_type *clause_data;
} clause_type;

typedef struct device_type {
    int (*seek) (void *, void *, unsigned, void *);
} device_type;

typedef struct driver_type {
    int (*copy) (void *, void *),
	(*dump) (void *, void *, unsigned *, void **),
	(*load) (void *, void *, unsigned *, void **);
} driver_type;

#if __LINK_CELL_A52A_LOADER__
typedef struct ea52al_type {
    const char *loader;
    unsigned flags;
} ea52al_type;
#endif				/* __LINK_CELL_A52A_LOADER__ */

typedef struct eid3wl_type {
    const char *parser;
    unsigned flags, timeout, transfer;
} eid3wl_type;

#if __ENABLE_EQUEUE__
typedef struct equeue_type {
    char *name, *rate;
    struct {
	int (*flat) (void **);
	void *data;
    } deck;
    struct {
	int (*slip) (void *), (*tail) (void *);
    } fine;
    struct {
	int (*look) (void *, unsigned *);
    } june;
    struct {
	unsigned size;
	void *data;
    } moon;
    struct {
	void *data, *link;
    } text;
    struct equeue_type *equeue_data;
    struct report_type *head_report, *tail_report;
    union {
	void *aime;
    } data;
    union {
	void *slip;
    } date;
    union {
	struct {
	    void *e2list, *e4less, *e8port;
	} aime;
    } slip;
    unsigned bits;
# if __ENABLE_CHANNEL_SERVER__
    unsigned channel, client;
# endif				/* __ENABLE_CHANNEL_SERVER__ */
    unsigned major, minor;
    void *dxflat;
} equeue_type;
#endif				/* __ENABLE_EQUEUE__ */

typedef struct extime_type {
    unsigned h, m, s, x;
} extime_type;

typedef struct id3tag_type {
    char *data[6];
    unsigned size[6];
} id3tag_type;

#if __ENABLE_EQUEUE__
typedef struct iqueue_type {
    struct equeue_type *head_equeue, *tail_equeue;
    unsigned lqueue;
} iqueue_type;
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_MPEG_EQUALIZER__
typedef struct l3band_type {
    double frequency, level;
    struct l3band_type *l3band_data;
} l3band_type;
#endif				/* __ENABLE_MPEG_EQUALIZER__ */

typedef struct mirror_type {
    const char *name, *path;
    struct {
	int argc;
	void *argv;
    } args;
} mirror_type;

typedef struct output_type {
    int (*fini) (void *), (*init) (void *), (*kill) (void *), (*sync) (void *);
} output_type;

typedef struct record_type {
    int (*data) (void *);
    struct record_type *record_data;
    void *text;
} record_type;

#if __ENABLE_EQUEUE__
typedef struct report_type {
    struct report_type *report_data;
    unsigned size;
} report_type;
#endif				/* __ENABLE_EQUEUE__ */

#if __ENABLE_EQUEUE__
typedef struct sample_type {
    struct {
	void *miss, *text;
    } function_set;
    void *operator1s, *operator2s;
} sample_type;
#endif				/* __ENABLE_EQUEUE__ */

typedef struct sequel_type {
    struct sequel_type *sequel_data;
    unsigned code;
    void *data;
} sequel_type;

typedef struct socket_type {
    int (*call) (void *, unsigned, void *, void *);
    unsigned page;
    void *back;
} socket_type;

typedef struct source_type {
    union {
	struct {
	    const char *path;
	    int fd;
	    unsigned size;
	} s1;
	struct {
	    lcard_type size;
	    struct seek_1f44_type *e66c;
	} s2;
    } _;
} source_type;

typedef struct string_type {
    const char *data;
    unsigned size;
} string_type;

typedef struct stream_type {
    char *resource;
    jmp_buf env;
    int synchronization;
    struct seek_1f44_type *e66c;
    unsigned audio_id, clip, size;
    unsigned char **e1f4_output;
    void *context, *data, *loader, *source, *system;
} stream_type;

#if __ENABLE_EQUEUE__
typedef struct stripe_type {
    struct iqueue_type iqueue;
} stripe_type;
#endif				/* __ENABLE_EQUEUE__ */

typedef struct target_type {
    int (*back) (void *), (*deck) (void *, void *), (*ever) (void *);
#if __ENABLE_FAST_INTER__
    int (*fast) (void *);
#endif				/* __ENABLE_FAST_INTER__ */
    int (*fini) (void *, unsigned, void **), (*flow) (void *, void *),
	(*head) (void *), (*init) (void *, unsigned, void **);
#if __ENABLE_x00696e6b__
    int (*line) (void *, unsigned, void **);
#endif				/* __ENABLE_x00696e6b__ */
    int (*link) (void *);
#if __ENABLE_x00696e6b__
    int (*lock) (void *, unsigned char *, unsigned);
#endif				/* __ENABLE_x00696e6b__ */
#if __EXPAND_x1f42MAPS__
    int (*plug) (void *, void **);
#endif				/* __ENABLE_x00696e6b__ */
    int (*seek) (void *), (*skip) (void *), (*type) (void *, void *),
	(*wind) (void *, unsigned *, unsigned);
#if __LINK_CELL_LOADER__
    int (*zero) (void *, unsigned);
#endif				/* __LINK_CELL_LOADER__ */
} target_type;

#if __WAIT_VIOLET__
typedef struct violet_type {
    int (*clip) (void *, int);
    pid_t wait;
    struct violet_type *violet_data;
    void *data;
} violet_type;
#endif				/* __WAIT_VIOLET__ */

typedef struct x1f42a_type {
    int audio_change_flag;
    lcard_type stream, system;
    union {
	const char *path;
	struct seek_1f44_type e66c;
    } source;
    unsigned PTS_0, PTS_1, audio_id, file_offset, frame_rate_d, frame_rate_n,
	frames, nentries, next_frame, nrecords, type;
#if __EXPAND_x1f42MAPS__
    unsigned artificial;
    void *qrecords;
#endif				/* __ENABLE_x00696e6b__ */
} x1f42a_type;

typedef struct x1f42e_type {
    unsigned count, index;
} x1f42e_type;

#if __LINK_A52A_LOADER__
typedef struct xa52a1_type {
    unsigned audio_id, frame_rate_d, frame_rate_n;
} xa52a1_type;
#endif				/* __LINK_A52A_LOADER__ */

typedef struct xepcm1_type {
    unsigned audio_id, frame_rate_d, frame_rate_n;
} xepcm1_type;

typedef struct xmpeg0_type {
    unsigned specification;
} xmpeg0_type;

typedef struct xmpeg1_type {
    const char *framerate;
    unsigned bitrate, frame_rate_d, frame_rate_n, layer, mode, mode_extension,
	specification;
} xmpeg1_type;

#endif				/* __Mylene1f4_TYPES_H__ */
