/*
 * sa2.c
 * Copyright (C) 2008-2012, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <errno.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <arpa/inet.h>

#include <netinet/in.h>

#include <sys/select.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/un.h>

#include <access.h>
#include <defs.h>
#include <inter.h>
#include <types.h>

#define __SEND_DEBUG__			0

#define SECOND_a(v)			((v).tv_sec)
#define SECOND_f(v)			((v).tv_usec)

#if __NONBLOCKING_REFLEX_TRANSFER__
# define READ_REFLEX(context) \
    READ(context)(context)
#else
# define READ_REFLEX(context)		Mylene200_read_reflex(context)
#endif				/* __NONBLOCKING_REFLEX_TRANSFER__ */

#if __NONBLOCKING_REFLEX_TRANSFER__
# define SEND_REFLEX(context) \
    SEND(context)(context)
#else
# define SEND_REFLEX(context)		send_reflex(context)
#endif				/* __NONBLOCKING_REFLEX_TRANSFER__ */

#define SWAP_CLIENT(context, here_client, last_client, post_client) \
    {									      \
	if ((last_client)) {						      \
	    struct client_type *miss_client;				      \
									      \
	    miss_client = NASSA5((context));				      \
									      \
	    if ((here_client) == miss_client) {				      \
	    } else {							      \
		NASSA5((context)) = (here_client);			      \
		(here_client)->client_data = NULL;			      \
		(here_client)->client_miss = miss_client;		      \
		(last_client)->client_data = post_client;		      \
		miss_client->client_data = (here_client);		      \
									      \
		if ((post_client)) {					      \
		    (post_client)->client_miss = (last_client);		      \
		}							      \
	    }								      \
	} else {							      \
	    struct client_type *miss_client;				      \
									      \
	    miss_client = NASSA5((context));				      \
									      \
	    if ((here_client) == miss_client) {				      \
	    } else {							      \
		CLIENT((context)) = (post_client);			      \
		NASSA5((context)) = (here_client);			      \
		(here_client)->client_data = NULL;			      \
		(here_client)->client_miss = miss_client;		      \
		miss_client->client_data = (here_client);		      \
									      \
		if ((post_client)) {					      \
		    (post_client)->client_miss = NULL;			      \
		}							      \
	    }								      \
	}								      \
    }

static int line_server(void *);
#if __NONBLOCKING_REFLEX_TRANSFER__
static int lock_reflex(void *);
#endif				/* __NONBLOCKING_REFLEX_TRANSFER__ */
static int miss_client(void *);
#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int miss_shrews(void *);
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */
static int miss_stripe(void *);
#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int miss_thread(void *);
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */
static int miss_window(void *);
static int push_client(void *);
static int send_reflex(void *);

static int
line_server(void *context)
{
    fd_set fd_set_0, fd_set_1, *set_0, *set_1, *set_o, *set_l;
    int reflex, server, status = 0;

    reflex = REFLEX(context);

    server = SERVER(context);

    set_l = &SET_1(context);
    set_o = &SET_0(context);

    set_0 = &fd_set_0;
    set_1 = &fd_set_1;

    while (1) {
	struct timeval *linetrap;

	memcpy(set_0, set_o, sizeof(fd_set));
	memcpy(set_1, set_l, sizeof(fd_set));

	if (0) {
	} else {
	    if (0) {
	    } else {
		linetrap = NULL;
	    }
	}

	status = select(NFDS(context), set_0, set_1, NULL, linetrap);
	if (status < 0) {
	    if (errno != EINTR) {
		Mylene200_fprintf_error
		    (context, "%s: failed select: %s\n", Mylene200(context),
		     strerror(errno));
		break;
	    }
	} else {
	    if (status) {
		unsigned count;

		count = status;

		if (FD_ISSET(server, set_0)) {
		    status = push_client(context);
		    if (status) {
			break;
		    } else {
			count--;
		    }
		}

		if (FD_ISSET(reflex, set_0)) {
		    status = READ_REFLEX(context);
		    if (status) {
			break;
		    } else {
			count--;
		    }
		}

		if (FD_ISSET(reflex, set_1)) {
		    status = SEND_REFLEX(context);
		    if (status) {
			break;
		    } else {
			count--;
		    }
		}

		if (count) {
		    struct client_type *client_data;

		    client_data = CLIENT(context);
		    if (client_data) {
			struct client_type /**last_client = NULL, */
			    *neon_client;

			neon_client = NASSA5(context);

			while (1) {
			    int client;
			    struct client_type *post_client;

			    post_client = client_data->client_data;

			    client = client_data->fd;

			    if (FD_ISSET(client, set_0)) {
				/*last_client = client_data;*/

				status = client_data->read
				    (context, client_data);
				if (status) {
				    break;
				} else {
				    count--;
				    if (count) {
				    } else {
					break;
				    }
				}

				if (FD_ISSET(client, set_1)) {
				    if (post_client) {
					if (post_client->client_miss
					    != client_data) {
					    client_data = NULL;
					}
				    } else {
					if (client_data != NASSA5(context)) {
					    client_data = NULL;
					}
				    }

				    count--;

				    if (client_data) {
					status = client_data->send
					    (context, client_data);
					if (status) {
					    break;
					} else {
					    if (count) {
					    } else {
						break;
					    }
					}
				    }
				}
			    } else {
				if (FD_ISSET(client, set_1)) {
				    /*last_client = client_data;*/

				    status = client_data->send
					(context, client_data);
				    if (status) {
					break;
				    } else {
					count--;
					if (count) {
					} else {
					    break;
					}
				    }
				} else {
#if 0
				    /*
				     * BUGS
				     *
				     *.last_client is wrongly set (maybe) - it
				     * may disappear after it is set
				     *.previously recorded NASSA5 is modified
				     */
				    SWAP_CLIENT(context, client_data,
						last_client, post_client);
#endif				/* 0 */
				}
			    }

			    if (client_data == neon_client) {
				break;
			    } else {
				client_data = post_client;
			    }
			}
		    }

		    if (count) {
			break;
		    }
		}
	    }
	}
    }

    return status;
}


#if __NONBLOCKING_REFLEX_TRANSFER__
static int
lock_reflex(void *context)
{
    int status;
    struct stripe_type *stripe_data;
    unsigned size;
    void *data;

    stripe_data = STRIPE(context);

    data = stripe_data->data;
    size = stripe_data->size;

    status = write(REFLEX(context), data, size);
    if (status == -1) {
	if (errno == EINTR) {
	    status = 0;
	} else {
	    Mylene200_lose_reflex(context);
	}
    } else {
	if (!status) {
	    status = -1;
	    Mylene200_fail_reflex(context);
	} else {
	    size -= status;

	    if (size) {
		stripe_data->data = (char *) data + status;
		stripe_data->size = size;

		status = 0;
	    } else {
		struct stripe_type *post_stripe;

		status = 0;

		post_stripe = stripe_data->stripe_data;

		if (STOYA7(context)) {
		    free(stripe_data);
		} else {
		    STOYA7(context) = stripe_data;
		}

		if (post_stripe) {
		    STRIPE(context) = post_stripe;
		} else {
		    STOYA6(context) = NULL;
		    STRIPE(context) = NULL;

		    FD_CLR(REFLEX(context), &SET_1(context));
		}
	    }
	}
    }

    return status;
}
#endif				/* __NONBLOCKING_REFLEX_TRANSFER__ */


static int
miss_client(void *context)
{
    int status = 0;
    struct client_type *client_data;

    client_data = CLIENT(context);
    while (client_data) {
	int excess;
	struct client_type *post_client;

	post_client = client_data->client_data;

	excess = Mylene200_free_client(context, client_data);
	if (!status) {
	    status = excess;
	}

	client_data = post_client;
    }

    return status;
}


#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int
miss_shrews(void *context)
{
    struct client_type *client_data;

    client_data = CLIENT(context);
    while (client_data) {
	unsigned bits;

	bits = client_data->bits;
	if (bits & Mylene200_MIND_CLIENT) {
	    client_data->shrews = NULL;
	    client_data->bits = bits & ~Mylene200_MIND_CLIENT;
	}

	client_data = client_data->client_data;
    }

    return 0;
}
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */


static int
miss_stripe(void *context)
{
    struct stripe_type *stripe_data;

    stripe_data = STOYA7(context);
    if (stripe_data) {
	free(stripe_data);
    }

    stripe_data = STRIPE(context);
    while (stripe_data) {
	struct stripe_type *post_stripe;

	post_stripe = stripe_data->stripe_data;

	free(stripe_data);

	stripe_data = post_stripe;
    }

    return 0;
}


#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int
miss_thread(void *context)
{
    struct thread_type *thread_data;

    thread_data = THREAD(context);
    while (thread_data) {
	struct thread_type *post_thread;

	post_thread = thread_data->thread_data;

	free(thread_data);

	thread_data = post_thread;
    }

    return 0;
}
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */


static int
miss_window(void *context)
{
    struct window_type *window_data;
#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
    unsigned flags;
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */

#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
    flags = FLAGS(context) & Mylene200_MISS_SHREWS;
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */

    window_data = WINDOW(context);
    while (window_data) {
	struct client_type *client_data;
	struct window_type *post_window;

	post_window = window_data->window_data;

	client_data = window_data->client_data;
	if (client_data) {
	    struct window_type *back_window;

	    back_window = window_data->last_window;
	    if (back_window) {
		back_window->window_data = window_data->miss_window;
	    } else {
		client_data->window = window_data->miss_window;
	    }
	}

#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
	if (flags) {
	    FLAGS(context) &= ~Mylene200_MISS_SHREWS;
	    flags = 0;
	} else {
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */
	    free(window_data);
#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
	}
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */

	window_data = post_window;
    }

    return 0;
}


static int
push_client(void *context)
{
    int new, status;
    socklen_t size;
    struct sockaddr_in address;

    size = sizeof(address);
    new = accept(SERVER(context), (struct sockaddr *) &address, &size);
    if (new == -1) {
	status = 0;
	if (errno != EINTR) {
	    Mylene200_fprintf_error
		(context, "%s: failed accept: %s\n", Mylene200(context),
		 strerror(errno));
	}
    } else {
#if __ENABLE_DEBUG__
	Mylene200_type_debug(context, " ** new connection %d\n", new);
#endif				/* __ENABLE_DEBUG__ */

	if (FD_SETSIZE < new + 1) {
	    status = 0;
	    close(new);
	} else {
	    void *client;

	    status = Mylene200_link_client(context, new, &client);
	    if (status) {
		close(new);
	    } else {
		struct client_type *client_data, *last_client;

		client_data = client;

		last_client = NASSA5(context);

		NASSA5(context) = client_data;

		client_data->client_data = NULL;

		if (!last_client) {
		    CLIENT(context) = client_data;
		    client_data->client_miss = NULL;
		} else {
		    client_data->client_miss = last_client;
		    last_client->client_data = client_data;
		}

#if __ENABLE_DEBUG__
		Mylene200_dump_clients(context);
#endif				/* __ENABLE_DEBUG__ */
	    }
	}
    }

    return status;
}


static int
send_reflex(void *context)
{
    int status;
    struct stripe_type *stripe_data;
    void *data;

    stripe_data = STRIPE(context);

    data = stripe_data->data;

    status = write(REFLEX(context), data, 1);
    if (status == -1) {
	if (errno == EINTR) {
	    status = 0;
	} else {
	    Mylene200_lose_reflex(context);
	}
    } else {
	if (!status) {
	    status = -1;
	    Mylene200_fail_reflex(context);
	} else {
	    unsigned size;

	    status = 0;

	    size = stripe_data->size;

	    size--;

#if __SEND_DEBUG__
	    Mylene200_fprintf_error
		(context, "writing Mylene1f4 %02x\n", *(unsigned char *) data);
#endif				/* __SEND_DEBUG__ */

	    if (size) {
		stripe_data->data = (char *) data + 1;
		stripe_data->size = size;
	    } else {
		struct stripe_type *post_stripe;

		post_stripe = stripe_data->stripe_data;

		if (STOYA7(context)) {
		    free(stripe_data);
		} else {
		    STOYA7(context) = stripe_data;
		}

		if (post_stripe) {
		    STRIPE(context) = post_stripe;
		} else {
		    STOYA6(context) = NULL;
		    STRIPE(context) = NULL;

		    FD_CLR(REFLEX(context), &SET_1(context));
		}
	    }
	}
    }

    return status;
}


int
Mylene200_line_server(void *context)
{
    int status;

#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
    BREAK6(context) = NULL;
    BREAK7(context) = NULL;
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */
    CLASS9(context) = NULL;
    CLIENT(context) = NULL;
#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
    JUDITH(context) = NULL;
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */
    NASSA5(context) = NULL;
    NFDS(context) = SERVER(context) + 1;
    STOYA6(context) = NULL;
    STOYA7(context) = NULL;
    STRIPE(context) = NULL;
#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
    THREAD(context) = NULL;
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */
    WINDOW(context) = NULL;

    Mylene200_flat_reflex(context);

    FD_SET(REFLEX(context), &SET_0(context));
    if (0) {
	FD_SET(REFLEX(context), &SET_1(context));
    }

    if (REFLEX(context) < NFDS(context)) {
    } else {
	NFDS(context) = REFLEX(context) + 1;
    }

#if __NONBLOCKING_REFLEX_TRANSFER__
    if (FLAGS(context) & Mylene200_LOCK_REFLEX) {
	SEND(context) = send_reflex;
    } else {
	SEND(context) = lock_reflex;
    }
#endif				/* __NONBLOCKING_REFLEX_TRANSFER__ */

#if __NONBLOCKING_REFLEX_TRANSFER__
    if (1) {
	READ(context) = Mylene200_read_reflex;
    }
#endif				/* __NONBLOCKING_REFLEX_TRANSFER__ */

#if __SEND_DEBUG__
    if (1) {
	SEND(context) = send_reflex;
    }
#endif				/* __SEND_DEBUG__ */

    status = line_server(context);

    if (1) {
	int excess;

	excess = miss_window(context);
	if (!status) {
	    status = excess;
	}
    }

#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
    if (1) {
	int excess;

	excess = miss_thread(context);
	if (!status) {
	    status = excess;
	}
    }
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */

    if (1) {
	int excess;

	excess = miss_stripe(context);
	if (!status) {
	    status = excess;
	}
    }

#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
    if (1) {
	int excess;

	excess = miss_shrews(context);
	if (!status) {
	    status = excess;
	}
    }
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */

    if (1) {
	int excess;

	excess = miss_client(context);
	if (!status) {
	    status = excess;
	}
    }

    return status;
}
