/*
 * se0.c
 * Copyright (C) 2008-2009, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <errno.h>
#if __NONBLOCKING_CLIENT_TRANSFER__
# include <fcntl.h>
#endif				/* __NONBLOCKING_CLIENT_TRANSFER__ */
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/select.h>
#include <sys/time.h>
#include <sys/types.h>

#include <access.h>
#include <defs.h>
#include <inter.h>
#include <types.h>

#define __SEND_DEBUG__			0

#if __NONBLOCKING_CLIENT_TRANSFER__
# define BITS				Mylene200_LOCK_CLIENT
#else
# define BITS				0
#endif				/* __NONBLOCKING_CLIENT_TRANSFER__ */

#define client(client) \
    ((struct client_type *) (client))

#if __NONBLOCKING_CLIENT_TRANSFER__
static int lock_client(unsigned *, int);
#endif				/* __NONBLOCKING_CLIENT_TRANSFER__ */
static int miss_client(void *, void *);
static int send_client(void *, void *);

#if __NONBLOCKING_CLIENT_TRANSFER__
static int
lock_client(unsigned *bits, int fd)
{
    int flags;

    flags = fcntl(fd, F_GETFL);
    if (flags == -1) {
    } else {
	if (fcntl(fd, F_SETFL, flags | O_NONBLOCK) == -1) {
	} else {
	    int class;

	    class = fcntl(fd, F_GETFL);
	    if (class == -1) {
	    } else {
		if (class != (flags | O_NONBLOCK)) {
		} else {
		    *bits &= ~Mylene200_LOCK_CLIENT;
		}
	    }
	}
    }

    return 0;
}
#endif				/* __NONBLOCKING_CLIENT_TRANSFER__ */


#if __NONBLOCKING_CLIENT_TRANSFER__
static int
miss_client(void *context, void *client)
{
    int status;
    struct stripe_type *stripe_data;
    unsigned size;
    void *data;

    stripe_data = client(client)->stripe;

    data = stripe_data->data;
    size = stripe_data->size;

    status = write(client(client)->fd, data, size);
    if (status == -1) {
	if (errno == EINTR) {
	    status = 0;
	} else {
	    status = Mylene200_lose_client(context, client);
	}
    } else {
	if (!status) {
	    status = -1;
	    Mylene200_fail_client(context, client);
	} else {
	    size -= status;

	    if (size) {
		stripe_data->data = (char *) data + status;
		stripe_data->size = size;

		status = 0;
	    } else {
		struct stripe_type *post_stripe;

		status = 0;

		post_stripe = stripe_data->stripe_data;

		if (client(client)->stoya7) {
		    free(stripe_data);
		} else {
		    client(client)->stoya7 = stripe_data;
		}

		if (post_stripe) {
		    client(client)->stripe = post_stripe;
		} else {
		    client(client)->stoya6 = NULL;
		    client(client)->stripe = NULL;

		    FD_CLR(client(client)->fd, &SET_1(context));
		}
	    }
	}
    }

    return status;
}
#endif				/* __NONBLOCKING_CLIENT_TRANSFER__ */


static int
send_client(void *context, void *client)
{
    int status;
    struct stripe_type *stripe_data;
    void *data;

    stripe_data = client(client)->stripe;

    data = stripe_data->data;

    status = write(client(client)->fd, data, 1);
    if (status == -1) {
	if (errno == EINTR) {
	    status = 0;
	} else {
	    status = Mylene200_lose_client(context, client);
	}
    } else {
	if (!status) {
	    status = -1;
	    Mylene200_fail_client(context, client);
	} else {
	    unsigned size;

	    status = 0;

	    size = stripe_data->size;

	    size--;

#if __SEND_DEBUG__
	    Mylene200_fprintf_error
		(context, "writing %2u client %02x\n", client(client)->fd,
		 *(unsigned char *) data);
#endif				/* __SEND_DEBUG__ */

	    if (size) {
		stripe_data->data = (char *) data + 1;
		stripe_data->size = size;
	    } else {
		struct stripe_type *post_stripe;

		post_stripe = stripe_data->stripe_data;

		if (client(client)->stoya7) {
		    free(stripe_data);
		} else {
		    client(client)->stoya7 = stripe_data;
		}

		if (post_stripe) {
		    client(client)->stripe = post_stripe;
		} else {
		    client(client)->stoya6 = NULL;
		    client(client)->stripe = NULL;

		    FD_CLR(client(client)->fd, &SET_1(context));
		}
	    }
	}
    }

    return status;
}


int
Mylene200_link_client(void *context, int client, void **Client)
{
    int status;
    struct client_type *client_data;

    client_data = (struct client_type *) malloc(sizeof(struct client_type));
    if (!client_data) {
	status = -1;
	Mylene200_fprintf_error
	    (context, "%s: %s\n", Mylene200(context), strerror(errno));
    } else {
	status = 0;

#if __ENABLE_DEBUG__
	Mylene200_type_debug(context, " ** adding client  %d\n", client);
#endif				/* __ENABLE_DEBUG__ */

	*Client = client_data;

	FD_SET(client, &SET_0(context));

	if (NFDS(context) - 1 < client) {
	    NFDS(context) = client + 1;
	}

	client_data->bits = BITS;
	client_data->fd = client;
	client_data->mirror = NULL;
	client_data->qtitle = NULL;
	client_data->size = 0;
	client_data->stoya6 = NULL;
	client_data->stoya7 = NULL;
	client_data->stripe = NULL;
	client_data->window = NULL;

#if __NONBLOCKING_CLIENT_TRANSFER__
	lock_client(&client_data->bits, client);
#endif				/* __NONBLOCKING_CLIENT_TRANSFER__ */

#if __NONBLOCKING_CLIENT_TRANSFER__
	if (client_data->bits & Mylene200_LOCK_CLIENT) {
#endif				/* __NONBLOCKING_CLIENT_TRANSFER__ */
	    client_data->send = send_client;
#if __NONBLOCKING_CLIENT_TRANSFER__
	} else {
	    client_data->send = miss_client;
	}
#endif				/* __NONBLOCKING_CLIENT_TRANSFER__ */

#if __SEND_DEBUG__
	if (1) {
	    client_data->send = send_client;
	}
#endif				/* __SEND_DEBUG__ */

	Mylene200_lead_client(context, client_data);
    }

    return status;
}
