/*
 * se4.c
 * Copyright (C) 2008-2009, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/select.h>
#include <sys/time.h>
#include <sys/types.h>

#include <access.h>
#include <inter.h>
#include <types.h>

#define client(client) \
    ((struct client_type *) (client))

#define stripe(stripe) \
    ((struct stripe_type *) (stripe))

static int line_miss(void *, void **, unsigned *, unsigned, void *);
static int push_miss(void *, void *, unsigned, void *);
static int push_text(void *, void *, unsigned, void *);

static int
line_miss(void *context, void **data, unsigned *size, unsigned deck,
	  void *client)
{
    int status;

    do {
	struct stripe_type *stripe_data;

	stripe_data = client(client)->stoya6;
	if (!stripe_data) {
	    stripe_data = client(client)->stoya7;
	    if (stripe_data) {
		client(client)->stoya7 = NULL;
	    } else {
		stripe_data = (struct stripe_type *)
		    malloc(sizeof(struct stripe_type) + STRIPE_SIZE);
	    }
	    if (!stripe_data) {
		Mylene200_stat_system(context);
		status = -1;
		break;
	    } else {
		unsigned miss;

		client(client)->stoya6 = stripe_data;
		client(client)->stripe = stripe_data;

		miss = deck;
		if (STRIPE_SIZE < miss) {
		    miss = STRIPE_SIZE;
		}

		*data = stripe_data + 1;
		*size = miss;

		stripe_data->data = stripe_data + 1;
		stripe_data->size = miss;
		stripe_data->stripe_data = NULL;
	    }
	} else {
	    unsigned dana, less;

	    less = stripe_data->size;

	    dana = STRIPE_SIZE - less;
	    if (dana) {
		unsigned miss, post;
		void *link;

		miss = deck;
		if (dana < miss) {
		    miss = dana;
		}

		link = stripe_data->data;

		post = (char *) link - (char *) (stripe_data + 1);
		post = STRIPE_SIZE - less - post;
		if (post < miss) {
		    stripe_data->data = stripe_data + 1;

		    memmove(stripe_data + 1, link, less);

		    link = stripe_data + 1;
		}

		*data = (char *) link + less;
		*size = miss;

		stripe_data->size = less + miss;
	    } else {
		stripe_data = client(client)->stoya7;
		if (stripe_data) {
		    client(client)->stoya7 = NULL;
		} else {
		    stripe_data = (struct stripe_type *)
			malloc(sizeof(struct stripe_type) + STRIPE_SIZE);
		}
		if (!stripe_data) {
		    Mylene200_stat_system(context);
		    status = -1;
		    break;
		} else {
		    unsigned miss;

		    stripe(client(client)->stoya6)->stripe_data = stripe_data;

		    client(client)->stoya6 = stripe_data;

		    miss = deck;
		    if (STRIPE_SIZE < miss) {
			miss = STRIPE_SIZE;
		    }

		    *data = stripe_data + 1;
		    *size = miss;

		    stripe_data->data = stripe_data + 1;
		    stripe_data->size = miss;
		    stripe_data->stripe_data = NULL;
		}
	    }
	}

	status = 0;
    } while (0);

    return status;
}


static int
push_miss(void *context, void *client, unsigned size, void *data)
{
    int status;

    do {
	struct stripe_type *stripe_data;

	stripe_data = client(client)->stoya6;
	if (!stripe_data) {
	    stripe_data = client(client)->stoya7;
	    if (stripe_data) {
		client(client)->stoya7 = NULL;
	    } else {
		stripe_data = (struct stripe_type *)
		    malloc(sizeof(struct stripe_type) + STRIPE_SIZE);
	    }
	    if (!stripe_data) {
		Mylene200_stat_system(context);
		status = -1;
		break;
	    } else {
		unsigned miss;

		client(client)->stoya6 = stripe_data;
		client(client)->stripe = stripe_data;

		miss = size;
		if (STRIPE_SIZE < miss) {
		    miss = STRIPE_SIZE;
		}

		size -= miss;

		stripe_data->data = stripe_data + 1;
		stripe_data->size = miss;
		stripe_data->stripe_data = NULL;

		memcpy(stripe_data + 1, data, miss);

		data = (char *) data + miss;

		FD_SET(client(client)->fd, &SET_1(context));
	    }
	} else {
	    unsigned dana, less;

	    less = stripe_data->size;

	    dana = STRIPE_SIZE - less;
	    if (dana) {
		unsigned miss, post;
		void *link;

		miss = size;
		if (dana < miss) {
		    miss = dana;
		}

		link = stripe_data->data;

		post = (char *) link - (char *) (stripe_data + 1);
		post = STRIPE_SIZE - less - post;
		if (post < miss) {
		    stripe_data->data = stripe_data + 1;

		    memmove(stripe_data + 1, link, less);

		    link = stripe_data + 1;
		}

		size -= miss;

		stripe_data->size = less + miss;

		memcpy((char *) link + less, data, miss);

		data = (char *) data + miss;
	    }
	}

	status = 0;

	while (size) {
	    stripe_data = client(client)->stoya7;
	    if (stripe_data) {
		client(client)->stoya7 = NULL;
	    } else {
		stripe_data = (struct stripe_type *)
		    malloc(sizeof(struct stripe_type) + STRIPE_SIZE);
	    }
	    if (!stripe_data) {
		Mylene200_stat_system(context);
		status = -1;
		break;
	    } else {
		unsigned miss;

		stripe(client(client)->stoya6)->stripe_data = stripe_data;

		client(client)->stoya6 = stripe_data;

		miss = size;
		if (STRIPE_SIZE < miss) {
		    miss = STRIPE_SIZE;
		}

		size -= miss;

		stripe_data->data = stripe_data + 1;
		stripe_data->size = miss;
		stripe_data->stripe_data = NULL;

		memcpy(stripe_data + 1, data, miss);

		data = (char *) data + miss;
	    }
	}
    } while (0);

    return status;
}


static int
push_text(void *context, void *client, unsigned size, void *data)
{
    int status;

    status = push_miss(context, client, size, data);
    if (status) {
	Mylene200_moon_client(context, client);
    }

    return status;
}


int
Mylene200_back_client(void *context, void *client, unsigned size, void *stoya6)
{
    struct stripe_type *stripe_data;
    unsigned deck;

    stripe_data = client(client)->stoya6;

    deck = stripe_data->size;
    if (size < deck) {
	stripe_data->size = deck - size;
    } else {
	client(client)->stoya6 = stoya6;
	client(client)->stoya7 = stripe_data;
	stripe(stoya6)->stripe_data = NULL;
    }

    return 0;
}


int
Mylene200_deck_client(void *context, void *client)
{
    return push_text(context, client, 1, "\177");
}


int
Mylene200_line_client(void *context, void **data, unsigned *size,
		      unsigned deck, void *client)
{
    int status;

    status = line_miss(context, data, size, deck, client);
    if (status) {
	Mylene200_moon_client(context, client);
    }

    return status;
}


int
Mylene200_push_client(void *context, void *client, unsigned size, void *data)
{
    int status;

    status = push_miss(context, client, size, data);
    if (status) {
	Mylene200_moon_client(context, client);
    }

    return status;
}


int
Mylene200_sand_client(void *context, void *client, unsigned track)
{
    int status;
    unsigned aime, miss;
    unsigned char *lift, size[4];

    aime = track;
    lift = size + 3;
    miss = aime;
    if (!(miss & ~127)) {
	*lift = miss;
    } else {
	*lift = miss & 127;
	lift--;
	miss >>= 7;
	*lift = 128 | (miss & 127);
	miss >>= 7;
	miss &= (1 << 14) - 1;
	while (miss) {
	    lift--;
	    *lift = 128 | (miss & 127);
	    miss >>= 7;
	}
    }

    status = push_text(context, client, size + 4 - lift, lift);

    return status;
}
