/*
 * se5.c
 * Copyright (C) 2008, 2009, 2010, 2011, 2013, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#include <defs.h>
#include <inter.h>
#include <types.h>

#define cassie(client) \
    wynona(client).data.ceil

#define client(client) \
    ((struct client_type *) (client))

#define eufrat(client) \
    cassie(client).data.byte

#define lilith(client) \
    cassie(client).data.sign

#define sienna(client) \
    cassie(client).data.text

#define wynona(client) \
    client(client)->data.miss

#if __REFLEX_SELECT__ != __DOUBLE_SELECT__
# define case_lose			NULL
# define case_well			NULL
#endif				/* __REFLEX_SELECT__ != __DOUBLE_SELECT__ */

#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
#else
# define case_fast			case_near
# define case_look			case_side
# define case_trip			case_mail
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */

static int case_away(void *, void *);
static int case_back(void *, void *);
static int case_bias(void *, void *);
static int case_byte(void *, void *);
static int case_ceil(void *, void *);
static int case_cell(void *, void *);
static int case_dana(void *, void *);
static int case_deck(void *, void *);
static int case_diva(void *, void *);
#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int case_fast(void *, void *);
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */
static int case_fine(void *, void *);
static int case_flat(void *, void *);
static int case_land(void *, void *);
static int case_lead(void *, void *);
static int case_left(void *, void *);
static int case_less(void *, void *);
static int case_lime(void *, void *);
#if __NONBLOCKING_CLIENT_TRANSFER__
static int case_line(void *, void *);
#endif				/* __NONBLOCKING_CLIENT_TRANSFER__ */
static int case_link(void *, void *, int *);
static int case_lock(void *, void *);
#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int case_look(void *, void *);
static int case_lose(void *, void *);
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */
static int case_mail(void *, void *);
static int case_mine(void *, void *);
static int case_miss(void *, void *);
static int case_mode(void *, void *);
static int case_more(void *, void *);
static int case_name(void *, void *);
static int case_near(void *, void *);
#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int case_news(void *, void *);
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */
static int case_nine(void *, void *);
static int case_node(void *, void *);
static int case_none(void *, void *);
static int case_note(void *, void *);
static int case_nova(void *, void *);
#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int case_null(void *, void *);
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */
static int case_pipe(void *, void *, int *);
static int case_post(void *, void *);
static int case_side(void *, void *);
static int case_sign(void *, void *);
static int case_slip(void *, void *);
static int case_tale(void *, void *);
static int case_text(void *, void *);
static int case_time(void *, void *);
#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int case_trip(void *, void *);
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */
static int case_type(void *, void *);
#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int case_well(void *, void *);
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */
static int ever_name(void *, void *);
static int line_away(void *, void *);
static int line_bias(void *, void *);
static int line_cell(void *, void *);
static int line_dana(void *, void *);
static int line_flat(void *, void *);
static int line_left(void *, void *);
static int line_mail(void *, void *);
static int line_mode(void *, void *);
static int line_nine(void *, void *);
static int line_none(void *, void *);
static int line_text(void *, void *);
static int line_time(void *, void *);
static int link_diva(void *, void *);
static int link_mine(void *, void *);
static int link_nova(void *, void *);
static int miss_byte(void *, void *);
static int miss_time(void *, void *);
static int post_name(void *, void *, int *);
static int push_cell(void *, void *, unsigned);
static int push_left(void *, void *, unsigned, unsigned);

static int (*byte_miss[]) (void *, void *) = {
/* *INDENT-OFF* */
    /* 00 -- EXIT         */
    case_deck,
    /* 01 -- SKIP         */
    case_deck,
    /* 02 -- SPACE        */
    case_deck,
    /* 03 -- CYCLE        */
    case_deck,
    /* 04 -- START        */
    case_ceil,
    /* 05 -- StarT        */
    case_deck,
    /* 06 -- BACK         */
    case_deck,
    /* 07 -- JOIN         */
    case_deck,
    /* 08 -- RAND         */
    case_deck,
    /* 09 -- SKIP1        */
    case_sign,
    /* 0a -- CYCLE_off    */
    case_deck,
    /* 0b -- CYCLE_On     */
    case_deck,
    /* 0c -- SPACE_off    */
    case_deck,
    /* 0d -- SPACE_On     */
    case_deck,
    /* 0e -- BACK1        */
    case_sign,
    /* 0f -- FORMAT       */
    case_none,
    /* 10 -- CYCLE_tail   */
    case_ceil,
    /* 11 -- ORDER_n      */
    case_deck,
    /* 12 -- ORDER_p      */
    case_deck,
    /* 13 -- ORDER_z      */
    case_deck,
    /* 14 -- LOAD         */
    case_deck,
    /* 15 -- CLEAR        */
    case_deck,
    /* 16 -- CYCLE_m      */
    case_sign,
    /* 17 -- CYCLE_p      */
    case_sign,
    /* 18 -- BEGIN        */
    case_time,
    /* 19 -- BegiN        */
    case_deck,
    /* 1a -- PICK         */
    case_sign,
    /* 1b -- PICK_m       */
    case_sign,
    /* 1c -- PICK_p       */
    case_sign,
    /* 1d -- SEEK         */
    case_time,
    /* 1e -- SEEK_m       */
    case_time,
    /* 1f -- SEEK_p       */
    case_time,
    /* 20 -- DETACH       */
    case_deck,
    /* 21 -- ATTACH       */
    case_deck,
    /* 22 -- QUEUE        */
    case_fast,
    /* 23 -- ABORT        */
    case_deck,
    /* 24 -- DEQUE        */
    case_deck,
    /* 25 -- DEQUE_c      */
    case_ceil,
    /* 26 -- DEQUE_all    */
    case_deck,
    /* 27 -- WIND         */
    case_deck,
    /* 28 -- DIVA         */
    case_diva,
    /* 29 -- DELETE       */
    case_sign,
    /* 2a -- SHUFFLE      */
    case_deck,
    /* 2b -- RESTORE      */
    case_deck,
    /* 2c -- HEAD         */
    case_deck,
    /* 2d -- TAIL         */
    case_deck,
    /* 2e -- CLASS_3      */
    case_nine,
    /* 2f -- WIND1        */
    case_sign,
    /* 30 -- LEVEL_3      */
    case_nine,
    /* 31 -- LESS1        */
    case_text,
    /* 32 -- LESS2        */
    case_text,
    /* 33 -- LESS3        */
    case_text,
    /* 34 -- HEAD_n       */
    case_sign,
    /* 35 -- TAIL_n       */
    case_sign,
    /* 36 -- SCALE_3      */
    case_sign,
    /* 37 -- INSERT       */
    case_look,
    /* 38 -- SUSPEND      */
    case_deck,
    /* 39 -- MOVE         */
    case_bias,
    /* 3a -- ADD          */
    case_text,
    /* 3b -- REMOVE       */
    case_ceil,
    /* 3c -- ReMOvE       */
    case_land,
    /* 3d -- Add          */
    case_type,
    /* 3e -- SuSpEnD      */
    case_sign,
    /* 3f -- FLUSH        */
    case_deck,
    /* 40 -- AdD          */
    case_flat,
    /* 41 -- AlbuM        */
    case_deck,
    /* 42 -- LbasE        */
    case_deck,
    /* 43 */
    NULL,
    /* 44 */
    NULL,
    /* 45 */
    NULL,
    /* 46 */
    NULL,
    /* 47 */
    NULL,
    /* 48 */
    NULL,
    /* 49 */
    NULL,
    /* 4a */
    NULL,
    /* 4b */
    NULL,
    /* 4c */
    NULL,
    /* 4d -- MatcH        */
    case_deck,
    /* 4e -- NoteS        */
    case_deck,
    /* 4f */
    NULL,
    /* 50 -- PresS        */
    case_deck,
    /* 51 -- LpatH        */
    case_deck,
    /* 52 */
    NULL,
    /* 53 */
    NULL,
    /* 54 -- TitlE        */
    case_deck,
    /* 55 */
    NULL,
    /* 56 */
    NULL,
    /* 57 */
    NULL,
    /* 58 */
    NULL,
    /* 59 */
    NULL,
    /* 5a */
    NULL,
    /* 5b -- POST         */
    case_well,
    /* 5c -- INSERT_s     */
    case_look,
    /* 5d -- INSERT_c     */
    case_look,
    /* 5e -- DELETE_c     */
    case_sign,
    /* 5f -- DELETE_s     */
    case_sign,
    /* 60 -- ADd          */
    case_less,
    /* 61 -- ALBUM        */
    case_text,
    /* 62 -- LBASE        */
    case_text,
    /* 63 */
    NULL,
    /* 64 */
    NULL,
    /* 65 */
    NULL,
    /* 66 */
    NULL,
    /* 67 */
    NULL,
    /* 68 */
    NULL,
    /* 69 */
    NULL,
    /* 6a */
    NULL,
    /* 6b */
    NULL,
    /* 6c */
    NULL,
    /* 6d -- MATCH        */
    case_text,
    /* 6e -- NOTES        */
    case_text,
    /* 6f */
    NULL,
    /* 70 -- PRESS        */
    case_text,
    /* 71 -- LPATH        */
    case_text,
    /* 72 */
    NULL,
    /* 73 */
    NULL,
    /* 74 -- TITLE        */
    case_text,
    /* 75 */
    NULL,
    /* 76 */
    NULL,
    /* 77 */
    NULL,
    /* 78 */
    NULL,
    /* 79 */
    NULL,
    /* 7a */
    NULL,
    /* 7b */
    NULL,
    /* 7c -- INSTANT      */
    case_trip,
    /* 7d -- SCRAP        */
    case_deck,
    /* 7e -- LOSE         */
    case_lose,
    /* 7f -- Flush        */
    case_back,
    /* 80 -- MAIL         */
    case_mail,
    /* 81 -- Album        */
    case_deck,
    /* 82 */
    NULL,
    /* 83 */
    NULL,
    /* 84 */
    NULL,
    /* 85 */
    NULL,
    /* 86 */
    NULL,
    /* 87 */
    NULL,
    /* 88 */
    NULL,
    /* 89 */
    NULL,
    /* 8a */
    NULL,
    /* 8b */
    NULL,
    /* 8c */
    NULL,
    /* 8d -- Match        */
    case_deck,
    /* 8e -- Notes        */
    case_deck,
    /* 8f */
    NULL,
    /* 90 */
    NULL,
    /* 91 */
    NULL,
    /* 92 */
    NULL,
    /* 93 */
    NULL,
    /* 94 -- Title        */
    case_deck,
    /* 95 */
    NULL,
    /* 96 */
    NULL,
    /* 97 */
    NULL,
    /* 98 */
    NULL,
    /* 99 */
    NULL,
    /* 9a */
    NULL,
    /* 9b */
    NULL,
    /* 9c */
    NULL,
    /* 9d */
    NULL,
    /* 9e */
    NULL,
    /* 9f */
    NULL,
    /* a0 */
    NULL,
    /* a1 */
    NULL,
    /* a2 */
    NULL,
    /* a3 */
    NULL,
    /* a4 */
    NULL,
    /* a5 */
    NULL,
    /* a6 */
    NULL,
    /* a7 */
    NULL,
    /* a8 */
    NULL,
    /* a9 */
    NULL,
    /* aa */
    NULL,
    /* ab */
    NULL,
    /* ac */
    NULL,
    /* ad */
    NULL,
    /* ae */
    NULL,
    /* af */
    NULL,
    /* b0 */
    NULL,
    /* b1 -- Less1        */
    case_deck,
    /* b2 -- Less2        */
    case_deck,
    /* b3 -- Less3        */
    case_deck,
    /* b4 */
    NULL,
    /* b5 */
    NULL,
    /* b6 */
    NULL,
    /* b7 */
    NULL,
    /* b8 */
    NULL,
    /* b9 */
    NULL,
    /* ba */
    NULL,
    /* bb */
    NULL,
    /* bc */
    NULL,
    /* bd */
    NULL,
    /* be */
    NULL,
    /* bf */
    NULL,
    /* c0 */
    NULL,
    /* c1 */
    NULL,
    /* c2 */
    NULL,
    /* c3 */
    NULL,
    /* c4 */
    NULL,
    /* c5 */
    NULL,
    /* c6 */
    NULL,
    /* c7 */
    NULL,
    /* c8 */
    NULL,
    /* c9 */
    NULL,
    /* ca */
    NULL,
    /* cb */
    NULL,
    /* cc */
    NULL,
    /* cd */
    NULL,
    /* ce */
    NULL,
    /* cf */
    NULL,
    /* d0 */
    NULL,
    /* d1 */
    NULL,
    /* d2 */
    NULL,
    /* d3 */
    NULL,
    /* d4 */
    NULL,
    /* d5 */
    NULL,
    /* d6 */
    NULL,
    /* d7 */
    NULL,
    /* d8 */
    NULL,
    /* d9 */
    NULL,
    /* da */
    NULL,
    /* db */
    NULL,
    /* dc */
    NULL,
    /* dd */
    NULL,
    /* de */
    NULL,
    /* df */
    NULL,
    /* e0 */
    NULL,
    /* e1 */
    NULL,
    /* e2 */
    NULL,
    /* e3 */
    NULL,
    /* e4 */
    NULL,
    /* e5 */
    NULL,
    /* e6 */
    NULL,
    /* e7 */
    NULL,
    /* e8 */
    NULL,
    /* e9 */
    NULL,
    /* ea */
    NULL,
    /* eb */
    NULL,
    /* ec */
    NULL,
    /* ed */
    NULL,
    /* ee */
    NULL,
    /* ef */
    NULL,
    /* f0 */
    NULL,
    /* f1 */
    NULL,
    /* f2 */
    NULL,
    /* f3 */
    NULL,
    /* f4 */
    NULL,
    /* f5 */
    NULL,
    /* f6 */
    NULL,
    /* f7 */
    NULL,
    /* f8 */
    NULL,
    /* f9 */
    NULL,
    /* fa */
    NULL,
    /* fb */
    NULL,
    /* fc */
    NULL,
    /* fd */
    NULL,
    /* fe */
    NULL,
    /* ff */
    NULL
/* *INDENT-ON* */
};

static int
case_away(void *context, void *client)
{
    int excess, status;

    client(client)->read = case_cell;

    wynona(client).data.ceil.miss = line_away;

    status = case_link(context, client, &excess);

    return status;
}


static int
case_back(void *context, void *client)
{
    int status;

    status = Mylene200_push_reflex
	(context, 1, &client(client)->data.miss.deck);
    if (!status) {
	status = Mylene200_lock_reflex(context, 0177, client);
	if (!status) {
	    client(client)->read = miss_byte;
	}
    }

    return status;
}


static int
case_bias(void *context, void *client)
{
    client(client)->read = case_left;

    wynona(client).data.ceil.miss = case_more;

    return 0;
}


static int
case_byte(void *context, void *client)
{
    int status;
    unsigned size;

    status = Mylene200_read_client
	(context, client, &cassie(client).data.byte.byte, 1, &size);
    if (status) {
    } else {
	if (size) {
	    status = wynona(client).data.ceil.miss(context, client);
	}
    }

    return status;
}


static int
case_ceil(void *context, void *client)
{
    client(client)->read = case_cell;

    wynona(client).data.ceil.deck = 0;
    wynona(client).data.ceil.miss = line_cell;

    return 0;
}


static int
case_cell(void *context, void *client)
{
    char data[1];
    int status;
    unsigned size;

    status = Mylene200_read_client(context, client, data, 1, &size);
    if (status) {
    } else {
	if (size) {
	    unsigned c;

	    c = *(unsigned char *) data;
	    if (c & 128) {
		c &= ~128;

		size = wynona(client).data.ceil.deck;
		size <<= 7;
		size += c;
		wynona(client).data.ceil.deck = size;
	    } else {
		size = wynona(client).data.ceil.deck;
		size <<= 7;
		size += c;
		wynona(client).data.ceil.deck = size;

		status = wynona(client).data.ceil.miss(context, client);
	    }
	}
    }

    return status;
}


static int
case_dana(void *context, void *client)
{
    client(client)->read = case_cell;

    wynona(client).data.ceil.deck = 0;
    wynona(client).data.ceil.miss = line_dana;

    return 0;
}


static int
case_deck(void *context, void *client)
{
    int status;

    status = Mylene200_push_reflex
	(context, 1, &client(client)->data.miss.deck);
    if (!status) {
	client(client)->read = miss_byte;
    }

    return status;
}


static int
case_diva(void *context, void *client)
{
    client(client)->read = case_cell;

    wynona(client).data.ceil.deck = 0;
    wynona(client).data.ceil.miss = case_name;

    return 0;
}


#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int
case_fast(void *context, void *client)
{
    int status;

    status = Mylene200_fast_thread(context, client);
    if (status) {
    } else {
	status = case_near(context, client);
    }

    return status;
}
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */


static int
case_fine(void *context, void *client)
{
    int excess, status;

    wynona(client).data.ceil.miss = line_mail;

    status = case_link(context, client, &excess);

    return status;
}


static int
case_flat(void *context, void *client)
{
    client(client)->read = case_left;

    wynona(client).data.ceil.miss = case_tale;

    return 0;
}


static int
case_land(void *context, void *client)
{
    client(client)->read = case_byte;

    wynona(client).data.ceil.miss = case_dana;

    return 0;
}


static int
case_lead(void *context, void *client)
{
    sienna(client).data.pair.call = wynona(client).data.ceil.deck;

    client(client)->read = case_cell;

    wynona(client).data.ceil.deck = 0;
    wynona(client).data.ceil.miss = case_fine;

    return 0;
}


static int
case_left(void *context, void *client)
{
    char data[1];
    int status;
    unsigned size;

    status = Mylene200_read_client(context, client, data, 1, &size);
    if (status) {
    } else {
	if (size) {
	    unsigned c;

	    c = *(unsigned char *) data;
	    if (c == 64) {
		status = Mylene200_post_client(context, client);
	    } else {
		cassie(client).data.sign.deck = c & 0x40;
		wynona(client).data.ceil.deck = c & 0077;

		if (c & 128) {
		    client(client)->read = case_cell;
		} else {
		    status = wynona(client).data.ceil.miss(context, client);
		}
	    }
	}
    }

    return status;
}


static int
case_less(void *context, void *client)
{
    client(client)->read = case_left;

    wynona(client).data.ceil.miss = case_lime;

    return 0;
}


static int
case_lime(void *context, void *client)
{
    eufrat(client).data.miss.deck = cassie(client).data.sign.deck;

    client(client)->read = case_byte;

    wynona(client).data.ceil.miss = case_lock;

    return 0;
}


#if __NONBLOCKING_CLIENT_TRANSFER__
static int
case_line(void *context, void *client)
{
    int status;
    unsigned miss, size;
    void *data;

    data = cassie(client).data.text.slip;
    miss = cassie(client).data.text.miss;

    status = Mylene200_read_client(context, client, data, miss, &size);
    if (status) {
    } else {
	if (size) {
	    miss -= size;
	    if (miss) {
		cassie(client).data.text.miss = miss;
		cassie(client).data.text.slip = (char *) data + size;
	    } else {
		status = wynona(client).data.ceil.miss(context, client);
	    }
	}
    }

    return status;
}
#endif				/* __NONBLOCKING_CLIENT_TRANSFER__ */


static int
case_link(void *context, void *client, int *class)
{
    int status;

    if (wynona(client).data.ceil.deck) {
	status = case_pipe(context, client, class);
    } else {
	cassie(client).data.text.miss = 0;
	status = wynona(client).data.ceil.miss(context, client);
	if (status) {
	} else {
	    *class = 0;
	}
    }

    return status;
}


static int
case_lock(void *context, void *client)
{
    unsigned byte, sign;

    byte = cassie(client).data.byte.byte;
    sign = eufrat(client).data.miss.deck;

    sienna(client).data.nine.byte = byte;
    sienna(client).data.nine.deck = wynona(client).data.ceil.deck;
    sienna(client).data.nine.sign = sign;

    client(client)->read = case_cell;

    wynona(client).data.ceil.deck = 0;
    wynona(client).data.ceil.miss = case_away;

    return 0;
}


#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int
case_look(void *context, void *client)
{
    int status;

    status = Mylene200_fast_thread(context, client);
    if (status) {
    } else {
	status = case_side(context, client);
    }

    return status;
}
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */


#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int
case_lose(void *context, void *client)
{
    client(client)->read = case_cell;

    wynona(client).data.ceil.deck = 0;
    wynona(client).data.ceil.miss = case_null;

    return 0;
}
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */


static int
case_mail(void *context, void *client)
{
    client(client)->read = case_cell;

    wynona(client).data.ceil.deck = 0;
    wynona(client).data.ceil.miss = case_note;

    return 0;
}


static int
case_mine(void *context, void *client)
{
    int excess, status;

    wynona(client).data.ceil.miss = line_none;

    status = case_link(context, client, &excess);

    return status;
}


static int
case_miss(void *context, void *client)
{
    int excess, status;

    wynona(client).data.ceil.miss = line_text;

    status = case_link(context, client, &excess);

    return status;
}


static int
case_mode(void *context, void *client)
{
    int excess, status;

    sienna(client).data.mode.byte = cassie(client).data.byte.byte;

    client(client)->read = case_cell;

    wynona(client).data.ceil.miss = line_mode;

    status = case_link(context, client, &excess);

    return status;
}


static int
case_more(void *context, void *client)
{
    client(client)->read = case_left;

    lilith(client).data.slip.deck = wynona(client).data.ceil.deck;
    lilith(client).data.slip.sign = cassie(client).data.sign.deck;

    wynona(client).data.ceil.miss = line_bias;

    return 0;
}


static int
case_name(void *context, void *client)
{
    int excess, status;

    wynona(client).data.ceil.miss = ever_name;

    status = case_link(context, client, &excess);

    return status;
}


static int
case_near(void *context, void *client)
{
    int status;

    status = link_diva(context, client);
    if (status) {
    } else {
	status = case_text(context, client);
    }

    return status;
}


#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int
case_news(void *context, void *client)
{
    int status;

    status = Mylene200_line_thread
	(context, client, wynona(client).data.ceil.deck);
    if (status) {
    } else {
	client(client)->read = miss_byte;
    }

    return status;
}
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */


static int
case_nine(void *context, void *client)
{
    client(client)->read = case_cell;

    wynona(client).data.ceil.deck = 0;
    wynona(client).data.ceil.miss = case_post;

    return 0;
}


static int
case_node(void *context, void *client)
{
    client(client)->read = case_cell;

    wynona(client).data.ceil.deck = 0;
    wynona(client).data.ceil.miss = case_mode;

    return 0;
}


static int
case_none(void *context, void *client)
{
    client(client)->read = case_cell;

    wynona(client).data.ceil.deck = 0;
    wynona(client).data.ceil.miss = case_mine;

    return 0;
}


static int
case_note(void *context, void *client)
{
    sienna(client).data.pair.deck = wynona(client).data.ceil.deck;

    client(client)->read = case_cell;

    wynona(client).data.ceil.deck = 0;
    wynona(client).data.ceil.miss = case_lead;

    return 0;
}


static int
case_nova(void *context, void *client)
{
    int excess, status;

    wynona(client).data.ceil.miss = line_flat;

    status = case_link(context, client, &excess);

    return status;
}


#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int
case_null(void *context, void *client)
{
    int status;

    status = Mylene200_flat_thread
	(context, client, wynona(client).data.ceil.deck);
    if (status) {
    } else {
	client(client)->read = miss_byte;
    }

    return status;
}
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */


static int
case_pipe(void *context, void *client, int *class)
{
    int status;

    do {
	unsigned size, text;
	void *data;

	size = client(client)->size;

	data = client(client)->mirror;

	text = wynona(client).data.ceil.deck;

	if (size < text) {
	    void *miss;

	    miss = malloc(text);
	    if (!miss) {
		status = Mylene200_miss_client(context, client);
		if (status) {
		} else {
		    *class = 1;
		}

		break;
	    } else {
		if (data) {
		    free(data);
		}

		data = miss;

		client(client)->size = text;

		client(client)->mirror = data;
	    }
	}

#if __NONBLOCKING_CLIENT_TRANSFER__
	if (client(client)->bits & Mylene200_LOCK_CLIENT) {
#endif				/* __NONBLOCKING_CLIENT_TRANSFER__ */
	    client(client)->read = case_slip;
#if __NONBLOCKING_CLIENT_TRANSFER__
	} else {
	    client(client)->read = case_line;
	}
#endif				/* __NONBLOCKING_CLIENT_TRANSFER__ */

	cassie(client).data.text.miss = text;
	cassie(client).data.text.slip = data;

	*class = 0;

	status = 0;
    } while (0);

    return status;
}


static int
case_post(void *context, void *client)
{
    client(client)->read = case_left;

    wynona(client).data.ceil.miss = line_nine;

    lilith(client).data.nine.deck = wynona(client).data.ceil.deck;

    return 0;
}


static int
case_side(void *context, void *client)
{
    int status;

    status = link_diva(context, client);
    if (status) {
    } else {
	status = case_flat(context, client);
    }

    return status;
}


static int
case_sign(void *context, void *client)
{
    client(client)->read = case_left;

    wynona(client).data.ceil.miss = line_left;

    return 0;
}


static int
case_slip(void *context, void *client)
{
    int status;
    unsigned size;
    void *data;

    data = cassie(client).data.text.slip;

    status = Mylene200_read_client(context, client, data, 1, &size);
    if (status) {
    } else {
	if (size) {
	    unsigned miss;

	    miss = cassie(client).data.text.miss;

	    miss--;
	    if (miss) {
		cassie(client).data.text.miss = miss;
		cassie(client).data.text.slip = (char *) data + 1;
	    } else {
		status = wynona(client).data.ceil.miss(context, client);
	    }
	}
    }

    return status;
}


static int
case_tale(void *context, void *client)
{
    client(client)->read = case_cell;

    sienna(client).data.slip.deck = wynona(client).data.ceil.deck;
    sienna(client).data.slip.sign = cassie(client).data.sign.deck;

    wynona(client).data.ceil.deck = 0;
    wynona(client).data.ceil.miss = case_nova;

    return 0;
}


static int
case_text(void *context, void *client)
{
    client(client)->read = case_cell;

    wynona(client).data.ceil.deck = 0;
    wynona(client).data.ceil.miss = case_miss;

    return 0;
}


static int
case_time(void *context, void *client)
{
    client(client)->read = case_cell;

    cassie(client).data.time.post = 3;

    wynona(client).data.ceil.deck = 0;
    wynona(client).data.ceil.miss = miss_time;

    return 0;
}


#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int
case_trip(void *context, void *client)
{
    int status;

    status = Mylene200_fast_thread(context, client);
    if (status) {
    } else {
	status = case_mail(context, client);
    }

    return status;
}
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */


static int
case_type(void *context, void *client)
{
    client(client)->read = case_byte;

    wynona(client).data.ceil.miss = case_node;

    return 0;
}


#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int
case_well(void *context, void *client)
{
    client(client)->read = case_cell;

    wynona(client).data.ceil.deck = 0;
    wynona(client).data.ceil.miss = case_news;

    return 0;
}
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */


static int
ever_name(void *context, void *client)
{
    int excess, status;

    status = post_name(context, client, &excess);
    if (status) {
    } else {
	if (excess) {
	} else {
	    client(client)->read = miss_byte;
	}
    }

    return status;
}


static int
line_away(void *context, void *client)
{
    int status;

    status = Mylene200_push_reflex
	(context, 1, &client(client)->data.miss.deck);
    if (status) {
    } else {
	status = push_left
	    (context, client, sienna(client).data.nine.sign,
	     sienna(client).data.nine.deck);
	if (status) {
	} else {
	    status = Mylene200_push_reflex
		(context, 1, &sienna(client).data.nine.byte);
	    if (status) {
	    } else {
		status = link_nova(context, client);
		if (status) {
		} else {
		    client(client)->read = miss_byte;
		}
	    }
	}
    }

    return status;
}


static int
line_bias(void *context, void *client)
{
    int status;

    status = Mylene200_push_reflex
	(context, 1, &client(client)->data.miss.deck);
    if (status) {
    } else {
	status = push_left
	    (context, client, lilith(client).data.slip.sign,
	     lilith(client).data.slip.deck);
	if (status) {
	} else {
	    status = push_left
		(context, client, cassie(client).data.sign.deck,
		 wynona(client).data.ceil.deck);
	    if (status) {
	    } else {
		client(client)->read = miss_byte;
	    }
	}
    }

    return status;
}


static int
line_cell(void *context, void *client)
{
    int status;

    status = Mylene200_push_reflex
	(context, 1, &client(client)->data.miss.deck);
    if (status) {
    } else {
	status = push_cell(context, client, wynona(client).data.ceil.deck);
	if (status) {
	} else {
	    client(client)->read = miss_byte;
	}
    }

    return status;
}


static int
line_dana(void *context, void *client)
{
    int status;

    status = Mylene200_push_reflex
	(context, 1, &client(client)->data.miss.deck);
    if (status) {
    } else {
	status = Mylene200_push_reflex
	    (context, 1, &cassie(client).data.byte.byte);
	if (status) {
	} else {
	    status = push_cell(context, client, wynona(client).data.ceil.deck);
	    if (status) {
	    } else {
		client(client)->read = miss_byte;
	    }
	}
    }

    return status;
}


static int
line_flat(void *context, void *client)
{
    int status;

    status = Mylene200_push_reflex
	(context, 1, &client(client)->data.miss.deck);
    if (status) {
    } else {
	status = push_left
	    (context, client, sienna(client).data.slip.sign,
	     sienna(client).data.slip.deck);
	if (status) {
	} else {
	    status = link_nova(context, client);
	    if (status) {
	    } else {
		client(client)->read = miss_byte;
	    }
	}
    }

    return status;
}


static int
line_left(void *context, void *client)
{
    int status;

    status = Mylene200_push_reflex
	(context, 1, &client(client)->data.miss.deck);
    if (status) {
    } else {
	status = push_left
	    (context, client, cassie(client).data.sign.deck,
	     wynona(client).data.ceil.deck);
	if (status) {
	} else {
	    client(client)->read = miss_byte;
	}
    }

    return status;
}


static int
line_mail(void *context, void *client)
{
    int status;

    status = Mylene200_push_reflex
	(context, 1, &client(client)->data.miss.deck);
    if (status) {
    } else {
	status = push_cell(context, client, sienna(client).data.pair.deck);
	if (status) {
	} else {
	    status = push_cell
		(context, client, sienna(client).data.pair.call);
	    if (status) {
	    } else {
		status = link_nova(context, client);
		if (status) {
		} else {
		    client(client)->read = miss_byte;
		}
	    }
	}
    }

    return status;
}


static int
line_mode(void *context, void *client)
{
    int status;

    status = Mylene200_push_reflex
	(context, 1, &client(client)->data.miss.deck);
    if (status) {
    } else {
	status = Mylene200_push_reflex
	    (context, 1, &sienna(client).data.mode.byte);
	if (status) {
	} else {
	    status = link_nova(context, client);
	    if (status) {
	    } else {
		client(client)->read = miss_byte;
	    }
	}
    }

    return status;
}


static int
line_nine(void *context, void *client)
{
    int status;

    status = Mylene200_push_reflex
	(context, 1, &client(client)->data.miss.deck);
    if (status) {
    } else {
	status = push_cell(context, client, lilith(client).data.nine.deck);
	if (status) {
	} else {
	    status = push_left
		(context, client, cassie(client).data.sign.deck,
		 wynona(client).data.ceil.deck);
	    if (status) {
	    } else {
		client(client)->read = miss_byte;
	    }
	}
    }

    return status;
}


static int
line_none(void *context, void *client)
{
    int status;

    status = link_mine(context, client);
    if (status) {
    } else {
	status = Mylene200_lock_reflex(context, 0017, client);
	if (status) {
	} else {
	    client(client)->read = miss_byte;
	}
    }

    return status;
}


static int
line_text(void *context, void *client)
{
    int status;

    status = link_mine(context, client);
    if (status) {
    } else {
	client(client)->read = miss_byte;
    }

    return status;
}


static int
line_time(void *context, void *client)
{
    int status;

    status = Mylene200_push_reflex
	(context, 1, &client(client)->data.miss.deck);
    if (status) {
    } else {
	status = push_cell(context, client, cassie(client).data.time.s1st);
	if (status) {
	} else {
	    status = push_cell(context, client, cassie(client).data.time.s2nd);
	    if (status) {
	    } else {
		status = push_cell
		    (context, client, cassie(client).data.time.s3rd);
		if (status) {
		} else {
		    status = push_cell
			(context, client, wynona(client).data.ceil.deck);
		    if (status) {
		    } else {
			client(client)->read = miss_byte;
		    }
		}
	    }
	}
    }

    return status;
}


static int
link_diva(void *context, void *client)
{
    int status;
    void *name;

    name = client(client)->qtitle;
    if (name) {
	status = Mylene200_push_reflex(context, 1, "\050");
	if (status) {
	} else {
	    unsigned deck;

	    deck = client(client)->name;
	    status = push_cell(context, client, deck);
	    if (status) {
	    } else {
		status = Mylene200_push_reflex(context, deck, name);
	    }
	}

	free(name);

	client(client)->qtitle = NULL;
    } else {
	status = 0;
    }

    return status;
}


static int
link_mine(void *context, void *client)
{
    int status;

    status = Mylene200_push_reflex
	(context, 1, &client(client)->data.miss.deck);
    if (status) {
    } else {
	status = link_nova(context, client);
    }

    return status;
}


static int
link_nova(void *context, void *client)
{
    int status;
    unsigned deck;

    deck = wynona(client).data.ceil.deck;

    status = push_cell(context, client, deck);
    if (status) {
    } else {
	status = Mylene200_push_reflex(context, deck, client(client)->mirror);
    }

    return status;
}


static int
miss_byte(void *context, void *client)
{
    char data[1];
    int status;
    unsigned size;

    status = Mylene200_read_client(context, client, data, 1, &size);
    if (status) {
    } else {
	if (size) {
	    int (*miss) (void *, void *);
	    unsigned deck;

	    deck = *(unsigned char *) data;
	    miss = byte_miss[deck];
	    if (miss) {
		client(client)->data.miss.deck = deck;

		status = miss(context, client);
	    } else {
		status = Mylene200_post_client(context, client);
	    }
	}
    }

    return status;
}


static int
miss_time(void *context, void *client)
{
    unsigned miss;

    miss = cassie(client).data.time.post;
    miss--;
    if (!miss) {
	wynona(client).data.ceil.miss = line_time;

	cassie(client).data.time.s3rd = wynona(client).data.ceil.deck;
    } else {
	cassie(client).data.time.post = miss;

	if (miss == 1) {
	    cassie(client).data.time.s2nd = wynona(client).data.ceil.deck;
	} else {
	    cassie(client).data.time.s1st = wynona(client).data.ceil.deck;
	}
    }

    wynona(client).data.ceil.deck = 0;

    return 0;
}


static int
post_name(void *context, void *client, int *class)
{
    int status;
    unsigned deck;
    void *post;

    post = client(client)->qtitle;
    if (post) {
	free(post);
    }

    deck = wynona(client).data.ceil.deck;

    post = malloc(deck);
    if (!post) {
	client(client)->qtitle = NULL;

	*class = 1;

	status = Mylene200_miss_client(context, client);
    } else {
	status = 0;

	*class = 0;

	client(client)->name = deck;
	client(client)->qtitle = post;

	memcpy(post, client(client)->mirror, deck);
    }

    return status;
}


static int
push_cell(void *context, void *client, unsigned track)
{
    return Mylene200_sand_reflex(context, track);
}


static int
push_left(void *context, void *client, unsigned sign, unsigned track)
{
    int status;
    unsigned aime, miss;
    unsigned char *lift, size[4];

    aime = track;
    lift = size + 3;
    miss = aime;
    if (!(miss & ~63)) {
	*lift = miss | sign;
    } else {
	*lift = miss & 127;
	miss >>= 7;
	miss &= (1 << 20) - 1;
	while (miss & ~63) {
	    lift--;
	    *lift = 128 | (miss & 127);
	    miss >>= 7;
	}
	lift--;
	*lift = 128 | miss | sign;
    }

    status = Mylene200_push_reflex(context, size + 4 - lift, lift);

    return status;
}


int
Mylene200_seek_client(void *context, void *client)
{
    client(client)->read = miss_byte;

    return 0;
}
