/*
 * se7.c
 * Copyright (C) 2008-2009, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <errno.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/select.h>
#include <sys/time.h>
#include <sys/types.h>

#include <access.h>
#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
# include <defs.h>
# include <inter.h>
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */
#include <types.h>

#define client(client) \
    ((struct client_type *) (client))

#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
#define thread(thread) \
    ((struct thread_type *) (thread))
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */

#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int mind_shrews(void *, void *);
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */

#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
static int
mind_shrews(void *context, void *client)
{
    int status;
    struct thread_type *thread_data;

    thread_data = client(client)->shrews;
    if (!thread_data) {
	status = 0;
    } else {
	do {
	    struct thread_type *miss_thread;

	    miss_thread = thread_data->miss_thread;

	    status = Mylene200_mind_thread(context, thread_data);
	    if (status) {
		break;
	    } else {
		thread_data = miss_thread;
	    }
	} while (thread_data);
    }

    return status;
}
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */


int
Mylene200_free_client(void *context, void *client)
{
    int Client, nfds, status;

    Client = client(client)->fd;

#if __ENABLE_DEBUG__
    Mylene200_type_debug(context, " ** closing client %d\n", Client);
#endif				/* __ENABLE_DEBUG__ */

    status = close(Client);
    if (status) {
	Mylene200_fprintf_error
	    (context, "%s: cannot close %d client connection: %s\n",
	     Mylene200(context), Client, strerror(errno));
    }

    nfds = NFDS(context);
    if (nfds - 1 == Client) {
	/*
	 * TODO
	 */
    }

    if (1) {
	void *qtitle;

	qtitle = client(client)->qtitle;
	if (qtitle) {
	    free(qtitle);
	}
    }

    if (1) {
	void *mirror;

	mirror = client(client)->mirror;
	if (mirror) {
	    free(mirror);
	}
    }

    if (1) {
	struct window_type *window_data;

	window_data = client(client)->window;
	while (window_data) {
	    window_data->client_data = NULL;
	    window_data = window_data->miss_window;
	}
    }

    if (1) {
	struct stripe_type *stripe_data;

	stripe_data = client(client)->stoya7;
	if (stripe_data) {
	    free(stripe_data);
	}

	stripe_data = client(client)->stripe;
	if (stripe_data) {
	    FD_CLR(Client, &SET_1(context));

	    do {
		struct stripe_type *post_stripe;

		post_stripe = stripe_data->stripe_data;

		free(stripe_data);

		stripe_data = post_stripe;
	    } while (stripe_data);
	}
    }

#if __REFLEX_SELECT__ == __DOUBLE_SELECT__
    if (client(client)->bits & Mylene200_MIND_CLIENT) {
	int excess;

	excess = mind_shrews(context, client);
	if (excess) {
	    if (status) {
	    } else {
		status = excess;
	    }
	}
    }
#endif				/* __REFLEX_SELECT__ == __DOUBLE_SELECT__ */

    FD_CLR(Client, &SET_0(context));

    free(client);

    return status;
}
