/*
 * signals.c
 * Copyright (C) 2003-2009, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <signal.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>

#include <inter.h>

typedef struct signal_type {
    char *name;
    int signal;
} signal_type;

static const char *fatal;
static const struct signal_type signals_0[] = {
/* *INDENT-OFF* */
    {	"SIGPIPE",	SIGPIPE		},
    {	NULL,		0		}
/* *INDENT-ON* */
}, signals_1[] = {
/* *INDENT-OFF* */
    {	"SIGHUP",	SIGHUP		},
    {	"SIGINT",	SIGINT		},
    {	"SIGQUIT",	SIGQUIT		},
    {	"SIGILL",	SIGILL		},
    {	"SIGABRT",	SIGABRT		},
    {	"SIGFPE",	SIGFPE		},
#if 0
    {	"SIGKILL",	SIGKILL		},
#endif				/* 0 */
    {	"SIGSEGV",	SIGSEGV		},
    {	"SIGALRM",	SIGALRM		},
    {	"SIGTERM",	SIGTERM		},
#if 1
    {	"SIGUSR1",	SIGUSR1		},
#endif				/* 0 */
    {	"SIGUSR2",	SIGUSR2		},
#if 0
    {	"SIGCHLD",	SIGCHLD		},
#endif				/* 0 */
#if 1
    {	"SIGCONT",	SIGCONT 	},
    {	"SIGSTOP",	SIGSTOP 	},
#endif				/* 0 */
    {	"SIGTSTP",	SIGTSTP		},
    {	"SIGTTIN",	SIGTTIN		},
    {	"SIGTTOU",	SIGTTOU		},
    {	NULL,		0		}
/* *INDENT-ON* */
}, signals_2[] = {
/* *INDENT-OFF* */
    {	"SIGKILL",	SIGKILL		},
    {	"SIGCHLD",	SIGCHLD		},
    {	NULL,		0		}
/* *INDENT-ON* */
};
static void (*after_function) (void), (*flush_function) (void);

static void
signal_handler(int signal)
{
#if 1
    char *name;
#endif				/* 0 */

    if (flush_function) {
	flush_function();
    }

#if 1
    fputs(fatal, stderr);
    fputs(": received signal ", stderr);
    if (Mylene200_try_signals(signal, &name)) {
	fprintf(stderr, "%d", signal);
    } else {
	fputs(name, stderr);
    }
    fputs(", exiting...\n", stderr);
#endif				/* 0 */

    if (after_function) {
	after_function();
    }

    exit(1);
}


int
Mylene200_air_signals(void)
{
    const struct signal_type *signal_data;

    signal(SIGCHLD, SIG_DFL);

    signal_data = signals_0;
    while (signal_data->name) {
	signal(signal_data->signal, SIG_DFL);
	signal_data++;
    }
    signal_data = signals_1;
    while (signal_data->name) {
	signal(signal_data->signal, SIG_DFL);
	signal_data++;
    }

    return 0;
}


int
Mylene200_all_signals(const char *error, void (*flush_up) (void),
		      void (*clean_up) (void))
{
    const struct signal_type *signal_data;

    fatal = error;

    flush_function = flush_up;
    after_function = clean_up;

    signal_data = signals_0;
    while (signal_data->name) {
	signal(signal_data->signal, SIG_IGN);
	signal_data++;
    }
    signal_data = signals_1;
    while (signal_data->name) {
	signal(signal_data->signal, signal_handler);
	signal_data++;
    }

    return 0;
}


int
Mylene200_try_signals(int signal, char **set)
{
    char *name;
    int status;
    const struct signal_type *signal_data, *sets[3];
    unsigned i;

    sets[0] = signals_0;
    sets[1] = signals_1;
    sets[2] = signals_2;

    for (i = 0; i < 3; i++) {
	signal_data = sets[i];
	name = signal_data->name;
	while (name) {
	    if (signal_data->signal == signal) {
		break;
	    } else {
		signal_data++;
		name = signal_data->name;
	    }
	}
	if (name) {
	    break;
	}
    }

    if (!name) {
	status = 1;
    } else {
	status = 0;
	*set = name;
    }

    return status;
}
