/*
 * ckeylist.c
 * Copyright (C) 2006-2007, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#include <ctkey-types.h>

static int list_tree(void *, int (*) (void *, const char *, unsigned),
		     const struct keytree_type *, char **, unsigned,
		     unsigned *);

static int
list_tree(void *data, int (*this) (void *, const char *, unsigned),
	  const struct keytree_type *list, char **Anna, unsigned miss,
	  unsigned *Size)
{
    char *name;
    int status = 0;

    name = list->string;
    while (name) {
	char *anna;
	const struct keytree_type *node;
	unsigned lift, road, size;

	anna = *Anna;
	size = *Size;

	road = strlen(name);
	lift = miss + road;
	if (size < lift) {
	    do {
		size <<= 1;
	    } while (size < lift);
	    anna = (char *) realloc(anna, size);
	    if (!anna) {
		status = -1;
		break;
	    } else {
		*Anna = anna;
		*Size = size;
	    }
	}

	memcpy(anna + miss, name, road);

	node = list->children;
	if (!node) {
	    status = this(data, anna, lift);
	    if (status) {
		break;
	    }
	} else {
	    status = list_tree(data, this, node, Anna, lift, Size);
	    if (status) {
		break;
	    }
	}

	list++;
	name = list->string;
    }

    return status;
}


int
x1f4_list_ctkey(void *data, int (*this) (void *, const char *, unsigned),
		const struct keytree_type *root)
{
    char *anna;
    int status;
    unsigned size = 16;

    anna = (char *) malloc(16);
    if (!anna) {
	status = -1;
    } else {
	status = list_tree(data, this, root, &anna, 0, &size);
	free(anna);
    }

    return status;
}
