/*
 * Mylene3e8.c
 * Copyright (C) 2002-2013, 2015, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if defined HAVE_FEATURES_H
# include <features.h>
#endif				/* HAVE_FEATURES_H */

#include <errno.h>
#if defined HAVE_LIBLEWD
# include <lewd.h>
#endif				/* HAVE_LIBLEWD */
#include <getopt.h>
#include <netdb.h>
#include <signal.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <arpa/inet.h>

#include <netinet/in.h>

#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/un.h>
#include <sys/utsname.h>

#if !defined HAVE_LIBLEWD
# include <akeylist.h>
# include <cardinal-w.h>
# include <cardinal-wx.h>
# include <cardinal-x.h>
# include <cardinal.h>
# include <ckeyhook.h>
# include <ckeylist.h>
# include <ckeytree.h>
# include <integral-x.h>
# include <ordinal.h>
# include <real.h>
# include <vkeyhook.h>
# include <vkeytree.h>
#endif				/* !HAVE_LIBLEWD */

/*
 * resume at 5c .. 5f, 7b .. 7d
 */
#define ABORT				0x23
#define ADD				0x3a
#define ADd				0x60
#define AdD				0x40
#define Add				0x3d
#define ALBUM				0x61
#define ALbUM				(ALBUM | JOIN << 8)
#define ASSERT				(DIVA | INSERT << 8)
#define ATTACH				0x21
#define AlBuM				(AlbuM | JOIN << 8)
#define AlbuM				0x41
#define Album				0x81
#define BACK				0x06
#define BACK1				0x0e
#define BEGIN				0x18
#define BegiN				0x19
#define CLASS_3				0x2e
#define CLEAR				0x15
#define CLeAR				(CLEAR | JOIN << 8)
#define CYCLE				0x03
#define CYCLE_On			0x0b
#define CYCLE_m				0x16
#define CYCLE_off			0x0a
#define CYCLE_p				0x17
#define CYCLE_tail			0x10
#define DELETE				0x29
#define DELETE_c			0x5e
#define DELETE_s			0x5f
#define DEQUE				0x24
#define DEQUE_all			0x26
#define DEQUE_c				0x25
#define DETACH				0x20
#define DIVA				0x28
#define EXIT				0x00
#define FLUSH				0x3f
#define FORMAT				0x0f
#define Flush				0x7f
#define HEAD				0x2c
#define HEAD_n				0x34
#define INSERT				0x37
#define INSERT_c			0x5d
#define INSERT_s			0x5c
#define INSTANT				0x7c
#define JOIN				0x07
#define LBASE				0x62
#define LBaSE				(LBASE | JOIN << 8)
#define LESS1				0x31
#define LESS2				0x32
#define LESS3				0x33
#define LEVEL_3				0x30
#define LOAD				0x14
#define LOSE				0x7e
#define LPATH				0x71
#define LPaTH				(LPATH | JOIN << 8)
#define LbAsE				(LbasE | JOIN << 8)
#define LbasE				0x42
#define Less1				0xb1
#define Less2				0xb2
#define Less3				0xb3
#define LpAtH				(LpatH | JOIN << 8)
#define LpatH				0x51
#define MAIL				0x80
#define MATCH				0x6d
#define MAtCH				(MATCH | JOIN << 8)
#define MOVE				0x39
#define MaTcH				(MatcH | JOIN << 8)
#define MatcH				0x4d
#define Match				0x8d
#define NOTES				0x6e
#define NOtES				(NOTES | JOIN << 8)
#define NoTeS				(NoteS | JOIN << 8)
#define NoteS				0x4e
#define Notes				0x8e
#define ORDER				ORDER_z
#define ORDER_n				0x11
#define ORDER_p				0x12
#define ORDER_z				0x13
#define PAUSE				SPACE_On
#define PICK				0x1a
#define PICK_m				0x1b
#define PICK_p				0x1c
#define PLAY				SPACE_off
#define POST				0x5b
#define PRESS				0x70
#define PReSS				(PRESS | JOIN << 8)
#define PrEsS				(PresS | JOIN << 8)
#define PresS				0x50
#define QUEUE				0x22
#define RAND				0x08
#define REMOVE				0x3b
#define RESTORE				0x2b
#define ReMOvE				0x3c
#define SCALE_3				0x36
#define SCRAP				0x7d
#define SEEK				0x1d
#define SEEK_m				0x1e
#define SEEK_p				0x1f
#define SHUFFLE				0x2a
#define SKIP				0x01
#define SKIP1				0x09
#define SPACE				0x02
#define SPACE_On			0x0d
#define SPACE_off			0x0c
#define START				0x04
#define SUSPEND				0x38
#define SuSpEnD				0x3e
#define StarT				0x05
#define TAIL				0x2d
#define TAIL_n				0x35
#define TEQUE				(DIVA | QUEUE << 8)
#define TITLE				0x74
#define TItLE				(TITLE | JOIN << 8)
#define TiTlE				(TitlE | JOIN << 8)
#define TitlE				0x54
#define Title				0x94
#define WIND				0x27
#define WIND1				0x2f

#define FRAME_A				0x41
#define FRAME_a				0x61
#define FRAME_c				0x63
#define FRAME_t				0x74
#define FRAME_y				0x79

#define FRAME_R				0

#define INDEX_C				0
#define INDEX_i				1

#define true(e)				1

#define SET_TIME(time, H, M, S, X) \
    {									      \
	((struct time_type *) (time))->h = (H);				      \
	((struct time_type *) (time))->m = (M);				      \
	((struct time_type *) (time))->s = (S);				      \
	((struct time_type *) (time))->x = (X);				      \
    }

typedef struct band_type {
    double delete;
    unsigned rank;
} band_type;

typedef struct c067_type {
    char *data, *link;
    int delete;
    const void *post;
} c067_type;

typedef struct cell_type {
    int delete;
} cell_type;

typedef struct e067_type {
    char *data;
    int delete;
} e067_type;

typedef struct e073_type {
    unsigned class, order;
} e073_type;

typedef struct e3e8_type {
    const char *self, *type;
    int connection, do_synchronize;
    void *data;
    const void *link;
} e3e8_type;

typedef struct flat_type {
    const char *album, *frame, *match, *notes, *sixth, *title;
    int delete;
} flat_type;

typedef struct link_type {
    int (*link) (void *), (*list) (void *, const char *, unsigned),
	(*type) (void *, int, char **);
    unsigned command;
} link_type;

typedef struct mail_type {
    char *text;
    unsigned major, minor;
} mail_type;

typedef struct miss_type {
    char *data;
} miss_type;

typedef struct mode_type {
    int append;
    void *file;
} mode_type;

typedef struct pair_type {
    char *major, *minor;
} pair_type;

typedef struct real_type {
    double data;
} real_type;

typedef struct s067_type {
    const void *post;
} s067_type;

typedef struct size_type {
    unsigned class;
} size_type;

typedef struct tail_type {
    int major, minor;
} tail_type;

typedef struct time_type {
    unsigned h, m, s, x;
} time_type;

typedef union data_type {
    struct band_type scale;
    struct c067_type order;
    struct cell_type inter;
    struct e067_type passe;
    struct e073_type slide;
    struct flat_type black;
    struct mail_type patch;
    struct miss_type press;
    struct pair_type class;
    struct real_type point;
    struct s067_type trans;
    struct size_type shift;
    struct tail_type white;
    struct time_type begin;
} data_type;

static int case_inet(void *, int, void *);
static int case_unix(void *, int, void *);
static int case_v1f4(void *, const void *);
static int case_v1f5(void *, const void *);
static int case_v1f6(void *, const void *);
static int case_v1f7(void *, const void *);
static int case_v1f8(void *, const void *);
static int case_v1f9(void *, const void *);
static int case_v1fa(void *, const void *);
static int case_x1f4(void *, int);
static int case_x3e8(void *, int, void *);
static int join_x1f4(void *);
static int link_a075(void *);
static int link_c042(void *);
static int link_c067(void *);
static int link_c124(void *);
static int link_c164(void *);
static int link_e004(void *);
static int link_e017(void *);
static int link_e035(void *);
static int link_e050(void *);
static int link_e047(void *);
static int link_e056(void *);
static int link_e057(void *);
static int link_e066(void *);
static int link_e067(void *);
static int link_e071(void *);
static int link_e074(void *);
static int link_e075(void *);
static int link_e140(void *);
static int link_e200(void *);
static int list_e002(void *, const char *, unsigned);
static int list_e003(void *, const char *, unsigned);
static int list_e004(void *, const char *, unsigned);
static int list_e023(void *, const char *, unsigned);
static int list_e031(void *, const char *, unsigned);
static int list_e035(void *, const char *, unsigned);
static int list_e042(void *, const char *, unsigned);
static int list_e044(void *, const char *, unsigned);
static int list_e051(void *, const char *, unsigned);
static int list_e056(void *, const char *, unsigned);
static int list_e057(void *, const char *, unsigned);
static int list_e066(void *, const char *, unsigned);
static int list_e071(void *, const char *, unsigned);
static int list_e072(void *, const char *, unsigned);
static int list_e073(void *, const char *, unsigned);
static int list_e200(void *, const char *, unsigned);
static int list_e224(void *, const char *, unsigned);
static int list_s042(void *, const char *, unsigned);
static int list_s067(void *, const char *, unsigned);
static int list_link(void *, const void *, const char *, unsigned);
static int list_miss(void *, const char *, unsigned);
static int list_mode(void *, const char *, unsigned);
static int miss_time(const char *, void *);
static int read_data(void *);
static int slip_data(void *, unsigned, const void *);
static int slip_text(void *, unsigned, const void *);
static int stat_read(void *, int);
static int type_a057(void *, int, char **);
static int type_c042(void *, int, char **);
static int type_c067(void *, int, char **);
static int type_e002(void *, int, char **);
static int type_e003(void *, int, char **);
static int type_e004(void *, int, char **);
static int type_e011(void *, int, char **);
static int type_e016(void *, int, char **);
static int type_e023(void *, int, char **);
static int type_e035(void *, int, char **);
static int type_e042(void *, int, char **);
static int type_e044(void *, int, char **);
static int type_e056(void *, int, char **);
static int type_e057(void *, int, char **);
static int type_e064(void *, int, char **);
static int type_e065(void *, int, char **);
static int type_e066(void *, int, char **);
static int type_e067(void *, int, char **);
static int type_e071(void *, int, char **);
static int type_e072(void *, int, char **);
static int type_e073(void *, int, char **);
static int type_e076(void *, int, char **);
static int type_e200(void *, int, char **);
static int type_e224(void *, int, char **);
static int type_s042(void *, int, char **);
static int type_s067(void *, int, char **);
static int type_s134(void *, int, char **);
static int type_s135(void *, int, char **);
static int type_v051(void *, int, char **);
static int type_v067(void *, int, char **);

static void SIGPIPE_handler(int);
static void detail_types(void);
static void stat_count(void *);
static void stat_value(const char *, unsigned, const char *);
static void usage(void);

static sig_atomic_t SIGPIPE_received;
static const struct link_type link_u025 = {
/* *INDENT-OFF* */
    link_c124,
    list_e224,
    type_e224,
    CLeAR
/* *INDENT-ON* */
}, link_u042 = {
/* *INDENT-OFF* */
    link_c042,
    NULL,
    NULL,
    0
/* *INDENT-ON* */
}, link_u067 = {
/* *INDENT-OFF* */
    link_c067,
    NULL,
    NULL,
    0
/* *INDENT-ON* */
}, link_u101 = {
/* *INDENT-OFF* */
    link_c124,
    list_e224,
    type_e224,
    AlBuM
/* *INDENT-ON* */
}, link_u102 = {
/* *INDENT-OFF* */
    link_c124,
    list_e224,
    type_e224,
    LbAsE
/* *INDENT-ON* */
}, link_u115 = {
/* *INDENT-OFF* */
    link_c124,
    list_e224,
    type_e224,
    MaTcH
/* *INDENT-ON* */
}, link_u116 = {
/* *INDENT-OFF* */
    link_c124,
    list_e224,
    type_e224,
    NoTeS
/* *INDENT-ON* */
}, link_u120 = {
/* *INDENT-OFF* */
    link_c124,
    list_e224,
    type_e224,
    PrEsS
/* *INDENT-ON* */
}, link_u121 = {
/* *INDENT-OFF* */
    link_c124,
    list_e224,
    type_e224,
    LpAtH
/* *INDENT-ON* */
}, link_u124 = {
/* *INDENT-OFF* */
    link_c124,
    list_e224,
    type_e224,
    TiTlE
/* *INDENT-ON* */
}, link_u141 = {
/* *INDENT-OFF* */
    link_c164,
    list_e042,
    type_e042,
    ALbUM
/* *INDENT-ON* */
}, link_u142 = {
/* *INDENT-OFF* */
    link_c164,
    list_e042,
    type_e042,
    LBaSE
/* *INDENT-ON* */
}, link_u155 = {
/* *INDENT-OFF* */
    link_c164,
    list_e042,
    type_e042,
    MAtCH
/* *INDENT-ON* */
}, link_u156 = {
/* *INDENT-OFF* */
    link_c164,
    list_e042,
    type_e042,
    NOtES
/* *INDENT-ON* */
}, link_u160 = {
/* *INDENT-OFF* */
    link_c164,
    list_e042,
    type_e042,
    PReSS
/* *INDENT-ON* */
}, link_u161 = {
/* *INDENT-OFF* */
    link_c164,
    list_e042,
    type_e042,
    LPaTH
/* *INDENT-ON* */
}, link_u164 = {
/* *INDENT-OFF* */
    link_c164,
    list_e042,
    type_e042,
    TItLE
/* *INDENT-ON* */
}, link_v000 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    EXIT
/* *INDENT-ON* */
}, link_v001 = {
/* *INDENT-OFF* */
    link_e047,
    NULL,
    NULL,
    SKIP
/* *INDENT-ON* */
}, link_v002 = {
/* *INDENT-OFF* */
    link_e047,
    list_e002,
    type_e002,
    SPACE
/* *INDENT-ON* */
}, link_v003 = {
/* *INDENT-OFF* */
    link_e047,
    list_e003,
    type_e003,
    CYCLE
/* *INDENT-ON* */
}, link_v004 = {
/* *INDENT-OFF* */
    link_e004,
    list_e004,
    type_e004,
    START
/* *INDENT-ON* */
}, link_v005 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    StarT
/* *INDENT-ON* */
}, link_v006 = {
/* *INDENT-OFF* */
    link_e047,
    NULL,
    NULL,
    BACK
/* *INDENT-ON* */
}, link_v007 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    JOIN
/* *INDENT-ON* */
}, link_v010 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    RAND
/* *INDENT-ON* */
}, link_v011 = {
/* *INDENT-OFF* */
    link_e057,
    list_e057,
    type_e011,
    SKIP1
/* *INDENT-ON* */
}, link_v012 = {
/* *INDENT-OFF* */
    link_e047,
    NULL,
    NULL,
    CYCLE_off
/* *INDENT-ON* */
}, link_v013 = {
/* *INDENT-OFF* */
    link_e047,
    NULL,
    NULL,
    CYCLE_On
/* *INDENT-ON* */
}, link_v014 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    SPACE_off
/* *INDENT-ON* */
}, link_v015 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    SPACE_On
/* *INDENT-ON* */
}, link_v016 = {
/* *INDENT-OFF* */
    link_e057,
    list_e057,
    type_e016,
    BACK1
/* *INDENT-ON* */
}, link_v017 = {
/* *INDENT-OFF* */
    link_e017,
    list_e042,
    type_e042,
    FORMAT
/* *INDENT-ON* */
}, link_v020 = {
/* *INDENT-OFF* */
    link_e004,
    NULL,
    NULL,
    CYCLE_tail
/* *INDENT-ON* */
}, link_v021 = {
/* *INDENT-OFF* */
    link_e047,
    list_e023,
    type_e023,
    ORDER_n
/* *INDENT-ON* */
}, link_v022 = {
/* *INDENT-OFF* */
    link_e047,
    list_e023,
    type_e023,
    ORDER_p
/* *INDENT-ON* */
}, link_v023 = {
/* *INDENT-OFF* */
    link_e047,
    list_e023,
    type_e023,
    ORDER
/* *INDENT-ON* */
}, link_v024 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    LOAD
/* *INDENT-ON* */
}, link_v025 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    CLEAR
/* *INDENT-ON* */
}, link_v026 = {
/* *INDENT-OFF* */
    link_e057,
    list_e031,
    type_a057,
    CYCLE_m
/* *INDENT-ON* */
}, link_v027 = {
/* *INDENT-OFF* */
    link_e057,
    list_e031,
    type_a057,
    CYCLE_p
/* *INDENT-ON* */
}, link_v030 = {
/* *INDENT-OFF* */
    link_e035,
    list_e035,
    type_e035,
    BEGIN
/* *INDENT-ON* */
}, link_v031 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    BegiN
/* *INDENT-ON* */
}, link_v032 = {
/* *INDENT-OFF* */
    link_e057,
    list_e031,
    type_a057,
    PICK
/* *INDENT-ON* */
}, link_v033 = {
/* *INDENT-OFF* */
    link_e057,
    list_e031,
    type_a057,
    PICK_m
/* *INDENT-ON* */
}, link_v034 = {
/* *INDENT-OFF* */
    link_e057,
    list_e031,
    type_a057,
    PICK_p
/* *INDENT-ON* */
}, link_v035 = {
/* *INDENT-OFF* */
    link_e035,
    list_e035,
    type_e035,
    SEEK
/* *INDENT-ON* */
}, link_v036 = {
/* *INDENT-OFF* */
    link_e035,
    list_e035,
    type_e035,
    SEEK_m
/* *INDENT-ON* */
}, link_v037 = {
/* *INDENT-OFF* */
    link_e035,
    list_e035,
    type_e035,
    SEEK_p
/* *INDENT-ON* */
}, link_v040 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    DETACH
/* *INDENT-ON* */
}, link_v041 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    ATTACH
/* *INDENT-ON* */
}, link_v042 = {
/* *INDENT-OFF* */
    link_e050,
    list_s042,
    type_s042,
    QUEUE
/* *INDENT-ON* */
}, link_v043 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    ABORT
/* *INDENT-ON* */
}, link_v044 = {
/* *INDENT-OFF* */
    link_e047,
    list_e044,
    type_e044,
    DEQUE
/* *INDENT-ON* */
}, link_v045 = {
/* *INDENT-OFF* */
    link_e004,
    NULL,
    NULL,
    DEQUE_c
/* *INDENT-ON* */
}, link_v046 = {
/* *INDENT-OFF* */
    link_e047,
    NULL,
    NULL,
    DEQUE_all
/* *INDENT-ON* */
}, link_v047 = {
/* *INDENT-OFF* */
    link_e047,
    NULL,
    NULL,
    WIND
/* *INDENT-ON* */
}, link_v051 = {
/* *INDENT-OFF* */
    link_e057,
    list_e051,
    type_v051,
    DELETE
/* *INDENT-ON* */
}, link_v052 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    SHUFFLE
/* *INDENT-ON* */
}, link_v053 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    RESTORE
/* *INDENT-ON* */
}, link_v054 = {
/* *INDENT-OFF* */
    link_e047,
    NULL,
    NULL,
    HEAD
/* *INDENT-ON* */
}, link_v055 = {
/* *INDENT-OFF* */
    link_e047,
    NULL,
    NULL,
    TAIL
/* *INDENT-ON* */
}, link_v056 = {
/* *INDENT-OFF* */
    link_e056,
    list_e056,
    type_e056,
    CLASS_3
/* *INDENT-ON* */
}, link_v057 = {
/* *INDENT-OFF* */
    link_e057,
    list_e057,
    type_e057,
    WIND1
/* *INDENT-ON* */
}, link_v060 = {
/* *INDENT-OFF* */
    link_e056,
    list_e056,
    type_e056,
    LEVEL_3
/* *INDENT-ON* */
}, link_v061 = {
/* *INDENT-OFF* */
    link_e050,
    list_e042,
    type_e042,
    LESS1
/* *INDENT-ON* */
}, link_v062 = {
/* *INDENT-OFF* */
    link_e050,
    list_e042,
    type_e042,
    LESS2
/* *INDENT-ON* */
}, link_v063 = {
/* *INDENT-OFF* */
    link_e050,
    list_e042,
    type_e042,
    LESS3
/* *INDENT-ON* */
}, link_v064 = {
/* *INDENT-OFF* */
    link_e057,
    list_e057,
    type_e064,
    HEAD_n
/* *INDENT-ON* */
}, link_v065 = {
/* *INDENT-OFF* */
    link_e057,
    list_e057,
    type_e065,
    TAIL_n
/* *INDENT-ON* */
}, link_v066 = {
/* *INDENT-OFF* */
    link_e066,
    list_e066,
    type_e066,
    SCALE_3
/* *INDENT-ON* */
}, link_v067 = {
/* *INDENT-OFF* */
    link_e067,
    list_s067,
    type_v067,
    INSERT
/* *INDENT-ON* */
}, link_v070 = {
/* *INDENT-OFF* */
    link_e047,
    NULL,
    NULL,
    SUSPEND
/* *INDENT-ON* */
}, link_v071 = {
/* *INDENT-OFF* */
    link_e071,
    list_e071,
    type_e071,
    MOVE
/* *INDENT-ON* */
}, link_v072 = {
/* *INDENT-OFF* */
    link_e050,
    list_e072,
    type_e072,
    ADD
/* *INDENT-ON* */
}, link_v073 = {
/* *INDENT-OFF* */
    link_e004,
    list_e073,
    type_e073,
    REMOVE
/* *INDENT-ON* */
}, link_v074 = {
/* *INDENT-OFF* */
    link_e074,
    NULL,
    NULL,
    ReMOvE
/* *INDENT-ON* */
}, link_v075 = {
/* *INDENT-OFF* */
    link_e075,
    NULL,
    NULL,
    Add
/* *INDENT-ON* */
}, link_v076 = {
/* *INDENT-OFF* */
    link_e057,
    list_e057,
    type_e076,
    SuSpEnD
/* *INDENT-ON* */
}, link_v077 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    FLUSH
/* *INDENT-ON* */
}, link_v100 = {
/* *INDENT-OFF* */
    link_e067,
    NULL,
    NULL,
    AdD
/* *INDENT-ON* */
}, link_v101 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    AlbuM
/* *INDENT-ON* */
}, link_v102 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    LbasE
/* *INDENT-ON* */
}, link_v115 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    MatcH
/* *INDENT-ON* */
}, link_v116 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    NoteS
/* *INDENT-ON* */
}, link_v120 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    PresS
/* *INDENT-ON* */
}, link_v121 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    LpatH
/* *INDENT-ON* */
}, link_v124 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    TitlE
/* *INDENT-ON* */
}, link_v134 = {
/* *INDENT-OFF* */
    link_e067,
    NULL,
    type_s134,
    INSERT_s
/* *INDENT-ON* */
}, link_v135 = {
/* *INDENT-OFF* */
    link_e067,
    NULL,
    type_s135,
    INSERT_c
/* *INDENT-ON* */
}, link_v136 = {
/* *INDENT-OFF* */
    link_e057,
    NULL,
    type_a057,
    DELETE_c
/* *INDENT-ON* */
}, link_v137 = {
/* *INDENT-OFF* */
    link_e057,
    NULL,
    type_a057,
    DELETE_s
/* *INDENT-ON* */
}, link_v140 = {
/* *INDENT-OFF* */
    link_e140,
    NULL,
    NULL,
    ADd
/* *INDENT-ON* */
}, link_v141 = {
/* *INDENT-OFF* */
    link_e050,
    list_e042,
    type_e042,
    ALBUM
/* *INDENT-ON* */
}, link_v142 = {
/* *INDENT-OFF* */
    link_e050,
    list_e042,
    type_e042,
    LBASE
/* *INDENT-ON* */
}, link_v155 = {
/* *INDENT-OFF* */
    link_e050,
    list_e042,
    type_e042,
    MATCH
/* *INDENT-ON* */
}, link_v156 = {
/* *INDENT-OFF* */
    link_e050,
    list_e042,
    type_e042,
    NOTES
/* *INDENT-ON* */
}, link_v160 = {
/* *INDENT-OFF* */
    link_e050,
    list_e042,
    type_e042,
    PRESS
/* *INDENT-ON* */
}, link_v161 = {
/* *INDENT-OFF* */
    link_e050,
    list_e042,
    type_e042,
    LPATH
/* *INDENT-ON* */
}, link_v164 = {
/* *INDENT-OFF* */
    link_e050,
    list_e042,
    type_e042,
    TITLE
/* *INDENT-ON* */
}, link_v174 = {
/* *INDENT-OFF* */
    link_e200,
    list_e200,
    type_e200,
    INSTANT
/* *INDENT-ON* */
}, link_v175 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    SCRAP
/* *INDENT-ON* */
}, link_v200 = {
/* *INDENT-OFF* */
    link_e200,
    list_e200,
    type_e200,
    MAIL
/* *INDENT-ON* */
}, link_v201 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    Album
/* *INDENT-ON* */
}, link_v215 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    Match
/* *INDENT-ON* */
}, link_v216 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    Notes
/* *INDENT-ON* */
}, link_v224 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    Title
/* *INDENT-ON* */
}, link_v261 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    Less1
/* *INDENT-ON* */
}, link_v262 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    Less2
/* *INDENT-ON* */
}, link_v263 = {
/* *INDENT-OFF* */
    link_e047,
    list_e224,
    type_e224,
    Less3
/* *INDENT-ON* */
};
static const struct x1f4_akeytree_type field_1[] = {
/* *INDENT-OFF* */
    {	"lbum",						NULL		},
    {	"rtist",					NULL		},
    {	NULL,						NULL		}
/* *INDENT-ON* */
}, field_0[] = {
/* *INDENT-OFF* */
    {	"a",						field_1		},
    {	"comment",					NULL		},
    {	"match",					NULL		},
    {	"title",					NULL		},
    {	"year",						NULL		},
    {	NULL,						NULL		}
/* *INDENT-ON* */
}, field_root[] = {
/* *INDENT-OFF* */
    {	"",						field_0		},
    {	NULL,						NULL		}
/* *INDENT-ON* */
};
static const struct x1f4_ckeytree_type frame_2[] = {
/* *INDENT-OFF* */
    {	"omment",	{	FRAME_c		},	NULL		},
    {	"urrent",	{	FRAME_R		},	NULL		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, frame_1[] = {
/* *INDENT-OFF* */
    {	"lbum",		{	FRAME_A		},	NULL		},
    {	"rtist",	{	FRAME_a		},	NULL		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, frame_0[] = {
/* *INDENT-OFF* */
    {	"a",		{	0		},	frame_1		},
    {	"c",		{	0		},	frame_2		},
    {	"match",	{	FRAME_a		},	NULL		},
    {	"title",	{	FRAME_t		},	NULL		},
    {	"year",		{	FRAME_y		},	NULL		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, frame_root[] = {
/* *INDENT-OFF* */
    {	"",		{	0		},	frame_0		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, index_0[] = {
/* *INDENT-OFF* */
    {	"current",	{	INDEX_C		},	NULL		},
    {	"initial",	{	INDEX_i		},	NULL		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, index_root[] = {
/* *INDENT-OFF* */
    {	"",		{	0		},	index_0		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, order_0[] = {
/* *INDENT-OFF* */
    {	"back",		{	ORDER_p		},	NULL		},
    {	"last",		{	ORDER_p		},	NULL		},
    {	"next",		{	ORDER_n		},	NULL		},
    {	"previous",	{	ORDER_p		},	NULL		},
    {	"random",	{	ORDER_z		},	NULL		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, order_root[] = {
/* *INDENT-OFF* */
    {	"",		{	0		},	order_0		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, scope_root[] = {
/* *INDENT-OFF* */
    {	"play",		{	0		},	NULL		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, swing_0[] = {
/* *INDENT-OFF* */
    {	"class",	{	1		},	NULL		},
    {	"start",	{	0		},	NULL		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, swing_root[] = {
/* *INDENT-OFF* */
    {	"",		{	0		},	swing_0		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, switch_0[] = {
/* *INDENT-OFF* */
    {	"ff",		{	0		},	NULL		},
    {	"n",		{	1		},	NULL		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, switch_root[] = {
/* *INDENT-OFF* */
    {	"o",		{	0		},	switch_0	},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
};
static const struct x1f4_vkeytree_type link_1_a[] = {
/* *INDENT-OFF* */
    {	"ert",		{	&link_v067	},	NULL		},
    {	"tant",		{	&link_v174	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_1_9[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v102	},	NULL		},
    {	"/",		{	&link_u102	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_1_8[] = {
/* *INDENT-OFF* */
    {	"move",		{	&link_v073	},	NULL		},
    {	"store",	{	&link_v053	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_1_7[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v115	},	NULL		},
    {	"/",		{	&link_u115	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_1_6[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v155	},	NULL		},
    {	"/",		{	&link_u155	},	NULL		},
    {	"@",		{	&link_v215	},	NULL		},
    {	"^",		{	NULL		},	link_1_7	},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_1_5[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v101	},	NULL		},
    {	"/",		{	&link_u101	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_1_4[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v141	},	NULL		},
    {	"/",		{	&link_u141	},	NULL		},
    {	"@",		{	&link_v201	},	NULL		},
    {	"^",		{	NULL		},	link_1_5	},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_1_3[] = {
/* *INDENT-OFF* */
    {	"il",		{	&link_v200	},	NULL		},
    {	"tch",		{	NULL		},	link_1_6	},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_1_2[] = {
/* *INDENT-OFF* */
    {	"a",		{	NULL		},	link_1_3	},
    {	"ove",		{	&link_v071	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_1_1[] = {
/* *INDENT-OFF* */
    {	"bort",		{	&link_v043	},	NULL		},
    {	"dd",		{	&link_v072	},	NULL		},
    {	"lbum",		{	NULL		},	link_1_4	},
    {	"rtist",	{	NULL		},	link_1_6	},
    {	"ttach",	{	&link_v041	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_1_0[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v016	},	NULL		},
    {	"*",		{	&link_v065	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_z[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v142	},	NULL		},
    {	"/",		{	&link_u142	},	NULL		},
    {	"^",		{	NULL		},	link_1_9	},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_y[] = {
/* *INDENT-OFF* */
    {	"ck",		{	NULL		},	link_1_0	},
    {	"sename",	{	NULL		},	link_0_z	},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_x[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v030	},	NULL		},
    {	"^",		{	&link_v031	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_w[] = {
/* *INDENT-OFF* */
    {	"a",		{	NULL		},	link_0_y	},
    {	"egin",		{	NULL		},	link_0_x	},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_v[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v116	},	NULL		},
    {	"/",		{	&link_u116	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_u[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v156	},	NULL		},
    {	"/",		{	&link_u156	},	NULL		},
    {	"@",		{	&link_v216	},	NULL		},
    {	"^",		{	NULL		},	link_0_v	},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_t[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v003	},	NULL		},
    {	"+",		{	&link_v027	},	NULL		},
    {	"-",		{	&link_v026	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_s[] = {
/* *INDENT-OFF* */
    {	"omment",	{	NULL		},	link_0_u	},
    {	"ycle",		{	NULL		},	link_0_t	},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_r[] = {
/* *INDENT-OFF* */
    {	"lete",		{	&link_v051	},	NULL		},
    {	"que",		{	&link_v044	},	NULL		},
    {	"tach",		{	&link_v040	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_q[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v120	},	NULL		},
    {	"/",		{	&link_u120	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_p[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v160	},	NULL		},
    {	"/",		{	&link_u160	},	NULL		},
    {	"^",		{	NULL		},	link_0_q	},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_o[] = {
/* *INDENT-OFF* */
    {	"ilter",	{	NULL		},	link_0_p	},
    {	"lush",		{	&link_v077	},	NULL		},
    {	"ormat",	{	&link_v017	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_n[] = {
/* *INDENT-OFF* */
    {	"amplification",
			{	&link_v066	},	NULL		},
    {	"frequency",	{	&link_v056	},	NULL		},
    {	"level",	{	&link_v060	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_m[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v023	},	NULL		},
    {	"^",		{	&link_v024	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_l[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v121	},	NULL		},
    {	"/",		{	&link_u121	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_k[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v161	},	NULL		},
    {	"/",		{	&link_u161	},	NULL		},
    {	"^",		{	NULL		},	link_0_l	},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_j[] = {
/* *INDENT-OFF* */
    {	"th",		{	NULL		},	link_0_k	},
    {	"use",		{	&link_v015	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_i[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v032	},	NULL		},
    {	"+",		{	&link_v034	},	NULL		},
    {	"-",		{	&link_v033	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_h[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v061	},	NULL		},
    {	"^",		{	&link_v261	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_g[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v062	},	NULL		},
    {	"^",		{	&link_v262	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_f[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v063	},	NULL		},
    {	"^",		{	&link_v263	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_e[] = {
/* *INDENT-OFF* */
    {	"1",		{	NULL		},	link_0_h	},
    {	"2",		{	NULL		},	link_0_g	},
    {	"3",		{	NULL		},	link_0_f	},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_d[] = {
/* *INDENT-OFF* */
    {	"a",		{	NULL		},	link_0_j	},
    {	"ick",		{	NULL		},	link_0_i	},
    {	"lay",		{	&link_v014	},	NULL		},
    {	"rogress",	{	NULL		},	link_0_e	},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_c[] = {
/* *INDENT-OFF* */
    {	"eue",		{	&link_v042	},	NULL		},
    {	"it",		{	&link_v000	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_b[] = {
/* *INDENT-OFF* */
    {	"and",		{	&link_v010	},	NULL		},
    {	"e",		{	NULL		},	link_1_8	},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_a[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v035	},	NULL		},
    {	"+",		{	&link_v037	},	NULL		},
    {	"-",		{	&link_v036	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_9[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v025	},	NULL		},
    {	"/",		{	&link_u025	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_8[] = {
/* *INDENT-OFF* */
    {	"ek",		{	NULL		},	link_0_a	},
    {	"lect^",	{	NULL		},	link_0_9	},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_7[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v011	},	NULL		},
    {	"*",		{	&link_v064	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_6[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v004	},	NULL		},
    {	"^",		{	&link_v005	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_5[] = {
/* *INDENT-OFF* */
    {	"crap",		{	&link_v175	},	NULL		},
    {	"e",		{	NULL		},	link_0_8	},
    {	"huffle",	{	&link_v052	},	NULL		},
    {	"kip",		{	NULL		},	link_0_7	},
    {	"pace",		{	&link_v002	},	NULL		},
    {	"tart",		{	NULL		},	link_0_6	},
    {	"uspend",	{	&link_v076	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_4[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v124	},	NULL		},
    {	"/",		{	&link_u124	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_3[] = {
/* *INDENT-OFF* */
    {	"",		{	&link_v164	},	NULL		},
    {	"/",		{	&link_u164	},	NULL		},
    {	"@",		{	&link_v224	},	NULL		},
    {	"^",		{	NULL		},	link_0_4	},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_0[] = {
/* *INDENT-OFF* */
    {	"a",		{	NULL		},	link_1_1	},
    {	"b",		{	NULL		},	link_0_w	},
    {	"c",		{	NULL		},	link_0_s	},
    {	"de",		{	NULL		},	link_0_r	},
    {	"exit",		{	&link_v000	},	NULL		},
    {	"f",		{	NULL		},	link_0_o	},
    {	"ins",		{	NULL		},	link_1_a	},
    {	"join",		{	&link_v007	},	NULL		},
    {	"l3",		{	NULL		},	link_0_n	},
    {	"m",		{	NULL		},	link_1_2	},
    {	"order",	{	NULL		},	link_0_m	},
    {	"p",		{	NULL		},	link_0_d	},
    {	"qu",		{	NULL		},	link_0_c	},
    {	"r",		{	NULL		},	link_0_b	},
    {	"s",		{	NULL		},	link_0_5	},
    {	"title",	{	NULL		},	link_0_3	},
    {	"wind",		{	&link_v057	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_root[] = {
/* *INDENT-OFF* */
    {	"",		{	NULL		},	link_0_0	},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
};

static int
case_inet(void *context, int connection, void *argv_1_)
{
    int status = 1;
    struct hostent *hostinfo;
    struct sockaddr_in address;
    struct utsname info;

    address.sin_family = AF_INET;
    address.sin_port = htons(*(unsigned *) argv_1_);
    uname(&info);
    if (!(hostinfo = gethostbyname(info.nodename))) {
	fprintf(stderr, "%s: cannot get host name: %s\n",
		((struct e3e8_type *) context)->self, hstrerror(h_errno));
    } else {
	address.sin_addr = *(struct in_addr *) hostinfo->h_addr;
	status = connect
	    (connection, (struct sockaddr *) &address, sizeof(address));
	if (status) {
	    fprintf(stderr, "%s: cannot connect %u: %s\n",
		    ((struct e3e8_type *) context)->self,
		    *(unsigned *) argv_1_, strerror(errno));
	} else {
	    status = case_x1f4(context, connection);
	}
    }

    return status;
}


static int
case_unix(void *context, int connection, void *argv_1_)
{
    int status = 1;
    struct sockaddr_un address;

    memset(&address, 0, sizeof(struct sockaddr_un));
    address.sun_family = AF_UNIX;
    strncpy(address.sun_path, argv_1_, sizeof(address.sun_path) - 1);
    if (1) {
	status = connect
	    (connection, (struct sockaddr *) &address, sizeof(address));
	if (status) {
	    fprintf(stderr, "%s: cannot connect `%s': %s\n",
		    ((struct e3e8_type *) context)->self, (char *) argv_1_,
		    strerror(errno));
	} else {
	    status = case_x1f4(context, connection);
	}
    }

    return status;
}


static int
case_v1f4(void *context, const void *argv_3_)
{
    unsigned char c;

    c = ((struct link_type *) argv_3_)->command;

    return slip_data(context, 1, &c);
}


static int
case_v1f5(void *context, const void *argv_3_)
{
    int aime, sign = 0;
    unsigned miss;
    unsigned char *lift, size[4];

    aime = *(int *) argv_3_;
    if (aime < 0) {
	sign = 64;
	aime = -aime;
    }
    lift = size + 3;
    miss = aime;
    if (!(miss & ~63)) {
	*lift = miss | sign;
    } else {
	*lift = miss & 127;
	miss >>= 7;
	miss &= (1 << 20) - 1;
	while (miss & ~63) {
	    lift--;
	    *lift = 128 | (miss & 127);
	    miss >>= 7;
	}
	lift--;
	*lift = 128 | miss | sign;
    }

    return slip_data(context, size + 4 - lift, lift);
}


static int
case_v1f6(void *context, const void *argv_3_)
{
    int status;
    unsigned aime, miss;
    unsigned char *lift, size[4];

    aime = strlen(argv_3_);
    lift = size + 3;
    miss = aime;
    if (!(miss & ~127)) {
	*lift = miss;
    } else {
	*lift = miss & 127;
	lift--;
	miss >>= 7;
	*lift = 128 | (miss & 127);
	miss >>= 7;
	while (miss) {
	    lift--;
	    *lift = 128 | (miss & 127);
	    miss >>= 7;
	}
    }

    status = slip_data(context, size + 4 - lift, lift);
    if (!status) {
	status = slip_data(context, aime, argv_3_);
    }

    return status;
}


static int
case_v1f7(void *context, const void *argv_3_)
{
    unsigned aime, miss;
    unsigned char *lift, size[4];

    aime = *(unsigned *) argv_3_;
    lift = size + 3;
    miss = aime;
    if (!(miss & ~127)) {
	*lift = miss;
    } else {
	*lift = miss & 127;
	lift--;
	miss >>= 7;
	*lift = 128 | (miss & 127);
	miss >>= 7;
	miss &= (1 << 14) - 1;
	while (miss) {
	    lift--;
	    *lift = 128 | (miss & 127);
	    miss >>= 7;
	}
    }

    return slip_data(context, size + 4 - lift, lift);
}


static int
case_v1f8(void *context, const void *argv_3_)
{
    const struct time_type *time_data;
    unsigned i = 4, miss, time[4], *unit;
    unsigned char *lift, size[12];

    time_data = argv_3_;
    time[0] = time_data->x;
    time[1] = time_data->s;
    time[2] = time_data->m;
    time[3] = time_data->h;
    unit = time;
    lift = size + 12;
    for (; i; i--) {
	unsigned aime;

	aime = *unit++;
	miss = aime;
	if (!(miss & ~127)) {
	    lift--;
	    *lift = miss;
	} else {
	    lift--;
	    *lift = miss & 127;
	    miss >>= 7;
	    lift--;
	    *lift = 128 | (miss & 127);
	    miss >>= 7;
	    miss &= (1 << 14) - 1;
	    while (miss) {
		lift--;
		*lift = 128 | (miss & 127);
		miss >>= 7;
	    }
	}
    }

    return slip_data(context, size + 12 - lift, lift);
}


static int
case_v1f9(void *context, const void *argv_3_)
{
    double real;
    int sign;
    unsigned data;

    real = *(const double *) argv_3_;
    if (real < 0) {
	real = -real;
	sign = 1;
    } else {
	sign = 0;
    }
    if (real < 32768) {
    } else {
	real = 32768;
    }
    data = real * 4096;
    if (data < 134217727) {
    } else {
	data = 134217727;
    }
    data += .5;
    if (sign) {
	sign = -data;
    } else {
	sign = data;
    }

    return case_v1f5(context, &sign);
}


static int
case_v1fa(void *context, const void *argv_3_)
{
    unsigned char c;

    c = *(unsigned *) argv_3_;

    return slip_data(context, 1, &c);
}


static int
case_x1f4(void *context, int connection)
{
    int status;

    status = slip_data(context, 16, "slippingaway\0\0\0\0");
    if (!status) {
	status =
	    ((struct link_type *)
	     ((struct e3e8_type *) context)->link)->link(context);
	if (!status) {
	    if (((struct e3e8_type *) context)->do_synchronize) {
		status = join_x1f4(context);
	    }
	}
    }

    return status;
}


static int
case_x3e8(void *context, int domain, void *argv_1_)
{
    int connection, status;

    SIGPIPE_received = 0;

    signal(SIGPIPE, SIGPIPE_handler);

    connection = socket(domain, SOCK_STREAM, 0);
    if (connection == -1) {
	status = 1;
	fprintf(stderr, "%s: cannot socket: %s\n",
		((struct e3e8_type *) context)->self, strerror(errno));
    } else {
	((struct e3e8_type *) context)->connection = connection;

	if (domain == AF_UNIX) {
	    status = case_unix(context, connection, argv_1_);
	} else {
	    status = case_inet(context, connection, argv_1_);
	}

	close(connection);
    }

    signal(SIGPIPE, SIG_DFL);

    return status;
}


static int
join_x1f4(void *context)
{
    int status;

    status = slip_data(context, 1, "\177");
    if (!status) {
	unsigned char c;

	status = read(((struct e3e8_type *) context)->connection, &c, 1);
	if (status != 1) {
	    status = stat_read(context, status);
	} else {
	    if (c == 127) {
		status = 0;
	    } else {
		fprintf(stderr, "%s: bad Mylene1f4 transmission received whi"
			"le waiting for synchonization reply\n",
			((struct e3e8_type *) context)->self);
	    }
	}
    }

    return status;
}


static int
link_a075(void *context)
{
    int status;

    if (1) {
	const char *album, *frame, *match, *notes, *sixth, *title;
	union data_type *class;
	unsigned a, f, field = 0, m, n, s, t;

	class = ((struct e3e8_type *) context)->data;

	album = class->black.album;
	frame = class->black.frame;
	match = class->black.match;
	notes = class->black.notes;
	sixth = class->black.sixth;
	title = class->black.title;

	if (!album) {
	    a = 0;
	} else {
	    a = strlen(album);
	    if (a) {
		a++;
		field |= 1 << 2;
	    }
	}

	if (!frame) {
	    f = 0;
	} else {
	    f = strlen(frame);
	    if (f) {
		f++;
		field |= 1 << 1;
	    }
	}

	if (!match) {
	    m = 0;
	} else {
	    m = strlen(match);
	    if (m) {
		m++;
		field |= 1 << 0;
	    }
	}

	if (!notes) {
	    n = 0;
	} else {
	    n = strlen(notes);
	    if (n) {
		n++;
		field |= 1 << 4;
	    }
	}

	if (!title) {
	    t = 0;
	} else {
	    t = strlen(title);
	    if (t) {
		t++;
		field |= 1 << 3;
	    }
	}

	{
	    s = strlen(sixth);
	}

	status = case_v1fa(context, &field);
	if (!status) {
	    field = a + f + m + n + t + s;

	    status = case_v1f7(context, &field);
	    if (!status) {
		status = slip_data(context, s, sixth);
		if (!status) {
		    do {
			if (n) {
			    status = slip_text(context, n, notes);
			    if (status) {
				break;
			    }
			}

			if (t) {
			    status = slip_text(context, t, title);
			    if (status) {
				break;
			    }
			}

			if (a) {
			    status = slip_text(context, a, album);
			    if (status) {
				break;
			    }
			}

			if (f) {
			    status = slip_text(context, f, frame);
			    if (status) {
				break;
			    }
			}

			if (m) {
			    status = slip_text(context, m, match);
			    if (status) {
				break;
			    }
			}
		    } while (0);
		}
	    }
	}
    }

    return status;
}


static int
link_c042(void *context)
{
    int status;
    struct link_type link_v050;
    const struct link_type *link_data;
    union data_type *class, data;

    link_data = ((struct e3e8_type *) context)->link;

    class = ((struct e3e8_type *) context)->data;

    ((struct e3e8_type *) context)->data = &data;

    ((struct e3e8_type *) context)->link = &link_v050;

    link_v050.command = DIVA;

    data.press.data = class->class.major;

    status = link_e050(context);
    if (!status) {
	((struct e3e8_type *) context)->link = &link_v042;

	data.press.data = class->class.minor;

	status = link_e050(context);
	if (!status) {
	}
    }

    ((struct e3e8_type *) context)->data = class;

    ((struct e3e8_type *) context)->link = link_data;

    return status;
}


static int
link_c067(void *context)
{
    int status;
    struct link_type link_v050;
    const struct link_type *link_data;
    union data_type *class, data;

    link_data = ((struct e3e8_type *) context)->link;

    class = ((struct e3e8_type *) context)->data;

    ((struct e3e8_type *) context)->data = &data;

    ((struct e3e8_type *) context)->link = &link_v050;

    link_v050.command = DIVA;

    data.press.data = class->order.link;

    status = link_e050(context);
    if (!status) {
	((struct e3e8_type *) context)->link = class->order.post;

	data.passe.data = class->order.data;
	data.passe.delete = class->order.delete;

	status = link_e067(context);
	if (!status) {
	}
    }

    ((struct e3e8_type *) context)->data = class;

    ((struct e3e8_type *) context)->link = link_data;

    return status;
}


static int
link_c124(void *context)
{
    int status;

    status = link_e047(context);
    if (!status) {
	struct link_type link_v007;
	const struct link_type *link_data;

	link_data = ((struct e3e8_type *) context)->link;

	((struct e3e8_type *) context)->link = &link_v007;

	link_v007.command = JOIN;

	status = link_e047(context);
	if (!status) {
	}

	((struct e3e8_type *) context)->link = link_data;
    }

    return status;
}


static int
link_c164(void *context)
{
    int status;

    status = link_e050(context);
    if (!status) {
	struct link_type link_v007;
	const struct link_type *link_data;

	link_data = ((struct e3e8_type *) context)->link;

	((struct e3e8_type *) context)->link = &link_v007;

	link_v007.command = JOIN;

	status = link_e047(context);
	if (!status) {
	}

	((struct e3e8_type *) context)->link = link_data;
    }

    return status;
}


static int
link_e004(void *context)
{
    int status;

    status = case_v1f4(context, ((struct e3e8_type *) context)->link);
    if (!status) {
	status = case_v1f7
	    (context,
	     &((union data_type *)
	       ((struct e3e8_type *) context)->data)->shift.class);
    }

    return status;
}


static int
link_e017(void *context)
{
    int status;

    status = link_e050(context);
    if (!status) {
	status = read_data(context);
    }

    return status;
}


static int
link_e035(void *context)
{
    int status;

    status = case_v1f4(context, ((struct e3e8_type *) context)->link);
    if (!status) {
	status = case_v1f8
	    (context,
	     &((union data_type *)
	       ((struct e3e8_type *) context)->data)->begin);
    }

    return status;
}


static int
link_e047(void *context)
{
    return case_v1f4(context, ((struct e3e8_type *) context)->link);
}


static int
link_e050(void *context)
{
    int status;

    status = case_v1f4(context, ((struct e3e8_type *) context)->link);
    if (!status) {
	status = case_v1f6
	    (context,
	     ((union data_type *)
	      ((struct e3e8_type *) context)->data)->press.data);
    }

    return status;
}


static int
link_e056(void *context)
{
    int status;

    status = case_v1f4(context, ((struct e3e8_type *) context)->link);
    if (!status) {
	union data_type *class;

	class = ((struct e3e8_type *) context)->data;
	status = case_v1f7(context, &class->scale.rank);
	if (!status) {
	    status = case_v1f9(context, &class->scale.delete);
	}
    }

    return status;
}


static int
link_e057(void *context)
{
    int status;

    status = case_v1f4(context, ((struct e3e8_type *) context)->link);
    if (!status) {
	status = case_v1f5
	    (context,
	     &((union data_type *)
	       ((struct e3e8_type *) context)->data)->inter.delete);
    }

    return status;
}


static int
link_e066(void *context)
{
    int status;

    status = case_v1f4(context, ((struct e3e8_type *) context)->link);
    if (!status) {
	status = case_v1f9
	    (context,
	     &((union data_type *)
	       ((struct e3e8_type *) context)->data)->point.data);
    }

    return status;
}


static int
link_e067(void *context)
{
    int status;

    status = case_v1f4(context, ((struct e3e8_type *) context)->link);
    if (!status) {
	union data_type *class;

	class = ((struct e3e8_type *) context)->data;
	status = case_v1f5(context, &class->passe.delete);
	if (!status) {
	    status = case_v1f6(context, class->passe.data);
	}
    }

    return status;
}


static int
link_e071(void *context)
{
    int status;

    status = case_v1f4(context, ((struct e3e8_type *) context)->link);
    if (!status) {
	status = case_v1f5
	    (context,
	     &((union data_type *)
	       ((struct e3e8_type *) context)->data)->white.major);
	if (!status) {
	    status = case_v1f5
		(context,
		 &((union data_type *)
		   ((struct e3e8_type *) context)->data)->white.minor);
	}
    }

    return status;
}


static int
link_e074(void *context)
{
    int status;

    status = case_v1f4(context, ((struct e3e8_type *) context)->link);
    if (!status) {
	status = case_v1fa
	    (context, 
	     &((union data_type *)
	       ((struct e3e8_type *) context)->data)->slide.class);
	if (!status) {
	    status = case_v1f7
		(context,
		 &((union data_type *)
		   ((struct e3e8_type *) context)->data)->slide.order);
	}
    }

    return status;
}


static int
link_e075(void *context)
{
    int status;

    status = case_v1f4(context, ((struct e3e8_type *) context)->link);
    if (!status) {
	status = link_a075(context);
    }

    return status;
}


static int
link_e140(void *context)
{
    int status;

    status = case_v1f4(context, ((struct e3e8_type *) context)->link);
    if (!status) {
	status = case_v1f5
	    (context,
	     &((union data_type *)
	       ((struct e3e8_type *) context)->data)->black.delete);
	if (!status) {
	    status = link_a075(context);
	}
    }

    return status;
}


static int
link_e200(void *context)
{
    int status;

    status = case_v1f4(context, ((struct e3e8_type *) context)->link);
    if (status) {
    } else {
	union data_type *class;

	class = ((struct e3e8_type *) context)->data;
	status = case_v1f7(context, &class->patch.major);
	if (status) {
	} else {
	    status = case_v1f7(context, &class->patch.minor);
	    if (status) {
	    } else {
		status = case_v1f6(context, class->patch.text);
	    }
	}
    }

    return status;
}


static int
list_e002(void *file, const char *data, unsigned size)
{
    struct mode_type mode;

    putchar('\n');
    fwrite(data, size, 1, file);
    mode.append = 0;
    mode.file = file;
    x1f4_list_ctkey(&mode, list_mode, switch_root);

    return 0;
}


static int
list_e003(void *file, const char *data, unsigned size)
{
    struct mode_type mode;

    putchar('\n');
    fwrite(data, size, 1, file);
    mode.append = 0;
    mode.file = file;
    x1f4_list_ctkey(&mode, list_mode, switch_root);
    putchar('\n');
    fwrite(data, size, 1, file);
    fputs(" <cardinal>", file);

    return 0;
}


static int
list_e004(void *file, const char *data, unsigned size)
{
    fputs(" <cardinal>", file);

    return 0;
}


static int
list_e023(void *file, const char *data, unsigned size)
{
    struct mode_type mode;

    mode.append = 0;
    mode.file = file;
    x1f4_list_ctkey(&mode, list_mode, order_root);

    return 0;
}


static int
list_e031(void *file, const char *data, unsigned size)
{
    fputs(" <integral>", file);

    return 0;
}


static int
list_e035(void *file, const char *data, unsigned size)
{
    fputs(" <c-time>", file);

    return 0;
}


static int
list_e042(void *file, const char *data, unsigned size)
{
    fputs(" <string>", file);

    return 0;
}


static int
list_e044(void *file, const char *data, unsigned size)
{
    putchar('\n');
    fwrite(data, size, 1, file);
    fputs(" all\n", file);
    fwrite(data, size, 1, file);
    fputs(" <cardinal>", file);

    return 0;
}


static int
list_e051(void *file, const char *data, unsigned size)
{
    struct mode_type mode;

    mode.file = file;

    fputs(" <integral>\n", file);
    fwrite(data, size, 1, file);
    mode.append = 0;
    x1f4_list_ctkey(&mode, list_mode, scope_root);
    mode.append = 0;
    x1f4_list_ctkey(&mode, list_mode, swing_root);
    fputs(" <integral>", file);

    return 0;
}


static int
list_e056(void *file, const char *data, unsigned size)
{
    fputs(" <cardinal> <real>", file);

    return 0;
}


static int
list_e057(void *file, const char *data, unsigned size)
{
    putchar('\n');
    fwrite(data, size, 1, file);
    fputs(" <integral>", file);

    return 0;
}


static int
list_e066(void *file, const char *data, unsigned size)
{
    fputs(" <real>", file);

    return 0;
}


static int
list_e071(void *file, const char *data, unsigned size)
{
    fputs(" <integral> <integral>", file);

    return 0;
}


static int
list_e072(void *file, const char *data, unsigned size)
{
    struct mode_type mode;

    fputs(" [", file);
    mode.append = 0;
    mode.file = file;
    x1f4_list_atkey(&mode, list_miss, field_root);
    fputs(" <string>]... [current <integral>]... <string>", file);

    return 0;
}


static int
list_e073(void *file, const char *data, unsigned size)
{
    struct mode_type mode;

    fputs(" <cardinal>\n", file);
    fwrite(data, size, 1, file);
    mode.append = 0;
    mode.file = file;
    x1f4_list_ctkey(&mode, list_mode, index_root);
    fputs(" <cardinal>", file);

    return 0;
}


static int
list_e200(void *file, const char *data, unsigned size)
{
    fputs(" <cardinal> <cardinal> <string>", file);

    return 0;
}


static int
list_e224(void *file, const char *data, unsigned size)
{
    return 0;
}


static int
list_s042(void *file, const char *data, unsigned size)
{
    fputs(" <string>\n", file);
    fwrite(data, size, 1, file);
    fputs(" <string> <string>", file);

    return 0;
}


static int
list_s067(void *file, const char *data, unsigned size)
{
    struct mode_type mode;

    mode.file = file;

    fputs(" <integral> <string>\n", file);
    fwrite(data, size, 1, file);
    fputs(" <integral> <string> <string>\n", file);
    fwrite(data, size, 1, file);
    mode.append = 0;
    x1f4_list_ctkey(&mode, list_mode, scope_root);
    mode.append = 0;
    x1f4_list_ctkey(&mode, list_mode, swing_root);
    fputs(" <integral> <string>\n", file);
    fwrite(data, size, 1, file);
    mode.append = 0;
    x1f4_list_ctkey(&mode, list_mode, scope_root);
    mode.append = 0;
    x1f4_list_ctkey(&mode, list_mode, swing_root);
    fputs(" <integral> <string> <string>", file);

    return 0;
}


static int
list_link(void *file, const void *node, const char *data, unsigned size)
{
    fwrite(data, size, 1, file);
    ((const struct link_type *) node)->list(file, data, size);
    fputc('\n', file);

    return 0;
}


static int
list_miss(void *mode, const char *data, unsigned size)
{
    struct mode_type *mode_data;
    void *file;

    mode_data = mode;
    file = mode_data->file;
    if (mode_data->append) {
	fputc('|', file);
    } else {
	mode_data->append = 1;
    }
    fwrite(data, size, 1, file);

    return 0;
}


static int
list_mode(void *mode, const char *data, unsigned size)
{
    struct mode_type *mode_data;
    void *file;

    mode_data = mode;
    file = mode_data->file;
    if (mode_data->append) {
	fputc('|', file);
    } else {
	fputc(' ', file);
	mode_data->append = 1;
    }
    fwrite(data, size, 1, file);

    return 0;
}


static int
miss_time(const char *specification, void *time)
{
    const char *s, *t;
    int status;
    unsigned a;

    s = specification;
    status = x1f4_parse_wcardinal(&a, s, &t, 10);
    if (!status) {
	SET_TIME(time, 0, 0, a, 0);
    } else if (s != t) {
	if (*t == ',') {
	    unsigned d;

	    status = x1f4_parse_cardinal(&d, t + 1, NULL, 10);
	    if (!status) {
		SET_TIME(time, 0, 0, a, d);
	    }
	} else if (*t == ':') {
	    unsigned b;

	    s = t + 1;
	    status = x1f4_parse_cardinal(&b, s, &t, 10);
	    if (!status) {
		SET_TIME(time, 0, a, b, 0);
	    } else if (s != t) {
		if (*t == ',') {
		    unsigned d;

		    status = x1f4_parse_cardinal(&d, t + 1, NULL, 10);
		    if (!status) {
			SET_TIME(time, 0, a, b, d);
		    }
		} else if (*t == ':') {
		    unsigned c;

		    s = t + 1;
		    status = x1f4_parse_cardinal(&c, s, &t, 10);
		    if (!status) {
			SET_TIME(time, a, b, c, 0);
		    } else if (s != t) {
			if (*t == ',') {
			    unsigned d;

			    status = x1f4_parse_cardinal(&d, t + 1, NULL, 10);
			    if (!status) {
				SET_TIME(time, a, b, c, d);
			    }
			}
		    }
		}
	    }
	}
    }

    return status;
}


static int
read_data(void *context)
{
    int connection, data = 0, status;
    unsigned size = 0;

    connection = ((struct e3e8_type *) context)->connection;

    while (1) {
	unsigned char c;

	status = read(connection, &c, 1);
	if (status != 1) {
	    status = stat_read(context, status);

	    break;
	}

	if (!data) {
	    if (c & 128) {
		c &= ~128;
	    } else {
		data = 1;
		if (!c) {
		    if (!size) {
			status = 0;
			break;
		    }
		}
	    }

	    size <<= 7;
	    size += c;
	} else {
	    putchar(c);
	    size--;
	    if (!size) {
		status = 0;
		break;
	    }
	}
    }

    return status;
}


static int
slip_data(void *context, unsigned size, const void *data)
{
    int connection, status;

    connection = ((struct e3e8_type *) context)->connection;

    status = write(connection, data, size);
    if (status != (int) size) {
	status = 1;
	fprintf(stderr, "%s: cannot write: %s\n",
		((struct e3e8_type *) context)->self, strerror(errno));
	if (SIGPIPE_received) {
	    fprintf(stderr, "%s: Mylene1f4 connection shut down\n",
		    ((struct e3e8_type *) context)->self);
	}
    } else {
	status = 0;
    }

    return status;
}


static int
slip_text(void *context, unsigned size, const void *data)
{
    char class = 0;
    int status;

    status = slip_data(context, 1, & class);
    if (!status) {
	status = slip_data(context, size - 1, data);
    }

    return status;
}


static int
stat_read(void *context, int status)
{
    if (!status) {
	fprintf(stderr, "%s: connection closed\n",
		((struct e3e8_type *) context)->self);
    } else {
	fprintf(stderr, "%s: cannot read connection: %s\n",
		((struct e3e8_type *) context)->self, strerror(errno));
    }

    return 1;
}


static int
type_a057(void *context, int argc, char **argv)
{
    int status;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    if (argc != 1) {
	stat_count(context);
	status = 1;
    } else {
	union data_type *class;

	class = mylene3e8->data;

	status = x1f4_parse_xintegral(&class->inter.delete, argv[0], NULL, 0);
	if (status) {
	    stat_value(mylene3e8->self, 0, argv[0]);
	}
    }

    return status;
}


static int
type_c042(void *context, int argc, char **argv)
{
    int status;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    if (0) {
    } else {
	union data_type *class;

	status = 0;

	class = mylene3e8->data;

	class->class.major = argv[0];
	class->class.minor = argv[1];
    }

    return status;
}


static int
type_c067(void *context, int argc, char **argv)
{
    int status;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    if (argc != 3) {
	stat_count(context);
	status = 1;
    } else {
	union data_type *class;
	const void *post;

	class = mylene3e8->data;

	post = class->trans.post;

	status = x1f4_parse_xintegral(&class->order.delete, argv[0], NULL, 0);
	if (status) {
	    stat_value(mylene3e8->self, 0, argv[0]);
	} else {
	    class->order.data = argv[2];
	    class->order.link = argv[1];
	    class->order.post = post;
	}
    }

    return status;
}


static int
type_e002(void *context, int argc, char **argv)
{
    int status;

    if (!argc) {
	status = 0;
    } else {
	struct e3e8_type *mylene3e8;

	mylene3e8 = context;

	if (argc != 1) {
	    stat_count(context);
	    status = 1;
	} else {
	    unsigned detail;

	    status = x1f4_find_ctkey(argv[0], switch_root, &detail);
	    if (status) {
		stat_value(mylene3e8->self, 0, argv[0]);
	    } else {
		if (!detail) {
		    mylene3e8->link = &link_v014;
		} else {
		    mylene3e8->link = &link_v015;
		}
	    }
	}
    }

    return status;
}


static int
type_e003(void *context, int argc, char **argv)
{
    int status;

    if (!argc) {
	status = 0;
    } else {
	struct e3e8_type *mylene3e8;

	mylene3e8 = context;

	if (argc != 1) {
	    stat_count(context);
	    status = 1;
	} else {
	    unsigned detail;

	    status = x1f4_find_ctkey(argv[0], switch_root, &detail);
	    if (!status) {
		if (!detail) {
		    mylene3e8->link = &link_v012;
		} else {
		    mylene3e8->link = &link_v013;
		}
	    } else {
		union data_type *class;

		class = mylene3e8->data;

		status = x1f4_parse_wxcardinal
		    (&class->shift.class, argv[0], NULL, 0);
		if (status) {
		    stat_value(mylene3e8->self, 0, argv[0]);
		} else {
		    mylene3e8->link = &link_v020;
		}
	    }
	}
    }

    return status;
}


static int
type_e004(void *context, int argc, char **argv)
{
    int status;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    if (argc != 1) {
	stat_count(context);
	status = 1;
    } else {
	union data_type *class;

	class = mylene3e8->data;

	status = x1f4_parse_wxcardinal(&class->shift.class, argv[0], NULL, 0);
	if (status) {
	    stat_value(mylene3e8->self, 0, argv[0]);
	}
    }

    return status;
}


static int
type_e011(void *context, int argc, char **argv)
{
    int status;

    if (!argc) {
	((struct e3e8_type *) context)->link = &link_v001;
	status = 0;
    } else {
	status = type_a057(context, argc, argv);
    }

    return status;
}


static int
type_e016(void *context, int argc, char **argv)
{
    int status;

    if (!argc) {
	((struct e3e8_type *) context)->link = &link_v006;
	status = 0;
    } else {
	status = type_a057(context, argc, argv);
    }

    return status;
}


static int
type_e023(void *context, int argc, char **argv)
{
    int status;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    if (argc != 1) {
	stat_count(context);
	status = 1;
    } else {
	unsigned detail;

	status = x1f4_find_ctkey(argv[0], order_root, &detail);
	if (status) {
	    stat_value(mylene3e8->self, 0, argv[0]);
	} else {
	    if (detail == ORDER_n) {
		((struct e3e8_type *) context)->link = &link_v021;
	    } else {
		if (detail == ORDER_p) {
		    ((struct e3e8_type *) context)->link = &link_v022;
		}
	    }
	}
    }

    return status;
}


static int
type_e035(void *context, int argc, char **argv)
{
    int status;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    if (argc != 1) {
	stat_count(context);
	status = 1;
    } else {
	status = miss_time
	    (argv[0], &((union data_type *) mylene3e8->data)->begin);
	if (status) {
	    stat_value(mylene3e8->self, 0, argv[0]);
	}
    }

    return status;
}


static int
type_e042(void *context, int argc, char **argv)
{
    int status;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    if (argc != 1) {
	stat_count(context);
	status = 1;
    } else {
	status = 0;
	((union data_type *) mylene3e8->data)->press.data = argv[0];
    }

    return status;
}


static int
type_e044(void *context, int argc, char **argv)
{
    int status;

    if (!argc) {
	status = 0;
    } else {
	struct e3e8_type *mylene3e8;

	mylene3e8 = context;

	if (argc != 1) {
	    stat_count(context);
	    status = 1;
	} else {
	    char *argv_0_;
	    int deque = 0;

	    argv_0_ = argv[0];

	    if (*argv_0_ == 0x61) {
		if (!argv_0_[1]) {
		    deque = 1;
		} else {
		    if (argv_0_[1] == 0x6c) {
			if (!argv_0_[2]) {
			    deque = 1;
			} else {
			    if (argv_0_[2] == 0x6c) {
				if (!argv_0_[3]) {
				    deque = 1;
				}
			    }
			}
		    }
		}
	    }

	    if (deque) {
		((struct e3e8_type *) context)->link = &link_v046;
		status = 0;
	    } else {
		status = x1f4_parse_wxcardinal
		    (&((union data_type *) mylene3e8->data)->shift.class,
		     argv_0_, NULL, 0);
		if (status) {
		    stat_value(mylene3e8->self, 0, argv_0_);
		} else {
		    ((struct e3e8_type *) context)->link = &link_v045;
		}
	    }
	}
    }

    return status;
}


static int
type_e056(void *context, int argc, char **argv)
{
    int status;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    if (argc != 2) {
	stat_count(context);
	status = 1;
    } else {
	union data_type *class;

	class = mylene3e8->data;

	status = x1f4_parse_wxcardinal(&class->scale.rank, argv[0], NULL, 0);
	if (status) {
	    stat_value(mylene3e8->self, 0, argv[0]);
	} else {
	    status = x1f4_parse_real(&class->scale.delete, argv[1], NULL);
	    if (status) {
		stat_value(mylene3e8->self, 1, argv[1]);
	    }
	}
    }

    return status;
}


static int
type_e057(void *context, int argc, char **argv)
{
    int status;

    if (!argc) {
	((struct e3e8_type *) context)->link = &link_v047;
	status = 0;
    } else {
	status = type_a057(context, argc, argv);
    }

    return status;
}


static int
type_e064(void *context, int argc, char **argv)
{
    int status;

    if (!argc) {
	((struct e3e8_type *) context)->link = &link_v054;
	status = 0;
    } else {
	status = type_a057(context, argc, argv);
    }

    return status;
}


static int
type_e065(void *context, int argc, char **argv)
{
    int status;

    if (!argc) {
	((struct e3e8_type *) context)->link = &link_v055;
	status = 0;
    } else {
	status = type_a057(context, argc, argv);
    }

    return status;
}


static int
type_e066(void *context, int argc, char **argv)
{
    int status;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    if (argc != 1) {
	stat_count(context);
	status = 1;
    } else {
	union data_type *class;

	class = mylene3e8->data;

	status = x1f4_parse_real(&class->point.data, argv[0], NULL);
	if (status) {
	    stat_value(mylene3e8->self, 0, argv[0]);
	}
    }

    return status;
}


static int
type_e067(void *context, int argc, char **argv)
{
    int status;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    if (argc != 2) {
	stat_count(context);
	status = 1;
    } else {
	union data_type *class;

	class = mylene3e8->data;

	status = x1f4_parse_xintegral(&class->passe.delete, argv[0], NULL, 0);
	if (status) {
	    stat_value(mylene3e8->self, 0, argv[0]);
	} else {
	    class->passe.data = argv[1];
	}
    }

    return status;
}


static int
type_e071(void *context, int argc, char **argv)
{
    int status;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    if (argc != 2) {
	stat_count(context);
	status = 1;
    } else {
	union data_type *class;

	class = mylene3e8->data;

	status = x1f4_parse_xintegral(&class->white.major, argv[0], NULL, 0);
	if (status) {
	    stat_value(mylene3e8->self, 0, argv[0]);
	} else {
	    status = x1f4_parse_xintegral
		(&class->white.minor, argv[1], NULL, 0);
	    if (status) {
		stat_value(mylene3e8->self, 1, argv[1]);
	    }
	}
    }

    return status;
}


static int
type_e072(void *context, int argc, char **argv)
{
    int status;

    if (argc == 1) {
	status = type_e042(context, argc, argv);
    } else {
	struct e3e8_type *mylene3e8;

	mylene3e8 = context;

	if (!(argc & 1)) {
	    stat_count(context);
	    status = 1;
	} else {
	    char *frame_R = NULL;
	    int tagged = 0;
	    union data_type *class;
	    unsigned count, frame_d = 0, i;

	    class = mylene3e8->data;

	    class->black.album = NULL;
	    class->black.match = NULL;
	    class->black.notes = NULL;
	    class->black.title = NULL;
	    class->black.frame = NULL;

	    count = argc >> 1;
	    i = count;
	    for (; i; i--) {
		unsigned frame;

		status = x1f4_find_ctkey(argv[0], frame_root, &frame);
		if (status) {
		    stat_value(mylene3e8->self, (count - i) << 1, argv[0]);
		    break;
		} else {
		    if (frame == FRAME_R) {
			frame_R = argv[1];
			frame_d = i;
		    } else {
			tagged = 1;

			switch (frame) {
			case FRAME_A:
			    class->black.album = argv[1];
			    break;
			case FRAME_a:
			    class->black.match = argv[1];
			    break;
			case FRAME_c:
			    class->black.notes = argv[1];
			    break;
			case FRAME_t:
			    class->black.title = argv[1];
			    break;
			default:
			    class->black.frame = argv[1];
			}
		    }

		    argv += 2;
		}
	    }

	    if (i) {
		status = 1;
	    } else {
		if (!frame_R) {
		    status = 0;

		    class->black.sixth = argv[0];

		    mylene3e8->link = &link_v075;
		} else {
		    int index;

		    status = x1f4_parse_xintegral(&index, frame_R, NULL, 0);
		    if (status) {
			stat_value
			    (mylene3e8->self,
			     ((count - frame_d) << 1) + 1, frame_R);
		    } else {
			if (!tagged) {
			    class->passe.data = argv[0];
			    class->passe.delete = index;

			    mylene3e8->link = &link_v100;
			} else {
			    class->black.delete = index;

			    class->black.sixth = argv[0];

			    mylene3e8->link = &link_v140;
			}
		    }
		}
	    }
	}
    }

    return status;
}


static int
type_e073(void *context, int argc, char **argv)
{
    int status;

    if (argc == 1) {
	status = type_e004(context, argc, argv);
    } else {
	struct e3e8_type *mylene3e8;

	mylene3e8 = context;

	if (argc != 2) {
	    stat_count(context);
	    status = 1;
	} else {
	    union data_type *class;

	    class = mylene3e8->data;

	    status = x1f4_find_ctkey(argv[0], index_root, &class->slide.class);
	    if (status) {
		stat_value(mylene3e8->self, 0, argv[0]);
	    } else {
		status = x1f4_parse_wxcardinal
		    (&class->slide.order, argv[1], NULL, 0);
		if (status) {
		    stat_value(mylene3e8->self, 1, argv[1]);
		} else {
		    mylene3e8->link = &link_v074;
		}
	    }
	}
    }

    return status;
}


static int
type_e076(void *context, int argc, char **argv)
{
    int status;

    if (!argc) {
	((struct e3e8_type *) context)->link = &link_v070;
	status = 0;
    } else {
	status = type_a057(context, argc, argv);
    }

    return status;
}


static int
type_e200(void *context, int argc, char **argv)
{
    int status;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    if (argc != 3) {
	stat_count(context);
	status = 1;
    } else {
	union data_type *class;

	class = mylene3e8->data;

	status = x1f4_parse_wxcardinal(&class->patch.major, argv[0], NULL, 0);
	if (status) {
	    stat_value(mylene3e8->self, 0, argv[0]);
	} else {
	    status = x1f4_parse_wxcardinal
		(&class->patch.minor, argv[1], NULL, 0);
	    if (status) {
		stat_value(mylene3e8->self, 1, argv[1]);
	    } else {
		class->patch.text = argv[2];
	    }
	}
    }

    return status;
}


static int
type_e224(void *context, int argc, char **argv)
{
    int status;

    if (argc) {
	stat_count(context);
	status = 1;
    } else {
	status = 0;
    }

    return status;
}


static int
type_s042(void *context, int argc, char **argv)
{
    int status;

    if (argc != 2) {
	status = type_e042(context, argc, argv);
    } else {
	((struct e3e8_type *) context)->link = &link_u042;

	status = type_c042(context, argc, argv);
    }

    return status;
}


static int
type_s067(void *context, int argc, char **argv)
{
    int status;

    if (argc != 3) {
	status = type_e067(context, argc, argv);
    } else {
	((struct e3e8_type *) context)->link = &link_u067;

	((union data_type *)
	 ((struct e3e8_type *) context)->data)->trans.post = &link_v067;

	status = type_c067(context, argc, argv);
    }

    return status;
}


static int
type_s134(void *context, int argc, char **argv)
{
    int status;

    if (argc != 3) {
	status = type_e067(context, argc, argv);
    } else {
	((struct e3e8_type *) context)->link = &link_u067;

	((union data_type *)
	 ((struct e3e8_type *) context)->data)->trans.post = &link_v134;

	status = type_c067(context, argc, argv);
    }

    return status;
}


static int
type_s135(void *context, int argc, char **argv)
{
    int status;

    if (argc != 3) {
	status = type_e067(context, argc, argv);
    } else {
	((struct e3e8_type *) context)->link = &link_u067;

	((union data_type *)
	 ((struct e3e8_type *) context)->data)->trans.post = &link_v135;

	status = type_c067(context, argc, argv);
    }

    return status;
}


static int
type_v051(void *context, int argc, char **argv)
{
    int status;

    if (argc < 2) {
	status = type_a057(context, argc, argv);
    } else {
	struct e3e8_type *mylene3e8;
	unsigned scope;

	mylene3e8 = context;

	status = x1f4_find_ctkey(argv[0], scope_root, &scope);
	if (status) {
	    stat_value(mylene3e8->self, 0, argv[0]);
	} else {
	    unsigned swing;

	    status = x1f4_find_ctkey(argv[1], swing_root, &swing);
	    if (status) {
		stat_value(mylene3e8->self, 1, argv[1]);
	    } else {
		if (swing) {
		    mylene3e8->link = &link_v136;
		} else {
		    mylene3e8->link = &link_v137;
		}

		status = ((struct link_type *) mylene3e8->link)->type
		    (context, argc - 2, argv + 2);
	    }
	}
    }

    return status;
}


static int
type_v067(void *context, int argc, char **argv)
{
    int status;

    if (argc < 4) {
	status = type_s067(context, argc, argv);
    } else {
	struct e3e8_type *mylene3e8;
	unsigned scope;

	mylene3e8 = context;

	status = x1f4_find_ctkey(argv[0], scope_root, &scope);
	if (status) {
	    stat_value(mylene3e8->self, 0, argv[0]);
	} else {
	    unsigned swing;

	    status = x1f4_find_ctkey(argv[1], swing_root, &swing);
	    if (status) {
		stat_value(mylene3e8->self, 1, argv[1]);
	    } else {
		if (!swing) {
		    mylene3e8->link = &link_v134;
		} else {
		    mylene3e8->link = &link_v135;
		}

		status = ((struct link_type *) mylene3e8->link)->type
		    (context, argc - 2, argv + 2);
	    }
	}
    }

    return status;
}


static void
SIGPIPE_handler(int none)
{
    SIGPIPE_received = 1;
}


static void
detail_types(void)
{
    puts("\
c-time				[[cardinal:]cardinal:]cardinal[,cardinal]");
}


static void
stat_count(void *context)
{
    const char *self;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    self = mylene3e8->self;

    fprintf(stderr, "%s: wrong number of arguments for the `", self);
    fputs(mylene3e8->type, stderr);
    fprintf(stderr, "' command\nType `%s --help' for more information.\n",
	    self);
}


static void
stat_value(const char *self, unsigned rank, const char *line)
{
    char suffix[2];

    x1f4_extend_ordinal(suffix, rank);
    fprintf(stderr, "%s: cannot parse command %u%.*s argument specification:"
	    " `%s'\n", self, rank + 1, 2, suffix, line);
}


static void
usage(void)
{
    puts("Usage: Mylene3e8 [OPTIONS] PATH|PORT COMMAND [ARGUMENT]...\n\
Control Mylene1f4 server.\n\
\n\
Options:\n\
  -i, --inet			connect to INET PORT server\n\
  -u, --unix			connect to UNIX PATH server\n\
  -l, --list			list available commands and exit\n\
  -t, --type			detail uncommon command argument types and e\
xit\n\
  -x, --synchronize		wait for command execution completion\n\
      --help			display this help and exit\n\
      --version			output version information and exit");
}


int
main(int argc, char **argv)
{
    int do_list = 0, hi_unix = 1, status = 1;
    struct e3e8_type e3e8;

    e3e8.do_synchronize = 0;

    unsetenv("POSIXLY_CORRECT");

    while (1) {
	char c;
	static struct option long_options[] = {
/* *INDENT-OFF* */
	    {	"inet",		0x00,	NULL,	0x69	},
	    {	"list",		0x00,	NULL,	0x6c	},
	    {	"type",		0x00,	NULL,	0x74	},
	    {   "synchronize",  0x00,   NULL,   0x78    },
	    {	"unix",		0x00,	NULL,	0x75	},
	    {	"version",	0x00,	NULL,	0x76	},
	    {	"help",		0x00,	NULL,	0x68	},
	    {	NULL,		0x00,	NULL,	0x00	}
/* *INDENT-ON* */
	};

	c = getopt_long(argc, argv, "iltux", long_options, NULL);

	if (!~c) {
	    break;
	}

	switch (c) {
	case 'i':
	    hi_unix = 0;
	    break;
	case 'l':
	    do_list = 1;
	    break;
	case 't':
	    do_list = 2;
	    break;
	case 'u':
	    hi_unix = 1;
	    break;
	case 'h':
	    usage();

	    return 0;
	case 'v':
	    printf("%s (%s) %s\n", argv[0], PACKAGE, VERSION);

	    return 0;
	case 'x':
	    e3e8.do_synchronize = 1;
	    break;
	case '?':
	    return 1;
	}
    }

    if (do_list) {
	if (do_list == 1) {
	    status = x1f4_hook_vtkey(stdout, list_link, link_root);
	} else {
	    status = 0;
	    detail_types();
	}
    } else {
	if (argc - optind < 2) {
	    fprintf(stderr, "%s: wrong number of arguments\nType `%s --help'"
		    " for more information.\n", argv[0], argv[0]);
	} else {
	    do {
		int domain;
		unsigned port;
		void *argv_1_;

		if (hi_unix) {
		    argv_1_ = argv[optind];
		    domain = AF_UNIX;
		} else {
		    domain = PF_INET;
		    argv_1_ = &port;
		    if (x1f4_parse_wxcardinal
			(argv_1_, argv[optind], NULL, 0)) {
			fprintf(stderr, "%s: cannot parse port specification"
				": `%s'\n", argv[0], argv[optind]);
			break;
		    } else if (65535 < port) {
			fprintf(stderr, "%s: port specification (`%u') is ou"
				"tside valid range (0 .. 65535)\n", argv[0],
				port);
			break;
		    }
		}

		e3e8.self = argv[0];

		{
		    const void *link;

		    optind++;

		    if (x1f4_find_vtkey(argv[optind], link_root, &link)) {
			fprintf(stderr, "%s: cannot parse command specificat"
				"ion: `%s'\n", argv[0], argv[optind]);
		    } else {
			const struct link_type *link_data;
			union data_type data;

			e3e8.data = &data;

			link_data = link;

			e3e8.link = link;

			e3e8.type = argv[optind];

			optind++;

			status = link_data->type
			    (&e3e8, argc - optind, argv + optind);
			if (!status) {
			    status = case_x3e8(&e3e8, domain, argv_1_);
			}
		    }
		}
	    } while (0);
	}
    }

    return status;
}
