/*
 * ev24.c
 * Copyright (C) 2006-2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if defined HAVE_FEATURES_H
# include <features.h>
#endif				/* HAVE_FEATURES_H */

#include <getopt.h>
#if defined HAVE_LIBx1f4i0
# include <libx1f4i0.h>
#endif				/* HAVE_LIBx1f4i0 */
#if defined HAVE_LIBx1f4l0
# include <libx1f4l0.h>
#endif				/* HAVE_LIBx1f4l0 */
#include <math.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <aime.h>
#if !defined HAVE_LIBx1f4i0
# include <cardinal-wx.h>
#endif				/* !HAVE_LIBx1f4i0 */
#include <f2.h>
#if !defined HAVE_LIBx1f4i0
# include <float1.h>
#endif				/* !HAVE_LIBx1f4i0 */
#include <inter.h>
#include <sf.h>
#include <types.h>

#define MAKE_DOUBLE(a, b)		a b
#define MAKE_SINGLE(a, b)		a

#define I_MODE(i)			(*((X1f4_E4_C_MODE *) (i)))
#define I_REAL(r)			(*((X1f4_E4_C_REAL *) (r)))
#define I_TEXT(r)			(*((X1f4_E4_C_TEXT *) (r)))

#define l_MODE(e, output) \
    {									      \
	X1f4_E4_C_MODE *l;						      \
									      \
	l = (output);							      \
	*l = (e);							      \
    }

#define l_USER(e, output) \
    {									      \
	X1f4_E4_C_USER *l;						      \
									      \
	l = (output);							      \
	*l = (X1f4_E4_C_USER) (e);					      \
    }

typedef struct context_type {
    struct x1f4_variable_type *variable_data;
    unsigned count;
    void **state;
} context_type;

static int c14xx_m__(void *, void **);
static int c23xx_m_m(void *, void **);
static int c32xx_m__(void *, void **);
static int c41xx_m_m(void *, void **);
static int c55xx_m__(void *, void **);
static int d14xx_m__(void *, void *, void **);
static int d32xx_m__(void *, void *, void **);
static int select_variable(const char *, unsigned, const void *,
			   const struct x1f4_variable_type **, void **);
static int test_cast(void *, const char *, unsigned);
static int type_ev24(void *);

static void usage(void);

static const int c_____m_m[] = {
/* *INDENT-OFF* */
    X1f4_E4_MODE,
    X1f4_E4_MODE
/* *INDENT-ON* */
};
static const struct x1f4_postmark_type e23pm_m_m[] = {
    {	MAKE_DOUBLE("<", ":"),	2				}
}, e41pm_m_m[] = {
    {	MAKE_DOUBLE(">", ":"),	2				}
};
static struct x1f4_2ndscope_type e23nd_m_m[1], e41nd_m_m[1];
static struct x1f4_linetext_type f14xx_m__ = {
/* *INDENT-OFF* */
    {	MAKE_DOUBLE("$", "~"),  d14xx_m__,
	X1f4_E4_MODE,		c_____m_m,	1,
	0,			2				},
    NULL
/* *INDENT-ON* */
}, f32xx_m__ = {
/* *INDENT-OFF* */
    {	MAKE_DOUBLE("$", "^"),  d32xx_m__,
	X1f4_E4_MODE,		c_____m_m,	1,
	0,			2				},
    NULL
/* *INDENT-ON* */
};
static const struct x1f4_linetext_type *const l55xx_m__[] = {
    &f14xx_m__,
    &f32xx_m__
};
static const struct x1f4_operator_type e14xx_m__[] = {
/* *INDENT-OFF* */
    {	MAKE_DOUBLE("$", "~"),  c14xx_m__,	0140,
	X1f4_E4_MODE,		c_____m_m,
	0,			2,
	NULL,			NULL				}
/* *INDENT-ON* */
}, e23xx_m_m[] = {
/* *INDENT-OFF* */
    {	MAKE_DOUBLE(":", ">"),  c23xx_m_m,	0160,
	X1f4_E4_MODE,		c_____m_m,
	X1f4_E4_E2ND_LINK,	2,
	NULL,			e23nd_m_m			}
/* *INDENT-ON* */
}, e32xx_m__[] = {
/* *INDENT-OFF* */
    {	MAKE_DOUBLE("$", "^"),  c32xx_m__,	0100,
	X1f4_E4_MODE,		c_____m_m,
	0,			2,
	NULL,			NULL				}
/* *INDENT-ON* */
}, e41xx_m_m[] = {
/* *INDENT-OFF* */
    {	MAKE_DOUBLE(":", "<"),  c41xx_m_m,	0120,
	X1f4_E4_MODE,		c_____m_m,
	X1f4_E4_E2ND_LINK,	2,
	NULL,			e41nd_m_m			}
/* *INDENT-ON* */
};
static struct x1f4_operator_type e55xx_m__[] = {
/* *INDENT-OFF* */
    {	MAKE_SINGLE("&", " "),  c55xx_m__,	0210,
	0,			c_____m_m,
	X1f4_E4_KEEP_CALL,	1,
	NULL,			NULL				}
/* *INDENT-ON* */
};
static const struct x1f4_operator_type *const ev24_e4_1_arithmetics[] = {
/* *INDENT-OFF* */
    e14xx_m__,
    e32xx_m__,
    e55xx_m__
/* *INDENT-ON* */
}, *const ev24_e4_2_arithmetics[] = {
/* *INDENT-OFF* */
    e23xx_m_m,
    e41xx_m_m
/* *INDENT-ON* */
};
static void *static_text;

static int
c14xx_m__(void *output, void **input)
{
    l_MODE(I_MODE(input[0]) + I_MODE(input[0]), output);

    return 0;
}


static int
c23xx_m_m(void *output, void **input)
{
    l_MODE(I_MODE(input[0]) + I_MODE(input[1]), output);

    return 0;
}


static int
c32xx_m__(void *output, void **input)
{
    l_MODE(I_MODE(input[0]) * I_MODE(input[0]), output);

    return 0;
}


static int
c41xx_m_m(void *output, void **input)
{
    l_MODE(I_MODE(input[0]) * I_MODE(input[1]), output);

    return 0;
}


static int
c55xx_m__(void *output, void **input)
{
    l_USER(l55xx_m__[I_MODE(input[0]) & 1], output);

    return 0;
}


static int
d14xx_m__(void *context, void *output, void **input)
{
    return c14xx_m__(output, input);
}


static int
d32xx_m__(void *context, void *output, void **input)
{
    return c32xx_m__(output, input);
}


static int
select_variable(const char *f, unsigned length, const void *context,
		const struct x1f4_variable_type **variable, void **state)
{
    int status = X1f4_E4_PARSE_ERROR;
    const struct context_type *context_data;
    const struct x1f4_variable_type *variable_data;

    context_data = context;
    variable_data = context_data->variable_data;
    if (variable_data) {
	while (variable_data->name) {
	    if (length == variable_data->length
		&& !memcmp((void *) f, variable_data->name, length)) {
		break;
	    }
	    variable_data++;
	}
	if (variable_data->name) {
	    status = 0;
	    *state = context_data->state
		[variable_data - context_data->variable_data];
	    *variable = variable_data;
	}
    }

    return status;
}


static int
test_cast(void *cast, const char *name, unsigned size)
{
    return size ? 1 ^ fwrite(name, size, 1, cast) : 0;
}


static int
type_ev24(void *e4fine)
{
    return x1f4_type_e4fine(e4fine, &e55xx_m__[0].type, l55xx_m__[0]);
}


static void
usage(void)
{
    puts("Usage: ev24 [OPTIONS] EXPRESSION [TYPE NAME VALUE]\n\
Test postfixed operators evaluation, evaluate EXPRESSION.\n\
\n\
Options:\n\
  -P, --print			print expression\n\
  -o, --optimize		enable optimizations\n\
  -p, --precision DIGITS	set precision for printing reals to DIGITS\n\
      --detail-constants	detail constants when printing expression\n\
      --detail-operators	detail operators when printing expression\n\
      --list-2			list available binary operators and exit\n\
      --help			display this help and exit\n\
      --version			output version information and exit");
}


int
main(int argc, char **argv)
{
    int do_optimize = 0, do_print = 0, list_functions = 0, status = 0;
    unsigned precision = 3, print_flags = 0;

    unsetenv("POSIXLY_CORRECT");

    {
	char *precision_specs = NULL;

	while (1) {
	    char c;
	    static struct option long_options[] = {
/* *INDENT-OFF* */
		{   "detail-constants",
				    0x00,   NULL,   0x12    },
		{   "detail-operators",
				    0x00,   NULL,   0x11    },
		{   "help",	    0x00,   NULL,   0x68    },
		{   "list-2",	    0x00,   NULL,   0x03    },
		{   "optimize",	    0x00,   NULL,   0x6f    },
		{   "version",	    0x00,   NULL,   0x76    },
		{   "precision",    0x01,   NULL,   0x70    },
		{   "print",	    0x00,   NULL,   0x50    },
		{   NULL,	    0x00,   NULL,   0x00    }
/* *INDENT-ON* */
	    };

	    c = getopt_long(argc, argv, "Pop:", long_options, NULL);

	    if (!~c) {
		break;
	    }

	    switch (c) {
	    case 003:
		list_functions = 3;
		break;
	    case 021:
		print_flags |= X1f4_E4_DETAIL_OPERATORS;
		break;
	    case 022:
		print_flags |= X1f4_E4_DETAIL_CONSTANTS;
		break;
	    case 'P':
		do_print = 1;
		break;
	    case 'o':
		do_optimize = 1;
		break;
	    case 'p':
		precision_specs = optarg;
		break;
	    case 'h':
		usage();

		return 0;
	    case 'v':
		printf("%s (%s) %s\n", argv[0], PACKAGE, VERSION);

		return 0;
	    case '?':
		return 1;
	    }
	}

	if (do_print) {
	    precision_specs = NULL;
	}

	if (precision_specs) {
	    if (x1f4_parse_wxcardinal(&precision, precision_specs, NULL, 0)) {
		fprintf(stderr, "%s: cannot parse precision specification: `"
			"%s'\n", argv[0], precision_specs);

		return 1;
	    }
	}
    }

    e23nd_m_m[0].bits = X1f4_E4_POST_MARK;
    e41nd_m_m[0].bits = X1f4_E4_POST_MARK;

    e23nd_m_m[0].post = e23pm_m_m;
    e41nd_m_m[0].post = e41pm_m_m;

    if (list_functions == 3) {
	const struct x1f4_operator_type *const *operAtor2s, *const *operator2s;

	x1f4_llink_operator2s(&operator2s);
	libx1f4i0_join_sets(&operAtor2s, operator2s, ev24_e4_2_arithmetics, 2);
	x1f4_print_operators(stdout, operAtor2s, 2);
	if (operAtor2s != operator2s) {
	    free((void *) operAtor2s);
	}
    } else if ((argc - optind) % 3 != 1) {
	status = 1;
	fprintf(stderr, "%s: wrong number of arguments\nType `%s --help' for"
		" more information.\n", argv[0], argv[0]);
    } else {
	do {
	    const char *expression;
	    struct context_type context;
	    struct x1f4_eerecord_type eerecord;
	    struct x1f4_variable_type *variable_data;
	    unsigned count, flags = X1f4_E4_DDACCESS | X1f4_E4_TRANSFER;
	    void *e4fine = NULL, **state = NULL, *x1f4_expression;
#if defined HAVE_LIBx1f4l0
	    void *valist = NULL;
#endif				/* HAVE_LIBx1f4l0 */

	    status = 1;

	    expression = argv[optind];
	    optind++;

	    static_text = &context;

	    status = libx1f4i0_lead_list
		(argv[0], argc, argv, optind, (void *) &variable_data, &state,
		 &count);
	    if (status) {
		break;
	    }

#if defined HAVE_LIBx1f4l0
	    {
		libx1f4i0_init_valist(&valist, 0, 0);
	    }
#endif				/* HAVE_LIBx1f4l0 */

#if defined HAVE_LIBx1f4l0
	    if (valist) {
		struct x1f4_e4fine_type pick_fine;

		libx1f4i0_lead_valist
		    (valist, 0, 0, &pick_fine.resource_set, NULL);

		x1f4_init_e4fine(&e4fine, X1f4_E4FINE_RESOURCE, &pick_fine);
	    } else {
#endif				/* HAVE_LIBx1f4l0 */
		x1f4_init_e4fine(&e4fine, 0, NULL);
#if defined HAVE_LIBx1f4l0
	    }
#endif				/* HAVE_LIBx1f4l0 */

	    if (e4fine) {
		type_ev24(e4fine);
	    }

	    {
		struct x1f4_attributes_type attributes;
		const struct x1f4_operator_type *const *operator1s,
		    *const *operator2s;

		context.count = count;
		context.state = state;
		context.variable_data = variable_data;

		attributes.bcollect_set.eerecord_data = &eerecord;
		attributes.function_set.get = libx1f4i0_select_function;
		attributes.function_set.context = x1f4_e4_defaults;
		x1f4_llink_operator1s(&operator1s);
		libx1f4i0_join_sets(&attributes.operator1s, operator1s,
				    ev24_e4_1_arithmetics, 3);
		x1f4_llink_operator2s(&operator2s);
		libx1f4i0_join_sets(&attributes.operator2s, operator2s,
				    ev24_e4_2_arithmetics, 2);
		attributes.terminator = 0;
		attributes.transfer_set.fine = e4fine;
		attributes.variable_set.get = select_variable;
		attributes.variable_set.context = &context;

		flags |= X1f4_E4_BCOLLECT | X1f4_E4_ESTORAGE;

#if defined HAVE_LIBx1f4l0
		if (valist) {
		    flags |= X1f4_E4_RESOURCE;
		    if (1) {
			libx1f4i0_lead_valist
			    (valist, 0, 0, &attributes.resource_set, NULL);
		    }
		}
#endif				/* HAVE_LIBx1f4l0 */

		{
		    status = x1f4_init_expression
			(&x1f4_expression, expression, flags, &attributes);
		}

		if (attributes.operator2s != operator1s) {
		    free((void *) attributes.operator1s);
		}

		if (attributes.operator2s != operator2s) {
		    free((void *) attributes.operator2s);
		}
	    }
	    if (status) {
		if (status == X1f4_E4_ALLOC_ERROR) {
		    perror(argv[0]);
		} else {
		    fprintf(stderr, "%s: cannot parse `%s'\n", argv[0],
			    expression);
		    fprintf(stderr, "%s: ", argv[0]);
		    x1f4_stat_expression(stderr, test_cast, &eerecord, NULL);
		    fprintf(stderr, "\n");
		}
	    } else {
		if (do_optimize) {
		    x1f4_line_expression(x1f4_expression);
		}

		if (do_print) {
		    x1f4_print_expression
			(stdout, x1f4_expression, print_flags);
		    putchar('\n');
		} else {
		    struct lead_type output;

		    status = x1f4_link_expression(x1f4_expression, &output);
		    if (status) {
			fprintf(stderr, "%s: cannot evaluate `%s'\n", argv[0],
				expression);
		    } else {
			libx1f4i0_type_data
			    (x1f4_type_expression(x1f4_expression), precision,
			     &output);
		    }
		}

		x1f4_fini_expression(&x1f4_expression);
	    }

	    if (e4fine) {
		x1f4_fini_e4fine(&e4fine);
	    }

#if defined HAVE_LIBx1f4l0
	    if (valist) {
		libx1f4i0_fini_valist(argv[0], valist, 0, 0, &status);
	    }
#endif				/* HAVE_LIBx1f4l0 */

	    libx1f4i0_fini_list(argv[0], (void *) &variable_data);
	} while (0);
    }

    return status;
}
