/*
 * ex16.c
 * Copyright (C) 2006-2013, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if defined HAVE_FEATURES_H
# include <features.h>
#endif				/* HAVE_FEATURES_H */

#include <getopt.h>
#if defined HAVE_LIBx1f4i0
# include <libx1f4i0.h>
#endif				/* HAVE_LIBx1f4i0 */
#if defined HAVE_LIBx1f4l0
# include <libx1f4l0.h>
#endif				/* HAVE_LIBx1f4l0 */
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <aime.h>
#if !defined HAVE_LIBx1f4i0
# include <cardinal-wx.h>
# include <float.n.h>
# include <float.x.h>
# include <float1.h>
#endif				/* !HAVE_LIBx1f4i0 */
#include <inter.h>
#if !defined HAVE_LIBx1f4i0
# include <lcardinal-x.h>
# include <lcardinal.n-d.h>
# include <lcardinal.p-d.h>
# include <lintegral-x.h>
# include <lintegral.n-d.h>
# include <lintegral.p-d.h>
# include <real.h>
#endif				/* !HAVE_LIBx1f4i0 */
#include <sf.h>
#include <types.h>

#define I_BILL(t)			(*((X1f4_E4_C_BILL *) (t)))
#define I_MODE(t)			(*((X1f4_E4_C_MODE *) (t)))
#define I_REAL(t)			(*((X1f4_E4_C_REAL *) (t)))
#define I_TEXT(t)			(*((X1f4_E4_C_TEXT *) (t)))

#define l_BILL(e, output) \
    {									      \
	X1f4_E4_C_BILL *l;						      \
									      \
	l = (output);							      \
	*l = (e);							      \
    }
#define l_MODE(e, output) \
    {									      \
	X1f4_E4_C_MODE *l;						      \
									      \
	l = (output);							      \
	*l = (e);							      \
    }
#define l_REAL(e, output) \
    {									      \
	X1f4_E4_C_REAL *l;						      \
									      \
	l = (output);							      \
	*l = (e);							      \
    }
#define l_TEXT(e, output) \
    {									      \
	X1f4_E4_C_TEXT *l;						      \
									      \
	l = (output);							      \
	*l = (e);							      \
    }

typedef struct context_type {
    struct x1f4_variable_type *variable_data;
    void **state;
} context_type;

typedef struct line_type {
    struct line_type *line_data;
} line_type;

typedef struct miss_type {
    int do_prefix, line;
#if defined HAVE_LIBx1f4l0
    int (*link) (void *, void **, unsigned);
#endif				/* HAVE_LIBx1f4l0 */
    struct line_type *line_data;
    unsigned precision;
    void *text;
} miss_type;

static int c16xx_b__(void *, void **);
static int c16xx_m__(void *, void **);
static int c16xx_r__(void *, void **);
static int fine_text(void *, void *, void **);
static int link_text(void *, char **, unsigned);
static int lock_composer(void *, int, int, const struct x1f4_linetext_type **);
static int lock_implicit(void *, int, int, const struct x1f4_operator_type **);
static int select_variable(const char *, unsigned, const void *,
			   const struct x1f4_variable_type **, void **);

static void usage(void);

static const int c_____b__[] = {
/* *INDENT-OFF* */
    X1f4_E4_BILL
/* *INDENT-ON* */
}, c_____m__[] = {
/* *INDENT-OFF* */
    X1f4_E4_MODE
/* *INDENT-ON* */
}, c_____r__[] = {
/* *INDENT-OFF* */
    X1f4_E4_REAL
/* *INDENT-ON* */
}, c_____t__[] = {
/* *INDENT-OFF* */
    X1f4_E4_TEXT
/* *INDENT-ON* */
};
static const struct x1f4_linetext_type e16a_set[] = {
/* *INDENT-OFF* */
    {	{	"(cardinal) ",		fine_text,
		X1f4_E4_BILL,		c_____t__,	1,
		X1f4_E4_TEXT_LINK,			11	},
	c_____b__							},
    {	{	"(integer) ",		fine_text,
		X1f4_E4_MODE,		c_____t__,	1,
		X1f4_E4_TEXT_LINK,			10	},
	c_____m__							},
    {	{	"(real) ",		fine_text,
		X1f4_E4_REAL,		c_____t__,	1,
		X1f4_E4_TEXT_LINK,			7	},
	c_____r__							},
    {	{	NULL,			NULL,
		0,			NULL,		0,
		0,					1	},
	NULL								}
/* *INDENT-ON* */
};
static const struct x1f4_operator_type e16xx_m__[] = {
/* *INDENT-OFF* */
    {	"(text) ",		c16xx_b__,	0000,
	X1f4_E4_TEXT,		c_____b__,
	0,			7,
	NULL,			NULL				},
    {	"(text) ",		c16xx_m__,	0000,
	X1f4_E4_TEXT,		c_____m__,
	0,			7,
	NULL,			NULL				},
    {	"(text) ",		c16xx_r__,	0000,
	X1f4_E4_TEXT,		c_____r__,
	0,			7,
	NULL,			NULL				},
    {	NULL,			NULL,		0000,
	X1f4_E4_VOID,		NULL,
	0,			0,
	NULL,			NULL				}
/* *INDENT-ON* */
};
static void *static_text;

static int
c16xx_b__(void *output, void **input)
{
    X1f4_E4_C_BILL mode;
    char *text;
    int status;
    struct miss_type *miss_data;
    unsigned size = 0;

    miss_data = static_text;

    mode = I_BILL(input[0]);

    x1f4_nprint_dlcardinal(&size, mode);

    if (miss_data->do_prefix) {
	size += 9;
    }

    status = link_text(static_text, &text, size + 1);
    if (status) {
    } else {
	l_TEXT(text, output);

	if (miss_data->do_prefix) {
	    memcpy(text, "cardinal:", 9);
	}

	text += size;
	*text = 0;
	x1f4_xprint_dlcardinal(&text, mode);
    }

    return status;
}


static int
c16xx_m__(void *output, void **input)
{
    X1f4_E4_C_MODE mode;
    char *text;
    int status;
    struct miss_type *miss_data;
    unsigned size = 0;

    miss_data = static_text;

    mode = I_MODE(input[0]);

    x1f4_nprint_dlintegral(&size, mode);

    if (miss_data->do_prefix) {
	size += 8;
    }

    status = link_text(static_text, &text, size + 1);
    if (status) {
    } else {
	l_TEXT(text, output);

	if (miss_data->do_prefix) {
	    memcpy(text, "integer:", 8);
	}

	text += size;
	*text = 0;
	x1f4_xprint_dlintegral(&text, mode);
    }

    return status;
}


static int
c16xx_r__(void *output, void **input)
{
    X1f4_E4_C_REAL real;
    char *text;
    int status;
    struct miss_type *miss_data;
    unsigned size = 0;

    miss_data = static_text;

    real = I_REAL(input[0]);

    x1f4_nprint_float(&size, miss_data->precision, real);

    if (miss_data->do_prefix) {
	size += 5;
    }

    status = link_text(static_text, &text, size + 1);
    if (status) {
    } else {
	l_TEXT(text, output);

	if (miss_data->do_prefix) {
	    memcpy(text, "real:", 5);
	    text += 5;
	}

	x1f4_xprint_float(text, miss_data->precision, real);
    }

    return status;
}


static int
fine_text(void *context, void *output, void **input)
{
    int status, type;

    type = *(const int *) context;
    if (type == X1f4_E4_BILL) {
	unsigned long bill;

	status = x1f4_parse_xlcardinal(&bill, I_TEXT(input[0]), NULL, 0);
	if (status) {
	} else {
	    l_BILL(bill, output);
	}
    } else {
	if (type == X1f4_E4_MODE) {
	    long mode;

	    status = x1f4_parse_xlintegral(&mode, I_TEXT(input[0]), NULL, 0);
	    if (status) {
	    } else {
		l_MODE(mode, output);
	    }
	} else {
	    double real;

	    status = x1f4_parse_real(&real, I_TEXT(input[0]), NULL);
	    if (status) {
	    } else {
		l_REAL(real, output);
	    }
	}
    }

    return status;
}


static int
link_text(void *context, char **text, unsigned size)
{
    char *copy;
    int status;
    struct miss_type *miss_data;

    miss_data = context;

    if (miss_data->line) {
#if defined HAVE_LIBx1f4l0
	if (miss_data->link) {
	    void *link;

	    status = miss_data->link(miss_data->text, &link, size);
	    if (!status) {
		*text = link;
	    }
	} else {
#endif				/* HAVE_LIBx1f4l0 */
	    copy = (char *) malloc(size);
	    if (!copy) {
		status = -1;
	    } else {
		status = 0;

		*text = copy;
	    }
#if defined HAVE_LIBx1f4l0
	}
#endif				/* HAVE_LIBx1f4l0 */
    } else {
	struct line_type *line_data;

	line_data = (struct line_type *)
	    malloc(sizeof(struct line_type) + size);
	if (!line_data) {
	    status = -1;
	} else {
	    status = 0;

	    copy = (char *) (line_data + 1);

	    *text = copy;

	    line_data->line_data = miss_data->line_data;
	    miss_data->line_data = line_data;
	}
    }

    return status;
}


static int
lock_composer(void *context, int outer, int inner,
	      const struct x1f4_linetext_type **linetext)
{
    int delete = 1;

    if (inner == X1f4_E4_TEXT) {
	const struct x1f4_linetext_type *linetext_data;

	linetext_data = e16a_set;
	while (linetext_data->function.name) {
	    if (linetext_data->function.type == outer) {
		*linetext = linetext_data;

		delete = 0;

		break;
	    }

	    linetext_data++;
	}
    }

    return delete;
}


static int
lock_implicit(void *context, int outer, int inner,
	      const struct x1f4_operator_type **operator)
{
    int delete = 1;

    if (outer == X1f4_E4_TEXT) {
	const struct x1f4_operator_type *operator_data;

	operator_data = e16xx_m__;
	while (operator_data->name) {
	    if (*operator_data->args == inner) {
		*operator = operator_data;

		delete = 0;

		break;
	    }

	    operator_data++;
	}
    }

    return delete;
}


static int
select_variable(const char *f, unsigned length, const void *context,
		const struct x1f4_variable_type **variable, void **state)
{
    int status = X1f4_E4_PARSE_ERROR;
    const struct context_type *context_data;
    const struct x1f4_variable_type *variable_data;

    context_data = context;
    variable_data = context_data->variable_data;
    if (variable_data) {
	while (variable_data->name) {
	    if (length == variable_data->length
		&& !memcmp((void *) f, variable_data->name, length)) {
		break;
	    }
	    variable_data++;
	}
	if (variable_data->name) {
	    status = 0;
	    *state = context_data->state
		[variable_data - context_data->variable_data];
	    *variable = variable_data;
	}
    }

    return status;
}


static void
usage(void)
{
    puts("Usage: ex16 [OPTIONS] EXPRESSION [TYPE NAME VALUE]\n\
Test application implicit conversions, evaluate EXPRESSION.\n\
\n\
Options:\n\
  -M, --stat-storage		stat expression memory storage requirements\n\
  -P, --print			print expression\n\
  -m, --stat-memory		stat memory operations\n\
  -o, --optimize		enable optimizations\n\
  -p, --precision DIGITS	set precision for printing reals to DIGITS\n\
  -t, --prefix			prefix text representation with original type\
\n\
      --detail-constants	detail constants when printing expression\n\
      --detail-operators	detail operators when printing expression\n\
      --help			display this help and exit\n\
      --version			output version information and exit");
}


int
main(int argc, char **argv)
{
    int do_memory = 0, do_optimize = 0, do_prefix = 0, do_print = 0,
	do_storage = 0, list_functions = 0, status = 1;
    unsigned precision = 3, print_flags = 0;

    unsetenv("POSIXLY_CORRECT");

    {
	char *precision_specs = NULL;

	while (1) {
	    char c;
	    static struct option long_options[] = {
/* *INDENT-OFF* */
		{   "detail-constants",
				    0x00,   NULL,   0x12    },
		{   "detail-operators",
				    0x00,   NULL,   0x11    },
		{   "help",	    0x00,   NULL,   0x68    },
		{   "optimize",	    0x00,   NULL,   0x6f    },
		{   "stat-memory",  0x00,   NULL,   0x6d    },
		{   "stat-storage", 0x00,   NULL,   0x4d    },
		{   "version",	    0x00,   NULL,   0x76    },
		{   "prefix",       0x00,   NULL,   0x74    },
		{   "precision",    0x01,   NULL,   0x70    },
		{   "print",	    0x00,   NULL,   0x50    },
		{   NULL,	    0x00,   NULL,   0x00    }
/* *INDENT-ON* */
	    };

	    c = getopt_long(argc, argv, "MPmop:t", long_options, NULL);

	    if (!~c) {
		break;
	    }

	    switch (c) {
	    case 021:
		print_flags |= X1f4_E4_DETAIL_OPERATORS;
		break;
	    case 022:
		print_flags |= X1f4_E4_DETAIL_CONSTANTS;
		break;
	    case 'M':
		do_storage = 1;
		break;
	    case 'P':
		do_print = 1;
		break;
	    case 'm':
		do_memory = 1;
		break;
	    case 'o':
		do_optimize = 1;
		break;
	    case 'p':
		precision_specs = optarg;
		break;
	    case 't':
		do_prefix = 1;
		break;
	    case 'h':
		usage();

		return 0;
	    case 'v':
		printf("%s (%s) %s\n", argv[0], PACKAGE, VERSION);

		return 0;
	    case '?':
		return 1;
	    }
	}

	if (list_functions) {
	    precision_specs = NULL;
	}

	if (do_print) {
	    precision_specs = NULL;
	}

	if (precision_specs) {
	    if (x1f4_parse_wxcardinal(&precision, precision_specs, NULL, 0)) {
		fprintf(stderr, "%s: cannot parse precision specification: `"
			"%s'\n", argv[0], precision_specs);

		return 1;
	    }
	}
    }

    if ((argc - optind) % 3 != 1) {
	fprintf(stderr, "%s: wrong number of arguments\nType `%s --help' for"
		" more information.\n", argv[0], argv[0]);
    } else {
	do {
	    const char *expression;
#if defined HAVE_LIBx1f4l0
	    struct list_type dlist;
#endif				/* HAVE_LIBx1f4l0 */
	    struct miss_type miss;
	    struct x1f4_variable_type *variable_data;
	    unsigned flags =
		X1f4_E4_COMPOSER | X1f4_E4_DDACCESS | X1f4_E4_IMPLICIT;
	    void **state = NULL, *x1f4_expression;
#if defined HAVE_LIBx1f4l0
	    void *valist = NULL;
#endif				/* HAVE_LIBx1f4l0 */

	    expression = argv[optind];
	    optind++;

	    static_text = &miss;

	    status = libx1f4i0_init_list
		(argv[0], argc, argv, optind, (void *) &variable_data, &state);
	    if (status) {
		break;
	    }

#if defined HAVE_LIBx1f4l0
	    libx1f4i0_init_valist(&valist, do_memory, do_storage);
#endif				/* HAVE_LIBx1f4l0 */

#if defined HAVE_LIBx1f4l0
	    miss.link = NULL;
#endif				/* HAVE_LIBx1f4l0 */

	    miss.line_data = NULL;

	    {
		if (0) {
		} else {
		    struct context_type context;
		    struct x1f4_attributes_type attributes;

		    context.state = state;
		    context.variable_data = variable_data;

		    attributes.composer_set.context = NULL;
		    attributes.composer_set.get = lock_composer;
		    attributes.function_set.get = libx1f4i0_select_function;
		    attributes.function_set.context = x1f4_e4_defaults;
		    attributes.implicit_set.context = NULL;
		    attributes.implicit_set.get = lock_implicit;
		    x1f4_llink_operator1s(&attributes.operator1s);
		    x1f4_llink_operator2s(&attributes.operator2s);
		    attributes.terminator = 0;
		    attributes.variable_set.get = select_variable;
		    attributes.variable_set.context = &context;

		    flags |= X1f4_E4_ESTORAGE;

#if defined HAVE_LIBx1f4l0
		    if (valist) {
			flags |= X1f4_E4_RESOURCE;
			libx1f4i0_line_valist
			    (valist, do_memory, do_storage,
			     &attributes.resource_set.free,
			     &attributes.resource_set.link,
			     &attributes.resource_set.mode,
			     &attributes.resource_set.context, &dlist);

			miss.link = attributes.resource_set.link;
			miss.text = attributes.resource_set.context;
		    }
#endif				/* HAVE_LIBx1f4l0 */

		    {
			status = x1f4_init_expression
			    (&x1f4_expression, expression, flags, &attributes);
		    }
		}
	    }
	    if (status) {
		if (status == X1f4_E4_ALLOC_ERROR) {
		    perror(argv[0]);
		} else {
		    fprintf(stderr, "%s: cannot parse `%s'\n", argv[0],
			    expression);
		}
	    } else {
		miss.line = 1;

		if (do_optimize) {
		    x1f4_line_expression(x1f4_expression);
		}

		miss.line = 0;

#if defined HAVE_LIBx1f4l0
		miss.link = NULL;
#endif				/* HAVE_LIBx1f4l0 */

#if defined HAVE_LIBx1f4l0
		if (valist) {
		    if (do_storage) {
			unsigned size;

			x1f4_size_xalist(valist, &size);
			fprintf(stderr, " %7u\n", size);
		    }
		}
#endif				/* HAVE_LIBx1f4l0 */

		if (do_print) {
		    x1f4_print_expression
			(stdout, x1f4_expression, print_flags);
		    putchar('\n');
		} else {
		    struct lead_type output;

		    miss.do_prefix = do_prefix;
		    miss.precision = precision;

		    status = x1f4_link_expression(x1f4_expression, &output);
		    if (status) {
			fprintf(stderr, "%s: cannot evaluate `%s'\n", argv[0],
				expression);
		    } else {
			libx1f4i0_type_data
			    (x1f4_type_expression(x1f4_expression), precision,
			     &output);
		    }
		}

		while (miss.line_data) {
		    struct line_type *line;

		    line = miss.line_data;
		    miss.line_data = line->line_data;
		    free(line);
		}

		x1f4_fini_expression(&x1f4_expression);
	    }

#if defined HAVE_LIBx1f4l0
	    if (valist) {
		libx1f4i0_fini_valist
		    (argv[0], valist, do_memory, do_storage, &status);
	    }
#endif				/* HAVE_LIBx1f4l0 */

	    libx1f4i0_fini_list(argv[0], (void *) &variable_data);
	} while (0);
    }

    return status;
}
