/*
 * tf24.c
 * Copyright (C) 2002-2015, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if defined HAVE_FEATURES_H
# include <features.h>
#endif				/* HAVE_FEATURES_H */

#include <getopt.h>
#include <signal.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/wait.h>

#include <aime.h>
#if !defined HAVE_LIBx1f4i0
# include <integral.v-d.h>
#endif				/* !HAVE_LIBx1f4i0 */
#include <inter.h>

#define DATA_BITS \
    X1f4_LXDATA_EVERLINK

#define DATE_BITS \
    X1f4_LXDATE_EVERLINK

#define DECQ_BITS \
    X1f4_LXDECQ_EVERLINK | X1f4_LXDECQ_RESETNEW

#define FILE_BITS \
    X1f4_LXFILE_EVERLINK | X1f4_LXFILE_SETCLOSE

#define LINE_BITS \
    X1f4_LXLINE_EVERLINK | X1f4_LXLINE_RESETNEW

#define LIST_BITS \
    X1f4_LXLIST_EVERLINK | X1f4_LXLIST_RESETNEW

#define PORT_BITS \
    X1f4_LXPORT_EVERLINK | X1f4_LXPORT_SETCLOSE

#define A113_BITS \
    X1f4_A1_BCOLLECT | X1f4_A1_COMPOSER | X1f4_A1_DDACCESS		      \
    | X1f4_A1_LEADCAST | X1f4_A1_OPTIMIZE | X1f4_A1_SCOMMENT		      \
    | X1f4_A1_SIDELIST | X1f4_A1_TRANSFER | X1f4_A1_TYPELIST

#define I_FLAT(i)			(*((X1f4_E4_C_USER *) (i)))
#define I_MODE(i)			(*((X1f4_E4_C_MODE *) (i)))

typedef struct context_type {
    char **argv;
    const char *program, *self;
    int argc, class, error, exit, immediate,
	(*line) (struct context_type *, unsigned *);
    struct x1f4_indexset_type *indexset_data;
    struct x1f4_linetext_type *linetext_data;
    unsigned call;
    void *logic;
    const void *data;
} context_type;

static int call_text(void *, void *, void **);
static int copy_error(void *, const char *, unsigned);
static int copy_output(void *, const char *, unsigned);
static int dumb_mode(void *, void *, void **);
static int ever_list(void *, void *, struct x1f4_variable_type *);
static int fini_xset(struct context_type *);
static int flat_text(void *);
static int flush_error(struct context_type *, const char *);
static int init_xset(struct context_type *);
static int line_flat(void *, int);
static int list_text(void *, const void *);
static int mode_argc(void *, void *, void **);
static int pick_shuffle(struct context_type *, unsigned *);
static int pick_text(void *, void *, void **);
static int post_flat(void *);
static int push_flat(void *, const char *, unsigned);
static int rule_fset(struct context_type *, const struct x1f4_function_type *);
static int rule_lset(struct context_type *, const struct x1f4_function_type *,
		     struct x1f4_cxflat_type *, struct x1f4_linetext_type *,
		     struct x1f4_dxflat_type *, char **);
static int run_logic(struct context_type *, int, char **);
static int seek_line(struct context_type *, unsigned, unsigned *);
static int select_context(const void *, const struct x1f4_function_type *,
			  void **);
static int select_logic(const char *, unsigned, const void *,
			const struct x1f4_function_type **);
static int set_composer(struct context_type *, void **,
			int (**) (void *, int, int,
				  const struct x1f4_linetext_type **));
static int set_logic(struct context_type *, const char *, const void *);
static int stat_argv(void *, int);
static int test_cast(void *, const char *, unsigned);
static int text_argv(void *, void *, void **);
static int type_fsdeck(struct context_type *);
static int wait_some(struct context_type *);

static void SIGCHLD_handler(int);
static void list_0010(struct context_type *);
static void list_0220(struct context_type *);
static void usage(void);

static int *static_class;
static const int c_____m__[] = {
/* *INDENT-OFF* */
    X1f4_E4_MODE
/* *INDENT-ON* */
};
static sig_atomic_t SIGCHLD_received;
static struct x1f4_cxflat_type types[] = {
/* *INDENT-OFF* */
    {	"X0",			0,		2,		},
    {	"X1",			0,		2,		},
    {	"X2",			0,		2,		},
    {	"X3",			0,		2,		},
    {	"X4",			0,		2,		},
    {	"X5",			0,		2,		},
    {	"X6",			0,		2,		},
    {	"X7",			0,		2,		}
/* *INDENT-ON* */
};
static struct x1f4_dxflat_type null_flat[] = {
/* *INDENT-OFF* */
    {	1,			NULL,				}
/* *INDENT-ON* */
};
/*
 * function names assumed below 3 characters long
 */
static const struct x1f4_function_type tf24_net[] = {
/* *INDENT-OFF* */
    {	"iX?",			pick_text,
	X1f4_E4_VOID,		c_____m__,	1,
	X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,	3		},
    {	"jX?",			call_text,
	X1f4_E4_MODE,		NULL,		1,
	X1f4_E4_KEEP_CALL,			3		},
    {	NULL,			NULL,
	0,			NULL,		0,
	0,					1		}
/* *INDENT-ON* */
}, tf24_set[] = {
/* *INDENT-OFF* */
    {	"argc",			mode_argc,
	X1f4_E4_MODE,		NULL,		0,
	X1f4_E4_KEEP_CALL,			4		},
    {	"argv",			text_argv,
	X1f4_E4_TEXT,		c_____m__,	1,
	X1f4_E4_KEEP_CALL,			4		},
    {	"exit",			dumb_mode,
	X1f4_E4_VOID,		c_____m__,	1,
	X1f4_E4_KEEP_CALL,			4		},
    {	NULL,			NULL,
	0,			NULL,		0,
	0,					1		}
/* *INDENT-ON* */
};
static struct x1f4_lxflat_type lxflat;
static struct x1f4_lxnear_type lxnear;
const static struct x1f4_lxtile_type tiles[] = {
/* *INDENT-OFF* */
    {	x1f4_note_lxcast,	0,		NULL		},
    {	x1f4_note_lxdata,	DATA_BITS,	NULL		},
    {	x1f4_note_lxdate,	DATE_BITS,	NULL		},
    {	x1f4_note_lxdecq,	DECQ_BITS,	NULL		},
    {	x1f4_note_lxfile,	FILE_BITS,	NULL		},
    {	x1f4_note_lxflat,	0,		&lxflat		},
    {	x1f4_note_lxline,	LINE_BITS,	NULL		},
    {	x1f4_note_lxlist,	LIST_BITS,	NULL		},
    {	x1f4_note_lxnear,	0,		&lxnear		},
    {	x1f4_note_lxport,	PORT_BITS,	NULL		},
    {	x1f4_note_lxtext,	0,		NULL		},
    {	x1f4_note_lxtrap,	0,		NULL		}
/* *INDENT-ON* */
};
static const struct x1f4_textpipe_type near_pipes[] = {
/* *INDENT-OFF* */
    {	"o",		copy_output,		1,	NULL	},
    {	"v",		copy_error,		1,	NULL	}
/* *INDENT-ON* */
};
static void *static_context;

static int
call_text(void *context, void *output, void **input)
{
    X1f4_E4_C_MODE *l;
    int *data;

    l = (void *) (output);
    data = ((struct x1f4_dxflat_type *) I_FLAT(input[0]))->data;
    *l = data ? *data : -1;

    return 0;
}


static int
copy_error(void *none, const char *data, unsigned size)
{
    fwrite(data, size, 1, stderr);

    return 0;
}


static int
copy_output(void *none, const char *data, unsigned size)
{
    fwrite(data, size, 1, stdout);

    return 0;
}


static int
dumb_mode(void *context, void *output, void **input)
{
    ((struct context_type *) context)->exit = I_MODE(input[0]) << 1 | 1;

    return 1;
}


static int
ever_list(void *none, void *data, struct x1f4_variable_type *variable_data)
{
    puts(variable_data->name);

    return 0;
}


static int
fini_xset(struct context_type *context_data)
{
    free(context_data->linetext_data);

    return x1f4_flat_indexset(context_data->indexset_data);
}


static int
flat_text(void *context)
{
    int status;
    struct context_type *context_data;
    struct x1f4_indexset_type *indexset_data;

    context_data = context;

    indexset_data = context_data->indexset_data;

    status = indexset_data->autodeck_set.deck
	(&indexset_data->autodeck_set.text);

    if (SIGCHLD_received) {
	int excess;

	excess = wait_some(context_data);
	if (excess) {
	    if (status) {
	    } else {
		status = excess;
	    }
	}
    }

    return status;
}


static int
flush_error(struct context_type *context_data, const char *program)
{
    if (context_data->error) {
    } else {
	fflush(stdout);
	fprintf(stderr, "%s:", context_data->self);
	if (program) {
	    fprintf(stderr, " %s:", program);
	}
	if (1) {
	    int (*line) (struct context_type *, unsigned *);

	    line = context_data->line;
	    if (line) {
		unsigned copy, miss;

		line(context_data, &copy);
		seek_line(context_data, copy, &miss);
		fprintf(stderr, " %u:", miss + 1);
	    }
	}

	fprintf(stderr, " cannot execute program\n");
    }

    return 17;
}


static int
init_xset(struct context_type *context_data)
{
    int status;
    struct x1f4_textport_type textport;
    unsigned textport_bits = 0;

    textport_bits = X1f4_TEXTPORT_CASTTYPE | X1f4_TEXTPORT_TEXTFLAT;

    textport.lasttype_set.type = X1f4_E4_LAST;

    textport.tilelong_set.lxtile_data = tiles;
    textport.tilelong_set.miss =
	sizeof(tiles) / sizeof(struct x1f4_lxtile_type);

    lxflat.datatype_set.cxflat_data = types;
    lxflat.datatype_set.miss = 8;

    lxnear.nearpipe_set.textpipe_data = near_pipes;
    lxnear.nearpipe_set.miss = 2;

    textport.textflat_set.data = context_data;
    textport.textflat_set.line = line_flat;
    textport.textflat_set.post = post_flat;
    textport.textflat_set.push = push_flat;

    textport.autodeck_set.class = &context_data->class;

    status = x1f4_fast_indexset
	(context_data->indexset_data, textport_bits, &textport);
    if (status) {
    } else {
	status = type_fsdeck(context_data);
	if (status) {
	    x1f4_flat_indexset(context_data->indexset_data);
	} else {
	}
    }

    return status;
}


static int
line_flat(void *context, int class)
{
    struct context_type *context_data;

    context_data = context;

    fflush(stdout);
    fputs(context_data->self, stderr);
    fputs(":", stderr);
    if (context_data->program) {
	fputs(" ", stderr);
	fputs(context_data->program, stderr);
	fputs(":", stderr);
    }
    if (1) {
	int (*line) (struct context_type *, unsigned *);

	line = context_data->line;
	if (line) {
	    unsigned copy, miss;

	    line(context, &copy);
	    seek_line(context, copy, &miss);
	    fprintf(stderr, " %u:", miss + 1);
	}
    }

    fputs(" ", stderr);

    return 0;
}


static int
list_text(void *context, const void *text)
{
    x1f4_print_htfunction
	(stdout, text, "", ((struct context_type *) context)->indexset_data,
	 x1f4_look_indexset);

    return 0;
}


static int
mode_argc(void *context, void *output, void **input)
{
    X1f4_E4_C_MODE *l;

    l = (void *) (output);
    *l = ((struct context_type *) static_context)->argc;

    return 0;
}


static int
pick_shuffle(struct context_type *context_data, unsigned *copy)
{
    return x1f4_seek_shuffle(context_data->logic, copy);
}


static int
pick_text(void *context, void *output, void **input)
{
    X1f4_E4_C_MODE mode;
    X1f4_E4_C_USER *l;

    mode = I_MODE(input[0]);

    l = (void *) (output);
    if (mode < 0) {
	if (mode < -8) {
	    *l = null_flat;
	} else {
	    *l = ((struct x1f4_dxflat_type *) context) + 8 + mode;
	}
    } else {
	if (mode < 8) {
	    *l = ((struct x1f4_dxflat_type *) context) + mode;
	} else {
	    *l = null_flat;
	}
    }

    return 0;
}


static int
post_flat(void *context)
{
    ((struct context_type *) context)->error = 1;

    fputs("\n", stderr);

    return 0;
}


static int
push_flat(void *context, const char *data, unsigned size)
{
    fwrite(data, size, 1, stderr);

    return 0;
}


static int
rule_fset(struct context_type *context_data,
	  const struct x1f4_function_type *function_data)
{
    const char *name;
    int status;

    name = function_data->name;
    if (name) {
	void *fsdeck;

	fsdeck = context_data->indexset_data->function_set.text;

	do {
	    status = x1f4_post_mxpath
		(fsdeck, name, function_data->length, function_data);
	    if (status) {
		break;
	    } else {
		function_data++;
	    }

	    name = function_data->name;
	} while (name);
    } else {
	status = 0;
    }

    return status;
}


static int
rule_lset(struct context_type *context_data,
	  const struct x1f4_function_type *function_data,
	  struct x1f4_cxflat_type *cxflat_data,
	  struct x1f4_linetext_type *linetext_data,
	  struct x1f4_dxflat_type *dxflat_data, char **text)
{
    const char *name;
    int status;

    name = function_data->name;
    if (name) {
	char *line;
	void *fsdeck;

	fsdeck = context_data->indexset_data->function_set.text;

	line = *text;

	do {
	    unsigned length;

	    linetext_data->function = *function_data;

	    linetext_data->context = dxflat_data;

	    length = function_data->length;

	    memcpy(line, name, length + 1);
	    line[length - 1] = cxflat_data->name[1];

	    linetext_data->function.name = line;

	    line += length + 1;

	    if (function_data->type == X1f4_E4_VOID) {
		linetext_data->function.type = cxflat_data->type;
	    } else {
		linetext_data->function.args = &cxflat_data->type;
	    }

	    status = x1f4_post_mxpath
		(fsdeck, linetext_data->function.name, length,
		 &linetext_data->function);
	    if (status) {
		break;
	    } else {
		function_data++;
	    }

	    linetext_data++;

	    name = function_data->name;
	} while (name);

	*text = line;
    } else {
	status = 0;
    }

    return status;
}


static int
run_logic(struct context_type *context_data, int argc, char **argv)
{
    int status;
    struct x1f4_indexset_type indexset;

    context_data->indexset_data = &indexset;

    status = init_xset(context_data);
    if (status) {
    } else {
	const char *program;

	program = argv[optind];
	optind++;

	SIGCHLD_received = 0;

	signal(SIGCHLD, SIGCHLD_handler);

	if (context_data->immediate) {
	    if (0) {
	    } else {
		context_data->data = program;

		context_data->program = NULL;

		status = set_logic(context_data, NULL, program);
	    }
	} else {
	    unsigned size;
	    void *data;

	    status = libx1f4i0_read_file(&data, &size, program, 1);
	    if (status) {
		libx1f4i0_stat_failure(argv[0], status, program);
	    } else {
		if (1) {
		    char *mine;

		    mine = data;
		    mine[size] = 0;
		}

		context_data->data = data;

		context_data->program = program;

		status = set_logic(context_data, program, data);

		free(data);
	    }
	}

	signal(SIGCHLD, SIG_DFL);

	if (1) {
	    int excess;

	    excess = fini_xset(context_data);
	    if (excess) {
		if (status) {
		} else {
		    status = excess;
		}
	    }
	}
    }

    return status;
}


static int
seek_line(struct context_type *context_data, unsigned copy, unsigned *miss)
{
    const char *data;
    unsigned line = 0;

    data = context_data->data;
    while (copy) {
	if (*data == 10) {
	    line++;
	}

	copy--;
	data++;
    }

    *miss = line;

    return 0;
}


static int
select_context(const void *context, const struct x1f4_function_type *miss,
	       void **text)
{
    *text = (void *)
	((const struct x1f4_linetext_type *)
	 ((const char *) miss
	  - offsetof(struct x1f4_linetext_type, function)))->context;

    return 0;
}


static int
select_logic(const char *f, unsigned length, const void *context,
	     const struct x1f4_function_type **function)
{
    int status;
    struct context_type *context_data;
    const void *mind;

    context_data = (void *) context;

    status = x1f4_find_mxpath
	(context_data->indexset_data->function_set.text, f, length, &mind);
    if (status) {
	status = 0;

	*function = mind;
    } else {
	status = 1;
    }

    return status;
}


static int
set_composer(struct context_type *context_data, void **text,
	     int (**lock) (void *, int, int,
			   const struct x1f4_linetext_type **))
{
    struct x1f4_lxslip_type *lxslip_data, *lxslip_text;

    lxslip_text = context_data->indexset_data->sliplong_set.lxslip_data;

    lxslip_data = lxslip_text;
    while (lxslip_data->note != x1f4_note_lxcast) {
	lxslip_data++;
    }

    *lock = x1f4_lock_lxcast;

    *text = lxslip_data->slip;

    while (lxslip_text->note != x1f4_note_lxtrap) {
	lxslip_text++;
    }

    x1f4_qfix_lxtrap(lxslip_text->slip, lxslip_data->slip, x1f4_lock_lxcast);

    return 0;
}


static int
set_logic(struct context_type *context_data, const char *program,
	  const void *data)
{
    int status;
    struct x1f4_a1_type a1;
    struct x1f4_a1record_type a1record;
    void *proGram;

    a1.autolead_set.context = context_data->indexset_data->autodeck_set.less;
    a1.autolead_set.link = x1f4_link_e4lf;
    a1.autolead_set.pick = x1f4_pick_e4lf;
    a1.autolink_set = context_data->indexset_data->autolink_set;
    a1.bcollect_set.a1record_data = &a1record;
    set_composer(context_data, &a1.composer_set.context, &a1.composer_set.get);
    a1.datatype_set.miss =
	context_data->indexset_data->datatype_set.datatype_data;
    a1.function_set.fix = select_context;
    a1.function_set.get = select_logic;
    a1.function_set.context = context_data;
    x1f4_llink_operator1s(&a1.operator1s);
    a1.operator2s = context_data->indexset_data->operator_set.operator2s;
    x1f4_pset_lxcast
	((void *) a1.composer_set.context, (void *) &a1.sidetype_set.miss);
    a1.transfer_set.fine = context_data->indexset_data->transfer_set.fine;
    a1.variable_set.context = context_data->indexset_data->variable_set.text;

    status = x1f4_init_shuffle(&proGram, data, A113_BITS, &a1);
    if (status) {
	if (status == X1f4_A1_ALLOC_ERROR) {
	    perror(context_data->self);
	} else {
	    struct x1f4_eelookup_type eelookup;

	    fprintf(stderr, "%s: cannot parse ", context_data->self);
	    if (program) {
		fprintf(stderr, "`%s'", program);
	    } else {
		fprintf(stderr, "program");
	    }
	    fprintf(stderr, "\n");
	    fprintf(stderr, "%s: ", context_data->self);
	    eelookup.type_l.context = context_data->indexset_data;
	    eelookup.type_l.fix = x1f4_look_indexset;
	    eelookup.type_q.e4fine =
		context_data->indexset_data->transfer_set.fine;
	    a1record.pick = 1;
	    x1f4_stat_shuffle(stderr, test_cast, data, &a1record, &eelookup);
	    fprintf(stderr, "\n");
	}
    } else {
	status = x1f4_pipe_e4ll
	    (context_data->indexset_data->autodeck_set.less, proGram);
	if (status) {
	    perror(context_data->self);
	} else {
	    X1f4_E4_C_MODE degree;

	    context_data->class = 0;
	    context_data->error = 0;

	    context_data->exit = 0;

	    context_data->line = pick_shuffle;

	    context_data->logic = proGram;

	    static_class = &context_data->class;

	    status = x1f4_long_shuffle
		(proGram, &context_data->class, context_data,
		 flat_text, &degree);
	    if (status) {
		if (context_data->exit) {
		    status = context_data->exit >> 1;
		} else {
		    flush_error(context_data, program);
		}
	    } else {
		status = degree;
	    }
	}

	x1f4_fini_shuffle(&proGram);
    }

    return status;
}


static int
stat_argv(void *context, int index)
{
    line_flat(context, -1);
    push_flat(context, "out of range argv index: ", 25);
    x1f4_vprint_dintegral(context, push_flat, index);
    post_flat(context);

    return -1;
}


static int
test_cast(void *cast, const char *name, unsigned size)
{
    return size ^ fwrite(name, 1, size, cast);
}


static int
text_argv(void *context, void *output, void **input)
{
    X1f4_E4_C_MODE index;
    int status = 0;
    struct context_type *context_data;

    context_data = static_context;

    index = I_MODE(input[0]);
    if (index < 0 || context_data->argc < index + 1) {
	status = stat_argv(static_context, index);
    } else {
	X1f4_E4_C_TEXT *l;

	l = (void *) (output);
	if (index) {
	    *l = context_data->argv[index];
	} else {
	    char *program;

	    program = (char *) context_data->program;

	    *l = program ? program : (char *) x1f4_c1_empty_string;
	}
    }

    return status;
}


static int
type_fsdeck(struct context_type *context_data)
{
    struct x1f4_linetext_type *linetext_data;

    rule_fset(context_data, tf24_set);

    linetext_data = malloc
	((sizeof(struct x1f4_linetext_type) << 4)
	 + (sizeof(struct x1f4_dxflat_type) << 6) + (sizeof(int) << 6)
	 + ((2 * (3 + 1)) << 4));
    if (linetext_data) {
	char *text;
	int *data;
	struct x1f4_cxflat_type *cxflat_data;
	struct x1f4_dxflat_type *dxflat_data;
	unsigned i = 8;

	context_data->linetext_data = linetext_data;

	dxflat_data = (void *) (linetext_data + 16);

	data = (void *) (dxflat_data + 64);

	text = (void *) (data + 64);

	cxflat_data = types;
	while (i) {
	    unsigned j;

	    i--;

	    rule_lset
		(context_data, tf24_net, cxflat_data, linetext_data,
		 dxflat_data, &text);
	    linetext_data += 2;
	    cxflat_data++;

	    j = 8;
	    while (j) {
		j--;
		dxflat_data->call = 1;
		dxflat_data->data = data;
		*data++ = i << 3 | j;
		dxflat_data++;
	    }
	}
    }

    return 0;
}


static int
wait_some(struct context_type *context_data)
{
    int status;
    struct x1f4_lxslip_type *lxslip_data;
    void *lxport;

    SIGCHLD_received = 0;

    lxslip_data = context_data->indexset_data->sliplong_set.lxslip_data;
    while (lxslip_data->note != x1f4_note_lxport) {
	lxslip_data++;
    }

    lxport = lxslip_data->slip;

    while (1) {
	pid_t wait;

	wait = waitpid(-1, NULL, WNOHANG);
	if (wait == -1) {
	    status = 0;
	    if (1) {
		break;
	    }
	} else {
	    if (wait) {
		x1f4_wait_lxport(lxport, wait);
	    } else {
		status = 0;
		if (1) {
		    break;
		}
	    }
	}
    }

    return status;
}


static void
SIGCHLD_handler(int signal)
{
    SIGCHLD_received = 1;
    *static_class = 1;
}


static void
list_0010(struct context_type *context_data)
{
    struct x1f4_indexset_type indexset;

    context_data->indexset_data = &indexset;

    context_data->argv = (void *) context_data;

    if (init_xset(context_data)) {
    } else {
	x1f4_lime_mxdeck(indexset.function_set.text, context_data, list_text);

	fini_xset(context_data);
    }
}


static void
list_0220(struct context_type *context_data)
{
    struct x1f4_indexset_type indexset;

    context_data->indexset_data = &indexset;

    context_data->argv = (void *) context_data;

    if (init_xset(context_data)) {
    } else {
	x1f4_list_state(indexset.variable_set.text, NULL, ever_list);

	fini_xset(context_data);
    }
}


static void
usage(void)
{
    puts("Usage: tf24 [OPTIONS] PROGRAM\n\
Execute PROGRAM.\n\
\n\
Options:\n\
  -c				execute the PROGRAM program instead program\n\
				read from the PROGRAM file\n\
      --data			list defined constants and exit\n\
      --list			list available functions and exit\n\
      --help			display this help and exit\n\
      --version			output version information and exit");
}


int
main(int argc, char **argv)
{
    int list_functions = 0, status = 0;
    struct context_type context;

    context.immediate = 0;

    unsetenv("POSIXLY_CORRECT");

    {
	int fast = ~0;

	while (1) {
	    char c;
	    static struct option long_options[] = {
/* *INDENT-OFF* */
		{   "data",	    0x00,   NULL,   0x02    },
		{   "help",	    0x00,   NULL,   0x68    },
		{   "list",	    0x00,   NULL,   0x01    },
		{   "print",	    0x00,   NULL,   0x50    },
		{   "version",	    0x00,   NULL,   0x76    },
		{   NULL,	    0x00,   NULL,   0x00    }
/* *INDENT-ON* */
	    };

	    c = getopt_long(argc, argv, "c", long_options, NULL);

	    if (!~c) {
		break;
	    }

	    switch (c) {
	    case 001:
		list_functions = 1;
		break;
	    case 002:
		list_functions = 2;
		break;
	    case 'c':
		context.immediate = 1;
		break;
	    case 'h':
	    case 'v':
		if (!~fast) {
		    fast = c;
		}
		break;
	    case '?':
		return 1;
	    }
	}

	if (~fast) {
	    switch (fast) {
	    case 'h':
		usage();
		break;
	    case 'v':
		printf("%s (%s) %s\n", argv[0], PACKAGE, VERSION);
	    }

	    return 0;
	}
    }

    context.argc = argc - optind;
    context.argv = argv + optind;

    if (list_functions) {
	if (list_functions == 1) {
	    list_0010(&context);
	} else {
	    if (list_functions == 2) {
		list_0220(&context);
	    }
	}
    } else {
	if (context.argv ? !(argc - optind) : (argc - optind) % 3 != 1) {
	    fprintf(stderr, "%s: wrong number of arguments\nTry `%s --help' "
		    "for more information.\n", argv[0], argv[0]);

	    return 1;
	} else {
	    context.self = argv[0];

	    static_context = &context;

	    status = run_logic(&context, argc, argv);
	}
    }

    return status;
}
