/*
 * channel29.c
 * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, Ciprian Niculescu
 * Copyright (C) 2013, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#if defined HAVE_FEATURES_H
# include <features.h>
#endif				/* HAVE_FEATURES_H */

#include <errno.h>
#if defined HAVE_LIBx1f4i0
# include <libx1f4i0.h>
#endif				/* HAVE_LIBx1f4i0 */
#include <getopt.h>
#include <netdb.h>
#include <signal.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <arpa/inet.h>

#include <netinet/in.h>

#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/un.h>
#include <sys/utsname.h>

#if !defined HAVE_LIBx1f4i0
# include <cardinal-w.h>
# include <cardinal-wx.h>
# include <cardinal-x.h>
# include <cardinal.h>
# include <ckeyhook.h>
# include <ckeylist.h>
# include <ckeytree.h>
# include <integral-x.h>
# include <ordinal.h>
# include <real.h>
# include <vkeyhook.h>
# include <vkeytree.h>
#endif				/* !HAVE_LIBx1f4i0 */

#define DIVA				0x28
#define INSERT				0x37
#define INSERT_c			0x5d
#define INSERT_s			0x5c
#define INSTANT				0x7c
#define QUEUE				0x22

#define true(e)				1

typedef struct c067_type {
    char *data, *link;
    int delete;
    const void *post;
} c067_type;

typedef struct e067_type {
    char *data;
    int delete;
} e067_type;

typedef struct e3e8_type {
    const char *self;
    int connection, do_synchronize;
    void *data;
    const void *link;
} e3e8_type;

typedef struct link_type {
    int (*link) (void *), (*list) (void *, const char *, unsigned),
	(*type) (void *, int, char **);
    unsigned command;
} link_type;

typedef struct mail_type {
    char *text;
    unsigned major, minor;
} mail_type;

typedef struct miss_type {
    char *data;
} miss_type;

typedef struct mode_type {
    int append;
    void *file;
} mode_type;

typedef struct pair_type {
    char *major, *minor;
} pair_type;

typedef struct s067_type {
    const void *post;
} s067_type;

typedef union data_type {
    struct c067_type order;
    struct e067_type passe;
    struct mail_type patch;
    struct miss_type press;
    struct pair_type class;
    struct s067_type trans;
} data_type;

static int case_inet(void *, int, void *);
static int case_unix(void *, int, void *);
static int case_v1f4(void *, const void *);
static int case_v1f5(void *, const void *);
static int case_v1f6(void *, const void *);
static int case_v1f7(void *, const void *);
static int case_x1f4(void *, int);
static int case_x3e8(void *, int, void *);
static int fast_dana(void *);
static int link_c042(void *);
static int link_c067(void *);
static int link_e050(void *);
static int link_e067(void *);
static int link_e174(void *);
static int link_e200(void *);
static int list_e200(void *, const char *, unsigned);
static int list_s042(void *, const char *, unsigned);
static int list_s067(void *, const char *, unsigned);
static int list_link(void *, const void *, const char *, unsigned);
static int list_mode(void *, const char *, unsigned);
static int read_dana(void *);
static int read_data(void *);
static int read_text(void *);
static int slip_data(void *, unsigned, const void *);
static int stat_read(void *, int);
static int type_c042(void *, int, char **);
static int type_c067(void *, int, char **);
static int type_e042(void *, int, char **);
static int type_e067(void *, int, char **);
static int type_e200(void *, int, char **);
static int type_s042(void *, int, char **);
static int type_s067(void *, int, char **);
static int type_s134(void *, int, char **);
static int type_s135(void *, int, char **);
static int type_v067(void *, int, char **);

static void SIGPIPE_handler(int);
static void stat_count(const char *);
static void stat_value(const char *, unsigned, const char *);
static void usage(void);

static sig_atomic_t SIGPIPE_received;
static const struct link_type link_u042 = {
/* *INDENT-OFF* */
    link_c042,
    NULL,
    NULL,
    0
/* *INDENT-ON* */
}, link_u067 = {
/* *INDENT-OFF* */
    link_c067,
    NULL,
    NULL,
    0
/* *INDENT-ON* */
}, link_v042 = {
/* *INDENT-OFF* */
    link_e050,
    list_s042,
    type_s042,
    QUEUE
/* *INDENT-ON* */
}, link_v067 = {
/* *INDENT-OFF* */
    link_e067,
    list_s067,
    type_v067,
    INSERT
/* *INDENT-ON* */
}, link_v134 = {
/* *INDENT-OFF* */
    link_e067,
    NULL,
    type_s134,
    INSERT_s
/* *INDENT-ON* */
}, link_v135 = {
/* *INDENT-OFF* */
    link_e067,
    NULL,
    type_s135,
    INSERT_c
/* *INDENT-ON* */
}, link_v174 = {
/* *INDENT-OFF* */
    link_e174,
    list_e200,
    type_e200,
    INSTANT
/* *INDENT-ON* */
};
static const struct x1f4_ckeytree_type scope_root[] = {
/* *INDENT-OFF* */
    {	"play",		{	0		},	NULL		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, swing_0[] = {
/* *INDENT-OFF* */
    {	"class",	{	1		},	NULL		},
    {	"start",	{	0		},	NULL		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
}, swing_root[] = {
/* *INDENT-OFF* */
    {	"",		{	0		},	swing_0		},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
};
static const struct x1f4_vkeytree_type link_1_a[] = {
/* *INDENT-OFF* */
    {	"ert",		{	&link_v067	},	NULL		},
    {	"tant",		{	&link_v174	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_c[] = {
/* *INDENT-OFF* */
    {	"eue",		{	&link_v042	},	NULL		},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_0_0[] = {
/* *INDENT-OFF* */
    {	"ins",		{	NULL		},	link_1_a	},
    {	"qu",		{	NULL		},	link_0_c	},
    {	NULL,		{	NULL		},	NULL		}
/* *INDENT-ON* */
}, link_root[] = {
/* *INDENT-OFF* */
    {	"",		{	NULL		},	link_0_0	},
    {	NULL,		{	0		},	NULL		}
/* *INDENT-ON* */
};

static int
case_inet(void *context, int connection, void *argv_1_)
{
    int status = 1;
    struct hostent *hostinfo;
    struct sockaddr_in address;
    struct utsname info;

    address.sin_family = AF_INET;
    address.sin_port = htons(*(unsigned *) argv_1_);
    uname(&info);
    if (!(hostinfo = gethostbyname(info.nodename))) {
	fprintf(stderr, "%s: cannot get host name: %s\n",
		((struct e3e8_type *) context)->self, hstrerror(h_errno));
    } else {
	address.sin_addr = *(struct in_addr *) hostinfo->h_addr;
	status = connect
	    (connection, (struct sockaddr *) &address, sizeof(address));
	if (status) {
	    fprintf(stderr, "%s: cannot connect: %s\n",
		    ((struct e3e8_type *) context)->self, strerror(errno));
	} else {
	    status = case_x1f4(context, connection);
	}
    }

    return status;
}


static int
case_unix(void *context, int connection, void *argv_1_)
{
    int status = 1;
    struct sockaddr_un address;

    memset(&address, 0, sizeof(struct sockaddr_un));
    address.sun_family = AF_UNIX;
    strncpy(address.sun_path, argv_1_, sizeof(address.sun_path) - 1);
    if (1) {
	status = connect
	    (connection, (struct sockaddr *) &address, sizeof(address));
	if (status) {
	    fprintf(stderr, "%s: cannot connect: %s\n",
		    ((struct e3e8_type *) context)->self, strerror(errno));
	} else {
	    status = case_x1f4(context, connection);
	}
    }

    return status;
}


static int
case_v1f4(void *context, const void *argv_3_)
{
    unsigned char c;

    c = ((struct link_type *) argv_3_)->command;

    return slip_data(context, 1, &c);
}


static int
case_v1f5(void *context, const void *argv_3_)
{
    int aime, sign = 0;
    unsigned miss;
    unsigned char *lift, size[4];

    aime = *(int *) argv_3_;
    if (aime < 0) {
	sign = 64;
	aime = -aime;
    }
    lift = size + 3;
    miss = aime;
    if (!(miss & ~63)) {
	*lift = miss | sign;
    } else {
	*lift = miss & 127;
	miss >>= 7;
	miss &= (1 << 20) - 1;
	while (miss & ~63) {
	    lift--;
	    *lift = 128 | (miss & 127);
	    miss >>= 7;
	}
	lift--;
	*lift = 128 | miss | sign;
    }

    return slip_data(context, size + 4 - lift, lift);
}


static int
case_v1f6(void *context, const void *argv_3_)
{
    int status;
    unsigned aime, miss;
    unsigned char *lift, size[4];

    aime = strlen(argv_3_);
    lift = size + 3;
    miss = aime;
    if (!(miss & ~127)) {
	*lift = miss;
    } else {
	*lift = miss & 127;
	lift--;
	miss >>= 7;
	*lift = 128 | (miss & 127);
	miss >>= 7;
	while (miss) {
	    lift--;
	    *lift = 128 | (miss & 127);
	    miss >>= 7;
	}
    }

    status = slip_data(context, size + 4 - lift, lift);
    if (!status) {
	status = slip_data(context, aime, argv_3_);
    }

    return status;
}


static int
case_v1f7(void *context, const void *argv_3_)
{
    unsigned aime, miss;
    unsigned char *lift, size[4];

    aime = *(unsigned *) argv_3_;
    lift = size + 3;
    miss = aime;
    if (!(miss & ~127)) {
	*lift = miss;
    } else {
	*lift = miss & 127;
	lift--;
	miss >>= 7;
	*lift = 128 | (miss & 127);
	miss >>= 7;
	miss &= (1 << 14) - 1;
	while (miss) {
	    lift--;
	    *lift = 128 | (miss & 127);
	    miss >>= 7;
	}
    }

    return slip_data(context, size + 4 - lift, lift);
}


static int
case_x1f4(void *context, int connection)
{
    const char *argv_0_;
    int status;

    argv_0_ = ((struct e3e8_type *) context)->self;

    status = slip_data(context, 16, "slippingaway\0\0\0\1");
    if (status) {
    } else {
	status =
	    ((struct link_type *)
	     ((struct e3e8_type *) context)->link)->link(context);
	if (status) {
	} else {
	    status = fast_dana(context);
	    if (status) {
	    } else {
		status = read_text(context);
	    }
	}
    }

    return status;
}


static int
case_x3e8(void *context, int domain, void *argv_1_)
{
    int connection, status;

    SIGPIPE_received = 0;

    signal(SIGPIPE, SIGPIPE_handler);

    connection = socket(domain, SOCK_STREAM, 0);
    if (connection == -1) {
	status = 1;
	fprintf(stderr, "%s: cannot socket: %s\n",
		((struct e3e8_type *) context)->self, strerror(errno));
    } else {
	((struct e3e8_type *) context)->connection = connection;

	if (domain == AF_UNIX) {
	    status = case_unix(context, connection, argv_1_);
	} else {
	    status = case_inet(context, connection, argv_1_);
	}

	close(connection);
    }

    signal(SIGPIPE, SIG_DFL);

    return status;
}


static int
fast_dana(void *context)
{
    int status;

    if (((struct e3e8_type *) context)->do_synchronize) {
	status = slip_data(context, 1, "\177");
    } else {
	status = 0;
    }

    return status;
}


static int
link_c042(void *context)
{
    int status;
    struct link_type link_v050;
    const struct link_type *link_data;
    union data_type *class, data;

    link_data = ((struct e3e8_type *) context)->link;

    class = ((struct e3e8_type *) context)->data;

    ((struct e3e8_type *) context)->data = &data;

    ((struct e3e8_type *) context)->link = &link_v050;

    link_v050.command = DIVA;

    data.press.data = class->class.major;

    status = link_e050(context);
    if (!status) {
	((struct e3e8_type *) context)->link = &link_v042;

	data.press.data = class->class.minor;

	status = link_e050(context);
	if (!status) {
	}
    }

    ((struct e3e8_type *) context)->data = class;

    ((struct e3e8_type *) context)->link = link_data;

    return status;
}


static int
link_c067(void *context)
{
    int status;
    struct link_type link_v050;
    const struct link_type *link_data;
    union data_type *class, data;

    link_data = ((struct e3e8_type *) context)->link;

    class = ((struct e3e8_type *) context)->data;

    ((struct e3e8_type *) context)->data = &data;

    ((struct e3e8_type *) context)->link = &link_v050;

    link_v050.command = DIVA;

    data.press.data = class->order.link;

    status = link_e050(context);
    if (!status) {
	((struct e3e8_type *) context)->link = class->order.post;

	data.passe.data = class->order.data;
	data.passe.delete = class->order.delete;

	status = link_e067(context);
	if (!status) {
	}
    }

    ((struct e3e8_type *) context)->data = class;

    ((struct e3e8_type *) context)->link = link_data;

    return status;
}


static int
link_e050(void *context)
{
    int status;

    status = case_v1f4(context, ((struct e3e8_type *) context)->link);
    if (!status) {
	status = case_v1f6
	    (context,
	     ((union data_type *)
	      ((struct e3e8_type *) context)->data)->press.data);
    }

    return status;
}


static int
link_e067(void *context)
{
    int status;

    status = case_v1f4(context, ((struct e3e8_type *) context)->link);
    if (!status) {
	union data_type *class;

	class = ((struct e3e8_type *) context)->data;
	status = case_v1f5(context, &class->passe.delete);
	if (!status) {
	    status = case_v1f6(context, class->passe.data);
	}
    }

    return status;
}


static int
link_e174(void *context)
{
    int status;

    status = link_e200(context);
    if (status) {
    } else {
	((struct e3e8_type *) context)->do_synchronize = 1;
    }

    return status;
}


static int
link_e200(void *context)
{
    int status;

    status = case_v1f4(context, ((struct e3e8_type *) context)->link);
    if (status) {
    } else {
	union data_type *class;

	class = ((struct e3e8_type *) context)->data;
	status = case_v1f7(context, &class->patch.major);
	if (status) {
	} else {
	    status = case_v1f7(context, &class->patch.minor);
	    if (status) {
	    } else {
		status = case_v1f6(context, class->patch.text);
	    }
	}
    }

    return status;
}


static int
list_e200(void *file, const char *data, unsigned size)
{
    fputs(" <cardinal> <cardinal> <string>", file);

    return 0;
}


static int
list_s042(void *file, const char *data, unsigned size)
{
    fputs(" <string>\n", file);
    fwrite(data, size, 1, file);
    fputs(" <string> <string>", file);

    return 0;
}


static int
list_s067(void *file, const char *data, unsigned size)
{
    struct mode_type mode;

    mode.file = file;

    fputs(" <integral> <string>\n", file);
    fwrite(data, size, 1, file);
    fputs(" <integral> <string> <string>\n", file);
    fwrite(data, size, 1, file);
    mode.append = 0;
    x1f4_list_ctkey(&mode, list_mode, scope_root);
    mode.append = 0;
    x1f4_list_ctkey(&mode, list_mode, swing_root);
    fputs(" <integral> <string>\n", file);
    fwrite(data, size, 1, file);
    mode.append = 0;
    x1f4_list_ctkey(&mode, list_mode, scope_root);
    mode.append = 0;
    x1f4_list_ctkey(&mode, list_mode, swing_root);
    fputs(" <integral> <string> <string>", file);

    return 0;
}


static int
list_link(void *file, const void *node, const char *data, unsigned size)
{
    fwrite(data, size, 1, file);
    ((const struct link_type *) node)->list(file, data, size);
    fputc('\n', file);

    return 0;
}


static int
list_mode(void *mode, const char *data, unsigned size)
{
    struct mode_type *mode_data;
    void *file;

    mode_data = mode;
    file = mode_data->file;
    if (mode_data->append) {
	fputc('|', file);
    } else {
	fputc(' ', file);
	mode_data->append = 1;
    }
    fwrite(data, size, 1, file);

    return 0;
}


static int
read_dana(void *context)
{
    int connection, status;

    connection = ((struct e3e8_type *) context)->connection;

    if (1) {
	unsigned char c;

	status = read(connection, &c, 1);
	if (status != 1) {
	    status = stat_read(context, status);
	} else {
	    if (c == 127) {
		status = 0;
	    } else {
		fprintf(stderr, "%s: bad Mylene1f4 transmission received whi"
			"le waiting for synchonization reply\n",
			((struct e3e8_type *) context)->self);
	    }
	}
    }

    return status;
}


static int
read_data(void *context)
{
    int connection, data = 0, status;
    unsigned size = 0;

    connection = ((struct e3e8_type *) context)->connection;

    while (1) {
	unsigned char c;

	status = read(connection, &c, 1);
	if (status != 1) {
	    status = stat_read(context, status);

	    break;
	}

	if (data) {
	    putchar(c);
	    size--;
	    if (size) {
	    } else {
		status = 0;
		break;
	    }
	} else {
	    if (c & 128) {
		c &= ~128;
	    } else {
		data = 1;
		if (c) {
		} else {
		    if (size) {
		    } else {
			status = 0;
			break;
		    }
		}
	    }

	    size <<= 7;
	    size += c;
	}
    }

    return status;
}


static int
read_text(void *context)
{
    int connection, status;

    connection = ((struct e3e8_type *) context)->connection;

    while (1) {
	unsigned char c;

	status = read(connection, &c, 1);
	if (status != 1) {
	    status = stat_read(context, status);

	    break;
	} else {
	    if (c == 1) {
		status = read_data(context);
		if (status) {
		    break;
		} else {
		    fflush(stdout);
		}
	    } else {
		if (c) {
		} else {
		    if (((struct e3e8_type *) context)->do_synchronize) {
			status = read_dana(context);
			if (1) {
			    break;
			}
		    }
		}

		status = 1;

		fprintf(stderr, "%s: unexpected Mylene1f4 transmission (x%02"
			"x) received while waiting for channel marker\n",
			((struct e3e8_type *) context)->self, (unsigned) c);

		break;
	    }
	}
    }

    return status;
}


static int
slip_data(void *context, unsigned size, const void *data)
{
    int connection, status;

    connection = ((struct e3e8_type *) context)->connection;

    status = write(connection, data, size);
    if (status != (int) size) {
	status = 1;
	fprintf(stderr, "%s: cannot write: %s\n",
		((struct e3e8_type *) context)->self, strerror(errno));
	if (SIGPIPE_received) {
	    fprintf(stderr, "%s: Mylene1f4 connection shut down\n",
		    ((struct e3e8_type *) context)->self);
	}
    } else {
	status = 0;
    }

    return status;
}


static int
stat_read(void *context, int status)
{
    if (!status) {
	fprintf(stderr, "%s: connection closed\n",
		((struct e3e8_type *) context)->self);
    } else {
	fprintf(stderr, "%s: cannot read connection: %s\n",
		((struct e3e8_type *) context)->self, strerror(errno));
    }

    return 1;
}


static int
type_c042(void *context, int argc, char **argv)
{
    int status;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    if (0) {
    } else {
	union data_type *class;

	status = 0;

	class = mylene3e8->data;

	class->class.major = argv[0];
	class->class.minor = argv[1];
    }

    return status;
}


static int
type_c067(void *context, int argc, char **argv)
{
    int status;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    if (argc != 3) {
	stat_count(mylene3e8->self);
	status = 1;
    } else {
	union data_type *class;
	const void *post;

	class = mylene3e8->data;

	post = class->trans.post;

	status = x1f4_parse_xintegral(&class->order.delete, argv[0], NULL, 0);
	if (status) {
	    stat_value(mylene3e8->self, 0, argv[0]);
	} else {
	    class->order.data = argv[2];
	    class->order.link = argv[1];
	    class->order.post = post;
	}
    }

    return status;
}


static int
type_e042(void *context, int argc, char **argv)
{
    int status;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    if (argc != 1) {
	stat_count(mylene3e8->self);
	status = 1;
    } else {
	status = 0;
	((union data_type *) mylene3e8->data)->press.data = argv[0];
    }

    return status;
}


static int
type_e067(void *context, int argc, char **argv)
{
    int status;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    if (argc != 2) {
	stat_count(mylene3e8->self);
	status = 1;
    } else {
	union data_type *class;

	class = mylene3e8->data;

	status = x1f4_parse_xintegral(&class->passe.delete, argv[0], NULL, 0);
	if (status) {
	    stat_value(mylene3e8->self, 0, argv[0]);
	} else {
	    class->passe.data = argv[1];
	}
    }

    return status;
}


static int
type_e200(void *context, int argc, char **argv)
{
    int status;
    struct e3e8_type *mylene3e8;

    mylene3e8 = context;

    if (argc != 3) {
	stat_count(mylene3e8->self);
	status = 1;
    } else {
	union data_type *class;

	class = mylene3e8->data;

	status = x1f4_parse_wxcardinal(&class->patch.major, argv[0], NULL, 0);
	if (status) {
	    stat_value(mylene3e8->self, 0, argv[0]);
	} else {
	    status = x1f4_parse_wxcardinal
		(&class->patch.minor, argv[1], NULL, 0);
	    if (status) {
		stat_value(mylene3e8->self, 1, argv[1]);
	    } else {
		class->patch.text = argv[2];
	    }
	}
    }

    return status;
}


static int
type_s042(void *context, int argc, char **argv)
{
    int status;

    if (argc != 2) {
	status = type_e042(context, argc, argv);
    } else {
	((struct e3e8_type *) context)->link = &link_u042;

	status = type_c042(context, argc, argv);
    }

    return status;
}


static int
type_s067(void *context, int argc, char **argv)
{
    int status;

    if (argc != 3) {
	status = type_e067(context, argc, argv);
    } else {
	((struct e3e8_type *) context)->link = &link_u067;

	((union data_type *)
	 ((struct e3e8_type *) context)->data)->trans.post = &link_v067;

	status = type_c067(context, argc, argv);
    }

    return status;
}


static int
type_s134(void *context, int argc, char **argv)
{
    int status;

    if (argc != 3) {
	status = type_e067(context, argc, argv);
    } else {
	((struct e3e8_type *) context)->link = &link_u067;

	((union data_type *)
	 ((struct e3e8_type *) context)->data)->trans.post = &link_v134;

	status = type_c067(context, argc, argv);
    }

    return status;
}


static int
type_s135(void *context, int argc, char **argv)
{
    int status;

    if (argc != 3) {
	status = type_e067(context, argc, argv);
    } else {
	((struct e3e8_type *) context)->link = &link_u067;

	((union data_type *)
	 ((struct e3e8_type *) context)->data)->trans.post = &link_v135;

	status = type_c067(context, argc, argv);
    }

    return status;
}


static int
type_v067(void *context, int argc, char **argv)
{
    int status;

    if (argc < 4) {
	status = type_s067(context, argc, argv);
    } else {
	struct e3e8_type *mylene3e8;
	unsigned scope;

	mylene3e8 = context;

	status = x1f4_find_ctkey(argv[0], scope_root, &scope);
	if (status) {
	    stat_value(mylene3e8->self, 0, argv[0]);
	} else {
	    unsigned swing;

	    status = x1f4_find_ctkey(argv[1], swing_root, &swing);
	    if (status) {
		stat_value(mylene3e8->self, 1, argv[1]);
	    } else {
		if (!swing) {
		    mylene3e8->link = &link_v134;
		} else {
		    mylene3e8->link = &link_v135;
		}

		status = ((struct link_type *) mylene3e8->link)->type
		    (context, argc - 2, argv + 2);
	    }
	}
    }

    return status;
}


static void
SIGPIPE_handler(int none)
{
    SIGPIPE_received = 1;
}


static void
stat_count(const char *self)
{
    fprintf(stderr, "%s: wrong number of arguments\nType `%s --help' for mor"
	    "e information.\n", self, self);
}


static void
stat_value(const char *self, unsigned rank, const char *line)
{
    char suffix[2];

    x1f4_extend_ordinal(suffix, rank);
    fprintf(stderr, "%s: cannot parse command %u%.*s argument specification:"
	    " `%s'\n", self, rank + 1, 2, suffix, line);
}


static void
usage(void)
{
    puts("Usage: channel29 [OPTIONS] PATH|PORT COMMAND [ARGUMENT]...\n\
Control Mylene1f4 server.\n\
\n\
Options:\n\
  -i, --inet			connect to INET PORT server\n\
  -u, --unix			connect to UNIX PATH server\n\
  -l, --list			list available commands and exit\n\
      --help			display this help and exit\n\
      --version			output version information and exit");
}


int
main(int argc, char **argv)
{
    int do_list = 0, hi_unix = 1, status = 1;
    struct e3e8_type e3e8;

    unsetenv("POSIXLY_CORRECT");

    while (1) {
	char c;
	static struct option long_options[] = {
/* *INDENT-OFF* */
	    {	"inet",		0x00,	NULL,	0x69	},
	    {	"list",		0x00,	NULL,	0x6c	},
	    {	"unix",		0x00,	NULL,	0x75	},
	    {	"version",	0x00,	NULL,	0x76	},
	    {	"help",		0x00,	NULL,	0x68	},
	    {	NULL,		0x00,	NULL,	0x00	}
/* *INDENT-ON* */
	};

	c = getopt_long(argc, argv, "ilu", long_options, NULL);

	if (!~c) {
	    break;
	}

	switch (c) {
	case 'i':
	    hi_unix = 0;
	    break;
	case 'l':
	    do_list = 1;
	    break;
	case 'u':
	    hi_unix = 1;
	    break;
	case 'h':
	    usage();

	    return 0;
	case 'v':
	    printf("%s (%s) %s\n", argv[0], PACKAGE, VERSION);

	    return 0;
	case '?':
	    return 1;
	}
    }

    if (do_list) {
	if (true(do_list == 1)) {
	    status = x1f4_hook_vtkey(stdout, list_link, link_root);
	} else {
	    status = 0;
	}
    } else {
	if (argc - optind < 2) {
	    fprintf(stderr, "%s: wrong number of arguments\nType `%s --help'"
		    " for more information.\n", argv[0], argv[0]);
	} else {
	    do {
		int domain;
		unsigned port;
		void *argv_1_;

		if (hi_unix) {
		    argv_1_ = argv[optind];
		    domain = AF_UNIX;
		} else {
		    domain = PF_INET;
		    argv_1_ = &port;
		    if (x1f4_parse_wxcardinal
			(argv_1_, argv[optind], NULL, 0)) {
			fprintf(stderr, "%s: cannot parse port specification"
				": `%s'\n", argv[0], argv[optind]);
			break;
		    } else if (65535 < port) {
			fprintf(stderr, "%s: port specification (`%u') is ou"
				"tside valid range (0 .. 65535)\n", argv[0],
				port);
			break;
		    }
		}

		e3e8.self = argv[0];

		{
		    const void *link;

		    if (x1f4_find_vtkey(argv[optind + 1], link_root, &link)) {
			fprintf(stderr, "%s: cannot parse command specificat"
				"ion: `%s'\n", argv[0], argv[optind + 1]);
		    } else {
			const struct link_type *link_data;
			union data_type data;

			e3e8.do_synchronize = 0;

			e3e8.data = &data;

			link_data = link;

			e3e8.link = link;

			optind += 2;

			status = link_data->type
			    (&e3e8, argc - optind, argv + optind);
			if (status) {
			} else {
			    status = case_x3e8(&e3e8, domain, argv_1_);
			}
		    }
		}
	    } while (0);
	}
    }

    return status;
}
