/*
 * x1f4i2.1.c
 * Copyright (C) 2005-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <config.h>

#include <unistd.h>

#include <sys/time.h>

#include <defs.h>
#include <types.h>

#define SECOND_A(v)			((v)->tv_sec)
#define SECOND_F(v)			((v)->tv_usec)

#if __REAL_ACCELERATION__ == 1
typedef double ACCELERATION;
#elif 7 < SIZEOF_UNSIGNED_LONG
typedef unsigned long ACCELERATION;
#else
typedef unsigned long long ACCELERATION;
#endif				/* __REAL_ACCELERATION__ == 1 */

extern int _aapple_audio_size(unsigned *);
extern int _aapple_audio_play(void *, unsigned char *, unsigned);
extern int _apple_audio_setup(void *);

int
x1f4_xlay_xaudio(void *xaudio, void *data, unsigned size, struct timeval *time)
{
    int excess, status;
    struct xaudio_type *xaudio_data;
    unsigned delay;

    xaudio_data = xaudio;

    if (!xaudio_data->started) {
	status = _apple_audio_setup(xaudio);
    } else {
	status = 0;
    }
    if (!status) {
	do {
	    excess = _aapple_audio_play(xaudio, data, size);
	    if (excess < 0) {
		status = excess;
		break;
	    } else {
//don't think we should be sleeping here
#if 0
		usleep((double) BLOCK_SIZE * 1000000 / xaudio_data->rate);
#endif

		if (excess) {
		    status += excess;

		    size -= excess;
		    if (!size) {
			break;
		    } else {
			data = (char *) data + excess;
		    }
		}
	    }
	} while (1);
    }
    if (status < 0) {
	status <<= SELF_SHIFT;
    } else {
	ACCELERATION advance;
	unsigned rate, second_a, second_f;

	_aapple_audio_size(&delay);

	advance = xaudio_data->size;
	advance -= delay;

	rate = xaudio_data->rate;

#if __REAL_ACCELERATION__ == 1
	advance /= rate;
	advance *= 1000000;
	advance += .5;

	second_a = advance / 1000000;
	second_f = advance - (double) second_a * 1000000;
	if (second_f < 1000000) {
	} else {
	    second_a++;
	    second_f -= 1000000;
	}
#else
	advance *= 2097152;
	advance += rate;
	advance /= rate << 1;

	second_a = advance >> 20;

	advance &= 1048575;
	advance *= 2000000;
	advance += 1048576;
	advance /= 2097152;

	second_f = advance;
#endif				/* __REAL_ACCELERATION__ == 1 */

	SECOND_A(time) = second_a;
	SECOND_F(time) = second_f;

	xaudio_data->size += status;
    }

    return status;
}
