/*
 * x1f4i2.5.c
 * Copyright (C) 2005-2008, 2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <poll.h>

#include <alsa/asoundlib.h>

#include <sys/time.h>

#include <defs.h>
#include <inter.h>
#include <trace.h>
#include <types.h>

#define SECOND_A(v)			((v)->tv_sec)
#define SECOND_F(v)			((v)->tv_usec)

#if __REAL_ACCELERATION__ == 1
typedef double ACCELERATION;
#elif 7 < SIZEOF_UNSIGNED_LONG
typedef unsigned long ACCELERATION;
#else
typedef unsigned long long ACCELERATION;
#endif				/* __REAL_ACCELERATION__ == 1 */

int
x1f4_qlay_xaudio(void *xaudio, void *data, unsigned size, struct timeval *time)
{
    int status;

    METHOD_ENTER();

    do {
	struct pollfd *dsound;
	struct xaudio_type *xaudio_data;
	unsigned copy, lock;
	void *asound;

	xaudio_data = xaudio;

	copy = xaudio_data->copy;
	lock = xaudio_data->lock;

	asound = xaudio_data->asound;
	dsound = xaudio_data->dsound;

	while (1) {
	    status = poll(dsound, lock, -1);
	    if (status < 0) {
		if (errno == EINTR) {
		} else {
		    status = _1f4_miss_xaudio(xaudio);
		    if (1) {
			break;
		    }
		}
	    } else {
		if (status) {
		    unsigned short plan;

		    status = snd_pcm_poll_descriptors_revents
			(asound, dsound, lock, &plan);
		    if (status) {
			status = _1f4_fail_xaudio(xaudio, status, REND_ERROR);
			if (1) {
			    break;
			}
		    }

		    if (plan & POLLOUT) {
			status = 0;
		    } else {
			if (plan) {
			    status = _1f4_fail_xaudio
				(xaudio, status, TRIP_ERROR);
			} else {
			    status = 0;
			    if (1) {
				asound = NULL;
			    }
			}
		    }

		    break;
		} else {
		    status = POLL_ERROR;
		    if (1) {
			break;
		    }
		}
	    }
	}

	if (status) {
	    break;
	} else {
	    if (asound) {
	    } else {
		break;
	    }
	}

	status = snd_pcm_writei(asound, data, size / copy);
	if (status < 0) {
	    if (status == -EAGAIN) {
		status = 0;
	    } else {
		status = _1f4_long_xaudio(xaudio, status);
		if (status) {
		    status = _1f4_fail_xaudio(xaudio, status, PUSH_ERROR);
		}
	    }

	    break;
	}

	if (0) {
	} else {
	    ACCELERATION advance;
	    snd_pcm_sframes_t frames;
	    unsigned delay, rate, second_a, second_f;

	    if (snd_pcm_delay(asound, &frames)) {
		delay = 0;
	    } else {
		delay = frames * xaudio_data->copy;
	    }

	    advance = xaudio_data->size;
	    advance -= delay;

	    rate = xaudio_data->rate;

#if __REAL_ACCELERATION__ == 1
	    advance /= rate;
	    advance *= 1000000;
	    advance += .5;

	    second_a = advance / 1000000;
	    second_f = advance - (double) second_a * 1000000;
	    if (second_f < 1000000) {
	    } else {
		second_a++;
		second_f -= 1000000;
	    }
#else
	    advance *= 2097152;
	    advance += rate;
	    advance /= rate << 1;

	    second_a = advance >> 20;

	    advance &= 1048575;
	    advance *= 2000000;
	    advance += 1048576;
	    advance /= 2097152;

	    second_f = advance;
#endif				/* __REAL_ACCELERATION__ == 1 */

	    SECOND_A(time) = second_a;
	    SECOND_F(time) = second_f;

	    status *= copy;

	    xaudio_data->size += status;
	}
    } while (0);

    METHOD_LEAVE(status);

    return status;
}
