/*
 * x1f4i2.a.c
 * Copyright (C) 2006-2008, 2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <alsa/asoundlib.h>

#include <sys/time.h>

#include <defs.h>
#include <inter.h>
#include <trace.h>
#include <types.h>

#define SECOND_A(v)			((v)->tv_sec)
#define SECOND_F(v)			((v)->tv_usec)

#if __REAL_ACCELERATION__ == 1
typedef double ACCELERATION;
#elif 7 < SIZEOF_UNSIGNED_LONG
typedef unsigned long ACCELERATION;
#else
typedef unsigned long long ACCELERATION;
#endif				/* __REAL_ACCELERATION__ == 1 */

int
x1f4_eset_xaudio(void *xaudio, struct timeval *time)
{
    int status;
    snd_pcm_sframes_t frames;
    struct xaudio_type *xaudio_data;

    METHOD_ENTER();

    xaudio_data = xaudio;

    status = snd_pcm_delay(xaudio_data->asound, &frames);
    if (status < 0) {
	status = snd_pcm_recover(xaudio_data->asound, status, 1);
	if (status) {
	} else {
	    status = snd_pcm_delay(xaudio_data->asound, &frames);
	}
    }

    if (status < 0) {
	status = _1f4_fail_xaudio(xaudio, status, DATE_ERROR);
    } else {
	ACCELERATION advance;
	unsigned rate, second_a, second_f;

	advance = xaudio_data->size;
	advance -= frames * xaudio_data->copy;

	status = 0;

	rate = xaudio_data->rate;

#if __REAL_ACCELERATION__ == 1
	advance /= rate;
	advance *= 1000000;
	advance += .5;

	second_a = advance / 1000000;
	second_f = advance - (double) second_a * 1000000;
	if (second_f < 1000000) {
	} else {
	    second_a++;
	    second_f -= 1000000;
	}
#else
	advance *= 2097152;
	advance += rate;
	advance /= rate << 1;

	second_a = advance >> 20;

	advance &= 1048575;
	advance *= 2000000;
	advance += 1048576;
	advance /= 2097152;

	second_f = advance;
#endif				/* __REAL_ACCELERATION__ == 1 */

	SECOND_A(time) = second_a;
	SECOND_F(time) = second_f;
    }

    METHOD_LEAVE(status);

    return status;
}
