/*
 * x1f4i2.b.c
 * Copyright (C) 2006-2007, 2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <poll.h>
#include <stdlib.h>

#include <alsa/asoundlib.h>

#include <defs.h>
#include <inter.h>
#include <trace.h>
#include <types.h>

static void
snd_lib_error_handler(const char *, int, const char *, int, const char *, ...);

static void
snd_lib_error_handler(const char *a, int b, const char *c, int d,
		      const char *e, ...)
{
}


int
x1f4_open_xaudio(void **xaudio, const char *dsp,
		 struct aerror_type *aerror_data)
{
    int status;
    struct xaudio_type *xaudio_data;

    METHOD_ENTER();

    xaudio_data = (struct xaudio_type *) malloc(sizeof(struct xaudio_type));
    if (!xaudio_data) {
	status = LINK_ERROR;
    } else {
	xaudio_data->xerror = aerror_data;

	status = snd_lib_error_set_handler(snd_lib_error_handler);
	if (status) {
	    status = _1f4_fail_xaudio(xaudio_data, status, HUSH_ERROR);
	} else {
	    snd_pcm_t *asound;

	    status = snd_pcm_open
		(&asound, dsp, SND_PCM_STREAM_PLAYBACK, SND_PCM_NONBLOCK);
	    if (status) {
		status = _1f4_fail_xaudio(xaudio_data, status, OPEN_ERROR);
	    } else {
		unsigned lock;
		void *dsound;

		lock = snd_pcm_poll_descriptors_count(asound);

		dsound = malloc(lock * sizeof(struct pollfd));
		if (!dsound) {
		    status = LINK_ERROR;
		} else {
		    if (0) {
		    } else {
			*xaudio = xaudio_data;

			xaudio_data->lock = lock;

			xaudio_data->asound = asound;
			xaudio_data->dsound = dsound;

			xaudio_data->missed = 0;

			snd_pcm_poll_descriptors(asound, dsound, lock);
		    }

		    if (status) {
			free(dsound);
		    }
		}

		if (status) {
		    snd_pcm_close(asound);
		}
	    }
	}

	if (status) {
	    free(xaudio_data);
	}
    }

    METHOD_LEAVE(status);

    return status;
}
