/*
 * x1f4i2.c.c
 * Copyright (C) 2005-2008, 2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>

#include <alsa/asoundlib.h>

#include <class.h>
#include <defs.h>
#include <inter.h>
#include <trace.h>
#include <types.h>

#define period_event			0

int
x1f4_mode_xaudio(void *xaudio, unsigned rate, unsigned channels,
		 unsigned format, unsigned flags)
{
    int status;
    snd_pcm_hw_params_t *sound1 = NULL;
    snd_pcm_sw_params_t *sound2 = NULL;
    struct xaudio_type *xaudio_data;

    METHOD_ENTER();

    xaudio_data = xaudio;

    do {
	int rid;
	snd_pcm_t *asound;
	snd_pcm_uframes_t frames;
	unsigned Rate, buffer = 500000, copy, period, sample;

	asound = xaudio_data->asound;

	status = snd_pcm_hw_params_malloc(&sound1);
        if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, ALSA_ERROR);
	    break;
	}

        status = snd_pcm_hw_params_any(asound, sound1);
        if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, ALSA_ERROR);
	    break;
	}

        status = snd_pcm_hw_params_set_rate_resample(asound, sound1, 1);
        if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, RESS_ERROR);
	    break;
	}

        status = snd_pcm_hw_params_set_access
	    (asound, sound1, SND_PCM_ACCESS_RW_INTERLEAVED);
        if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, PLAN_ERROR);
	    break;
	}

	FIX_CLASS(format, sample, format);

        status = snd_pcm_hw_params_set_format(asound, sound1, format);
        if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, TYPE_ERROR);
	    break;
	}

        status = snd_pcm_hw_params_set_channels(asound, sound1, channels);
        if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, UNIT_ERROR);
	    break;
	}

	Rate = rate;

        status = snd_pcm_hw_params_set_rate_near(asound, sound1, &Rate, 0);
        if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, ALSA_ERROR);
	    break;
	}
        if (Rate ^ rate) {
	    status = _1f4_fail_xaudio(xaudio, rate, RATE_ERROR);
	    if (1) {
		break;
	    }
	}

        status = snd_pcm_hw_params_set_buffer_time_near
	    (asound, sound1, &buffer, &rid);
        if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, ALSA_ERROR);
	    break;
	}

	period = buffer >> 3;

        status = snd_pcm_hw_params_get_buffer_size(sound1, &frames);
        if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, SIZE_ERROR);
	    break;
        }

	buffer = frames;

	copy = channels * sample;

	xaudio_data->buffer = frames * copy;

        status = snd_pcm_hw_params_set_period_time_near
	    (asound, sound1, &period, &rid);
        if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, NEAR_ERROR);
	    break;
        }

        status = snd_pcm_hw_params_get_period_size(sound1, &frames, &rid);
        if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, EVEN_ERROR);
	    break;
        }

	period = frames;

	xaudio_data->period = frames * copy;

        status = snd_pcm_hw_params(asound, sound1);
        if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, COPY_ERROR);
	    break;
        }

	status = snd_pcm_sw_params_malloc(&sound2);
        if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, ALSA_ERROR);
	    break;
	}

        status = snd_pcm_sw_params_current(asound, sound2);
        if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, ALSA_ERROR);
	    break;
        }

#if 0
        status = snd_pcm_sw_params_set_start_threshold
	    (asound, sound2, (buffer / period) * period);
        if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, ALSA_ERROR);
	    break;
        }

        status = snd_pcm_sw_params_set_avail_min
	    (asound, sound2, period_event ? buffer : period);
        if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, ALSA_ERROR);
	    break;
        }
#endif				/* 0 */

        if (period_event) {
	    status = snd_pcm_sw_params_set_period_event(asound, sound2, 1);
	    if (status) {
		status = _1f4_fail_xaudio(xaudio, status, ALSA_ERROR);
		break;
	    }
        }

        status = snd_pcm_sw_params(asound, sound2);
        if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, RULE_ERROR);
	    break;
        }

	xaudio_data->copy = copy;
	xaudio_data->rate = copy * rate;
	xaudio_data->size = 0;
    } while (0);

    if (sound2) {
	snd_pcm_sw_params_free(sound2);
    }

    if (sound1) {
	snd_pcm_hw_params_free(sound1);
    }

    METHOD_LEAVE(status);

    return status;
}
