/*
 * x1f4i2.h.c
 * Copyright (C) 2006-2007, 2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <errno.h>
#include <stdio.h>
#include <string.h>

#include <alsa/asoundlib.h>

#include <defs.h>
#include <types.h>

static void file_alsa(void *, struct aerror_type *);
static void file_miss(void *, struct aerror_type *);
static void file_rate(void *, struct aerror_type *);

static void
file_alsa(void *file, struct aerror_type *aerror_data)
{
    if (aerror_data) {
	fprintf(file, ": %s", snd_strerror(aerror_data->excess));
    }
}


static void
file_miss(void *file, struct aerror_type *aerror_data)
{
    if (aerror_data) {
	fprintf(file, "%s", strerror(aerror_data->excess));
    } else {
	fprintf(file, "%s", strerror(errno));
    }
}


static void
file_rate(void *file, struct aerror_type *aerror_data)
{
    if (aerror_data) {
	fprintf(file, " near %dHz", aerror_data->excess);
    }
}


int
x1f4_file_xaudio(void *file, int condition, const char *dsp,
		 struct aerror_type *aerror_data)
{
    switch (condition) {
    case ALSA_ERROR:
	fprintf(file, "alsa error");
	file_alsa(file, aerror_data);
	break;
    case COPY_ERROR:
	fprintf(file, "cannot configure alsa hard");
	file_alsa(file, aerror_data);
	break;
    case DATE_ERROR:
	fprintf(file, "cannot get stream time");
	file_alsa(file, aerror_data);
	break;
    case DROP_ERROR:
	fprintf(file, "cannot drop stream");
	file_alsa(file, aerror_data);
	break;
    case EVEN_ERROR:
	fprintf(file, "cannot get period time");
	file_alsa(file, aerror_data);
	break;
    case HUSH_ERROR:
	fprintf(file, "cannot hush crappy alsa");
	file_alsa(file, aerror_data);
	break;
    case LINK_ERROR:
	fprintf(file, "%s", strerror(ENOMEM));
	break;
    case LOCK_ERROR:
	fprintf(file, "cannot set nonblocking mode");
	file_alsa(file, aerror_data);
	break;
    case LOSE_ERROR:
	fprintf(file, "can't lose device %s", dsp);
	file_alsa(file, aerror_data);
	break;
    case MISS_ERROR:
	file_miss(file, aerror_data);
	break;
    case NEAR_ERROR:
	fprintf(file, "cannot set period time");
	file_alsa(file, aerror_data);
	break;
    case OPEN_ERROR:
	fprintf(file, "can't open device %s", dsp);
	file_alsa(file, aerror_data);
	break;
    case PLAN_ERROR:
	fprintf(file, "cannot set access mode");
	file_alsa(file, aerror_data);
	break;
    case POLL_ERROR:
	fprintf(file, "poll gave up before time");
	break;
    case PUSH_ERROR:
	fprintf(file, "cannot write data");
	file_alsa(file, aerror_data);
	break;
    case RATE_ERROR:
	fprintf(file, "cannot set sampling rate");
	file_rate(file, aerror_data);
	break;
    case REND_ERROR:
	fprintf(file, "cannot get transfer state");
	file_rate(file, aerror_data);
	break;
    case RESS_ERROR:
	fprintf(file, "cannot set resampling mode");
	file_alsa(file, aerror_data);
	break;
    case RULE_ERROR:
	fprintf(file, "cannot configure alsa soft");
	file_alsa(file, aerror_data);
	break;
    case SIZE_ERROR:
	fprintf(file, "cannot set buffer size");
	file_alsa(file, aerror_data);
	break;
    case TRIP_ERROR:
	fprintf(file, "unknown transfer state");
	file_rate(file, aerror_data);
	break;
    case TYPE_ERROR:
	fprintf(file, "cannot set data format");
	file_alsa(file, aerror_data);
	break;
    case UNIT_ERROR:
	fprintf(file, "cannot set channels count");
	file_alsa(file, aerror_data);
	break;
    case WAIT_ERROR:
	fprintf(file, "cannot wait stream");
	file_alsa(file, aerror_data);
	break;
    default:
	;
    }

    return -1;
}
