/*
 * x1f4i2.p.c
 * Copyright (C) 2006-2008, 2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <unistd.h>

#include <alsa/asoundlib.h>

#include <sys/time.h>

#include <defs.h>
#include <inter.h>
#include <trace.h>
#include <types.h>

#define SECOND_A(v)			((v)->tv_sec)
#define SECOND_F(v)			((v)->tv_usec)

#if __REAL_ACCELERATION__ == 1
typedef double ACCELERATION;
#elif 7 < SIZEOF_UNSIGNED_LONG
typedef unsigned long ACCELERATION;
#else
typedef unsigned long long ACCELERATION;
#endif				/* __REAL_ACCELERATION__ == 1 */

int
x1f4_slip_xaudio(void *xaudio, unsigned *loss, int now, struct timeval *time)
{
    int status;
    snd_pcm_t *asound;
    struct xaudio_type *xaudio_data;
    unsigned delay = 0;

    METHOD_ENTER();

    xaudio_data = xaudio;

    asound = xaudio_data->asound;

    if (now) {
	int excess;

	status = _1f4_mind_xaudio(xaudio, &delay);

	if (loss) {
	    *loss = delay;
	}

	excess = snd_pcm_drop(asound);
	if (excess) {
	    if (status) {
	    } else {
		status = _1f4_fail_xaudio(xaudio, status, DROP_ERROR);
	    }
	}
    } else {
#if 1
	status = snd_pcm_nonblock(asound, 0);
	if (status) {
	    status = _1f4_fail_xaudio(xaudio, status, LOCK_ERROR);
	} else {
	    status = snd_pcm_drain(asound);
	    if (status) {
		status = _1f4_fail_xaudio(xaudio, status, WAIT_ERROR);
	    } else {
		status = snd_pcm_nonblock(asound, 1);
		if (status) {
		    status = _1f4_fail_xaudio(xaudio, status, LOCK_ERROR);
		}
	    }
	}
#else
	status = 0;

	while (1) {
	    snd_pcm_state_t fail;

	    fail = snd_pcm_state(asound);
	    if (fail == SND_PCM_STATE_RUNNING) {
		usleep(16384);
	    } else {
		break;
	    }
	}
#endif				/* 0 */
    }
    if (status) {
    } else {
	ACCELERATION advance;
	unsigned rate, second_a, second_f;

	advance = xaudio_data->size;
	advance -= delay;

	rate = xaudio_data->rate;

#if __REAL_ACCELERATION__ == 1
	advance /= rate;
	advance *= 1000000;
	advance += .5;

	second_a = advance / 1000000;
	second_f = advance - (double) second_a * 1000000;
	if (second_f < 1000000) {
	} else {
	    second_a++;
	    second_f -= 1000000;
	}
#else
	advance *= 2097152;
	advance += rate;
	advance /= rate << 1;

	second_a = advance >> 20;

	advance &= 1048575;
	advance *= 2000000;
	advance += 1048576;
	advance /= 2097152;

	second_f = advance;
#endif				/* __REAL_ACCELERATION__ == 1 */

	SECOND_A(time) = second_a;
	SECOND_F(time) = second_f;
    }

    METHOD_LEAVE(status);

    return status;
}
