/*
 * xaudio.0.c
 * Copyright (C) 2005-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>

#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <class.h>
#include <defs.h>
#include <types.h>

#define OUTBURST			512

static int init_xaudio(struct xaudio_type *, const char *, unsigned, unsigned,
		       unsigned, unsigned);

static int
init_xaudio(struct xaudio_type *xaudio_data, const char *dsp, unsigned rate,
	    unsigned channels, unsigned format, unsigned flags)
{
    int fd, status;

#if defined __linux__
    fd = open(dsp, O_WRONLY | O_NONBLOCK);
#else
    fd = open(dsp, O_WRONLY);
#endif				/* __linux__ */
    if (fd == -1) {
	status = OPEN_ERROR;
    } else {
	do {
	    int whatever;

#if defined __linux__
	    if (fcntl(fd, F_SETFL, 0) < 0) {
		status = DSP0_ERROR;
		break;
	    }
#endif				/* __linux__ */

#if 0
	    if (ioctl(fd, SNDCTL_DSP_RESET, NULL) == -1) {
		status = DSPX_ERROR;
		break;
	    }
#endif				/* 0 */

#if defined __linux__
	    if (ioctl(fd, SNDCTL_DSP_NONBLOCK) == -1) {
		status = DSPB_ERROR;
		break;
	    }
#endif				/* __linux__ */

#if defined FD_CLOEXEC && defined F_SETFD
	    fcntl(fd, F_SETFD, FD_CLOEXEC);
#endif				/* FD_CLOEXEC && F_SETFD */

	    FIX_CLASS(whatever, format);
	    if (ioctl(fd, SNDCTL_DSP_SETFMT, &whatever) == -1) {
		status = DSPF_ERROR;
		break;
	    }

	    if (channels < 3) {
		whatever = channels - 1;
		if (ioctl(fd, SNDCTL_DSP_STEREO, &whatever) == -1) {
		    status = DSPS_ERROR;
		    break;
		}
	    } else {
		whatever = channels;
		if (ioctl(fd, SNDCTL_DSP_CHANNELS, &whatever) == -1) {
		    status = DSPC_ERROR;
		    break;
		}
	    }

	    whatever = rate;
	    if (ioctl(fd, SNDCTL_DSP_SPEED, &whatever) == -1) {
		status = DSPV_ERROR;
		break;
	    }

	    if (1) {
		struct audio_buf_info info;

		if (ioctl(fd, SNDCTL_DSP_GETOSPACE, &info) != -1) {
		    xaudio_data->bytes = info.fragstotal * info.fragsize;
		    xaudio_data->outburst = info.fragsize;
		} else {
		    if (ioctl(fd, SNDCTL_DSP_GETBLKSIZE, &whatever) == -1) {
			xaudio_data->outburst = OUTBURST;
		    } else {
			xaudio_data->outburst = whatever;
		    }

		    xaudio_data->bytes = xaudio_data->outburst;
		}
	    }

	    xaudio_data->sleep = (unsigned)
		(((double) xaudio_data->outburst * 1000000)
		 / (rate * channels << 1) + .5);

	    status = 0;
	} while (0);

	if (status) {
	    close(fd);
	} else {
	    xaudio_data->fd = fd;
	}
    }

    return status;
}


int
libxaudio_init_xaudio(void **xaudio, const char *dsp, unsigned rate,
		      unsigned channels, unsigned format, unsigned flags)
{
    int status;
    struct xaudio_type *xaudio_data;

    xaudio_data = (struct xaudio_type *) malloc(sizeof(struct xaudio_type));
    if (!xaudio_data) {
	status = MISS_ERROR;
    } else {
	status = init_xaudio(xaudio_data, dsp, rate, channels, format, flags);
	if (status) {
	    free(xaudio_data);
	} else {
	    *xaudio = xaudio_data;
	}
    }

    return status;
}
