/*
 * xaudio.4.c
 * Copyright (C) 2005-2007, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <unistd.h>

#include <sys/ioctl.h>
#include <sys/soundcard.h>

#include <defs.h>
#include <types.h>

#define __SYNC_FLUSH_FINI__		0
#define __WAIT_FLUSH_FINI__		1

#define __FLUSH_FINI__			__WAIT_FLUSH_FINI__

static int fini_xaudio(struct xaudio_type *, int);

static int
fini_xaudio(struct xaudio_type *xaudio_data, int now)
{
    int fd, status;

    fd = xaudio_data->fd;

    if (now) {
#if defined SNDCTL_DSP_RESET
	status = ioctl(fd, SNDCTL_DSP_RESET, NULL);
	if (status) {
	    status = DSPX_ERROR;
	}
# else
	status = 0;
#endif				/* SNDCTL_DSP_RESET */
    } else {
#if __FLUSH_FINI__ == __SYNC_FLUSH_FINI__
# if defined SNDCTL_DSP_SYNC
	status = ioctl(fd, SNDCTL_DSP_SYNC, NULL);
	if (status) {
	    status = DSPY_ERROR;
	}
# else
	status = 0;
# endif				/* SNDCTL_DSP_SYNC */
#else
	struct audio_buf_info info;

	if (ioctl(fd, SNDCTL_DSP_GETOSPACE, &info) == -1) {
# if defined SNDCTL_DSP_SYNC
	    status = ioctl(fd, SNDCTL_DSP_SYNC, NULL);
	    if (status) {
		status = DSPY_ERROR;
	    }
# else
	    status = 0;
# endif				/* SNDCTL_DSP_SYNC */
	} else {
# if 0
	    usleep(xaudio_data->sleep * (info.fragstotal - info.fragments));
# else
	    double sleep, space;

	    space = info.bytes;
	    space /= info.fragsize;
	    sleep = info.fragstotal;
	    sleep -= space;
	    sleep *= xaudio_data->sleep;

	    usleep(sleep);
# endif				/* 0 */
# if defined SNDCTL_DSP_RESET
	    status = ioctl(fd, SNDCTL_DSP_RESET, NULL);
	    if (status) {
		status = DSPX_ERROR;
	    }
# else
	    status = 0;
# endif				/* SNDCTL_DSP_RESET */
	}
#endif				/* __FLUSH_FINI__ == __SYNC_FLUSH_FINI__ */
    }

    {
	int excess;

	excess = close(fd);
	if (excess) {
	    if (!status) {
		status = LOSE_ERROR;
	    }
	}
    }

    return status;
}


int
libxaudio_fini_xaudio(void **xaudio, int now)
{
    int status;
    struct xaudio_type *xaudio_data;

    xaudio_data = *xaudio;

    status = fini_xaudio(xaudio_data, now);
    free(xaudio_data);

    return status;
}
