/*
 * xaudio.c.c
 * Copyright (C) 2006-2011, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>

#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <class.h>
#include <defs.h>
#include <types.h>

#define OUTBURST			512

static int mode_xaudio(struct xaudio_type *, unsigned, unsigned, unsigned,
		       unsigned);

static int
mode_xaudio(struct xaudio_type *xaudio_data, unsigned rate, unsigned channels,
	    unsigned format, unsigned flags)
{
    int status;

    do {
	int fd, whatever;

	fd = xaudio_data->fd;

	FIX_CLASS(whatever, format);
	if (ioctl(fd, SNDCTL_DSP_SETFMT, &whatever) == -1) {
	    status = DSPF_ERROR;
	    break;
	}

	if (channels < 3) {
	    whatever = channels - 1;
	    if (ioctl(fd, SNDCTL_DSP_STEREO, &whatever) == -1) {
		status = DSPS_ERROR;
		break;
	    }
	} else {
	    whatever = channels;
	    if (ioctl(fd, SNDCTL_DSP_CHANNELS, &whatever) == -1) {
		status = DSPC_ERROR;
		break;
	    }
	}

	whatever = rate;
	if (ioctl(fd, SNDCTL_DSP_SPEED, &whatever) == -1) {
	    status = DSPV_ERROR;
	    break;
	}

	if (1) {
	    struct audio_buf_info info;

	    if (ioctl(fd, SNDCTL_DSP_GETOSPACE, &info) != -1) {
		xaudio_data->bytes = info.fragstotal * info.fragsize;
		xaudio_data->outburst = info.fragsize;
	    } else {
		if (ioctl(fd, SNDCTL_DSP_GETBLKSIZE, &whatever) == -1) {
		    /*
		     * leave _outburst_ untouched (it was set before)?
		     */
		    xaudio_data->outburst = OUTBURST;
		} else {
		    xaudio_data->outburst = whatever;
		}

		xaudio_data->bytes = xaudio_data->outburst;
	    }
	}

	xaudio_data->sleep = (unsigned)
	    (((double) xaudio_data->outburst * 1000000)
	     / (rate * channels << 1) + .5);

	status = 0;
    } while (0);

    return status;
}


int
libxaudio_mode_xaudio(void *xaudio, unsigned rate, unsigned channels,
		      unsigned format, unsigned flags)
{
    int status;

    status = mode_xaudio(xaudio, rate, channels, format, flags);

    return status;
}
