/*
 * xaudio.h.c
 * Copyright (C) 2006-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <errno.h>
#include <stdio.h>
#include <string.h>

#include <defs.h>

int
libxaudio_file_xaudio(void *file, int condition, const char *dsp)
{
    switch (condition) {
#if defined __linux__
    case DSP0_ERROR:
	fprintf(file, "cannot set file status flags on %s: %s", dsp,
		strerror(errno));
	break;
#endif				/* __linux__ */
#if defined __linux__
    case DSPB_ERROR:
	fprintf(file, "cannot set %s in non blocking mode: %s", dsp,
		strerror(errno));
	break;
#endif				/* __linux__ */
    case DSPC_ERROR:
	fprintf(file, "cannot set %s channels: %s", dsp, strerror(errno));
	break;
    case DSPF_ERROR:
	fprintf(file, "cannot set %s format: %s", dsp, strerror(errno));
	break;
    case DSPO_ERROR:
	fprintf(file, "cannot get output space on %s: %s", dsp,
		strerror(errno));
	break;
    case DSPQ_ERROR:
	fprintf(file, "cannot get audio data classes for %s: %s", dsp,
		strerror(errno));
	break;
    case DSPS_ERROR:
	fprintf(file, "cannot set %s stereo: %s", dsp, strerror(errno));
	break;
    case DSPV_ERROR:
	fprintf(file, "cannot set %s speed: %s", dsp, strerror(errno));
	break;
    case DSPX_ERROR:
	fprintf(file, "cannot reset %s: %s", dsp, strerror(errno));
	break;
    case DSPY_ERROR:
	fprintf(file, "cannot synchronize %s: %s", dsp, strerror(errno));
	break;
    case LOSE_ERROR:
	fprintf(file, "cannot close %s: %s", dsp, strerror(errno));
	break;
    case MISS_ERROR:
	fprintf(file, "%s", strerror(errno));
	break;
    case OPEN_ERROR:
	fprintf(file, "cannot open %s: %s", dsp, strerror(errno));
	break;
    default:
	fprintf(file, "cannot write %s: %s", dsp, strerror(errno));
    }

    return -1;
}
