/*
 * cquery.c
 * Copyright (C) 2002-2008, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <config.h>

#if defined HAVE_FEATURES_H
# include <features.h>
#endif				/* HAVE_FEATURES_H */

#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libxaudio.h>

#define DEVICE				"/dev/dsp"

static int file_audio(const char *, int, const char *);

static void usage(void);

static int
file_audio(const char *self, int error, const char *path)
{
    char *name;
    unsigned size;

    size = strlen(path);

    name = (char *) malloc(size + 3);
    if (name) {
	*name = '`';
	name++;
	memcpy(name, path, size);
	name += size;
	*name = '\'';
	name[1] = 0;
	name -= size + 1;

	fprintf(stderr, "%s: ", self);
	libxaudio_file_xaudio(stderr, error, name);
	fprintf(stderr, "\n");

	free(name);
    }

    return -1;
}


static void
usage(void)
{
    puts("Usage: cquery [OPTIONS]\n\
Query supported audio classes.\n\
\n\
Options:\n\
  --help			display this help and exit\n\
  --version			output version information and exit");
}


int
main(int argc, char **argv)
{
    const char *device = NULL;
    int fast = ~0, status = 0;

    unsetenv("POSIXLY_CORRECT");

    while (1) {
	char c;
	static struct option long_options[] = {
/* *INDENT-OFF* */
	    {	"version",	0x00,	NULL,	0x76	},
	    {	"help",		0x00,	NULL,	0x68	},
	    {	NULL,		0x00,	NULL,	0x00	}
/* *INDENT-ON* */
	};

	c = getopt_long(argc, argv, "", long_options, NULL);

	if (!~c) {
	    break;
	}

	switch (c) {
	case 'h':
	case 'v':
	    if (!~fast) {
		fast = c;
	    }
	    break;
	case '?':
	    return 1;
	}
    }

    if (~fast) {
	switch (fast) {
	case 'h':
	    usage();
	    break;
	case 'v':
	    printf("%s (%s) %s\n", argv[0], PACKAGE, VERSION);
	}

	return 0;
    }

    if (0 < argc - optind) {
	fprintf(stderr, "%s: too many arguments\nTry `%s --help' for more in"
		"formation.\n", argv[0], argv[0]);

	return 1;
    }

    if (!device) {
	device = DEVICE;
    }

    {
	void *xaudio;

	status = libxaudio_open_xaudio(&xaudio, device);
	if (status) {
	    file_audio(argv[0], status, device);
	} else {
	    unsigned cquery;

	    status = libxaudio_list_xaudio(xaudio, &cquery);
	    if (status) {
		file_audio(argv[0], status, device);
	    } else {
		if (cquery & LIBxaudio_S010_CLASS) {
		    puts("signed 8bit");
		}
		if (cquery & LIBxaudio_S16B_CLASS) {
		    puts("signed 16bit big endian");
		}
		if (cquery & LIBxaudio_S16L_CLASS) {
		    puts("signed 16bit little endian");
		}
		if (cquery & LIBxaudio_U010_CLASS) {
		    puts("unsigned 8bit");
		}
		if (cquery & LIBxaudio_U16B_CLASS) {
		    puts("unsigned 16bit big endian");
		}
		if (cquery & LIBxaudio_U16L_CLASS) {
		    puts("unsigned 16bit little endian");
		}
	    }

	    {
		int excess;

		excess = libxaudio_lose_xaudio(&xaudio);
		if (excess) {
		    status = 1;
		    file_audio(argv[0], excess, device);
		}
	    }
	}
    }

    return status;
}
