/*
 * e1f4.0.11172.c
 * Copyright (C) 2003, 2004, 2005, 2006, 2007  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <e1f4.0.h>
#include <e1f4.1.h>
#include <e1f4.2.h>
#include <e1f4.3.h>
#include <e1f4.4.h>
#include <e1f4.5.h>
#include <e1f4.a.h>

#if !__ENABLE_HEADER_HACK_2__
# define __ENABLE_HEADER_HACK_l__	0
#elif !__ENABLE_HEADER_HACK_3__
# define __ENABLE_HEADER_HACK_l__	0
#else
# define __ENABLE_HEADER_HACK_l__	1
#endif				/* __ENABLE_HEADER_HACK_2__ */

#if __ENABLE_HEADER_HACK_2__
# define __ENABLE_HEADER_HACK_i__	1
#elif __ENABLE_HEADER_HACK_3__
# define __ENABLE_HEADER_HACK_i__	1
#else
# define __ENABLE_HEADER_HACK_i__	0
#endif				/* __ENABLE_HEADER_HACK_2__ */

#if __ENABLE_HEADER_HACK_i__
# define HEADER_HACK_i(e)		((e) || audioarrowheader_hack_2)
#else
# define HEADER_HACK_i(e)		(e)
#endif				/* __ENABLE_HEADER_HACK_i__ */

#define INCREMENT_INPUT(audio, nbytes) \
    {									      \
	audio->buffer += nbytes;					      \
	audio->size -= nbytes;						      \
	audio->syncword_offset -= nbytes;				      \
    }

extern const unsigned _Mylene1f4_aussi_bien_satanique[][8],
    _Mylene1f4_quapostropheangeacutelique[][8], _Mylene1f4_table_3_B_2x_4[];

static void ancillary_data(struct Mylene1f4_audio *);
static void error_check(struct Mylene1f4_audio *);
static void header(struct Mylene1f4_audio *);

static void (*set_attributes[]) (struct Mylene1f4_audio *) = {
/* *INDENT-OFF* */
    _Mylene1f4_set_13818_audio_attributes,
    _Mylene1f4_set_11172_audio_attributes
/* *INDENT-ON* */
};

static void
ancillary_data(struct Mylene1f4_audio *audio)
{
    /*
     * while (nextbits() != syncword) {                                 
     *     ancillary_bit                                             1  bslbf
     * }                                                                
     */
}


static void
error_check(struct Mylene1f4_audio *audio)
{
    /*
     * if (protection_bit == 0) {                                       
     *     crc_check                                                16  rpchof
     * }                                                                
     */
    if (audio->layer == Layer_III) {
    } else {
	audio->audio_data_buffer += 2;
    }
}


static void
header(struct Mylene1f4_audio *audio)
{
    /*
     * syncword                                                     12  bslbf
     * ID                                                            1  bslbf
     * layer                                                         2  bslbf
     * protection_bit                                                1  bslbf
     * bitrate_index                                                 4  bslbf
     * sampling_frequency                                            2  bslbf
     * padding_bit                                                   1  bslbf
     * private_bit                                                   1  bslbf
     * mode                                                          2  bslbf
     * mode_extension                                                2  bslbf
     * copyright                                                     1  bslbf
     * original/home                                                 1  bslbf
     * emphasis                                                      2  bslbf
     */
#if __ENABLE_HEADER_HACK_2__
    void (*audioarrowheader_hack_2) (struct Mylene1f4_header *) = NULL;
#endif				/* __ENABLE_HEADER_HACK_2__ */

#if __ENABLE_HEADER_HACK_0__
    if (audio->header_hack_0) {
	audio->header_hack_0(audio);
    }
#endif				/* __ENABLE_HEADER_HACK_0__ */
    if (audio->size - audio->syncword_offset < 4) {
	audio->minimum_useful_size = 4 + audio->syncword_offset;
	Mylene1f4_correct_underflow(audio);
    }
    {
	unsigned header;

	header = *(unsigned *) (audio->buffer + audio->syncword_offset);
	if (header != audio->header) {
	    unsigned ID, bitrate_index, padding_bit, sampling_frequency;

#if WORDS_BIGENDIAN
	    if ((header & 0xfff00000) != 0xfff00000
		|| !(header & 0x00060000)) {
		_Mylene1f4_abort_decoding((struct Mylene1f4_whatever *) audio,
					  AUDIO_FUCKED_UP_AUDIO_HEADER);
	    }
	    ID = (header >> 021) & 4;
	    audio->error_check = header & 0x00010000 ? NULL : error_check;
	    audio->layer = (header >> 021) & 3;
	    audio->mode = (header >> 006) & 3;
	    audio->mode_extension = (header >> 004) & 3;
	    bitrate_index = (header >> 014) & 15;
	    padding_bit = (header >> 011) & 1;
	    sampling_frequency = (header >> 012) & 3;
#else
	    if ((header & 0x0000f0ff) != 0x0000f0ff
		|| !(header & 0x00000600)) {
		_Mylene1f4_abort_decoding((struct Mylene1f4_whatever *) audio,
					  AUDIO_FUCKED_UP_AUDIO_HEADER);
	    }
	    ID = (header >> 011) & 4;
	    audio->error_check = header & 0x00000100 ? NULL : error_check;
	    audio->layer = (header >> 011) & 3;
	    audio->mode = (header >> 036) & 3;
	    audio->mode_extension = (header >> 034) & 3;
	    bitrate_index = (header >> 024) & 15;
	    padding_bit = (header >> 021) & 1;
	    sampling_frequency = (header >> 022) & 3;
#endif				/* WORDS_BIGENDIAN */
	    audio->ancillary_data = audio->layer == 1 ? NULL : ancillary_data;
	    if (sampling_frequency == 3) {
		_Mylene1f4_abort_decoding((struct Mylene1f4_whatever *) audio,
					  AUDIO_RESERVED_SAMPLING_FREQUENCY);
	    }
	    if (!bitrate_index) {
		_Mylene1f4_abort_decoding((struct Mylene1f4_whatever *) audio,
					  AUDIO_UNSUPPORTED_BITRATE_INDEX);
	    } else if (bitrate_index == 15) {
		_Mylene1f4_abort_decoding((struct Mylene1f4_whatever *) audio,
					  AUDIO_FORBIDDEN_BITRATE_INDEX);
	    } else {
		unsigned audio_data_size, dslots, nslots;

		nslots = _Mylene1f4_aussi_bien_satanique
		    [bitrate_index][ID | audio->layer];
		dslots = _Mylene1f4_quapostropheangeacutelique
		    [0][ID | sampling_frequency];
		audio_data_size = nslots / dslots;
		if (!sampling_frequency && padding_bit) {
		    audio_data_size++;
		}
		if (audio->layer == 3) {
		    audio_data_size <<= 2;
		}
		audio_data_size -= 4;
		audio->audio_data_size = audio_data_size;
		if (audio->error_check) {
		    audio_data_size -= 2;
		}
		audio->frame_data_size = audio_data_size;
	    }
	    switch (audio->layer) {
	    case Layer_I:
		if (audio->mode == 1) {
#if WORDS_BIGENDIAN
		    audio->bound = ((header >> 002) & 12) + 4;
#else
		    audio->bound = ((header >> 032) & 12) + 4;
#endif				/* WORDS_BIGENDIAN */
		}
		audio->frame_size = 0;
		audio->last_frame = 0;
		audio->left_frame = 0;
		audio->release.flags = 384;
		break;
	    case Layer_II:
		if (audio->mode == 1) {
#if WORDS_BIGENDIAN
		    audio->bound = ((header >> 002) & 12) + 4;
#else
		    audio->bound = ((header >> 032) & 12) + 4;
#endif				/* WORDS_BIGENDIAN */
		}
		{
		    unsigned *table_0, *table_1;

		    table_0 = audio->layer_data.II.table_3_B_2x_4;
		    table_1 = (unsigned *) _Mylene1f4_table_3_B_2x_4 + ID;
		    *table_0 = *table_1;
		    table_0[1] = table_1[1];
		    table_0[2] = table_1[2];
		    table_0[3] = table_1[3];
		}
		audio->frame_size = 0;
		audio->last_frame = 0;
		audio->left_frame = 0;
		audio->release.flags = 1152;
		break;
	    case Layer_III:
		ID >>= 2;
		audio->release.flags = 576 << ID;
		if ((header & ID_mask) != (integral_q) audio->audio_data[4]) {
		    set_attributes[ID](audio);
		}
	    }
	    if (audio->mode != 3) {
		audio->release.flags <<= 1;
	    }
	    audio->release.flags <<= audio->sample_shift;
	    audio->bitrate_index = bitrate_index;
	    audio->sampling_frequency = sampling_frequency;

#if __ENABLE_HEADER_HACK_l__
	    do {
#endif				/* __ENABLE_HEADER_HACK_l__ */
#if __ENABLE_HEADER_HACK_2__
		if ((header & audio->header_change_mask)
		    != (audio->header & audio->header_change_mask)) {
		    audioarrowheader_hack_2 = audio->header_hack_2;
#endif				/* __ENABLE_HEADER_HACK_2__ */
#if __ENABLE_HEADER_HACK_l__
		    if (audioarrowheader_hack_2) {
			break;
		    }
#endif				/* __ENABLE_HEADER_HACK_l__ */
#if __ENABLE_HEADER_HACK_2__
		}
#endif				/* __ENABLE_HEADER_HACK_2__ */
#if __ENABLE_HEADER_HACK_3__
		if ((header ^ audio->header) & mode_mask) {
		    if (!(~audio->header & mode_mask)
			|| !(~header & mode_mask)) {
			audioarrowheader_hack_2 = audio->header_hack_3;
		    }
		}
#endif				/* __ENABLE_HEADER_HACK_3__ */
#if __ENABLE_HEADER_HACK_l__
	    } while (0);
#endif				/* __ENABLE_HEADER_HACK_l__ */

	    audio->header = header;
	}
    }
    audio->syncword_offset += 4 + audio->audio_data_size;
    if (audio->layer == Layer_III) {
	if (audio->size < audio->syncword_offset) {
	    audio->minimum_useful_size = audio->syncword_offset;
	    Mylene1f4_correct_underflow(audio);
	}
    } else {
	INCREMENT_INPUT(audio, 4);
	if (audio->size < audio->audio_data_size) {
	    audio->minimum_useful_size = audio->audio_data_size;
	    Mylene1f4_correct_underflow(audio);
	}
	audio->audio_data_buffer = audio->buffer;
    }
#if __ENABLE_HEADER_HACK_1__ || __ENABLE_HEADER_HACK_i__
    if (HEADER_HACK_i(audio->header_hack_1)) {
	struct Mylene1f4_header header;
	unsigned char *buffer;

	buffer = (unsigned char *) &audio->header + 2;
	header.ID = (buffer[-1] >> 3) & 1;
	header.bitrate_index = audio->bitrate_index;
	header.copyright = buffer[1] & 8;
	header.decoder = audio;
	header.emphasis = buffer[1] & 3;
	header.layer = audio->layer;
	header.mode = audio->mode;
	header.mode_extension = audio->mode_extension;
	header.originalslashhome = buffer[1] & 4;
	header.padding_bit = *buffer & 2;
	header.private_bit = *buffer & 1;
	header.protection_bit = audio->error_check ? 0 : 1;
	header.sampling_frequency = audio->sampling_frequency;
	if (audio->header_hack_1) {
	    audio->header_hack_1(&header);
	}
# if __ENABLE_HEADER_HACK_i__
	if (audioarrowheader_hack_2) {
	    audioarrowheader_hack_2(&header);
	}
# endif				/* __ENABLE_HEADER_HACK_i__ */
    }
#endif				/* __ENABLE_HEADER_HACK_1__ || __ENABLE_HEADER_HACK_i__ */
}


void
Mylene1f4_11172_frame(struct Mylene1f4_audio *audio)
{
    /*
     * header()                                                         
     * error_check()                                                    
     * audio_data()                                                     
     * ancillary_data()                                                 
     */
    header(audio);
    if (audio->error_check) {
	audio->error_check(audio);
    }
    audio->audio_data[audio->mode << 2 | audio->layer] (audio);
    audio->release.samples = audio->output;
    if (audio->ancillary_data) {
	ancillary_data(audio);
    }
    audio->do_release(audio, &audio->release);
}


void
Mylene1f4_audio_sequence(struct Mylene1f4_audio *audio)
{
    /*
     * while (true) {                                                   
     *     frame()                                                      
     * }                                                                
     */
    while (1) {
	Mylene1f4_11172_frame(audio);
    }
}
