/*
 * e1f4.2.c
 * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <e1f4.0.h>
#include <e1f4.1.h>
#include <e1f4.3.h>

#if WORDS_BIGENDIAN
# define FRAME_HEADER(header) \
    (((header) & 0xfff00000) == 0xfff00000 && ((header) & 0x00060000)	      \
     && ((header) & 0x00000c00) != 0x00000c00 && ((header) & 0x0000f000)      \
     && ((header) & 0x0000f000) != 0x0000f000)
#else
# define FRAME_HEADER(header) \
    (((header) & 0x0000f0ff) == 0x0000f0ff && ((header) & 0x00000600)	      \
     && ((header) & 0x000c0000) != 0x000c0000 && ((header) & 0x00f00000)      \
     && ((header) & 0x00f00000) != 0x00f00000)
#endif				/* WORDS_BIGENDIAN */

#define correct_underflow(audio, mad) \
    if ((audio)->size < (mad)) {					      \
	(audio)->minimum_useful_size = (mad);				      \
	Mylene1f4_correct_underflow((audio));				      \
    }

void
Mylene1f4_seek_audio(struct Mylene1f4_audio *audio)
{
    while (1) {
	unsigned header;

	correct_underflow(audio, 4);
	header = *(unsigned *) audio->buffer;
	if (FRAME_HEADER(header)) {
	    unsigned offset = 0;

	    Mylene1f4_syncword_offset(header, &offset);
	    if (offset) {
		correct_underflow(audio, offset + 4);
		header = *(unsigned *) (audio->buffer + offset);
		if (FRAME_HEADER(header)) {
		    break;
		}
	    }
	}
	audio->buffer++;
	audio->size--;
    }
}
