/*
 * e1f4.8.11172.c
 * Copyright (C) 2003, 2004, 2005, 2006  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <e1f4.0.h>
#include <e1f4.2.h>
#include <e1f4.3.h>
#include <e1f4.4.h>

void
Mylene1f4_header_semantics(unsigned header, struct Mylene1f4_header *semantics)
{
#if WORDS_BIGENDIAN
    if ((header & 0xfff00000) != 0xfff00000 || !(header & 0x00060000)) {
	semantics->decoder = NULL;

	return;
    } else {
	semantics->decoder = (void *) 1;
    }

    semantics->ID = (header >> 023) & 1;
    semantics->bitrate_index = (header >> 014) & 15;
    semantics->copyright = (header >> 003) & 1;
    semantics->emphasis = (header >> 000) & 3;
    semantics->layer = (header >> 021) & 3;
    semantics->mode = (header >> 006) & 3;
    semantics->mode_extension = (header >> 004) & 3;
    semantics->originalslashhome = (header >> 002) & 1;
    semantics->padding_bit = (header >> 011) & 1;
    semantics->private_bit = (header >> 010) & 1;
    semantics->protection_bit = (header >> 020) & 1;
    semantics->sampling_frequency = (header >> 012) & 3;
#else
    if ((header & 0x0000f0ff) != 0x0000f0ff || !(header & 0x00000600)) {
	semantics->decoder = NULL;

	return;
    } else {
	semantics->decoder = (void *) 1;
    }

    semantics->ID = (header >> 013) & 1;
    semantics->bitrate_index = (header >> 024) & 15;
    semantics->copyright = (header >> 033) & 1;
    semantics->emphasis = (header >> 030) & 3;
    semantics->layer = (header >> 011) & 3;
    semantics->mode = (header >> 036) & 3;
    semantics->mode_extension = (header >> 034) & 3;
    semantics->originalslashhome = (header >> 032) & 1;
    semantics->padding_bit = (header >> 021) & 1;
    semantics->private_bit = (header >> 020) & 1;
    semantics->protection_bit = (header >> 010) & 1;
    semantics->sampling_frequency = (header >> 022) & 3;
#endif				/* WORDS_BIGENDIAN */
}


void
Mylene1f4_release_size(unsigned header, unsigned *release_size)
{
    unsigned layer, release;

#if WORDS_BIGENDIAN
    layer = (header >> 021) & 3;
#else
    layer = (header >> 011) & 3;
#endif				/* WORDS_BIGENDIAN */
    switch (layer) {
    case Layer_I:
	release = 384;
	break;
    case Layer_II:
	release = 1152;
	break;
    default:
#if WORDS_BIGENDIAN
	release = 576 << ((header >> 023) & 1);
#else
	release = 576 << ((header >> 013) & 1);
#endif				/* WORDS_BIGENDIAN */
    }
#if WORDS_BIGENDIAN
    if (((header >> 006) & 3) != 3) {
	release <<= 1;
    }
#else
    if (((header >> 036) & 3) != 3) {
	release <<= 1;
    }
#endif				/* WORDS_BIGENDIAN */

    *release_size = release;
}
