/*
 * e1f4.b.11172.c
 * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <e1f4.1.h>
#include <e1f4.3.h>

#define WRITE_SAMPLE(samples, sum) \
    {									      \
	*(samples) = (sum);						      \
    }

extern Mylene1f4_real _Mylene1f4_decwin[544];

void
Mylene1f4_polyphase_synthesis_a(struct Mylene1f4_audio *audio,
				Mylene1f4_real *fraction, void *output,
				unsigned nblocks)
{
    Mylene1f4_real (*buffer)[0x110], *buffer_1, *samples;
    unsigned wrapped, wrapped_1;

    for (; nblocks; nblocks--) {
	samples = (Mylene1f4_real *) output;

	wrapped = audio->wrapped_1[0];

	wrapped--;
	wrapped &= 0xf;

	buffer = ((Mylene1f4_real (*)[2][0x110]) audio->polyphase_buffer)[0];

	if (wrapped & 0x1) {
	    buffer_1 = buffer[0];
	    wrapped_1 = wrapped;
	    audio->dct_64(buffer[1] + ((wrapped + 1) & 0xf),
			  buffer[0] + wrapped, fraction);
	} else {
	    buffer_1 = buffer[1];
	    wrapped_1 = wrapped + 1;
	    audio->dct_64
		(buffer[0] + wrapped, buffer[1] + wrapped + 1, fraction);
	}

	audio->wrapped_1[0] = wrapped;

	{
	    Mylene1f4_real *window;
	    register int j;

	    window = _Mylene1f4_decwin + 16 - wrapped_1;

	    for (j = 16; j;
		 j--, buffer_1 += 0x10, window += 0x20, samples += 1) {
		Mylene1f4_real sum;

		sum = +window[0x0] * buffer_1[0x0];
		sum -= window[0x1] * buffer_1[0x1];
		sum += window[0x2] * buffer_1[0x2];
		sum -= window[0x3] * buffer_1[0x3];
		sum += window[0x4] * buffer_1[0x4];
		sum -= window[0x5] * buffer_1[0x5];
		sum += window[0x6] * buffer_1[0x6];
		sum -= window[0x7] * buffer_1[0x7];
		sum += window[0x8] * buffer_1[0x8];
		sum -= window[0x9] * buffer_1[0x9];
		sum += window[0xa] * buffer_1[0xa];
		sum -= window[0xb] * buffer_1[0xb];
		sum += window[0xc] * buffer_1[0xc];
		sum -= window[0xd] * buffer_1[0xd];
		sum += window[0xe] * buffer_1[0xe];
		sum -= window[0xf] * buffer_1[0xf];

		WRITE_SAMPLE(samples, sum);
	    }

	    {
		Mylene1f4_real sum;

		sum = +window[0x0] * buffer_1[0x0];
		sum += window[0x2] * buffer_1[0x2];
		sum += window[0x4] * buffer_1[0x4];
		sum += window[0x6] * buffer_1[0x6];
		sum += window[0x8] * buffer_1[0x8];
		sum += window[0xa] * buffer_1[0xa];
		sum += window[0xc] * buffer_1[0xc];
		sum += window[0xe] * buffer_1[0xe];
		WRITE_SAMPLE(samples, sum);
		buffer_1 -= 0x10, window -= 0x20, samples += 1;
	    }

	    window += (wrapped_1 << 1) - 1;

	    for (j = 15; j;
		 j--, buffer_1 -= 0x10, window -= 0x20, samples += 1) {
		Mylene1f4_real sum;

		sum = -window[-0x0] * buffer_1[0x0];
		sum -= window[-0x1] * buffer_1[0x1];
		sum -= window[-0x2] * buffer_1[0x2];
		sum -= window[-0x3] * buffer_1[0x3];
		sum -= window[-0x4] * buffer_1[0x4];
		sum -= window[-0x5] * buffer_1[0x5];
		sum -= window[-0x6] * buffer_1[0x6];
		sum -= window[-0x7] * buffer_1[0x7];
		sum -= window[-0x8] * buffer_1[0x8];
		sum -= window[-0x9] * buffer_1[0x9];
		sum -= window[-0xa] * buffer_1[0xa];
		sum -= window[-0xb] * buffer_1[0xb];
		sum -= window[-0xc] * buffer_1[0xc];
		sum -= window[-0xd] * buffer_1[0xd];
		sum -= window[-0xe] * buffer_1[0xe];
		sum -= window[-0xf] * buffer_1[0xf];

		WRITE_SAMPLE(samples, sum);
	    }
	}

	fraction += 32;
	output = (Mylene1f4_real *) output + 32;
    }
}


void
Mylene1f4_polyphase_synthesis_b(struct Mylene1f4_audio *audio,
				Mylene1f4_real *fraction, unsigned channel,
				void *output, unsigned nblocks)
{
    Mylene1f4_real (*buffer)[0x110], *buffer_1, *samples;
    unsigned wrapped, wrapped_1;

    for (; nblocks; nblocks--) {
	samples = (Mylene1f4_real *) output;

	wrapped = audio->wrapped_1[channel];

	wrapped--;
	wrapped &= 0xf;

	samples += channel;
	buffer =
	    ((Mylene1f4_real (*)[2][0x110]) audio->polyphase_buffer)[channel];

	if (wrapped & 0x1) {
	    buffer_1 = buffer[0];
	    wrapped_1 = wrapped;
	    audio->dct_64(buffer[1] + ((wrapped + 1) & 0xf),
			  buffer[0] + wrapped, fraction);
	} else {
	    buffer_1 = buffer[1];
	    wrapped_1 = wrapped + 1;
	    audio->dct_64
		(buffer[0] + wrapped, buffer[1] + wrapped + 1, fraction);
	}

	audio->wrapped_1[channel] = wrapped;

	{
	    Mylene1f4_real *window;
	    register int j;

	    window = _Mylene1f4_decwin + 16 - wrapped_1;

	    for (j = 16; j;
		 j--, buffer_1 += 0x10, window += 0x20, samples += 2) {
		Mylene1f4_real sum;

		sum = +window[0x0] * buffer_1[0x0];
		sum -= window[0x1] * buffer_1[0x1];
		sum += window[0x2] * buffer_1[0x2];
		sum -= window[0x3] * buffer_1[0x3];
		sum += window[0x4] * buffer_1[0x4];
		sum -= window[0x5] * buffer_1[0x5];
		sum += window[0x6] * buffer_1[0x6];
		sum -= window[0x7] * buffer_1[0x7];
		sum += window[0x8] * buffer_1[0x8];
		sum -= window[0x9] * buffer_1[0x9];
		sum += window[0xa] * buffer_1[0xa];
		sum -= window[0xb] * buffer_1[0xb];
		sum += window[0xc] * buffer_1[0xc];
		sum -= window[0xd] * buffer_1[0xd];
		sum += window[0xe] * buffer_1[0xe];
		sum -= window[0xf] * buffer_1[0xf];

		WRITE_SAMPLE(samples, sum);
	    }

	    {
		Mylene1f4_real sum;

		sum = +window[0x0] * buffer_1[0x0];
		sum += window[0x2] * buffer_1[0x2];
		sum += window[0x4] * buffer_1[0x4];
		sum += window[0x6] * buffer_1[0x6];
		sum += window[0x8] * buffer_1[0x8];
		sum += window[0xa] * buffer_1[0xa];
		sum += window[0xc] * buffer_1[0xc];
		sum += window[0xe] * buffer_1[0xe];
		WRITE_SAMPLE(samples, sum);
		buffer_1 -= 0x10, window -= 0x20, samples += 2;
	    }

	    window += (wrapped_1 << 1) - 1;

	    for (j = 15; j;
		 j--, buffer_1 -= 0x10, window -= 0x20, samples += 2) {
		Mylene1f4_real sum;

		sum = -window[-0x0] * buffer_1[0x0];
		sum -= window[-0x1] * buffer_1[0x1];
		sum -= window[-0x2] * buffer_1[0x2];
		sum -= window[-0x3] * buffer_1[0x3];
		sum -= window[-0x4] * buffer_1[0x4];
		sum -= window[-0x5] * buffer_1[0x5];
		sum -= window[-0x6] * buffer_1[0x6];
		sum -= window[-0x7] * buffer_1[0x7];
		sum -= window[-0x8] * buffer_1[0x8];
		sum -= window[-0x9] * buffer_1[0x9];
		sum -= window[-0xa] * buffer_1[0xa];
		sum -= window[-0xb] * buffer_1[0xb];
		sum -= window[-0xc] * buffer_1[0xc];
		sum -= window[-0xd] * buffer_1[0xd];
		sum -= window[-0xe] * buffer_1[0xe];
		sum -= window[-0xf] * buffer_1[0xf];

		WRITE_SAMPLE(samples, sum);
	    }
	}

	fraction += 32;
	output = (Mylene1f4_real *) output + 64;
    }
}
