/*
 * e1f4.f.c
 * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <e1f4.0.h>
#include <e1f4.1.h>
#include <e1f4.2.h>
#include <e1f4.3.h>
#include <e1f4.4.h>
#include <e1f4.a.h>
#include <e1f4.f.h>

#define AUDIO_ATTRIBUTES		struct Mylene1f4_audio_attributes

#define e1f4_FORBIDDEN			Mylene1f4_FORBIDDEN
#define e1f4_FREE_FORMAT		Mylene1f4_FREE_FORMAT

#define SET_DATA__1___5___9___d___0(audio, process) \
    {									      \
	audio->audio_data[0x1] = process##_audio_data_III[0x0];		      \
	audio->audio_data[0x5] = process##_audio_data_III[0x1];		      \
	audio->audio_data[0x9] = process##_audio_data_III[0x2];		      \
	audio->audio_data[0xd] = process##_audio_data_III[0x3];		      \
    }

#define SET_DATA__1___5___9___d___1(audio, process) \
    {									      \
	audio->audio_data[0x1] = process##_audio_data_III[0x4];		      \
	audio->audio_data[0x5] = process##_audio_data_III[0x4];		      \
	audio->audio_data[0x9] = process##_audio_data_III[0x4];		      \
	audio->audio_data[0xd] = process##_audio_data_III[0x4];		      \
    }

#define SET_DATA___23__67__ab__ef_0(audio, process) \
    {									      \
	audio->audio_data[0x2] = process##_audio_data__II[0x0];		      \
	audio->audio_data[0x3] = process##_audio_data___I[0x0];		      \
	audio->audio_data[0x6] = process##_audio_data__II[0x1];		      \
	audio->audio_data[0x7] = process##_audio_data___I[0x1];		      \
	audio->audio_data[0xa] = process##_audio_data__II[0x2];		      \
	audio->audio_data[0xb] = process##_audio_data___I[0x2];		      \
	audio->audio_data[0xe] = process##_audio_data__II[0x3];		      \
	audio->audio_data[0xf] = process##_audio_data___I[0x3];		      \
    }

#define SET_DATA___23__67__ab__ef_1(audio, process) \
    {									      \
	audio->audio_data[0x2] = process##_audio_data__II[0x4];		      \
	audio->audio_data[0x3] = process##_audio_data___I[0x4];		      \
	audio->audio_data[0x6] = process##_audio_data__II[0x4];		      \
	audio->audio_data[0x7] = process##_audio_data___I[0x4];		      \
	audio->audio_data[0xa] = process##_audio_data__II[0x4];		      \
	audio->audio_data[0xb] = process##_audio_data___I[0x4];		      \
	audio->audio_data[0xe] = process##_audio_data__II[0x4];		      \
	audio->audio_data[0xf] = process##_audio_data___I[0x4];		      \
    }

extern void
 (*_Mylene1f4_4294967279_audio_data_III[]) (struct Mylene1f4_audio *),
    (*_Mylene1f4_4294967291_audio_data_III[]) (struct Mylene1f4_audio *),
    (*_Mylene1f4_4294967291_audio_data__II[]) (struct Mylene1f4_audio *),
    (*_Mylene1f4_4294967291_audio_data___I[]) (struct Mylene1f4_audio *),
    (*_Mylene1f4_futile_____audio_data_III[]) (struct Mylene1f4_audio *),
    (*_Mylene1f4_futile_____audio_data__II[]) (struct Mylene1f4_audio *),
    (*_Mylene1f4_futile_____audio_data___I[]) (struct Mylene1f4_audio *);

static const char *error_names[] = {
/* *INDENT-OFF* */
    /* no available description */
    "no available description",
    /* x01: AUDIO_DATA_FAULT */
    "audio data fault",
    /* x02: BAD_IMPLEMENTATION */
    "bad implementation",
    /* x03: FORBIDDEN_BITRATE_INDEX */
    "forbidden bitrate index",
    /* x04: FUCKED_UP_AUDIO_HEADER */
    "fucked up audio header",
    /* x05: INVALID_BACKWARD_SEEK */
    "invalid backward seek",
    /* x06: NULL_BLOCK_TYPE */
    "null block type",
    /* x07: RESERVED_SAMPLING_FREQUENCY */
    "reserved sampling frequency",
    /* x08: SHORT_SAMPLES_SECTION */
    "short samples section",
    /* x09: UNSUPPORTED_BITRATE_INDEX */
    "unsupported bitrate index"
/* *INDENT-ON* */
}, *frame_rate_strings[] = {
/* *INDENT-OFF* */
    "",
    "38.28125",
    "19.140625",
    "57.421875",
    "",
    "41.(6)",
    "20.8(3)",
    "62.5",
    "",
    "27.(7)",
    "13.(8)",
    "41.(6)",
    "",
    "",
    "",
    "",
    "",
    "38.28125",
    "38.28125",
    "114.84375",
    "",
    "41.(6)",
    "41.(6)",
    "125",
    "",
    "27.(7)",
    "27.(7)",
    "83.(3)",
    "",
    "",
    "",
    ""
/* *INDENT-ON* */
};
static const unsigned header_to_bitrate[][3] = {
/* *INDENT-OFF* */
    {	e1f4_FREE_FORMAT, e1f4_FREE_FORMAT, e1f4_FREE_FORMAT	},
    {	               8,                8,               32	},
    {	              16,               16,               48	},
    {	              24,               24,               56	},
    {	              32,               32,               64	},
    {	              40,               40,               80	},
    {	              48,               48,               96	},
    {	              56,               56,              112	},
    {	              64,               64,              128	},
    {	              80,               80,              144	},
    {	              96,               96,              160	},
    {	             112,              112,              176	},
    {	             128,              128,              192	},
    {	             144,              144,              224	},
    {	             160,              160,              256	},
    {	  e1f4_FORBIDDEN,   e1f4_FORBIDDEN,   e1f4_FORBIDDEN	},
    {	e1f4_FREE_FORMAT, e1f4_FREE_FORMAT, e1f4_FREE_FORMAT	},
    {	              32,               32,               32	},
    {	              40,               48,               64	},
    {	              48,               56,               96	},
    {	              56,               64,              128	},
    {	              64,               80,              160	},
    {	              80,               96,              192	},
    {	              96,              112,              224	},
    {	             112,              128,              256	},
    {	             128,              160,              288	},
    {	             160,              192,              320	},
    {	             192,              224,              352	},
    {	             224,              256,              384	},
    {	             256,              320,              416	},
    {	             320,              384,              448	},
    {	  e1f4_FORBIDDEN,   e1f4_FORBIDDEN,   e1f4_FORBIDDEN	}
/* *INDENT-ON* */
}, header_to_sampling_frequency[] = {
/* *INDENT-OFF* */
		   22050,
		   24000,
		   16000,
      Mylene1f4_RESERVED,
		   44100,
		   48000,
		   32000,
      Mylene1f4_RESERVED
/* *INDENT-ON* */
}, samples_per_frame[] = {
/* *INDENT-OFF* */
     Mylene1f4_FORBIDDEN,
		     576,
		    1152,
		     384,
     Mylene1f4_FORBIDDEN,
		    1152,
		    1152,
		     384
/* *INDENT-ON* */
};

void
_Mylene1f4_set_13818_audio_attributes(struct Mylene1f4_audio *audio)
{
    if (!audio->audio_data[4]) {
    } else {
	audio->audio_data[4] = (void (*)(struct Mylene1f4_audio *)) 0;
	if (audio->audio_data[0x1]
	    == _Mylene1f4_4294967291_audio_data_III[0x0]) {
	    SET_DATA__1___5___9___d___0(audio, _Mylene1f4_4294967279);
	} else if (audio->audio_data[0x1]
		   == _Mylene1f4_4294967291_audio_data_III[0x4]) {
	    SET_DATA__1___5___9___d___1(audio, _Mylene1f4_4294967279);
	}
    }
}


void
Mylene1f4_audio_error_string(unsigned error, char **error_name)
{
    if (error < AUDIO_NUMBER_OF_ERROR_CODES) {
	*error_name = (char *) error_names[error];
    } else {
	*error_name = NULL;
    }
}


void
Mylene1f4_audio_size(unsigned *size_return)
{
    *size_return = sizeof(struct Mylene1f4_audio);
}


void
Mylene1f4_audio_release_size(void *audio, unsigned *release_size_return)
{
    *release_size_return =
	((struct Mylene1f4_audio *) audio)->release.flags & 32767;
}


void
Mylene1f4_bitrate(struct Mylene1f4_header *header, unsigned *bitrate)
{
    if (header->layer) {
	*bitrate = header_to_bitrate
	    [header->ID << 4 | header->bitrate_index][header->layer - 1];
    } else {
	*bitrate = Mylene1f4_FORBIDDEN;
    }
}


void
Mylene1f4_clear_state(void *audio)
{
    ((struct Mylene1f4_audio *) audio)->frame_size = 0;
    ((struct Mylene1f4_audio *) audio)->last_frame = 0;
    ((struct Mylene1f4_audio *) audio)->left_frame = 0;
}


void
Mylene1f4_framerate(struct Mylene1f4_header *header, unsigned *numerator,
		    unsigned *denominator)
{
    if (!header->layer || header->sampling_frequency == 3) {
	*numerator = Mylene1f4_FORBIDDEN;
    } else {
	*denominator = samples_per_frame[header->ID << 2 | header->layer];
	*numerator = header_to_sampling_frequency
	    [header->ID << 2 | header->sampling_frequency];
    }
}


void
Mylene1f4_framerate_string(struct Mylene1f4_header *header,
			   const char **string)
{
    if (!header->layer || header->sampling_frequency == 3) {
	*string = NULL;
    } else {
	*string = frame_rate_strings
	    [header->ID << 4 | header->sampling_frequency << 2
	     | header->layer];
    }
}


void
Mylene1f4_get_ID(void *audio, unsigned *ID)
{
    *ID = ((unsigned char *) &((struct Mylene1f4_audio *) audio)->header)[1]
	>> 3 & 1;
}


void
Mylene1f4_get_audio_attributes(void *audio_return,
			       AUDIO_ATTRIBUTES *audio_attributes,
			       unsigned mask, unsigned extra_mask)
{
    struct Mylene1f4_audio *audio;

    audio = (struct Mylene1f4_audio *) audio_return;
    if (mask & Mylene1f4_AUDIO_ABORT_DECODING_MASK) {
	audio_attributes->abort_decoding =
	    (void (*)(void *)) audio->abort_decoding;
    }
    if (mask & Mylene1f4_AUDIO_BUFFER_MASK) {
	audio_attributes->buffer = audio->buffer;
    }
    if (mask & Mylene1f4_AUDIO_CORRECT_UNDERFLOW_MASK) {
	audio_attributes->correct_underflow =
	    (void (*)(void *)) audio->optimistique_moi;
    }
    if (mask & Mylene1f4_AUDIO_DCT_64_MASK) {
	audio_attributes->dct_64 = audio->dct_64;
    }
    if (mask & Mylene1f4_AUDIO_DO_RELEASE_MASK) {
	audio_attributes->do_release =
	    (void (*)(void *, struct Mylene1f4_release *)) audio->do_release;
    }
#if __ENABLE_EQUALIZER_0__
    if (mask & Mylene1f4_AUDIO_EQUALIZER_0_MASK) {
	audio_attributes->equalizer_0 = audio->x_equalizer_0;
    }
#endif				/* __ENABLE_EQUALIZER_0__ */
#if __ENABLE_EQUALIZER_0__
    if (mask & Mylene1f4_AUDIO_EQUALIZE_0_MASK) {
	audio_attributes->equalize_0 = audio->x_equalize_0;
    }
#endif				/* __ENABLE_EQUALIZER_0__ */
    if (mask & Mylene1f4_AUDIO_MINIMUM_USEFUL_SIZE_MASK) {
	audio_attributes->minimum_useful_size = audio->minimum_useful_size;
    }
    if (mask & Mylene1f4_AUDIO_OUTPUT_MASK) {
	audio_attributes->output = audio->output;
    }
    if (mask & Mylene1f4_AUDIO_POLYPHASE_BUFFER_MASK) {
	audio_attributes->polyphase_buffer = audio->polyphase_buffer;
    }
    if (mask & Mylene1f4_AUDIO_POLYPHASE_SYNTHESIS_0_MASK) {
	audio_attributes->polyphase_synthesis_0 =
	    (void (*)(void *, Mylene1f4_real *, void *, unsigned))
	    audio->polyphase_synthesis_0;
    }
    if (mask & Mylene1f4_AUDIO_POLYPHASE_SYNTHESIS_1_MASK) {
	audio_attributes->polyphase_synthesis_1 =
	    (void (*)(void *, Mylene1f4_real *, unsigned, void *, unsigned))
	    audio->polyphase_synthesis_1;
    }
    if (mask & Mylene1f4_AUDIO_PROCESS_MASK) {
	if (audio->audio_data[0xf]
	    == _Mylene1f4_4294967291_audio_data___I[0x3]) {
	    audio_attributes->process = Mylene1f4_4294967291;
	} else if (audio->audio_data[0xf]
		   == _Mylene1f4_futile_____audio_data___I[0x3]) {
	    audio_attributes->process = Mylene1f4_FUTILE;
	} else if (audio->audio_data[0xf]
		   == _Mylene1f4_4294967291_audio_data___I[0x4]) {
	    audio_attributes->process = Mylene1f4_UTTERLY_FUTILE;
	} else {
	}
    }
    if (mask & Mylene1f4_AUDIO_SAMPLE_SHIFT_MASK) {
	audio_attributes->sample_shift = audio->sample_shift;
    }
    if (mask & Mylene1f4_AUDIO_SIZE_MASK) {
	audio_attributes->size = audio->size;
    }
    if (mask & Mylene1f4_AUDIO_SYNCWORD_OFFSET_MASK) {
	audio_attributes->syncword_offset = audio->syncword_offset;
    }
    if (mask & Mylene1f4_AUDIO_TRANSFORM_BUFFER_MASK) {
	audio_attributes->transform_buffer = audio->transform_buffer;
    }
#if __ENABLE_HEADER_HACK_2__
    if (extra_mask & Mylene1f4_AUDIO_HEADER_CHANGE_MASK) {
	audio_attributes->header_change_mask = audio->user_header_change_mask;
    }
#endif				/* __ENABLE_HEADER_HACK_2__ */
#if __ENABLE_HEADER_HACK_0__
    if (extra_mask & Mylene1f4_AUDIO_HEADER_HACK_0_MASK) {
	audio_attributes->header_hack_0 =
	    (void (*)(void *)) audio->header_hack_0;
    }
#endif				/* __ENABLE_HEADER_HACK_0__ */
#if __ENABLE_HEADER_HACK_1__
    if (extra_mask & Mylene1f4_AUDIO_HEADER_HACK_1_MASK) {
	audio_attributes->header_hack_1 = audio->header_hack_1;
    }
#endif				/* __ENABLE_HEADER_HACK_1__ */
#if __ENABLE_HEADER_HACK_2__
    if (extra_mask & Mylene1f4_AUDIO_HEADER_HACK_2_MASK) {
	audio_attributes->header_hack_2 = audio->header_hack_2;
    }
#endif				/* __ENABLE_HEADER_HACK_2__ */
#if __ENABLE_HEADER_HACK_3__
    if (extra_mask & Mylene1f4_AUDIO_HEADER_HACK_3_MASK) {
	audio_attributes->header_hack_3 = audio->header_hack_3;
    }
#endif				/* __ENABLE_HEADER_HACK_3__ */
}


void
Mylene1f4_get_audio_data_size(void *audio, unsigned *audio_data_size)
{
    *audio_data_size = ((struct Mylene1f4_audio *) audio)->audio_data_size;
}


void
Mylene1f4_get_bitrate(void *audio, unsigned *bitrate)
{
    unsigned layer;

    layer = ((struct Mylene1f4_audio *) audio)->layer;
    if (layer) {
	unsigned IDsl4, bitrate_index;

	IDsl4 = ((unsigned char *)
		 &((struct Mylene1f4_audio *) audio)->header)[1] << 1 & 16;
	bitrate_index = ((struct Mylene1f4_audio *) audio)->bitrate_index;
	*bitrate = header_to_bitrate[IDsl4 | bitrate_index][layer - 1];
    } else {
	*bitrate = Mylene1f4_FORBIDDEN;
    }
}


void
Mylene1f4_get_bitrate_index(void *audio, unsigned *bitrate_index)
{
    *bitrate_index = ((struct Mylene1f4_audio *) audio)->bitrate_index;
}


void
Mylene1f4_get_header(void *audio, unsigned *header)
{
    *header = ((struct Mylene1f4_audio *) audio)->header;
}


void
Mylene1f4_get_layer(void *audio, unsigned *layer)
{
    *layer = ((struct Mylene1f4_audio *) audio)->layer;
}


void
Mylene1f4_get_mode(void *audio, unsigned *mode)
{
    *mode = ((struct Mylene1f4_audio *) audio)->mode;
}


void
Mylene1f4_get_mode_extension(void *audio, unsigned *mode_extension)
{
    *mode_extension = ((struct Mylene1f4_audio *) audio)->mode_extension;
}


void
Mylene1f4_get_output(void *audio, unsigned char ***output)
{
    *output = &((struct Mylene1f4_audio *) audio)->output;
}


void
Mylene1f4_get_sampling_frequency(void *audio, unsigned *sampling_frequency)
{
    unsigned IDsl2;

    IDsl2 = ((unsigned char *)
	     &((struct Mylene1f4_audio *) audio)->header)[1] >> 1 & 4;
    *sampling_frequency = header_to_sampling_frequency
	[IDsl2 | ((struct Mylene1f4_audio *) audio)->sampling_frequency];
}


void
Mylene1f4_reset_audio(void *audio_return)
{
    struct Mylene1f4_audio *audio;

    audio = (struct Mylene1f4_audio *) audio_return;
    audio->audio_data[0x0] = (void (*)(struct Mylene1f4_audio *)) 11172;
    audio->audio_data[0x1] = _Mylene1f4_futile_____audio_data_III[0x0];
    audio->audio_data[0x2] = _Mylene1f4_futile_____audio_data__II[0x0];
    audio->audio_data[0x3] = _Mylene1f4_futile_____audio_data___I[0x0];
    audio->audio_data[0x4] = (void (*)(struct Mylene1f4_audio *)) ID_mask;
    audio->audio_data[0x5] = _Mylene1f4_futile_____audio_data_III[0x1];
    audio->audio_data[0x6] = _Mylene1f4_futile_____audio_data__II[0x1];
    audio->audio_data[0x7] = _Mylene1f4_futile_____audio_data___I[0x1];
    audio->audio_data[0x8] = (void (*)(struct Mylene1f4_audio *)) 11172;
    audio->audio_data[0x9] = _Mylene1f4_futile_____audio_data_III[0x2];
    audio->audio_data[0xa] = _Mylene1f4_futile_____audio_data__II[0x2];
    audio->audio_data[0xb] = _Mylene1f4_futile_____audio_data___I[0x2];
    audio->audio_data[0xc] = (void (*)(struct Mylene1f4_audio *)) 11172;
    audio->audio_data[0xd] = _Mylene1f4_futile_____audio_data_III[0x3];
    audio->audio_data[0xe] = _Mylene1f4_futile_____audio_data__II[0x3];
    audio->audio_data[0xf] = _Mylene1f4_futile_____audio_data___I[0x3];
    audio->buffer = NULL;
    audio->dct_64 = Mylene1f4_dct64_0;
    audio->error = 0;
    audio->frame_size = 0;
    audio->header = ~0;
    audio->last_frame = 0;
    audio->layer = ~Layer_III;
    audio->left_frame = 0;
    audio->minimum_useful_size = 64;
    audio->polyphase_synthesis_0 = Mylene1f4_polyphase_synthesis_0;
    audio->polyphase_synthesis_1 = Mylene1f4_polyphase_synthesis_1;
    audio->sample_shift = 1;
    audio->size = 0;
    audio->syncword_offset = 0;
    audio->wrapped_0[0] = 0;
    audio->wrapped_0[1] = 0;
    audio->wrapped_1[0] = 1;
    audio->wrapped_1[1] = 1;
#if __ENABLE_HEADER_HACK_2__
    audio->header_change_mask = 0;
#endif				/* __ENABLE_HEADER_HACK_2__ */
#if __ENABLE_HEADER_HACK_0__
    audio->header_hack_0 = NULL;
#endif				/* __ENABLE_HEADER_HACK_0__ */
#if __ENABLE_HEADER_HACK_1__
    audio->header_hack_1 = NULL;
#endif				/* __ENABLE_HEADER_HACK_1__ */
#if __ENABLE_HEADER_HACK_2__
    audio->header_hack_2 = NULL;
#endif				/* __ENABLE_HEADER_HACK_2__ */
#if __ENABLE_HEADER_HACK_3__
    audio->header_hack_3 = NULL;
#endif				/* __ENABLE_HEADER_HACK_3__ */
#if __ENABLE_HEADER_HACK_2__
    audio->user_header_change_mask = 0xffffffff;
#endif				/* __ENABLE_HEADER_HACK_2__ */
#if __ENABLE_EQUALIZER_0__
    audio->x_equalize_0 = NULL;
#endif				/* __ENABLE_EQUALIZER_0__ */
#if __ENABLE_EQUALIZER_0__
    audio->x_equalizer_0 = NULL;
#endif				/* __ENABLE_EQUALIZER_0__ */
}


void
Mylene1f4_reset_audio_attributes(void *audio_return, unsigned mask,
				 unsigned extra_mask)
{
    struct Mylene1f4_audio *audio;

    audio = (struct Mylene1f4_audio *) audio_return;
    if (mask & Mylene1f4_AUDIO_BUFFER_MASK) {
	_Mylene1f4_set_buffer(audio, NULL);
    }
    if (mask & Mylene1f4_AUDIO_DCT_64_MASK) {
	audio->dct_64 = Mylene1f4_dct64_0;
    }
#if __ENABLE_EQUALIZER_0__
    if (mask & Mylene1f4_AUDIO_EQUALIZER_0_MASK) {
	audio->x_equalizer_0 = NULL;
    }
#endif				/* __ENABLE_EQUALIZER_0__ */
#if __ENABLE_EQUALIZER_0__
    if (mask & Mylene1f4_AUDIO_EQUALIZE_0_MASK) {
	audio->x_equalize_0 = NULL;
    }
#endif				/* __ENABLE_EQUALIZER_0__ */
    if (mask & Mylene1f4_AUDIO_POLYPHASE_SYNTHESIS_0_MASK) {
	audio->polyphase_synthesis_0 = Mylene1f4_polyphase_synthesis_0;
    }
    if (mask & Mylene1f4_AUDIO_POLYPHASE_SYNTHESIS_1_MASK) {
	audio->polyphase_synthesis_1 = Mylene1f4_polyphase_synthesis_1;
    }
    if (mask & Mylene1f4_AUDIO_PROCESS_MASK) {
	SET_DATA___23__67__ab__ef_0(audio, _Mylene1f4_futile____);
	if (audio->audio_data[4]
	    == (void (*)(struct Mylene1f4_audio *)) ID_mask) {
	    SET_DATA__1___5___9___d___0(audio, _Mylene1f4_futile____);
	} else {
	    SET_DATA__1___5___9___d___0(audio, _Mylene1f4_futile____);
	}
    }
    if (mask & Mylene1f4_AUDIO_SAMPLE_SHIFT_MASK) {
	audio->sample_shift = 1;
    }
    if (mask & Mylene1f4_AUDIO_SIZE_MASK) {
	audio->size = 0;
    }
    if (mask & Mylene1f4_AUDIO_SYNCWORD_OFFSET_MASK) {
	audio->syncword_offset = 0;
    }
#if __ENABLE_HEADER_HACK_2__
    if (extra_mask & Mylene1f4_AUDIO_HEADER_CHANGE_MASK) {
	audio->user_header_change_mask = 0xffffffff;
	if (audio->header_hack_2) {
	    audio->header_change_mask = 0xffffffff;
	} else {
	    audio->header_change_mask = 0;
	}
    }
#endif				/* __ENABLE_HEADER_HACK_2__ */
#if __ENABLE_HEADER_HACK_0__
    if (extra_mask & Mylene1f4_AUDIO_HEADER_HACK_0_MASK) {
	audio->header_hack_0 = NULL;
    }
#endif				/* __ENABLE_HEADER_HACK_0__ */
#if __ENABLE_HEADER_HACK_1__
    if (extra_mask & Mylene1f4_AUDIO_HEADER_HACK_1_MASK) {
	audio->header_hack_1 = NULL;
    }
#endif				/* __ENABLE_HEADER_HACK_1__ */
#if __ENABLE_HEADER_HACK_2__
    if (extra_mask & Mylene1f4_AUDIO_HEADER_HACK_2_MASK) {
	audio->header_change_mask = 0;
	audio->header_hack_2 = NULL;
    }
#endif				/* __ENABLE_HEADER_HACK_2__ */
#if __ENABLE_HEADER_HACK_3__
    if (extra_mask & Mylene1f4_AUDIO_HEADER_HACK_3_MASK) {
	audio->header_hack_3 = NULL;
    }
#endif				/* __ENABLE_HEADER_HACK_3__ */
}


void
Mylene1f4_reset_state(void *audio)
{
    Mylene1f4_clear_state(audio);

    ((struct Mylene1f4_audio *) audio)->size = 0;
    ((struct Mylene1f4_audio *) audio)->syncword_offset = 0;
}


void
Mylene1f4_sampling_frequency(struct Mylene1f4_header *header,
			     unsigned *sampling_frequency)
{
    *sampling_frequency = header_to_sampling_frequency
	[header->ID << 2 | header->sampling_frequency];
}


void
Mylene1f4_set_audio_attributes(void *audio_return,
			       AUDIO_ATTRIBUTES *audio_attributes,
			       unsigned mask, unsigned extra_mask)
{
    struct Mylene1f4_audio *audio;

    audio = (struct Mylene1f4_audio *) audio_return;
    if (mask & Mylene1f4_AUDIO_ABORT_DECODING_MASK) {
	audio->abort_decoding = (void (*)(struct Mylene1f4_audio *))
	    audio_attributes->abort_decoding;
    }
    if (mask & Mylene1f4_AUDIO_BUFFER_MASK) {
	_Mylene1f4_set_buffer(audio, audio_attributes->buffer);
    }
    if (mask & Mylene1f4_AUDIO_CORRECT_UNDERFLOW_MASK) {
	audio->optimistique_moi = (void (*)(struct Mylene1f4_audio *))
	    audio_attributes->correct_underflow;
    }
    if (mask & Mylene1f4_AUDIO_DCT_64_MASK) {
	audio->dct_64 = audio_attributes->dct_64;
    }
    if (mask & Mylene1f4_AUDIO_DO_RELEASE_MASK) {
	audio->do_release =
	    (void (*)(struct Mylene1f4_audio *, struct Mylene1f4_release *))
	    audio_attributes->do_release;
    }
#if __ENABLE_EQUALIZER_0__
    if (mask & Mylene1f4_AUDIO_EQUALIZER_0_MASK) {
	audio->x_equalizer_0 = audio_attributes->equalizer_0;
    }
#endif				/* __ENABLE_EQUALIZER_0__ */
#if __ENABLE_EQUALIZER_0__
    if (mask & Mylene1f4_AUDIO_EQUALIZE_0_MASK) {
	audio->x_equalize_0 = audio_attributes->equalize_0;
    }
#endif				/* __ENABLE_EQUALIZER_0__ */
    if (mask & Mylene1f4_AUDIO_OUTPUT_MASK) {
	audio->output = audio_attributes->output;
    }
    if (mask & Mylene1f4_AUDIO_POLYPHASE_BUFFER_MASK) {
	Mylene1f4_real *polyphase;
	unsigned i = 1088;

	audio->polyphase_buffer = audio_attributes->polyphase_buffer;
	polyphase = (Mylene1f4_real *) audio->polyphase_buffer;
	for (; i; i--) {
	    *polyphase++ = 0;
	}
    }
    if (mask & Mylene1f4_AUDIO_POLYPHASE_SYNTHESIS_0_MASK) {
	audio->polyphase_synthesis_0 =
	    (void (*)(struct Mylene1f4_audio *, Mylene1f4_real *, void *,
		      unsigned))
	    audio_attributes->polyphase_synthesis_0;
    }
    if (mask & Mylene1f4_AUDIO_POLYPHASE_SYNTHESIS_1_MASK) {
	audio->polyphase_synthesis_1 =
	    (void (*)(struct Mylene1f4_audio *, Mylene1f4_real *, unsigned,
		      void *, unsigned))
	    audio_attributes->polyphase_synthesis_1;
    }
    if (mask & Mylene1f4_AUDIO_PROCESS_MASK) {
	if (audio_attributes->process == Mylene1f4_4294967291) {
	    SET_DATA___23__67__ab__ef_0(audio, _Mylene1f4_4294967291);
	} else if (audio_attributes->process == Mylene1f4_FUTILE) {
	    SET_DATA___23__67__ab__ef_0(audio, _Mylene1f4_futile____);
	} else if (audio_attributes->process == Mylene1f4_UTTERLY_FUTILE) {
	    SET_DATA___23__67__ab__ef_1(audio, _Mylene1f4_4294967291);
	} else {
	}
	if (audio->audio_data[4]
	    == (void (*)(struct Mylene1f4_audio *)) ID_mask) {
	    if (audio_attributes->process == Mylene1f4_4294967291) {
		SET_DATA__1___5___9___d___0(audio, _Mylene1f4_4294967291);
	    } else if (audio_attributes->process == Mylene1f4_FUTILE) {
		SET_DATA__1___5___9___d___0(audio, _Mylene1f4_futile____);
	    } else if (audio_attributes->process == Mylene1f4_UTTERLY_FUTILE) {
		SET_DATA__1___5___9___d___1(audio, _Mylene1f4_4294967291);
	    } else {
	    }
	} else {
	    if (audio_attributes->process == Mylene1f4_4294967291) {
		SET_DATA__1___5___9___d___0(audio, _Mylene1f4_4294967279);
	    } else if (audio_attributes->process == Mylene1f4_FUTILE) {
		SET_DATA__1___5___9___d___0(audio, _Mylene1f4_futile____);
	    } else if (audio_attributes->process == Mylene1f4_UTTERLY_FUTILE) {
		SET_DATA__1___5___9___d___1(audio, _Mylene1f4_4294967279);
	    } else {
	    }
	}
    }
    if (mask & Mylene1f4_AUDIO_SAMPLE_SHIFT_MASK) {
	audio->sample_shift = audio_attributes->sample_shift;
    }
    if (mask & Mylene1f4_AUDIO_SIZE_MASK) {
	audio->size = audio_attributes->size;
    }
    if (mask & Mylene1f4_AUDIO_SYNCWORD_OFFSET_MASK) {
	audio->syncword_offset = audio_attributes->syncword_offset;
    }
    if (mask & Mylene1f4_AUDIO_TRANSFORM_BUFFER_MASK) {
	Mylene1f4_real *transform;
	unsigned i = 2304;

	audio->transform_buffer = audio_attributes->transform_buffer;
	transform = (Mylene1f4_real *) audio->transform_buffer;
	for (; i; i--) {
	    *transform++ = 0;
	}
    }
#if __ENABLE_HEADER_HACK_2__
    if (extra_mask & Mylene1f4_AUDIO_HEADER_CHANGE_MASK) {
# if WORDS_BIGENDIAN
	audio->user_header_change_mask = audio_attributes->header_change_mask;
	if (audio->header_hack_2) {
	    audio->header_change_mask = audio->user_header_change_mask;
	} else {
	    audio->header_change_mask = 0;
	}
# else
	unsigned user_header_change_mask;

	user_header_change_mask = audio_attributes->header_change_mask;
	user_header_change_mask = (user_header_change_mask & 0xff000000) >> 030
	    | (user_header_change_mask & 0x00ff0000) >> 010
	    | (user_header_change_mask & 0x0000ff00) << 010
	    | (user_header_change_mask & 0x000000ff) << 030;
	audio->user_header_change_mask = user_header_change_mask;
	if (audio->header_hack_2) {
	    audio->header_change_mask = user_header_change_mask;
	} else {
	    audio->header_change_mask = 0;
	}
# endif				/* WORDS_BIGENDIAN */
    }
#endif				/* __ENABLE_HEADER_HACK_2__ */
#if __ENABLE_HEADER_HACK_0__
    if (extra_mask & Mylene1f4_AUDIO_HEADER_HACK_0_MASK) {
	audio->header_hack_0 = (void (*)(struct Mylene1f4_audio *))
	    audio_attributes->header_hack_0;
    }
#endif				/* __ENABLE_HEADER_HACK_0__ */
#if __ENABLE_HEADER_HACK_1__
    if (extra_mask & Mylene1f4_AUDIO_HEADER_HACK_1_MASK) {
	audio->header_hack_1 = audio_attributes->header_hack_1;
    }
#endif				/* __ENABLE_HEADER_HACK_1__ */
#if __ENABLE_HEADER_HACK_2__
    if (extra_mask & Mylene1f4_AUDIO_HEADER_HACK_2_MASK) {
	audio->header_hack_2 = audio_attributes->header_hack_2;
	if (audio->header_hack_2) {
	    audio->header_change_mask = audio->user_header_change_mask;
	} else {
	    audio->header_change_mask = 0;
	}
    }
#endif				/* __ENABLE_HEADER_HACK_2__ */
#if __ENABLE_HEADER_HACK_3__
    if (extra_mask & Mylene1f4_AUDIO_HEADER_HACK_3_MASK) {
	audio->header_hack_3 = audio_attributes->header_hack_3;
    }
#endif				/* __ENABLE_HEADER_HACK_3__ */
}
