/*
 * e1f4.f.h
 * Copyright (C) 2003, 2004, 2005, 2006, 2007  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __Mylene1f4_F_H__
#define __Mylene1f4_F_H__

#include <e1f4.3.h>

/*
 * audio attribute mask definitions
 */
#define Mylene1f4_AUDIO_ABORT_DECODING_MASK \
					(1L << 0x00)
#define Mylene1f4_AUDIO_BUFFER_MASK	(1L << 0x01)
#define Mylene1f4_AUDIO_CORRECT_UNDERFLOW_MASK \
					(1L << 0x02)
#define Mylene1f4_AUDIO_DCT_64_MASK	(1L << 0x03)
#define Mylene1f4_AUDIO_DO_RELEASE_MASK	(1L << 0x04)
#if __ENABLE_EQUALIZER_0__
# define Mylene1f4_AUDIO_EQUALIZER_0_MASK \
					(1L << 0x05)
# define Mylene1f4_AUDIO_EQUALIZE_0_MASK \
					(1L << 0x06)
#endif				/* __ENABLE_EQUALIZER_0__ */
#define Mylene1f4_AUDIO_MINIMUM_USEFUL_SIZE_MASK \
					(1L << 0x07)
#define Mylene1f4_AUDIO_OUTPUT_MASK	(1L << 0x08)
#define Mylene1f4_AUDIO_POLYPHASE_BUFFER_MASK \
					(1L << 0x09)
#define Mylene1f4_AUDIO_POLYPHASE_SYNTHESIS_0_MASK \
					(1L << 0x0a)
#define Mylene1f4_AUDIO_POLYPHASE_SYNTHESIS_1_MASK \
					(1L << 0x0b)
#define Mylene1f4_AUDIO_PROCESS_MASK	(1L << 0x0c)
#define Mylene1f4_AUDIO_SAMPLE_SHIFT_MASK \
					(1L << 0x0d)
#define Mylene1f4_AUDIO_SIZE_MASK	(1L << 0x0e)
#define Mylene1f4_AUDIO_SYNCWORD_OFFSET_MASK \
					(1L << 0x0f)
#define Mylene1f4_AUDIO_TRANSFORM_BUFFER_MASK \
					(1L << 0x10)
#define Mylene1f4_AUDIO_TRANSFORM_SYNTHESIS_MASK \
					(1L << 0x11)

/*
 * bit rate definitions
 */
#define Mylene1f4_FORBIDDEN		0
#define Mylene1f4_FREE_FORMAT		((unsigned) ~0)

/*
 * extra audio attribute mask definitions
 */
#if __ENABLE_HEADER_HACK_2__
# define Mylene1f4_AUDIO_HEADER_CHANGE_MASK \
					(1L << 0x00)
#endif				/* __ENABLE_HEADER_HACK_2__ */
#if __ENABLE_HEADER_HACK_0__
# define Mylene1f4_AUDIO_HEADER_HACK_0_MASK \
					(1L << 0x01)
#endif				/* __ENABLE_HEADER_HACK_0__ */
#if __ENABLE_HEADER_HACK_1__
# define Mylene1f4_AUDIO_HEADER_HACK_1_MASK \
					(1L << 0x02)
#endif				/* __ENABLE_HEADER_HACK_1__ */
#if __ENABLE_HEADER_HACK_2__
# define Mylene1f4_AUDIO_HEADER_HACK_2_MASK \
					(1L << 0x03)
#endif				/* __ENABLE_HEADER_HACK_2__ */
#if __ENABLE_HEADER_HACK_3__
# define Mylene1f4_AUDIO_HEADER_HACK_3_MASK \
					(1L << 0x04)
#endif				/* __ENABLE_HEADER_HACK_3__ */

/*
 * process definitions
 */
#define Mylene1f4_4294967291		0
#define Mylene1f4_FUTILE		1
#define Mylene1f4_UTTERLY_FUTILE	2

/*
 * sampling_frequency definitions
 */
#define Mylene1f4_RESERVED		((unsigned) ~0)

typedef struct Mylene1f4_audio_attributes {
    unsigned minimum_useful_size, process, sample_shift, size, syncword_offset;
#if __ENABLE_HEADER_HACK_2__
    unsigned header_change_mask;
#endif				/* __ENABLE_HEADER_HACK_2__ */
    unsigned char *buffer, *output;
    void (*abort_decoding) (void *), (*correct_underflow) (void *),
	(*dct_64) (Mylene1f4_real *, Mylene1f4_real *, Mylene1f4_real *),
	(*do_release) (void *, struct Mylene1f4_release *),
	*polyphase_buffer,
	(*polyphase_synthesis_0) (void *, Mylene1f4_real *, void *, unsigned),
	(*polyphase_synthesis_1) (void *, Mylene1f4_real *, unsigned, void *,
				  unsigned), *transform_buffer,
	(*transform_synthesis) (void);
#if __ENABLE_EQUALIZER_0__
    void (*equalize_0) (void *, Mylene1f4_real *), *equalizer_0;
#endif				/* __ENABLE_EQUALIZER_0__ */
#if __ENABLE_HEADER_HACK_0__
    void (*header_hack_0) (void *);
#endif				/* __ENABLE_HEADER_HACK_0__ */
#if __ENABLE_HEADER_HACK_1__
    void (*header_hack_1) (struct Mylene1f4_header *);
#endif				/* __ENABLE_HEADER_HACK_1__ */
#if __ENABLE_HEADER_HACK_2__
    void (*header_hack_2) (struct Mylene1f4_header *);
#endif				/* __ENABLE_HEADER_HACK_2__ */
#if __ENABLE_HEADER_HACK_3__
    void (*header_hack_3) (struct Mylene1f4_header *);
#endif				/* __ENABLE_HEADER_HACK_3__ */
} Mylene1f4_audio_attributes;

#endif				/* __Mylene1f4_F_H__ */
