/*
 * h262.0.11172.c
 * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.1.h>
#include <h262.2.h>
#include <h262.3.h>
#include <h262.a.h>

#define INCREMENT_INPUT(system, nbytes) \
    {									      \
	system->size -= nbytes;						      \
	system->buffer += nbytes;					      \
    }

static void pack(struct seek_h262_system *);

static void
pack(struct seek_h262_system *system)
{
    /*
     * pack_start_code                                              32  bslbf
     * '0010'                                                        4  bslbf
     * system_clock_reference[32..30]                                3  bslbf
     * marker_bit                                                    1  bslbf
     * system_clock_reference[29..15]                               15  bslbf
     * marker_bit                                                    1  bslbf
     * system_clock_reference[14..0]                                15  bslbf
     * marker_bit                                                    1  bslbf
     * marker_bit                                                    1  bslbf
     * mux_rate                                                     22  uimsbf
     * marker_bit                                                    1  bslbf
     * if (nextbits() == system_header_start_code) {                    
     *     system_header()                                              
     * }                                                                
     * while (nextbits() == packet_start_code) {                 
     *     packet()                                                 
     * }                                                                
     */
    seek_h262_artificial_pack_header(system);
    while (system_header_start_code < system->start_code) {
	seek_h262_packet(system);
	system->next_start_code(system);
    }
}


void
seek_h262_artificial_pack_header(struct seek_h262_system *system)
{
    if (system->size < 8) {
	system->correct_underflow(system);
    } {
	unsigned *buffer;

	buffer = (unsigned *) system->buffer;
#if WORDS_BIGENDIAN
	if ((*buffer & 0xf1000100) != 0x21000100) {
	    if ((*buffer & 0xf0000000) != 0x20000000) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) system,
		     SYSTEM_FAILED_PACK_MARKER);
	    } else {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) system,
		     SYSTEM_UNSET_MARKER_BITS);
	    }
	}
#else
	if ((*buffer & 0x000100f1) != 0x00010021) {
	    if ((*buffer & 0x000000f0) != 0x00000020) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) system,
		     SYSTEM_FAILED_PACK_MARKER);
	    } else {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) system,
		     SYSTEM_UNSET_MARKER_BITS);
	    }
	}
#endif				/* WORDS_BIGENDIAN */
	buffer++;
#if WORDS_BIGENDIAN
	if ((*buffer & 0x01800001) != 0x01800001) {
	    _seek_h262_abort_decoding
		((struct seek_h262_whatever *) system,
		 SYSTEM_UNSET_MARKER_BITS);
	}
#else
	if ((*buffer & 0x01008001) != 0x01008001) {
	    _seek_h262_abort_decoding
		((struct seek_h262_whatever *) system,
		 SYSTEM_UNSET_MARKER_BITS);
	}
#endif				/* WORDS_BIGENDIAN */
    }
#if __ENABLE_X000001BA___HACK_1__
    if (system->x000001ba___hack_1) {
# define mux_rate			program_mux_rate
	struct seek_h262_x000001ba__ artificial_pack_header;
	unsigned char *buffer;

	buffer = system->buffer;
	artificial_pack_header.decoder = system;
	artificial_pack_header.mux_rate = *(unsigned *) (buffer + 4);
# if WORDS_BIGENDIAN
	artificial_pack_header.mux_rate =
	    (artificial_pack_header.mux_rate >> 1) & 4194304;
# else
	artificial_pack_header.mux_rate =
	    (artificial_pack_header.mux_rate & 0x7f00) << 7
	    | (artificial_pack_header.mux_rate & 0xff0000) >> 9
	    | (artificial_pack_header.mux_rate & 0xfe000000) >> 25;
	artificial_pack_header.system_clock_reference_0 =
	    *(unsigned *) (buffer + 4) >> 3;
# endif				/* WORDS_BIGENDIAN */
	artificial_pack_header.system_clock_reference_0 =
	    (*buffer & 0x08) >> 3;
	artificial_pack_header.system_clock_reference_1 =
	    (*buffer & 0x06) << 035 | buffer[1] << 026
	    | (buffer[2] & 0xfe) << 016 | buffer[3] << 007
	    | (buffer[4] & 0xfe) >> 001;
	system->x000001ba___hack_1(&artificial_pack_header);
    }
#endif				/* __ENABLE_X000001BA___HACK_1__ */
    INCREMENT_INPUT(system, 8);
    system->next_start_code(system);
    if (system->start_code == system_header_start_code) {
	_seek_h262_system_header(system);
	system->next_start_code(system);
    }
}


void
seek_h262_iso11172_stream(struct seek_h262_system *system)
{
    /*
     * do {                                                             
     *     pack()                                                       
     * } while (nextbits() == pack_start_code)                          
     * iso_11172_end_code                                           32  bslbf
     */
    system->next_start_code(system);
    do {
	pack(system);
    } while (system->start_code == pack_start_code);
    if (system->start_code != iso_11172_end_code) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) system, SYSTEM_IMPROPER_START_CODE);
    }
}


void
seek_h262_packet(struct seek_h262_system *system)
{
    /*
     * packet_start_code_prefix                                     24  bslbf
     * stream_id                                                     8  uimsbf
     * packet_length                                                16  uimsbf
     * if (packet_start_code != private_stream_2) {                     
     *     while (nextbits() == '1') {                                  
     *         stuffing_byte                                         8 bslbf
     *     }                                                            
     *     if (nextbits() == '01') {                                    
     *         '01'                                                  2  bslbf
     *         STD_buffer_scale                                      1  bslbf
     *         STD_buffer_size                                      13  uimsbf
     *     }                                                            
     *     if (nextbits() == '0010') {                                  
     *         '0010'                                                4  bslbf
     *         presentation_time_stamp[32..30]                       3  bslbf
     *         marker_bit                                            1  bslbf
     *         presentation_time_stamp[29..15]                      15  bslbf
     *         marker_bit                                            1  bslbf
     *         presentation_time_stamp[14..0]                       15  bslbf
     *         marker_bit                                            1  bslbf
     *     } else if (nextbits() == '0011') {                           
     *         '0011'                                                4  bslbf
     *         presentation_time_stamp[32..30]                       3  bslbf
     *         marker_bit                                            1  bslbf
     *         presentation_time_stamp[29..15]                      15  bslbf
     *         marker_bit                                            1  bslbf
     *         presentation_time_stamp[14..0]                       15  bslbf
     *         marker_bit                                            1  bslbf
     *         '0001'                                                4  bslbf
     *         decoding_time_stamp[32..30]                           3  bslbf
     *         marker_bit                                            1  bslbf
     *         decoding_time_stamp[29..15]                          15  bslbf
     *         marker_bit                                            1  bslbf
     *         decoding_time_stamp[14..0]                           15  bslbf
     *         marker_bit                                            1  bslbf
     *     } else {                                                     
     *         '0000 1111'                                           8  bslbf
     *     }                                                            
     * }                                                                
     * for (i = 0; i < N; i++) {                                        
     *      packet_data_byte                                         8  bslbf
     * }                                                                
     */
#if __ENABLE_X000001FF___HACK_0__
    if (system->x000001ff___hack_0) {
	system->x000001ff___hack_0(system);
    }
#endif				/* __ENABLE_X000001FF___HACK_0__ */
    if (system->size < 2) {
	system->correct_underflow(system);
    } {
	unsigned packet_length;

	{
	    unsigned char *buffer;

	    buffer = system->buffer;
	    packet_length = *buffer << 8 | buffer[1];
	}
	INCREMENT_INPUT(system, 2);
	if (system->start_code != private_stream_2) {
	    do {
		unsigned size;
		unsigned char *buffer;

		if (!system->size) {
		    system->correct_underflow(system);
		}
		buffer = system->buffer;
		size = system->size;
		if (*buffer & 0x80) {
		    do {
			if (!packet_length) {
			    _seek_h262_abort_decoding
				((struct seek_h262_whatever *) system,
				 SYSTEM_INVALID_PACKET_LENGTH);
			}
			packet_length--;
			buffer++;
			size--;
			if (!size) {
			    break;
			}
		    } while (*buffer & 0x80);
		    system->buffer = buffer;
		    if (size) {
			system->size = size;
			break;
		    } else {
			system->size = 0;
		    }
		} else {
		    break;
		}
	    } while (1);
	    if (!system->size) {
		system->correct_underflow(system);
	    }
	    if ((*system->buffer & 0xc0) == 0x40) {
		if (packet_length < 2) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PACKET_LENGTH);
		}
		if (system->size < 2) {
		    system->correct_underflow(system);
		}
		packet_length -= 2;
		INCREMENT_INPUT(system, 2);
	    }
	    if (!system->size) {
		system->correct_underflow(system);
	    }
	    if ((*system->buffer & 0xf0) == 0x20) {
		unsigned char *buffer;

		if (packet_length < 5) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PACKET_LENGTH);
		}
		if (system->size < 5) {
		    system->correct_underflow(system);
		}
		buffer = system->buffer;
		if ((*buffer & 0xf1) != 0x21 || !(buffer[2] & 1)
		    || !(buffer[4] & 1)) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_UNSET_MARKER_BITS);
		}
		packet_length -= 5;
#if __ENABLE_X000001FF___HACK_1__
		if (system->x000001ff___hack_1) {
		    struct seek_h262_x000001ff__ packet;

		    packet.decoder = system;
		    packet.x.x0.PTS_0 = (*buffer & 0x08) >> 3;
		    packet.x.x0.DTS_0 = packet.x.x0.PTS_0;
		    packet.x.x0.PTS_1 = (*buffer & 0x06) << 035
			| buffer[1] << 026 | (buffer[2] & 0xfe) << 016
			| buffer[3] << 007 | (buffer[4] & 0xfe) >> 001;
		    packet.x.x0.DTS_1 = packet.x.x0.PTS_1;
		    system->x000001ff___hack_1(&packet);
		}
#endif				/* __ENABLE_X000001FF___HACK_1__ */
		INCREMENT_INPUT(system, 5);
	    } else if ((*system->buffer & 0xf0) == 0x30) {
		unsigned char *buffer;

		if (packet_length < 10) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PACKET_LENGTH);
		}
		if (system->size < 10) {
		    system->correct_underflow(system);
		}
		buffer = system->buffer;
		if ((*buffer & 0xf1) != 0x31 || !(buffer[2] & 1)
		    || !(buffer[4] & 1) || (buffer[5] & 0xf1) != 0x11
		    || !(buffer[7] & 1) || !(buffer[9] & 1)) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_UNSET_MARKER_BITS);
		}
		packet_length -= 10;
#if __ENABLE_X000001FF___HACK_1__
		if (system->x000001ff___hack_1) {
		    struct seek_h262_x000001ff__ packet;

		    packet.decoder = system;
		    packet.x.x0.PTS_0 = (*buffer & 0x08) >> 3;
		    packet.x.x0.PTS_1 = (*buffer & 0x06) << 035
			| buffer[1] << 026 | (buffer[2] & 0xfe) << 016
			| buffer[3] << 007 | (buffer[4] & 0xfe) >> 001;
		    packet.x.x0.DTS_0 = (buffer[5] & 0x08) >> 3;
		    packet.x.x0.DTS_1 = (buffer[5] & 0x06) << 035
			| buffer[6] << 026 | (buffer[7] & 0xfe) << 016
			| buffer[8] << 007 | (buffer[9] & 0xfe) >> 001;
		    system->x000001ff___hack_1(&packet);
		}
#endif				/* __ENABLE_X000001FF___HACK_1__ */
		INCREMENT_INPUT(system, 10);
	    } else if (*system->buffer == 0x0f) {
		if (!packet_length) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PACKET_LENGTH);
		}
		packet_length--;
		INCREMENT_INPUT(system, 1);
	    } else {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) system,
		     SYSTEM_INVALID_PACKET);
	    }
	}
#if __ENABLE_X000001FF___HACK_2__
	if (system->x000001ff___hack_2) {
	    struct seek_h262_x000001ff__ packet;

	    packet.decoder = system;
	    packet.x.x1.data_size = packet_length;
	    system->x000001ff___hack_2(&packet);
	}
#endif				/* __ENABLE_X000001FF___HACK_2__ */
	system->packet_data(system, packet_length);
    }
}
