/*
 * h262.1.11172.c
 * Copyright (C) 2003, 2004, 2005  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.1.h>
#include <h262.2.h>
#include <h262.3.h>
#include <h262.4.h>
#include <h262.6.h>
#include <h262.a.h>

#define INCREMENT_INPUT(video, nbytes) \
    {									      \
	video->size -= nbytes;						      \
	video->buffer += nbytes;					      \
    }

#if 0
extern const unsigned char _seek_h262_B_2a[];
#endif				/* 0 */
extern const unsigned char _seek_h262_B_2b[], _seek_h262_B_2c[];

static void artificial_extension_and_user_data(struct seek_h262_video *);
static void artificial_sequence_extension(struct seek_h262_video *);
static void artificial_subsequent_sequence_extension(struct seek_h262_video *);
static void motion_backward(struct seek_h262_video *);
static void motion_forward(struct seek_h262_video *);
static void utterly_futile_data(struct seek_h262_video *);

static void group_of_pictures_header(struct seek_h262_video *);
static void picture(struct seek_h262_video *);

static void
artificial_extension_and_user_data(struct seek_h262_video *video)
{
    if (video->start_code == extension_start_code) {
#if __ENABLE_X000001B5___HACK_0__
	if (video->x000001b5___hack_0) {
	    video->x000001b5___hack_0(video);
	}
#endif				/* __ENABLE_X000001B5___HACK_0__ */
	_seek_h262_artificial_extension_or_user_data(video);
    }
    if (video->start_code == user_data_start_code) {
#if __ENABLE_X000001B2___HACK_0__
	if (video->x000001b2___hack_0) {
	    video->x000001b2___hack_0(video);
	}
#endif				/* __ENABLE_X000001B2___HACK_0__ */
	_seek_h262_artificial_extension_or_user_data(video);
    }
}


static void
artificial_sequence_extension(struct seek_h262_video *video)
{
    video->mb_height = (video->vertical_size + 15) >> 4;
    video->mb_width = (video->horizontal_size + 15) >> 4;
    video->macroblocks_count = video->mb_height * video->mb_width;
    video->Irulan_release.flags = 0;
    video->chrominance_line_size = video->mb_width << 3;
    video->luminance_line_size = video->chrominance_line_size << 1;
#if __ENABLE_X000001B5_1_HACK_1__
    if (video->x000001b5_1_hack_1) {
	struct seek_h262_x000001b5_1 sequence_extension;

# if WORDS_BIGENDIAN
	sequence_extension.bit_rate = video->sequence_header[1] >> 14;
# else
	{
	    unsigned bit_rate;

	    bit_rate = video->sequence_header[1];
	    sequence_extension.bit_rate = (bit_rate & 0x000000ff) << 10
		| (bit_rate & 0x0000ff00) >> 6 | (bit_rate & 0x00c00000) >> 22;
	}
# endif				/* WORDS_BIGENDIAN */
	sequence_extension.chroma_format = H2V2;
	sequence_extension.decoder = video;
	sequence_extension.horizontal_size = video->horizontal_size;
# if WORDS_BIGENDIAN
	sequence_extension.load_intra_quantiser_matrix =
	    video->sequence_header[1] & 0x00000002;
	sequence_extension.load_non_intra_quantiser_matrix =
	    video->sequence_header[1] & 0x00000001;
# else
	sequence_extension.load_intra_quantiser_matrix =
	    video->sequence_header[1] & 0x02000000;
	sequence_extension.load_non_intra_quantiser_matrix =
	    video->sequence_header[1] & 0x01000000;
# endif				/* WORDS_BIGENDIAN */
	sequence_extension.progressive_sequence = 1024;
	sequence_extension.synthetic = 1;
	sequence_extension.vertical_size = video->vertical_size;

	_seek_h262_dump_quantiser
	    (video->intra_quantiser_matrix[0],
	     sequence_extension.intra_quantiser_matrix);
	_seek_h262_dump_quantiser
	    (video->non_intra_quantiser_matrix[0],
	     sequence_extension.non_intra_quantiser_matrix);

	video->x000001b5_1_hack_1(&sequence_extension);
    }
#endif				/* __ENABLE_X000001B5_1_HACK_1__ */

    _seek_h262_compute_11172_motion_limit(video);

    _seek_h262_alloc_storage
	(video, video->macroblocks_count << 8, video->macroblocks_count << 6);
}


static void
artificial_subsequent_sequence_extension(struct seek_h262_video *video)
{
#if __ENABLE_X000001B5_1_HACK_2__
    if (video->x000001b5_1_hack_2) {
	struct seek_h262_x000001b5_3 quant_matrix_extension;

	quant_matrix_extension.decoder = video;
# if WORDS_BIGENDIAN
	quant_matrix_extension.load_intra_quantiser_matrix =
	    video->sequence_header[1] & 0x00000002;
	quant_matrix_extension.load_non_intra_quantiser_matrix =
	    video->sequence_header[1] & 0x00000001;
# else
	quant_matrix_extension.load_intra_quantiser_matrix =
	    video->sequence_header[1] & 0x02000000;
	quant_matrix_extension.load_non_intra_quantiser_matrix =
	    video->sequence_header[1] & 0x01000000;
# endif				/* WORDS_BIGENDIAN */

	_seek_h262_dump_quantiser
	    (video->intra_quantiser_matrix[0],
	     quant_matrix_extension.intra_quantiser_matrix);
	_seek_h262_dump_quantiser
	    (video->non_intra_quantiser_matrix[0],
	     quant_matrix_extension.non_intra_quantiser_matrix);

	video->x000001b5_1_hack_2(&quant_matrix_extension);
    }
#endif				/* __ENABLE_X000001B5_1_HACK_2__ */
}


static void
motion_backward(struct seek_h262_video *video)
{
    _seek_h262_motion_component
	(video->backward_f_code, &video->backward_f_code_limit,
	 &video->backward_f_code_mask, &video->backward_f_code_range,
	 &video->backward_B_a_01);
}


static void
motion_forward(struct seek_h262_video *video)
{
    _seek_h262_motion_component
	(video->forward_f_code, &video->forward_f_code_limit,
	 &video->forward_f_code_mask, &video->forward_f_code_range,
	 &video->forward_B_a_01);
}


static void
utterly_futile_data(struct seek_h262_video *video)
{
#if __ENABLE_X000001B8___HACK_0__
    if (video->x000001b8___hack_0) {
	video->x000001b8___hack_0(video);
    }
#endif				/* __ENABLE_X000001B8___HACK_0__ */
    if (video->size < 4) {
	video->optimistique_moi(video);
    }
    if (!(video->buffer[1] & 0x08)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_INVALID_TIME_CODE);
    }
    INCREMENT_INPUT(video, 4);
    video->next_start_code(video);
    artificial_extension_and_user_data(video);
    if (video->start_code != picture_start_code) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_IMPROPER_START_CODE);
    }
}


static void
group_of_pictures_header(struct seek_h262_video *video)
{
    /*
     * group_start_code                                             32  bslbf
     * time_code                                                    25  bslbf
     * closed_gop                                                    1  uimsbf
     * broken_link                                                   1  uimsbf
     * next_start_code()                                                
     * if (nextbits() == extension_start_code) {                        
     *     extension_start_code                                      32 bslbf
     *     while (nextbits() != '0000 0000 0000 0000 0000 0001') {      
     *         sequence_extension_data                                8 
     *     }                                                            
     *     next_start_code()                                            
     * }                                                                
     * if (nextbits() == user_data_start_code) {                        
     *     user_data_start_code                                      32 bslbf
     *     while (nextbits() != '0000 0000 0000 0000 0000 0001') {      
     *         user_data                                              8 
     *     }                                                            
     *     next_start_code()                                            
     * }                                                                
     * do {                                                             
     *     picture()                                                    
     * } while (nextbits() == picture_start_code)                       
     */
    utterly_futile_data(video);
    do {
	picture(video);
    } while (video->start_code == picture_start_code);
}


static void
picture(struct seek_h262_video *video)
{
    /*
     * picture_start_code                                           32  bslbf
     * temporal_reference                                           10  uimsbf
     * picture_coding_type                                           3  uimsbf
     * vbv_delay                                                    16  uimsbf
     * if (picture_coding_type == 2 || picture_coding_type == 3) {      
     *     full_pel_forward_vector                                   1  
     *     forward_f_code                                            3  uimsbf
     * }                                                                
     * if (picture_coding_type == 3) {                                  
     *     full_pel_backward_vector                                  1  
     *     backward_f_code                                           3  uimsbf
     * }                                                                
     * while (nextbits() == '1') {                                      
     *     extra_bit_picture                                         1  "1"
     *     extra_information_picture                                 8  
     * }                                                                
     * extra_bit_picture                                             1  "0"
     * next_start_code()                                                
     * if (nextbits() == extension_start_code) {                        
     *     extension_start_code                                      32 bslbf
     *     while (nextbits() != '0000 0000 0000 0000 0000 0001') {      
     *         sequence_extension_data                                8 
     *     }                                                            
     *     next_start_code()                                            
     * }                                                                
     * if (nextbits() == user_data_start_code) {                        
     *     user_data_start_code                                      32 bslbf
     *     while (nextbits() != '0000 0000 0000 0000 0000 0001') {      
     *         user_data                                              8 
     *     }                                                            
     *     next_start_code()                                            
     * }                                                                
     * do {                                                             
     *    slice()                                                       
     * } while (nextbits() == slice_start_code)                         
     * next_start_code()                                                
     */
#if __ENABLE_X00000100___HACK_1__
    unsigned temporal_reference;
#endif				/* __ENABLE_X00000100___HACK_1__ */

#if __ENABLE_X00000100___HACK_0__
    if (video->x00000100___hack_0) {
	video->x00000100___hack_0(video);
    }
#endif				/* __ENABLE_X00000100___HACK_0__ */

    if (video->size < 5) {
	video->optimistique_moi(video);
    } {
	unsigned mask, nbytes;
	unsigned char *buffer;

	buffer = video->buffer - 4;
	{
	    unsigned picture_coding_type;

	    picture_coding_type = buffer[5] & 0x38;
	    if (!picture_coding_type || 0x20 < picture_coding_type) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_INVALID_PICTURE_CODING_TYPE);
	    }
#if WORDS_BIGENDIAN
	    temporal_reference = *((unsigned *) buffer + 1) >> 22;
#else
	    temporal_reference = buffer[4] << 2 | buffer[5] >> 6;
#endif				/* WORDS_BIGENDIAN */
	    if (picture_coding_type == 0x10) {
		video->forward_f_code = (buffer[7] & 3) << 1 | buffer[8] >> 7;
		if (!video->forward_f_code) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video, 0);
		}
		video->forward_f_code--;
		motion_forward(video);
		if (buffer[7] & 0x04) {
		    video->forward_f_code_range--;
		    video->full_pel_forward_vector = 1;
		} else {
		    video->full_pel_forward_vector = 0;
		}
		mask = 0x40;
		nbytes = 5;
	    } else if (picture_coding_type == 0x18) {
		video->forward_f_code = (buffer[7] & 3) << 1 | buffer[8] >> 7;
		video->backward_f_code = (buffer[8] & 56) >> 3;
		if (!video->forward_f_code || !video->backward_f_code) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video, 0);
		}
		video->backward_f_code--;
		motion_backward(video);
		if (buffer[8] & 0x40) {
		    video->backward_f_code_range--;
		    video->full_pel_backward_vector = 1;
		} else {
		    video->full_pel_backward_vector = 0;
		}
		video->forward_f_code--;
		motion_forward(video);
		if (buffer[7] & 0x04) {
		    video->forward_f_code_range--;
		    video->full_pel_forward_vector = 1;
		} else {
		    video->full_pel_forward_vector = 0;
		}
		mask = 0x04;
		nbytes = 5;
	    } else {
		mask = 0x04;
		nbytes = 4;
	    }
	    switch (picture_coding_type) {
	    case 0x08:
#if 0
		video->B_2345678_01 = (unsigned char *) _seek_h262_B_2a;
#endif				/* 0 */
		video->slice_minimum_useful_size = intra_coded_MACROBLOCK_SIZE;
		break;
	    case 0x10:
		video->B_2345678_01 = (unsigned char *) _seek_h262_B_2b;
#if __Berkeley_FUCK_UP__
		video->block_0_0[0] = video->block_0_0[1];
		video->block_0_3[0] = video->block_0_3[1];
#endif				/* __Berkeley_FUCK_UP__ */
		video->slice_minimum_useful_size =
		    predictive_coded_MACROBLOCK_SIZE;
		break;
	    case 0x18:
		video->B_2345678_01 = (unsigned char *) _seek_h262_B_2c;
#if __Berkeley_FUCK_UP__
		video->block_0_0[0] = video->block_0_0[2];
		video->block_0_3[0] = video->block_0_3[2];
#endif				/* __Berkeley_FUCK_UP__ */
		video->slice_minimum_useful_size =
		    bidirectionally_predictive_coded_MACROBLOCK_SIZE;
		break;
	    case 0x20:
		video->slice_minimum_useful_size =
		    dc_intra_coded_MACROBLOCK_SIZE;
	    }
	    video->picture_coding_type = picture_coding_type >> 3;
	}
	buffer += nbytes + 3;
	while (*buffer & mask) {
	    mask >>= 1;
	    if (!mask) {
		buffer += 2;
		mask = 0x80;
		nbytes += 2;
	    } else {
		buffer++;
		nbytes++;
	    }
	    if (video->size < nbytes) {
		INCREMENT_INPUT(video, nbytes - 1);
		video->optimistique_moi(video);
		buffer = video->buffer;
		nbytes = 1;
	    }
	}
	INCREMENT_INPUT(video, nbytes);
    }
#if __ENABLE_X00000100___HACK_1__
    if (video->x00000100___hack_1) {
	struct seek_h262_x00000100__ picture_header;

	picture_header.decoder = video;
	picture_header.picture_coding_type = video->picture_coding_type;
	picture_header.temporal_reference = temporal_reference;

	if (video->picture_coding_type == predictive_coded) {
	    picture_header.forward_f_code = video->forward_f_code;
	    picture_header.full_pel_forward_vector =
		video->full_pel_forward_vector;
	} else if (video->picture_coding_type
		   == bidirectionally_predictive_coded) {
	    picture_header.backward_f_code = video->backward_f_code;
	    picture_header.forward_f_code = video->forward_f_code;
	    picture_header.full_pel_backward_vector =
		video->full_pel_backward_vector;
	    picture_header.full_pel_forward_vector =
		video->full_pel_forward_vector;
	}

	video->x00000100___hack_1(&picture_header);
    }
#endif				/* __ENABLE_X00000100___HACK_1__ */
    {
	unsigned char *base_address;

	base_address = video->Mylene_release.Y;
	video->blocks[0].base_address = base_address;
	video->blocks[1].base_address = base_address + 8;
	base_address += video->mb_width << 7;
	video->blocks[2].base_address = base_address;
	video->blocks[3].base_address = base_address + 8;
	video->blocks[4].base_address = video->Mylene_release.Cb;
	video->blocks[5].base_address = video->Mylene_release.Cr;
    }
    video->next_start_code(video);
    if (video->picture_coding_type == bidirectionally_predictive_coded) {
	video->______skipped_macroblocks_x__ =
	    video->frame_skipped_macroblocks_x18;
    } else if (video->picture_coding_type == predictive_coded) {
	video->______skipped_macroblocks_x__ =
	    video->frame_skipped_macroblocks_x10;
    } else {
    }
    artificial_extension_and_user_data(video);
    {
	struct seek_h262_release swap_release;

	if (video->picture_coding_type != bidirectionally_predictive_coded) {
	    swap_release = video->Iran_release;
	    video->Iran_release = video->Irulan_release;
	    if (video->Iran_release.flags) {
		video->do_release(video, &video->Iran_release);
	    }
	}
#if WORDS_BIGENDIAN
	if (video->start_code < 0x00000101 || 0x000001af < video->start_code) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				      VIDEO_IMPROPER_START_CODE);
	}
#else
	if (video->start_code < 0x01010000 || 0xaf010000 < video->start_code) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				      VIDEO_IMPROPER_START_CODE);
	}
#endif				/* WORDS_BIGENDIAN */
	video->minimum_useful_size = 4;
#if __ENABLE_X000001AF___HACK_x__
	if (video->x000001af___hack_X) {
	    video->x000001af___hack_X(video);
	} else {
#endif				/* __ENABLE_X000001AF___HACK_x__ */
	    unsigned slice_select;

	    slice_select = video->picture_coding_type;
#if WORDS_BIGENDIAN
	    slice_select |= (~video->sequence_header[1] & 1) << 2;
#else
	    slice_select |= (~video->sequence_header[1] & 16777216) >> 22;
#endif				/* WORDS_BIGENDIAN */
#if __ENABLE_X000001AF___HACK_0__
	    if (video->x000001af___hack_0) {
# if WORDS_BIGENDIAN
		do {
		    video->x000001af___hack_0(video);
		    video->slices[slice_select](video);
		} while (0x00000100 < video->start_code
			 && video->start_code < 0x000001b0);
# else
		do {
		    video->x000001af___hack_0(video);
		    video->slices[slice_select](video);
		} while (0x01000000 < video->start_code
			 && video->start_code < 0xb0010000);
# endif				/* WORDS_BIGENDIAN */
	    } else {
#endif				/* __ENABLE_X000001AF___HACK_0__ */
		video->slices[slice_select](video);
#if __ENABLE_X000001AF___HACK_0__
	    }
#endif				/* __ENABLE_X000001AF___HACK_0__ */
#if __ENABLE_X000001AF___HACK_x__
	}
#endif				/* __ENABLE_X000001AF___HACK_x__ */
	video->minimum_useful_size = 64;
	if (video->picture_coding_type == bidirectionally_predictive_coded) {
	    video->Mylene_release.flags = bidirectionally_predictive_coded;
	    video->do_release(video, &video->Mylene_release);
	} else {
	    video->Mylene_release.flags = video->picture_coding_type;
	    video->Irulan_release = video->Mylene_release;
	    video->Mylene_release = swap_release;
	}
    }
}


void
_seek_h262_next_11172_x00000100(struct seek_h262_video *video)
{
    _seek_h262_set_11172_video_attributes(video);
    artificial_sequence_extension(video);
    artificial_extension_and_user_data(video);
    if (video->start_code != group_start_code) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_IMPROPER_START_CODE);
    }
    utterly_futile_data(video);
}


void
_seek_h262_video_sequence_11172(struct seek_h262_video *video)
{
    /*
     * next_start_code()                                                
     * do {                                                             
     *     sequence_header()                                            
     *     do {                                                         
     *         group_of_pictures_header()                               
     *     } while (nextbits() == group_start_code)                     
     * } while (nextbits() != sequence_end_code)                        
     * sequence_end_code                                            32  bslbf
     */
    _seek_h262_set_11172_video_attributes(video);
    artificial_sequence_extension(video);
    artificial_extension_and_user_data(video);
    do {
	if (video->start_code != group_start_code) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				      VIDEO_IMPROPER_START_CODE);
	}
	do {
	    group_of_pictures_header(video);
	} while (video->start_code == group_start_code);
	if (video->start_code == sequence_end_code) {
	    break;
	}
	seek_h262_subsequent_sequence_header(video);
	artificial_subsequent_sequence_extension(video);
	artificial_extension_and_user_data(video);
    } while (1);
    if (video->Irulan_release.flags) {
	video->do_release(video, &video->Irulan_release);
    }
}


void
seek_h262_next_11172_x00000100(struct seek_h262_video *video)
{
    picture(video);
    if (video->start_code == sequence_end_code) {
    } else if (video->start_code != picture_start_code) {
	if (video->start_code != group_start_code) {
	    seek_h262_subsequent_sequence_header(video);
	    artificial_subsequent_sequence_extension(video);
	    artificial_extension_and_user_data(video);
	    if (video->start_code != group_start_code) {
		_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
					  VIDEO_IMPROPER_START_CODE);
	    }
	}
	utterly_futile_data(video);
    }
}
