/*
 * h262.1.c
 * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.1.h>
#include <h262.2.h>
#include <h262.3.h>
#include <h262.a.h>

#define INCREMENT_INPUT(system, nbytes) \
    {									      \
	system->size -= nbytes;						      \
	system->buffer += nbytes;					      \
    }

static void pack(struct seek_h262_system *);

static void
pack(struct seek_h262_system *system)
{
    /*
     * pack_header()                                                    
     * while (nextbits() == packet_start_code_prefix) {                 
     *     PES_packet()                                                 
     * }                                                                
     */
    seek_h262_pack_header(system);
    while (system_header_start_code < system->start_code) {
	seek_h262_PES_packet(system);
	system->next_start_code(system);
    }
}


void
_seek_h262_system_header(struct seek_h262_system *system)
{
    /*
     * system_header_start_code                                     32  bslbf
     * header_length                                                16  uimsbf
     * marker_bit                                                    1  bslbf
     * rate_bound                                                   22  uimsbf
     * marker_bit                                                    1  bslbf
     * audio_bound                                                   6  uimsbf
     * fixed_flag                                                    1  bslbf
     * CSPS_flag                                                     1  bslbf
     * system_audio_lock_flag                                        1  bslbf
     * system_video_lock_flag                                        1  bslbf
     * marker_bit                                                    1  bslbf
     * video_bound                                                   5  uimsbf
     * reserved_byte                                                 8  bslbf
     * while (next_bits() == '1') {                                     
     *     stream_id                                                 8  uimsbf
     *     '11'                                                      2  bslbf
     *     P-STD_buffer_bound_scale                                  1  bslbf
     *     P-STD_buffer_size_bound                                  13  uimsbf
     * }                                                                
     */
    unsigned header_length;

    if (system->size < 8) {
	system->correct_underflow(system);
    } {
	unsigned *buffer;

	buffer = (unsigned *) system->buffer;
#if WORDS_BIGENDIAN
	if ((*buffer & 0x00008000) != 0x00008000) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) system,
				      SYSTEM_UNSET_MARKER_BIT);
	}
#else
	if ((*buffer & 0x00800000) != 0x00800000) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) system,
				      SYSTEM_UNSET_MARKER_BIT);
	}
#endif				/* WORDS_BIGENDIAN */
#if WORDS_BIGENDIAN
	header_length = *buffer >> 16;
#else
	header_length = (*buffer & 0xff) << 16 | (*buffer & 0xff00) >> 8;
#endif				/* WORDS_BIGENDIAN */
	header_length += 2;
	if (header_length < 8) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) system,
				      SYSTEM_INVALID_HEADER_LENGTH);
	}
	buffer++;
#if 0
# if WORDS_BIGENDIAN
	if ((*buffer & 0x010000ff) != 0x010000ff) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) system,
				      ...);
	}
# else
	if ((*buffer & 0xff000001) != 0xff000001) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) system,
				      ...);
	}
# endif				/* WORDS_BIGENDIAN */
#else
# if WORDS_BIGENDIAN
	if (!(*buffer & 0x01000000)) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) system,
				      SYSTEM_UNSET_MARKER_BIT);
	}
# else
	if (!(*buffer & 0x00000001)) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) system,
				      SYSTEM_UNSET_MARKER_BIT);
	}
# endif				/* WORDS_BIGENDIAN */
#endif				/* 0 */
    }
#if __ENABLE_X000001BB___HACK_1__
    if (system->x000001bb___hack_1) {
	struct seek_h262_x000001bb__ system_header;
	unsigned char *buffer;

	buffer = system->buffer;
	system_header.decoder = system;
	system_header.x.x0.CSPS_flag = buffer[5] & 1;
	system_header.x.x0.audio_bound = buffer[5] >> 2;
	system_header.x.x0.fixed_flag = buffer[5] & 2;
	system_header.x.x0.rate_bound = *(unsigned *) (buffer + 1);
# if WORDS_BIGENDIAN
	system_header.x.x0.rate_bound =
	    (system_header.x.x0.rate_bound >> 1) & 4194304;
# else
	system_header.x.x0.rate_bound =
	    (system_header.x.x0.rate_bound & 0x7f00) << 7
	    | (system_header.x.x0.rate_bound & 0xff0000) >> 9
	    | (system_header.x.x0.rate_bound & 0xfe000000) >> 25;
# endif				/* WORDS_BIGENDIAN */
	system_header.x.x0.system_audio_lock_flag = buffer[6] & 0x80;
	system_header.x.x0.system_video_lock_flag = buffer[6] & 0x40;
	system_header.x.x0.video_bound = buffer[6] & 31;
	system->x000001bb___hack_1(&system_header);
    }
#endif				/* __ENABLE_X000001BB___HACK_1__ */
#if __ENABLE_X000001BB___HACK_2__
    if (system->x000001bb___hack_2) {
	INCREMENT_INPUT(system, 8);
	header_length -= 8;
	while (2 < header_length) {
	    unsigned char *buffer;

	    if (system->size < 3) {
		system->correct_underflow(system);
	    }
	    buffer = system->buffer;
	    if (*buffer & 0x80) {
		struct seek_h262_x000001bb__ system_header;

		system_header.decoder = system;
		system_header.x.x1.stream_id = *buffer;
		system_header.x.x1.PhyphenSTD_buffer_bound_scale =
		    buffer[1] & 0x20;
		system_header.x.x1.PhyphenSTD_buffer_size_bound =
		    (buffer[1] & 0x1f) << 8 | buffer[2];
		system->x000001bb___hack_2(&system_header);
		INCREMENT_INPUT(system, 3);
		header_length -= 3;
	    } else {
		break;
	    }
	}
    }
#endif				/* __ENABLE_X000001BB___HACK_2__ */
    if (system->size < header_length) {
	system->seek_stream(system, header_length);
    } else {
	INCREMENT_INPUT(system, header_length);
    }
}


void
_seek_h262_xprogram_stream_map(struct seek_h262_system *system)
{
    /*
     * packet_start_code_prefix                                     24  bslbf
     * map_stream_id                                                 8  uimsbf
     * program_stream_map_length                                    16  uimsbf
     * current_next_indicator                                        1  bslbf
     * reserved                                                      2  bslbf
     * program_stream_map_version                                    5  uimsbf
     * reserved                                                      7  bslbf
     * marker_bit                                                    1  bslbf
     * program_stream_info_length                                   16  uimsbf
     * for (i = 0; i < N; i++) {                                        
     *     descriptor();                                                
     * }                                                                
     * elementary_stream_map_length                                 16  uimsbf
     * for (i = 0; i < N; i++) {                                        
     *     stream_type                                               8  uimsbf
     *     elementary_stream_id                                      8  uimsbf
     *     elementary_stream_info_length                            16  uimsbf
     *     for (i = 0; i < N; i++) {                                    
     *         descriptor();                                            
     *     }                                                            
     * }                                                                
     * CRC_32                                                       32  rpchof
     */
    unsigned program_stream_map_length = 2;

    if (system->size < 6) {
	system->correct_underflow(system);
    }
    {
	unsigned char *buffer;

	buffer = system->buffer;
	program_stream_map_length += *buffer << 8 | buffer[1];
	if ((buffer[2] & 0x60) != 0x60 || buffer[3] != 0xff) {
	    _seek_h262_abort_decoding
		((struct seek_h262_whatever *) system,
		 SYSTEM_INVALID_PROGRAM_STREAM_MAP_PES_PACKET);
	}
#if __ENABLE_X000001BC___HACK_1__
	if (system->x000001bc___hack_1) {
	    struct seek_h262_x000001bc__ xprogram_stream_map;

	    xprogram_stream_map.decoder = system;
	    xprogram_stream_map.x.x0.current_next_indicator = buffer[2] & 0x80;
	    xprogram_stream_map.x.x0.program_stream_map_version =
		buffer[2] & 0x1f;
	    system->x000001bc___hack_1(&xprogram_stream_map);
	}
#endif				/* __ENABLE_X000001BC___HACK_1__ */
	{
	    unsigned program_stream_info_length;

	    program_stream_info_length = buffer[4] << 8 | buffer[5];
	    if (program_stream_map_length < program_stream_info_length + 12) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) system,
		     SYSTEM_INVALID_PROGRAM_STREAM_MAP_PES_PACKET);
	    }
	    program_stream_map_length -= program_stream_info_length + 6;
	    INCREMENT_INPUT(system, 6);
#if __ENABLE_X000001BC___HACK_2__
	    if (system->x000001bc___hack_2) {
		while (program_stream_info_length) {
		    struct seek_h262_x000001bc__ xprogram_stream_map;
		    unsigned descriptor_length;
		    unsigned char *data;

		    if (system->size < 2) {
			system->correct_underflow(system);
		    }
		    buffer = system->buffer;
		    descriptor_length = buffer[1];
		    if (program_stream_info_length < descriptor_length + 2) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_INVALID_PROGRAM_STREAM_MAP_PES_PACKET);
		    }
		    program_stream_info_length -= descriptor_length + 2;
		    INCREMENT_INPUT(system, 2);
		    data = xprogram_stream_map.x.x1.data;
		    xprogram_stream_map.decoder = system;
		    xprogram_stream_map.x.x1.descriptor_length =
			descriptor_length;
		    xprogram_stream_map.x.x1.descriptor_tag = *buffer;
		    while (descriptor_length) {
			unsigned transfer;

			if (!system->size) {
			    system->correct_underflow(system);
			}
			transfer = system->size < descriptor_length
			    ? system->size : descriptor_length;
			descriptor_length -= transfer;
			buffer = system->buffer;
			INCREMENT_INPUT(system, transfer);
			while (transfer--) {
			    *data++ = *buffer++;
			}
		    }
		    system->x000001bc___hack_2(&xprogram_stream_map);
		}
	    } else {
#endif				/* __ENABLE_X000001BC___HACK_2__ */
		if (system->size < program_stream_info_length) {
		    system->seek_stream(system, program_stream_info_length);
		} else {
		    INCREMENT_INPUT(system, program_stream_info_length);
		}
#if __ENABLE_X000001BC___HACK_2__
	    }
#endif				/* __ENABLE_X000001BC___HACK_2__ */
	}
    }
    if (system->size < 2) {
	system->correct_underflow(system);
    }
    {
	unsigned char *buffer;

	buffer = system->buffer;
	if (program_stream_map_length != (*buffer << 8 | buffer[1]) + 6) {
	    _seek_h262_abort_decoding
		((struct seek_h262_whatever *) system,
		 SYSTEM_INVALID_PROGRAM_STREAM_MAP_PES_PACKET);
	}
	INCREMENT_INPUT(system, 2);
	program_stream_map_length -= 2;
    }
    while (4 < program_stream_map_length) {
	if (system->size < 4) {
	    system->correct_underflow(system);
	}
	{
	    unsigned elementary_stream_info_length;
	    unsigned char *buffer;

	    buffer = system->buffer;
	    elementary_stream_info_length = buffer[2] << 8 | buffer[3];
	    if (program_stream_map_length
		< elementary_stream_info_length + 8) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) system,
		     SYSTEM_INVALID_PROGRAM_STREAM_MAP_PES_PACKET);
	    }
#if __ENABLE_X000001BC___HACK_3__
	    if (system->x000001bc___hack_3) {
		struct seek_h262_x000001bc__ xprogram_stream_map;

		xprogram_stream_map.decoder = system;
		xprogram_stream_map.x.x2.elementary_stream_id = buffer[1];
		xprogram_stream_map.x.x2.stream_type = *buffer;
		system->x000001bc___hack_3(&xprogram_stream_map);
	    }
#endif				/* __ENABLE_X000001BC___HACK_3__ */
	    program_stream_map_length -= elementary_stream_info_length + 4;
	    INCREMENT_INPUT(system, 4);
#if __ENABLE_X000001BC___HACK_4__
	    if (system->x000001bc___hack_4) {
		while (elementary_stream_info_length) {
		    struct seek_h262_x000001bc__ xprogram_stream_map;
		    unsigned descriptor_length;
		    unsigned char *data;

		    if (system->size < 2) {
			system->correct_underflow(system);
		    }
		    buffer = system->buffer;
		    descriptor_length = buffer[1];
		    if (elementary_stream_info_length
			< descriptor_length + 2) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_INVALID_PROGRAM_STREAM_MAP_PES_PACKET);
		    }
		    elementary_stream_info_length -= descriptor_length + 2;
		    INCREMENT_INPUT(system, 2);
		    data = xprogram_stream_map.x.x3.data;
		    xprogram_stream_map.decoder = system;
		    xprogram_stream_map.x.x3.descriptor_length =
			descriptor_length;
		    xprogram_stream_map.x.x3.descriptor_tag = *buffer;
		    while (descriptor_length) {
			unsigned transfer;

			if (!system->size) {
			    system->correct_underflow(system);
			}
			transfer = system->size < descriptor_length
			    ? system->size : descriptor_length;
			descriptor_length -= transfer;
			buffer = system->buffer;
			INCREMENT_INPUT(system, transfer);
			while (transfer--) {
			    *data++ = *buffer++;
			}
		    }
		    system->x000001bc___hack_4(&xprogram_stream_map);
		}
	    } else {
#endif				/* __ENABLE_X000001BC___HACK_4__ */
		if (system->size < elementary_stream_info_length) {
		    system->seek_stream(system, elementary_stream_info_length);
		} else {
		    INCREMENT_INPUT(system, elementary_stream_info_length);
		}
#if __ENABLE_X000001BC___HACK_4__
	    }
#endif				/* __ENABLE_X000001BC___HACK_4__ */
	}
    }
    if (system->size < 4) {
	system->seek_stream(system, 4);
    } else {
	INCREMENT_INPUT(system, 4);
    }
}


void
seek_h262_pack_header(struct seek_h262_system *system)
{
    /*
     * pack_start_code                                              32  bslbf
     * '01'                                                          2  bslbf
     * system_clock_reference_base[32..30]                           3  bslbf
     * marker_bit                                                    1  bslbf
     * system_clock_reference_base[29..15]                          15  bslbf
     * marker_bit                                                    1  bslbf
     * system_clock_reference_base[14..0]                           15  bslbf
     * marker_bit                                                    1  bslbf
     * system_clock_reference_extension                              9  uimsbf
     * marker_bit                                                    1  bslbf
     * program_mux_rate                                             22  uimsbf
     * marker_bit                                                    1  bslbf
     * marker_bit                                                    1  bslbf
     * reserved                                                      5  bslbf
     * pack_stuffing_length                                          3  uimsbf
     * for (i = 0; i < pack_stuffing_length; i++) {                     
     *     stuffing_byte                                             8  bslbf
     * }                                                                
     * if (nextbits() == system_header_start_code) {                    
     *     system_header()                                              
     * }                                                                
     */
    if (system->size < 10) {
	system->correct_underflow(system);
    } {
	unsigned *buffer;

	buffer = (unsigned *) system->buffer;
#if WORDS_BIGENDIAN
	if ((*buffer & 0xc4000400) != 0x44000400) {
	    if ((*(unsigned char *) buffer & 0xc0) != 0x40) {
		_seek_h262_abort_decoding((struct seek_h262_whatever *) system,
					  SYSTEM_FAILED_PACK_HEADER_MARKER);
	    }
	    if ((*buffer & 0x04000400) != 0x04000400) {
		_seek_h262_abort_decoding((struct seek_h262_whatever *) system,
					  SYSTEM_UNSET_MARKER_BITS);
	    }
	}
#else
	if ((*buffer & 0x000400c4) != 0x00040044) {
	    if ((*(unsigned char *) buffer & 0xc0) != 0x40) {
		_seek_h262_abort_decoding((struct seek_h262_whatever *) system,
					  SYSTEM_FAILED_PACK_HEADER_MARKER);
	    }
	    if ((*buffer & 0x00040004) != 0x00040004) {
		_seek_h262_abort_decoding((struct seek_h262_whatever *) system,
					  SYSTEM_UNSET_MARKER_BITS);
	    }
	}
#endif				/* WORDS_BIGENDIAN */
	buffer++;
#if WORDS_BIGENDIAN
	if ((*buffer & 0x04010000) != 0x04010000) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) system,
				      SYSTEM_UNSET_MARKER_BITS);
	}
#else
	if ((*buffer & 0x00000104) != 0x00000104) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) system,
				      SYSTEM_UNSET_MARKER_BITS);
	}
#endif				/* WORDS_BIGENDIAN */
	buffer++;
	if ((*(unsigned char *) buffer & 0x03) != 0x03) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) system,
				      SYSTEM_UNSET_MARKER_BITS);
	}
    }
#if __ENABLE_X000001BA___HACK_1__
    if (system->x000001ba___hack_1) {
	struct seek_h262_x000001ba__ pack_header;
	unsigned char *buffer;

	buffer = system->buffer;
	pack_header.decoder = system;
	pack_header.program_mux_rate = *(unsigned *) (buffer + 5);
# if WORDS_BIGENDIAN
	pack_header.program_mux_rate =
	    (pack_header.program_mux_rate >> 2) & 4194304;
# else
	pack_header.program_mux_rate =
	    (pack_header.program_mux_rate & 0xff00) << 6
	    | (pack_header.program_mux_rate & 0xff0000) >> 10
	    | (pack_header.program_mux_rate & 0xfc000000) >> 26;
# endif				/* WORDS_BIGENDIAN */
	pack_header.system_clock_reference_0 = (buffer[4] & 0x03) << 010
	    | (buffer[5] & 0xfe) | (*buffer & 0x20) >> 5;
	pack_header.system_clock_reference_1 = (*buffer & 0x18) << 033
	    | (*buffer & 0x03) << 034 | buffer[1] << 024
	    | (buffer[2] & 0xf8) << 014 | (buffer[2] & 0x03) << 015
	    | buffer[3] << 005 | (buffer[4] & 0xf8) >> 003;
	system->x000001ba___hack_1(&pack_header);
    }
#endif				/* __ENABLE_X000001BA___HACK_1__ */
    INCREMENT_INPUT(system, 10);
    {
	unsigned stuffing_byte;

	stuffing_byte = system->buffer[-1] & 7;
	if (stuffing_byte) {
	    unsigned char *buffer;

	    if (system->size < stuffing_byte) {
		system->correct_underflow(system);
	    }
	    buffer = system->buffer;
	    system->size -= stuffing_byte;
	    for (; stuffing_byte; stuffing_byte--) {
		if (~*buffer++) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_FAILED_STUFFING_BYTE);
		}
	    }
	    system->buffer = buffer;
	}
    }
    system->next_start_code(system);
    if (system->start_code == system_header_start_code) {
	_seek_h262_system_header(system);
	system->next_start_code(system);
    }
}


void
seek_h262_MPEG2_program_stream(struct seek_h262_system *system)
{
    /*
     * do {                                                             
     *     pack()                                                       
     * } while (nextbits() == pack_start_code)                          
     * MPEG_program_end_code                                        32  bslbf
     */
    system->next_start_code(system);
    do {
	pack(system);
    } while (system->start_code == pack_start_code);
    if (system->start_code != MPEG_program_end_code) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) system, SYSTEM_IMPROPER_START_CODE);
    }
}


void
seek_h262_PES_packet(struct seek_h262_system *system)
{
    /*
     * packet_start_code_prefix                                     24  bslbf
     * stream_id                                                     8  uimsbf
     * PES_packet_length                                            16  uimsbf
     * if (stream_id != private_stream_2 && stream_id != padding_stream) {
     *     '10'                                                      2  bslbf
     *     PES_scrambling_control                                    2  bslbf
     *     PES_priority                                              1  bslbf
     *     data_alignment_indicator                                  1  bslbf
     *     copyright                                                 1  bslbf
     *     original_or_copy                                          1  bslbf
     *     PTS_DTS_flags                                             2  bslbf
     *     ESCR_flag                                                 1  bslbf
     *     ES_rate_flag                                              1  bslbf
     *     DSM_trick_mode_flag                                       1  bslbf
     *     additional_copy_flag                                      1  bslbf
     *     PES_CRC_flag                                              1  bslbf
     *     PES_extension_flag                                        1  bslbf
     *     PES_header_data_length                                    8  uimsbf
     *     if (PTS_DTS_flags == '10') {                                 
     *         '0010'                                                4  bslbf
     *         PTS[32..30]                                           3  bslbf
     *         marker_bit                                            1  bslbf
     *         PTS[29..15]                                          15  bslbf
     *         marker_bit                                            1  bslbf
     *         PTS[14..0]                                           15  bslbf
     *         marker_bit                                            1  bslbf
     *     }                                                            
     *     if (PTS_DTS_flags == '11') {                                 
     *         '0011'                                                4  bslbf
     *         PTS[32..30]                                           3  bslbf
     *         marker_bit                                            1  bslbf
     *         PTS[29..15]                                          15  bslbf
     *         marker_bit                                            1  bslbf
     *         PTS[14..0]                                           15  bslbf
     *         marker_bit                                            1  bslbf
     *         '0001'                                                4  bslbf
     *         DTS[32..30]                                           3  bslbf
     *         marker_bit                                            1  bslbf
     *         DTS[29..15]                                          15  bslbf
     *         marker_bit                                            1  bslbf
     *         DTS[14..0]                                           15  bslbf
     *         marker_bit                                            1  bslbf
     *     }                                                            
     *     if (ESCR_flag == '1') {                                      
     *         reserved                                              2  bslbf
     *         ESCR_base[32..30]                                     3  bslbf
     *         marker_bit                                            1  bslbf
     *         ESCR_base[29..15]                                    15  bslbf
     *         marker_bit                                            1  bslbf
     *         ESCR_base[14..0]                                     15  bslbf
     *         marker_bit                                            1  bslbf
     *         ESCR_extension                                        9  uimsbf
     *         marker_bit                                            1  bslbf
     *     }                                                            
     *     if (ES_rate_flag == '1') {                                   
     *         marker_bit                                            1  bslbf
     *         ES_rate                                              22  uimsbf
     *         marker_bit                                            1  bslbf
     *     }                                                            
     *     if (DSM_trick_mode_flag == '1') {                            
     *         trick_mode_control                                    3  uimsbf
     *         if (trick_mode_control == '000') {                       
     *             field_id                                          2  bslbf
     *             intra_slice_refresh                               1  bslbf
     *             frequency_truncation                              2  bslbf
     *         } else if (trick_mode_control == '001') {                
     *             field_rep_cntrl                                   5  uimsbf
     *         } else if (trick_mode_control == '010') {                
     *             field_id                                          2  uimsbf
     *             reserved                                          3  bslbf
     *         } else if (trick_mode_control == '011') {                
     *             field_id                                          2  bslbf
     *             intra_slice_refresh                               1  bslbf
     *             frequency_truncation                              2  bslbf
     *         }                                                        
     *     }                                                            
     *     if (additional_copy_flag == '1') {                           
     *         marker_bit                                            1  bslbf
     *         additional_copy_info                                  7  bslbf
     *     }                                                            
     *     if (PES_CRC_flag == '1') {                                   
     *         previous_PES_packet_CRC                              16  bslbf
     *     }                                                            
     *     if (PES_extension_flag) {                                    
     *         PES_private_data_flag                                 1  bslbf
     *         pack_header_field_flag                                1  bslbf
     *         program_packet_sequence_control_flag                  1  bslbf
     *         P-STD_buffer_flag                                     1  bslbf
     *         reserved                                              3  bslbf
     *         PES_extension_field_flag                              1  bslbf
     *     }                                                            
     *     if (PES_private_data_flag == '1') {                          
     *         PES_private_data                                    128  bslbf
     *     }                                                            
     *     if (pack_header_field_flag == '1') {                         
     *         pack_field_length                                     8  uimsbf
     *         pack_header()                                            
     *     }                                                            
     *     if (program_packet_sequence_counter_flag == '1') {           
     *         marker_bit                                            1  bslbf
     *         program_packet_sequence_counter                       7  uimsbf
     *         marker_bit                                            1  bslbf
     *         original_stuff_length                                 7  uimsbf
     *     }                                                            
     *     if (P-STD_buffer_flag == '1') {                              
     *         '01'                                                  2  bslbf
     *         P-STD_buffer_scale                                    1  bslbf
     *         P-STD_buffer_size                                    13  uimsbf
     *     }                                                            
     *     if (PES_extension_field_flag == '1') {                       
     *         marker_bit                                            1  bslbf
     *         PES_extension_field_length                            7  uimsbf
     *         for (i = 0; i < PES_extension_field_length; i++) {       
     *              reserved                                         8  bslbf
     *         }                                                        
     *     }                                                            
     *     for (i = 0; i < N; i++) {                                    
     *          stuffing_byte                                        8  bslbf
     *     }                                                            
     *     for (i = 0; i < N; i++) {                                    
     *          PES_packet_data_byte                                 8  bslbf
     *     }                                                            
     * } else if (stream_id == private_stream_2) {                      
     *     for (i = 0; i < PES_packet_length; i++) {                    
     *          PES_packet_data_byte                                 8  bslbf
     *     }                                                            
     * } else if (stream_id == padding_stream) {                        
     *     for (i = 0; i < N; i++) {                                    
     *          padding_byte                                         8  bslbf
     *     }                                                            
     * }                                                                
     */
#if __ENABLE_X000001FF___HACK_0__
    if (system->x000001ff___hack_0) {
	system->x000001ff___hack_0(system);
    }
#endif				/* __ENABLE_X000001FF___HACK_0__ */
    if (system->size < 2) {
	system->correct_underflow(system);
    }
    if (system->start_code == program_stream_map) {
#if __ENABLE_X000001FF___HACK_2__
	if (system->x000001ff___hack_2) {
	    struct seek_h262_x000001ff__ PES_packet;

	    PES_packet.decoder = system;
	    PES_packet.x.x1.data_size = 0;
	    system->x000001ff___hack_2(&PES_packet);
	}
#endif				/* __ENABLE_X000001FF___HACK_2__ */
	_seek_h262_xprogram_stream_map(system);
    } else {
	unsigned PES_packet_length;

	PES_packet_length = *system->buffer << 8 | system->buffer[1];
	INCREMENT_INPUT(system, 2);
	if (system->start_code == padding_stream
	    || system->start_code == private_stream_2) {
	} else if (PES_packet_length < 3) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) system,
				      SYSTEM_INVALID_PES_PACKET_LENGTH);
	} else {
	    unsigned PES_header_data_length, flags;
	    unsigned char *buffer;

	    if (system->size < 3) {
		system->correct_underflow(system);
	    }
	    buffer = system->buffer;
	    if ((*buffer & 0xc0) != 0x80) {
		_seek_h262_abort_decoding((struct seek_h262_whatever *) system,
					  SYSTEM_FAILED_PES_PACKET_MARKER);
	    }
	    flags = buffer[1];
	    if (system->start_code == program_stream_directory) {
		if (*buffer & 0x04 || flags) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PROGRAM_STREAM_DIRECTORY_PES_PACKET);
		}
	    }
	    PES_packet_length -= 3;
	    PES_header_data_length = buffer[2];
	    if (84 < PES_header_data_length
		|| PES_packet_length < PES_header_data_length) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) system,
		     SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
	    }
	    if (system->size < PES_header_data_length + 3) {
		INCREMENT_INPUT(system, 3);
		system->correct_underflow(system);
		buffer = system->buffer;
	    } else {
		buffer += 3;
	    }
	    PES_packet_length -= PES_header_data_length;
	    if ((flags & 0xc0) == 0x40) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) system,
		     SYSTEM_INVALID_PTS_DTS_FLAGS);
	    } else if ((flags & 0xc0) == 0x80) {
		if (PES_header_data_length < 5) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		}
		if ((*buffer & 0xf1) != 0x21 || !(buffer[2] & 1)
		    || !(buffer[4] & 1)) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_UNSET_MARKER_BITS);
		}
#if __ENABLE_X000001FF___HACK_1__
		if (system->x000001ff___hack_1) {
		    struct seek_h262_x000001ff__ packet;

		    packet.decoder = system;
		    packet.x.x0.PTS_0 = (*buffer & 0x08) >> 3;
		    packet.x.x0.DTS_0 = packet.x.x0.PTS_0;
		    packet.x.x0.PTS_1 = (*buffer & 0x06) << 035
			| buffer[1] << 026 | (buffer[2] & 0xfe) << 016
			| buffer[3] << 007 | (buffer[4] & 0xfe) >> 001;
		    packet.x.x0.DTS_1 = packet.x.x0.PTS_1;
		    system->x000001ff___hack_1(&packet);
		}
#endif				/* __ENABLE_X000001FF___HACK_1__ */
		buffer += 5;
		PES_header_data_length -= 5;
	    } else if (flags & 0xc0) {
		if (PES_header_data_length < 10) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		}
		if ((*buffer & 0xf1) != 0x31 || !(buffer[2] & 1)
		    || !(buffer[4] & 1) || (buffer[5] & 0xf1) != 0x11
		    || !(buffer[7] & 1) || !(buffer[9] & 1)) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_UNSET_MARKER_BITS);
		}
#if __ENABLE_X000001FF___HACK_1__
		if (system->x000001ff___hack_1) {
		    struct seek_h262_x000001ff__ packet;

		    packet.decoder = system;
		    packet.x.x0.PTS_0 = (*buffer & 0x08) >> 3;
		    packet.x.x0.PTS_1 = (*buffer & 0x06) << 035
			| buffer[1] << 026 | (buffer[2] & 0xfe) << 016
			| buffer[3] << 007 | (buffer[4] & 0xfe) >> 001;
		    packet.x.x0.DTS_0 = (buffer[5] & 0x08) >> 3;
		    packet.x.x0.DTS_1 = (buffer[5] & 0x06) << 035
			| buffer[6] << 026 | (buffer[7] & 0xfe) << 016
			| buffer[8] << 007 | (buffer[9] & 0xfe) >> 001;
		    system->x000001ff___hack_1(&packet);
		}
#endif				/* __ENABLE_X000001FF___HACK_1__ */
		buffer += 10;
		PES_header_data_length -= 10;
	    }
	    if (flags & 0x20) {
		if (PES_header_data_length < 6) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		}
		if (!(*buffer & 0x40) || !(buffer[2] & 0x40)
		    || !(buffer[4] & 0x40) || !(buffer[5] & 1)) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_UNSET_MARKER_BITS);
		}
		buffer += 6;
		PES_header_data_length -= 6;
	    }
	    if (flags & 0x10) {
		if (PES_header_data_length < 3) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		}
		if (!(*buffer & 0x80) || !(buffer[2] & 1)) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_UNSET_MARKER_BITS);
		}
		buffer += 3;
		PES_header_data_length -= 3;
	    }
	    if (flags & 0x08) {
		if (!PES_header_data_length) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		}
		buffer++;
		PES_header_data_length--;
	    }
	    if (flags & 0x04) {
		if (!PES_header_data_length) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		}
		if (!(*buffer & 0x80)) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_UNSET_MARKER_BIT);
		}
		buffer++;
		PES_header_data_length--;
	    }
	    if (flags & 0x02) {
		if (PES_header_data_length < 2) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		}
		buffer += 2;
		PES_header_data_length -= 2;
	    }
	    if (flags & 0x01) {
		if (!PES_header_data_length) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		}
		flags = *buffer;
		buffer++;
		PES_header_data_length--;
	    } else {
		flags = 0;
	    }
	    if (flags) {
		if (flags & 0x80) {
		    if (PES_header_data_length < 16) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		    }
		    buffer += 16;
		    PES_header_data_length -= 16;
		}
		if (flags & 0x40) {
#if 0
		    if (!PES_header_data_length) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		    }
		    buffer++;
		    PES_header_data_length--;
#else
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_BAD_IMPLEMENTATION);
#endif				/* 0 */
		}
		if (flags & 0x20) {
		    if (PES_header_data_length < 2) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		    }
		    if (!(*buffer & 0x80) || !(buffer[1] & 0x80)) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_UNSET_MARKER_BITS);
		    }
		    buffer += 2;
		    PES_header_data_length -= 2;
		}
		if (flags & 0x10) {
		    if (PES_header_data_length < 2) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		    }
		    if ((*buffer & 0xc0) != 0x40) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_FAILED_P_STD_BUFFER_MARKER);
		    }
		    buffer += 2;
		    PES_header_data_length -= 2;
		}
		if (flags & 0x01) {
		    if (!PES_header_data_length) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		    }
		    if (!(*buffer & 0x80)) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_UNSET_MARKER_BIT);
		    }
		    flags = (*buffer & 0x7f) + 1;
		    if (PES_header_data_length < flags) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		    }
		    buffer += flags;
		    PES_header_data_length -= flags;
		}
	    }
	    if (32 < PES_header_data_length) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) system,
		     SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
	    }
	    if (system->size < PES_header_data_length) {
		system->size -= buffer - system->buffer;
		system->buffer = buffer;
		system->correct_underflow(system);
		buffer = system->buffer;
	    }
	    for (; PES_header_data_length; PES_header_data_length--) {
		if (*buffer != 0xff) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_FAILED_STUFFING_BYTE);
		}
		buffer++;
	    }
	    system->size -= buffer - system->buffer;
	    system->buffer = buffer;
	}
#if __ENABLE_X000001FF___HACK_2__
	if (system->x000001ff___hack_2) {
	    struct seek_h262_x000001ff__ PES_packet;

	    PES_packet.decoder = system;
	    PES_packet.x.x1.data_size = PES_packet_length;
	    system->x000001ff___hack_2(&PES_packet);
	}
#endif				/* __ENABLE_X000001FF___HACK_2__ */
	system->packet_data(system, PES_packet_length);
    }
}
