/*
 * h262.1.part4.c
 * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.1.h>
#include <h262.2.h>
#include <h262.3.h>
#include <h262.a.h>

#define INCREMENT_INPUT(system, nbytes) \
    {									      \
	system->size -= nbytes;						      \
	system->buffer += nbytes;					      \
    }

void
seek_h262_noisy_PES_packet(struct seek_h262_system *system)
{
#if __ENABLE_X000001FF___HACK_0__
    if (system->x000001ff___hack_0) {
	system->x000001ff___hack_0(system);
    }
#endif				/* __ENABLE_X000001FF___HACK_0__ */
    if (system->size < 2) {
	system->correct_underflow(system);
    }
    if (system->start_code == program_stream_map) {
#if __ENABLE_X000001FF___HACK_2__
	if (system->x000001ff___hack_2) {
	    struct seek_h262_x000001ff__ PES_packet;

	    PES_packet.decoder = system;
	    PES_packet.x.x1.data_size = 0;
	    system->x000001ff___hack_2(&PES_packet);
	}
#endif				/* __ENABLE_X000001FF___HACK_2__ */
	_seek_h262_xprogram_stream_map(system);
    } else {
	unsigned PES_packet_length;

	PES_packet_length = *system->buffer << 8 | system->buffer[1];
	INCREMENT_INPUT(system, 2);
	if (system->start_code == padding_stream
	    || system->start_code == private_stream_2) {
	} else if (PES_packet_length < 3) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) system,
				      SYSTEM_INVALID_PES_PACKET_LENGTH);
	} else {
	    unsigned PES_header_data_length, flags;
	    unsigned char *buffer;

	    if (system->size < 3) {
		system->correct_underflow(system);
	    }
	    buffer = system->buffer;
	    if ((*buffer & 0xc0) != 0x80) {
		_seek_h262_abort_decoding((struct seek_h262_whatever *) system,
					  SYSTEM_FAILED_PES_PACKET_MARKER);
	    }
	    flags = buffer[1];
	    if (system->start_code == program_stream_directory) {
		if (*buffer & 0x04 || flags) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PROGRAM_STREAM_DIRECTORY_PES_PACKET);
		}
	    }
	    PES_packet_length -= 3;
	    PES_header_data_length = buffer[2];
	    if (84 < PES_header_data_length
		|| PES_packet_length < PES_header_data_length) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) system,
		     SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
	    }
#if __ENABLE_X000001S1___HACK_3__
	    if (system->x000001s1___hack_3) {
		struct seek_h262_x000001s1__ PES_packet;
		unsigned class;

		class = buffer[0];

		PES_packet.decoder = system;
		PES_packet.x.x2.DSM_trick_mode_flag = (flags >> 3) & 1;
		PES_packet.x.x2.ESCR_flag = (flags >> 5) & 1;
		PES_packet.x.x2.ES_rate_flag = (flags >> 4) & 1;
		PES_packet.x.x2.PES_CRC_flag = (flags >> 1) & 1;
		PES_packet.x.x2.PES_extension_flag = flags & 1;
		PES_packet.x.x2.PES_header_data_length =
		    PES_header_data_length;
		PES_packet.x.x2.PES_priority = (class >> 3) & 1;
		PES_packet.x.x2.PES_scrambling_control = (class >> 4) & 3;
		PES_packet.x.x2.PTS_DTS_flags = flags >> 6;
		PES_packet.x.x2.additional_copy_flag = (flags >> 2) & 1;
		PES_packet.x.x2.copyright = (class >> 1) & 1;
		PES_packet.x.x2.data_alignment_indicator = (class >> 2) & 1;
		PES_packet.x.x2.original_or_copy = class & 1;
		system->x000001s1___hack_3(&PES_packet);
	    }
#endif				/* __ENABLE_X000001S1___HACK_3__ */
	    if (system->size < PES_header_data_length + 3) {
		INCREMENT_INPUT(system, 3);
		system->correct_underflow(system);
		buffer = system->buffer;
	    } else {
		buffer += 3;
	    }
	    PES_packet_length -= PES_header_data_length;
	    if ((flags & 0xc0) == 0x40) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) system,
		     SYSTEM_INVALID_PTS_DTS_FLAGS);
	    } else if ((flags & 0xc0) == 0x80) {
		if (PES_header_data_length < 5) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		}
		if ((*buffer & 0xf1) != 0x21 || !(buffer[2] & 1)
		    || !(buffer[4] & 1)) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_UNSET_MARKER_BITS);
		}
#if __ENABLE_X000001FF___HACK_1__
		if (system->x000001ff___hack_1) {
		    struct seek_h262_x000001ff__ packet;

		    packet.decoder = system;
		    packet.x.x0.PTS_0 = (*buffer & 0x08) >> 3;
		    packet.x.x0.DTS_0 = packet.x.x0.PTS_0;
		    packet.x.x0.PTS_1 = (*buffer & 0x06) << 035
			| buffer[1] << 026 | (buffer[2] & 0xfe) << 016
			| buffer[3] << 007 | (buffer[4] & 0xfe) >> 001;
		    packet.x.x0.DTS_1 = packet.x.x0.PTS_1;
		    system->x000001ff___hack_1(&packet);
		}
#endif				/* __ENABLE_X000001FF___HACK_1__ */
		buffer += 5;
		PES_header_data_length -= 5;
	    } else if (flags & 0xc0) {
		if (PES_header_data_length < 10) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		}
		if ((*buffer & 0xf1) != 0x31 || !(buffer[2] & 1)
		    || !(buffer[4] & 1) || (buffer[5] & 0xf1) != 0x11
		    || !(buffer[7] & 1) || !(buffer[9] & 1)) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_UNSET_MARKER_BITS);
		}
#if __ENABLE_X000001FF___HACK_1__
		if (system->x000001ff___hack_1) {
		    struct seek_h262_x000001ff__ packet;

		    packet.decoder = system;
		    packet.x.x0.PTS_0 = (*buffer & 0x08) >> 3;
		    packet.x.x0.PTS_1 = (*buffer & 0x06) << 035
			| buffer[1] << 026 | (buffer[2] & 0xfe) << 016
			| buffer[3] << 007 | (buffer[4] & 0xfe) >> 001;
		    packet.x.x0.DTS_0 = (buffer[5] & 0x08) >> 3;
		    packet.x.x0.DTS_1 = (buffer[5] & 0x06) << 035
			| buffer[6] << 026 | (buffer[7] & 0xfe) << 016
			| buffer[8] << 007 | (buffer[9] & 0xfe) >> 001;
		    system->x000001ff___hack_1(&packet);
		}
#endif				/* __ENABLE_X000001FF___HACK_1__ */
		buffer += 10;
		PES_header_data_length -= 10;
	    }
	    if (flags & 0x20) {
		if (PES_header_data_length < 6) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		}
		if (!(*buffer & 0x40) || !(buffer[2] & 0x40)
		    || !(buffer[4] & 0x40) || !(buffer[5] & 1)) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_UNSET_MARKER_BITS);
		}
#if __ENABLE_X000001S1___HACK_4__
		if (system->x000001s1___hack_4) {
		    struct seek_h262_x000001s1__ packet;

		    packet.decoder = system;
		    packet.x.x3.ESCR_base_0 = (*buffer & 0x20) >> 5;
		    packet.x.x3.ESCR_base_1 = (*buffer & 0x18) << 033
			| (buffer[0] & 0x03) << 034 | buffer[1] << 024
			| (buffer[2] & 0xf8) << 014 | (buffer[2] & 0x03) << 015
			| buffer[3] << 005 | (buffer[4] & 0xf8) >> 003;
		    packet.x.x3.ESCR_extension =
			(buffer[4] & 0x03) << 007 | (buffer[5] & 0xfe) >> 001;
		    system->x000001s1___hack_4(&packet);
		}
#endif				/* __ENABLE_X000001S1___HACK_4__ */
		buffer += 6;
		PES_header_data_length -= 6;
	    }
	    if (flags & 0x10) {
		if (PES_header_data_length < 3) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		}
		if (!(*buffer & 0x80) || !(buffer[2] & 1)) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_UNSET_MARKER_BITS);
		}
#if __ENABLE_X000001S1___HACK_5__
		if (system->x000001s1___hack_5) {
		    struct seek_h262_x000001s1__ packet;

		    packet.decoder = system;
		    packet.x.x4.ES_rate = (*buffer & 0x7f) << 017
			| buffer[1] << 007 | (buffer[2] & 0xfe) >> 001;
		    system->x000001s1___hack_5(&packet);
		}
#endif				/* __ENABLE_X000001S1___HACK_5__ */
		buffer += 3;
		PES_header_data_length -= 3;
	    }
	    if (flags & 0x08) {
		if (!PES_header_data_length) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		}
#if __ENABLE_X000001S1___HACK_6__
		if (system->x000001s1___hack_6) {
		    struct seek_h262_x000001s1__ packet;
		    unsigned class, field;

		    field = *buffer;
		    class = field >> 5;

		    packet.decoder = system;
		    packet.x.x5.trick_mode_control = class;
		    if (!class) {
			packet.x.x5.v.v0.field_id = field >> 3 & 3;
			packet.x.x5.v.v0.frequency_truncation = field & 3;
			packet.x.x5.v.v0.intra_slice_refresh = field >> 2 & 1;
		    } else if (class == 1) {
			packet.x.x5.v.v1.field_rep_cntrl = field & 31;
		    } else if (class == 2) {
			packet.x.x5.v.v2.field_id = field >> 3 & 3;
		    } else if (class == 3) {
			packet.x.x5.v.v3.field_id = field >> 3 & 3;
			packet.x.x5.v.v3.frequency_truncation = field & 3;
			packet.x.x5.v.v3.intra_slice_refresh = field >> 2 & 1;
		    }
		    system->x000001s1___hack_6(&packet);
		}
#endif				/* __ENABLE_X000001S1___HACK_6__ */
		buffer++;
		PES_header_data_length--;
	    }
	    if (flags & 0x04) {
		if (!PES_header_data_length) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		}
		if (!(*buffer & 0x80)) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_UNSET_MARKER_BIT);
		}
#if __ENABLE_X000001S1___HACK_7__
		if (system->x000001s1___hack_7) {
		    struct seek_h262_x000001s1__ packet;

		    packet.decoder = system;
		    packet.x.x6.additional_copy_info = *buffer & 0x7f;
		    system->x000001s1___hack_7(&packet);
		}
#endif				/* __ENABLE_X000001S1___HACK_7__ */
		buffer++;
		PES_header_data_length--;
	    }
	    if (flags & 0x02) {
		if (PES_header_data_length < 2) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		}
#if __ENABLE_X000001S1___HACK_8__
		if (system->x000001s1___hack_8) {
		    struct seek_h262_x000001s1__ packet;

		    packet.decoder = system;
		    packet.x.x7.previous_PES_packet_CRC =
			*buffer << 010 | buffer[1];
		    system->x000001s1___hack_8(&packet);
		}
#endif				/* __ENABLE_X000001S1___HACK_8__ */
		buffer += 2;
		PES_header_data_length -= 2;
	    }
	    if (flags & 0x01) {
		if (!PES_header_data_length) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		}
		flags = *buffer;
#if __ENABLE_X000001S1___HACK_9__
		if (system->x000001s1___hack_9) {
		    struct seek_h262_x000001s1__ packet;

		    packet.decoder = system;
		    packet.x.x8.PES_extension_field_flag = flags & 1;
		    packet.x.x8.PES_private_data_flag = flags >> 007 & 1;
		    packet.x.x8.PhyphenSTD_buffer_flag = flags >> 004 & 1;
		    packet.x.x8.pack_header_field_flag = flags >> 006 & 1;
		    packet.x.x8.program_packet_sequence_control_flag =
			flags >> 005 & 1;
		    system->x000001s1___hack_9(&packet);
		}
#endif				/* __ENABLE_X000001S1___HACK_9__ */
		buffer++;
		PES_header_data_length--;
	    } else {
		flags = 0;
	    }
	    if (flags) {
		if (flags & 0x80) {
		    if (PES_header_data_length < 16) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		    }
		    buffer += 16;
		    PES_header_data_length -= 16;
		}
		if (flags & 0x40) {
#if 0
		    if (!PES_header_data_length) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		    }
		    buffer++;
		    PES_header_data_length--;
#else
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_BAD_IMPLEMENTATION);
#endif				/* 0 */
		}
		if (flags & 0x20) {
		    if (PES_header_data_length < 2) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		    }
		    if (!(*buffer & 0x80) || !(buffer[1] & 0x80)) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_UNSET_MARKER_BITS);
		    }
		    buffer += 2;
		    PES_header_data_length -= 2;
		}
		if (flags & 0x10) {
		    if (PES_header_data_length < 2) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		    }
		    if ((*buffer & 0xc0) != 0x40) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_FAILED_P_STD_BUFFER_MARKER);
		    }
		    buffer += 2;
		    PES_header_data_length -= 2;
		}
		if (flags & 0x01) {
		    if (!PES_header_data_length) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		    }
		    if (!(*buffer & 0x80)) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_UNSET_MARKER_BIT);
		    }
		    flags = (*buffer & 0x7f) + 1;
		    if (PES_header_data_length < flags) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) system,
			     SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
		    }
		    buffer += flags;
		    PES_header_data_length -= flags;
		}
	    }
	    if (32 < PES_header_data_length) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) system,
		     SYSTEM_INVALID_PES_HEADER_DATA_LENGTH);
	    }
	    if (system->size < PES_header_data_length) {
		system->size -= buffer - system->buffer;
		system->buffer = buffer;
		system->correct_underflow(system);
		buffer = system->buffer;
	    }
	    for (; PES_header_data_length; PES_header_data_length--) {
		if (*buffer != 0xff) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) system,
			 SYSTEM_FAILED_STUFFING_BYTE);
		}
		buffer++;
	    }
	    system->size -= buffer - system->buffer;
	    system->buffer = buffer;
	}
#if __ENABLE_X000001FF___HACK_2__
	if (system->x000001ff___hack_2) {
	    struct seek_h262_x000001ff__ PES_packet;

	    PES_packet.decoder = system;
	    PES_packet.x.x1.data_size = PES_packet_length;
	    system->x000001ff___hack_2(&PES_packet);
	}
#endif				/* __ENABLE_X000001FF___HACK_2__ */
	system->packet_data(system, PES_packet_length);
    }
}
