/*
 * h262.2.c
 * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.1.h>
#include <h262.2.h>
#include <h262.3.h>
#include <h262.4.h>
#include <h262.a.h>
#include <h262.f.l7.h>

#define Frame_based_prediction_or_16x8_MC \
					2

#define INCREMENT_INPUT(video, nbytes) \
    {									      \
	video->size -= nbytes;						      \
	video->buffer += nbytes;					      \
    }

extern unsigned char _seek_h262_B_2_01[], _seek_h262_B_3_01[],
    _seek_h262_B_4_01[], _seek_h262_B_5_01[], _seek_h262_B_6_01[],
    _seek_h262_B_7_01[], _seek_h262_B_8_01[], _seek_h262_B_e_012_0[],
    _seek_h262_B_e_012_1[], _seek_h262_B_f_012_0[], _seek_h262_B_f_012_1[];

static void copy_sixteen(unsigned *, unsigned *);

static void artificial_extension_data(struct seek_h262_video *);

static void extension_and_user_data(struct seek_h262_video *, unsigned);
static void extension_data(struct seek_h262_video *, unsigned);
static void group_of_pictures_header(struct seek_h262_video *);
static void motion_backward(struct seek_h262_video *);
static void motion_forward(struct seek_h262_video *);
static void picture_coding_extension(struct seek_h262_video *);
static void picture_data(struct seek_h262_video *);
static void picture_display_extension(struct seek_h262_video *);
static void picture_header(struct seek_h262_video *);
static void picture_spatial_scalable_extension(struct seek_h262_video *);
static void picture_temporal_scalable_extension(struct seek_h262_video *);
static void sequence_display_extension(struct seek_h262_video *);
static void sequence_extension(struct seek_h262_video *);
static void sequence_header(struct seek_h262_video *);
static void sequence_scalable_extension(struct seek_h262_video *);
static void user_data(struct seek_h262_video *);

static void subsequent_sequence_display_extension(struct seek_h262_video *);
static void subsequent_sequence_extension(struct seek_h262_video *);
static void subsequent_sequence_scalable_extension(struct seek_h262_video *);

static const unsigned x7_6[] = {
/* *INDENT-OFF* */
    0x00, 0x02, 0x04, 0x06, 0x08, 0x0a, 0x0c, 0x0e,
    0x10, 0x12, 0x14, 0x16, 0x18, 0x1a, 0x1c, 0x1e,
    0x20, 0x22, 0x24, 0x26, 0x28, 0x2a, 0x2c, 0x2e,
    0x30, 0x32, 0x34, 0x36, 0x38, 0x3a, 0x3c, 0x3e,
    0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
    0x08, 0x0a, 0x0c, 0x0e, 0x10, 0x12, 0x14, 0x16,
    0x18, 0x1c, 0x20, 0x24, 0x28, 0x2c, 0x30, 0x34,
    0x38, 0x40, 0x48, 0x50, 0x58, 0x60, 0x68, 0x70
/* *INDENT-ON* */
};
static const unsigned char default_intra_quantiser_matrix[] = {
/* *INDENT-OFF* */
    0x08, 0x10, 0x10, 0x13, 0x10, 0x13, 0x16, 0x16,
    0x16, 0x16, 0x16, 0x16, 0x1a, 0x18, 0x1a, 0x1b,
    0x1b, 0x1b, 0x1a, 0x1a, 0x1a, 0x1a, 0x1b, 0x1b,
    0x1b, 0x1d, 0x1d, 0x1d, 0x22, 0x22, 0x22, 0x1d,
    0x1d, 0x1d, 0x1b, 0x1b, 0x1d, 0x1d, 0x20, 0x20,
    0x22, 0x22, 0x25, 0x26, 0x25, 0x23, 0x23, 0x22,
    0x23, 0x26, 0x26, 0x28, 0x28, 0x28, 0x30, 0x30,
    0x2e, 0x2e, 0x38, 0x38, 0x3a, 0x45, 0x45, 0x53
/* *INDENT-ON* */
};

static void
copy_sixteen(unsigned *major, unsigned *minor)
{
    *major++ = *minor++;
    *major++ = *minor++;
    *major++ = *minor++;
    *major++ = *minor++;
    *major++ = *minor++;
    *major++ = *minor++;
    *major++ = *minor++;
    *major++ = *minor++;
    *major++ = *minor++;
    *major++ = *minor++;
    *major++ = *minor++;
    *major++ = *minor++;
    *major++ = *minor++;
    *major++ = *minor++;
    *major++ = *minor++;
    *major++ = *minor++;
}


static void
artificial_extension_data(struct seek_h262_video *video)
{
    if ((*video->buffer & 0xf0) == 0x00
	|| (*video->buffer & 0xf0) == 0x40
	|| (*video->buffer & 0xf0) == 0x60
	|| 0xa0 < (*video->buffer & 0xf0)) {
    } else {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_INVALID_EXTENSION_IDENTIFIER);
    }
#if __ENABLE_X000001B5___HACK_0__
    if (video->x000001b5___hack_0) {
	video->x000001b5___hack_0(video);
    }
#endif				/* __ENABLE_X000001B5___HACK_0__ */
    _seek_h262_artificial_extension_or_user_data(video);
}


static void
extension_and_user_data(struct seek_h262_video *video, unsigned i)
{
    /*
     * while (nextbits() == extension_start_code                        
     *        || nextbits() == user_data_start_code) {                  
     *     if (i != 1) {                                                
     *         if (nextbits() == extension_start_code) {                
     *             extension_data(i)                                    
     *         }                                                        
     *     }                                                            
     *     if (nextbits() == user_data_start_code) {                    
     *         user_data()                                              
     *     }                                                            
     * }                                                                
     */
    while (1) {
	if (video->start_code == user_data_start_code) {
	    user_data(video);
	} else if (video->start_code == extension_start_code) {
	    extension_data(video, i);
	} else {
	    break;
	}
    }
}


static void
extension_data(struct seek_h262_video *video, unsigned i)
{
    /*
     * while (nextbits() == extension_start_code) {                     
     *     extension_start_code                                     32  bslbf
     *     if (i == 0) {                                                
     *         slash* follows sequence_extension() *slash               
     *         if (nextbits() == "Sequence Display Extension ID") {     
     *             sequence_display_extension()                         
     *         }                                                        
     *         if (nextbits() == "Sequence Scalable Extension ID") {    
     *             sequence_scalable_extension()                        
     *         }                                                        
     *     }                                                            
     *     slash* Note: extension never follows a                       
     *          * group_of_pictures_header()                            
     *          *slash                                                  
     *     if (i == 2) {                                                
     *         slash* follows picture_coding_extension() *slash         
     *         if (nextbits() == "Quant Matrix Extension ID") {         
     *             quant_matrix_extension()                             
     *         }                                                        
     *         if (nextbits() == "Picture Pan Scan Extension ID") {     
     *             picture_display_extension()                          
     *         }                                                        
     *         if (nextbits() == "Picture Spatial Scalable Extension ID") {
     *             picture_spatial_scalable_extension()                 
     *         }                                                        
     *         if (nextbits() == "Picture Temporal Scalable Ext. ID") { 
     *             picture_temporal_scalable_extension()                
     *         }                                                        
     *     }                                                            
     * }                                                                
     */
    while (1) {
	if (video->start_code != extension_start_code) {
	    break;
	}
	if (!i) {
	    if (!video->size) {
		video->optimistique_moi(video);
	    }
	    if ((*video->buffer & 0xf0) == Sequence_Display_Extension_ID) {
		if (video->sequence_display_extension_seen) {
		    subsequent_sequence_display_extension(video);
		} else {
		    sequence_display_extension(video);
		}
	    } else if ((*video->buffer & 0xf0) ==
		       Sequence_Scalable_Extension_ID) {
		if (video->sequence_scalable_extension_seen) {
		    subsequent_sequence_scalable_extension(video);
		} else {
		    sequence_scalable_extension(video);
		}
	    } else {
		artificial_extension_data(video);
	    }
	} else if (i == 1) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				      VIDEO_INVALID_EXTENSION_IDENTIFIER);
	} else {
	    if (!video->size) {
		video->optimistique_moi(video);
	    }
	    if ((*video->buffer & 0xf0) == Quant_Matrix_Extension_ID) {
		seek_h262_quant_matrix_extension(video);
	    } else if ((*video->buffer & 0xf0) == Picture_Display_Extension_ID) {
		/*
		 * What the fuck "Picture Pan Scan Extension ID" is?
		 * Assume "Picture Display Extension ID"...
		 */
		picture_display_extension(video);
	    } else if ((*video->buffer & 0xf0) ==
		       Picture_Spatial_Scalable_Extension_ID) {
		picture_spatial_scalable_extension(video);
	    } else if ((*video->buffer & 0xf0) ==
		       Picture_Temporal_Scalable_Extension_ID) {
		/*
		 * What the fuck "Picture Temporal Scalable Ext. ID" is?
		 * Assume "Picture Temporal Scalable Extension ID"...
		 */
		picture_temporal_scalable_extension(video);
	    } else {
		artificial_extension_data(video);
	    }
	}
    }
}


static void
group_of_pictures_header(struct seek_h262_video *video)
{
    /*
     * group_start_code                                             32  bslbf
     * time_code                                                    25  bslbf
     * closed_gop                                                    1  uimsbf
     * broken_link                                                   1  uimsbf
     * next_start_code()                                                
     */
#if __ENABLE_X000001B8___HACK_0__
    if (video->x000001b8___hack_0) {
	video->x000001b8___hack_0(video);
    }
#endif				/* __ENABLE_X000001B8___HACK_0__ */
    if (video->size < 4) {
	video->optimistique_moi(video);
    }
    if (!(video->buffer[1] & 0x08)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_INVALID_TIME_CODE);
    }
    {
	INCREMENT_INPUT(video, 4);
	video->next_start_code(video);
    }
}


static void
motion_backward(struct seek_h262_video *video)
{
    _seek_h262_motion_component
	(video->f_code_1_0, &video->f_code_1_0_limit, &video->f_code_1_0_mask,
	 &video->f_code_1_0_range, &video->B_a_01_1_0);
    _seek_h262_motion_component
	(video->f_code_1_1, &video->f_code_1_1_limit, &video->f_code_1_1_mask,
	 &video->f_code_1_1_range, &video->B_a_01_1_1);
}


static void
motion_forward(struct seek_h262_video *video)
{
    _seek_h262_motion_component
	(video->f_code_0_0, &video->f_code_0_0_limit, &video->f_code_0_0_mask,
	 &video->f_code_0_0_range, &video->B_a_01_0_0);
    _seek_h262_motion_component
	(video->f_code_0_1, &video->f_code_0_1_limit, &video->f_code_0_1_mask,
	 &video->f_code_0_1_range, &video->B_a_01_0_1);
}


static void
picture_coding_extension(struct seek_h262_video *video)
{
    /*
     * extension_start_code                                         32  bslbf
     * extension_start_code_identifier                               4  uimsbf
     * f_code[0][0]             slash* forward horizontal *slash     4  uimsbf
     * f_code[0][1]             slash* forward vertical *slash       4  uimsbf
     * f_code[1][0]             slash* backward horizontal *slash    4  uimsbf
     * f_code[1][1]             slash* backward vertical *slash      4  uimsbf
     * intra_dc_precision                                            2  uimsbf
     * picture_structure                                             2  uimsbf
     * top_field_first                                               1  uimsbf
     * frame_pred_frame_dct                                          1  uimsbf
     * concealment_motion_vectors                                    1  uimsbf
     * q_scale_type                                                  1  uimsbf
     * intra_vlc_format                                              1  uimsbf
     * alternate_scan                                                1  uimsbf
     * repeat_first_field                                            1  uimsbf
     * chroma_420_type                                               1  uimsbf
     * progressive_frame                                             1  uimsbf
     * composite_display_flag                                        1  uimsbf
     * if (composite_display_flag) {                                    
     *     v_axis                                                    1  uimsbf
     *     field_sequence                                            3  uimsbf
     *     sub_carrier                                               1  uimsbf
     *     burst_amplitude                                           7  uimsbf
     *     sub_carrier_phase                                         8  uimsbf
     * }                                                                
     * next_start_code()                                                
     */
#if __ENABLE_X000001B5_8_HACK_0__
    if (video->x000001b5_8_hack_0) {
	video->x000001b5_8_hack_0(video);
    }
#endif				/* __ENABLE_X000001B5_8_HACK_0__ */
    if (video->size < 7) {
	video->optimistique_moi(video);
    }
    {
	unsigned char *buffer;

	buffer = video->buffer - 4;
	if (video->start_code != extension_start_code) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				      VIDEO_IMPROPER_START_CODE);
	} else if ((buffer[4] & 0xf0) != Picture_Coding_Extension_ID) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				      VIDEO_INVALID_EXTENSION_IDENTIFIER);
	}
	if (buffer[8] & 0x40) {
	    INCREMENT_INPUT(video, 7);
	} else {
	    INCREMENT_INPUT(video, 5);
	}
	{
	    unsigned concealment_motion_vectors, f_code, frame_pred_frame_dct,
		picture_structure, progressive_frame, repeat_first_field,
		top_field_first;

	    concealment_motion_vectors = buffer[7] & 0x20;
	    f_code = *((unsigned *) buffer + 1);
	    if (video->picture_coding_type == intra_coded
		&& !concealment_motion_vectors) {
#if WORDS_BIGENDIAN
		if ((f_code & 0x0ffff000) != 0x0ffff000) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_INVALID_F_CODES);
		    _seek_h262_detail_x08_f_codes(video, &f_code);
		}
#else
		if ((f_code & 0x00f0ff0f) != 0x00f0ff0f) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_INVALID_F_CODES);
		    _seek_h262_detail_x08_f_codes(video, &f_code);
		}
#endif				/* WORDS_BIGENDIAN */
	    } else if (video->picture_coding_type == intra_coded
		       || video->picture_coding_type == predictive_coded) {
#if WORDS_BIGENDIAN
		if ((f_code & 0x000ff000) != 0x000ff000
		    || !(f_code & 0x0f000000)
		    || 0x09000000 < (f_code & 0x0f000000)
		    || !(f_code & 0x00f00000)
		    || 0x00900000 < (f_code & 0x00f00000)) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_INVALID_F_CODES);
		    _seek_h262_detail_x10_f_codes(video, &f_code);
		}
		f_code -= 0x01100000;
		video->f_code_0_0 = (f_code & 0x0f000000) >> 24;
		video->f_code_0_1 = (f_code & 0x00f00000) >> 20;
#else
		if ((f_code & 0x00f00f00) != 0x00f00f00
		    || !(f_code & 0x0000000f)
		    || 0x00000009 < (f_code & 0x0000000f)
		    || !(f_code & 0x0000f000)
		    || 0x00009000 < (f_code & 0x0000f000)) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_INVALID_F_CODES);
		    _seek_h262_detail_x10_f_codes(video, &f_code);
		}
		f_code -= 0x00001001;
		video->f_code_0_0 = f_code & 0x0000000f;
		video->f_code_0_1 = (f_code & 0x0000f000) >> 12;
#endif				/* WORDS_BIGENDIAN */
		motion_forward(video);
	    } else {
#if WORDS_BIGENDIAN
		if (!(f_code & 0x0f000000)
		    || 0x09000000 < (f_code & 0x0f000000)
		    || !(f_code & 0x00f00000)
		    || 0x00900000 < (f_code & 0x00f00000)
		    || !(f_code & 0x000f0000)
		    || 0x00090000 < (f_code & 0x000f0000)
		    || !(f_code & 0x0000f000)
		    || 0x00009000 < (f_code & 0x0000f000)) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_INVALID_F_CODES);
		    _seek_h262_detail_x18_f_codes(video, &f_code);
		}
		f_code -= 0x01111000;
		video->f_code_0_0 = (f_code & 0x0f000000) >> 24;
		video->f_code_0_1 = (f_code & 0x00f00000) >> 20;
		video->f_code_1_0 = (f_code & 0x000f0000) >> 16;
		video->f_code_1_1 = (f_code & 0x0000f000) >> 12;
#else
		if (!(f_code & 0x0000000f)
		    || 0x00000009 < (f_code & 0x0000000f)
		    || !(f_code & 0x0000f000)
		    || 0x00009000 < (f_code & 0x0000f000)
		    || !(f_code & 0x00000f00)
		    || 0x00000900 < (f_code & 0x00000f00)
		    || !(f_code & 0x00f00000)
		    || 0x00900000 < (f_code & 0x00f00000)) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_INVALID_F_CODES);
		    _seek_h262_detail_x18_f_codes(video, &f_code);
		}
		f_code -= 0x00101101;
		video->f_code_0_0 = *((unsigned char *) &f_code) & 15;
		video->f_code_0_1 = ((unsigned char *) &f_code)[1] >> 4;
		video->f_code_1_0 = ((unsigned char *) &f_code)[1] & 15;
		video->f_code_1_1 = ((unsigned char *) &f_code)[2] >> 4;
#endif				/* WORDS_BIGENDIAN */
		motion_backward(video);
		motion_forward(video);
	    }
	    picture_structure = buffer[6] & 0x03;
	    if (!picture_structure) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_INVALID_PICTURE_STRUCTURE);
	    }
	    progressive_frame = buffer[8] & 0x80;
	    frame_pred_frame_dct = buffer[7] & 0x40;
	    repeat_first_field = buffer[7] & 0x02;
	    top_field_first = buffer[7] & 0x80;
	    if (frame_pred_frame_dct) {
		if (picture_structure != 0x03) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_INVALID_FIELD_FRAME_PRED_FRAME_DCT);
		}
	    } else {
		if (progressive_frame) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_INVALID_PROGRESSIVE_FRAME_PRED_FRAME_DCT);
		}
	    }
	    if (video->progressive_sequence) {
		if (picture_structure == 0x03) {
		    if (progressive_frame) {
			video->picture_substructure =
			    Frame_picture_mask | progressive_frame_mask;
			if (repeat_first_field) {
			    video->picture_substructure |=
				repeat_first_field_mask;
			}
			if (top_field_first) {
			    video->picture_substructure |=
				top_field_first_mask;
			}
		    } else if (repeat_first_field) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) video,
			     VIDEO_INVALID_REPEAT_FIRST_FIELD);
		    } else {
			video->picture_substructure = Frame_picture_mask;
			if (top_field_first) {
			    video->picture_substructure |=
				top_field_first_mask;
			}
		    }
		} else {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_INVALID_PROGRESSIVE_SEQUENCE_PICTURE_STRUCTURE);
		}
	    } else {
		if (progressive_frame) {
		    if (picture_structure != 0x03) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) video, 0);
		    } else if (video->expect_field) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) video, 0);
		    } else {
			video->dmv_abort = ~0;
			video->expect_field = 0;
			video->picture_substructure =
			    Frame_picture_mask | progressive_frame_mask;
			if (repeat_first_field) {
			    video->picture_substructure |=
				repeat_first_field_mask;
			}
			if (top_field_first) {
			    video->picture_substructure |=
				top_field_first_mask;
			}
		    }
		} else if (picture_structure == 0x03) {
		    if (video->expect_field) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) video, 0);
		    } else if (repeat_first_field) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) video, 0);
		    } else {
			video->dmv_abort = ~0;
			video->expect_field = 0;
			video->picture_substructure = Frame_picture_mask;
			if (top_field_first) {
			    video->picture_substructure |=
				top_field_first_mask;
			}
		    }
		} else {
		    if (repeat_first_field) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) video, 0);
		    } else if (top_field_first) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) video, 0);
		    } else {
			video->dmv_abort =
			    video->deinterlaced_release_coding_type
			    == intra_coded
			    && video->picture_coding_type == predictive_coded
			    ? Dual_Prime : ~0;
			video->expect_field ^= 1;
			if (picture_structure == 0x02) {
			    video->picture_substructure = 0;
			} else {
			    video->picture_substructure = Top_Field_mask;
			}
		    }
		}
	    }
	    video->concealment_motion_vectors = concealment_motion_vectors;
	    video->decode_dct_type =
		picture_structure == 0x03 && !frame_pred_frame_dct
		? macroblock_pattern_mask : 0;
	    video->decode_motion_type =
		picture_structure != 0x03 || !frame_pred_frame_dct;
	    video->frame_pred_frame_dct = frame_pred_frame_dct;
	    video->intra_dc_precision = (buffer[6] & 0x0c) >> 2;
	    video->intra_dc_default = 128 << video->intra_dc_precision;
	    video->intra_dc_precision = 3 - video->intra_dc_precision;
	    video->picture_structure = picture_structure;
	    video->q_scale = (unsigned *) x7_6 + ((buffer[7] & 0x10) << 1);
	    if (buffer[7] & 0x08) {
		video->B_ef_012_0 = _seek_h262_B_f_012_0;
		video->B_ef_012_1 = _seek_h262_B_f_012_1;
	    } else {
		video->B_ef_012_0 = _seek_h262_B_e_012_0;
		video->B_ef_012_1 = _seek_h262_B_e_012_1;
	    }
	    video->alternate_scan = (buffer[7] & 0x04) >> 2;
	    if (picture_structure == Frame_picture) {
		if (top_field_first) {
		    video->m_0_1 = 3;
		    video->m_1_0 = 1;
		} else {
		    video->m_0_1 = 1;
		    video->m_1_0 = 3;
		}
		if (video->picture_coding_type
		    == bidirectionally_predictive_coded) {
		    video->______skipped_macroblocks_x__ =
			video->frame_skipped_macroblocks_x18;
		} else if (video->picture_coding_type == predictive_coded) {
		    video->______skipped_macroblocks_x__ =
			video->frame_skipped_macroblocks_x10;
		    video->predictive_coded_motion_type =
			Frame_based_prediction_or_16x8_MC;
		} else {
		}
		video->______macroblock[0] = video->frame_macroblock[0];
		video->______macroblock[1] = video->frame_macroblock[1];
		video->______macroblock[2] = video->frame_macroblock[2];
		video->______macroblock[3] = video->frame_macroblock[3];
		video->______macroblock[4] = video->frame_macroblock[4];
		video->______macroblock[5] = video->frame_macroblock[5];
		video->______macroblock[6] = video->frame_macroblock[6];
		video->______macroblock[7] = video->frame_macroblock[7];
		video->luminance_slice_size = video->mb_width << 8;
		switch (video->chroma_format) {
		case H1V1:
		    video->chrominance_slice_size = video->mb_width << 8;
		    break;
		case H2V1:
		    video->chrominance_slice_size = video->mb_width << 7;
		    break;
		case H2V2:
		    video->chrominance_slice_size = video->mb_width << 6;
		}
	    } else {
#if 0
		video->m_0_1 = video->m_1_0 = 1;
#endif				/* 0 */
		if (video->picture_coding_type
		    == bidirectionally_predictive_coded) {
		    video->______skipped_macroblocks_x__ =
			video->field_skipped_macroblocks_x18;
		} else if (video->picture_coding_type == predictive_coded) {
		    video->______skipped_macroblocks_x__ =
			video->field_skipped_macroblocks_x10;
		    video->predictive_coded_motion_type =
			Field_based_prediction;
		} else {
		}
		video->______macroblock[0] = video->field_macroblock[0];
		video->______macroblock[1] = video->field_macroblock[1];
		video->______macroblock[2] = video->field_macroblock[2];
		video->______macroblock[3] = video->field_macroblock[3];
		video->luminance_slice_size = video->mb_width << 9;
		switch (video->chroma_format) {
		case H1V1:
		    video->chrominance_slice_size = video->mb_width << 9;
		    break;
		case H2V1:
		    video->chrominance_slice_size = video->mb_width << 8;
		    break;
		case H2V2:
		    video->chrominance_slice_size = video->mb_width << 7;
		}
	    }
	}
	video->next_start_code(video);
    }
#if __ENABLE_X000001B5_8_HACK_1__
    if (video->x000001b5_8_hack_1) {
	struct seek_h262_x000001b5_8 picture_coding_extension;

	picture_coding_extension.alternate_scan = video->alternate_scan;
	picture_coding_extension.concealment_motion_vectors =
	    video->concealment_motion_vectors;
	picture_coding_extension.decode_dct_type = video->decode_dct_type;
	picture_coding_extension.decode_motion_type =
	    video->decode_motion_type;
	picture_coding_extension.decoder = video;
	picture_coding_extension.f_code_0_0 = video->f_code_0_0;
	picture_coding_extension.f_code_0_1 = video->f_code_0_1;
	picture_coding_extension.f_code_1_0 = video->f_code_1_0;
	picture_coding_extension.f_code_1_1 = video->f_code_1_1;
	picture_coding_extension.frame_pred_frame_dct =
	    video->frame_pred_frame_dct;
	picture_coding_extension.intra_dc_precision =
	    3 - video->intra_dc_precision;
	picture_coding_extension.intra_vlc_format =
	    video->B_ef_012_0 == _seek_h262_B_f_012_0;
	picture_coding_extension.picture_structure = video->picture_structure;
	picture_coding_extension.progressive_frame =
	    video->picture_substructure & progressive_frame_mask;
	picture_coding_extension.q_scale_type = video->q_scale == x7_6 ? 0 : 1;
	picture_coding_extension.repeat_first_field =
	    video->picture_substructure & repeat_first_field_mask;
	picture_coding_extension.top_field_first =
	    video->picture_substructure & top_field_first_mask;

	video->x000001b5_8_hack_1(&picture_coding_extension);
    }
#endif				/* __ENABLE_X000001B5_8_HACK_1__ */
}


static void
picture_data(struct seek_h262_video *video)
{
    /*
     * do {                                                             
     *    slice()                                                       
     * } while (nextbits() == slice_start_code)                         
     * next_start_code()                                                
     */
    struct seek_h262_release swap_release;

#if WORDS_BIGENDIAN
    if (video->start_code < 0x00000101 || 0x000001af < video->start_code) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_IMPROPER_START_CODE);
    }
#else
    if (video->start_code < 0x01010000 || 0xaf010000 < video->start_code) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_IMPROPER_START_CODE);
    }
#endif				/* WORDS_BIGENDIAN */
    if (video->picture_coding_type != bidirectionally_predictive_coded) {
	if (video->picture_structure == Frame_picture) {
	    swap_release = video->Iran_release;
	    video->Iran_release = video->Irulan_release;
	    if (video->Iran_release.flags) {
		video->do_release(video, &video->Iran_release);
	    }
	} else {
	    if (video->expect_field) {
		swap_release = video->Iran_release;
		video->Iran_release = video->Irulan_release;
		if (video->Iran_release.flags) {
		    video->do_release(video, &video->Iran_release);
		}
	    } else {
		swap_release = video->Irulan_release;
		video->Irulan_release = video->Mylene_release;
	    }
	    if (video->picture_coding_type != intra_coded) {
		_seek_h262_set_references(video);
	    }
	}
    } else {
	if (video->picture_structure != Frame_picture) {
	    if (video->picture_coding_type != intra_coded) {
		_seek_h262_set_references(video);
	    }
	}
    }
    video->minimum_useful_size = 4;
#if __ENABLE_X000001AF___HACK_x__
    if (video->x000001af___hack_X) {
	video->x000001af___hack_X(video);
    } else {
#endif				/* __ENABLE_X000001AF___HACK_x__ */
	unsigned slice_select;

	slice_select = video->picture_coding_type;
#if __ENABLE_X000001AF___HACK_0__
	if (video->x000001af___hack_0) {
# if WORDS_BIGENDIAN
	    do {
		video->x000001af___hack_0(video);
		video->slices[slice_select](video);
	    } while (0x00000100 < video->start_code
		     && video->start_code < 0x000001b0);
# else
	    do {
		video->x000001af___hack_0(video);
		video->slices[slice_select](video);
	    } while (0x01000000 < video->start_code
		     && video->start_code < 0xb0010000);
# endif				/* WORDS_BIGENDIAN */
	} else {
#endif				/* __ENABLE_X000001AF___HACK_0__ */
	    video->slices[slice_select](video);
#if __ENABLE_X000001AF___HACK_0__
	}
#endif				/* __ENABLE_X000001AF___HACK_0__ */
#if __ENABLE_X000001AF___HACK_x__
    }
#endif				/* __ENABLE_X000001AF___HACK_x__ */
    video->minimum_useful_size = 64;
    if (video->expect_field) {
	if (video->picture_coding_type == bidirectionally_predictive_coded) {
	} else {
	    video->Irulan_release = swap_release;
	}
    } else {
	video->Mylene_release.flags = video->deinterlaced_release_coding_type
	    | video->picture_substructure;
	if (video->picture_coding_type == bidirectionally_predictive_coded) {
	    video->do_release(video, &video->Mylene_release);
	} else {
	    video->Irulan_release = video->Mylene_release;
	    video->Mylene_release = swap_release;
	}
    }
}


static void
picture_display_extension(struct seek_h262_video *video)
{
    /*
     * extension_start_code_identifier                               4  uimsbf
     * for (i = 0; i < number_of_frame_centre_offsets; i++) {           
     *     frame_centre_horizontal_offset                           16  simsbf
     *     marker_bit                                                1  bslbf
     *     frame_centre_vertical_offset                             16  simsbf
     *     marker_bit                                                1  bslbf
     * }                                                                
     * next_start_code()                                                
     */
#if __ENABLE_X000001B5_7_HACK_1__
    unsigned nbytes = 5;
#endif				/* __ENABLE_X000001B5_7_HACK_1__ */

#if __ENABLE_X000001B5_7_HACK_0__
    if (video->x000001b5_7_hack_0) {
	video->x000001b5_7_hack_0(video);
    }
#endif				/* __ENABLE_X000001B5_7_HACK_0__ */
    if (!video->sequence_display_extension_seen) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_UNEXPECTED_PICTURE_DISPLAY_EXTENSION);
    } else {
#if !__ENABLE_X000001B5_7_HACK_1__
	unsigned nbytes = 5;
#endif				/* !__ENABLE_X000001B5_7_HACK_1__ */

	if (video->progressive_sequence
	    || (video->picture_structure != Frame_picture)) {
	} else if (video->picture_substructure & repeat_first_field_mask) {
	    nbytes = 14;
	} else {
	    nbytes = 9;
	}
	if (video->size < nbytes) {
	    video->optimistique_moi(video);
	}
	{
	    unsigned char *buffer;

	    buffer = video->buffer;
	    if (!(buffer[2] & 0x08) || !(buffer[4] & 0x04)
		|| (8 < nbytes
		    && (!(buffer[6] & 0x02) || !(buffer[8] & 0x01)
			|| (13 < nbytes
			    && (!(buffer[11] & 0x80)
				|| !(buffer[13] & 0x40)))))) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_UNSET_MARKER_BITS);
	    }
	}
	INCREMENT_INPUT(video, nbytes);
    }
#if __ENABLE_X000001B5_7_HACK_1__
    if (video->x000001b5_7_hack_1) {
	struct seek_h262_x000001b5_7 picture_display_extension;

	picture_display_extension.decoder = video;
	{
	    unsigned frame_centre_offset, i = 0, mask = 0x0f, nbits = 4,
		number_of_frame_centre_offsets;
	    unsigned char *buffer;

	    buffer = video->buffer - nbytes;
	    number_of_frame_centre_offsets = nbytes >> 2;
	    for (; i < number_of_frame_centre_offsets; i++) {
		if (nbits) {
		    frame_centre_offset =
			(*buffer & mask) << (16 - nbits)
			| buffer[1] << (8 - nbits) | buffer[2] >> nbits;
		    buffer += 2;
		    mask >>= 1;
		    nbits--;
		} else {
		    frame_centre_offset = buffer[1] << 8 | buffer[2];
		    buffer += 3;
		    mask = 0x7f;
		    nbits = 7;
		}
		if (frame_centre_offset & 0x8000) {
		    frame_centre_offset |= ~0 << 16;
		}
		picture_display_extension.x[i].frame_centre_horizontal_offset =
		    frame_centre_offset;
		if (nbits) {
		    frame_centre_offset =
			(*buffer & mask) << (16 - nbits)
			| buffer[1] << (8 - nbits) | buffer[2] >> nbits;
		    buffer += 2;
		    mask >>= 1;
		    nbits--;
		} else {
		    frame_centre_offset = buffer[1] << 8 | buffer[2];
		    buffer += 3;
		    mask = 0x7f;
		    nbits = 7;
		}
		if (frame_centre_offset & 0x8000) {
		    frame_centre_offset |= ~0 << 16;
		}
		picture_display_extension.x[i].frame_centre_vertical_offset =
		    frame_centre_offset;
	    }
	}

	video->x000001b5_7_hack_1(&picture_display_extension);
    }
#endif				/* __ENABLE_X000001B5_7_HACK_1__ */
    video->next_start_code(video);
}


static void
picture_header(struct seek_h262_video *video)
{
    /*
     * picture_start_code                                           32  bslbf
     * temporal_reference                                           10  uimsbf
     * picture_coding_type                                           3  uimsbf
     * vbv_delay                                                    16  uimsbf
     * if (picture_coding_type == 2 || picture_coding_type == 3) {      
     *     full_pel_forward_vector                                   1  
     *     forward_f_code                                            3  uimsbf
     * }                                                                
     * if (picture_coding_type == 3) {                                  
     *     full_pel_backward_vector                                  1  
     *     backward_f_code                                           3  uimsbf
     * }                                                                
     * while (nextbits() == '1') {                                      
     *     extra_bit_picture    slash* with the value "1" *slash     1  uimsbf
     *     extra_information_picture                                 8  
     * }                                                                
     * extra_bit_picture        slash* with the value "0" *slash     1  uimsbf
     * next_start_code()                                                
     */
    unsigned mask, nbytes;

#if __ENABLE_X00000100___HACK_0__
    if (video->x00000100___hack_0) {
	video->x00000100___hack_0(video);
    }
#endif				/* __ENABLE_X00000100___HACK_0__ */
    if (video->size < 5) {
	video->optimistique_moi(video);
    }
    {
	unsigned picture_coding_type, temporal_reference;
	unsigned char *buffer;

	buffer = video->buffer;
	picture_coding_type = (buffer[1] & 0x38) >> 3;
	if (!picture_coding_type || 0x03 < picture_coding_type) {
	    _seek_h262_abort_decoding
		((struct seek_h262_whatever *) video,
		 VIDEO_INVALID_PICTURE_CODING_TYPE);
	}
#if WORDS_BIGENDIAN
	temporal_reference = *((unsigned *) buffer + 0) >> 22;
#else
	temporal_reference = buffer[0] << 2 | buffer[1] >> 6;
#endif				/* WORDS_BIGENDIAN */
	if (video->expect_field) {
	    if (video->temporal_reference != temporal_reference) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_INVALID_FIELD_TEMPORAL_REFERENCE);
	    } else if (picture_coding_type
		       != video->deinterlaced_release_coding_type) {
		if (picture_coding_type == 0x02
		    && video->deinterlaced_release_coding_type == 0x01) {
		} else {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_INVALID_FIELD_PICTURE_CODING_TYPE);
		}
	    }
	} else {
	    video->deinterlaced_release_coding_type = picture_coding_type;
	}
	if (picture_coding_type == 0x02) {
#if WORDS_BIGENDIAN
	    if ((*(unsigned *) (buffer + 2) & 0x00078000) != 0x00038000) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_INVALID_11172_PICTURE_HEADER_INFORMATION);
	    }
#else
	    if ((*(unsigned *) (buffer + 2) & 0x00800700) != 0x00800300) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_INVALID_11172_PICTURE_HEADER_INFORMATION);
	    }
#endif				/* WORDS_BIGENDIAN */
	    mask = 0x40;
	    nbytes = 5;
#if __Berkeley_FUCK_UP__
	    video->block_0_0[0] = video->block_0_0[1];
	    video->block_0_1[0] = video->block_0_1[1];
	    video->block_0_2[0] = video->block_0_2[1];
	    video->block_0_3[0] = video->block_0_3[1];
	    video->block_0_6[0] = video->block_0_6[1];
	    video->block_1_0[0] = video->block_1_0[1];
	    video->block_1_1[0] = video->block_1_1[1];
	    video->block_1_2[0] = video->block_1_2[1];
	    video->block_1_3[0] = video->block_1_3[1];
	    video->block_1_6[0] = video->block_1_6[1];
#endif				/* __Berkeley_FUCK_UP__ */
	} else if (picture_coding_type == 0x03) {
#if WORDS_BIGENDIAN
	    if ((*(unsigned *) (buffer + 2) & 0x0007f800) != 0x0003b800) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_INVALID_11172_PICTURE_HEADER_INFORMATION);
	    }
#else
	    if ((*(unsigned *) (buffer + 2) & 0x00f80700) != 0x00b80300) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_INVALID_11172_PICTURE_HEADER_INFORMATION);
	    }
#endif				/* WORDS_BIGENDIAN */
	    mask = 0x04;
	    nbytes = 5;
#if __Berkeley_FUCK_UP__
	    video->block_0_0[0] = video->block_0_0[2];
	    video->block_0_1[0] = video->block_0_1[2];
	    video->block_0_2[0] = video->block_0_2[2];
	    video->block_0_3[0] = video->block_0_3[2];
	    video->block_0_6[0] = video->block_0_6[2];
	    video->block_1_0[0] = video->block_1_0[2];
	    video->block_1_1[0] = video->block_1_1[2];
	    video->block_1_2[0] = video->block_1_2[2];
	    video->block_1_3[0] = video->block_1_3[2];
	    video->block_1_6[0] = video->block_1_6[2];
#endif				/* __Berkeley_FUCK_UP__ */
	} else {
	    mask = 0x04;
	    nbytes = 4;
	}
	if (video->scalable_mode != SNR_scalability) {
	    switch (picture_coding_type) {
	    case 0x01:
		video->B_2345678_01 = _seek_h262_B_2_01;
		video->slice_minimum_useful_size =
		    MACROBLOCK_SIZE(B2, video->chroma_format);
		break;
	    case 0x02:
		video->B_2345678_01 = _seek_h262_B_3_01;
		video->slice_minimum_useful_size =
		    MACROBLOCK_SIZE(B3, video->chroma_format);
		break;
	    case 0x03:
		video->B_2345678_01 = _seek_h262_B_4_01;
		video->slice_minimum_useful_size =
		    MACROBLOCK_SIZE(B4, video->chroma_format);
	    }
	}
	video->picture_coding_type = picture_coding_type;
	video->temporal_reference = temporal_reference;
    }
#if __ENABLE_X00000100___HACK_1__
    if (video->x00000100___hack_1) {
	struct seek_h262_x00000100__ picture_header;

	picture_header.decoder = video;
	picture_header.picture_coding_type = video->picture_coding_type;
	picture_header.temporal_reference = video->temporal_reference;

	video->x00000100___hack_1(&picture_header);
    }
#endif				/* __ENABLE_X00000100___HACK_1__ */
#if __ENABLE_X00000100___HACK_2__
    if (video->x00000100___hack_2) {
	_seek_h262_x00000100___hack_2(video);
    }
#endif				/* __ENABLE_X00000100___HACK_2__ */
    {
	unsigned char *buffer;

	buffer = video->buffer + nbytes - 1;
	while (*buffer & mask) {
	    mask >>= 1;
	    if (!mask) {
		buffer += 2;
		mask = 0x80;
		nbytes += 2;
	    } else {
		buffer++;
		nbytes++;
	    }
	    if (video->size < nbytes) {
		INCREMENT_INPUT(video, nbytes - 1);
		video->optimistique_moi(video);
		buffer = video->buffer;
		nbytes = 1;
	    }
	}
	video->picture_spatial_scalable_extension_seen = 0;
	video->spatial_temporal_weight_code_table_index = 0;
	INCREMENT_INPUT(video, nbytes);
    }
    video->next_start_code(video);
}


static void
picture_spatial_scalable_extension(struct seek_h262_video *video)
{
    /*
     * extension_start_code_identifier                               4  uimsbf
     * lower_layer_temporal_reference                               10  uimsbf
     * marker_bit                                                    1  bslbf
     * lower_layer_horizontal_offset                                15  simsbf
     * marker_bit                                                    1  bslbf
     * lower_layer_vertical_offset                                  15  simsbf
     * spatial_temporal_weight_code_table_index                      2  uimsbf
     * lower_layer_progressive_frame                                 1  uimsbf
     * lower_layer_deinterlaced_field_select                         1  uimsbf
     * next_start_code()                                                
     */
#if __ENABLE_X000001B5_9_HACK_0__
    if (video->x000001b5_9_hack_0) {
	video->x000001b5_9_hack_0(video);
    }
#endif				/* __ENABLE_X000001B5_9_HACK_0__ */
    if (video->size < 7) {
	video->optimistique_moi(video);
    }
    {
	unsigned char *buffer;

	buffer = video->buffer;
	if (!(buffer[1] & 0x02) || !(buffer[3] & 0x02)) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				      VIDEO_UNSET_MARKER_BITS);
	}
	if (video->scalable_mode != SNR_scalability) {
	    switch (video->picture_coding_type) {
	    case bidirectionally_predictive_coded:
		video->B_2345678_01 = _seek_h262_B_7_01;
		video->slice_minimum_useful_size =
		    MACROBLOCK_SIZE(B7, video->chroma_format);
		break;
	    case intra_coded:
		video->B_2345678_01 = _seek_h262_B_5_01;
		video->slice_minimum_useful_size =
		    MACROBLOCK_SIZE(B5, video->chroma_format);
		break;
	    case predictive_coded:
		video->B_2345678_01 = _seek_h262_B_6_01;
		video->slice_minimum_useful_size =
		    MACROBLOCK_SIZE(B6, video->chroma_format);
	    }
	}
	video->spatial_temporal_weight_code_table_index = buffer[5] & 3;
	video->picture_spatial_scalable_extension_seen = 1;
	INCREMENT_INPUT(video, 7);
    }
#if __ENABLE_X000001B5_9_HACK_1__
    if (video->x000001b5_9_hack_1) {
	struct seek_h262_x000001b5_9 picture_spatial_scalable_extension;

	picture_spatial_scalable_extension.decoder = video;
	{
	    unsigned lower_layer_offset;
	    unsigned char *buffer;

	    buffer = video->buffer - 7;
	    picture_spatial_scalable_extension.
		lower_layer_deinterlaced_field_select =
		(buffer[6] & 0x40) >> 6;
	    lower_layer_offset = buffer[2] << 6 | buffer[3] >> 2;
	    if (buffer[1] & 0x01) {
		lower_layer_offset |= ~0 << 14;
	    }
	    picture_spatial_scalable_extension.lower_layer_horizontal_offset =
		lower_layer_offset;
	    picture_spatial_scalable_extension.lower_layer_progressive_frame =
		buffer[6] >> 7;
	    picture_spatial_scalable_extension.lower_layer_temporal_reference =
		(*buffer & 0x0f) << 6 | buffer[1] >> 2;
	    lower_layer_offset = buffer[4] << 6 | (buffer[5] & 0xfc) >> 2;
	    if (buffer[3] & 0x01) {
		lower_layer_offset |= ~0 << 14;
	    }
	    picture_spatial_scalable_extension.lower_layer_vertical_offset =
		lower_layer_offset;
	    picture_spatial_scalable_extension.
		spatial_temporal_weight_code_table_index =
		video->spatial_temporal_weight_code_table_index;
	}

	video->x000001b5_9_hack_1(&picture_spatial_scalable_extension);
    }
#endif				/* __ENABLE_X000001B5_9_HACK_1__ */
    video->next_start_code(video);
}


static void
picture_temporal_scalable_extension(struct seek_h262_video *video)
{
    /*
     * extension_start_code_identifier                               4  uimsbf
     * reference_select_code                                         2  uimsbf
     * forward_temporal_reference                                   10  uimsbf
     * marker_bit                                                    1  bslbf
     * backward_temporal_reference                                  10  uimsbf
     * next_start_code()                                                
     */
#if __ENABLE_X000001B5_A_HACK_0__
    if (video->x000001b5_a_hack_0) {
	video->x000001b5_a_hack_0(video);
    }
#endif				/* __ENABLE_X000001B5_A_HACK_0__ */
    if (video->size < 4) {
	video->optimistique_moi(video);
    }
    {
	unsigned char *buffer;

	buffer = video->buffer;
	if (!(buffer[2] & 0x80)) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				      VIDEO_UNSET_MARKER_BIT);
	}
	INCREMENT_INPUT(video, 4);
    }
#if __ENABLE_X000001B5_A_HACK_1__
    if (video->x000001b5_a_hack_1) {
	struct seek_h262_x000001b5_a picture_temporal_scalable_extension;

	picture_temporal_scalable_extension.decoder = video;
	{
	    unsigned char *buffer;

	    buffer = video->buffer - 4;
	    picture_temporal_scalable_extension.backward_temporal_reference =
		(buffer[2] & 0x7f) << 3 | (buffer[3] >> 5);
	    picture_temporal_scalable_extension.forward_temporal_reference =
		(*buffer & 0x03) << 8 | buffer[1];
	    picture_temporal_scalable_extension.reference_select_code =
		(*buffer & 0x0c) >> 2;
	}

	video->x000001b5_a_hack_1(&picture_temporal_scalable_extension);
    }
#endif				/* __ENABLE_X000001B5_A_HACK_1__ */
    video->next_start_code(video);
}


static void
sequence_display_extension(struct seek_h262_video *video)
{
    /*
     * extension_start_code_identifier                               4  uimsbf
     * video_format                                                  3  uimsbf
     * colour_description                                            1  uimsbf
     * if (colour_description) {                                        
     *     colour_primaries                                          8  uimsbf
     *     transfer_characteristics                                  8  uimsbf
     *     matrix_coefficients                                       8  uimsbf
     * }                                                                
     * display_horizontal_size                                      14  uimsbf
     * marker_bit                                                    1  bslbf
     * display_vertical_size                                        14  uimsbf
     * next_start_code()                                                
     */
#if __ENABLE_X000001B5_2_HACK_1__
    unsigned nbytes = 5;
#endif				/* __ENABLE_X000001B5_2_HACK_1__ */

#if __ENABLE_X000001B5_2_HACK_0__
    if (video->x000001b5_2_hack_0) {
	video->x000001b5_2_hack_0(video);
    }
#endif				/* __ENABLE_X000001B5_2_HACK_0__ */
    if (video->size < 8) {
	video->optimistique_moi(video);
    }
    {
	unsigned char *buffer;

	buffer = video->buffer;
	if (*buffer & 1) {
	    if (!(buffer[5] & 2)) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_UNSET_MARKER_BIT);
	    }
#if __ENABLE_X000001B5_2_HACK_1__
	    nbytes = 8;
#endif				/* __ENABLE_X000001B5_2_HACK_1__ */
	    INCREMENT_INPUT(video, 8);
	} else {
	    if (!(buffer[2] & 2)) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_UNSET_MARKER_BIT);
	    }
	    INCREMENT_INPUT(video, 5);
	}
	video->sequence_display_extension[0] = *buffer;
	video->sequence_display_extension[1] = buffer[1];
	video->sequence_display_extension[2] = buffer[2];
	video->sequence_display_extension[3] = buffer[3];
	video->sequence_display_extension[4] = buffer[4];
	if (*buffer & 1) {
	    video->sequence_display_extension[5] = buffer[5];
	    video->sequence_display_extension[6] = buffer[6];
	    video->sequence_display_extension[7] = buffer[7];
	}
	video->sequence_display_extension_seen = 1;
    }
#if __ENABLE_X000001B5_2_HACK_1__
    if (video->x000001b5_2_hack_1) {
	struct seek_h262_x000001b5_2 sequence_display_extension;

	sequence_display_extension.decoder = video;
	{
	    unsigned char *buffer;

	    buffer = video->buffer - nbytes;
	    sequence_display_extension.colour_description = *buffer & 1;
	    if (sequence_display_extension.colour_description) {
		sequence_display_extension.colour_primaries = buffer[1];
		sequence_display_extension.matrix_coefficients = buffer[3];
		sequence_display_extension.transfer_characteristics =
		    buffer[2];
		buffer += 3;
	    }
	    sequence_display_extension.display_horizontal_size =
		buffer[1] << 6 | buffer[2] >> 2;
	    sequence_display_extension.display_vertical_size =
		(buffer[2] & 0x01) << 13 | buffer[3] << 5 | buffer[4] >> 3;
	    sequence_display_extension.video_format = (*buffer & 0x0e) >> 1;
	}

	video->x000001b5_2_hack_1(&sequence_display_extension);
    }
#endif				/* __ENABLE_X000001B5_2_HACK_1__ */
    video->next_start_code(video);
}


static void
sequence_extension(struct seek_h262_video *video)
{
    /*
     * extension_start_code                                         32  bslbf
     * extension_start_code_identifier                               4  uimsbf
     * profile_and_level_indication                                  8  uimsbf
     * progressive_sequence                                          1  uimsbf
     * chroma_format                                                 2  uimsbf
     * horizontal_size_extension                                     2  uimsbf
     * vertical_size_extension                                       2  uimsbf
     * bit_rate_extension                                           12  uimsbf
     * marker_bit                                                    1  bslbf
     * vbv_buffer_size_extension                                     8  uimsbf
     * low_delay                                                     1  uimsbf
     * frame_rate_extension_n                                        2  uimsbf
     * frame_rate_extension_d                                        5  uimsbf
     * next_start_code()                                                
     */
#if __ENABLE_X000001B5_1_HACK_0__
    if (video->x000001b5_1_hack_0) {
	video->x000001b5_1_hack_0(video);
    }
#endif				/* __ENABLE_X000001B5_1_HACK_0__ */
    if (video->size < 6) {
	video->optimistique_moi(video);
    }
    {
	unsigned *buffer;

	buffer = (unsigned *) video->buffer;
	if (((*(unsigned char *) buffer) & 0xf0) != Sequence_Extension_ID) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				      VIDEO_INVALID_EXTENSION_IDENTIFIER);
	} {
	    unsigned packed_fields;

	    packed_fields = *buffer;
#if WORDS_BIGENDIAN
	    if (!(packed_fields & 0x00000001)) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_UNSET_MARKER_BIT);
	    }
	    if (!(packed_fields & 0x00060000)) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_INVALID_CHROMA_FORMAT);
	    }
	    video->horizontal_size |= (packed_fields & 0x00018000) >> 3;
	    video->chroma_format =
		4 + (1 << ((packed_fields & 0x00060000) >> 17));
	    video->progressive_sequence = packed_fields & 0x00080000;
	    video->vertical_size |= (packed_fields & 0x00006000) >> 1;
	    if (packed_fields & 0x00080000) {
		video->dmv_abort = ~0;
	    }
#else
	    if (!(packed_fields & 0x01000000)) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_UNSET_MARKER_BIT);
	    }
	    if (!(packed_fields & 0x00000600)) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_INVALID_CHROMA_FORMAT);
	    }
	    video->horizontal_size |= (packed_fields & 0x00800000) >> 11
		| (packed_fields & 0x00000100) << 5;
	    video->chroma_format =
		4 + (1 << ((packed_fields & 0x00000600) >> 9));
	    video->progressive_sequence = packed_fields & 0x00000800;
	    video->vertical_size |= (packed_fields & 0x00600000) >> 9;
	    if (packed_fields & 0x00000800) {
		video->dmv_abort = ~0;
	    }
#endif				/* WORDS_BIGENDIAN */
	    if (video->chroma_format != H2V2) {
		_seek_h262_copy_quantiser
		    (video->chroma_intra_quantiser_matrix[0],
		     video->intra_quantiser_matrix[0]);
		_seek_h262_copy_quantiser
		    (video->chroma_non_intra_quantiser_matrix[0],
		     video->non_intra_quantiser_matrix[0]);
	    }
#if WORDS_BIGENDIAN
	    if (packed_fields & 0x00080000) {
		video->mb_height = (video->vertical_size + 15) >> 4;
	    } else {
		video->mb_height = ((video->vertical_size + 31) >> 5) << 1;
	    }
#else
	    if (packed_fields & 0x00000800) {
		video->mb_height = (video->vertical_size + 15) >> 4;
	    } else {
		video->mb_height = ((video->vertical_size + 31) >> 5) << 1;
	    }
#endif				/* WORDS_BIGENDIAN */
	    video->mb_width = (video->horizontal_size + 15) >> 4;
	    video->macroblocks_count = video->mb_height * video->mb_width;
	    video->luminance_line_size = video->mb_width << 4;
#if WORDS_BIGENDIAN
	    switch (packed_fields & 0x00060000) {
	    case 0x00020000:
		video->chrominance_line_size = video->mb_width << 3;
		break;
	    case 0x00040000:
		if (!(packed_fields & 0x00080000)) {
		}
		video->chrominance_line_size = video->mb_width << 3;
		break;
	    case 0x00060000:
		if (!(packed_fields & 0x00080000)) {
		}
		video->chrominance_line_size = video->mb_width << 4;
	    }
	    if (!(packed_fields & 0x00080000)) {
	    }
#else
	    switch (packed_fields & 0x00000600) {
	    case 0x00000200:
		video->chrominance_line_size = video->mb_width << 3;
		break;
	    case 0x00000400:
		if (!(packed_fields & 0x00000800)) {
		}
		video->chrominance_line_size = video->mb_width << 3;
		break;
	    case 0x00000600:
		if (!(packed_fields & 0x00000800)) {
		}
		video->chrominance_line_size = video->mb_width << 4;
	    }
	    if (!(packed_fields & 0x00000800)) {
	    }
#endif				/* WORDS_BIGENDIAN */
	    video->Irulan_release.flags = 0;
	}
	video->sequence_display_extension_seen = 0;
	video->sequence_scalable_extension_seen = 0;
	INCREMENT_INPUT(video, 6);
	video->sequence_extension[0] = ((unsigned char *) buffer)[0];
	video->sequence_extension[1] = ((unsigned char *) buffer)[1];
	video->sequence_extension[2] = ((unsigned char *) buffer)[2];
	video->sequence_extension[3] = ((unsigned char *) buffer)[3];
	video->sequence_extension[4] = ((unsigned char *) buffer)[4];
	video->sequence_extension[5] = ((unsigned char *) buffer)[5];
	video->next_start_code(video);
    }
#if __ENABLE_X000001B5_1_HACK_1__
    if (video->x000001b5_1_hack_1) {
	struct seek_h262_x000001b5_1 sequence_extension;

# if WORDS_BIGENDIAN
	sequence_extension.bit_rate = video->sequence_header[1] >> 14;
# else
	{
	    unsigned bit_rate;

	    bit_rate = video->sequence_header[1];
	    sequence_extension.bit_rate = (bit_rate & 0x000000ff) << 10
		| (bit_rate & 0x0000ff00) >> 6 | (bit_rate & 0x00c00000) >> 22;
	}
# endif				/* WORDS_BIGENDIAN */
	sequence_extension.bit_rate |=
	    (video->sequence_extension[2] & 0x1f) << 25;
	sequence_extension.bit_rate |=
	    (video->sequence_extension[3] & 0xf7) << 17;
	sequence_extension.chroma_format = video->chroma_format;
	sequence_extension.decoder = video;
	if (video->chroma_format != H2V2) {
	    _seek_h262_dump_quantiser
		(video->chroma_intra_quantiser_matrix[0],
		 sequence_extension.chroma_intra_quantiser_matrix);
	    _seek_h262_dump_quantiser
		(video->chroma_non_intra_quantiser_matrix[0],
		 sequence_extension.chroma_non_intra_quantiser_matrix);
	}
	sequence_extension.frame_rate_extension_d =
	    video->sequence_extension[5] & 31;
	sequence_extension.frame_rate_extension_n =
	    (video->sequence_extension[5] & 96) >> 5;
	sequence_extension.horizontal_size = video->horizontal_size;
# if WORDS_BIGENDIAN
	sequence_extension.load_intra_quantiser_matrix =
	    video->sequence_header[1] & 0x00000002;
	sequence_extension.load_non_intra_quantiser_matrix =
	    video->sequence_header[1] & 0x00000001;
# else
	sequence_extension.load_intra_quantiser_matrix =
	    video->sequence_header[1] & 0x02000000;
	sequence_extension.load_non_intra_quantiser_matrix =
	    video->sequence_header[1] & 0x01000000;
# endif				/* WORDS_BIGENDIAN */
	sequence_extension.profile_and_level_indication =
	    (video->sequence_extension[0] & 15) << 4
	    | video->sequence_extension[1] >> 4;
	sequence_extension.progressive_sequence = video->progressive_sequence;
	sequence_extension.synthetic = 0;
	sequence_extension.vertical_size = video->vertical_size;

	_seek_h262_dump_quantiser
	    (video->intra_quantiser_matrix[0],
	     sequence_extension.intra_quantiser_matrix);
	_seek_h262_dump_quantiser
	    (video->non_intra_quantiser_matrix[0],
	     sequence_extension.non_intra_quantiser_matrix);

	video->x000001b5_1_hack_1(&sequence_extension);
    }
#endif				/* __ENABLE_X000001B5_1_HACK_1__ */

    _seek_h262_compute_13818_motion_limit(video);

    _seek_h262_alloc_storage
	(video, video->macroblocks_count << 8,
	 (video->macroblocks_count << 6) * ((video->chroma_format - 4) >> 1));
}


static void
sequence_header(struct seek_h262_video *video)
{
    /*
     * ISO/IEC 13818-2
     */
    /*
     * sequence_header_code                                         32  bslbf
     * horizontal_size_value                                        12  uimsbf
     * vertical_size_value                                          12  uimsbf
     * aspect_ratio_information                                      4  uimsbf
     * frame_rate_code                                               4  uimsbf
     * bit_rate_value                                               18  uimsbf
     * marker_bit                                                    1  bslbf
     * vbv_buffer_size_value                                        10  uimsbf
     * constrained_parameters_flag                                   1  
     * load_intra_quantizer_matrix                                   1  
     * if (load_intra_quantizer_matrix) {                               
     *     intra_quantizer_matrix[64]                           8 * 64  uimsbf
     * }                                                             1  
     * load_non_intra_quantizer_matrix                                  
     * if (load_non_intra_quantizer_matrix) {                           
     *     non_intra_quantizer_matrix[64]                       8 * 64  uimsbf
     * }                                                                
     * next_start_code()                                                
     */
    /*
     * ISO/IEC 11172-2
     */
    /*
     * sequence_header_code                                         32  bslbf
     * horizontal_size                                              12  uimsbf
     * vertical_size                                                12  uimsbf
     * pel_aspect_ratio                                              4  uimsbf
     * picture_rate                                                  4  uimsbf
     * bit_rate                                                     18  uimsbf
     * marker_bit                                                    1  "1"
     * vbv_buffer_size                                              10  uimsbf
     * constrained_parameter_flag                                    1  
     * load_intra_quantizer_matrix                                   1  
     * if (load_intra_quantizer_matrix) {                               
     *     intra_quantizer_matrix[64]                           8 * 64  uimsbf
     * }                                                             1  
     * load_non_intra_quantizer_matrix                                  
     * if (load_non_intra_quantizer_matrix) {                           
     *     non_intra_quantizer_matrix[64]                       8 * 64  uimsbf
     * }                                                                
     * next_start_code()                                                
     * if (nextbits() == extension_start_code) {                        
     *     extension_start_code                                      32 bslbf
     *     while (nextbits() != '0000 0000 0000 0000 0000 0001') {      
     *         sequence_extension_data                                8 
     *     }                                                            
     *     next_start_code()                                            
     * }                                                                
     * if (nextbits() == user_data_start_code) {                        
     *     user_data_start_code                                      32 bslbf
     *     while (nextbits() != '0000 0000 0000 0000 0000 0001') {      
     *         user_data                                              8 
     *     }                                                            
     *     next_start_code()                                            
     * }                                                                
     */
#if __ENABLE_X000001B3___HACK_0__
    if (video->x000001b3___hack_0) {
	video->x000001b3___hack_0(video);
    }
#endif				/* __ENABLE_X000001B3___HACK_0__ */
    if (video->size < 8) {
	video->optimistique_moi(video);
    }
    {
	unsigned *buffer;

	buffer = (unsigned *) video->buffer;
	if (video->start_code != sequence_header_code) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				      VIDEO_IMPROPER_START_CODE);
	} {
	    unsigned packed_fields;

	    packed_fields = *buffer;
#if WORDS_BIGENDIAN
	    if (!(packed_fields & 0xfff00000) || !(packed_fields & 0x000fff00)
		|| !(packed_fields & 0x000000f0)
		|| !(packed_fields & 0x0000000f)) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_INVALID_SEQUENCE_HEADER);
	    }
	    video->horizontal_size = (packed_fields & 0xfff00000) >> 20;
	    video->vertical_size = (packed_fields & 0x000fff00) >> 8;
#else
	    if (!(packed_fields & 0x0000f0ff) || !(packed_fields & 0x00ff0f00)
		|| !(packed_fields & 0xf0000000)
		|| !(packed_fields & 0x0f000000)) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_INVALID_SEQUENCE_HEADER);
	    }
	    video->horizontal_size = (packed_fields & 0x000000ff) << 4
		| (packed_fields & 0x0000f000) >> 12;
	    video->vertical_size = (packed_fields & 0x00ff0000) >> 16
		| (packed_fields & 0x00000f00);
#endif				/* WORDS_BIGENDIAN */
	}
	if (!((unsigned char *) buffer)[6] & 0x20) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				      VIDEO_UNSET_MARKER_BIT);
	}
	INCREMENT_INPUT(video, 8);
	video->sequence_header[0] = *buffer;
#if WORDS_BIGENDIAN
	video->sequence_header[1] = buffer[1] & 0xfffffffe;
#else
	video->sequence_header[1] = buffer[1] & 0xfeffffff;
#endif				/* WORDS_BIGENDIAN */
	buffer += 2;
	if (((unsigned char *) buffer)[-1] & 0x02) {
	    unsigned char save_buffer, quantiser_matrix[64];

	    save_buffer = ((unsigned char *) buffer)[-1];
	    if (video->size < 64) {
		video->optimistique_moi(video);
		buffer = (unsigned *) video->buffer;
	    }
	    copy_sixteen((unsigned *) quantiser_matrix, buffer);
	    {
		unsigned i = 63;
		unsigned char *buffer;

		buffer = quantiser_matrix + 63;
		for (; i; i--) {
		    *buffer = (*buffer >> 1) | (buffer[-1] << 7);
		    if (!*buffer--) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) video,
			     VIDEO_NULL_INTRA_QUANTISER);
		    }
		}
		*buffer = (*buffer >> 1) | (save_buffer << 7);
		if (*buffer != 8) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_INVALID_INTRA_QUANTISER);
		}
	    }
	    _seek_h262_set_quantiser
		(quantiser_matrix, video->intra_quantiser_matrix[0]);
	    INCREMENT_INPUT(video, 64);
	    buffer += 16;
	} else {
	    _seek_h262_set_quantiser
		(default_intra_quantiser_matrix,
		 video->intra_quantiser_matrix[0]);
	}
#if WORDS_BIGENDIAN
	video->sequence_header[1] |= buffer[-1] & 0x00000001;
#else
	video->sequence_header[1] |= buffer[-1] & 0x01000000;
#endif				/* WORDS_BIGENDIAN */
	if (((unsigned char *) buffer)[-1] & 0x01) {
	    unsigned char quantiser_matrix[64];

	    if (video->size < 64) {
		video->optimistique_moi(video);
		buffer = (unsigned *) video->buffer;
	    }
	    INCREMENT_INPUT(video, 64);
	    copy_sixteen((unsigned *) quantiser_matrix, buffer);
	    {
		unsigned i = 64;
		unsigned char *buffer;

		buffer = quantiser_matrix;
		for (; i; i--) {
		    if (!*buffer++) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) video,
			     VIDEO_NULL_NON_INTRA_QUANTISER);
		    }
		}
	    }
	    _seek_h262_set_quantiser
		(quantiser_matrix, video->non_intra_quantiser_matrix[0]);
	} else {
	    _seek_h262_reset_quantiser(video->non_intra_quantiser_matrix[0]);
	}
	video->next_start_code(video);
    }
#if __ENABLE_X000001B3___HACK_1__
    if (video->x000001b3___hack_1) {
	struct seek_h262_x000001b3__ sequence_header;

# if WORDS_BIGENDIAN
	sequence_header.aspect_ratio_information =
	    (video->sequence_header[0] >> 4) & 15;
# else
	sequence_header.aspect_ratio_information =
	    (video->sequence_header[0] >> 28) & 15;
# endif				/* WORDS_BIGENDIAN */
	sequence_header.decoder = video;
# if WORDS_BIGENDIAN
	sequence_header.frame_rate_code = video->sequence_header[0] & 15;
# else
	sequence_header.frame_rate_code =
	    (video->sequence_header[0] >> 24) & 15;
# endif				/* WORDS_BIGENDIAN */

	video->x000001b3___hack_1(&sequence_header);
    }
#endif				/* __ENABLE_X000001B3___HACK_1__ */
}


static void
sequence_scalable_extension(struct seek_h262_video *video)
{
    /*
     * extension_start_code_identifier                               4  uimsbf
     * scalable_mode                                                 2  uimsbf
     * layer_id                                                      4  uimsbf
     * if (scalable_mode == "spatial scalability") {                    
     *     lower_layer_prediction_horizontal_size                   14  uimsbf
     *     marker_bit                                                1  bslbf
     *     lower_layer_prediction_vertical_size                     14  uimsbf
     *     horizontal_subsampling_factor_m                           5  uimsbf
     *     horizontal_subsampling_factor_n                           5  uimsbf
     *     vertical_subsampling_factor_m                             5  uimsbf
     *     vertical_subsampling_factor_n                             5  uimsbf
     * }                                                                
     * if (scalable_mode == "temporal scalability") {                   
     *     picture_mux_enable                                        1  uimsbf
     *     if (picture_mux_enable) {                                    
     *         mux_to_progressive_sequence                           1  uimsbf
     *     }                                                            
     *     picture_mux_order                                         3  uimsbf
     *     picture_mux_factor                                        3  uimsbf
     * }                                                                
     * next_start_code()                                                
     */
#if __ENABLE_X000001B5_5_HACK_0__
    if (video->x000001b5_5_hack_0) {
	video->x000001b5_5_hack_0(video);
    }
#endif				/* __ENABLE_X000001B5_5_HACK_0__ */
    if (video->size < 2) {
	video->optimistique_moi(video);
    }
    {
	unsigned scalable_mode;
	unsigned char *buffer;

	buffer = video->buffer;
	scalable_mode = *buffer & 0x0c;
	if (scalable_mode == 0x04) {
	    if (video->size < 8) {
		video->optimistique_moi(video);
		buffer = video->buffer;
	    }
	    if (!(buffer[3] & 0x80)) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_UNSET_MARKER_BIT);
	    }
	    INCREMENT_INPUT(video, 8);
	} else if (scalable_mode == 0x08) {
	    video->B_2345678_01 = _seek_h262_B_8_01;
	    video->slice_minimum_useful_size =
		MACROBLOCK_SIZE(B8, video->chroma_format);
	    INCREMENT_INPUT(video, 2);
	} else if (scalable_mode == 0x0c) {
	    if (video->size < 3) {
		video->optimistique_moi(video);
		buffer = video->buffer;
	    }
	    INCREMENT_INPUT(video, 3);
	} else {
	    INCREMENT_INPUT(video, 2);
	}
	video->scalable_mode = scalable_mode;
	video->sequence_scalable_extension[0] = *buffer;
	video->sequence_scalable_extension[1] = buffer[1];
	if (scalable_mode == 0x04) {
	    video->sequence_scalable_extension[2] = buffer[2];
	    video->sequence_scalable_extension[3] = buffer[3];
	    video->sequence_scalable_extension[4] = buffer[4];
	    video->sequence_scalable_extension[5] = buffer[5];
	    video->sequence_scalable_extension[6] = buffer[6];
	    video->sequence_scalable_extension[7] = buffer[7];
	} else if (scalable_mode == 0x0c) {
	    video->sequence_scalable_extension[2] = buffer[2];
	}
	video->sequence_scalable_extension_seen |= 1;
    }
#if __ENABLE_X000001B5_5_HACK_1__
    if (video->x000001b5_5_hack_1) {
	struct seek_h262_x000001b5_5 sequence_scalable_extension;

	sequence_scalable_extension.decoder = video;
	if (video->scalable_mode == spatial_scalability) {
	    unsigned char *buffer;

	    buffer = video->buffer - 8;
	    sequence_scalable_extension.layer_id =
		(*buffer & 0x03) << 2 | buffer[1] >> 6;
	    sequence_scalable_extension.x.x0.horizontal_subsampling_factor_m =
		(buffer[4] & 0x01) << 4 | buffer[5] >> 4;
	    sequence_scalable_extension.x.x0.horizontal_subsampling_factor_n =
		(buffer[5] & 0x0f) << 1 | buffer[6] >> 7;
	    sequence_scalable_extension.x.x0.
		lower_layer_prediction_horizontal_size =
		(buffer[1] & 0x3f) << 8 | buffer[2];
	    sequence_scalable_extension.x.x0.
		lower_layer_prediction_vertical_size =
		(buffer[3] & 0x7f) << 7 | buffer[4] >> 1;
	    sequence_scalable_extension.x.x0.vertical_subsampling_factor_m =
		(buffer[6] & 0x7c) >> 2;
	    sequence_scalable_extension.x.x0.vertical_subsampling_factor_n =
		(buffer[6] & 0x03) << 3 | buffer[7] >> 5;
	} else if (video->scalable_mode == temporal_scalability) {
	    unsigned char *buffer;

	    buffer = video->buffer - 3;
	    sequence_scalable_extension.layer_id =
		(*buffer & 0x03) << 2 | buffer[1] >> 6;
	    if (buffer[1] & 0x20) {
		sequence_scalable_extension.x.x1.picture_mux_enable = 1;
		sequence_scalable_extension.x.x1.mux_to_progressive_sequence =
		    (buffer[1] & 0x10) >> 4;
		sequence_scalable_extension.x.x1.picture_mux_order =
		    (buffer[1] & 0x0e) >> 1;
		sequence_scalable_extension.x.x1.picture_mux_factor =
		    (buffer[1] & 0x01) << 2 | buffer[2] >> 6;
	    } else {
		sequence_scalable_extension.x.x1.picture_mux_enable = 0;
		sequence_scalable_extension.x.x1.picture_mux_order =
		    (buffer[1] & 0x1c) >> 2;
		sequence_scalable_extension.x.x1.picture_mux_factor =
		    (buffer[1] & 0x03) << 1 | buffer[2] >> 7;
	    }
	} else {
	    unsigned char *buffer;

	    buffer = video->buffer - 2;
	    sequence_scalable_extension.layer_id =
		(*buffer & 0x03) << 2 | buffer[1] >> 6;
	}
	sequence_scalable_extension.scalable_mode = video->scalable_mode;

	video->x000001b5_5_hack_1(&sequence_scalable_extension);
    }
#endif				/* __ENABLE_X000001B5_5_HACK_1__ */
    video->next_start_code(video);
}


static void
user_data(struct seek_h262_video *video)
{
    /*
     * user_data_start_code                                         32  bslbf
     * while (nextbits() != '0000 0000 0000 0000 0000 0001') {          
     *     user_data                                                 8  
     * }                                                                
     * next_start_code()                                                
     */
#if __ENABLE_X000001B2___HACK_0__
    if (video->x000001b2___hack_0) {
	video->x000001b2___hack_0(video);
    }
#endif				/* __ENABLE_X000001B2___HACK_0__ */
    _seek_h262_artificial_extension_or_user_data(video);
}


static void
subsequent_sequence_display_extension(struct seek_h262_video *video)
{
#if __ENABLE_X000001B5_2_HACK_0__
    if (video->x000001b5_2_hack_0) {
	video->x000001b5_2_hack_0(video);
    }
#endif				/* __ENABLE_X000001B5_2_HACK_0__ */
    if (video->size < 8) {
	video->optimistique_moi(video);
    }
    if (*video->buffer & 1) {
	if (video->sequence_display_extension[0] != video->buffer[0]
	    || video->sequence_display_extension[1] != video->buffer[1]
	    || video->sequence_display_extension[2] != video->buffer[2]
	    || video->sequence_display_extension[3] != video->buffer[3]
	    || video->sequence_display_extension[4] != video->buffer[4]
	    || video->sequence_display_extension[5] != video->buffer[5]
	    || video->sequence_display_extension[6] != video->buffer[6]
	    || video->sequence_display_extension[7] != video->buffer[7]) {
	    _seek_h262_abort_decoding
		((struct seek_h262_whatever *) video,
		 VIDEO_CHANGED_SEQUENCE_DISPLAY_EXTENSION);
	}
	INCREMENT_INPUT(video, 8);
    } else {
	if (video->sequence_display_extension[0] != video->buffer[0]
	    || video->sequence_display_extension[1] != video->buffer[1]
	    || video->sequence_display_extension[2] != video->buffer[2]
	    || video->sequence_display_extension[3] != video->buffer[3]
	    || video->sequence_display_extension[4] != video->buffer[4]) {
	    _seek_h262_abort_decoding
		((struct seek_h262_whatever *) video,
		 VIDEO_CHANGED_SEQUENCE_DISPLAY_EXTENSION);
	}
	INCREMENT_INPUT(video, 5);
    }
    video->sequence_display_extension_seen |= 1;
    video->next_start_code(video);
}


static void
subsequent_sequence_extension(struct seek_h262_video *video)
{
#if __ENABLE_X000001B5_1_HACK_0__
    if (video->x000001b5_1_hack_0) {
	video->x000001b5_1_hack_0(video);
    }
#endif				/* __ENABLE_X000001B5_1_HACK_0__ */
    if (video->size < 6) {
	video->optimistique_moi(video);
    }
    if (video->start_code != extension_start_code) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_IMPROPER_START_CODE);
    } else if (video->sequence_extension[0] != video->buffer[0]
	       || video->sequence_extension[1] != video->buffer[1]
	       || video->sequence_extension[2] != video->buffer[2]
	       || video->sequence_extension[3] != video->buffer[3]
	       || video->sequence_extension[4] != video->buffer[4]
	       || video->sequence_extension[5] != video->buffer[5]) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_SEQUENCE_EXTENSION);
	_seek_h262_changed_sequence_extension(video);
    } else {
	INCREMENT_INPUT(video, 6);
	video->next_start_code(video);
	video->sequence_display_extension_seen &= ~1;
	video->sequence_scalable_extension_seen &= ~1;
	if (video->chroma_format != H2V2) {
	    _seek_h262_copy_quantiser
		(video->chroma_intra_quantiser_matrix[0],
		 video->intra_quantiser_matrix[0]);
	    _seek_h262_copy_quantiser
		(video->chroma_non_intra_quantiser_matrix[0],
		 video->non_intra_quantiser_matrix[0]);
	}
    }
#if __ENABLE_X000001B5_1_HACK_2__
    if (video->x000001b5_1_hack_2) {
	struct seek_h262_x000001b5_3 quant_matrix_extension;

	quant_matrix_extension.decoder = video;
	if (video->chroma_format != H2V2) {
	    _seek_h262_dump_quantiser
		(video->chroma_intra_quantiser_matrix[0],
		 quant_matrix_extension.chroma_intra_quantiser_matrix);
	    _seek_h262_dump_quantiser
		(video->chroma_non_intra_quantiser_matrix[0],
		 quant_matrix_extension.chroma_non_intra_quantiser_matrix);
	}
# if WORDS_BIGENDIAN
	quant_matrix_extension.load_intra_quantiser_matrix =
	    video->sequence_header[1] & 0x00000002;
	quant_matrix_extension.load_non_intra_quantiser_matrix =
	    video->sequence_header[1] & 0x00000001;
# else
	quant_matrix_extension.load_intra_quantiser_matrix =
	    video->sequence_header[1] & 0x02000000;
	quant_matrix_extension.load_non_intra_quantiser_matrix =
	    video->sequence_header[1] & 0x01000000;
# endif				/* WORDS_BIGENDIAN */

	_seek_h262_dump_quantiser
	    (video->intra_quantiser_matrix[0],
	     quant_matrix_extension.intra_quantiser_matrix);
	_seek_h262_dump_quantiser
	    (video->non_intra_quantiser_matrix[0],
	     quant_matrix_extension.non_intra_quantiser_matrix);

	video->x000001b5_1_hack_2(&quant_matrix_extension);
    }
#endif				/* __ENABLE_X000001B5_1_HACK_2__ */
}


static void
subsequent_sequence_scalable_extension(struct seek_h262_video *video)
{
#if __ENABLE_X000001B5_5_HACK_0__
    if (video->x000001b5_5_hack_0) {
	video->x000001b5_5_hack_0(video);
    }
#endif				/* __ENABLE_X000001B5_5_HACK_0__ */
    if (video->size < 2) {
	video->optimistique_moi(video);
    }
    if (video->scalable_mode == spatial_scalability) {
	if (video->size < 8) {
	    video->optimistique_moi(video);
	}
	if (video->sequence_scalable_extension[0] != video->buffer[0]
	    || video->sequence_scalable_extension[1] != video->buffer[1]
	    || video->sequence_scalable_extension[2] != video->buffer[2]
	    || video->sequence_scalable_extension[3] != video->buffer[3]
	    || video->sequence_scalable_extension[4] != video->buffer[4]
	    || video->sequence_scalable_extension[5] != video->buffer[5]
	    || video->sequence_scalable_extension[6] != video->buffer[6]
	    || video->sequence_scalable_extension[7] != video->buffer[7]) {
	    _seek_h262_abort_decoding
		((struct seek_h262_whatever *) video,
		 VIDEO_CHANGED_SEQUENCE_SCALABLE_EXTENSION);
	}
	INCREMENT_INPUT(video, 8);
    } else if (video->scalable_mode == temporal_scalability) {
	if (video->size < 3) {
	    video->optimistique_moi(video);
	}
	if (video->sequence_scalable_extension[0] != video->buffer[0]
	    || video->sequence_scalable_extension[1] != video->buffer[1]
	    || video->sequence_scalable_extension[2] != video->buffer[2]) {
	    _seek_h262_abort_decoding
		((struct seek_h262_whatever *) video,
		 VIDEO_CHANGED_SEQUENCE_SCALABLE_EXTENSION);
	}
	INCREMENT_INPUT(video, 3);
    } else {
	if (video->sequence_scalable_extension[0] != video->buffer[0]
	    || video->sequence_scalable_extension[1] != video->buffer[1]) {
	    _seek_h262_abort_decoding
		((struct seek_h262_whatever *) video,
		 VIDEO_CHANGED_SEQUENCE_SCALABLE_EXTENSION);
	}
	INCREMENT_INPUT(video, 2);
    }
    video->sequence_scalable_extension_seen |= 1;
    video->next_start_code(video);
}


void
_seek_h262_artificial_extension_or_user_data(struct seek_h262_video *video)
{
    unsigned size;
    unsigned char *buffer;

    buffer = video->buffer;
    size = video->size;
    while (1) {
	if (size < 3) {
	    video->buffer = buffer;
	    video->size = size;
	    video->optimistique_moi(video);
	    buffer = video->buffer;
	    size = video->size;
	}
	if (!*buffer && !buffer[1] && buffer[2] == 0x01) {
	    break;
	}
	buffer++;
	size--;
    }
    video->buffer = buffer;
    video->size = size;
    if (size < 4) {
	video->optimistique_moi(video);
    }
    video->start_code = *(unsigned *) video->buffer;
    INCREMENT_INPUT(video, 4);
}


void
seek_h262_next_x00000100(struct seek_h262_video *video)
{
    video->minimum_useful_size = 64;
    video->next_start_code(video);
    sequence_header(video);
    video->expect_field = 0;
    if (video->start_code == extension_start_code) {
	video->scalable_mode = 0;
	sequence_extension(video);
	extension_and_user_data(video, 0);
	video->sequence_display_extension_seen <<= 1;
	video->sequence_scalable_extension_seen <<= 1;
	if (video->start_code == group_start_code) {
	    group_of_pictures_header(video);
	    extension_and_user_data(video, 1);
	}
	if (video->start_code != picture_start_code) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				      VIDEO_IMPROPER_START_CODE);
	}
    } else {
	_seek_h262_next_11172_x00000100(video);
    }
}


void
seek_h262_next_13818_x00000100(struct seek_h262_video *video)
{
    do {
	picture_header(video);
	picture_coding_extension(video);
	extension_and_user_data(video, 2);
	picture_data(video);
	if (video->start_code == sequence_end_code) {
	    if (video->expect_field) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_UNEXPECTED_SEQUENCE_END_CODE);
	    }
	} else if (video->start_code != picture_start_code) {
	    if (video->start_code != group_start_code) {
		seek_h262_subsequent_sequence_header(video);
		subsequent_sequence_extension(video);
		extension_and_user_data(video, 0);
		if (video->sequence_display_extension_seen
		    && video->sequence_display_extension_seen < 3) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_MISSING_SEQUENCE_DISPLAY_EXTENSION);
		}
		if (video->sequence_scalable_extension_seen
		    && video->sequence_scalable_extension_seen < 3) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_MISSING_SEQUENCE_SCALABLE_EXTENSION);
		}
	    }
	    if (video->start_code == group_start_code) {
		group_of_pictures_header(video);
		extension_and_user_data(video, 1);
	    }
	    if (video->start_code != picture_start_code) {
		_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
					  VIDEO_IMPROPER_START_CODE);
	    }
	}
    } while (video->expect_field);
}


void
seek_h262_quant_matrix_extension(struct seek_h262_video *video)
{
    /*
     * extension_start_code_identifier                               4  uimsbf
     * load_intra_quantiser_matrix                                   1  uimsbf
     * if (load_intra_quantiser_matrix) {                               
     *     intra_quantiser_matrix[64]                           8 * 64  uimsbf
     * }                                                                
     * load_non_intra_quantiser_matrix                               1  uimsbf
     * if (load_non_intra_quantiser_matrix) {                           
     *     non_intra_quantiser_matrix[64]                       8 * 64  uimsbf
     * }                                                                
     * load_chroma_intra_quantiser_matrix                            1  uimsbf
     * if (load_chroma_intra_quantiser_matrix) {                        
     *     chroma_intra_quantiser_matrix[64]                    8 * 64  uimsbf
     * }                                                                
     * load_chroma_non_intra_quantiser_matrix                        1  uimsbf
     * if (load_chroma_non_intra_quantiser_matrix) {                    
     *     chroma_non_intra_quantiser_matrix[64]                8 * 64  uimsbf
     * }                                                                
     * next_start_code()                                                
     */
#if __ENABLE_X000001B5_3_HACK_1__
    unsigned load__________________quantiser_matrix = 0;
#endif				/* __ENABLE_X000001B5_3_HACK_1__ */

#if __ENABLE_X000001B5_3_HACK_0__
    if (video->x000001b5_3_hack_0) {
	video->x000001b5_3_hack_0(video);
    }
#endif				/* __ENABLE_X000001B5_3_HACK_0__ */
    if (!video->size) {
	video->optimistique_moi(video);
    }
    {
	int do_chroma = 0, do_chroma_non = 0;
	unsigned char *buffer;

	INCREMENT_INPUT(video, 1);
	buffer = video->buffer;
#if __ENABLE_X000001B5_3_HACK_1__
	load__________________quantiser_matrix |= buffer[-1] & 0x08;
#endif				/* __ENABLE_X000001B5_3_HACK_1__ */
	if (buffer[-1] & 0x08) {
	    unsigned char save_buffer, quantiser_matrix[64];

	    save_buffer = buffer[-1];
	    if (video->size < 64) {
		video->optimistique_moi(video);
		buffer = video->buffer;
	    }
	    copy_sixteen((unsigned *) quantiser_matrix, (unsigned *) buffer);
	    {
		unsigned i = 63;
		unsigned char *buffer;

		buffer = quantiser_matrix + 63;
		for (; i; i--) {
		    *buffer = (*buffer >> 3) | (buffer[-1] << 5);
		    if (!*buffer--) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) video,
			     VIDEO_NULL_INTRA_QUANTISER);
		    }
		}
		*buffer = (*buffer >> 3) | (save_buffer << 5);
		if (*buffer != 8) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_INVALID_INTRA_QUANTISER);
		}
	    }
	    _seek_h262_set_quantiser
		(quantiser_matrix, video->intra_quantiser_matrix[0]);
	    INCREMENT_INPUT(video, 64);
	    buffer += 64;
	    do_chroma = 1;
	}
#if __ENABLE_X000001B5_3_HACK_1__
	load__________________quantiser_matrix |= buffer[-1] & 0x04;
#endif				/* __ENABLE_X000001B5_3_HACK_1__ */
	if (buffer[-1] & 0x04) {
	    unsigned char save_buffer, quantiser_matrix[64];

	    save_buffer = buffer[-1];
	    if (video->size < 64) {
		video->optimistique_moi(video);
		buffer = video->buffer;
	    }
	    copy_sixteen((unsigned *) quantiser_matrix, (unsigned *) buffer);
	    {
		unsigned i = 63;
		unsigned char *buffer;

		buffer = quantiser_matrix + 63;
		for (; i; i--) {
		    *buffer = (*buffer >> 2) | (buffer[-1] << 6);
		    if (!*buffer--) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) video,
			     VIDEO_NULL_NON_INTRA_QUANTISER);
		    }
		}
		*buffer = (*buffer >> 2) | (save_buffer << 6);
		if (!*buffer) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_NULL_NON_INTRA_QUANTISER);
		}
	    }
	    _seek_h262_set_quantiser
		(quantiser_matrix, video->non_intra_quantiser_matrix[0]);
	    INCREMENT_INPUT(video, 64);
	    buffer += 64;
	    do_chroma_non = 1;
	}
#if __ENABLE_X000001B5_3_HACK_1__
	load__________________quantiser_matrix |= buffer[-1] & 0x02;
#endif				/* __ENABLE_X000001B5_3_HACK_1__ */
	if (buffer[-1] & 0x02) {
	    unsigned char save_buffer, quantiser_matrix[64];

	    save_buffer = buffer[-1];
	    if (video->size < 64) {
		video->optimistique_moi(video);
		buffer = video->buffer;
	    }
	    copy_sixteen((unsigned *) quantiser_matrix, (unsigned *) buffer);
	    {
		unsigned i = 63;
		unsigned char *buffer;

		buffer = quantiser_matrix + 63;
		for (; i; i--) {
		    *buffer = (*buffer >> 1) | (buffer[-1] << 7);
		    if (!*buffer--) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) video,
			     VIDEO_NULL_CHROMA_INTRA_QUANTISER);
		    }
		}
		*buffer = (*buffer >> 1) | (save_buffer << 7);
		if (*buffer != 8) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_INVALID_CHROMA_INTRA_QUANTISER);
		}
	    }
	    _seek_h262_set_quantiser
		(quantiser_matrix,
		 video->chroma_intra_quantiser_matrix[0]);
	    INCREMENT_INPUT(video, 64);
	    buffer += 64;
	} else if (do_chroma) {
	    _seek_h262_copy_quantiser
		(video->chroma_intra_quantiser_matrix[0],
		 video->intra_quantiser_matrix[0]);
	}
#if __ENABLE_X000001B5_3_HACK_1__
	load__________________quantiser_matrix |= buffer[-1] & 0x01;
#endif				/* __ENABLE_X000001B5_3_HACK_1__ */
	if (buffer[-1] & 0x01) {
	    unsigned char quantiser_matrix[64];

	    if (video->size < 64) {
		video->optimistique_moi(video);
		buffer = video->buffer;
	    }
	    copy_sixteen((unsigned *) quantiser_matrix, (unsigned *) buffer);
	    {
		unsigned i = 64;
		unsigned char *buffer;

		buffer = quantiser_matrix;
		for (; i; i--) {
		    if (!*buffer++) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) video,
			     VIDEO_NULL_CHROMA_NON_INTRA_QUANTISER);
		    }
		}
	    }
	    _seek_h262_set_quantiser
		(quantiser_matrix,
		 video->chroma_non_intra_quantiser_matrix[0]);
	    INCREMENT_INPUT(video, 64);
	    buffer += 64;
	} else if (do_chroma_non) {
	    _seek_h262_copy_quantiser
		(video->chroma_non_intra_quantiser_matrix[0],
		 video->non_intra_quantiser_matrix[0]);
	}
	video->next_start_code(video);
    }
#if __ENABLE_X000001B5_3_HACK_1__
    if (video->x000001b5_3_hack_1) {
	struct seek_h262_x000001b5_3 quant_matrix_extension;

	quant_matrix_extension.decoder = video;
	if (video->chroma_format != H2V2) {
	    _seek_h262_dump_quantiser
		(video->chroma_intra_quantiser_matrix[0],
		 quant_matrix_extension.chroma_intra_quantiser_matrix);
	    _seek_h262_dump_quantiser
		(video->chroma_non_intra_quantiser_matrix[0],
		 quant_matrix_extension.chroma_non_intra_quantiser_matrix);
	}
# if WORDS_BIGENDIAN
	quant_matrix_extension.load_chroma_intra_quantiser_matrix =
	    load__________________quantiser_matrix & 0x00000002;
	quant_matrix_extension.load_chroma_non_intra_quantiser_matrix =
	    load__________________quantiser_matrix & 0x00000001;
	quant_matrix_extension.load_intra_quantiser_matrix =
	    load__________________quantiser_matrix & 0x00000008;
	quant_matrix_extension.load_non_intra_quantiser_matrix =
	    load__________________quantiser_matrix & 0x00000004;
# else
	load__________________quantiser_matrix <<= 030;
	quant_matrix_extension.load_chroma_intra_quantiser_matrix =
	    load__________________quantiser_matrix & 0x02000000;
	quant_matrix_extension.load_chroma_non_intra_quantiser_matrix =
	    load__________________quantiser_matrix & 0x01000000;
	quant_matrix_extension.load_intra_quantiser_matrix =
	    load__________________quantiser_matrix & 0x08000000;
	quant_matrix_extension.load_non_intra_quantiser_matrix =
	    load__________________quantiser_matrix & 0x04000000;
# endif				/* WORDS_BIGENDIAN */

	_seek_h262_dump_quantiser
	    (video->intra_quantiser_matrix[0],
	     quant_matrix_extension.intra_quantiser_matrix);
	_seek_h262_dump_quantiser
	    (video->non_intra_quantiser_matrix[0],
	     quant_matrix_extension.non_intra_quantiser_matrix);

	video->x000001b5_3_hack_1(&quant_matrix_extension);
    }
#endif				/* __ENABLE_X000001B5_3_HACK_1__ */
}


void
seek_h262_subsequent_sequence_header(struct seek_h262_video *video)
{
#if __ENABLE_X000001B3___HACK_0__
    if (video->x000001b3___hack_0) {
	video->x000001b3___hack_0(video);
    }
#endif				/* __ENABLE_X000001B3___HACK_0__ */
    if (video->size < 8) {
	video->optimistique_moi(video);
    }
    if (video->start_code != sequence_header_code) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_IMPROPER_START_CODE);
#if WORDS_BIGENDIAN
    } else if ((video->sequence_header[1] & 0xfffffffc)
	       != (((unsigned *) video->buffer)[1] & 0xfffffffc)
	       || (video->sequence_header[0]
		   != ((unsigned *) video->buffer)[0])) {
#else
    } else if ((video->sequence_header[1] & 0xfcffffff)
	       != (((unsigned *) video->buffer)[1] & 0xfcffffff)
	       || (video->sequence_header[0]
		   != ((unsigned *) video->buffer)[0])) {
#endif				/* WORDS_BIGENDIAN */
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_SEQUENCE_HEADER);
	_seek_h262_changed_sequence_header(video);
    } {
	unsigned char *buffer;

	INCREMENT_INPUT(video, 8);
	buffer = video->buffer;
#if WORDS_BIGENDIAN
	video->sequence_header[1] &= 0xfffffffc;
	video->sequence_header[1] |= buffer[-1] & 0x02;
#else
	video->sequence_header[1] &= 0xfcffffff;
	video->sequence_header[1] |= (buffer[-1] & 0x02) << 030;
#endif				/* WORDS_BIGENDIAN */
	if (buffer[-1] & 0x02) {
	    unsigned char save_buffer, quantiser_matrix[64];

	    save_buffer = buffer[-1];
	    if (video->size < 64) {
		video->optimistique_moi(video);
		buffer = video->buffer;
	    }
	    copy_sixteen((unsigned *) quantiser_matrix, (unsigned *) buffer);
	    {
		unsigned i = 63;
		unsigned char *buffer;

		buffer = quantiser_matrix + 63;
		for (; i; i--) {
		    *buffer = (*buffer >> 1) | (buffer[-1] << 7);
		    if (!*buffer--) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) video,
			     VIDEO_NULL_INTRA_QUANTISER);
		    }
		}
		*buffer = (*buffer >> 1) | (save_buffer << 7);
		if (*buffer != 8) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_IMPROPER_INTRA_QUANTISER);
		}
	    }
	    _seek_h262_set_quantiser
		(quantiser_matrix, video->intra_quantiser_matrix[0]);
	    INCREMENT_INPUT(video, 64);
	    buffer += 64;
	} else {
	    _seek_h262_set_quantiser
		(default_intra_quantiser_matrix,
		 video->intra_quantiser_matrix[0]);
	}
#if WORDS_BIGENDIAN
	video->sequence_header[1] |= buffer[-1] & 0x01;
#else
	video->sequence_header[1] |= (buffer[-1] & 0x01) << 030;
#endif				/* WORDS_BIGENDIAN */
	if (buffer[-1] & 0x01) {
	    unsigned char quantiser_matrix[64];

	    if (video->size < 64) {
		video->optimistique_moi(video);
		buffer = video->buffer;
	    }
	    copy_sixteen((unsigned *) quantiser_matrix, (unsigned *) buffer);
	    {
		unsigned i = 64;
		unsigned char *buffer;

		buffer = quantiser_matrix;
		for (; i; i--) {
		    if (!*buffer++) {
			_seek_h262_abort_decoding
			    ((struct seek_h262_whatever *) video,
			     VIDEO_NULL_NON_INTRA_QUANTISER);
		    }
		}
	    }
	    _seek_h262_set_quantiser
		(quantiser_matrix, video->non_intra_quantiser_matrix[0]);
	    INCREMENT_INPUT(video, 64);
	} else {
	    _seek_h262_reset_quantiser(video->non_intra_quantiser_matrix[0]);
	}
	video->next_start_code(video);
    }
}


void
seek_h262_video_sequence(struct seek_h262_video *video)
{
    /*
     * next_start_code()                                                
     * sequence_header()                                                
     * if (next_bits() == extension_start_code) {                       
     *     sequence_extension()                                         
     *     do {                                                         
     *         extension_and_user_data(0)                               
     *         do {                                                     
     *             if (nextbits() == group_start_code) {                
     *                 group_of_pictures_header()                       
     *                 extension_and_user_data(1)                       
     *             }                                                    
     *             picture_header()                                     
     *             picture_coding_extension()                           
     *             extension_and_user_data(2)                           
     *             picture_data()                                       
     *         } while (nextbits() == picture_start_code                
     *                  || nextbits() == group_start_code)              
     *         if (next_bits() != sequence_end_code) {                  
     *             sequence_header()                                    
     *             sequence_extension()                                 
     *         }                                                        
     *     } while (nextbits() != sequence_end_code)                    
     * } else {                                                         
     *     slash* ISO/IEC 11172-2 *slash                            
     * }                                                                
     * sequence_end_code                                            32  bslbf
     */
    video->minimum_useful_size = 64;
    video->next_start_code(video);
    sequence_header(video);
    video->expect_field = 0;
    if (video->start_code == extension_start_code) {
	video->scalable_mode = 0;
	sequence_extension(video);
	extension_and_user_data(video, 0);
	video->sequence_display_extension_seen <<= 1;
	video->sequence_scalable_extension_seen <<= 1;
	do {
	    while (1) {
		if (video->start_code == group_start_code) {
		    group_of_pictures_header(video);
		    extension_and_user_data(video, 1);
		}
		if (video->start_code != picture_start_code) {
		    _seek_h262_abort_decoding
			((struct seek_h262_whatever *) video,
			 VIDEO_IMPROPER_START_CODE);
		}
		picture_header(video);
		picture_coding_extension(video);
		extension_and_user_data(video, 2);
		picture_data(video);
		if (video->start_code == picture_start_code
		    || video->start_code == group_start_code) {
		} else {
		    break;
		}
	    }
	    if (video->start_code == sequence_end_code) {
		break;
	    }
	    seek_h262_subsequent_sequence_header(video);
	    subsequent_sequence_extension(video);
	    extension_and_user_data(video, 0);
	    if (video->sequence_display_extension_seen
		&& video->sequence_display_extension_seen < 3) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_MISSING_SEQUENCE_DISPLAY_EXTENSION);
	    }
	    if (video->sequence_scalable_extension_seen
		&& video->sequence_scalable_extension_seen < 3) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_MISSING_SEQUENCE_SCALABLE_EXTENSION);
	    }
	} while (1);
	if (video->expect_field) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				      VIDEO_UNEXPECTED_SEQUENCE_END_CODE);
	} else if (video->Irulan_release.flags) {
	    video->do_release(video, &video->Irulan_release);
	}
    } else {
	_seek_h262_video_sequence_11172(video);
    }
}
