/*
 * h262.2.h
 * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __SEEK_H262_2_H__
#define __SEEK_H262_2_H__

#include <h262.0.h>

/*
 * chroma format definitions
 */
#define H1V1				014
#define H2V1				010
#define H2V2				006

/*
 * coefficient definitions
 */
#define End_of_Block			0xfe
#define Escape				0xff
#define x2_End_of_Block_End_of_Block	0x0002fefe

/*
 * extension start code identifier definitions
 */
#define Picture_Coding_Extension_ID	0x80
#define Picture_Display_Extension_ID	0x70
#define Picture_Spatial_Scalable_Extension_ID \
					0x90
#define Picture_Temporal_Scalable_Extension_ID \
					0xa0
#define Quant_Matrix_Extension_ID	0x30
#define Sequence_Display_Extension_ID	0x20
#define Sequence_Extension_ID		0x10
#define Sequence_Scalable_Extension_ID	0x50

/*
 * f code definitions
 */
#if WORDS_BIGENDIAN
# define f_code_0__0_SHIFT		030
# define f_code_0__1_SHIFT		024
# define f_code_1__0_SHIFT		020
# define f_code_1__1_SHIFT		014
#else
# define f_code_0__0_SHIFT		000
# define f_code_0__1_SHIFT		014
# define f_code_1__0_SHIFT		010
# define f_code_1__1_SHIFT		024
#endif				/* WORDS_BIGENDIAN */

/*
 * picture coding type definitions
 */
#define bidirectionally_predictive_coded \
					0x03
#define dc_intra_coded			0x04
#define intra_coded			0x01
#define predictive_coded		0x02

/*
 * picture structure definitions
 */
#define Bottom_Field			0x02
#define Frame_picture			0x03
#define Top_Field			0x01

/*
 * picture substructure definitions
 */
#define Frame_picture_mask		(1L << 0x06)
#define Top_Field_mask			(1L << 0x03)
#define progressive_frame_mask		(1L << 0x05)
#define repeat_first_field_mask		(1L << 0x04)
#define top_field_first_mask		(1L << 0x03)

/*
 * prediction type definitions
 */
#define Field_based_prediction		1
#define Frame_based_prediction		2
#define x_16x8_MC			2
#define Dual_Prime			3

/*
 * scalable mode definitions
 */
#define SNR_scalability			0x08
#define data_partitioning		0x00
#define spatial_scalability		0x04
#define temporal_scalability		0x0c

/*
 * start code definitions
 */
#if WORDS_BIGENDIAN
# define MPEG_program_end_code		0x000001b9
# define extension_start_code		0x000001b5
# define group_start_code		0x000001b8
# define iso_11172_end_code		0x000001b9
# define pack_start_code		0x000001ba
# if 0
#  define packet_start_code_prefix	0x000001
# endif				/* 0 */
# define padding_stream			0x000001be
# define picture_start_code		0x00000100
# if 0
#  define private_stream_1		0x000001bd
# endif				/* 0 */
# define private_stream_2		0x000001bf
# define program_stream_directory	0x000001ff
# define program_stream_map		0x000001bc
# define sequence_end_code		0x000001b7
# define sequence_header_code		0x000001b3
# define system_header_start_code	0x000001bb
# define user_data_start_code		0x000001b2
#else
# define MPEG_program_end_code		0xb9010000
# define extension_start_code		0xb5010000
# define group_start_code		0xb8010000
# define iso_11172_end_code		0xb9010000
# define pack_start_code		0xba010000
# if 0
#  define packet_start_code_prefix	0x010000
# endif				/* 0 */
# define padding_stream			0xbe010000
# define picture_start_code		0x00010000
# if 0
#  define private_stream_1		0xbd010000
# endif				/* 0 */
# define private_stream_2		0xbf010000
# define program_stream_directory	0xff010000
# define program_stream_map		0xbc010000
# define sequence_end_code		0xb7010000
# define sequence_header_code		0xb3010000
# define system_header_start_code	0xbb010000
# define user_data_start_code		0xb2010000
#endif				/* WORDS_BIGENDIAN */

/*
 * yet more definitions...
 */
#define macroblock_intra_mask		(1L << 0x03)
#define macroblock_motion_backward_mask (1L << 0x05)
#define macroblock_motion_forward_mask	(1L << 0x06)
#define macroblock_pattern_mask		(1L << 0x04)
#define macroblock_quant_mask		(1L << 0x07)
#if 0
# define permitted_spatial_temporal_weight_classes_mask \
					(3L << 0x00)
#endif				/* 0 */
#define spatial_temporal_weight_code_flag_mask \
					(1L << 0x02)

#endif				/* __SEEK_H262_2_H__ */
