/*
 * h262.4.c
 * Copyright (C) 2003, 2004, 2005, 2006, 2007  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.1.h>
#include <h262.2.h>
#include <h262.3.h>
#include <h262.3.l7.h>
#include <h262.4.d4.h>
#include <h262.5.d4.h>
#include <h262.6.d4.h>
#include <h262.a.h>
#include <h262.e.l7.h>

#define Frame_based_prediction_or_16x8_MC \
					2

#define __RUN_FUTILE__			0

#define ABORT_DECODING(video, buffer, size, error) \
    {									      \
	_seek_h262_abort_divergent_decoding				      \
	    ((struct seek_h262_whatever *) video, buffer, size, error);	      \
    }

#if __ENABLE_X000001AF___HACK_0__
# define MAMAN_A_TORT(video) \
    (picture_start_code < video->start_code				      \
     && video->start_code < video->break_code)
#elif WORDS_BIGENDIAN
# define MAMAN_A_TORT(video) \
    (picture_start_code < video->start_code && video->start_code < 0x000001b0)
#else
# define MAMAN_A_TORT(video) \
    (picture_start_code < video->start_code && video->start_code < 0xb0010000)
#endif				/* __ENABLE_X000001AF___HACK_0__ */

extern unsigned char _seek_h262_B_1_01[], _seek_h262_B_9_01[],
    _seek_h262_B_e_012_0[], _seek_h262_B_e_012_1[], _seek_h262_B_f_012_0[],
    _seek_h262_B_f_012_1[];
extern const unsigned char _seek_h262_B_c_0[], _seek_h262_B_c_1[],
    _seek_h262_B_d_0[], _seek_h262_B_d_1[];

#if !__INLINE_OPTIMISTIQUE_MOI__
static void optimistique_moi(struct seek_h262_video *, unsigned char **,
			     unsigned *);
#endif				/* !__INLINE_OPTIMISTIQUE_MOI__ */

static const unsigned char B_b_0[] = "\0\0\1\377", B_b_1[] = "\1\1\2\2";
static const unsigned char x7_19[] = {
/* *INDENT-OFF* */
    0x01, 0x01, 0x01, 0x01, 0x03, 0x01, 0x03, 0x01,
    0x02, 0x01, 0x02, 0x01, 0x02, 0x02, 0x03, 0x01
/* *INDENT-ON* */
};

#if !__INLINE_OPTIMISTIQUE_MOI__
static void
optimistique_moi(struct seek_h262_video *video, unsigned char **buffer,
		 unsigned *size)
{
    video->buffer = *buffer;
    video->size = *size;
    video->optimistique_moi(video);
    *buffer = video->buffer;
    *size = video->size;
}
#endif				/* !__INLINE_OPTIMISTIQUE_MOI__ */


void
_seek_h262_GRAYSCALE_x___13818_slices(struct seek_h262_video *video)
{
    __13818_MACROBLOCK_decls;

    __MACROBLOCK_setup();

    video->macroblock = &macroblock;
    _seek_h262_set_GRAYSCALE_13818_blocks(video);

    do {
	unsigned cache_buffer, cache_size, dc_dct_pred, quantiser_scale = 0,
	    size;
	unsigned char *buffer;

#if WORDS_BIGENDIAN
	macroblock.mb_row = (video->start_code & 0xff) - 1;
#else
	macroblock.mb_row = (video->start_code >> 24) - 1;
#endif				/* WORDS_BIGENDIAN */
	buffer = video->buffer;
	size = video->size;
	OPTIMISTIQUE_MOI_X(4);
	cache_buffer =
	    *buffer << 24 | buffer[1] << 16 | buffer[2] << 8 | buffer[3];
	buffer += 4;
	cache_size = 32;
	size -= 4;
	if (2800 < video->vertical_size) {
	    if (127 < macroblock.mb_row) {
		ABORT_DECODING(video, buffer, size, VIDEO_SLICE_FAULT);
	    }
	    cache_size = 29;
	    macroblock.mb_row |= (cache_buffer >> 22) & ~127;
	}
	if (video->picture_structure == Frame_picture) {
	    if (video->mb_height < macroblock.mb_row + 1) {
		ABORT_DECODING
		    (video, buffer, size, VIDEO_INVALID_SLICE_START_CODE);
	    }
	} else {
	    if (video->mb_height < (macroblock.mb_row << 1) + 1) {
		ABORT_DECODING
		    (video, buffer, size, VIDEO_INVALID_SLICE_START_CODE);
	    }
	}

	_seek_h262_set_13818_slice_blocks(video);

	{
	    if (video->sequence_scalable_extension_seen) {
		if (video->scalable_mode == data_partitioning) {
		    cache_size -= 7;
		}
	    }
	    cache_size -= 5;
	    quantiser_scale = cache_buffer >> cache_size & 31;
	    if (!quantiser_scale) {
		ABORT_DECODING
		    (video, buffer, size, VIDEO_NULL_SLICE_QUANTISER_SCALE);
	    }
	    quantiser_scale = video->q_scale[quantiser_scale];
	    if (cache_buffer >> (cache_size - 1) & 1) {
		do {
		    cache_size -= 9;
		    if (cache_size < 9) {
			OPTIMISTIQUE_MOI_X(2);
			cache_buffer =
			    cache_buffer << 16 | *buffer << 8 | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
#if __ENABLE_X000001AF___HACK_3__
		    if (video->x000001af___hack_3) {
			struct seek_h262_x000001af__ slice;

			slice.decoder = video;
			slice.x.x2.extra_information_slice =
			    cache_buffer >> cache_size & 255;

			video->x000001af___hack_3(&slice);
		    }
#endif				/* __ENABLE_X000001AF___HACK_0__ */
		} while (cache_buffer >> (cache_size - 1) & 1);
	    }
	    cache_size--;
	}
	dc_dct_pred = video->intra_dc_default;
	macroblock.previous_macroblock_address =
	    macroblock.mb_row * video->mb_width - 1;
	macroblock.PMV_0_0_0 = 0;
	macroblock.PMV_0_0_1 = 0;
	macroblock.PMV_0_1_0 = 0;
	macroblock.PMV_0_1_1 = 0;
	macroblock.PMV_1_0_0 = 0;
	macroblock.PMV_1_0_1 = 0;
	macroblock.PMV_1_1_0 = 0;
	macroblock.PMV_1_1_1 = 0;
	if (cache_size < 9) {
	    OPTIMISTIQUE_MOI_X(3);
	    cache_buffer = cache_buffer << 24
		| *buffer << 16 | buffer[1] << 8 | buffer[2];
	    buffer += 3;
	    cache_size += 24;
	    size -= 3;
	} else if (cache_size < 17) {
	    OPTIMISTIQUE_MOI_X(2);
	    cache_buffer = cache_buffer << 16 | *buffer << 8 | buffer[1];
	    buffer += 2;
	    cache_size += 16;
	    size -= 2;
	} else if (cache_size < 25) {
	    OPTIMISTIQUE_MOI_1(0);
	    cache_buffer = cache_buffer << 8 | *buffer++;
	    cache_size += 8;
	    size--;
	}
	if (cache_buffer >> (cache_size - 1) & 1) {
	    macroblock.macroblock_address_increment = 1;
	    cache_size--;
	    macroblock.macroblock_address =
		macroblock.previous_macroblock_address + 1;
	    macroblock.mb_column = 0;
	} else {
	    unsigned offset;

	    offset = cache_buffer >> (cache_size - 11) & 2047;
	    if (offset != 8) {
		offset = ((unsigned short *) _seek_h262_B_1_01)[offset];
		cache_size -= offset >> 8;
		macroblock.macroblock_address_increment = offset & 255;
	    } else {
		macroblock.macroblock_address_increment = 33;
		cache_size -= 11;
		do {
		    offset = cache_buffer >> (cache_size - 11) & 2047;
		    if (offset == 8) {
			macroblock.macroblock_address_increment += 33;
			cache_size -= 11;
		    } else {
			offset =
			    ((unsigned short *) _seek_h262_B_1_01)[offset];
			cache_size -= offset >> 8;
			macroblock.macroblock_address_increment +=
			    offset & 255;
		    }
		    if (cache_size < 11) {
			OPTIMISTIQUE_MOI_X(2);
			cache_buffer =
			    cache_buffer << 16 | *buffer << 8 | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
		} while (offset == 8);
	    }
	    macroblock.macroblock_address =
		macroblock.previous_macroblock_address
		+ macroblock.macroblock_address_increment;
	    macroblock.mb_column =
		macroblock.macroblock_address % video->mb_width;
	    if (1) {
		macroblock.macroblock_address_increment = 1;
	    }
	}
	do {
	    unsigned dct_type = 0;

	    OPTIMISTIQUE_MOI_c(13818);

	    macroblock.packed_flags =
		((unsigned short *) video->B_2345678_01)
		[cache_buffer >> (cache_size - 9) & 511];
	    cache_size -= macroblock.packed_flags >> 8;
	    macroblock.packed_flags &= 255;
	    if (macroblock.packed_flags
		& spatial_temporal_weight_code_flag_mask
		&& video->spatial_temporal_weight_code_table_index) {
		if (cache_size < 2) {
		    cache_buffer = cache_buffer << 8 | *buffer++;
		    cache_size += 8;
		    size--;
		}
		cache_size -= 2;
		macroblock.spatial_temporal_weight_code =
		    cache_buffer >> cache_size & 3;
	    } else {
		macroblock.spatial_temporal_weight_code = 0;
	    }
	    if (cache_size < 17) {
		cache_buffer = cache_buffer << 16 | *buffer << 8 | buffer[1];
		buffer += 2;
		cache_size += 16;
		size -= 2;
	    }
	    if (macroblock.packed_flags & macroblock_intra_mask) {
		seek_h262_coefficient *coefficients;
		seek_h262_scan_index *inverse_scan;
		unsigned chroma_format;
		unsigned char *B_ef_012_0, *B_ef_012_1;

		if (video->decode_dct_type) {
		    cache_size--;
		    dct_type = cache_buffer >> cache_size & 1;
		}
		if (macroblock.packed_flags & macroblock_quant_mask) {
		    cache_size -= 5;
		    quantiser_scale = cache_buffer >> cache_size & 31;
		    if (!quantiser_scale) {
			ABORT_DECODING
			    (video, buffer, size,
			     VIDEO_NULL_MACROBLOCK_QUANTISER_SCALE);
		    }
		    quantiser_scale = video->q_scale[quantiser_scale];
		} {
		    if (video->concealment_motion_vectors) {
			if (video->picture_structure != Frame_picture) {
			    motion_vertical_field_select(0, 0);
			}
			subsequent_motion_vector(0, 0);
			if (!cache_size) {
			    cache_buffer = *buffer << 8 | buffer[1];
			    buffer += 2;
			    cache_size = 16;
			    size -= 2;
			}
			cache_size--;
			macroblock.PMV_1_0_0 = macroblock.PMV_0_0_0;
			macroblock.PMV_1_0_1 = macroblock.PMV_0_0_1;
		    } else {
			macroblock.PMV_0_0_0 = 0;
			macroblock.PMV_0_0_1 = 0;
			macroblock.PMV_0_1_0 = 0;
			macroblock.PMV_0_1_1 = 0;
			macroblock.PMV_1_0_0 = 0;
			macroblock.PMV_1_0_1 = 0;
			macroblock.PMV_1_1_0 = 0;
			macroblock.PMV_1_1_1 = 0;
		    }
		}
		B_ef_012_0 = video->B_ef_012_0;
		B_ef_012_1 = video->B_ef_012_1;
		coefficients = macroblock.coefficients + _1ST_BLOCK_OFFSET;
		inverse_scan =
		    video->intra_quantiser_matrix[video->alternate_scan];
		chroma_format = 4;
		for (; chroma_format; chroma_format--) {
		    int sum;

		    if (cache_size < 17) {
			cache_buffer = cache_buffer << 16 | *buffer << 8
			    | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
		    {
			int dc_dct_size;

			dc_dct_size = cache_buffer >> (cache_size - 9) & 511;
			if (dc_dct_size < 448) {
			    dc_dct_size =
				((short *) _seek_h262_B_c_0)[dc_dct_size >> 2];
			    cache_size -= dc_dct_size & 255;
			    dc_dct_pred += dc_dct_size >> 8;
			} else {
			    int dct_diff;
			    unsigned mask;

			    if (cache_size < 25) {
				cache_buffer = cache_buffer << 8 | *buffer++;
				cache_size += 8;
				size--;
			    }
			    dc_dct_size =
				((unsigned short *) _seek_h262_B_c_1)
				[dc_dct_size & 63];
			    cache_size -= dc_dct_size >> 8;
			    dc_dct_size &= 255;
			    mask = (1 << dc_dct_size) - 1;
			    dct_diff = cache_buffer >> cache_size & mask;
			    if (!(dct_diff & ~(mask >> 1))) {
				dct_diff -= mask;
			    }
			    dc_dct_pred += dct_diff;
			}
			sum = dc_dct_pred << video->intra_dc_precision;
		    }
		    if (sum < 0) {
			sum = 0;
		    } else if (2047 < sum) {
			sum = 2047;
		    }
		    *coefficients = sum;
		    DECODE_13818_B__BLOCK(B_ef_012_0, B_ef_012_1);
		    coefficients += BLOCK_SIZE;
		}
		chroma_format = video->chroma_format - 4;
		for (; chroma_format; chroma_format--) {
		    if (cache_size < 17) {
			cache_buffer = cache_buffer << 16 | *buffer << 8
			    | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
		    {
			int dc_dct_size;

			dc_dct_size = cache_buffer >> (cache_size - 10) & 1023;
			if (dc_dct_size < 960) {
			    dc_dct_size =
				((short *) _seek_h262_B_d_0)[dc_dct_size >> 2];
			    cache_size -= dc_dct_size & 255;
			} else {
			    if (cache_size < 25) {
				cache_buffer = cache_buffer << 8 | *buffer++;
				cache_size += 8;
				size--;
			    }
			    dc_dct_size =
				((unsigned short *) _seek_h262_B_d_1)
				[dc_dct_size & 63];
			    cache_size -= dc_dct_size >> 8;
			}
		    }
		    DETACH_13818_B__BLOCK(B_ef_012_0, B_ef_012_1);
		}
		video->macroblock = &macroblock;
#if __RUN_FUTILE__
		macroblock.previous_macroblock_address =
		    macroblock.macroblock_address;
#else
		video->______macroblock[dct_type << 2] (video);
#endif				/* __RUN_FUTILE__ */
	    } else {
		unsigned motion_type = Frame_based_prediction_or_16x8_MC;

		if (macroblock.packed_flags
		    & (macroblock_motion_backward_mask
		       | macroblock_motion_forward_mask)) {
		    if (video->decode_motion_type) {
			cache_size -= 2;
			motion_type = cache_buffer >> cache_size & 3;
			if (motion_type == video->dmv_abort) {
			    ABORT_DECODING(video, buffer, size,
					   VIDEO_INVALID_MOTION_TYPE);
			}
		    }
		    if (macroblock.packed_flags & video->decode_dct_type) {
			cache_size--;
			dct_type = cache_buffer >> cache_size & 1;
		    }
		    if (macroblock.packed_flags & macroblock_quant_mask) {
			cache_size -= 5;
			quantiser_scale = cache_buffer >> cache_size & 31;
			if (!quantiser_scale) {
			    ABORT_DECODING
				(video, buffer, size,
				 VIDEO_NULL_MACROBLOCK_QUANTISER_SCALE);
			}
			quantiser_scale = video->q_scale[quantiser_scale];
		    }
		    if (video->picture_structure == Frame_picture) {
		        switch (motion_type) {
			case Dual_Prime:
			    if (macroblock.packed_flags
				& macroblock_motion_forward_mask) {
				motion_vector_0(0, 0);
			    }
			    if (macroblock.packed_flags
				& macroblock_motion_backward_mask) {
				subsequent_motion_vector_0(0, 1);
			    }
			    macroblock.PMV_1_0_0 = macroblock.PMV_0_0_0;
			    macroblock.PMV_1_0_1 = macroblock.PMV_0_0_1;
			    break;
			case Field_based_prediction:
			    {
				unsigned motion_vector_count;

				motion_vector_count =
				    video->
				    spatial_temporal_weight_code_table_index
				    << 2
				    | macroblock.spatial_temporal_weight_code;
				motion_vector_count =
				    x7_19[motion_vector_count] < 2 ? 2 : 1;
				if (macroblock.packed_flags
				    & macroblock_motion_forward_mask) {
				    motion_vertical_field_select(0, 0);
				    motion_vector_1(0, 0);
				    if (motion_vector_count != 1) {
					motion_vertical_field_select(1, 0);
					subsequent_motion_vector_1(1, 0);
				    }
				}
				if (macroblock.packed_flags
				    & macroblock_motion_backward_mask) {
				    motion_vertical_field_select(0, 1);
				    subsequent_motion_vector_1(0, 1);
				    if (motion_vector_count != 1) {
					motion_vertical_field_select(1, 1);
					subsequent_motion_vector_1(1, 1);
				    }
				}
			    }
			    break;
			default:
			    if (macroblock.packed_flags
				& macroblock_motion_forward_mask) {
				motion_vector_2(0, 0);
				macroblock.PMV_1_0_0 = macroblock.PMV_0_0_0;
				macroblock.PMV_1_0_1 = macroblock.PMV_0_0_1;
			    }
			    if (macroblock.packed_flags
				& macroblock_motion_backward_mask) {
				subsequent_motion_vector_2(0, 1);
				macroblock.PMV_1_1_0 = macroblock.PMV_0_1_0;
				macroblock.PMV_1_1_1 = macroblock.PMV_0_1_1;
			    }
			}
		    } else {
			switch (motion_type) {
			case Dual_Prime:
			    if (macroblock.packed_flags
				& macroblock_motion_forward_mask) {
				motion_vector_3(0, 0);
			    }
			    if (macroblock.packed_flags
				& macroblock_motion_backward_mask) {
				subsequent_motion_vector_3(0, 1);
			    }
			    macroblock.PMV_1_0_0 = macroblock.PMV_0_0_0;
			    macroblock.PMV_1_0_1 = macroblock.PMV_0_0_1;
			    break;
			case Field_based_prediction:
			    if (macroblock.packed_flags
				& macroblock_motion_forward_mask) {
				motion_vertical_field_select(0, 0);
				motion_vector_4(0, 0);
				macroblock.PMV_1_0_0 = macroblock.PMV_0_0_0;
				macroblock.PMV_1_0_1 = macroblock.PMV_0_0_1;
			    }
			    if (macroblock.packed_flags
				& macroblock_motion_backward_mask) {
				motion_vertical_field_select(0, 1);
				subsequent_motion_vector_4(0, 1);
				macroblock.PMV_1_1_0 = macroblock.PMV_0_1_0;
				macroblock.PMV_1_1_1 = macroblock.PMV_0_1_1;
			    }
			    break;
			default:
			    if (macroblock.packed_flags
				& macroblock_motion_forward_mask) {
				motion_vertical_field_select(0, 0);
				motion_vector_5(0, 0);
				motion_vertical_field_select(1, 0);
				subsequent_motion_vector_5(1, 0);
			    }
			    if (macroblock.packed_flags
				& macroblock_motion_backward_mask) {
				motion_vertical_field_select(0, 1);
				subsequent_motion_vector_5(0, 1);
				motion_vertical_field_select(1, 1);
				subsequent_motion_vector_5(1, 1);
			    }
			}
		    }
		} else {
		    if (macroblock.packed_flags & video->decode_dct_type) {
			cache_size--;
			dct_type = cache_buffer >> cache_size & 1;
		    }
		    if (macroblock.packed_flags & macroblock_quant_mask) {
			cache_size -= 5;
			quantiser_scale = cache_buffer >> cache_size & 31;
			if (!quantiser_scale) {
			    ABORT_DECODING
				(video, buffer, size,
				 VIDEO_NULL_MACROBLOCK_QUANTISER_SCALE);
			}
			quantiser_scale = video->q_scale[quantiser_scale];
		    }
		    if (video->picture_coding_type == predictive_coded) {
			if (video->deinterlaced_release_coding_type
			    == intra_coded) {
			    ABORT_DECODING
				(video, buffer, size, VIDEO_SLICE_FAULT);
			}
			macroblock.PMV_0_0_0 = 0;
			macroblock.PMV_0_0_1 = 0;
			macroblock.PMV_0_1_0 = 0;
			macroblock.PMV_0_1_1 = 0;
			macroblock.PMV_1_0_0 = 0;
			macroblock.PMV_1_0_1 = 0;
			macroblock.PMV_1_1_0 = 0;
			macroblock.PMV_1_1_1 = 0;
			macroblock.motion_vertical_field_select_0_0 =
			    video->picture_structure >> 1;
			macroblock.vector_0_0_0 = 0;
			macroblock.vector_0_0_1 = 0;
			motion_type = video->predictive_coded_motion_type;
		    }
		}
		dc_dct_pred = video->intra_dc_default;
		if (macroblock.packed_flags & macroblock_pattern_mask) {
		    seek_h262_coefficient *coefficients;
		    seek_h262_scan_index *inverse_scan;
		    unsigned chroma_format, chroma_pattern_code = 0,
			pattern_code;

		    if (cache_size < 15) {
			cache_buffer =
			    cache_buffer << 16 | *buffer << 8 | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
		    pattern_code = ((unsigned short *) _seek_h262_B_9_01)
			[cache_buffer >> (cache_size - 9) & 511];
		    cache_size -= pattern_code & 15;
		    pattern_code >>= 4;
		    chroma_pattern_code = pattern_code >> 8;
		    pattern_code &= 255;
		    chroma_pattern_code |= chroma_pattern_code >> 1;
		    chroma_pattern_code &= 3;
		    chroma_format = video->chroma_format;
		    if (chroma_format == H2V2) {
		    } else if (chroma_format == H2V1) {
			cache_size -= 2;
			chroma_pattern_code = chroma_pattern_code << 2
			    | (cache_buffer >> cache_size & 3);
		    } else if (chroma_format == H1V1) {
			cache_size -= 6;
			chroma_pattern_code = chroma_pattern_code << 6
			    | (cache_buffer >> cache_size & 63);
		    }
		    inverse_scan = video->non_intra_quantiser_matrix
			[video->alternate_scan];
		    macroblock.pattern_code = pattern_code;
		    coefficients = macroblock.coefficients + _1ST_BLOCK_OFFSET;
		    chroma_format = 1;
		    for (; chroma_format < 341; chroma_format <<= 2) {
			if (pattern_code & chroma_format) {
			    DECODE_13818_Be_BLOCK();
			}
			coefficients += BLOCK_SIZE;
		    }
		    for (; chroma_pattern_code; chroma_pattern_code >>= 1) {
			if (chroma_pattern_code & 1) {
			    DETACH_13818_Be_BLOCK();
			}
		    }
		    macroblock.dc_sparse = pattern_code;
		    macroblock.pattern_code &= ~macroblock.dc_sparse;
		} else {
		    macroblock.dc_sparse = 0;
		    macroblock.pattern_code = 0;
		}
		video->macroblock = &macroblock;
#if __RUN_FUTILE__
		macroblock.previous_macroblock_address =
		macroblock.macroblock_address;
#else
		video->______macroblock[dct_type << 2 | motion_type](video);
#endif				/* __RUN_FUTILE__ */
	    }
	    if (cache_size < 23) {
		if (cache_size < 7) {
		    cache_buffer = cache_buffer << 24
			| *buffer << 16 | buffer[1] << 8 | buffer[2];
		    buffer += 3;
		    cache_size += 24;
		    size -= 3;
		} else if (cache_size < 15) {
		    cache_buffer =
			cache_buffer << 16 | *buffer << 8 | buffer[1];
		    buffer += 2;
		    cache_size += 16;
		    size -= 2;
		} else {
		    cache_buffer = cache_buffer << 8 | *buffer++;
		    cache_size += 8;
		    size--;
		}
	    }
	    {
		unsigned offset;

		offset = cache_buffer >> (cache_size - 23) & 8388607;
		if (!offset) {
		    break;
		}

		if (offset & 4194304) {
		    macroblock.macroblock_address_increment = 1;
		    cache_size--;
		    macroblock.macroblock_address =
			macroblock.previous_macroblock_address + 1;
		    macroblock.mb_column++;
		    if (macroblock.macroblock_address == video->mb_width) {
			macroblock.mb_column = 0;
		    }
		} else {
		    offset >>= 12;
		    if (offset != 8) {
			offset =
			    ((unsigned short *) _seek_h262_B_1_01)[offset];
			cache_size -= offset >> 8;
			macroblock.macroblock_address_increment = offset & 255;
		    } else {
			macroblock.macroblock_address_increment = 33;
			cache_size -= 11;
			do {
			    offset = cache_buffer >> (cache_size - 11) & 2047;
			    if (offset == 8) {
				macroblock.macroblock_address_increment += 33;
				cache_size -= 11;
			    } else {
				offset = ((unsigned short *) _seek_h262_B_1_01)
				    [offset];
				cache_size -= offset >> 8;
				macroblock.macroblock_address_increment +=
				    offset & 255;
			    }
			    if (cache_size < 11) {
				OPTIMISTIQUE_MOI_X(2);
				cache_buffer = cache_buffer << 16
				    | *buffer << 8 | buffer[1];
				buffer += 2;
				cache_size += 16;
				size -= 2;
			    }
			} while (offset == 8);
		    }
		    macroblock.macroblock_address =
			macroblock.previous_macroblock_address
			+ macroblock.macroblock_address_increment;
		    macroblock.mb_column =
			macroblock.macroblock_address % video->mb_width;
		    if (video->picture_coding_type == intra_coded) {
			if (video->scalable_mode == SNR_scalability
			    || video->picture_spatial_scalable_extension_seen) {
			} else {
			    ABORT_DECODING
				(video, buffer, size,
				 VIDEO_INVALID_MACROBLOCK_ADDRESS_INCREMENT);
			}
		    } else if (video->picture_coding_type == predictive_coded) {
			if (video->deinterlaced_release_coding_type
			    == intra_coded) {
			    ABORT_DECODING
				(video, buffer, size,
				 VIDEO_INVALID_MACROBLOCK_ADDRESS_INCREMENT);
			}
			macroblock.PMV_0_0_0 = 0;
			macroblock.PMV_0_0_1 = 0;
			macroblock.PMV_0_1_0 = 0;
			macroblock.PMV_0_1_1 = 0;
			macroblock.PMV_1_0_0 = 0;
			macroblock.PMV_1_0_1 = 0;
			macroblock.PMV_1_1_0 = 0;
			macroblock.PMV_1_1_1 = 0;
		    }
#if __RUN_FUTILE__
#else
		    video->______skipped_macroblocks_x__(video);
#endif				/* __RUN_FUTILE__ */
		    dc_dct_pred = video->intra_dc_default;
		}
	    }
	} while (1);
	cache_size &= ~7;
	cache_buffer &= (1 << cache_size) - 1;
	video->buffer = buffer;
	video->size = size;
	if (cache_buffer) {
	    if (cache_size != 24 || cache_buffer != 1) {
		ABORT_DECODING(video, buffer, size, VIDEO_SLICE_FAULT);
	    }
	    if (!size) {
		video->optimistique_moi(video);
	    }
#if WORDS_BIGENDIAN
	    video->start_code = 0x00000100 | *video->buffer;
#else
	    video->start_code = 0x00010000 | *video->buffer << 24;
#endif				/* WORDS_BIGENDIAN */
	    video->buffer++;
	    video->size--;
	} else if (cache_size) {
	    video->start_code = cache_size >> 3;
	    video->neXt_start_code(video);
	} else {
	    video->next_start_code(video);
	}
    } while (MAMAN_A_TORT(video));
}
