/*
 * h262.4.part3.c
 * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.1.h>
#include <h262.2.h>
#include <h262.3.h>
#include <h262.a.h>

#define __PAIR_MISS_F_CODES_INSANITY__	0
#define __ZERO_MISS_F_CODES_INSANITY__	1

#define __HERE_MISS_F_CODES_INSANITY__	__ZERO_MISS_F_CODES_INSANITY__

extern unsigned char _seek_h262_B_a_01[], _seek_h262_B_a_01_1[],
    _seek_h262_B_a_01_2[], _seek_h262_B_a_01_3[];

void
_seek_h262_alloc_storage(struct seek_h262_video *video,
			 unsigned luminance_size, unsigned chrominance_size)
{
    unsigned char *storage[9];

    video->alloc_storage(video, storage, luminance_size, chrominance_size);

    video->Iran_release.Cb = storage[0];
    video->Iran_release.Cr = storage[1];
    video->Iran_release.Y = storage[2];
    video->Irulan_release.Cb = storage[3];
    video->Irulan_release.Cr = storage[4];
    video->Irulan_release.Y = storage[5];
    video->Mylene_release.Cb = storage[6];
    video->Mylene_release.Cr = storage[7];
    video->Mylene_release.Y = storage[8];
}


void
_seek_h262_compute_11172_motion_limit(struct seek_h262_video *video)
{
    video->motion_limit[0] =
	(video->macroblocks_count << 8) - 16 - video->luminance_line_size * 15;
    video->motion_limit[1] = video->motion_limit[0] - 1;
    video->motion_limit[2] =
	video->motion_limit[0] - video->luminance_line_size;
    video->motion_limit[3] = video->motion_limit[2] - 1;
}


void
_seek_h262_compute_13818_motion_limit(struct seek_h262_video *video)
{
    video->block_1_motion_limit[0] =
	(video->macroblocks_count << 8) - 16 - video->luminance_line_size * 14;
    video->block_1_motion_limit[1] = video->block_1_motion_limit[0] - 1;
    video->block_1_motion_limit[2] =
	video->block_1_motion_limit[0] - video->luminance_line_size;
    video->block_1_motion_limit[3] = video->block_1_motion_limit[2] - 1;
    video->block_2_motion_limit[0] =
	(video->macroblocks_count << 8) - 16 - video->luminance_line_size * 15;
    video->block_2_motion_limit[1] = video->block_2_motion_limit[0] - 1;
    video->block_2_motion_limit[2] =
	video->block_2_motion_limit[0] - video->luminance_line_size;
    video->block_2_motion_limit[3] = video->block_2_motion_limit[2] - 1;
    video->block_9_motion_limit[0] =
	(video->macroblocks_count << 8) - 16 - video->luminance_line_size * 30;
    video->block_9_motion_limit[1] = video->block_9_motion_limit[0] - 1;
    video->block_9_motion_limit[2] =
	video->block_9_motion_limit[0] - video->luminance_line_size * 2;
    video->block_9_motion_limit[3] = video->block_9_motion_limit[2] - 1;
    video->block_a_motion_limit[0] =
	(video->macroblocks_count << 8) - 16 - video->luminance_line_size * 14;
    video->block_a_motion_limit[1] = video->block_a_motion_limit[0] - 1;
    video->block_a_motion_limit[2] =
	video->block_a_motion_limit[0] - video->luminance_line_size * 2;
    video->block_a_motion_limit[3] = video->block_a_motion_limit[2] - 1;
}


void
_seek_h262_detail_x08_f_codes(struct seek_h262_video *video, unsigned *e_code)
{
    unsigned f_code;

    f_code = *e_code;
    if (!(f_code & 15 << f_code_0__0_SHIFT)) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_NULL_F_CODE_0__0_);
    } else {
	if ((f_code & 15 << f_code_0__0_SHIFT) != 15 << f_code_0__0_SHIFT) {
	    _seek_h262_abort_decoding
		((struct seek_h262_whatever *) video,
		 VIDEO_INVALID_F_CODE_0__0_);
	}
    }
    if (!(f_code & 15 << f_code_0__1_SHIFT)) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_NULL_F_CODE_0__1_);
    } else {
	if ((f_code & 15 << f_code_0__1_SHIFT) != 15 << f_code_0__1_SHIFT) {
	    _seek_h262_abort_decoding
		((struct seek_h262_whatever *) video,
		 VIDEO_INVALID_F_CODE_0__1_);
	}
    }
    if (!(f_code & 15 << f_code_1__0_SHIFT)) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_NULL_F_CODE_1__0_);
    } else {
	if ((f_code & 15 << f_code_1__0_SHIFT) != 15 << f_code_1__0_SHIFT) {
	    _seek_h262_abort_decoding
		((struct seek_h262_whatever *) video,
		 VIDEO_INVALID_F_CODE_1__0_);
	}
    }
    if (!(f_code & 15 << f_code_1__1_SHIFT)) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_NULL_F_CODE_1__1_);
    } else {
	if ((f_code & 15 << f_code_1__1_SHIFT) != 15 << f_code_1__1_SHIFT) {
	    _seek_h262_abort_decoding
		((struct seek_h262_whatever *) video,
		 VIDEO_INVALID_F_CODE_1__1_);
	}
    }
}


void
_seek_h262_detail_x10_f_codes(struct seek_h262_video *video, unsigned *e_code)
{
    unsigned d_code, f_code;

    f_code = *e_code;

    d_code = f_code;

    if (!(f_code & 15 << f_code_0__0_SHIFT)) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_NULL_F_CODE_0__0_);
    } else {
	if (9 << f_code_0__0_SHIFT < (f_code & 15 << f_code_0__0_SHIFT)) {
	    if ((f_code & 15 << f_code_0__0_SHIFT)
		== 15 << f_code_0__0_SHIFT) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_MISS_F_CODE_0__0_);
#if __HERE_MISS_F_CODES_INSANITY__ == __PAIR_MISS_F_CODES_INSANITY__
		if ((f_code & 15 << f_code_0__1_SHIFT)
		    == 15 << f_code_0__1_SHIFT) {
#endif				/* __HERE_MISS_F_CODES_INSANITY__ == __PA... */
		    d_code ^= 14 << f_code_0__0_SHIFT;
#if __HERE_MISS_F_CODES_INSANITY__ == __PAIR_MISS_F_CODES_INSANITY__
		}
#endif				/* __HERE_MISS_F_CODES_INSANITY__ == __PA... */
	    } else {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_INVALID_F_CODE_0__0_);
	    }
	}
    }
    if (!(f_code & 15 << f_code_0__1_SHIFT)) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_NULL_F_CODE_0__1_);
    } else {
	if (9 << f_code_0__1_SHIFT < (f_code & 15 << f_code_0__1_SHIFT)) {
	    if ((f_code & 15 << f_code_0__1_SHIFT)
		== 15 << f_code_0__1_SHIFT) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_MISS_F_CODE_0__1_);
#if __HERE_MISS_F_CODES_INSANITY__ == __PAIR_MISS_F_CODES_INSANITY__
		if ((f_code & 15 << f_code_0__0_SHIFT)
		    == 15 << f_code_0__0_SHIFT) {
#endif				/* __HERE_MISS_F_CODES_INSANITY__ == __PA... */
		    d_code ^= 14 << f_code_0__1_SHIFT;
#if __HERE_MISS_F_CODES_INSANITY__ == __PAIR_MISS_F_CODES_INSANITY__
		}
#endif				/* __HERE_MISS_F_CODES_INSANITY__ == __PA... */
	    } else {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_INVALID_F_CODE_0__1_);
	    }
	}
    }
    if (!(f_code & 15 << f_code_1__0_SHIFT)) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_NULL_F_CODE_1__0_);
    } else {
	if ((f_code & 15 << f_code_1__0_SHIFT) != 15 << f_code_1__0_SHIFT) {
	    _seek_h262_abort_decoding
		((struct seek_h262_whatever *) video,
		 VIDEO_INVALID_F_CODE_1__0_);
	}
    }
    if (!(f_code & 15 << f_code_1__1_SHIFT)) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_NULL_F_CODE_1__1_);
    } else {
	if ((f_code & 15 << f_code_1__1_SHIFT) != 15 << f_code_1__1_SHIFT) {
	    _seek_h262_abort_decoding
		((struct seek_h262_whatever *) video,
		 VIDEO_INVALID_F_CODE_1__1_);
	}
    }

    *e_code = d_code;
}


void
_seek_h262_detail_x18_f_codes(struct seek_h262_video *video, unsigned *e_code)
{
    unsigned d_code, f_code;

    f_code = *e_code;

    d_code = f_code;

    if (!(f_code & 15 << f_code_0__0_SHIFT)) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_NULL_F_CODE_0__0_);
    } else {
	if (9 << f_code_0__0_SHIFT < (f_code & 15 << f_code_0__0_SHIFT)) {
	    if ((f_code & 15 << f_code_0__0_SHIFT)
		== 15 << f_code_0__0_SHIFT) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_MISS_F_CODE_0__0_);
#if __HERE_MISS_F_CODES_INSANITY__ == __PAIR_MISS_F_CODES_INSANITY__
		if ((f_code & 15 << f_code_0__1_SHIFT)
		    == 15 << f_code_0__1_SHIFT) {
#endif				/* __HERE_MISS_F_CODES_INSANITY__ == __PA... */
		    d_code ^= 14 << f_code_0__0_SHIFT;
#if __HERE_MISS_F_CODES_INSANITY__ == __PAIR_MISS_F_CODES_INSANITY__
		}
#endif				/* __HERE_MISS_F_CODES_INSANITY__ == __PA... */
	    } else {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_INVALID_F_CODE_0__0_);
	    }
	}
    }
    if (!(f_code & 15 << f_code_0__1_SHIFT)) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_NULL_F_CODE_0__1_);
    } else {
	if (9 << f_code_0__1_SHIFT < (f_code & 15 << f_code_0__1_SHIFT)) {
	    if ((f_code & 15 << f_code_0__1_SHIFT)
		== 15 << f_code_0__1_SHIFT) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_MISS_F_CODE_0__1_);
#if __HERE_MISS_F_CODES_INSANITY__ == __PAIR_MISS_F_CODES_INSANITY__
		if ((f_code & 15 << f_code_0__0_SHIFT)
		    == 15 << f_code_0__0_SHIFT) {
#endif				/* __HERE_MISS_F_CODES_INSANITY__ == __PA... */
		    d_code ^= 14 << f_code_0__1_SHIFT;
#if __HERE_MISS_F_CODES_INSANITY__ == __PAIR_MISS_F_CODES_INSANITY__
		}
#endif				/* __HERE_MISS_F_CODES_INSANITY__ == __PA... */
	    } else {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_INVALID_F_CODE_0__1_);
	    }
	}
    }
    if (!(f_code & 15 << f_code_1__0_SHIFT)) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_NULL_F_CODE_1__0_);
    } else {
	if (9 << f_code_1__0_SHIFT < (f_code & 15 << f_code_1__0_SHIFT)) {
	    if ((f_code & 15 << f_code_1__0_SHIFT)
		== 15 << f_code_1__0_SHIFT) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_MISS_F_CODE_1__0_);
#if __HERE_MISS_F_CODES_INSANITY__ == __PAIR_MISS_F_CODES_INSANITY__
		if ((f_code & 15 << f_code_1__1_SHIFT)
		    == 15 << f_code_1__1_SHIFT) {
#endif				/* __HERE_MISS_F_CODES_INSANITY__ == __PA... */
		    d_code ^= 14 << f_code_1__0_SHIFT;
#if __HERE_MISS_F_CODES_INSANITY__ == __PAIR_MISS_F_CODES_INSANITY__
		}
#endif				/* __HERE_MISS_F_CODES_INSANITY__ == __PA... */
	    } else {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_INVALID_F_CODE_1__0_);
	    }
	}
    }
    if (!(f_code & 15 << f_code_1__1_SHIFT)) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_NULL_F_CODE_1__1_);
    } else {
	if (9 << f_code_1__1_SHIFT < (f_code & 15 << f_code_1__1_SHIFT)) {
	    if ((f_code & 15 << f_code_1__1_SHIFT)
		== 15 << f_code_1__1_SHIFT) {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_MISS_F_CODE_1__1_);
#if __HERE_MISS_F_CODES_INSANITY__ == __PAIR_MISS_F_CODES_INSANITY__
		if ((f_code & 15 << f_code_1__0_SHIFT)
		    == 15 << f_code_1__0_SHIFT) {
#endif				/* __HERE_MISS_F_CODES_INSANITY__ == __PA... */
		    d_code ^= 14 << f_code_1__1_SHIFT;
#if __HERE_MISS_F_CODES_INSANITY__ == __PAIR_MISS_F_CODES_INSANITY__
		}
#endif				/* __HERE_MISS_F_CODES_INSANITY__ == __PA... */
	    } else {
		_seek_h262_abort_decoding
		    ((struct seek_h262_whatever *) video,
		     VIDEO_INVALID_F_CODE_1__1_);
	    }
	}
    }

    *e_code = d_code;
}


void
_seek_h262_motion_component(unsigned f_code, unsigned *f_code_limit,
			    unsigned *f_code_mask, unsigned *f_code_range,
			    unsigned char **B_a_01)
{
#define _seek_h262_B_a_01_0		_seek_h262_B_a_01
#define _seek_h262_B_a_01_x		_seek_h262_B_a_01
    switch (f_code) {
    case 0:
	*B_a_01 = _seek_h262_B_a_01_0;
	*f_code_limit = 0;
	*f_code_mask = 0;
	*f_code_range = 27;
	break;
    case 1:
	*B_a_01 = _seek_h262_B_a_01_1;
	*f_code_limit = 36;
	*f_code_mask = 1;
	*f_code_range = 26;
	break;
    case 2:
	*B_a_01 = _seek_h262_B_a_01_2;
	*f_code_limit = 48;
	*f_code_mask = 3;
	*f_code_range = 25;
	break;
    case 3:
	*B_a_01 = _seek_h262_B_a_01_3;
	*f_code_limit = 48;
	*f_code_mask = 7;
	*f_code_range = 24;
	break;
    default:
	*B_a_01 = _seek_h262_B_a_01_x;
	*f_code_limit = 1024;
	*f_code_mask = (1 << f_code) - 1;
	*f_code_range = 27 - f_code;
    }
#undef _seek_h262_B_a_01_x
#undef _seek_h262_B_a_01_0
}


void
_seek_h262_set_references(struct seek_h262_video *video)
{
    if (video->picture_structure == Frame_picture) {
    } else {
	struct seek_h262_release *release_0, *release_1;

	if (video->picture_coding_type
	    == bidirectionally_predictive_coded) {
	    release_0 = &video->Iran_release;
	    release_1 = &video->Iran_release;
	} else if (video->expect_field) {
	    release_0 = &video->Iran_release;
	    release_1 = &video->Iran_release;
	} else if (video->picture_structure == Bottom_Field) {
	    release_0 = &video->Mylene_release;
	    release_1 = &video->Iran_release;
	} else {
	    release_0 = &video->Iran_release;
	    release_1 = &video->Mylene_release;
	}
	video->field_references[0].Cb = release_0->Cb;
	video->field_references[0].Cr = release_0->Cr;
	video->field_references[0].Y = release_0->Y;
	video->field_references[1].Cb = release_1->Cb;
	video->field_references[1].Cr = release_1->Cr;
	video->field_references[1].Y = release_1->Y;
    }
}


void
seek_h262_picture_data(void *video_return)
{
    struct seek_h262_video *video;
    unsigned slice_select;

    video = (struct seek_h262_video *) video_return;
#if WORDS_BIGENDIAN
    if (video->start_code < 0x00000101 || 0x000001af < video->start_code) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_IMPROPER_START_CODE);
    }
#else
    if (video->start_code < 0x01010000 || 0xaf010000 < video->start_code) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_IMPROPER_START_CODE);
    }
#endif				/* WORDS_BIGENDIAN */
    slice_select = video->picture_coding_type;
    if (((integral_q) video->slices[0] & 65535) == 11172) {
#if WORDS_BIGENDIAN
	slice_select |= (~video->sequence_header[1] & 1) << 2;
#else
	slice_select |= (~video->sequence_header[1] & 16777216) >> 22;
#endif				/* WORDS_BIGENDIAN */
    }
#if __ENABLE_X000001AF___HACK_0__
    if (video->x000001af___hack_0) {
# if WORDS_BIGENDIAN
	do {
	    video->x000001af___hack_0(video);
	    video->slices[slice_select](video);
	} while (0x00000100 < video->start_code
		 && video->start_code < 0x000001b0);
# else
	do {
	    video->x000001af___hack_0(video);
	    video->slices[slice_select](video);
	} while (0x01000000 < video->start_code
		 && video->start_code < 0xb0010000);
# endif				/* WORDS_BIGENDIAN */
    } else {
#endif				/* __ENABLE_X000001AF___HACK_0__ */
	video->slices[slice_select](video);
#if __ENABLE_X000001AF___HACK_0__
    }
#endif				/* __ENABLE_X000001AF___HACK_0__ */
}
