/*
 * h262.5.c
 * Copyright (C) 2003, 2004, 2005  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.1.h>
#include <h262.2.h>
#include <h262.3.h>
#include <h262.5.d4.h>
#include <h262.a.h>
#include <h262.e.l7.h>

#define Frame_based_prediction_or_16x8_MC \
					2

#define ABORT_DECODING(video, buffer, size, error) \
    {									      \
	_seek_h262_abort_divergent_decoding				      \
	    ((struct seek_h262_whatever *) video, buffer, size, error);	      \
    }

#if __ENABLE_X000001AF___HACK_0__
# define MAMAN_A_TORT(video) \
    (picture_start_code < video->start_code				      \
     && video->start_code < video->break_code)
#elif WORDS_BIGENDIAN
# define MAMAN_A_TORT(video) \
    (picture_start_code < video->start_code && video->start_code < 0x000001b0)
#else
# define MAMAN_A_TORT(video) \
    (picture_start_code < video->start_code && video->start_code < 0xb0010000)
#endif				/* __ENABLE_X000001AF___HACK_0__ */

#if WORDS_BIGENDIAN
# define do_something_0() \
    {									      \
	cache_size -= _seek_h262_B_a_01[(vector & 4094) + 1];		      \
    }
#else
# define do_something_0() \
    {									      \
	cache_size -= _seek_h262_B_a_01[vector & 4094];			      \
    }
#endif				/* WORDS_BIGENDIAN */

#define do_something_1(r, s, t) \
    {									      \
	if (cache_size < 12) {						      \
	    OPTIMISTIQUE_MOI_X(2);					      \
	    cache_buffer = cache_buffer << 16 | *buffer << 8 | buffer[1];     \
	    buffer += 2;						      \
	    cache_size += 16;						      \
	    size -= 2;							      \
	}								      \
	vector = cache_buffer >> (cache_size - 12);			      \
	if (vector & 2048) {						      \
	    cache_size--;						      \
	} else {							      \
	    do_something_0();						      \
	    if (cache_size < video->f_code_##s##_##t) {			      \
		OPTIMISTIQUE_MOI_1(0);					      \
		cache_buffer = cache_buffer << 8 | *buffer++;		      \
		cache_size += 8;					      \
		size--;							      \
	    }								      \
	    cache_size -= video->f_code_##s##_##t;			      \
	}								      \
    }

#define do_something_2(t) \
    {									      \
	if (cache_size < 2) {						      \
	    OPTIMISTIQUE_MOI_1(0);					      \
	    cache_buffer = cache_buffer << 8 | *buffer++;		      \
	    cache_size += 8;						      \
	    size--;							      \
	}								      \
	cache_size--;							      \
	cache_size -= cache_buffer >> cache_size & 1;			      \
    }

#define do_something_3(r, s) \
    {									      \
	if (!cache_size) {						      \
	    OPTIMISTIQUE_MOI_X(2);					      \
	    cache_buffer = *buffer << 8 | buffer[1];			      \
	    buffer += 2;						      \
	    cache_size = 16;						      \
	    size -= 2;							      \
	}								      \
	cache_size--;							      \
    }

#define motion_vector_0(r, s) \
    {									      \
	int vector;							      \
									      \
	do_something_1(r, s, 0);					      \
	do_something_2(0);						      \
	do_something_1(r, s, 1);					      \
	do_something_2(1);						      \
    }

#define subsequent_motion_vector_0(r, s) \
    {									      \
	int vector;							      \
									      \
	do_something_1(r, s, 0);					      \
	do_something_1(r, s, 1);					      \
    }

#define motion_vector_1(r, s) \
    {									      \
	int vector;							      \
									      \
	do_something_1(r, s, 0);					      \
	do_something_1(r, s, 1);					      \
    }

#define subsequent_motion_vector_1(r, s) \
    {									      \
	int vector;							      \
									      \
	do_something_1(r, s, 0);					      \
	do_something_1(r, s, 1);					      \
    }

extern unsigned char _seek_h262_B_1_01[], _seek_h262_B_9_01[],
    _seek_h262_B_a_01[], _seek_h262_B_e_012_0[], _seek_h262_B_e_012_1[],
    _seek_h262_B_f_012_0[], _seek_h262_B_f_012_1[];
extern const unsigned char _seek_h262_B_c_0[], _seek_h262_B_c_1[],
    _seek_h262_B_d_0[], _seek_h262_B_d_1[];

#if !__INLINE_OPTIMISTIQUE_MOI__
static void optimistique_moi(struct seek_h262_video *, unsigned char **,
			     unsigned *);
#endif				/* !__INLINE_OPTIMISTIQUE_MOI__ */

static const unsigned char x7_19[] = "\1\1\1\1\3\1\3\1\2\1\2\1\2\2\3\1";

#if !__INLINE_OPTIMISTIQUE_MOI__
static void
optimistique_moi(struct seek_h262_video *video, unsigned char **buffer,
		 unsigned *size)
{
    video->buffer = *buffer;
    video->size = *size;
    video->optimistique_moi(video);
    *buffer = video->buffer;
    *size = video->size;
}
#endif				/* !__INLINE_OPTIMISTIQUE_MOI__ */


void
_seek_h262___________x___13818_slices(struct seek_h262_video *video)
{
    do {
	unsigned cache_buffer, cache_size, macroblock_address,
	    macroblock_address_increment, mb_row, packed_flags,
	    previous_macroblock_address, size, spatial_temporal_weight_code =
	    0;
	unsigned char *buffer;

	buffer = video->buffer;
	size = video->size;
#if WORDS_BIGENDIAN
	mb_row = (video->start_code & 0xff) - 1;
#else
	mb_row = (video->start_code >> 24) - 1;
#endif				/* WORDS_BIGENDIAN */
	if (video->picture_structure == Frame_picture) {
	    if (video->mb_height < mb_row + 1) {
		ABORT_DECODING(video, buffer, size, VIDEO_SLICE_FAULT);
	    }
	} else {
	    if (video->mb_height < (mb_row << 1) + 1) {
		ABORT_DECODING(video, buffer, size, VIDEO_SLICE_FAULT);
	    }
	}
	OPTIMISTIQUE_MOI_X(4);
	cache_buffer =
	    *buffer << 24 | buffer[1] << 16 | buffer[2] << 8 | buffer[3];
	buffer += 4;
	cache_size = 32;
	size -= 4;
	{
	    if (2800 < video->vertical_size) {
		if (127 < mb_row) {
		    ABORT_DECODING(video, buffer, size, VIDEO_SLICE_FAULT);
		}
		cache_size = 29;
		mb_row |= (cache_buffer >> 22) & ~127;
	    }
	    if (video->sequence_scalable_extension_seen) {
		if (video->scalable_mode == data_partitioning) {
		    cache_size -= 7;
		}
	    }
	    cache_size -= 5;
	    if (!(cache_buffer >> cache_size & 31)) {
		ABORT_DECODING
		    (video, buffer, size, VIDEO_NULL_SLICE_QUANTISER_SCALE);
	    }
	    if (cache_buffer >> (cache_size - 1) & 1) {
		do {
		    cache_size -= 9;
		    if (cache_size < 9) {
			OPTIMISTIQUE_MOI_X(2);
			cache_buffer =
			    cache_buffer << 16 | *buffer << 8 | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
#if __ENABLE_X000001AF___HACK_3__
		    if (video->x000001af___hack_3) {
			struct seek_h262_x000001af__ slice;

			slice.decoder = video;
			slice.x.x2.extra_information_slice =
			    cache_buffer >> cache_size & 255;

			video->x000001af___hack_3(&slice);
		    }
#endif				/* __ENABLE_X000001AF___HACK_0__ */
		} while (cache_buffer >> (cache_size - 1) & 1);
	    }
	    cache_size--;
	}
	previous_macroblock_address = mb_row * video->mb_width - 1;
	if (cache_size < 9) {
	    OPTIMISTIQUE_MOI_X(3);
	    cache_buffer = cache_buffer << 24
		| *buffer << 16 | buffer[1] << 8 | buffer[2];
	    buffer += 3;
	    cache_size += 24;
	    size -= 3;
	} else if (cache_size < 17) {
	    OPTIMISTIQUE_MOI_X(2);
	    cache_buffer = cache_buffer << 16 | *buffer << 8 | buffer[1];
	    buffer += 2;
	    cache_size += 16;
	    size -= 2;
	} else if (cache_size < 25) {
	    OPTIMISTIQUE_MOI_1(0);
	    cache_buffer = cache_buffer << 8 | *buffer++;
	    cache_size += 8;
	    size--;
	}
	if (cache_buffer >> (cache_size - 1) & 1) {
	    macroblock_address_increment = 1;
	    cache_size--;
	    macroblock_address = previous_macroblock_address + 1;
	} else {
	    unsigned offset;

	    offset = cache_buffer >> (cache_size - 11) & 2047;
	    if (offset != 8) {
		offset = ((unsigned short *) _seek_h262_B_1_01)[offset];
		cache_size -= offset >> 8;
		macroblock_address_increment = offset & 255;
	    } else {
		macroblock_address_increment = 33;
		cache_size -= 11;
		do {
		    offset = cache_buffer >> (cache_size - 11) & 2047;
		    if (offset == 8) {
			macroblock_address_increment += 33;
			cache_size -= 11;
		    } else {
			offset =
			    ((unsigned short *) _seek_h262_B_1_01)[offset];
			cache_size -= offset >> 8;
			macroblock_address_increment += offset & 255;
		    }
		    if (cache_size < 11) {
			OPTIMISTIQUE_MOI_X(2);
			cache_buffer =
			    cache_buffer << 16 | *buffer << 8 | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
		} while (offset == 8);
	    }
	    macroblock_address = previous_macroblock_address
		+ macroblock_address_increment;
	    if (1) {
		macroblock_address_increment = 1;
	    }
	}
	do {
	    OPTIMISTIQUE_MOI_c(13818);

	    packed_flags =
		((unsigned short *) video->B_2345678_01)
		[cache_buffer >> (cache_size - 9) & 511];
	    cache_size -= packed_flags >> 8;
	    packed_flags &= 255;
	    if (packed_flags & spatial_temporal_weight_code_flag_mask
		&& video->spatial_temporal_weight_code_table_index) {
		if (cache_size < 2) {
		    cache_buffer = cache_buffer << 8 | *buffer++;
		    cache_size += 8;
		    size--;
		}
		cache_size -= 2;
		spatial_temporal_weight_code = cache_buffer >> cache_size & 3;
	    }
	    if (cache_size < 17) {
		cache_buffer = cache_buffer << 16 | *buffer << 8 | buffer[1];
		buffer += 2;
		cache_size += 16;
		size -= 2;
	    }
	    if (packed_flags & macroblock_intra_mask) {
		unsigned i = 0;
		unsigned char *B_ef_012_0, *B_ef_012_1;

		if (video->decode_dct_type) {
		    cache_size--;
		}
		if (packed_flags & macroblock_quant_mask) {
		    cache_size -= 5;
		    if (!(cache_buffer >> cache_size & 31)) {
			ABORT_DECODING
			    (video, buffer, size,
			     VIDEO_NULL_MACROBLOCK_QUANTISER_SCALE);
		    }
		} {
		    if (video->concealment_motion_vectors) {
			if (video->picture_structure != Frame_picture) {
			    do_something_3(0, 0);
			}
			subsequent_motion_vector_1(0, 0);
			if (!cache_size) {
			    cache_buffer = *buffer << 8 | buffer[1];
			    buffer += 2;
			    cache_size = 16;
			    size -= 2;
			}
			cache_size--;
		    }
		}
		B_ef_012_0 = video->B_ef_012_0;
		B_ef_012_1 = video->B_ef_012_1;
		for (; i < 4; i++) {
		    if (cache_size < 17) {
			cache_buffer = cache_buffer << 16 | *buffer << 8
			    | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
		    {
			int dc_dct_size;

			dc_dct_size = cache_buffer >> (cache_size - 9) & 511;
			if (dc_dct_size < 448) {
			    dc_dct_size =
				((short *) _seek_h262_B_c_0)[dc_dct_size >> 2];
			    cache_size -= dc_dct_size & 255;
			} else {
			    if (cache_size < 25) {
				cache_buffer = cache_buffer << 8 | *buffer++;
				cache_size += 8;
				size--;
			    }
			    dc_dct_size =
				((unsigned short *) _seek_h262_B_c_1)
				[dc_dct_size & 63];
			    cache_size -= dc_dct_size >> 8;
			}
		    }
		    DETACH_13818_B__BLOCK(B_ef_012_0, B_ef_012_1);
		}
		i = video->chroma_format - 4;
		for (; i; i--) {
		    if (cache_size < 17) {
			cache_buffer = cache_buffer << 16 | *buffer << 8
			    | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
		    {
			int dc_dct_size;

			dc_dct_size = cache_buffer >> (cache_size - 10) & 1023;
			if (dc_dct_size < 960) {
			    dc_dct_size =
				((short *) _seek_h262_B_d_0)[dc_dct_size >> 2];
			    cache_size -= dc_dct_size & 255;
			} else {
			    if (cache_size < 25) {
				cache_buffer = cache_buffer << 8 | *buffer++;
				cache_size += 8;
				size--;
			    }
			    dc_dct_size =
				((unsigned short *) _seek_h262_B_d_1)
				[dc_dct_size & 63];
			    cache_size -= dc_dct_size >> 8;
			}
		    }
		    DETACH_13818_B__BLOCK(B_ef_012_0, B_ef_012_1);
		}
	    } else {
		if (packed_flags
		    & (macroblock_motion_backward_mask
		       | macroblock_motion_forward_mask)) {
		    unsigned motion_type = Frame_based_prediction_or_16x8_MC;

		    if (video->decode_motion_type) {
			cache_size -= 2;
			motion_type = cache_buffer >> cache_size & 3;
			if (motion_type == video->dmv_abort) {
			    ABORT_DECODING(video, buffer, size,
					   VIDEO_INVALID_MOTION_TYPE);
			}
		    }
		    if (packed_flags & video->decode_dct_type) {
			cache_size--;
		    }
		    if (packed_flags & macroblock_quant_mask) {
			cache_size -= 5;
			if (!(cache_buffer >> cache_size & 31)) {
			    ABORT_DECODING
				(video, buffer, size,
				 VIDEO_NULL_MACROBLOCK_QUANTISER_SCALE);
			}
		    }
		    if (video->picture_structure == Frame_picture) {
			unsigned motion_vector_count;

			switch (motion_type) {
			case Dual_Prime:
			    if (packed_flags
				& macroblock_motion_forward_mask) {
				motion_vector_0(0, 0);
			    }
			    if (packed_flags
				& macroblock_motion_backward_mask) {
				subsequent_motion_vector_0(0, 1);
			    }
			    break;
			case Field_based_prediction:
			    motion_vector_count =
				video->spatial_temporal_weight_code_table_index
				<< 2 | spatial_temporal_weight_code;
			    motion_vector_count =
				x7_19[motion_vector_count] < 2 ? 2 : 1;
			    if (packed_flags
				& macroblock_motion_forward_mask) {
				do_something_3(0, 0);
				motion_vector_1(0, 0);
				if (motion_vector_count != 1) {
				    do_something_3(1, 0);
				    subsequent_motion_vector_1(1, 0);
				}
			    }
			    if (packed_flags
				& macroblock_motion_backward_mask) {
				do_something_3(0, 1);
				subsequent_motion_vector_1(0, 1);
				if (motion_vector_count != 1) {
				    do_something_3(1, 1);
				    subsequent_motion_vector_1(1, 1);
				}
			    }
			    break;
			default:
			    if (packed_flags
				& macroblock_motion_forward_mask) {
				motion_vector_1(0, 0);
			    }
			    if (packed_flags
				& macroblock_motion_backward_mask) {
				subsequent_motion_vector_1(0, 1);
			    }
			}
		    } else {
			switch (motion_type) {
			case Dual_Prime:
			    if (packed_flags
				& macroblock_motion_forward_mask) {
				motion_vector_0(0, 0);
			    }
			    if (packed_flags
				& macroblock_motion_backward_mask) {
				subsequent_motion_vector_0(0, 1);
			    }
			    break;
			case Field_based_prediction:
			    if (packed_flags
				& macroblock_motion_forward_mask) {
				do_something_3(0, 0);
				motion_vector_1(0, 0);
			    }
			    if (packed_flags
				& macroblock_motion_backward_mask) {
				do_something_3(0, 1);
				subsequent_motion_vector_1(0, 1);
			    }
			    break;
			default:
			    if (packed_flags
				& macroblock_motion_forward_mask) {
				do_something_3(0, 0);
				motion_vector_1(0, 0);
				do_something_3(1, 0);
				subsequent_motion_vector_1(1, 0);
			    }
			    if (packed_flags
				& macroblock_motion_backward_mask) {
				do_something_3(0, 1);
				subsequent_motion_vector_1(0, 1);
				do_something_3(1, 1);
				subsequent_motion_vector_1(1, 1);
			    }
			}
		    }
		} else {
		    if (packed_flags & video->decode_dct_type) {
			cache_size--;
		    }
		    if (packed_flags & macroblock_quant_mask) {
			cache_size -= 5;
			if (!(cache_buffer >> cache_size & 31)) {
			    ABORT_DECODING
				(video, buffer, size,
				 VIDEO_NULL_MACROBLOCK_QUANTISER_SCALE);
			}
		    }
		    if (video->picture_coding_type == predictive_coded) {
			if (video->deinterlaced_release_coding_type
			    == intra_coded) {
			    ABORT_DECODING
				(video, buffer, size, VIDEO_SLICE_FAULT);
			}
		    }
		}
		if (packed_flags & macroblock_pattern_mask) {
		    unsigned pattern_code;

		    if (cache_size < 15) {
			cache_buffer =
			    cache_buffer << 16 | *buffer << 8 | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
		    pattern_code = ((unsigned short *) _seek_h262_B_9_01)
			[cache_buffer >> (cache_size - 9) & 511];
		    cache_size -= pattern_code & 15;
		    pattern_code =
			(pattern_code >> 4 | pattern_code >> 9) & 63;
		    switch (video->chroma_format) {
		    case H1V1:
			cache_size -= 6;
			pattern_code = pattern_code << 6
			    | (cache_buffer >> cache_size & 63);
			break;
		    case H2V1:
			cache_size -= 2;
			pattern_code = pattern_code << 2
			    | (cache_buffer >> cache_size & 3);
		    }
		    for (; pattern_code; pattern_code >>= 1) {
			if (pattern_code & 1) {
			    DETACH_13818_Be_BLOCK();
			}
		    }
		}
	    }
	    if (cache_size < 23) {
		if (cache_size < 7) {
		    cache_buffer = cache_buffer << 24
			| *buffer << 16 | buffer[1] << 8 | buffer[2];
		    buffer += 3;
		    cache_size += 24;
		    size -= 3;
		} else if (cache_size < 15) {
		    cache_buffer =
			cache_buffer << 16 | *buffer << 8 | buffer[1];
		    buffer += 2;
		    cache_size += 16;
		    size -= 2;
		} else {
		    cache_buffer = cache_buffer << 8 | *buffer++;
		    cache_size += 8;
		    size--;
		}
	    }
	    previous_macroblock_address = macroblock_address;
	    {
		unsigned offset;

		offset = cache_buffer >> (cache_size - 23) & 8388607;
		if (!offset) {
		    break;
		}

		if (offset & 4194304) {
		    macroblock_address_increment = 1;
		    cache_size--;
		    macroblock_address = previous_macroblock_address + 1;
		} else {
		    offset >>= 12;
		    if (offset != 8) {
			offset =
			    ((unsigned short *) _seek_h262_B_1_01)[offset];
			cache_size -= offset >> 8;
			macroblock_address_increment = offset & 255;
		    } else {
			macroblock_address_increment = 33;
			cache_size -= 11;
			do {
			    offset = cache_buffer >> (cache_size - 11) & 2047;
			    if (offset == 8) {
				macroblock_address_increment += 33;
				cache_size -= 11;
			    } else {
				offset = ((unsigned short *) _seek_h262_B_1_01)
				    [offset];
				cache_size -= offset >> 8;
				macroblock_address_increment += offset & 255;
			    }
			    if (cache_size < 11) {
				OPTIMISTIQUE_MOI_X(2);
				cache_buffer = cache_buffer << 16
				    | *buffer << 8 | buffer[1];
				buffer += 2;
				cache_size += 16;
				size -= 2;
			    }
			} while (offset == 8);
		    }
		    macroblock_address = previous_macroblock_address
			+ macroblock_address_increment;
		    if (video->picture_coding_type == intra_coded) {
			if (video->scalable_mode == SNR_scalability
			    || video->picture_spatial_scalable_extension_seen) {
			} else {
			    ABORT_DECODING
				(video, buffer, size,
				 VIDEO_INVALID_MACROBLOCK_ADDRESS_INCREMENT);
			}
		    } else if (video->picture_coding_type == predictive_coded) {
			if (video->deinterlaced_release_coding_type
			    == intra_coded) {
			    ABORT_DECODING
				(video, buffer, size,
				 VIDEO_INVALID_MACROBLOCK_ADDRESS_INCREMENT);
			}
		    }
		}
	    }
	} while (1);
	cache_size &= ~7;
	cache_buffer &= (1 << cache_size) - 1;
	video->buffer = buffer;
	video->size = size;
	if (cache_buffer) {
	    if (cache_size != 24 || cache_buffer != 1) {
		ABORT_DECODING(video, buffer, size, VIDEO_SLICE_FAULT);
	    }
	    if (!size) {
		video->optimistique_moi(video);
	    }
#if WORDS_BIGENDIAN
	    video->start_code = 0x00000100 | *video->buffer;
#else
	    video->start_code = 0x00010000 | *video->buffer << 24;
#endif				/* WORDS_BIGENDIAN */
	    video->buffer++;
	    video->size--;
	} else if (cache_size) {
	    video->start_code = cache_size >> 3;
	    video->neXt_start_code(video);
	} else {
	    video->next_start_code(video);
	}
    } while (MAMAN_A_TORT(video));
}
