/*
 * h262.7.11172.c
 * Copyright (C) 2003, 2004, 2005, 2006, 2007  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.1.d4.h>
#include <h262.1.h>
#include <h262.2.h>
#include <h262.3.h>
#include <h262.3.l7.h>
#include <h262.6.d4.h>
#include <h262.a.h>
#include <h262.d.l7.h>

#define __RUN_FUTILE__			0

#define ABORT_DECODING(video, buffer, size, error) \
    {									      \
	_seek_h262_abort_divergent_decoding				      \
	    ((struct seek_h262_whatever *) video, buffer, size, error);	      \
    }

#if __ENABLE_X000001AF___HACK_0__
# define MAMAN_A_TORT(video) \
    (picture_start_code < video->start_code				      \
     && video->start_code < video->break_code)
#elif WORDS_BIGENDIAN
# define MAMAN_A_TORT(video) \
    (picture_start_code < video->start_code && video->start_code < 0x000001b0)
#else
# define MAMAN_A_TORT(video) \
    (picture_start_code < video->start_code && video->start_code < 0xb0010000)
#endif				/* __ENABLE_X000001AF___HACK_0__ */

extern unsigned char _seek_h262_B_1_01[], _seek_h262_B_5a[], _seek_h262_B_5b[],
    _seek_h262_B_e_012_0[], _seek_h262_B_e_012_1[];

#if !__INLINE_OPTIMISTIQUE_MOI__
static void optimistique_moi(struct seek_h262_video *, unsigned char **,
			     unsigned *);
#endif				/* !__INLINE_OPTIMISTIQUE_MOI__ */

static const unsigned char chroma_size_0[] = {
/* *INDENT-OFF* */
    0x08, 0x08, 0x08, 0x08, 0x07, 0x07, 0x07, 0x07,
    0x09, 0x09, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06,
    0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05,
    0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04,
    0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04,
    0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04,
    0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03,
    0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03
/* *INDENT-ON* */
}, chroma_size_1[] = {
/* *INDENT-OFF* */
    0x11, 0x10, 0x0f, 0x0f, 0x0e, 0x0e, 0x0e, 0x0e,
    0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d,
    0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b,
    0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b
/* *INDENT-ON* */
};

#if !__INLINE_OPTIMISTIQUE_MOI__
static void
optimistique_moi(struct seek_h262_video *video, unsigned char **buffer,
		 unsigned *size)
{
    video->buffer = *buffer;
    video->size = *size;
    video->optimistique_moi(video);
    *buffer = video->buffer;
    *size = video->size;
}
#endif				/* !__INLINE_OPTIMISTIQUE_MOI__ */


void
_seek_h262_GRAYSCALE_x08_11172_slices(struct seek_h262_video *video)
{
    __11172_MACROBLOCK_decls;

    __MACROBLOCK_setup();

    video->macroblock = &macroblock;
    _seek_h262_set_GRAYSCALE_11172_blocks(video);

    do {
	unsigned cache_buffer, cache_size = 27, dc_dct_pred = 1024,
	    quantiser_scale, size, slice_address;
	unsigned char *buffer;

	buffer = video->buffer;
	size = video->size;
#if WORDS_BIGENDIAN
	slice_address = (video->start_code & 0xff) - 1;
#else
	slice_address = (video->start_code >> 030) - 1;
#endif				/* WORDS_BIGENDIAN */
	OPTIMISTIQUE_MOI_X(4);
	cache_buffer =
	    *buffer << 24 | buffer[1] << 16 | buffer[2] << 8 | buffer[3];
	buffer += 4;
	size -= 4;
	{
	    quantiser_scale = cache_buffer >> 27;
	    if (!quantiser_scale) {
		ABORT_DECODING
		    (video, buffer, size, VIDEO_NULL_SLICE_QUANTISER_SCALE);
	    }
	    if (cache_buffer >> (cache_size - 1) & 1) {
		do {
		    cache_size -= 9;
		    if (cache_size < 9) {
			OPTIMISTIQUE_MOI_X(2);
			cache_buffer =
			    cache_buffer << 16 | *buffer << 8 | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
#if __ENABLE_X000001AF___HACK_3__
		    if (video->x000001af___hack_3) {
			struct seek_h262_x000001af__ slice;

			slice.decoder = video;
			slice.x.x2.extra_information_slice =
			    cache_buffer >> cache_size & 255;

			video->x000001af___hack_3(&slice);
		    }
#endif				/* __ENABLE_X000001AF___HACK_0__ */
		} while (cache_buffer >> (cache_size - 1) & 1);
	    }
	    cache_size--;
	}
	slice_address *= video->mb_width;
	video->mb_offset = slice_address << 8;
	video->mb_advance = video->mb_offset + (video->mb_width << 4);
	slice_address--;
	macroblock.previous_macroblock_address = slice_address;
	video->macroblock = &macroblock;
	if (cache_size < 12) {
	    OPTIMISTIQUE_MOI_X(2);
	    cache_buffer = cache_buffer << 16 | *buffer << 8 | buffer[1];
	    buffer += 2;
	    cache_size += 16;
	    size -= 2;
	}
	{
	    unsigned offset;

	    offset = cache_buffer >> (cache_size - 11) & 2047;
	    while (offset == 15) {
		cache_size -= 11;
		if (cache_size < 11) {
		    OPTIMISTIQUE_MOI_X(2);
		    cache_buffer =
			cache_buffer << 16 | *buffer << 8 | buffer[1];
		    buffer += 2;
		    cache_size += 16;
		    size -= 2;
		}
		offset = cache_buffer >> (cache_size - 11) & 2047;
	    }
	    if (offset & 1024) {
		cache_size--;
		macroblock.macroblock_address_increment = 1;
		macroblock.macroblock_address =
		    macroblock.previous_macroblock_address + 1;
	    } else {
		macroblock.macroblock_address_increment = 0;
		while (offset == 8) {
		    macroblock.macroblock_address_increment += 33;
		    cache_size -= 11;
		    if (cache_size < 11) {
			OPTIMISTIQUE_MOI_X(2);
			cache_buffer =
			    cache_buffer << 16 | *buffer << 8 | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
		    offset = cache_buffer >> (cache_size - 11) & 2047;
		}
		offset = ((unsigned short *) _seek_h262_B_1_01)[offset];
		cache_size -= offset >> 8;
		macroblock.macroblock_address_increment += offset & 255;
		macroblock.macroblock_address =
		    macroblock.previous_macroblock_address
		    + macroblock.macroblock_address_increment;
		if (1) {
		    macroblock.macroblock_address_increment = 1;
		}
		{
		    unsigned mb_column;

		    mb_column =
			macroblock.macroblock_address % video->mb_width;
		    video->mb_advance =
			(macroblock.macroblock_address - mb_column) << 8;
		    video->mb_offset = video->mb_advance + (mb_column << 4);
		    video->mb_advance += video->mb_width << 4;
		}
		if (cache_size < 7) {
		    OPTIMISTIQUE_MOI_1(0);
		    cache_buffer = cache_buffer << 8 | *buffer++;
		    cache_size += 8;
		    size--;
		}
	    }
	}
	do {
	    OPTIMISTIQUE_MOI_c(11172);

	    cache_size--;
	    macroblock.packed_flags = (cache_buffer >> cache_size & 1) ^ 1;
	    cache_size -= macroblock.packed_flags;
	    macroblock.packed_flags = (macroblock.packed_flags << 7) | 8;
	    if (macroblock.packed_flags & macroblock_quant_mask) {
		cache_size -= 5;
		quantiser_scale = cache_buffer >> cache_size & 31;
		if (!quantiser_scale) {
		    ABORT_DECODING(video, buffer, size,
				   VIDEO_NULL_MACROBLOCK_QUANTISER_SCALE);
		}
	    }
	    {
		seek_h262_coefficient *coefficients;
		seek_h262_scan_index *inverse_scan;
		unsigned chroma_format = 4;

		coefficients = macroblock.coefficients + _1ST_BLOCK_OFFSET;
		inverse_scan = video->intra_quantiser_matrix[0];
		for (; chroma_format; chroma_format--) {
		    unsigned i = 0;
#if __ENABLE_BLOCK_META_DATA__
		    unsigned meta_block;
#endif				/* __ENABLE_BLOCK_META_DATA__ */

		    if (cache_size < 16) {
			cache_buffer =
			    cache_buffer << 16 | *buffer << 8 | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
		    {
			int dc_dct_size;

			dc_dct_size =
			    ((short *) _seek_h262_B_5a)
			    [cache_buffer >> (cache_size - 6) & 63];
			cache_size -= dc_dct_size & 127;
			if (dc_dct_size & 128) {
			    int dct_diff;
			    unsigned mask;

			    mask = dc_dct_size >> 8 & 255;
			    dct_diff = cache_buffer >> cache_size & mask;
			    if (!(dct_diff & ~(mask >> 1))) {
				dct_diff -= mask;
			    }
			    dc_dct_pred += dct_diff << 3;
			} else {
			    dc_dct_pred += dc_dct_size >> 8;
			}
			*coefficients = dc_dct_pred;
		    }
		    if (*coefficients < 0) {
			*coefficients = 0;
		    } else if (2047 < *coefficients) {
			*coefficients = 2047;
		    }
		    DECODE_11172_B__BLOCK();
		    coefficients += BLOCK_SIZE;
		}
		chroma_format = 2;
		for (; chroma_format; chroma_format--) {
		    if (cache_size < 16) {
			cache_buffer =
			    cache_buffer << 16 | *buffer << 8 | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
#if WORDS_BIGENDIAN
		    cache_size -= _seek_h262_B_5b
			[(cache_buffer >> (cache_size - 8) & 254) + 1] & 127;
#else
		    cache_size -= _seek_h262_B_5b
			[(cache_buffer >> (cache_size - 8) & 254) + 0] & 127;
#endif				/* WORDS_BIGENDIAN */
		    DETACH_11172_B__BLOCK(chroma_size_0, chroma_size_1);
		}
#if __RUN_FUTILE__
		macroblock.previous_macroblock_address =
		    macroblock.macroblock_address;
#else
		video->______macroblock[4] (video);
#endif				/* __RUN_FUTILE__ */
	    }
	    if (cache_size < 23) {
		if (cache_size < 7) {
		    cache_buffer = cache_buffer << 24
			| *buffer << 16 | buffer[1] << 8 | buffer[2];
		    buffer += 3;
		    cache_size += 24;
		    size -= 3;
		} else if (cache_size < 15) {
		    cache_buffer =
			cache_buffer << 16 | *buffer << 8 | buffer[1];
		    buffer += 2;
		    cache_size += 16;
		    size -= 2;
		} else {
		    cache_buffer = cache_buffer << 8 | *buffer++;
		    cache_size += 8;
		    size--;
		}
	    }
	    {
		unsigned offset;

		offset = cache_buffer >> (cache_size - 23) & 8388607;
		if (!offset) {
		    break;
		}

		offset >>= 12;
		while (offset == 15) {
		    cache_size -= 11;
		    if (cache_size < 11) {
			OPTIMISTIQUE_MOI_X(2);
			cache_buffer =
			    cache_buffer << 16 | *buffer << 8 | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
		    offset = cache_buffer >> (cache_size - 11) & 2047;
		}
		if (offset & 1024) {
		    cache_size--;
		    macroblock.macroblock_address_increment = 1;
		    macroblock.macroblock_address =
			macroblock.previous_macroblock_address + 1;
		    video->mb_offset += 16;
		    if (video->mb_offset == video->mb_advance) {
			video->mb_advance += video->mb_width << 8;
			video->mb_offset =
			    video->mb_advance - (video->mb_width << 4);
		    }
		} else {
		    ABORT_DECODING(video, buffer, size,
				   VIDEO_INVALID_MACROBLOCK_ADDRESS_INCREMENT);
		}
	    }
	} while (1);
	cache_size &= ~7;
	cache_buffer &= (1 << cache_size) - 1;
	video->buffer = buffer;
	video->size = size;
	if (cache_buffer) {
	    if (cache_size != 24 || cache_buffer != 1) {
		ABORT_DECODING(video, buffer, size, VIDEO_SLICE_FAULT);
	    }
	    if (!size) {
		video->optimistique_moi(video);
	    }
#if WORDS_BIGENDIAN
	    video->start_code = 0x00000100 | *video->buffer;
#else
	    video->start_code = 0x00010000 | *video->buffer << 24;
#endif				/* WORDS_BIGENDIAN */
	    video->buffer++;
	    video->size--;
	} else if (cache_size) {
	    video->start_code = cache_size >> 3;
	    video->neXt_start_code(video);
	} else {
	    video->next_start_code(video);
	}
    } while (MAMAN_A_TORT(video));
}
