/*
 * h262.7.c
 * Copyright (C) 2003, 2004, 2005  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.2.h>
#include <h262.3.h>
#include <h262.4.h>
#include <h262.a.h>

#if 0
# define CHECK_MOTION_LIMITS(half_flags, limits, offset) \
    {									      \
	if ((int) (offset) < 0						      \
	    || video->block_##limits##_motion_limit[half_flags] < (offset)) { \
	    return;							      \
	}								      \
    }
#else
# define CHECK_MOTION_LIMITS(half_flags, limits, offset) \
    {									      \
	if (video->block_##limits##_motion_limit[(half_flags)] < (offset)) {  \
	    return;							      \
	}								      \
    }
#endif				/* 0 */

#define DOWNLOAD_RELEASE(s, video, i) \
    {									      \
	Cb_reference##s = (video)->i##_release.Cb;			      \
	Cr_reference##s = (video)->i##_release.Cr;			      \
	Y_reference##s = (video)->i##_release.Y;			      \
    }

#define DOWNLOAD_ReLEASE(s, video, i) \
    {									      \
	struct seek_h262_field_reference *field_reference;		      \
									      \
	field_reference = &(video)->field_references[i];		      \
	Cb_reference##s = field_reference->Cb;				      \
	Cr_reference##s = field_reference->Cr;				      \
	Y_reference##s = field_reference->Y;				      \
    }

#define DOWNLOAD_xMOTION_VECTOR(v, macroblock, l) \
    {									      \
	vector##v##_0 = (macroblock)->vector##l##_0;			      \
	vector##v##_1 = (macroblock)->vector##l##_1;			      \
    }

#define DOwNLOAD_xMOTION_VECTOR(v, macroblock, l) \
    {									      \
	vector##v##_0 = (macroblock)->vector##l##_0;			      \
	vector##v##_1 = (macroblock)->vector##l##_1;			      \
	motion_vertical_field_select##v =				      \
	    (macroblock)->motion_vertical_field_select##l;		      \
    }

#define MACROBLOCK_INIT(structure) \
    {									      \
	macroblock = video->macroblock;					      \
	if (video->mb_width < macroblock->mb_column + 1) {		      \
	    return;							      \
	}								      \
	macroblock->previous_macroblock_address =			      \
	    macroblock->macroblock_address;				      \
	c_offset = macroblock->chrominance_slice_address		      \
	    + (macroblock->mb_column << 3);				      \
	y_offset = macroblock->luminance_slice_address			      \
	    + (macroblock->mb_column << 4);				      \
	pattern_code = macroblock->pattern_code;			      \
	pattern_code |= macroblock->dc_sparse << 1;			      \
	blocks = video->blocks;						      \
	line_size = video->luminance_line_size;				      \
    }

#define NON_BIDIRECTIONALLY_PREDICTIVE_MACROBLOCK(macroblock) \
    (((macroblock)->packed_flags					      \
      & (macroblock_motion_backward_mask | macroblock_motion_forward_mask))   \
     != (macroblock_motion_backward_mask | macroblock_motion_forward_mask))

#define SET_C_REFERENCES(s, offset, c, _0_x) \
    {									      \
	Cb_reference##s += offset;					      \
	Cr_reference##s += offset;					      \
	c##_half_flags =						      \
	    (vector##_0_x##_0 & 1) | (vector##_0_x##_1 & 1) << 1;	      \
    }

#define SET_C_REFERENCeS(s, offset, c, _0_x) \
    {									      \
	Cb_reference##s += offset;					      \
	Cr_reference##s += offset;					      \
	c##_half_flags =						      \
	    ((vector##_0_x##_0 & 1) | (vector##_0_x##_1 & 1) << 1) << 2;      \
    }

#define SET_C_REFEReNCES(s, offset, c, _0_x) \
    {									      \
	Cb_reference##s += offset;					      \
	Cr_reference##s += offset;					      \
	c##_half_flags |=						      \
	    (vector##_0_x##_0 & 1) | (vector##_0_x##_1 & 1) << 1;	      \
    }

#define SET_X_REFERENCES(s, c, y) \
    {									      \
	Cb_reference##s += c##_offset;					      \
	Cr_reference##s += c##_offset;					      \
	Y_reference##s += y##_offset;					      \
	c##_half_flags = 0;						      \
	y##_half_flags = 0;						      \
    }

#define SET_X_REFEReNCES(s, c, y) \
    {									      \
	Cb_reference##s += c##_offset;					      \
	Cr_reference##s += c##_offset;					      \
	Y_reference##s += y##_offset;					      \
    }

#define SET_X_ReFERENCES(s, c, y, line_size, vertical_field_select) \
    {									      \
	Cb_reference##s += c##_offset					      \
	    + ((line_size) >> 1) * (vertical_field_select);		      \
	Cr_reference##s += c##_offset					      \
	    + ((line_size) >> 1) * (vertical_field_select);		      \
	Y_reference##s += y##_offset + (line_size) * (vertical_field_select); \
	c##_half_flags = 0;						      \
	y##_half_flags = 0;						      \
    }

#define SET_X_ReFEReNCES(s, c, y, line_size, vertical_field_select) \
    {									      \
	Cb_reference##s += c##_offset					      \
	    + ((line_size) >> 1) * (vertical_field_select);		      \
	Cr_reference##s += c##_offset					      \
	    + ((line_size) >> 1) * (vertical_field_select);		      \
	Y_reference##s += y##_offset + (line_size) * (vertical_field_select); \
    }

#define SET_Y_REFERENCES(s, offset, y, _0_x) \
    {									      \
	Y_reference##s += offset;					      \
	y##_half_flags =						      \
	    (vector##_0_x##_0 & 1) | (vector##_0_x##_1 & 1) << 1;	      \
    }

#define SET_Y_REFERENCeS(s, offset, y, _0_x) \
    {									      \
	Y_reference##s += offset;					      \
	y##_half_flags =						      \
	    ((vector##_0_x##_0 & 1) | (vector##_0_x##_1 & 1) << 1) << 2;      \
    }

#define SET_Y_REFEReNCES(s, offset, y, _0_x) \
    {									      \
	Y_reference##s += offset;					      \
	y##_half_flags |=						      \
	    (vector##_0_x##_0 & 1) | (vector##_0_x##_1 & 1) << 1;	      \
    }

#define VERTICAL_FIELD_CORRECT(motion_vertical_field_select, vector) \
    ((vector & ~1) | motion_vertical_field_select)

static void macroblock_0(struct seek_h262_video *);
static void macroblock_1(struct seek_h262_video *);
static void macroblock_2(struct seek_h262_video *);
static void macroblock_3(struct seek_h262_video *);
static void macroblock_4(struct seek_h262_video *);
static void macroblock_5(struct seek_h262_video *);
static void macroblock_6(struct seek_h262_video *);
static void macroblock_7(struct seek_h262_video *);
static void macroblock_8(struct seek_h262_video *);
static void macroblock_9(struct seek_h262_video *);
static void macroblock_a(struct seek_h262_video *);
static void macroblock_b(struct seek_h262_video *);

void (*_seek_h262_YCbCr_422_macroblock[]) (struct seek_h262_video *) = {
/* *INDENT-OFF* */
    macroblock_0,
    macroblock_1,
    macroblock_2,
    macroblock_3,
    macroblock_4,
    macroblock_5,
    macroblock_6,
    macroblock_7,
    macroblock_8,
    macroblock_9,
    macroblock_a,
    macroblock_b
/* *INDENT-ON* */
};

static const unsigned reflex16[] = {
/* *INDENT-OFF* */
    0x0000, 0x1000, 0x2000, 0x3000, 0x0400, 0x1400, 0x2400, 0x3400,
    0x0800, 0x1800, 0x2800, 0x3800, 0x0c00, 0x1c00, 0x2c00, 0x3c00
/* *INDENT-ON* */
};

static void
macroblock_0(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;

    macroblock = (struct seek_h262_13818_macroblock *) video->macroblock;
    if (video->mb_width < macroblock->mb_column + 1) {
	return;
    }
    macroblock->previous_macroblock_address = macroblock->macroblock_address;
    {
	struct seek_h262_block *blocks;
	unsigned line_size, mb_column;

	blocks = video->blocks;
	line_size = video->luminance_line_size;
	mb_column = macroblock->mb_column;
	video->idct_set_block(blocks, 4, mb_column << 4, line_size);
	video->idct_set_block(blocks + 4, 4, mb_column << 3, line_size >> 1);
    }
}


static void
macroblock_1(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned c_offset, line_size, pattern_code, y_offset;

    MACROBLOCK_INIT(0);

    if (NON_BIDIRECTIONALLY_PREDICTIVE_MACROBLOCK(macroblock)) {
	unsigned f = 1;

	for (; f < 3; f++) {
	    unsigned c_half_flags, y_half_flags;
	    unsigned char *Cb_references, *Cr_references, *Y_references;

	    {
		int vector_x_x_0, vector_x_x_1;
		unsigned motion_vertical_field_select_x_x, offset;

		if (macroblock->packed_flags
		    & macroblock_motion_backward_mask) {
		    DOWNLOAD_RELEASE(s, video, Irulan);
		    if (f == 1) {
			DOwNLOAD_xMOTION_VECTOR(_x_x, macroblock, _0_1);
		    } else {
			DOwNLOAD_xMOTION_VECTOR(_x_x, macroblock, _1_1);
		    }
		} else {
		    DOWNLOAD_RELEASE(s, video, Iran);
		    if (f == 1) {
			DOwNLOAD_xMOTION_VECTOR(_x_x, macroblock, _0_0);
		    } else {
			DOwNLOAD_xMOTION_VECTOR(_x_x, macroblock, _1_0);
		    }
		}
		offset = y_offset + (vector_x_x_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_x, vector_x_x_1))
		    * line_size;
		SET_Y_REFERENCES(s, offset, y, _x_x);
		CHECK_MOTION_LIMITS(y_half_flags, 1, offset);
		vector_x_x_0 /= 2;
		offset = c_offset + (vector_x_x_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_x, vector_x_x_1))
		    * (line_size >> 1);
		SET_C_REFERENCES(s, offset, c, _x_x);
	    }
	    {
		seek_h262_constructor constructor;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructor = video->block_0_2[y_half_flags];
#else
		constructor = video->block_0_2[0][y_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = video->Mylene_release.Y + y_offset;
		if (f == 2) {
		    predictions += line_size;
		}
		constructor(predictions, Y_references, line_size << 1);
	    }
	    {
		seek_h262_constructor constructor;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructor = video->block_0_0[c_half_flags];
#else
		constructor = video->block_0_0[0][c_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = video->Mylene_release.Cb + c_offset;
		if (f == 2) {
		    predictions += line_size >> 1;
		}
		constructor(predictions, Cb_references, line_size);
		predictions = video->Mylene_release.Cr + c_offset;
		if (f == 2) {
		    predictions += line_size >> 1;
		}
		constructor(predictions, Cr_references, line_size);
	    }
	}
    } else {
	unsigned f = 1;

	for (; f < 3; f++) {
	    unsigned c_half_flags, y_half_flags;
	    unsigned char *Cb_references_0, *Cb_references_1, *Cr_references_0,
		*Cr_references_1, *Y_references_0, *Y_references_1;

	    {
		int vector_x_1_0, vector_x_1_1;
		unsigned motion_vertical_field_select_x_1, offset;

		DOWNLOAD_RELEASE(s_0, video, Irulan);
		if (f == 1) {
		    DOwNLOAD_xMOTION_VECTOR(_x_1, macroblock, _0_1);
		} else {
		    DOwNLOAD_xMOTION_VECTOR(_x_1, macroblock, _1_1);
		}
		offset = y_offset + (vector_x_1_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_1, vector_x_1_1))
		    * line_size;
		SET_Y_REFERENCeS(s_0, offset, y, _x_1);
		CHECK_MOTION_LIMITS(y_half_flags >> 02, 1, offset);
		vector_x_1_0 /= 2;
		offset = c_offset + (vector_x_1_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_1, vector_x_1_1))
		    * (line_size >> 1);
		SET_C_REFERENCeS(s_0, offset, c, _x_1);
	    }
	    {
		int vector_x_0_0, vector_x_0_1;
		unsigned motion_vertical_field_select_x_0, offset;

		DOWNLOAD_RELEASE(s_1, video, Iran);
		if (f == 1) {
		    DOwNLOAD_xMOTION_VECTOR(_x_0, macroblock, _0_0);
		} else {
		    DOwNLOAD_xMOTION_VECTOR(_x_0, macroblock, _1_0);
		}
		offset = y_offset + (vector_x_0_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_0, vector_x_0_1))
		    * line_size;
		SET_Y_REFEReNCES(s_1, offset, y, _x_0);
		CHECK_MOTION_LIMITS(y_half_flags & 0x3, 1, offset);
		vector_x_0_0 /= 2;
		offset = c_offset + (vector_x_0_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_0, vector_x_0_1))
		    * (line_size >> 1);
		SET_C_REFEReNCES(s_1, offset, c, _x_0);
	    }
	    {
		seek_h262_constructOr constructOr;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructOr = video->block_1_2[y_half_flags];
#else
		constructOr = video->block_1_2[0][y_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = video->Mylene_release.Y + y_offset;
		if (f == 2) {
		    predictions += line_size;
		}
		constructOr(predictions, Y_references_0, Y_references_1,
			    line_size << 1);
	    }
	    {
		seek_h262_constructOr constructOr;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructOr = video->block_1_0[c_half_flags];
#else
		constructOr = video->block_1_0[0][c_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = video->Mylene_release.Cb + c_offset;
		if (f == 2) {
		    predictions += line_size >> 1;
		}
		constructOr
		    (predictions, Cb_references_0, Cb_references_1, line_size);
		predictions = video->Mylene_release.Cr + c_offset;
		if (f == 2) {
		    predictions += line_size >> 1;
		}
		constructOr
		    (predictions, Cr_references_0, Cr_references_1, line_size);
	    }
	}
    }

    video->idct_add_block(blocks, pattern_code & 255,
			  macroblock->mb_column << 4, line_size);
    video->idct_add_block(blocks + 4, pattern_code >> 8,
			  macroblock->mb_column << 3, line_size >> 1);
}


static void
macroblock_2(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned c_offset, line_size, pattern_code, y_offset;

    MACROBLOCK_INIT(0);

    if (NON_BIDIRECTIONALLY_PREDICTIVE_MACROBLOCK(macroblock)) {
	seek_h262_constructor constructor;
	unsigned c_half_flags, y_half_flags;
	unsigned char *Cb_references, *Cr_references, *Y_references;

	{
	    int vector_0_x_0, vector_0_x_1;

	    if (macroblock->packed_flags & macroblock_motion_backward_mask) {
		DOWNLOAD_RELEASE(s, video, Irulan);
		DOWNLOAD_xMOTION_VECTOR(_0_x, macroblock, _0_1);
	    } else {
		DOWNLOAD_RELEASE(s, video, Iran);
		DOWNLOAD_xMOTION_VECTOR(_0_x, macroblock, _0_0);
	    }
	    if (vector_0_x_0 | vector_0_x_1) {
		unsigned offset;

		offset = y_offset + (vector_0_x_0 >> 1)
		    + (vector_0_x_1 >> 1) * line_size;
		SET_Y_REFERENCES(s, offset, y, _0_x);
		CHECK_MOTION_LIMITS(y_half_flags, 2, offset);
		vector_0_x_0 /= 2;
		offset = c_offset + (vector_0_x_0 >> 1)
		    + (vector_0_x_1 >> 1) * (line_size >> 1);
		SET_C_REFERENCES(s, offset, c, _0_x);
	    } else {
		SET_X_REFERENCES(s, c, y);
	    }
	}
#if !__Berkeley_FUCK_UP__
	constructor = video->block_0_3[y_half_flags];
#else
	constructor = video->block_0_3[0][y_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	constructor
	    (video->Mylene_release.Y + y_offset, Y_references, line_size);
	video->idct_add_block(blocks, pattern_code & 255,
			      macroblock->mb_column << 4, line_size);
#if !__Berkeley_FUCK_UP__
	constructor = video->block_0_1[c_half_flags];
#else
    	constructor = video->block_0_1[0][c_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	line_size >>= 1;
	constructor
	    (video->Mylene_release.Cb + c_offset, Cb_references, line_size);
	constructor
	    (video->Mylene_release.Cr + c_offset, Cr_references, line_size);
	video->idct_add_block(blocks + 4, pattern_code >> 8,
			      macroblock->mb_column << 3, line_size);
    } else {
	seek_h262_constructOr constructOr;
	unsigned c_half_flags, y_half_flags;
	unsigned char *Cb_references_0, *Cb_references_1, *Cr_references_0,
	    *Cr_references_1, *Y_references_0, *Y_references_1;

	{
	    int vector_0_1_0, vector_0_1_1;

	    DOWNLOAD_RELEASE(s_0, video, Irulan);
	    DOWNLOAD_xMOTION_VECTOR(_0_1, macroblock, _0_1);
	    if (vector_0_1_0 | vector_0_1_1) {
		unsigned offset;

		offset = y_offset + (vector_0_1_0 >> 1)
		    + (vector_0_1_1 >> 1) * line_size;
		SET_Y_REFERENCeS(s_0, offset, y, _0_1);
		CHECK_MOTION_LIMITS(y_half_flags >> 02, 2, offset);
		vector_0_1_0 /= 2;
		offset = c_offset + (vector_0_1_0 >> 1)
		    + (vector_0_1_1 >> 1) * (line_size >> 1);
		SET_C_REFERENCeS(s_0, offset, c, _0_1);
	    } else {
		SET_X_REFERENCES(s_0, c, y);
	    }
	}
	{
	    int vector_0_0_0, vector_0_0_1;

	    DOWNLOAD_RELEASE(s_1, video, Iran);
	    DOWNLOAD_xMOTION_VECTOR(_0_0, macroblock, _0_0);
	    if (vector_0_0_0 | vector_0_0_1) {
		unsigned offset;

		offset = y_offset + (vector_0_0_0 >> 1)
		    + (vector_0_0_1 >> 1) * line_size;
		SET_Y_REFEReNCES(s_1, offset, y, _0_0);
		CHECK_MOTION_LIMITS(y_half_flags & 0x3, 2, offset);
		vector_0_0_0 /= 2;
		offset = c_offset + (vector_0_0_0 >> 1)
		    + (vector_0_0_1 >> 1) * (line_size >> 1);
		SET_C_REFEReNCES(s_1, offset, c, _0_0);
	    } else {
		SET_X_REFEReNCES(s_1, c, y);
	    }
	}
#if !__Berkeley_FUCK_UP__
	constructOr = video->block_1_3[y_half_flags];
#else
	constructOr = video->block_1_3[0][y_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	constructOr(video->Mylene_release.Y + y_offset, Y_references_0,
		    Y_references_1, line_size);
	video->idct_add_block(blocks, pattern_code & 255,
			      macroblock->mb_column << 4, line_size);
#if !__Berkeley_FUCK_UP__
	constructOr = video->block_1_1[c_half_flags];
#else
    	constructOr = video->block_1_1[0][c_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	line_size >>= 1;
	constructOr(video->Mylene_release.Cb + c_offset, Cb_references_0,
		    Cb_references_1, line_size);
	constructOr(video->Mylene_release.Cr + c_offset, Cr_references_0,
		    Cr_references_1, line_size);
	video->idct_add_block(blocks + 4, pattern_code >> 8,
			      macroblock->mb_column << 3, line_size);
    }
}


static void
macroblock_3(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned c_offset, line_size, pattern_code, y_offset;

    MACROBLOCK_INIT(0);

    {
	unsigned c_half_flags, f = 1, y_half_flags;
	unsigned char *Cb_references_0, *Cb_references_1, *Cr_references_0,
	    *Cr_references_1, *Y_references_0, *Y_references_1;

	{
	    int scaled_vector_0_0_1, vector_0_0_0, vector_0_0_1;

	    DOWNLOAD_RELEASE(s_0, video, Iran);
	    DOWNLOAD_xMOTION_VECTOR(_0_0, macroblock, _0_0);
	    scaled_vector_0_0_1 = vector_0_0_1 & ~1;
	    {
		unsigned offset;

		offset = y_offset + (vector_0_0_0 >> 1)
		    + scaled_vector_0_0_1 * line_size;
		y_half_flags =
		    ((vector_0_0_0 & 1) | (vector_0_0_1 & 1) << 1) << 2;
		CHECK_MOTION_LIMITS(y_half_flags >> 02, 2, offset);
		Y_references_0 += offset;
	    }
	    vector_0_0_0 /= 2;
	    scaled_vector_0_0_1 = vector_0_0_1 & ~1;
	    {
		unsigned offset;

		offset = c_offset + (vector_0_0_0 >> 1)
		    + scaled_vector_0_0_1 * (line_size >> 1);
		Cb_references_0 += offset;
		Cr_references_0 += offset;
	    }
	    c_half_flags =
		((vector_0_0_0 & 1) | (vector_0_0_1 & 1) << 1) << 2;
	}
	for (; f < 3; f++) {
	    {
		int scaled_vector_x_0_1, vector_x_0_0, vector_x_0_1;

		DOWNLOAD_RELEASE(s_1, video, Iran);
		DOWNLOAD_xMOTION_VECTOR(_x_0, macroblock, _0_0);
		if (f == 1) {
		    if (vector_x_0_0 < 1) {
			vector_x_0_0 =
			    ((vector_x_0_0 * (int) video->m_1_0) >> 1)
			    + macroblock->dmvector_0;
		    } else {
			vector_x_0_0 =
			    ((vector_x_0_0 * (int) video->m_1_0 + 1) >> 1)
			    + macroblock->dmvector_0;
		    }
		    if (vector_x_0_1 < 1) {
			vector_x_0_1 =
			    ((vector_x_0_1 * (int) video->m_1_0) >> 1)
			    - 1 + macroblock->dmvector_1;
		    } else {
			vector_x_0_1 =
			    ((vector_x_0_1 * (int) video->m_1_0 + 1) >> 1)
			    - 1 + macroblock->dmvector_1;
		    }
		    scaled_vector_x_0_1 = vector_x_0_1 | 1;
		} else {
		    if (vector_x_0_0 < 1) {
			vector_x_0_0 =
			    ((vector_x_0_0 * (int) video->m_0_1) >> 1)
			    + macroblock->dmvector_0;
		    } else {
			vector_x_0_0 =
			    ((vector_x_0_0 * (int) video->m_0_1 + 1) >> 1)
			    + macroblock->dmvector_0;
		    }
		    if (vector_x_0_1 < 1) {
			vector_x_0_1 =
			    ((vector_x_0_1 * (int) video->m_0_1) >> 1)
			    + 1 + macroblock->dmvector_1;
		    } else {
			vector_x_0_1 =
			    ((vector_x_0_1 * (int) video->m_0_1 + 1) >> 1)
			    + 1 + macroblock->dmvector_1;
		    }
		    scaled_vector_x_0_1 = vector_x_0_1 & ~1;
		}
		{
		    unsigned offset;

		    offset = y_offset + (vector_x_0_0 >> 1)
			+ scaled_vector_x_0_1 * line_size;
		    y_half_flags = (y_half_flags & ~3)
			| (vector_x_0_0 & 1) | (vector_x_0_1 & 1) << 1;
		    CHECK_MOTION_LIMITS(y_half_flags & 0x3, 1, offset);
		    Y_references_1 += offset;
		}
		vector_x_0_0 /= 2;
		if (f == 1) {
		    scaled_vector_x_0_1 = vector_x_0_1 | 1;
		} else {
		    scaled_vector_x_0_1 = vector_x_0_1 & ~1;
		}
		{
		    unsigned offset;

		    offset = c_offset + (vector_x_0_0 >> 1)
			+ scaled_vector_x_0_1 * (line_size >> 1);
		    Cb_references_1 += offset;
		    Cr_references_1 += offset;
		}
		c_half_flags = (c_half_flags & ~3)
		    | (vector_x_0_0 & 1) | (vector_x_0_1 & 1) << 1;
	    }
	    {
		seek_h262_constructOr constructOr;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructOr = video->block_1_2[y_half_flags];
#else
		constructOr = video->block_1_2[0][y_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = video->Mylene_release.Y + y_offset;
		if (f == 2) {
		    Y_references_0 += line_size;
		    predictions += line_size;
		}
		constructOr(predictions, Y_references_0, Y_references_1,
			    line_size << 1);
	    }
	    {
		seek_h262_constructOr constructOr;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructOr = video->block_1_0[c_half_flags];
#else
		constructOr = video->block_1_0[0][c_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = video->Mylene_release.Cb + c_offset;
		if (f == 2) {
		    Cb_references_0 += line_size >> 1;
		    predictions += line_size >> 1;
		}
		constructOr
		    (predictions, Cb_references_0, Cb_references_1, line_size);
		predictions = video->Mylene_release.Cr + c_offset;
		if (f == 2) {
		    Cr_references_0 += line_size >> 1;
		    predictions += line_size >> 1;
		}
		constructOr
		    (predictions, Cr_references_0, Cr_references_1, line_size);
	    }
	}
    }

    video->idct_add_block(blocks, pattern_code & 255,
			  macroblock->mb_column << 4, line_size);
    video->idct_add_block(blocks + 4, pattern_code >> 8,
			  macroblock->mb_column << 3, line_size >> 1);
}


static void
macroblock_4(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;

    macroblock = (struct seek_h262_13818_macroblock *) video->macroblock;
    if (video->mb_width < macroblock->mb_column + 1) {
	return;
    }
    macroblock->previous_macroblock_address = macroblock->macroblock_address;
    {
	struct seek_h262_block *blocks;
	unsigned line_size, mb_column;

	blocks = video->blocks + 8;
	line_size = video->luminance_line_size;
	mb_column = macroblock->mb_column;
	video->idct_set_block(blocks, 4, mb_column << 4, line_size << 1);
	video->idct_set_block(blocks + 4, 4, mb_column << 3, line_size);
    }
}


static void
macroblock_5(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned c_offset, line_size, pattern_code, y_offset;

    MACROBLOCK_INIT(0);

    if (NON_BIDIRECTIONALLY_PREDICTIVE_MACROBLOCK(macroblock)) {
	unsigned f = 1;

	for (; f < 3; f++) {
	    unsigned c_half_flags, y_half_flags;
	    unsigned char *Cb_references, *Cr_references, *Y_references;

	    {
		int vector_x_x_0, vector_x_x_1;
		unsigned motion_vertical_field_select_x_x, offset;

		if (macroblock->packed_flags
		    & macroblock_motion_backward_mask) {
		    DOWNLOAD_RELEASE(s, video, Irulan);
		    if (f == 1) {
			DOwNLOAD_xMOTION_VECTOR(_x_x, macroblock, _0_1);
		    } else {
			DOwNLOAD_xMOTION_VECTOR(_x_x, macroblock, _1_1);
		    }
		} else {
		    DOWNLOAD_RELEASE(s, video, Iran);
		    if (f == 1) {
			DOwNLOAD_xMOTION_VECTOR(_x_x, macroblock, _0_0);
		    } else {
			DOwNLOAD_xMOTION_VECTOR(_x_x, macroblock, _1_0);
		    }
		}
		offset = y_offset + (vector_x_x_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_x, vector_x_x_1))
		    * line_size;
		SET_Y_REFERENCES(s, offset, y, _x_x);
		CHECK_MOTION_LIMITS(y_half_flags, 1, offset);
		vector_x_x_0 /= 2;
		offset = c_offset + (vector_x_x_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_x, vector_x_x_1))
		    * (line_size >> 1);
		SET_C_REFERENCES(s, offset, c, _x_x);
	    }
	    {
		seek_h262_constructor constructor;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructor = video->block_0_2[y_half_flags];
#else
		constructor = video->block_0_2[0][y_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = video->Mylene_release.Y + y_offset;
		if (f == 2) {
		    predictions += line_size;
		}
		constructor(predictions, Y_references, line_size << 1);
	    }
	    {
		seek_h262_constructor constructor;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructor = video->block_0_0[c_half_flags];
#else
		constructor = video->block_0_0[0][c_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = video->Mylene_release.Cb + c_offset;
		if (f == 2) {
		    predictions += line_size >> 1;
		}
		constructor(predictions, Cb_references, line_size);
		predictions = video->Mylene_release.Cr + c_offset;
		if (f == 2) {
		    predictions += line_size >> 1;
		}
		constructor(predictions, Cr_references, line_size);
	    }
	}
    } else {
	unsigned f = 1;

	for (; f < 3; f++) {
	    unsigned c_half_flags, y_half_flags;
	    unsigned char *Cb_references_0, *Cb_references_1, *Cr_references_0,
		*Cr_references_1, *Y_references_0, *Y_references_1;

	    {
		int vector_x_1_0, vector_x_1_1;
		unsigned motion_vertical_field_select_x_1, offset;

		DOWNLOAD_RELEASE(s_0, video, Irulan);
		if (f == 1) {
		    DOwNLOAD_xMOTION_VECTOR(_x_1, macroblock, _0_1);
		} else {
		    DOwNLOAD_xMOTION_VECTOR(_x_1, macroblock, _1_1);
		}
		offset = y_offset + (vector_x_1_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_1, vector_x_1_1))
		    * line_size;
		SET_Y_REFERENCeS(s_0, offset, y, _x_1);
		CHECK_MOTION_LIMITS(y_half_flags >> 02, 1, offset);
		vector_x_1_0 /= 2;
		offset = c_offset + (vector_x_1_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_1, vector_x_1_1))
		    * (line_size >> 1);
		SET_C_REFERENCeS(s_0, offset, c, _x_1);
	    }
	    {
		int vector_x_0_0, vector_x_0_1;
		unsigned motion_vertical_field_select_x_0, offset;

		DOWNLOAD_RELEASE(s_1, video, Iran);
		if (f == 1) {
		    DOwNLOAD_xMOTION_VECTOR(_x_0, macroblock, _0_0);
		} else {
		    DOwNLOAD_xMOTION_VECTOR(_x_0, macroblock, _1_0);
		}
		offset = y_offset + (vector_x_0_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_0, vector_x_0_1))
		    * line_size;
		SET_Y_REFEReNCES(s_1, offset, y, _x_0);
		CHECK_MOTION_LIMITS(y_half_flags & 0x3, 1, offset);
		vector_x_0_0 /= 2;
		offset = c_offset + (vector_x_0_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_0, vector_x_0_1))
		    * (line_size >> 1);
		SET_C_REFEReNCES(s_1, offset, c, _x_0);
	    }
	    {
		seek_h262_constructOr constructOr;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructOr = video->block_1_2[y_half_flags];
#else
		constructOr = video->block_1_2[0][y_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = video->Mylene_release.Y + y_offset;
		if (f == 2) {
		    predictions += line_size;
		}
		constructOr(predictions, Y_references_0, Y_references_1,
			    line_size << 1);
	    }
	    {
		seek_h262_constructOr constructOr;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructOr = video->block_1_1[c_half_flags];
#else
		constructOr = video->block_1_1[0][c_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = video->Mylene_release.Cb + c_offset;
		if (f == 2) {
		    predictions += line_size >> 1;
		}
		constructOr
		    (predictions, Cb_references_0, Cb_references_1, line_size);
		predictions = video->Mylene_release.Cr + c_offset;
		if (f == 2) {
		    predictions += line_size >> 1;
		}
		constructOr
		    (predictions, Cr_references_0, Cr_references_1, line_size);
	    }
	}
    }

    video->idct_add_block(blocks + 8, pattern_code & 255,
			  macroblock->mb_column << 4, line_size << 1);
    video->idct_add_block(blocks + 12, pattern_code >> 8,
			  macroblock->mb_column << 3, line_size);
}


static void
macroblock_6(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned c_offset, line_size, pattern_code, y_offset;

    MACROBLOCK_INIT(0);

    if (NON_BIDIRECTIONALLY_PREDICTIVE_MACROBLOCK(macroblock)) {
	seek_h262_constructor constructor;
	unsigned c_half_flags, y_half_flags;
	unsigned char *Cb_references, *Cr_references, *Y_references;

	{
	    int vector_0_x_0, vector_0_x_1;

	    if (macroblock->packed_flags & macroblock_motion_backward_mask) {
		DOWNLOAD_RELEASE(s, video, Irulan);
		DOWNLOAD_xMOTION_VECTOR(_0_x, macroblock, _0_1);
	    } else {
		DOWNLOAD_RELEASE(s, video, Iran);
		DOWNLOAD_xMOTION_VECTOR(_0_x, macroblock, _0_0);
	    }
	    if (vector_0_x_0 | vector_0_x_1) {
		unsigned offset;

		offset = y_offset + (vector_0_x_0 >> 1)
		    + (vector_0_x_1 >> 1) * line_size;
		SET_Y_REFERENCES(s, offset, y, _0_x);
		CHECK_MOTION_LIMITS(y_half_flags, 2, offset);
		vector_0_x_0 /= 2;
		offset = c_offset + (vector_0_x_0 >> 1)
		    + (vector_0_x_1 >> 1) * (line_size >> 1);
		SET_C_REFERENCES(s, offset, c, _0_x);
	    } else {
		SET_X_REFERENCES(s, c, y);
	    }
	}
#if !__Berkeley_FUCK_UP__
	constructor = video->block_0_3[y_half_flags];
#else
	constructor = video->block_0_3[0][y_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	constructor
	    (video->Mylene_release.Y + y_offset, Y_references, line_size);
	video->idct_add_block(blocks + 8, pattern_code & 255,
			      macroblock->mb_column << 4, line_size << 1);
#if !__Berkeley_FUCK_UP__
	constructor = video->block_0_1[c_half_flags];
#else
    	constructor = video->block_0_1[0][c_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	line_size >>= 1;
	constructor
	    (video->Mylene_release.Cb + c_offset, Cb_references, line_size);
	constructor
	    (video->Mylene_release.Cr + c_offset, Cr_references, line_size);
	video->idct_add_block(blocks + 12, pattern_code >> 8,
			      macroblock->mb_column << 3, line_size << 1);
    } else {
	seek_h262_constructOr constructOr;
	unsigned c_half_flags, y_half_flags;
	unsigned char *Cb_references_0, *Cb_references_1, *Cr_references_0,
	    *Cr_references_1, *Y_references_0, *Y_references_1;

	{
	    int vector_0_1_0, vector_0_1_1;

	    DOWNLOAD_RELEASE(s_0, video, Irulan);
	    DOWNLOAD_xMOTION_VECTOR(_0_1, macroblock, _0_1);
	    if (vector_0_1_0 | vector_0_1_1) {
		unsigned offset;

		offset = y_offset + (vector_0_1_0 >> 1)
		    + (vector_0_1_1 >> 1) * line_size;
		SET_Y_REFERENCeS(s_0, offset, y, _0_1);
		CHECK_MOTION_LIMITS(y_half_flags >> 02, 2, offset);
		vector_0_1_0 /= 2;
		offset = c_offset + (vector_0_1_0 >> 1)
		    + (vector_0_1_1 >> 1) * (line_size >> 1);
		SET_C_REFERENCeS(s_0, offset, c, _0_1);
	    } else {
		SET_X_REFERENCES(s_0, c, y);
	    }
	}
	{
	    int vector_0_0_0, vector_0_0_1;

	    DOWNLOAD_RELEASE(s_1, video, Iran);
	    DOWNLOAD_xMOTION_VECTOR(_0_0, macroblock, _0_0);
	    if (vector_0_0_0 | vector_0_0_1) {
		unsigned offset;

		offset = y_offset + (vector_0_0_0 >> 1)
		    + (vector_0_0_1 >> 1) * line_size;
		SET_Y_REFEReNCES(s_1, offset, y, _0_0);
		CHECK_MOTION_LIMITS(y_half_flags & 0x3, 2, offset);
		vector_0_0_0 /= 2;
		offset = c_offset + (vector_0_0_0 >> 1)
		    + (vector_0_0_1 >> 1) * (line_size >> 1);
		SET_C_REFEReNCES(s_1, offset, c, _0_0);
	    } else {
		SET_X_REFEReNCES(s_1, c, y);
	    }
	}
#if !__Berkeley_FUCK_UP__
	constructOr = video->block_1_3[y_half_flags];
#else
	constructOr = video->block_1_3[0][y_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	constructOr(video->Mylene_release.Y + y_offset, Y_references_0,
		    Y_references_1, line_size);
	video->idct_add_block(blocks + 8, pattern_code & 255,
			      macroblock->mb_column << 4, line_size << 1);
#if !__Berkeley_FUCK_UP__
	constructOr = video->block_1_1[c_half_flags];
#else
    	constructOr = video->block_1_1[0][c_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	line_size >>= 1;
	constructOr(video->Mylene_release.Cb + c_offset, Cb_references_0,
		    Cb_references_1, line_size);
	constructOr(video->Mylene_release.Cr + c_offset, Cr_references_0,
		    Cr_references_1, line_size);
	video->idct_add_block(blocks + 12, pattern_code >> 8,
			      macroblock->mb_column << 3, line_size << 1);
    }
}


static void
macroblock_7(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned c_offset, line_size, pattern_code, y_offset;

    MACROBLOCK_INIT(0);

    {
	unsigned c_half_flags, f = 1, y_half_flags;
	unsigned char *Cb_references_0, *Cb_references_1, *Cr_references_0,
	    *Cr_references_1, *Y_references_0, *Y_references_1;

	{
	    int scaled_vector_0_0_1, vector_0_0_0, vector_0_0_1;

	    DOWNLOAD_RELEASE(s_0, video, Iran);
	    DOWNLOAD_xMOTION_VECTOR(_0_0, macroblock, _0_0);
	    scaled_vector_0_0_1 = vector_0_0_1 & ~1;
	    {
		unsigned offset;

		offset = y_offset + (vector_0_0_0 >> 1)
		    + scaled_vector_0_0_1 * line_size;
		y_half_flags =
		    ((vector_0_0_0 & 1) | (vector_0_0_1 & 1) << 1) << 2;
		CHECK_MOTION_LIMITS(y_half_flags >> 02, 2, offset);
		Y_references_0 += offset;
	    }
	    vector_0_0_0 /= 2;
	    {
		unsigned offset;

		offset = c_offset + (vector_0_0_0 >> 1)
		    + scaled_vector_0_0_1 * (line_size >> 1);
		Cb_references_0 += offset;
		Cr_references_0 += offset;
	    }
	    c_half_flags =
		((vector_0_0_0 & 1) | (vector_0_0_1 & 1) << 1) << 2;
	}
	for (; f < 3; f++) {
	    {
		int scaled_vector_x_0_1, vector_x_0_0, vector_x_0_1;

		DOWNLOAD_RELEASE(s_1, video, Iran);
		DOWNLOAD_xMOTION_VECTOR(_x_0, macroblock, _0_0);
		if (f == 1) {
		    if (vector_x_0_0 < 1) {
			vector_x_0_0 =
			    ((vector_x_0_0 * (int) video->m_1_0) >> 1)
			    + macroblock->dmvector_0;
		    } else {
			vector_x_0_0 =
			    ((vector_x_0_0 * (int) video->m_1_0 + 1) >> 1)
			    + macroblock->dmvector_0;
		    }
		    if (vector_x_0_1 < 1) {
			vector_x_0_1 =
			    ((vector_x_0_1 * (int) video->m_1_0) >> 1)
			    - 1 + macroblock->dmvector_1;
		    } else {
			vector_x_0_1 =
			    ((vector_x_0_1 * (int) video->m_1_0 + 1) >> 1)
			    - 1 + macroblock->dmvector_1;
		    }
		    scaled_vector_x_0_1 = vector_x_0_1 | 1;
		} else {
		    if (vector_x_0_0 < 1) {
			vector_x_0_0 =
			    ((vector_x_0_0 * (int) video->m_0_1) >> 1)
			    + macroblock->dmvector_0;
		    } else {
			vector_x_0_0 =
			    ((vector_x_0_0 * (int) video->m_0_1 + 1) >> 1)
			    + macroblock->dmvector_0;
		    }
		    if (vector_x_0_1 < 1) {
			vector_x_0_1 =
			    ((vector_x_0_1 * (int) video->m_0_1) >> 1)
			    + 1 + macroblock->dmvector_1;
		    } else {
			vector_x_0_1 =
			    ((vector_x_0_1 * (int) video->m_0_1 + 1) >> 1)
			    + 1 + macroblock->dmvector_1;
		    }
		    scaled_vector_x_0_1 = vector_x_0_1 & ~1;
		}
		{
		    unsigned offset;

		    offset = y_offset + (vector_x_0_0 >> 1)
			+ scaled_vector_x_0_1 * line_size;
		    y_half_flags = (y_half_flags & ~3)
			| (vector_x_0_0 & 1) | (vector_x_0_1 & 1) << 1;
		    CHECK_MOTION_LIMITS(y_half_flags & 0x3, 1, offset);
		    Y_references_1 += offset;
		}
		vector_x_0_0 /= 2;
		{
		    unsigned offset;

		    offset = c_offset + (vector_x_0_0 >> 1)
			+ scaled_vector_x_0_1 * (line_size >> 1);
		    Cb_references_1 += offset;
		    Cr_references_1 += offset;
		}
		c_half_flags = (c_half_flags & ~3)
		    | (vector_x_0_0 & 1) | (vector_x_0_1 & 1) << 1;
	    }
	    {
		seek_h262_constructOr constructOr;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructOr = video->block_1_2[y_half_flags];
#else
		constructOr = video->block_1_2[0][y_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = video->Mylene_release.Y + y_offset;
		if (f == 2) {
		    Y_references_0 += line_size;
		    predictions += line_size;
		}
		constructOr(predictions, Y_references_0, Y_references_1,
			    line_size << 1);
	    }
	    {
		seek_h262_constructOr constructOr;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructOr = video->block_1_0[c_half_flags];
#else
		constructOr = video->block_1_0[0][c_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = video->Mylene_release.Cb + c_offset;
		if (f == 2) {
		    Cb_references_0 += line_size >> 1;
		    predictions += line_size >> 1;
		}
		constructOr
		    (predictions, Cb_references_0, Cb_references_1, line_size);
		predictions = video->Mylene_release.Cr + c_offset;
		if (f == 2) {
		    Cr_references_0 += line_size >> 1;
		    predictions += line_size >> 1;
		}
		constructOr
		    (predictions, Cr_references_0, Cr_references_1, line_size);
	    }
	}
    }

    video->idct_add_block(blocks + 8, pattern_code & 255,
			  macroblock->mb_column << 4, line_size << 1);
    video->idct_add_block(blocks + 12, pattern_code >> 8,
			  macroblock->mb_column << 3, line_size);
}


static void
macroblock_8(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;

    macroblock = (struct seek_h262_13818_macroblock *) video->macroblock;
    if (video->mb_width < macroblock->mb_column + 1) {
	return;
    }
    macroblock->previous_macroblock_address = macroblock->macroblock_address;
    {
	struct seek_h262_block *blocks;
	unsigned line_size, mb_column;

	blocks = video->blocks;
	line_size = video->luminance_line_size;
	mb_column = macroblock->mb_column;
	video->idct_set_block(blocks, 4, mb_column << 4, line_size << 1);
	video->idct_set_block(blocks + 4, 4, mb_column << 3, line_size);
    }
}


static void
macroblock_9(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned c_offset, line_size, pattern_code, y_offset;

    MACROBLOCK_INIT(1);

    if (NON_BIDIRECTIONALLY_PREDICTIVE_MACROBLOCK(macroblock)) {
	seek_h262_constructor constructor;
	unsigned c_half_flags, y_half_flags;
	unsigned char *Cb_references, *Cr_references, *Y_references;

	{
	    int vector_0_x_0, vector_0_x_1;
	    unsigned motion_vertical_field_select_0_x;

	    if (macroblock->packed_flags & macroblock_motion_backward_mask) {
		DOWNLOAD_RELEASE(s, video, Irulan);
		DOwNLOAD_xMOTION_VECTOR(_0_x, macroblock, _0_1);
	    } else {
		DOwNLOAD_xMOTION_VECTOR(_0_x, macroblock, _0_0);
		DOWNLOAD_ReLEASE(s, video, motion_vertical_field_select_0_x);
	    }
	    if (vector_0_x_0 | vector_0_x_1) {
		unsigned offset;

		offset = y_offset + (vector_0_x_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_0_x, vector_0_x_1))
		    * line_size;
		SET_Y_REFERENCES(s, offset, y, _0_x);
		CHECK_MOTION_LIMITS(y_half_flags, 9, offset);
		vector_0_x_0 /= 2;
		offset = c_offset + (vector_0_x_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_0_x, vector_0_x_1))
		    * (line_size >> 1);
		SET_C_REFERENCES(s, offset, c, _0_x);
	    } else {
		SET_X_ReFERENCES
		    (s, c, y, line_size, motion_vertical_field_select_0_x);
	    }
	}
#if !__Berkeley_FUCK_UP__
	constructor = video->block_0_3[y_half_flags];
#else
	constructor = video->block_0_3[0][y_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	constructor(blocks[0].base_address + (macroblock->mb_column << 4),
		    Y_references, line_size << 1);
	video->idct_add_block(blocks, pattern_code & 255,
			      macroblock->mb_column << 4, line_size << 1);
#if !__Berkeley_FUCK_UP__
	constructor = video->block_0_1[c_half_flags];
#else
    	constructor = video->block_0_1[0][c_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	constructor(blocks[4].base_address + (macroblock->mb_column << 3),
		    Cb_references, line_size);
	constructor(blocks[5].base_address + (macroblock->mb_column << 3),
		    Cr_references, line_size);
	video->idct_add_block(blocks + 4, pattern_code >> 8,
			      macroblock->mb_column << 3, line_size);
    } else {
	seek_h262_constructOr constructOr;
	unsigned c_half_flags, y_half_flags;
	unsigned char *Cb_references_0, *Cb_references_1, *Cr_references_0,
	    *Cr_references_1, *Y_references_0, *Y_references_1;

	{
	    int vector_0_1_0, vector_0_1_1;

	    DOWNLOAD_RELEASE(s_0, video, Irulan);
	    DOWNLOAD_xMOTION_VECTOR(_0_1, macroblock, _0_1);
	    if (vector_0_1_0 | vector_0_1_1) {
		unsigned offset;

		offset = y_offset + (vector_0_1_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (macroblock->motion_vertical_field_select_0_1,
			vector_0_1_1))
		    * line_size;
		SET_Y_REFERENCeS(s_0, offset, y, _0_1);
		CHECK_MOTION_LIMITS(y_half_flags >> 02, 9, offset);
		vector_0_1_0 /= 2;
		offset = c_offset + (vector_0_1_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (macroblock->motion_vertical_field_select_0_1,
			vector_0_1_1))
		    * (line_size >> 1);
		SET_C_REFERENCeS(s_0, offset, c, _0_1);
	    } else {
		SET_X_ReFERENCES(s_0, c, y, line_size,
				 macroblock->motion_vertical_field_select_0_1);
	    }
	}
	{
	    int vector_0_0_0, vector_0_0_1;

	    DOWNLOAD_RELEASE(s_1, video, Iran);
	    DOWNLOAD_xMOTION_VECTOR(_0_0, macroblock, _0_0);
	    if (vector_0_0_0 | vector_0_0_1) {
		unsigned offset;

		offset = y_offset + (vector_0_0_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (macroblock->motion_vertical_field_select_0_0,
			vector_0_0_1))
		    * line_size;
		SET_Y_REFEReNCES(s_1, offset, y, _0_0);
		CHECK_MOTION_LIMITS(y_half_flags & 0x3, 9, offset);
		vector_0_0_0 /= 2;
		offset = c_offset + (vector_0_0_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (macroblock->motion_vertical_field_select_0_0,
			vector_0_0_1))
		    * (line_size >> 1);
		SET_C_REFEReNCES(s_1, offset, c, _0_0);
	    } else {
		SET_X_ReFEReNCES(s_1, c, y, line_size,
				 macroblock->motion_vertical_field_select_0_0);
	    }
	}
#if !__Berkeley_FUCK_UP__
	constructOr = video->block_1_3[y_half_flags];
#else
	constructOr = video->block_1_3[0][y_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	constructOr(blocks[0].base_address + (macroblock->mb_column << 4),
		    Y_references_0, Y_references_1, line_size << 1);
	video->idct_add_block(blocks, pattern_code & 255,
			      macroblock->mb_column << 4, line_size << 1);
#if !__Berkeley_FUCK_UP__
	constructOr = video->block_1_1[c_half_flags];
#else
    	constructOr = video->block_1_1[0][c_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	constructOr(blocks[4].base_address + (macroblock->mb_column << 3),
		    Cb_references_0, Cb_references_1, line_size);
	constructOr(blocks[5].base_address + (macroblock->mb_column << 3),
		    Cr_references_0, Cr_references_1, line_size);
	video->idct_add_block(blocks + 4, pattern_code >> 8,
			      macroblock->mb_column << 3, line_size);
    }
}


static void
macroblock_a(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned c_offset, line_size, pattern_code, y_offset;

    MACROBLOCK_INIT(1);

    if (NON_BIDIRECTIONALLY_PREDICTIVE_MACROBLOCK(macroblock)) {
	unsigned f = 1;

	for (; f < 3; f++) {
	    unsigned c_half_flags, y_half_flags;
	    unsigned char *Cb_references, *Cr_references, *Y_references;

	    {
		int vector_x_x_0, vector_x_x_1;
		unsigned motion_vertical_field_select_x_x, offset;

		if (macroblock->packed_flags
		    & macroblock_motion_backward_mask) {
		    DOWNLOAD_RELEASE(s, video, Irulan);
		    if (f == 1) {
			DOwNLOAD_xMOTION_VECTOR(_x_x, macroblock, _0_1);
		    } else {
			DOwNLOAD_xMOTION_VECTOR(_x_x, macroblock, _1_1);
		    }
		} else {
		    if (f == 1) {
			DOwNLOAD_xMOTION_VECTOR(_x_x, macroblock, _0_0);
		    } else {
			DOwNLOAD_xMOTION_VECTOR(_x_x, macroblock, _1_0);
		    }
		    DOWNLOAD_ReLEASE
			(s, video, motion_vertical_field_select_x_x);
		}
		offset = y_offset + (vector_x_x_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_x, vector_x_x_1))
		    * line_size;
		if (f == 2) {
		    offset += line_size << 4;
		}
		SET_Y_REFERENCES(s, offset, y, _x_x);
		CHECK_MOTION_LIMITS(y_half_flags, a, offset);
		vector_x_x_0 /= 2;
		offset = c_offset + (vector_x_x_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_x, vector_x_x_1))
		    * (line_size >> 1);
		if (f == 2) {
		    offset += line_size << 3;
		}
		SET_C_REFERENCES(s, offset, c, _x_x);
	    }
	    {
		seek_h262_constructor constructor;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructor = video->block_0_2[y_half_flags];
#else
		constructor = video->block_0_2[0][y_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = blocks[0].base_address
		    + (macroblock->mb_column << 4);
		if (f == 2) {
		    predictions += line_size << 4;
		}
		constructor(predictions, Y_references, line_size << 1);
	    }
	    {
		seek_h262_constructor constructor;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructor = video->block_0_0[c_half_flags];
#else
		constructor = video->block_0_0[0][c_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = blocks[4].base_address
		    + (macroblock->mb_column << 3);
		if (f == 2) {
		    predictions += line_size << 3;
		}
		constructor(predictions, Cb_references, line_size);
		predictions = blocks[5].base_address
		    + (macroblock->mb_column << 3);
		if (f == 2) {
		    predictions += line_size << 3;
		}
		constructor(predictions, Cr_references, line_size);
	    }
	}
    } else {
	unsigned f = 1;

	for (; f < 3; f++) {
	    unsigned c_half_flags, y_half_flags;
	    unsigned char *Cb_references_0, *Cb_references_1, *Cr_references_0,
		*Cr_references_1, *Y_references_0, *Y_references_1;

	    {
		int vector_x_1_0, vector_x_1_1;
		unsigned motion_vertical_field_select_x_1, offset;

		DOWNLOAD_RELEASE(s_0, video, Irulan);
		if (f == 1) {
		    DOwNLOAD_xMOTION_VECTOR(_x_1, macroblock, _0_1);
		} else {
		    DOwNLOAD_xMOTION_VECTOR(_x_1, macroblock, _1_1);
		}
		offset = y_offset + (vector_x_1_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_1, vector_x_1_1))
		    * line_size;
		if (f == 2) {
		    offset += line_size << 4;
		}
		SET_Y_REFERENCeS(s_0, offset, y, _x_1);
		CHECK_MOTION_LIMITS(y_half_flags >> 02, a, offset);
		vector_x_1_0 /= 2;
		offset = c_offset + (vector_x_1_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_1, vector_x_1_1))
		    * (line_size >> 1);
		if (f == 2) {
		    offset += line_size << 3;
		}
		SET_C_REFERENCeS(s_0, offset, c, _x_1);
	    }
	    {
		int vector_x_0_0, vector_x_0_1;
		unsigned motion_vertical_field_select_x_0, offset;

		DOWNLOAD_RELEASE(s_1, video, Iran);
		if (f == 1) {
		    DOwNLOAD_xMOTION_VECTOR(_x_0, macroblock, _0_0);
		} else {
		    DOwNLOAD_xMOTION_VECTOR(_x_0, macroblock, _1_0);
		}
		offset = y_offset + (vector_x_0_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_0, vector_x_0_1))
		    * line_size;
		if (f == 2) {
		    offset += line_size << 4;
		}
		SET_Y_REFEReNCES(s_1, offset, y, _x_0);
		CHECK_MOTION_LIMITS(y_half_flags & 0x3, a, offset);
		vector_x_0_0 /= 2;
		offset = c_offset + (vector_x_0_0 >> 1)
		    + (VERTICAL_FIELD_CORRECT
		       (motion_vertical_field_select_x_0, vector_x_0_1))
		    * (line_size >> 1);
		if (f == 2) {
		    offset += line_size << 3;
		}
		SET_C_REFEReNCES(s_1, offset, c, _x_0);
	    }
	    {
		seek_h262_constructOr constructOr;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructOr = video->block_1_2[y_half_flags];
#else
		constructOr = video->block_1_2[0][y_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = blocks[0].base_address
		    + (macroblock->mb_column << 4);
		if (f == 2) {
		    predictions += line_size << 4;
		}
		constructOr(predictions, Y_references_0, Y_references_1,
			    line_size << 1);
	    }
	    {
		seek_h262_constructOr constructOr;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructOr = video->block_1_0[c_half_flags];
#else
		constructOr = video->block_1_0[0][c_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = blocks[4].base_address
		    + (macroblock->mb_column << 3);
		if (f == 2) {
		    predictions += line_size << 3;
		}
		constructOr
		    (predictions, Cb_references_0, Cb_references_1, line_size);
		predictions = blocks[5].base_address
		    + (macroblock->mb_column << 3);
		if (f == 2) {
		    predictions += line_size << 3;
		}
		constructOr
		    (predictions, Cr_references_0, Cr_references_1, line_size);
	    }
	}
    }

    video->idct_add_block(blocks, pattern_code & 255,
			  macroblock->mb_column << 4, line_size << 1);
    video->idct_add_block(blocks + 4, pattern_code >> 8,
			  macroblock->mb_column << 3, line_size);
}


static void
macroblock_b(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned c_offset, line_size, pattern_code, y_offset;

    MACROBLOCK_INIT(1);

    {
	unsigned c_half_flags, y_half_flags;
	unsigned char *Cb_references_0, *Cb_references_1, *Cr_references_0,
	    *Cr_references_1, *Y_references_0, *Y_references_1;

	{
	    int scaled_vector_0_0_1, vector_0_0_0, vector_0_0_1;

	    DOWNLOAD_RELEASE(s_0, video, Iran);
	    DOWNLOAD_xMOTION_VECTOR(_0_0, macroblock, _0_0);
	    if (video->picture_structure == Top_Field) {
		scaled_vector_0_0_1 = vector_0_0_1 & ~1;
	    } else {
		scaled_vector_0_0_1 = vector_0_0_1 | 1;
	    }
	    {
		unsigned offset;

		offset = y_offset + (vector_0_0_0 >> 1)
		    + scaled_vector_0_0_1 * line_size;
		y_half_flags =
		    ((vector_0_0_0 & 1) | (vector_0_0_1 & 1) << 1) << 2;
		CHECK_MOTION_LIMITS(y_half_flags >> 02, 9, offset);
		Y_references_0 += offset;
	    }
	    vector_0_0_0 /= 2;
	    {
		unsigned offset;

		offset = c_offset + (vector_0_0_0 >> 1)
		    + scaled_vector_0_0_1 * (line_size >> 1);
		Cb_references_0 += offset;
		Cr_references_0 += offset;
	    }
	    c_half_flags =
		((vector_0_0_0 & 1) | (vector_0_0_1 & 1) << 1) << 2;
	}
	{
	    int scaled_vector_x_0_1, vector_x_0_0, vector_x_0_1;

	    DOWNLOAD_RELEASE(s_1, video, Irulan);
	    DOWNLOAD_xMOTION_VECTOR(_x_0, macroblock, _0_0);
	    if (vector_x_0_0 < 1) {
		vector_x_0_0 = ((vector_x_0_0) >> 1)
		    + macroblock->dmvector_0;
	    } else {
		vector_x_0_0 = ((vector_x_0_0 + 1) >> 1)
		    + macroblock->dmvector_0;
	    }
	    if (video->picture_structure == Top_Field) {
		if (vector_x_0_1 < 1) {
		    vector_x_0_1 = ((vector_x_0_1) >> 1)
			- 1 + macroblock->dmvector_1;
		} else {
		    vector_x_0_1 = ((vector_x_0_1 + 1) >> 1)
			- 1 + macroblock->dmvector_1;
		}
		scaled_vector_x_0_1 = vector_x_0_1 | 1;
	    } else {
		if (vector_x_0_1 < 1) {
		    vector_x_0_1 = ((vector_x_0_1) >> 1)
			+ 1 + macroblock->dmvector_1;
		} else {
		    vector_x_0_1 = ((vector_x_0_1 + 1) >> 1)
			+ 1 + macroblock->dmvector_1;
		}
		scaled_vector_x_0_1 = vector_x_0_1 & ~1;
	    }
	    {
		unsigned offset;

		offset = y_offset + (vector_x_0_0 >> 1)
		    + scaled_vector_x_0_1 * line_size;
		y_half_flags |=
		    (vector_x_0_0 & 1) | (vector_x_0_1 & 1) << 1;
		CHECK_MOTION_LIMITS(y_half_flags & 0x3, 9, offset);
		Y_references_1 += offset;
	    }
	    vector_x_0_0 /= 2;
	    {
		unsigned offset;

		offset = c_offset + (vector_x_0_0 >> 1)
		    + scaled_vector_x_0_1 * (line_size >> 1);
		Cb_references_1 += offset;
		Cr_references_1 += offset;
	    }
	    c_half_flags |=
		(vector_x_0_0 & 1) | (vector_x_0_1 & 1) << 1;
	}
	{
	    seek_h262_constructOr constructOr;

#if !__Berkeley_FUCK_UP__
	    constructOr = video->block_1_3[y_half_flags];
#else
	    constructOr = video->block_1_3[0][y_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	    constructOr(blocks[0].base_address + (macroblock->mb_column << 4),
			Y_references_0, Y_references_1, line_size << 1);
	}
	{
	    seek_h262_constructOr constructOr;

#if !__Berkeley_FUCK_UP__
	    constructOr = video->block_1_1[c_half_flags];
#else
	    constructOr = video->block_1_1[0][c_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	    constructOr(blocks[4].base_address + (macroblock->mb_column << 3),
			Cb_references_0, Cb_references_1, line_size);
	    constructOr(blocks[5].base_address + (macroblock->mb_column << 3),
			Cr_references_0, Cr_references_1, line_size);
	}
    }

    video->idct_add_block(blocks, pattern_code & 255,
			  macroblock->mb_column << 4, line_size << 1);
    video->idct_add_block(blocks + 4, pattern_code >> 8,
			  macroblock->mb_column << 3, line_size);
}
