/*
 * h262.7.part3.c
 * Copyright (C) 2003, 2004  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.1.h>
#include <h262.3.h>
#include <h262.a.h>

void
_seek_h262_changed_sequence_extension(struct seek_h262_video *video)
{
    unsigned char *sequence_extension;

    sequence_extension = video->buffer;
    if ((sequence_extension[0] & 15) != (video->sequence_extension[0] & 15)
	|| ((sequence_extension[1] & 240)
	    != (video->sequence_extension[1] & 240))) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_PROFILE_AND_LEVEL_INDICATION);
    }
    if ((sequence_extension[1] & 8) != (video->sequence_extension[1] & 8)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_PROGRESSIVE_SEQUENCE);
    }
    if ((sequence_extension[1] & 6) != (video->sequence_extension[1] & 6)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_CHROMA_FORMAT);
    }
    if ((sequence_extension[1] & 1) != (video->sequence_extension[1] & 1)
	|| ((sequence_extension[2] & 128)
	    != (video->sequence_extension[2] & 128))) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_HORIZONTAL_SIZE_EXTENSION);
    }
    if ((sequence_extension[2] & 96) != (video->sequence_extension[2] & 96)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_VERTICAL_SIZE_EXTENSION);
    }
    if ((sequence_extension[2] & 31) != (video->sequence_extension[2] & 31)
	|| ((sequence_extension[3] & 254)
	    != (video->sequence_extension[3] & 254))) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_BIT_RATE_EXTENSION);
    }
    if ((sequence_extension[3] & 1) != (video->sequence_extension[3] & 1)) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_UNSET_MARKER_BIT);
    }
    if (sequence_extension[4] != video->sequence_extension[4]) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_VBV_BUFFER_SIZE_EXTENSION);
    }
    if ((sequence_extension[5] & 128)
	!= (video->sequence_extension[5] & 128)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_LOW_DELAY);
    }
    if ((sequence_extension[5] & 96) != (video->sequence_extension[5] & 96)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_FRAME_RATE_EXTENSION_N);
    }
    if ((sequence_extension[5] & 31) != (video->sequence_extension[5] & 31)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_FRAME_RATE_EXTENSION_D);
    }
}


void
_seek_h262_changed_sequence_header(struct seek_h262_video *video)
{
    unsigned *sequence_header;

    sequence_header = (unsigned *) video->buffer;
#if WORDS_BIGENDIAN
    if ((sequence_header[0] & 0xfff00000)
	!= (video->sequence_header[0] & 0xfff00000)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_HORIZONTAL_SIZE_VALUE);
    }
    if ((sequence_header[0] & 0x000fff00)
	!= (video->sequence_header[0] & 0x000fff00)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_VERTICAL_SIZE_VALUE);
    }
    if ((sequence_header[0] & 0x000000f0)
	!= (video->sequence_header[0] & 0x000000f0)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_ASPECT_RATIO_INFORMATION);
    }
    if ((sequence_header[0] & 0x0000000f)
	!= (video->sequence_header[0] & 0x0000000f)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_FRAME_RATE_CODE);
    }
    if ((sequence_header[1] & 0xffffc000)
	!= (video->sequence_header[1] & 0xffffc000)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_BIT_RATE_VALUE);
    }
    if ((sequence_header[1] & 0x00002000)
	!= (video->sequence_header[1] & 0x00002000)) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_UNSET_MARKER_BIT);
    }
    if ((sequence_header[1] & 0x00001ff8)
	!= (video->sequence_header[1] & 0x00001ff8)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_VBV_BUFFER_SIZE_VALUE);
    }
    if ((sequence_header[1] & 0x00000004)
	!= (video->sequence_header[1] & 0x00000004)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_CONSTRAINED_PARAMETERS_FLAG);
    }
#else
    if ((sequence_header[0] & 0x0000f0ff)
	!= (video->sequence_header[0] & 0x0000f0ff)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_HORIZONTAL_SIZE_VALUE);
    }
    if ((sequence_header[0] & 0x00ff0f00)
	!= (video->sequence_header[0] & 0x00ff0f00)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_VERTICAL_SIZE_VALUE);
    }
    if ((sequence_header[0] & 0xf0000000)
	!= (video->sequence_header[0] & 0xf0000000)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_ASPECT_RATIO_INFORMATION);
    }
    if ((sequence_header[0] & 0x0f000000)
	!= (video->sequence_header[0] & 0x0f000000)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_FRAME_RATE_CODE);
    }
    if ((sequence_header[1] & 0x00c0ffff)
	!= (video->sequence_header[1] & 0x00c0ffff)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_BIT_RATE_VALUE);
    }
    if ((sequence_header[1] & 0x00200000)
	!= (video->sequence_header[1] & 0x00200000)) {
	_seek_h262_abort_decoding
	    ((struct seek_h262_whatever *) video, VIDEO_UNSET_MARKER_BIT);
    }
    if ((sequence_header[1] & 0xf81f0000)
	!= (video->sequence_header[1] & 0xf81f0000)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_VBV_BUFFER_SIZE_VALUE);
    }
    if ((sequence_header[1] & 0x04000000)
	!= (video->sequence_header[1] & 0x04000000)) {
	_seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				  VIDEO_CHANGED_CONSTRAINED_PARAMETERS_FLAG);
    }
#endif				/* WORDS_BIGENDIAN */
}
