/*
 * h262.9.11172.c
 * Copyright (C) 2003, 2004, 2005  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.1.h>
#include <h262.2.h>
#include <h262.3.h>
#include <h262.5.h>
#include <h262.a.h>
#include <h262.d.l7.h>

#define ABORT_DECODING(video, buffer, size, error) \
    {									      \
	_seek_h262_abort_divergent_decoding				      \
	    ((struct seek_h262_whatever *) video, buffer, size, error);	      \
    }

#if __ENABLE_X000001AF___HACK_0__
# define MAMAN_A_TORT(video) \
    (picture_start_code < video->start_code				      \
     && video->start_code < video->break_code)
#elif WORDS_BIGENDIAN
# define MAMAN_A_TORT(video) \
    (picture_start_code < video->start_code && video->start_code < 0x000001b0)
#else
# define MAMAN_A_TORT(video) \
    (picture_start_code < video->start_code && video->start_code < 0xb0010000)
#endif				/* __ENABLE_X000001AF___HACK_0__ */

extern unsigned char _seek_h262_B_1_01[], _seek_h262_B_5a[], _seek_h262_B_5b[];

#if !__INLINE_OPTIMISTIQUE_MOI__
static void optimistique_moi(struct seek_h262_video *, unsigned char **,
			     unsigned *);
#endif				/* !__INLINE_OPTIMISTIQUE_MOI__ */

#if !__INLINE_OPTIMISTIQUE_MOI__
static void
optimistique_moi(struct seek_h262_video *video, unsigned char **buffer,
		 unsigned *size)
{
    video->buffer = *buffer;
    video->size = *size;
    video->optimistique_moi(video);
    *buffer = video->buffer;
    *size = video->size;
}
#endif				/* !__INLINE_OPTIMISTIQUE_MOI__ */


void
_seek_h262___________x20_11172_slices(struct seek_h262_video *video)
{
    do {
	unsigned cache_buffer, cache_size = 27, size, slice_address = 0;
	unsigned char *buffer;

	buffer = video->buffer;
	size = video->size;
	OPTIMISTIQUE_MOI_X(4);
	cache_buffer =
	    *buffer << 24 | buffer[1] << 16 | buffer[2] << 8 | buffer[3];
	buffer += 4;
	size -= 4;
	{
	    if (!(cache_buffer >> 27)) {
		ABORT_DECODING
		    (video, buffer, size, VIDEO_NULL_SLICE_QUANTISER_SCALE);
	    }
	    if (cache_buffer >> (cache_size - 1) & 1) {
		do {
		    cache_size -= 9;
		    if (cache_size < 9) {
			OPTIMISTIQUE_MOI_X(2);
			cache_buffer =
			    cache_buffer << 16 | *buffer << 8 | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
#if __ENABLE_X000001AF___HACK_3__
		    if (video->x000001af___hack_3) {
			struct seek_h262_x000001af__ slice;

			slice.decoder = video;
			slice.x.x2.extra_information_slice =
			    cache_buffer >> cache_size & 255;

			video->x000001af___hack_3(&slice);
		    }
#endif				/* __ENABLE_X000001AF___HACK_0__ */
		} while (cache_buffer >> (cache_size - 1) & 1);
	    }
	    cache_size--;
	}
	do {
	    {
		unsigned offset;

		while (1) {
		    if (cache_size < 11) {
			OPTIMISTIQUE_MOI_X(2);
			cache_buffer =
			    cache_buffer << 16 | *buffer << 8 | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
		    offset = cache_buffer >> (cache_size - 11) & 2047;
		    if (offset == 15) {
			cache_size -= 11;
		    } else {
			break;
		    }
		}
		if (offset & 1024) {
		    cache_size--;
#if 0
		} else if (slice_address) {
		    ABORT_DECODING(video, buffer, size,
				   VIDEO_INVALID_MACROBLOCK_ADDRESS_INCREMENT);
#endif				/* 0 */
		} else {
		    while (offset == 8) {
			cache_size -= 11;
			if (cache_size < 11) {
			    OPTIMISTIQUE_MOI_X(2);
			    cache_buffer =
				cache_buffer << 16 | *buffer << 8 | buffer[1];
			    buffer += 2;
			    cache_size += 16;
			    size -= 2;
			}
			offset = cache_buffer >> (cache_size - 11) & 2047;
		    }
		    cache_size -=
			((unsigned short *) _seek_h262_B_1_01)[offset] >> 8;
		}
	    }
	    slice_address = ~0;

	    OPTIMISTIQUE_MOI_c(11172);

	    if (!cache_size) {
		cache_buffer = *buffer++;
		cache_size = 8;
		size--;
	    }
	    cache_size--;
	    if (!(cache_buffer >> cache_size & 1)) {
		ABORT_DECODING
		    (video, buffer, size, VIDEO_INVALID_MACROBLOCK_TYPE);
	    }
	    {
		unsigned chroma_format = 6;

		for (; chroma_format; chroma_format--) {
		    if (cache_size < 16) {
			cache_buffer =
			    cache_buffer << 16 | *buffer << 8 | buffer[1];
			buffer += 2;
			cache_size += 16;
			size -= 2;
		    }
		    if (chroma_format < 3) {
#if WORDS_BIGENDIAN
			cache_size -= _seek_h262_B_5b
			    [(cache_buffer >> (cache_size - 8) & 254) + 1]
			    & 127;
#else
			cache_size -= _seek_h262_B_5b
			    [(cache_buffer >> (cache_size - 8) & 254) + 0]
			    & 127;
#endif				/* WORDS_BIGENDIAN */
		    } else {
#if WORDS_BIGENDIAN
			cache_size -= _seek_h262_B_5a
			    [(cache_buffer >> (cache_size - 7) & 126) + 1]
			    & 127;
#else
			cache_size -= _seek_h262_B_5a
			    [(cache_buffer >> (cache_size - 7) & 126) + 0]
			    & 127;
#endif				/* WORDS_BIGENDIAN */
		    }
		}
	    }
	    if (!cache_size) {
		cache_buffer = *buffer++;
		cache_size = 8;
		size--;
	    }
	    cache_size--;
	    if (!(cache_buffer >> cache_size & 1)) {
		ABORT_DECODING(video, buffer, size, VIDEO_SLICE_FAULT);
	    }
	    if (cache_size < 23) {
		if (cache_size < 7) {
		    cache_buffer = cache_buffer << 24
			| *buffer << 16 | buffer[1] << 8 | buffer[2];
		    buffer += 3;
		    cache_size += 24;
		    size -= 3;
		} else if (cache_size < 15) {
		    cache_buffer =
			cache_buffer << 16 | *buffer << 8 | buffer[1];
		    buffer += 2;
		    cache_size += 16;
		    size -= 2;
		} else {
		    cache_buffer = cache_buffer << 8 | *buffer++;
		    cache_size += 8;
		    size--;
		}
	    }
	} while (cache_buffer >> (cache_size - 23) & 8388607);
	cache_size &= ~7;
	cache_buffer &= (1 << cache_size) - 1;
	video->buffer = buffer;
	video->size = size;
	if (cache_buffer) {
	    if (cache_size != 24 || cache_buffer != 1) {
		ABORT_DECODING(video, buffer, size, VIDEO_SLICE_FAULT);
	    }
	    if (!size) {
		video->optimistique_moi(video);
	    }
#if WORDS_BIGENDIAN
	    video->start_code = 0x00000100 | *video->buffer;
#else
	    video->start_code = 0x00010000 | *video->buffer << 24;
#endif				/* WORDS_BIGENDIAN */
	    video->buffer++;
	    video->size--;
	} else if (cache_size) {
	    video->start_code = cache_size >> 3;
	    video->neXt_start_code(video);
	} else {
	    video->next_start_code(video);
	}
    } while (MAMAN_A_TORT(video));
}
