/*
 * h262.9.c
 * Copyright (C) 2003, 2004, 2005  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.2.h>
#include <h262.3.h>
#include <h262.4.h>
#include <h262.6.d4.h>

#if 0
# define CHECK_MOTION_LIMITS(half_flags, limits, offset) \
    {									      \
	if ((int) (offset) < 0						      \
	    || video->block_##limits##_motion_limit[half_flags] < (offset)) { \
	    return;							      \
	}								      \
    }
#else
# define CHECK_MOTION_LIMITS(half_flags, limits, offset) \
    {									      \
	if (video->block_##limits##_motion_limit[(half_flags)] < (offset)) {  \
	    return;							      \
	}								      \
    }
#endif				/* 0 */

#define MACROBLOCK_INIT(structure) \
    {									      \
	macroblock = video->macroblock;					      \
	if (video->mb_width < macroblock->mb_column + 1) {		      \
	    return;							      \
	}								      \
	macroblock->previous_macroblock_address =			      \
	    macroblock->macroblock_address;				      \
	offset = macroblock->luminance_slice_address			      \
	    + (macroblock->mb_column << 4);				      \
	pattern_code = macroblock->pattern_code;			      \
	pattern_code |= macroblock->dc_sparse << 1;			      \
	line_size = video->luminance_line_size;				      \
    }

#define NON_BIDIRECTIONALLY_PREDICTIVE_MACROBLOCK(macroblock) \
    (((macroblock)->packed_flags					      \
      & (macroblock_motion_backward_mask | macroblock_motion_forward_mask))   \
     != (macroblock_motion_backward_mask | macroblock_motion_forward_mask))

static void macroblock_0(struct seek_h262_video *);
static void macroblock_1(struct seek_h262_video *);
static void macroblock_2(struct seek_h262_video *);
static void macroblock_3(struct seek_h262_video *);
static void macroblock_4(struct seek_h262_video *);
static void macroblock_5(struct seek_h262_video *);
static void macroblock_6(struct seek_h262_video *);
static void macroblock_7(struct seek_h262_video *);
static void macroblock_8(struct seek_h262_video *);
static void macroblock_9(struct seek_h262_video *);
static void macroblock_a(struct seek_h262_video *);
static void macroblock_b(struct seek_h262_video *);

void (*_seek_h262_GRAYSCALE_macroblock[]) (struct seek_h262_video *) = {
/* *INDENT-OFF* */
    macroblock_0,
    macroblock_1,
    macroblock_2,
    macroblock_3,
    macroblock_4,
    macroblock_5,
    macroblock_6,
    macroblock_7,
    macroblock_8,
    macroblock_9,
    macroblock_a,
    macroblock_b
/* *INDENT-ON* */
};

static void
macroblock_0(struct seek_h262_video *video)
{
    {
	struct seek_h262_13818_macroblock *macroblock;

	macroblock = (struct seek_h262_13818_macroblock *) video->macroblock;
	if (video->mb_width < macroblock->mb_column + 1) {
	    return;
	}
	macroblock->previous_macroblock_address =
	    macroblock->macroblock_address;
	video->idct_set_block(video->blocks, 4, macroblock->mb_column << 4,
			      video->luminance_line_size);
    }
}


static void
macroblock_1(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    unsigned line_size, pattern_code, offset;

    MACROBLOCK_INIT(0);

    if (NON_BIDIRECTIONALLY_PREDICTIVE_MACROBLOCK(macroblock)) {
	unsigned f = 1;

	for (; f < 3; f++) {
	    seek_h262_constructor *constructor;
	    unsigned char *references;

#if !__Berkeley_FUCK_UP__
	    constructor = video->block_0_2;
#else
	    constructor = video->block_0_2[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	    {
		int vector_x_x_0, vector_x_x_1;
		unsigned h_offset, half_flags,
		    motion_vertical_field_select_x_x;

		if (macroblock->packed_flags
		    & macroblock_motion_backward_mask) {
		    references = video->Irulan_release.Y;
		    if (f == 1) {
			vector_x_x_0 = macroblock->vector_0_1_0;
			vector_x_x_1 = macroblock->vector_0_1_1;
			motion_vertical_field_select_x_x =
			    macroblock->motion_vertical_field_select_0_1;
		    } else {
			vector_x_x_0 = macroblock->vector_1_1_0;
			vector_x_x_1 = macroblock->vector_1_1_1;
			motion_vertical_field_select_x_x =
			    macroblock->motion_vertical_field_select_1_1;
		    }
		} else {
		    references = video->Iran_release.Y;
		    if (f == 1) {
			vector_x_x_0 = macroblock->vector_0_0_0;
			vector_x_x_1 = macroblock->vector_0_0_1;
			motion_vertical_field_select_x_x =
			    macroblock->motion_vertical_field_select_0_0;
		    } else {
			vector_x_x_0 = macroblock->vector_1_0_0;
			vector_x_x_1 = macroblock->vector_1_0_1;
			motion_vertical_field_select_x_x =
			    macroblock->motion_vertical_field_select_1_0;
		    }
		}
		half_flags = (vector_x_x_0 & 1) | (vector_x_x_1 & 1) << 1;
		h_offset = offset + (vector_x_x_0 >> 1)
		    + ((vector_x_x_1 & ~1)
		       | motion_vertical_field_select_x_x) * line_size;
		CHECK_MOTION_LIMITS(half_flags, 1, h_offset);
		references += h_offset;
		constructor += half_flags;
	    }
	    {
		unsigned char *predictions;

		predictions = video->Mylene_release.Y + offset;
		if (f == 2) {
		    predictions += line_size;
		}
		(*constructor)(predictions, references, line_size << 1);
	    }
	}
    } else {
	unsigned f = 1;

	for (; f < 3; f++) {
	    seek_h262_constructOr *constructOr;
	    unsigned char *references_0, *references_1;

#if !__Berkeley_FUCK_UP__
	    constructOr = video->block_1_2;
#else
	    constructOr = video->block_1_2[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	    {
		int vector_x_1_0, vector_x_1_1;
		unsigned h_offset, half_flags,
		    motion_vertical_field_select_x_1;

		references_0 = video->Irulan_release.Y;
		if (f == 1) {
		    vector_x_1_0 = macroblock->vector_0_1_0;
		    vector_x_1_1 = macroblock->vector_0_1_1;
		    motion_vertical_field_select_x_1 =
			macroblock->motion_vertical_field_select_0_1;
		} else {
		    vector_x_1_0 = macroblock->vector_1_1_0;
		    vector_x_1_1 = macroblock->vector_1_1_1;
		    motion_vertical_field_select_x_1 =
			macroblock->motion_vertical_field_select_1_1;
		}
		half_flags = (vector_x_1_0 & 1) | (vector_x_1_1 & 1) << 1;
		h_offset = offset + (vector_x_1_0 >> 1)
		    + ((vector_x_1_1 & ~1)
		       | motion_vertical_field_select_x_1) * line_size;
		CHECK_MOTION_LIMITS(half_flags, 1, h_offset);
		references_0 += h_offset;
		constructOr += half_flags << 2;
	    }
	    {
		int vector_x_0_0, vector_x_0_1;
		unsigned h_offset, half_flags,
		    motion_vertical_field_select_x_0;

		references_1 = video->Iran_release.Y;
		if (f == 1) {
		    vector_x_0_0 = macroblock->vector_0_0_0;
		    vector_x_0_1 = macroblock->vector_0_0_1;
		    motion_vertical_field_select_x_0 =
			macroblock->motion_vertical_field_select_0_0;
		} else {
		    vector_x_0_0 = macroblock->vector_1_0_0;
		    vector_x_0_1 = macroblock->vector_1_0_1;
		    motion_vertical_field_select_x_0 =
			macroblock->motion_vertical_field_select_1_0;
		}
		half_flags = (vector_x_0_0 & 1) | (vector_x_0_1 & 1) << 1;
		h_offset = offset + (vector_x_0_0 >> 1)
		    + ((vector_x_0_1 & ~1)
		       | motion_vertical_field_select_x_0) * line_size;
		CHECK_MOTION_LIMITS(half_flags, 1, h_offset);
		references_1 += h_offset;
		constructOr += half_flags;
	    }
	    {
		unsigned char *predictions;

		predictions = video->Mylene_release.Y + offset;
		if (f == 2) {
		    predictions += line_size;
		}
		(*constructOr)(predictions, references_0, references_1,
			       line_size << 1);
	    }
	}
    }

    video->idct_add_block(video->blocks, pattern_code & 255,
			  macroblock->mb_column << 4, line_size);
}


static void
macroblock_2(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    unsigned line_size, pattern_code, offset;

    MACROBLOCK_INIT(0);

    if (NON_BIDIRECTIONALLY_PREDICTIVE_MACROBLOCK(macroblock)) {
	seek_h262_constructor *constructor;
	unsigned char *references;

#if !__Berkeley_FUCK_UP__
	constructor = video->block_0_3;
#else
	constructor = video->block_0_3[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	{
	    int vector_0_x_0, vector_0_x_1;

	    if (macroblock->packed_flags & macroblock_motion_backward_mask) {
		references = video->Irulan_release.Y;
		vector_0_x_0 = macroblock->vector_0_1_0;
		vector_0_x_1 = macroblock->vector_0_1_1;
	    } else {
		references = video->Iran_release.Y;
		vector_0_x_0 = macroblock->vector_0_0_0;
		vector_0_x_1 = macroblock->vector_0_0_1;
	    }
	    if (vector_0_x_0 | vector_0_x_1) {
		unsigned half_flags, h_offset;

		half_flags = (vector_0_x_0 & 1) | (vector_0_x_1 & 1) << 1;
		h_offset = offset + (vector_0_x_0 >> 1)
		    + (vector_0_x_1 >> 1) * line_size;
		CHECK_MOTION_LIMITS(half_flags, 2, h_offset);
		references += h_offset;
		constructor += half_flags;
	    } else {
		references += offset;
	    }
	}
	(*constructor)
	    (video->Mylene_release.Y + offset, references, line_size);
    } else {
	seek_h262_constructOr *constructOr;
	unsigned char *references_0, *references_1;

#if !__Berkeley_FUCK_UP__
	constructOr = video->block_1_3;
#else
	constructOr = video->block_1_3[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	{
	    int vector_0_1_0, vector_0_1_1;

	    references_0 = video->Irulan_release.Y;
	    vector_0_1_0 = macroblock->vector_0_1_0;
	    vector_0_1_1 = macroblock->vector_0_1_1;
	    if (vector_0_1_0 | vector_0_1_1) {
		unsigned half_flags, h_offset;

		half_flags = (vector_0_1_0 & 1) | (vector_0_1_1 & 1) << 1;
		h_offset = offset + (vector_0_1_0 >> 1)
		    + (vector_0_1_1 >> 1) * line_size;
		CHECK_MOTION_LIMITS(half_flags, 2, h_offset);
		references_0 += h_offset;
		constructOr += half_flags << 2;
	    } else {
		references_0 += offset;
	    }
	}
	{
	    int vector_0_0_0, vector_0_0_1;

	    references_1 = video->Iran_release.Y;
	    vector_0_0_0 = macroblock->vector_0_0_0;
	    vector_0_0_1 = macroblock->vector_0_0_1;
	    if (vector_0_0_0 | vector_0_0_1) {
		unsigned half_flags, h_offset;

		half_flags = (vector_0_0_0 & 1) | (vector_0_0_1 & 1) << 1;
		h_offset = offset + (vector_0_0_0 >> 1)
		    + (vector_0_0_1 >> 1) * line_size;
		CHECK_MOTION_LIMITS(half_flags, 2, h_offset);
		references_1 += h_offset;
		constructOr += half_flags;
	    } else {
		references_1 += offset;
	    }
	}
	(*constructOr)(video->Mylene_release.Y + offset, references_0,
		       references_1, line_size);
    }

    video->idct_add_block(video->blocks, pattern_code & 255,
			  macroblock->mb_column << 4, line_size);
}


static void
macroblock_3(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    unsigned line_size, pattern_code, offset;

    MACROBLOCK_INIT(0);

    {
	unsigned f = 1, half_flags;
	unsigned char *references_0, *references_1;

	{
	    int scaled_vector_0_0_1, vector_0_0_0, vector_0_0_1;

	    references_0 = video->Iran_release.Y;
	    vector_0_0_0 = macroblock->vector_0_0_0;
	    vector_0_0_1 = macroblock->vector_0_0_1;
	    scaled_vector_0_0_1 = vector_0_0_1 & ~1;
	    {
		unsigned h_offset;

		h_offset = offset + (vector_0_0_0 >> 1)
		    + scaled_vector_0_0_1 * line_size;
		half_flags =
		    ((vector_0_0_0 & 1) | (vector_0_0_1 & 1) << 1) << 2;
		CHECK_MOTION_LIMITS(half_flags >> 02, 2, h_offset);
		references_0 += h_offset;
	    }
	}
	for (; f < 3; f++) {
	    {
		int scaled_vector_x_0_1, vector_x_0_0, vector_x_0_1;

		references_1 = video->Iran_release.Y;
		vector_x_0_0 = macroblock->vector_0_0_0;
		vector_x_0_1 = macroblock->vector_0_0_1;
		if (f == 1) {
		    if (vector_x_0_0 < 0) {
			vector_x_0_0 =
			    ((vector_x_0_0 * (int) video->m_1_0) >> 1)
			    + macroblock->dmvector_0;
		    } else {
			vector_x_0_0 =
			    ((vector_x_0_0 * (int) video->m_1_0 + 1) >> 1)
			    + macroblock->dmvector_0;
		    }
		    if (vector_x_0_1 < 0) {
			vector_x_0_1 =
			    ((vector_x_0_1 * (int) video->m_1_0) >> 1)
			    - 1 + macroblock->dmvector_1;
		    } else {
			vector_x_0_1 =
			    ((vector_x_0_1 * (int) video->m_1_0 + 1) >> 1)
			    - 1 + macroblock->dmvector_1;
		    }
		    scaled_vector_x_0_1 = vector_x_0_1 | 1;
		} else {
		    if (vector_x_0_0 < 0) {
			vector_x_0_0 =
			    ((vector_x_0_0 * (int) video->m_0_1) >> 1)
			    + macroblock->dmvector_0;
		    } else {
			vector_x_0_0 =
			    ((vector_x_0_0 * (int) video->m_0_1 + 1) >> 1)
			    + macroblock->dmvector_0;
		    }
		    if (vector_x_0_1 < 0) {
			vector_x_0_1 =
			    ((vector_x_0_1 * (int) video->m_0_1) >> 1)
			    + 1 + macroblock->dmvector_1;
		    } else {
			vector_x_0_1 =
			    ((vector_x_0_1 * (int) video->m_0_1 + 1) >> 1)
			    + 1 + macroblock->dmvector_1;
		    }
		    scaled_vector_x_0_1 = vector_x_0_1 & ~1;
		}
		{
		    unsigned h_offset;

		    h_offset = offset + (vector_x_0_0 >> 1)
			+ scaled_vector_x_0_1 * line_size;
		    half_flags = (half_flags & ~3)
			| (vector_x_0_0 & 1) | (vector_x_0_1 & 1) << 1;
		    CHECK_MOTION_LIMITS(half_flags & 0x3, 1, h_offset);
		    references_1 += h_offset;
		}
	    }
	    {
		seek_h262_constructOr constructOr;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructOr = video->block_1_2[half_flags];
#else
		constructOr = video->block_1_2[0][half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = video->Mylene_release.Y + offset;
		if (f == 2) {
		    predictions += line_size;
		    references_0 += line_size;
		}
		constructOr(predictions, references_0, references_1,
			    line_size << 1);
	    }
	}
    }

    video->idct_add_block(video->blocks, pattern_code & 255,
			  macroblock->mb_column << 4, line_size);
}


static void
macroblock_4(struct seek_h262_video *video)
{
    {
	struct seek_h262_13818_macroblock *macroblock;

	macroblock = (struct seek_h262_13818_macroblock *) video->macroblock;
	if (video->mb_width < macroblock->mb_column + 1) {
	    return;
	}
	macroblock->previous_macroblock_address =
	    macroblock->macroblock_address;
	video->idct_set_block(video->blocks + video->chroma_format, 4,
			      macroblock->mb_column << 4,
			      video->luminance_line_size << 1);
    }
}


static void
macroblock_5(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    unsigned line_size, pattern_code, offset;

    MACROBLOCK_INIT(0);

    if (NON_BIDIRECTIONALLY_PREDICTIVE_MACROBLOCK(macroblock)) {
	unsigned f = 1;

	for (; f < 3; f++) {
	    seek_h262_constructor *constructor;
	    unsigned char *references;

#if !__Berkeley_FUCK_UP__
	    constructor = video->block_0_2;
#else
	    constructor = video->block_0_2[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	    {
		int vector_x_x_0, vector_x_x_1;
		unsigned h_offset, half_flags,
		    motion_vertical_field_select_x_x;

		if (macroblock->packed_flags
		    & macroblock_motion_backward_mask) {
		    references = video->Irulan_release.Y;
		    if (f == 1) {
			vector_x_x_0 = macroblock->vector_0_1_0;
			vector_x_x_1 = macroblock->vector_0_1_1;
			motion_vertical_field_select_x_x =
			    macroblock->motion_vertical_field_select_0_1;
		    } else {
			vector_x_x_0 = macroblock->vector_1_1_0;
			vector_x_x_1 = macroblock->vector_1_1_1;
			motion_vertical_field_select_x_x =
			    macroblock->motion_vertical_field_select_1_1;
		    }
		} else {
		    references = video->Iran_release.Y;
		    if (f == 1) {
			vector_x_x_0 = macroblock->vector_0_0_0;
			vector_x_x_1 = macroblock->vector_0_0_1;
			motion_vertical_field_select_x_x =
			    macroblock->motion_vertical_field_select_0_0;
		    } else {
			vector_x_x_0 = macroblock->vector_1_0_0;
			vector_x_x_1 = macroblock->vector_1_0_1;
			motion_vertical_field_select_x_x =
			    macroblock->motion_vertical_field_select_1_0;
		    }
		}
		half_flags = (vector_x_x_0 & 1) | (vector_x_x_1 & 1) << 1;
		h_offset = offset + (vector_x_x_0 >> 1)
		    + ((vector_x_x_1 & ~1)
		       | motion_vertical_field_select_x_x) * line_size;
		CHECK_MOTION_LIMITS(half_flags, 1, h_offset);
		references += h_offset;
		constructor += half_flags;
	    }
	    {
		unsigned char *predictions;

		predictions = video->Mylene_release.Y + offset;
		if (f == 2) {
		    predictions += line_size;
		}
		(*constructor)(predictions, references, line_size << 1);
	    }
	}
    } else {
	unsigned f = 1;

	for (; f < 3; f++) {
	    seek_h262_constructOr *constructOr;
	    unsigned char *references_0, *references_1;

#if !__Berkeley_FUCK_UP__
	    constructOr = video->block_1_2;
#else
	    constructOr = video->block_1_2[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	    {
		int vector_x_1_0, vector_x_1_1;
		unsigned h_offset, half_flags,
		    motion_vertical_field_select_x_1;

		references_0 = video->Irulan_release.Y;
		if (f == 1) {
		    vector_x_1_0 = macroblock->vector_0_1_0;
		    vector_x_1_1 = macroblock->vector_0_1_1;
		    motion_vertical_field_select_x_1 =
			macroblock->motion_vertical_field_select_0_1;
		} else {
		    vector_x_1_0 = macroblock->vector_1_1_0;
		    vector_x_1_1 = macroblock->vector_1_1_1;
		    motion_vertical_field_select_x_1 =
			macroblock->motion_vertical_field_select_1_1;
		}
		half_flags = (vector_x_1_0 & 1) | (vector_x_1_1 & 1) << 1;
		h_offset = offset + (vector_x_1_0 >> 1)
		    + ((vector_x_1_1 & ~1)
		       | motion_vertical_field_select_x_1) * line_size;
		CHECK_MOTION_LIMITS(half_flags, 1, h_offset);
		references_0 += h_offset;
		constructOr += half_flags << 2;
	    }
	    {
		int vector_x_0_0, vector_x_0_1;
		unsigned h_offset, half_flags,
		    motion_vertical_field_select_x_0;

		references_1 = video->Iran_release.Y;
		if (f == 1) {
		    vector_x_0_0 = macroblock->vector_0_0_0;
		    vector_x_0_1 = macroblock->vector_0_0_1;
		    motion_vertical_field_select_x_0 =
			macroblock->motion_vertical_field_select_0_0;
		} else {
		    vector_x_0_0 = macroblock->vector_1_0_0;
		    vector_x_0_1 = macroblock->vector_1_0_1;
		    motion_vertical_field_select_x_0 =
			macroblock->motion_vertical_field_select_1_0;
		}
		half_flags = (vector_x_0_0 & 1) | (vector_x_0_1 & 1) << 1;
		h_offset = offset + (vector_x_0_0 >> 1)
		    + ((vector_x_0_1 & ~1)
		       | motion_vertical_field_select_x_0) * line_size;
		CHECK_MOTION_LIMITS(half_flags, 1, h_offset);
		references_1 += h_offset;
		constructOr += half_flags;
	    }
	    {
		unsigned char *predictions;

		predictions = video->Mylene_release.Y + offset;
		if (f == 2) {
		    predictions += line_size;
		}
		(*constructOr)(predictions, references_0, references_1,
			       line_size << 1);
	    }
	}
    }

    video->idct_add_block(video->blocks + video->chroma_format,
			  pattern_code & 255, macroblock->mb_column << 4,
			  line_size << 1);
}


static void
macroblock_6(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    unsigned line_size, pattern_code, offset;

    MACROBLOCK_INIT(0);

    if (NON_BIDIRECTIONALLY_PREDICTIVE_MACROBLOCK(macroblock)) {
	seek_h262_constructor *constructor;
	unsigned char *references;

#if !__Berkeley_FUCK_UP__
	constructor = video->block_0_3;
#else
	constructor = video->block_0_3[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	{
	    int vector_0_x_0, vector_0_x_1;

	    if (macroblock->packed_flags & macroblock_motion_backward_mask) {
		references = video->Irulan_release.Y;
		vector_0_x_0 = macroblock->vector_0_1_0;
		vector_0_x_1 = macroblock->vector_0_1_1;
	    } else {
		references = video->Iran_release.Y;
		vector_0_x_0 = macroblock->vector_0_0_0;
		vector_0_x_1 = macroblock->vector_0_0_1;
	    }
	    if (vector_0_x_0 | vector_0_x_1) {
		unsigned half_flags, h_offset;

		half_flags = (vector_0_x_0 & 1) | (vector_0_x_1 & 1) << 1;
		h_offset = offset + (vector_0_x_0 >> 1)
		    + (vector_0_x_1 >> 1) * line_size;
		CHECK_MOTION_LIMITS(half_flags, 2, h_offset);
		references += h_offset;
		constructor += half_flags;
	    } else {
		references += offset;
	    }
	}
	(*constructor)
	    (video->Mylene_release.Y + offset, references, line_size);
    } else {
	seek_h262_constructOr *constructOr;
	unsigned char *references_0, *references_1;

#if !__Berkeley_FUCK_UP__
	constructOr = video->block_1_3;
#else
	constructOr = video->block_1_3[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	{
	    int vector_0_1_0, vector_0_1_1;

	    references_0 = video->Irulan_release.Y;
	    vector_0_1_0 = macroblock->vector_0_1_0;
	    vector_0_1_1 = macroblock->vector_0_1_1;
	    if (vector_0_1_0 | vector_0_1_1) {
		unsigned half_flags, h_offset;

		half_flags = (vector_0_1_0 & 1) | (vector_0_1_1 & 1) << 1;
		h_offset = offset + (vector_0_1_0 >> 1)
		    + (vector_0_1_1 >> 1) * line_size;
		CHECK_MOTION_LIMITS(half_flags, 2, h_offset);
		references_0 += h_offset;
		constructOr += half_flags << 2;
	    } else {
		references_0 += offset;
	    }
	}
	{
	    int vector_0_0_0, vector_0_0_1;

	    references_1 = video->Iran_release.Y;
	    vector_0_0_0 = macroblock->vector_0_0_0;
	    vector_0_0_1 = macroblock->vector_0_0_1;
	    if (vector_0_0_0 | vector_0_0_1) {
		unsigned half_flags, h_offset;

		half_flags = (vector_0_0_0 & 1) | (vector_0_0_1 & 1) << 1;
		h_offset = offset + (vector_0_0_0 >> 1)
		    + (vector_0_0_1 >> 1) * line_size;
		CHECK_MOTION_LIMITS(half_flags, 2, h_offset);
		references_1 += h_offset;
		constructOr += half_flags;
	    } else {
		references_1 += offset;
	    }
	}
	(*constructOr)(video->Mylene_release.Y + offset, references_0,
		       references_1, line_size);
    }

    video->idct_add_block(video->blocks + video->chroma_format,
			  pattern_code & 255, macroblock->mb_column << 4,
			  line_size << 1);
}


static void
macroblock_7(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    unsigned line_size, pattern_code, offset;

    MACROBLOCK_INIT(0);

    {
	unsigned f = 1, half_flags;
	unsigned char *references_0, *references_1;

	{
	    int scaled_vector_0_0_1, vector_0_0_0, vector_0_0_1;

	    references_0 = video->Iran_release.Y;
	    vector_0_0_0 = macroblock->vector_0_0_0;
	    vector_0_0_1 = macroblock->vector_0_0_1;
	    scaled_vector_0_0_1 = vector_0_0_1 & ~1;
	    {
		unsigned h_offset;

		h_offset = offset + (vector_0_0_0 >> 1)
		    + scaled_vector_0_0_1 * line_size;
		half_flags =
		    ((vector_0_0_0 & 1) | (vector_0_0_1 & 1) << 1) << 2;
		CHECK_MOTION_LIMITS(half_flags >> 02, 2, h_offset);
		references_0 += h_offset;
	    }
	}
	for (; f < 3; f++) {
	    {
		int scaled_vector_x_0_1, vector_x_0_0, vector_x_0_1;

		references_1 = video->Iran_release.Y;
		vector_x_0_0 = macroblock->vector_0_0_0;
		vector_x_0_1 = macroblock->vector_0_0_1;
		if (f == 1) {
		    if (vector_x_0_0 < 0) {
			vector_x_0_0 =
			    ((vector_x_0_0 * (int) video->m_1_0) >> 1)
			    + macroblock->dmvector_0;
		    } else {
			vector_x_0_0 =
			    ((vector_x_0_0 * (int) video->m_1_0 + 1) >> 1)
			    + macroblock->dmvector_0;
		    }
		    if (vector_x_0_1 < 0) {
			vector_x_0_1 =
			    ((vector_x_0_1 * (int) video->m_1_0) >> 1)
			    - 1 + macroblock->dmvector_1;
		    } else {
			vector_x_0_1 =
			    ((vector_x_0_1 * (int) video->m_1_0 + 1) >> 1)
			    - 1 + macroblock->dmvector_1;
		    }
		    scaled_vector_x_0_1 = vector_x_0_1 | 1;
		} else {
		    if (vector_x_0_0 < 0) {
			vector_x_0_0 =
			    ((vector_x_0_0 * (int) video->m_0_1) >> 1)
			    + macroblock->dmvector_0;
		    } else {
			vector_x_0_0 =
			    ((vector_x_0_0 * (int) video->m_0_1 + 1) >> 1)
			    + macroblock->dmvector_0;
		    }
		    if (vector_x_0_1 < 0) {
			vector_x_0_1 =
			    ((vector_x_0_1 * (int) video->m_0_1) >> 1)
			    + 1 + macroblock->dmvector_1;
		    } else {
			vector_x_0_1 =
			    ((vector_x_0_1 * (int) video->m_0_1 + 1) >> 1)
			    + 1 + macroblock->dmvector_1;
		    }
		    scaled_vector_x_0_1 = vector_x_0_1 & ~1;
		}
		{
		    unsigned h_offset;

		    h_offset = offset + (vector_x_0_0 >> 1)
			+ scaled_vector_x_0_1 * line_size;
		    half_flags = (half_flags & ~3)
			| (vector_x_0_0 & 1) | (vector_x_0_1 & 1) << 1;
		    CHECK_MOTION_LIMITS(half_flags & 0x3, 1, h_offset);
		    references_1 += h_offset;
		}
	    }
	    {
		seek_h262_constructOr constructOr;
		unsigned char *predictions;

#if !__Berkeley_FUCK_UP__
		constructOr = video->block_1_2[half_flags];
#else
		constructOr = video->block_1_2[0][half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
		predictions = video->Mylene_release.Y + offset;
		if (f == 2) {
		    predictions += line_size;
		    references_0 += line_size;
		}
		constructOr(predictions, references_0, references_1,
			    line_size << 1);
	    }
	}
    }

    video->idct_add_block(video->blocks + video->chroma_format,
			  pattern_code & 255, macroblock->mb_column << 4,
			  line_size << 1);
}


static void
macroblock_8(struct seek_h262_video *video)
{
    {
	struct seek_h262_13818_macroblock *macroblock;

	macroblock = (struct seek_h262_13818_macroblock *) video->macroblock;
	if (video->mb_width < macroblock->mb_column + 1) {
	    return;
	}
	macroblock->previous_macroblock_address =
	    macroblock->macroblock_address;
	video->idct_set_block(video->blocks, 4, macroblock->mb_column << 4,
			      video->luminance_line_size << 1);
    }
}


static void
macroblock_9(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned line_size, pattern_code, offset;

    MACROBLOCK_INIT(1);
    blocks = video->blocks;

    if (NON_BIDIRECTIONALLY_PREDICTIVE_MACROBLOCK(macroblock)) {
	seek_h262_constructor *constructor;
	unsigned char *references;

#if !__Berkeley_FUCK_UP__
	constructor = video->block_0_3;
#else
	constructor = video->block_0_3[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	{
	    int vector_0_x_0, vector_0_x_1;
	    unsigned motion_vertical_field_select_0_x;

	    if (macroblock->packed_flags & macroblock_motion_backward_mask) {
		references = video->Irulan_release.Y;
		vector_0_x_0 = macroblock->vector_0_1_0;
		vector_0_x_1 = macroblock->vector_0_1_1;
		motion_vertical_field_select_0_x =
		    macroblock->motion_vertical_field_select_0_1;
	    } else {
		vector_0_x_0 = macroblock->vector_0_0_0;
		vector_0_x_1 = macroblock->vector_0_0_1;
		motion_vertical_field_select_0_x =
		    macroblock->motion_vertical_field_select_0_0;
		references = video->field_references
		    [motion_vertical_field_select_0_x].Y;
	    }
	    if (vector_0_x_0 | vector_0_x_1) {
		unsigned h_offset, half_flags;

		half_flags = (vector_0_x_0 & 1) | (vector_0_x_1 & 1) << 1;
		h_offset = offset + (vector_0_x_0 >> 1)
		    + ((vector_0_x_1 & ~1)
		       | motion_vertical_field_select_0_x) * line_size;
		CHECK_MOTION_LIMITS(half_flags, 9, h_offset);
		references += h_offset;
		constructor += half_flags;
	    } else {
		references += offset
		    + motion_vertical_field_select_0_x * line_size;
	    }
	}
	(*constructor)
	    (blocks->base_address + (macroblock->mb_column << 4), references,
	     line_size << 1);
    } else {
	seek_h262_constructOr *constructOr;
	unsigned char *references_0, *references_1;

#if !__Berkeley_FUCK_UP__
	constructOr = video->block_1_3;
#else
	constructOr = video->block_1_3[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	{
	    int vector_0_1_0, vector_0_1_1;

	    references_0 = video->Irulan_release.Y;
	    vector_0_1_0 = macroblock->vector_0_1_0;
	    vector_0_1_1 = macroblock->vector_0_1_1;
	    if (vector_0_1_0 | vector_0_1_1) {
		unsigned h_offset, half_flags;

		half_flags = (vector_0_1_0 & 1) | (vector_0_1_1 & 1) << 1;
		h_offset = offset + (vector_0_1_0 >> 1)
		    + ((vector_0_1_1 & ~1)
		       | macroblock->motion_vertical_field_select_0_1)
		    * line_size;
		CHECK_MOTION_LIMITS(half_flags, 9, h_offset);
		references_0 += h_offset;
		constructOr += half_flags << 2;
	    } else {
		references_0 += offset
		    + macroblock->motion_vertical_field_select_0_1 * line_size;
	    }
	}
	{
	    int vector_0_0_0, vector_0_0_1;

	    references_1 = video->Iran_release.Y;
	    vector_0_0_0 = macroblock->vector_0_0_0;
	    vector_0_0_1 = macroblock->vector_0_0_1;
	    if (vector_0_0_0 | vector_0_0_1) {
		unsigned h_offset, half_flags;

		half_flags = (vector_0_0_0 & 1) | (vector_0_0_1 & 1) << 1;
		h_offset = offset + (vector_0_0_0 >> 1)
		    + ((vector_0_0_1 & ~1)
		       | macroblock->motion_vertical_field_select_0_0)
		    * line_size;
		CHECK_MOTION_LIMITS(half_flags, 9, h_offset);
		references_1 += h_offset;
		constructOr += half_flags;
	    } else {
		references_1 += offset
		    + macroblock->motion_vertical_field_select_0_0 * line_size;
	    }
	}
	(*constructOr)
	    (blocks->base_address + (macroblock->mb_column << 4),
	     references_0, references_1, line_size << 1);
    }

    video->idct_add_block(blocks, pattern_code & 255,
			  macroblock->mb_column << 4, line_size << 1);
}


static void
macroblock_a(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned line_size, pattern_code, offset;

    MACROBLOCK_INIT(1);
    blocks = video->blocks;

    if (NON_BIDIRECTIONALLY_PREDICTIVE_MACROBLOCK(macroblock)) {
	unsigned f = 1;

	for (; f < 3; f++) {
	    seek_h262_constructor *constructor;
	    unsigned char *references;

#if !__Berkeley_FUCK_UP__
	    constructor = video->block_0_2;
#else
	    constructor = video->block_0_2[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	    {
		int vector_x_x_0, vector_x_x_1;
		unsigned h_offset, half_flags,
		    motion_vertical_field_select_x_x;

		if (macroblock->packed_flags
		    & macroblock_motion_backward_mask) {
		    references = video->Irulan_release.Y;
		    if (f == 1) {
			vector_x_x_0 = macroblock->vector_0_1_0;
			vector_x_x_1 = macroblock->vector_0_1_1;
			motion_vertical_field_select_x_x =
			    macroblock->motion_vertical_field_select_0_1;
		    } else {
			vector_x_x_0 = macroblock->vector_1_1_0;
			vector_x_x_1 = macroblock->vector_1_1_1;
			motion_vertical_field_select_x_x =
			    macroblock->motion_vertical_field_select_1_1;
		    }
		} else {
		    if (f == 1) {
			vector_x_x_0 = macroblock->vector_0_0_0;
			vector_x_x_1 = macroblock->vector_0_0_1;
			motion_vertical_field_select_x_x =
			    macroblock->motion_vertical_field_select_0_0;
		    } else {
			vector_x_x_0 = macroblock->vector_1_0_0;
			vector_x_x_1 = macroblock->vector_1_0_1;
			motion_vertical_field_select_x_x =
			    macroblock->motion_vertical_field_select_1_0;
		    }
		    references = video->field_references
			[motion_vertical_field_select_x_x].Y;
		}
		half_flags = (vector_x_x_0 & 1) | (vector_x_x_1 & 1) << 1;
		h_offset = offset + (vector_x_x_0 >> 1)
		    + ((vector_x_x_1 & ~1)
		       | motion_vertical_field_select_x_x) * line_size;
		if (f == 2) {
		    h_offset += line_size << 4;
		}
		CHECK_MOTION_LIMITS(half_flags, a, h_offset);
		references += h_offset;
		constructor += half_flags;
	    }
	    {
		unsigned char *predictions;

		predictions = blocks->base_address
		    + (macroblock->mb_column << 4);
		if (f == 2) {
		    predictions += line_size << 4;
		}
		(*constructor)(predictions, references, line_size << 1);
	    }
	}
    } else {
	unsigned f = 1;

	for (; f < 3; f++) {
	    seek_h262_constructOr *constructOr;
	    unsigned char *references_0, *references_1;

#if !__Berkeley_FUCK_UP__
	    constructOr = video->block_1_2;
#else
	    constructOr = video->block_1_2[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	    {
		int vector_x_1_0, vector_x_1_1;
		unsigned h_offset, half_flags,
		    motion_vertical_field_select_x_1;

		references_0 = video->Irulan_release.Y;
		if (f == 1) {
		    vector_x_1_0 = macroblock->vector_0_1_0;
		    vector_x_1_1 = macroblock->vector_0_1_1;
		    motion_vertical_field_select_x_1 =
			macroblock->motion_vertical_field_select_0_1;
		} else {
		    vector_x_1_0 = macroblock->vector_1_1_0;
		    vector_x_1_1 = macroblock->vector_1_1_1;
		    motion_vertical_field_select_x_1 =
			macroblock->motion_vertical_field_select_1_1;
		}
		half_flags = (vector_x_1_0 & 1) | (vector_x_1_1 & 1) << 1;
		h_offset = offset + (vector_x_1_0 >> 1)
		    + ((vector_x_1_1 & ~1)
		       | motion_vertical_field_select_x_1) * line_size;
		CHECK_MOTION_LIMITS(half_flags, a, h_offset);
		references_0 += h_offset;
		constructOr += half_flags << 2;
	    }
	    {
		int vector_x_0_0, vector_x_0_1;
		unsigned h_offset, half_flags,
		    motion_vertical_field_select_x_0;

		references_1 = video->Iran_release.Y;
		if (f == 1) {
		    vector_x_0_0 = macroblock->vector_0_0_0;
		    vector_x_0_1 = macroblock->vector_0_0_1;
		    motion_vertical_field_select_x_0 =
			macroblock->motion_vertical_field_select_0_0;
		} else {
		    vector_x_0_0 = macroblock->vector_1_0_0;
		    vector_x_0_1 = macroblock->vector_1_0_1;
		    motion_vertical_field_select_x_0 =
			macroblock->motion_vertical_field_select_1_0;
		}
		half_flags = (vector_x_0_0 & 1) | (vector_x_0_1 & 1) << 1;
		h_offset = offset + (vector_x_0_0 >> 1)
		    + ((vector_x_0_1 & ~1)
		       | motion_vertical_field_select_x_0) * line_size;
		CHECK_MOTION_LIMITS(half_flags, a, h_offset);
		references_1 += h_offset;
		constructOr += half_flags;
	    }
	    {
		unsigned char *predictions;

		predictions = blocks->base_address
		    + (macroblock->mb_column << 4);
		if (f == 2) {
		    predictions += line_size << 4;
		}
		(*constructOr)(predictions, references_0, references_1,
			       line_size << 1);
	    }
	}
    }

    video->idct_add_block(blocks, pattern_code & 255,
			  macroblock->mb_column << 4, line_size << 1);
}


static void
macroblock_b(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned line_size, pattern_code, offset;

    MACROBLOCK_INIT(1);
    blocks = video->blocks;

    {
	seek_h262_constructOr *constructOr;
	unsigned char *references_0, *references_1;

#if !__Berkeley_FUCK_UP__
	constructOr = video->block_1_3;
#else
	constructOr = video->block_1_3[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	{
	    int scaled_vector_0_0_1, vector_0_0_0, vector_0_0_1;

	    references_0 = video->Iran_release.Y;
	    vector_0_0_0 = macroblock->vector_0_0_0;
	    vector_0_0_1 = macroblock->vector_0_0_1;
	    if (video->picture_structure == Top_Field) {
		scaled_vector_0_0_1 = vector_0_0_1 & ~1;
	    } else {
		scaled_vector_0_0_1 = vector_0_0_1 | 1;
	    }
	    {
		unsigned h_offset, half_flags;

		h_offset = offset + (vector_0_0_0 >> 1)
		    + scaled_vector_0_0_1 * line_size;
		half_flags = (vector_0_0_0 & 1) | (vector_0_0_1 & 1) << 1;
		CHECK_MOTION_LIMITS(half_flags, 9, h_offset);
		references_0 += h_offset;
		constructOr += half_flags << 2;
	    }
	}
	{
	    int scaled_vector_x_0_1, vector_x_0_0, vector_x_0_1;

	    references_1 = video->Irulan_release.Y;
	    vector_x_0_0 = macroblock->vector_0_0_0;
	    vector_x_0_1 = macroblock->vector_0_0_1;
	    if (vector_x_0_0 < 1) {
		vector_x_0_0 = ((vector_x_0_0) >> 1)
		    + macroblock->dmvector_0;
	    } else {
		vector_x_0_0 = ((vector_x_0_0 + 1) >> 1)
		    + macroblock->dmvector_0;
	    }
	    if (video->picture_structure == Top_Field) {
		if (vector_x_0_1 < 1) {
		    vector_x_0_1 = ((vector_x_0_1) >> 1)
			- 1 + macroblock->dmvector_1;
		} else {
		    vector_x_0_1 = ((vector_x_0_1 + 1) >> 1)
			- 1 + macroblock->dmvector_1;
		}
		scaled_vector_x_0_1 = vector_x_0_1 | 1;
	    } else {
		if (vector_x_0_1 < 1) {
		    vector_x_0_1 = ((vector_x_0_1) >> 1)
			+ 1 + macroblock->dmvector_1;
		} else {
		    vector_x_0_1 = ((vector_x_0_1 + 1) >> 1)
			+ 1 + macroblock->dmvector_1;
		}
		scaled_vector_x_0_1 = vector_x_0_1 & ~1;
	    }
	    {
		unsigned h_offset, half_flags;

		h_offset = offset + (vector_x_0_0 >> 1)
		    + scaled_vector_x_0_1 * line_size;
		half_flags = (vector_x_0_0 & 1) | (vector_x_0_1 & 1) << 1;
		CHECK_MOTION_LIMITS(half_flags, 9, h_offset);
		references_1 += h_offset;
		constructOr += half_flags;
	    }
	}
	{
	    (*constructOr)(blocks->base_address + (macroblock->mb_column << 4),
			   references_0, references_1, line_size << 1);
	}
    }

    video->idct_add_block(blocks, pattern_code & 255,
			  macroblock->mb_column << 4, line_size << 1);
}
