/*
 * h262.a.c
 * Copyright (C) 2003, 2004, 2005, 2006  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.2.h>
#include <h262.3.h>

#define __RUN_FUTILE__			0

static void GRAYSCALE_macroblocks_0(struct seek_h262_video *);
static void GRAYSCALE_macroblocks_1(struct seek_h262_video *);
static void YCbCr_420_macroblocks_0(struct seek_h262_video *);
static void YCbCr_420_macroblocks_1(struct seek_h262_video *);
static void YCbCr_422_macroblocks_0(struct seek_h262_video *);
static void YCbCr_422_macroblocks_1(struct seek_h262_video *);
static void YCbCr_444_macroblocks_0(struct seek_h262_video *);
static void YCbCr_444_macroblocks_1(struct seek_h262_video *);

void (*_seek_h262_GRAYSCALE_macroblocks[]) (struct seek_h262_video *) = {
/* *INDENT-OFF* */
    GRAYSCALE_macroblocks_0,
    GRAYSCALE_macroblocks_1
/* *INDENT-ON* */
}, (*_seek_h262_YCbCr_4xx_macroblocks[]) (struct seek_h262_video *) = {
/* *INDENT-OFF* */
    YCbCr_420_macroblocks_0,
    YCbCr_420_macroblocks_1,
    YCbCr_422_macroblocks_0,
    YCbCr_422_macroblocks_1,
    YCbCr_444_macroblocks_0,
    YCbCr_444_macroblocks_1
/* *INDENT-ON* */
};

static void
GRAYSCALE_macroblocks_0(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    unsigned offset, skipped;

    macroblock = (struct seek_h262_13818_macroblock *) video->macroblock;
    if (video->mb_width < macroblock->mb_column + 1) {
	return;
    }
    skipped = macroblock->macroblock_address_increment - 1;
    offset =
	macroblock->luminance_slice_address + (macroblock->mb_column << 4);
#if !__RUN_FUTILE__
    {
	unsigned ii = 16, skip_line;
	unsigned_q *prediction, *reference;

	prediction = (unsigned_q *) (video->Mylene_release.Y + offset);
	reference = (unsigned_q *) (video->Irulan_release.Y + offset);
	skip_line = (skipped << 4) + video->luminance_line_size;
	for (; ii; ii--) {
	    unsigned iii;

	    for (iii = skipped; iii; iii--) {
		*--prediction = *--reference;
		*--prediction = *--reference;
# if SIZEOF_LONG != 8
		*--prediction = *--reference;
		*--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
	    }
	    prediction = (unsigned_q *)
		((unsigned char *) prediction + skip_line);
	    reference = (unsigned_q *)
		((unsigned char *) reference + skip_line);
	}
    }
#endif				/* !__RUN_FUTILE__ */
    macroblock->previous_macroblock_address += skipped;
}


static void
GRAYSCALE_macroblocks_1(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    struct seek_h262_field_reference *field_reference;
    unsigned offset, skipped;

    macroblock = (struct seek_h262_13818_macroblock *) video->macroblock;
    if (video->mb_width < macroblock->mb_column + 1) {
	return;
    }
    skipped = macroblock->macroblock_address_increment - 1;
    offset =
	macroblock->luminance_slice_address + (macroblock->mb_column << 4);
    if (video->picture_structure == Bottom_Field) {
	offset += video->luminance_line_size;
	field_reference = video->field_references + 1;
    } else {
	field_reference = video->field_references + 0;
    }
#if !__RUN_FUTILE__
    {
	unsigned ii = 16, skip_line;
	unsigned_q *prediction, *reference;

	prediction = (unsigned_q *) (video->Mylene_release.Y + offset);
	reference = (unsigned_q *) (field_reference->Y + offset);
	skip_line = (skipped << 4) + (video->luminance_line_size << 1);
	for (; ii; ii--) {
	    unsigned iii;

	    for (iii = skipped; iii; iii--) {
		*--prediction = *--reference;
		*--prediction = *--reference;
# if SIZEOF_LONG != 8
		*--prediction = *--reference;
		*--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
	    }
	    prediction = (unsigned_q *)
		((unsigned char *) prediction + skip_line);
	    reference = (unsigned_q *)
		((unsigned char *) reference + skip_line);
	}
    }
#endif				/* !__RUN_FUTILE__ */
    macroblock->previous_macroblock_address += skipped;
}


static void
YCbCr_420_macroblocks_0(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    unsigned offset, skipped;

    macroblock = (struct seek_h262_13818_macroblock *) video->macroblock;
    if (video->mb_width < macroblock->mb_column + 1) {
	return;
    }
    skipped = macroblock->macroblock_address_increment - 1;
    offset =
	macroblock->luminance_slice_address + (macroblock->mb_column << 4);
#if !__RUN_FUTILE__
    {
	unsigned ii = 16, skip_line;
	unsigned_q *prediction, *reference;

	prediction = (unsigned_q *) (video->Mylene_release.Y + offset);
	reference = (unsigned_q *) (video->Irulan_release.Y + offset);
	skip_line = (skipped << 4) + video->luminance_line_size;
	for (; ii; ii--) {
	    unsigned iii;

	    for (iii = skipped; iii; iii--) {
		*--prediction = *--reference;
		*--prediction = *--reference;
# if SIZEOF_LONG != 8
		*--prediction = *--reference;
		*--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
	    }
	    prediction = (unsigned_q *)
		((unsigned char *) prediction + skip_line);
	    reference = (unsigned_q *)
		((unsigned char *) reference + skip_line);
	}
    }
#endif				/* !__RUN_FUTILE__ */
    offset >>= 2;
    offset += macroblock->mb_column << 2;
#if !__RUN_FUTILE__
    {
	unsigned skip_line;

	skip_line = (skipped << 3) + video->chrominance_line_size;
	{
	    unsigned ii = 8;
	    unsigned_q *prediction, *reference;

	    prediction = (unsigned_q *) (video->Mylene_release.Cb + offset);
	    reference = (unsigned_q *) (video->Irulan_release.Cb + offset);
	    for (; ii; ii--) {
		unsigned iii;

		for (iii = skipped; iii; iii--) {
		    *--prediction = *--reference;
# if SIZEOF_LONG != 8
		    *--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
		}
		prediction = (unsigned_q *)
		    ((unsigned char *) prediction + skip_line);
		reference = (unsigned_q *)
		    ((unsigned char *) reference + skip_line);
	    }
	}
	{
	    unsigned ii = 8;
	    unsigned_q *prediction, *reference;

	    prediction = (unsigned_q *) (video->Mylene_release.Cr + offset);
	    reference = (unsigned_q *) (video->Irulan_release.Cr + offset);
	    for (; ii; ii--) {
		unsigned iii;

		for (iii = skipped; iii; iii--) {
		    *--prediction = *--reference;
# if SIZEOF_LONG != 8
		    *--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
		}
		prediction = (unsigned_q *)
		    ((unsigned char *) prediction + skip_line);
		reference = (unsigned_q *)
		    ((unsigned char *) reference + skip_line);
	    }
	}
    }
#endif				/* !__RUN_FUTILE__ */
    macroblock->previous_macroblock_address += skipped;
}


static void
YCbCr_420_macroblocks_1(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    struct seek_h262_field_reference *field_reference;
    unsigned offset_2, offset_4, skipped;

    macroblock = (struct seek_h262_13818_macroblock *) video->macroblock;
    if (video->mb_width < macroblock->mb_column + 1) {
	return;
    }
    skipped = macroblock->macroblock_address_increment - 1;
    offset_4 = macroblock->luminance_slice_address;
    offset_2 = macroblock->mb_column << 4;
    if (video->picture_structure == Bottom_Field) {
	offset_2 += video->luminance_line_size;
	field_reference = video->field_references + 1;
    } else {
	field_reference = video->field_references + 0;
    }
#if !__RUN_FUTILE__
    {
	unsigned ii = 16, skip_line;
	unsigned_q *prediction, *reference;

	skip_line = offset_2 + offset_4;
	prediction = (unsigned_q *) (video->Mylene_release.Y + skip_line);
	reference = (unsigned_q *) (field_reference->Y + skip_line);
	skip_line = (skipped << 4) + (video->luminance_line_size << 1);
	for (; ii; ii--) {
	    unsigned iii;

	    for (iii = skipped; iii; iii--) {
		*--prediction = *--reference;
		*--prediction = *--reference;
# if SIZEOF_LONG != 8
		*--prediction = *--reference;
		*--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
	    }
	    prediction = (unsigned_q *)
		((unsigned char *) prediction + skip_line);
	    reference = (unsigned_q *)
		((unsigned char *) reference + skip_line);
	}
    }
#endif				/* !__RUN_FUTILE__ */
    offset_4 >>= 2;
    offset_2 >>= 1;
    offset_2 += offset_4;
#if !__RUN_FUTILE__
    {
	unsigned skip_line;

	skip_line = (skipped << 3) + (video->chrominance_line_size << 1);
	{
	    unsigned ii = 8;
	    unsigned_q *prediction, *reference;

	    prediction = (unsigned_q *) (video->Mylene_release.Cb + offset_2);
	    reference = (unsigned_q *) (field_reference->Cb + offset_2);
	    for (; ii; ii--) {
		unsigned iii;

		for (iii = skipped; iii; iii--) {
		    *--prediction = *--reference;
# if SIZEOF_LONG != 8
		    *--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
		}
		prediction = (unsigned_q *)
		    ((unsigned char *) prediction + skip_line);
		reference = (unsigned_q *)
		    ((unsigned char *) reference + skip_line);
	    }
	}
	{
	    unsigned ii = 8;
	    unsigned_q *prediction, *reference;

	    prediction = (unsigned_q *) (video->Mylene_release.Cr + offset_2);
	    reference = (unsigned_q *) (field_reference->Cr + offset_2);
	    for (; ii; ii--) {
		unsigned iii;

		for (iii = skipped; iii; iii--) {
		    *--prediction = *--reference;
# if SIZEOF_LONG != 8
		    *--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
		}
		prediction = (unsigned_q *)
		    ((unsigned char *) prediction + skip_line);
		reference = (unsigned_q *)
		    ((unsigned char *) reference + skip_line);
	    }
	}
    }
#endif				/* !__RUN_FUTILE__ */
    macroblock->previous_macroblock_address += skipped;
}


static void
YCbCr_422_macroblocks_0(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    unsigned offset, skipped;

    macroblock = (struct seek_h262_13818_macroblock *) video->macroblock;
    if (video->mb_width < macroblock->mb_column + 1) {
	return;
    }
    skipped = macroblock->macroblock_address_increment - 1;
    offset =
	macroblock->luminance_slice_address + (macroblock->mb_column << 4);
#if !__RUN_FUTILE__
    {
	unsigned ii = 16, skip_line;
	unsigned_q *prediction, *reference;

	prediction = (unsigned_q *) (video->Mylene_release.Y + offset);
	reference = (unsigned_q *) (video->Irulan_release.Y + offset);
	skip_line = (skipped << 4) + video->luminance_line_size;
	for (; ii; ii--) {
	    unsigned iii;

	    for (iii = skipped; iii; iii--) {
		*--prediction = *--reference;
		*--prediction = *--reference;
# if SIZEOF_LONG != 8
		*--prediction = *--reference;
		*--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
	    }
	    prediction = (unsigned_q *)
		((unsigned char *) prediction + skip_line);
	    reference = (unsigned_q *)
		((unsigned char *) reference + skip_line);
	}
    }
#endif				/* !__RUN_FUTILE__ */
    offset >>= 1;
#if !__RUN_FUTILE__
    {
	unsigned skip_line;

	skip_line = (skipped << 3) + video->chrominance_line_size;
	{
	    unsigned ii = 16;
	    unsigned_q *prediction, *reference;

	    prediction =
		(unsigned_q *) (video->Mylene_release.Cb + offset);
	    reference =
		(unsigned_q *) (video->Irulan_release.Cb + offset);
	    for (; ii; ii--) {
		unsigned iii;

		for (iii = skipped; iii; iii--) {
		    *--prediction = *--reference;
# if SIZEOF_LONG != 8
		    *--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
		}
		prediction = (unsigned_q *)
		    ((unsigned char *) prediction + skip_line);
		reference = (unsigned_q *)
		    ((unsigned char *) reference + skip_line);
	    }
	}
	{
	    unsigned ii = 16;
	    unsigned_q *prediction, *reference;

	    prediction =
		(unsigned_q *) (video->Mylene_release.Cr + offset);
	    reference =
		(unsigned_q *) (video->Irulan_release.Cr + offset);
	    for (; ii; ii--) {
		unsigned iii;

		for (iii = skipped; iii; iii--) {
		    *--prediction = *--reference;
# if SIZEOF_LONG != 8
		    *--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
		}
		prediction = (unsigned_q *)
		    ((unsigned char *) prediction + skip_line);
		reference = (unsigned_q *)
		    ((unsigned char *) reference + skip_line);
	    }
	}
    }
#endif				/* !__RUN_FUTILE__ */
    macroblock->previous_macroblock_address += skipped;
}


static void
YCbCr_422_macroblocks_1(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    struct seek_h262_field_reference *field_reference;
    unsigned offset, skipped;

    macroblock = (struct seek_h262_13818_macroblock *) video->macroblock;
    if (video->mb_width < macroblock->mb_column + 1) {
	return;
    }
    skipped = macroblock->macroblock_address_increment - 1;
    offset =
	macroblock->luminance_slice_address + (macroblock->mb_column << 4);
    if (video->picture_structure == Bottom_Field) {
	offset += video->luminance_line_size;
	field_reference = video->field_references + 1;
    } else {
	field_reference = video->field_references + 0;
    }
#if !__RUN_FUTILE__
    {
	unsigned ii = 16, skip_line;
	unsigned_q *prediction, *reference;

	prediction = (unsigned_q *) (video->Mylene_release.Y + offset);
	reference = (unsigned_q *) (field_reference->Y + offset);
	skip_line = (skipped << 4) + (video->luminance_line_size << 1);
	for (; ii; ii--) {
	    unsigned iii;

	    for (iii = skipped; iii; iii--) {
		*--prediction = *--reference;
		*--prediction = *--reference;
# if SIZEOF_LONG != 8
		*--prediction = *--reference;
		*--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
	    }
	    prediction = (unsigned_q *)
		((unsigned char *) prediction + skip_line);
	    reference = (unsigned_q *)
		((unsigned char *) reference + skip_line);
	}
    }
#endif				/* !__RUN_FUTILE__ */
    offset >>= 1;
#if !__RUN_FUTILE__
    {
	unsigned skip_line;

	skip_line = (skipped << 3) + (video->chrominance_line_size << 1);
	{
	    unsigned ii = 16;
	    unsigned_q *prediction, *reference;

	    prediction = (unsigned_q *) (video->Mylene_release.Cb + offset);
	    reference = (unsigned_q *) (field_reference->Cb + offset);
	    for (; ii; ii--) {
		unsigned iii;

		for (iii = skipped; iii; iii--) {
		    *--prediction = *--reference;
# if SIZEOF_LONG != 8
		    *--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
		}
		prediction = (unsigned_q *)
		    ((unsigned char *) prediction + skip_line);
		reference = (unsigned_q *)
		    ((unsigned char *) reference + skip_line);
	    }
	}
	{
	    unsigned ii = 16;
	    unsigned_q *prediction, *reference;

	    prediction = (unsigned_q *) (video->Mylene_release.Cr + offset);
	    reference = (unsigned_q *) (field_reference->Cr + offset);
	    for (; ii; ii--) {
		unsigned iii;

		for (iii = skipped; iii; iii--) {
		    *--prediction = *--reference;
# if SIZEOF_LONG != 8
		    *--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
		}
		prediction = (unsigned_q *)
		    ((unsigned char *) prediction + skip_line);
		reference = (unsigned_q *)
		    ((unsigned char *) reference + skip_line);
	    }
	}
    }
#endif				/* !__RUN_FUTILE__ */
    macroblock->previous_macroblock_address += skipped;
}


static void
YCbCr_444_macroblocks_0(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    unsigned offset, skip_line, skipped;

    macroblock = (struct seek_h262_13818_macroblock *) video->macroblock;
    if (video->mb_width < macroblock->mb_column + 1) {
	return;
    }
    skipped = macroblock->macroblock_address_increment - 1;
    offset =
	macroblock->luminance_slice_address + (macroblock->mb_column << 4);
    skip_line = (skipped << 4) + video->luminance_line_size;
#if !__RUN_FUTILE__
    {
	unsigned ii = 16;
	unsigned_q *prediction, *reference;

	prediction = (unsigned_q *) (video->Mylene_release.Y + offset);
	reference = (unsigned_q *) (video->Irulan_release.Y + offset);
	for (; ii; ii--) {
	    unsigned iii;

	    for (iii = skipped; iii; iii--) {
		*--prediction = *--reference;
		*--prediction = *--reference;
# if SIZEOF_LONG != 8
		*--prediction = *--reference;
		*--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
	    }
	    prediction = (unsigned_q *)
		((unsigned char *) prediction + skip_line);
	    reference = (unsigned_q *)
		((unsigned char *) reference + skip_line);
	}
    }
    {
	unsigned ii = 16;
	unsigned_q *prediction, *reference;

	prediction = (unsigned_q *) (video->Mylene_release.Cb + offset);
	reference = (unsigned_q *) (video->Irulan_release.Cb + offset);
	for (; ii; ii--) {
	    unsigned iii;

	    for (iii = skipped; iii; iii--) {
		*--prediction = *--reference;
		*--prediction = *--reference;
# if SIZEOF_LONG != 8
		*--prediction = *--reference;
		*--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
	    }
	    prediction = (unsigned_q *)
		((unsigned char *) prediction + skip_line);
	    reference = (unsigned_q *)
		((unsigned char *) reference + skip_line);
	}
    }
    {
	unsigned ii = 16;
	unsigned_q *prediction, *reference;

	prediction = (unsigned_q *) (video->Mylene_release.Cr + offset);
	reference = (unsigned_q *) (video->Irulan_release.Cr + offset);
	for (; ii; ii--) {
	    unsigned iii;

	    for (iii = skipped; iii; iii--) {
		*--prediction = *--reference;
		*--prediction = *--reference;
# if SIZEOF_LONG != 8
		*--prediction = *--reference;
		*--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
	    }
	    prediction = (unsigned_q *)
		((unsigned char *) prediction + skip_line);
	    reference = (unsigned_q *)
		((unsigned char *) reference + skip_line);
	}
    }
#endif				/* !__RUN_FUTILE__ */
    macroblock->previous_macroblock_address += skipped;
}


static void
YCbCr_444_macroblocks_1(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    struct seek_h262_field_reference *field_reference;
    unsigned offset, skip_line, skipped;

    macroblock = (struct seek_h262_13818_macroblock *) video->macroblock;
    if (video->mb_width < macroblock->mb_column + 1) {
	return;
    }
    skipped = macroblock->macroblock_address_increment - 1;
    offset =
	macroblock->luminance_slice_address + (macroblock->mb_column << 4);
    if (video->picture_structure == Bottom_Field) {
	offset += video->luminance_line_size;
	field_reference = video->field_references + 1;
    } else {
	field_reference = video->field_references + 0;
    }
    skip_line = (skipped << 4) + (video->luminance_line_size << 1);
#if !__RUN_FUTILE__
    {
	unsigned ii = 16;
	unsigned_q *prediction, *reference;

	prediction = (unsigned_q *) (video->Mylene_release.Y + offset);
	reference = (unsigned_q *) (field_reference->Y + offset);
	for (; ii; ii--) {
	    unsigned iii;

	    for (iii = skipped; iii; iii--) {
		*--prediction = *--reference;
		*--prediction = *--reference;
# if SIZEOF_LONG != 8
		*--prediction = *--reference;
		*--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
	    }
	    prediction = (unsigned_q *)
		((unsigned char *) prediction + skip_line);
	    reference = (unsigned_q *)
		((unsigned char *) reference + skip_line);
	}
    }
    {
	unsigned ii = 16;
	unsigned_q *prediction, *reference;

	prediction = (unsigned_q *) (video->Mylene_release.Cb + offset);
	reference = (unsigned_q *) (field_reference->Cb + offset);
	for (; ii; ii--) {
	    unsigned iii;

	    for (iii = skipped; iii; iii--) {
		*--prediction = *--reference;
		*--prediction = *--reference;
# if SIZEOF_LONG != 8
		*--prediction = *--reference;
		*--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
	    }
	    prediction = (unsigned_q *)
		((unsigned char *) prediction + skip_line);
	    reference = (unsigned_q *)
		((unsigned char *) reference + skip_line);
	}
    }
    {
	unsigned ii = 16;
	unsigned_q *prediction, *reference;

	prediction = (unsigned_q *) (video->Mylene_release.Cr + offset);
	reference = (unsigned_q *) (field_reference->Cr + offset);
	for (; ii; ii--) {
	    unsigned iii;

	    for (iii = skipped; iii; iii--) {
		*--prediction = *--reference;
		*--prediction = *--reference;
# if SIZEOF_LONG != 8
		*--prediction = *--reference;
		*--prediction = *--reference;
# endif				/* SIZEOF_LONG != 8 */
	    }
	    prediction = (unsigned_q *)
		((unsigned char *) prediction + skip_line);
	    reference = (unsigned_q *)
		((unsigned char *) reference + skip_line);
	}
    }
#endif				/* !__RUN_FUTILE__ */
    macroblock->previous_macroblock_address += skipped;
}


void
_seek_h262___________macroblocks(struct seek_h262_video *video)
{
    struct seek_h262_13818_macroblock *macroblock;
    unsigned i = 1, skipped;

    macroblock = (struct seek_h262_13818_macroblock *) video->macroblock;
    macroblock->dc_sparse = 0;
    macroblock->pattern_code = 0;
    skipped = macroblock->macroblock_address_increment;
    macroblock->macroblock_address -= skipped - 1;
    macroblock->mb_column -= skipped - 1;
    if (macroblock->packed_flags & macroblock_motion_forward_mask) {
	macroblock->motion_vertical_field_select_0_0 =
	    video->picture_structure >> 1;
	macroblock->vector_0_0_0 = macroblock->PMV_0_0_0;
	macroblock->vector_0_0_1 = macroblock->PMV_0_0_1;
    }
    if (macroblock->packed_flags & macroblock_motion_backward_mask) {
	macroblock->motion_vertical_field_select_0_1 =
	    video->picture_structure >> 1;
	macroblock->vector_0_1_0 = macroblock->PMV_0_1_0;
	macroblock->vector_0_1_1 = macroblock->PMV_0_1_1;
    }
    for (; i < skipped; i++) {
	video->______macroblock
	    [video->picture_structure == Frame_picture
	     ? Frame_based_prediction : Field_based_prediction] (video);
	macroblock->macroblock_address++;
	macroblock->mb_column++;
    }
}
