/*
 * h262.b.11172.c
 * Copyright (C) 2003, 2004, 2005, 2006, 2007  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.2.h>
#include <h262.3.h>
#include <h262.4.h>
#include <h262.6.d4.h>
#include <h262.a.h>

#if 0
# define CHECK_MOTION_LIMITS(half_flags, offset) \
    {									      \
	if ((int) (offset) < 0						      \
	    || video->motion_limit[half_flags] < (offset)) {		      \
	    return;							      \
	}								      \
    }
#else
# define CHECK_MOTION_LIMITS(half_flags, offset) \
    {									      \
	if (video->motion_limit[(half_flags)] < (offset)) {		      \
	    return;							      \
	}								      \
    }
#endif				/* 0 */

static void macroblock_0(struct seek_h262_video *);
static void macroblock_1(struct seek_h262_video *);
static void macroblock_2(struct seek_h262_video *);
static void macroblock_3(struct seek_h262_video *);
static void macroblock_4(struct seek_h262_video *);
static void macroblock_7(struct seek_h262_video *);

void (*_seek_h262_11172_YCbCr_____macroblock[]) (struct seek_h262_video *) = {
/* *INDENT-OFF* */
    macroblock_0,
    macroblock_1,
    macroblock_2,
    macroblock_3,
    macroblock_4,
    NULL,
    NULL,
    macroblock_7
/* *INDENT-ON* */
};

static const unsigned reflex64[] = {
/* *INDENT-OFF* */
    0x000, 0x400, 0x100, 0x500, 0x040, 0x440, 0x140, 0x540,
    0x010, 0x410, 0x110, 0x510, 0x050, 0x450, 0x150, 0x550,
    0x004, 0x404, 0x104, 0x504, 0x044, 0x444, 0x144, 0x544,
    0x014, 0x414, 0x114, 0x514, 0x054, 0x454, 0x154, 0x554,
    0x001, 0x401, 0x101, 0x501, 0x041, 0x441, 0x141, 0x541,
    0x011, 0x411, 0x111, 0x511, 0x051, 0x451, 0x151, 0x551,
    0x005, 0x405, 0x105, 0x505, 0x045, 0x445, 0x145, 0x545,
    0x015, 0x415, 0x115, 0x515, 0x055, 0x455, 0x155, 0x555,
/* *INDENT-ON* */
};

static void
macroblock_0(struct seek_h262_video *video)
{
    struct seek_h262_11172_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned line_size, offset, pattern_code;

    macroblock = video->macroblock;
    if (!(macroblock->macroblock_address < video->macroblocks_count)) {
	return;
    }
    macroblock->previous_macroblock_address = macroblock->macroblock_address;
    blocks = video->blocks;
    line_size = video->luminance_line_size;
    pattern_code = macroblock->pattern_code;
    pattern_code |= macroblock->dc_sparse << 1;
    {
	seek_h262_constructor constructor;
	unsigned char *predictions, *references;

#if !__Berkeley_FUCK_UP__
	constructor = *video->block_0_3;
#else
	constructor = *video->block_0_3[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	offset = video->mb_offset;
	predictions = video->Mylene_release.Y + offset;
	references = video->Iran_release.Y + offset;
	constructor(predictions, references, line_size);
	video->idct_add_block(blocks, pattern_code & 255, offset, line_size);
#if !__Berkeley_FUCK_UP__
	constructor = *video->block_0_0;
#else
	constructor = *video->block_0_0[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	offset = video->mb_chroma;
	line_size >>= 1;
	constructor(video->Mylene_release.Cb + offset,
		    video->Iran_release.Cb + offset, line_size);
	constructor(video->Mylene_release.Cr + offset,
		    video->Iran_release.Cr + offset, line_size);
	video->idct_add_block
	    (blocks + 4, pattern_code >> 8, offset, line_size);
    }
}


static void
macroblock_1(struct seek_h262_video *video)
{
    struct seek_h262_11172_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned c_offset, line_size, pattern_code, y_offset;

    macroblock = video->macroblock;
    if (!(macroblock->macroblock_address < video->macroblocks_count)) {
	return;
    }
    macroblock->previous_macroblock_address = macroblock->macroblock_address;
    blocks = video->blocks;
    line_size = video->luminance_line_size;
    pattern_code = macroblock->pattern_code;
    pattern_code |= macroblock->dc_sparse << 1;
    y_offset = video->mb_offset;
    c_offset = video->mb_chroma;
    {
	seek_h262_constructor constructor;
	unsigned chrominance_half_flags, luminance_half_flags;
	unsigned char *Cb_references, *Cr_references, *Y_references,
	    *predictions;

	{
	    int vector_0_x_0, vector_0_x_1;

	    {
		Cb_references = video->Iran_release.Cb;
		Cr_references = video->Iran_release.Cr;
		Y_references = video->Iran_release.Y;
		vector_0_x_0 = macroblock->recon_right_for;
		vector_0_x_1 = macroblock->recon_down_for;
	    }
	    if (vector_0_x_0 | vector_0_x_1) {
		unsigned offset;

		offset = y_offset + (vector_0_x_0 >> 1)
		    + (vector_0_x_1 >> 1) * line_size;
		luminance_half_flags =
		    (vector_0_x_0 & 1) | (vector_0_x_1 & 1) << 1;
		CHECK_MOTION_LIMITS(luminance_half_flags, offset);
		Y_references += offset;
		vector_0_x_0 /= 2;
		vector_0_x_1 /= 2;
		offset = c_offset + (vector_0_x_0 >> 1)
		    + (vector_0_x_1 >> 1) * (line_size >> 1);
		Cb_references += offset;
		Cr_references += offset;
		chrominance_half_flags =
		    (vector_0_x_0 & 1) | (vector_0_x_1 & 1) << 1;
	    } else {
		Cb_references += c_offset;
		Cr_references += c_offset;
		Y_references += y_offset;
		chrominance_half_flags = 0;
		luminance_half_flags = 0;
	    }
	}
#if !__Berkeley_FUCK_UP__
	constructor = video->block_0_3[luminance_half_flags];
#else
	constructor = video->block_0_3[0][luminance_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	predictions = video->Mylene_release.Y + y_offset;
	constructor(predictions, Y_references, line_size);
	video->idct_add_block(blocks, pattern_code & 255, y_offset, line_size);
#if !__Berkeley_FUCK_UP__
	constructor = video->block_0_0[chrominance_half_flags];
#else
	constructor = video->block_0_0[0][chrominance_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	line_size >>= 1;
	constructor
	    (video->Mylene_release.Cb + c_offset, Cb_references, line_size);
	constructor
	    (video->Mylene_release.Cr + c_offset, Cr_references, line_size);
	video->idct_add_block
	    (blocks + 4, pattern_code >> 8, c_offset, line_size);
    }
}


static void
macroblock_2(struct seek_h262_video *video)
{
    struct seek_h262_11172_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned c_offset, line_size, pattern_code, y_offset;

    macroblock = video->macroblock;
    if (!(macroblock->macroblock_address < video->macroblocks_count)) {
	return;
    }
    macroblock->previous_macroblock_address = macroblock->macroblock_address;
    blocks = video->blocks;
    line_size = video->luminance_line_size;
    pattern_code = macroblock->pattern_code;
    pattern_code |= macroblock->dc_sparse << 1;
    y_offset = video->mb_offset;
    c_offset = video->mb_chroma;
    {
	seek_h262_constructor constructor;
	unsigned chrominance_half_flags, luminance_half_flags;
	unsigned char *Cb_references, *Cr_references, *Y_references,
	    *predictions;

	{
	    int vector_0_x_0, vector_0_x_1;

	    {
		Cb_references = video->Irulan_release.Cb;
		Cr_references = video->Irulan_release.Cr;
		Y_references = video->Irulan_release.Y;
		vector_0_x_0 = macroblock->recon_right_back;
		vector_0_x_1 = macroblock->recon_down_back;
	    }
	    if (vector_0_x_0 | vector_0_x_1) {
		unsigned offset;

		offset = y_offset + (vector_0_x_0 >> 1)
		    + (vector_0_x_1 >> 1) * line_size;
		luminance_half_flags =
		    (vector_0_x_0 & 1) | (vector_0_x_1 & 1) << 1;
		CHECK_MOTION_LIMITS(luminance_half_flags, offset);
		Y_references += offset;
		vector_0_x_0 /= 2;
		vector_0_x_1 /= 2;
		offset = c_offset + (vector_0_x_0 >> 1)
		    + (vector_0_x_1 >> 1) * (line_size >> 1);
		Cb_references += offset;
		Cr_references += offset;
		chrominance_half_flags =
		    (vector_0_x_0 & 1) | (vector_0_x_1 & 1) << 1;
	    } else {
		Cb_references += c_offset;
		Cr_references += c_offset;
		Y_references += y_offset;
		chrominance_half_flags = 0;
		luminance_half_flags = 0;
	    }
	}
#if !__Berkeley_FUCK_UP__
	constructor = video->block_0_3[luminance_half_flags];
#else
	constructor = video->block_0_3[0][luminance_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	predictions = video->Mylene_release.Y + y_offset;
	constructor(predictions, Y_references, line_size);
	video->idct_add_block(blocks, pattern_code & 255, y_offset, line_size);
#if !__Berkeley_FUCK_UP__
	constructor = video->block_0_0[chrominance_half_flags];
#else
	constructor = video->block_0_0[0][chrominance_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	line_size >>= 1;
	constructor
	    (video->Mylene_release.Cb + c_offset, Cb_references, line_size);
	constructor
	    (video->Mylene_release.Cr + c_offset, Cr_references, line_size);
	video->idct_add_block
	    (blocks + 4, pattern_code >> 8, c_offset, line_size);
    }
}


static void
macroblock_3(struct seek_h262_video *video)
{
    struct seek_h262_11172_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned c_offset, line_size, pattern_code, y_offset;

    macroblock = video->macroblock;
    if (!(macroblock->macroblock_address < video->macroblocks_count)) {
	return;
    }
    macroblock->previous_macroblock_address = macroblock->macroblock_address;
    blocks = video->blocks;
    line_size = video->luminance_line_size;
    pattern_code = macroblock->pattern_code;
    pattern_code |= macroblock->dc_sparse << 1;
    y_offset = video->mb_offset;
    c_offset = video->mb_chroma;
    {
	seek_h262_constructOr constructOr;
	unsigned chrominance_half_flags = 0, luminance_half_flags = 0;
	unsigned char *Cb_references_0, *Cb_references_1, *Cr_references_0,
	    *Cr_references_1, *Y_references_0, *Y_references_1, *predictions;

	{
	    int vector_0_1_0, vector_0_1_1;

	    Cb_references_0 = video->Irulan_release.Cb;
	    Cr_references_0 = video->Irulan_release.Cr;
	    Y_references_0 = video->Irulan_release.Y;
	    vector_0_1_0 = macroblock->recon_right_back;
	    vector_0_1_1 = macroblock->recon_down_back;
	    if (vector_0_1_0 | vector_0_1_1) {
		unsigned offset;

		offset = y_offset + (vector_0_1_0 >> 1)
		    + (vector_0_1_1 >> 1) * line_size;
		luminance_half_flags =
		    (vector_0_1_0 & 1) | (vector_0_1_1 & 1) << 1;
		CHECK_MOTION_LIMITS(luminance_half_flags, offset);
		Y_references_0 += offset;
		luminance_half_flags <<= 2;
		vector_0_1_0 /= 2;
		vector_0_1_1 /= 2;
		offset = c_offset + (vector_0_1_0 >> 1)
		    + (vector_0_1_1 >> 1) * (line_size >> 1);
		Cb_references_0 += offset;
		Cr_references_0 += offset;
		chrominance_half_flags =
		    ((vector_0_1_0 & 1) | (vector_0_1_1 & 1) << 1) << 2;
	    } else {
		Cb_references_0 += c_offset;
		Cr_references_0 += c_offset;
		Y_references_0 += y_offset;
	    }
	}
	{
	    int vector_0_0_0, vector_0_0_1;

	    Cb_references_1 = video->Iran_release.Cb;
	    Cr_references_1 = video->Iran_release.Cr;
	    Y_references_1 = video->Iran_release.Y;
	    vector_0_0_0 = macroblock->recon_right_for;
	    vector_0_0_1 = macroblock->recon_down_for;
	    if (vector_0_0_0 | vector_0_0_1) {
		unsigned offset;

		offset = y_offset + (vector_0_0_0 >> 1)
		    + (vector_0_0_1 >> 1) * line_size;
		luminance_half_flags |=
		    (vector_0_0_0 & 1) | (vector_0_0_1 & 1) << 1;
		CHECK_MOTION_LIMITS(luminance_half_flags & 3, offset);
		Y_references_1 += offset;
		vector_0_0_0 /= 2;
		vector_0_0_1 /= 2;
		offset = c_offset + (vector_0_0_0 >> 1)
		    + (vector_0_0_1 >> 1) * (line_size >> 1);
		Cb_references_1 += offset;
		Cr_references_1 += offset;
		chrominance_half_flags |=
		    (vector_0_0_0 & 1) | (vector_0_0_1 & 1) << 1;
	    } else {
		Cb_references_1 += c_offset;
		Cr_references_1 += c_offset;
		Y_references_1 += y_offset;
	    }
	}
#if !__Berkeley_FUCK_UP__
	constructOr = video->block_1_3[luminance_half_flags];
#else
	constructOr = video->block_1_3[0][luminance_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	predictions = video->Mylene_release.Y + y_offset;
	constructOr(predictions, Y_references_0, Y_references_1, line_size);
	video->idct_add_block(blocks, pattern_code & 255, y_offset, line_size);
#if !__Berkeley_FUCK_UP__
	constructOr = video->block_1_0[chrominance_half_flags];
#else
	constructOr = video->block_1_0[0][chrominance_half_flags];
#endif				/* !__Berkeley_FUCK_UP__ */
	line_size >>= 1;
	constructOr(video->Mylene_release.Cb + c_offset, Cb_references_0,
		    Cb_references_1, line_size);
	constructOr(video->Mylene_release.Cr + c_offset, Cr_references_0,
		    Cr_references_1, line_size);
	video->idct_add_block
	    (blocks + 4, pattern_code >> 8, c_offset, line_size);
    }
}


static void
macroblock_4(struct seek_h262_video *video)
{
    unsigned offset;

    {
	struct seek_h262_11172_macroblock *macroblock;

	macroblock = video->macroblock;
	if (!(macroblock->macroblock_address < video->macroblocks_count)) {
	    return;
	}
	offset = macroblock->macroblock_address;
	macroblock->previous_macroblock_address = offset;
    }
    {
	struct seek_h262_block *blocks;
	unsigned line_size;

	blocks = video->blocks;
	line_size = video->luminance_line_size;
	offset = video->mb_offset;
	video->idct_set_block(blocks, 4, offset, line_size);
	offset = video->mb_chroma;
	video->idct_set_block(blocks + 4, 2, offset, line_size >> 1);
    }
}


static void
macroblock_7(struct seek_h262_video *video)
{
    seek_h262_coefficient *coefficients;
    unsigned offset, *output;

    {
	struct seek_h262_11172_macroblock *macroblock;

	macroblock = video->macroblock;
	if (!(macroblock->macroblock_address < video->macroblocks_count)) {
	    return;
	} {
	    offset = macroblock->macroblock_address;
	    macroblock->previous_macroblock_address = offset;
	    offset = video->mb_offset;
	    offset >>= 2;
	    output = (unsigned *) video->Mylene_release.Y + offset;
	    offset = video->mb_chroma;
	    offset >>= 2;
	}
	coefficients = macroblock->coefficients;
    }
    {
	unsigned block_0, block_1, i = 8, line_size;

	line_size = (video->luminance_line_size - 16) >> 2;
	block_0 = coefficients[_1ST_BLOCK_OFFSET] >> 3;
	block_0 |= block_0 << 010;
	block_0 |= block_0 << 020;
	block_1 = coefficients[_2ND_BLOCK_OFFSET] >> 3;
	block_1 |= block_1 << 010;
	block_1 |= block_1 << 020;
	for (; i; i--) {
	    *output++ = block_0;
	    *output++ = block_0;
	    *output++ = block_1;
	    *output++ = block_1;
	    output += line_size;
	}
	block_0 = coefficients[_3RD_BLOCK_OFFSET] >> 3;
	block_0 |= block_0 << 010;
	block_0 |= block_0 << 020;
	block_1 = coefficients[_4TH_BLOCK_OFFSET] >> 3;
	block_1 |= block_1 << 010;
	block_1 |= block_1 << 020;
	for (i = 8; i; i--) {
	    *output++ = block_0;
	    *output++ = block_0;
	    *output++ = block_1;
	    *output++ = block_1;
	    output += line_size;
	}
	line_size >>= 1;
	output = (unsigned *) video->Mylene_release.Cb + offset;
	block_0 = coefficients[_5TH_BLOCK_OFFSET] >> 3;
	block_0 |= block_0 << 010;
	block_0 |= block_0 << 020;
	for (i = 8; i; i--) {
	    *output++ = block_0;
	    *output++ = block_0;
	    output += line_size;
	}
	output = (unsigned *) video->Mylene_release.Cr + offset;
	block_1 = coefficients[_6TH_BLOCK_OFFSET] >> 3;
	block_1 |= block_1 << 010;
	block_1 |= block_1 << 020;
	for (i = 8; i; i--) {
	    *output++ = block_1;
	    *output++ = block_1;
	    output += line_size;
	}
    }
}
