/*
 * h262.d.11172.c
 * Copyright (C) 2003, 2004, 2005, 2006, 2007  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.2.h>
#include <h262.3.h>
#include <h262.4.h>
#include <h262.6.d4.h>
#include <h262.a.h>

#if 0
# define CHECK_MOTION_LIMITS(half_flags, offset) \
    {									      \
	if ((int) (offset) < 0						      \
	    || video->motion_limit[half_flags] < (offset)) {		      \
	    return;							      \
	}								      \
    }
#else
# define CHECK_MOTION_LIMITS(half_flags, offset) \
    {									      \
	if (video->motion_limit[(half_flags)] < (offset)) {		      \
	    return;							      \
	}								      \
    }
#endif				/* 0 */

static void macroblock_0(struct seek_h262_video *);
static void macroblock_1(struct seek_h262_video *);
static void macroblock_2(struct seek_h262_video *);
static void macroblock_3(struct seek_h262_video *);
static void macroblock_4(struct seek_h262_video *);
static void macroblock_7(struct seek_h262_video *);

void (*_seek_h262_11172_GRAYSCALE_macroblock[]) (struct seek_h262_video *) = {
/* *INDENT-OFF* */
    macroblock_0,
    macroblock_1,
    macroblock_2,
    macroblock_3,
    macroblock_4,
    NULL,
    NULL,
    macroblock_7
/* *INDENT-ON* */
};

static void
macroblock_0(struct seek_h262_video *video)
{
    struct seek_h262_11172_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned line_size, offset, pattern_code;

    macroblock = video->macroblock;
    if (!(macroblock->macroblock_address < video->macroblocks_count)) {
	return;
    }
    offset = macroblock->macroblock_address;
    macroblock->previous_macroblock_address = offset;
    blocks = video->blocks;
    line_size = video->luminance_line_size;
    pattern_code = macroblock->pattern_code;
    pattern_code |= macroblock->dc_sparse << 1;
    {
	seek_h262_constructor constructor;

#if !__Berkeley_FUCK_UP__
	constructor = *video->block_0_3;
#else
	constructor = *video->block_0_3[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	offset = video->mb_offset;
	constructor(video->Mylene_release.Y + offset,
		    video->Iran_release.Y + offset, line_size);
	video->idct_add_block(blocks, pattern_code & 255, offset, line_size);
    }
}


static void
macroblock_1(struct seek_h262_video *video)
{
    struct seek_h262_11172_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned line_size, offset, pattern_code;

    macroblock = video->macroblock;
    if (!(macroblock->macroblock_address < video->macroblocks_count)) {
	return;
    }
    offset = macroblock->macroblock_address;
    macroblock->previous_macroblock_address = offset;
    blocks = video->blocks;
    line_size = video->luminance_line_size;
    pattern_code = macroblock->pattern_code;
    pattern_code |= macroblock->dc_sparse << 1;
    {
	seek_h262_constructor *constructor;
	unsigned char *references;

#if !__Berkeley_FUCK_UP__
	constructor = video->block_0_3;
#else
	constructor = video->block_0_3[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	offset = video->mb_offset;
	{
	    int vector_0_x_0, vector_0_x_1;

	    {
		references = video->Iran_release.Y;
		vector_0_x_0 = macroblock->recon_right_for;
		vector_0_x_1 = macroblock->recon_down_for;
	    }
	    if (vector_0_x_0 | vector_0_x_1) {
		unsigned half_flags, h_offset;

		h_offset = offset + (vector_0_x_0 >> 1)
		    + (vector_0_x_1 >> 1) * line_size;
		half_flags = (vector_0_x_0 & 1) | (vector_0_x_1 & 1) << 1;
		CHECK_MOTION_LIMITS(half_flags, h_offset);
		references += h_offset;
		constructor += half_flags;
	    } else {
		references += offset;
	    }
	}
	(*constructor)
	    (video->Mylene_release.Y + offset, references, line_size);
	video->idct_add_block(blocks, pattern_code & 255, offset, line_size);
    }
}


static void
macroblock_2(struct seek_h262_video *video)
{
    struct seek_h262_11172_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned line_size, offset, pattern_code;

    macroblock = video->macroblock;
    if (!(macroblock->macroblock_address < video->macroblocks_count)) {
	return;
    }
    offset = macroblock->macroblock_address;
    macroblock->previous_macroblock_address = offset;
    blocks = video->blocks;
    line_size = video->luminance_line_size;
    pattern_code = macroblock->pattern_code;
    pattern_code |= macroblock->dc_sparse << 1;
    {
	seek_h262_constructor *constructor;
	unsigned char *references;

#if !__Berkeley_FUCK_UP__
	constructor = video->block_0_3;
#else
	constructor = video->block_0_3[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	offset = video->mb_offset;
	{
	    int vector_0_x_0, vector_0_x_1;

	    {
		references = video->Irulan_release.Y;
		vector_0_x_0 = macroblock->recon_right_back;
		vector_0_x_1 = macroblock->recon_down_back;
	    }
	    if (vector_0_x_0 | vector_0_x_1) {
		unsigned half_flags, h_offset;

		h_offset = offset + (vector_0_x_0 >> 1)
		    + (vector_0_x_1 >> 1) * line_size;
		half_flags = (vector_0_x_0 & 1) | (vector_0_x_1 & 1) << 1;
		CHECK_MOTION_LIMITS(half_flags, h_offset);
		references += h_offset;
		constructor += half_flags;
	    } else {
		references += offset;
	    }
	}
	(*constructor)
	    (video->Mylene_release.Y + offset, references, line_size);
	video->idct_add_block(blocks, pattern_code & 255, offset, line_size);
    }
}


static void
macroblock_3(struct seek_h262_video *video)
{
    struct seek_h262_11172_macroblock *macroblock;
    struct seek_h262_block *blocks;
    unsigned line_size, offset, pattern_code;

    macroblock = video->macroblock;
    if (!(macroblock->macroblock_address < video->macroblocks_count)) {
	return;
    }
    offset = macroblock->macroblock_address;
    macroblock->previous_macroblock_address = offset;
    blocks = video->blocks;
    line_size = video->luminance_line_size;
    pattern_code = macroblock->pattern_code;
    pattern_code |= macroblock->dc_sparse << 1;
    {
	seek_h262_constructOr *constructOr;
	unsigned char *references_0, *references_1;

#if !__Berkeley_FUCK_UP__
	constructOr = video->block_1_3;
#else
	constructOr = video->block_1_3[0];
#endif				/* !__Berkeley_FUCK_UP__ */
	offset = video->mb_offset;
	{
	    int vector_0_1_0, vector_0_1_1;

	    references_0 = video->Irulan_release.Y;
	    vector_0_1_0 = macroblock->recon_right_back;
	    vector_0_1_1 = macroblock->recon_down_back;
	    if (vector_0_1_0 | vector_0_1_1) {
		unsigned half_flags, h_offset;

		h_offset = offset + (vector_0_1_0 >> 1)
		    + (vector_0_1_1 >> 1) * line_size;
		half_flags = (vector_0_1_0 & 1) | (vector_0_1_1 & 1) << 1;
		CHECK_MOTION_LIMITS(half_flags, h_offset);
		references_0 += h_offset;
		constructOr += half_flags << 2;
	    } else {
		references_0 += offset;
	    }
	}
	{
	    int vector_0_0_0, vector_0_0_1;

	    references_1 = video->Iran_release.Y;
	    vector_0_0_0 = macroblock->recon_right_for;
	    vector_0_0_1 = macroblock->recon_down_for;
	    if (vector_0_0_0 | vector_0_0_1) {
		unsigned half_flags, h_offset;

		h_offset = offset + (vector_0_0_0 >> 1)
		    + (vector_0_0_1 >> 1) * line_size;
		half_flags = (vector_0_0_0 & 1) | (vector_0_0_1 & 1) << 1;
		CHECK_MOTION_LIMITS(half_flags, h_offset);
		references_1 += h_offset;
		constructOr += half_flags;
	    } else {
		references_1 += offset;
	    }
	}
	(*constructOr)(video->Mylene_release.Y + offset, references_0,
		       references_1, line_size);
	video->idct_add_block(blocks, pattern_code & 255, offset, line_size);
    }
}


static void
macroblock_4(struct seek_h262_video *video)
{
    unsigned offset;

    {
	struct seek_h262_11172_macroblock *macroblock;

	macroblock = video->macroblock;
	if (!(macroblock->macroblock_address < video->macroblocks_count)) {
	    return;
	}
	offset = macroblock->macroblock_address;
	macroblock->previous_macroblock_address = offset;
	offset = video->mb_offset;
    }
    {
	video->idct_set_block
	    (video->blocks, 4, offset, video->luminance_line_size);
    }
}


static void
macroblock_7(struct seek_h262_video *video)
{
    seek_h262_coefficient *coefficients;
    unsigned *output;

    {
	struct seek_h262_11172_macroblock *macroblock;

	macroblock = video->macroblock;
	if (!(macroblock->macroblock_address < video->macroblocks_count)) {
	    return;
	} {
	    unsigned offset;

	    offset = macroblock->macroblock_address;
	    macroblock->previous_macroblock_address = offset;
	    output = (unsigned *) video->Mylene_release.Y
		+ (video->mb_offset >> 2);
	}
	coefficients = macroblock->coefficients;
    }
    {
	unsigned block_0, block_1, i = 8, line_size;

	line_size = (video->luminance_line_size - 16) >> 2;
	block_0 = coefficients[_1ST_BLOCK_OFFSET] >> 3;
	block_0 |= block_0 << 010;
	block_0 |= block_0 << 020;
	block_1 = coefficients[_2ND_BLOCK_OFFSET] >> 3;
	block_1 |= block_1 << 010;
	block_1 |= block_1 << 020;
	for (; i; i--) {
	    *output++ = block_0;
	    *output++ = block_0;
	    *output++ = block_1;
	    *output++ = block_1;
	    output += line_size;
	}
	block_0 = coefficients[_3RD_BLOCK_OFFSET] >> 3;
	block_0 |= block_0 << 010;
	block_0 |= block_0 << 020;
	block_1 = coefficients[_4TH_BLOCK_OFFSET] >> 3;
	block_1 |= block_1 << 010;
	block_1 |= block_1 << 020;
	for (i = 8; i; i--) {
	    *output++ = block_0;
	    *output++ = block_0;
	    *output++ = block_1;
	    *output++ = block_1;
	    output += line_size;
	}
    }
}
