/*
 * h262.e.part3.c
 * Copyright (C) 2003, 2004, 2005, 2006  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.1.h>
#include <h262.2.h>
#include <h262.3.h>
#include <h262.4.h>
#include <h262.a.h>
#include <h262.e.h>

#define X08_				(intra_coded)
#define X08q				(intra_coded | 4)
#define X10_				(predictive_coded)
#define X10q				(predictive_coded | 4)
#define X18_				(bidirectionally_predictive_coded)
#define X18q				(bidirectionally_predictive_coded | 4)
#define X20_				(dc_intra_coded)

static const char *picture_coding_type_descriptions[] = {
/* *INDENT-OFF* */
    "intra_coded",
    "predictive_coded",
    "bidirectionally_predictive_coded",
    "dc_intra_coded"
/* *INDENT-ON* */
};

void
seek_h262_get_do_release(struct seek_h262_video *video,
			 void (**do_release) (void *,
			                      struct seek_h262_release *))
{
    *do_release =
	(void (*)(void *, struct seek_h262_release *)) video->do_release;
}


void
seek_h262_get_future_release(struct seek_h262_video *video,
			     struct seek_h262_release **release)
{
    *release = &video->Irulan_release;
}


void
seek_h262_get_inverse_scans(struct seek_h262_video *video,
			    unsigned *inverse_scan_0, unsigned *inverse_scan_1)
{
    unsigned j = 0;

    for (; j < 2; j++) {
	unsigned *major;

	major = j ? inverse_scan_1 : inverse_scan_0;
	if (major) {
	    seek_h262_scan_index *minor;
	    unsigned i = 64;

	    minor = video->intra_quantiser_matrix[j];

	    for (; i; i--) {
		*major++ = *minor++;
	    }
	}
    }
}


void
seek_h262_get_past_release(struct seek_h262_video *video,
			   struct seek_h262_release **release)
{
    *release = &video->Iran_release;
}


void
seek_h262_get_picture_coding_type(struct seek_h262_video *video,
				  unsigned *picture_coding_type)
{
    *picture_coding_type = video->picture_coding_type;
}


void
seek_h262_get_releases(struct seek_h262_video *video,
		       struct seek_h262_release **releases)
{
    *releases = &video->Iran_release;
    releases[1] = &video->Irulan_release;
    releases[2] = &video->Mylene_release;
}


void
seek_h262_get_slices_colorspace(struct seek_h262_video *video,
				unsigned *color_space)
{
    seek_h262_get_colorspace(video, color_space);
}


void
seek_h262_picture_coding_type_description(unsigned picture_coding_type,
					  char **description)
{
    *description = picture_coding_type && picture_coding_type < 5
	? (char *) picture_coding_type_descriptions[picture_coding_type - 1]
	: "";
}


void
seek_h262_reset_inverse_scans(struct seek_h262_video *video)
{
    video->inverse_scan_0 = NULL;
    video->inverse_scan_1 = NULL;

    _seek_h262_set_quantisers(video);
}


void
seek_h262_set_do_release(struct seek_h262_video *video,
			 void (*do_release)(void *,
					    struct seek_h262_release *))
{
    video->do_release =
	(void (*)(struct seek_h262_video *, struct seek_h262_release *))
	do_release;
}


void
seek_h262_set_inverse_scans(struct seek_h262_video *video,
			    unsigned *inverse_scan_0, unsigned *inverse_scan_1)
{
    video->inverse_scan_0 = inverse_scan_0;
    video->inverse_scan_1 = inverse_scan_1;

    _seek_h262_set_quantisers(video);
}


void
seek_h262_set_slices_colorspace(struct seek_h262_video *video,
				unsigned color_space)
{
    if (((integral_q) video->slices[0] & 65535) == 11172) {
	if (color_space == SEEK_H262_GRAYSCALE) {
	    video->slices[X08_] = _seek_h262_GRAYSCALE_x08_11172_slices;
	    video->slices[X08q] = _seek_h262_GRAYSCALE_x08_11172_slices;
	    video->slices[X10_] = _seek_h262_GRAYSCALE_x___11172_slices;
	    video->slices[X10q] = _seek_h262_GRAYSCALE_x___11172_slicEs;
	    video->slices[X18_] = _seek_h262_GRAYSCALE_x___11172_slices;
	    video->slices[X18q] = _seek_h262_GRAYSCALE_x___11172_slicEs;
	    video->slices[X20_] = _seek_h262_GRAYSCALE_x20_11172_slices;
	} else if (color_space == SEEK_H262_SKIP) {
	    video->slices[X08_] = _seek_h262_SKIP______x_________slices;
	    video->slices[X08q] = _seek_h262_SKIP______x_________slices;
	    video->slices[X10_] = _seek_h262_SKIP______x_________slices;
	    video->slices[X10q] = _seek_h262_SKIP______x_________slices;
	    video->slices[X18_] = _seek_h262_SKIP______x_________slices;
	    video->slices[X18q] = _seek_h262_SKIP______x_________slices;
	    video->slices[X20_] = _seek_h262_SKIP______x_________slices;
	} else if (color_space == SEEK_H262_YCbCr) {
	    video->slices[X08_] = _seek_h262_YCbCr_____x08_11172_slices;
	    video->slices[X08q] = _seek_h262_YCbCr_____x08_11172_slices;
	    video->slices[X10_] = _seek_h262_YCbCr_____x___11172_slices;
	    video->slices[X10q] = _seek_h262_YCbCr_____x___11172_slicEs;
	    video->slices[X18_] = _seek_h262_YCbCr_____x___11172_slices;
	    video->slices[X18q] = _seek_h262_YCbCr_____x___11172_slicEs;
	    video->slices[X20_] = _seek_h262_YCbCr_____x20_11172_slices;
	} else {
	    video->slices[X08_] = _seek_h262___________x08_11172_slices;
	    video->slices[X08q] = _seek_h262___________x08_11172_slices;
	    video->slices[X10_] = _seek_h262___________x___11172_slices;
	    video->slices[X10q] = _seek_h262___________x___11172_slices;
	    video->slices[X18_] = _seek_h262___________x___11172_slices;
	    video->slices[X18q] = _seek_h262___________x___11172_slices;
	    video->slices[X20_] = _seek_h262___________x20_11172_slices;
	}
    } else {
	if (color_space == SEEK_H262_GRAYSCALE) {
	    video->slices[X08_] = _seek_h262_GRAYSCALE_x___13818_slices;
	    video->slices[X10_] = _seek_h262_GRAYSCALE_x___13818_slices;
	    video->slices[X18_] = _seek_h262_GRAYSCALE_x___13818_slices;
	    video->slices[X20_] = _seek_h262_GRAYSCALE_x___13818_slices;
	} else if (color_space == SEEK_H262_SKIP) {
	    video->slices[X08_] = _seek_h262_SKIP______x_________slices;
	    video->slices[X10_] = _seek_h262_SKIP______x_________slices;
	    video->slices[X18_] = _seek_h262_SKIP______x_________slices;
	    video->slices[X20_] = _seek_h262_SKIP______x_________slices;
	} else if (color_space == SEEK_H262_YCbCr) {
	    video->slices[X08_] = _seek_h262_YCbCr_____x___13818_slices;
	    video->slices[X10_] = _seek_h262_YCbCr_____x___13818_slices;
	    video->slices[X18_] = _seek_h262_YCbCr_____x___13818_slices;
	    video->slices[X20_] = _seek_h262_YCbCr_____x___13818_slices;
	} else {
	    video->slices[X08_] = _seek_h262___________x___13818_slices;
	    video->slices[X10_] = _seek_h262___________x___13818_slices;
	    video->slices[X18_] = _seek_h262___________x___13818_slices;
	    video->slices[X20_] = _seek_h262___________x___13818_slices;
	}
    }
}


void
seek_h262_skip_frames(struct seek_h262_video *video,
		      int (*stop_callback) (struct seek_h262_video *),
		      unsigned flags, unsigned skip_color_space)
{
    struct seek_h262_release swap_release;
    unsigned color_space, last_coding = 0, this_coding = 0;
    void (*next_x00000100) (struct seek_h262_video *);

    seek_h262_get_slices_colorspace(video, &color_space);
    if (color_space == SEEK_H262_none) {
	flags &= ~SEEK_H262_SKIP_bidirectionally_predictive_coded;
    }
    if (!(flags & SEEK_H262_SKIP_bidirectionally_predictive_coded)) {
	flags &= ~SEEK_H262_SAVE_PAST;
    }

    if (flags & SEEK_H262_SAVE_PAST) {
	swap_release = video->Iran_release;
    }

    next_x00000100 = ((integral_q) video->slices[0] & 65535) == 11172
	? seek_h262_next_11172_x00000100 : seek_h262_next_13818_x00000100;

    while (!stop_callback(video)) {
	if (video->start_code != picture_start_code) {
	    _seek_h262_abort_decoding((struct seek_h262_whatever *) video,
				      VIDEO_IMPROPER_START_CODE);
	}

	if (flags & SEEK_H262_SKIP_bidirectionally_predictive_coded) {
	    if (video->size < 2) {
		video->optimistique_moi(video);
	    }
	    this_coding = (video->buffer[1] & 0x38) == 0x18 ? 1 : 0;
	    if (this_coding != last_coding) {
		seek_h262_set_slices_colorspace
		    (video, this_coding ? skip_color_space : color_space);
		last_coding = this_coding;
	    }
	}

	next_x00000100(video);

	if (flags & SEEK_H262_SAVE_PAST) {
	    if (!this_coding) {
		video->Mylene_release = video->Iran_release;
		video->Iran_release = swap_release;
	    }
	}
    }

    if (last_coding) {
	seek_h262_set_slices_colorspace(video, color_space);
    }
}


void
seek_h262_time_code_semantics
    (unsigned time_code,
     struct seek_h262_time_code_semantics_ *time_code_semantics_)
{
    time_code_semantics_->drop_frame_flag = time_code >> 030;
    time_code_semantics_->time_code_hours = time_code >> 023 & 31;
    time_code_semantics_->time_code_minutes = time_code >> 015 & 63;
    time_code_semantics_->time_code_pictures = time_code >> 000 & 63;
    time_code_semantics_->time_code_seconds = time_code >> 006 & 63;
}
