/*
 * h262.e.part4.c
 * Copyright (C) 2007, 2008  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.1.h>
#include <h262.3.h>
#include <h262.e.h>

void
seek_h262_get_aspect_ratio_information(struct seek_h262_video *video,
				       unsigned *reply,
				       unsigned *numerator_return,
				       unsigned *denominator_return)
{
#if WORDS_BIGENDIAN
    seek_h262_aspect_ratio_information
	((video->sequence_header[0] >> 004) & 31, reply, numerator_return,
	 denominator_return);
#else
    seek_h262_aspect_ratio_information
	((video->sequence_header[0] >> 034) & 31, reply, numerator_return,
	 denominator_return);
#endif				/* WORDS_BIGENDIAN */
}


void
seek_h262_get_frame_rate(struct seek_h262_video *video,
			 unsigned *numerator_return,
			 unsigned *denominator_return)
{
    unsigned frame_rate_d, frame_rate_n;

#if WORDS_BIGENDIAN
    seek_h262_frame_rate_code_to_frame_rate_value
	(video->sequence_header[0] & 15, &frame_rate_n, &frame_rate_d);
#else
    seek_h262_frame_rate_code_to_frame_rate_value
	((video->sequence_header[0] >> 24) & 15, &frame_rate_n, &frame_rate_d);
#endif				/* WORDS_BIGENDIAN */
    if (((integral_q) video->slices[0] & 65535) == 13818) {
	if (frame_rate_n != SEEK_H262_FORBIDDEN
	    && frame_rate_n != SEEK_H262_RESERVED) {
	    unsigned frame_rate_extension_d, frame_rate_extension_n;

	    frame_rate_extension_d = video->sequence_extension[5] & 31;
	    frame_rate_extension_n = (video->sequence_extension[5] & 96) >> 5;
	    if (frame_rate_extension_d | frame_rate_extension_n) {
		frame_rate_d *= frame_rate_extension_d + 1;
		frame_rate_n *= frame_rate_extension_n + 1;
	    }
	}
    }

    *denominator_return = frame_rate_d;
    *numerator_return = frame_rate_n;
}


void
seek_h262_get_chroma_format(struct seek_h262_video *video,
			    unsigned *chroma_format)
{
    *chroma_format = video->chroma_format;
}


void
seek_h262_get_horizontal_size(struct seek_h262_video *video,
			      unsigned *horizontal_size)
{
    *horizontal_size = video->horizontal_size;
}


void
seek_h262_get_pel_aspect_ratio(struct seek_h262_video *video,
			       unsigned *numerator_return,
			       unsigned *denominator_return)
{
#if WORDS_BIGENDIAN
    seek_h262_pel_aspect_ratio
	((video->sequence_header[0] >> 004) & 31, numerator_return,
	 denominator_return);
#else
    seek_h262_pel_aspect_ratio
	((video->sequence_header[0] >> 034) & 31, numerator_return,
	 denominator_return);
#endif				/* WORDS_BIGENDIAN */
}


void
seek_h262_get_vertical_size(struct seek_h262_video *video,
			    unsigned *vertical_size)
{
    *vertical_size = video->vertical_size;
}
