/*
 * h262.f.c
 * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <h262.0.h>
#include <h262.1.h>
#include <h262.3.h>
#include <h262.4.h>
#include <h262.a.h>
#include <h262.f.h>

#define SYSTEM_ATTRIBUTES		struct seek_h262_system_attributes

static const char *error_names[] = {
/* *INDENT-OFF* */
    /* no available description */
    "no available description",
    /* FAILED_START_CODE_LOOKUP */
    "failed start code lookup",
    /* x02: BAD_IMPLEMENTATION */
    "bad implementation",
    /* x03: FAILED_PACK_HEADER_MARKER */
    "failed pack header marker",
    /* x04: FAILED_PACK_MARKER */
    "failed pack marker",
    /* x05: FAILED_PES_PACKET_MARKER */
    "failed PES packet marker",
    /* x06: FAILED_P_STD_BUFFER_MARKER */
    "failed P-STD buffer marker",
    /* x07: FAILED_STUFFING_BYTE */
    "failed stuffing byte",
    /* x08: IMPROPER_START_CODE */
    "improper start code",
    /* x09: INVALID_HEADER_LENGTH */
    "invalid header length",
    /* x0a: INVALID_PACKET */
    "invalid packet",
    /* x0b: INVALID_PACKET_LENGTH */
    "invalid packet length",
    /* x0c: INVALID_PES_HEADER_DATA_LENGTH */
    "invalid PES header data length",
    /* x0d: INVALID_PES_PACKET_LENGTH */
    "invalid PES packet length",
    /* x0e: INVALID_PROGRAM_STREAM_DIRECTORY_PES_PACKET */
    "invalid program stream directory PES packet",
    /* x0f: INVALID_PROGRAM_STREAM_MAP_PES_PACKET */
    "invalid program stream map PES packet",
    /* x10: INVALID_PTS_DTS_FLAGS */
    "invalid PTS_DTS flags",
    /* x11: UNSET_MARKER_BIT */
    "unset marker bit",
    /* x12: UNSET_MARKER_BITS */
    "unset marker bits"
/* *INDENT-ON* */
}, *start_code_names[] = {
/* *INDENT-OFF* */
    "picture_start_code",				/* 00 */
    "slice_start_code",					/* 01 */
    "slice_start_code",					/* 02 */
    "slice_start_code",					/* 03 */
    "slice_start_code",					/* 04 */
    "slice_start_code",					/* 05 */
    "slice_start_code",					/* 06 */
    "slice_start_code",					/* 07 */
    "slice_start_code",					/* 08 */
    "slice_start_code",					/* 09 */
    "slice_start_code",					/* 0a */
    "slice_start_code",					/* 0b */
    "slice_start_code",					/* 0c */
    "slice_start_code",					/* 0d */
    "slice_start_code",					/* 0e */
    "slice_start_code",					/* 0f */
    "slice_start_code",					/* 10 */
    "slice_start_code",					/* 11 */
    "slice_start_code",					/* 12 */
    "slice_start_code",					/* 13 */
    "slice_start_code",					/* 14 */
    "slice_start_code",					/* 15 */
    "slice_start_code",					/* 16 */
    "slice_start_code",					/* 17 */
    "slice_start_code",					/* 18 */
    "slice_start_code",					/* 19 */
    "slice_start_code",					/* 1a */
    "slice_start_code",					/* 1b */
    "slice_start_code",					/* 1c */
    "slice_start_code",					/* 1d */
    "slice_start_code",					/* 1e */
    "slice_start_code",					/* 1f */
    "slice_start_code",					/* 20 */
    "slice_start_code",					/* 21 */
    "slice_start_code",					/* 22 */
    "slice_start_code",					/* 23 */
    "slice_start_code",					/* 24 */
    "slice_start_code",					/* 25 */
    "slice_start_code",					/* 26 */
    "slice_start_code",					/* 27 */
    "slice_start_code",					/* 28 */
    "slice_start_code",					/* 29 */
    "slice_start_code",					/* 2a */
    "slice_start_code",					/* 2b */
    "slice_start_code",					/* 2c */
    "slice_start_code",					/* 2d */
    "slice_start_code",					/* 2e */
    "slice_start_code",					/* 2f */
    "slice_start_code",					/* 30 */
    "slice_start_code",					/* 31 */
    "slice_start_code",					/* 32 */
    "slice_start_code",					/* 33 */
    "slice_start_code",					/* 34 */
    "slice_start_code",					/* 35 */
    "slice_start_code",					/* 36 */
    "slice_start_code",					/* 37 */
    "slice_start_code",					/* 38 */
    "slice_start_code",					/* 39 */
    "slice_start_code",					/* 3a */
    "slice_start_code",					/* 3b */
    "slice_start_code",					/* 3c */
    "slice_start_code",					/* 3d */
    "slice_start_code",					/* 3e */
    "slice_start_code",					/* 3f */
    "slice_start_code",					/* 40 */
    "slice_start_code",					/* 41 */
    "slice_start_code",					/* 42 */
    "slice_start_code",					/* 43 */
    "slice_start_code",					/* 44 */
    "slice_start_code",					/* 45 */
    "slice_start_code",					/* 46 */
    "slice_start_code",					/* 47 */
    "slice_start_code",					/* 48 */
    "slice_start_code",					/* 49 */
    "slice_start_code",					/* 4a */
    "slice_start_code",					/* 4b */
    "slice_start_code",					/* 4c */
    "slice_start_code",					/* 4d */
    "slice_start_code",					/* 4e */
    "slice_start_code",					/* 4f */
    "slice_start_code",					/* 50 */
    "slice_start_code",					/* 51 */
    "slice_start_code",					/* 52 */
    "slice_start_code",					/* 53 */
    "slice_start_code",					/* 54 */
    "slice_start_code",					/* 55 */
    "slice_start_code",					/* 56 */
    "slice_start_code",					/* 57 */
    "slice_start_code",					/* 58 */
    "slice_start_code",					/* 59 */
    "slice_start_code",					/* 5a */
    "slice_start_code",					/* 5b */
    "slice_start_code",					/* 5c */
    "slice_start_code",					/* 5d */
    "slice_start_code",					/* 5e */
    "slice_start_code",					/* 5f */
    "slice_start_code",					/* 60 */
    "slice_start_code",					/* 61 */
    "slice_start_code",					/* 62 */
    "slice_start_code",					/* 63 */
    "slice_start_code",					/* 64 */
    "slice_start_code",					/* 65 */
    "slice_start_code",					/* 66 */
    "slice_start_code",					/* 67 */
    "slice_start_code",					/* 68 */
    "slice_start_code",					/* 69 */
    "slice_start_code",					/* 6a */
    "slice_start_code",					/* 6b */
    "slice_start_code",					/* 6c */
    "slice_start_code",					/* 6d */
    "slice_start_code",					/* 6e */
    "slice_start_code",					/* 6f */
    "slice_start_code",					/* 70 */
    "slice_start_code",					/* 71 */
    "slice_start_code",					/* 72 */
    "slice_start_code",					/* 73 */
    "slice_start_code",					/* 74 */
    "slice_start_code",					/* 75 */
    "slice_start_code",					/* 76 */
    "slice_start_code",					/* 77 */
    "slice_start_code",					/* 78 */
    "slice_start_code",					/* 79 */
    "slice_start_code",					/* 7a */
    "slice_start_code",					/* 7b */
    "slice_start_code",					/* 7c */
    "slice_start_code",					/* 7d */
    "slice_start_code",					/* 7e */
    "slice_start_code",					/* 7f */
    "slice_start_code",					/* 80 */
    "slice_start_code",					/* 81 */
    "slice_start_code",					/* 82 */
    "slice_start_code",					/* 83 */
    "slice_start_code",					/* 84 */
    "slice_start_code",					/* 85 */
    "slice_start_code",					/* 86 */
    "slice_start_code",					/* 87 */
    "slice_start_code",					/* 88 */
    "slice_start_code",					/* 89 */
    "slice_start_code",					/* 8a */
    "slice_start_code",					/* 8b */
    "slice_start_code",					/* 8c */
    "slice_start_code",					/* 8d */
    "slice_start_code",					/* 8e */
    "slice_start_code",					/* 8f */
    "slice_start_code",					/* 90 */
    "slice_start_code",					/* 91 */
    "slice_start_code",					/* 92 */
    "slice_start_code",					/* 93 */
    "slice_start_code",					/* 94 */
    "slice_start_code",					/* 95 */
    "slice_start_code",					/* 96 */
    "slice_start_code",					/* 97 */
    "slice_start_code",					/* 98 */
    "slice_start_code",					/* 99 */
    "slice_start_code",					/* 9a */
    "slice_start_code",					/* 9b */
    "slice_start_code",					/* 9c */
    "slice_start_code",					/* 9d */
    "slice_start_code",					/* 9e */
    "slice_start_code",					/* 9f */
    "slice_start_code",					/* a0 */
    "slice_start_code",					/* a1 */
    "slice_start_code",					/* a2 */
    "slice_start_code",					/* a3 */
    "slice_start_code",					/* a4 */
    "slice_start_code",					/* a5 */
    "slice_start_code",					/* a6 */
    "slice_start_code",					/* a7 */
    "slice_start_code",					/* a8 */
    "slice_start_code",					/* a9 */
    "slice_start_code",					/* aa */
    "slice_start_code",					/* ab */
    "slice_start_code",					/* ac */
    "slice_start_code",					/* ad */
    "slice_start_code",					/* ae */
    "slice_start_code",					/* af */
    "reserved",						/* b0 */
    "reserved",						/* b1 */
    "user_data_start_code",				/* b2 */
    "sequence_header_code",				/* b3 */
    "sequence_error_code",				/* b4 */
    "extension_start_code",				/* b5 */
    "reserved",						/* b6 */
    "sequence_end_code",				/* b7 */
    "group_start_code",					/* b8 */
    "MPEG_program_end_code",				/* b9 */
    "pack_start_code",					/* ba */
    "system_header_start_code",				/* bb */
    "program_stream_map",				/* bc */
    "private_stream_1",					/* bd */
    "padding_stream",					/* be */
    "private_stream_2",					/* bf */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* c0 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* c1 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* c2 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* c3 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* c4 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* c5 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* c6 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* c7 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* c8 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* c9 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* ca */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* cb */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* cc */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* cd */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* ce */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* cf */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* d0 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* d1 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* d2 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* d3 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* d4 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* d5 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* d6 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* d7 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* d8 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* d9 */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* da */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* db */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* dc */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* dd */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* de */
    "ISO/IEC 11172-3 or 13818-3 audio stream",		/* df */
    "ISO/IEC 11172-2 or 13818-2 video stream",		/* e0 */
    "ISO/IEC 11172-2 or 13818-2 video stream",		/* e1 */
    "ISO/IEC 11172-2 or 13818-2 video stream",		/* e2 */
    "ISO/IEC 11172-2 or 13818-2 video stream",		/* e3 */
    "ISO/IEC 11172-2 or 13818-2 video stream",		/* e4 */
    "ISO/IEC 11172-2 or 13818-2 video stream",		/* e5 */
    "ISO/IEC 11172-2 or 13818-2 video stream",		/* e6 */
    "ISO/IEC 11172-2 or 13818-2 video stream",		/* e7 */
    "ISO/IEC 11172-2 or 13818-2 video stream",		/* e8 */
    "ISO/IEC 11172-2 or 13818-2 video stream",		/* e9 */
    "ISO/IEC 11172-2 or 13818-2 video stream",		/* ea */
    "ISO/IEC 11172-2 or 13818-2 video stream",		/* eb */
    "ISO/IEC 11172-2 or 13818-2 video stream",		/* ec */
    "ISO/IEC 11172-2 or 13818-2 video stream",		/* ed */
    "ISO/IEC 11172-2 or 13818-2 video stream",		/* ee */
    "ISO/IEC 11172-2 or 13818-2 video stream",		/* ef */
    "ECM",						/* f0 */
    "EMM",						/* f1 */
    "DSM CC",						/* f2 */
    "ISO/IEC 13522 stream",				/* f3 */
    "reserved data stream",				/* f4 */
    "reserved data stream",				/* f5 */
    "reserved data stream",				/* f6 */
    "reserved data stream",				/* f7 */
    "reserved data stream",				/* f8 */
    "reserved data stream",				/* f9 */
    "reserved data stream",				/* fa */
    "reserved data stream",				/* fb */
    "reserved data stream",				/* fc */
    "reserved data stream",				/* fd */
    "reserved data stream",				/* fe */
    "program stream directory"				/* ff */
};

void
seek_h262_get_system_attributes(void *system_return,
				SYSTEM_ATTRIBUTES *system_attributes,
				unsigned mask)
{
    struct seek_h262_system *system;

    system = (struct seek_h262_system *) system_return;
    if (mask & SEEK_H262_SYSTEM_ABORT_DECODING_MASK) {
	system_attributes->abort_decoding =
	    (void (*)(void *)) system->abort_decoding;
    }
    if (mask & SEEK_H262_SYSTEM_BUFFER_MASK) {
	system_attributes->buffer = system->buffer;
    }
    if (mask & SEEK_H262_SYSTEM_CORRECT_UNDERFLOW_MASK) {
	system_attributes->correct_underflow =
	    (void (*)(void *)) system->correct_underflow;
    }
    if (mask & SEEK_H262_SYSTEM_MINIMUM_USEFUL_SIZE_MASK) {
	system_attributes->minimum_useful_size = system->minimum_useful_size;
    }
    if (mask & SEEK_H262_SYSTEM_NEXT_START_CODE_MASK) {
	system_attributes->next_start_code =
	    (void (*)(void *)) system->next_start_code;
    }
    if (mask & SEEK_H262_SYSTEM_PACKET_DATA_MASK) {
	system_attributes->packet_data =
	    (void (*)(void *, unsigned)) system->packet_data;
    }
    if (mask & SEEK_H262_SYSTEM_SEEK_STREAM_MASK) {
	system_attributes->seek_stream =
	    (void (*)(void *, unsigned)) system->seek_stream;
    }
    if (mask & SEEK_H262_SYSTEM_SIZE_MASK) {
	system_attributes->size = system->size;
    }
#if __ENABLE_X000001BA___HACK_1__
    if (mask & SEEK_H262_SYSTEM_X000001BA___HACK_1_MASK) {
	system_attributes->x000001ba___hack_1 = system->x000001ba___hack_1;
    }
#endif				/* __ENABLE_X000001BA___HACK_1__ */
#if __ENABLE_X000001BB___HACK_1__
    if (mask & SEEK_H262_SYSTEM_X000001BB___HACK_1_MASK) {
	system_attributes->x000001bb___hack_1 = system->x000001bb___hack_1;
    }
#endif				/* __ENABLE_X000001BB___HACK_1__ */
#if __ENABLE_X000001BB___HACK_2__
    if (mask & SEEK_H262_SYSTEM_X000001BB___HACK_2_MASK) {
	system_attributes->x000001bb___hack_2 = system->x000001bb___hack_2;
    }
#endif				/* __ENABLE_X000001BB___HACK_2__ */
#if __ENABLE_X000001BC___HACK_1__
    if (mask & SEEK_H262_SYSTEM_X000001BC___HACK_1_MASK) {
	system_attributes->x000001bc___hack_1 = system->x000001bc___hack_1;
    }
#endif				/* __ENABLE_X000001BC___HACK_1__ */
#if __ENABLE_X000001BC___HACK_2__
    if (mask & SEEK_H262_SYSTEM_X000001BC___HACK_2_MASK) {
	system_attributes->x000001bc___hack_2 = system->x000001bc___hack_2;
    }
#endif				/* __ENABLE_X000001BC___HACK_2__ */
#if __ENABLE_X000001BC___HACK_3__
    if (mask & SEEK_H262_SYSTEM_X000001BC___HACK_3_MASK) {
	system_attributes->x000001bc___hack_3 = system->x000001bc___hack_3;
    }
#endif				/* __ENABLE_X000001BC___HACK_3__ */
#if __ENABLE_X000001BC___HACK_4__
    if (mask & SEEK_H262_SYSTEM_X000001BC___HACK_4_MASK) {
	system_attributes->x000001bc___hack_4 = system->x000001bc___hack_4;
    }
#endif				/* __ENABLE_X000001BC___HACK_4__ */
#if __ENABLE_X000001FF___HACK_0__
    if (mask & SEEK_H262_SYSTEM_X000001FF___HACK_0_MASK) {
	system_attributes->x000001ff___hack_0 =
	    (void (*)(void *)) system->x000001ff___hack_0;
    }
#endif				/* __ENABLE_X000001FF___HACK_0__ */
#if __ENABLE_X000001FF___HACK_1__
    if (mask & SEEK_H262_SYSTEM_X000001FF___HACK_1_MASK) {
	system_attributes->x000001ff___hack_1 = system->x000001ff___hack_1;
    }
#endif				/* __ENABLE_X000001FF___HACK_1__ */
#if __ENABLE_X000001FF___HACK_2__
    if (mask & SEEK_H262_SYSTEM_X000001FF___HACK_2_MASK) {
	system_attributes->x000001ff___hack_2 = system->x000001ff___hack_2;
    }
#endif				/* __ENABLE_X000001FF___HACK_2__ */
#if __ENABLE_X000001S1___HACK_3__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_3_MASK) {
	system_attributes->x000001s1___hack_3 = system->x000001s1___hack_3;
    }
#endif				/* __ENABLE_X000001S1___HACK_3__ */
#if __ENABLE_X000001S1___HACK_4__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_4_MASK) {
	system_attributes->x000001s1___hack_4 = system->x000001s1___hack_4;
    }
#endif				/* __ENABLE_X000001S1___HACK_4__ */
#if __ENABLE_X000001S1___HACK_5__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_5_MASK) {
	system_attributes->x000001s1___hack_5 = system->x000001s1___hack_5;
    }
#endif				/* __ENABLE_X000001S1___HACK_5__ */
#if __ENABLE_X000001S1___HACK_6__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_6_MASK) {
	system_attributes->x000001s1___hack_6 = system->x000001s1___hack_6;
    }
#endif				/* __ENABLE_X000001S1___HACK_6__ */
#if __ENABLE_X000001S1___HACK_7__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_7_MASK) {
	system_attributes->x000001s1___hack_7 = system->x000001s1___hack_7;
    }
#endif				/* __ENABLE_X000001S1___HACK_7__ */
#if __ENABLE_X000001S1___HACK_8__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_8_MASK) {
	system_attributes->x000001s1___hack_8 = system->x000001s1___hack_8;
    }
#endif				/* __ENABLE_X000001S1___HACK_8__ */
#if __ENABLE_X000001S1___HACK_9__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_9_MASK) {
	system_attributes->x000001s1___hack_9 = system->x000001s1___hack_9;
    }
#endif				/* __ENABLE_X000001S1___HACK_9__ */
}


void
seek_h262_reset_system(void *system_return)
{
    struct seek_h262_system *system;

    system = (struct seek_h262_system *) system_return;
    system->buffer = NULL;
    system->error = 0;
    system->minimum_useful_size = 64;
    system->next_start_code =
	(void (*)(struct seek_h262_system *)) seek_h262_next_start_code;
    system->size = 0;
#if __ENABLE_X000001BA___HACK_1__
    system->x000001ba___hack_1 = NULL;
#endif				/* __ENABLE_X000001BA___HACK_1__ */
#if __ENABLE_X000001BB___HACK_1__
    system->x000001bb___hack_1 = NULL;
#endif				/* __ENABLE_X000001BB___HACK_1__ */
#if __ENABLE_X000001BB___HACK_2__
    system->x000001bb___hack_2 = NULL;
#endif				/* __ENABLE_X000001BB___HACK_2__ */
#if __ENABLE_X000001BC___HACK_1__
    system->x000001bc___hack_1 = NULL;
#endif				/* __ENABLE_X000001BC___HACK_1__ */
#if __ENABLE_X000001BC___HACK_2__
    system->x000001bc___hack_2 = NULL;
#endif				/* __ENABLE_X000001BC___HACK_2__ */
#if __ENABLE_X000001BC___HACK_3__
    system->x000001bc___hack_3 = NULL;
#endif				/* __ENABLE_X000001BC___HACK_3__ */
#if __ENABLE_X000001BC___HACK_4__
    system->x000001bc___hack_4 = NULL;
#endif				/* __ENABLE_X000001BC___HACK_4__ */
#if __ENABLE_X000001FF___HACK_0__
    system->x000001ff___hack_0 = NULL;
#endif				/* __ENABLE_X000001FF___HACK_0__ */
#if __ENABLE_X000001FF___HACK_1__
    system->x000001ff___hack_1 = NULL;
#endif				/* __ENABLE_X000001FF___HACK_1__ */
#if __ENABLE_X000001FF___HACK_2__
    system->x000001ff___hack_2 = NULL;
#endif				/* __ENABLE_X000001FF___HACK_2__ */
#if __ENABLE_X000001S1___HACK_3__
    system->x000001s1___hack_3 = NULL;
#endif				/* __ENABLE_X000001S1___HACK_3__ */
#if __ENABLE_X000001S1___HACK_4__
    system->x000001s1___hack_4 = NULL;
#endif				/* __ENABLE_X000001S1___HACK_4__ */
#if __ENABLE_X000001S1___HACK_5__
    system->x000001s1___hack_5 = NULL;
#endif				/* __ENABLE_X000001S1___HACK_5__ */
#if __ENABLE_X000001S1___HACK_6__
    system->x000001s1___hack_6 = NULL;
#endif				/* __ENABLE_X000001S1___HACK_6__ */
#if __ENABLE_X000001S1___HACK_7__
    system->x000001s1___hack_7 = NULL;
#endif				/* __ENABLE_X000001S1___HACK_7__ */
#if __ENABLE_X000001S1___HACK_8__
    system->x000001s1___hack_8 = NULL;
#endif				/* __ENABLE_X000001S1___HACK_8__ */
#if __ENABLE_X000001S1___HACK_9__
    system->x000001s1___hack_9 = NULL;
#endif				/* __ENABLE_X000001S1___HACK_9__ */
}


void
seek_h262_reset_system_attributes(void *system_return, unsigned mask)
{
    struct seek_h262_system *system;

    system = (struct seek_h262_system *) system_return;
    if (mask & SEEK_H262_SYSTEM_BUFFER_MASK) {
	system->buffer = NULL;
    }
    if (mask & SEEK_H262_SYSTEM_NEXT_START_CODE_MASK) {
	system->next_start_code =
	    (void (*)(struct seek_h262_system *)) seek_h262_next_start_code;
    }
    if (mask & SEEK_H262_SYSTEM_SIZE_MASK) {
	system->size = 0;
    }
#if __ENABLE_X000001BA___HACK_1__
    if (mask & SEEK_H262_SYSTEM_X000001BA___HACK_1_MASK) {
	system->x000001ba___hack_1 = NULL;
    }
#endif				/* __ENABLE_X000001BA___HACK_1__ */
#if __ENABLE_X000001BB___HACK_1__
    if (mask & SEEK_H262_SYSTEM_X000001BB___HACK_1_MASK) {
	system->x000001bb___hack_1 = NULL;
    }
#endif				/* __ENABLE_X000001BB___HACK_1__ */
#if __ENABLE_X000001BB___HACK_2__
    if (mask & SEEK_H262_SYSTEM_X000001BB___HACK_2_MASK) {
	system->x000001bb___hack_2 = NULL;
    }
#endif				/* __ENABLE_X000001BB___HACK_2__ */
#if __ENABLE_X000001BC___HACK_1__
    if (mask & SEEK_H262_SYSTEM_X000001BC___HACK_1_MASK) {
	system->x000001bc___hack_1 = NULL;
    }
#endif				/* __ENABLE_X000001BC___HACK_1__ */
#if __ENABLE_X000001BC___HACK_2__
    if (mask & SEEK_H262_SYSTEM_X000001BC___HACK_2_MASK) {
	system->x000001bc___hack_2 = NULL;
    }
#endif				/* __ENABLE_X000001BC___HACK_2__ */
#if __ENABLE_X000001BC___HACK_3__
    if (mask & SEEK_H262_SYSTEM_X000001BC___HACK_3_MASK) {
	system->x000001bc___hack_3 = NULL;
    }
#endif				/* __ENABLE_X000001BC___HACK_3__ */
#if __ENABLE_X000001BC___HACK_4__
    if (mask & SEEK_H262_SYSTEM_X000001BC___HACK_4_MASK) {
	system->x000001bc___hack_4 = NULL;
    }
#endif				/* __ENABLE_X000001BC___HACK_4__ */
#if __ENABLE_X000001FF___HACK_0__
    if (mask & SEEK_H262_SYSTEM_X000001FF___HACK_0_MASK) {
	system->x000001ff___hack_0 = NULL;
    }
#endif				/* __ENABLE_X000001FF___HACK_0__ */
#if __ENABLE_X000001FF___HACK_1__
    if (mask & SEEK_H262_SYSTEM_X000001FF___HACK_1_MASK) {
	system->x000001ff___hack_1 = NULL;
    }
#endif				/* __ENABLE_X000001FF___HACK_1__ */
#if __ENABLE_X000001FF___HACK_2__
    if (mask & SEEK_H262_SYSTEM_X000001FF___HACK_2_MASK) {
	system->x000001ff___hack_2 = NULL;
    }
#endif				/* __ENABLE_X000001FF___HACK_2__ */
#if __ENABLE_X000001S1___HACK_3__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_3_MASK) {
	system->x000001s1___hack_3 = NULL;
    }
#endif				/* __ENABLE_X000001S1___HACK_3__ */
#if __ENABLE_X000001S1___HACK_4__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_4_MASK) {
	system->x000001s1___hack_4 = NULL;
    }
#endif				/* __ENABLE_X000001S1___HACK_4__ */
#if __ENABLE_X000001S1___HACK_5__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_5_MASK) {
	system->x000001s1___hack_5 = NULL;
    }
#endif				/* __ENABLE_X000001S1___HACK_5__ */
#if __ENABLE_X000001S1___HACK_6__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_6_MASK) {
	system->x000001s1___hack_6 = NULL;
    }
#endif				/* __ENABLE_X000001S1___HACK_6__ */
#if __ENABLE_X000001S1___HACK_7__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_7_MASK) {
	system->x000001s1___hack_7 = NULL;
    }
#endif				/* __ENABLE_X000001S1___HACK_7__ */
#if __ENABLE_X000001S1___HACK_8__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_8_MASK) {
	system->x000001s1___hack_8 = NULL;
    }
#endif				/* __ENABLE_X000001S1___HACK_8__ */
#if __ENABLE_X000001S1___HACK_9__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_9_MASK) {
	system->x000001s1___hack_9 = NULL;
    }
#endif				/* __ENABLE_X000001S1___HACK_9__ */
}


void
seek_h262_set_system_attributes(void *system_return,
				SYSTEM_ATTRIBUTES *system_attributes,
				unsigned mask)
{
    struct seek_h262_system *system;

    system = (struct seek_h262_system *) system_return;
    if (mask & SEEK_H262_SYSTEM_ABORT_DECODING_MASK) {
	system->abort_decoding = (void (*)(struct seek_h262_system *))
	    system_attributes->abort_decoding;
    }
    if (mask & SEEK_H262_SYSTEM_BUFFER_MASK) {
	system->buffer = system_attributes->buffer;
    }
    if (mask & SEEK_H262_SYSTEM_CORRECT_UNDERFLOW_MASK) {
	system->correct_underflow = (void (*)(struct seek_h262_system *))
	    system_attributes->correct_underflow;
    }
    if (mask & SEEK_H262_SYSTEM_NEXT_START_CODE_MASK) {
	system->next_start_code = (void (*)(struct seek_h262_system *))
	    system_attributes->next_start_code;
    }
    if (mask & SEEK_H262_SYSTEM_PACKET_DATA_MASK) {
	system->packet_data = (void (*)(struct seek_h262_system *, unsigned))
	    system_attributes->packet_data;
    }
    if (mask & SEEK_H262_SYSTEM_SEEK_STREAM_MASK) {
	system->seek_stream = (void (*)(struct seek_h262_system *, unsigned))
	    system_attributes->seek_stream;
    }
    if (mask & SEEK_H262_SYSTEM_SIZE_MASK) {
	system->size = system_attributes->size;
    }
#if __ENABLE_X000001BA___HACK_1__
    if (mask & SEEK_H262_SYSTEM_X000001BA___HACK_1_MASK) {
	system->x000001ba___hack_1 = system_attributes->x000001ba___hack_1;
    }
#endif				/* __ENABLE_X000001BA___HACK_1__ */
#if __ENABLE_X000001BB___HACK_1__
    if (mask & SEEK_H262_SYSTEM_X000001BB___HACK_1_MASK) {
	system->x000001bb___hack_1 = system_attributes->x000001bb___hack_1;
    }
#endif				/* __ENABLE_X000001BB___HACK_1__ */
#if __ENABLE_X000001BB___HACK_2__
    if (mask & SEEK_H262_SYSTEM_X000001BB___HACK_2_MASK) {
	system->x000001bb___hack_2 = system_attributes->x000001bb___hack_2;
    }
#endif				/* __ENABLE_X000001BB___HACK_2__ */
#if __ENABLE_X000001BC___HACK_1__
    if (mask & SEEK_H262_SYSTEM_X000001BC___HACK_1_MASK) {
	system->x000001bc___hack_1 = system_attributes->x000001bc___hack_1;
    }
#endif				/* __ENABLE_X000001BC___HACK_1__ */
#if __ENABLE_X000001BC___HACK_2__
    if (mask & SEEK_H262_SYSTEM_X000001BC___HACK_2_MASK) {
	system->x000001bc___hack_2 = system_attributes->x000001bc___hack_2;
    }
#endif				/* __ENABLE_X000001BC___HACK_2__ */
#if __ENABLE_X000001BC___HACK_3__
    if (mask & SEEK_H262_SYSTEM_X000001BC___HACK_3_MASK) {
	system->x000001bc___hack_3 = system_attributes->x000001bc___hack_3;
    }
#endif				/* __ENABLE_X000001BC___HACK_3__ */
#if __ENABLE_X000001BC___HACK_4__
    if (mask & SEEK_H262_SYSTEM_X000001BC___HACK_4_MASK) {
	system->x000001bc___hack_4 = system_attributes->x000001bc___hack_4;
    }
#endif				/* __ENABLE_X000001BC___HACK_4__ */
#if __ENABLE_X000001FF___HACK_0__
    if (mask & SEEK_H262_SYSTEM_X000001FF___HACK_0_MASK) {
	system->x000001ff___hack_0 = (void (*)(struct seek_h262_system *))
	    system_attributes->x000001ff___hack_0;
    }
#endif				/* __ENABLE_X000001FF___HACK_0__ */
#if __ENABLE_X000001FF___HACK_1__
    if (mask & SEEK_H262_SYSTEM_X000001FF___HACK_1_MASK) {
	system->x000001ff___hack_1 = system_attributes->x000001ff___hack_1;
    }
#endif				/* __ENABLE_X000001FF___HACK_1__ */
#if __ENABLE_X000001FF___HACK_2__
    if (mask & SEEK_H262_SYSTEM_X000001FF___HACK_2_MASK) {
	system->x000001ff___hack_2 = system_attributes->x000001ff___hack_2;
    }
#endif				/* __ENABLE_X000001FF___HACK_2__ */
#if __ENABLE_X000001S1___HACK_3__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_3_MASK) {
	system->x000001s1___hack_3 = system_attributes->x000001s1___hack_3;
    }
#endif				/* __ENABLE_X000001S1___HACK_3__ */
#if __ENABLE_X000001S1___HACK_4__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_4_MASK) {
	system->x000001s1___hack_4 = system_attributes->x000001s1___hack_4;
    }
#endif				/* __ENABLE_X000001S1___HACK_4__ */
#if __ENABLE_X000001S1___HACK_5__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_5_MASK) {
	system->x000001s1___hack_5 = system_attributes->x000001s1___hack_5;
    }
#endif				/* __ENABLE_X000001S1___HACK_5__ */
#if __ENABLE_X000001S1___HACK_6__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_6_MASK) {
	system->x000001s1___hack_6 = system_attributes->x000001s1___hack_6;
    }
#endif				/* __ENABLE_X000001S1___HACK_6__ */
#if __ENABLE_X000001S1___HACK_7__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_7_MASK) {
	system->x000001s1___hack_7 = system_attributes->x000001s1___hack_7;
    }
#endif				/* __ENABLE_X000001S1___HACK_7__ */
#if __ENABLE_X000001S1___HACK_8__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_8_MASK) {
	system->x000001s1___hack_8 = system_attributes->x000001s1___hack_8;
    }
#endif				/* __ENABLE_X000001S1___HACK_8__ */
#if __ENABLE_X000001S1___HACK_9__
    if (mask & SEEK_H262_SYSTEM_X000001S1___HACK_9_MASK) {
	system->x000001s1___hack_9 = system_attributes->x000001s1___hack_9;
    }
#endif				/* __ENABLE_X000001S1___HACK_9__ */
}


void
seek_h262_start_code_name(unsigned start_code, char **start_code_name)
{
    *start_code_name = (char *) start_code_names[start_code & 255];
}


void
seek_h262_system_error_string(unsigned error, char **error_name)
{
    if (error < SYSTEM_NUMBER_OF_ERROR_CODES) {
	*error_name = (char *) error_names[error];
    } else {
	*error_name = NULL;
    }
}


void
seek_h262_system_size(unsigned *size_return)
{
    *size_return = sizeof(struct seek_h262_system);
}
