/*
 * h262.f.h
 * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008  Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __SEEK_H262_F_H__
#define __SEEK_H262_F_H__

#include <h262.0.h>
#include <h262.3.h>

/*
 * system attribute mask definitions
 */
#define SEEK_H262_SYSTEM_ABORT_DECODING_MASK \
					(1L << 0x00)
#define SEEK_H262_SYSTEM_BUFFER_MASK	(1L << 0x01)
#define SEEK_H262_SYSTEM_CORRECT_UNDERFLOW_MASK \
					(1L << 0x02)
#define SEEK_H262_SYSTEM_MINIMUM_USEFUL_SIZE_MASK \
					(1L << 0x03)
#define SEEK_H262_SYSTEM_MPEG2_PROGRAM_STREAM_ID_MASK \
					(1L << 0x04)
#define SEEK_H262_SYSTEM_NEXT_START_CODE_MASK \
					(1L << 0x05)
#define SEEK_H262_SYSTEM_PACKET_DATA_MASK \
					(1L << 0x06)
#define SEEK_H262_SYSTEM_SEEK_STREAM_MASK \
					(1L << 0x07)
#define SEEK_H262_SYSTEM_SIZE_MASK	(1L << 0x08)
#if __ENABLE_X000001BA___HACK_1__
# define SEEK_H262_SYSTEM_X000001BA___HACK_1_MASK \
					(1L << 0x09)
#endif				/* __ENABLE_X000001BA___HACK_1__ */
#if __ENABLE_X000001BB___HACK_1__
# define SEEK_H262_SYSTEM_X000001BB___HACK_1_MASK \
					(1L << 0x0a)
#endif				/* __ENABLE_X000001BB___HACK_1__ */
#if __ENABLE_X000001BB___HACK_2__
# define SEEK_H262_SYSTEM_X000001BB___HACK_2_MASK \
					(1L << 0x0b)
#endif				/* __ENABLE_X000001BB___HACK_2__ */
#if __ENABLE_X000001BC___HACK_1__
# define SEEK_H262_SYSTEM_X000001BC___HACK_1_MASK \
					(1L << 0x0c)
#endif				/* __ENABLE_X000001BC___HACK_1__ */
#if __ENABLE_X000001BC___HACK_2__
# define SEEK_H262_SYSTEM_X000001BC___HACK_2_MASK \
					(1L << 0x0d)
#endif				/* __ENABLE_X000001BC___HACK_2__ */
#if __ENABLE_X000001BC___HACK_3__
# define SEEK_H262_SYSTEM_X000001BC___HACK_3_MASK \
					(1L << 0x0e)
#endif				/* __ENABLE_X000001BC___HACK_3__ */
#if __ENABLE_X000001BC___HACK_4__
# define SEEK_H262_SYSTEM_X000001BC___HACK_4_MASK \
					(1L << 0x0f)
#endif				/* __ENABLE_X000001BC___HACK_4__ */
#if __ENABLE_X000001FF___HACK_0__
# define SEEK_H262_SYSTEM_X000001FF___HACK_0_MASK \
					(1L << 0x10)
#endif				/* __ENABLE_X000001FF___HACK_0__ */
#if __ENABLE_X000001FF___HACK_1__
# define SEEK_H262_SYSTEM_X000001FF___HACK_1_MASK \
					(1L << 0x11)
#endif				/* __ENABLE_X000001FF___HACK_1__ */
#if __ENABLE_X000001FF___HACK_2__
# define SEEK_H262_SYSTEM_X000001FF___HACK_2_MASK \
					(1L << 0x12)
#endif				/* __ENABLE_X000001FF___HACK_2__ */
#if __ENABLE_X000001S1___HACK_3__
# define SEEK_H262_SYSTEM_X000001S1___HACK_3_MASK \
					(1L << 0x13)
#endif				/* __ENABLE_X000001S1___HACK_3__ */
#if __ENABLE_X000001S1___HACK_4__
# define SEEK_H262_SYSTEM_X000001S1___HACK_4_MASK \
					(1L << 0x14)
#endif				/* __ENABLE_X000001S1___HACK_4__ */
#if __ENABLE_X000001S1___HACK_5__
# define SEEK_H262_SYSTEM_X000001S1___HACK_5_MASK \
					(1L << 0x15)
#endif				/* __ENABLE_X000001S1___HACK_5__ */
#if __ENABLE_X000001S1___HACK_6__
# define SEEK_H262_SYSTEM_X000001S1___HACK_6_MASK \
					(1L << 0x16)
#endif				/* __ENABLE_X000001S1___HACK_6__ */
#if __ENABLE_X000001S1___HACK_7__
# define SEEK_H262_SYSTEM_X000001S1___HACK_7_MASK \
					(1L << 0x17)
#endif				/* __ENABLE_X000001S1___HACK_7__ */
#if __ENABLE_X000001S1___HACK_8__
# define SEEK_H262_SYSTEM_X000001S1___HACK_8_MASK \
					(1L << 0x18)
#endif				/* __ENABLE_X000001S1___HACK_8__ */
#if __ENABLE_X000001S1___HACK_9__
# define SEEK_H262_SYSTEM_X000001S1___HACK_9_MASK \
					(1L << 0x19)
#endif				/* __ENABLE_X000001S1___HACK_9__ */

typedef struct seek_h262_system_attributes {
    unsigned minimum_useful_size, size;
    unsigned char *buffer;
    void (*abort_decoding) (void *), (*correct_underflow) (void *),
	(*next_start_code) (void *), (*packet_data) (void *, unsigned),
	(*seek_stream) (void *, unsigned);
#if __ENABLE_X000001BA___HACK_1__
    void (*x000001ba___hack_1) (struct seek_h262_x000001ba__ *);
#endif				/* __ENABLE_X000001BA___HACK_1__ */
#if __ENABLE_X000001BB___HACK_1__
    void (*x000001bb___hack_1) (struct seek_h262_x000001bb__ *);
#endif				/* __ENABLE_X000001BB___HACK_1__ */
#if __ENABLE_X000001BB___HACK_2__
    void (*x000001bb___hack_2) (struct seek_h262_x000001bb__ *);
#endif				/* __ENABLE_X000001BB___HACK_2__ */
#if __ENABLE_X000001BC___HACK_1__
    void (*x000001bc___hack_1) (struct seek_h262_x000001bc__ *);
#endif				/* __ENABLE_X000001BC___HACK_1__ */
#if __ENABLE_X000001BC___HACK_2__
    void (*x000001bc___hack_2) (struct seek_h262_x000001bc__ *);
#endif				/* __ENABLE_X000001BC___HACK_2__ */
#if __ENABLE_X000001BC___HACK_3__
    void (*x000001bc___hack_3) (struct seek_h262_x000001bc__ *);
#endif				/* __ENABLE_X000001BC___HACK_3__ */
#if __ENABLE_X000001BC___HACK_4__
    void (*x000001bc___hack_4) (struct seek_h262_x000001bc__ *);
#endif				/* __ENABLE_X000001BC___HACK_4__ */
#if __ENABLE_X000001FF___HACK_0__
    void (*x000001ff___hack_0) (void *);
#endif				/* __ENABLE_X000001FF___HACK_0__ */
#if __ENABLE_X000001FF___HACK_1__
    void (*x000001ff___hack_1) (struct seek_h262_x000001ff__ *);
#endif				/* __ENABLE_X000001FF___HACK_1__ */
#if __ENABLE_X000001FF___HACK_2__
    void (*x000001ff___hack_2) (struct seek_h262_x000001ff__ *);
#endif				/* __ENABLE_X000001FF___HACK_2__ */
#if __ENABLE_X000001S1___HACK_3__
    void (*x000001s1___hack_3) (struct seek_h262_x000001s1__ *);
#endif				/* __ENABLE_X000001S1___HACK_3__ */
#if __ENABLE_X000001S1___HACK_4__
    void (*x000001s1___hack_4) (struct seek_h262_x000001s1__ *);
#endif				/* __ENABLE_X000001S1___HACK_4__ */
#if __ENABLE_X000001S1___HACK_5__
    void (*x000001s1___hack_5) (struct seek_h262_x000001s1__ *);
#endif				/* __ENABLE_X000001S1___HACK_5__ */
#if __ENABLE_X000001S1___HACK_6__
    void (*x000001s1___hack_6) (struct seek_h262_x000001s1__ *);
#endif				/* __ENABLE_X000001S1___HACK_6__ */
#if __ENABLE_X000001S1___HACK_7__
    void (*x000001s1___hack_7) (struct seek_h262_x000001s1__ *);
#endif				/* __ENABLE_X000001S1___HACK_7__ */
#if __ENABLE_X000001S1___HACK_8__
    void (*x000001s1___hack_8) (struct seek_h262_x000001s1__ *);
#endif				/* __ENABLE_X000001S1___HACK_8__ */
#if __ENABLE_X000001S1___HACK_9__
    void (*x000001s1___hack_9) (struct seek_h262_x000001s1__ *);
#endif				/* __ENABLE_X000001S1___HACK_9__ */
} seek_h262_system_attributes;

#endif				/* __SEEK_H262_F_H__ */
